import CellPainter from "../../tr-grid-util/es6/CellPainter.js";
import { Ext } from "../../tr-grid-util/es6/Ext.js";
import ElfUtil from "../../tr-grid-util/es6/ElfUtil.js";
import { ElementWrapper } from "../../tr-grid-util/es6/ElementWrapper.js";
import { FieldFormatter, DATE_TIME } from "../../tr-grid-util/es6/FieldFormatter.js";
import NumberFormatter from "../../tr-grid-util/es6/NumberFormatter.js";
import { PercentBar } from "../../tr-grid-util/es6/PercentBar.js";
import { Table } from "../../tr-grid-util/es6/Table.js";

declare namespace PreviewTable {

    type Bar = {
        isShowValue?: boolean|null,
        isUsedMovement?: boolean|null
    };

    type ConfigDateTime = {
        formatDate?: string|null,
        isUsedGMT: boolean
    };

    type ConfigDisplayColor = {
        background?: string|null,
        textColor?: string|null
    };

    type Conditional = {
        isShowValue?: boolean|null,
        isUsedMovement?: boolean|null,
        conditionSequence: ((...params: any[]) => any)|null,
        colors: Colors|null
    };

    type ConfigDisplayStyle = {
        mode: string,
        alignment?: string|null,
        bar?: Bar|null,
        heatmap?: Heatmap|null,
        conditionalColor: Conditional|null
    };

    type ConfigUpdateStyleValue = {
        isTextChangedStyle?: boolean|null,
        isCellChangedStyle?: boolean|null,
        colors?: string|null,
        isBackgroundBlink?: boolean|null
    };

    type ConfigValueFormat = {
        mode: string,
        decimalPlaces?: number|null,
        isUsedSeparator?: boolean|null,
        isShowPlusSignh?: boolean|null,
        scaled?: string|null,
        isPercentMultiply100?: boolean|null
    };

    type DisplayStyleTapGeneralMode = {
        aligment?: string|null
    };

    type DisplayStyleTapBarMode = {
        showValues?: boolean|null,
        useMovementColor?: boolean|null,
        alignment?: string|null
    };

    type DisplayStyleTapHeatMapMode = {
        mode?: string|null,
        midPoint?: number|null,
        alignment?: string|null
    };

    type DisplayStyleTap = {
        mode?: string|null,
        general?: DisplayStyleTapGeneralMode|null,
        bar?: DisplayStyleTapBarMode|null,
        heatmap?: DisplayStyleTapHeatMapMode|null
    };

    type FormatTab = {
        formatType?: string|null,
        decimalPlaces?: string|null,
        separator?: boolean|null,
        plusSign?: boolean|null,
        scalingUnit?: string|null,
        multiplyBy100?: boolean|null
    };

    type Heatmap = {
        midPoint?: string|null,
        applied?: string|null
    };

    type Colors = {
        color: string,
        background: string
    };

}

declare class PreviewTable extends ElementWrapper {

    constructor(elem: Element|null);

    public getStructureFormat(): any;

    public loadDisplayData(config: any): void;

    public setDateFormatString(formatDate: string): void;

    public setDateFormatUseGMT(isUsedGMT?: boolean|null): void;

    public setDateFormatLanguage(language: string): void;

    public setDisplayStyleAlignment(alignment: string): void;

    public setDisplayStyleMode(mode: string): void;

    public setDisplayStyleHeatmap(applied: string, midPoint: string): void;

    public setDisplayStylePercentBarShowValue(isShowValue: boolean): void;

    public setDisplayStylePercentBarIsUsedMovement(isUsedMovement: boolean): void;

    public setDisplayStyleConditional(conditionDefs: (PreviewTable.Conditional)[]|null): void;

    public setFormatType(type: string): void;

    public setUpdateStyleValueCell(isCellChangedStyle?: boolean|null): void;

    public setUpdateStyleValueText(isTextChangedStyle?: boolean|null): void;

    public setUpdateStyleValueCellBlinking(isBackgroundBlink?: boolean|null): void;

    public setValueFormatDecimalPlaces(decimalPlaces: number): void;

    public setValueFormatGeneral(): void;

    public setValueFormatShowPercentSignWithMultiply100(isPercentMultiply100: boolean): void;

    public setValueFormatScaled(scaled: string): void;

    public setValueFormatShowPlusSign(isShowPlusSign: boolean): void;

    public setValueFormatShowSeparatorSign(isUsedSeparator: boolean): void;

}

export default PreviewTable;
export { PreviewTable };
