import Ext from "../../../tr-grid-util/es6/Ext.js";
import EventDispatcher from "../grid/event/EventDispatcher.js";
import DataView from "./DataView.js"; // eslint-disable-line
import DataTable from "./DataTable.js"; // eslint-disable-line
import DataCache from "./DataCache.js"; // eslint-disable-line
import ColumnStats from "./ColumnStats.js"; // eslint-disable-line

declare namespace WrappedView {

    type Options = {
        dataView: DataView|null,
        wrapIndex: number
    };

}

declare class WrappedView extends EventDispatcher {

    constructor(options: WrappedView.Options|null);

    public dispose(): void;

    public getData(rid: string, cid: string): { [key: string]:  any };

    public getRowData(rid: string): { [key: string]:  any };

    public getColumnData(cid: string, inclNonDataRow?: boolean|null): any[];

    public getRowId(rowIndex: number): string;

    public getAllRowIds(shallowCopy?: boolean|null, inclNonDataRow?: boolean|null): (string)[];

    public getVisibleRowIds(shallowCopy?: boolean|null): (string)[];

    public getSortedRowIds(shallowCopy?: boolean|null): (string)[];

    public getRowIndex(rid: string): number;

    public getRowCount(): number;

    public getDataAt(rowIndex: number, cid: string): { [key: string]:  any };

    public getRowDataAt(rowIndex: number): { [key: string]:  any };

    public getPreviousData(rid: string, cid: string): { [key: string]:  any };

    public getMultipleRowData(rids: (string)[]|null, opt_from?: number|null, opt_to?: number|null): (any)[];

    public getAllRowData(): (any)[];

    public setData(rid: string, cid: string, value: any): boolean;

    public setRowData(rid: string, values?: { [key: string]:  any }|null, opt_eventArg?: any): boolean;

    public setColumnData(cid: string, valueList: any[]|null, opt_ridList?: (string)[]|null): void;

    public setDataAt(rowIndex: number, cid: string, value: any): boolean;

    public setRowDataAt(rowIndex: number, values?: { [key: string]:  any }|null, opt_eventArg?: any): boolean;

    public insertRow(rowRef: string|number|null, opt_values?: { [key: string]:  any }|null, opt_rid?: string|null|null): string;

    public removeRow(rowRef: number|string|null): void;

    public removeRows(rowRefs: (number|string)[]|null): void;

    public moveRow(fromRef: (number|string)[]|number|string|null, toRef: number|string|null): (string)[]|string|null;

    public getDataTable(): DataTable|null;

    public getDataSource(): DataView|null;

    public setDataSource(source?: DataView|null): void;

    public setConflationRate(milliSecond: number): void;

    public getConflationRate(): number;

    public sort(cid: string|(string)[]|null, opt_sortOrder?: string|number|(string|number)[]|null, sortLogics?: ((...params: any[]) => any)|(((...params: any[]) => any))[]|null, noOptimization?: boolean|null): void;

    public sortOnce(cid: string|(string)[]|null, opt_sortOrder?: string|number|(string|number)[]|null): void;

    public setSortingLogic(func: DataTable.SortLogic|null): void;

    public setColumnSortingLogic(cid: string, func: DataTable.SortLogic|null): void;

    public isSorting(): boolean;

    public hideRow(rId: string|number|null, opt_show?: boolean|null): void;

    public hideRows(rowRefs: (string|number)[]|null, opt_show?: boolean|null): void;

    public showRows(rowRefs: (string|number)[]|null, opt_show?: boolean|null): void;

    public unhideAllRows(): void;

    public hasHiddenRow(): boolean;

    public filterIn(cid: string|((...params: any[]) => any), value?: any, opt_filteringOut?: boolean|null): boolean;

    public filterOut(cid: string|((...params: any[]) => any), value?: any): boolean;

    public filterInOnce(cid: string|((...params: any[]) => any)|null, value?: any, opt_filteringOut?: boolean|null): void;

    public filterOutOnce(cid: string|((...params: any[]) => any), value?: any): void;

    public clearFilter(): void;

    public disablePageCorrection(opt_disable?: boolean|null): void;

    public setPage(pageIndex: number): boolean;

    public gotoPage(): void;

    public nextPage(): boolean;

    public previousPage(): boolean;

    public prevPage(): void;

    public lastPage(): boolean;

    public firstPage(): boolean;

    public getCurrentPage(): number;

    public getPageIndex(): void;

    public isOnLastPage(): boolean;

    public isOnFirstPage(): boolean;

    public getPageSize(): number;

    public getPageCount(): number;

    public getTotalPages(): void;

    public setPageSize(pageSize: number): void;

    public groupBy(cid: string|((...params: any[]) => any)|(string|((...params: any[]) => any))[]|null, opt_predefinedGroups?: any[]|null): void;

    public addGroup(value: any|any[]): void;

    public sortGroups(compare?: ((...params: any[]) => any)|null): void;

    public moveGroup(groupRef: number|string|null, destIndex: number): void;

    public setPredefinedGroups(predefinedGroups: any[]|null): void;

    public addFooter(opt_num?: number|null): void;

    public removeFooter(): void;

    public getGroupByRowId(rowId: string): DataView|null;

    public getGroup(groupRef: number|string|null): DataView|null;

    public getAllGroups(out_ary?: any[]|null): (DataView)[];

    public getGroupList(): (string)[]|null;

    public getGroupId(): string;

    public getGroupValue(): any;

    public getGroupRowId(): string;

    public hasGroup(): boolean;

    public getGroupRows(): null;

    public getGroupLevel(rowRef?: (number|string)|null): number;

    public enableColumnStats(cid: string, opt_enabled?: boolean|null, opt_refId?: string|null): ColumnStats|null;

    public getColumnStats(cid: string): ColumnStats|null;

    public hide(opt_hidden?: boolean|null): boolean;

    public isHidden(): boolean;

    public isHiddenRow(rowId: string): boolean;

    public collapse(opt_collapsed?: boolean|null): void;

    public collapseAllGroups(opt_collapsed?: boolean|null): void;

    public toggleCollapsedState(): void;

    public isCollapsed(): boolean;

    public enableContentAsHeader(bool?: boolean|null): void;

    public getVisibleRowCount(): number;

    public isRowDataInGroup(rowData: any, groupID?: string|null): boolean;

    public isRowIdInGroup(rowId: string, groupID?: string|null): boolean;

    public addRowExpansion(rid: string, count?: number|null): void;

    public removeRowExpansion(rid: string): void;

    public toggleRowExpansion(rowRef: string|number|null, force?: boolean|null, count?: number|null): void;

    public removeAllRowExpansions(): void;

    public getExpansionParents(from?: number|null, to?: number|null): (string)[]|null;

    public isRowExpansion(rid: string): boolean;

    public hasRowExpansion(rid: string): boolean;

    public getRowExpansionId(rid: string, idx?: number|null): string;

    public getRowExpansionIndex(rid: string): number;

    public searchAll(searchLogic: ((...params: any[]) => any)|null, opt_out?: (string)[]|null): (string)[];

    public searchPrev(rowRef: number|string|null, searchLogic: ((...params: any[]) => any)|null): number;

    public searchNext(rowRef: number|string|null, searchLogic: ((...params: any[]) => any)|null): number;

    public stall(opt_bool?: boolean|null): boolean;

    public enableAutoGroupRemoval(opt_bool?: boolean|null): boolean;

    public enableAutoGroupHiding(opt_bool?: boolean|null): boolean;

    public synchronizeRowOrder(): void;

    public setSegmentSeparator(rowRef: string|number|null, enabled?: boolean|null): boolean;

    public unsetAllSegmentSeparators(): boolean;

    public hasSegmentation(): boolean;

    public isSegmentSeparator(rid: string): boolean;

    public getSegmentParentRowId(rid: string): string;

    public collapseSegment(rowRef: string|number|null, collapsed?: boolean|null): boolean;

    public expandSegment(rowRef: string|number|null, expanded?: boolean|null): boolean;

    public expandAllSegments(): boolean;

    public isSegmentCollapsed(rowRef: string|number|null): boolean;

    public fillSegment(segmentRef: string|number|null): void;

    public fillSegments(): void;

    public addSegmentChild(segmentRef: string|number|null, rowRef: string|number|null): boolean;

    public addSegmentChildren(segmentRef: string|number|null, rowRefs: (string|number)[]|null): boolean;

    public removeSegmentChild(segmentRef: string|number|null, rowRef: string|number|null): boolean;

    public removeSegmentChildren(segmentRef: string|number|null, rowRefs: (string|number)[]|null): boolean;

    public removeAllSegmentChildren(): boolean;

    public getSegmentIds(): (string)[]|null;

    public getSegmentChildIds(segmentRef: string|number|null): (string)[]|null;

    public sortSegments(compare: ((...params: any[]) => any)|null): void;

    public getWrapSize(): number;

    public getWrappedViews(): (WrappedView)[]|null;

    public wrapView(wrapSize?: number|null): void;

    public dump(opt_options?: any): (any)[];

    public log(opt_options?: any): void;

    public isRowFiltered(rid: string, rowData: any): boolean;

}

declare function fromRids(): DataTable|null;

declare function complexData(opt_options?: any): void;

export default WrappedView;
export { WrappedView };
