import Ext from "../../../tr-grid-util/es6/Ext.js";
import GroupDefinitions from "../../../tr-grid-util/es6/GroupDefinitions.js"; // eslint-disable-line
import { isEmptyObject } from "../../../tr-grid-util/es6/Util.js";
import ElementWrapper from "./components/ElementWrapper.js";
import ILayoutGrid from "./ILayoutGrid.js"; // eslint-disable-line
import LayoutGrid from "./LayoutGrid.js";
import DataTable from "../data/DataTable.js";
import DataView from "../data/DataView.js";
import Util from "./util/util.js";
import SectionSettings from "./util/SectionSettings.js";
import Virtualizer from "./util/Virtualizer.js";
import Conflator from "./util/Conflator.js";
import TrackLayout from "./util/TrackLayout.js";
import HScrollbar from "./components/HScrollbar.js";
import VScrollbar from "./components/VScrollbar.js";
import VirtualizedLayoutGrid from "./VirtualizedLayoutGrid.js";

declare namespace Core {

    type SectionReference = number|string|ILayoutGrid|SectionSettings|null;

    type MouseInfo = {
        hit: boolean,
        x: number,
        y: number,
        colIndex: number,
        sectionIndex: number,
        rowIndex: number,
        sectionType: string,
        sectionSettings: SectionSettings|null,
        section: ILayoutGrid|null,
        cellElement: Element|null,
        dataSource: DataView|null
    };

    type BatchInfo = {
        reset?: string|null,
        insertion?: string|null,
        removal?: string|null,
        moving?: string|null
    };

    type CellReference = Core.MouseInfo|ElementWrapper|Element|null;

    type ColumnOptions = {
        width?: number|null,
        minWidth?: number|null,
        minSize?: number|null,
        size?: number|null,
        visible?: boolean|null,
        scalable?: boolean|null,
        styles?: { [key: string]:  string }|null,
        classes?: { [key: string]:  string }|null,
        dataColumnName?: string|null,
        renderingHandler?: ((...params: any[]) => any)|null,
        dataBindingHandler?: ((...params: any[]) => any)|null,
        columnData?: string|null,
        stationary?: boolean|null,
        leftPinned?: boolean|null,
        rightPinned?: boolean|null,
        layout?: TrackLayout|null
    };

}

declare class Core extends ElementWrapper {

    constructor(opt_initializer?: Element|null);

    public static getVersion(): string;

    public isDisposed(): boolean;

    public getConfigObject(gridOptions?: any): any;

    public getContentWidth(): number;

    public getSectionCount(): number;

    public addSection(opt_type?: string|null, opt_sectionName?: string|null): ILayoutGrid|null|null;

    public addSectionAt(at: number, opt_type?: string|null, opt_sectionName?: string|null): ILayoutGrid|null|null;

    public removeSection(sectionRef: Core.SectionReference|null): ILayoutGrid|null;

    public removeSectionAt(at: number): ILayoutGrid|null;

    public removeLastSection(): ILayoutGrid|null;

    public removeAllSections(): void;

    public setBottomPaddingSection(paddingSize: boolean|number|null): void;

    public getSection(sectionRef: Core.SectionReference|null): ILayoutGrid|null;

    public getSectionSettings(sectionRef: Core.SectionReference|null): SectionSettings|null;

    public getAllSections(opt_type?: string|null): (ILayoutGrid)[];

    public getAllSectionSettings(opt_type?: string|null): (SectionSettings)[];

    public getSectionIndex(identifier?: (MouseEvent|Core.SectionReference)|null): number;

    public getSectionTop(sectionRef: Core.SectionReference|null): number;

    public getPreviousSection(identifier?: (MouseEvent|Core.SectionReference)|null, opt_type?: string|null): ILayoutGrid|null;

    public getNextSection(identifier?: (MouseEvent|Core.SectionReference)|null, opt_type?: string|null): ILayoutGrid|null;

    public getLastSection(): ILayoutGrid|null;

    public getCell(sectionRef: Core.SectionReference|null, colIndex: number, rowIndex: number, opt_includeSpan?: boolean|null): ElementWrapper|null;

    public getCellInfo(cellRef: Core.CellReference|null): Core.MouseInfo|null;

    public getNextCellInRow(cellRef: Core.CellReference|null, step?: number|null): Core.MouseInfo|null;

    public getPrevCellInRow(cellRef: Core.CellReference|null, step?: number|null): Core.MouseInfo|null;

    public getNextCell(cellRef: Core.CellReference|null): Core.MouseInfo|null;

    public getPrevCell(cellRef: Core.CellReference|null): Core.MouseInfo|null;

    public getColumnCount(): number;

    public setColumnCount(num: number): void;

    public insertColumn(index: number, jsonObj?: Core.ColumnOptions|null): void;

    public insertColumns(at: number, cols?: (number|Core.ColumnOptions|(Core.ColumnOptions)[])|null): void;

    public removeColumnAt(index: number): any;

    public removeAllColumns(): void;

    public getStationaryColumnIndex(): number;

    public isColumnMoveAllowed(fromColIndex: number|(number)[]|null, toColIndex: number): boolean;

    public isColumnMovable(colIndex: number): boolean;

    public moveColumn(fromCol: number|(number)[]|null, destCol: number): boolean;

    public moveColumnById(srcCol: number|string|null, destCol?: (number|string)|null): boolean;

    public reorderColumns(colRefs: number|string|(number|string)[]|null, destCol?: (number|string)|null): boolean;

    public addRow(opt_num?: number|null): void;

    public removeRow(opt_num?: number|null): void;

    public getRowCount(): number;

    public setSectionRowCount(sectionRef: Core.SectionReference|null, num: number): void;

    public getRowIndex(sectionIndex: number, rowIndex: number): number;

    public removeAllRows(): void;

    public getDefaultRowHeight(): number;

    public setDefaultRowHeight(val: number, opt_includeTitle?: boolean|null): void;

    public getDataSource(): DataView|null;

    public setDataConflationRate(milliSecond: number): void;

    public getDataConflationRate(): number;

    public setDataSource(dataSource: DataTable|DataView|null): DataView|null;

    public getColumnRenderEvent(): void;

    public getColumnDataBindingEvent(c: number): any;

    public setColumnStyle(colIndex: number, style: string, value: string, opt_type?: string|null): void;

    public getColumnBackgroundColor(colIndex: number): string;

    public setColumnBackgroundColor(colIndex: number, color?: string|null): void;

    public enableColumnClass(colIndex: number, clsName: string, enabled?: boolean|null, opt_type?: string|null): boolean;

    public hasColumnClass(colIndex: number, clsName: string, opt_type?: string|null): boolean;

    public setColumnAlignment(colIndex: number, alignment?: string|null): void;

    public getColumnAlignment(colIndex: number): string;

    public getColumnLeft(colIndex: number): number;

    public getColumnRight(colIndex: number): number;

    public getColumnWidth(colIndex: number): number;

    public getColumnCustomLaneSize(colIndex: number): number;

    public setColumnWidth(colIndex: number, px: number, opt_scalable?: boolean|null): boolean;

    public setColumnWidths(columnWidths: (number)[]|null, opt_proportion?: (boolean|(boolean)[])|null): boolean;

    public isColumnVisible(colIndex: number): boolean;

    public hideColumn(colIndex: number, hidden?: boolean|null): void;

    public hideColumns(colIndices: (number)[]|null, hidden?: boolean|null): void;

    public showAllColumns(): void;

    public getMinimumColumnWidth(colIndex: number): number;

    public setMinimumColumnWidth(colIndex: number, width: number): void;

    public setMinimumColumnWidths(minWidths: (number|null)[]|null, proportion?: boolean|null): boolean;

    public getColumnScalability(colIndex: number): boolean;

    public setColumnScalability(val: number|boolean|null, opt_scalable?: boolean|null): boolean;

    public setDefaultColumnScalable(scalable: boolean): void;

    public resetColumnWidths(): boolean;

    public setColumnDefaultWidth(colIndex: number, width: number): void;

    public preserveProportion(preserve: boolean): void;

    public isProportionPreserved(): boolean;

    public preserveGridSize(preserve: boolean): void;

    public getColumnData(colIndex: number): any;

    public setAlwaysRenderColumn(colIndex: number, alwaysRender?: boolean|null): void;

    public getFitContentWidth(): boolean;

    public setFitContentWidth(bool: boolean): void;

    public loadPlugin(pluginRef: any, opt_options?: any): any;

    public unloadPlugin(pluginRef: any): any|null;

    public getPlugin(pluginName: string): any|null;

    public getPlugins(): any[];

    public getPluginList(): (string)[];

    public freezeSection(opt_frozenSection?: (string|number|null)|null): void;

    public resetSectionFreezing(): void;

    public freezeFooter(frozenCount: number): void;

    public getFooterCount(): number;

    public freezeColumn(frozenColIndex?: (number|null)|null, numRightColumn?: number|null): void;

    public isPinnedColumn(colIndex: number): boolean;

    public getFrozenColumnCount(): number;

    public getPinnedLeftColumnCount(): number;

    public getPinnedRightColumnCount(): number;

    public getFirstPinnedRightIndex(): number;

    public updateLayout(): void;

    public updateScrollbars(updateHeight: boolean, updateWidth: boolean): void;

    public freezeLayout(opt_bool?: boolean|null): boolean;

    public unfreezeLayout(opt_bool?: boolean|null): void;

    public isLayoutFrozen(): boolean;

    public toggleBorders(forcedVal?: boolean|null): void;

    public toggleGridlines(forcedVal?: boolean|null): void;

    public toggleVerticalLines(forcedVal?: boolean|null): void;

    public toggleHorizontalLines(forcedVal?: boolean|null): void;

    public toggleContentVerticalLines(forcedVal?: boolean|null): void;

    public getDataColumnMap(): (string)[]|null;

    public setDataColumnMap(opt_dataMap?: (string)[]|null): void;

    public getDataColumnName(colIndex: number): string;

    public setDataColumnName(colIndex: number, opt_dataColumnName?: (string|null)|null): void;

    public setDefaultRenderingHandler(handler: ((...params: any[]) => any)|null): void;

    public setDefaultDataBindingHandler(handler: ((...params: any[]) => any)|null): void;

    public setColumnRenderingHandler(c: number, func: any): void;

    public setColumnDataBindingHandler(c: number, func: any): void;

    public getColumnRenderingHandler(c: number): ((...params: any[]) => any)|null;

    public getColumnDataBindingHandler(c: number): ((...params: any[]) => any)|null;

    public enableRowHighlighting(opt_enabled?: boolean|null): void;

    public getRelativePosition(obj: ElementWrapper|Element|Event|MouseEvent, context?: any): Core.MouseInfo;

    public getVScrollbar(): ElementWrapper;

    public getHScrollbar(): ElementWrapper;

    public autoHideScrollbars(opt_hide?: boolean|null): void;

    public scrollToColumn(colIndex: number, leftOfView?: boolean|null): boolean;

    public getScrollLeft(): number;

    public setScrollLeft(pixels: number): void;

    public scrollRight(pixels: number): void;

    public getScrollWidth(): number;

    public getYScrollVal(sectionRef: Core.SectionReference|null, rowIndex: number, topOfView?: boolean|null): number|null|null;

    public scrollToRow(sectionRef: Core.SectionReference|null, rowIndex: number, topOfView?: boolean|null): void;

    public getVerticalViewInfo(): any;

    public getVScrollView(): any;

    public getScrollTop(): number;

    public setScrollTop(pixels: number): void;

    public scrollDown(pixels: number): void;

    public getScrollHeight(): number;

    public restoreScrollbars(): void;

    public getRowVirtualizer(): Virtualizer;

    public getColumnVirtualizer(): Virtualizer;

    public enableRowVirtualization(opt_enabled: boolean): void;

    public isRowVirtualizationEnabled(): boolean;

    public enableColumnVirtualization(opt_enabled?: boolean|null): void;

    public isColumnVirtualizationEnabled(): boolean;

    public setRowScrollingStep(rowCount: number): void;

    public getVMouseWheelHandler(): ((...params: any[]) => any)|null;

    public synchronizeVScrollbar(subGrid: Core|null): void;

    public synchronizeHScrollbar(subGrid: Core|null): void;

    public updateRowData(sectionRef?: Core.SectionReference|null, fromRowIndex?: number|null, lastRowIndex?: number|null, userParam?: any): void;

    public rerender(): void;

    public requestRowRefresh(): void;

    public deactivateRendering(disabled?: boolean|null): void;

    public reserveRightSpace(size: number): boolean;

    public focus(): void;

    public isBinding(): boolean;

    public hideRow(rowId: number | string|null, hidden?: boolean|null): void;

    public hideRows(rowIds: number | string | (number | string)[]|null, hidden?: boolean|null): void;

    public showRows(rowIds: number | string | (number | string)[]|null, shown?: boolean|null): void;

    public unhideAllRows(): void;

    public hasHiddenRow(): boolean;

    public selectColumn(colIndex: number, selected?: boolean|null): void;

    public isSelectedColumn(colIndex: number): boolean;

    public updateColumnBounds(): void;

    public getColumnRect(startColIndex: number, endColIndex: number): any;

    public getRowRect(startRowIndex: number, endRowIndex: number): any;

    public normalizeConfig(configObj: any): any;

    public setColumnGrouping(groupDefs: GroupDefinitions|null): void;

    public getColumnGroupParentId(colRef: string|number|null): string;

    public getColumnGroupChildIds(groupId: string): (string)[]|null;

    public getValidColumnList(colIds: (string)[]|null, columnMap?: any): (string)[];

    public createColumnMap(colRefs?: (string|number)[]|null): any;

    public startBatch(batchType: string): boolean;

    public stopBatch(batchType: string): boolean;

    public getColumnId(colIndex: number): string;

    public getColumnIds(): (string)[];

    public setColumnField(colIndex: number, field: string): void;

    public getColumnField(colIndex: number): string;

    public getColumnFields(): (string)[];

    public getColumnIndex(colRef: string|number|null): number;

    public getColumnIndices(colRefs: (number|string)[]|null): (number)[];

    public getColumnIndexMap(): { [key: string]:  number };

}

declare function num(opt_type?: string|null): (ILayoutGrid)[];

declare function mPos(sectionRef: Core.SectionReference|null): number;

declare function rowIndex(cellRef: Core.CellReference|null, step?: number|null): Core.MouseInfo|null;

declare function rowIndex(cellRef: Core.CellReference|null): Core.MouseInfo|null;

declare function section(cellRef: Core.CellReference|null): Core.MouseInfo|null;

declare function section(): number;

declare function path(obj: ElementWrapper|Element|Event|MouseEvent, context?: any): Core.MouseInfo;

declare function cellElement(): ElementWrapper;

declare function toR(): void;

export default Core;
export { Core };
