/* eslint-disable */
import ElementWrapper from "./components/ElementWrapper.js";
import Cell from "./components/Cell.js";
import TrackLayout from "./util/TrackLayout.js";

/**
* @interface
* @extends {ElementWrapper}
*/
let ILayoutGrid = function () {};

/** @public
* @param {number} x
* @param {number} y
* @param {boolean=} opt_omitSpan if true cell spans will be omitted from calculation
* @return {Object|null} Return object with "colIndex" and "rowIndex" property
*/
ILayoutGrid.prototype.hitTest = function (x, y, opt_omitSpan) { };
/** @public
* @param {number} x
* @return {number} Positive index if hit, otherwise negative
*/
ILayoutGrid.prototype.hitTestX = function (x) { };
/** @public
* @param {number} y
* @return {number} Positive index if hit, otherwise negative
*/
ILayoutGrid.prototype.hitTestY = function (y) { };

/** @public
* @return {number}
*/
ILayoutGrid.prototype.getDefaultColumnWidth = function () { };
/** @public
* @param {number} val
*/
ILayoutGrid.prototype.setDefaultColumnWidth = function (val) { };
/** @public
* @return {number}
*/
ILayoutGrid.prototype.getDefaultRowHeight = function () { };
/** @public
* @param {number} val
*/
ILayoutGrid.prototype.setDefaultRowHeight = function (val) { };
/** @public
* @param {number} colIndex
* @return {number}
*/
ILayoutGrid.prototype.getColumnWidth = function (colIndex) { };
/** @public
* @param {number} colIndex
* @param {number} val
* @return {boolean}
*/
ILayoutGrid.prototype.setColumnWidth = function (colIndex, val) { };
/** @public
* @param {number} rowIndex
* @return {number}
*/
ILayoutGrid.prototype.getRowHeight = function (rowIndex) { };
/** @public
* @param {number} rowIndex
* @param {number} val
* @return {boolean}
*/
ILayoutGrid.prototype.setRowHeight = function (rowIndex, val) { };
/** @public
* @param {number} colIndex
* @return {boolean}
*/
ILayoutGrid.prototype.isColumnVisible = function (colIndex) { };
/** @public
* @param {number} colIndex
* @param {boolean=} val
* @return {boolean}
*/
ILayoutGrid.prototype.hideColumn = function (colIndex, val) { };
/** @public
* @param {number} rowIndex
* @return {boolean}
*/
ILayoutGrid.prototype.isRowVisible = function (rowIndex) { };
/** @public
* @param {number} rowIndex
* @param {boolean=} val
* @return {boolean}
*/
ILayoutGrid.prototype.hideRow = function (rowIndex, val) { };
/** @public
* @param {number} colIndex
* @return {number}
*/
ILayoutGrid.prototype.getMinimumColumnWidth = function (colIndex) { };
/** @public
* @param {number} colIndex
* @param {number} val
* @return {boolean}
*/
ILayoutGrid.prototype.setMinimumColumnWidth = function (colIndex, val) { };
/** @public
* @param {number} rowIndex
* @return {number}
*/
ILayoutGrid.prototype.getMinimumRowHeight = function (rowIndex) { };
/** @public
* @param {number} rowIndex
* @param {number} val
* @return {boolean}
*/
ILayoutGrid.prototype.setMinimumRowHeight = function (rowIndex, val) { };

/** @public
* @param {number} colIndex
* @return {number}
*/
ILayoutGrid.prototype.getCellLeft = function (colIndex) { };
/** @public
* @param {number} rowIndex
* @return {number}
*/
ILayoutGrid.prototype.getCellTop = function (rowIndex) { };
/** @public
* @param {number} colIndex
* @param {number} rowIndex
* @return {number}
*/
ILayoutGrid.prototype.getCellWidth = function (colIndex, rowIndex) { };
/** @public
* @param {number} colIndex
* @param {number} rowIndex
* @return {number}
*/
ILayoutGrid.prototype.getCellHeight = function (colIndex, rowIndex) { };

/** @public
* @return {boolean}
*/
ILayoutGrid.prototype.hasCellSpan = function () { };
/** @description Returns zero or negative value, if the cell is spanned over (i.e. occupied by the spanning cell). <br>
* Negative value is distance from the spanning cell (i.e. -1 means one cell from the spanning cell and -2 means two cells from the spanning cell) <br>
* Return one, if the cell has no span (normal cell). <br>
* Return value greater than one, if the cell is spanning (getting expanded)
* @public
* @param {number} colIndex
* @param {number} rowIndex
* @return {number}
*/
ILayoutGrid.prototype.getCellRowSpan = function (colIndex, rowIndex) { };
/** @public
* @param {number} colIndex
* @param {number} rowIndex
* @param {number} rowSpan
* @return {boolean}
*/
ILayoutGrid.prototype.setCellRowSpan = function (colIndex, rowIndex, rowSpan) { };
/** @description Returns zero or negative value, if the cell is spanned over (i.e. occupied by the spanning cell). <br>
* Negative value is distance from the spanning cell (i.e. -1 means one cell from the spanning cell and -2 means two cells from the spanning cell) <br>
* Return one, if the cell has no span (normal cell). <br>
* Return value greater than one, if the cell is spanning (getting expanded)
* @public
* @param {number} colIndex
* @param {number} rowIndex
* @return {number}
*/
ILayoutGrid.prototype.getCellColSpan = function (colIndex, rowIndex) { };
/** @public
* @param {number} colIndex
* @param {number} rowIndex
* @param {number} colSpan
* @return {boolean}
*/
ILayoutGrid.prototype.setCellColSpan = function (colIndex, rowIndex, colSpan) { };
/** @public
* @return {boolean}
*/
ILayoutGrid.prototype.clearCellSpans = function () { };
/** @public
* @param {number} colIndex
* @return {boolean}
*/
ILayoutGrid.prototype.clearColumnSpans = function (colIndex) { };

/** @public
*/
ILayoutGrid.prototype.clearColumnCustomizations = function () { };
/** @public
*/
ILayoutGrid.prototype.clearRowCustomizations = function () { };

/** @public
* @param {number} colIndex
* @return {boolean}
*/
ILayoutGrid.prototype.getColumnScalability = function (colIndex) { };
/** @public
* @param {number} colIndex
* @param {boolean} val
* @return {boolean}
*/
ILayoutGrid.prototype.setColumnScalability = function (colIndex, val) { };
/** @public
* @param {number} rowIndex
* @return {boolean}
*/
ILayoutGrid.prototype.getRowScalability = function (rowIndex) { };
/** @public
* @param {number} rowIndex
* @param {boolean} val
* @return {boolean}
*/
ILayoutGrid.prototype.setRowScalability = function (rowIndex, val) { };

/** WARNING: Any modification to individual cell outside this class will not reflect the change as a whole. Please use with great care.
* @public
* @param {number} colIndex
* @param {number} rowIndex
* @param {boolean=} opt_includeSpan Default is false (cells are retrieved by their index)
* @return {ElementWrapper}
*/
ILayoutGrid.prototype.getCell = function (colIndex, rowIndex, opt_includeSpan) { };
/** @public
* @param {number} colIndex
* @param {number} rowIndex
* @return {Element|Node|NodeList}
*/
ILayoutGrid.prototype.getCellContent = function (colIndex, rowIndex) { };
/** Set data to the specified cell
* @public
* @param {number} colIndex
* @param {number} rowIndex
* @param {string|Element|ElementWrapper|*} data this can be HTML element, string, number, date, image
* @return {Element}
*/
ILayoutGrid.prototype.setCellContent = function (colIndex, rowIndex, data) { };

/** @public
* @param {number} index
* @return {ElementWrapper}
*/
ILayoutGrid.prototype.getColumn = function (index) { };
/** @public
* @return {number}
*/
ILayoutGrid.prototype.getColumnCount = function () { };
/** @public
* @param {number} val
*/
ILayoutGrid.prototype.setColumnCount = function (val) { };
/** @public
* @return {number}
*/
ILayoutGrid.prototype.getRowCount = function () { };
/** @public
* @param {number} val
* @param {boolean=} noBinding
*/
ILayoutGrid.prototype.setRowCount = function (val, noBinding) { };
/** @public
* @return {number}
*/
ILayoutGrid.prototype.getVisibleRowCount = function () { };
/** @public
* @param {number} val Number of row to be added from the last row
*/
ILayoutGrid.prototype.addRow = function (val) { };
/** @public
* @param {number} val Number of row to be removed from the last row
*/
ILayoutGrid.prototype.removeRow = function (val) { };

/** Deprecated. Shorthand method to set border visibility.
* @public
* @param {boolean} lft
* @param {boolean} top
* @param {boolean} rgt
* @param {boolean} btm
*/
ILayoutGrid.prototype.setBorders = function (lft, top, rgt, btm) { };

/** @public
* @param {number} colIndex
* @param {string} styleName
* @return {string|null} value
*/
ILayoutGrid.prototype.getColumnStyle = function (colIndex, styleName) { };

/** @public
* @param {number} colIndex
* @param {string} styleName
* @param {string} value
*/
ILayoutGrid.prototype.setColumnStyle = function (colIndex, styleName, value) { };

/** @public
* @param {number} colIndex
* @param {string} className
* @param {boolean=} enabled
*/
ILayoutGrid.prototype.enableColumnClass = function (colIndex, className, enabled) { };
/** @public
 * @return {boolean}
 */
ILayoutGrid.prototype.hasColumnClass = function (colIndex, className) { };
/** Apply CSS class to each cell in the row
* @public
* @param {number} rowIndex
* @param {string} className
* @param {boolean} enabled
* @see {@link ILayoutGrid.#setRowStyle}
*/
ILayoutGrid.prototype.enableRowClass = function (rowIndex, className, enabled) { };
/** Set CSS style to each cell in the row
* @public
* @param {number} rowIndex
* @param {string} styleName
* @param {string|null=} value
* @param {boolean=} enabled If set to false, the specified style will be removed instead
* @see {@link ILayoutGrid.#enableRowClass}
*/
ILayoutGrid.prototype.setRowStyle = function (rowIndex, styleName, value, enabled) { };

/** @public
* @param {!ElementWrapper|Element|Event|MouseEvent} obj ElementWrapper, element or mouse event whose position we're calculating.
* @return {!Object}
*/
ILayoutGrid.prototype.getRelativePosition = function (obj) { };

/** @public
* @return {number}
*/
ILayoutGrid.prototype.getContentWidth = function () { };
/** @public
* @return {number}
*/
ILayoutGrid.prototype.getContentHeight = function () { };
/** @public
* @param {number} rowIndex
* @return {Array.<ElementWrapper>} Return null for non existing row
* @see {@link ILayoutGrid.#setRowStyle}
* @see {@link ILayoutGrid.#enableRowClass}
*/
ILayoutGrid.prototype.getCellsByRowIndex = function(rowIndex) { };

/** @public
* @param {boolean=} opt_bool
*/
ILayoutGrid.prototype.disableHorizontalGridLine = function (opt_bool) { };
/** @public
* @param {boolean=} opt_bool
*/
ILayoutGrid.prototype.disableVerticalGridLine = function (opt_bool) { };

/** @public
* @param {boolean} enabled
*/
ILayoutGrid.prototype.enableShading = function (enabled) { };
/** @public
* @param {boolean} enabled
*/
ILayoutGrid.prototype.setTransposedShading = function (enabled) { };

/** When mouse hovering over a row, it will be highlighted
* @public
* @param {boolean=} opt_bool
*/
ILayoutGrid.prototype.enableRowHighlighting = function (opt_bool) { };
/** @public
* @return {number} rowIndex Negative number means no row is highlighted
*/
ILayoutGrid.prototype.getRowHighlight = function () { };
/** @public
* @param {number} rowIndex Negative number will cause deselection
*/
ILayoutGrid.prototype.setRowHighlight = function (rowIndex) { };
/** @public
* @return {Array.<number>}
*/
ILayoutGrid.prototype.getSelectedRows = function () { };
/** @public
* @return {number}
*/
ILayoutGrid.prototype.getSelectedRowCount = function () { };
/** @public
 * @param {number} at
 * @return {boolean}
 */
ILayoutGrid.prototype.isSelectedRow = function (at) { };
/** @public
 * @return {number}
 */
ILayoutGrid.prototype.getRowAnchor = function () { };
/** @public
 * @param {number} at
 */
ILayoutGrid.prototype.setRowAnchor = function (at) { };
/** @public
* @param {number} rowIndex
* @param {boolean=} opt_selected
*/
ILayoutGrid.prototype.setSelectedRow = function (rowIndex, opt_selected) { };

/** @public
 * @param {number} rowIndex
 */
ILayoutGrid.prototype.selectSingleRow = function (rowIndex) { };
/** @public
 * @param {number} rowIndex
 * * @param {number} length can be negative to select upward
 */
ILayoutGrid.prototype.selectRowRange = function (rowIndex, length) { };
/** @public
* @return {number} Number of selection removed
 */
ILayoutGrid.prototype.clearSelectedRows = function () { };
/** @public
 * @return {number}
 */
ILayoutGrid.prototype.getFirstSelectedRow = function () { };
/** @public
 * @return {number}
 */
ILayoutGrid.prototype.getLastSelectedRow = function () { };

/** @public
* @ignore
* @param {number} colIndex
* @param {boolean=} opt_bool
*/
ILayoutGrid.prototype.activateColumn = function (colIndex, opt_bool) { };
/** @public
* @param {number=} from INCLUSIVE
* @param {number=} to EXCLUSIVE
* @param {Array.<number>=} opt_updateRange
*/
ILayoutGrid.prototype.activateRows = function (from, to, opt_updateRange) { };
/** @public
* @ignore
* @param {number} numRows
*/
ILayoutGrid.prototype.moveRowsToTop = function (numRows) {};

/** @public
 * @return {number}
 */
ILayoutGrid.prototype.getFirstIndexInView = function () { };

/** @public
 * @return {number}
 */
ILayoutGrid.prototype.getLastIndexInView = function () { };

/** @public
 * @param {number} index
 * @return {boolean}
 */
ILayoutGrid.prototype.isInCurrentView = function (index) { };

/** @public
* @ignore
* @return {boolean}
*/
ILayoutGrid.prototype.getFrozenLayout = function () { };
/** @public
* @ignore
* @param {boolean} bool
*/
ILayoutGrid.prototype.setFrozenLayout = function (bool) { };

/** @public
* @ignore
* @param {number} index
* @param {Object|null=} opt_json
* @return {!ElementWrapper}
*/
ILayoutGrid.prototype.insertColumn = function (index, opt_json) { };

/** @public
* @ignore
* @param {number} index
* @return {Object|null}
*/
ILayoutGrid.prototype.removeColumnAt = function (index) { };

/** @public
* @ignore
* @param {number} from INCLUSIVE
* @param {number} to INCLUSIVE
*/
ILayoutGrid.prototype.moveColumn = function (from, to) { };

/** @public
* @ignore
* @param {boolean} bool
*/
ILayoutGrid.prototype.enableRowVirtualization = function (bool) { };
/** @public
* @ignore
* @param {boolean} bool
*/
ILayoutGrid.prototype.enableColumnVirtualization = function (bool) { };

/** @public
* @param {Element|Node} colElem
* @return {number}
*/
ILayoutGrid.prototype.getColumnIndex = function (colElem) { };
/** @public
* @param {number} colIndex
* @param {Element} cellElem
* @return {number}
*/
ILayoutGrid.prototype.getCellIndex = function (colIndex, cellElem) { };

/** @public
* @return {number}
*/
ILayoutGrid.prototype.getIndex = function () { };
/** @public
* @ignore
* @param {number} index
*/
ILayoutGrid.prototype.setIndex = function (index) { };
/** @public
* @return {number}
*/
ILayoutGrid.prototype.getRowOffset = function () { };
/** @public
* @ignore
* @param {number} index
*/
ILayoutGrid.prototype.setRowOffset = function (index) { };

/** @public
* @ignore
*/
ILayoutGrid.prototype.updateLayout = function () { };

/** This will make the specified cell horizontally span to fit entire row regardless of position of the cell -- cell will shift to the left most of the section.<br>
* If onlyToTheRight is true, the cell will not extend to the left most position<br>
* This is different from cell colSpan, which the cell remain in the same position. tr-stretched class will also be added to the cell
* @public
* @param {number|Cell} cellRef Either column index or cell reference is acceptable
* @param {number=} rowIndex If the cell instance is given as the first parameter, this parameter will be ignored.
* @param {boolean=} opt_stretching Default is true
* @param {boolean=} onlyToTheRight=false By default, cell will be stretched to both left and right directions, covering entire row. Set to true to expand only to the right
* @return {Cell} The cell being stretched
* @example
* grid.stretchCell(1, 0); // The second cell on the first row will be stretched covering the entire row
* grid.stretchCell(cellInstance, 0, true); // The given cell will be stretched covering the entire row
*/
ILayoutGrid.prototype.stretchCell = function (cellRef, rowIndex, opt_stretching, onlyToTheRight) {};
/**
* @public
* @param {number|Cell} cellRef
* @param {number=} rowIndex
* @return {Cell} The cell being un-stretched
* @see {@link ILayoutGrid#stretchCell}
*/
ILayoutGrid.prototype.unstretchCell = function (cellRef, rowIndex) {};
/**
* @public
*/
ILayoutGrid.prototype.unstretchAllCells = function () {};

/**
* @public
* @param {number|Cell} cellRef
* @param {number=} rowIndex
* @return {Cell}
*/
ILayoutGrid.prototype.getStretchedCell = function (cellRef, rowIndex) {};
/**
* @public
* @return {!Array.<Cell>} The cells that are stretched
* @see {@link ILayoutGrid#stretchCell}
*/
ILayoutGrid.prototype.getStretchedCells = function () {};

/**
* @public
* @see {@link ILayoutGrid#stretchCell}
* @return {boolean} Return true if there is any change
*/
ILayoutGrid.prototype.updateStretchedCells = function () {};

/** Get immediate parent of all child columns (i.e. base grid element)
* @public
* @return {Element}
*/
ILayoutGrid.prototype.getColumnHost = function () {};

/**
* @public
* @ignore
* @param {string} method
*/
ILayoutGrid.prototype.setHidingMethod = function (method) {};
/**
* @public
* @ignore
* @return {string}
*/
ILayoutGrid.prototype.getHidingMethod = function () {};
/**
* @public
* @ignore
* @param {number} colIndex
* @param {boolean} bool
* @param {number} flag Default is 0
* @return {boolean}
*/
ILayoutGrid.prototype.setColumnVisibility = function (colIndex, bool, flag) {};
/**
* @public
* @ignore
* @return {TrackLayout}
*/
ILayoutGrid.prototype.getRowLayout = function () {};

/** @public
* @param {number} size
* @return {Element}
*/
ILayoutGrid.prototype.reserveRightSpace = function (size) {};
/** @public
* @return {Element}
*/
ILayoutGrid.prototype.getReservedSpace = function () {};

/** @public
* @return {CSSStyleDeclaration} CSS live style object
* @see {@link https://developer.mozilla.org/en-US/docs/Web/API/Window/getComputedStyle}
*/
ILayoutGrid.prototype.getComputedStyle = function () {};

/** Column can be virtualized. Column that is in view will be active, while the one that is not in view will be inactive
* @public
* @param {number} colIndex
* @return {boolean}
*/
ILayoutGrid.prototype.isColumnActive = function (colIndex) { };

/**
* @public
* @param {number} rowIndex
* @param {function(number):Array.<{attribute:string, value:string, remove:boolean}>} func
*/
ILayoutGrid.prototype.setRowAttributes = function (rowIndex, func) {};

/** @public
* @param {number} rowIndex
* @return {Object}
*/
ILayoutGrid.prototype.getContextRow = function (rowIndex) {};

/** @public
* @param {number} colIndex
* @param {boolean=} selected
*/
ILayoutGrid.prototype.selectColumn = function (colIndex, selected) {};
/** @public
* @param {number} colIndex
* @return {boolean}
*/
ILayoutGrid.prototype.isSelectedColumn = function (colIndex) {};
/** @public
* @param {number} colIndex
* @param {number} rowIndex
* @param {boolean=} selected
*/
ILayoutGrid.prototype.selectCell = function (colIndex, rowIndex, selected) {};
/** @public
* @param {number} colIndex
* @param {number} rowIndex
* @param {number} width
* @param {number} height
*/
ILayoutGrid.prototype.setCellBounds = function (colIndex, rowIndex, width, height) {};
/** @public
* @ignore
* @return {!TrackLayout}
*/
ILayoutGrid.prototype.getHorizontalLayout = function () {};
/** @public
* @ignore
* @param {number} lftIdx
* @param {number} rgtIdx
* @param {!Array.<number>} outPositions
* @param {!Array.<boolean>} outNoBorders
*/
ILayoutGrid.prototype.calculateColumnBounds = function (lftIdx, rgtIdx, outPositions, outNoBorders) {};
/** @public
* @ignore
* @param {!Array.<Array>} posAry Left and right bound positions in pixel
* @param {!Array.<Array>} noBorderAry Boolean values indicating existence of left and right CSS borders
* @param {number=} topPx Top position of bound
*/
ILayoutGrid.prototype.updateColumnBounds = function (posAry, noBorderAry, topPx) {};
/** @public
* @ignore
*/
ILayoutGrid.prototype.updateColumnSeparators = function () {};

export default ILayoutGrid;
export { ILayoutGrid };
