/* eslint-disable */
import Ext from "../../../../tr-grid-util/es6/Ext.js";
import ElementWrapper from "./ElementWrapper.js";
/* eslint-enable */

/** @constructor
* @extends {ElementWrapper}
*/
let CellFloatingPanel = function () {
	this._items = [];
	this._element = null;
};
Ext.inherits(CellFloatingPanel, ElementWrapper);

/** @public
* @type {Array.<Element>}
*/
CellFloatingPanel.prototype._items = [];

/** @public
 * @returns {boolean}
*/
CellFloatingPanel.prototype.hasItem = function () {
	return !!this._items.length;
};
/** @public
 * @param {Element} icon Element of item
 * @returns {boolean}
*/
CellFloatingPanel.prototype.containItem = function (icon) {
	return this._items.indexOf(icon) > -1;
};
/** @public
* @param {Element} item
* @param {number=} opt_order The greater the order, the farther the position to the right
*/
CellFloatingPanel.prototype.insertItem = function (item, opt_order) {
	let items = this._items;
	if(items.indexOf(item) > -1) {
		return; // There is no need to insert the same icon
	}
	items.push(item);

	// Ordering icons
	if(opt_order) {
		item._order = opt_order;
	}
	if (items.length) {
		items.sort(CellFloatingPanel._iconComparer);
	}
	this._updatePanelItems();
};
/** @public
* @param {(Element|string)=} elemRef This can be the element or className (string) of the icon. If not specified, the last item will be remove.
* @return {Element|null} removed element
*/
CellFloatingPanel.prototype.removeItem = function (elemRef) {
	let items = this._items;
	let targetIndex = -1;
	if (!elemRef) {
		targetIndex = items.length - 1;
	} else {
		for(let i = items.length; --i >= 0;) {
			let ch = items[i];
			if (typeof elemRef === "string") {
				if(ch.classList.contains(elemRef)) {
					targetIndex = i;
					break;
				}
			} else {
				if (ch === elemRef) {
					targetIndex = i;
					break;
				}
			}
		}
	}
	let removedItem = null;
	if (targetIndex > -1) {
		removedItem = items.splice(targetIndex, 1)[0];
	}
	this._updatePanelItems();
	return removedItem;
};
/** @public
*/
CellFloatingPanel.prototype.clearItems = function () {
	let items = this._items;
	if (!items.length) {
		return;
	}
	items.length = 0;
	this._updatePanelItems();
};

/** @private
* @param {Element} elemA
* @param {Element} elemB
* @return {number}
*/
CellFloatingPanel._iconComparer = function(elemA, elemB) {
	if(!elemA._order) {
		return (!elemB._order) ? 0 : -1;
	} else if(!elemB._order) {
		return 1;
	}
	return (elemA._order - elemB._order);
};

/** @private
*/
CellFloatingPanel.prototype._updatePanelItems = function() {
	let items = this._items;
	let elem = this._element;
	if (!items.length) {
		return;
	}
	if (!this._element) {
		elem = this._element = document.createElement("div");
	}
	while (elem.lastChild) {
		elem.removeChild(elem.lastChild);
	}
	for (let i = 0; i < items.length; i++) {
		elem.appendChild(items[i]);
	}
};

export default CellFloatingPanel;
export { CellFloatingPanel };
