
/** @constructor
* @param {number} x
* @param {number} y
*/
let CellSpan = function (x, y) {
	this.indexX = x;
	this.indexY = y;
};

/** @public
 * @param {number} x
 * @param {number} y
* @return {boolean}
*/
CellSpan.prototype.isSpanIndices = function (x, y) {
	return x === this.indexX && y === this.indexY;
};
/** @public
 * @param {number} x
 * @param {number} y
* @return {boolean}
*/
CellSpan.prototype.isOccupiedIndices = function (x, y) {
	return x !== this.indexX || y !== this.indexY;
};

/** @public
* @param {!Array.<CellSpan>} allSpans
* @param {!Object.<string, CellSpan>} occupiedMap
*/
CellSpan.prototype.register = function (allSpans, occupiedMap) {
	this._register(allSpans, occupiedMap, true);
};
/** @public
* @param {!Array.<CellSpan>} allSpans
* @param {!Object.<string, CellSpan>} occupiedMap
*/
CellSpan.prototype.unregister = function (allSpans, occupiedMap) {
	this._register(allSpans, occupiedMap, false);
};
/** @private
* @param {!Array.<CellSpan>} allSpans
* @param {!Object.<string, CellSpan>} occupiedMap
* @param {boolean} bool
*/
CellSpan.prototype._register = function (allSpans, occupiedMap, bool) {
	let at = allSpans.indexOf(this);
	if(bool) {
		if(at < 0) {
			allSpans.push(this);
		}
	} else {
		if(at >= 0) {
			allSpans.splice(at, 1);
		}
	}

	let obj = bool ? this : null;
	let endCol = this.indexX + this.colSpan;
	let endRow = this.indexY + this.rowSpan;
	for (let c = this.indexX; c < endCol; ++c) {
		for (let r = this.indexY; r < endRow; ++r) {
			occupiedMap[c + "," + r] = obj;
		}
	}
};

/** @public
* @type {number}
*/
CellSpan.prototype.indexX;
/** @public
* @type {number}
*/
CellSpan.prototype.indexY;
/** @public
* @type {number}
*/
CellSpan.prototype.colSpan = 1;
/** @public
* @type {number}
*/
CellSpan.prototype.rowSpan = 1;
/** @public
* @type {number}
*/
CellSpan.prototype.prevColSpan = 1;
/** @public
* @type {number}
*/
CellSpan.prototype.prevRowSpan = 1;
/** @public
* @type {number}
*/
CellSpan.prototype.retVal = 0;

export default CellSpan;
export { CellSpan };
