/* eslint-disable */
import Ext from "../../../../tr-grid-util/es6/Ext.js";
import ElementWrapper from "../components/ElementWrapper.js";
import Scrollbar from "./Scrollbar.js";
/* eslint-enable */

/** @constructor
* @extends {Scrollbar}
*/
let HScrollbar = function () {
	let t = this; // Create a shorthand to shorthen file size
	t._vertical = false;
	t._onDeactivated = t._onDeactivated.bind(t);
	t._onActivated = t._onActivated.bind(t);
	t._hostClassName = "hscrollbar-host";
	HScrollbar.base(this, 'constructor'); // Call base constructor

	// Styling
	t._element.classList.add("grid-hscroll");
	t._trackContent.style.height = "1px";

	t.disableMouseWheel(false); // Mousewheel for horizontal scrollbar is enabled by default

	t.listen("activated", t._onActivated);
	t.listen("deactivated", t._onDeactivated);
};
Ext.inherits(HScrollbar, Scrollbar);

/** @type {number}
* @private
*/
HScrollbar.prototype._pinnedLeftColumnCount = 0;
/** @type {number}
* @private
*/
HScrollbar.prototype._pinnedRightColumnCount = 0;

/** For internal use only
* @public
* @ignore
* @param {ElementWrapper} grid Grid object is not used in this case
* @param {!Array.<ElementWrapper>} sections
* @param {number} startColIndex
* @param {number=} pinnedRightColumn Number of pinned columns on the right side
*/
HScrollbar.prototype.setScrollContent = function (grid, sections, startColIndex, pinnedRightColumn) {
	this._clearAllPanes();

	let sectionCount = sections.length;
	if(sectionCount <= 0) { return; }

	this._pinnedLeftColumnCount = startColIndex >= 0 ? startColIndex : 0;
	this._pinnedRightColumnCount = pinnedRightColumn > 0 ? pinnedRightColumn : 0;
	let section = sections[0];
	let colCount = section.getColumnCount();
	let endColIndex = colCount - pinnedRightColumn; // WARNING: If number of pinned column is greater than the scrollable range, the pinning will not be applied
	if(!endColIndex || endColIndex < 0) {
		endColIndex = colCount;
	}

	// Insert panes in the correct slot
	for (let s = 0; s < sectionCount; ++s) {
		section = sections[s];

		let pane = this._addPaneAt(s);
		let paneSlider = this._paneSliders[s];
		let paneElem = pane.getElement();
		let sliderElem = paneSlider.getElement();

		let c;
		let column;
		for (c = 0; c < startColIndex; ++c) {
			column = /** @type{Column} */(section.getColumn(c));
			column.setScrollState(paneElem, null, false);
		}
		let scrollHost = section.getColumnHost();
		pane.setParent(scrollHost);
		let columns = [];
		let content = [];
		for (c = startColIndex; c < endColIndex; ++c) {
			column = /** @type{Column} */(section.getColumn(c));
			column.setScrollState(null, sliderElem, this.isActive());

			columns.push(column);
			content.push(column.getElement());
		}
		for (c = endColIndex; c < colCount; ++c) {
			column = /** @type{Column} */(section.getColumn(c));
			column.setScrollState(null, null, false);
		}

		let rs = section.getReservedSpace();
		if(rs) {
			if(this.isActive() && !this._pinnedRightColumnCount) {
				paneSlider.addContent(rs);
				content.push(rs);
			} else {
				scrollHost.appendChild(rs);
			}
			section._updateRightSpaceStyle();
		}
		section._updateLayers(); // Order of the layers may be changed due to change in rightspace and/or column pinning
		paneSlider._titleSection = section.hasClass("title"); // To cache virtualized title section
		paneSlider._columns = columns; // To cache virtualized columns
		paneSlider._content = content; // To cache virtualized columns
	}
	this.invalidateContent(); // Delay content update
};

/** @override
*/
HScrollbar.prototype.setWidth = function (val) {
	this.setTrackSize(val, val);
	return true;
};
/** @override
*/
HScrollbar.prototype.getWidth = function () {
	return this.getPaneSize(); // Track size may not represent what the client want
};

/** @override */
HScrollbar.prototype.getScrollLeft = function () {
	return this._getScrollValue();
};
/** @public
* @param {number} px
*/
HScrollbar.prototype.setScrollLeft = function (px) {
	this.update();
	this._scrollTo(px); // "scroll" event may be fired
};

/** @public
* @return {number}
*/
HScrollbar.prototype.getPinnedLeftColumnCount = function () {
	return this.isEnabled() ? this._pinnedLeftColumnCount : 0;
};
/** @public
* @return {number}
*/
HScrollbar.prototype.getPinnedRightColumnCount = function () {
	return this.isEnabled() ? this._pinnedRightColumnCount : 0;
};

/** @public
* @ignore
* @param {number} val
*/
HScrollbar.prototype.setPaneLeft = function (val) {
	let px = val && this.isActive() ? (-val + "px") : "";
	for (let i = this._panes.length; --i >= 0; ) {
		this._paneSliders[i].setStyle("marginLeft", px);
	}
};

/** @public
* @override
* @return {boolean}
*/
HScrollbar.prototype.isEndOfHorizontalScroll = function () {
	return this._isEndOfScroll();
};
/** @public
* @return {number}
*/
HScrollbar.prototype.getContentWidth = function () {
	let paneSlider = this._paneSliders[0]; // Get content width from the first pane slider
	let sum = 0;
	if (paneSlider) {
		let colList = /** @type{Array} */(paneSlider._content);
		for (let i = colList.length; --i >= 0; ) {
			let elem = colList[i];
			let width = elem ? elem.style.width : 0; // HACK
			if(width) {
				sum += parseFloat(width); // TODO: parseFloat is not fast
			}
		}
	}
	return sum;
};
/** @override
* @protected
* @return {number}
*/
HScrollbar.prototype.calculateContentSize = function () {
	return this.getContentWidth();
};

/** Horizontal scrollbar will be shrunk if vertical scrollbar exists at the same time
* @override
* @public
* @return {number}
*/
HScrollbar.prototype.getTrackSize = function () {
	// Horizontal scrollbar may be overlap with vertical scrollbar
	if(this._footerCount <= 0) { // Horizontal scrollbar is NOT on top of a footer section and may be overlap
		if(this._otherScrollbar) { // The other scrollbar exists
			if(this._otherScrollbar.isActive() && !this._pinnedRightColumnCount) { // The other scrollbar is active and hence it is overlap with this scrollbar
				return this._trackSize - this.getTrackThickness();
			}
		}
	}
	return this._trackSize;
};

/** @private
* @param {Object} e
*/
HScrollbar.prototype._onActivated = function (e) {
	this._updateActivation(true);
};
/** @private
* @param {Object} e
*/
HScrollbar.prototype._onDeactivated = function (e) {
	this._updateActivation(false);
};
/** @private
* @param {boolean} bool
*/
HScrollbar.prototype._updateActivation = function (bool) {
	let sectCount = this._paneSliders.length;
	for(let s = 0; s < sectCount; ++s) {
		let paneSlider = this._paneSliders[s];
		let columns = paneSlider._columns;
		let colCount = columns ? columns.length : 0;
		for(let c = 0; c < colCount; ++c) {
			let column = columns[c];
			if(column.getElement()) { // Column may have been disposed
				column.enableAbsolutePosition(bool);
			}
		}
	}
};

HScrollbar._proto = HScrollbar.prototype;

export default HScrollbar;
export { HScrollbar };
