/* eslint-disable */
import Ext from "../../../../tr-grid-util/es6/Ext.js";
import ElementWrapper from "../components/ElementWrapper.js";
import Scrollbar from "./Scrollbar.js";
/* eslint-enable */

/** @constructor
* @extends {Scrollbar}
*/
let VScrollbar = function () {
	let t = this; // Create a shorthand to shorthen file size
	t._vertical = true;
	t._hostClassName = "vscrollbar-host";
	VScrollbar.base(this, 'constructor'); // Call base constructor

	t._onScroll = t._onScroll.bind(t);

	t._addPaneAt(); // Vertical scroll bar will have only one pane for multiple sections

	// Styling
	t._element.classList.add("grid-vscroll");
	t._trackContent.style.width = "1px";

	t.disableMouseWheel(false); // Mousewheel for vertical scrollbar is enabled by default

	t.listen("scroll", t._onScroll); // For backward compatability
	t._addEvent("contentScrolled");
};
Ext.inherits(VScrollbar, Scrollbar);


/** For internal usage only
* @public
* @ignore
* @param {ElementWrapper} grid
* @param {Array.<ElementWrapper>} sections
* @param {number} startingSection
*/
VScrollbar.prototype.setScrollContent = function (grid, sections, startingSection) {
	this._clearAllPanes();

	let sectionCount = sections.length;
	if(sectionCount <= 0) { return; }

	// Insert panes in the correct slot
	let section;
	let pane = this._panes[0]; // Vertical scrollbar has only one pane and one paneSlider
	let paneSlider = this._paneSliders[0];

	let s = 0;
	while(s < startingSection) {
		section = sections[s++];
		section.setParent(grid);
	}

	pane.setParent(grid);
	let dest = sectionCount - this._footerCount;

	while(s < dest) {
		section = sections[s++];
		section.setParent(paneSlider);
	}

	while(s < sectionCount) {
		section = sections[s++];
		section.setParent(grid);
	}
	this.invalidateContent(); // Delay content update
};

/** @override
*/
VScrollbar.prototype.setHeight = function (val) {
	this.setTrackSize(val, val);
	return true;
};
/** @override
*/
VScrollbar.prototype.getHeight = function () {
	return this.getPaneSize(); // Track size may not represent what the client want
};

/** @override */
VScrollbar.prototype.getScrollTop = function () {
	return this._getScrollValue();
};
/** @public
* @param {number} px
*/
VScrollbar.prototype.setScrollTop = function (px) {
	this.update();
	this._scrollTo(px); // "scroll" event may be fired
};

/** @public
* @override
* @return {boolean}
*/
VScrollbar.prototype.isEndOfVerticalScroll = function () {
	return this._isEndOfScroll();
};
/** @public
* @return {number}
*/
VScrollbar.prototype.getContentHeight = function () {
	let nodeList = this._paneSliders[0].getContent(true);
	let sum = 0;
	for (let i = nodeList.length; --i >= 0; ) {
		sum += nodeList[i].offsetHeight; // TODO: Slow
	}
	return sum;
};
/** @override
* @protected
* @return {number}
*/
VScrollbar.prototype.calculateContentSize = function () {
	return this.getContentHeight();
};

/** @public
* @ignore
* @return {ElementWrapper}
*/
VScrollbar.prototype.getPane = function () {
	return this._panes[0];
};

/** @private
* @param {!Object} e
*/
VScrollbar.prototype._onScroll = function (e) {
	this._dispatch("contentScrolled", e); // For backward compatibility
};

VScrollbar._proto = VScrollbar.prototype;

export default VScrollbar;
export { VScrollbar };
