import Ext from "../../../../tr-grid-util/es6/Ext.js";
import EventDispatcher from "../event/EventDispatcher.js";
import Core from "../Core.js";
import ILayoutGrid from "../ILayoutGrid.js";
import ElementWrapper from "../components/ElementWrapper.js";
import DataTable from "../../data/DataTable.js";
import { isEmptyObject } from "../../../../tr-grid-util/es6/Util.js";

declare namespace SortableTitlePlugin {

    type SortOrder = "a" | "ascending" | "d" | "descending" | "n" | null|null;

    type InitialSort = {
        colIndex?: number|null,
        colId?: string|null,
        field?: string|null,
        sortOrder?: SortableTitlePlugin.SortOrder|null,
        order?: SortableTitlePlugin.SortOrder|null
    };

    type SortingDefinition = {
        colIndex?: number|null,
        colId?: string|null,
        field?: string|null,
        sortOrder?: SortableTitlePlugin.SortOrder|null,
        order?: SortableTitlePlugin.SortOrder|null
    };

    type ColumnOptions = {
        sortable?: boolean|null,
        sortBy?: string|null,
        sort?: string|null,
        sortOrder?: string|null,
        sortLogic?: DataTable.SortLogic|null,
        sortingLogic?: DataTable.SortLogic|null,
        sorter?: DataTable.SortLogic|null,
        sortingSequence?: (SortableTitlePlugin.SortOrder)[]|null
    };

    type Options = {
        initialSort?: (SortableTitlePlugin.InitialSort|(SortableTitlePlugin.SortingDefinition)[])|null,
        multicolumn?: (boolean|number)|null,
        multiColumn?: (boolean|number)|null,
        threeStatesSorting?: boolean|null,
        sortLogic?: { [key: string]:  DataTable.SortLogic }|null,
        disableDoubleClickToSort?: boolean|null,
        sortableColumns?: boolean|null,
        sortableIndicator?: boolean|null,
        indicatorOnly?: boolean|null,
        mode?: string|null,
        disabled?: boolean|null,
        columnSorted?: ((...params: any[]) => any)|null,
        preClicked?: ((...params: any[]) => any)|null,
        clicked?: ((...params: any[]) => any)|null,
        preDataSorting?: ((...params: any[]) => any)|null,
        sortingSequence?: (SortableTitlePlugin.SortOrder)[]|null,
        rowSorting?: boolean|null
    };

}

declare class SortableTitlePlugin extends EventDispatcher {

    constructor(options?: SortableTitlePlugin.Options|null);

    public getName(): string;

    public initialize(host: Core, options?: any): void;

    public unload(host: Core): void;

    public afterInit(): void;

    public config(options?: any): void;

    public getConfigObject(gridOptions?: any): any;

    public getColumnMenu(colIndex: number, config: any): any;

    public getIcons(): { [key: string]:  string }|null;

    public setSortLogics(sortLogic: { [key: string]:  ((...params: any[]) => any) }|null): void;

    public setSortLogic(field: string, func: ((...params: any[]) => any)|null): void;

    public getSortOrder(colIndex?: number|null): string;

    public getSortedColumnIndex(priority?: number|null): number;

    public getSortPriority(colIndex: number, colRef?: string|null): number;

    public isColumnSorted(colIndex: number): boolean;

    public getSortedColumns(): (any)[]|null;

    public isSorting(): boolean;

    public sortColumn(colRef: number|string|null, sortOrder?: string|null, opt_arg?: any): void;

    public sortColumns(sortOptions: (SortableTitlePlugin.SortingDefinition)[]|null, opt_arg?: any): void;

    public clearSortState(opt_arg?: any): void;

    public refresh(): void;

    public restoreUserActions(): void;

    public setSortableColumnCount(count: number): void;

    public getSortingStates(): (any)[]|null;

    public updateSortSymbols(): void;

    public getDataColumnMap(): (string)[];

    public setDataColumnMap(dataMap: (string|null)[]|null): void;

    public getColumnSortingField(colIndex: number): string;

    public setColumnSortingField(colIndex: number, field: string|null|null, sortLogic?: ((...params: any[]) => any)|null, rowSorting?: boolean|null): void;

    public getColumnSortingFields(): (string)[];

    public setFirstSortOrder(colIdentifier: string|number|(string|number)[]|null, sortOrder?: string|null): void;

    public disableTwoStateSorting(disabled?: boolean|null): void;

    public freezeIndicator(bool?: boolean|null): void;

    public disableSortSymbols(disabled?: boolean|null): void;

    public disableDataSorting(disabled?: boolean|null): void;

    public disableDoubleClickToSort(disabled?: boolean|null): void;

    public disableCellClick(cell: ElementWrapper|null, disabled?: boolean|null): void;

    public disableColumnSorting(columns: number|(number)[]|null, disabled?: boolean|null): void;

    public setClickDelayInterval(opt_ms?: number|null): void;

    public getSortingSequence(colRef?: (number|string)|null): (string)[];

    public setSortingSequence(sequence: (string)[]|null, colRef?: (number|string)|null): void;

    public clearAllColumnSortingSequences(): void;

    public sortSeparators(comparer: ((...params: any[]) => any)|null): void;

}

declare function len(disabled?: boolean|null): void;

export default SortableTitlePlugin;
export { SortableTitlePlugin };
