
/** @description Reverter is used executed the given method after the specified delay. If Reverter is activated the second time within the delay, the delay time will be extended.
* @constructor
* @param {Function|number=} opt_func
* @param {number|Function=} opt_ms The delay time in millisecond before executing the function
*/
let Reverter = function (opt_func, opt_ms) {
	this._onInterval = this._onInterval.bind(this);

	if(typeof opt_func == "function") {
		this._func = opt_func;
	} else if (typeof opt_ms == "function") { // Parameter can be swapped places
		this._func = opt_ms;
	}

	if(typeof opt_ms == "number") {
		this._delay = opt_ms;
	} else if (typeof opt_func == "number") { // Parameter can be swapped places
		this._delay = opt_func;
	}
};

/** @type {Function}
* @private
*/
Reverter.prototype._func = null;
/** @type {number}
* @private
*/
Reverter.prototype._delay = 0;
/** @type {number}
* @private
*/
Reverter.prototype._timerId = 0;
/** @type {boolean}
* @private
*/
Reverter.prototype._pending = false;


/** @public
*/
Reverter.prototype.start = function () {
	if(this._func) {
		if(!this._timerId) {
			this._timerId = setInterval(this._onInterval, this._delay);
		} else {
			this._pending = true;
		}
	}
};

/** @public
*/
Reverter.prototype.dispose = function () {
	if(this._timerId) {
		clearInterval(this._timerId);
		this._timerId = 0;
	}
	this._func = null; // Release user bound function
};


/** @private
*/
Reverter.prototype._onInterval = function () {
	if(this._pending) {
		this._pending = false;
	} else {
		this._func();
		clearInterval(this._timerId);
		this._timerId = 0;
	}
};

Reverter._proto = Reverter.prototype;

export default Reverter;
export { Reverter };
