import FormatterBuilder from "./FormatterBuilder.js";

/** @typedef {Object} CoralRadioButtonFormatter~Options
* @description FormatterBuilder options
* @property {number=} initialIndex
*/

/** @private
* @function
* @param {Element} element
* @param {Object} ctx
*/
let onElementCreated = function (element, ctx) {
	let dv = ctx.grid.getDataSource();
	let rid = dv.getRowId(ctx.rowIndex);
	let index = dv.getRowIndex(rid);
	let checked = index === ctx.initialIndex;
	if (checked) {
		ctx.previousId = rid;
		ctx.setData(ctx.field, checked);
	}
	ctx.value = checked;
};

let onElementUpdated = function (element, ctx) {
	let dv = ctx.grid.getDataSource();
	let rid = dv.getRowId(ctx.rowIndex);
	let checked = (rid === ctx.previousId) && ctx.value;
	element.checked = checked;
};

/** To bound data and view of the current focus radio button
* @private
* @param {*} e
* @param {*} ctx
*/
let _changeHandler = function (e, ctx) {
	ctx.value = e.detail.value;
	ctx.setData(ctx.field, ctx.value);

	let dv = ctx.grid.getDataSource();
	let rid = dv.getRowId(ctx.rowIndex);
	if (ctx.previousId && ctx.previousId !== rid) {
		let row = dv.getRowData(ctx.previousId);
		if (row) {
			if (row["ROW_DEF"]) {
				row["ROW_DEF"].setData(ctx.field, false);
			} else {
				dv.setData(ctx.previousId, ctx.field, false);
			}
		}
	}
	ctx.previousId = dv.getRowId(ctx.rowIndex);
};

/** @constructor
* @param {*=} options
*/
let CoralRadioButtonFormatter = function (options) {

};

/** @public
* @param {(FormatterBuilder.Options|CoralRadioButtonFormatter.Options|Object)=} options
* @return {Object}
*/
CoralRadioButtonFormatter.create = function (options) {
	let defaultOpt = {
		tagName: "ef-radio-button",
		refName: "CoralRadioButtonFormatter",
		styles: {
			margin: 0
		},
		onElementCreated: onElementCreated,
		onElementUpdated: onElementUpdated,
		changeHook: "checked-changed",
		changeHandler: _changeHandler,
		initialIndex: -1 // does not checked by default
	};

	return FormatterBuilder.create(options, defaultOpt);
};

export default CoralRadioButtonFormatter;
export { CoralRadioButtonFormatter };
