import FormatterBuilder from "./FormatterBuilder.js";
import ElfDate from "../../tr-grid-util/es6/ElfDate.js";
import ElfUtil from "../../tr-grid-util/es6/ElfUtil.js";

let onDuplexContextCreated = function (ctx, options) {
	ctx.ElfDate = ElfDate;
	ctx.fromField = options.from;
	ctx.toField = options.to;
};

let _onElementUpdated = function (element, ctx) {
	let from = ctx.ElfDate.from(ctx.getData(ctx.fromField));
	let to = ctx.ElfDate.from(ctx.getData(ctx.toField));

	if(!ctx.ElfDate.equal(element.from, from)) {
		element.from = from;
	}
	if(!ctx.ElfDate.equal(element.to, to)) {
		element.to = to;
	}
};

let _onElfV4ElementUpdated = function (element, ctx) {
	let from = ctx.ElfDate.from(ctx.getData(ctx.fromField));
	let to = ctx.ElfDate.from(ctx.getData(ctx.toField));

	element.values = [ctx.ElfDate.toYMD(from), ctx.ElfDate.toYMD(to)];
};

let _onFieldFromBound = function (e, ctx) {
	let from = ctx.ElfDate.from(e.currentTarget.from);
	let currentData = ctx.getData(ctx.fromField);
	if (!ctx.ElfDate.equal(currentData, from)) {
		ctx.setData(ctx.fromField, from);
	}
};

let _onFieldToBound = function (e, ctx) {
	let to = ctx.ElfDate.from(e.currentTarget.to);
	if (to == null) {
		return;
	}
	let currentData = ctx.getData(ctx.toField);
	if (!ctx.ElfDate.equal(currentData, to)) {
		ctx.setData(ctx.toField, to);
	}
};

/** @private
* @param {Object} e
* @param {Object} ctx
*/
let _onElfV4ValueChanged = function (e, ctx) {
	let dates = e.currentTarget.values;
	let from = null;
	let to = null;
	if(typeof dates === "string") {
		dates = dates.split(",");
	}
	if(Array.isArray(dates)) {
		from = ctx.ElfDate.from(dates[0]);
		to = ctx.ElfDate.from(dates[1]);
	}
	if(!from || !to) {
		return;
	}

	let currentData = ctx.getData(ctx.fromField);
	if (!ctx.ElfDate.equal(currentData, from)) {
		ctx.setData(ctx.fromField, from);
	}

	currentData = ctx.getData(ctx.toField);
	if (!ctx.ElfDate.equal(currentData, to)) {
		ctx.setData(ctx.toField, to);
	}
};

/** @constructor
* @param {*=} options
*/
let DuplexEmeraldDateTimePickerFormatter = function (options) {

};

/** @public
* @param {(FormatterBuilder.Options|Object)=} options
* @return {Object}
*/
DuplexEmeraldDateTimePickerFormatter.create = function (options) {
	let defaultOpt = {
		tagName: "ef-datetime-picker",
		refName: "DuplexEmeraldDateTimePickerFormatter",
		onContextCreated: onDuplexContextCreated,
		onElementUpdated: _onElementUpdated,
		attributes: {
			range: "",
			duplex: ""
		},
		styles: {
			margin: "0"
		}
	};

	if(ElfUtil.getElfVersion() >= 4) {
		defaultOpt.onElementUpdated = _onElfV4ElementUpdated;
		defaultOpt.changeHook = "value-changed";
		defaultOpt.changeHandler = _onElfV4ValueChanged;
	} else {
		defaultOpt.changeHooks = ["from-changed", "to-changed"];
		defaultOpt.changeHandlers = [_onFieldFromBound, _onFieldToBound];
	}

	return FormatterBuilder.create(options, defaultOpt);
};


export default DuplexEmeraldDateTimePickerFormatter;
export { DuplexEmeraldDateTimePickerFormatter };
