import FormatterBuilder from "./FormatterBuilder.js";
import ElfDate from "../../tr-grid-util/es6/ElfDate.js";

/** Due to stamping techinue is used in the FormatterBuilder, we have to store the constructor in context object
* @private
* @param {Object} ctx Context object
* @param {Object} options User options
*/
let onContextCreated = function (ctx, options) {
	ctx.ElfDate = ElfDate;
};

/** @private
* @function
* @param {Element} element
* @param {Object} ctx
*/
let onElementUpdated = function (element, ctx) {
	if (!ctx.ElfDate.equal(element.value, ctx.value)) {
		ctx.ElfDate.setDate(element, ctx.value);
	}
};

let _changeHandler = function (e, ctx) {
	ctx.value = ctx.ElfDate.getDate(e.detail);
	let currentData = ctx.getData(ctx.field);
	if (!ctx.ElfDate.equal(currentData, ctx.value)) {
		ctx.setData(ctx.field, ctx.value);
	}
};

/** @constructor
* @param {*=} options
*/
let EFDateTimePickerFormatter = function (options) {

};

/** @public
* @param {(FormatterBuilder.Options|Object)=} options
* @return {Object}
*/
EFDateTimePickerFormatter.create = function (options) {
	let defaultOpt = {
		tagName: "ef-datetime-picker",
		refName: "EFDateTimePickerFormatter",
		onContextCreated: onContextCreated,
		onElementUpdated: onElementUpdated,
		changeHook: "value-changed",
		changeHandler: _changeHandler,
		styles: {
			margin: "0"
		}
	};

	return FormatterBuilder.create(options, defaultOpt);
};

export default EFDateTimePickerFormatter;
export { EFDateTimePickerFormatter };
