import FormatterBuilder from "./FormatterBuilder.js";

/** @typedef {Object} EFIconFormatter~Options
* @description FormatterBuilder options
* @property {(string|Object)=} icon
* @property {(number|string)=} size
*/

/** @private
* @function
* @param {Element} element
* @param {Object} ctx
*/
let onElementUpdated = function (element, ctx) {
	let icon = ctx.value;
	if (ctx.icon != null && typeof ctx.icon === "object") {
		icon = ctx.icon[ctx.value];
	}
	if (typeof ctx.icon === "string") {
		icon = ctx.icon;
	}
	element.icon = icon;
	element.size = ctx.size;
};

/** @constructor
* @param {*=} options
*/
let EFIconFormatter = function (options) {

};

/** @public
* @param {(FormatterBuilder.Options|EFIconFormatter.Options|Object)=} options
* @return {Object}
*/
EFIconFormatter.create = function (options) {
	let defaultOpt = {
		tagName: "ef-icon",
		refName: "EFIconFormatter",
		onElementUpdated: onElementUpdated,
		icon: null, // use icon from ctx.value
		size: 15
	};

	return FormatterBuilder.create(options, defaultOpt);
};

export default EFIconFormatter;
export { EFIconFormatter };
