import FormatterBuilder from "./FormatterBuilder.js";

/** @typedef {Object} EFRadioButtonFormatter~Options
* @description FormatterBuilder options
* @property {number=} initialIndex
*/

/** @private
* @function
* @param {number|string} rowRef
* @param {boolean=} checked
* @returns {boolean} Returns true if there is a change
*/
let _selectRadio = function (rowRef, checked) {
	if(!this.grid) {
		return false;
	}
	let dv = this.grid.getDataSource();
	if(!dv) {
		return false;
	}

	let rowIndex = -1;
	let rid = "";
	if(typeof rowRef == "string") {
		rid = rowRef;
		rowIndex = dv.getRowIndex(rid);
	} else {
		rowIndex = rowRef;
		rid = dv.getRowId(rowIndex);
	}
	if(!(rowIndex >= 0) || !rid) {
		return false;
	}

	let dirty = false;
	if (checked !== false) {
		if (this.previousId && this.previousId !== rid) {
			this.rowIndex = dv.getRowIndex(this.previousId);
			this.setData(this.field, false);
		}
		this.rowIndex = rowIndex;
		this.previousId = rid;
		this.setData(this.field, true);
		this.value = checked;
		dirty = true;
	} else if(this.previousId == rid) {
		this.rowIndex = rowIndex;
		this.previousId = "";
		this.setData(this.field, false);
		this.value = false;
		dirty = true;
	}
	return dirty;
};

/** @private
* @function
* @param {Element} element
* @param {Object} ctx
*/
let onElementCreated = function (element, ctx) {
	let checked = false;
	let rid = "";
	if(ctx.initialIndex >= 0) {
		let dv = ctx.grid.getDataSource();
		rid = dv.getRowId(ctx.rowIndex);
		if(!ctx.initialRid) {
			ctx.initialRid = dv.getRowId(ctx.initialIndex);
		}
		checked = (rid === ctx.initialRid);
	}
	if (checked) {
		ctx.previousId = rid;
		ctx.setData(ctx.field, checked);
		ctx.value = checked;
	}
};

/** @private
* @function
* @param {Element} element
* @param {Object} ctx
*/
let onElementUpdated = function (element, ctx) {
	let checked = ctx.value ? true : false;
	if(checked) {
		checked = (ctx.grid.getDataSource().getRowId(ctx.rowIndex) === ctx.previousId);
	}
	element.checked = checked;
};

/** To bound data and view of the current focus radio button
* @private
* @param {*} e
* @param {*} ctx
*/
let _changeHandler = function (e, ctx) {
	ctx.selectRadio(ctx.rowIndex, true);
};

/** @constructor
* @param {*=} options
*/
let EFRadioButtonFormatter = function (options) {

};

/** @public
* @param {(FormatterBuilder.Options|EFRadioButtonFormatter.Options|Object)=} options
* @return {Object}
*/
EFRadioButtonFormatter.create = function (options) {
	let defaultOpt = {
		tagName: "ef-radio-button",
		refName: "EFRadioButtonFormatter",
		styles: {
			margin: 0
		},
		selectRadio: _selectRadio,
		onElementCreated: onElementCreated,
		onElementUpdated: onElementUpdated,
		changeHook: "checked-changed",
		changeHandler: _changeHandler,
		initialIndex: -1 // does not checked by default
	};

	return FormatterBuilder.create(options, defaultOpt);
};

export default EFRadioButtonFormatter;
export { EFRadioButtonFormatter };
