import FormatterBuilder from "./FormatterBuilder.js";
import CoralItems from "../../tr-grid-util/es6/CoralItems.js";

/** @typedef {Object} EFSelectFormatter~Options
* @description EFSelectFormatter options
* @property {Array=} data Item list to be displayed on the coral select. The given object should contains at least value properties. If string is given, it will be treated as the value.
* @property {Array=} entries Alias to `data` property
* @property {Array=} items Alias to `data` property
*/

/** @private
* @param {Object} ctx Context object
* @param {Object} options User options
*/
let onContextCreated = function (ctx, options) {
	let userItems = options.data || options.entries || options.items;
	ctx.data = CoralItems.create(userItems);
};

/** @private
* @function
* @param {Element} element
* @param {Object} ctx
*/
let onElementCreated = function (element, ctx) {
	if (ctx.data) {
		element.data = ctx.data;
	}
};

/** @private
* @function
* @param {Element} element
* @param {Object} ctx
*/
let onElementUpdated = function (element, ctx) {
	element.value = ctx.value;
};

let _changeHandler = function (e, ctx) {
	ctx.value = e.detail.value;
	ctx.setData(ctx.field, ctx.value);
};

/** @constructor
* @param {*=} options
*/
let EFSelectFormatter = function (options) {

};

/** @public
* @param {(FormatterBuilder.Options|EFSelectFormatter.Options|Object)=} options
* @return {Object}
*/
EFSelectFormatter.create = function (options) {
	let defaultOpt = {
		tagName: "ef-select",
		refName: "EFSelectFormatter",
		styles: {
			maxWidth: "100%",
			margin: "0"
		},
		onContextCreated: onContextCreated,
		onElementCreated: onElementCreated,
		onElementUpdated: onElementUpdated,
		changeHook: "value-changed",
		changeHandler: _changeHandler,
		data: null
	};

	return FormatterBuilder.create(options, defaultOpt);
};

export default EFSelectFormatter;
export { EFSelectFormatter };
