import FormatterBuilder from "./FormatterBuilder.js";
import ElfUtil from "../../tr-grid-util/es6/ElfUtil.js";

let onElementUpdated = function (element, ctx) {
	element.value = ctx.value;
};

let onElementCreated = function (element, ctx) {
	element.setAttribute("type", "number");
};

let _changeHandler = function (e, ctx) {
	ctx.value = e.detail.value;
	ctx.setData(ctx.field, ctx.value);
};

let _inputChangeHandler = function (e, ctx) {
	ctx.value = +e.currentTarget.value;
	ctx.setData(ctx.field, ctx.value);
};

/** @constructor
* @param {*=} options
*/
let NumericInputFormatter = function (options) {

};

/** @public
* @param {(FormatterBuilder.Options|Object)=} options
* @return {Object}
*/
NumericInputFormatter.create = function (options) {
	let tn = "input";
	if(ElfUtil.hasComponent("ef-number-field")) {
		tn = "ef-number-field";
	} else if(ElfUtil.hasComponent("ef-input")) {
		tn = "ef-input";
	}

	let defaultOpt = {
		tagName: tn,
		refName: "NumericInputFormatter",
		onElementUpdated: onElementUpdated,
		changeHook: "value-changed",
		changeHandler: _changeHandler,
		styles: {
			margin: "0",
			width: "100%"
		}
	};
	if(tn !== "ef-number-field") {
		defaultOpt.onElementCreated = onElementCreated;
		if(tn === "input") {
			defaultOpt.changeHook = "change";
			defaultOpt.changeHandler = _inputChangeHandler;
		}
	}

	return FormatterBuilder.create(options, defaultOpt);
};

export default NumericInputFormatter;
export { NumericInputFormatter };
