import FormatterBuilder from "./FormatterBuilder.js";
import {PercentBar} from "../../tr-grid-util/es6/PercentBar.js";

/** @typedef {Object} PercentBarFormatter~Options
* @description PercentBarFormatter options
* @property {string=} alignment="left" Possible values are l, r, c, left, right, and center
* @property {boolean=} movementColor=true If disabled, percent bar will not use up and down colors
* @property {string=} barColor="" Custom color for bar element. This will override the movementColor
* @property {boolean=} textHidden=false If enabled, text label will not be shown
* @property {(number|string)=} textWidth=56 Number of pixel or percentage width
*/

/** Due to stamping technique is used in the FormatterBuilder, we have to store the constructor in context object
* @private
* @param {Object} ctx Context object
* @param {Object} options User options
*/
let onContextCreated = function (ctx, options) {
	ctx.PercentBar = PercentBar;
};

/** @private
* @param {Element} element
* @param {Object} ctx Context object
*/
let onElementCreated = function (element, ctx) {
	element["_percentBar"] = new ctx.PercentBar(element, ctx);
};

/** @private
* @param {Element} element
* @param {Object} ctx Context object
*/
let onElementUpdated = function (element, ctx) {
	element["_percentBar"].setValue(ctx.value);
};

/** @constructor
* @param {*=} options
*/
let PercentBarFormatter = function (options) {

};

/** @public
* @param {(FormatterBuilder.Options|PercentBarFormatter.Options|Object)=} options
* @return {Object}
*/
PercentBarFormatter.create = function (options) {
	let defaultOpt = {
		tagName: "div",
		refName: "PercentBarFormatter",
		onContextCreated: onContextCreated,
		onElementCreated: onElementCreated,
		onElementUpdated: onElementUpdated
	};

	return FormatterBuilder.create(options, defaultOpt);
};

export default PercentBarFormatter;
export { PercentBarFormatter };
