import FormatterBuilder from "./FormatterBuilder.js";

/** @typedef {Object} SimpleImageFormatter~Options
* @description FormatterBuilder options
* @property {(string|number)=} height
* @property {(string|number)=} width
*/

/** @private
* @function
* @param {Element} element
* @param {Object} ctx
*/
let onElementUpdated = function (element, ctx) {
	element.src = ctx.value;
};

/** @constructor
* @param {*=} options
*/
let SimpleImageFormatter = function (options) {

};

/** @public
* @param {(FormatterBuilder.Options|SimpleImageFormatter.Options|Object)=} options
* @return {Object}
*/
SimpleImageFormatter.create = function (options) {
	let defaultOpt = {
		tagName: "img",
		refName: "SimpleImageFormatter",
		styles: {
			position: "absolute",
			maxHeight: "100%",
			top: 0,
			bottom: 0,
			left: 0,
			right: 0,
			margin: "auto"
		},
		onElementUpdated: onElementUpdated,
		height: "auto",
		width: "auto"
	};

	defaultOpt.attributes = {
		height: defaultOpt.height,
		width: defaultOpt.width
	};
	if (options) {
		if (options.height != null) {
			defaultOpt.attributes.height = options.height;
		}
		if (options.width != null) {
			defaultOpt.attributes.width = options.width;
		}
	}

	return FormatterBuilder.create(options, defaultOpt);
};

export default SimpleImageFormatter;
export { SimpleImageFormatter };
