import FormatterBuilder from "./FormatterBuilder.js";

/** @typedef {Object} SimpleLinkFormatter~Options
* @description FormatterBuilder options
* @property {string=} label
* @property {string=} urlField Field used for href attributed
*/

/** @private
* @param {Element} element
* @param {Object} ctx Context object
*/
let onElementUpdated = function (element, ctx) {
	let label = typeof ctx.label === "string" ? ctx.label : ctx.value;
	element.textContent = label;
	if(ctx.urlField) {
		element.setAttribute("href", ctx.getData(ctx.urlField));
	}
};

/** @constructor
* @param {*=} options
*/
let SimpleLinkFormatter = function (options) {

};

/** @public
* @param {(FormatterBuilder.Options|SimpleLinkFormatter.Options|Object)=} options
* @return {Object}
*/
SimpleLinkFormatter.create = function (options) {
	let defaultOpt = {
		tagName: "a",
		refName: "SimpleLinkFormatter",
		onElementUpdated: onElementUpdated,
		attributes: {
			class: 'text'
		},
		label: null,
		urlField: null
	};

	return FormatterBuilder.create(options, defaultOpt);
};

export default SimpleLinkFormatter;
export { SimpleLinkFormatter };
