import FormatterBuilder from "./FormatterBuilder.js";
import TickCodes from "../../tr-grid-util/es6/TickCodes.js";
import ElfUtil from "../../tr-grid-util/es6/ElfUtil.js";

/** @typedef {Object} SimpleTickerFormatter~Options
* @description FormatterBuilder options
* @private
* @property {string=} positiveColor Color code for up icon
* @property {string=} negativeColor Color code for down icon
*/

let onContextCreated = function (ctx, options) {
	if(ElfUtil.getElfVersion() > 3) {
		ctx._upIcon = "arrow-up-fill";
		ctx._downIcon = "arrow-down-fill";
	} else {
		ctx._upIcon = "arrow-up";
		ctx._downIcon = "arrow-down";
	}
	ctx.TickCodes = TickCodes;
};

let onElementUpdated = function (element, ctx) {
	let value = ctx.TickCodes[ctx.value];

	if (value === 1) {
		element.icon = ctx._upIcon;
		element.style.color = ctx.positiveColor;
	} else if (value === -1) {
		element.icon = ctx._downIcon;
		element.style.color = ctx.negativeColor;
	} else {
		element.icon = "";
	}
};

/** @constructor
* @param {*=} options
*/
let SimpleTickerFormatter = function (options) {

};

/** @public
* @param {(FormatterBuilder.Options|SimpleTickerFormatter~Options|Object)=} options
* @return {Object}
*/
SimpleTickerFormatter.create = function (options) {
	let defaultOpt = {
		tagName: "ef-icon",
		refName: "SimpleTickerFormatter",
		onContextCreated: onContextCreated,
		onElementUpdated: onElementUpdated,
		styles: {
			cursor: "default",
			verticalAlign: "middle"
		},
		positiveColor: "#39C46E",
		negativeColor: "#F5475B"
	};

	return FormatterBuilder.create(options, defaultOpt);
};

export default SimpleTickerFormatter;
export { SimpleTickerFormatter };
