import FormatterBuilder from "./FormatterBuilder.js";
import ElfUtil from "../../tr-grid-util/es6/ElfUtil.js";

/** @typedef {Object} SimpleToggleFormatter~Options
* @description FormatterBuilder options
* @property {(string|Element)=} element1
* @property {(string|Element)=} element2
*/

/** @private
* @function
* @param {Node} n
* @return {Node}
*/
let _provideCloneMethod = function (n) {
	if (n && n.cloneNode) {
		return n;
	} else { // Workaround: custom element may not have cloneNode() method
		let span = document.createElement("span");
		if (n) {
			span.appendChild(n);
		}
		return span;
	}
};

/** @private
* @function
* @param {string=} icon
* @return {Element}
*/
let _createCoralButton = function (icon) {
	let btn = document.createElement("ef-button");
	let styles = "margin: 0;";
	if(ElfUtil.isHaloTheme()) {
		styles += " min-width: 26px;";
	}
	btn.setAttribute("style", styles);
	if (icon) {
		btn.setAttribute("icon", icon);
	}
	return btn;
};

/** @private
* @function
* @param {*} n
* @return {Element}
*/
let _convertToElement = function (n) {
	if (n) {
		if (typeof n === "string") {
			let div = document.createElement("div");
			div.innerHTML = n.trim();
			return div.firstChild;
		}
		return n;
	}
	return null;
};

let onContextCreated = function (ctx, options) {
	ctx.element1 = _provideCloneMethod(ctx.element1);
	ctx.element2 = _provideCloneMethod(ctx.element2);
};

let onElementCreated = function (element, ctx) {
	element._element1 = ctx.element1.cloneNode(true);
	element._element2 = ctx.element2.cloneNode(true);
};

let onElementUpdated = function (element, ctx) {
	let prevChecked = element.firstChild ? element.firstChild === element._element1 : NaN;
	let checked = ctx.value;
	if (checked !== prevChecked) {
		if (element.firstChild) {
			element.removeChild(element.firstChild);
		}
		element.appendChild(checked ? element._element1 : element._element2);
	}
};

/** To uncheck and toggle data of the previous focus radio button
* @private
* @param {*} e
* @param {*} ctx
*/
let onClick = function (e, ctx) {
	let checked = ctx.getData(ctx.field);
	ctx.value = !checked;
	ctx.setData(ctx.field, ctx.value);
};

/** @constructor
* @param {*=} options
*/
let SimpleToggleFormatter = function (options) {

};

/** @public
* @param {(FormatterBuilder.Options|SimpleToggleFormatter.Options|Object)=} options
* @return {Object}
*/
SimpleToggleFormatter.create = function (options) {
	let defaultOpt = {
		tagName: "div",
		refName: "SimpleToggleFormatter",
		onContextCreated: onContextCreated,
		onElementCreated: onElementCreated,
		onElementUpdated: onElementUpdated,
		events: {
			"click": onClick
		},
		element1: _createCoralButton("tick"),
		element2: _createCoralButton()
	};

	if (options) {
		if (options.element1) {
			options.element1 = _convertToElement(options.element1);
		}
		if (options.element2) {
			options.element2 = _convertToElement(options.element2);
		}
	}

	return FormatterBuilder.create(options, defaultOpt);
};

export default SimpleToggleFormatter;
export { SimpleToggleFormatter };
