import FieldDefinition from "./FieldDefinition.js";
import PredefinedFormula from "./PredefinedFormula.js";
import Formula from "../../tr-grid-util/es6/formula/Formula.js";
import Engine from "../../tr-grid-util/es6/formula/Engine.js";

declare namespace ColumnDefinition {

    type SortLogic = (valA: any, valB: any, order: number, ctx: any) => void;

    type Options = {
        field?: string|null,
        dataType?: string|null,
        formulaReference?: string|null,
        require?: (string)[]|null,
        name?: string|null,
        notRealTimeField?: boolean|null,
        tooltip?: (boolean|string|null)|null,
        headerBinding?: ((...params: any[]) => any)|null,
        headerRenderer?: ((...params: any[]) => any)|null,
        binding?: ((...params: any[]) => any)|null,
        renderer?: ((...params: any[]) => any)|null,
        sortLogic?: ColumnDefinition.SortLogic|null,
        sortingLogic?: ColumnDefinition.SortLogic|null,
        sorter?: ColumnDefinition.SortLogic|null,
        rowSorting?: boolean|null,
        sortable?: boolean|null,
        sort?: string|null,
        sortOrder?: string|null,
        styles?: any,
        width?: number|null,
        minWidth?: number|null,
        scalable?: boolean|null,
        className?: string|null,
        class?: string|null,
        textAlign?: string|null,
        alignment?: string|null,
        headerAlignment?: string|null,
        titleAlignment?: string|null,
        hidden?: boolean|null,
        id?: string|null,
        textSelect?: boolean|null,
        keepModel?: boolean|null,
        stationary?: boolean|null,
        leftPinned?: boolean|null,
        rightPinned?: boolean|null,
        info?: any,
        focusable?: boolean|null,
        backgroundColor?: string|null
    };

}

declare class ColumnDefinition {

    constructor(columnOption?: ColumnDefinition.Options|null, hostGrid?: any);

    public dispose(): void;

    public initialize(columnOption?: ColumnDefinition.Options|null): void;

    public getId(): string;

    public getRequireFields(): (string)[];

    public getRequiredFields(): (string)[];

    public getRenderer(): ((...params: any[]) => any);

    public getHeaderRenderer(): ((...params: any[]) => any)|null;

    public getName(): string;

    public isDefaultName(): boolean;

    public getFieldDefinition(): string;

    public getFieldDataType(): string;

    public getDataType(): string;

    public setDataType(dataType: string): void;

    public static getDataType(field: string): string;

    public getTooltip(): string;

    public getField(): string;

    public getAllFields(): (string)[];

    public isRealTimeField(): boolean;

    public isTimeSeries(): boolean;

    public isTimeSeriesChild(): boolean;

    public isFormulaField(): boolean;

    public static isFormulaField(field: string): boolean;

    public static isAdcField(field: string): boolean;

    public static isRealTimeField(field: string): boolean;

    public static getRealTimeFields(fields: (string)[]|null, colDef: ColumnDefinition|null): (string)[];

    public isSortable(): boolean;

    public isScalable(): boolean;

    public getInitialSortOrder(): string;

    public getWidth(): number;

    public getMinWidth(): number;

    public getTextAlign(): string;

    public isFieldEmpty(): boolean;

    public getHeaderAlignment(): string;

    public getClasses(): (string)[];

    public getConfigObject(colOptions?: any): any;

    public isHidden(): boolean;

    public isTextSelect(): boolean;

    public hide(bool?: boolean|null): void;

    public setHeaderRenderer(func: ((...params: any[]) => any)|null): void;

    public setRenderer(func: ((...params: any[]) => any)|null): void;

    public getParent(): ColumnDefinition|null;

    public getChildren(): (ColumnDefinition)[]|null;

    public addRenderer(func: ((...params: any[]) => any)|null): void;

    public activateRenderer(id?: string|null, func?: ((...params: any[]) => any)|null): boolean;

    public getSorter(): ((...params: any[]) => any)|null;

    public setSorter(func?: ColumnDefinition.SortLogic|null): void;

    public isRowSorting(): boolean;

    public isAutoGenerated(): boolean;

    public setName(str: string): void;

    public getUserModel(): any;

    public clearUserModel(): void;

    public setColumnInfo(obj: any): void;

    public getColumnInfo(): any;

    public isFocusable(): boolean;

    public getBackgroundColor(): string;

    public setBackgroundColor(color: string): void;

}

declare const COL_DEF: string;

export {ColumnDefinition, COL_DEF};
export default ColumnDefinition;
