// TODO: Merge this with Fields in mobile-mon repository
/* eslint-disable */
import Grid from "./Grid.js";
import RowDefinition from "./RowDefinition.js";
/* eslint-enable */
import { Deferred } from "../../tr-grid-util/es6/Deferred.js";

/** @type {string}
* @private
* @constant
*/
const SYNAPSE_URL =
	"/synapse/service/suggestions/suggest/?"
	+ "hits=1" // search only 1 result
	+ "&profile=" + encodeURIComponent("Field Selector");

/** @type {!RegExp}
* @private
* @constant
* @description
* ^TR.						=> start with TR.
* \w+						=> any field with string and value
* [\(]						=> open bucket (
* [\w\-=,]*				=> any property name and follow by = EX. SDATE=2011-11-11, PRIOD=123123
* (?:EDATE|SDATE)	=> non-capturing group match EDATE or SDATE
* [\w\-=,]+				=> another propertie param
* [\)]$						=> end with only )
* i								=> for match both upper and lower cases
*/
const TimeSeriesRegEx = /^TR\.\w+\([\w\-=,]*(?:EDATE|SDATE)=+[\w\-=,]*[ \w]*\)$/i;

/* @namespace */
let FieldDefinition = {};

/** @type {Object.<string, Object>}
* @private
*/
FieldDefinition._defs = {};
/** @type {Object.<string, Object>}
* @private
*/
FieldDefinition._defaults = {
	"CF_NAME": {
		name: "Name",
		rank: 2800003,
		fieldDataType: "ALPHANUMERIC",
		id: "6307",
		IsMonitorOnlyField: false,
		IsRealtimePortfolioField: false,
		IsRealtimeField: true,
		source: "IDN",
		sourceRank: 100003,
		description: "The name of the instrument"
	},
	"CF_LAST": {
		name: "Last",
		rank: 2800024,
		fieldDataType: "PRICE",
		id: "5364",
		IsMonitorOnlyField: false,
		IsRealtimePortfolioField: false,
		IsRealtimeField: true,
		source: "IDN",
		sourceRank: 100024,
		description: "The latest trade price or value."
	},
	"PCTCHNG": {
		name: "Pct. Chng",
		rank: 2709440,
		fieldDataType: "PRICE",
		id: "56",
		IsMonitorOnlyField: false,
		IsRealtimePortfolioField: false,
		IsRealtimeField: true,
		source: "IDN",
		sourceRank: 9440,
		description: "Percentage change in the latest trade price or value from the historic close."
	},
	"CF_NETCHNG": {
		name: "Net. Chng",
		rank: 2800027,
		fieldDataType: "PRICE",
		id: "5367",
		IsMonitorOnlyField: false,
		IsRealtimePortfolioField: false,
		IsRealtimeField: true,
		source: "IDN",
		sourceRank: 100027,
		description: "Net Change is the difference between latest trade price or value and the historic closing value or settlement price."
	},
	"CF_CLOSE": {
		name: "Close",
		rank: 2800023,
		fieldDataType: "PRICE",
		id: "5360",
		IsMonitorOnlyField: false,
		IsRealtimePortfolioField: false,
		IsRealtimeField: true,
		source: "IDN",
		sourceRank: 100023,
		description: "The last trade price, settlement value or closing value."
	},
	"CF_CURR": {
		name: "Currency",
		rank: 2800021,
		fieldDataType: "ENUMERATED",
		id: "6452",
		IsMonitorOnlyField: false,
		IsRealtimePortfolioField: false,
		IsRealtimeField: true,
		source: "IDN",
		sourceRank: 100021,
		description: "The currency that the instrument is quoted in"
	},
	"CF_BID": {
		name: "Bid",
		rank: 2800025,
		fieldDataType: "PRICE",
		id: "5359",
		IsMonitorOnlyField: false,
		IsRealtimePortfolioField: false,
		IsRealtimeField: true,
		source: "IDN",
		sourceRank: 100025,
		description: "The latest Bid price."
	},
	"CF_ASK": {
		name: "Ask",
		rank: 2800026,
		fieldDataType: "PRICE",
		id: "5358",
		IsMonitorOnlyField: false,
		IsRealtimePortfolioField: false,
		IsRealtimeField: true,
		source: "IDN",
		sourceRank: 100026,
		description: "The latest Ask price."
	}
};

/** @type {Object.<string, Object>}
* @private
*/
FieldDefinition._timeSeriesChildren = {};

/** api-key to call synapse service
* @type {string}
* @private
* @const
*/
FieldDefinition._synapse = "";
/**
* @type {string}
* @private
* @const
*/
FieldDefinition._lang = "en";
/**
* @type {boolean}
* @private
*/
FieldDefinition._timeSeriesExpansion = true;

/**
 * @type {Object.<string, boolean>}
 * @private
 */
FieldDefinition._loadingField = {};

/** @public
* @function
* @param {string} field
* @param {Object} def
*/
FieldDefinition.set = function(field, def) {
	if (!FieldDefinition._defs[field]) {
		FieldDefinition._defs[field] = def;
	} else {
		for (let key in def) {
			FieldDefinition._defs[field][key] = def[key];
		}
	}
};
/** Get field definition object
* @public
* @function
*/
FieldDefinition.resetAll = function() {
	let defs = FieldDefinition._defs = {};
	let defaultFields = FieldDefinition._defaults;
	for(let key in defaultFields) {
		defs[key] = defaultFields[key];
	}
};
/** Get field definition object
* @public
* @function
* @param {string} field
* @return {Object}
*/
FieldDefinition.get = function(field) {
	return FieldDefinition._defs[field] || null;
};
/** Get field definition object
* @public
* @function
* @param {string} field
* @return {Object}
*/
FieldDefinition.getFieldInfo = FieldDefinition.get;

/** @public
* @function
* @param {string} field
* @return {boolean}
*/
FieldDefinition.hasFieldInfo = function(field) {
	let val = FieldDefinition.get(field);
	return (val && val.field) ? true : false; // Already preventing an error caused by accessing a property on a null value
};

/** @public
* @function
* @param {string} field
* @return {Array}
*/
FieldDefinition.getTimeSeriesChildren = function(field) {
	return FieldDefinition._timeSeriesChildren[field] || null;
};
/** @public
* @function
* @param {string} tsDef
* @param {Object} childDef
*/
FieldDefinition.addTimeSeriesChild = function(tsDef, childDef) {
	if(!FieldDefinition._timeSeriesChildren[tsDef]) {
		FieldDefinition._timeSeriesChildren[tsDef] = [];
	}
	FieldDefinition._timeSeriesChildren[tsDef].push(childDef);
};
/** @public
* @function
* @param {string} field
*/
FieldDefinition.remove = function(field) {
	delete FieldDefinition._defs[field];
};

/** @public
* @function
* @param {Grid~SynapseConfig} config
*/
FieldDefinition.setSynapseConfig = function (config) {
	FieldDefinition._synapse = config;
};

/** @public
* @function
* @param {boolean} disabled=true if disable it, time series will not be expand
*/
FieldDefinition.disableTimeSeriesExpansion = function(disabled) {
	FieldDefinition._timeSeriesExpansion = disabled === false;
};

/** @public
* @param {string} field
* @return {boolean}
*/
FieldDefinition.isFormula = function(field) {
	return field.charAt(0) === "=";
};
/** @public
* @param {string} field
* @return {boolean}
*/
FieldDefinition.isAdc = function(field) {
	return field.indexOf("TR.") === 0;
};

/** @public
* @param {string} field
* @return {boolean}
*/
FieldDefinition.isRealTimeField = function(field) {
	if (!field) {
		return false;
	}

	if(FieldDefinition.isAdc(field)) { // Note that all TimeSeries fields are ADC field
		return false;
	}

	if(FieldDefinition.isFormula(field)) {
		return false;
	}

	if(FieldDefinition.isTimeSeriesChild(field)) {
		return false;
	}

	return FieldDefinition.getFieldProperty(field, "IsRealtimeField") !== false;
};

/** @public
* @param {string} field
* @return {boolean}
*/
FieldDefinition.isTimeSeriesChild = function(field) {
	return FieldDefinition.getFieldProperty(field, "timeSeriesParent") ? true : false;
};

/** @public
* @param {string} field
* @return {boolean}=true if field is time series field
*/
FieldDefinition.isTimeSeries = function (field) {
	if (!field) {
		return false;
	}

	// If we disable time series expansion, the grid won't have a time series field.
	if(!FieldDefinition._timeSeriesExpansion) {
		return false;
	}

	// We can check time series using a cache to avoid duplicating checks in regular expressions.
	let timeSeriesField = FieldDefinition.getFieldProperty(field, "timeSeries");
	if (timeSeriesField != null) {
		return timeSeriesField ? true : false;
	}

	return TimeSeriesRegEx.test(field);
};

/**
 * Set property value into field definition
 * @private
 * @param {string} field field definition
 * @param {string} propertyName
 * @param {*} value
 */
FieldDefinition.setFieldProperty = function(field, propertyName, value) {
	if(!FieldDefinition._defs[field]) {
		FieldDefinition._defs[field] = {};
	}
	FieldDefinition._defs[field][propertyName] = value;
};

/**
 * Set property value into field definition
 * @private
 * @param {string} field field definition
 * @param {string} propertyName
 * @return {*}
 */
FieldDefinition.getFieldProperty = function(field, propertyName) {
	let fieldDef = FieldDefinition._defs[field];
	if(fieldDef) {
		return fieldDef[propertyName];
	}
	return null;
};

/** To get more info about field via synapse service
* @private
* @param {string} field
* @returns {Promise}
*/
FieldDefinition.loadFieldInfo = function (field) {
	if (FieldDefinition._loadingField[field]) {
		return FieldDefinition._loadingField[field].promise;
	}

	let defer = new Deferred();
	let synapse = FieldDefinition._synapse;
	// cannot call request without synapse config
	if (!synapse) {
		defer.resolve(null);
	}
	// already have field definition then return
	else if (FieldDefinition.hasFieldInfo(field)) {
		defer.resolve(FieldDefinition.get(field));
	}
	// in debug using mock data instead
	else if (synapse.debug) {
		let mockData = this._mockOnLoadEndData(field, defer);
		setTimeout(onLoadEnd.bind(null, mockData), 1000);
	}
	// cannot call request without apiKey and  contextApp
	else if (!synapse.apiKey || !synapse.contextApp) {
		defer.resolve(null); // cannot request without apiKey and contextApp
	}
	// everything fine, call synapse service
	else {
		let queryUrl = SYNAPSE_URL
			+ "&api-key=" + encodeURIComponent(synapse.apiKey)
			+ "&contextApp=" + encodeURIComponent(synapse.contextApp)
			+ "&language=" + encodeURIComponent(FieldDefinition._lang)
			+ "&query=" + encodeURIComponent(field);
		if (synapse.auth) {
			queryUrl += "&auth=" + encodeURIComponent(synapse.auth);
		}
		// TODO: handle client timeout
		let xmr = new XMLHttpRequest();
		xmr._defer = defer;
		xmr._field = field;
		FieldDefinition._loadingField[field] = defer;

		// for now we care only successful case
		xmr.addEventListener("loadend", onLoadEnd);
		xmr.addEventListener("timeout", onError);
		xmr.addEventListener("error", onError);
		xmr.open("GET", queryUrl, true); // true for asynchronous
		xmr.send();
	}
	return defer.promise;
};

/**
* @private
* @param {Event} e
*/
function onError(e) {
	// provide information for timeout and error cases and will be used in onLoadEnd handler
	e.currentTarget._error = {
		type: e.type
	};
}

/** @private
* @function
* @param {string} field
* @param {Object} defer
* @returns {Object}
*/
FieldDefinition._mockOnLoadEndData = function (field, defer) {
	let fieldUpperCase = field.toUpperCase();
	let fieldLowerCase = fieldUpperCase.toLowerCase();
	let label;
	if (fieldLowerCase.indexOf("_") === 2) { // transform XX_ABCD -> Abcd
		label = fieldUpperCase[3] + fieldLowerCase.substring(4);
	} else {
		label = fieldUpperCase[0] + fieldLowerCase.substring(1);
	}
	let dataType = (fieldUpperCase.indexOf("DATE") >= 0) ? "DATE" : "PRICE";
	let item = {
		cmd: label,
		explanation: null,
		fr: false,
		id: "1635614732",
		navigation: null,
		p: {
			Rank: 2800024,
			fdt: dataType,
			fid: "5364",
			fimo: false,
			firp: false,
			firt: true,
			fl: label,
			fn: fieldUpperCase,
			fsrc: "IDN",
			fsrnk: 100024
		},
		position: 0,
		preview: false,
		relations: {},
		score: 2840024,
		source: "87",
		subtitle: "",
		symbol: label,
		title: "title mockup of " + fieldUpperCase,
		vc: "FLD"
	};

	let response = {
		action: '\"' + fieldUpperCase + '\" SRCH',
		assetClassifierExplanation: "AssetClassifier is disabled.",
		assetClassifierStatus: "NOT_INVOKED",
		// header: { request: { … }, response: { … } }, // whatever
		result: [{ name: "Fields", default: false, size: 1, hits: [item], hasMore: false }],
		search: "cpurl://apps.cp./apps/SearchAll?Search.Value=" + fieldLowerCase + "*"
	};
	response = JSON.stringify(response);
	return {
		currentTarget: {
			_defer: defer,
			_field: field,
			status: 200,
			response: response,
			responseText: response
		}
	};
};

/**
* @private
* @param {Event} e
*/
function onLoadEnd(e) {
	let xmr = e.currentTarget;
	let defer = xmr._defer;
	let field = xmr._field;

	delete FieldDefinition._loadingField[field];

	if (xmr.status === 200) { // case success
		let res = JSON.parse(xmr.responseText);
		let fieldDef = null;

		let result = res.result && res.result[0];
		if (result) {
			let item = result.hits && result.hits[0];
			if (item && item.p) {
				let profile = item.p;
				if (profile.fn.toUpperCase() === field.toUpperCase()) {
					fieldDef = FieldDefinition.get(field) || {};

					fieldDef.field = profile.fn; // name
					fieldDef.name = profile.fl; // label
					fieldDef.rank = profile.Rank;
					fieldDef.fieldDataType = profile.fdt; // data type
					fieldDef.id = profile.fid;
					fieldDef.IsMonitorOnlyField = profile.fimo;
					fieldDef.IsRealtimePortfolioField = profile.firp;
					fieldDef.IsRealtimeField = profile.firt;
					fieldDef.source = profile.fsrc;
					fieldDef.sourceRank = profile.fsrnk;
					fieldDef.description = item.title;

					FieldDefinition.set(field, fieldDef);
				}
			}
		}
		if(fieldDef) {
			defer.resolve(fieldDef);
		} else {
			defer.reject("No definition for " + field);
		}
	} else if (xmr._error) { // case error && time out
		xmr._error.field = field;
		defer.reject(xmr._error);
	} else { // case status not 200
		defer.reject({
			field: field,
			status: xmr.status,
			statusText: xmr.statusText,
			request: xmr
		});
	}
}

(function() { // Add "field" property to all default field definitions. This reduces file size and redundancy
	for(let key in FieldDefinition._defaults) {
		FieldDefinition._defaults[key]["field"] = key;
		FieldDefinition._defs[key] = FieldDefinition._defaults[key];
	}
})();

export {FieldDefinition};
export default FieldDefinition;
