/* eslint-disable */
import Grid from "./Grid.js";
import {Ext} from "../../tr-grid-util/es6/Ext.js";
import {EventDispatcher} from "../../tr-grid-util/es6/EventDispatcher.js";
import {FieldDefinition} from "./FieldDefinition.js";
import { isEmptyObject } from "../../tr-grid-util/es6/Util.js";

/** This class make requesting ADC field easier by encapsulating JET.Data APIs and mapping data to the row
* @constructor
* @extends {EventDispatcher}
*/
let SnapshotFiller = function () {
	this._onRequest = this._onRequest.bind(this);

	this._rics = {};
	this._fields = {};
	this._adcOptions = { // TODO: support requesting level parameter
		productId: "001"
	};
};
Ext.inherits(SnapshotFiller, EventDispatcher);

/** @private
* @type {Object}
*/
SnapshotFiller._mockAdc = null;

/** @type {number}
* @private
*/
SnapshotFiller.prototype._timerId = 0;
/** @type {!Object.<string, number>}
* @private
*/
SnapshotFiller.prototype._rics;
/** @type {!Object.<string, number>}
* @private
*/
SnapshotFiller.prototype._fields;
/** @type {!*}
* @private
*/
SnapshotFiller.prototype._rtk;
/** @type {!Grid~ADCOptions}
* @private
*/
SnapshotFiller.prototype._adcOptions = null;


/** @public
* @param {*} rtk RTK instance
*/
SnapshotFiller.prototype.dispose = function () {
	this.removeAllEventListeners();
	if(this._timerId) {
		clearTimeout(this._timerId);
		this._timerId = 0;
	}
	this._rics = this._fields = this._rtk = this._adcOptions = null;
};

/** @public
* @param {*} rtk RTK instance
*/
SnapshotFiller.prototype.setRTK = function (rtk) {
	this._rtk = rtk;
};
/** @public
* @param {string} str 
* @param {*} value 
*/
SnapshotFiller.setMockAdc = function(str, value) {
	if(!SnapshotFiller._mockAdc) {
		SnapshotFiller._mockAdc  = {};
	}
	SnapshotFiller._mockAdc[str] = value;
};

/** @public
* @param {string} str 
* @return {*} 
*/
SnapshotFiller.getMockAdc = function(str) {
	if(!SnapshotFiller._mockAdc) {
		return null;
	}
	return SnapshotFiller._mockAdc[str];
};

/** @public
*/
SnapshotFiller.clearMockAdc = function() {
	SnapshotFiller._mockAdc = null;
}

/** @public
* @param {Grid~ADCOptions} adcOptions ADC requesting level parameter options
*/
SnapshotFiller.prototype.setADCOptions = function (adcOptions) {

	let val = adcOptions["productId"];
	if(val) {
		this._adcOptions.productId = val;
	}
	// TODO: support requesting level parameter https://confluence.refinitiv.com/display/ADC/Request+level+parameters
	// let val = adcOptions["lang"];
	// if(val) {
	// 	this._adcOptions.lang = val;
	// }
	// let val = adcOptions["cache"];
	// if(val) {
	// 	this._adcOptions.cache = val;
	// }
};

/** @public
* @param {string} ric
*/
SnapshotFiller.prototype.addRic = function (ric) {
	// TODO: since we use ric instead of rowDef
	// We need to find a way to check isSubscribing and isAutoGenerated before add ric
	// if (!rowDef.isSubscribing()) { // // Only real-time row (non-empty non-header) can be registered for snapshot.
	// 	if (!rowDef.isAutoGenerated()) {  // However, chain constituents (auto generated) are still eligible.
	// 		return;
	// 	}
	// }

	if (ric && !this._rics[ric]) {
		this._rics[ric] = 1;
		this._request();
	}
};
/** @public
* @param {Array.<string>} rics
* @return {boolean}
*/
SnapshotFiller.prototype.addRics = function (rics) {
	let len = rics ? rics.length : 0;
	let dirty = false;
	for(let i = 0; i < len; ++i) {
		let ric = rics[i];
		if (ric && !this._rics[ric]) {
			this._rics[ric] = 1;
			dirty = true;
		}
	}
	if(dirty) {
		this._request();
		return true;
	}
	return false;
};
/** @public
* @param {string} str
*/
SnapshotFiller.prototype.addField = function (str) {
	if(this._addField(str)) {
		this._request();
	}
};
/** @public
* @param {Array.<string>} fields
* @return {boolean}
*/
SnapshotFiller.prototype.addFields = function (fields) {
	let len = fields.length;
	let dirty = 0;
	for(let i = 0; i < len; ++i) {
		dirty |= this._addField(fields[i]);
	}
	if(dirty) {
		this._request();
		return true;
	}
	return false;
};

/** @private
* @param {string} str
* @return {number}
*/
SnapshotFiller.prototype._addField = function (str) {
	if(SnapshotFiller.isValidField(str)) {
		if(!this._fields[str]) {
			this._fields[str] = 1;
			return 1;
		}
	}
	return 0;
};
/** @public
* @function
* @param {string} str
* @returns {boolean}
*/
SnapshotFiller.isValidField = function(str) {
	// Only field with TR. is considered as ADC field
	return (str && str.indexOf("TR.") === 0) ? true : false;
};

/** @private
*/
SnapshotFiller.prototype._request = function () {
	if(!this._timerId) {
		this._timerId = setTimeout(this._onRequest, 0);
	}
};
/** @private
*/
SnapshotFiller.prototype._onRequest = function () {
	this._timerId = 0;

	let jet = window["JET"]; // Assume that JET is already exist.
	if (this._rtk) {
		if (!this._rtk.Data) {
			return; // Require RTK.Data plugin to get snapshot data
		}
	} else if (!jet || !jet["ContainerDescription"] || (typeof jet["Data"] !== "function")) {
		return; // Require JET to get snapshot data or JET is not ready to use
	}

	// Retrieve instruments and ADC fields
	let instruments = Object.keys(this._rics);

	let i;
	let fields = [];
	let timeSeriesFields = [];
	for (let field in this._fields) {
		if(!FieldDefinition.isTimeSeries(field)) {
			fields.push(field);
		} else {
			timeSeriesFields.push(field);
		}
	}
	let fieldLen = fields.length;
	let timeSeriesFieldLen = timeSeriesFields.length;

	if ( (!fieldLen && !timeSeriesFieldLen ) || !instruments.length) { // No ADC field or real-time RIC
		return;
	}

	// Clean up members, preparing for the next request
	this._rics = {};
	this._fields = {};

	let onSuccess, payload;
	if (this._rtk) {
		let strFields;
		// Request time serie fields
		if(timeSeriesFields.length > 0) {
			// request time series field
			let strtTimeSeriesFields = timeSeriesFields.join(',');
			payload = {
				"method": "select",
				"formula": strtTimeSeriesFields,
				"identifiers": instruments,
				"productId": this._adcOptions.productId,
				"output": "Col,date|,Row,In|,va,T,NoEmptyTickers" // For customize output server, for more information please visit "https://confluence.refinitiv.com/display/ADC/Data+Cloud+Output+Format"
			};
			onSuccess = this._onRTKTimeSeriesSuccess.bind(this, timeSeriesFields);
			let responseMockAdc = SnapshotFiller.getMockAdc(payload.formula);
			if(responseMockAdc) {
				setTimeout(function() {
					onSuccess(responseMockAdc);
				}, 10); // Mock Async operation
			} else {
				this._rtk.Data.Adc
				.request(payload)
				.then(onSuccess)
				.catch(function (err) {
					console.log(err);
				});
			}
		}

		// Request normal adc field
		if(fields.length > 0) {
			strFields = fields.join(',');
			payload = {
				"method": "select",
				"formula": strFields,
				"identifiers": instruments,
				"productId": this._adcOptions.productId,
				"output": "Col,In,va,T,NoEmptyTickers" // For customize output server, for more information please visit "https://confluence.refinitiv.com/display/ADC/Data+Cloud+Output+Format"
			};
			onSuccess = this._onRTKSuccess.bind(this, fields);
			this._rtk.Data.Adc
				.request(payload)
				.then(onSuccess)
				.catch(function (err) {
					console.log(err);
				});
		}


	} else {
		let reqFields = [];
		for(i = 0; i < fieldLen; i++) {
			reqFields.push({ "name": fields[i] });
		}
		payload = {
			"instruments": instruments,
			"fields": reqFields
		};
		onSuccess = this._onJETSuccess.bind(this, fields);
		jet["Data"]("datagrid")
			.then(function (service) { return service["request"](payload); })
			.then(JSON.parse)
			.then(onSuccess)
			.catch(function (err) {
				console.log(err);
			});
	}
};

/** @private
* @function
* @param {Array.<string>} fields
* @param {string} serverResult
*/
SnapshotFiller.prototype._onJETSuccess = function (fields, serverResult) {
	this._dispatch("adcDataReceived", serverResult);
	let data2D = serverResult["data"];
	let svHeaders = serverResult["headers"] && serverResult["headers"][0];

	if (!Array.isArray(data2D) || !Array.isArray(svHeaders)) {
		return; //TODO: Return Promise.reject(errMsg);
	}


	// Server will return field only in UPPERCASE
	// ex. requestField = TR.Volume ===> serverField = TR.VOLUME
	// so we need convert UPPERCASE to be original
	let i, field, ric;
	let j = 1; //  to skip instrument index, use j = 1
	let fLength = fields.length;
	let hLength = svHeaders.length;
	let headers = new Array(hLength);
	for (i = 0; i < fLength && j < hLength; i++) {
		field = fields[i];
		if (svHeaders[j].field === field.toUpperCase()) {
			headers[j] = field;
			j++;
		}
	}

	let len = data2D.length;
	let fieldLen = headers.length;
	let ricMap = {};

	// TODO: Freeze the data view before setting multiple data
	for (i = 0; i < len; i++) {
		let dataRow = data2D[i];
		ric = dataRow[0];

		if (ricMap[ric]) {
			continue; // we interested only first row of ric
		}

		let snapData = ricMap[ric] = {};

		// loop for create rowData for update
		for (j = 1; j < fieldLen; j++) { //  to skip instrument index, use j = 1
			let value = dataRow[j];
			if (value != null && value !== "") {
				field = headers[j];
				snapData[field] = value;
			}
		}
	}

	// return result only ric that has update data
	let updatedData = {};
	for (ric in ricMap) {
		let obj = ricMap[ric];
		if (!isEmptyObject(obj)) {
			updatedData[ric] = obj;
		}
	}

	this._dispatch("dataChanged", {
		data: updatedData
	});
};

/** @private
* @function
* @param {Array.<string>} fields
* @param {string} serverResult
*/
SnapshotFiller.prototype._onRTKSuccess = function (fields, serverResult) {
	this._dispatch("adcDataReceived", serverResult);
	let data2D = serverResult["rows"];
	let svHeaders = serverResult["rows"] && serverResult["rows"][0];
	if (!Array.isArray(data2D) || !Array.isArray(svHeaders)) {
		return; //TODO: Return Promise.reject(errMsg);
	}

	// Server will return field only in UPPERCASE
	// ex. requestField = TR.Volume ===> serverField = TR.VOLUME
	// so we need convert UPPERCASE to be original
	let i, field, ric;
	let j = 1; //  to skip instrument index, use j = 1
	let fLength = fields.length;
	let hLength = svHeaders.length;
	let headers = new Array(hLength);
	for (i = 0; i < fLength && j < hLength; i++) {
		field = fields[i];
		if (svHeaders[j].r.toUpperCase() === field.toUpperCase()) {
			headers[j] = field;
			j++;
		}
	}

	let len = data2D.length;
	let fieldLen = headers.length;
	let ricMap = {};

	// TODO: Freeze the data view before setting multiple data
	for (i = 1; i < len; i++) { // to skip column header index, use i = 1
		let dataRow = data2D[i];
		ric = dataRow[0];

		if (ricMap[ric]) {
			continue; // we interested only first row of ric
		}
		let snapData = ricMap[ric] = {};

		// loop for create rowData for update
		for (j = 1; j < fieldLen; j++) { // to skip instrument index, use j = 1
			let value = dataRow[j];
			if (value != null && value !== "") {
				if(typeof value !== 'object') {
					field = headers[j];
					snapData[field] = value;
				}
				// TODO : handled when a cell has a mistake and the value appears as {f: "1"} ( description error in fault attribute at index 1 ),
				// Therefore, we need to store information to the error field for this cell.
				// else {}

			}
		}
	}

	// return result only ric that has update data
	let updatedData = {};
	for (ric in ricMap) {
		let obj = ricMap[ric];
		if (!isEmptyObject(obj)) {
			updatedData[ric] = obj;
		}
	}

	this._dispatch("dataChanged", {
		data: updatedData
	});
};


/** @private
* @function
* @param {Array.<string>} fields
* @param {string} serverResult
*/
SnapshotFiller.prototype._onRTKTimeSeriesSuccess = function (fields, serverResult) {

	// TODO: noti the user for change structure
	this._dispatch("adcDataReceived", serverResult);
	let data2D = serverResult["rows"];
	let svHeaders = serverResult["rows"] && serverResult["rows"][0];
	if (!Array.isArray(data2D) || !Array.isArray(svHeaders)) {
		return; // TODO: Return Promise.reject(errMsg);
	}
	// Extract header dates
	let headerDates = svHeaders.slice(2).map(header => header.v);
	// Create necessary variables
	let fileNameToTSField = {};
	let count = 0;
	// Convert map to fieldMap
	let data2DLength = data2D.length;
	for (let i = 1; i < data2DLength; i++) {
		let dataRow = data2D[i];
		let fieldName = dataRow[1];
		if (!fileNameToTSField[fieldName]) {
			fileNameToTSField[fieldName] = fields[count++];
		}
	}
	// Process data and populate ricMap and childrenFieldToParent
	let ricMap = {};
	let childrenFieldToParent = {};
	for (let i = 1; i < data2DLength; i++) {
		let dataRow = data2D[i];
		let ric = dataRow[0];
		let fieldName = dataRow[1];

		if (!ricMap[ric]) {
			ricMap[ric] = {};
		}
		let snapData = ricMap[ric];
		if (headerDates.length === 1) {
			// Time series with only 1 column, it will be a normal adc field
			snapData[fileNameToTSField[fieldName]] = dataRow[2];
		} else {
			// Multiple columns, it will be a time series field
			for (let k = 0; k < headerDates.length; k++) {
				let field = fileNameToTSField[fieldName].replace("TR.", "") + "_" + headerDates[k];
				let celLData =  dataRow[2 + k];
				if(celLData !== null) {
					childrenFieldToParent[field] = fileNameToTSField[fieldName];
					snapData[field] = celLData;
				}
			}
		}
	}

	// Return result only for ric that has updated data
	let dataMapping = {};
	for (let ric in ricMap) {
		let obj = ricMap[ric];
		if (!isEmptyObject(obj)) {
			dataMapping[ric] = obj;
		}
	}

	// Dispatch events based on the header length
	let eventData = {
		data: dataMapping,
	};
	if (headerDates.length > 1) {
		eventData.timeSeries = true;
		eventData.childrenFieldToParent = childrenFieldToParent;
	}
	this._dispatch("dataChanged", eventData);
};

export { SnapshotFiller };
export default SnapshotFiller;
