import { Ext } from "../../tr-grid-util/es6/Ext.js";
import { GridPlugin } from "../../tr-grid-util/es6/GridPlugin.js";
import { Dom } from "../../tr-grid-util/es6/Dom.js";
import { Popup } from "../../tr-grid-util/es6/Popup.js";
import { ElfUtil } from "../../tr-grid-util/es6/ElfUtil.js";
import { isTouchDevice, injectCss, prettifyCss } from "../../tr-grid-util/es6/Util.js";
import { ElfDate } from "../../tr-grid-util/es6/ElfDate.js";
import { CoralItems } from "../../tr-grid-util/es6/CoralItems.js";
import { Conflator } from "../../tr-grid-util/es6/Conflator.js";

declare namespace InCellEditingPlugin {

    type Options = {
        balloonMode?: boolean|null,
        editableTitle?: boolean|null,
        editableContent?: boolean|null,
        autoCommitText?: boolean|null,
        dataBinding?: boolean|null,
        popupElement?: Element|null,
        doubleClick?: boolean|null,
        tabToMove?: boolean|null,
        contentSource?: string|null,
        inlineStyling?: boolean|null,
        disablingScroll?: boolean|null,
        uiBlocking?: boolean|null,
        preEditorOpening?: ((...params: any[]) => any)|null,
        editorOpened?: ((...params: any[]) => any)|null,
        beforeCommit?: ((...params: any[]) => any)|null,
        editorClosed?: ((...params: any[]) => any)|null,
        keyUp?: ((...params: any[]) => any)|null,
        beforeRowCommit?: ((...params: any[]) => any)|null,
        rowEditorClosed?: ((...params: any[]) => any)|null,
        autoSuggest?: Element|null,
        closingOnScroll?: boolean|null,
        autoHiding?: boolean|null,
        readonly?: boolean|null,
        starterText?: (string|boolean)|null
    };

    type Cache = {
        editor: Element|null,
        balloonPopup: Element|null,
        inputElement: Element|null
    };

    type EditorOptions = {
        type?: string|null,
        entries?: any[]|null,
        attributes?: any
    };

    type ColumnOptions = {
        editableContent?: boolean|null,
        inCellEditing?: InCellEditingPlugin.EditorOptions|null
    };

}

declare class InCellEditingPlugin extends GridPlugin {

    constructor(options?: InCellEditingPlugin.Options|null);

    public initialize(host: any, options: any): void;

    public beforeProcessOption(optionName: string, optionVal: any): any;

    public config(options?: any): void;

    public getConfigObject(out_obj?: any): any;

    public openEditor(colIndex: number, rowIndex: number, sectionRef?: any, grid?: any): void;

    public closeCellEditor(opt_forceCommit?: boolean|null): void;

    public enableBalloonMode(opt_enabled?: boolean|null): void;

    public enableTitleEditing(opt_enabled?: boolean|null): void;

    public disableTitleEditing(opt_disabled?: boolean|null): void;

    public disableCellEditing(cell: any, opt_disabled?: boolean|null): void;

    public disableDataBinding(opt_disabled?: boolean|null): void;

    public disableDblClick(opt_disabled?: boolean|null): void;

    public showStarterText(shown?: boolean|null): void;

    public _requestUpdateStarterText(): void;

    public updateStarterText(shown?: boolean|null): void;

    public isEditing(): boolean;

    public getTextBox(columnIndex?: number|null, grid?: any): Element|null;

    public getInitialText(): string;

    public _getRowIndex(rowId: string): number;

    public getActiveColIndex(): number;

    public getActiveRowIndex(): number;

    public getInputElementPosition(): any;

    public enableAutoCommitText(opt_enable?: boolean|null): void;

    public isColumnEditable(colIndex: number): boolean;

    public setColumnEditability(colIndex: number, bool?: (boolean|null)|null): void;

    public enableReadonly(enabled?: boolean|null): void;

    public disableReadonly(disabled?: boolean|null): void;

    public openRowEditor(rowIndex: number, grid?: any): void;

    public closeRowEditor(isCommit?: boolean|null): void;

    public toggleRowEditor(rowIndex: number, isCommit?: boolean|null, grid?: any): void;

    public setPopupElement(elm: Element|null): void;

    public setText(content: any): void;

    public getText(): string;

    public getValue(): any;

    public clearText(): void;

    public appendText(word: string): void;

    public commitText(word?: string|null): void;

    public getBalloonPopup(): Popup|null|null;

    public getAutoSuggest(): Element|null;

    public setDisablingScroll(disabled?: boolean|null): boolean;

    public requestShowEditorPopup(): void;

}

export default InCellEditingPlugin;
export { InCellEditingPlugin, InCellEditingPlugin as InCellEditing, InCellEditingPlugin as InCellEditingExtension };
