import { Ext } from "../../tr-grid-util/es6/Ext.js";
import { ElementWrapper } from "../../tr-grid-util/es6/ElementWrapper.js"; // ElementWrapper doesnot have setStyle and etc.
import { Dom } from "../../tr-grid-util/es6/Dom.js";

/**
* @constructor
* @extends {ElementWrapper}
*/
let CellWriter = function () {
};
Ext.inherits(CellWriter, ElementWrapper);

/** @private
* @type {Element}
*/
CellWriter.prototype._flexRow = null;
/** @private
* @type {Element}
*/
CellWriter.prototype._floatingPanel = null;
/** @private
* @type {Element}
*/
CellWriter.prototype._frontPanel = null;

/** @private
* @type {Element}
*/
CellWriter.prototype._parent = null;


/** @public
* @return {Node} content
*/
CellWriter.prototype.getParent = function () {
	if(!this._parent) {
		this._parent = document.createElement("div");
	}
	return this._parent;
};
/** @public
* @return {Node} content
*/
CellWriter.prototype.getContent = function () {
	return this._elem.children[0] || null;
};
/** @public
* @param {*} content
* @param {boolean=} tooltip
* @return {Element}
*/
CellWriter.prototype.setContent = function (content, tooltip) {
	Dom.removeChildren(this._elem);
	if(content == null) {
		return null;
	}
	if(content["getElement"]) {
		content = /** @type{Element} */(content["getElement"]());
	} else if(content.nodeType !== 1) {
		content = this._createTextContent(content);
	}

	this._elem.appendChild(/** @type{Node} */(content));
	return /** @type{Element} */(content);
};

/**
 * Set value to title. If value is absent then a title will be removed
 * @public
 * @param {string} str
 */
CellWriter.prototype.setTooltip = function (str) {
	if(str) {
		this._elem.setAttribute("title", str);
	} else {
		this._elem.removeAttribute("title");
	}
};
/** @public
* @ignore
* @param {string} type
* @param {*=} tooltip
*/
CellWriter.prototype.setTooltipInfo = function(type, tooltip) {
};
/** @public
* @ignore
*/
CellWriter.prototype.updateTooltip = function() {
};

/** @public
* @param {string} str
*/
CellWriter.prototype.setTextContent = function (str) {
	Dom.removeChildren(this._elem);
	this._elem.appendChild(this._createTextContent(str));
};
/** @public
* @param {*} n
* @return {Element}
*/
CellWriter.prototype.addContent = function (n) {
	this._elem.appendChild(/** @type{Node} */(n));
	return /** @type{Element} */(n);
};
/** @private
* @param {string} str
* @return {Element}
*/
CellWriter.prototype._createTextContent = function (str) {
	let div = Dom.div("text");
	if(str != null) {
		div.textContent = str;
		this["percentText"] = str; // HACK
	}
	return div;
};

/** @public
* @param {string} str
* @param {*} val
*/
CellWriter.prototype.setStyle = function (str, val) {
	this._elem.style[str] = val;
};
/** @private
* @returns {!Element} flex row
*/
CellWriter.prototype._insertFlexRow = function () {
	let flexRow = this._flexRow;
	if(!flexRow) {
		flexRow = this._flexRow = document.createElement("div");
		flexRow.className = "tr-printing-flex-row";
	}

	let elem = this._elem;
	if(elem.firstChild !== flexRow) {
		while(elem.firstChild) {
			flexRow.appendChild(elem.firstChild);
		}
		elem.appendChild(flexRow);
	}

	return flexRow;
};
/** @public
* @param {*} n
*/
CellWriter.prototype.removeIcon = function (n) {};
/** @public
* @param {Element} elem
*/
CellWriter.prototype.updateIcon = function (elem) {
	if(elem && elem.nodeType === 1) {
		let flexRow = this._insertFlexRow();
		let fp = this._frontPanel;
		if(!fp) {
			fp = this._frontPanel = document.createElement("div");
			fp.className = "tr-printing-float-right"; // WARNING: This may need to be float-left
			flexRow.insertBefore(fp, flexRow.firstChild);
		}
		fp.appendChild(elem.cloneNode(true)); // WARNING: This may not need to be cloned
	}
};
/** @public
*/
CellWriter.prototype.unlisten = function () {};
/** @public
*/
CellWriter.prototype.listen = function () {};

/** @public
* @param {string} str
*/
CellWriter.prototype.addClass = function (str) {
	this._elem.classList.add(str);
};
/** @public
* @param {string} str
* @returns {boolean}
*/
CellWriter.prototype.hasClass = function (str) {
	return this._elem.classList.contains(str);
};
/** @public
* @param {string} str
*/
CellWriter.prototype.removeClass = function (str) {
	this._elem.classList.remove(str);
};
/** @public
* @param {string} str
* @param {boolean} bool
*/
CellWriter.prototype.enableClass = function (str, bool) {
	if(bool || bool == null) {
		this.addClass(str);
	} else {
		this.removeClass(str);
	}
};

/** @public
* @param {Element} elem
* @param {number} order
*/
CellWriter.prototype.insertFloatingIcon = function (elem, order) {
	if(elem && elem.nodeType === 1) {
		let flexRow = this._insertFlexRow();
		let fp = this._floatingPanel;
		if(!fp) {
			fp = this._floatingPanel = document.createElement("div");
			fp.className = "tr-printing-float-right";
			flexRow.appendChild(fp);
		}
		fp.appendChild(elem.cloneNode(true)); // WARNING: This may not need to be cloned
	}
};
/** @public
* @param {string|Element} elemRef
* @param {number} order
*/
CellWriter.prototype.removeFloatingIcon = function (elemRef, order) {};
/** @public
* @return {null}
*/
CellWriter.prototype.getSection = function () {
	return null;
};

/**
* @public
* @param {Element} elem
*/
CellWriter.prototype.cloak = function (elem) {
	this.percentText = ""; // Hack
	this._flexRow = this._frontPanel = this._floatingPanel = null;
	elem.className = "cell";
	this._elem = elem;
};



export default CellWriter;
export { CellWriter };
