/** @namespace */
let CoralItems = {};

/** Create new array with valid object items to be consumed by ef-select's data property from the given input
* @public
* @function
* @param {Array} userItems This can be array of anything (e.g. texts, numbers, objects or etc.).
* @return {Array.<Object>} Valid data objects for generating ef-item by ef-select
* @example
* CoralItems.create(["a", "b"]); // [{label: "a", value: "a"}, {label: "b", value: "b"}]
* CoralItems.create([33]); // [{label: "33", value: 33}]
* CoralItems.create([null]); // [{label: "", value: ""}]
* CoralItems.create([{value: false}]); // [{label: "false", value: false}]
* CoralItems.create([{label: "a", value: "b"}]); // [{label: "a", value: "b"}]
* CoralItems.create("invalid parameter"); // null
*/
CoralItems.create = function(userItems) {
	if(Array.isArray(userItems)) {
		let len = userItems.length;
		let ary = new Array(len);
		for(let i = 0; i < len; ++i) {
			let userItem = userItems[i];
			if(userItem != null) {
				if (typeof userItem !== "object") {
					userItem = { value: userItem };
				}
			} else {
				userItem = {value: ""};
			}
			let label = userItem.label;
			if(!label && typeof label !== "string") { // 0, false, null, undefined, NaN
				userItem.label = userItem.value + "";
			}
			ary[i] = userItem;
		}
		return ary;
	}
	return null;
};

export default CoralItems;
export {CoralItems};
