/** Delay class combines {@link Deferred} and Timeout method together.
* @constructor
* @param {number=} ms
* @param {*=} ctx Context object that will passed to resolve func as the first parameter
*/
let Delay = function(ms, ctx){
	this.promise = new Promise(this._executor.bind(this));
	if(ctx != null) {
		this._ctx = ctx;
	}

	setTimeout(this._onTimeout.bind(this), ms > 0 ? ms : 0);
};
/** A shorthand for Creating a Delay class object
* @public
* @function
* @param {number=} ms
* @param {*=} ctx
* @return {Delay}
*/
Delay.for = function(ms, ctx) {
	return new Delay(ms, ctx);
};

/** @type {Promise<*>}
* @public
*/
Delay.prototype.promise = null;
/** @type {Function}
* @private
*/
Delay.prototype._resolve = null;
/** @type {Function}
* @private
*/
Delay.prototype._reject = null;

/** @type {number}
* @private
*/
Delay.prototype._timerId = 0;
/** @type {*}
* @public
*/
Delay.prototype._ctx = null; // Context object

/** @public */
Delay.prototype.resolve = function() {
	if(this._timerId) {
		this._onTimeout();
	}
};
/** @public */
Delay.prototype.reject = function() {
	if(this._timerId) {
		clearTimeout(this._timerId);
		this._timerId = 0;
		this._reject((this._ctx != null) ? this._ctx : this);
		this._ctx = null;
	}
};
/** @public
* @return {boolean}
*/
Delay.prototype.isTimeout = function() {
	return !this._timerId;
};

/** @private
* @param {Function} resolve
* @param {Function} reject
*/
Delay.prototype._executor = function(resolve, reject) {
	this._resolve = resolve;
	this._reject = reject;
};
/** @private */
Delay.prototype._onTimeout = function() {
	this._timerId = 0;
	this._resolve((this._ctx != null) ? this._ctx : this);
	this._ctx = null;
};

/** Create a promise of a Delay
* @public
* @function
* @param {number=} ms
* @param {*=} ctx
* @return {Promise}
*/
let promiseDelay = function(ms, ctx) {
	return Delay.for(ms, ctx).promise;
};

export default Delay;
export {Delay, promiseDelay};
