import {Ext} from './Ext.js';
import {EventDispatcher} from './EventDispatcher.js';

/** @constructor
* @extends {EventDispatcher}
*/
let ElementWrapper = function() {};
Ext.inherits(ElementWrapper, EventDispatcher);
/** @type {Element}
* @protected
* @ignore
*/
ElementWrapper.prototype._elem = null;


/** @public
* @return {Element}
*/
ElementWrapper.prototype.getElement = function() {
	return this._elem;
};
/** @public
* @param {Node|ElementWrapper} parent
*/
ElementWrapper.prototype.setParent = function(parent) {
	if(parent) {
		if(parent["getElement"]) {
			parent = /** @type{Node} */(parent["getElement"]());
		}
		parent.appendChild(this._elem);
	} else {
		parent = this._elem.parentNode;
		if(parent) {
			parent.removeChild(this._elem);
		}
	}
};
/** @public
* @return {!Object}
*/
ElementWrapper.prototype.getBoundingClientRect = function() {
	return this._elem.getBoundingClientRect();
};
/** Get mouse position relative to this element
* @public
* @param {Event} e MouseEvent should be given
* @param {Object=} retObj
* @return {!Object}
*/
ElementWrapper.prototype.getMousePosition = function(e, retObj) {
	if(!retObj) { retObj = {}; }
	let bounds = this.getBoundingClientRect();
	retObj["x"] = e.clientX - bounds["left"];
	retObj["y"] = e.clientY - bounds["top"];

	return /**@type {!Object}*/(retObj);
};

/** @public
* @override
* @param {string} type
* @param {Function} listener
*/
ElementWrapper.prototype.addEventListener = function(type, listener) {
	ElementWrapper.base(this, "addEventListener", type, listener);
	// TODO: This won't work with Custom Element
	if(this._elem["on" + type] !== undefined) { // eslint-disable-line
		this._elem.addEventListener(type, listener, false);
	}
};
/** @public
* @override
* @param {string} type
* @param {Function} listener
*/
ElementWrapper.prototype.removeEventListener = function(type, listener) {
	ElementWrapper.base(this, "removeEventListener", type, listener);
	if(this._elem["on" + type] !== undefined) { // eslint-disable-line
		this._elem.removeEventListener(type, listener, false);
	}
};

export default ElementWrapper;
export {ElementWrapper};
