import { Ext } from "./Ext.js";
import { ElementWrapper } from "./ElementWrapper.js";
import { ElfUtil } from "./ElfUtil.js";

/**
* @constructor
* @extends {ElementWrapper}
*/
let ExpanderIcon = function () {
	let elem;
	if (ExpanderIcon._iconName) {
		elem = document.createElement("ef-icon");
		elem.setAttribute("icon", ExpanderIcon._iconName);
	} else {
		elem = document.createElement("span");
		let fallback = document.createElement("span");
		fallback.className = "fallback-arrow";
		fallback.textContent = "\u25bc";
		elem.appendChild(fallback);
	}
	this._elem = elem;
	elem.classList.add("expander");

	elem.addEventListener("mousedown", ExpanderIcon._stopPropagation, false);
	elem.addEventListener("click", ExpanderIcon._stopPropagation, false);
};
Ext.inherits(ExpanderIcon, ElementWrapper);

/** @type {string}
* @private
*/
ExpanderIcon._iconName = ""; // static variable

/** @private
* @param {Event} e
*/
ExpanderIcon._stopPropagation = function(e) {
	e.stopPropagation();
};
/** @private
*/
ExpanderIcon._onThemeLoaded = function() {
	if(ElfUtil.isHaloTheme()) {
		ExpanderIcon._iconName = "down";
	}
};

/** @public
*/
ExpanderIcon.loadExpanderStyles = function() {
	if(!ExpanderIcon._iconName) {
		let themeName = ElfUtil.getThemeName();

		if(themeName) {
			ExpanderIcon._onThemeLoaded(); // synchronous
		} else {
			ElfUtil.getThemeColors().then(ExpanderIcon._onThemeLoaded).catch(ExpanderIcon._onThemeLoaded); // asynchronous
		}
	}
};

/** @private
* @param {Object} cell
* @param {boolean=} removal
* @return {ExpanderIcon} Return ExpanderIcon object only if Expander is newly created
*/
ExpanderIcon.renderOn = function (cell, removal) {
	let newExpander = null;
	if (removal) {
		cell.removeIcon();
	} else {
		let expander = cell._expanderIcon;
		if (!expander) {
			expander = cell._expanderIcon = newExpander = new ExpanderIcon();
		}

		cell.updateIcon(expander.getElement());
	}
	return newExpander;
};

/** @public
*/
ExpanderIcon.prototype.expand = function () {
	this._elem.classList.remove("closed");
};
/** @public
*/
ExpanderIcon.prototype.collapse = function () {
	this._elem.classList.add("closed");
};
/** @public
* @return {boolean}
*/
ExpanderIcon.prototype.isExpanded = function () {
	return !this._elem.classList.contains("closed");
};
/** @public
* @return {boolean}
*/
ExpanderIcon.prototype.isCollapsed = function () {
	return this._elem.classList.contains("closed");
};
/** @public
* @param {boolean=} collapsed Set to true to force collapsing state. Set to false to force expanding state
*/
ExpanderIcon.prototype.toggle = function (collapsed) {
	if(collapsed == null) {
		collapsed = !this.isCollapsed();
	}
	if(collapsed) {
		this.collapse();
	} else {
		this.expand();
	}
};


export default ExpanderIcon;
export { ExpanderIcon };
