import { FilterOperators, OperatorFunctions } from "./FilterOperators.js";
import { ExpressionParser } from "./ExpressionParser.js";

declare namespace FilterBuilder {

    type Options = {
        field?: (string|null)|null,
        formatter?: ((...params: any[]) => any)|null,
        formattedField?: string|null,
        useUTCTime?: boolean|null,
        rawDataAccessor?: ((...params: any[]) => any)|null,
        formattedDataAccessor?: ((...params: any[]) => any)|null
    };

    type Condition = any[]|null;

    type Conditions = (FilterBuilder.Condition)[]|null;

}

declare class FilterBuilder {

    constructor(options?: FilterBuilder.Options|null);

    public reset(): void;

    public init(options?: FilterBuilder.Options|null): void;

    public setFieldDefinition(fieldName: string, formatter?: ((...params: any[]) => any)|null, formattedField?: string|null): void;

    public setDefaultUTCUsage(useUTCTime: boolean): void;

    public setDataAccessors(rawDataAccessor?: ((...params: any[]) => any)|null, formattedDataAccessor?: ((...params: any[]) => any)|null): void;

    public addCondition(oper: string, value: string|number|boolean|Date|null, connector?: string|null, useUTCTime?: boolean|null): boolean;

    public addConditions(conditions: FilterBuilder.Conditions|null): number;

    public getConditions(): FilterBuilder.Conditions|null;

    public buildFilter(): ((...params: any[]) => any)|null;

    public buildMonitorFilter(filterStr: string, field: string, formatter?: ((...params: any[]) => any)|null, formattedField?: string|null): ((...params: any[]) => any)|null;

    public parse(condition: any, field?: string|null, formatter?: ((...params: any[]) => any)|null, formattedField?: string|null, rawDataAccessor?: ((...params: any[]) => any)|null, formattedDataAccessor?: ((...params: any[]) => any)|null): ((...params: any[]) => any)|null;

}

declare function stringToDateObject(str: any): any;

declare function buildFilterFromObjectMap(obj: any, field: string, rawDataAccessor?: ((...params: any[]) => any)|null): ((...params: any[]) => any)|null;

export default FilterBuilder;
export { FilterBuilder, buildFilterFromObjectMap, stringToDateObject };
