/** @namespace */
let Icon = {};

/** @public
* @function
* @param {string=} iconName
* @param {string=} className
* @return {!Element} ef-icon element
*/
Icon.create = function(iconName, className) {
	let icon = document.createElement("ef-icon");
	if(className) {
		icon.className = className;
	}
	if(iconName) {
		icon.setAttribute("icon", iconName);
	}
	return icon;
};

/** Create a container element with an icon element in it.
* @public
* @function
* @param {string=} iconName
* @param {string=} className Class name for the containing box
* @param {(string|Element)=} boxRef
* @return {!Element} Element that contains the newly created icon
*/
Icon.box = function(iconName, className, boxRef) {
	let boxElem = null;
	if(boxRef) {
		if(typeof boxRef == "string") {
			boxElem = document.createElement(boxRef);
		} else if(boxRef.nodeType === 1) {
			boxElem = /** @type{Element} */(boxRef);
		}
	}
	if(!boxElem) {
		boxElem = document.createElement("div");
	}
	if(className) {
		boxElem.className = className;
	}

	boxElem.appendChild(Icon.create(iconName));
	return boxElem;
};

/** @public
* @function
* @param {Element} iconElem ef-icon element
* @param {string=} iconName
*/
Icon.change = function(iconElem, iconName) {
	let curIcon = iconElem.getAttribute("icon");
	if(curIcon !== iconName) {
		iconElem.setAttribute("icon", iconName);
	}
};

export default Icon;
export {Icon};
