import { Ext } from './Ext.js';
import { EventDispatcher } from './EventDispatcher.js';
import { Dom } from './Dom.js';

declare namespace Popup {

    type Options = {
        positioning?: Popup.Positioning|null,
        autoHiding?: boolean|null,
        hoverToShow?: boolean|null,
        autoClipping?: boolean|null,
        attachedElement?: Element|null,
        focusElement?: Element|null,
        focusElements?: (Element)[]|null,
        parentElement?: Element|null,
        contentElement?: Element|null,
        popupChild?: Popup|null,
        uiBlocking?: boolean|null,
        hideOnScroll?: boolean|null,
        zIndexReference?: Element|null
    };

    type Positioning = string;

}

declare class Popup extends EventDispatcher {

    constructor(popupElem?: (Element|Popup.Options)|null, options?: Popup.Options|null);

    public dispose(): void;

    public getElement(): Element|null;

    public appendChild(childElem: Element|null): void;

    public disable(opt_bool?: boolean|null): void;

    public enable(opt_bool?: boolean|null): void;

    public init(options?: Popup.Options|null): void;

    public attachTo(elem: Element|null, positioning?: Popup.Positioning|null): void;

    public getAttachedElement(): Element|null;

    public setAttachedElement(elem: Element|null, positioning?: Popup.Positioning|null): void;

    public addFocusElement(elem: Element|null): void;

    public removeFocusElement(elem: Element|null): void;

    public addPopupChild(popup?: Popup|null): Popup;

    public setPopupGap(gap?: number|null): void;

    public setParentElement(parentElement: Element|null): Element|null;

    public getZIndexReference(): Element|null;

    public setZIndexReference(elem: Element|null): void;

    public updateZIndex(): void;

    public show(opt_shown?: boolean|null, parentElement?: Element|null): void;

    public hide(opt_hidden?: boolean|null): void;

    public toggle(force?: boolean|null, parentElement?: Element|null): void;

    public isShown(): boolean;

    public isHidden(): boolean;

    public contains(elems: Element|(Element)[]|null): boolean;

    public setPositioning(posType: Popup.Positioning|null): void;

    public disableAutoHiding(opt_disabled?: boolean|null): void;

    public disableAutoRepositioning(opt_disabled?: boolean|null): void;

    public disableAutoClipping(opt_disabled?: boolean|null): void;

    public disableHideOnScroll(opt_disabled?: boolean|null): void;

    public updatePosition(fallback?: boolean|null): void;

    public enableUIBlocking(bool?: boolean|null): boolean;

    public getOverlayElement(): Element|null;

}

declare function containElements(parentElem: Element|null, e: MouseEvent|null): boolean;

export { Popup, containElements };
export default Popup;
