import { Dom } from "./Dom.js";
import { Ext } from "./Ext.js";
import { ElementWrapper } from "./ElementWrapper.js";
import { SubTable } from "./SubTable.js";

declare namespace Table {

    type Options = {
        colCount?: number|null,
        rowCount?: number|null,
        cellWidth?: number|null,
        cellHeight?: number|null,
        width?: number|null,
        height?: number|null,
        header?: number|null,
        footer?: number|null
    };

}

declare class Table extends ElementWrapper {

    constructor(elem?: Element|null, options?: Table.Options|null);

    public getTableElement(): Element|null;

    public setCRWH(col: number, row: number, width: number, height: number): void;

    public init(options: Table.Options|null): void;

    public addColumns(count?: number|null): (Element)[]|null;

    public removeColumns(opt_count?: number|null): void;

    public removeAllColumns(): void;

    public setColumnCount(val: number): void;

    public addRows(count?: number|null): (Element)[];

    public removeRows(count?: number|null): void;

    public removeAllRows(): void;

    public setRowCount(val: number): void;

    public getColumnCount(): number;

    public getRowCount(): number;

    public getCell(c: number, r: number): Element|null;

    public getCellsInColumn(c: number): (Element)[]|null;

    public getCellsInRow(r: number): (Element)[]|null;

    public getAllCells(): (Element)[];

    public getAllRows(): (Element)[];

    public getRows(): (Element)[];

    public getRow(r: number): Element|null;

    public spanBlock(c1: number, c2: number, r1: number, r2: number): Element|null;

    public spanHorizontally(r: number, bool: boolean): Element|null;

    public setColMinWidths(val: number|string|(string|number)[]|null, colIndex?: number|null): void;

    public setColumnWidths(val: number|string|(string|number)[]|null, colIndex?: number|null): void;

    public setColBackgroundColors(val: string|(string)[]|null, colIndex?: number|null): void;

    public setColBGColors(val: string|(string)[]|null, colIndex?: number|null): void;

    public setColBorders(val: number|string|(number|string)[]|null, colIndex?: number|null): void;

    public setSize(width: number, height: number): void;

    public setCellSize(defaultWidth: number, defaultHeight: number): void;

    public setDefaultColumnWidth(val?: (number|null)|null): void;

    public getDefaultColumnWidth(): number|null|null;

    public setDefaultRowHeight(val?: (number|null)|null): void;

    public getDefaultRowHeight(): number|null|null;

    public distributeColumnWidth(): void;

    public log(opt_rowLimit?: number|null): (any)[]|null;

    public getTextContents(): (string)[][];

    public getCellTextContent(c: number, r: number): string;

    public getColumnTextContent(c: number): string;

    public getRowTextContent(r: number): string;

    public getTableTextContent(): string;

    public fixateTableWidth(opt_bool?: boolean|null): void;

    public loosenTableWidth(opt_bool?: boolean|null): void;

    public addHeaderRows(opt_count?: number|null): (Element)[];

    public getHeaderCell(c: number, r: number): Element|null;

    public getAllHeaderCells(): (Element)[];

    public removeHeaderRows(opt_count?: number|null): void;

    public addFooterRows(opt_count?: number|null): (Element)[];

    public getFooterCell(c: number, r: number): Element|null;

    public getAllFooterCells(): (Element)[];

    public removeFooterRows(opt_count?: number|null): void;

    public getColumnIndex(e: Event|null): number;

    public getRowIndex(e: Event|null): number;

    public getColumnLeft(colIndex: number, rowIndex?: number|null): number;

    public getRowTop(rowRef: number|Element|null): number;

    public getCellPosition(cell: Element|null, ret_obj?: any): any;

    public cloak(tblElem: Element|null): void;

    public getBody(): SubTable|null;

    public getHeader(): SubTable|null;

    public getFooter(): SubTable|null;

}

export default Table;
export { Table };
