import {ADCSubscription} from "./ADCSubscription.js";

/** @namespace */
let ADCService = {};

/** @type {Object}
* @private
*/
ADCService._adcSubscription = null;
/** @type {string}
* @private
*/
ADCService._uuid = "GUEST";
/** @type {string}
* @private
*/
ADCService._adcProductId = "ANONYMOUS";
/** @type {string}
* @private
*/
ADCService._adcUrl = "/datacloud/rest/select";

/** @public
* @return {Object}
*/
ADCService.create = function() {
	let adcSub = ADCService._adcSubscription;
	if (!adcSub) {
		if (window["JET"]) { // ADC only need JET
			adcSub = ADCService._adcSubscription = new ADCSubscription();
			ADCService._updateDataCloudSettings();
		}
	}
	return adcSub || null;
};

/** @public
* @param {string=} uuid
* @param {string=} productId
* @param {string=} adcUrl
* @param {string=} lang
*/
ADCService.setDataCloudSettings = function(uuid, productId, adcUrl, lang) {
	if(uuid) {
		ADCService._uuid = uuid;
	}
	if(productId) {
		ADCService._adcProductId = productId;
	}
	if(adcUrl) {
		ADCService._adcUrl = adcUrl;
	}
	if(lang) {
		ADCService._lang = lang;
	}
	// In case set the settings after the service was created.
	ADCService._updateDataCloudSettings();
};
/** @private */
ADCService._updateDataCloudSettings = function() {
	if(ADCService._adcSubscription) {
		ADCService._adcSubscription.setDataCloudSettings(
			ADCService._uuid,
			ADCService._adcProductId,
			ADCService._adcUrl,
			ADCService._lang
		);
	}
};


export {ADCService};
