import {TSISubscription} from "./TSISubscription.js";

/** @public
* @namespace
*/
let TSIService = {};

/** @type {Object}
* @private
*/
TSIService._tsiSubscription = null;

/** @type {string}
* @private
*/
TSIService._tsiSharedServiceUrl = "/Apps/shared-services/service/TsMultiRICTimeSeries/GetMultiRICTimeSeries";
/** @type {string}
* @private
*/
TSIService._tsiTaServiceUrl = "/Apps/TAService";

/** @public
* @return {Object}
*/
TSIService.create = function() {
	let tsiSub = TSIService._tsiSubscription;
	if (!tsiSub) {
		tsiSub = new TSISubscription();
		TSIService._tsiSubscription = tsiSub;
		TSIService.setTSISettings();
	}
	return tsiSub;
};

/** @public
* @param {string=} sharedServiceUrl
* @param {string=} taServiceUrl
*/
TSIService.setTSISettings = function(sharedServiceUrl, taServiceUrl) {
	if(sharedServiceUrl) {
		TSIService._tsiSharedServiceUrl = sharedServiceUrl;
	}
	if(taServiceUrl) {
		TSIService._tsiTaServiceUrl = taServiceUrl;
	}
	if(TSIService._tsiSubscription) { // In case set the settings after the service was created.
		TSIService._tsiSubscription.setTSISettings(TSIService._tsiSharedServiceUrl, TSIService._tsiTaServiceUrl);
	}
};

export {TSIService};
