/**
 * @namespace Internal
 * @ignore
 */
let Internal = {};

/**
 * @description This method calculates Restricted Indicator (RI) status from RI_FLAG, RI_CODE, STATUS and PRC_QL2 fields
 * @param  {string} riFlag
 * @param  {*} riCode
 * @param  {number} status
 * @param  {string} prcQl2
 * @return {Object}
 * @ignore
 */
Internal["_RI_STATUS"] = function(riFlag, riCode, status, prcQl2) {
	let val = "";
	let isDelayed = false;
	let delayedText;

	if (riFlag) {
		if (riFlag === "R" || riFlag === "E" || riFlag === "Q") {
			val = riFlag;
		} else {
			val = "I";
		}
	} else if(riCode) {
		val = "I";
	}

	if(status === 5) { // Delayed RIC
		isDelayed = true;
	}

	if (prcQl2 && typeof prcQl2 === "string") {
		delayedText = prcQl2.toLowerCase();
	}

	return {
		"restricted": val,
		"isDelayed": isDelayed,
		"delayedText": delayedText
	};
};

/**
 * @description This method combines Bid / Offer
 * @param  {string|null} bid
 * @param  {string} bidFormatted
 * @param  {string|null} ask
 * @param  {string} askFormatted
 * @return {{bid:(string|null), offer:(string|null)}}
 */
Internal["_BID_OFFER"] = function(bid, bidFormatted, ask, askFormatted) {

	return {
		"bid": bidFormatted,
		"offer": askFormatted
	};
};

/**
 * @ignore
 * @description Calculate the percentage from low, high and last value.
 * @param {number} lowVal
 * @param {number} lastVal
 * @param {number} highVal
 * @return {number}
 */
Internal["_PERCENT"] = function (lowVal, lastVal, highVal) {
	if ((!lowVal && lowVal !== 0) ||
		(!lastVal && lastVal !== 0) ||
		(!highVal && highVal !== 0)) {
		return NaN;
	}
	if (highVal === lowVal) {
		return 50;
	}
	let percent = (lastVal - lowVal) / (highVal - lowVal);
	if (percent < 0) {
		return 0;
	}
	if (percent > 1) {
		return 100;
	}
	return percent * 100;
};

export default Internal;
export { Internal };
