/**
 * @namespace GridMath
 * @description
 * {lang=en} Mathmatic functions
 * {lang=ja} 数学関数
 * {lang=zh} 数学函数
 * {lang=hant} 數學函數
 * {lang=ko} 수학 함수
 * {lang=ru} Mathmatic functions
 * {lang=de} Mathematische Funktionen
 */
let GridMath = {};

/**
 * @function ABS
 * @memberof GridMath
 * @description
 *    {lang=en} Returns the absolute value of a number
 *    {lang=ja} 数値の絶対値を返します
 *    {lang=zh} 返回一个数字的绝对值
 *    {lang=hant} 返回一個數字的絕對值
 *    {lang=ko} 숫자의 절댓값을 반환
 *    {lang=ru} Returns the absolute value of a number
 *    {lang=de} Gibt den absoluten Wert einer Zahl aus
 * @param  {number} x
 *    {lang=en} Number
 *    {lang=ja} 数値
 *    {lang=zh} 数字
 *    {lang=hant} 數字
 *    {lang=ko} 숫자
 *    {lang=ru} Number
 *    {lang=de} Zahl
 * @return {number}
 * @example
 *    =ABS(-0.005)
 */
GridMath["ABS"] = Math.abs;

/**
 * @function ACOS
 * @memberof GridMath
 * @description
 *    {lang=en} Returns the arccosine of a number
 *    {lang=ja} 数値のアークコサインを返します
 *    {lang=zh} 返回一个数字的反余弦
 *    {lang=hant} 返回一個數字的反餘弦
 *    {lang=ko} 숫자의 아크코사인을 반환
 *    {lang=ru} Returns the arccosine of a number
 *    {lang=de} Gibt den Arkussinus einer Zahl aus
 * @param  {number} x
 *    {lang=en} Number
 *    {lang=ja} 数値
 *    {lang=zh} 数字
 *    {lang=hant} 數字
 *    {lang=ko} 숫자
 *    {lang=ru} Number
 *    {lang=de} Zahl
 * @return {number}
 * @example
 *    =ACOS(0.5)
 */
GridMath["ACOS"] = Math.acos;

/**
 * @function ACOSH
 * @memberof GridMath
 * @description
 *    {lang=en} Returns the inverse hyperbolic cosine of a number
 *    {lang=ja} 数値の双曲線逆余弦を返します
 *    {lang=zh} 返回一个数字的反双曲余弦
 *    {lang=hant} 返回一個數字的反雙曲餘弦
 *    {lang=ko} 숫자의 역쌍곡코사인을 반환
 *    {lang=ru} Returns the inverse hyperbolic cosine of a number
 *    {lang=de} Gibt den umgekehrten hyperbolischen Kosinus einer Zahl aus
 * @param  {number} x
 *    {lang=en} Number
 *    {lang=ja} 数値
 *    {lang=zh} 数字
 *    {lang=hant} 數字
 *    {lang=ko} 숫자
 *    {lang=ru} Number
 *    {lang=de} Zahl
 * @return {number}
 * @example
 *    =ACOSH(0.5)
 */
GridMath["ACOSH"] = function(x) {
	return Math.log(x + Math.sqrt(x * x - 1));
};

/**
 * @function ASIN
 * @memberof GridMath
 * @description
 *    {lang=en} Returns the arcsine of a numberr
 *    {lang=ja} 数値のアークサインを返します
 *    {lang=zh} 返回一个数字的反正弦
 *    {lang=hant} 返回一個數字的反正弦
 *    {lang=ko} 숫자의 아크사인을 반환
 *    {lang=ru} Returns the arcsine of a numberr
 *    {lang=de} Gibt den Arkussinus einer Zahl aus
 * @param  {number} x
 *    {lang=en} Number
 *    {lang=ja} 数値
 *    {lang=zh} 数字
 *    {lang=hant} 數字
 *    {lang=ko} 숫자
 *    {lang=ru} Number
 *    {lang=de} Zahl
 * @return {number}
 * @example
 *    =ASIN(0.5)
 */
GridMath["ASIN"] = Math.asin;

/**
 * @function ASINH
 * @memberof GridMath
 * @description
 *    {lang=en} Returns the inverse hyperbolic sine of a number
 *    {lang=ja} 数値の双曲線逆正弦を返します
 *    {lang=zh} 返回一个数字的反双曲正弦
 *    {lang=hant} 返回一個數字的反雙曲正弦
 *    {lang=ko} 숫자의 역쌍곡사인을 반환
 *    {lang=ru} Returns the inverse hyperbolic sine of a number
 *    {lang=de} Gibt den umgekehrten hyperbolischen Sinus einer Zahl aus
 * @param  {number} x
 *    {lang=en} Number
 *    {lang=ja} 数値
 *    {lang=zh} 数字
 *    {lang=hant} 數字
 *    {lang=ko} 숫자
 *    {lang=ru} Number
 *    {lang=de} Zahl
 * @return {number}
 * @example
 *    =ASINH(0.5)
 */
GridMath["ASINH"] = function(x) {
	if (x === -Infinity) {
		return x;
	} else {
		return Math.log(x + Math.sqrt(x * x + 1));
	}
};

/**
 * @function ATAN
 * @memberof GridMath
 * @description
 *    {lang=en} Returns the arctangent of a number
 *    {lang=ja} 数値のアークタンジェントを返します
 *    {lang=zh} 返回一个数字的反正切
 *    {lang=hant} 返回一個數字的反正切
 *    {lang=ko} 숫자의 아크탄젠트를 반환
 *    {lang=ru} Returns the arctangent of a number
 *    {lang=de} Gibt den Arkustangens einer Zahl aus
 * @param  {number} x
 *    {lang=en} Number
 *    {lang=ja} 数値
 *    {lang=zh} 数字
 *    {lang=hant} 數字
 *    {lang=ko} 숫자
 *    {lang=ru} Number
 *    {lang=de} Zahl
 * @return {number}
 * @example
 *    =ATAN(-1)
 */
GridMath["ATAN"] = Math.atan;

/**
 * @function ATAN2
 * @memberof GridMath
 * @description
 *    {lang=en} Returns the arctangent from x- and y-coordinates
 *    {lang=ja} 指定された x-y 座標のアークタンジェントを返します
 *    {lang=zh} 返回 x- 和 y-坐标的反正切
 *    {lang=hant} 返回 x- 和 y-坐標的反正切
 *    {lang=ko} x좌표와 y좌표에서 아크탄젠트를 반환
 *    {lang=ru} Returns the arctangent from x- and y-coordinates
 *    {lang=de} Gibt den Arkustangens aus x- und y-Koordinaten aus
 * @param  {number} x
 *    {lang=en} Number
 *    {lang=ja} 数値
 *    {lang=zh} 数字
 *    {lang=hant} 數字
 *    {lang=ko} 숫자
 *    {lang=ru} Number
 *    {lang=de} Zahl
 * @param  {number} y
 *    {lang=en} Number
 *    {lang=ja} 数値
 *    {lang=zh} 数字
 *    {lang=hant} 數字
 *    {lang=ko} 숫자
 *    {lang=ru} Number
 *    {lang=de} Zahl
 * @return {number}
 * @example
 *    =ATAN2(2,-1)
 */
GridMath["ATAN2"] = function(x, y) {
	return Math.atan2(y, x);
};

/**
 * @function ATANH
 * @memberof GridMath
 * @description
 *    {lang=en} Returns the inverse hyperbolic tangent of a number
 *    {lang=ja} 数値の双曲線逆正接を返します
 *    {lang=zh} 返回一个数字的反双曲正切
 *    {lang=hant} 返回一個數字的反雙曲正切
 *    {lang=ko} 숫자의 역쌍곡탄젠트를 반환
 *    {lang=ru} Returns the inverse hyperbolic tangent of a number
 *    {lang=de} Gibt den umgekehrten hyperbolischen Tangens einer Zahl aus
 * @param  {number} x
 *    {lang=en} Number
 *    {lang=ja} 数値
 *    {lang=zh} 数字
 *    {lang=hant} 數字
 *    {lang=ko} 숫자
 *    {lang=ru} Number
 *    {lang=de} Zahl
 * @return {number}
 * @example
 *    =ATANH(-1)
 */
GridMath["ATANH"] = function(x) {
	return Math.log((1 + x) / (1 - x)) / 2;
};

/**
 * @function CEILING
 * @memberof GridMath
 * @description
 *    {lang=en} Rounds a number to the nearest integer or to the nearest multiple of significance
 *    {lang=ja} 指定された基準値の倍数のうち、最も近い値に数値を切り上げます
 *    {lang=zh} 将一个数字四舍五入到最近的整数或最近的指定基数的倍数
 *    {lang=hant} 將一個數字四捨五入到最近的整數或最近的指定基數的倍數
 *    {lang=ko} 숫자를 가장 가까운 정수 또는 가장 가까운 significance의 배수로 반올림
 *    {lang=ru} Rounds a number to the nearest integer or to the nearest multiple of significance
 *    {lang=de} Rundet eine Zahl auf die nächste Ganzzahl oder auf das nächste Vielfache der Signifikanz
 * @param  {number} num
 *    {lang=en} Number
 *    {lang=ja} 数値
 *    {lang=zh} 数字
 *    {lang=hant} 數字
 *    {lang=ko} 숫자
 *    {lang=ru} Number
 *    {lang=de} Zahl
 * @param  {number} multiple
 *    {lang=en} The multiple to use when rounding
 *    {lang=ja} 基準値の倍数
 *    {lang=zh} 四舍五入时使用的乘数
 *    {lang=hant} 四捨五入時使用的乘數
 *    {lang=ko} 반올림할 때 사용할 배수
 *    {lang=ru} The multiple to use when rounding
 *    {lang=de} Das beim Runden zu verwendende Vielfache
 * @return {number}
 * @example
 *    =CEILING(4.88291516)
 */
GridMath["CEILING"] = function(num, multiple) { //
	if(!multiple) {
		multiple = 1;
	}
	num /= multiple;
	return Math.ceil(num) * multiple;
};

/**
 * @function CEIL
 * @memberof GridMath
 * @description
 *    {lang=en} Rounds a number to the nearest integer or to the nearest multiple of significance
 *    {lang=ja} 指定された基準値の倍数のうち、最も近い値に数値を切り上げます
 *    {lang=zh} 将一个数字四舍五入到最近的整数或最近的指定基数的倍数
 *    {lang=hant} 將一個數字四捨五入到最近的整數或最近的指定基數的倍數
 *    {lang=ko} 숫자를 가장 가까운 정수 또는 가장 가까운 significance의 배수로 반올림
 *    {lang=ru} Rounds a number to the nearest integer or to the nearest multiple of significance
 *    {lang=de} Rundet eine Zahl auf die nächste Ganzzahl oder auf das nächste Vielfache der Signifikanz
 * @param  {number} num
 *    {lang=en} Number
 *    {lang=ja} 数値
 *    {lang=zh} 数字
 *    {lang=hant} 數字
 *    {lang=ko} 숫자
 *    {lang=ru} Number
 *    {lang=de} Zahl
 * @param  {number} multiple
 *    {lang=en} The multiple to use when rounding
 *    {lang=ja} 基準値の倍数
 *    {lang=zh} 四舍五入时使用的乘数
 *    {lang=hant} 四捨五入時使用的乘數
 *    {lang=ko} 반올림할 때 사용할 배수
 *    {lang=ru} The multiple to use when rounding
 *    {lang=de} Das beim Runden zu verwendende Vielfache
 * @return {number}
 * @example
 *    =CEIL(4.88291516)
 */
GridMath["CEIL"] = GridMath["CEILING"];

/**
 * @function COMBIN
 * @memberof GridMath
 * @description
 *    {lang=en} Returns the number of combinations for a given number of objects
 *    {lang=ja} すべての項目から指定された個数を選択するときの組み合わせ数を返します
 *    {lang=zh} 返回给定数目的对象的组合数
 *    {lang=hant} 返回給定數目的對象的組合數
 *    {lang=ko} 주어진 개체 수에 대한 조합 수를 반환
 *    {lang=ru} Returns the number of combinations for a given number of objects
 *    {lang=de} Gibt die Anzahl der Kombinationen für eine gegebene Anzahl von Objekten aus
 * @param  {number} num
 *    {lang=en} Number
 *    {lang=ja} 数値
 *    {lang=zh} 数字
 *    {lang=hant} 數字
 *    {lang=ko} 숫자
 *    {lang=ru} Number
 *    {lang=de} Zahl
 * @param  {number} num_chosen
 *    {lang=en} Number
 *    {lang=ja} 数値
 *    {lang=zh} 数字
 *    {lang=hant} 數字
 *    {lang=ko} 숫자
 *    {lang=ru} Number
 *    {lang=de} Zahl
 * @return {number}
 * @example
 *    =COMBIN(10,3)
 */
GridMath["COMBIN"] = function(num, num_chosen) {
	let fact = GridMath["FACT"];
	return fact(num) / (fact(num_chosen) * fact(num - num_chosen));
};

/**
 * @function COS
 * @memberof GridMath
 * @description
 *    {lang=en} Returns the cosine of a number
 *    {lang=ja} 数値のコサインを返します
 *    {lang=zh} 返回一个数字的余弦
 *    {lang=hant} 返回一個數字的餘弦
 *    {lang=ko} 숫자의 코사인을 반환
 *    {lang=ru} Returns the cosine of a number
 *    {lang=de} Gibt den Kosinus einer Zahl aus
 * @param  {number} x
 *    {lang=en} Number
 *    {lang=ja} 数値
 *    {lang=zh} 数字
 *    {lang=xh-Hant} 數字
 *    {lang=ko} 숫자
 *    {lang=ru} Number
 *    {lang=de} Zahl
 * @return {number}
 * @example
 *    =COS(PI()/3)
 */
GridMath["COS"] = Math.cos; //

/**
 * @function COSH
 * @memberof GridMath
 * @description
 *    {lang=en} Returns the hyperbolic cosine of a number
 *    {lang=ja} 数値の双曲線余弦を返します
 *    {lang=zh} 返回一个数字的双曲余弦
 *    {lang=hant} 返回一個數字的雙曲餘弦
 *    {lang=ko} 숫자의 쌍곡코사인을 반환
 *    {lang=ru} Returns the hyperbolic cosine of a number
 *    {lang=de} Gibt den hyperbolischen Kosinus einer Zahl aus
 * @param  {number} x
 *    {lang=en} Number
 *    {lang=ja} 数値
 *    {lang=zh} 数字
 *    {lang=hant} 數字
 *    {lang=ko} 숫자
 *    {lang=ru} Number
 *    {lang=de} Zahl
 * @return {number}
 * @example
 *    =COSH(3)
 */
GridMath["COSH"] = function(x) {
	let y = Math.exp(x);
	return (y + 1 / y) / 2;
};

/**
 * @function DEGREES
 * @memberof GridMath
 * @description
 *    {lang=en} Converts radians to degrees
 *    {lang=ja} ラジアンで表された角度を度に変換します
 *    {lang=zh} 将弧度转换成角度
 *    {lang=hant} 將弧度轉換成角度
 *    {lang=ko} 라디안을 도로 변환
 *    {lang=ru} Converts radians to degrees
 *    {lang=de} Wandelt Radianten in Grad um
 * @param  {number} radians
 *    {lang=en} Number
 *    {lang=ja} 数値
 *    {lang=zh} 数字
 *    {lang=hant} 數字
 *    {lang=ko} 숫자
 *    {lang=ru} Number
 *    {lang=de} Zahl
 * @return {number}
 * @example
 *    =DEGREES(1)
 */
GridMath["DEGREES"] = function(radians) {
	return radians / (Math.PI / 180);
};

/**
 * @function EVEN
 * @memberof GridMath
 * @description
 *    {lang=en} Rounds a number up to the nearest even integer
 *    {lang=ja} 数値を切り上げ、最も近い偶数にします
 *    {lang=zh} 将一个数字向上舍入到最近的偶数
 *    {lang=hant} 將一個數字向上舍入到最近的偶數
 *    {lang=ko} 가장 가까운 짝수인 정수로 숫자를 올림
 *    {lang=ru} Rounds a number up to the nearest even integer
 *    {lang=de} Rundet eine Zahl auf die nächste gerade Ganzzahl auf
 * @param  {number} num
 *    {lang=en} Number
 *    {lang=ja} 数値
 *    {lang=zh} 数字
 *    {lang=hant} 數字
 *    {lang=ko} 숫자
 *    {lang=ru} Number
 *    {lang=de} Zahl
 * @return {number}
 * @example
 *    =EVEN(3.152)
 */
GridMath["EVEN"] = function(num) {
	num = Math.ceil(num);
	if(num % 2) {
		return (num < 0) ? num - 1 : num + 1;
	}
	return num;
};

/**
 * @function EXP
 * @memberof GridMath
 * @description
 *    {lang=en} Returns e^x, where x is the argument, and e is Euler's number (also known as Napier's constant), the base of the natural logarithms.
 *    {lang=ja} 指定された e (オイラー数又はナピア constant) を底とする数値のべき乗 e^x を返します (x は引数)。
 *    {lang=zh} 返回 e 的 x 次方, 其中 x 是参数，e 是欧拉指数 (也叫做 Napier 常数), 是自然对数的的底。
 *    {lang=hant} 返回 e 的 x 次方, 其中 x 是參數，e 是歐拉指數 (也叫做 Napier 常數), 是自然對數的的底。
 *    {lang=ko} e^x를 반환합니다. 여기서 x는 인수이고 e는 자연로그의 밑인 오일러의 수(네이피어 상수라고도 함)입니다.
 *    {lang=ru} Returns e^x, where x is the argument, and e is Euler's number (also known as Napier's constant), the base of the natural logarithms.
 *    {lang=de} Gibt e^x aus, wobei x das Argument und e die Euler-Zahl (auch als Napier-Konstante bekannt), die Basis der natürlichen Logarithmen, ist.
 * @param  {number} x
 *    {lang=en} Number
 *    {lang=ja} 数値
 *    {lang=zh} 数字
 *    {lang=hant} 數字
 *    {lang=ko} 숫자
 *    {lang=ru} Number
 *    {lang=de} Zahl
 * @return {number}
 * @example
 *    =EXP(1)
 */
GridMath["EXP"] = Math.exp; // Returns e raised to the power of a given number

/**
 * @function FACT
 * @memberof GridMath
 * @description
 *    {lang=en} Returns the factorial of a number
 *    {lang=ja} 数値の階乗を返します
 *    {lang=zh} 返回一个数字的阶乘
 *    {lang=hant} 返回一個數字的階乘
 *    {lang=ko} 숫자의 계승을 반환
 *    {lang=ru} Returns the factorial of a number
 *    {lang=de} Gibt den faktoriellen Wert einer Zahl aus
 * @param  {number} num
 *    {lang=en} Number
 *    {lang=ja} 数値
 *    {lang=zh} 数字
 *    {lang=hant} 數字
 *    {lang=ko} 숫자
 *    {lang=ru} Number
 *    {lang=de} Zahl
 * @return {number}
 * @example
 *    =FACT(10)
 */
GridMath["FACT"] = function(num) {
	if(num < 0) { return NaN; }
	num = Math.floor(num);
	let ret = 1;
	while(num > 1) {
		ret *= num;
		--num;
	}
	return ret;
};

/**
 * @function FACTDOUBLE
 * @memberof GridMath
 * @description
 *    {lang=en} Returns the double factorial of a number
 *    {lang=ja} 数値の二重階乗を返します
 *    {lang=zh} 返回一个数字的双阶乘
 *    {lang=hant} 返回一個數字的雙階乘
 *    {lang=ko} 숫자의 겹계승을 반환
 *    {lang=ru} Returns the double factorial of a number
 *    {lang=de} Gibt den doppelten faktoriellen Wert einer Zahl aus
 * @param  {number} num
 *    {lang=en} Number
 *    {lang=ja} 数値
 *    {lang=zh} 数字
 *    {lang=hant} 數字
 *    {lang=ko} 숫자
 *    {lang=ru} Number
 *    {lang=de} Zahl
 * @return {number}
 * @example
 *    =FACTDOUBLE(10)
 */
GridMath["FACTDOUBLE"] = function(num) {
	if(num < 0) { return NaN; }
	num = Math.floor(num);
	let ret = 1;
	while(num > 1) {
		ret *= num;
		num -= 2;
	}
	return ret;
};

/**
 * @function FLOOR
 * @memberof GridMath
 * @description
 *    {lang=en} Rounds a number down, toward zero
 *    {lang=ja} 指定された基準値の倍数のうち、最も近い値に数値を切り捨てます
 *    {lang=zh} 对一个数字四舍五入，靠近零值
 *    {lang=hant} 對一個數字四捨五入，靠近零值
 *    {lang=ko} 숫자를 0을 향해 내림
 *    {lang=ru} Rounds a number down, toward zero
 *    {lang=de} Rundet eine Zahl ab, auf Null
 * @param  {number} num
 *    {lang=en} Number
 *    {lang=ja} 数値
 *    {lang=zh} 数字
 *    {lang=hant} 數字
 *    {lang=ko} 숫자
 *    {lang=ru} Number
 *    {lang=de} Zahl
 * @param  {number} multiple
 *    {lang=en} The multiple to use when rounding
 *    {lang=ja} 基準値の倍数
 *    {lang=zh} 四舍五入时使用的乘数
 *    {lang=hant} 四捨五入時使用的乘數
 *    {lang=ko} 반올림할 때 사용할 배수
 *    {lang=ru} The multiple to use when rounding
 *    {lang=de} Das beim Runden zu verwendende Vielfache
 * @return {number}
 * @example
 *    =FLOOR(4.882961516)
 */
GridMath["FLOOR"] = function(num, multiple) {
	if(!multiple) {
		multiple = 1;
	}
	num /= multiple;
	return Math.floor(num) * multiple;
};

/**
 * @function GCD
 * @memberof GridMath
 * @description
 *    {lang=en} Returns a number representing the largest positive integer that divides the numbers without a remainder
 *    {lang=ja} 指定された数値の最大公約数を返します
 *    {lang=zh} 返回一个数字，代表整除该数的最大正整数
 *    {lang=hant} 返回一個數字，代表整除該數的最大正整數
 *    {lang=ko} 나머지 없이 숫자를 나누는 가장 큰 양의 정수를 나타내는 숫자를 반환
 *    {lang=ru} Returns a number representing the largest positive integer that divides the numbers without a remainder
 *    {lang=de} Gibt eine Zahl aus, welche die größte positive Ganzzahl darstellt, die die Zahlen ohne Rest dividiert
 * @param  {number} a
 *    {lang=en} The first number.
 *    {lang=ja} 数値 1
 *    {lang=zh} 第一个数字。
 *    {lang=hant} 第一個數字。
 *    {lang=ko} 첫 번째 숫자.
 *    {lang=ru} The first number.
 *    {lang=de} Die erste Zahl.
 * @param  {number} b
 *    {lang=en} The second number.
 *    {lang=ja} 数値 2
 *    {lang=zh} 第二个数字。
 *    {lang=hant} 第二個數字。
 *    {lang=ko} 두 번째 숫자.
 *    {lang=ru} The second number.
 *    {lang=de} Die zweite Zahl.
 * @return {number}
 * @example
 *    =GCD(222,70)
 */
GridMath["GCD"] = function(a, b) {
	if (a < 0) a = -a;
	if (b < 0) b = -b;
	if (b > a) { let temp = a; a = b; b = temp;}
	while (b != 0) {
		a %= b;
		if (a == 0) return b;
		b %= a;
	}
	return a;
};

/**
 * @function INT
 * @memberof GridMath
 * @description
 *    {lang=en} Rounds a number down to the nearest integer
 *    {lang=ja} 数値を切り捨て、最も近い整数を返します
 *    {lang=zh} 将一个数字向下舍入到最近的整数
 *    {lang=hant} 將一個數字向下舍入到最近的整數
 *    {lang=ko} 가장 가까운 정수로 숫자를 내림
 *    {lang=ru} Rounds a number down to the nearest integer
 *    {lang=de} Rundet eine Zahl auf die nächste Ganzzahl ab
 * @param  {number} x
 *    {lang=en} Number
 *    {lang=ja} 数値
 *    {lang=zh} 数字
 *    {lang=hant} 數字
 *    {lang=ko} 숫자
 *    {lang=ru} Number
 *    {lang=de} Zahl
 * @return {number}
 * @example
 *    =INT(4.882961516)
 */
GridMath["INT"] = Math.floor;

/**
 * @function LCM
 * @memberof GridMath
 * @description
 *    {lang=en} Returns the least common multiple. Supports any number of arguments LCM(number1, [number2], ...)
 *    {lang=ja} 最小公倍数を返します。この関数は、任意の引数 LCM をサポートしています (数値 1, 数値 2, ...)
 *    {lang=zh} 返回最小公倍数。支持任何数量的参数 LCM(number1, [number2], ...)
 *    {lang=hant} 返回最小公倍數。支持任何數量的參數 LCM(number1, [number2], ...)
 *    {lang=ko} 최소 공배수를 반환합니다. 인수를 제한 없이 지원합니다. LCM(number1, [number2], ...)
 *    {lang=ru} Returns the least common multiple. Supports any number of arguments LCM(number1, [number2], ...)
 *    {lang=de} Gibt das kleinste gemeinsame Vielfache auf. Unterstützt eine beliebige Anzahl von Argumenten LCM(Zahl1, [Zahl2], ...)
 * @return {number}
 * @example
 *    =LCM(10,15)
 */
GridMath["LCM"] = function(/* ... args*/) {
	let args = (arguments[0] instanceof Array) ? arguments[0] : arguments;
	let argLen = args.length;
	let a = Math.abs(args[0]);
	for (let i = 1; i < argLen; i++) {
		let b = Math.abs(args[i]);
		let c = a;
		while (a && b){
			if(a > b) {
				a %= b;
			} else {
				b %= a;
			}
		}
		a = Math.abs(c * args[i]) / (a + b);
	}
	return a;
};

/**
 * @function LN
 * @memberof GridMath
 * @description
 *    {lang=en} Returns the natural logarithm of a number
 *    {lang=ja} 数値の自然対数を返します。
 *    {lang=zh} 返回一个数字的自然对数
 *    {lang=hant} 返回一個數字的自然對數
 *    {lang=ko} 숫자의 자연로그를 반환
 *    {lang=ru} Returns the natural logarithm of a number
 *    {lang=de} Gibt den natürlichen Logarithmus einer Zahl aus
 * @param  {number} x
 *    {lang=en} Number
 *    {lang=ja} 数値
 *    {lang=zh} 数字
 *    {lang=hant} 數字
 *    {lang=ko} 숫자
 *    {lang=ru} Number
 *    {lang=de} Gibt den natürlichen Logarithmus einer Zahl aus
 * @return {number}
 * @example
 *    =LN(0.1353352832)
 */
GridMath["LN"] = Math.log;

/**
 * @function LOG
 * @memberof GridMath
 * @description
 *    {lang=en} Returns the logarithm of a number to a specified base
 *    {lang=ja} 指定された数を底とする数値の対数を返します。
 *    {lang=zh} 以指定的底，返回某个数字的对数
 *    {lang=hant} 以指定的底，返回某個數字的對數
 *    {lang=ko} 숫자의 로그를 지정된 밑으로 반환
 *    {lang=ru} Returns the logarithm of a number to a specified base
 *    {lang=de} Gibt den Logarithmus einer Zahl zu einer angegebenen Basis aus
 * @param  {number} num
 *    {lang=en} Number
 *    {lang=ja} 数値
 *    {lang=zh} 数字
 *    {lang=hant} 數字
 *    {lang=ko} 숫자
 *    {lang=ru} Number
 *    {lang=de} Zahl
 * @param  {number} base
 *    {lang=en} Base
 *    {lang=ja} 底
 *    {lang=zh} 底
 *    {lang=hant} 底
 *    {lang=ko} 밑
 *    {lang=ru} Base
 *    {lang=de} Basis
 * @return {number}
 * @example
 *    =LOG(10)
 */
GridMath["LOG"] = function(num, base) {
	if(base == null) { base = 10; }
	return Math.log(num) / Math.log(base);
};

/**
 * @function LOG10
 * @memberof GridMath
 * @description
 *    {lang=en} Returns the base-10 logarithm of a number
 *    {lang=ja} 10を底とする数値の対数 (常用対数) を返します
 *    {lang=zh} 返回一个数字以10为底的对数
 *    {lang=hant} 返回一個數字以10為底的對數
 *    {lang=ko} 숫자에 대해 밑이 10인 로그를 반환
 *    {lang=ru} Returns the base-10 logarithm of a number
 *    {lang=de} Gibt den Basis-10-Logarithmus einer Zahl aus
 * @param  {number} x
 *    {lang=en} Number
 *    {lang=ja} 数値
 *    {lang=zh} 数字
 *    {lang=hant} 數字
 *    {lang=ko} 숫자
 *    {lang=ru} Number
 *    {lang=de} Zahl
 * @return {number}
 * @example
 *    =LOG10(100000)
 */
GridMath["LOG10"] = function(x) {
	return Math.log(x) / Math.LN10;
};

/**
 * @function MOD
 * @memberof GridMath
 * @description
 *    {lang=en} Returns the remainder from division, i.e. Modulus
 *    {lang=ja} 除算の余りを返します
 *    {lang=zh} 返回余数
 *    {lang=hant} 返回餘數
 *    {lang=ko} 나누기에서 나머지(Modulus)를 반환
 *    {lang=ru} Returns the remainder from division, i.e. Modulus
 *    {lang=de} Gibt den Rest aus der Division aus, d. h. den Teilungsrest
 * @param  {number} num
 *    {lang=en} Number
 *    {lang=ja} 数値
 *    {lang=zh} 数字
 *    {lang=hant} 數字
 *    {lang=ko} 숫자
 *    {lang=ru} Number
 *    {lang=de} Zahl
 * @param  {number} denominator
 *    {lang=en} Denominator
 *    {lang=ja} 除数
 *    {lang=zh} 分母
 *    {lang=hant} 分母
 *    {lang=ko} 분모
 *    {lang=ru} Denominator
 *    {lang=de} Nenner
 * @return {number}
 * @example
 *    =MOD(5,2)
 */
GridMath["MOD"] = function(num, denominator) {
	return num % denominator;
};

/**
 * @function ODD
 * @memberof GridMath
 * @description
 *    {lang=en} Rounds a number up to the nearest odd integer
 *    {lang=ja} 数値を切り上げ、最も近い奇数にします
 *    {lang=zh} 将一个数字向上舍入到最近的奇数
 *    {lang=hant} 將一個數字向上舍入到最近的奇數
 *    {lang=ko} 가장 가까운 홀수인 정수로 숫자를 올림
 *    {lang=ru} Rounds a number up to the nearest odd integer
 *    {lang=de} Rundet eine Zahl auf die nächste ungerade Ganzzahl auf
 * @param  {number} num
 *    {lang=en} Number
 *    {lang=ja} 数値
 *    {lang=zh} 数字
 *    {lang=hant} 數字
 *    {lang=ko} 숫자
 *    {lang=ru} Number
 *    {lang=de} Zahl
 * @return {number}
 * @example
 *    =ODD(3.152)
 */
GridMath["ODD"] = function(num) { //
	num = Math.ceil(num);
	if((num % 2) === 0) {
		return (num < 0) ? num - 1 : num + 1;
	}
	return num;
};

/**
 * @function POWER
 * @memberof GridMath
 * @description
 *    {lang=en} Returns the result of a number raised to a power
 *    {lang=ja} 数値のべき乗を返します
 *    {lang=zh} 返回一个数字升幂的结果
 *    {lang=hant} 返回一個數字升冪的結果
 *    {lang=ko} 지수의 거듭제곱 결과를 반환
 *    {lang=ru} Returns the result of a number raised to a power
 *    {lang=de} Gibt das Ergebnis einer Zahl aus, die zu einer Potenz erhöht wird
 * @param  {number} base
 *    {lang=en} Base
 *    {lang=ja} 基準
 *    {lang=zh} 底
 *    {lang=hant} 底
 *    {lang=ko} 밑
 *    {lang=ru} Base
 *    {lang=de} Basis
 * @param  {number} exponent
 *    {lang=en} Exponent
 *    {lang=ja} 指数
 *    {lang=zh} 指数
 *    {lang=hant} 指數
 *    {lang=ko} 지수
 *    {lang=ru} Exponent
 *    {lang=de} Hochzahl
 * @return {number}
 * @example
 *    =POWER(2,10)
 */
GridMath["POWER"] = Math.pow;

/**
 * @function PRODUCT
 * @memberof GridMath
 * @description
 *    {lang=en}  Multiplies its arguments. Any number of arguments are supported PRODUCT(number1, [number2], ...)
 *    {lang=ja}  引数の積を返します。任意の引数が PRODUCT でサポートされています  (数値 1, 数値 2, ...)
 *    {lang=jp}  乘以其参数。支持任意数字的参数 PRODUCT(number1, [number2], ...)
 * @return {number}
 * @example
 *    =PRODUCT(1,-2,3)
 */
GridMath["PRODUCT"] = function(/* ... args*/) {
	let args = (arguments[0] instanceof Array) ? arguments[0] : arguments;
	let prod = args[0];
	if(prod == null || prod !== prod) { return NaN; }

	for(let i = args.length; --i >= 1;) {
		prod *= args[i];
	}
	return prod;
};

/**
 * @function PROD
 * @memberof GridMath
 * @description
 *    {lang=en}  Multiplies its arguments. Any number of arguments are supported PROD(number1, [number2], ...)
 *    {lang=ja}  引数の積を返します。任意の引数が PROD でサポートされています  (数値 1, 数値 2, ...)
 *    {lang=zh}  乘以其参数。支持任意数量的参数 PROD(number1, [number2], ...)
 *    {lang=hant}  乘以其參數。支持任意數量的參數 PROD(number1, [number2], ...)
 *    {lang=ko}  그 인수를 곱합니다. 인수를 제한 없이 지원합니다. PROD(number1, [number2], ...)
 *    {lang=ru}  Multiplies its arguments. Any number of arguments are supported PROD(number1, [number2], ...)
 *    {lang=de}  Multipliziert seine Argumente. Beliebig viele Argumente werden unterstützt PROD(Zahl1, [Zahl2], ...)
 * @return {number}
 * @example
 *    =PROD(1,-2,3)
 */
GridMath["PROD"] = GridMath["PRODUCT"];

/**
 * @function QUOTIENT
 * @memberof GridMath
 * @description
 *    {lang=en}  Returns the integer portion of a division
 *    {lang=ja}  除算の商の整数部を返します
 *    {lang=zh}  返回除法中的整数部分
 *    {lang=hant}  返回除法中的整數部分
 *    {lang=ko}  나눗셈의 정수 부분을 반환
 *    {lang=ru}  Returns the integer portion of a division
 *    {lang=de}  Gibt den ganzzahligen Anteil einer Division aus
 * @param  {number} num
 *    {lang=en} Number
 *    {lang=ja} 数値
 *    {lang=zh} 数字
 *    {lang=hant} 數字
 *    {lang=ko} 숫자
 *    {lang=ru} Number
 *    {lang=de} Zahl
 * @param  {number} denominator
 *    {lang=en} Denominator
 *    {lang=ja} 除数
 *    {lang=zh} 分母
 *    {lang=hant} 分母
 *    {lang=ko} 분모
 *    {lang=ru} Denominator
 *    {lang=de} Nenner
 * @return {number}
 * @example
 *    =QUOTIENT(10,3)
 */
GridMath["QUOTIENT"] = function(num, denominator) { //
	return Math.floor(num / denominator);
};

/**
 * @function RADIANS
 * @memberof GridMath
 * @description
 *    {lang=en} Converts degrees to radians
 *    {lang=ja} 度単位で表された角度をラジアンに変換します
 *    {lang=zh} 将角度转换成弧度
 *    {lang=hant} 將角度轉換成弧度
 *    {lang=ko} 도를 라디안으로 변환
 *    {lang=ru} Converts degrees to radians
 *    {lang=de} Wandelt Grad in Radianten um
 * @param  {number} degrees
 *    {lang=en} Degree
 *    {lang=ja} 角度
 *    {lang=zh} 角度
 *    {lang=hant} 角度
 *    {lang=ko} 도
 *    {lang=ru} Degree
 *    {lang=de} Grad
 * @return {number}
 * @example
 *    =RADIANS(45)
 */
GridMath["RADIANS"] = function(degrees) {
	return  degrees * (Math.PI / 180);
};

/**
 * @function RAND
 * @memberof GridMath
 * @description
 *    {lang=en} Returns a random number between 0 and 1
 *    {lang=ja} 0 以上 1 未満の乱数を返します
 *    {lang=zh} 返回一个 0 和 1 之间的随机数
 *    {lang=hant} 返回一個 0 和 1 之間的隨機數
 *    {lang=ko} 0과 1 사이의 난수를 반환
 *    {lang=ru} Returns a random number between 0 and 1
 *    {lang=de} Gibt eine Zufallszahl zwischen 0 und 1 aus
 * @return {number}
 * @example
 *    =RAND()
 */
GridMath["RAND"] = Math.random;

/**
 * @function RANDBETWEEN
 * @memberof GridMath
 * @description
 *    {lang=en} Returns a random number between the numbers you specify
 *    {lang=ja} 指定された範囲内の整数の乱数を返します
 *    {lang=zh} 返回一个您指定的数字之间的随机数
 *    {lang=hant} 返回一個您指定的數字之間的隨機數
 *    {lang=ko} 지정한 숫자 사이에 있는 난수를 반환
 *    {lang=ru} Returns a random number between the numbers you specify
 *    {lang=de} Gibt eine Zufallszahl zwischen den von Ihnen angegebenen Zahlen aus
 * @param  {number} bottom
 *    {lang=en} Low number
 *    {lang=ja} 最小値
 *    {lang=zh} 最小数
 *    {lang=hant} 最小數
 *    {lang=ko} 작은 수
 *    {lang=ru} Low number
 *    {lang=de} Niedrige Zahl
 * @param  {number} top
 *    {lang=en} High number
 *    {lang=ja} 最大値
 *    {lang=zh} 最大数
 *    {lang=hant} 最大數
 *    {lang=ko} 큰 수
 *    {lang=ru} High number
 *    {lang=de} Hohe Zahl
 * @return {number}
 * @example
 *    =RANDBETWEEN(2,10)
 */
GridMath["RANDBETWEEN"] = function(bottom, top) {  //
	return Math.floor(Math.random() * (top - bottom + 1)) + bottom;
};

/**
 * @function ROUND
 * @memberof GridMath
 * @description
 *    {lang=en} Rounds a number to a specified number of digits
 *    {lang=ja} 数値を四捨五入して指定された桁数にします
 *    {lang=zh} 将一个数字四舍五入到指定的位数
 *    {lang=hant} 將一個數字四捨五入到指定的位數
 *    {lang=ko} 숫자를 지정된 자릿수로 반올림
 *    {lang=ru} Rounds a number to a specified number of digits
 *    {lang=de} Rundet eine Zahl auf eine bestimmte Anzahl an Ziffern
 * @param  {number} num
 *    {lang=en} Number
 *    {lang=ja} 数値
 *    {lang=zh} 数字
 *    {lang=hant} 數字
 *    {lang=ko} 숫자
 *    {lang=ru} Number
 *    {lang=de} Zahl
 * @param  {number} num_digits
 *    {lang=en} Number of digits. Note that it can go below zero.
 *    {lang=ja} 桁数
 *    {lang=zh} 数字位数。请注意这可能小于零。
 *    {lang=hant} 數字位數。請注意這可能小於零。
 *    {lang=ko} 자릿수. 0 이하가 될 수 있습니다.
 *    {lang=ru} Number of digits. Note that it can go below zero.
 *    {lang=de} Anzahl der Ziffern. Bitte beachten Sie, dass sie unter Null gehen kann.
 * @return {number}
 * @example
 *    =ROUND(54.943, 1)
 */
GridMath["ROUND"] = function(num, num_digits) {
	if(!num_digits) {
		return Math.round(num);
	}
	let divisor = Math.pow(10, num_digits); // Slow!
	return Math.round(num * divisor) / divisor;
};

/**
 * @function ROUNDDOWN
 * @memberof GridMath
 * @description
 *    {lang=en} Rounds a number down, toward zero
 *    {lang=ja} 数値を指定された桁数で切り捨てます
 *    {lang=zh} 将一个数字向下舍入，趋向于零
 *    {lang=hant} 將一個數字向下舍入，趨向於零
 *    {lang=ko} 숫자를 0을 향해 내림
 *    {lang=ru} Rounds a number down, toward zero
 *    {lang=de} Rundet eine Zahl ab, auf Null
 * @param  {number} num
 *    {lang=en} Number
 *    {lang=ja} 数値
 *    {lang=zh} 数字
 *    {lang=hant} 數字
 *    {lang=ko} 숫자
 *    {lang=ru} Number
 *    {lang=de} Zahl
 * @param  {number} num_digits
 *    {lang=en} Number of digits
 *    {lang=ja} 桁数
 *    {lang=zh} 数字位数
 *    {lang=hant} 數字位數
 *    {lang=ko} 자릿수
 *    {lang=ru} Number of digits
 *    {lang=de} Anzahl der Ziffern
 * @return {number}
 * @example
 *    =ROUNDDOWN(54.928, 2)
 */
GridMath["ROUNDDOWN"] = function(num, num_digits) {
	if(!num_digits) {
		return Math.floor(num);
	}
	let divisor = Math.pow(10, num_digits); // Slow!
	return Math.floor(num * divisor) / divisor;
};

/**
 * @function ROUNDUP
 * @memberof GridMath
 * @description
 *    {lang=en} Rounds a number up, away from zero
 *    {lang=ja} 数値を指定された桁数で切り上げます
 *    {lang=zh} 将一个数字向上舍入，远离零值
 *    {lang=hant} 將一個數字向上舍入，遠離零值
 *    {lang=ko} 숫자를 0에서 멀리 올림
 *    {lang=ru} Rounds a number up, away from zero
 *    {lang=de} Rundet eine Zahl auf, weg von Null
 * @param  {number} num
 *    {lang=en} Number
 *    {lang=ja} 数値
 *    {lang=zh} 数字
 *    {lang=hant} 數字
 *    {lang=ko} 숫자
 *    {lang=ru} Number
 *    {lang=de} Zahl
 * @param  {number} num_digits
 *    {lang=en} Number of digits. Note that it can go below zero.
 *    {lang=ja} 桁数
 *    {lang=zh} 数字位数。请注意这可能小于零。
 *    {lang=hant} 數字位數。請注意這可能小於零。
 *    {lang=ko} 자릿수. 0 이하가 될 수 있습니다.
 *    {lang=ru} Number of digits. Note that it can go below zero.
 *    {lang=de} Anzahl der Ziffern. Bitte beachten Sie, dass sie unter Null gehen kann.
 * @return {number}
 * @example
 *    =ROUNDUP(54.928, 2)
 */
GridMath["ROUNDUP"] = function(num, num_digits) { //
	if(!num_digits) {
		return Math.ceil(num);
	}
	let divisor = Math.pow(10, num_digits); // Slow!
	return Math.ceil(num * divisor) / divisor;
};

/**
 * @function SERIESSUM
 * @memberof GridMath
 * @description
 *    {lang=en} Returns the sum of a power series based on the formula
 *    {lang=ja} 数式で定義されるべき数を返します
 *    {lang=zh} 返回一个幂级数基于公式的和
 *    {lang=hant} 返回一個冪級數基於公式的和
 *    {lang=ko} 수식을 기반으로 멱급수의 합을 반환
 *    {lang=ru} Returns the sum of a power series based on the formula
 *    {lang=de} Gibt die Summe einer Potenzreihe basierend auf der Formel aus
 * @param  {number} x
 *    {lang=en} The input value to the power series.
 *    {lang=ja} べき級数に代入する値を指定します。
 *    {lang=zh} 幂级数的输入值。
 *    {lang=hant} 冪級數的輸入值。
 *    {lang=ko} 멱급수에 대한 입력값입니다.
 *    {lang=ru} The input value to the power series.
 *    {lang=de} Der Eingangswert für die Potenzreihe.
 * @param  {number} n
 *    {lang=en} The first power to which x is to be raised.
 *    {lang=ja} x のべき乗の初期値を指定します。
 *    {lang=zh} x 升幂的第一个幂。
 *    {lang=hant} x 升冪的第一個冪。
 *    {lang=ko} x의 1제곱.
 *    {lang=ru} The first power to which x is to be raised.
 *    {lang=de} Die erste Potenz, auf die x angehoben werden soll.
 * @param  {number} m
 *    {lang=en} The step size that n is increased by, on each successive power of x.
 *    {lang=ja} 級数の各項ごとに n を増加させる値を指定します。
 *    {lang=zh} 每个 x 的连续幂，n 增加的步长。
 *    {lang=hant} 每個 x 的連續冪，n 增加的步長。
 *    {lang=ko} n이 x의 각 연속 지수에 의해 증분되는 양.
 *    {lang=ru} The step size that n is increased by, on each successive power of x.
 *    {lang=de} Die Schrittgröße, um die n bei jeder aufeinanderfolgenden Potenz von x erhöht wird.
 * @param  {Array.<number>} coefficients
 *    {lang=en} An array of coefficients that multiply each successive power of x.
 *    {lang=ja} x のべき乗である各項にかける一連の係数を指定します。
 *    {lang=zh} 乘以每个 x 的连续幂的系数的数组。
 *    {lang=hant} 乘以每個 x 的連續冪的係數的數組。
 *    {lang=ko} x의 각 연속 지수를 곱하는 계수들의 배열.
 *    {lang=ru} An array of coefficients that multiply each successive power of x.
 *    {lang=de} Eine Anordnung von Koeffizienten, die jede aufeinanderfolgende Potenz von x multipliziert.
 * @return {number}
 * @example
 * // WARNING: coefficients variable is an Array in which user have no knowledge about how to create an Array in JavaScript syntax. Unsure how to set the array
 *    =SERIESSUM( 5, 1, 1,5 )
 */
GridMath["SERIESSUM"] = function(x, n, m, coefficients) { //
	// WARNING: coefficients variable is an Array in which user have no knowledge about how to create an Array in JavaScript syntax.
	let sum = 0;
	for(let i = coefficients.length; --i >= 0;) {
		sum += coefficients[i] * Math.pow(x, n + i * m);
	}
	return sum;
};

/**
 * @function SIGN
 * @memberof GridMath
 * @description
 *    {lang=en} Returns the sign of a number
 *    {lang=ja} 数値の正負を調べます
 *    {lang=zh} 返回数字的符号
 *    {lang=ko} 숫자의 사인을 반환
 *    {lang=ru} Returns the sign of a number
 * @param  {number} num
 *    {lang=en} Number
 *    {lang=ja} 数値
 *    {lang=zh} 数字
 *    {lang=ko} 숫자
 *    {lang=ru} Number
 * @return {number}
 * @example
 *    =SIGN(-496)
 */
GridMath["SIGN"] = function(num) {
	if(num > 0) {
		return 1;
	} else if(num < 0) {
		return -1;
	}
	return 0; // num may have a undefined, null, NaN, or empty string value
};

/**
 * @function SIN
 * @memberof GridMath
 * @description
 *    {lang=en} Returns the sine of the given angle
 *    {lang=ja} 角度のサインを返します
 *    {lang=zh} 返回给定角的正弦
 *    {lang=hant} 返回給定角的正弦
 *    {lang=ko} 주어진 각도의 사인을 반환
 *    {lang=ru} Returns the sine of the given angle
 *    {lang=de} Gibt den Sinus des gegebenen Winkels aus
 * @param  {number} angle
 *    {lang=en} Angle
 *    {lang=ja} 角度
 *    {lang=zh} 角
 *    {lang=hant} 角
 *    {lang=ko} 각도
 *    {lang=ru} Angle
 *    {lang=de} Winkel
 * @return {number}
 * @example
 *    =SIN(-PI()/6)
 */
GridMath["SIN"] = Math.sin; //

/**
 * @function SINH
 * @memberof GridMath
 * @description
 *    {lang=en} Returns the hyperbolic sine of a number
 *    {lang=ja} 数値の双曲サインを返します
 *    {lang=zh} 返回一个数字的双正弦
 *    {lang=hant} 返回一個數字的雙正弦
 *    {lang=ko} 숫자의 쌍곡사인을 반환
 *    {lang=ru} Returns the hyperbolic sine of a number
 *    {lang=de} Gibt den hyperbolischen Sinus einer Zahl aus
 * @param  {number} x
 *    {lang=en} Number
 *    {lang=ja} 数値
 *    {lang=zh} 数字
 *    {lang=hant} 數字
 *    {lang=ko} 숫자
 *    {lang=ru} Number
 *    {lang=de} Zahl
 * @return {number}
 * @example
 *    =SINH(4)
 */
GridMath["SINH"] = function(x) {
	let y = Math.exp(x);
	return (y - 1 / y) / 2;
};

/**
 * @function SQRT
 * @memberof GridMath
 * @description
 *    {lang=en} Returns a positive square root
 *    {lang=ja} 正の平方根を返します
 *    {lang=zh} 返回一个正的平方根
 *    {lang=hant} 返回一個正的平方根
 *    {lang=ko} 양의 제곱근을 반환
 *    {lang=ru} Returns a positive square root
 *    {lang=de} Gibt eine positive Quadratwurzel aus
 * @param  {number} x
 *    {lang=en} Number
 *    {lang=ja} 数値
 *    {lang=zh} 数字
 *    {lang=hant} 數字
 *    {lang=ko} 숫자
 *    {lang=ru} Number
 *    {lang=de} Zahl
 * @return {number}
 * @example
 *    =SQRT(4)
 */
GridMath["SQRT"] = Math.sqrt;

/**
 * @function SQRTPI
 * @memberof GridMath
 * @description
 *    {lang=en} Returns the square root of (multiple * pi)
 *    {lang=ja} 数値 x n の平方根の値を計算します
 *    {lang=zh} 返回 (multiple * pi) 的平方根
 *    {lang=hant} 返回 (multiple * pi) 的平方根
 *    {lang=ko} (배수 * 파이) 제곱근을 반환
 *    {lang=ru} Returns the square root of (multiple * pi)
 *    {lang=de} Gibt die Quadratwurzel von (Vielfaches * pi) aus
 * @param  {number} multiple
 *    {lang=en} Multiple
 *    {lang=ja} 数値
 *    {lang=zh} 乘数
 *    {lang=hant} 乘數
 *    {lang=ko} 배수
 *    {lang=ru} Multiple
 *    {lang=de} Mehrere
 * @return {number}
 * @example
 *    =SQRTPI(3)
 */
GridMath["SQRTPI"] = function(multiple) {
	return Math.sqrt(multiple * Math.PI);
};

/**
 * @function SUM
 * @memberof GridMath
 * @description
 *    {lang=en} Returns the summation of all the arguments. This function supports any number of arguments SUM(number1, [number2], ...)
 *    {lang=ja} すべての引数の合計を返します。この関数は、任意の値の引数 SUM(数値 1, 数値 2, ...) をサポートしています
 *    {lang=zh} 返回全部参数的和。这个函数支持任意数量的参数 SUM(number1, [number2], ...)
 *    {lang=hant} 返回全部參數的和。這個函數支持任意數量的參數 SUM(number1, [number2], ...)
 *    {lang=ko} 모든 인수의 합계를 반환합니다. 이 함수는 인수를 제한 없이 지원합니다. SUM(number1, [number2], ...)
 *    {lang=ru} Returns the summation of all the arguments. This function supports any number of arguments SUM(number1, [number2], ...)
 *    {lang=de} Gibt die Summenbildung aller Argumente aus. Diese Funktion unterstützt eine beliebige Anzahl von Argumenten SUM(Zahl1, [Zahl2], ...)
 * @return {number}
 * @example
 *    =SUM(4,-500)
 */
GridMath["SUM"] = function(/* ... args*/) {
	let args = (arguments[0] instanceof Array) ? arguments[0] : arguments;
	let sum = 0;
	for(let i = args.length; --i >= 0;) {
		sum += args[i];
	}
	return sum;
};

/**
 * @function SUMPRODUCT
 * @memberof GridMath
 * @description
 *    {lang=en} Returns the sum of the products of corresponding array components.
 * 				This function supports any number of arguments SUMPRODUCT(number1, [number2], ...)
 *    {lang=ja} 指定された配列で対応する要素の積の合計を返します。
 * 				この関数は、任意の値の引数 SUMPRODUCT(数値 1, 数値 2, ...)
 をサポートしています。
 *    {lang=zh} 返回相应数组元素的乘积的和。
 * 				这个函数支持任意数量的参数 SUMPRODUCT(number1, [number2], ...)
 *    {lang=ko} 해당하는 배열 요소의 곱의 합계를 반환합니다.
 *    {lang=ru} Returns the sum of the products of corresponding array components.
 * 				This function supports any number of arguments SUMPRODUCT(number1, [number2], ...)
 * @return {number}
 * @example
 *    =SUMPRODUCT(2,3)
 */
GridMath["SUMPRODUCT"] = function(/* ... args*/) {
	let args = (arguments[0] instanceof Array) ? arguments[0] : arguments;
	let argLen = args.length;
	if(argLen === 0) { return 0; }

	let i;
	let products = [];
	for(i = 0; i < argLen; ++i) {
		products[i] = 1;
	}
	for(i = 0; i < argLen; ++i) {
		products[i] *= args[i];
	}
	let sum = 0;
	for(i = 0; i < argLen; ++i) {
		sum += products[i];
	}
	return sum;
};

/**
 * @function SUMSQ
 * @memberof GridMath
 * @description
 *    {lang=en} Returns the sum of the squares of the arguments. This function supports any number of arguments SUMSQ(number1, [number2], ...)
 *    {lang=ja} 引数の2乗の合計を返します。この関数は、任意の値の引数 SUMSQ(数値 1, 数値 2, ...) をサポートしています
 *    {lang=zh} 返回参数的平方的和。这个函数支持任意数量的参数 SUMSQ(number1, [number2], ...)
 *    {lang=hant} 返回參數的平方的和。這個函數支持任意數量的參數 SUMSQ(number1, [number2], ...)
 *    {lang=ko} 인수의 제곱의 합을 반환합니다. 이 함수는 인수를 제한 없이 지원합니다. SUMSQ(number1, [number2], ...)
 *    {lang=ru} Returns the sum of the squares of the arguments. This function supports any number of arguments SUMSQ(number1, [number2], ...)
 *    {lang=de} Gibt die Summe der Quadratwurzeln der Argumente aus. Diese Funktion unterstützt eine beliebige Anzahl von Argumenten SUMSQ(Zahl1, [Zahl2], ...)
 * @return {number}
 * @example
 *    =SUMSQ(3)
 */
GridMath["SUMSQ"] = function(/* ... args*/) {
	let args = (arguments[0] instanceof Array) ? arguments[0] : arguments;
	let sum = 0;
	for(let i = args.length; --i >= 0;) {
		let val = args[i];
		sum += (val * val);
	}
	return sum;
};


// /**
//  * @function SUMX2MY2
//  * @memberof GridMath
//  * @description
//  *    {lang=en} Returns the sum of the difference of squares of corresponding values in two arrays
//  *    {lang=ja} 2つの配列の対応する配列要素の平方差の合計を返します
//  *    {lang=zh} 返回两个数组中相应值的平方的差的和
//  *    {lang=ko} 두 배열에서 해당하는 값의 제곱의 차의 합을 반환
//  *    {lang=ru} Returns the sum of the difference of squares of corresponding values in two arrays
//  * @param  {number} aryX
//  *    {lang=en} First array
//  *    {lang=ja} 配列 1
//  *    {lang=zh} 第一个数组
//  *    {lang=ko} 첫 번째 배열
//  *    {lang=ru} First array
//  * @param  {number} aryY
//  *    {lang=en} Second array
//  *    {lang=ja} 配列 2
//  *    {lang=zh} 第二个数组
//  *    {lang=ko} 두 번째 배열
//  *    {lang=ru} Second array
//  * @return {number}
//  * @example
//  *    <Example can be provided here>
//  */
// GridMath["SUMX2MY2"] = function(aryX, aryY) {
// 	let sum = 0;
// 	for(let i = aryX.length; --i >= 0;) {
// 		let valX = aryX[i];
// 		let valY = aryY[i];
// 		sum += (valX * valX) - (valY * valY);
// 	}
// 	return sum;
// };

// *
//  * @function SUMX2PY2
//  * @memberof GridMath
//  * @description
//  *    {lang=en} Returns the sum of the sum of squares of corresponding values in two arrays
//  *    {lang=ja} 2つの配列の対応する値の平方和を返します
//  *    {lang=zh} 返回两个数组中相应的值的平方的和
//  *    {lang=ko} 두 배열에서 해당하는 값의 제곱합의 합을 반환
//  *    {lang=ru} Returns the sum of the sum of squares of corresponding values in two arrays
//  * @param  {number} aryX
//  *    {lang=en} First array
//  *    {lang=ja} 配列 1
//  *    {lang=zh} 第一个数组
//  *    {lang=ko} 첫 번째 배열
//  *    {lang=ru} First array
//  * @param  {number} aryY
//  *    {lang=en} Second array
//  *    {lang=ja} 配列 2
//  *    {lang=zh} 第二个数组
//  *    {lang=ko} 두 번째 배열
//  *    {lang=ru} Second array
//  * @return {number}
//  * @example
//  *    <Example can be provided here>

// GridMath["SUMX2PY2"] = function(aryX, aryY) {
// 	let sum = 0;
// 	for(let i = aryX.length; --i >= 0;) {
// 		let valX = aryX[i];
// 		let valY = aryY[i];
// 		sum += (valX * valX) + (valY * valY);
// 	}
// 	return sum;
// };

// /**
//  * @function SUMXMY2
//  * @memberof GridMath
//  * @description
//  *    {lang=en} Returns the sum of squares of differences of corresponding values in two arrays
//  *    {lang=ja} 2つの配列の対応する値の差の平方和を返します
//  *    {lang=zh} 返回两个数组中相应的值的差的平方的和
//  *    {lang=ko} 두 배열에서 해당하는 값의 차의 제곱합을 반환
//  *    {lang=ru} Returns the sum of squares of differences of corresponding values in two arrays
//  * @param  {number} aryX
//  *    {lang=en} First array
//  *    {lang=ja} 配列 1
//  *    {lang=zh} 第一个数组
//  *    {lang=ko} 첫 번째 배열
//  *    {lang=ru} First array
//  * @param  {number} aryY
//  *    {lang=en} Second array
//  *    {lang=ja} 配列 2
//  *    {lang=zh} 第二个数组
//  *    {lang=ko} 두 번째 배열
//  *    {lang=ru} Second array
//  * @return {number}
//  * @example
//  *    <Example can be provided here>
//  */
// GridMath["SUMXMY2"] = function(aryX, aryY) {
// 	let sum = 0;
// 	for(let i = aryX.length; --i >= 0;) {
// 		let XMY = aryX[i] - aryY[i]; // X minus Y
// 		sum += (XMY * XMY);
// 	}
// 	return sum;
// };

/**
 * @function TAN
 * @memberof GridMath
 * @description
 *    {lang=en} Returns the tangent of a number
 *    {lang=ja} 角度のタンジェントを返します
 *    {lang=zh} 返回一个数字的正切
 *    {lang=hant} 返回一個數字的正切
 *    {lang=ko} 숫자의 탄젠트를 반환
 *    {lang=ru} Returns the tangent of a number
 *    {lang=de} Gibt den Tangens einer Zahl aus
 * @param  {number} x
 *    {lang=en} Number
 *    {lang=ja} 数値
 *    {lang=zh} 数字
 *    {lang=hant} 數字
 *    {lang=ko} 숫자
 *    {lang=ru} Number
 *    {lang=de} Zahl
 * @return {number}
 * @example
 *    =TAN(-PI()/4)
 */
GridMath["TAN"] = Math.tan;

/**
 * @function TANH
 * @memberof GridMath
 * @description
 *    {lang=en} Returns the hyperbolic tangent of a number
 *    {lang=ja} 数値の双曲タンジェントを返します
 *    {lang=zh} 返回一个数字的双曲正切
 *    {lang=hant} 返回一個數字的雙曲正切
 *    {lang=ko} 숫자의 쌍곡탄젠트를 반환
 *    {lang=ru} Returns the hyperbolic tangent of a number
 *    {lang=de} Gibt den hyperbolischen Tangens einer Zahl aus
 * @param  {number} x
 *    {lang=en} Number
 *    {lang=ja} 数値
 *    {lang=zh} 数字
 *    {lang=hant} 數字
 *    {lang=ko} 숫자
 *    {lang=ru} Number
 *    {lang=de} Zahl
 * @return {number}
 * @example
 *    =TANH(4)
 */
GridMath["TANH"] = function(x) {
	if (x === Infinity) {
		return 1;
	} else if (x === -Infinity) {
		return -1;
	}
	let y = Math.exp(2 * x);
	return (y - 1) / (y + 1);
};

/**
 * @function TRUNC
 * @memberof GridMath
 * @description
 *    {lang=en} Truncates a number to an integer
 *    {lang=ja} 数値の小数部を切り捨て、整数または指定された桁数にします
 *    {lang=zh} 将一个数字截断成整数
 *    {lang=hant} 將一個數字截斷成整數
 *    {lang=ko} 숫자를 정수로 자름
 *    {lang=ru} Truncates a number to an integer
 *    {lang=de} Kürzt eine Zahl auf eine Ganzzahl
 * @param  {number} num
 *    {lang=en} Number
 *    {lang=ja} 数値
 *    {lang=zh} 数字
 *    {lang=hant} 數字
 *    {lang=ko} 숫자
 *    {lang=ru} Number
 *    {lang=de} Zahl
 * @param  {number} num_digits
 *    {lang=en} Number of digits
 *    {lang=ja} 桁数
 *    {lang=zh} 数字位数
 *    {lang=hant} 數字位數
 *    {lang=ko} 자릿수
 *    {lang=ru} Number of digits
 *    {lang=de} Anzahl der Ziffern
 * @return {number}
 * @example
 *    =TRUNC(CF_LAST,1)
 */
GridMath["TRUNC"] = function(num, num_digits) {
	if(!num_digits) {
		return Math.floor(num);
	}
	let divisor = Math.pow(10, num_digits);
	return Math.floor(num * divisor) / divisor;
};

export default GridMath;
export { GridMath as Math };
