import { DataGenerator } from "./DataGenerator.js";
import { invalidFieldDict } from "./MockUtil.js";
import { DateTime } from "../DateTime.js";

let dataGen = new DataGenerator();

/** @private
* @namespace
*/
let Adc = {};

/** @private
* @function
* @param {Object} payload
* @param {Object=} mockResponse
* @return {Promise}
*/
Adc.request = function (payload, mockResponse) {
	if (mockResponse) {
		return Promise.resolve(JSON.stringify(mockResponse));
	}

	// build row header
	let rows = [];
	let i, f, len, row, j;
	let identifiers, formula, fields, invalidDict;
	if(payload.output === "Col,date|,Row,In|,va,T,NoEmptyTickers") {

		identifiers = payload.identifiers;
		formula =  payload.formula.trim().split(/(?=,TR.)/);// TODO: check each field with another way, this doesn't work when user use some comma (,) formula
		fields = [];

		// invalidFieldDict is a dictionary of non exist field
		// so we must remove invalid field to make "mocking api" return result more like a "real api".
		invalidDict = invalidFieldDict;
		for (i = 0; i < formula.length; i++) {
			f = formula[i];
			if (!invalidDict[f]) {
				fields.push(f);
			}
		}

		let fieldCount = fields.length;
		// formula = payload.formula.trim().split(",TR"); // TODO: split wit

		rows.push([
			{
				"h": true,
				"i": "instrument",
				"v": "Instrument"
			},
			{
				"h": true,
				"i": "date",
				"v": "Date"
			}
		]);

		for (let index = 0; index < fieldCount; index++) {
			let timeSeriesField = fields[index];

			let regex = /TR.(.*)(?=\()/;
			let result = timeSeriesField.match(regex);
			let field = result[0].replace("TR.", "");
			let phrase = timeSeriesField.toLowerCase();
			let startDateRegex = /sdate=(.*)[,]/;
			let endDateRegex = /edate=(.*)[)]/;
			let startDateMatch = phrase.match(startDateRegex);
			let endDateMatch = phrase.match(endDateRegex);

			let startDate = startDateMatch ? startDateMatch[1] : DateTime.format(new Date(), "YYYY-MM-DD");
			let endDate = endDateMatch[1];

			let swapDateTmp;
			if(startDate > endDate) {
				swapDateTmp = startDate;
				startDate = endDate;
				endDate = swapDateTmp;
			}

			let msBetweenDate = (+new Date(endDate)) - (+new Date(startDate));

			let msInDay = 1000 * 60 * 60 * 24;
			let betweenDay = msBetweenDate / msInDay;
			// let betweenDay = Math.floor(Math.random() * 10); // For random length


			for (i = 0; i <= betweenDay; i++) {
				let date = new Date(new Date(startDate).setDate(new Date(startDate).getDate() + i));
				let fieldValue = {
					"h": true,
					"v": DateTime.format(date, "YYYY-MM-DDT00:00:00")
				};
				if(index === 0) { // add header 1 time
					rows[0].push(fieldValue);
				}
			}

			let rics = identifiers;
			for (i = 0; i < rics.length; i++) {

				let ric = rics[i];
				let rowValue = [
					ric,
					field
				];
				for (j = 1; j <= betweenDay + 1; j++) {
					let generatedValue = DataGenerator.generateRecord(field);
					rowValue.push(generatedValue[field]);

				}
				rows.push(rowValue);
			}
			// The example rows should be look like
			/*
			[
				[
				{
					"h": true,
					"i": "instrument",
					"v": "Instrument"
				},
				{
					"h": true,
					"i": "date",
					"v": "Date"
				},
				{
					"h": true,
					"v": "2011-08-11T00:00:00"
				},
				{
					"h": true,
					"v": "2011-08-12T00:00:00"
				},
				{
					"h": true,
					"v": "2011-08-15T00:00:00"
				}
				],
				[
				"IBM",
				"Price Close", // NOTE: this cannot be detech we join it with space Ex. PriceClose
				159.25449372,
				160.6585848,
				165.23382036
				],
				[
				"GOOGL.O",
				"Price Close",
				14.066881653,
				14.107921423,
				13.944262828
				]
			]
			*/
		}

	} else {

		identifiers = payload.identifiers;
		formula = Adc.splitFields(payload.formula);
		fields = [];

		// invalidFieldDict is a dictionary of non exist field
		// so we must remove invalid field to make "mocking api" return result more like a "real api".
		invalidDict = invalidFieldDict;
		for (i = 0; i < formula.length; i++) {
			f = formula[i];
			if (!invalidDict[f]) {
				fields.push(f);
			}
		}

		rows[0] = [{
			"h": true,
			"i": "instrument",
			"v": "Instrument"
		}];
		for (i = 0; i < fields.length; i++) {
			f = fields[i];
			rows[0].push({
				"h": true,
				"r": f.toUpperCase().trim()
				// "v": "Price Close", // Doesn't use
				// "p": "I_TRP_PH_PriceClose" // Doesn't use this property
			});
		}

		// build data
		let rowMap = {};

		len = identifiers.length;
		let rowData = dataGen.generate(fields, len);
		for (i = 0; i < len; ++i) {
			let inst = identifiers[i];
			row = rowMap[inst];
			if (!row) {
				row = rowMap[inst] = rowData[i];
				row.unshift(inst); // prepend instrument on each row
			}
			rows.push(row);
		}

		// There is a chance that rtk will return multiple row data per instrument
		// so we must create mock up for this case
		if (rows.length > 0) {
			let chance = dataGen.randInt(1, 10);
			if (chance <= 3) { // chance 30%
				let pos = dataGen.randInt(0, rows.length - 1); // random row pos
				row = rows[pos];
				len = row.length;
				let mockupRow = new Array(len);
				mockupRow[0] = row[0]; // 1st index is for instrument
				for (i = 1; i < len; i++) {
					mockupRow[i] = ''; // real case will return null or empty string
				}
				rows.splice(pos + 1, 0, mockupRow);
			}
		}
	}


	/*
	response type is :
	{
		cols: number,
		faults: Array<obj>,
		rows: Array<obj>,
		status: string
	}
	Currently, we use only the rows property for cache ADC data
	*/
	return Promise.resolve(JSON.stringify({
		rows: rows
		// TODO : supports faults property
	}));
};

/**
 * Splits a string of comma-separated fields into an array of individual field names with optional parentheses and contents.
 *
 * @param {string} strFields - The string of comma-separated fields to split.
 * @returns {Array<string>} - An array of individual field names with optional parentheses and contents.
 */
Adc.splitFields = function(strFields) {
	 if(!strFields) {
		return [];
	 }

	// Split the input string using the regular expression
	// regex is match commas that are not inside parentheses
	/*
		,          		- matches a comma
		(?![^()]*\) 	- negative lookahead assertion that matches if the comma is not followed by:
			[^()]*   	- any characters that are not opening or closing parentheses
			\)			- a closing parenthesis
		)          		- ending delimiter of the regular expression
	*/
	let fields = strFields.split(/,(?![^()]*\))/);
	fields = fields.map(function(field) {
		return field.trim();
	});

	return fields;
};

export { Adc };
