import DataSet from "./DataSet.js";

declare namespace DataGenerator {

    type DataOptions = {
        seed?: number|null,
        numRows?: number|null,
        rowCount?: number|null
    };

    type FieldInformation = {
        type?: string|null,
        prefix?: string|null,
        suffix?: string|null,
        min?: number|null,
        max?: number|null,
        prec?: string|null,
        fixedValue?: string|null,
        members?: any[]|null,
        generate?: ((...params: any[]) => any)|null
    };

}

declare class DataGenerator {

    constructor(seed?: number|null);

    public generate(fields: string|(string)[]|null, options?: (number|DataGenerator.DataOptions)|null): (any[])[];

    public generateRecords(fields: string|(string)[]|null, options?: (number|any)|null): (any)[];

    public generateQuoteData(field: string, options?: DataGenerator.DataOptions|null): any;

    public getSeed(): number|null;

    public setSeed(seed: number): void;

    public randBetween(min?: number|null, max?: number|null): number;

    public randInt(min: number, max: number): number;

    public randIndex(max: number): number;

    public randNumber(min?: number|null, max?: number|null, prec?: (number|null)|null): number;

    public randBoolean(): boolean;

    public randMember(set: any[]|null): any;

    public randDate(min?: (number|any)|null, max?: (number|any)|null): Date|null;

    public randString(min?: number|null, max?: number|null): string;

    public static generate(fields: string|(string)[]|null, options?: DataGenerator.DataOptions|null): (any[])[];

    public static generateRecord(fields: string|(string)[]|null, options?: DataGenerator.DataOptions|null): any;

    public static generateRecords(fields: string|(string)[]|null, options?: (number|DataGenerator.DataOptions)|null): (any)[];

    public static addFieldInfo(field: string, options: DataGenerator.FieldInformation|((...params: any[]) => any)|null): void;

    public static randNumber(min?: number|null, max?: number|null, prec?: (number|null)|null, seed?: number|null): number;

    public static randInt(min: number, max: number, seed?: number|null): number;

    public static randBetween(min?: number|null, max?: number|null, seed?: number|null): number;

    public static randIndex(max: number, seed?: number|null): number;

    public static randBoolean(seed?: number|null): boolean;

    public static randMember(set: any[]|null, seed?: number|null): any;

    public static randDate(min?: (number|any)|null, max?: (number|any)|null, seed?: number|null): Date|null;

    public static randString(min?: number|null, max?: number|null, subType?: string|null, seed?: number|null): string;

    public static toRecords(data2D: (any[])[]|null, fields?: (string)[]|null): (any)[]|null;

}

declare function getFieldInfo(field: string): any;

declare function addFieldInfo(field: string, options: DataGenerator.FieldInformation|((...params: any[]) => any)|null): void;

declare function pseudoRandNumber(seed: number): number;

declare function randNumber(min?: number|null, max?: number|null, prec?: (number|null)|null, seed?: number|null): number;

declare function randInt(min: number, max: number, seed?: number|null): number;

declare function randBetween(min?: number|null, max?: number|null, seed?: number|null): number;

declare function randIndex(max: number, seed?: number|null): number;

declare function randBoolean(seed?: number|null): boolean;

declare function randMember(set: any[]|null, seed?: number|null): any;

declare function randDate(min?: (number|any)|null, max?: (number|any)|null, seed?: number|null): Date|null;

declare function randString(min?: number|null, max?: number|null, seed?: number|null): string;

declare function toRecords(data2D: (any[])[]|null, fields?: (string|number)[]|null): (any)[]|null;

declare function getSeed(): number;

declare function generate(fields: string|(string)[]|null, options?: DataGenerator.DataOptions|null): (any[])[];

declare function generateRecord(fields: string|(string)[]|null, options?: (number|DataGenerator.DataOptions)|null, seed?: number|null): any;

declare function generateRecords(fields: string|(string)[]|null, options?: (number|DataGenerator.DataOptions)|null): (any)[];

declare function generateQuoteData(field: string, options?: DataGenerator.DataOptions|null, seed?: number|null): any;

export default DataGenerator;
export {
	DataGenerator,
	addFieldInfo,
	getSeed,
	generate,
	generateRecord,
	generateRecords,
	generateQuoteData,
	randBetween,
	randInt,
	randIndex,
	randNumber,
	randBoolean,
	randMember,
	randDate,
	randString,
	pseudoRandNumber,
	toRecords
};
