import { randIndex, generateQuoteData } from "./DataGenerator.js";

/** @constructor */
let MockQuotes = function() {
};

/** @public
* @param {string=} opt_id
* @return {MockSubscription}
*/
MockQuotes.prototype.create = function(opt_id) {
	return new MockSubscription(opt_id);
};


/** @constructor
* @param {string=} opt_id
*/
let MockSubscription = function(opt_id) {
	this._onRowUpdated = this._onRowUpdated.bind(this);

	this._fieldMap = {};
	this._prevData = {};
	if(opt_id != null) {
		this["id"] = opt_id + "";
	} else {
		this["id"] = "_sub" + MockSubscription._runningId++;
	}
};

/** @type {Array.<string>}
* @private
*/
MockSubscription.prototype._rics = null;

/** @type {boolean}
* @private
*/
MockSubscription.prototype._isChain = false;

/** @type {Object}
* @private
*/
MockSubscription.prototype._fieldMap = null;

/** @type {Object}
* @private
*/
MockSubscription.prototype._prevData = null;

/** @type {Function}
* @private
*/
MockSubscription.prototype._updateHandler = null;

/** @type {number}
* @private
*/
MockSubscription.prototype._updateTimerId = 0;

/** @type {number}
* @private
*/
MockSubscription.prototype._updateInterval = 1000;

/** @type {number}
* @private
*/
MockSubscription._runningId = 0;

/** @public
* @param {Array.<string>} fields
* @return {MockSubscription}
*/
MockSubscription.prototype.formattedFields = function(fields) {
	for(let i = fields.length; --i >= 0;){
		this._fieldMap[fields[i]] = true;
	}
	return this;
};
/** @public
* @param {Array.<string>} fields
* @return {MockSubscription}
*/
MockSubscription.prototype.rawFields = function(fields) {
	for(let i = fields.length; --i >= 0;){
		this._fieldMap[fields[i]] = true;
	}
	return this;
};

/** @public
* @param {string} ric
* @return {MockSubscription}
*/
MockSubscription.prototype.chain = function(ric) {
	this._rics = Array.isArray(ric) ? ric[0] : [ric];
	this._isChain = true;
	return this;
};

/** @public
* @param {string|Array.<string>} ric
* @return {MockSubscription}
*/
MockSubscription.prototype.rics = function(ric) {
	this._rics = Array.isArray(ric) ? ric : [ric];
	return this;
};

/** @public
* @return {MockSubscription}
*/
MockSubscription.prototype.start = function() {
	this._onRowUpdated();
	return this;
};
/** @public
* @return {MockSubscription}
*/
MockSubscription.prototype.stop = function() {
	if(this._updateTimerId){
		clearInterval(this._updateTimerId);
		this._updateTimerId = 0;
	}
	return this;
};

/** @public
* @param {Function} handler
* @return {MockSubscription}
*/
MockSubscription.prototype.onNewRow = function(handler) {
	return this;
};
/** @public
* @param {Function} handler
* @return {MockSubscription}
*/
MockSubscription.prototype.onUpdate = function(handler) {
	this._updateHandler = handler;
	return this;
};
/** @public
* @param {Function} handler
* @return {MockSubscription}
*/
MockSubscription.prototype.onRemoveRow = function(handler) {
	return this;
};

/** @private */
MockSubscription.prototype._onRowUpdated = function() {
	if(!this._updateHandler){
		return;
	}
	//TODO: random which fields to be updated
	let len = this._rics ? this._rics.length : 0;
	let ric;
	if(len === 1) {
		ric = this._rics[0];
		this._updateHandler(this, ric, this._getUpdateData(ric));
	} else {
		for(let i = 0; i < len; ++i) {
			if(!randIndex(3)) {
				ric = this._rics[i];
				this._updateHandler(this, ric, this._getUpdateData(ric));
			}
		}
	}
	if(!this._updateTimerId) {
		this._updateTimerId = setInterval(this._onRowUpdated, this._updateInterval);
	}
};

/** @private
* @param {string} ric
* @return {Object}
*/
MockSubscription.prototype._getUpdateData = function(ric) {
	let data = {"id": this["id"]};
	let options = {"text": ric};
	for(let field in this._fieldMap) {
		let fInfo = generateQuoteData(field, options);
		let v = fInfo.value;
		let fv = fInfo.formattedValue;
		if(fInfo.changeOnly) {
			if(this._prevData[field] === v) {
				continue;
			}
		}
		this._prevData[field] = v;
		data[field] = {"raw": v, "formatted": fv || (v + "")};
	}
	return data;
};


export {MockQuotes, MockSubscription};
