import { Adc } from "./Adc.js";
import { DataGenerator } from "./DataGenerator.js";
import { invalidFieldDict, setInvalidFields } from "./MockUtil.js";

let dataGen = new DataGenerator();

/** @private
* @namespace
*/
let DataGrid = {};

/** @private
* @function
* @param {Object} payload
* @param {Object=} mockResponse
* @return {Promise}
*/
DataGrid.request = function (payload, mockResponse) {
	if (mockResponse) {
		return Promise.resolve(JSON.stringify(mockResponse));
	}

	let i, f, len, row;
	let instruments = payload.instruments;
	let fields = payload.fields;

	// _invalidFieldDict is a dictionary of non exist field
	// so we must remove invalid field to make "mocking api" return result more like a "real api".
	let invalidDict = invalidFieldDict;
	fields = [];
	for (i = 0; i < payload.fields.length; i++) {
		f = payload.fields[i];
		if (!invalidDict[f.name]) {
			fields.push(f);
		}
	}

	// for filter duplicate instrument
	let rowMap = {};

	// build data
	len = instruments.length;
	let data2D = [];
	let rowData = dataGen.generate(fields, len);
	for (i = 0; i < len; ++i) {
		let inst = instruments[i];
		row = rowMap[inst];
		if (!row) {
			row = rowMap[inst] = rowData[i];
			row.unshift(inst); // prepend instrument on each row
		}
		data2D.push(row);
	}

	// There is a chance that rtk will return multiple row data per instrument
	// so we must create mock up for this case
	if (data2D.length > 0) {
		let chance = dataGen.randInt(1, 10);
		if (chance <= 3) { // chance 30%
			let pos = dataGen.randInt(0, data2D.length - 1); // random row pos
			row = data2D[pos];
			len = row.length;
			let mockupRow = new Array(len);
			mockupRow[0] = row[0]; // 1st index is for instrument
			for (i = 1; i < len; i++) {
				mockupRow[i] = ''; // real case will return null or empty string
			}
			data2D.splice(pos + 1, 0, mockupRow);
		}
	}

	// build header
	let headers = [{
		"displayName": "Instrument"
	}];
	for (i = 0; i < fields.length; i++) {
		f = fields[i];
		headers.push({
			"displayName": f.name,
			"field": f.name.toUpperCase() // server only return in uppercase
		});
	}

	return Promise.resolve(JSON.stringify({
		data: data2D,
		headers: [headers]
	}));
};

/** @public
* @function
* @param {string} dataType
* @return {Promise}
*/
let mockDataAPI = function (dataType) {
	if (dataType === "datagrid") {
		return Promise.resolve(DataGrid);
	}  else if(dataType === "adc") {
		return Promise.resolve(Adc);
	} else {
		return Promise.reject(dataType + " is not supported.");
	}
};

/** to mock invalid fields.
* @public
* @param {Array<string>} fields ex. TR.NonExistField, CF_IGNORE_FIELD
*/
mockDataAPI.setInvalidFields = setInvalidFields;

export { mockDataAPI, DataGrid, Adc, setInvalidFields };
