import { Ext } from "../../tr-grid-util/es6/Ext.js";
import { GridPlugin } from "../../tr-grid-util/es6/GridPlugin.js";
import { Conflator } from "../../tr-grid-util/es6/Conflator.js";
import { Icon } from "../../tr-grid-util/es6/Icon.js";
import { ElfUtil } from "../../tr-grid-util/es6/ElfUtil.js";
import { GroupDefinitions } from "../../tr-grid-util/es6/GroupDefinitions.js";
import { Dom } from "../../tr-grid-util/es6/Dom.js";
import { injectCss, prettifyCss } from "../../tr-grid-util/es6/Util.js";
import { preventDefault } from "../../tr-grid-util/es6/EventDispatcher.js";

declare namespace ColumnStackPlugin {

    type Options = {
        fields: (string)[]|null,
        stacks: (ColumnStackPlugin.StackDefinition)[]|null,
        autoStacking?: boolean|null,
        clicked?: ((...params: any[]) => any)|null,
        menuElement?: Element|null,
        menuItemClicked?: ((...params: any[]) => any)|null
    };

    type ColumnOptions = {
        stackId?: string|null,
        stack?: (string|ColumnStackPlugin.StackOptions)|null
    };

    type StackOptions = {
        id: string,
        spreading?: boolean|null,
        collapsed?: boolean|null
    };

    type StackDefinition = {
        id: string,
        spreading?: boolean|null,
        collapsed?: boolean|null,
        children?: (string)[]|null,
        fields?: (string)[]|null,
        name?: string|null,
        activeColumn?: string|null
    };

    type StackConfiguration = {
        spreading?: boolean|null,
        collapsed?: boolean|null,
        activeColumn?: string|null
    };

}

declare class ColumnStackPlugin extends GridPlugin {

    constructor();

    public initialize(host: any, options?: any): void;

    public unload(host: any): void;

    public config(options?: any): void;

    public getConfigObject(gridOptions?: any): any;

    public getMemberIndices(colIndex: number): (number)[]|null;

    public collapseGroup(colIndex: number, collapsed?: boolean|null): (number)[]|null;

    public expandGroup(colIndex: number): (number)[]|null;

    public isColumnStackable(colIndices: (number)[]|null): boolean;

    public isColumnStacked(colIndices?: (number)[]|null): boolean;

    public isInactiveStackedColumn(colIndex: number): boolean;

    public isActiveStackedColumn(colIndex: number): boolean;

    public isStackingMode(colIndex: number): boolean;

    public isCollapsingMode(colIndex: number): boolean;

    public isColumnInCollection(colIndex: number): boolean;

    public isInCollection(colIndex: number): boolean;

    public isColumnCollapsed(colIndex: number): boolean;

    public isColumnActive(colIndex: number): boolean;

    public getStackId(colRef: number|string|null): string;

    public stackColumns(colRefs?: (number|string)[]|null, stackId?: string|null, options?: ColumnStackPlugin.StackConfiguration|null): boolean;

    public setStack(colRefs?: (number|string)[]|null, activeColRef?: (number|string)|null): boolean;

    public setStacks(stacks: (ColumnStackPlugin.StackDefinition)[]|null): void;

    public getStacks(): (ColumnStackPlugin.StackDefinition)[];

    public unstackColumns(colIndices?: (number)[]|null): boolean;

    public removeStack(stackId: string): boolean;

    public removeAllStacks(enableUpdateUI?: boolean|null): boolean;

    public setActiveColumn(activeColumn: number|string|null): boolean;

    public swapColumn(colRef: number|Event|null, swappingIndex: number): boolean;

    public getStackMemberIndices(stackId: string): (number)[];

    public getStackMemberIds(stackId: string): (string)[];

    public getColumnIdsByIndex(colIndices: number|(number)[]|null): (string)[];

    public getColumnIndicesByColumnIds(colIds: string|(string)[]|null): (string)[];

    public getColumnIdsByFields(fields: string|(string)[]|null): (string)[];

    public addColumnToStack(colRef: number|string|null, stackId: string): void;

    public removeColumnFromStack(colRef: number|string|null): void;

    public reorderStackColumns(colRefs: (number|string)[]|null, stackId: string): void;

    public setStackName(stackId: string, name: string): void;

    public getStackName(stackId: string): string;

    public getActiveColumnId(stackId: string): string;

    public getActiveColumnField(stackId: string): string;

    public getActiveColumnIndex(stackId: string): number;

    public addStackChild(stackId: string, colRef: number|string|null): void;

    public removeStackChild(stackId: string, colRef: number|string|null): void;

    public unsetParent(colRef: number|string|null): void;

    public reorderColumns(colList: (number|string)[]|null, destCol?: (number|string)|null): boolean;

    public moveColumnById(srcCol: number|string|null, destCol?: (number|string)|null): boolean;

    public hideStack(stackId: string, hidden?: boolean|null): void;

    public showStack(stackId: string): void;

    public isStackHidden(stackId: string): boolean|null|null;

    public moveStack(stackId: string, destCol?: (number|string)|null): boolean;

    public pinStack(stackId: string, side?: string|null): void;

    public unpinStack(stackId: string, dest?: (number|string)|null): void;

}

export default ColumnStackPlugin;
export { ColumnStackPlugin, ColumnStackPlugin as ColumnStack, ColumnStackPlugin as ColumnStackExtension };
