import {Ext} from "../../tr-grid-util/es6/Ext.js";
import {GridPlugin} from "../../tr-grid-util/es6/GridPlugin.js";
import {extendObject, injectCss, prettifyCss} from "../../tr-grid-util/es6/Util.js";
import {CellPainter} from "../../tr-grid-util/es6/CellPainter.js";
import {FilterBuilder} from "../../tr-grid-util/es6/FilterBuilder.js";
import {ElfUtil} from "../../tr-grid-util/es6/ElfUtil.js";

declare namespace ConditionalColoringPlugin {

    type Options = {
        predefinedColors?: any,
        blinkingDuration?: number|null,
        insertionBlinking?: (boolean|string)|null
    };

    type ColumnOptions = {
        conditions?: (ConditionalColoringPlugin.Condition)[]|null,
        colorText?: (string|boolean|ConditionalColoringPlugin.ColorText)|null,
        tickColor?: (string|boolean|ConditionalColoringPlugin.ColorText)|null,
        blinking?: (string|boolean|ConditionalColoringPlugin.Blinking)|null,
        field?: string|null
    };

    type ConditionalColoringOptions = {
        conditions?: (ConditionalColoringPlugin.Condition)[]|null,
        colorText?: (string|boolean|ConditionalColoringPlugin.ColorText)|null,
        tickColor?: (string|boolean|ConditionalColoringPlugin.ColorText)|null,
        field?: string|null
    };

    type Condition = {
        expression?: (string|((...params: any[]) => any)|any[])|null,
        backgroundColor?: string|null,
        color?: string|null,
        cssClass?: string|null,
        field?: string|null
    };

    type Blinking = {
        border?: boolean|null,
        field?: string|null,
        up?: string|null,
        down?: string|null,
        level?: (string|boolean)|null,
        duration?: number|null
    };

    type ColorText = {
        field?: string|null,
        upClass?: string|null,
        downClass?: string|null,
        levelClass?: string|null
    };

}

declare class ConditionalColoringPlugin extends GridPlugin {

    constructor();

    public hasMultiTableSupport(): boolean;

    public initialize(host: any, options?: any): void;

    public unload(host: any): void;

    public beforeProcessOption(optionName: string, optionVal: any): any;

    public config(options?: any): void;

    public getConfigObject(gridOptions?: any): any;

    public getColumnColoring(colIndex: number, options?: any): ConditionalColoringPlugin.ColumnOptions;

    public setColumnColoring(colIndex: number, columnOptions?: (ConditionalColoringPlugin.ColumnOptions|null)|null): void;

    public setConditionalColoring(colIndex: number, coloringOptions?: ConditionalColoringPlugin.ConditionalColoringOptions|null): void;

    public setColumnBlinking(colIndex: number, blinkingOptions?: (boolean|ConditionalColoringPlugin.Blinking)|null, field?: string|null): void;

    public blinkRow(rowIndex: number, blinkSignal: number, host?: any): void;

    public blinkCell(cell: any, colIndex: number, blinkSignal: number, rowData?: any): void;

    public setPredefinedColors(predefinedColors: any): void;

    public setInsertionBlinking(blinking: boolean|string|null): void;

    public getColumnPainter(colIndex: number): CellPainter|null;

    public applyColor(colIndex: number, cell: any, rowData?: any): void;

    public static cleanUpPrevRows(): void;

    public static setThemeColors(colors: { [key: string]:  string }|null): void;

    public reloadThemeColors(): Promise<any>|null;

}

export default ConditionalColoringPlugin;
export { ConditionalColoringPlugin, ConditionalColoringPlugin as ConditionalColoring, ConditionalColoringPlugin as ConditionalColoringExtension };
