import ElementWrapper from "./components/ElementWrapper.js";
import Cell from "./components/Cell.js";
import TrackLayout from "./util/TrackLayout.js";

declare interface ILayoutGrid extends ElementWrapper {

     hitTest(x: number, y: number, opt_omitSpan?: boolean|null): any|null|null;

     hitTestX(x: number): number;

     hitTestY(y: number): number;

     getDefaultColumnWidth(): number;

     setDefaultColumnWidth(val: number): void;

     getDefaultRowHeight(): number;

     setDefaultRowHeight(val: number): void;

     getColumnWidth(colIndex: number): number;

     setColumnWidth(colIndex: number, val: number): boolean;

     getRowHeight(rowIndex: number): number;

     setRowHeight(rowIndex: number, val: number): boolean;

     isColumnVisible(colIndex: number): boolean;

     hideColumn(colIndex: number, val?: boolean|null): boolean;

     isRowVisible(rowIndex: number): boolean;

     hideRow(rowIndex: number, val?: boolean|null): boolean;

     getMinimumColumnWidth(colIndex: number): number;

     setMinimumColumnWidth(colIndex: number, val: number): boolean;

     getMinimumRowHeight(rowIndex: number): number;

     setMinimumRowHeight(rowIndex: number, val: number): boolean;

     getCellLeft(colIndex: number): number;

     getCellTop(rowIndex: number): number;

     getCellWidth(colIndex: number, rowIndex: number): number;

     getCellHeight(colIndex: number, rowIndex: number): number;

     hasCellSpan(): boolean;

     getCellRowSpan(colIndex: number, rowIndex: number): number;

     setCellRowSpan(colIndex: number, rowIndex: number, rowSpan: number): boolean;

     getCellColSpan(colIndex: number, rowIndex: number): number;

     setCellColSpan(colIndex: number, rowIndex: number, colSpan: number): boolean;

     clearCellSpans(): boolean;

     clearColumnSpans(colIndex: number): boolean;

     clearColumnCustomizations(): void;

     clearRowCustomizations(): void;

     getColumnScalability(colIndex: number): boolean;

     setColumnScalability(colIndex: number, val: boolean): boolean;

     getRowScalability(rowIndex: number): boolean;

     setRowScalability(rowIndex: number, val: boolean): boolean;

     getCell(colIndex: number, rowIndex: number, opt_includeSpan?: boolean|null): ElementWrapper|null;

     getCellContent(colIndex: number, rowIndex: number): Element|Node|NodeList|null;

     setCellContent(colIndex: number, rowIndex: number, data: string|Element|ElementWrapper|any): Element|null;

     getColumn(index: number): ElementWrapper|null;

     getColumnCount(): number;

     setColumnCount(val: number): void;

     getRowCount(): number;

     setRowCount(val: number, noBinding?: boolean|null): void;

     getVisibleRowCount(): number;

     addRow(val: number): void;

     removeRow(val: number): void;

     setBorders(lft: boolean, top: boolean, rgt: boolean, btm: boolean): void;

     getColumnStyle(colIndex: number, styleName: string): string|null|null;

     setColumnStyle(colIndex: number, styleName: string, value: string): void;

     enableColumnClass(colIndex: number, className: string, enabled?: boolean|null): void;

     hasColumnClass(): boolean;

     enableRowClass(rowIndex: number, className: string, enabled: boolean): void;

     setRowStyle(rowIndex: number, styleName: string, value?: string|null|null, enabled?: boolean|null): void;

     getRelativePosition(obj: ElementWrapper|Element|Event|MouseEvent): any;

     getContentWidth(): number;

     getContentHeight(): number;

     getCellsByRowIndex(rowIndex: number): (ElementWrapper)[]|null;

     disableHorizontalGridLine(opt_bool?: boolean|null): void;

     disableVerticalGridLine(opt_bool?: boolean|null): void;

     enableShading(enabled: boolean): void;

     setTransposedShading(enabled: boolean): void;

     enableRowHighlighting(opt_bool?: boolean|null): void;

     getRowHighlight(): number;

     setRowHighlight(rowIndex: number): void;

     getSelectedRows(): (number)[]|null;

     getSelectedRowCount(): number;

     isSelectedRow(at: number): boolean;

     getRowAnchor(): number;

     setRowAnchor(at: number): void;

     setSelectedRow(rowIndex: number, opt_selected?: boolean|null): void;

     selectSingleRow(rowIndex: number): void;

     selectRowRange(rowIndex: number, length: number): void;

     clearSelectedRows(): number;

     getFirstSelectedRow(): number;

     getLastSelectedRow(): number;

     activateRows(from?: number|null, to?: number|null, opt_updateRange?: (number)[]|null): void;

     getFirstIndexInView(): number;

     getLastIndexInView(): number;

     isInCurrentView(index: number): boolean;

     getColumnIndex(colElem: Element|Node|null): number;

     getCellIndex(colIndex: number, cellElem: Element|null): number;

     getIndex(): number;

     getRowOffset(): number;

     stretchCell(cellRef: number|Cell|null, rowIndex?: number|null, opt_stretching?: boolean|null, onlyToTheRight?: boolean|null): Cell|null;

     unstretchCell(cellRef: number|Cell|null, rowIndex?: number|null): Cell|null;

     unstretchAllCells(): void;

     getStretchedCell(cellRef: number|Cell|null, rowIndex?: number|null): Cell|null;

     getStretchedCells(): (Cell)[];

     updateStretchedCells(): boolean;

     getColumnHost(): Element|null;

     reserveRightSpace(size: number): Element|null;

     getReservedSpace(): Element|null;

     getComputedStyle(): CSSStyleDeclaration|null;

     isColumnActive(colIndex: number): boolean;

     setRowAttributes(rowIndex: number, func: ((...params: any[]) => any)[]|null): void;

     getContextRow(rowIndex: number): any;

     selectColumn(colIndex: number, selected?: boolean|null): void;

     isSelectedColumn(colIndex: number): boolean;

     selectCell(colIndex: number, rowIndex: number, selected?: boolean|null): void;

     setCellBounds(colIndex: number, rowIndex: number, width: number, height: number): void;

}

export default ILayoutGrid;
export { ILayoutGrid };
