import ExtensionOptions from "../ExtensionOptions";
import Extensions from "../Extensions";
import { Dom } from "../../../tr-grid-util/es6/Dom.js";
import { Conflator } from "../../../tr-grid-util/es6/Conflator.js";
import { Ext } from "../../../tr-grid-util/es6/Ext.js";
import { EventDispatcher } from "../../../tr-grid-util/es6/EventDispatcher.js";
import { Engine } from "../../../tr-grid-util/es6/formula/Engine.js";
import { cloneObject, extendObject, arrayToObject, isEmptyObject, deepEqual } from "../../../tr-grid-util/es6/Util.js";
import { DateTime } from "../../../tr-grid-util/es6/DateTime.js";

import { RowDefinition, ROW_DEF } from "./RowDefinition.js";
import { ColumnDefinition, COL_DEF } from "./ColumnDefinition.js";
import SnapshotFiller from "./SnapshotFiller.js";
import StyleLoader from "./StyleLoader.js";
import FieldDefinition from "./FieldDefinition.js";
import DataConnector from "./DataConnector.js";
import RowDefSorter from "./RowDefSorter.js";

import { Core } from "../Core/grid/Core.js";
import { SortableTitlePlugin } from "../Core/grid/plugins/SortableTitlePlugin.js";
import { DataCache } from "../Core/data/DataCache.js";
import { DataTable } from "../Core/data/DataTable.js";
import { DataView } from "../Core/data/DataView.js";
import { ElementWrapper } from "../Core/grid/components/ElementWrapper.js";

declare namespace Grid {

    type SynapseConfig = {
        apiKey: string,
        contextApp: string,
        auth?: string|null,
        debug?: boolean|null
    };

    type ADCOptions = {
        productId?: string|null
    };

    type GridConfigObject = ExtensionOptions &  {
        columns?: (ColumnDefinition.Options|string)[]|null,
        defaultColumnOptions?: ColumnDefinition.Options|null,
        rows?: (RowDefinition.Options)[]|null,
        scrollbar?: boolean|null,
        linearWheelScrolling?: boolean|null,
        stepScroll?: boolean|null,
        autoHideScrollbar?: boolean|null,
        pageSize?: string|null,
        rowHeight?: number|null,
        headerRowHeight?: number|null,
        rowHighlighting?: boolean|null,
        autoLayoutUpdate?: (boolean|number)|null,
        rowVirtualization?: boolean|null,
        columnVirtualization?: boolean|null,
        topFreezingCount?: (number|null|boolean)|null,
        bottomFreezingCount?: (number|null)|null,
        borders?: boolean|null,
        gridlines?: boolean|null,
        verticalLines?: boolean|null,
        contentVerticalLines?: boolean|null,
        horizontalLines?: boolean|null,
        contentRightPadding?: number|null,
        contentBottomPadding?: number|null,
        dataConflationRate?: number|null,
        textSelect?: boolean|null,
        lang?: string|null,
        formulaEngine?: boolean|null,
        adcPollingInterval?: number|null,
        fieldCaching?: boolean|null,
        timeSeriesExpansion?: boolean|null,
        childDataField?: string|null,
        sorting?: SortableTitlePlugin.Options|null
    };

    type GridOptions = ExtensionOptions & {
        columns?: (ColumnDefinition.Options|string)[]|null,
        defaultColumnOptions?: ColumnDefinition.Options|null,
        rows?: (RowDefinition.Options)[]|null,
        fields?: (string)[]|null,
        rics?: (string)[]|null,
        plugins?: Extensions[],
        extensions?: Extensions[],
        scrollbar?: boolean|null,
        linearWheelScrolling?: boolean|null,
        stepScroll?: boolean|null,
        autoHideScrollbar?: boolean|null,
        pageSize?: string|null,
        rowHeight?: number|null,
        headerRowHeight?: number|null,
        noColumnDragging?: boolean|null,
        columnReorder?: boolean|null,
        rowHighlighting?: boolean|null,
        autoLayoutUpdate?: (boolean|number)|null,
        columnFreezing?: number|null,
        freezeColumn?: number|null,
        pinnedRightColumns?: number|null,
        staticDataRows?: any[]|null,
        dataFields?: (string)[]|null,
        rowVirtualization?: boolean|null,
        columnVirtualization?: boolean|null,
        topFreezingCount?: (number|null|boolean)|null,
        bottomFreezingCount?: (number|null)|null,
        borders?: boolean|null,
        gridlines?: boolean|null,
        verticalLines?: boolean|null,
        contentVerticalLines?: boolean|null,
        horizontalLines?: boolean|null,
        RTK?: any,
        ADC?: Grid.ADCOptions|null,
        synapse?: Grid.SynapseConfig|null,
        contentRightPadding?: number|null,
        contentBottomPadding?: number|null,
        dataConflationRate?: number|null,
        autoDateConversion?: boolean|null,
        textSelect?: boolean|null,
        lang?: string|null,
        scrollbarParent?: Element|null,
        formulaEngine?: boolean|null,
        adcPollingInterval?: number|null,
        fieldCaching?: boolean|null,
        timeSeriesExpansion?: boolean|null,
        childDataField?: string|null,
        topSection?: boolean|null,
        sorting?: SortableTitlePlugin.Options|null,
        dataComposed?: ((...params: any[]) => any)|null,
        beforeContentBinding?: ((...params: any[]) => any)|null,
        firstRendered?: ((...params: any[]) => any)|null,
        afterContentBinding?: ((...params: any[]) => any)|null
    };

    type RowReference = number|string|RowDefinition|null;

    type ColumnReference = number|string|ColumnDefinition|null;

}

declare class Grid extends EventDispatcher {

    constructor(placeholder?: (Element|null)|null, config?: Grid.GridOptions|null);

    public dispose(): void;

    public getElement(): Element|null;

    public removeEventListeners(): void;

    public listen(type: string, handler: ((...params: any[]) => any)|null): void;

    public initSubscription(): void;

    public updateLayout(): void;

    public updateRowData(opt_rowIndex?: number|null): void;

    public addHeaderSection(sectionName?: string|null): any;

    public addFooterSection(sectionName?: string|null): any;

    public setParent(elem: Element|null): void;

    public insertBefore(elem: Element|null): void;

    public getRelativePosition(obj: ElementWrapper|Element|Event|MouseEvent): any;

    public getCoreGrid(): Core|null;

    public initialize(gridOption?: Grid.GridOptions|null): void;

    public getConfigObject(gridOptions?: any): Grid.GridConfigObject;

    public setLocale(lang: string): void;

    public insertColumn(columnOption: ColumnDefinition.Options|string|null, idx?: number|null): void;

    public replaceColumn(columnOption: ColumnDefinition.Options|string|null, colRef: Grid.ColumnReference|null): void;

    public getFieldInfo(field: string): any;

    public loadFieldInfo(field: string): Promise<any>|null;

    public setColumns(columns: (any)[]|null): void;

    public restoreColumns(columns: (any)[]|null, byId?: boolean|null): void;

    public setFields(ary: (string)[]|null): void;

    public removeColumn(colRef: Grid.ColumnReference|null): void;

    public removeColumns(colRefs: (Grid.ColumnReference)[]|null): void;

    public removeAllColumns(): void;

    public moveColumn(fromColIndex: number|(number)[]|null, toColIndex: number): boolean;

    public moveColumnById(srcCol: number|string|null, destCol?: (number|string)|null): boolean;

    public reorderColumns(colRefs: number|string|(number|string)[]|null, destCol?: (number|string)|null): boolean;

    public hideColumn(colRef: Grid.ColumnReference|null, hidden?: boolean|null): void;

    public hideColumns(colRefs: (Grid.ColumnReference)[]|null, hidden?: boolean|null): void;

    public showAllColumns(): void;

    public addDataFields(fieldRef: string|(string)[]|null, referrer?: string|null): boolean;

    public removeDataFields(fieldRef: string|(string)[]|null, referrer?: string|null): boolean;

    public removeFieldReferrer(referrer: string): void;

    public sortColumn(colRef: Grid.ColumnReference|null, opt_order?: string|null): void;

    public setColumnName(colIndex: number, str: string): void;

    public getColumnBackgroundColor(colRef: Grid.ColumnReference|null): string;

    public setColumnBackgroundColor(colRef: Grid.ColumnReference|null, color: string): void;

    public setColumnRenderer(colRef: Grid.ColumnReference|null, func?: ((...params: any[]) => any)|null): void;

    public activateColumnRenderer(colRef: Grid.ColumnReference|null, id?: string|null, func?: ((...params: any[]) => any)|null): void;

    public setColumnFormatter(colIndex: number, formatter?: any|((...params: any[]) => any)|null): void;

    public setColumnSorter(colRef: Grid.ColumnReference|null, func?: ColumnDefinition.SortLogic|null): void;

    public insertRow(rowOption?: (RowDefinition.Options|string)|null, rowRef?: Grid.RowReference|null): RowDefinition|null;

    public insertSegmentSeparator(rowOption?: RowDefinition.Options|null, rowRef?: Grid.RowReference|null): RowDefinition|null;

    public insertRows(rowOptions: (RowDefinition.Options|string)[]|null, rowRef?: Grid.RowReference|null, opt_fields?: (string)[]|null): void;

    public addStaticDataRows(dataRows: any[]|null, fields?: (string)[]|null): void;

    public updateDataSet(records: (any)[]|null, rowIdentifier?: string|null): void;

    public removeRow(rowRef: Grid.RowReference|null): RowDefinition|null;

    public removeRows(rowRefs: (Grid.RowReference)[]|null): (RowDefinition)[]|null;

    public removeAllRows(): void;

    public hideRow(rowRef: Grid.RowReference|null, hidden?: boolean|null): void;

    public hideRows(rowRefs: Grid.RowReference | (Grid.RowReference)[]|null, hidden?: boolean|null): void;

    public showRows(rowRefs: Grid.RowReference | (Grid.RowReference)[]|null, shown?: boolean|null): void;

    public unhideAllRows(): void;

    public hasHiddenRow(): boolean;

    public setRic(rowRef: Grid.RowReference|null, str: string, options?: RowDefinition.Options|null): RowDefinition|null;

    public unlinkChain(rowRef: Grid.RowReference|null): void;

    public setRowContent(rowRef: Grid.RowReference|null, str: string): void;

    public toggleRowExpansion(rid: string|number|null, force?: boolean|null): void;

    public setRics(ary: (string)[]|null): void;

    public getRowCount(): number;

    public getPageItemCount(): number;

    public getDataRowCount(): number;

    public getColumnCount(): number;

    public getColumnDataType(colIndex: number): string;

    public getColumnDefinition(colIndex: number): ColumnDefinition|null;

    public getColumnDefinitions(): (ColumnDefinition)[];

    public getColumnDefinitionById(colId: string): ColumnDefinition|null;

    public getColumnDefinitionsById(colIds: (string)[]|null): ColumnDefinition|null;

    public getRowType(rowRef: number|string|null): string;

    public getRowDefinition(rowRef: number|string|null): RowDefinition|null;

    public getRowDefinitions(): (RowDefinition)[];

    public getAllRowDefinitions(inclAutoGenerated?: boolean|null): (RowDefinition)[];

    public getRowData(rowIndex: number): any;

    public getMultipleRowData(obj?: any): (any)[];

    public setRicData(ric: string, values: any): void;

    public getAllRics(): (string)[]|null;

    public hasRic(): boolean;

    public getRic(rowRef?: (string|number)|null): string;

    public setRowData(rowRef: Grid.RowReference|null, values: any): void;

    public setStaticRowData(rowRef: Grid.RowReference|null, values: any): void;

    public setStaticData(rowRef: Grid.RowReference|null, field: string, value: any): void;

    public clearColumnData(colRefs: Grid.ColumnReference|(Grid.ColumnReference)[]|null): void;

    public getColumnIndex(colRef: Grid.ColumnReference|null): number;

    public getColumnIndices(colRefs: (Grid.ColumnReference)[]|null): (number)[];

    public getColumnId(colIndex: number): string;

    public getColumnIds(): (string)[];

    public getColumnField(colIndex: number): string;

    public getColumnFields(): (string)[];

    public getColumnNames(): (string)[];

    public getAllFields(): (string)[];

    public freezeColumn(colIndex?: number|null, pinnedRightColumns?: number|null): void;

    public pinColumn(colRef: Grid.ColumnReference|(Grid.ColumnReference)[]|null, side?: string|null): boolean;

    public unpinColumn(colRef: Grid.ColumnReference|(Grid.ColumnReference)[]|null, dest?: Grid.ColumnReference|null): boolean;

    public unpinAllColumns(): boolean;

    public updateColumnTitle(): void;

    public isSorting(): boolean;

    public getSortedColumnIndex(): number;

    public getSortOrder(): string;

    public clearSort(): void;

    public getSortingStates(): (any)[];

    public getDataView(): DataView|null;

    public setPage(pageIndex: number): boolean;

    public setPageSize(size: number): void;

    public getPageIndex(): number;

    public getPageSize(): number;

    public getPageCount(): number;

    public toggleChain(rowRef: Grid.RowReference|null): void;

    public setClassification(rowRef: Grid.RowReference|null, fields: (string)[]|null): boolean;

    public contains(elem: Element|null): boolean;

    public isFocused(): boolean;

    public focus(): void;

    public requestRowRefresh(): void;

    public updateLayout(): void;

    public getExtension(str: string): any;

    public activateADCPolling(activated?: boolean|null): void;

    public refreshADCData(): boolean;

    public logDT(opt_options?: any): void;

    public logDV(opt_options?: any): void;

    public replaceRow(rowRef: Grid.RowReference|null, rowOption?: (RowDefinition.Options|string)|null): RowDefinition|null;

    public scrollToColumn(colIndex: number, leftOfView?: boolean|null): boolean;

    public scrollToRow(rowIndex: number, topOfView?: boolean|null): void;

    public getScrollLeft(): number;

    public getScrollTop(): number;

    public setScrollLeft(pixels: number): void;

    public setScrollTop(pixels: number): void;

    public scrollRight(pixels: number): void;

    public scrollDown(pixels: number): void;

    public getScrollWidth(): number;

    public getScrollHeight(): number;

    public getVScrollView(): any;

}

declare function borders(gridOptions?: any): any;

declare function idx(colRef: Grid.ColumnReference|null): void;

declare function colCount(rowRef: number|string|null): string;

export { Grid };
export default Grid;
