import { arrayToObject } from "../../../tr-grid-util/es6/Util.js";
import { DataCache } from "../Core/data/DataCache.js";
import { DataTable } from "../Core/data/DataTable.js";

declare namespace RowDefinition {

    type Options = {
        ric?: string|null,
        permId?: string|null,
        values?: (any[]|any)|null,
        fields?: (string)[]|null,
        asChain?: boolean|null,
        chainRic?: string|null,
        collapsed?: boolean|null,
        label?: (string|null)|null,
        hidden?: boolean|null,
        realTime?: boolean|null,
        info?: any,
        rowId?: string|null
    };

    type RowTypes = {
        CONTENT: string,
        CHAIN: string,
        CONSTITUENT: string,
        GROUP_HEADER: string,
        SUBGROUP_HEADER: string,
        GROUP_MEMBER: string
    };

}

declare class RowDefinition {

    constructor(rowOptions?: RowDefinition.Options|null);

    public dispose(): void;

    public isDisposed(): boolean;

    public initialize(rowOptions?: RowDefinition.Options|null): void;

    public getRowId(): string;

    public getConfigObject(rowOptions?: RowDefinition.Options|null): RowDefinition.Options|null;

    public getDataId(): string;

    public getType(): string;

    public setDataSource(dataSource: DataCache|null, subs?: any): boolean;

    public getDataSource(): DataCache|null;

    public getRowData(): any;

    public getData(field: string): any;

    public setStaticRowData(data: { [key: string]:  any }|any[], indexToFieldMap?: (string)[]|null): void;

    public setStaticData(field: string, value: any): void;

    public updateRowData(data: { [key: string]:  any }|any[], indexToFieldMap?: (string)[]|null): void;

    public copyRowData(srcRowDef: RowDefinition|null): void;

    public cloneRowData(obj?: any, exceptionObj?: any): any;

    public setRowData(data: { [key: string]:  any }): void;

    public resetRowData(): void;

    public setData(field: string, value: any): void;

    public getUserInput(): string;

    public getRic(): string;

    public getPermId(): string;

    public getSymbol(): string;

    public getDisplayText(): string;

    public getLabel(): string|null|null;

    public isRowHeader(): boolean;

    public isChain(): boolean;

    public isConstituent(): boolean;

    public static hasChain(rowDef: RowDefinition|null): boolean;

    public isChainCollapsed(): boolean;

    public isChainExpanded(): boolean;

    public isAutoGenerated(): boolean;

    public isRealTimeRow(): boolean;

    public subscribeForUpdates(subs?: any): boolean;

    public unsubscribeForUpdates(keepData?: boolean|null): null;

    public isSubscribing(): boolean;

    public getSubId(): string;

    public addUpdate(changes: any): boolean;

    public getUpdates(): { [key: string]:  number }|null;

    public resetUpdates(): void;

    public registerToView(view: DataView|null, destRowId?: string|null): boolean;

    public unlinkChain(): void;

    public collapseChain(): boolean;

    public expandChain(): boolean;

    public toggleChain(): boolean;

    public getChildren(): (RowDefinition)[]|null;

    public getDescendants(ary?: (RowDefinition)[]|null): (RowDefinition)[];

    public getAllDescendantIds(): (string)[]|null;

    public getChildCount(): number;

    public countChildInView(): number;

    public getParent(): RowDefinition|null;

    public getDepthLevel(): number;

    public setRowInfo(obj: any): void;

    public getRowInfo(): any;

    public static toRowId(rowDef: RowDefinition|null): string;

    public static dispose(rowDef: RowDefinition|null): void;

}

declare const ROW_DEF: string;

declare const ROW_TYPES: RowDefinition.RowTypes;

export {RowDefinition, ROW_DEF, ROW_TYPES};
export default RowDefinition;
