/******/ (() => { // webpackBootstrap
/******/ 	"use strict";

// UNUSED EXPORTS: Grid, default

;// ./node_modules/tr-grid-util/es6/Dom.js
/** @namespace */
let Dom = {};

/** Shorthand for creating an element with class
* @public
* @function
* @param {string} tagName
* @param {string=} className
* @return {!Element}
*/
Dom.create = function(tagName, className) {
	let elem = document.createElement(tagName);
	if(className) {
		elem.className = className;
	}
	return elem;
};
/** Shorthand for creating a div element with class
* @public
* @function
* @param {string=} className
* @return {!Element}
*/
Dom.div = function(className) {
	return Dom.create("div", className);
};
/** Shorthand for creating a span element with textContent
* @public
* @function
* @param {(string|number)=} textContent
* @param {string=} className
* @return {!Element}
*/
Dom.text = function(textContent, className) {
	let elem = Dom.create("span", className);
	if(textContent || textContent === 0) {
		elem.textContent = textContent;
	}
	return elem;
};

/** Shorthand for appending multiple children
* @public
* @function
* @param {Element} parentElem
* @param {...Node|Array} children
* @return {Element} Parent element
*/
Dom.appendChild = function(parentElem, children) {
	if(!parentElem) {
		return null;
	}
	let i, ary;
	if(Array.isArray(arguments[1])) {
		ary = arguments[1];
		i = 0;
	} else {
		ary = arguments;
		i = 1;
	}
	let len = ary.length;
	while(i < len) {
		parentElem.appendChild(ary[i]);
		++i;
	}
	return parentElem;
};
/** Shorthand for appending multiple children in a tree like manner
* @public
* @function
* @param {Element} parentElem
* @param {...Node|Array} children The first child from the given array will be treated as a parent of all subsequence items.
* @return {Element} Parent element
* @example
* Dom.buildTree(p1,
*   ch1,
*   ch2,
*   [ch3_p1,
*	 ch3_ch1,
*	 ch3_ch2
*   ]
* );
*/
Dom.buildTree = function(parentElem, children) {
	if(!parentElem) {
		return null;
	}
	let args = arguments;
	let len = args.length;
	for(let i = 1; i < len; ++i) {
		let child = args[i];
		if(child) {
			let childElem;
			if(Array.isArray(child)) {
				childElem = Dom.buildTree.apply(null, child);
			} else if(child.nodeType === 1) {
				childElem = /** @type{Element} */(child);
			}
			if(childElem) {
				parentElem.appendChild(childElem);
			}
		}
	}
	return parentElem;
};
/** Put the given element out of the document
* @function
* @param {Node|Element} child
* @return {Element} Parent Element
*/
Dom.removeParent = function (child) {
	if(!child) { return null; }
	let parent = child.parentNode;
	if(parent) {
		parent.removeChild(child);
	}
	return parent;
};
/** Safely set parent for the specified child element
* @function
* @param {Node|Element} child
* @param {Node|Element} parent
* @return {Element} Parent Element
*/
Dom.setParent = function (child, parent) {
	if(!child) { return null; }
	if(parent) {
		parent.appendChild(child);
		return parent;
	} else {
		return Dom.removeParent(child);
	}
};
/** Remove all child nodes
* @function
* @param {Node|Element} elem
* @param {number=} count
*/
Dom.removeChildren = function (elem, count) {
	if(elem) {
		let childCount = elem.childNodes.length;
		if(count == null || count > childCount) {
			count = childCount;
		}
		while(--count >= 0) {
			elem.removeChild(elem.lastChild);
		}
	}
};
/** Ensure that element will contain only the given nodes (all other existing nodes are removed).
* @public
* @function
* @param {Element} elem
* @param {*} val Element, Node, string, number, or everything else.
*/
Dom.setContent = function(elem, val) {
	if(elem) {
		if(val || val === 0) {
			let nt = val.nodeType;
			if(nt === 1 || nt === 3) {
				if(val !== elem.lastChild) {
					Dom.removeChildren(elem);
					elem.appendChild(/** @type{Node} */(val));
				}
			} else if(Array.isArray(val)) {
				Dom.removeChildren(elem); // Could be slow, if duplicates are given
				Dom.appendChild(elem, val);
			} else {
				elem.textContent = val; // Implicit text conversion
			}
		} else {
			Dom.removeChildren(elem);
		}
	}
};

/** A quick way to toggle attribute on and off.
* @function
* @param {Element} elem
* @param {string} attr Attribute name (e.g. disabled, readonly, style, class)
* @param {*=} bool Truthy or falsy value. If specified, this method is equivalent to setter method.
*/
Dom.toggleAttribute = function (elem, attr, bool) {
	if(bool) {
		elem.setAttribute(attr, (typeof bool === "string") ? bool : "");
	} else if(bool == null) {
		if(elem.hasAttribute(attr)) {
			elem.removeAttribute(attr);
		} else {
			elem.setAttribute(attr, "");
		}
	} else {
		elem.removeAttribute(attr);
	}
};
/** A quick way to toggle class on and off. Note that this is a replacement for classList.toggle() as IE10 doesn't support it.
* @see {@link https://developer.mozilla.org/en-US/docs/Web/API/Element/classList}
* @function
* @param {Element} elem
* @param {string} classStr Non space class name
* @param {*=} bool Truthy or falsy value. If specified, this method is equivalent to setter method.
*/
Dom.toggleClass = function (elem, classStr, bool) {
	if(bool) {
		elem.classList.add(classStr);
	} else if(bool == null) {
		if(elem.classList.contains(classStr)) {
			elem.classList.remove(classStr);
		} else {
			elem.classList.add(classStr);
		}
	} else {
		elem.classList.remove(classStr);
	}
};
/** A quick way to add multiple classes without altering existing classes in the element. This does not necessarily mean that this method has high performance.
* @function
* @param {Element} elem
* @param {...string} classes CSS Class names without any space in the name. Array can be given as well
* @see {@link https://developer.mozilla.org/en-US/docs/Web/API/Element/classList}
*/
Dom.addClasses = function (elem, classes) {
	if(elem) {
		let ary = Array.isArray(classes) ? classes : arguments;
		let len = ary.length;
		for(let i = 0; i < len; ++i) {
			let str = ary[i];
			if(str && typeof str == "string") {
				elem.classList.add(str);
			}
		}
	}
};
/** A quick way to remove multiple classes without altering unrelated classes in the element. This does not necessarily mean that this method has high performance.
* @function
* @param {Element} elem
* @param {...string} classes CSS Class names without any space in the name. Array can be given as well
* @see {@link https://developer.mozilla.org/en-US/docs/Web/API/Element/classList}
*/
Dom.removeClasses = function (elem, classes) {
	if(elem) {
		let ary = Array.isArray(classes) ? classes : arguments;
		let len = ary.length;
		for(let i = 0; i < len; ++i) {
			let str = ary[i];
			if(str && typeof str == "string") {
				elem.classList.remove(str);
			}
		}
	}
};

/** @public
* @function
* @param {Event} e
*/
Dom.preventDefault = function(e) {
	e.preventDefault();
	e.stopPropagation();
};
/** @public
* @function
* @param {Event} e
*/
Dom.stopPropagation = function(e) {
	e.stopPropagation();
};


/** Get client prosition from the given target. Supports only for modern browser (Gecko 1.9+, modern Webkit)
* @public
* @function
* @param {!Element|Event|MouseEvent} target
* @param {*=} retObj An object that stores returned values
* @return {!Object} Return object with property x and y
*/
Dom.getClientPosition = function(target, retObj) {
	let pos = retObj || {"x": 0, "y": 0};
	if(target["getBoundingClientRect"]){
		let box = target["getBoundingClientRect"]();
		pos["x"] = box.left;
		pos["y"] = box.top;
	} else { //Event
		pos["x"] = target.clientX;
		pos["y"] = target.clientY;
	}

	return /**@type {!Object} */(pos);
};
/** Get position of A element that is relative to the B element. <br>
* Top-left of B is used as registration a point (0, 0) for x and y. <br>
* For example, if A is located to right of B, "x" will be positive, If A is located below B, "y" will be positive.
* @public
* @function
* @param {!Element|Event|MouseEvent} A HTMLElement, Browser's event, or mouse event object
* @param {!Element|Event|MouseEvent} B HTMLElement, Browser's event, or mouse event object
* @param {*=} retObj An object that stores returned values
* @return {!Object} an object with "x" and "y" values
* @example
* Dom.getRelativePosition(popupA, document.body); // Get position of popupA that is relative to the document body
*/
Dom.getRelativePosition = function(A, B, retObj) {
	retObj = Dom.getClientPosition(A, retObj);
	let bp = Dom.getClientPosition(B);

	retObj["x"] -= bp["x"];
	retObj["y"] -= bp["y"];
	return /**@type {!Object} */(retObj);
};
/** The method returns closest ancestor element or the element itself with specified class string. Otherwise, it returns null
* @public
* @function
* @param {Element|Node|undefined} elem
* @param {string} classStr The class string can only be one single class without spaces
* @return {Element}
*/
Dom.closestElement = function(elem, classStr) {
	let n = elem;
	while (n) {
		if(n.classList) { // DocumentFragment does not have classList property
			if (n.classList.contains(classStr)) {
				return /** @type{Element} */(n);
			}
		}
		n = n.parentNode || n.host; // ShadowRoot has host, but not parentNode
	}
	return null;
};
/** The method returns closest ancestor element or the element itself with the specified tagName. Otherwise, it returns null
* @public
* @function
* @param {Element|Node|undefined} elem
* @param {string} tn Tag name in uppercase format (e.g. "DIV")
* @return {Element}
*/
Dom.closestTagName = function(elem, tn) {
	let n = elem;
	while(n) {
		if(n.tagName) {
			if(n.tagName == tn) {
				return /** @type{Element} */(n);
			}
		}
		n = n.parentNode || n.host; // ShadowRoot has host, but not parentNode
	}
	return null;
};

/* harmony default export */ const es6_Dom = ((/* unused pure expression or super */ null && (Dom)));


;// ./node_modules/tr-grid-util/es6/Conflator.js
/** Conflator is used to stop multiple executions of a function in the specified period of time by merging multiple calls into one single call. <br>
* Conflator guarantees that only one single call will be executed.
* @constructor
* @param {(Function|number)=} func Bound function that will use this conflator
* @param {(number|Function)=} ms The delay time in millisecond before executing the function
* @param {*=} thisObj "this" object to be bound with the given function. If the function is already bound, there is no need to provide thisObj parameter
* @example
* let c = new Conflator(function() { console.log("Executed"); }, 1000);
* for(let i = 10; --i >= 0;) {
*   c.conflate(i); // Only one "Executed" text will be logged to console after one second
* }
*/
let Conflator = function (func, ms, thisObj) {
	this._onConflated = this._onConflated.bind(this);

	this._data = [];

	if(typeof func == "function") {
		this.setFunction(func, thisObj);
	} else if (typeof ms == "function") { // Parameter can be swapped places
		this.setFunction(ms, thisObj);
	}

	if(typeof ms == "number") {
		this.setConflatingTime(ms);
	} else if (typeof func == "number") { // Parameter can be swapped places
		this.setConflatingTime(func);
	}
};

/** @type {!Array}
* @private
*/
Conflator.prototype._data; // Array to store all conflated data
/** @type {Function}
* @private
*/
Conflator.prototype._func = null;
/** @type {number}
* @private
*/
Conflator.prototype._delay = 0;
/** @type {number}
* @private
*/
Conflator.prototype._timerId = 0;
/** @type {boolean}
* @private
*/
Conflator.prototype._conflated = false;
/** @type {boolean}
* @private
*/
Conflator.prototype._disabled = false;


/** Any existing timer and stored data will be cleared.
* @public
* @return {Array} All stored data. Return null if there is no data
*/
Conflator.prototype.reset = function () {
	if(this._timerId) {
		clearTimeout(this._timerId);
		this._timerId = 0;
	}

	return this.popAllData();
};
/** All stored data will be cleared
* @public
* @return {Array} All stored data. Return null if there is no data
*/
Conflator.prototype.popAllData = function() {
	if(this._data.length) {
		let data = this._data;
		this._data = [];
		return data;
	}
	return null;
};
/** @public */
Conflator.prototype.dispose = function () {
	if(this._timerId) {
		clearTimeout(this._timerId);
		this._timerId = 0;
	}
	if(this._data.length) {
		this._data.length = 0;
	}
	this._func = null; // Release user bound function
};

/** @public
* @param {*=} obj Any data that is needed to be stored for later usage
* @return {boolean}
*/
Conflator.prototype.conflate = function (obj) {
	if(this._disabled) {
		return false;
	}
	if(obj != null) {
		this._data.push(obj);
	}
	if(this._conflated) {
		return false; // Conflating operation is already completed
	}
	if(!this._timerId) {
		this._timerId = setTimeout(this._onConflated, this._delay);
	}
	return true;
};
/** @public
* @return {boolean}
*/
Conflator.prototype.isConflating = function () {
	return (this._timerId > 0 && !this._conflated && !this._disabled);
};

/** @private */
Conflator.prototype._onConflated = function () {
	this._conflated = true;

	try {
		this._func();
	} catch (err) {
		console.error(err.message || err);
	}

	this._timerId = 0;
	this._data.length = 0;
	this._conflated = false;
};

/** @public
* @return {!Array}
*/
Conflator.prototype.getData = function () {
	return this._data;
};
/** @public
* @param {Function} func Bound function that will use this conflator
* @param {*=} thisObj "this" object to be bound with the given function. If the function is already bound, there is no need to provide thisObj parameter
*/
Conflator.prototype.setFunction = function (func, thisObj) {
	this._func = func;
	if(func) {
		if(thisObj) {
			this._func = func.bind(thisObj);
		} else {
			this._func = func;
		}
	} else {
		this._func = null;
	}
};
/** @public
* @param {number} ms The delay time in millisecond before executing the function
*/
Conflator.prototype.setConflatingTime = function (ms) {
	this._delay = ms || 0;
	if(this._delay < 0) {
		this._delay = 0;
	}
};

/** @public
* @return {number} milli second
*/
Conflator.prototype.getConflatingTime = function () {
	return this._delay;
};

/** @public
* @param {boolean=} opt_disabled
*/
Conflator.prototype.disable = function (opt_disabled) {
	this._disabled = opt_disabled !== false;
};
/** @public
* @return {boolean}
*/
Conflator.prototype.isEnabled = function () {
	return !this._disabled;
};
/** @public
* @param {boolean=} opt_enabled
*/
Conflator.prototype.enable = function (opt_enabled) {
	this._disabled = opt_enabled === false;
};

/* harmony default export */ const es6_Conflator = ((/* unused pure expression or super */ null && (Conflator)));


;// ./node_modules/tr-grid-util/es6/Ext.js
/** Provide ability for prototype based Class to inherits another class
* @namespace
* @example
* let BaseClass = function() {};
* BaseClass.prototype.method = function() {};
* BaseClass.prototype._member = 1;
* let DerivedClass = function() {};
* Ext.inherits(DerivedClass, BaseClass); // Derived class will have methods and members of the base class
* // To call base class method
* DeriveClass["base"](this, "methodName", ...params);
*/
let Ext = {};

/** @public
* @function
* @param {Function} childCtor
* @param {Function} parentCtor
*/
Ext.inherits = function (childCtor, parentCtor) {
	/** @constructor
* @ignore
*/
	function tempCtor() { }
	tempCtor.prototype = parentCtor.prototype;
	childCtor.prototype = new tempCtor();
	childCtor.prototype.constructor = childCtor;

	/** Calls superclass constructor/method. <br>
* This function is only available if you use goog.inherits to <br>
* express inheritance relationships between classes. <br>
* NOTE: This is a replacement for goog.base and for superClass_ <br>
* property defined in childCtor.
* @param {!Object} me Should always be "this".
* @param {(string|null)=} methodName The method name to call. Calling superclass constructor can be done with the special string 'constructor'.
* @param {...*} var_args The arguments to pass to superclass <br>
* method/constructor.
* @return {*} The return value of the superclass method/constructor.
*/
	childCtor["base"] = function(me, methodName, var_args) {
		if(!methodName) { methodName = 'constructor'; }
		// Copying using loop to avoid deop due to passing arguments object to function. This is faster in many JS engines as of late 2014.
		let len = arguments.length;
		let args = new Array(len); // http://jsperf.com/creating-an-array
		for (let i = 2; i < len; i++) {
			args[i - 2] = arguments[i];
		}
		return parentCtor.prototype[methodName].apply(me, args);
	};
};

/* harmony default export */ const es6_Ext = (Ext);


;// ./node_modules/tr-grid-util/es6/EventDispatcher.js
/** Abstract base class that provides event management methods for derived class
* @constructor
*/
let EventDispatcher = function () {};

/** @type {Object.<string, Function>}
* @protected
* @ignore
*/
EventDispatcher.prototype._events = null;
/** @type {number}
* @protected
* @ignore
*/
EventDispatcher.prototype._listenerCount = 0;

//#region public function of event
/** Add the function handler into the specific event.
* @public
* @param {string} type Event name
* @param {Function} handler Event handler
*/
EventDispatcher.prototype.addEventListener = function(type, handler) {
	if(!type || !handler) { return; }
	if(!this._events) {
		this._events = {};
	}

	let listeners = this._events[type];
	if(listeners) {
		if(listeners.indexOf(handler) < 0) {
			listeners.push(handler);
			++this._listenerCount;
		}
	} else {
		this._events[type] = [handler];
		++this._listenerCount;
	}
};
/** Remove the function handler from the specific event.
* @public
* @param {string} type Event name
* @param {Function} handler Event handler
*/
EventDispatcher.prototype.removeEventListener = function(type, handler) {
	let listeners = (this._events) ? this._events[type] : null;
	if(listeners) {
		let at = listeners.indexOf(handler);
		if(at >= 0) {
			listeners.splice(at, 1);
			--this._listenerCount;
		}
	}
};
/** Remove all function handlers from all events.
* @public
*/
EventDispatcher.prototype.removeAllEventListeners = function() {
	this._events = null;
	this._listenerCount = 0;
};

/** Check whether the specified event has a function handler.
* @public
* @param {string} type Event name
* @return {boolean}
*/
EventDispatcher.prototype.hasListener = function(type) {
	if(this._listenerCount) {
		if(type) {
			let listeners = this._events ? this._events[type] : null;
			return listeners ? (listeners.length > 0) : false;
		} else {
			return true;
		}
	} else {
		return false;
	}
};
/** Get event listener (function handler) from the specified event.
* @public
* @param {string} type Event name
* @param {number=} idx=0 Index of the listener to be retrieved. This is used when there are multiple event listeners
* @return {Function}
*/
EventDispatcher.prototype.getListener = function(type, idx) {
	if(type) {
		let listeners = this._events ? this._events[type] : null;
		if(listeners) {
			if(!idx) {
				idx = 0;
			}
			return listeners[idx] || null;
		}
	}
	return null;
};
/** A shorthand to retrieve function from an object and add it as an Eventlistener
* @public
* @function
* @param {Object} obj Object that contains a handler with the same name as the given `type`
* @param {string} type Event name
* @example
* let obj = {"mouseUp": function(e) { console.log(e); }};
* plugin.addListener(obj, "mouseUp");
* plugin.addListener(obj, "mouseDown");
*/
EventDispatcher.prototype.addListener = function(obj, type) {
	let func = obj ? obj[type] : null;
	if(typeof func === "function") {
		this.addEventListener(type, func);
	}
};

/** Executed before dispatching, allowing modification of user's event argument object
* @protected
* @ignore
* @param {string} type Event name
* @param {Object} eventArg User's event argument object
* @return {Object} New event argument object
*/
EventDispatcher.prototype._prepareEventArguments = function(type, eventArg) {
	return eventArg;
};

/** Publish the data to the listener.
* @protected
* @ignore
* @param {string} type Event name
* @param {Object} eventArg Event arguments
*/
EventDispatcher.prototype._dispatch = function(type, eventArg) {
	let listeners = this._events ? this._events[type] : null;
	if(listeners) {
		eventArg = this._prepareEventArguments(type, eventArg);
		let len = listeners.length;
		for(let i = 0; i < len; ++i) {
			listeners[i](eventArg);
		}
	}
};

/** @public
* @function
* @param {Event} e
*/
let preventDefault = function(e) {
	if(e && e.preventDefault) {
		e.preventDefault();
		e.stopPropagation();
	}
};

/** @public
* @function
* @param {Event} e
*/
EventDispatcher.preventDefault = preventDefault;

/* harmony default export */ const es6_EventDispatcher = (EventDispatcher);


;// ./node_modules/tr-grid-util/es6/formula/VariableToken.js

/** @constructor
* @param {string} str
* @param {number=} index
*/
let VariableToken = function(str, index) {
	str = str.trim();

	if(this._inputConverter[str]) {
		str = this._inputConverter[str];
	}

	this._input = str;
	if(index != null) {
		this._index = index;
	}

	if(this._constants[str] != null) {
		this._value = this._constants[str];
		this._type = typeof this._value;
		return;
	}

	let firstChar = str.charAt(0);
	let lastChar = (str.length > 1) ? str.charAt(str.length - 1) : "";
	if(firstChar === '"' && lastChar === '"') {
		this._value = str.substring(1, str.length - 1);
		this._type = "string";
		return;
	}
	if(firstChar === "'" && lastChar === "'") {
		this._value = str.substring(1, str.length - 1);
		this._type = "string";
		return;
	}
	if(firstChar === "(" && lastChar === ")") {
		this._resolveVariableGroup(str);
		return;
	}
	if(firstChar === "[" && lastChar === "]") {
		this._resolveArray(str); // Array has undefined value (this._value == null)
		return;
	}

	let adc = str.match(/TR\.\w+\(/);
	if(adc) { // ADC field with parameters
		this._type = "dynamic"; // ADC field with params always a dynamic token field
		return;
	}

	let funcs = str.match(/[\w._]+\(/g);
	if(funcs && funcs.length === 1) { // There must be only one method in the given str
		this._resolveFunction(str, funcs[0]); // Function has undefined value (this._value == null)
		return;
	}

	let words = str.match(/[\w._♦]+/g);
	if(words && words.length > 1) { // A group of expression without parentheses
		this._resolveVariableGroup(str);
		return;
	}

	if(firstChar === "♦") {
		this._type = "token";
		return;
	}

	let num = Number(str);
	if(num === num) { // If not NaN
		this._value = num;
		this._type = "number";
		return;
	}

	this._type = "dynamic";
};

/** @private
* @type {string}
*/
VariableToken.prototype._input = "";
/** @private
* @type {number}
*/
VariableToken.prototype._index = -1;
/** @private
* @type {string}
*/
VariableToken.prototype._type = "";
/** @private
* @type {Array.<VariableToken>}
*/
VariableToken.prototype._subVars = null;
/** @private
* @type {string}
*/
VariableToken.prototype._funcName = "";

/** @private
* @type {*}
*/
VariableToken.prototype._value;

/** @private
* @type {string}
*/
VariableToken.prototype._valueWithTokenField = "";

/** Static variables for input conversion
* @private
* @type {Object.<string, string>}
*/
VariableToken.prototype._inputConverter = {
	"TRUE": "true",
	"FALSE": "false",
	"PI": "Math.PI",
	"TRUE()": "true",
	"FALSE()": "false",
	"PI()": "Math.PI"
};
/** Static variables for defining constant
* @private
* @type {Object.<string, number>}
*/
VariableToken.prototype._constants = {
	"true": true,
	"false": false,
	"null": null,
	"NaN": NaN,
	"Math.E": Math.E, // returns Euler's number
	"Math.PI": Math.PI, // returns PI
	"Math.SQRT2": Math.SQRT2, // returns the square root of 2
	"Math.SQRT1_2": Math.SQRT1_2, // returns the square root of 1/2
	"Math.LN2": Math.LN2, // returns the natural logarithm of 2
	"Math.LN10": Math.LN10, // returns the natural logarithm of 10
	"Math.LOG2E": Math.LOG2E, // returns base 2 logarithm of E
	"Math.LOG10E": Math.LOG10E
};

/** @public
* @return {string}
*/
VariableToken.prototype.getExpression = function() {
	if(this._type === "dynamic") {
		return 'this.__D("' + this._input + '", __R)';
	}
	if(this._type === "group") {
		return /** @type{string} */(this._value);
	}
	if(this._type === "function") {
		if(this._subVars) {
			return "this.__F[" + this._index + "](" + this._subVars.join(", ") + ")";
		} else {
			return "this.__F[" + this._index + "]()";
		}
	}
	if(this._type === "array") {
		if(this._subVars) {
			return "[" + this._subVars.join(", ") + "]";
		} else {
			return "[]";
		}
	}
	return this._input;
};

/** @public
 * @return {string}
 */
VariableToken.prototype.getInputWithTokenField = function() {
	if(this._type === "dynamic") {
		return '♠' + this._input + '♥';
	}
	if(this._type === "group") {
		return /** @type{string} */(this._valueWithTokenField);
	}
	if(this._type === "function") {
		return this._funcName + "(" + this.getChildInputWithToken() + ")";
	}
	if(this._type === "array") {
		return "[" + this.getChildInputWithToken() + "]";
	}
	return this._input;
};

/** @public
 * @return {string}
 */
VariableToken.prototype.getChildInputWithToken = function() {
	let len = this._subVars ? this._subVars.length : 0;
	if(len <= 0) {
		return "";
	}
	let inputs = new Array(len);
	for(let i = 0; i < len; ++i) {
		inputs[i] = this._subVars[i].getInputWithTokenField();
	}
	return inputs.join(", ");
};

/** @public
* @function
* @return {string}
*/
VariableToken.prototype.toString = VariableToken.prototype.getExpression;

/** @public
* @return {*}
*/
VariableToken.prototype.getValue = function() { return this._value; };
/** @public
* @return {string}
*/
VariableToken.prototype.getInput = function() { return this._input; };
/** @public
* @return {number}
*/
VariableToken.prototype.getIndex = function() { return this._index; };
/** @public
* @param {number} num
*/
VariableToken.prototype.setIndex = function(num) { this._index = num; };

/** @public
* @return {string}
*/
VariableToken.prototype.getFunctionName = function() { return this._funcName; };

/** @public
* @param {Object=} opt_obj
* @return {string}
*/
VariableToken.prototype.getFieldNames = function(opt_obj) {
	if(this._subVars) {
		if(opt_obj) {
			let len = this._subVars.length;
			for(let i = 0; i < len; ++i) {
				this._subVars[i].getFieldNames(opt_obj);
			}
		}
		return this._subVars[0].getFieldNames();
	} else if(this._type === "dynamic") {
		if(opt_obj) {
			opt_obj[this._input] = true;
		}
		return this._input;
	}

	return "";
};

/** @private
* @param {string} str
* @returns {string}
*/
VariableToken.prototype._resolveVariableGroup = function(str) {
	let i = this._subVars ? this._subVars.length : 0;
	this._type = "group";

	this._value = str.replace(/[\w._♦]+/g, function(varStr) {
		if(!this._subVars) {
			this._subVars = [];
		}
		let varToken = new VariableToken(varStr, this._subVars.length);
		this._subVars.push(varToken);
		return varToken.getExpression();
	}.bind(this));

	this._valueWithTokenField = str.replace(/[\w._♦]+/g, function(varStr) {
		return this._subVars[i++].getInputWithTokenField();
	}.bind(this));

	return this._value;
};

/** @private
* @param {string} str
* @param {string} matchedName
*/
VariableToken.prototype._resolveFunction = function(str, matchedName) {
	this._type = "function";
	this._funcName = matchedName.substr(0, matchedName.length - 1); // Extract function name

	// str = str.replace(/ /g, ""); // Replace all white spaces
	str = str.replace(matchedName, ""); // Strip function name and "("
	str = str.substr(0, str.length - 1); // Strip ")"
	if(str) {
		this._subVars = [];
		let params = str.split(",");
		let len = params.length;
		for(let i = 0; i < len; ++i) {
			this._subVars[i] = new VariableToken(params[i], i);
		}
	}
};
/** @private
* @param {string} str
*/
VariableToken.prototype._resolveArray = function(str) {
	this._type = "array";

	str = str.substr(1, str.length - 1); // Strip "[" and "]"
	if(str) {
		this._subVars = [];
		let params = str.split(",");
		let len = params.length;
		for(let i = 0; i < len; ++i) {
			this._subVars[i] = new VariableToken(params[i], i);
		}
	}
};

/* harmony default export */ const formula_VariableToken = (VariableToken);


;// ./node_modules/tr-grid-util/es6/formula/Formula.js




/** Trigger when data within the realtime formula has been changed
* @event Formula#dataCacheChanged
*/

/** @constructor
* @extends {EventDispatcher}
* @param {string=} expression
* @param {Object=} context
*/
let Formula = function(expression, context) {
	this._input = expression || "";
	if(context != null) {
		this._context = context;
		if (context["_DATA"]) { // _DATA is an internal context used for getting cached data
			this["__D"] = context["_DATA"];
		}
	}
	this["__F"] = [];
	this._fnTokens = [];
	this._fields = [];
	this._fieldMap = {};
	this._dataCache = {};
	this._intervalServices = {};
	this._formulaCount = 1; // Initial count
	// this._addEvent("dataCacheChanged");
	// this._addEvent("dataRequired");
	// this._addEvent("formulaChanged");
	this._onDataCacheChanged = this._onDataCacheChanged.bind(this);
	this.dispatchDataChanged = this.dispatchDataChanged.bind(this);

	this._parse(this._input);
};
es6_Ext.inherits(Formula, es6_EventDispatcher);

/** @type {Object.<string, boolean>}
* @const
* @public
*/
Formula._isBoundFormula = {
	"RECORD": true,
	"TR": true,
	"MTXSERIES": true,
	"NOW": true,
	"TODAY": true,
	// Adfin - StylesFunction
	"ADSTYLEATTRIBUTE": true,
	// Adfin - BdFunction
	"ADBONDDERIV": true,
	"ADBONDYIELD": true,
	"BDSETTLE": true,
	"ACCRUED": true,
	// Adfin - CfFunction
	"CFYLD": true,
	// Adfin - DfFunction
	"DFFORMATDATE": true,
	"DFADDPERIOD": true,
	"DFADDWD": true,
	"DFADDMONTHS": true,
	"DFADDYEARS": true,
	"DFADJUSTTOWD": true,
	"DFLASTWD": true,
	"DFFINDDATED": true,
	"DFCOUNTDAYS": true,
	"DFCOUNTWD": true,
	// Adfin - FxFunction
	"ADFXSWPTODEP": true,
	"FXCALCPERIOD": true,
	// Adfin - OpFunction
	"OPCALCDERIV": true,
	"OPIMPLIEDVOL": true,
	// Adfin - UtFunction
	"ADFORMAT": true,
	"ADHISTORYVALUE": true


};
/** @private
* @type {string}
*/
Formula.prototype._input = ""; // Initial user input
/** @private
* @type {string}
*/
Formula.prototype._expression = ""; // Evaluated expression
/** @private
* @type {string}
*/
Formula.prototype._alias = ""; // Alias name of formula
/** @private
* @type {Function}
*/
Formula.prototype._main = null; // Function resulted from resolving an expression

/** @private
* @type {Array.<Function>}
*/
Formula.prototype["__F"] = null; // Sub-methods that is used by this formula
/** @private
* @type {Array.<VariableToken>}
*/
Formula.prototype._fnTokens = null; // Sub-methods meta-data
/** @private
* @type {Array.<string>}
*/
Formula.prototype._fields = null; // Required field names from the expression
/** @private
* @type {Object.<string, boolean>}
*/
Formula.prototype._fieldMap = null; // Required field names from the expression

/** @private
* @type {string}
*/
Formula.prototype._reference = "";

/** @private
* @type {Object.<string, Function>}
*/
Formula.prototype._context = null;

/** @private
* @type {*}
*/
Formula.prototype._rtService = null;

/** @private
* @type {Object.<string, *>}
*/
Formula.prototype._intervalServices = null;

/** @private
* @type {Object.<string, *>}
*/
Formula.prototype._dataCache = null;

/** @private
* @type {string}
*/
Formula.prototype._dataLastError = "";

/** @private
* @type {string}
*/
Formula.prototype._inputWithTokenField = "";

/** @private
* @type {number}
*/
Formula.prototype._formulaCount = 0;

/** @private
* @type {Object.<string, Function>}
*/
Formula._invalidOps = {
	"+=": true,
	"-=": true,
	"*=": true,
	"/=": true,
	"|=": true,
	"&=": true,
	"^=": true
};

/** @public
*/
Formula.prototype.dispose = function() {
	if(this._rtService) {
		this._rtService["dispose"]();
		this._rtService = null;
	}
	for(let serviceName in this._intervalServices) {
		// No need to dispose the static object
		this._intervalServices[serviceName]["removeEventListener"]("dataChanged", this.dispatchDataChanged);
	}
	this._intervalServices = null;
	this.unlistenAll();
};

/** @public
* @param {Object.<string, *>} rowData
* @return {*}
*/
Formula.prototype.calculate = function(rowData) {
	if (this._main) {
		this._dataLastError = ""; // Clear last realtime error message
		let res = this._main(rowData);
		switch(this._dataLastError) {
		case "Pending data":		// Don't show data if realtime data is still pending
			res = null;
			break;
		case "Pending - no erase":
			res = "...";			// Data update is pending - do not erase previous value
			break;
		}
		return res;
	}
	return null;
};

/** Function resulted from resolving an expression
* @public
* @return {Function} Returns function that takes an JSON object as a parameter. Return null if function is not valid JavaScript codes.
*/
Formula.prototype.getFunction = function() { return this._main; };
/** @public
* @return {Array.<string>}
*/
Formula.prototype.getFieldNames = function() { return this._fields; };

/** For internal use only
* @public
* @ignore
* @param {string} str
*/
Formula.prototype._setReference = function(str) {
	this._reference = str;
};
/** @public
* @return {string}
*/
Formula.prototype.getReference = function() {
	return this._reference;
};

/** Evaluated expression
* @public
* @return {string}
*/
Formula.prototype.getExpression = function() {
	return this._expression;
};
/** @public
* @function
* @return {string}
*/
Formula.prototype.toString = Formula.prototype.getReference;

/** Initial user input
* @public
* @return {string}
*/
Formula.prototype.getInput = function() { return this._input; };

/** Get alias name of formula. If alias does not exist, user input is returned instead
* @public
* @return {string}
*/
Formula.prototype.getAlias = function() { return this._alias || this._input; };
/** Set alias name of formula
* @public
* @param {string} str
*/
Formula.prototype.setAlias = function(str) {
	this._alias = str;
};
/** Add formula count
* @public
*/
Formula.prototype.addFormulaCount = function() {
	this._formulaCount++;
};
/** Subtract formula count. Return true if formula count is zero.
* @public
* @return {boolean}
*/
Formula.prototype.subtractFormulaCount = function() {
	return --this._formulaCount === 0;
};

/** @public
*/
Formula.prototype.updateContext = function() {
	if(!this._context) { return; }

	let len = this._fnTokens.length;
	for(let i = 0; i < len; ++i) {
		let fname = this._fnTokens[i].getFunctionName();
		this["__F"][i] = this._context[fname] || Formula._emptyFunction;
		if (Formula._isBoundFormula[fname]) {
			this["__F"][i] = this["__F"][i].bind(this);
		}
		//this.__F[i] = this._logger.bind(this, i);
	}
};

/** @private
* @param {number} index
*/
Formula.prototype._logger = function(index) {
	console.log("[" + index + "] " + this._fnTokens[index].getFunctionName() + " is executed" );
};
/** @private
*/
Formula._emptyFunction = function() {};

/** @private
* @param {string} str
* @return {boolean}
*/
Formula.prototype._detectCodeInjection = function(str) {
	let injection = false;

	if(str.match(/\+\+|--/)) { // ++ and -- which modifies current variable is not allowed
		injection = true;
	}

	if(!injection) {
		let matches = str.match(/([^=]=+)/g) || [];
		for (let i = matches.length; --i >= 0;) {
			let m = matches[i];
			let lastTwoChars = m.substring(m.length - 2);
			if (Formula._invalidOps[lastTwoChars]) {
				injection = true;
				break;
			}
		}
	}

	// if(injection) {
	// console.log("WARNING: Code injection detected");
	// }

	return injection;
};
/** Upper case only formula expression. All other cases are ignored
* @public
* @param {string} exp
* @return {string}
*/
Formula.toUpperCase = function(exp) {
	if(!exp || exp.charAt(0) !== "=") { //Prevent impacting to UDIP search field
		return exp || "";
	}

	let tokens = [];
	let calcHierarchy = [];

	// Tokenize string constants first because they has enclosure (single quotes and double quotes)
	exp = Formula.tokenizeQuotes(exp, tokens, calcHierarchy);

	exp = exp.toUpperCase(); // Upper case the remaining

	// May have the new string token from ADC field with parameter
	// Regex for adc fileds
	// TR 				: start with tr
	// (\.[\w]+)+ 		: dot follow by any character at least 1
	// \( 				: start parentheses
	// [^\(\)]* 		: any except parentheses
	// \) 				: end parentheses
	// (\.[a-zA-Z]+)* 	: dot follow by any character (optional)

	exp = exp.replace(/TR(\.[\w ]+)+\([^\(\)]*\)(\.[a-zA-Z]+)*/g, Formula._encloseWithTR);

	exp = Formula.resolveTokens(exp, tokens, calcHierarchy); // Restore the string

	return exp;
};
/** @description Wrap TR function to ADC field with parameter e.g.
* TR.Revenue(Scale=2)
* TR.Revenue(Scale=2).calcdate,
* TR.Revenue(Period=FY0,Scale=2)
* @private
* @param {string} adcField
* @return {string}
*/
Formula._encloseWithTR = function(adcField) {
	return "TR(\"" + adcField + "\")";
};
/** @public
* @ignore
* @param {string} exp
* @param {Array.<VariableToken>=} tokens
* @param {Array.<Array.<number>>=} calcHierarchy
* @return {string}
*/
Formula.tokenizeQuotes = function(exp, tokens, calcHierarchy) {
	if(!tokens) {
		tokens = [];
	}
	if(!calcHierarchy) {
		calcHierarchy = [];
	}

	let index;
	let matched = false;
	let tokenIndices = [];
	do {
		matched = false;
		exp = exp.replace(/(["'])(?:(?=(\\?))\2.)*?\1/g, function(str) { // eslint-disable-line
			matched = true;
			index = tokens.length;
			tokens.push(new formula_VariableToken(str, index));
			tokenIndices.push(index);
			return "♦" + index + "♦";
		});
	} while(matched);
	calcHierarchy.push(tokenIndices);
	return exp;
};
/** @public
* @ignore
* @param {string} exp
* @param {Array.<VariableToken>=} tokens
* @param {Array.<Array.<number>>=} calcHierarchy
* @return {string}
*/
Formula.tokenizeAdcWithParams = function(exp, tokens, calcHierarchy) {
	if(!tokens) {
		tokens = [];
	}
	if(!calcHierarchy) {
		calcHierarchy = [];
	}

	let index;
	let matched = false;
	let tokenIndices = [];
	do {
		matched = false;
		exp = exp.replace(/TR\.\w+\([^\)]*\)+/g, function(str) { // eslint-disable-line
			matched = true;
			index = tokens.length;
			tokens.push(new formula_VariableToken(str, index));
			tokenIndices.push(index);
			return "♦" + index + "♦";
		});
	} while(matched);
	calcHierarchy.push(tokenIndices);
	return exp;
};
/** @public
* @ignore
* @param {string} exp
* @param {Array.<VariableToken>=} tokens
* @param {Array.<Array.<number>>=} calcHierarchy
* @return {string}
*/
Formula.replaceUnwantedCharacters = function(exp, tokens, calcHierarchy) {
	// Replace all double spaces with single space
	while(exp.indexOf("  ") >= 0) {
		exp = exp.replace(/ {2}/g, " ");
	}

	// Remove all dollar signs (e.g. $CF_BID + $CF_ASK)
	exp = exp.replace(/\$/g, "");

	// Replace single ampersand with plus sign to support string concatenation with ampersand
	exp = exp.replace(/\&+/g, function(str) {
		if(str === "&") {
			return "+";
		}
		return str;
	});

	/**
	 * Replace all assignment operator with equality operator, by using
	 * regular expression for detecting only "=" that is not a starting point of a formula,
	 * so the character "=" is not replaced with "==" during conversion.
	 *
	 * @example
	 * =ABS(COL5+COL6) -> =ABS(COL5+COL6)
	 * =(COL5=COL6)    -> =(COL5==COL6)
	 */
	exp = exp.replace(/(<=|>=|!=|!==|===|==|=(?!([\w]*\([^()]*\))))/g, function(m) {
		if (m === "=") {
			return "==";
		}

		return m;
	});

	return exp;
};
/** @private
* @param {string} exp
* @param {Array.<VariableToken>=} tokens
* @param {Array.<Array.<number>>=} calcHierarchy
* @return {string}
*/
Formula.prototype._tokenizeParentheses = function(exp, tokens, calcHierarchy) {
	if (!tokens) {
		tokens = [];
	}

	if(!calcHierarchy) {
		calcHierarchy = [];
	}

	let fnStrToIndex = {}; // To avoid duplicate function

	/**
	 * It is better to not duplicate content of regular expressions to avoid any possible typos
	 * or discrepancies.
	 * Therefore we copy existing regular expression and assign new flags to it instead.
	 */
	let firstMatchRE = /[\w._]*(?:=|=[\w]*)?\([^()]*\)/;
	let globalRE = new RegExp(firstMatchRE.source, 'g');

	while (exp.match(firstMatchRE)) {
		let tokenIndices = [];

		exp = exp.replace(globalRE, function (fnStr, group) { // eslint-disable-line
			// Remove the first equal sign.
			if (fnStr.charAt(0) === "=") {
				fnStr = fnStr.substr(1, fnStr.length);
			}

			let index = fnStrToIndex[fnStr];

			if (index == null) {
				index = tokens.length;
				tokens.push(new formula_VariableToken(fnStr, index));
				fnStrToIndex[fnStr] = index;
			}

			tokenIndices.push(index);

			return "♦" + index + "♦";
		});

		calcHierarchy.push(tokenIndices);
	}

	return exp;
};
/** @private
* @param {string} exp
* @param {Array.<VariableToken>=} tokens
* @param {Array.<Array.<number>>=} calcHierarchy
* @return {string}
*/
Formula.prototype._tokenizeConstants = function(exp, tokens, calcHierarchy) {
	if(!tokens) {
		tokens = [];
	}
	if(!calcHierarchy) {
		calcHierarchy = [];
	}

	let tokenIndices = [];
	exp = exp.replace(/[\w._♦]+/g, function(varStr) {
		if (varStr.charAt(0) === "♦") {
			return varStr;
		}
		let index = tokens.length;
		tokens.push(new formula_VariableToken(varStr, index));
		tokenIndices.push(index);
		return "♦" + index + "♦";
	});
	calcHierarchy.push(tokenIndices);
	return exp;
};
/** @public
* @param {string} exp
* @param {Array.<VariableToken>} tokens
* @param {Array.<Array.<number>>} calcHierarchy
* @return {string}
*/
Formula.resolveTokens = function(exp, tokens, calcHierarchy) {
	for(let i = calcHierarchy.length; --i >= 0;) { // Must be done in reverse order
		let indices = calcHierarchy[i];
		let len = indices.length;
		for(let j = 0; j < len; ++j) {
			let index = indices[j];
			let token = tokens[index];
			exp = exp.replace(new RegExp("♦" + index + "♦", "g"), function() { // eslint-disable-line
				return token.getExpression();
			});
		}
	}
	return exp;
};

/** @private
* @param {string} exp
* @param {Array.<VariableToken>} tokens
* @param {Array.<Array.<number>>} calcHierarchy
* @return {string}
*/
Formula._getInputWithTokenField = function(exp, tokens, calcHierarchy) {
	for(let i = calcHierarchy.length; --i >= 0;) { // Must be done in reverse order
		let indices = calcHierarchy[i];
		let len = indices.length;
		for(let j = 0; j < len; ++j) {
			let index = indices[j];
			let token = tokens[index];
			exp = exp.replace(new RegExp("♦" + index + "♦", "g"), function() { // eslint-disable-line
				return token.getInputWithTokenField();
			});
		}
	}
	return "=" + exp;
};

/** @private
* @param {string} exp
* @return {Function}
*/
Formula.prototype._parse = function(exp) {
	this._expression = "";
	this._main = null;

	this["__F"].length = 0;
	this._fnTokens.length = 0;
	this._fields.length = 0;

	if(!exp) { return null; }

	if(exp.charAt(0) === "=") { // Remove the first equal sign
		exp = exp.substr(1);
	}
	exp = exp.trim();
	exp = exp.replace(/<>/g, "!=");
	exp = exp.replace(/‛|‘|’/g, "'"); // Left and right single quote will be converted to simple single quote
	exp = exp.replace(/‟|“|”/g, '"'); // Left and right double quote will be converted to simple double quote

	if(!exp || this._detectCodeInjection(exp)) { return null; }

	let tokens = [];
	let calcHierarchy = [];

	exp = Formula.tokenizeQuotes(exp, tokens, calcHierarchy); // Tokenize string constants first because they has enclosure (single quotes and double quotes)

	exp = Formula.tokenizeAdcWithParams(exp, tokens, calcHierarchy);

	exp = Formula.replaceUnwantedCharacters(exp, tokens, calcHierarchy);

	exp = this._tokenizeParentheses(exp, tokens, calcHierarchy); // Tokenize all methods and parenthesis group

	exp = this._tokenizeConstants(exp, tokens, calcHierarchy); // Tokenize the rest of the constants and dynamic values

	// Used for change field name in formula
	this._inputWithTokenField = Formula._getInputWithTokenField(exp, tokens, calcHierarchy);

	// Resolve each function tokens into valid JavaScript codes
	exp = Formula.resolveTokens(exp, tokens, calcHierarchy);

	// Re-organize function structure and index, and retrieve field data
	let fields = {};
	let len = tokens.length;
	for(let i = 0; i < len; ++i) {
		let token = tokens[i];
		if(token.getFunctionName()) {
			let fIndex = this._fnTokens.length;
			exp = exp.replace(new RegExp("this\\.__F\\[" + i + "\\]", "g"), "this.__F[" + fIndex + "]");
			token.setIndex(fIndex);
			this._fnTokens.push(token);
		}
		token.getFieldNames(fields);
	}

	this._expression = exp; // The expression has been finalized
	this.updateContext();

	this._main = this._createFunction(this._expression);
	if(!this._main) {
		return null;
	}

	this._fieldMap = fields;
	this._fields = Object.keys(fields);
	return this._main;
};

/** @private
* @param {string} expression
* @return {Function}
*/
Formula.prototype._createFunction = function (expression) {
	let func = null;
	try {
		func = new Function("__R", "return " + expression + ";");
		func = func.bind(this);
		func({}); // Try to execute the method once to check error
	} catch(err) {
		func = null;
		// window.console.log('Expression: "' + this._input + '" contains the following error: \n' + err.message);
	}
	return func;
};

/** @private
* @param {string} colName
* @param {Object.<string, *>} rowData
* @return {*}
*/
Formula.prototype["__D"] = function (colName, rowData) {
	return rowData[colName];
};
/** @public
* @param {string} field
* @param {string=} opt_src
* @returns {boolean}
*/
Formula.prototype.addField = function(field, opt_src) {
	if(this._fieldMap[field]) { // Existing in this._fields
		return false;
	}
	if(field.indexOf("undefined") >= 0) { // We can remove this block if the ADC service can ignore the invalid requested field.
		// REMARK: ADC cannot support the invalid parameter. It will respond with ERROR and do not return any ADC data anymore.
		return false; // WORKAROUND: Field name must not contain with string 'undefined'
	}

	this._fields.push(field);
	this._fieldMap[field] = true;
	this._dispatch("dataRequired", { "field": field, "src": opt_src || "adc" });
	return true;
};

/** Change field in formula
* @public
* @param {string} oldName
* @param {string} newName
* @return {boolean}
*/
Formula.prototype.changeField = function (oldName, newName) {
	let index = this._fields.indexOf(oldName);
	if (index < 0) {
		return false;
	}
	this._fields.splice(index, 1);
	if (this._fields.indexOf(newName) < 0) {
		this._fields.push(newName);
	}
	let oldInput = this._input;
	let oldTokenField = "♠" + oldName + "♥";
	let newTokenField = "♠" + newName + "♥";
	this._inputWithTokenField = this._inputWithTokenField.replace(new RegExp("♠[^♠]*♥", "g"), function(str) {
		return (str === oldTokenField) ? newTokenField : str;
	});
	this._input = this._inputWithTokenField.replace(new RegExp("♠[^♠]*♥", "g"), function(str) {
		return str.substr(1, str.length - 2); // Remove "♠" and "♥"
	});
	this._expression = this._expression.replace(new RegExp('this\\.__D\\("' + oldName + '", __R\\)', "g"), 'this.__D("' + newName + '", __R)');
	this._main = this._createFunction(this._expression);
	this._dispatch("formulaChanged", { "oldInput": oldInput, "formula": this });
	return true;
};
/** Change multiple field in formula
* @public
* @param {Array.<string>} oldNames
* @param {Array.<string>} newNames
* @return {boolean}
*/
Formula.prototype.changeFields = function (oldNames, newNames) {
	if(!Array.isArray(oldNames) || !Array.isArray(newNames)) {
		return false;
	}
	if(oldNames.length !== newNames.length) {
		return false;
	}

	let oldInput = this._input;
	for(let i = oldNames.length; --i >= 0;) {
		let oldName = oldNames[i];
		let newName = newNames[i];

		let index = this._fields.indexOf(oldName);
		if (index < 0) {
			continue;
		}

		this._fields.splice(index, 1);
		if (this._fields.indexOf(newName) < 0) {
			this._fields.push(newName);
		}

		let oldTokenField = "♠" + oldName + "♥";
		let newTokenField = "♠" + newName + "♥";

		this._inputWithTokenField = this._inputWithTokenField.replace(new RegExp("♠[^♠]*♥", "g"), function(str) { // eslint-disable-line
			return (str === oldTokenField) ? newTokenField : str;
		});
		this._input = this._inputWithTokenField.replace(new RegExp("♠[^♠]*♥", "g"), function(str) { // eslint-disable-line
			return str.substr(1, str.length - 2); // Remove "♠" and "♥"
		});
		this._expression = this._expression.replace(new RegExp('this\\.__D\\("' + oldName + '", __R\\)', "g"), 'this.__D("' + newName + '", __R)');
	}

	this._main = this._createFunction(this._expression);
	this._dispatch("formulaChanged", { "oldInput": oldInput, "formula": this });
	return true;
};

//#region Realtime service
/** @public
* @return {*}
*/
Formula.prototype.getRealtimeService = function() {
	return this._rtService;
};

/** @public
* @param {Object} rtService
*/
Formula.prototype.setRealtimeService = function(rtService) {
	if (this._rtService) {
		this._rtService["dispose"]();
	}
	this._rtService = rtService;
	this._rtService["addEventListener"]("dataChanged", this._onDataCacheChanged);
};
//#endregion Realtime service

//#region Interval service
/** @public
 * @param {string} funcName
* @return {*}
*/
Formula.prototype.getIntervalService = function(funcName) {
	return this._intervalServices[funcName];
};

/** @public
* @param {string} funcName
* @param {Object} newService
*/
Formula.prototype.setIntervalService = function(funcName, newService) {
	if(!newService) {
		return;
	}

	let service = this._intervalServices[funcName];
	if (service) {
		service["removeEventListener"]("dataChanged", this.dispatchDataChanged); // Service will stop only if no event listener anymore
	}
	this._intervalServices[funcName] = newService;
	newService["addEventListener"]("dataChanged", this.dispatchDataChanged);
};
//#endregion Interval service

//#region formula data cache
/** @public
 * @param {string} symbol
* @return {*}
*/
Formula.prototype.getCachedData = function(symbol) {
	return this._dataCache[symbol];
};

/** @public
* @param {string} symbol
* @param {Object.<string,*>} values
*/
Formula.prototype.setCachedData = function(symbol, values) {
	this._dataCache[symbol] = values;
};

/** @public
* @param {string} msg
*/
Formula.prototype.setDataError = function(msg) {
	this._dataLastError = msg;
};

/** @public
* @param {Object=} e
*/
Formula.prototype.dispatchDataChanged = function(e) {
	// Just dispatch the event. No need to cache any data in formula.
	this._dispatch("dataCacheChanged", {});
};

/** @public
* @param {Object} e
*/
Formula.prototype._onDataCacheChanged = function(e) {
	// Cache data for real-time function since real-time function has its own RIC list. Then dispatch the event.
	let symbol = e['ric'];
	let values = /** @type{Object.<string, *>} */(e['values']);
	let data = this._dataCache[symbol];
	if (data) {
		for (let field in values) {
			data[field] = values[field];
		}
	} else {
		this._dataCache[symbol] = values;
	}
	this._dispatch("dataCacheChanged", {});
};
//#endregion formula data cache

Formula._proto = Formula.prototype;

/* harmony default export */ const formula_Formula = (Formula);


;// ./node_modules/tr-grid-util/es6/locale/translation-en.js
/**
* @namespace
*/
let translationEn = {
	"sun": "Sun",
	"mon": "Mon",
	"tue": "Tue",
	"wed": "Wed",
	"thu": "Thu",
	"fri": "Fri",
	"sat": "Sat",
	"Sunday": "Sunday",
	"Monday": "Monday",
	"Tuesday": "Tuesday",
	"Wednesday": "Wednesday",
	"Thursday": "Thursday",
	"Friday": "Friday",
	"Saturday": "Saturday",

	"jan": "Jan",
	"feb": "Feb",
	"mar": "Mar",
	"apr": "Apr",
	"may": "May",
	"jun": "Jun",
	"jul": "Jul",
	"aug": "Aug",
	"sep": "Sep",
	"oct": "Oct",
	"nov": "Nov",
	"dec": "Dec",
	"January": "January",
	"February": "February",
	"March": "March",
	"April": "April",
	"May": "May",
	"June": "June",
	"July": "July",
	"August": "August",
	"September": "September",
	"October": "October",
	"November": "November",
	"December": "December",

	"am": "am",
	"AM": "AM",
	"pm": "pm",
	"PM": "PM"
};

/* harmony default export */ const translation_en = (translationEn);

;// ./node_modules/tr-grid-util/es6/locale/translation-de.js
/**
* @namespace
*/
let translationDe = {
	"sun": "So",
	"mon": "Mo",
	"tue": "Di",
	"wed": "Mi",
	"thu": "Do",
	"fri": "Fr",
	"sat": "Sa",
	"Sunday": "Sonntag",
	"Monday": "Montag",
	"Tuesday": "Dienstag",
	"Wednesday": "Mittwoch",
	"Thursday": "Donnerstag",
	"Friday": "Freitag",
	"Saturday": "Samstag",

	"jan": "Jan",
	"feb": "Feb",
	"mar": "Mrz",
	"apr": "Apr",
	"may": "Mai",
	"jun": "Jun",
	"jul": "Jul",
	"aug": "Aug",
	"sep": "Sep",
	"oct": "Okt",
	"nov": "Nov",
	"dec": "Dez",
	"January": "Januar",
	"February": "Februar",
	"March": "März",
	"April": "April",
	"May": "Mai",
	"June": "Juni",
	"July": "Juli",
	"August": "August",
	"September": "September",
	"October": "Oktober",
	"November": "November",
	"December": "Dezember",

	"am": "am",
	"AM": "AM",
	"pm": "pm",
	"PM": "PM"
};

/* harmony default export */ const translation_de = (translationDe);

;// ./node_modules/tr-grid-util/es6/locale/translation-ja.js
/**
* @namespace
*/
let translationJa = {
	'sun': '日',
	'mon': '月',
	'tue': '火',
	'wed': '水',
	'thu': '木',
	'fri': '金',
	'sat': '土',
	'Sunday': '日曜日',
	'Monday': '月曜日',
	'Tuesday': '火曜日',
	'Wednesday': '水曜日',
	'Thursday': '木曜日',
	'Friday': '金曜日',
	'Saturday': '土曜日',

	'jan': '1月',
	'feb': '2月',
	'mar': '3月',
	'apr': '4月',
	'may': '5月',
	'jun': '6月',
	'jul': '7月',
	'aug': '8月',
	'sep': '9月',
	'oct': '10月',
	'nov': '11月',
	'dec': '12月',
	'January': '1月',
	'February': '2月',
	'March': '3月',
	'April': '4月',
	'May': '5月',
	'June': '6月',
	'July': '7月',
	'August': '8月',
	'September': '9月',
	'October': '10月',
	'November': '11月',
	'December': '12月',

	'am': '午前',
	'AM': '午前',
	'pm': '午後',
	'PM': '午後'
};

/* harmony default export */ const translation_ja = (translationJa);

;// ./node_modules/tr-grid-util/es6/locale/translation-zh.js
/**
* @namespace
*/
let translationZh = {
	'sun': '周日',
	'mon': '周一',
	'tue': '周二',
	'wed': '周三',
	'thu': '周四',
	'fri': '周五',
	'sat': '周六',
	'Sunday': '星期日',
	'Monday': '星期一',
	'Tuesday': '星期二',
	'Wednesday': '星期三',
	'Thursday': '星期四',
	'Friday': '星期五',
	'Saturday': '星期六',

	'jan': '1月',
	'feb': '2月',
	'mar': '3月',
	'apr': '4月',
	'may': '5月',
	'jun': '6月',
	'jul': '7月',
	'aug': '8月',
	'sep': '9月',
	'oct': '10月',
	'nov': '11月',
	'dec': '12月',
	'January': '一月',
	'February': '二月',
	'March': '三月',
	'April': '四月',
	'May': '五月',
	'June': '六月',
	'July': '七月',
	'August': '八月',
	'September': '九月',
	'October': '十月',
	'November': '十一月',
	'December': '十二月',

	'am': '上午',
	'AM': '上午',
	'pm': '下午',
	'PM': '下午'
};

/* harmony default export */ const translation_zh = (translationZh);

;// ./node_modules/tr-grid-util/es6/locale/translation-zh-hant.js
/**
* @namespace
*/
let translationZhHant = {
	'sun': '週日',
	'mon': '週一',
	'tue': '週二',
	'wed': '週三',
	'thu': '週四',
	'fri': '週五',
	'sat': '週六',
	'Sunday': '星期日',
	'Monday': '星期一',
	'Tuesday': '星期二',
	'Wednesday': '星期三',
	'Thursday': '星期四',
	'Friday': '星期五',
	'Saturday': '星期六',

	'jan': '1月',
	'feb': '2月',
	'mar': '3月',
	'apr': '4月',
	'may': '5月',
	'jun': '6月',
	'jul': '7月',
	'aug': '8月',
	'sep': '9月',
	'oct': '10月',
	'nov': '11月',
	'dec': '12月',
	'January': '一月',
	'February': '二月',
	'March': '三月',
	'April': '四月',
	'May': '五月',
	'June': '六月',
	'July': '七月',
	'August': '八月',
	'September': '九月',
	'October': '十月',
	'November': '十一月',
	'December': '十二月',

	'am': '上午',
	'AM': '上午',
	'pm': '下午',
	'PM': '下午'
};

/* harmony default export */ const translation_zh_hant = (translationZhHant);

;// ./node_modules/tr-grid-util/es6/locale/translation.js






/**
* @namespace
*/
let translation = {
	"en": translation_en,
	"de": translation_de,
	"ja": translation_ja,
	"zh": translation_zh,
	"zh-Hant": translation_zh_hant
};

/* harmony default export */ const locale_translation = (translation);

;// ./node_modules/tr-grid-util/es6/Color.js
/** @namespace */
let Color = {};

/** @private
* @constant
* @type {RegExp}
*/
const NumRegExp = /\d+/g;

/** Convert CSS rgb or rgba formats to CSS hex color string (with # prefix)
* @public
* @param {string} rgbCode RGB values without # prefix
* @return {string} RGB in hex code (with # prefix)
* @example
* rgb2Hex("rgb(255, 255, 0)"); // "#FFFF00"
* rgb2Hex("rgba(255, 255, 0, 1)"); // "#FFFF00"
* rgb2Hex("255 255.0"); // "#FFFF00"
* rgb2Hex("#FFFF00"); // "#FFFF00"
* rgb2Hex("#1a1a1a"); // "#1a1a1a"
* rgb2Hex("2552550"); // "2552550"
* rgb2Hex("invalid"); // "invalid"
* rgb2Hex(null); // ""
*/
let rgb2Hex = function (rgbCode) {
	if(!rgbCode || typeof rgbCode !== "string") {
		return "";
	}
	if(rgbCode.charAt(0) === "#") {
		return rgbCode;
	}
	let rgb = rgbCode.match(NumRegExp);
	if(!rgb || rgb.length < 3) {
		return rgbCode;
	}

	let hex = "#";
	for(let i = 0; i < 3; i++) {
		let num = +rgb[i];
		if(!(num >= 16)) { // Handle NaN case
			hex += "0";
		}
		hex += (num) ? num.toString(16).toUpperCase() : "0";
	}
	return hex;
};

/** @public
* @function
* @param {Array.<number>} triplet
* @return {string} resultColor
*/
let num2Hex = function (triplet) {
	let rgb = triplet[2] | (triplet[1] << 8) | (triplet[0] << 16);
	return ("#" + (0x1000000 + rgb).toString(16).slice(1));
};
/** Note that Chrome, IE, and Firefox store color in rgb representation.
* @public
* @function
* @param {Array.<number>} triplet
* @return {string} Color string in RGB represetation (e.g. rgb(100, 44, 1))
*/
let num2Rgb = function (triplet) {
	return "rgb(" + triplet[0] + ", " + triplet[1] + ", " + triplet[2] + ")";
};
/** @public
* @function
* @param {string} hex
* @return {Array.<number>} Array of size 3 which contains [red, green, blue]
*/
let hex2Num = function (hex) {
	let hexInt = parseInt(hex.replace(/[^0-9A-F]/gi, ""), 16);
	let r = (hexInt >> 16) & 255;
	let g = (hexInt >> 8) & 255;
	let b = hexInt & 255;
	return [r, g, b];
};
/** @public
* @function
* @param {string} hex Color string with leading # character (e.g. #FFAA00)
* @return {string} Color string in RGB represetation (e.g. rgb(100, 44, 1))
*/
let hex2Rgb = function (hex) {
	if(hex) {
		let hexInt = parseInt(hex.replace(/[^0-9A-F]/gi, ""), 16);
		let r = (hexInt >> 16) & 255;
		let g = (hexInt >> 8) & 255;
		let b = hexInt & 255;
		return "rgb(" + r + ", " + g + ", " + b + ")";
	}
	return "";
};

/** @public
* @function
* @param {number} color A color component (e.g., R, G, or B) with value between 0 and 255 (inclusive)
* @return {number} Normalized luminance value between 0 and 1
*/
let getColorLuminance = function (color) {
	if(!color || color < 0) {
		return 0;
	}
	if(color >= 255) {
		return 1;
	}
	let normalizedColor = color / 255;
	if(normalizedColor <= 0.03928) {
		return normalizedColor / 12.92;
	}
	return Math.pow((normalizedColor + 0.055) / 1.055, 2.4);
};
/** The relative brightness of any point in a colorspace, normalized to 0 for darkest black and 1 for lightest white (https://www.w3.org/TR/WCAG20/#relativeluminancedef)
* @public
* @function
* @param {Array.<number>} triplet
* @return {number} Normalized value between 0 and 1
*/
let getRelativeLuminance = function (triplet) {
	let R = getColorLuminance(triplet[0]);
	let G = getColorLuminance(triplet[1]);
	let B = getColorLuminance(triplet[2]);

	return 0.2126 * R + 0.7152 * G + 0.0722 * B;
};
/** @public
* @function
* @param {number} lumin1
* @param {number} lumin2
* @return {number} Contrast ratios can range from 1 to 21 (commonly written 1:1 to 21:1).
*/
let getContrastRatio = function (lumin1, lumin2) {
	if(lumin1 === lumin2) {
		return 1;
	}
	let darker = lumin1;
	let lighter = lumin2;

	if(lumin1 > lumin2) {
		lighter = lumin1;
		darker = lumin2;
	}
	return (lighter + 0.05) / (darker + 0.05);
};
/** @public
* @function
* @param {Array.<number>} triplet
* @return {string} white or black color in hex code
*/
let getContrastColor = function (triplet) {
	let luminance = getRelativeLuminance(triplet);
	let contrastW = getContrastRatio(1, luminance);
	let contrastB = getContrastRatio(0, luminance);

	if (contrastB >= contrastW) { // Brighter color has more impact to human eye than the darker color
		return "#000000";
	}
	return "#ffffff";
};

/** Blend two colors into single color with the specified ratio
* @public
* @function
* @param {string} baseColor
* @param {string} maxColor
* @param {number} ratio [0, 1]
* @return {Array.<number>} resultColor
*/
let blendColor = function (baseColor, maxColor, ratio) { // This can be optimized further
	if (ratio > 1) {
		ratio = 1;
	} else if(ratio < 0) {
		ratio = 0;
	}

	let baseColorTriplet = hex2Num(baseColor);
	let maxColorTriplet = hex2Num(maxColor);
	let blendResult = [];
	for (let i = 0; i < 3; ++i) {
		let gap = (maxColorTriplet[i] - baseColorTriplet[i]) * ratio;
		blendResult.push(baseColorTriplet[i] + gap);
	}

	return blendResult;
};


Color.rgb2Hex = rgb2Hex;
Color.num2Hex = num2Hex;
Color.num2Rgb = num2Rgb;
Color.hex2Num = hex2Num;
Color.hex2Rgb = hex2Rgb;
Color.getColorLuminance = getColorLuminance;
Color.getRelativeLuminance = getRelativeLuminance;
Color.getContrastRatio = getContrastRatio;
Color.getContrastColor = getContrastColor;
Color.blendColor = blendColor;

/* harmony default export */ const es6_Color = ((/* unused pure expression or super */ null && (Color)));


;// ./node_modules/tr-grid-util/es6/Util.js


/** @namespace */
let Util = {};

/** This is a shorthand for fetch() API by POST method and with json body <br>
* WARNING: fetch is not supported in IE (including IE11)
* @public
* @see {@link https://developer.mozilla.org/en-US/docs/Web/API/Fetch_API/Using_Fetch}
* @function
* @param {string} url
* @param {Object|string} obj
* @return {!Promise<Response>}
*/
Util.post = function(url, obj) { // Not supported in IE
	return Util._post(url, obj).then(Util.toJSON);
};
/** Request a server response from server as text by POST method <br>
* WARNING: fetch is not supported in IE (including IE11)
* @public
* @see {@link https://developer.mozilla.org/en-US/docs/Web/API/Fetch_API/Using_Fetch}
* @function
* @param {string} url
* @param {Object|string} obj
* @return {!Promise<Response>}
*/
Util.requestText = function(url, obj) { // Not supported in IE
	return Util._post(url, obj, "text/plain").then(Util.toText);
};
/** Request a server response from server by POST method and with url-encoded body <br>
* WARNING: fetch is not supported in IE (including IE11)
* @public
* @see {@link https://developer.mozilla.org/en-US/docs/Web/API/Fetch_API/Using_Fetch}
* @function
* @param {string} url
* @param {string} obj
* @return {!Promise<Response>}
*/
Util.requestByUrlEncoded = function(url, obj) { // Not supported in IE
	return Util._post(url, obj, "application/x-www-form-urlencoded").then(Util.toJSON);
};

/** @public
* @function
* @param {Response} resp
* @return {!Promise<Object>}
*/
Util.toJSON = function (resp) {
	if(Util._logError(resp)) {
		return Promise.reject(resp);
	}
	return resp.json();
};
/** @public
* @function
* @param {Response} resp
* @return {!Promise<string>}
*/
Util.toText = function (resp) {
	if(Util._logError(resp)) {
		return Promise.reject(resp);
	}
	return resp.text();
};

/** @private
* @see {@link https://en.wikipedia.org/wiki/List_of_HTTP_status_codes}
* @function
* @param {Response} resp
* @return {boolean}
*/
Util._logError = function(resp) {
	if(resp && resp.status >= 300) {
		console.log("Response " + resp.status + ": " + resp.statusText);
		return true;
	}
	return false;
};
/** @private
* @function
* @param {string} url
* @param {Object|string} obj
* @param {string=} contentType
* @return {!Promise<Response>}
*/
Util._post = function(url, obj, contentType) {
	let options = {
		method: obj ? "POST" : "GET",
		headers: { "Content-Type": contentType || "application/json" }
	};
	if(obj) {
		options.body = typeof obj === "string" ? obj : JSON.stringify(obj);
	}

	return fetch(url, options);
};

/** WARNING: Only one level of property tree is affected (i.e. no recursive or nested operation). <br>
* Property with undefined or null value will not have an impact on the object. <br>
* Existing properties will be overridden.
* @public
* @function
* @param {Object} obj Object that is extended (new properties will be added to this object)
* @param {Object=} extender Master object (no modification will be made on the extender)
* @param {Array.<string>=} limiters Specify property to be extended
* @return {Object}
*/
let extendObject = function (obj, extender, limiters) {
	if(!obj) { // null undefined NaN empty string and 0
		return null;
	}
	if(!extender || obj === extender) {
		return obj;
	}

	let key;
	if(limiters) {
		let len = limiters.length;
		for(let i = 0; i < len; ++i) {
			key = limiters[i];
			if(key) {
				extendProperty(obj, extender, key);
			}
		}
	} else {
		for(key in extender) {
			extendProperty(obj, extender, key);
		}
	}
	return obj;
};
/** WARNING: Only one level of property tree is affected (i.e. no recursive or nested operation). <br>
* Property with undefined or null value will not be cloned.
* @public
* @function
* @param {Object} obj
* @param {Array.<string>=} limiters
* @return {Object}
*/
let cloneObject = function (obj, limiters) {
	return extendObject({}, obj, limiters);
};

/** Check empty object
* @public
* @function
* @param {Object} obj
* @return {boolean}=true, if the obj is empty
*/
let isEmptyObject = function (obj) {
	for (let key in obj) {
		return false;
	}
	return true;
};

/** @public
* @param {Array=} data
* @param {Array.<string>=} fields In case of the given data is an array, this param will be used for mapping index to field
* @return {Object|null}
*/
let arrayToObject = function(data, fields) {
	if(!Array.isArray(data)) {
		return data;
	} else if(!fields) {
		return null;
	}
	let ary = data;
	data = {};
	let len = ary.length;
	for(let i = 0; i < len; ++i) {
		let field = fields[i];
		// eslint-disable-next-line no-undefined
		if(field && ary[i] !== undefined) {
			data[field] = ary[i];
		}
	}
	return data;
};
/** Replace the specified property with the value from the extender. If the value is an array, the value will be added to the array instead of replacement
* @public
* @function
* @param {Object} obj
* @param {Object} extender
* @param {string} propName
* @example
* extendProperty({a: 0}, {a: 1}, "a"); // {a: 1}
* extendProperty({a: 0}, {b: 1}, "b"); // {a: 0, b: 1}
* extendProperty({a: [0]}, {a: 1}, "a"); // {a: [0, 1]}
* extendProperty({a: [0]}, {a: [1, 2]}, "a"); // {a: [0, 1, 2]}
*/
let extendProperty = function (obj, extender, propName) {
	let val = extender[propName];
	if(val != null) {
		let objVal = obj[propName];
		if(Array.isArray(objVal)) {
			obj[propName] = objVal.concat(val);
		} else if(Array.isArray(val) && objVal) {
			obj[propName] = val.concat(objVal); // TODO: Preserve the order
		} else {
			obj[propName] = val; // WARNING: Overriding existing data
		}
	}
};


/** Compare the values of obj1 and obj2. if they are the same object or have the same value, the result will be true. Otherwise, it will return false.
* @public
* @param {*} obj1
* @param {*} obj2
* @return {boolean}
*/
let deepEqual = function (obj1, obj2) {

	if(obj1 === obj2) {
		return true;
	}

	if (typeof obj1 !== typeof obj2) {
		return false;
	}

	if (typeof obj1 === "object") {

		// We need to split array and object for performance reason
		if(Array.isArray(obj1)) {
			if(obj1.length !== obj2.length) {
				return false;
			}

			for (let i = 0; i < obj1.length; i++) {
				if (!deepEqual(obj1[i], obj2[i])) { // The array may not be a match if the elements are not sorted, so it will not be considered equal if there is a mismatch.
					return false;
				}
			}

		} else {

			if (Object.keys(obj1).length !== Object.keys(obj2).length) {
				return false;
			}

			for (let key in obj1) {
				if (!deepEqual(obj1[key], obj2[key])) {
					return false;
				}
			}
		}


		return true;
	}

	return false;

};
/** Array.push() is much faster than Array.concat(). The downside is the parameters must be an array, whereas concat() has no such requirement. <br>
* This method excels in extending known array property in an object.
* @public
* @function
* @param {Object} obj
* @param {string} propName
* @param {*} ary
* @return {Array} Returns the result of the extended array
* @see {@link https://dev.to/uilicious/javascript-array-push-is-945x-faster-than-array-concat-1oki}
* @example
* let obj = {};
* extendArrayProperty(obj, "prop1", 1); // [1]
* extendArrayProperty(obj, "prop1", 2); // [1, 2]
* extendArrayProperty(obj, "prop1", [3, 4]); // [1, 2, 3, 4]
* obj.prop2 = 5;
* extendArrayProperty(obj, "prop2", 6); // [5, 6]
* extendArrayProperty(obj, "prop2", [7]); // [5, 6, 7]
* extendArrayProperty(obj, "prop2", null); // null
*/
let extendArrayProperty = function (obj, propName, ary) {
	let objAry = null;
	if(ary) {
		let objVal = obj[propName];
		if(objVal) {
			if(Array.isArray(objVal)) {
				objAry = objVal;
			} else {
				objAry = obj[propName] = [objVal];
			}
		} else {
			objAry = obj[propName] = [];
		}
		// objAry is guaranteed to be an array at this point

		if(Array.isArray(ary)) {
			if(ary.length) {
				if(ary.length > 1) {
					Array.prototype.push.apply(objAry, ary);
				} else {
					objAry.push(ary[0]);
				}
			}
		} else {
			objAry.push(ary);
		}
	}
	return objAry;
};

/** @private
* @param {string|Array.<string>} item
* @return {string}
*/
let _encloseBracket = function(item) {
	return Array.isArray(item) ? "{\n" + item.join("\n") + "\n}" : item;
};
/** @private
* @param {string} str
* @return {string}
*/
let _indentBracketContent = function(str){
	return str.replace(/\n+/g, "\n\t").replace(/\n\t}$/, "\n}");
};
/** @public
* @param {string|Array} css
* @return {string} prettified CSS string
* @example
* prettifyCss(["div", [
*   "color: red;",
*   "padding: 20px;"
* ],
* ".class", [
*   "margin: 0;"
* ]
* ]);
*/
let prettifyCss = function(css) {
	if(css) {
		let cssStr = "";
		if (Array.isArray(css)) {
			let ary = css.map(_encloseBracket);
			cssStr = ary.join("\n").replace(/{\s*{/g, "{").replace(/\s+{/g, " {");
		} else {
			cssStr = (typeof css === "string") ? css : css + "";
		}

		if(cssStr) {
			return cssStr.replace(/{[\w\W]*?}/g, _indentBracketContent);
		}
	}
	return "";
};
/** Get shadow root of the given element regardless of whether the element is in the DOM or not. The immediate shadow root parent is returned in case of nested shadow roots. Any other case return null.
* @public
* @param {Element} elem
* @return {DocumentFragment}
*/
let getShadowRoot = function(elem) {
	if(elem) {
		let rootNode;
		if(elem.shadowRoot) {
			rootNode = elem.shadowRoot;
		} else if(elem.getRootNode) {
			rootNode = elem.getRootNode();
		}
		if(rootNode) { // getRootNode does not supported in IE. It should be implemented by the polyfills
			if(rootNode !== document) { // element that is in the DOM and not in the shadow has document as its root node
				if(rootNode !== elem) { // element that is NOT in the DOM and not in the shadow has itself as its root node
					return rootNode;
				}
			}
		}
	}
	return null;
};
/** Attach style tag to the proper document (in case of element inside a shadow root)
* @public
* @param {string} cssStr Valid CSS string (e.g. "div {color: red;}")
* @param {Element=} targetContext Element that needs the CSS
* @return {Element} New style tag
*/
let injectCss = function(cssStr, targetContext) {
	if(!cssStr) {
		return null;
	}

	let styleTag = document.createElement("style");
	styleTag.textContent = "\n" + cssStr + "\n";

	let styleHost = getShadowRoot(targetContext);
	let isInShadow = true;
	if(!styleHost) {
		isInShadow = false;
		styleHost = document.head;
	}

	// Find a place to insert the style tag
	let beforeElem;
	if(isInShadow) {
		if(styleHost.children && styleHost.children.length) {
			beforeElem = styleHost.children[0];
		}
	}
	if(beforeElem) {
		styleHost.insertBefore(styleTag, beforeElem);
	} else {
		styleHost.appendChild(styleTag);
	}
	return styleTag;
};

/** return true if browser is IE or Edge
* @public
* @return {boolean}
*/
let isIE = function () {
	let ua = window.navigator.userAgent;
	return (ua.indexOf('MSIE ') > 0) || (ua.indexOf('Trident/') > 0) || (ua.indexOf('Edge/') > 0);
};

/** return true if device is mac
* @public
* @return {boolean}
*/
let isMac = function () {
	return /Mac/.test(navigator.platform);
};

/** Return true if device is mac
* @public
* @return {boolean}
*/
let isTouchDevice = function () {
	if ((navigator["maxTouchPoints"] && navigator["maxTouchPoints"] < 256) ||
		(navigator["msMaxTouchPoints"] && navigator["msMaxTouchPoints"] < 256)) {
		return true;
	}
	return false;
};

/** parse nested object in to array
* @public
* @param {Object} obj
* @param {Array=} ary
* @return {Array}
*/
let nestedObjectToArray = function (obj, ary) {
	if (!ary) {
		ary = [];
	}
	for (let key in obj) {
		let element = obj[key];
		if ('object' === typeof element) {
			nestedObjectToArray(element, ary);
		} else {
			ary.push(element);
		}
	}
	return ary;
};

/** transform data to tab seperated value
* @public
* @param {*} data
* @return {string}
*/
let prepareTSVContent = function (data) {
	if (data == null) {
		return "";
	}

	let content = (typeof data === 'string') ? data : data.toString();

	if (!content.length) { return ""; }

	// Replace any new line and tab
	if (content.indexOf("\n") >= 0) {
		content = content.replace(/[\r\n]/g, " ");
	}

	if (content.indexOf("\t") >= 0) {
		content = content.replace(/\t/g, " ");
	}

	// Trim front and back spaces
	if (content.charAt(0) === " " || content.charAt(content.length - 1) === " ") {
		content = content.trim();
	}

	return content;
};

/* harmony default export */ const es6_Util = ((/* unused pure expression or super */ null && (Util)));


;// ./node_modules/tr-grid-util/es6/Deferred.js
/** Deferred promise should be used when asynchronous method does not return promise object (e.g. typical AJAX calls, or native setTimeout()). In other words, deferred promise is used to allow any non-promise logics to be worked in promise pattern.
* @constructor
* @param {Function=} resolveHandler
* @param {Function=} rejectHandler
* @param {*=} ctx Context object that will be provided as the second parameter of the given handlers
* @example
* let def = new Deferred(handler);
* setTimeout(def.resolve, 2000);
* let def2 = new Deferred();
* setTimeout(function() {
*   def2.resolve();
* }, 4000);
*/
let Deferred = function(resolveHandler, rejectHandler, ctx) {
	this.resolve = this.resolve.bind(this);
	this.reject = this.reject.bind(this);

	if(typeof resolveHandler === "function") {
		this._resolveHandler = resolveHandler;
	}
	if(typeof rejectHandler === "function") {
		this._rejectHandler = rejectHandler;
	}
	if(ctx != null) {
		this._ctx = ctx;
	}
	this.promise = new Promise((function(resolve, reject) {
		this._resolve = resolve;
		this._reject = reject;
	}).bind(this));
};
/** @type {Promise<*>}
* @public
*/
Deferred.prototype.promise = null;
/** @type {Function}
* @private
*/
Deferred.prototype._resolve = null; // Always supplied from Promise object
/** @type {Function}
* @private
*/
Deferred.prototype._reject = null; // Always supplied from Promise object
/** @type {Function}
* @private
*/
Deferred.prototype._resolveHandler = null;
/** @type {Function}
* @private
*/
Deferred.prototype._rejectHandler = null;
/** Context object that will be provided as the second parameter of the given handlers
* @type {*}
* @private
*/
Deferred.prototype._ctx = null;
/** @type {boolean}
* @private
*/
Deferred.prototype._fulfilled = false;

/** @public
* @param {*=} result
*/
Deferred.prototype.resolve = function(result) {
	if(this._resolve) { // Promise can be resolved only once
		if(this._resolveHandler) {
			let handlerRes = this._resolveHandler(result, this._ctx || this);
			if(handlerRes != null) {
				result = handlerRes;
			}
		}
		this._resolve(result); // WARNING: Synchronous call. The standard promise behavior requires asynchronous response.
		this._finally();
	}
};
/** @public
* @param {*=} result
*/
Deferred.prototype.reject = function(result) {
	if(this._reject) { // Promise can be resolved only once
		if(this._rejectHandler) {
			let handlerRes = this._rejectHandler(result, this._ctx || this);
			if(handlerRes != null) {
				result = handlerRes;
			}
		}
		this._reject(result); // WARNING: Synchronous call
		this._finally();
	}
};
/** @public
* @return {boolean}
*/
Deferred.prototype.isFulfilled = function() {
	return this._fulfilled;
};

/** @private */
Deferred.prototype._finally = function() {
	// Release all references to avoid memory leak
	this._fulfilled = true;
	this._ctx = this._resolveHandler = this._rejectHandler = null;
	this._resolve = this._reject = null;
};

/* harmony default export */ const es6_Deferred = ((/* unused pure expression or super */ null && (Deferred)));


;// ./node_modules/tr-grid-util/es6/ElementObserver.js

/** @namespace
*/
let ElementObserver = {};

/** @private
* @param {Element} elem
* @param {MutationRecord} mutation
*/
let _onLanguageMutated = function (elem, mutation) {
	if (mutation.type == "attributes" && mutation.attributeName == "lang") {
		elem.lang = mutation.target.lang || "en";
	}
};

/** @private
* @param {Function} listener
* @param {string=} attributeName
* @param {MutationRecord} mutation
*/
let _onAttributeMutated = function (listener, attributeName, mutation) {
	if (mutation.type === "attributes") {
		if (attributeName != null) {
			if (mutation.attributeName === attributeName) {
				listener(mutation.target.getAttribute(attributeName));
			}
		} else {
			listener(mutation.target);
		}
	}
};

/** @private
* @return {string}
*/
ElementObserver._getNewId = function () {
	let id = "observer_" + (++ElementObserver._id);
	return id;
};

/** Observe any element
* @private
* @param {Element} elem
* @param {Function} listener
* @param {Object=} opt_option
*/
ElementObserver._addObserver = function(elem, listener, opt_option) {
	let option = opt_option;
	if (!option) {
		option = {};
	}
	option.attributes = true; //configure it to listen to attribute changes

	ElementObserver._observer.observe(elem, option);
	ElementObserver._addListener(elem, listener);
};

/** @private
* @param {Array.<MutationRecord>} mutations
*/
ElementObserver._onMutationCallback = function(mutations) {
	for (let i = 0; i < mutations.length; i++) {
		let mutation = mutations[i];
		let listeners = ElementObserver._listeners[mutation.target._observeId];
		for (let j = 0; j < listeners.length; j++) {
			listeners[j](mutation);
		}
	}
};

/** @private
* @param {Element} elem
* @param {Function} fn
*/
ElementObserver._addListener = function(elem, fn) {
	let listeners = ElementObserver._listeners;
	if (!listeners) {
		listeners = ElementObserver._listeners = {};
	}
	if (!elem._observeId) {
		let id = ElementObserver._getNewId();
		elem._observeId = id;
	}
	if (!listeners[elem._observeId]) { // Always check to support the using of separated ElfUtil module in testing page
		listeners[elem._observeId] = [];
	}

	let listener = listeners[elem._observeId];
	if (listener.indexOf(fn) < 0) {
		listener.push(fn);
	}
};

/** Add a listener to a html lang attribute
* @public
* @param {Element} element An element within the DOM tree to watch for changes
*/
ElementObserver.addLanguageListener = function(element) {
	if (!element) { return; }

	element.lang = document.documentElement.lang || "en";

	ElementObserver._addObserver(document.documentElement, _onLanguageMutated.bind(null, element));
};

/** Add a listener to a html attribute
* @public
* @param {Element} element An element within the DOM tree to watch for changes
* @param {Function} listener A function which will be called on each attribute change
* @param {string=} attributeName If not specified, listener will be called on every attribute change
*/
ElementObserver.addAttributeListener = function(element, listener, attributeName) {
	if (!element || !listener) { return; }
	ElementObserver._addObserver(element, _onAttributeMutated.bind(null, listener, attributeName));
};

/**
* @type {MutationObserver}
* @private
*/
ElementObserver._observer = new MutationObserver(ElementObserver._onMutationCallback);
/**
* @type {Obect}
* @private
*/
ElementObserver._listeners = {};
/**
* @type {number}
* @private
*/
ElementObserver._id = 0;

/* harmony default export */ const es6_ElementObserver = ((/* unused pure expression or super */ null && (ElementObserver)));


;// ./node_modules/tr-grid-util/es6/ElfUtil.js




/** Dialog supporting language (selection dialog, formater dialog and filter dialog)
* @type {!Object.<string, boolean>}
* @private
* @const
*/
let LANGUAGE_SUPPORTS = {
	"en": true,
	"de": true,
	"ja": true,
	"zh": true,
	"zh-Hant": true
};

/** @namespace */
let ElfUtil = {};
/** @type {Deferred}
* @private
*/
ElfUtil._deferred = null;
/** @type {Promise<Object>}
* @public
*/
ElfUtil.themeReady = null;
/** Default theme colors from Solar Charcoal + American color profile
* @type {!Object.<string, string>}
* @private
* @const
*/
ElfUtil._defaultColors = {
	"up": "#39C46E",
	"down": "#F5475B",
	"level": "#C2C2C2",
	"tickUp": "#39C46E",
	"tickDown": "#F5475B",
	"baseGrid": "#1A1A1C", // For CellPainter. In Solar Pearl, the value would be #FFFFFF
	"baseText": "#C2C2C2", // For CellPainter. In Solar Pearl, the value would be #505050
	"trackColor": "#FF9933" // For PercentBar.
};
/** @type {Object.<string, string>}
* @public
*/
ElfUtil.themeColors = ElfUtil._defaultColors;
/** @type {Object.<string, number>}
* @private
*/
ElfUtil._components = {};
/** @type {string}
* @private
*/
ElfUtil._themeName = "";
/** @type {number}
* @private
*/
ElfUtil._elfVersion = 0;
/** @type {Element}
* @private
*/
ElfUtil._dummyIcon = null;
/** @type {boolean}
* @private
*/
ElfUtil._iconLoaded = false;
/** @type {string|null}
* @private
*/
ElfUtil._profileName = null;
/** @type {Array}
* @private
*/
ElfUtil._callbacks = [];
/** @type {boolean}
* @private
*/
ElfUtil._observed = false;
/** @type {boolean}
* @private
*/
ElfUtil._pendingResolve = false;
/** @type {boolean}
* @private
*/
ElfUtil._themeLoaded = false;

/** @type {Object.<string, Object>}
* @private
*/
ElfUtil._icons = {
	"elf-theme-halo": {
		"rowGrouping": {
			"expander": "down" // also used in rt-grid
		},
		"sorting": {
			"ascending": "hollow-arrow-up",
			"descending": "hollow-arrow-down",
			"sortable": "sort-up-down"
		},
		"columnStack": {
			"stack": "map-layers",
			"expand": "right",
			"collapse": "left"
		}
	},
	"elf-theme-solar": {
		"columnStack": {
			"stack": "list",
			"expand": "right",
			"collapse": "left"
		}
	}
};
// Elf v5 was change theme name to halo-theme and solar-theme
ElfUtil._icons["halo-theme"] = ElfUtil._icons["elf-theme-halo"];
ElfUtil._icons["solar-theme"] = ElfUtil._icons["elf-theme-solar"];


/** Set ELF version for testing purpose
* @public
* @function
* @param {number} ver
*/
ElfUtil.setElfVersion = function (ver) {
	ElfUtil._elfVersion = ver;
};
/** Check ELF version
* @public
* @function
* @return {number}
*/
ElfUtil.getElfVersion = function () {
	if (!ElfUtil._elfVersion) {
		let val = 0;
		if (window.EFX_GRID) {
			let version = window.EFX_GRID.version;
			version = version.split(".");
			val = +version[0];
		} else if (window.elf) {
			if (window.elf.version) {
				let ver = window.elf.version.match(/\d/); // example version code: 4.0.1
				if (ver && ver[0]) {
					val = parseInt(ver[0], 10);
				}
			} else {
				if (window.elf.customElements) {
					val = 3;
				} else if (window.customElements) {
					val = 4;
				}
			}
		} else if (window.Polymer) {
			val = 1;
		}
		ElfUtil._elfVersion = val;
	}

	return ElfUtil._elfVersion;
};

/** Check if ELF component is available for usage. Returns version number of the available component.
* @public
* @function
* @param {string} compName
* @return {number}
* @example
* ElfUtil.hasComponent("ef-button"); // 1 or 3
* ElfUtil.hasComponent("unknown-button"); // 0
*/
ElfUtil.hasComponent = function (compName) {
	let val = ElfUtil._components[compName];
	if (val == null) {
		val = 0;
		let ver = ElfUtil.getElfVersion();
		if (compName) {
			let elem;
			if (ver === 1) {
				elem = document.createElement(compName);
				if (elem.is === compName) {
					val = ver;
				}
			} else if (ver > 1) {
				if (
					window.customElements && window.customElements.get(compName) // Elf v3 v4
					|| document.createElement(compName).constructor !== HTMLElement // In case doesn't polyfill elf
				) {
					val = ver;
				}
			}
			ElfUtil._components[compName] = val; // TODO: Add safeguard for retriving version before ELF
		}
	}
	return val;
};


/** Convert language code (ISO 639-1) and country code (ISO 3166-1 alpha-2) format to ISO 639-1
* @param {string} lang string of language code or country code
* @returns {string} language code after convert to ISO 639-1 (2 character) or empty string
* @example
* ElfUtil.toLangString("ja-JP"); // convert to "ja"
*/
ElfUtil.toLangString = function (lang) {
	if(!lang) return "";
	lang = lang.toLowerCase();
	if(lang === "zh-hant") {
		return "zh-Hant";
	}
	let mainLang = lang.replace(/-.*/, "");
	if(LANGUAGE_SUPPORTS[mainLang]){
		return mainLang;
	}
	return "";
};

/** Check if ELF theme is available for usage. Returns true if theme is available.
* @public
* @function
* @param {string} themeName
* @return {boolean}
* @example
* ElfUtil.hasTheme("elf-theme-solar"); // true or false
*/
ElfUtil.hasTheme = function (themeName) {
	return themeName === ElfUtil.getThemeName();
};

/** Check the current ELF theme. If not available, return empty string.
* @public
* @function
* @param {Element=} optElem Deprecated param: Function will always use <html> instead
* @return {string}
*/
ElfUtil.getThemeName = function (optElem) {
	if (!ElfUtil._themeName) {
		// get theme name directly from element <html> --> document.documentElement
		ElfUtil._themeName = ElfUtil.getCssVariable("--theme-name", document.documentElement);
	}

	return ElfUtil._themeName;
};

/** This method is using for unit test
* @public
* @ignore
* @function
* @param {string} themeName
*/
ElfUtil.setThemeName = function (themeName) {
	ElfUtil._themeName = themeName;
};

/** @public
* @function
* @return {boolean}
*/
ElfUtil.isThemeLoaded = function () {
	return ElfUtil._themeLoaded;
};

/** @public
* @function
* @return {Object}
*/
ElfUtil.getColors = function() {
	return ElfUtil.themeColors;
};

/** Check the current ELF theme is Halo theme
* @public
* @function
* @param {string=} themeName
* @return {boolean}
*/
ElfUtil.isHaloTheme = function (themeName) {
	if(themeName == null) {
		themeName = ElfUtil.getThemeName();
	}
	if(themeName) {
		return themeName.indexOf("halo") >= 0;
	}

	return false;
};

/** Inject appropriate icon list (based on theme and ELF version) to the given object.
* @public
* @function
* @param {Object} configObj Object to mutate
*/
ElfUtil.injectIcons = function (configObj) {
	if (typeof configObj !== "object") { return; }

	let theme = ElfUtil.getThemeName();
	configObj.themeName = theme;

	let ver = ElfUtil.getElfVersion();
	configObj.elfVersion = ver;

	let icons = ElfUtil._icons[theme];
	if (icons) {
		configObj.icons = icons;
	}
};

/** Get icons array list
* @public
* @function
* @returns {!Array.<string>} Always return a list of icons for the existing theme
*/
ElfUtil.getIconList = function () {
	let iconStruct = ElfUtil._icons[ElfUtil.getThemeName()];
	return iconStruct ? nestedObjectToArray(iconStruct) : [];
};

/** @private
* @function
*/
ElfUtil._onIconPreloaded = function() {
	if(ElfUtil._dummyIcon) {
		let pn = ElfUtil._dummyIcon.parentNode;
		if(pn) {
			pn.removeChild(ElfUtil._dummyIcon);
		}
		ElfUtil._dummyIcon = null;
	}
};
/** Prepare and get icons array list. This method ensures that preloading can be executed only once.
* @public
* @function
* @returns {Array.<string>} Return null if there is no icon to be preloaded. Otherwise, a list of icons is returned.
*/
ElfUtil.prepareIconPreloading = function() {
	if(!ElfUtil._iconLoaded) {
		ElfUtil._iconLoaded = true;
		let iconList = ElfUtil.getIconList();
		if(iconList.length) {
			setTimeout(ElfUtil._onIconPreloaded, 10);
			let dummyIcon = ElfUtil._dummyIcon = document.createElement("ef-icon");
			dummyIcon.style.transform = "scale(0)";
			document.body.appendChild(dummyIcon);
			return iconList;
		}
	}
	return null;
};

/** Gets any defined css variables by name/key
* @param {string} varName css variable names (with leading --)
* @param {Element=} optElem target element, fallback to document body
* @returns {string} CSS variable
* @example
* ElfUtil.getCssVariable("--valid-name");
* ElfUtil.getCssVariable("--valid-name", spanElement);
*/
ElfUtil.getCssVariable = function (varName, optElem) {
	let result = "";
	let elem = optElem || document.body;
	if (window && typeof varName === "string" && elem.nodeType === 1) {
		if (varName.match(/^--/)) {
			result = window.getComputedStyle(elem).getPropertyValue(varName);
			result = result.replace(/[\"\']/g, "");
		}
	}
	return result;
};
/** Gets multiple CSS variables from an object map for better performance
* @param {Object.<string, string>} obj Key can be anything, while value is the CSS variable name
* @param {Element=} optElem target element, fallback to document body
* @returns {Object} Returns the same given object
* @example
* ElfUtil.getCssVariables({
*   "name1": "--var1",
*   "name2": "--var2"
* }, spanElement); // Css varriable names will be replaced with their value
*/
ElfUtil.getCssVariables = function (obj, optElem) {
	let elem = optElem || document.body;
	if (window && elem.nodeType === 1) {
		let computedStyle = window.getComputedStyle(elem);
		for(let key in obj) {
			let varName = obj[key];
			if (varName && varName.match(/^--/)) {
				obj[key] = computedStyle.getPropertyValue(varName).replace(/\"/g, ""); // eslint-disable-line
			}
		}
	}
	return obj || null;
};

/** @public
* @param {Object} rtk
*/
ElfUtil.setRTK = function(rtk) {
	if(rtk != null) {
		ElfUtil._rtk = rtk;
	}
};

/** Get current profile name set on the root html element (document.documentElement)
* @public
* @return {string} Current profile name
*/
ElfUtil.getMovementColorProfile = function() {
	return document.documentElement.getAttribute("movement-color-profile");
};

/** @private
* @param {Function} cb
*/
ElfUtil._addThemeChangedCallback = function(cb) {
	let callbacks = ElfUtil._callbacks;
	if(callbacks.indexOf(cb) < 0) {
		callbacks.push(cb);
	}
};

/** @public
* @return {Promise<Object>} A promise of object of theme colors
*/
ElfUtil.reloadThemeColors = function() {
	if (ElfUtil._deferred && ElfUtil._deferred.isFulfilled()) {
		ElfUtil._profileName = "";
		ElfUtil._themeLoaded = false;
		ElfUtil.themeReady = null;
		return ElfUtil.getThemeColors();
	} else {
		return ElfUtil.themeReady;
	}
};

/** Gets current theme colors from the document and returns a promise. <br>
* WANRING: This method sets movement color profile to html tag automatically, if JET.Settings exists or RTK.Settings is available. <br>
* To re-request/reset theme colors, set ElfUtil.themeReady variable to null
* @public
* @param {Function=} themeChangedCb
* @return {Promise<Object>} A promise of object of theme colors
*/
ElfUtil.getThemeColors = function(themeChangedCb) {
	if(typeof themeChangedCb === "function") {
		ElfUtil._addThemeChangedCallback(themeChangedCb);
	}

	if(ElfUtil.themeReady) {
		return ElfUtil.themeReady;
	}

	let d = ElfUtil._deferred = new Deferred();
	ElfUtil.themeReady = d.promise;

	let profileName = ElfUtil.getMovementColorProfile();
	if(profileName) {
		if(profileName !== ElfUtil._profileName) {
			setTimeout(ElfUtil._profileNameRetrieved, 100); // TODO: Find a proper way to ensure that theme is ready
		} else {
			ElfUtil._deferred.resolve(ElfUtil.themeColors);
		}
	} else {
		let options = {
			providerName: "Configuration",
			settingName: "RDE_USER_CURRENT_TICK_COLOR"
		};

		let jet = window ? window.JET : null;
		ElfUtil._themeLoaded = false;
		if(ElfUtil._rtk && ElfUtil._rtk.Settings) {
			ElfUtil._rtk.Settings.getAsync(options)
				.then(ElfUtil._loadingProfileSuccess)
				.catch(ElfUtil._loadingProfileFailure);
		} else if(jet && jet.Settings) {
			try {
				jet.Settings.read(ElfUtil._loadingProfileSuccess, options);
			} catch (err) {
				ElfUtil._loadingProfileFailure();
			}
		} else {
			setTimeout(ElfUtil._profileNameRetrieved, 100); // TODO: Find a proper way to ensure that theme is ready
		}
	}

	return d.promise;
};

/** @private
* @param {string} profileName
*/
let movementColorProfileChanged = function(profileName) {
	if(profileName !== ElfUtil._profileName) {
		ElfUtil._retrieveThemeColors(profileName);

		if(ElfUtil._pendingResolve) {
			ElfUtil._pendingResolve = false;
			ElfUtil._deferred.resolve(ElfUtil.themeColors);
		}

		let callbacks = ElfUtil._callbacks;
		for (let i = 0; i < callbacks.length; i++) {
			callbacks[i]();
		}
	}
};

/** Get user's color profile from JET or RTK and set it to html tag (document.documentElement)
* @private
* @param {string} profileName Returned from JET.Settings or RTK.Settings
*/
ElfUtil._loadingProfileSuccess = function(profileName) {
	if(profileName) {
		document.documentElement.setAttribute("movement-color-profile", profileName.toLowerCase());
	}

	if(ElfUtil._observed) { // Let attribute listener does the job
		ElfUtil._pendingResolve = true;
	} else {
		ElfUtil._profileNameRetrieved(profileName);
	}
};

/** @private
*/
ElfUtil._loadingProfileFailure = function() {
	ElfUtil._retrieveThemeColors();
	if(!ElfUtil._observed) {
		ElfUtil._observed = true;
		ElementObserver.addAttributeListener(document.documentElement, movementColorProfileChanged, "movement-color-profile");
	}
	ElfUtil._deferred.reject("Failed to get movement color profile from settings.");
};

/** @private
*/
ElfUtil._profileNameRetrieved = function() {
	let profileName = ElfUtil.getMovementColorProfile();
	ElfUtil._retrieveThemeColors(profileName);
	if(!ElfUtil._observed) {
		ElfUtil._observed = true;
		ElementObserver.addAttributeListener(document.documentElement, movementColorProfileChanged, "movement-color-profile");
	}
	ElfUtil._deferred.resolve(ElfUtil.themeColors);
};

/** Get theme colors from document
* @public
* @ignore
* @param {string} profileName Movement color profile name
*/
ElfUtil._retrieveThemeColors = function(profileName) {
	ElfUtil._themeLoaded = true;
	ElfUtil._profileName = profileName;

	let colors = ElfUtil.themeColors = ElfUtil.getCssVariables({
		"primary": "--color-scheme-primary", // Usually used in headers, and selection
		"secondary": "--color-scheme-secondary",
		"tertiary": "--color-scheme-tertiary", // Usually black or white
		"complementary": "--color-scheme-complementary", // Usually black or white
		"up": "--color-scheme-positive",
		"down": "--color-scheme-negative",
		"level": "--color-scheme-neutral",
		"tickUp": "--color-scheme-tickup",
		"tickDown": "--color-scheme-tickdown"
	});
	if(!colors["up"]) {
		colors["up"] = ElfUtil._defaultColors["up"];
		colors["down"] = ElfUtil._defaultColors["down"];
		colors["level"] = ElfUtil._defaultColors["level"];
		colors["tickUp"] = ElfUtil._defaultColors["tickUp"];
		colors["tickDown"] = ElfUtil._defaultColors["tickDown"];
	}

	// To be consistent with Elf theme variables
	colors["positive"] = colors["up"];
	colors["negative"] = colors["down"];
	colors["neutral"] = colors["level"];

	// Retrieve table styles to be used in grid and extensions
	let tableElem = document.createElement("table");
	tableElem.style.visibility = "hidden";
	tableElem.style.position = "fixed";
	tableElem.style.left = "0"; // Activate fixed position
	tableElem.style.top = "0";
	tableElem.style.width = "0"; // Prevent creating scrollbar
	let trElem = tableElem.insertRow(-1);
	let tdElem = trElem.insertCell(-1);

	document.body.appendChild(tableElem);
	let trStyles = window.getComputedStyle(trElem);
	colors["tableBg"] = trStyles.backgroundColor; // Usually equivalent to --color-scheme-tertiary

	let tdStyles = window.getComputedStyle(tdElem);
	colors["tableText"] = tdStyles.color; // Usually equivalent to --color-scheme-neutral
	colors["tableBorder"] = tdStyles.borderColor;
	if(colors["tableText"] === colors["tableBorder"]) {
		// trElem.className = "pearl";
		tdElem.className = "tr-lg";
		tdStyles = window.getComputedStyle(tdElem);
		colors["tableBg"] = tdStyles.backgroundColor;
		colors["tableText"] = tdStyles.color;
		colors["tableBorder"] = tdStyles.borderColor; // In Solar Pearl, the value would be #DADDE0
		if(colors["tableText"] === colors["tableBorder"]) { // Last fallback for non ELF coloring
			colors["tableBg"] = ElfUtil._defaultColors["baseGrid"];
			colors["tableText"] = ElfUtil._defaultColors["baseText"];
			colors["tableBorder"] = "";
		}
	}
	// WARNING: Computed styles return rgb/rgba color format instead of hex format
	colors["tableBg"] = rgb2Hex(colors["tableBg"]);
	colors["tableText"] = rgb2Hex(colors["tableText"]);
	colors["tableBorder"] = rgb2Hex(colors["tableBorder"]);
	document.body.removeChild(tableElem);

	// For CellPainter backward compatability
	colors["baseGrid"] = colors["tableBg"];
	colors["baseText"] = colors["tableText"];
	colors["trackColor"] = colors["primary"] || ElfUtil._defaultColors["trackColor"];
};

/* harmony default export */ const es6_ElfUtil = (ElfUtil);


;// ./node_modules/tr-grid-util/es6/DateTime.js



/** @type {!Date}
* @public
* @constant
*/
const INVALID_DATE = new Date(NaN); // eslint-disable-line
/** JavaScript date starts at Jan 01 1970, while Excel date starts at Jan 01 1900 (before JavaScript date). This value is the difference in days between the two values.
* @type {number}
* @private
* @constant
*/
const _DIFF_DAYS = 25567 + 2; // Start 1 on 1 Jan 1900 + 1 for error on 29 Feb 1900 (Not existing date but existing in window)
/** Starting date (1 Jan 1900) of Excel date in milliseconds counting from JavaScript date
* @type {number}
* @private
* @constant
*/
const _1_JAN_1900 = Date.UTC(1900, 0, 1); // -2208988800000
/** @type {number}
* @private
* @constant
*/
const _1_MAR_1900 = Date.UTC(1900, 2, 1); // -2203891200000
/** @type {number}
* @public
* @constant
*/
const MILLISECONDS_IN_DAY = 24 * 60 * 60 * 1000; // 86400000
/** 86400 seconds in a day
* @type {number}
* @public
* @constant
*/
const SECONDS_IN_DAY = 24 * 60 * 60; // eslint-disable-line
/** 3600 seconds in a day
* @type {number}
* @public
* @constant
*/
const SECONDS_IN_HOUR = (/* unused pure expression or super */ null && (60 * 60)); // eslint-disable-line
/** @type {number}
* @public
* @constant
*/
const SECONDS_IN_MINUTE = 60; // eslint-disable-line

/** @type {Array.<string>}
* @public
* @constant
*/
const SHORT_DAYS = ["sun", "mon", "tue", "wed", "thu", "fri", "sat"];
/** @type {Array.<string>}
* @public
* @constant
*/
const FULL_DAYS = ["Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"];
/** @type {Array.<string>}
* @public
* @constant
*/
const SHORT_MONTHS = ["jan", "feb", "mar", "apr", "may", "jun", "jul", "aug", "sep", "oct", "nov", "dec"];
/** @type {Array.<string>}
* @public
* @constant
*/
const FULL_MONTHS = ["January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December"];

/** Add zero prefix to the number. WARNING: Does not support negative number
* @ignore
* @function
* @param {number} num
* @return {string}
*/
let prefixZero = function(num) {
	return (num > 9) ? "" + num : "0" + num;
};

/** Add double zero prefix to the number. WARNING: Does not support negative number
* @ignore
* @function
* @param {number} num
* @return {string}
*/
let prefixDoubleZero = function(num) {
	if (num > 99) {
		return "" + num;
	} else if (num > 9) {
		return "0" + num;
	} else {
		return "00" + num;
	}
};

/** @type {!Object.<string, Function>}
* @ignore
* @description Map of date pattern convertion
*/
let Parsers = {
	// Interday time
	"D": function(date, locale) { return "" + date.getDate(); },
	"DD": function(date, locale) { return prefixZero(date.getDate()); },
	"DDD": function(date, locale) { return locale[ SHORT_DAYS[date.getDay()] ]; },
	"D_": function(date, locale) { return locale[ FULL_DAYS[date.getDay()] ]; },
	"M": function(date, locale) { return "" + (date.getMonth() + 1); },
	"MM": function(date, locale) { return prefixZero(date.getMonth() + 1); },
	"MMM": function(date, locale) { return locale[ SHORT_MONTHS[date.getMonth()] ]; },
	"M_": function(date, locale) { return locale[ FULL_MONTHS[date.getMonth()] ]; },
	"YY": function(date, locale) { return ("" + date.getFullYear()).slice(-2); },
	"Y_": function(date, locale) { return "" + date.getFullYear(); },
	// Intraday time
	"H": function(date, locale) { return "" + date.getHours(); },
	"H_": function(date, locale) { return prefixZero(date.getHours()); },
	"h": function(date, locale) {
		let temp = date.getHours() % 12;
		return (temp === 0) ? "12" : "" + temp;
	},
	"h_": function(date, locale) {
		let temp = date.getHours() % 12;
		temp = (temp === 0) ? 12 : temp;
		return prefixZero(temp);
	},
	"m": function(date, locale) { return "" + date.getMinutes(); },
	"m_": function(date, locale) { return prefixZero(date.getMinutes()); },
	"s": function(date, locale) { return "" + date.getSeconds(); },
	"s_": function(date, locale) { return prefixZero(date.getSeconds()); },
	"S": function(date, locale) { return ~~(date.getMilliseconds() / 100); },
	"SS": function(date, locale) { return prefixZero(~~(date.getMilliseconds() / 10)); },
	"S_": function(date, locale) { return prefixDoubleZero(date.getMilliseconds()); },
	// AM/PM suffix
	"A": function(date, locale) { return (date.getHours() < 12) ? locale["AM"] : locale["PM"]; },
	"a": function(date, locale) { return (date.getHours() < 12) ? locale["am"] : locale["pm"]; }
};

/** @namespace
*/
let DateTime = {};

/**
* @type {string}
* @private
*/
DateTime._dateTimePattern = "M/D/YY h:mm A";
/**
* @type {string}
* @private
*/
DateTime._localeSetting = "en";
/**
* @type {Object.<string, string>}
* @private
*/
DateTime._activeLocaleMap = null;
/**
* @type {Date}
* @private
*/
DateTime._activeInput = null;


/** @public
* @function
* @description Set the default date time format in formula.
* @param {string=} format
*/
DateTime.setDateTimePattern = function(format) {
	DateTime._dateTimePattern = format || "M/D/YY h:mm A";
};
/** @public
* @function
* @description Set the locale language for localization.
* @param {string=} lang
*/
DateTime.setLocale = function(lang) {
	let mainLang = es6_ElfUtil.toLangString(lang);
	DateTime._localeSetting = locale_translation[mainLang] ? mainLang : "en";
};

/**
* @public
* @function
* @description The TEXT function converts an date & time number since 1 Jan, 1900 into a text in customizable format. Default is YYYY-MM-DD HH:mm:ss.
* @param  {number|string|Date} dateTimeNumber A date & time number since 1 Jan, 1900 or date & time text in format YYYY-MM-DDTHH:mm:ss
* @param  {string=} dateTimeFormat A customized format of the result. Default format is M/D/YY h:mm A.
* @param  {string=} timezone GMT (Greenwich Mean Time) or LOCAL (Local timezone). Default is GMT.
* @return {string}
* @example
* ---------------------------------------------------------------------
* | Group  | Character   | English Output                             |
* ---------------------------------------------------------------------
* | Year        | YY / yy     | 00 01 ... 16 17                       |
* |             | YYYY / yyyy | 2000 2001 ... 2016 2017               |
* | Month       | M           | 1 2 ... 11 12                         |
* |             | MM          | 01 02 ... 11 12                       |
* |             | MMM         | Jan Feb ... Nov Dec                   |
* |             | MMMM        | January February ... November December|
* | Day         | D / d       | 1 2 ... 30 31                         |
* |             | DD / dd     | 01 02 ... 30 31                       |
* |             | DDD / ddd   | Sun Mon ... Fri Sat                   |
* |             | DDDD / dddd | Sunday Monday ... Friday Saturday     |
* | Hour        | H           | 0 1 ... 22 23                         |
* |             | HH          | 00 01 ... 22 23                       |
* |             | h           | 1 2 ... 11 12                         |
* |             | hh          | 01 02 ... 11 12                       |
* | Minute      | m           | 0 1 ... 58 59                         |
* |             | mm          | 00 01 ... 58 59                       |
* | Second      | s           | 0 1 ... 58 59                         |
* |             | ss          | 00 01 ... 58 59                       |
* | Millisecond | S           | 0 1 ... 8 9                           |
* |             | SS          | 00 01 ... 98 99                       |
* |             | SSS         | 001 002 ... 998 999                   |
* | AM/PM       | A           | AM PM                                 |
* |             | a           | am pm                                 |
* ---------------------------------------------------------------------
*/
DateTime.format = function(dateTimeNumber, dateTimeFormat, timezone) {
	let date = null;
	let dtTxt = ""; // new Date() is on par with Date.parse in term of performance.
	let localTime = !!(timezone && timezone.toUpperCase() === "LOCAL");
	if(typeof dateTimeNumber === "number") {
		if(dateTimeNumber > 1e8) { // Maximum date is 100 millions and any number greater than that is assumed to be millisecond
			date = new Date(dateTimeNumber); // convert milisecond
		} else { // The given number is assumed to be in Excel format (i.e. integer represents date and fraction represents time)
			// This number is a product of other formulas (e.g. TODAY(), NOW()), which are all local time.
			date = DateTime.fromDateNumber(dateTimeNumber);
			localTime = true; // Assume that the given date is already in local time
		}
	} else if(dateTimeNumber instanceof Date) {
		// Monitor timestamp should pass the value as Date object
		// IDN data should pass the value as Date object. E.g. CF_DATE, CF_TIME
		date = new Date(dateTimeNumber); // Always clone the passing object as Date object is mutable
	} else {
		// Pass the value with string format, expected to be ISO8601 format (YYYY-MM-DDTHH:mm:ss)
		dtTxt = dateTimeNumber + "";
		let isISO8601 = dtTxt.match(/^\d{4}(-\d{2}(-\d{2}(T\d{2}\:\d{2}\:\d{2})?)?)?$/);
		dtTxt = dtTxt.replace(/\-/g, "\/"); // Convert to YYYY/MM/DD HH:mm:ss (common for all browsers)
		if(isISO8601) {
			dtTxt = dtTxt.replace(/T/g, " "); // Convert to YYYY/MM/DD HH:mm:ss (common for all browsers)
		}

		date = new Date(dtTxt);
	}

	if(DateTime.isInvalidDate(date)) { // Invalid date
		return dateTimeNumber || ""; // TODO: Check if we need to convert the original object to string
	}

	// Convert non-local time (GMT) to local time
	if(!localTime) {
		date.setTime(date.getTime() + date.getTimezoneOffset() * 60000);
	}

	return DateTime.formatDateObject(date, dateTimeFormat);
};
/** Format the Date object to specified format.
* @public
* @function
* @param  {Date} date Date object
* @param  {string=} dateTimeFormat Datetime format. Referred to TEXT function.
* @return {string} Return empty string, if the given date object is invalid
*/
DateTime.formatDateObject = function(date, dateTimeFormat) {
	if(DateTime.isInvalidDate(date)) { // Invalid date
		return "";
	}

	if(!dateTimeFormat) {
		dateTimeFormat = DateTime._dateTimePattern;
	}

	DateTime._activeInput = date;
	DateTime._activeLocaleMap = locale_translation[DateTime._localeSetting] || DateTimeLocale["en"];

	let result = dateTimeFormat.replace(/D+|M+|Y+|H+|S+|A+|d+|m+|y+|h+|s+|a+|\[[^\]]*\]+/g, DateTime._parseDatePattern);
	DateTime._activeInput = null;

	return result;
};

/** @type {!Object.<string, boolean>}
* @private
*/
DateTime._lowerCaseDatePatterns = {
	"d": true,
	"y": true
};
/**
* @private
* @function
* @param  {string} match
* @return {string}
*/
DateTime._parseDatePattern = function(match) {
	let fchar = (match) ? match.charAt(0) : "";
	if(fchar === "[") { // Static text
		return match.substring(1, match.length - 1);
	}

	let parser = Parsers[match];
	if(!parser) {
		let mod = match;
		if(DateTime._lowerCaseDatePatterns[fchar]) {
			mod = match.toUpperCase();
			fchar = fchar.toUpperCase();
			parser = Parsers[mod];
		}
		if(!parser) {
			mod = fchar + "_";
			parser = Parsers[mod];
		}
	}
	if(parser) {
		return parser(DateTime._activeInput, DateTime._activeLocaleMap);
	}
	return match;
};

/** Check if the given date obj is valid without any automatic data type conversion.
* @public
* @function
* @param  {Date} dateObj Date number since 1 Jan, 1900 (value is 1)
* @return {boolean}
*/
DateTime.isValidDate = function(dateObj) {
	if(dateObj) {
		if(dateObj.getTime) {
			let t = dateObj.getTime();
			return t === t;
		}
	}
	return false;
};
/** Check if the given date obj is invalid without any automatic data type conversion.
* @public
* @function
* @param  {Date} dateObj Date number since 1 Jan, 1900 (value is 1)
* @return {boolean}
*/
DateTime.isInvalidDate = function(dateObj) {
	if(dateObj) {
		if(dateObj.getTime) {
			let t = dateObj.getTime();
			return t !== t;
		}
	}
	return true;
};

/** Convert the date number (Excel date format) since 1 Jan, 1900 (value is 1) to Date object.
* @public
* @function
* @param  {number} dateNumber Date number since 1 Jan, 1900 (value is 1)
* @return {!Date}
* @see {@link DateTime#dateNumberToMs}
* @see {@link DateTime#timeNumberToMs}
*/
DateTime.fromDateNumber = function(dateNumber) {
	let datePart = DateTime.dateNumberToMs(dateNumber);
	if(datePart === datePart) {
		let timePart = DateTime.timeNumberToMs(dateNumber);
		return new Date(datePart + timePart);
	}
	return new Date(NaN);
};
/** Convert the date number (Excel date format) to milliseconds. WARNING: This discards any intraday time information (keep only date part).
* @public
* @function
* @param  {number} dateNumber Date number since 1 Jan, 1900 (value is 1)
* @return {number} Returns NaN, if the given number is invalid. Otherwise, returns positive number in milliseconds
* @see {@link DateTime#timeNumberToMs}
* @see {@link DateTime#fromDateNumber}
*/
DateTime.dateNumberToMs = function(dateNumber) {
	dateNumber = Math.floor(+dateNumber); // Remove the floating point to prevent calculation error
	if(!dateNumber || dateNumber <= 0 || dateNumber === 60) { // Invalid input // 60 is 29 Feb 1900 that is not existing
		return NaN;
	}

	let num = dateNumber - _DIFF_DAYS;
	if(dateNumber < 60) {
		num++; // +1 Before 1 Mar 1900 because there is not existing 29 Feb 1900 but existing in Window.
	}
	return num * MILLISECONDS_IN_DAY;
};
/** Convert the time number (Excel time format) with range between 0 and 1 to milliseconds. WARNING: This discards date and millisecond from the given number.
* @public
* @function
* @param  {number} dateNumber Date number since 1 Jan, 1900 (value is 1)
* @return {number}
* @see {@link DateTime#dateNumberToMs}
* @see {@link DateTime#fromDateNumber}
*/
DateTime.timeNumberToMs = function(dateNumber) {
	if(typeof dateNumber !== "number") {
		dateNumber = (+dateNumber);
	}
	let timeNumber = dateNumber - Math.floor(dateNumber);
	if(!timeNumber) { // NaN or 0
		return 0;
	}
	if(timeNumber < 0) {
		timeNumber = -timeNumber;
	}

	let secondFromMidnight = Math.round(timeNumber * SECONDS_IN_DAY);
	return secondFromMidnight * 1000;
};

/** Convert string or number to date object
* @public
* @param {string|number} dateInput
* @return {Object}
*/
DateTime.toDateObject = function(dateInput) {
	let date = null;
	if(dateInput instanceof Date) {
		date = dateInput;
	} else if(typeof dateInput === "number") {
		if(dateInput > 3e9 || dateInput <= 0) { // 3e9 milliseconds is Thu Feb 05 1970 00:20:00
			date = new Date(dateInput); // convert milisecond
		} else if(dateInput > 1e5) { // 1e5 days is Tue Oct 17 2243
			// The given number is assumed to be second
			date = new Date(dateInput * 1000);
		} else {
			// The given number is assumed to be in Excel format (i.e. integer represents date and fraction represents time)
			// Excel stores dates as the number of days since January 0, 1900.
			// This number is a product of other formulas (e.g. TODAY(), NOW()), which are all local time.
			date = DateTime.fromDateNumber(dateInput);
		}
	} else if(!dateInput) {
		date = new Date(NaN);
	} else {
		// Pass the value with string format, expected to be ISO8601 format (YYYY-MM-DDTHH:mm:ss)
		let dtTxt = dateInput + "";
		let isISO8601 = dtTxt.match(/^\d{4}(-\d{2}(-\d{2}(T\d{2}\:\d{2}\:\d{2})?)?)?$/);
		dtTxt = dtTxt.replace(/\-/g, "\/"); // Convert to YYYY/MM/DD HH:mm:ss (common for all browsers)
		if(isISO8601) {
			dtTxt = dtTxt.replace(/T/g, " "); // Convert to YYYY/MM/DD HH:mm:ss (common for all browsers)
		}

		date = new Date(dtTxt);
	}

	return date;
};

/** Convert the JavaScript date object to the date number since 1 Jan, 1900 (value is 1). WARNING: This discards any intraday time information (keep only date part).
* @public
* @param  {Date} dateObj JavaScript date object
* @return {number}
*/
DateTime.toDateNumber = function(dateObj) {
	return DateTime._millisecondToDateNumber(Date.UTC(
		dateObj.getUTCFullYear(),
		dateObj.getUTCMonth(),
		dateObj.getUTCDate()
	));
};
/** Convert the JavaScript date object to the time number in a range between 0 and 1. WARNING: This discards date and millisecond from the given date object.
* @public
* @param  {Date} dateObj JavaScript date object
* @return {number}
*/
DateTime.toTimeNumber = function(dateObj) {
	return Date.UTC(
		1970, 0, 1,
		dateObj.getUTCHours(),
		dateObj.getUTCMinutes(),
		dateObj.getUTCSeconds()
	) / MILLISECONDS_IN_DAY;
};
/** Convert the time in millisecond of JavaScript date object to date number since 1 Jan, 1900 (value is 1).
* @private
* @function
* @param  {number} millisecond Milliseconds of JavaScript date object (UTC)
* @return {number} Date number
*/
DateTime._millisecondToDateNumber = function(millisecond) {
	millisecond = +millisecond;
	if(millisecond !== millisecond || millisecond < _1_JAN_1900) { // Invalid input
		return NaN;
	} else {
		let calDate = (millisecond / MILLISECONDS_IN_DAY) + _DIFF_DAYS;
		if(millisecond < _1_MAR_1900) {
			calDate--; // -1 before 1 Mar 1900 because there is not existing 29 Feb 1900 but existing in Window.
		}
		return calDate;
	}
};

/** Returns YYYY-MM-DD (UTC)
* @public
* @function
* @param  {Date} dateObj
* @return {string}
*/
DateTime.toYYYYMMDD = function(dateObj) {
	let str = dateObj ? dateObj.toJSON() : "";
	if(str) {
		return str.slice(0, 10);
	}
	return "";
};


/* harmony default export */ const es6_DateTime = (DateTime);


;// ./node_modules/tr-grid-util/es6/formula/functions/Info.js

/**
 * @namespace Info
 * @description
 *    {lang=en} Valid/Invalid value checking functions
 *    {lang=ja} 有効/無効 の確認関数
 *    {lang=zh} 有效/无效值检查函数
 *    {lang=hant} 有效/無效值檢查函數
 *    {lang=ko} 유효/무효 값 확인 함수
 *    {lang=ru} Valid/Invalid value checking functions
 *    {lang=de} Prüffunktionen für gültige/ungültige Werte
 */
let Info = {};

/**
 * @function ISEMPTY
 * @memberof Info
 * @description
 *    {lang=en} The ISEMPTY function tests if an initial supplied expression (or value) contains a value.
 *    {lang=ja} ISEMPTY 関数は、最初に指定した式 (または値) に空白があるかどうかをテストします。
 *    {lang=zh} ISEMPTY 函数测试初始提供的表达式 (或值) 是否有空值。
 *    {lang=hant} ISEMPTY 函數測試初始提供的表達式 (或值) 是否有空值。
 *    {lang=ko} ISEMPTY 함수는 초기 입력한 식(또는 값)에 값이 들어 있는지 테스트합니다.
 *    {lang=ru} The ISEMPTY function tests if an initial supplied expression (or value) contains a value.
 *    {lang=de} Die ISEMPTY-Funktion prüft, ob ein ursprünglich gelieferter Ausdruck (oder Wert) einen Wert enthält.
 * @param  {*} value
 *    {lang=en} expression (or value)
 *    {lang=ja} 式 (または値)
 *    {lang=zh} 表达式 (或值)
 *    {lang=hant} 表達式 (或值)
 *    {lang=ko} 식(또는 값)
 *    {lang=ru} expression (or value)
 *    {lang=de} Ausdruck (oder Wert)
 * @return {number}
 * @example
 *    =ISEMPTY(COL3)
 */
Info["ISEMPTY"] = function(value) { // undefined, null, and empty string return 1. 0 and NaN return 0
	return (value === value && value !== 0 && !value) ? 1 : 0;
};
/**
 * @function ISERR
 * @memberof Info
 * @description
 *    {lang=en} The ISERR function tests if an initial supplied expression (or value) returns an Error, except #N/A
 *    {lang=ja} ISERR 関数は、最初に指定した式 (または値) が #N/A 以外のエラーを返すかどうかをテストします。
 *    {lang=zh} ISERR 函数测试初始提供的表达式 (或值) 是否返回任何错误，#N/A 错误除外
 *    {lang=hant} ISERR 函數測試初始提供的表達式 (或值) 是否返回任何錯誤，#N/A 錯誤除外
 *    {lang=ko} ISERR 함수는 초기 입력한 식(또는 값)이 #N/A를 제외한 오류를 반환하는지 테스트합니다.
 *    {lang=ru} The ISERR function tests if an initial supplied expression (or value) returns an Error, except #N/A
 *    {lang=de} Die ISERR-Funktion prüft, ob ein anfänglich gelieferter Ausdruck (oder Wert) einen Fehler ausgibt, außer #N/A
 * @param  {*} value
 *    {lang=en} expression (or value)
 *    {lang=ja} 式 (または値)
 *    {lang=zh} 表达式 (或值)
 *    {lang=hant} 表達式 (或值)
 *    {lang=ko} 식(또는 값)
 *    {lang=ru} expression (or value)
 *    {lang=de} Ausdruck (oder Wert)
 * @return {number}
 * @example
 *    =ISERR(COL3)
 */
Info["ISERR"] = function(value) { // undefined, null, and NaN return 1.  0 and empty string return 0
	return (value !== value || value == null) ? 1 : 0;
};
/**
 * @function ISNA
 * @memberof Info
 * @description
 *    {lang=en} The ISNA function tests if an initial supplied expression (or value) returns an #N/A Error, and if so, returns true; Otherwise the function returns false.
 *    {lang=ja} ISNA 関数は、最初に指定された式 (または値) が #N/A エラーを返すかどうかをテストします。#N/A エラーの場合、TRUE を返し、それ以外は FALSE を返します。
 *    {lang=zh} ISNA 函数测试初始提供的表达式 (或值) 是否返回 #N/A 错误，如果是，返回 true; 否则函数返回 false。
 *    {lang=hant} ISNA 函數測試初始提供的表達式 (或值) 是否返回 #N/A 錯誤，如果是，返回 true; 否則函數返回 false。
 *    {lang=ko} ISNA 함수는 초기 입력한 식(또는 값)이 #N/A 오류를 반환하는지 테스트하고 #N/A Error를 반환하면 TRUE를 반환하지 않으면 FALSE를 반환합니다.
 *    {lang=ru} The ISNA function tests if an initial supplied expression (or value) returns an #N/A Error, and if so, returns true; Otherwise the function returns false.
 *    {lang=de} Die ISNA-Funktion prüft, ob ein anfänglich gelieferter Ausdruck (oder Wert) einen #N/A-Fehler ausgibt, und wenn ja, wird „Wahr“ ausgegeben; andernfalls gibt die Funktion „Falsch“ aus.
 * @param  {*} value
 *    {lang=en} expression (or value)
 *    {lang=ja} 式 (または値)
 *    {lang=zh} 表达式 (或值)
 *    {lang=hant} 表達式 (或值)
 *    {lang=ko} 식(또는 값)
 *    {lang=ru} expression (or value)
 *    {lang=de} Ausdruck (oder Wert)
 * @return {number}
 * @example
 *    =ISNA(COL3)
 */
Info["ISNA"] = function(value) {
	return (value === "") ? 1 : 0;
};
/**
 * @function ISNUMBER
 * @memberof Info
 * @description
 *    {lang=en} The ISNUMBER function tests if a supplied value is a number. If so, the function returns TRUE; Otherwise it returns FALSE.
 *    {lang=ja} ISNUMBER 関数は、指定された値が数値かどうかをテストします。文字列の場合 TRUE を返し、それ以外は FALSE を返します。
 *    {lang=zh} ISNUMBER 函数测试提供的值是否是数字。如果是，函数返回 TRUE; 否则返回 FALSE。
 *    {lang=hant} ISNUMBER 函數測試提供的值是否是數字。如果是，函數返回 TRUE; 否則返回 FALSE。
 *    {lang=ko} ISNUMBER 함수는 입력한 값이 숫자인지 테스트합니다. 숫자라면 이 함수는 TRUE를, 숫자가 아니면 FALSE를 반환합니다.
 *    {lang=ru} The ISNUMBER function tests if a supplied value is a number. If so, the function returns TRUE; Otherwise it returns FALSE.
 *    {lang=de} Die ISNUMBER-Funktion prüft, ob ein gelieferter Wert eine Zahl ist. Wenn dies der Fall ist, gibt die Funktion WAHR aus; andernfalls gibt sie FALSCH aus.
 * @param  {*} value
 *    {lang=en} expression (or value)
 *    {lang=ja} 式 (または値)
 *    {lang=zh} 表达式 (或值)
 *    {lang=hant} 表達式 (或值)
 *    {lang=ko} 식(또는 값)
 *    {lang=ru} expression (or value)
 *    {lang=de} Ausdruck (oder Wert)
 * @return {number}
 * @example
 *    =ISNUMBER(CF_NETCHNG)
 */
Info["ISNUMBER"] = function(value) {
	return (typeof value === "number") ? 1 : 0;
};
/**
 * @function ISSTRING
 * @memberof Info
 * @description
 *    {lang=en} The ISSTRING function tests if a supplied value is a string. If so, the function returns TRUE; Otherwise it returns FALSE.
 *    {lang=ja} ISSTRING 関数は、指定された値が文字列かどうかをテストします。文字列の場合 TRUE を返し、それ以外は FALSE を返します。
 *    {lang=zh} ISSTRING 函数测试提供的值是否是字符串。如果是，函数返回 TRUE; 否则返回 FALSE。
 *    {lang=hant} ISSTRING 函數測試提供的值是否是字符串。如果是，函數返回 TRUE; 否則返回 FALSE。
 *    {lang=ko} ISSTRING 함수는 입력한 값이 문자열인지 테스트합니다. 문자열이라면 이 함수는 TRUE를, 문자가 아니면 FALSE를 반환합니다.
 *    {lang=ru} The ISSTRING function tests if a supplied value is a string. If so, the function returns TRUE; Otherwise it returns FALSE.
 *    {lang=de} Die ISSTRING-Funktion prüft, ob ein gelieferter Wert eine Zeichenfolge ist. Wenn dies der Fall ist, gibt die Funktion WAHR aus; andernfalls gibt sie FALSCH aus.
 * @param  {*} value
 *    {lang=en} expression (or value)
 *    {lang=ja} 式 (または値)
 *    {lang=zh} 表达式 (或值)
 *    {lang=hant} 表達式 (或值)
 *    {lang=ko} 식(또는 값)
 *    {lang=ru} expression (or value)
 *    {lang=de} Ausdruck (oder Wert)
 * @return {number}
 * @example
 *    =ISSTRING(CF_NAME)
 */
Info["ISSTRING"] = function(value) {
	return (typeof value === "string") ? 1 : 0;
};
/**
 * @function ISNASTRING
 * @memberof Info
 * @description
 *    {lang=en} The ISNASTRING  function tests if a supplied value is a none-empty string. The function returns TRUE for an empty string, Otherwise it returns FALSE
 *    			Zero, null, undefined, number, array with item and object with key will return 0
 *    {lang=ja} ISNASTRING 関数は、指定された値が空白でない文字列かどうかをテストします。
 *    			ゼロ, null, 未定義, 数値, 項目のある配列およびキー付オブジェクトは 0 を返します。
 *    {lang=zh} ISNASTRING  函数测试提供的值是否是非空字符串。该函数对空字符串返回 TRUE，否则返回 FALSE。
 *    			零、空、未定义、数字、数组与项目，以及对象与关键字返回 0。
 *    {lang=hant} ISNASTRING 函數測試提供的值是否是非空字符串。該函數對空字符串返回 TRUE，否則返回 FALSE。
 *    			零、空、未定義、數字、數組與項目，以及對象與關鍵字返回 0。
 *    {lang=ko} ISNASTRING 함수는 입력한 값이 비어 있지 않은 문자열인지 테스트합니다. 이 함수는 빈 문자열의 경우 TRUE를, 그렇지 않으면 FALSE를 반환합니다.
 *    {lang=ru} The ISNASTRING  function tests if a supplied value is a none-empty string. The function returns TRUE for an empty string, Otherwise it returns FALSE
 *    			Zero, null, undefined, number, array with item and object with key will return 0
 *    {lang=de} Die ISNASTRING-Funktion prüft, ob ein gelieferter Wert eine nicht leere Zeichenfolge ist. Die Funktion gibt für eine leere Zeichenkette WAHR aus, andernfalls gibt sie FALSCH aus.
 *    			Zero, null, undefined, number, array with item and object with key will return 0
 * @param  {*} value
 *    {lang=en} expression (or value)
 *    {lang=ja} 式 (または値)
 *    {lang=zh} 表达式 (或值)
 *    {lang=hant} 表達式 (或值)
 *    {lang=ko} 식(또는 값)
 *    {lang=ru} expression (or value)
 *    {lang=de} Ausdruck (oder Wert)
 * @return {number}
 * @example
 *    =ISNASTRING(CF_SOURCE)
 */
Info["ISNASTRING"] = function(value) {
	if(value) {
		if(value.length === 0) {
			return 1; // []
		}
		if(value.constructor === Object) {
			for(let key in value) {
				return 0; // {"a": 1}
			}
			return 1; // {}
		}
	} else if(value === "") {
		return 1; // ""
	}
	return  0; // "abc", [1, 2, 3], 0, 1, null, undefined, NaN
};

/* harmony default export */ const functions_Info = (Info);


;// ./node_modules/tr-grid-util/es6/formula/functions/Logic.js
/**
 * @namespace Logic
 * @description
 *    {lang=en} Functions that handle logical conditions of data
 *    {lang=ja} データの理論関数
 *    {lang=zh} 处理数据的逻辑条件的函数
 *    {lang=hant} 處理數據的邏輯條件的函數
 *    {lang=ko} 데이터의 논리적 조건을 처리하는 함수
 *    {lang=ru} Functions that handle logical conditions of data
 *    {lang=de} Funktionen, die logische Bedingungen von Daten verarbeiten
 */
let Logic = {};

/**
 * @function AND
 * @memberof Logic
 * @description
 *    {lang=en} The AND function returns 1 (TRUE), if all arguments in List are true. It returns 0 (FALSE) if one or more arguments in List are false. Use operand && for a shorthand.
 *    {lang=ja} AND 関数は、リストのすべての引数が TRUEの 場合、1 (TRUE) を返します。リストの1つ以上の引数が FALSEの 場合、0 (FALSE) を返します。省略形は、オペランド [ && ] を使用します。
 *    {lang=zh} 如果列表中的全部参数为真，则 AND 函数返回 1 (TRUE)。如果一个或多个列表中的参数为假，则返回 0 (FALSE)。 速记使用操作符 &&。
 *    {lang=hant} 如果列表中的全部參數為真，則 AND 函數返回 1 (TRUE)。如果一個或多個列表中的參數為假，則返回 0 (FALSE)。速記使用操作符 &&。
 *    {lang=ko} AND 함수는 목록의 모든 인수가 참이면 1(TRUE)을 반환합니다. 목록에 있는 하나 이상의 인수가 거짓이면 0(FALSE)을 반환합니다. 약칭으로 피연산자 &&를 사용합니다.
 *    {lang=ru} The AND function returns 1 (TRUE), if all arguments in List are true. It returns 0 (FALSE) if one or more arguments in List are false. Use operand && for a shorthand.
 *    {lang=de} Die UND-Funktion gibt 1 (WAHR) aus, wenn alle Argumente in der Liste wahr sind. Sie gibt 0 (FALSCH) aus, wenn eines oder mehrere Argumente in der Liste falsch sind. Verwenden Sie Operand && für eine Kurzschrift.
 * @return {number}
 * @example
 *    =AND(5>2, 1+1=2)
 *    =5>2 && 1+1=2
 */
Logic["AND"] = function(/* ... args*/) {
	let args = (arguments[0] instanceof Array) ? arguments[0] : arguments;
	for(let i = args.length; --i >= 0;) {
		if(!args[i]) {
			return 0;
		}
	}
	return 1;
};

/**
 * @function FALSE
 * @memberof Logic
 * @description
 *    {lang=en} The FALSE function returns the value 0 (FALSE).
 *    {lang=ja} FALSE 関数は、値 0 (FALSE) を返します。
 *    {lang=zh} FALSE 函数返回值 0 (FALSE)。
 *    {lang=hant} FALSE 函數返回值 0 (FALSE)。
 *    {lang=ko} FALSE 함수는 값 0(FALSE)을 반환합니다.
 *    {lang=ru} The FALSE function returns the value 0 (FALSE).
 *    {lang=de} Die FALSCH-Funktion gibt den Wert 0 (FALSCH) aus.
 * @return {number}
 * @example
 *    =FALSE()
 */
Logic["FALSE"] = function() {
	return 0;
};

/**
 * @function IF
 * @memberof Logic
 * @description
 *    {lang=en} The IF function returns Expression 1 if Cond is TRUE, (a numeric value  0). Otherwise, it returns Expression 2.
 *    {lang=ja} IF 関数は、指定された式が TRUE の場合、理論式 1を返します (数値 0)。 それ以外は、理論式 2を返します。
 *    {lang=zh} 如果条件为 TRUE，则 IF 函数返回表达式 1, (数值为  0)。否则，返回表达式 2。
 *    {lang=hant} 如果條件為 TRUE，則 IF 函數返回表達式 1, (數值為 0)。否則，返回表達式 2。
 *    {lang=ko} IF 함수는 Cond가 TRUE인 경우 Expression 1을 반환합니다(숫자 값 0). 그렇지 않으면 Expression 2를 반환합니다.
 *    {lang=ru} The IF function returns Expression 1 if Cond is TRUE, (a numeric value  0). Otherwise, it returns Expression 2.
 *    {lang=de} Die IF-Funktion gibt Ausdruck 1 aus, wenn die Bedingung WAHR ist (ein numerischer Wert 0). Andernfalls gibt sie Ausdruck 2 aus.
 * @param  {boolean|string} cond
 *    {lang=en} Boolean condition
 *    {lang=ja} ブール条件
 *    {lang=zh} 布尔条件
 *    {lang=hant} 布爾條件
 *    {lang=ko} 불리언 조건
 *    {lang=ru} Boolean condition
 *    {lang=de} Boolesche Bedingung
 * @param  {number|string} exp1
 *    {lang=en} Expression 1
 *    {lang=ja} 理論式 1
 *    {lang=zh} 表达式 1
 *    {lang=hant} 表達式 1
 *    {lang=ko} Expression 1
 *    {lang=ru} Expression 1
 *    {lang=de} Ausdruck 1
 * @param  {number|string} exp2
 *    {lang=en} Expression 2
 *    {lang=ja} 理論式 2
 *    {lang=zh} 表達式 2
 *    {lang=hant} Expression 2
 *    {lang=ko} Expression 2
 *    {lang=ru} Expression 2
 *    {lang=de} Ausdruck 2
 * @return {number|string}
 * @example
 *    =IF(CF_LAST > 0, "Positive", "Negative")
 */
Logic["IF"] = function(cond, exp1, exp2) {
	return cond ? exp1 : exp2;
};

/**
 * @function NOT
 * @memberof Logic
 * @description
 *    {lang=en} The NOT function negate the value of the condition. Use operand ! for a shorthand.
 *    {lang=ja} NOT 関数は、FALSE の場合は TRUEを、TRUE の場合は FALSE を返します。省略形は、オペランド [ !]を使用します。
 *    {lang=zh} NOT 函数否定条件值。速记使用操作符 !。
 *    {lang=hant} NOT 函數否定條件值。速記使用操作符 !。
 *    {lang=ko} NOT 함수는 조건의 값을 무효화합니다. 약칭으로 피연산자 !를 사용하십시오.
 *    {lang=ru} The NOT function negate the value of the condition. Use operand ! for a shorthand.
 *    {lang=de} Die NICHT-Funktion negiert den Wert der Bedingung. Verwenden Sie Operand ! für eine Kurzschrift.
 * @param  {number|string} a
 *    {lang=en} Cond
 *    {lang=ja} 条件
 *    {lang=zh} 条件
 *    {lang=hant} 條件
 *    {lang=ko} 조건
 *    {lang=ru} Cond
 *    {lang=de} Beding.
 * @return {number}
 * @example
 *    =NOT(CF_NETCHNG > 0)
 *    =!(CF_NETCHNG > 0)
 */
Logic["NOT"] = function(a) {
	return !a ? 1 : 0;
};

/**
 * @function OR
 * @memberof Logic
 * @description
 *    {lang=en} The OR function returns 1 (TRUE), if any argument in List is true. It returns 0 (FALSE), if all arguments in List are FALSE. Use operand || for a shorthand.
 *    {lang=ja} OR 関数は、リストのいずれかの引数が TRUE の場合、1 (TRUE) を返します。リストのすべての引数が FALSE の場合、0 (FALSE) を返します。省略形は、オペランド [ || ] を使用します。
 *    {lang=zh} 如果列表中的任意参数为真，则 OR 函数返回 1 (TRUE)。如果列表中的全部参数为假，则返回 0 (FALSE)。速记使用操作符 ||。
 *    {lang=hant} 如果列表中的任意參數為真，則 OR 函數返回 1 (TRUE)。如果列表中的全部參數為假，則返回 0 (FALSE)。速記使用操作符 ||。
 *    {lang=ko} OR 함수는 목록의 어떤 인수가 참이면 1(TRUE)을 반환합니다. 목록에 있는 모든 인수가 거짓이면 0(FALSE)을 반환합니다. 약칭으로 피연산자 ||를 사용합니다.
 *    {lang=ru} The OR function returns 1 (TRUE), if any argument in List is true. It returns 0 (FALSE), if all arguments in List are FALSE. Use operand || for a shorthand.
 *    {lang=de} Die ODER-Funktion gibt 1 (WAHR) aus, wenn eines der Argumente in der Liste wahr ist. Sie gibt 0 (FALSCH) aus, wenn alle Argumente in der Liste FALSCH sind. Verwenden Sie Operand || für eine Kurzschrift.
 * @return {number}
 * @example
 *    =OR(5>10, 4+2=8, 10<1)
 *    =5>10 || 4+2=8 || 10<1
 */
Logic["OR"] = function(/* ... args*/) {
	let args = (arguments[0] instanceof Array) ? arguments[0] : arguments;
	for(let i = args.length; --i >= 0;) {
		if(args[i]) {
			return 1;
		}
	}
	return 0;
};

/**
 * @function TRUE
 * @memberof Logic
 * @description
 *    {lang=en} The TRUE function returns the value 1 (TRUE).
 *    {lang=ja} TRUE 関数は、理論値 1 (TRUE) を返します。
 *    {lang=zh} TRUE 函数返回值 1 (TRUE)。
 *    {lang=hant} TRUE 函數返回值 1 (TRUE)。
 *    {lang=ko} TRUE 함수는 값 1(TRUE)을 반환합니다.
 *    {lang=ru} The TRUE function returns the value 1 (TRUE).
 *    {lang=de} Die WAHR-Funktion gibt den Wert 0 (WAHR) aus.
 * @return {number}
 * @example
 *    =TRUE()
 */
Logic["TRUE"] = function() {
	return 1;
};

/* harmony default export */ const functions_Logic = (Logic);


;// ./node_modules/tr-grid-util/es6/formula/functions/Math.js
/**
 * @namespace GridMath
 * @description
 * {lang=en} Mathmatic functions
 * {lang=ja} 数学関数
 * {lang=zh} 数学函数
 * {lang=hant} 數學函數
 * {lang=ko} 수학 함수
 * {lang=ru} Mathmatic functions
 * {lang=de} Mathematische Funktionen
 */
let GridMath = {};

/**
 * @function ABS
 * @memberof GridMath
 * @description
 *    {lang=en} Returns the absolute value of a number
 *    {lang=ja} 数値の絶対値を返します
 *    {lang=zh} 返回一个数字的绝对值
 *    {lang=hant} 返回一個數字的絕對值
 *    {lang=ko} 숫자의 절댓값을 반환
 *    {lang=ru} Returns the absolute value of a number
 *    {lang=de} Gibt den absoluten Wert einer Zahl aus
 * @param  {number} x
 *    {lang=en} Number
 *    {lang=ja} 数値
 *    {lang=zh} 数字
 *    {lang=hant} 數字
 *    {lang=ko} 숫자
 *    {lang=ru} Number
 *    {lang=de} Zahl
 * @return {number}
 * @example
 *    =ABS(-0.005)
 */
GridMath["ABS"] = Math.abs;

/**
 * @function ACOS
 * @memberof GridMath
 * @description
 *    {lang=en} Returns the arccosine of a number
 *    {lang=ja} 数値のアークコサインを返します
 *    {lang=zh} 返回一个数字的反余弦
 *    {lang=hant} 返回一個數字的反餘弦
 *    {lang=ko} 숫자의 아크코사인을 반환
 *    {lang=ru} Returns the arccosine of a number
 *    {lang=de} Gibt den Arkussinus einer Zahl aus
 * @param  {number} x
 *    {lang=en} Number
 *    {lang=ja} 数値
 *    {lang=zh} 数字
 *    {lang=hant} 數字
 *    {lang=ko} 숫자
 *    {lang=ru} Number
 *    {lang=de} Zahl
 * @return {number}
 * @example
 *    =ACOS(0.5)
 */
GridMath["ACOS"] = Math.acos;

/**
 * @function ACOSH
 * @memberof GridMath
 * @description
 *    {lang=en} Returns the inverse hyperbolic cosine of a number
 *    {lang=ja} 数値の双曲線逆余弦を返します
 *    {lang=zh} 返回一个数字的反双曲余弦
 *    {lang=hant} 返回一個數字的反雙曲餘弦
 *    {lang=ko} 숫자의 역쌍곡코사인을 반환
 *    {lang=ru} Returns the inverse hyperbolic cosine of a number
 *    {lang=de} Gibt den umgekehrten hyperbolischen Kosinus einer Zahl aus
 * @param  {number} x
 *    {lang=en} Number
 *    {lang=ja} 数値
 *    {lang=zh} 数字
 *    {lang=hant} 數字
 *    {lang=ko} 숫자
 *    {lang=ru} Number
 *    {lang=de} Zahl
 * @return {number}
 * @example
 *    =ACOSH(0.5)
 */
GridMath["ACOSH"] = function(x) {
	return Math.log(x + Math.sqrt(x * x - 1));
};

/**
 * @function ASIN
 * @memberof GridMath
 * @description
 *    {lang=en} Returns the arcsine of a numberr
 *    {lang=ja} 数値のアークサインを返します
 *    {lang=zh} 返回一个数字的反正弦
 *    {lang=hant} 返回一個數字的反正弦
 *    {lang=ko} 숫자의 아크사인을 반환
 *    {lang=ru} Returns the arcsine of a numberr
 *    {lang=de} Gibt den Arkussinus einer Zahl aus
 * @param  {number} x
 *    {lang=en} Number
 *    {lang=ja} 数値
 *    {lang=zh} 数字
 *    {lang=hant} 數字
 *    {lang=ko} 숫자
 *    {lang=ru} Number
 *    {lang=de} Zahl
 * @return {number}
 * @example
 *    =ASIN(0.5)
 */
GridMath["ASIN"] = Math.asin;

/**
 * @function ASINH
 * @memberof GridMath
 * @description
 *    {lang=en} Returns the inverse hyperbolic sine of a number
 *    {lang=ja} 数値の双曲線逆正弦を返します
 *    {lang=zh} 返回一个数字的反双曲正弦
 *    {lang=hant} 返回一個數字的反雙曲正弦
 *    {lang=ko} 숫자의 역쌍곡사인을 반환
 *    {lang=ru} Returns the inverse hyperbolic sine of a number
 *    {lang=de} Gibt den umgekehrten hyperbolischen Sinus einer Zahl aus
 * @param  {number} x
 *    {lang=en} Number
 *    {lang=ja} 数値
 *    {lang=zh} 数字
 *    {lang=hant} 數字
 *    {lang=ko} 숫자
 *    {lang=ru} Number
 *    {lang=de} Zahl
 * @return {number}
 * @example
 *    =ASINH(0.5)
 */
GridMath["ASINH"] = function(x) {
	if (x === -Infinity) {
		return x;
	} else {
		return Math.log(x + Math.sqrt(x * x + 1));
	}
};

/**
 * @function ATAN
 * @memberof GridMath
 * @description
 *    {lang=en} Returns the arctangent of a number
 *    {lang=ja} 数値のアークタンジェントを返します
 *    {lang=zh} 返回一个数字的反正切
 *    {lang=hant} 返回一個數字的反正切
 *    {lang=ko} 숫자의 아크탄젠트를 반환
 *    {lang=ru} Returns the arctangent of a number
 *    {lang=de} Gibt den Arkustangens einer Zahl aus
 * @param  {number} x
 *    {lang=en} Number
 *    {lang=ja} 数値
 *    {lang=zh} 数字
 *    {lang=hant} 數字
 *    {lang=ko} 숫자
 *    {lang=ru} Number
 *    {lang=de} Zahl
 * @return {number}
 * @example
 *    =ATAN(-1)
 */
GridMath["ATAN"] = Math.atan;

/**
 * @function ATAN2
 * @memberof GridMath
 * @description
 *    {lang=en} Returns the arctangent from x- and y-coordinates
 *    {lang=ja} 指定された x-y 座標のアークタンジェントを返します
 *    {lang=zh} 返回 x- 和 y-坐标的反正切
 *    {lang=hant} 返回 x- 和 y-坐標的反正切
 *    {lang=ko} x좌표와 y좌표에서 아크탄젠트를 반환
 *    {lang=ru} Returns the arctangent from x- and y-coordinates
 *    {lang=de} Gibt den Arkustangens aus x- und y-Koordinaten aus
 * @param  {number} x
 *    {lang=en} Number
 *    {lang=ja} 数値
 *    {lang=zh} 数字
 *    {lang=hant} 數字
 *    {lang=ko} 숫자
 *    {lang=ru} Number
 *    {lang=de} Zahl
 * @param  {number} y
 *    {lang=en} Number
 *    {lang=ja} 数値
 *    {lang=zh} 数字
 *    {lang=hant} 數字
 *    {lang=ko} 숫자
 *    {lang=ru} Number
 *    {lang=de} Zahl
 * @return {number}
 * @example
 *    =ATAN2(2,-1)
 */
GridMath["ATAN2"] = function(x, y) {
	return Math.atan2(y, x);
};

/**
 * @function ATANH
 * @memberof GridMath
 * @description
 *    {lang=en} Returns the inverse hyperbolic tangent of a number
 *    {lang=ja} 数値の双曲線逆正接を返します
 *    {lang=zh} 返回一个数字的反双曲正切
 *    {lang=hant} 返回一個數字的反雙曲正切
 *    {lang=ko} 숫자의 역쌍곡탄젠트를 반환
 *    {lang=ru} Returns the inverse hyperbolic tangent of a number
 *    {lang=de} Gibt den umgekehrten hyperbolischen Tangens einer Zahl aus
 * @param  {number} x
 *    {lang=en} Number
 *    {lang=ja} 数値
 *    {lang=zh} 数字
 *    {lang=hant} 數字
 *    {lang=ko} 숫자
 *    {lang=ru} Number
 *    {lang=de} Zahl
 * @return {number}
 * @example
 *    =ATANH(-1)
 */
GridMath["ATANH"] = function(x) {
	return Math.log((1 + x) / (1 - x)) / 2;
};

/**
 * @function CEILING
 * @memberof GridMath
 * @description
 *    {lang=en} Rounds a number to the nearest integer or to the nearest multiple of significance
 *    {lang=ja} 指定された基準値の倍数のうち、最も近い値に数値を切り上げます
 *    {lang=zh} 将一个数字四舍五入到最近的整数或最近的指定基数的倍数
 *    {lang=hant} 將一個數字四捨五入到最近的整數或最近的指定基數的倍數
 *    {lang=ko} 숫자를 가장 가까운 정수 또는 가장 가까운 significance의 배수로 반올림
 *    {lang=ru} Rounds a number to the nearest integer or to the nearest multiple of significance
 *    {lang=de} Rundet eine Zahl auf die nächste Ganzzahl oder auf das nächste Vielfache der Signifikanz
 * @param  {number} num
 *    {lang=en} Number
 *    {lang=ja} 数値
 *    {lang=zh} 数字
 *    {lang=hant} 數字
 *    {lang=ko} 숫자
 *    {lang=ru} Number
 *    {lang=de} Zahl
 * @param  {number} multiple
 *    {lang=en} The multiple to use when rounding
 *    {lang=ja} 基準値の倍数
 *    {lang=zh} 四舍五入时使用的乘数
 *    {lang=hant} 四捨五入時使用的乘數
 *    {lang=ko} 반올림할 때 사용할 배수
 *    {lang=ru} The multiple to use when rounding
 *    {lang=de} Das beim Runden zu verwendende Vielfache
 * @return {number}
 * @example
 *    =CEILING(4.88291516)
 */
GridMath["CEILING"] = function(num, multiple) { //
	if(!multiple) {
		multiple = 1;
	}
	num /= multiple;
	return Math.ceil(num) * multiple;
};

/**
 * @function CEIL
 * @memberof GridMath
 * @description
 *    {lang=en} Rounds a number to the nearest integer or to the nearest multiple of significance
 *    {lang=ja} 指定された基準値の倍数のうち、最も近い値に数値を切り上げます
 *    {lang=zh} 将一个数字四舍五入到最近的整数或最近的指定基数的倍数
 *    {lang=hant} 將一個數字四捨五入到最近的整數或最近的指定基數的倍數
 *    {lang=ko} 숫자를 가장 가까운 정수 또는 가장 가까운 significance의 배수로 반올림
 *    {lang=ru} Rounds a number to the nearest integer or to the nearest multiple of significance
 *    {lang=de} Rundet eine Zahl auf die nächste Ganzzahl oder auf das nächste Vielfache der Signifikanz
 * @param  {number} num
 *    {lang=en} Number
 *    {lang=ja} 数値
 *    {lang=zh} 数字
 *    {lang=hant} 數字
 *    {lang=ko} 숫자
 *    {lang=ru} Number
 *    {lang=de} Zahl
 * @param  {number} multiple
 *    {lang=en} The multiple to use when rounding
 *    {lang=ja} 基準値の倍数
 *    {lang=zh} 四舍五入时使用的乘数
 *    {lang=hant} 四捨五入時使用的乘數
 *    {lang=ko} 반올림할 때 사용할 배수
 *    {lang=ru} The multiple to use when rounding
 *    {lang=de} Das beim Runden zu verwendende Vielfache
 * @return {number}
 * @example
 *    =CEIL(4.88291516)
 */
GridMath["CEIL"] = GridMath["CEILING"];

/**
 * @function COMBIN
 * @memberof GridMath
 * @description
 *    {lang=en} Returns the number of combinations for a given number of objects
 *    {lang=ja} すべての項目から指定された個数を選択するときの組み合わせ数を返します
 *    {lang=zh} 返回给定数目的对象的组合数
 *    {lang=hant} 返回給定數目的對象的組合數
 *    {lang=ko} 주어진 개체 수에 대한 조합 수를 반환
 *    {lang=ru} Returns the number of combinations for a given number of objects
 *    {lang=de} Gibt die Anzahl der Kombinationen für eine gegebene Anzahl von Objekten aus
 * @param  {number} num
 *    {lang=en} Number
 *    {lang=ja} 数値
 *    {lang=zh} 数字
 *    {lang=hant} 數字
 *    {lang=ko} 숫자
 *    {lang=ru} Number
 *    {lang=de} Zahl
 * @param  {number} num_chosen
 *    {lang=en} Number
 *    {lang=ja} 数値
 *    {lang=zh} 数字
 *    {lang=hant} 數字
 *    {lang=ko} 숫자
 *    {lang=ru} Number
 *    {lang=de} Zahl
 * @return {number}
 * @example
 *    =COMBIN(10,3)
 */
GridMath["COMBIN"] = function(num, num_chosen) {
	let fact = GridMath["FACT"];
	return fact(num) / (fact(num_chosen) * fact(num - num_chosen));
};

/**
 * @function COS
 * @memberof GridMath
 * @description
 *    {lang=en} Returns the cosine of a number
 *    {lang=ja} 数値のコサインを返します
 *    {lang=zh} 返回一个数字的余弦
 *    {lang=hant} 返回一個數字的餘弦
 *    {lang=ko} 숫자의 코사인을 반환
 *    {lang=ru} Returns the cosine of a number
 *    {lang=de} Gibt den Kosinus einer Zahl aus
 * @param  {number} x
 *    {lang=en} Number
 *    {lang=ja} 数値
 *    {lang=zh} 数字
 *    {lang=xh-Hant} 數字
 *    {lang=ko} 숫자
 *    {lang=ru} Number
 *    {lang=de} Zahl
 * @return {number}
 * @example
 *    =COS(PI()/3)
 */
GridMath["COS"] = Math.cos; //

/**
 * @function COSH
 * @memberof GridMath
 * @description
 *    {lang=en} Returns the hyperbolic cosine of a number
 *    {lang=ja} 数値の双曲線余弦を返します
 *    {lang=zh} 返回一个数字的双曲余弦
 *    {lang=hant} 返回一個數字的雙曲餘弦
 *    {lang=ko} 숫자의 쌍곡코사인을 반환
 *    {lang=ru} Returns the hyperbolic cosine of a number
 *    {lang=de} Gibt den hyperbolischen Kosinus einer Zahl aus
 * @param  {number} x
 *    {lang=en} Number
 *    {lang=ja} 数値
 *    {lang=zh} 数字
 *    {lang=hant} 數字
 *    {lang=ko} 숫자
 *    {lang=ru} Number
 *    {lang=de} Zahl
 * @return {number}
 * @example
 *    =COSH(3)
 */
GridMath["COSH"] = function(x) {
	let y = Math.exp(x);
	return (y + 1 / y) / 2;
};

/**
 * @function DEGREES
 * @memberof GridMath
 * @description
 *    {lang=en} Converts radians to degrees
 *    {lang=ja} ラジアンで表された角度を度に変換します
 *    {lang=zh} 将弧度转换成角度
 *    {lang=hant} 將弧度轉換成角度
 *    {lang=ko} 라디안을 도로 변환
 *    {lang=ru} Converts radians to degrees
 *    {lang=de} Wandelt Radianten in Grad um
 * @param  {number} radians
 *    {lang=en} Number
 *    {lang=ja} 数値
 *    {lang=zh} 数字
 *    {lang=hant} 數字
 *    {lang=ko} 숫자
 *    {lang=ru} Number
 *    {lang=de} Zahl
 * @return {number}
 * @example
 *    =DEGREES(1)
 */
GridMath["DEGREES"] = function(radians) {
	return radians / (Math.PI / 180);
};

/**
 * @function EVEN
 * @memberof GridMath
 * @description
 *    {lang=en} Rounds a number up to the nearest even integer
 *    {lang=ja} 数値を切り上げ、最も近い偶数にします
 *    {lang=zh} 将一个数字向上舍入到最近的偶数
 *    {lang=hant} 將一個數字向上舍入到最近的偶數
 *    {lang=ko} 가장 가까운 짝수인 정수로 숫자를 올림
 *    {lang=ru} Rounds a number up to the nearest even integer
 *    {lang=de} Rundet eine Zahl auf die nächste gerade Ganzzahl auf
 * @param  {number} num
 *    {lang=en} Number
 *    {lang=ja} 数値
 *    {lang=zh} 数字
 *    {lang=hant} 數字
 *    {lang=ko} 숫자
 *    {lang=ru} Number
 *    {lang=de} Zahl
 * @return {number}
 * @example
 *    =EVEN(3.152)
 */
GridMath["EVEN"] = function(num) {
	num = Math.ceil(num);
	if(num % 2) {
		return (num < 0) ? num - 1 : num + 1;
	}
	return num;
};

/**
 * @function EXP
 * @memberof GridMath
 * @description
 *    {lang=en} Returns e^x, where x is the argument, and e is Euler's number (also known as Napier's constant), the base of the natural logarithms.
 *    {lang=ja} 指定された e (オイラー数又はナピア constant) を底とする数値のべき乗 e^x を返します (x は引数)。
 *    {lang=zh} 返回 e 的 x 次方, 其中 x 是参数，e 是欧拉指数 (也叫做 Napier 常数), 是自然对数的的底。
 *    {lang=hant} 返回 e 的 x 次方, 其中 x 是參數，e 是歐拉指數 (也叫做 Napier 常數), 是自然對數的的底。
 *    {lang=ko} e^x를 반환합니다. 여기서 x는 인수이고 e는 자연로그의 밑인 오일러의 수(네이피어 상수라고도 함)입니다.
 *    {lang=ru} Returns e^x, where x is the argument, and e is Euler's number (also known as Napier's constant), the base of the natural logarithms.
 *    {lang=de} Gibt e^x aus, wobei x das Argument und e die Euler-Zahl (auch als Napier-Konstante bekannt), die Basis der natürlichen Logarithmen, ist.
 * @param  {number} x
 *    {lang=en} Number
 *    {lang=ja} 数値
 *    {lang=zh} 数字
 *    {lang=hant} 數字
 *    {lang=ko} 숫자
 *    {lang=ru} Number
 *    {lang=de} Zahl
 * @return {number}
 * @example
 *    =EXP(1)
 */
GridMath["EXP"] = Math.exp; // Returns e raised to the power of a given number

/**
 * @function FACT
 * @memberof GridMath
 * @description
 *    {lang=en} Returns the factorial of a number
 *    {lang=ja} 数値の階乗を返します
 *    {lang=zh} 返回一个数字的阶乘
 *    {lang=hant} 返回一個數字的階乘
 *    {lang=ko} 숫자의 계승을 반환
 *    {lang=ru} Returns the factorial of a number
 *    {lang=de} Gibt den faktoriellen Wert einer Zahl aus
 * @param  {number} num
 *    {lang=en} Number
 *    {lang=ja} 数値
 *    {lang=zh} 数字
 *    {lang=hant} 數字
 *    {lang=ko} 숫자
 *    {lang=ru} Number
 *    {lang=de} Zahl
 * @return {number}
 * @example
 *    =FACT(10)
 */
GridMath["FACT"] = function(num) {
	if(num < 0) { return NaN; }
	num = Math.floor(num);
	let ret = 1;
	while(num > 1) {
		ret *= num;
		--num;
	}
	return ret;
};

/**
 * @function FACTDOUBLE
 * @memberof GridMath
 * @description
 *    {lang=en} Returns the double factorial of a number
 *    {lang=ja} 数値の二重階乗を返します
 *    {lang=zh} 返回一个数字的双阶乘
 *    {lang=hant} 返回一個數字的雙階乘
 *    {lang=ko} 숫자의 겹계승을 반환
 *    {lang=ru} Returns the double factorial of a number
 *    {lang=de} Gibt den doppelten faktoriellen Wert einer Zahl aus
 * @param  {number} num
 *    {lang=en} Number
 *    {lang=ja} 数値
 *    {lang=zh} 数字
 *    {lang=hant} 數字
 *    {lang=ko} 숫자
 *    {lang=ru} Number
 *    {lang=de} Zahl
 * @return {number}
 * @example
 *    =FACTDOUBLE(10)
 */
GridMath["FACTDOUBLE"] = function(num) {
	if(num < 0) { return NaN; }
	num = Math.floor(num);
	let ret = 1;
	while(num > 1) {
		ret *= num;
		num -= 2;
	}
	return ret;
};

/**
 * @function FLOOR
 * @memberof GridMath
 * @description
 *    {lang=en} Rounds a number down, toward zero
 *    {lang=ja} 指定された基準値の倍数のうち、最も近い値に数値を切り捨てます
 *    {lang=zh} 对一个数字四舍五入，靠近零值
 *    {lang=hant} 對一個數字四捨五入，靠近零值
 *    {lang=ko} 숫자를 0을 향해 내림
 *    {lang=ru} Rounds a number down, toward zero
 *    {lang=de} Rundet eine Zahl ab, auf Null
 * @param  {number} num
 *    {lang=en} Number
 *    {lang=ja} 数値
 *    {lang=zh} 数字
 *    {lang=hant} 數字
 *    {lang=ko} 숫자
 *    {lang=ru} Number
 *    {lang=de} Zahl
 * @param  {number} multiple
 *    {lang=en} The multiple to use when rounding
 *    {lang=ja} 基準値の倍数
 *    {lang=zh} 四舍五入时使用的乘数
 *    {lang=hant} 四捨五入時使用的乘數
 *    {lang=ko} 반올림할 때 사용할 배수
 *    {lang=ru} The multiple to use when rounding
 *    {lang=de} Das beim Runden zu verwendende Vielfache
 * @return {number}
 * @example
 *    =FLOOR(4.882961516)
 */
GridMath["FLOOR"] = function(num, multiple) {
	if(!multiple) {
		multiple = 1;
	}
	num /= multiple;
	return Math.floor(num) * multiple;
};

/**
 * @function GCD
 * @memberof GridMath
 * @description
 *    {lang=en} Returns a number representing the largest positive integer that divides the numbers without a remainder
 *    {lang=ja} 指定された数値の最大公約数を返します
 *    {lang=zh} 返回一个数字，代表整除该数的最大正整数
 *    {lang=hant} 返回一個數字，代表整除該數的最大正整數
 *    {lang=ko} 나머지 없이 숫자를 나누는 가장 큰 양의 정수를 나타내는 숫자를 반환
 *    {lang=ru} Returns a number representing the largest positive integer that divides the numbers without a remainder
 *    {lang=de} Gibt eine Zahl aus, welche die größte positive Ganzzahl darstellt, die die Zahlen ohne Rest dividiert
 * @param  {number} a
 *    {lang=en} The first number.
 *    {lang=ja} 数値 1
 *    {lang=zh} 第一个数字。
 *    {lang=hant} 第一個數字。
 *    {lang=ko} 첫 번째 숫자.
 *    {lang=ru} The first number.
 *    {lang=de} Die erste Zahl.
 * @param  {number} b
 *    {lang=en} The second number.
 *    {lang=ja} 数値 2
 *    {lang=zh} 第二个数字。
 *    {lang=hant} 第二個數字。
 *    {lang=ko} 두 번째 숫자.
 *    {lang=ru} The second number.
 *    {lang=de} Die zweite Zahl.
 * @return {number}
 * @example
 *    =GCD(222,70)
 */
GridMath["GCD"] = function(a, b) {
	if (a < 0) a = -a;
	if (b < 0) b = -b;
	if (b > a) { let temp = a; a = b; b = temp;}
	while (b != 0) {
		a %= b;
		if (a == 0) return b;
		b %= a;
	}
	return a;
};

/**
 * @function INT
 * @memberof GridMath
 * @description
 *    {lang=en} Rounds a number down to the nearest integer
 *    {lang=ja} 数値を切り捨て、最も近い整数を返します
 *    {lang=zh} 将一个数字向下舍入到最近的整数
 *    {lang=hant} 將一個數字向下舍入到最近的整數
 *    {lang=ko} 가장 가까운 정수로 숫자를 내림
 *    {lang=ru} Rounds a number down to the nearest integer
 *    {lang=de} Rundet eine Zahl auf die nächste Ganzzahl ab
 * @param  {number} x
 *    {lang=en} Number
 *    {lang=ja} 数値
 *    {lang=zh} 数字
 *    {lang=hant} 數字
 *    {lang=ko} 숫자
 *    {lang=ru} Number
 *    {lang=de} Zahl
 * @return {number}
 * @example
 *    =INT(4.882961516)
 */
GridMath["INT"] = Math.floor;

/**
 * @function LCM
 * @memberof GridMath
 * @description
 *    {lang=en} Returns the least common multiple. Supports any number of arguments LCM(number1, [number2], ...)
 *    {lang=ja} 最小公倍数を返します。この関数は、任意の引数 LCM をサポートしています (数値 1, 数値 2, ...)
 *    {lang=zh} 返回最小公倍数。支持任何数量的参数 LCM(number1, [number2], ...)
 *    {lang=hant} 返回最小公倍數。支持任何數量的參數 LCM(number1, [number2], ...)
 *    {lang=ko} 최소 공배수를 반환합니다. 인수를 제한 없이 지원합니다. LCM(number1, [number2], ...)
 *    {lang=ru} Returns the least common multiple. Supports any number of arguments LCM(number1, [number2], ...)
 *    {lang=de} Gibt das kleinste gemeinsame Vielfache auf. Unterstützt eine beliebige Anzahl von Argumenten LCM(Zahl1, [Zahl2], ...)
 * @return {number}
 * @example
 *    =LCM(10,15)
 */
GridMath["LCM"] = function(/* ... args*/) {
	let args = (arguments[0] instanceof Array) ? arguments[0] : arguments;
	let argLen = args.length;
	let a = Math.abs(args[0]);
	for (let i = 1; i < argLen; i++) {
		let b = Math.abs(args[i]);
		let c = a;
		while (a && b){
			if(a > b) {
				a %= b;
			} else {
				b %= a;
			}
		}
		a = Math.abs(c * args[i]) / (a + b);
	}
	return a;
};

/**
 * @function LN
 * @memberof GridMath
 * @description
 *    {lang=en} Returns the natural logarithm of a number
 *    {lang=ja} 数値の自然対数を返します。
 *    {lang=zh} 返回一个数字的自然对数
 *    {lang=hant} 返回一個數字的自然對數
 *    {lang=ko} 숫자의 자연로그를 반환
 *    {lang=ru} Returns the natural logarithm of a number
 *    {lang=de} Gibt den natürlichen Logarithmus einer Zahl aus
 * @param  {number} x
 *    {lang=en} Number
 *    {lang=ja} 数値
 *    {lang=zh} 数字
 *    {lang=hant} 數字
 *    {lang=ko} 숫자
 *    {lang=ru} Number
 *    {lang=de} Gibt den natürlichen Logarithmus einer Zahl aus
 * @return {number}
 * @example
 *    =LN(0.1353352832)
 */
GridMath["LN"] = Math.log;

/**
 * @function LOG
 * @memberof GridMath
 * @description
 *    {lang=en} Returns the logarithm of a number to a specified base
 *    {lang=ja} 指定された数を底とする数値の対数を返します。
 *    {lang=zh} 以指定的底，返回某个数字的对数
 *    {lang=hant} 以指定的底，返回某個數字的對數
 *    {lang=ko} 숫자의 로그를 지정된 밑으로 반환
 *    {lang=ru} Returns the logarithm of a number to a specified base
 *    {lang=de} Gibt den Logarithmus einer Zahl zu einer angegebenen Basis aus
 * @param  {number} num
 *    {lang=en} Number
 *    {lang=ja} 数値
 *    {lang=zh} 数字
 *    {lang=hant} 數字
 *    {lang=ko} 숫자
 *    {lang=ru} Number
 *    {lang=de} Zahl
 * @param  {number} base
 *    {lang=en} Base
 *    {lang=ja} 底
 *    {lang=zh} 底
 *    {lang=hant} 底
 *    {lang=ko} 밑
 *    {lang=ru} Base
 *    {lang=de} Basis
 * @return {number}
 * @example
 *    =LOG(10)
 */
GridMath["LOG"] = function(num, base) {
	if(base == null) { base = 10; }
	return Math.log(num) / Math.log(base);
};

/**
 * @function LOG10
 * @memberof GridMath
 * @description
 *    {lang=en} Returns the base-10 logarithm of a number
 *    {lang=ja} 10を底とする数値の対数 (常用対数) を返します
 *    {lang=zh} 返回一个数字以10为底的对数
 *    {lang=hant} 返回一個數字以10為底的對數
 *    {lang=ko} 숫자에 대해 밑이 10인 로그를 반환
 *    {lang=ru} Returns the base-10 logarithm of a number
 *    {lang=de} Gibt den Basis-10-Logarithmus einer Zahl aus
 * @param  {number} x
 *    {lang=en} Number
 *    {lang=ja} 数値
 *    {lang=zh} 数字
 *    {lang=hant} 數字
 *    {lang=ko} 숫자
 *    {lang=ru} Number
 *    {lang=de} Zahl
 * @return {number}
 * @example
 *    =LOG10(100000)
 */
GridMath["LOG10"] = function(x) {
	return Math.log(x) / Math.LN10;
};

/**
 * @function MOD
 * @memberof GridMath
 * @description
 *    {lang=en} Returns the remainder from division, i.e. Modulus
 *    {lang=ja} 除算の余りを返します
 *    {lang=zh} 返回余数
 *    {lang=hant} 返回餘數
 *    {lang=ko} 나누기에서 나머지(Modulus)를 반환
 *    {lang=ru} Returns the remainder from division, i.e. Modulus
 *    {lang=de} Gibt den Rest aus der Division aus, d. h. den Teilungsrest
 * @param  {number} num
 *    {lang=en} Number
 *    {lang=ja} 数値
 *    {lang=zh} 数字
 *    {lang=hant} 數字
 *    {lang=ko} 숫자
 *    {lang=ru} Number
 *    {lang=de} Zahl
 * @param  {number} denominator
 *    {lang=en} Denominator
 *    {lang=ja} 除数
 *    {lang=zh} 分母
 *    {lang=hant} 分母
 *    {lang=ko} 분모
 *    {lang=ru} Denominator
 *    {lang=de} Nenner
 * @return {number}
 * @example
 *    =MOD(5,2)
 */
GridMath["MOD"] = function(num, denominator) {
	return num % denominator;
};

/**
 * @function ODD
 * @memberof GridMath
 * @description
 *    {lang=en} Rounds a number up to the nearest odd integer
 *    {lang=ja} 数値を切り上げ、最も近い奇数にします
 *    {lang=zh} 将一个数字向上舍入到最近的奇数
 *    {lang=hant} 將一個數字向上舍入到最近的奇數
 *    {lang=ko} 가장 가까운 홀수인 정수로 숫자를 올림
 *    {lang=ru} Rounds a number up to the nearest odd integer
 *    {lang=de} Rundet eine Zahl auf die nächste ungerade Ganzzahl auf
 * @param  {number} num
 *    {lang=en} Number
 *    {lang=ja} 数値
 *    {lang=zh} 数字
 *    {lang=hant} 數字
 *    {lang=ko} 숫자
 *    {lang=ru} Number
 *    {lang=de} Zahl
 * @return {number}
 * @example
 *    =ODD(3.152)
 */
GridMath["ODD"] = function(num) { //
	num = Math.ceil(num);
	if((num % 2) === 0) {
		return (num < 0) ? num - 1 : num + 1;
	}
	return num;
};

/**
 * @function POWER
 * @memberof GridMath
 * @description
 *    {lang=en} Returns the result of a number raised to a power
 *    {lang=ja} 数値のべき乗を返します
 *    {lang=zh} 返回一个数字升幂的结果
 *    {lang=hant} 返回一個數字升冪的結果
 *    {lang=ko} 지수의 거듭제곱 결과를 반환
 *    {lang=ru} Returns the result of a number raised to a power
 *    {lang=de} Gibt das Ergebnis einer Zahl aus, die zu einer Potenz erhöht wird
 * @param  {number} base
 *    {lang=en} Base
 *    {lang=ja} 基準
 *    {lang=zh} 底
 *    {lang=hant} 底
 *    {lang=ko} 밑
 *    {lang=ru} Base
 *    {lang=de} Basis
 * @param  {number} exponent
 *    {lang=en} Exponent
 *    {lang=ja} 指数
 *    {lang=zh} 指数
 *    {lang=hant} 指數
 *    {lang=ko} 지수
 *    {lang=ru} Exponent
 *    {lang=de} Hochzahl
 * @return {number}
 * @example
 *    =POWER(2,10)
 */
GridMath["POWER"] = Math.pow;

/**
 * @function PRODUCT
 * @memberof GridMath
 * @description
 *    {lang=en}  Multiplies its arguments. Any number of arguments are supported PRODUCT(number1, [number2], ...)
 *    {lang=ja}  引数の積を返します。任意の引数が PRODUCT でサポートされています  (数値 1, 数値 2, ...)
 *    {lang=jp}  乘以其参数。支持任意数字的参数 PRODUCT(number1, [number2], ...)
 * @return {number}
 * @example
 *    =PRODUCT(1,-2,3)
 */
GridMath["PRODUCT"] = function(/* ... args*/) {
	let args = (arguments[0] instanceof Array) ? arguments[0] : arguments;
	let prod = args[0];
	if(prod == null || prod !== prod) { return NaN; }

	for(let i = args.length; --i >= 1;) {
		prod *= args[i];
	}
	return prod;
};

/**
 * @function PROD
 * @memberof GridMath
 * @description
 *    {lang=en}  Multiplies its arguments. Any number of arguments are supported PROD(number1, [number2], ...)
 *    {lang=ja}  引数の積を返します。任意の引数が PROD でサポートされています  (数値 1, 数値 2, ...)
 *    {lang=zh}  乘以其参数。支持任意数量的参数 PROD(number1, [number2], ...)
 *    {lang=hant}  乘以其參數。支持任意數量的參數 PROD(number1, [number2], ...)
 *    {lang=ko}  그 인수를 곱합니다. 인수를 제한 없이 지원합니다. PROD(number1, [number2], ...)
 *    {lang=ru}  Multiplies its arguments. Any number of arguments are supported PROD(number1, [number2], ...)
 *    {lang=de}  Multipliziert seine Argumente. Beliebig viele Argumente werden unterstützt PROD(Zahl1, [Zahl2], ...)
 * @return {number}
 * @example
 *    =PROD(1,-2,3)
 */
GridMath["PROD"] = GridMath["PRODUCT"];

/**
 * @function QUOTIENT
 * @memberof GridMath
 * @description
 *    {lang=en}  Returns the integer portion of a division
 *    {lang=ja}  除算の商の整数部を返します
 *    {lang=zh}  返回除法中的整数部分
 *    {lang=hant}  返回除法中的整數部分
 *    {lang=ko}  나눗셈의 정수 부분을 반환
 *    {lang=ru}  Returns the integer portion of a division
 *    {lang=de}  Gibt den ganzzahligen Anteil einer Division aus
 * @param  {number} num
 *    {lang=en} Number
 *    {lang=ja} 数値
 *    {lang=zh} 数字
 *    {lang=hant} 數字
 *    {lang=ko} 숫자
 *    {lang=ru} Number
 *    {lang=de} Zahl
 * @param  {number} denominator
 *    {lang=en} Denominator
 *    {lang=ja} 除数
 *    {lang=zh} 分母
 *    {lang=hant} 分母
 *    {lang=ko} 분모
 *    {lang=ru} Denominator
 *    {lang=de} Nenner
 * @return {number}
 * @example
 *    =QUOTIENT(10,3)
 */
GridMath["QUOTIENT"] = function(num, denominator) { //
	return Math.floor(num / denominator);
};

/**
 * @function RADIANS
 * @memberof GridMath
 * @description
 *    {lang=en} Converts degrees to radians
 *    {lang=ja} 度単位で表された角度をラジアンに変換します
 *    {lang=zh} 将角度转换成弧度
 *    {lang=hant} 將角度轉換成弧度
 *    {lang=ko} 도를 라디안으로 변환
 *    {lang=ru} Converts degrees to radians
 *    {lang=de} Wandelt Grad in Radianten um
 * @param  {number} degrees
 *    {lang=en} Degree
 *    {lang=ja} 角度
 *    {lang=zh} 角度
 *    {lang=hant} 角度
 *    {lang=ko} 도
 *    {lang=ru} Degree
 *    {lang=de} Grad
 * @return {number}
 * @example
 *    =RADIANS(45)
 */
GridMath["RADIANS"] = function(degrees) {
	return  degrees * (Math.PI / 180);
};

/**
 * @function RAND
 * @memberof GridMath
 * @description
 *    {lang=en} Returns a random number between 0 and 1
 *    {lang=ja} 0 以上 1 未満の乱数を返します
 *    {lang=zh} 返回一个 0 和 1 之间的随机数
 *    {lang=hant} 返回一個 0 和 1 之間的隨機數
 *    {lang=ko} 0과 1 사이의 난수를 반환
 *    {lang=ru} Returns a random number between 0 and 1
 *    {lang=de} Gibt eine Zufallszahl zwischen 0 und 1 aus
 * @return {number}
 * @example
 *    =RAND()
 */
GridMath["RAND"] = Math.random;

/**
 * @function RANDBETWEEN
 * @memberof GridMath
 * @description
 *    {lang=en} Returns a random number between the numbers you specify
 *    {lang=ja} 指定された範囲内の整数の乱数を返します
 *    {lang=zh} 返回一个您指定的数字之间的随机数
 *    {lang=hant} 返回一個您指定的數字之間的隨機數
 *    {lang=ko} 지정한 숫자 사이에 있는 난수를 반환
 *    {lang=ru} Returns a random number between the numbers you specify
 *    {lang=de} Gibt eine Zufallszahl zwischen den von Ihnen angegebenen Zahlen aus
 * @param  {number} bottom
 *    {lang=en} Low number
 *    {lang=ja} 最小値
 *    {lang=zh} 最小数
 *    {lang=hant} 最小數
 *    {lang=ko} 작은 수
 *    {lang=ru} Low number
 *    {lang=de} Niedrige Zahl
 * @param  {number} top
 *    {lang=en} High number
 *    {lang=ja} 最大値
 *    {lang=zh} 最大数
 *    {lang=hant} 最大數
 *    {lang=ko} 큰 수
 *    {lang=ru} High number
 *    {lang=de} Hohe Zahl
 * @return {number}
 * @example
 *    =RANDBETWEEN(2,10)
 */
GridMath["RANDBETWEEN"] = function(bottom, top) {  //
	return Math.floor(Math.random() * (top - bottom + 1)) + bottom;
};

/**
 * @function ROUND
 * @memberof GridMath
 * @description
 *    {lang=en} Rounds a number to a specified number of digits
 *    {lang=ja} 数値を四捨五入して指定された桁数にします
 *    {lang=zh} 将一个数字四舍五入到指定的位数
 *    {lang=hant} 將一個數字四捨五入到指定的位數
 *    {lang=ko} 숫자를 지정된 자릿수로 반올림
 *    {lang=ru} Rounds a number to a specified number of digits
 *    {lang=de} Rundet eine Zahl auf eine bestimmte Anzahl an Ziffern
 * @param  {number} num
 *    {lang=en} Number
 *    {lang=ja} 数値
 *    {lang=zh} 数字
 *    {lang=hant} 數字
 *    {lang=ko} 숫자
 *    {lang=ru} Number
 *    {lang=de} Zahl
 * @param  {number} num_digits
 *    {lang=en} Number of digits. Note that it can go below zero.
 *    {lang=ja} 桁数
 *    {lang=zh} 数字位数。请注意这可能小于零。
 *    {lang=hant} 數字位數。請注意這可能小於零。
 *    {lang=ko} 자릿수. 0 이하가 될 수 있습니다.
 *    {lang=ru} Number of digits. Note that it can go below zero.
 *    {lang=de} Anzahl der Ziffern. Bitte beachten Sie, dass sie unter Null gehen kann.
 * @return {number}
 * @example
 *    =ROUND(54.943, 1)
 */
GridMath["ROUND"] = function(num, num_digits) {
	if(!num_digits) {
		return Math.round(num);
	}
	let divisor = Math.pow(10, num_digits); // Slow!
	return Math.round(num * divisor) / divisor;
};

/**
 * @function ROUNDDOWN
 * @memberof GridMath
 * @description
 *    {lang=en} Rounds a number down, toward zero
 *    {lang=ja} 数値を指定された桁数で切り捨てます
 *    {lang=zh} 将一个数字向下舍入，趋向于零
 *    {lang=hant} 將一個數字向下舍入，趨向於零
 *    {lang=ko} 숫자를 0을 향해 내림
 *    {lang=ru} Rounds a number down, toward zero
 *    {lang=de} Rundet eine Zahl ab, auf Null
 * @param  {number} num
 *    {lang=en} Number
 *    {lang=ja} 数値
 *    {lang=zh} 数字
 *    {lang=hant} 數字
 *    {lang=ko} 숫자
 *    {lang=ru} Number
 *    {lang=de} Zahl
 * @param  {number} num_digits
 *    {lang=en} Number of digits
 *    {lang=ja} 桁数
 *    {lang=zh} 数字位数
 *    {lang=hant} 數字位數
 *    {lang=ko} 자릿수
 *    {lang=ru} Number of digits
 *    {lang=de} Anzahl der Ziffern
 * @return {number}
 * @example
 *    =ROUNDDOWN(54.928, 2)
 */
GridMath["ROUNDDOWN"] = function(num, num_digits) {
	if(!num_digits) {
		return Math.floor(num);
	}
	let divisor = Math.pow(10, num_digits); // Slow!
	return Math.floor(num * divisor) / divisor;
};

/**
 * @function ROUNDUP
 * @memberof GridMath
 * @description
 *    {lang=en} Rounds a number up, away from zero
 *    {lang=ja} 数値を指定された桁数で切り上げます
 *    {lang=zh} 将一个数字向上舍入，远离零值
 *    {lang=hant} 將一個數字向上舍入，遠離零值
 *    {lang=ko} 숫자를 0에서 멀리 올림
 *    {lang=ru} Rounds a number up, away from zero
 *    {lang=de} Rundet eine Zahl auf, weg von Null
 * @param  {number} num
 *    {lang=en} Number
 *    {lang=ja} 数値
 *    {lang=zh} 数字
 *    {lang=hant} 數字
 *    {lang=ko} 숫자
 *    {lang=ru} Number
 *    {lang=de} Zahl
 * @param  {number} num_digits
 *    {lang=en} Number of digits. Note that it can go below zero.
 *    {lang=ja} 桁数
 *    {lang=zh} 数字位数。请注意这可能小于零。
 *    {lang=hant} 數字位數。請注意這可能小於零。
 *    {lang=ko} 자릿수. 0 이하가 될 수 있습니다.
 *    {lang=ru} Number of digits. Note that it can go below zero.
 *    {lang=de} Anzahl der Ziffern. Bitte beachten Sie, dass sie unter Null gehen kann.
 * @return {number}
 * @example
 *    =ROUNDUP(54.928, 2)
 */
GridMath["ROUNDUP"] = function(num, num_digits) { //
	if(!num_digits) {
		return Math.ceil(num);
	}
	let divisor = Math.pow(10, num_digits); // Slow!
	return Math.ceil(num * divisor) / divisor;
};

/**
 * @function SERIESSUM
 * @memberof GridMath
 * @description
 *    {lang=en} Returns the sum of a power series based on the formula
 *    {lang=ja} 数式で定義されるべき数を返します
 *    {lang=zh} 返回一个幂级数基于公式的和
 *    {lang=hant} 返回一個冪級數基於公式的和
 *    {lang=ko} 수식을 기반으로 멱급수의 합을 반환
 *    {lang=ru} Returns the sum of a power series based on the formula
 *    {lang=de} Gibt die Summe einer Potenzreihe basierend auf der Formel aus
 * @param  {number} x
 *    {lang=en} The input value to the power series.
 *    {lang=ja} べき級数に代入する値を指定します。
 *    {lang=zh} 幂级数的输入值。
 *    {lang=hant} 冪級數的輸入值。
 *    {lang=ko} 멱급수에 대한 입력값입니다.
 *    {lang=ru} The input value to the power series.
 *    {lang=de} Der Eingangswert für die Potenzreihe.
 * @param  {number} n
 *    {lang=en} The first power to which x is to be raised.
 *    {lang=ja} x のべき乗の初期値を指定します。
 *    {lang=zh} x 升幂的第一个幂。
 *    {lang=hant} x 升冪的第一個冪。
 *    {lang=ko} x의 1제곱.
 *    {lang=ru} The first power to which x is to be raised.
 *    {lang=de} Die erste Potenz, auf die x angehoben werden soll.
 * @param  {number} m
 *    {lang=en} The step size that n is increased by, on each successive power of x.
 *    {lang=ja} 級数の各項ごとに n を増加させる値を指定します。
 *    {lang=zh} 每个 x 的连续幂，n 增加的步长。
 *    {lang=hant} 每個 x 的連續冪，n 增加的步長。
 *    {lang=ko} n이 x의 각 연속 지수에 의해 증분되는 양.
 *    {lang=ru} The step size that n is increased by, on each successive power of x.
 *    {lang=de} Die Schrittgröße, um die n bei jeder aufeinanderfolgenden Potenz von x erhöht wird.
 * @param  {Array.<number>} coefficients
 *    {lang=en} An array of coefficients that multiply each successive power of x.
 *    {lang=ja} x のべき乗である各項にかける一連の係数を指定します。
 *    {lang=zh} 乘以每个 x 的连续幂的系数的数组。
 *    {lang=hant} 乘以每個 x 的連續冪的係數的數組。
 *    {lang=ko} x의 각 연속 지수를 곱하는 계수들의 배열.
 *    {lang=ru} An array of coefficients that multiply each successive power of x.
 *    {lang=de} Eine Anordnung von Koeffizienten, die jede aufeinanderfolgende Potenz von x multipliziert.
 * @return {number}
 * @example
 * // WARNING: coefficients variable is an Array in which user have no knowledge about how to create an Array in JavaScript syntax. Unsure how to set the array
 *    =SERIESSUM( 5, 1, 1,5 )
 */
GridMath["SERIESSUM"] = function(x, n, m, coefficients) { //
	// WARNING: coefficients variable is an Array in which user have no knowledge about how to create an Array in JavaScript syntax.
	let sum = 0;
	for(let i = coefficients.length; --i >= 0;) {
		sum += coefficients[i] * Math.pow(x, n + i * m);
	}
	return sum;
};

/**
 * @function SIGN
 * @memberof GridMath
 * @description
 *    {lang=en} Returns the sign of a number
 *    {lang=ja} 数値の正負を調べます
 *    {lang=zh} 返回数字的符号
 *    {lang=ko} 숫자의 사인을 반환
 *    {lang=ru} Returns the sign of a number
 * @param  {number} num
 *    {lang=en} Number
 *    {lang=ja} 数値
 *    {lang=zh} 数字
 *    {lang=ko} 숫자
 *    {lang=ru} Number
 * @return {number}
 * @example
 *    =SIGN(-496)
 */
GridMath["SIGN"] = function(num) {
	if(num > 0) {
		return 1;
	} else if(num < 0) {
		return -1;
	}
	return 0; // num may have a undefined, null, NaN, or empty string value
};

/**
 * @function SIN
 * @memberof GridMath
 * @description
 *    {lang=en} Returns the sine of the given angle
 *    {lang=ja} 角度のサインを返します
 *    {lang=zh} 返回给定角的正弦
 *    {lang=hant} 返回給定角的正弦
 *    {lang=ko} 주어진 각도의 사인을 반환
 *    {lang=ru} Returns the sine of the given angle
 *    {lang=de} Gibt den Sinus des gegebenen Winkels aus
 * @param  {number} angle
 *    {lang=en} Angle
 *    {lang=ja} 角度
 *    {lang=zh} 角
 *    {lang=hant} 角
 *    {lang=ko} 각도
 *    {lang=ru} Angle
 *    {lang=de} Winkel
 * @return {number}
 * @example
 *    =SIN(-PI()/6)
 */
GridMath["SIN"] = Math.sin; //

/**
 * @function SINH
 * @memberof GridMath
 * @description
 *    {lang=en} Returns the hyperbolic sine of a number
 *    {lang=ja} 数値の双曲サインを返します
 *    {lang=zh} 返回一个数字的双正弦
 *    {lang=hant} 返回一個數字的雙正弦
 *    {lang=ko} 숫자의 쌍곡사인을 반환
 *    {lang=ru} Returns the hyperbolic sine of a number
 *    {lang=de} Gibt den hyperbolischen Sinus einer Zahl aus
 * @param  {number} x
 *    {lang=en} Number
 *    {lang=ja} 数値
 *    {lang=zh} 数字
 *    {lang=hant} 數字
 *    {lang=ko} 숫자
 *    {lang=ru} Number
 *    {lang=de} Zahl
 * @return {number}
 * @example
 *    =SINH(4)
 */
GridMath["SINH"] = function(x) {
	let y = Math.exp(x);
	return (y - 1 / y) / 2;
};

/**
 * @function SQRT
 * @memberof GridMath
 * @description
 *    {lang=en} Returns a positive square root
 *    {lang=ja} 正の平方根を返します
 *    {lang=zh} 返回一个正的平方根
 *    {lang=hant} 返回一個正的平方根
 *    {lang=ko} 양의 제곱근을 반환
 *    {lang=ru} Returns a positive square root
 *    {lang=de} Gibt eine positive Quadratwurzel aus
 * @param  {number} x
 *    {lang=en} Number
 *    {lang=ja} 数値
 *    {lang=zh} 数字
 *    {lang=hant} 數字
 *    {lang=ko} 숫자
 *    {lang=ru} Number
 *    {lang=de} Zahl
 * @return {number}
 * @example
 *    =SQRT(4)
 */
GridMath["SQRT"] = Math.sqrt;

/**
 * @function SQRTPI
 * @memberof GridMath
 * @description
 *    {lang=en} Returns the square root of (multiple * pi)
 *    {lang=ja} 数値 x n の平方根の値を計算します
 *    {lang=zh} 返回 (multiple * pi) 的平方根
 *    {lang=hant} 返回 (multiple * pi) 的平方根
 *    {lang=ko} (배수 * 파이) 제곱근을 반환
 *    {lang=ru} Returns the square root of (multiple * pi)
 *    {lang=de} Gibt die Quadratwurzel von (Vielfaches * pi) aus
 * @param  {number} multiple
 *    {lang=en} Multiple
 *    {lang=ja} 数値
 *    {lang=zh} 乘数
 *    {lang=hant} 乘數
 *    {lang=ko} 배수
 *    {lang=ru} Multiple
 *    {lang=de} Mehrere
 * @return {number}
 * @example
 *    =SQRTPI(3)
 */
GridMath["SQRTPI"] = function(multiple) {
	return Math.sqrt(multiple * Math.PI);
};

/**
 * @function SUM
 * @memberof GridMath
 * @description
 *    {lang=en} Returns the summation of all the arguments. This function supports any number of arguments SUM(number1, [number2], ...)
 *    {lang=ja} すべての引数の合計を返します。この関数は、任意の値の引数 SUM(数値 1, 数値 2, ...) をサポートしています
 *    {lang=zh} 返回全部参数的和。这个函数支持任意数量的参数 SUM(number1, [number2], ...)
 *    {lang=hant} 返回全部參數的和。這個函數支持任意數量的參數 SUM(number1, [number2], ...)
 *    {lang=ko} 모든 인수의 합계를 반환합니다. 이 함수는 인수를 제한 없이 지원합니다. SUM(number1, [number2], ...)
 *    {lang=ru} Returns the summation of all the arguments. This function supports any number of arguments SUM(number1, [number2], ...)
 *    {lang=de} Gibt die Summenbildung aller Argumente aus. Diese Funktion unterstützt eine beliebige Anzahl von Argumenten SUM(Zahl1, [Zahl2], ...)
 * @return {number}
 * @example
 *    =SUM(4,-500)
 */
GridMath["SUM"] = function(/* ... args*/) {
	let args = (arguments[0] instanceof Array) ? arguments[0] : arguments;
	let sum = 0;
	for(let i = args.length; --i >= 0;) {
		sum += args[i];
	}
	return sum;
};

/**
 * @function SUMPRODUCT
 * @memberof GridMath
 * @description
 *    {lang=en} Returns the sum of the products of corresponding array components.
 * 				This function supports any number of arguments SUMPRODUCT(number1, [number2], ...)
 *    {lang=ja} 指定された配列で対応する要素の積の合計を返します。
 * 				この関数は、任意の値の引数 SUMPRODUCT(数値 1, 数値 2, ...)
 をサポートしています。
 *    {lang=zh} 返回相应数组元素的乘积的和。
 * 				这个函数支持任意数量的参数 SUMPRODUCT(number1, [number2], ...)
 *    {lang=ko} 해당하는 배열 요소의 곱의 합계를 반환합니다.
 *    {lang=ru} Returns the sum of the products of corresponding array components.
 * 				This function supports any number of arguments SUMPRODUCT(number1, [number2], ...)
 * @return {number}
 * @example
 *    =SUMPRODUCT(2,3)
 */
GridMath["SUMPRODUCT"] = function(/* ... args*/) {
	let args = (arguments[0] instanceof Array) ? arguments[0] : arguments;
	let argLen = args.length;
	if(argLen === 0) { return 0; }

	let i;
	let products = [];
	for(i = 0; i < argLen; ++i) {
		products[i] = 1;
	}
	for(i = 0; i < argLen; ++i) {
		products[i] *= args[i];
	}
	let sum = 0;
	for(i = 0; i < argLen; ++i) {
		sum += products[i];
	}
	return sum;
};

/**
 * @function SUMSQ
 * @memberof GridMath
 * @description
 *    {lang=en} Returns the sum of the squares of the arguments. This function supports any number of arguments SUMSQ(number1, [number2], ...)
 *    {lang=ja} 引数の2乗の合計を返します。この関数は、任意の値の引数 SUMSQ(数値 1, 数値 2, ...) をサポートしています
 *    {lang=zh} 返回参数的平方的和。这个函数支持任意数量的参数 SUMSQ(number1, [number2], ...)
 *    {lang=hant} 返回參數的平方的和。這個函數支持任意數量的參數 SUMSQ(number1, [number2], ...)
 *    {lang=ko} 인수의 제곱의 합을 반환합니다. 이 함수는 인수를 제한 없이 지원합니다. SUMSQ(number1, [number2], ...)
 *    {lang=ru} Returns the sum of the squares of the arguments. This function supports any number of arguments SUMSQ(number1, [number2], ...)
 *    {lang=de} Gibt die Summe der Quadratwurzeln der Argumente aus. Diese Funktion unterstützt eine beliebige Anzahl von Argumenten SUMSQ(Zahl1, [Zahl2], ...)
 * @return {number}
 * @example
 *    =SUMSQ(3)
 */
GridMath["SUMSQ"] = function(/* ... args*/) {
	let args = (arguments[0] instanceof Array) ? arguments[0] : arguments;
	let sum = 0;
	for(let i = args.length; --i >= 0;) {
		let val = args[i];
		sum += (val * val);
	}
	return sum;
};


// /**
//  * @function SUMX2MY2
//  * @memberof GridMath
//  * @description
//  *    {lang=en} Returns the sum of the difference of squares of corresponding values in two arrays
//  *    {lang=ja} 2つの配列の対応する配列要素の平方差の合計を返します
//  *    {lang=zh} 返回两个数组中相应值的平方的差的和
//  *    {lang=ko} 두 배열에서 해당하는 값의 제곱의 차의 합을 반환
//  *    {lang=ru} Returns the sum of the difference of squares of corresponding values in two arrays
//  * @param  {number} aryX
//  *    {lang=en} First array
//  *    {lang=ja} 配列 1
//  *    {lang=zh} 第一个数组
//  *    {lang=ko} 첫 번째 배열
//  *    {lang=ru} First array
//  * @param  {number} aryY
//  *    {lang=en} Second array
//  *    {lang=ja} 配列 2
//  *    {lang=zh} 第二个数组
//  *    {lang=ko} 두 번째 배열
//  *    {lang=ru} Second array
//  * @return {number}
//  * @example
//  *    <Example can be provided here>
//  */
// GridMath["SUMX2MY2"] = function(aryX, aryY) {
// 	let sum = 0;
// 	for(let i = aryX.length; --i >= 0;) {
// 		let valX = aryX[i];
// 		let valY = aryY[i];
// 		sum += (valX * valX) - (valY * valY);
// 	}
// 	return sum;
// };

// *
//  * @function SUMX2PY2
//  * @memberof GridMath
//  * @description
//  *    {lang=en} Returns the sum of the sum of squares of corresponding values in two arrays
//  *    {lang=ja} 2つの配列の対応する値の平方和を返します
//  *    {lang=zh} 返回两个数组中相应的值的平方的和
//  *    {lang=ko} 두 배열에서 해당하는 값의 제곱합의 합을 반환
//  *    {lang=ru} Returns the sum of the sum of squares of corresponding values in two arrays
//  * @param  {number} aryX
//  *    {lang=en} First array
//  *    {lang=ja} 配列 1
//  *    {lang=zh} 第一个数组
//  *    {lang=ko} 첫 번째 배열
//  *    {lang=ru} First array
//  * @param  {number} aryY
//  *    {lang=en} Second array
//  *    {lang=ja} 配列 2
//  *    {lang=zh} 第二个数组
//  *    {lang=ko} 두 번째 배열
//  *    {lang=ru} Second array
//  * @return {number}
//  * @example
//  *    <Example can be provided here>

// GridMath["SUMX2PY2"] = function(aryX, aryY) {
// 	let sum = 0;
// 	for(let i = aryX.length; --i >= 0;) {
// 		let valX = aryX[i];
// 		let valY = aryY[i];
// 		sum += (valX * valX) + (valY * valY);
// 	}
// 	return sum;
// };

// /**
//  * @function SUMXMY2
//  * @memberof GridMath
//  * @description
//  *    {lang=en} Returns the sum of squares of differences of corresponding values in two arrays
//  *    {lang=ja} 2つの配列の対応する値の差の平方和を返します
//  *    {lang=zh} 返回两个数组中相应的值的差的平方的和
//  *    {lang=ko} 두 배열에서 해당하는 값의 차의 제곱합을 반환
//  *    {lang=ru} Returns the sum of squares of differences of corresponding values in two arrays
//  * @param  {number} aryX
//  *    {lang=en} First array
//  *    {lang=ja} 配列 1
//  *    {lang=zh} 第一个数组
//  *    {lang=ko} 첫 번째 배열
//  *    {lang=ru} First array
//  * @param  {number} aryY
//  *    {lang=en} Second array
//  *    {lang=ja} 配列 2
//  *    {lang=zh} 第二个数组
//  *    {lang=ko} 두 번째 배열
//  *    {lang=ru} Second array
//  * @return {number}
//  * @example
//  *    <Example can be provided here>
//  */
// GridMath["SUMXMY2"] = function(aryX, aryY) {
// 	let sum = 0;
// 	for(let i = aryX.length; --i >= 0;) {
// 		let XMY = aryX[i] - aryY[i]; // X minus Y
// 		sum += (XMY * XMY);
// 	}
// 	return sum;
// };

/**
 * @function TAN
 * @memberof GridMath
 * @description
 *    {lang=en} Returns the tangent of a number
 *    {lang=ja} 角度のタンジェントを返します
 *    {lang=zh} 返回一个数字的正切
 *    {lang=hant} 返回一個數字的正切
 *    {lang=ko} 숫자의 탄젠트를 반환
 *    {lang=ru} Returns the tangent of a number
 *    {lang=de} Gibt den Tangens einer Zahl aus
 * @param  {number} x
 *    {lang=en} Number
 *    {lang=ja} 数値
 *    {lang=zh} 数字
 *    {lang=hant} 數字
 *    {lang=ko} 숫자
 *    {lang=ru} Number
 *    {lang=de} Zahl
 * @return {number}
 * @example
 *    =TAN(-PI()/4)
 */
GridMath["TAN"] = Math.tan;

/**
 * @function TANH
 * @memberof GridMath
 * @description
 *    {lang=en} Returns the hyperbolic tangent of a number
 *    {lang=ja} 数値の双曲タンジェントを返します
 *    {lang=zh} 返回一个数字的双曲正切
 *    {lang=hant} 返回一個數字的雙曲正切
 *    {lang=ko} 숫자의 쌍곡탄젠트를 반환
 *    {lang=ru} Returns the hyperbolic tangent of a number
 *    {lang=de} Gibt den hyperbolischen Tangens einer Zahl aus
 * @param  {number} x
 *    {lang=en} Number
 *    {lang=ja} 数値
 *    {lang=zh} 数字
 *    {lang=hant} 數字
 *    {lang=ko} 숫자
 *    {lang=ru} Number
 *    {lang=de} Zahl
 * @return {number}
 * @example
 *    =TANH(4)
 */
GridMath["TANH"] = function(x) {
	if (x === Infinity) {
		return 1;
	} else if (x === -Infinity) {
		return -1;
	}
	let y = Math.exp(2 * x);
	return (y - 1) / (y + 1);
};

/**
 * @function TRUNC
 * @memberof GridMath
 * @description
 *    {lang=en} Truncates a number to an integer
 *    {lang=ja} 数値の小数部を切り捨て、整数または指定された桁数にします
 *    {lang=zh} 将一个数字截断成整数
 *    {lang=hant} 將一個數字截斷成整數
 *    {lang=ko} 숫자를 정수로 자름
 *    {lang=ru} Truncates a number to an integer
 *    {lang=de} Kürzt eine Zahl auf eine Ganzzahl
 * @param  {number} num
 *    {lang=en} Number
 *    {lang=ja} 数値
 *    {lang=zh} 数字
 *    {lang=hant} 數字
 *    {lang=ko} 숫자
 *    {lang=ru} Number
 *    {lang=de} Zahl
 * @param  {number} num_digits
 *    {lang=en} Number of digits
 *    {lang=ja} 桁数
 *    {lang=zh} 数字位数
 *    {lang=hant} 數字位數
 *    {lang=ko} 자릿수
 *    {lang=ru} Number of digits
 *    {lang=de} Anzahl der Ziffern
 * @return {number}
 * @example
 *    =TRUNC(CF_LAST,1)
 */
GridMath["TRUNC"] = function(num, num_digits) {
	if(!num_digits) {
		return Math.floor(num);
	}
	let divisor = Math.pow(10, num_digits);
	return Math.floor(num * divisor) / divisor;
};

/* harmony default export */ const functions_Math = (GridMath);


;// ./node_modules/tr-grid-util/es6/formula/functions/Statistics.js
/**
 * @namespace Statistic
 * @description
 *    {lang=en} Statistic functions
 *    {lang=ja} 統計関数
 *    {lang=zh} 统计函数
 *    {lang=hant} 統計函數
 *    {lang=ko} 통계 함수
 *    {lang=ru} Statistic functions
 *    {lang=de} Statistische Funktionen
 */
let Stat = {};

/**
* @function AVERAGE
* @memberof Statistic
* @description
*    {lang=en} Returns the average (arithmetic mean) of the arguments. This function supports any number of arguments AVERAGE(number1, [number2], [number3], ...)
*    {lang=ja} 引数の平均値 (算術平均) を返します。この関数は、任意の値の引数 AVERAGE をサポートしています (数値 1, 数値 2, 数値 3, ...)
*    {lang=zh} 返回参数的平均值 (算数平均数)。这个函数支持任意数量的参数 AVERAGE(number1, [number2], [number3], ...)
*    {lang=hant} 返回參數的平均值 (算數平均數)。這個函數支持任意數量的參數 AVERAGE(number1, [number2], [number3], ...)
*    {lang=ko} 인수의 평균(산술 평균)을 반환합니다. 이 함수는 인수를 제한 없이 지원합니다. AVERAGE(number1, [number2], [number3], ...)
*    {lang=ru} Returns the average (arithmetic mean) of the arguments. This function supports any number of arguments AVERAGE(number1, [number2], [number3], ...)
*    {lang=de} Gibt den Durchschnitt (arithmetische Mitte) der Argumente aus. Diese Funktion unterstützt den DURCHSCHNITT einer beliebigen Anzahl von Argumenten (Zahl1, [Zahl2], [Zahl3], ...)
* @return {number}
* @example
*    =AVERAGE(6.10,13.20,15.40)
*/
Stat["AVERAGE"] = function(/* ... args*/) {
	let args = (arguments[0] instanceof Array) ? arguments[0] : arguments;
	let argLen = args.length;
	if(argLen <= 0) { return NaN; }

	let sum = 0;
	for(let i = argLen; --i >= 0;) {
		sum += args[i];
	}
	return sum / argLen;
};

/**
* @function AVG
* @memberof Statistic
* @description
*    {lang=en} Returns the average (arithmetic mean) of the arguments. This function supports any number of arguments AVG(number1, [number2], [number3], ...)
*    {lang=ja} 引数の平均値 (算術平均) を返します。この関数は、任意の値の引数 AVG をサポートしています (数値 1, 数値 2, 数値 3, ...)
*    {lang=zh} 返回参数的平均值 (算数平均数)。这个函数支持任意数量的参数 AVG(number1, [number2], [number3], ...)
*    {lang=hant} 返回參數的平均值 (算數平均數)。這個函數支持任意數量的參數 AVG(number1, [number2], [number3], ...)
*    {lang=ko} 인수의 평균(산술 평균)을 반환합니다. 이 함수는 인수를 제한 없이 지원합니다. AVG(number1, [number2], [number3], ...)
*    {lang=ru} Returns the average (arithmetic mean) of the arguments. This function supports any number of arguments AVG(number1, [number2], [number3], ...)
*    {lang=de} Gibt den Durchschnitt (arithmetische Mitte) der Argumente aus. Diese Funktion unterstützt den DURCHSCHN. einer beliebigen Anzahl von Argumenten (Zahl1, [Zahl2], [Zahl3], ...)
* @return {number}
* @example
*    =AVG(6.10,13.20,15.40)
*/
Stat["AVG"] = Stat["AVERAGE"];

/**
* @function COUNT
* @memberof Statistic
* @description
*    {lang=en} Returns the number of entries in a number field that is in a range or array of numbers.
*              This function supports any number of arguments COUNT(number1, [number2], [number3], ...)
*    {lang=ja} 数値列のフィールドに入力された数値の数を返します。
*              この関数は、任意の値の引数 COUNT をサポートしています (数値 1, 数値 2, 数値 3, ...)
*    {lang=zh} 返回在某数字范围或某数组中的数字项输入的次数。
*              这个函数支持任意数量的参数 COUNT(number1, [number2], [number3], ...)
*    {lang=hant} 返回在某數字範圍或某數組中的數字項輸入的次數。
*              這個函數支持任意數量的參數 COUNT(number1, [number2], [number3], ...)
*    {lang=ko} 범위 또는 숫자 배열에 있는 숫자 필드의 항목 수를 반환합니다.
*    {lang=ru} Returns the number of entries in a number field that is in a range or array of numbers.
*              This function supports any number of arguments COUNT(number1, [number2], [number3], ...)
*    {lang=de} Gibt die Anzahl der Einträge in einem Zahlenfeld aus, das sich in einem Bereich oder einer Anordnung von Zahlen befindet.
*              This function supports any number of arguments COUNT(number1, [number2], [number3], ...)
* @return {number}
* @example
*    =COUNT(6.10,13.20,15.40)
*/
Stat["COUNT"] = function(/* ... args*/) {
	let args = (arguments[0] instanceof Array) ? arguments[0] : arguments;
	let count = 0;
	for(let i = args.length; --i >= 0;) {
		let arg = args[i];
		if(arg != null && arg == arg) {
			++count;
		}
	}
	return count;
};

/**
* @function MAX
* @memberof Statistic
* @description
*    {lang=en} Returns the largest value from a supplied set of numeric values.
*              This function supports any number of arguments MAX(number1, [number2], [number3], ...)
*    {lang=ja} 指定された数値の中から最大値を返します。
*              この関数は、任意の値の引数 MAX をサポートしています (数値 1, 数値 2, 数値 3, ...)
*    {lang=zh} 返回所提供的数值集中的最大值。
*              这个函数支持任意数量的参数 MAX(number1, [number2], [number3], ...)
*    {lang=hant} 返回所提供的數值集中的最大值。
*              這個函數支持任意數量的參數 MAX(number1, [number2], [number3], ...)
*    {lang=ko} 제공된 숫자 값의 집합에서 가장 큰 값을 반환합니다.
*    {lang=ru} Returns the largest value from a supplied set of numeric values.
*              This function supports any number of arguments MAX(number1, [number2], [number3], ...)
*    {lang=de} Gibt den größten Wert aus einem gelieferten Satz numerischer Werte aus.
*              This function supports any number of arguments MAX(number1, [number2], [number3], ...)
* @return {number}
* @example
*    =MAX(6.10,13.20,15.40)
*/
Stat["MAX"] = function(/* ... args*/) {
	let args = (arguments[0] instanceof Array) ? arguments[0] : arguments;
	if(args.length <= 0) { return NaN; }

	let max = args[0];
	for(let i = args.length; --i >= 1;) {
		if(args[i] > max) {
			max = args[i];
		}
	}
	return max;
};

/**
* @function MIN
* @memberof Statistic
* @description
*    {lang=en} Returns the smallest value from a supplied set of numeric values.
*              This function supports any number of arguments MIN(number1, [number2], [number3], ...)
*    {lang=ja} 指定された数値の中から最小値を返します。
*              この関数は、任意の値の引数 MIN をサポートしています (数値 1, 数値 2, 数値 3, ...)
*    {lang=zh} 返回所提供的数值集中的最小值。
*              这个函数支持任意数量的参数 MIN(number1, [number2], [number3], ...)
*    {lang=hant} 返回所提供的數值集中的最小值。
*              這個函數支持任意數量的參數 MIN(number1, [number2], [number3], ...)
*    {lang=ko} 제공된 숫자 값의 집합에서 가장 작은 값을 반환합니다.
*    {lang=ru} Returns the smallest value from a supplied set of numeric values.
*              This function supports any number of arguments MIN(number1, [number2], [number3], ...)
*    {lang=de} Gibt den kleinsten Wert aus einem gelieferten Satz numerischer Werte aus.
*              This function supports any number of arguments MIN(number1, [number2], [number3], ...)
* @return {number}
* @example
*    =MIN(6.10,13.20,15.40)
*/
Stat["MIN"] = function(/* ... args*/) {
	let args = (arguments[0] instanceof Array) ? arguments[0] : arguments;
	if(args.length <= 0) { return NaN; }

	let min = args[0];
	for(let i = args.length; --i >= 1;) {
		if(args[i] < min) {
			min = args[i];
		}
	}
	return min;
};

/**
* @function STDEV
* @memberof Statistic
* @description
*    {lang=en} Estimates standard deviation based on a sample (unbias standard deviation for small sample size)
*              This function supports any number of arguments STDEV(number1, [number2], [number3], ...)
*    {lang=ja} 標本に基づいて予想標準偏差を返します (小さいサンプル サイズの偏りのない標準偏差)
*              この関数は、任意の値の引数 STDEV をサポートしています (数値 1, 数値 2, 数値 3, ...)
*    {lang=zh} 基于样本的预估标准差 (小样本规模的无偏标准差)
*              这个函数支持任意数量的参数 STDEV(number1, [number2], [number3], ...)
*    {lang=hant} 基於樣本的預估標準差 (小樣本規模的無偏標準差)
*              這個函數支持任意數量的參數 STDEV(number1, [number2], [number3], ...)
*    {lang=ko} 표본을 기준으로 표준편차를 추정(표본 크기가 작으면 편의가 없는 표준편차)
*    {lang=ru} Estimates standard deviation based on a sample (unbias standard deviation for small sample size)
*              This function supports any number of arguments STDEV(number1, [number2], [number3], ...)
*    {lang=de} Schätzt die Standardabweichung basierend auf einer Stichprobe (objektive Standardabweichung für kleine Stichprobengrößen)
*              This function supports any number of arguments STDEV(number1, [number2], [number3], ...)
* @return {number}
* @example
*    =STDEV(6.10,13.20,15.40)
*/
Stat["STDEV"] = function(/* ... args*/) {
	let args = (arguments[0] instanceof Array) ? arguments[0] : Array.prototype.slice.call(arguments);
	let argLen = args.length;
	if(argLen <= 0) { return NaN; }
	if(argLen === 1) { return 0; }

	let mean = Stat["AVERAGE"](args);

	let sum = 0;
	for(let i = argLen; --i >= 0;) {
		let diff = args[i] - mean;
		sum += diff * diff;
	}
	return Math.sqrt(sum / (argLen - 1));
};

/**
* @function STDEVP
* @memberof Statistic
* @description
*    {lang=en} Calculates standard deviation based on the entire population (bias standard deviation for small sample size)
*              This function supports any number of arguments STDEVP(number1, [number2], [number3], ...)
*    {lang=ja} 母集団の標準偏差を返します (小さいサンプル サイズのバイアス標準偏差)
*              この関数は、任意の値の引数 STDEVP をサポートしています (数値 1, 数値 2, 数値 3, ...)
*    {lang=zh} 基于整个总体计算标准差 (小样本规模的无偏标准差)
*              这个函数支持任意数量的参数 STDEVP(number1, [number2], [number3], ...)
*    {lang=hant} 基於整個總體計算標準差 (小樣本規模的無偏標準差)
*              這個函數支持任意數量的參數 STDEVP(number1, [number2], [number3], ...)
*    {lang=ko} 전체 모집단을 기준으로 표준편차를 계산(표본 크기가 작은 경우 편의가 있는 표준편차)
*    {lang=ru} Calculates standard deviation based on the entire population (bias standard deviation for small sample size)
*              This function supports any number of arguments STDEVP(number1, [number2], [number3], ...)
*    {lang=de} Berechnet die Standardabweichung basierend auf der Gesamtpopulation (Tendez-Standardabweichung für kleine Stichprobengrößen)
*              This function supports any number of arguments STDEVP(number1, [number2], [number3], ...)
* @return {number}
* @example
*    =STDEVP(6.10,13.20,15.40)
*/
Stat["STDEVP"] = function(/* ... args*/) {
	let args = (arguments[0] instanceof Array) ? arguments[0] : Array.prototype.slice.call(arguments);
	let argLen = args.length;
	if(argLen <= 0) { return NaN; }
	if(argLen === 1) { return 0; }

	let mean = Stat["AVERAGE"](args);

	let sum = 0;
	for(let i = argLen; --i >= 0;) {
		let diff = args[i] - mean;
		sum += diff * diff;
	}
	return Math.sqrt(sum / argLen);
};

/* harmony default export */ const Statistics = (Stat);


;// ./node_modules/tr-grid-util/es6/formula/functions/Text.js
/**
 * @namespace Text
 * @description
 *    {lang=en} Text manipulation functions
 *    {lang=ja} 文字列に関する関数
 *    {lang=zh} 文本控制函数
 *    {lang=ko} 텍스트 조작 함수
 *    {lang=ru} Text manipulation functions
 */
let Text = {};

let _toString = function(text) {
	if(!text && text !== 0) {
		return "";
	}

	return text + "";
};

/**
 * @function EXACT
 * @memberof Text
 * @description
 *   {lang=en} The EXACT function returns 1(TRUE) if text string 1 is same as text string 2. Otherwise returns 0(FALSE). Use operand = for a shorthand.
 *   {lang=ja} EXACT 関数は、2つの文字列を比較して同じ場合は 1(TRUE) を返します。異なる場合は 0(FALSE) を返します。オペランド [ = ] を使用します。
 *   {lang=zh} 如果文本字符串 1 与文本字符串 2 相同，则 EXACT 函数返回 1(TRUE)。否则返回 0(FALSE)。快速记录请使用操作符 =
 *   {lang=ko} EXACT 함수는 텍스트 문자열 1이 텍스트 문자열 2와 같으면 1(TRUE)을 반환합니다. 다르면 0(FALSE)을 반환합니다. 약칭으로 피연산자 =를 사용합니다.
 *   {lang=ru} The EXACT function returns 1(TRUE) if text string 1 is same as text string 2. Otherwise returns 0(FALSE). Use operand = for a shorthand.
 * @param  {string} text1
 *   {lang=en} The text string 1.
 *   {lang=ja} 文字列 1
 *   {lang=zh} 文本字符串 1。
 *   {lang=ko} 텍스트 문자열 1.
 *   {lang=ru} The text string 1.
 * @param  {string} text2
 *   {lang=en} The text string 2.
 *   {lang=ja} 文字列 2
 *   {lang=zh} 文本字符串 2。
 *   {lang=ko} 텍스트 문자열 2.
 *   {lang=ru} The text string 2.
 * @return {number}
 * @example
 *   =EXACT(CF_CURR,"USD")
 *   =CF_CURR="USD"
 */
Text["EXACT"] = function(text1, text2) {
	text1 = text1 + "";
	text2 = text2 + "";
	return (text1 === text2) ? 1 : 0;
};
/**
 * @function FIND
 * @memberof Text
 * @description
 *   {lang=en} The FIND function returns the position of a specified character or sub-string within a supplied text string.
 *             REMARK: The function is case-sensitive. The first character is counted as 0.
 *   {lang=ja} FIND 関数は、指定された文字列内で指定された文字の位置を返します。
 *             注記: この関数は、大文字と小文字を区別します。最初の文字は [0] としてカウントされます。
 *   {lang=zh} FIND 函数返回指定的字符或子字符串在所提供的文本字符串中的位置。
 *   {lang=ko} FIND 함수는 제공된 텍스트 문자열 내에서 지정된 문자 또는 하위 문자열의 위치를 반환합니다.
 *   {lang=ru} The FIND function returns the position of a specified character or sub-string within a supplied text string.
 *             注意: 该函数区分大小写。首个字符计为 0。
 * @param  {string} findText
 *   {lang=en} The character or sub-string that you wish to find.
 *   {lang=ja} 検索文字列
 *   {lang=zh} 您希望查找的字符或子字符串。
 *   {lang=ko} 찾으려는 문자나 하위 문자열.
 *   {lang=ru} The character or sub-string that you wish to find.
 * @param  {string} withinText
 *   {lang=en} The text string that is to be searched.
 *   {lang=ja} 検索する文字列
 *   {lang=zh} 要搜索的文本字符串。
 *   {lang=ko} 검색할 텍스트 문자열.
 *   {lang=ru} The text string that is to be searched.
 * @param  {number=} opt_startNum
 *   {lang=en} An optional argument that specifies the position in the within_text string, from which the search should begin.
 *   {lang=ja} 検索文字列内で検索を開始する位置を指定するオプショナル引数。
 *   {lang=zh} 可选参数，指定搜索开始于 within_text 字符串中的位置。
 *   {lang=ko} 검색을 시작해야 하는 within_text 문자열의 위치를 지정하는 선택적 인수.
 *   {lang=ru} An optional argument that specifies the position in the within_text string, from which the search should begin.
 * @return {number}
 * @example
 *    =FIND("r","surfthewave",5)
 */
Text["FIND"] = function(findText, withinText, opt_startNum) {
	findText = _toString(findText);
	if(!findText) { // Invalid findText
		return 0;
	}
	withinText = _toString(withinText);
	if(!withinText) { // Invalid base text
		return 0;
	}
	opt_startNum = +opt_startNum;
	if (!opt_startNum || opt_startNum < 0) {
		opt_startNum = 0;
	}
	let pos = withinText.indexOf(findText, opt_startNum);
	return pos;
};
/**
 * @function LEFT
 * @memberof Text
 * @description
 *   {lang=en} The LEFT function returns a specified number of characters from the start of a supplied text string.
 *             REMARK: If opt_numChars, (the number of returned characters), is more than the length of text string, return the whole string.
 *   {lang=ja} LEFT 関数は、指定された文字列の先頭から指定された数の文字を返します。
 *             注記: opt_numChars (返される文字数) がテキスト文字列の長さを超えている場合、文字列全体を返します。
 *   {lang=zh} LEFT 函数返回从所提供的文本字符串起始处的指定数量的字符。
 *             注意: 如果 opt_numChars, (返回的字符数), 多于文本字符串的长度，则返回整个字符串。
 *   {lang=hant} LEFT 函數返回從所提供的文本字符串起始處的指定數量的字符。
 *             注意: 如果 opt_numChars, (返回的字符數), 多於文本字符串的長度，則返回整個字符串。
 *   {lang=ko} LEFT 함수는 제공된 텍스트 문자열의 시작 부분에서 지정된 수의 문자를 반환합니다.
 *   {lang=ru} The LEFT function returns a specified number of characters from the start of a supplied text string.
 *   {lang=de} Die Funktion LINKS gibt eine bestimmte Anzahl von Zeichen ab dem Beginn einer bereitgestellten Textzeichenfolge zurück.
 *             REMARK: If opt_numChars, (the number of returned characters), is more than the length of text string, return the whole string.
 * @param {string} text
 *   {lang=en} The original text string.
 *   {lang=ja} 元の文字列
 *   {lang=zh} 初始文本字符串。
 *   {lang=hant} 初始文本字符串。
 *   {lang=ko} 원본 텍스트 문자열.
 *   {lang=ru} The original text string.
 *   {lang=de} Die ursprüngliche Textzeichenfolge.
 * @param {number} opt_numChars
 *   {lang=en} An optional argument that specifies the number of characters to be returned from the start of the supplied text.
 *   {lang=ja} 指定された文字列の先頭から指定された文字数を返すオプショナル引数です。
 *   {lang=zh} 可选参数，指定从所提供的文本起始处要返回的字符数。
 *   {lang=hant} 可選參數，指定從所提供的文本起始處要返回的字符數。
 *   {lang=ko} 제공된 텍스트의 시작 부분에서 반환할 문자 수를 지정하는 선택적 인수.
 *   {lang=ru} An optional argument that specifies the number of characters to be returned from the start of the supplied text.
 *   {lang=de} Ein optionales Argument, das die Anzahl der Zeichen spezifiziert, die vom Anfang des bereitgestellten Textes zurückgegeben werden sollen.
 * @return {string}
 * @example
 *    =LEFT("Surf the Wave",4)
 */
Text["LEFT"] = function(text, opt_numChars) {
	opt_numChars = +opt_numChars;
	if(!opt_numChars) {
		opt_numChars = 0;
	}
	if(opt_numChars < 0) {
		opt_numChars = 1;
	}
	return _toString(text).substr(0, opt_numChars);
};
/**
 * @function LENGTH
 * @memberof Text
 * @description
 *   {lang=en} The LENGTH function returns a number of characters of a text string.
 *   {lang=ja} LENGTH 関数は、文字列の文字数を返します。
 *   {lang=zh} LENGTH 函数返回文本字符串的一定数量的字符。
 *   {lang=hant} LENGTH 函數返回文本字符串的一定數量的字符。
 *   {lang=ko} LENGTH 함수는 텍스트 문자열의 문자 수를 반환합니다.
 *   {lang=ru} The LENGTH function returns a number of characters of a text string.
 *   {lang=de} Die Funktion LÄNGE gibt eine Anzahl von Zeichen einer Textzeichenfolge zurück.
 * @param  {string} text
 *   {lang=en} Text string.
 *   {lang=ja} 文字列
 *   {lang=zh} 文本字符串。
 *   {lang=hant} 文本字符串。
 *   {lang=ko} 텍스트 문자열.
 *   {lang=ru} Text string.
 *   {lang=de} Textzeichenfolge.
 * @return {number}
 * @example
 *    =LENGTH("Markets")
 */
Text["LENGTH"] = function(text) {
	return _toString(text).length;
};
/**
 * @function LEN
 * @memberof Text
 * @description
 *   {lang=en} The LEN function returns a number of characters of a text string.
 *   {lang=ja} LEN 関数は、文字列の文字数を返します。
 *   {lang=zh} LEN 函数返回文本字符串的一定数量的字符。
 *   {lang=hant} LEN 函數返回文本字符串的一定數量的字符。
 *   {lang=ko} LEN 함수는 텍스트 문자열의 문자 수를 반환합니다.
 *   {lang=ru} The LEN function returns a number of characters of a text string.
 *   {lang=de} Die Funktion LEN gibt eine Anzahl von Zeichen einer Textzeichenfolge zurück.
 * @param  {string} text
 *   {lang=en} Text string.
 *   {lang=ja} 文字列
 *   {lang=zh} 文本字符串。
 *   {lang=hant} 文本字符串。
 *   {lang=ko} 텍스트 문자열.
 *   {lang=ru} Text string.
 *   {lang=de} Textzeichenfolge.
 * @return {number}
 * @example
 *    =LEN("Markets")
 */
Text["LEN"] = Text["LENGTH"];

/**
 * @function LOWER
 * @memberof Text
 * @description
 *   {lang=en} The LOWER function converts all uppercase letters in text string to lowercase.
 *   {lang=ja} LOWER 関数は、文字列のすべての文字を小文字に変換します。
 *   {lang=zh} LOWER 函数将文本字符串中的全部大写字母转换成小写。
 *   {lang=hant} LOWER 函數將文本字符串中的全部大寫字母轉換成小寫。
 *   {lang=ko} LOWER 함수는 텍스트 문자열의 모든 대문자를 소문자로 변환합니다.
 *   {lang=ru} The LOWER function converts all uppercase letters in text string to lowercase.
 *   {lang=de} Die Funktion KLEINBUCHSTABEN konvertiert alle Großbuchstaben in der Textzeichenfolge in Kleinbuchstaben.
 * @param  {string} text
 *   {lang=en} Text string.
 *   {lang=ja} 文字列
 *   {lang=zh} 文本字符串。
 *   {lang=hant} 文本字符串。
 *   {lang=ko} 텍스트 문자열.
 *   {lang=ru} Text string.
 *   {lang=de} Textzeichenfolge.
 * @return {string}
 * @example
 *    =LOWER("HELLO WORLD")
 */
Text["LOWER"] = function(text) {
	return _toString(text).toLowerCase();
};

/**
 * @function MID
 * @memberof Text
 * @description
 *   {lang=en} The MID function returns a specified number of characters from the middle of a supplied text string.
 *             REMARK: The first character is counted as 0.
 *   {lang=ja} MID 関数は、文字列の指定された位置から指定された数の文字を返します。
 *             注記: 最初の文字は [0] としてカウントされます。
 *   {lang=zh} MID 函数从所提供的文本字符串的中间返回指定数量的字符。
  *             注意: 首字符计为 0。
 *   {lang=hant} MID 函數從所提供的文本字符串的中間返回指定數量的字符。
 *             注意: 首字符計為 0。
 *   {lang=ko} MID 함수는 제공된 텍스트 문자열의 중간 부분에서 지정된 수의 문자를 반환합니다.
 *   {lang=ru} The MID function returns a specified number of characters from the middle of a supplied text string.
 *             REMARK: The first character is counted as 0.
 *   {lang=de} Die Funktion MITTE gibt eine bestimmte Anzahl von Zeichen aus der Mitte einer bereitgestellten Textzeichenfolge zurück.
 *             REMARK: The first character is counted as 0.
 * @param  {string} text
 *   {lang=en} The original text string.
 *   {lang=ja} 元の文字列
 *   {lang=zh} 初始文本字符串。
 *   {lang=hant} 初始文本字符串。
 *   {lang=ko} 원본 텍스트 문자열.
 *   {lang=ru} The original text string.
 *   {lang=de} Die ursprüngliche Textzeichenfolge.
 * @param  {number} startNum
 *   {lang=en} An integer that specifies the position of the first character that you want to be returned.
 *   {lang=ja} 返される最初の文字の位置を指定する整数。
 *   {lang=zh} 一个整数，指定您想要返回的首字符的位置。
 *   {lang=hant} 一個整數，指定您想要返回的首字符的位置。
 *   {lang=ko} 반환할 첫 번째 문자의 위치를 지정하는 정수입니다.
 *   {lang=ru} An integer that specifies the position of the first character that you want to be returned.
 *   {lang=de} Eine Ganzzahl, die die Position des ersten Zeichens angibt, das zurückgegeben werden soll.
 * @param  {number} numChars
 *   {lang=en} An integer that specifies the number of characters (beginning with start_num), to be returned from the supplied text.
 *   {lang=ja} 指定された文字列から返される文字数を指定する整数。
 *   {lang=zh} 一个整数，指定从所提供的文本要返回的字符数 (以 start_num 开头)。
 *   {lang=hant} 一個整數，指定從所提供的文本要返回的字符數 (以 start_num 開頭)。
 *   {lang=ko} 제공된 텍스트에서 반환할 문자 수(start_num으로 시작하는)를 지정하는 정수입니다.
 *   {lang=ru} An integer that specifies the number of characters (beginning with start_num), to be returned from the supplied text.
 *   {lang=de} Eine Ganzzahl, die die Anzahl der Zeichen (beginnend mit start_num) spezifiziert, die vom bereitgestellten Text zurückgegeben werden soll.
 * @return {string}
 * @example
 *    =MID("antidisestablishmentarianism",7,13)
 */
Text["MID"] = function(text, startNum, numChars) {
	return _toString(text).substr(startNum, numChars);
};
/**
 * @function REPLACE
 * @memberof Text
 * @description
 *   {lang=en} The REPLACE function replaces all or part of a text string with another string.
 *   {lang=ja} REPLACE 関数は、指定された数の文字を新しい文字で置き換えた文字列を返します。
 *   {lang=zh} REPLACE 函数将全部或部分文本字符串替代为另一字符串。
 *   {lang=hant} REPLACE 函數將全部或部分文本字符串替代為另一字符串。
 *   {lang=ko} REPLACE 함수는 텍스트 문자열 전부 또는 일부를 다른 문자열로 바꿉니다.
 *   {lang=ru} The REPLACE function replaces all or part of a text string with another string.
 *   {lang=de} Die Funktion ERSETZEN tauscht eine Textzeichenfolge ganz oder teilweise mit einer anderen Zeichenfolge aus.
 * @param  {string} oldText
 *   {lang=en} The original text string, that you want to replace a part of.
 *   {lang=ja} 置き換えをする元の文字列。
 *   {lang=zh} 您想要部分替代的初始文本字符串。
 *   {lang=hant} 您想要部分替代的初始文本字符串。
 *   {lang=ko} 일부를 바꾸려는 원본 텍스트 문자열.
 *   {lang=ru} The original text string, that you want to replace a part of.
 *   {lang=de} Die ursprüngliche Textzeichenfolge, von der Sie einen Teil ersetzen möchten.
 * @param  {number} numChars
 *   {lang=en} The number of characters to replace.
 *   {lang=ja} 置き換えをする文字数。
 *   {lang=zh} 要替代的字符数。
 *   {lang=hant} 要替代的字符數。
 *   {lang=ko} 바꿀 문자 수.
 *   {lang=ru} The number of characters to replace.
 *   {lang=de} Die Anzahl der zu ersetzenden Zeichen.
 * @param  {number} startNum
 *   {lang=en} The position, within old_text, of the first character that you want to replace.
 *   {lang=ja} 置き換えをする文字列中で、置き換えを開始する文字の位置。
 *   {lang=zh} 您想要替代的旧文本中首字符的位置。
 *   {lang=hant} 您想要替代的舊文本中首字符的位置。
 *   {lang=ko} old_text 내에서 바꾸려는 첫 번째 문자의 위치.
 *   {lang=ru} The position, within old_text, of the first character that you want to replace.
 *   {lang=de} Die Position, innerhalb von old_text, des ersten Zeichens, das Sie ersetzen möchten.
 * @param  {string} newText
 *   {lang=en} The replacement text.
 *   {lang=ja} 置き換える文字列。
 *   {lang=zh} 替代文本。
 *   {lang=hant} 替代文本。
 *   {lang=ko} 대체 텍스트.
 *   {lang=ru} The replacement text.
 *   {lang=de} Der Ersatztext.
 * @return {string}
 * @example
 *    =REPLACE("Rollerblading",7,6,"skates")
 */
Text["REPLACE"] = function(oldText, numChars, startNum, newText) {
	newText = newText + "";
	oldText = _toString(oldText);
	let arrOldText = oldText.split("");
	arrOldText.splice(startNum, numChars, newText);
	return arrOldText.join("");
};
/**
 * @function RIGHT
 * @memberof Text
 * @description
 *   {lang=en} The RIGHT function returns a specified number of characters from the end of a supplied text string.
 *   {lang=ja} RIGHT 関数は、指定された文字列の末尾から指定された文字数を返します。
 *   {lang=zh} RIGHT 函数从所提供的文本字符串末尾返回指定数量的字符。
 *   {lang=hant} RIGHT 函數從所提供的文本字符串末尾返回指定數量的字符。
 *   {lang=ko} RIGHT 함수는 제공된 텍스트 문자열의 끝부분에서 지정된 수의 문자를 반환합니다.
 *   {lang=ru} The RIGHT function returns a specified number of characters from the end of a supplied text string.
 *   {lang=de} Die Funktion RECHTS gibt eine bestimmte Anzahl von Zeichen vom Ende einer bereitgestellten Textzeichenfolge zurück.
 * @param  {string} text
 *   {lang=en} The original text string.
 *   {lang=ja} 元の文字列。
 *   {lang=zh} 初始文本字符串。
 *   {lang=hant} 初始文本字符串。
 *   {lang=ko} 원본 텍스트 문자열.
 *   {lang=ru} The original text string.
 *   {lang=de} Die ursprüngliche Textzeichenfolge.
 * @param  {number} opt_numChars
 *   {lang=en} An optional argument that specifies the number of characters to be returned from the end of the supplied text.
 *   {lang=ja} 指定された文字列の末尾から指定された文字数を返すオプショナル引数です。
 *   {lang=zh} 可选参数，指定从所提供文本的末尾返回的字符数。
 *   {lang=hant} 可選參數，指定從所提供文本的末尾返回的字符數。
 *   {lang=ko} 제공된 텍스트의 끝부분에서 반환할 문자 수를 지정하는 선택적 인수.
 *   {lang=ru} An optional argument that specifies the number of characters to be returned from the end of the supplied text.
 *   {lang=de} Ein optionales Argument, das die Anzahl der Zeichen spezifiziert, die vom Ende des bereitgestellten Textes zurückgegeben werden sollen.
 * @return {string}
 * @example
 *    =RIGHT("Portishead",4)
 */
Text["RIGHT"] = function(text, opt_numChars) {
	text = _toString(text);
	opt_numChars = +opt_numChars;
	if(!opt_numChars) {
		opt_numChars = 0;
	}
	if(opt_numChars < 0) {
		opt_numChars = 1;
	}
	let startNum = text.length - opt_numChars;
	return text.substr(startNum, opt_numChars);
};
/**
 * @function TRIM
 * @memberof Text
 * @description
 *   {lang=en} The TRIM function removes leading, trailing, and consecutive spaces from a string.
 *   {lang=ja} TRIM 関数は、指定された文字列の余分な空白を削除します。
 *   {lang=zh} TRIM 函数从一个字符串中删除开头、结尾与连续的空格。
 *   {lang=hant} TRIM 函數從一個字符串中刪除開頭、結尾與連續的空格。
 *   {lang=ko} TRIM 함수는 문자열에서 앞과 뒤, 연속된 공백을 제거합니다.
 *   {lang=ru} The TRIM function removes leading, trailing, and consecutive spaces from a string.
 *   {lang=de} Die Funktion KÜRZEN entfernt führende, nachfolgende und aufeinanderfolgende Leerzeichen aus einer Zeichenfolge.
 * @param  {string} text
 *   {lang=en} Text string.
 *   {lang=ja} 文字列
 *   {lang=zh} 文本字符串。
 *   {lang=hant} 文本字符串。
 *   {lang=ko} 텍스트 문자열.
 *   {lang=ru} Text string.
 *   {lang=de} Textzeichenfolge.
 * @return {string}
 * @example
 *    =TRIM(" Information Management Systems ")
 */
Text["TRIM"] = function(text) {
	return _toString(text).trim();
};
/**
 * @function UPPER
 * @memberof Text
 * @description
 *   {lang=en} The UPPER function converts all lowercase letters in text string to uppercase.
 *   {lang=ja} UPPER 関数は、文字列のすべての小文字を大文字に変換します。
 *   {lang=zh} UPPER 函数将文本字符串中的全部小写字母转换成大写。
 *   {lang=hant} UPPER 函數將文本字符串中的全部小寫字母轉換成大寫。
 *   {lang=ko} UPPER 함수는 텍스트 문자열의 모든 소문자를 대문자로 변환합니다.
 *   {lang=ru} The UPPER function converts all lowercase letters in text string to uppercase.
 *   {lang=de} Die Funktion GROSSBUCHSTABEN konvertiert alle Kleinbuchstaben in der Textzeichenfolge in Großbuchstaben.
 * @param  {string} text
 *   {lang=en} Text string.
 *   {lang=ja} 文字列
 *   {lang=zh} 文本字符串。
 *   {lang=hant} 文本字符串。
 *   {lang=ko} 텍스트 문자열.
 *   {lang=ru} Text string.
 *   {lang=de} Textzeichenfolge.
 * @return {string}
 * @example
 *    =UPPER("Hello World")
 */
Text["UPPER"] = function(text) {
	return _toString(text).toUpperCase();
};
/**
 * @function VALUE
 * @memberof Text
 * @description
 *   {lang=en} The VALUE function converts a string to a number.
 *   {lang=ja} VALUE 関数は、文字列を数値に変換します。
 *   {lang=zh} VALUE 函数将字符串转换成数字。
 *   {lang=hant} VALUE 函數將字符串轉換成數字。
 *   {lang=ko} VALUE 함수는 문자열을 숫자로 변환합니다.
 *   {lang=ru} The VALUE function converts a string to a number.
 *   {lang=de} Die Funktion WERT konvertiert eine Zeichenfolge in eine Zahl.
 * @param {string} text
 *   {lang=en} text
 *   {lang=ja} 文字列
 *   {lang=zh} 文本
 *   {lang=hant} 文本
 *   {lang=ko} 텍스트
 *   {lang=ru} text
 *   {lang=de} Text
 * @return {number}
 * @example
 *    =VALUE("987.1")
 */
Text["VALUE"] = function(text) {
	return +text;
};
/**
 * @function CHAR
 * @memberof Text
 * @description
 *   {lang=en} The CHAR function converts a number to an ASCII character.
 *   {lang=ja} CHAR 関数は、数値を ASCII 文字に変換します。
 *   {lang=zh} CHAR 函数将数字转换成 ASCII 字符。
 *   {lang=hant} CHAR 函數將數字轉換成 ASCII 字符。
 *   {lang=ko} CHAR 함수는 숫자를 ASCII 문자로 변환합니다.
 *   {lang=ru} The CHAR function converts a number to an ASCII character.
 *   {lang=de} Die Funktion CHAR konvertiert eine Zahl in ein ASCII-Zeichen.
 * @param {number} code
 *   {lang=en} character code
 *   {lang=ja} 文字コード
 *   {lang=zh} 字符代码
 *   {lang=hant} 字符代碼
 *   {lang=ko} 문자 코드
 *   {lang=ru} character code
 *   {lang=de} Zeichencode
 * @return {string}
 * @example
 *    =CHAR(1)
 */
Text["CHAR"] = function(code) {
	let num = +code;
	return String.fromCharCode(num);
};
/**
 * @function CLEAN
 * @memberof Text
 * @description
 *   {lang=en} The CLEAN function removes non-ASCII characters, (characters not in the range 32-127 inclusive), from the string.
 *   {lang=ja} CLEAN 関数は、ASCII 以外の文字 (32-127 の範囲にない文字) を文字列から削除します。
 *   {lang=zh} CLEAN 函数从字符串中删除非 ASCII 字符 (不在 32-127 范围中的字符)。
 *   {lang=hant} CLEAN 函數從字符串中刪除非 ASCII 字符 (不在 32-127 範圍中的字符)。
 *   {lang=ko} CLEAN 함수는 문자열에서 비 ASCII 문자(32-127 범위에 없는 문자)를 제거합니다.
 *   {lang=ru} The CLEAN function removes non-ASCII characters, (characters not in the range 32-127 inclusive), from the string.
 *   {lang=de} Die Funktion SÄUBERN entfernt Nicht-ASCII-Zeichen (Zeichen, die nicht im Bereich 32-127 einschließlich liegen) aus der Zeichenfolge.
 * @param {string} text
 *   {lang=en} Text
 *   {lang=ja} 文字列
 *   {lang=zh} 文本
 *   {lang=hant} 文本
 *   {lang=ko} 텍스트
 *   {lang=ru} Text
 *   {lang=de} Text
 * @return {string}
 * @example
 *    =CLEAN("Kobérña")
 */
Text["CLEAN"] = function(text) {
	let arrText = _toString(text).split("");
	for(let i = arrText.length; --i >= 0;) {
		let ascii = arrText[i].charCodeAt(0);
		if(ascii < 32 || ascii > 127) {
			arrText[i] = "";
		}
	}
	return arrText.join("");
};
/**
 * @function CODE
 * @memberof Text
 * @description
 *   {lang=en} The CODE function returns the ASCII code that corresponds to the first character in a string.
 *   {lang=ja} CODE 関数は、文字列の先頭文字の ASCII コードを返します。
 *   {lang=zh} CODE 函数返回对应字符串中首字符的 ASCII 代码。
 *   {lang=hant} CODE 函數返回對應字符串中首字符的 ASCII 代碼。
 *   {lang=ko} CODE 함수는 문자열의 첫 번째 문자에 해당하는 ASCII 코드를 반환합니다.
 *   {lang=ru} The CODE function returns the ASCII code that corresponds to the first character in a string.
 *   {lang=de} Die Funktion CODE gibt den ASCII-Code zurück, welcher dem ersten Zeichen in einer Zeichenfolge entspricht.
 * @param  {string} text
 *   {lang=en} Text
 *   {lang=ja} 文字列
 *   {lang=zh} 文本
 *   {lang=hant} 文本
 *   {lang=ko} 텍스트
 *   {lang=ru} Text
 *   {lang=de} Text
 * @return {number}
 * @example
 *    =CODE("A")
 */
Text["CODE"] = function(text) {
	return _toString(text).charCodeAt(0);
};
/**
 * @function PROPER
 * @memberof Text
 * @description
 *   {lang=en} The PROPER function converts the first letter of all words in sentence to uppercase, and converts all remaining letters to lowercase.
 *   {lang=ja} ROPER 関数は、文中の各単語の先頭文字を大文字に変換し、残りの文字をすべて小文字に変換します。
 *   {lang=zh} PROPER 函数将句子中全部单词的首字母转换成大写，其他字母全部保持小写。
 *   {lang=hant} PROPER 函數將句子中全部單詞的首字母轉換成大寫，其他字母全部保持小寫。
 *   {lang=ko} PROPER 함수는 문장의 모든 단어의 첫 글자를 대문자로 변환하고 나머지 문자를 모두 소문자로 변환합니다.
 *   {lang=ru} The PROPER function converts the first letter of all words in sentence to uppercase, and converts all remaining letters to lowercase.
 *   {lang=de} Die Funktion RICHTIG konvertiert den ersten Buchstaben aller Wörter im Satz in Großbuchstaben und alle verbleibenden Buchstaben in Kleinbuchstaben.
 * @param  {string} text
 *   {lang=en} Text
 *   {lang=ja} 文字列
 *   {lang=zh} 文本
 *   {lang=hant} 文本
 *   {lang=ko} 텍스트
 *   {lang=ru} Text
 *   {lang=de} Text
 * @return {string}
 * @example
 *    =PROPER("A STRANGE BLACK CAT ON YOUR PORCH BRINGS PROSPERITY.")
 */
Text["PROPER"] = function(text) {
	text = _toString(text);
	let arrTokens = text.split(/[\W_]/g);
	let arrSyms = text.match(/[\W_]/g);
	let arrLen = arrTokens.length;
	let result = "";
	for(let i = 0; i < arrLen; i++) {
		let token = arrTokens[i];
		let len = token.length;
		if(len === 1) {
			result += token.toUpperCase();
		} else if(len !== 0) {
			result += token.charAt(0).toUpperCase() + token.substr(1).toLowerCase();
		}

		let sym = arrSyms[i];
		if(sym) {
			result += sym;
		}
	}
	return result;
};
/**
 * @function REPEAT
 * @memberof Text
 * @description
 *   {lang=en} The REPEAT function duplicates the text string in specific times.
 *   {lang=ja} REPEAT 関数は、特定の時刻にテキスト文字を複製します。
 *   {lang=zh} REPEAT 函数将文本字符串重复指定次数。
 *   {lang=hant} REPEAT 函數將文本字符串重複指定次數。
 *   {lang=ko} REPEAT 함수는 텍스트 문자열을 지정된 횟수만큼 복제합니다.
 *   {lang=ru} The REPEAT function duplicates the text string in specific times.
 *   {lang=de} Die Funktion WIEDERHOLEN dupliziert die Textzeichenfolge zu bestimmten Zeiten.
 * @param  {string} text
 *   {lang=en} Text
 *   {lang=ja} 文字列
 *   {lang=zh} 文本
 *   {lang=hant} 文本
 *   {lang=ko} 텍스트
 *   {lang=ru} Text
 *   {lang=de} Text
 * @param  {number} numRep
 *   {lang=en} Number of times
 *   {lang=ja} 回数
 *   {lang=zh} 次数
 *   {lang=hant} 次數
 *   {lang=ko} 횟수
 *   {lang=ru} Number of times
 *   {lang=de} Mehrfach
 * @return {string}
 * @example
 *    =REPEAT("Ho! ",3)
 */
Text["REPEAT"] = function(text, numRep) {
	numRep = +numRep;
	if(!numRep || numRep < 0) {
		return "";
	}

	text = _toString(text);
	if(text === "") {
		return "";
	}

	let newText = text;
	for(let i = 1; i < numRep; i++) {
		newText += text;
	}
	return newText;
};
/**
 * @function STRING
 * @memberof Text
 * @description
 *   {lang=en} The STRING function rounds the number to a specified precision, and then converts it into a string.
 *   {lang=ja} STRING 関数は、数値を指定された少数に処理した後、文字列に返します。
 *   {lang=zh} STRING 函数将数字四舍五入到指定精度，然后将其转换成字符串。
 *   {lang=hant} STRING 函數將數字四捨五入到指定精度，然後將其轉換成字符串。
 *   {lang=ko} STRING 함수는 숫자를 지정된 정밀도로 반올림한 다음 문자열로 변환합니다.
 *   {lang=ru} The STRING function rounds the number to a specified precision, and then converts it into a string.
 *   {lang=de} Die Funktion STRING rundet die Zahl auf eine bestimmte Genauigkeit und konvertiert sie dann in eine Zeichenfolge.
 * @param {number} num1
 *   {lang=en} Number 1
 *   {lang=ja} 数値 1
 *   {lang=zh} 数字 1
 *   {lang=hant} 數字 1
 *   {lang=ko} 숫자 1
 *   {lang=ru} Number 1
 *   {lang=de} Nummer 1
 * @param {number} num2
 *   {lang=en} Number 2
 *   {lang=ja} 数値 2
 *   {lang=zh} 数字 2
 *   {lang=hant} 數字 2
 *   {lang=ko} 숫자 2
 *   {lang=ru} Number 2
 *   {lang=de} Nummer 2
 * @return {string}
 * @example
 *    =STRING(12.129,2)
 */
Text["STRING"] = function(num1, num2) {
	num1 = +num1;
	num2 = +num2;
	if(!num1) {
		return "";
	}
	if(!num2 || num2 < 0) {
		num2 = 0;
	} else if(num2 > 9) {
		num2 = 9;
	}

	return (num1).toFixed(num2);
};

/* harmony default export */ const functions_Text = (Text);


;// ./node_modules/tr-grid-util/es6/formula/Engine.js








// import Excel from "./functions/Excel.js";
// import AdFin from "./functions/AdFin.js";


/** @constructor
* @extends {EventDispatcher}
*/
let Engine = function() {
	this._context = {};
	Engine._addContext(this._context, Engine.getContext()); // Clone context for supporting more than one Engine in App
	this._context["_DATA"] = this._getData.bind(this);  // _DATA is an internal context used for getting cached data
	this._setData = this._defaultFormulaDataSetter.bind(this); // Default setter for adding the result into rowData
	this._formulas = [];
	this._formulaMap = {};
	this._referenceMap = {};
	// this.addEventListener("dataChanged");
	// this.addEventListener("dataRequired");
	// this.addEventListener("formulaChanged");
};
es6_Ext.inherits(Engine, es6_EventDispatcher);
/** @private
* @type {Object.<string, *>}
*/
Engine.prototype._context = null;
/** @private
* @type {Object.<string, *>}
*/
Engine.prototype._rowData = null;
/** @private
* @type {?Function}
*/
Engine.prototype._setData = null;
/** @private
* @type {Array.<Formula>}
*/
Engine.prototype._formulas = null;
/** @private
* @type {Object.<string, Formula>}
*/
Engine.prototype._formulaMap = null;

/** @private
* @type {Object.<string, string>}
*/
Engine.prototype._referenceMap = null;
/** @private
* @type {Object.<string, *>}
*/
Engine._context = null; // Static variable for all instances


/** @public
* @function
* @return {!Object.<string, *>}
*/
Engine.getContext = function() {
	let context = Engine._context;
	if(!context) {
		context = {};
		Engine._context = context;

		// Engine._addContext(context, AdFin);
		Engine._addContext(context, es6_DateTime);
		// Engine._addContext(context, Excel);
		Engine._addContext(context, functions_Info);
		Engine._addContext(context, functions_Logic);
		Engine._addContext(context, functions_Math);
		Engine._addContext(context, Statistics);
		Engine._addContext(context, functions_Text);
	}
	return context;
};
/** @public
* @param {Object.<string, Function>} funcs
*/
Engine.addContext = function(funcs) {
	let context = Engine.getContext();
	Engine._addContext(context, funcs);
};
/** @private
* @param {Object} context
* @param {Object.<string, Function>} funcs
*/
Engine._addContext = function(context, funcs) {
	for(let key in funcs) {
		context[key] = funcs[key];
	}
};

/** @public
* @param {Function} func
*/
Engine.prototype.setFormulaDataGetter = function(func) {
	if(typeof func !== "function") {
		return;
	}
	this._context["_DATA"] = func;
};
/** @public
* @param {Function} func
*/
Engine.prototype.setFormulaDataSetter = function(func) {
	if(typeof func !== "function") {
		return;
	}
	this._setData = func;
};
/** @private
* @param {Formula} formula
* @param {*} result
* @param {Object.<string, *>} rowData
* @param {Object.<string, *>} opt_changes
*/
Engine.prototype._defaultFormulaDataSetter = function(formula, result, rowData, opt_changes) {
	if(result !== "...") {			// Data update is pending - do not erase previous value
		let alias = formula.getAlias();
		rowData[alias] = result;
		if (opt_changes) {
			opt_changes[alias] = result;
		}
	}
};

/** @public
* @param {Formula|string} userInput
* @return {!Formula}
*/
Engine.prototype.addFormula = function(userInput) {
	let f;
	if(userInput instanceof formula_Formula){
		f = userInput;
	} else {
		this._rowData = {}; // Formula will be tested in the initialization process. So we need to mock the cached data.
		f = new formula_Formula(/** @type{string}*/(userInput), this._context);
		this._rowData = null;
	}

	f.addEventListener("dataCacheChanged", this._dispatch.bind(this, "dataChanged"));
	f.addEventListener("dataRequired", this._dispatch.bind(this, "dataRequired"));
	f.addEventListener("formulaChanged", this._dispatch.bind(this, "formulaChanged"));
	this._formulas.push(f); // The one in the back will be executed last
	return f;
};
/** @public
* @ignore
* @param {Formula|string} userInput
* @return {!Formula}
*/
Engine.prototype.mockFormula = function(userInput) {
	let f;
	if(userInput instanceof formula_Formula){
		f = new formula_Formula(userInput.getInput(), this._context);
	} else {
		f = new formula_Formula(/** @type{string}*/(userInput), this._context);
	}

	return f;
};
/** @public
* @param {Formula|string|number=} ref
* @return {Formula}
*/
Engine.prototype.removeFormula = function(ref) {
	if(ref instanceof formula_Formula) { // Convert Formula to index
		ref = this._formulas.indexOf(/** @type{Formula} */(ref));
	}

	let f;
	if(typeof ref === "number") {
		f = this._formulas[ref];
		if(f) {
			this._formulas.splice(ref, 1);
			if(f.getReference()) {
				delete this._formulaMap[f.getReference()];
				delete this._referenceMap[f.getReference()];
			}
			f.dispose();
		}
	} else if (typeof ref === "string") {
		f = this._formulaMap[ref];
		if(f) {
			let at = this._formulas.indexOf(f);
			if(at >= 0) {
				this._formulas.splice(at, 1);
				delete this._formulaMap[ref];
				delete this._referenceMap[ref];
			}
			f.dispose();
		}
	}

	return f || null;
};
/** @public
*/
Engine.prototype.removeAllFormulas = function() {
	let formulas = this._formulas;
	for(let i = formulas.length; --i >= 0;) {
		delete this._referenceMap[formulas[i].getReference()];
	}

	this._formulas.length = 0;
	this._formulaMap = {};
};

/** @public
* @param {string|number} opt_ref Index (number) or formula reference (string)
* @return {Formula}
*/
Engine.prototype.getFormula = function(opt_ref) {
	let f;
	if(typeof opt_ref === "number") {
		f = this._formulas[opt_ref];
	} else {
		f = this._formulaMap[opt_ref];
	}
	return f || null;
};
/** @public
* @param {string} exp E.g "CF_BID + CF_ASK"
* @return {Formula}
*/
Engine.prototype.getFormulaByExpression = function(exp) {
	let len = this._formulas.length;
	for(let i = 0; i < len; ++i) {
		let f = this._formulas[i];
		if(f.getInput() === exp) {
			return f;
		}
	}
	return null;
};
/** @public
* @return {Array.<Formula>}
*/
Engine.prototype.getAllFormulas = function() {
	return this._formulas;
};
/** @public
* @return {Object.<string, string>}
*/
Engine.prototype.getAllFormulaReferences = function() {
	return this._referenceMap;
};
/** @public
* @return {number}
*/
Engine.prototype.getFormulaCount = function() {
	return this._formulas.length;
};
/** @private
 * @param {string} ref
 * @return {string}
 */
Engine.prototype._generateUniqueReference = function(ref) {
	let uniRef = ref;
	let index = 0;
	while(this._referenceMap[uniRef] != null) {
		uniRef = ref + "_" + ++index;
	}
	return uniRef;
};
/** @public
 * @param {string|Formula} src
 * @param {string} ref
 * @return {string}
 */
Engine.prototype.addReference = function(src, ref) {
	if (!src || !ref) {
		return "";
	}
	ref = this._generateUniqueReference(ref);
	let colName = "";
	if (src instanceof formula_Formula) {
		let res = this._setFormulaReference(src, ref);
		if (!res) {
			return "";
		}
		colName = src.getAlias();
	} else {
		colName = /** @type{string} */(src);
	}
	this._referenceMap[ref] = colName;
	return ref;
};
/** @public
 * @param {string} oldRef
 * @param {string} newRef
 * @return {boolean}
 */
Engine.prototype.renameReference = function(oldRef, newRef) {
	if (!newRef) {
		return this.removeReference(oldRef);
	}
	if (!oldRef || this._referenceMap[oldRef] == null || oldRef === newRef) {
		return false;
	}
	let f = this._formulaMap[oldRef];
	if (f && !this.isValidReference(f, newRef)) {
		return false;
	}

	// Change the reference being used in all formulas
	for (let i = this._formulas.length; --i >= 0;) {
		let formula = this._formulas[i];
		if (formula.changeField(oldRef, newRef)) {
			let ref = formula.getReference();
			if (ref) {
				this._referenceMap[ref] = formula.getAlias();
			}
		}
	}

	if (f) {
		this._setFormulaReference(f, newRef);
	}

	this._referenceMap[newRef] = this._referenceMap[oldRef];
	delete this._referenceMap[oldRef];
	return true;
};
/** @public
 * @param {string} ref
 * @return {boolean}
 */
Engine.prototype.removeReference = function(ref) {
	if (!ref || this._referenceMap[ref] == null) {
		return false;
	}
	let f = this._formulaMap[ref];
	if (f != null) {
		f._setReference("");
		delete this._formulaMap[ref];
	}
	delete this._referenceMap[ref];
	return true;
};
/** @public
 * @param {string} refName
 * @return {string}
 */
Engine.prototype.resolveReference = function(refName) {
	return this._referenceMap[refName] || refName;
};

/** Get value from cached row data
 *  @private
 * @param {string} refName
 * @param {Object.<string, *>} rowData WARNING: This param will not be used!
 * @return {*}
 */
Engine.prototype._getData = function(refName, rowData) {
	//  TODO: Assign default value based on refName in case of non existing data
	if (!this._rowData) {
		return null;
	}
	let value = this._rowData[this.resolveReference(refName)];

	if (value) {
		let num = Number(value);

		if (num === num) { // Convert text to number for formula calculation
			value = num;
		}
	}

	return value;
};
/** @private
* @param {Formula} formula
* @param {string} ref
* @return {boolean}
*/
Engine.prototype._setFormulaReference = function(formula, ref) {
	if(this._formulaMap[ref]) {
		console.warn("Duplicate formula reference detected.");
		return false;
	}
	let prevRef = formula.getReference();
	if(prevRef === ref) {
		return false;
	}

	if(ref) { // Empty string is no need to be in formula map
		this._formulaMap[ref] = formula;
	}
	formula._setReference(ref);
	let dependencies = this._sortDependencies(this._formulas, this._formulaMap);
	if (!dependencies) { // Circular dependencies detected
		formula._setReference(prevRef);
		delete this._formulaMap[ref];
		return false;
	}

	// Conclude
	this._formulas = dependencies;

	delete this._formulaMap[prevRef]; // Delete old reference of formula
	return true;
};
/** @public
* @param {Object.<string, *>} rowData
* @param {Object.<string, *>=} opt_changes
*/
Engine.prototype.calculate = function(rowData, opt_changes) {
	this._rowData = rowData; // Cache for resolving reference

	let len =  this._formulas.length;
	for(let i = 0; i < len; ++i) { // This must be done in normal order
		let f = this._formulas[i];
		// if opt_changes is undefined then recalculate all function
		// if opt_changes is exist then check formula need to recalculate
		if (opt_changes) {
			// Don't recalculate formula if fields are not changed and not update from realtime formula
			let needRecalculate = (opt_changes[f.getAlias()] != null); // Update from realtime formula
			if (!needRecalculate) {
				let fields = f.getFieldNames();
				for (let j = fields.length; --j >= 0;) {
					if (opt_changes[this.resolveReference(fields[j])] != null) {
						needRecalculate = true;
						break;
					}
				}
				if (!needRecalculate) {
					continue;
				}
			}
		}

		let result = f.calculate(rowData);
		this._setData(f, result, rowData, opt_changes);
	}

	this._rowData = null;
};

/**
 * @public
 * @param {Formula} formula
 * @param {string} ref
 * @return {boolean}
 */
Engine.prototype.isValidReference = function(formula, ref) {
	if(!ref) {
		return true;
	}
	if(this._referenceMap[ref]) {
		return false; // duplicate reference detected.
	}

	let prevRef = formula.getReference();
	let parentChildrenMap = this._createParentChildrenMap(this._formulas);

	// Replace the old reference with new reference
	if (prevRef) {
		delete parentChildrenMap[prevRef];
		for (let parent in parentChildrenMap) {
			let children = parentChildrenMap[parent];
			let index = children.indexOf(prevRef);
			if (index !== -1) {
				children = children.slice(0); // Clone. The orignal children is an array in formula object. We should not modify it.
				children.splice(index, 1);
				index = children.indexOf(ref);
				if (index === -1) {
					children.push(ref);
				}
				parentChildrenMap[parent] = children;
			}
		}
	}
	parentChildrenMap[ref] = formula.getFieldNames();

	return (!!this._sortParentChildrenMap(parentChildrenMap));
};

/** @private
 * @param {Array.<Formula>} formulas
 * @param {Object.<string, Formula>} formulaMap	Contains all initial formulas in a map form
 * @return {Array.<Formula>}
 */
Engine.prototype._sortDependencies = function(formulas, formulaMap) {
	let formulaCount = formulas.length;
	if (formulaCount === 1) {
		return formulas;
	}

	let parentChildrenMap = this._createParentChildrenMap(formulas);
	let sortedParents = this._sortParentChildrenMap(parentChildrenMap);
	if (!sortedParents) {
		return null;
	}

	let noDepFormulas = formulas.filter(function(f) {
		return (!f.getReference());
	});

	let results = [];
	if (formulaMap) {
		for(let i = sortedParents.length; --i >= 0;) {
			let f = formulaMap[sortedParents[i]];
			if(f) {
				results.push(f); // The one in the back will be executed last
			}
		}
		if(noDepFormulas.length > 0) {
			results = results.concat(noDepFormulas);
		}
	}
	return results;
};

/**
 * @private
 * @param {Array.<Formula>} formulas
 * @return {Object.<string, Array.<string>>}
 */
Engine.prototype._createParentChildrenMap = function(formulas) {
	let i, parentChildrenMap = {};
	for (i = formulas.length; --i >= 0;) {
		let f = formulas[i];
		let ref = f.getReference();
		if (ref) {
			parentChildrenMap[ref] = f.getFieldNames();
		}
	}
	return parentChildrenMap;
};

/** @description https://en.wikipedia.org/wiki/Topological_sorting
 * @private
 * @param {Object.<string, Array.<string>>} ParentChildrenMap
 * @return {Array.<string>}
 */
Engine.prototype._sortParentChildrenMap = function(ParentChildrenMap) {
	let edges = {};
	let i, child, children, childCount;
	let parent;
	for (parent in ParentChildrenMap) {
		children = ParentChildrenMap[parent];
		childCount = children.length;
		for(let j = 0; j < childCount; ++j) {
			child = children[j];
			edges[child] = (edges[child] + 1) || 1;
		}
	}

	// Collect nodes with no edge (root nodes)
	let rootNodes = [];
	for(parent in ParentChildrenMap) {
		if(!edges[parent]) {
			rootNodes.push(parent);
		}
	}

	// Navigate from root nodes down to all the leaf nodes
	let sortedParents = [];
	while(rootNodes.length > 0) {
		parent = rootNodes.shift();
		sortedParents.push(parent);

		children = ParentChildrenMap[parent];
		childCount = (children) ? children.length : 0;
		for(i = 0; i < childCount; ++i) {
			child = children[i];
			if(!(--edges[child] > 0)) {
				rootNodes.push(child); // This child has no more edge; and hence, it becomes root node
			}
		}
	}

	// Check for circular reference
	for(child in edges) {
		if(edges[child]) {
			return null;
		}
	}

	return sortedParents;
};

Engine._proto = Engine.prototype;

/* harmony default export */ const formula_Engine = ((/* unused pure expression or super */ null && (Engine)));


;// ./node_modules/@grid/core/es6/grid/util/util.js
/**
* @namespace
*/
let util_Util = {};

// TODO: move to tr-grid-util

/** @const
* @type {boolean}
*/
util_Util.isIE = (function () {
	let ua = window.navigator.userAgent;
	return (ua.indexOf('MSIE ') > 0) || (ua.indexOf('Trident/') > 0) || (ua.indexOf('Edge/') > 0);
}());
/** @const
* @type {boolean}
*/
util_Util.isTouchDevice = (function () {
	if ((navigator["maxTouchPoints"] && navigator["maxTouchPoints"] < 256) ||
		(navigator["msMaxTouchPoints"] && navigator["msMaxTouchPoints"] < 256)) {
		return true;
	}
	return false;
}());
/** @const
* @type {boolean}
*/
util_Util.isSafari = (function () {
	let rejectedExpression = /Chrome|Android|CriOS|FxiOS|EdgiOS/i;
	let safariExpression = /Safari/i;
	let agent = navigator.userAgent;
	if (rejectedExpression.test(agent)) {
		return false;
	}
	return safariExpression.test(agent);
}());

/** @public
 * @type {boolean}
 */
util_Util.isMobile = (function () {
	return /Android|webOS|iPhone|iPad|iPod|BlackBerry|IEMobile|Opera Mini/i.test(navigator.userAgent);
}());

/** Retrieve an element from Novakit control or any object that has "getElement" method.
* @public
* @ignore
* @param {*} obj If an HTMLElement is given, the same HTMLElement will be returned
* @return {Element} The return value is always element or null
*/
util_Util.getElement = function (obj) {
	if (util_Util.isElement(obj)) {
		return /** @type{Element} */(obj);
	} else {
		if (obj && util_Util.isFunction(obj["getElement"])) {
			return obj["getElement"]();
		}
	}
	return null;
};
/** Some browsers do not support HTMLElement, so we use nodeType to identify the type instead. Works on IE7
* @public
* @ignore
* @param {*} val
* @return {boolean}
*/
util_Util.isElement = function (val) {
	if (val) {
		return val.nodeType === 1;
	}
	return false;
};
/**
 * Returns `true` if the passed value is empty, `false` otherwise. The value is deemed to be empty if it is either:
 * - `null`
 * - `undefined`
 * - a zero-length array
 * - a zero-length string (Unless the `allowEmptyString` parameter is set to `true`)
 *
 * @param {*} value The value to test.
 * @param {boolean} [allowEmptyString=false] `true` to allow empty strings.
 * @return {boolean}
 */
util_Util.isEmpty = function (value, allowEmptyString) {
	return (value == null || typeof value === "undefined") ||
		(!allowEmptyString ? value === "" : false) ||
		(util_Util.isArray(value) && value.length === 0);
};
/**
* @public
* @ignore
* @param {*} val
* @return {boolean}
*/
util_Util.isNumber = function (val) {
	return typeof val === "number" && isFinite(val);
};
/**
* @public
* @ignore
* @param {*} val
* @return {boolean}
*/
util_Util.isString = function (val) {
	return typeof val === "string";
};
/** This version of isArray is faster than the one provided by goog.isArray. It works with both literal Array ([]) and Array class (new Array()) for Chrome
* WARNING: It does not work when called across iframe, since Array object is a different object in each iframe
* @public
* @ignore
* @param {*} val
* @return {boolean}
*/
util_Util.isArray = Array.isArray;
/** This version of isFunction is faster than the one provided by goog.isFunction. However It only works for Modern Browser and not IE
* @public
* @ignore
* @param {*} val
* @return {boolean}
*/
util_Util.isFunction = function (val) {
	return typeof val === "function";
};
/** Supports only for modern browser (Gecko 1.9+, modern Webkit)
* @public
* @ignore
* @param {!Element|Event|MouseEvent} target
* @param {*=} opt_retObj An object that stores returned values
* @return {!Object}
*/
util_Util.getClientPosition = function (target, opt_retObj) {
	let pos = opt_retObj || {};
	if (target["getBoundingClientRect"]) {
		let box = target["getBoundingClientRect"]();
		pos["x"] = box.left;
		pos["y"] = box.top;
	} else { // Event
		pos["x"] = target.clientX;
		pos["y"] = target.clientY;
	}
	return /**@type {!Object}*/(pos);
};
/** Top-left of B is used as registration a point (0, 0) for x and y.
 * For example, if A is located to right of B, "x" will be positive, If A is located below B, "y" will be positive.
* @public
* @ignore
* @param {!Element|Event|MouseEvent} A HTMLElement, Browser's event, or Browser's mouse event object
* @param {!Element|Event|MouseEvent} B HTMLElement, Browser's event, or Browser's mouse event object
* @param {*=} opt_retObj An object that stores returned values
* @return {!Object} an object with "x" and "y" values
*/
util_Util.getRelativePosition = function (A, B, opt_retObj) {
	opt_retObj = util_Util.getClientPosition(A, opt_retObj);
	let bp = util_Util.getClientPosition(B);

	opt_retObj["x"] -= bp["x"];
	opt_retObj["y"] -= bp["y"];
	return /**@type {!Object}*/(opt_retObj);
};

/** Move an item in the given array to the specified destination. If the destination equal or greater than array length, the array will be expanded
* @public
* @ignore
* @param {Array} ary
* @param {number} from INCLUSIVE
* @param {number} to INCLUSIVE
* @return {Array} ary
*/
util_Util.moveArrayItem = function (ary, from, to) {
	if (!ary) { return null; }

	let len = ary.length;
	if (from >= len && to >= len) {
		return /** @type{Array} */(ary); // No item needs to be moved
	}

	let item = ary.splice(from, 1)[0];
	if (to < len) {
		ary.splice(to, 0, item);
	} else {
		ary[to] = item;
	}
	return ary;
};
/** @public
* @function
* @param {Array} ary
* @param {Array} items
* @return {Array}
*/
util_Util.pushArrayItems = function(ary, items) {
	let len = items ? items.length : 0;
	if(len > 0) {
		if(len < 1e5) { // Maximum number of arguments that the spread operator can take is exactly 125052
			if(len > 1) {
				Array.prototype.push.apply(ary, items);
			} else {
				ary.push(items[0]);
			}
		} else {
			let start = 0;
			let end = 0;
			while(start < len) {
				end += 1e5;
				Array.prototype.push.apply(ary, items.slice(start, end));
				start = end;
			}
		}
	}
	return ary;
};
/** Insert multiple empty items to an array
* @public
* @function
* @param {Array} ary
* @param {number} at Zero or positive number
* @param {number} chunkSize
* @example
* Util.insertEmptyItems(["a", "b"], 1, 2); // ["a", , , "b"]
*/
util_Util.insertEmptyItems = function (ary, at, chunkSize) {
	if(chunkSize > 0) {
		if(chunkSize > 1) {
			let tailEnd = ary.slice(at);
			ary.length = at; // Remove the tail end part
			ary.length += chunkSize;
			util_Util.pushArrayItems(ary, tailEnd);
		} else {
			ary.splice(at, 0, null);
		}
	}
};
/** Move all items from the specify index to the front/top of the array
* @public
* @function
* @param {Array} ary
* @param {number} numItems Index of the first item to be moved to the top
* @return {Array}
* @example
* Util.cycleArray(["a", "b", "c"], 2); // ["c", "a", "b"]
*/
util_Util.cycleArray = function (ary, numItems) {
	if(numItems > 0 && ary) {
		if(numItems < ary.length) {
			let front = ary.splice(0, numItems);
			return ary.concat(front);
		}
	}
	return ary || null;
};

/** STL implementation of lowerBound() of the C++ vector class with no recursion
* @public
* @ignore
* @param {Array.<*>} ary The given array must already be sorted in ascending order
* @param {*} value the value to be searched
* @param {number=} left the begin index of the search range is INCLUSIVE
* @param {number=} right the end index of the search range is EXCLUSIVE
* @param {function(?, ?):boolean=} opt_lessFunc "Less than comparison functor"
* @return {number} Return the lowest index that gives equal or greater than the searched value
*/
util_Util.lowerBound = function (ary, value, left, right, opt_lessFunc) {
	if (left == null || left < 0) {
		left = 0;
	}
	if (right == null || right < 0 || right > ary.length) {
		right = ary.length;
	}
	if (left > right) {
		left = right;
	}
	if (opt_lessFunc == null) {
		opt_lessFunc = util_Util._defaultLessComparator;
	}

	let mid = 0;
	while (right > left) {
		mid = (left + right) >> 1;
		if (opt_lessFunc(ary[mid], value)) {
			left = mid + 1;
		} else {
			right = mid;
		}
	}
	return left; // May return ary.length
};

/** Upper bouund is the greatest index that has value that is always greater than the given value
* @public
* @ignore
* @param {Array.<*>} ary The given array must already be sorted in ascending order
* @param {*} value the value to be searched
* @param {number=} left the begin index of the search range is INCLUSIVE
* @param {number=} right the end index of the search range is EXCLUSIVE
* @param {function(?, ?):boolean=} opt_lessFunc "Less than comparison functor"
* @return {number} Return the highest index that gives equal or greater than the searched value
 */
util_Util.upperBound = function (ary, value, left, right, opt_lessFunc) {
	let at = util_Util.lowerBound(ary, value, left, right, opt_lessFunc);
	while (ary[at] === value) { ++at; }
	return at;
};

/** This function will replace the string by sequence arguments. (like String.format in JAVA)
 * @param {...string} format - First argument is string to format, the following arguments are the string to substitute
 * @example Util.stringFormat("There is a {0} in a {1}.", "box", "room"); // Result: "There is a box in a room."
 * @return {string}
 */
util_Util.stringFormat = function (format) {
	let args = Array.prototype.slice.call(arguments, 1);
	return format.replace(/{(\d+)}/g, function (match, number) {
		return (args[number] != null) ? args[number] : match;
	});
};
/**
* @private
* @param {*} left
* @param {*} right
* @return {boolean}
*/
util_Util._defaultLessComparator = function (left, right) {
	return left < right;
};
/**
* @public
* @function
* @ignore
 */
util_Util._deprecatedFunction = function () {
	console.warn("Calling deprecated method");
};

/**
 * @public
 * @function
 * @ignore
 * @param {number} lowVal
 * @param {number} lastVal
 * @param {number} highVal
 * @return {number}
 */
util_Util.calculatePercent = function (lowVal, lastVal, highVal) {
	if ((!lowVal && lowVal !== 0) ||
		(!lastVal && lastVal !== 0) ||
		(!highVal && highVal !== 0)) {
		return NaN;
	}
	if (highVal === lowVal) {
		return 50;
	}
	let percent = (lastVal - lowVal) / (highVal - lowVal);
	if (percent < 0) {
		return 0;
	}
	if (percent > 1) {
		return 100;
	}
	return percent * 100;
};

/** @description The method returns closest ancestor element or the element itself with specified class string. Otherwise, it returns null
 * @public
 * @function
 * @param {Element|Node|undefined} elem
 * @param {string} classStr The class string can only be one single class without spaces
 * @return {Element}
 */
util_Util.closestElement = function (elem, classStr) {
	let n = elem;
	while (n) {
		if(n.classList) { // DocumentFragment does not have classList property
			if (n.classList.contains(classStr)) {
				return /** @type{Element} */(n);
			}
		}
		n = n.parentNode || n.host; // ShadowRoot has host, but not parentNode
	}
	return null;
};
/** @public
* @function
* @ignore
* @param {Event} e
 */
util_Util._preventDefault = function (e) {
	e.preventDefault();
	e.stopPropagation();
};

/* harmony default export */ const util = (util_Util);


;// ./node_modules/@grid/core/es6/grid/event/EventListeners.js


/** @constructor
* @param {*|null=} opt_sender The "sender" reference added automatically to event argument when dispatching event
*/
let EventListeners = function (opt_sender) {
	this.dispatch = this.dispatch.bind(this);

	this._listeners = [];
	this.setSender(opt_sender);
};

//#region Public methods
/** @public
* @param {Function} listener should be already bound by the bind method
* @param {number=} opt_priority The higher the priority, the sooner the listener get executed. Undefined priority listener will be executed last
*/
EventListeners.prototype.listen = function (listener, opt_priority) {
	if (listener && util.isFunction(listener)) {
		if(this._listeners.indexOf(listener) < 0) {
			if(opt_priority == null) {
				this._listeners.push(listener);
			} else {
				listener._priority = opt_priority;
				let at = util.lowerBound(this._listeners, listener, 0, this._listeners.length, EventListeners.listenerComparer);
				this._listeners.splice(at, 0, listener);
			}
		}
	}
};
/** @public
* @param {Function} listener
*/
EventListeners.prototype.unlisten = function (listener) {
	if (listener) {
		let index = this._listeners.indexOf(listener);
		if (index >= 0) {
			this._listeners.splice(index, 1);
		}
	}
};
/** @public
*/
EventListeners.prototype.unlistenAll = function () {
	this._listeners.length = 0;
};

/** @public
* @return {number}
*/
EventListeners.prototype.getListenerCount = function () {
	return this._listeners.length;
};
/** @public
* @return {boolean}
*/
EventListeners.prototype.hasListener = function () {
	return this._listeners.length > 0;
};

/** @public
* @param {*} eventArg
*/
EventListeners.prototype.dispatch = function (eventArg) {
	if (!this._enabled || this._listeners.length <= 0) { return; }

	if (this._dispatching) {
		window.console.log("WARNING: There is an error in one of your binding methods.");
	}
	this._dispatching = true;

	if (eventArg == null) { eventArg = {}; }

	if (this._sender !== null) {
		eventArg["sender"] = this._sender;
	}

	if (this._extender !== null) {
		this._extender(/** @type{Object} */(eventArg));
	}

	for (let i = 0; i < this._listeners.length; i++) {
		this._listeners[i](eventArg);
	}

	if(this._sender !== null) {
		eventArg["sender"] = null; // Clear ref variable to prevent memory leak
	}
	this._dispatching = false;
};
/** @public
* @return {boolean}
*/
EventListeners.prototype.isDispatching = function () {
	return this._dispatching;
};

/** @public
* @return {boolean}
*/
EventListeners.prototype.isEnabled = function () {
	return this._enabled;
};
/** @public
* @param {boolean=} opt_enabled
*/
EventListeners.prototype.enable = function (opt_enabled) {
	this._enabled = (opt_enabled !== false);
};
/** @public
* @param {boolean=} opt_disabled
*/
EventListeners.prototype.disable = function (opt_disabled) {
	this._enabled = (opt_disabled === false);
};

/** Specifies "sender" reference to be added to the given event argument
* @public
* @param {*|null} sender
*/
EventListeners.prototype.setSender = function (sender) {
	this._sender = sender || null;
};
/** Specifies a handler to be executed before every dispatching
* @public
* @param {Function=} extender
*/
EventListeners.prototype.setEventArgExtender = function (extender) {
	this._extender = (extender != null) ? extender : null;
};
//#endregion Public methods

//#region Private
EventListeners.listenerComparer = function(a, b) {
	if(b._priority == null) { return true; }
	return a._priority >= b._priority; // Descending order
};

/** @private
 * @type {Array.<function(*)>}
 */
EventListeners.prototype._listeners = null;

/** @private
 * @type {*|null}
 */
EventListeners.prototype._sender = null;
/** @private
 * @type {function(Object)|null}
 */
EventListeners.prototype._extender = null;

/** @private
 * @type {boolean}
 */
EventListeners.prototype._enabled = true;
/** @private
 * @type {boolean}
 */
EventListeners.prototype._dispatching = false;
//#endregion  Private

EventListeners._proto = EventListeners.prototype; // To allow compression of prototype keyword

/* harmony default export */ const event_EventListeners = (EventListeners);


;// ./node_modules/@grid/core/es6/grid/event/EventDispatcher.js


/** Abstract base class that provides event management methods for derived class
 * @constructor
 */
let EventDispatcher_EventDispatcher = function () {
	/* throw("Cannot instantiate an abstract class"); */
	/*
	this._addEvent("customEvent"); // Add this line in your derived class, if you want to add custom event to your class object
	*/
};
/** @protected
* @ignore
* @type {Object.<string, !EventListeners>}
*/
EventDispatcher_EventDispatcher.prototype._customEvents = null;

//#region Public
/**
* @public
* @param {string} type Any event provided by derived class
* @param {Function} handler Callback method
* @param {number=} opt_priority The higher the priority, the sooner the listener get executed. Undefined priority listener will be executed last
* @example
* let grid = new Core();
* grid.listen("click", function(e) { console.log("Grid is clicked"); });
*/
EventDispatcher_EventDispatcher.prototype.listen = function (type, handler, opt_priority) {
	if (this._customEvents && this._customEvents[type]) {
		this._customEvents[type].listen(handler, opt_priority);
	}
};
/** Removes an event listener which was added with listen(). The same function handler must be sent in, or else it will not be removed
* @public
* @param {string} type The name of the event without the 'on' prefix.
* @param {Function} handler The listener function to remove.
*/
EventDispatcher_EventDispatcher.prototype.unlisten = function (type, handler) {
	if(this._customEvents && this._customEvents[type]) {
		this._customEvents[type].unlisten(handler);
	}
};
/** Removes all listeners from an object. If no type is specified, it will remove all listeners that have been registered. <br>
* You can also optionally remove listeners of a particular type.
* @public
* @param {string=} type Type of event to, default is all types.
*/
EventDispatcher_EventDispatcher.prototype.unlistenAll = function (type) {
	if(!type) {
		for (let key in this._customEvents) {
			this._customEvents[key].unlistenAll();
		}
	} else if(this._customEvents && this._customEvents[type]) {
		this._customEvents[type].unlistenAll();
	}
};
//#endregion Public

//#region Protected
/** Adds custom event
* @protected
* @ignore
* @param {string} type
*/
EventDispatcher_EventDispatcher.prototype._addEvent = function (type) {
	if(!type) { return; }

	if(!this._customEvents) {
		this._customEvents = {};
	}
	let e = this._customEvents[type];
	if(!e) {
		this._customEvents[type] = new event_EventListeners(this);
	}
};
/** Adds multiple events at once
* @protected
* @ignore
* @param {...string} types
*/
EventDispatcher_EventDispatcher.prototype._addEvents = function (types) {
	let ary = Array.isArray(types) ? types : arguments;
	let len = ary.length;
	for(let i = 0; i < len; ++i) {
		this._addEvent(ary[i]);
	}
};
/** Check if the custom event is disptaching as dispatching for custom event is syncronous operations. Native events will always return false. This is useful to stop infinite loop.
* @protected
* @ignore
* @param {string} type
* @return {boolean} Returns false if the event is not dispatching
*/
EventDispatcher_EventDispatcher.prototype._isEventDispatching = function (type) {
	let evt = this._customEvents[type];
	if(evt) {
		return evt.isDispatching();
	}
	return false;
};
/** Disable custom event from operating. This does NOT work with native events.
* @protected
* @ignore
* @param {string} type
* @param {boolean=} disabled
*/
EventDispatcher_EventDispatcher.prototype._disableEvent = function (type, disabled) {
	let evt = this._customEvents[type];
	if(evt) {
		evt.disable(disabled);
	}
};
/** This is used in conjunction with _dispatch to avoid creating event argument without actual listener
* @protected
* @ignore
* @param {string} type
* @return {boolean}
*/
EventDispatcher_EventDispatcher.prototype._hasListener = function (type) {
	if(this._customEvents && this._customEvents[type]) {
		return this._customEvents[type].hasListener();
	}
	return false;
};
/** Only custom event that is added by _addEvent() can be dispatched
* @protected
* @ignore
* @param {string} type
* @param {!Object} eventArg
*/
EventDispatcher_EventDispatcher.prototype._dispatch = function (type, eventArg) {
	this._customEvents[type].dispatch(eventArg); // Intentional no error checking. You must add custom event first
};

EventDispatcher_EventDispatcher._proto = EventDispatcher_EventDispatcher.prototype;

/* harmony default export */ const event_EventDispatcher = (EventDispatcher_EventDispatcher);


;// ./node_modules/@grid/core/es6/data/DataCache.js



/** @description Trigger when data within the table has been changed
 * @event DataCache#dataChanged
 * @property {boolean} globalChange Indicates a big change. User should expect all data has been change
 * @property {string} type Type of changes. Possible values are "inserted", "removed", "updated", and undefined
 * @property {string} rid Row Id
 * @property {Object.<string, *>} rowData Column values of the changed row in JSON object format
 */

/** @description Trigger before DataCache#dataChanged. Perform any data update during the event will NOT cause more dataChanged events
 * @event DataCache#dataComposed
 */

/** Class for caching data and firing event when data is changed
 * @constructor
 * @extends {EventDispatcher}
 */
let DataCache = function () {
	this._rows = {};

	this._addEvent("dataComposed");
	this._addEvent("dataChanged");
};
es6_Ext.inherits(DataCache, event_EventDispatcher);

/** @protected
 * @ignore
 * @type {!Object.<string, Object>}
 */
DataCache.prototype._rows;

/** @protected
 * @ignore
 * @type {boolean}
 */
DataCache.prototype._composing = false;

//#region Public
/**
 * @public
 */
DataCache.prototype.dispose = function () {
	this.unlistenAll();
	this.clearAllData(true);
};

/** Deprecated. Built-in Data Service is deprecated due to the lack of flexibility.
 * @public
 * @ignore
 */
DataCache.prototype.setSubscriptions = function () {};

/** Deprecated. Built-in Data Service is deprecated due to the lack of flexibility.
 * @public
 * @ignore
 * @return {Object} Always returns null
 */
DataCache.prototype.getSubscriptions = function () { return null; };

/** Deprecated. Built-in Data Service is deprecated due to the lack of flexibility.
 * @public
 * @ignore
 */
DataCache.prototype.addSubscription = function () {};

/** Deprecated. Built-in Data Service is deprecated due to the lack of flexibility.
 * @public
 * @ignore
 */
DataCache.prototype.removeSubscription = function () {};

/** Deprecated. Built-in Data Service is deprecated due to the lack of flexibility.
 * @public
 * @ignore
 */
DataCache.prototype.startAllSubscriptions = function () {};

/** Deprecated. Built-in Data Service is deprecated due to the lack of flexibility.
 * @public
 * @ignore
 */
DataCache.prototype.stopAllSubscriptions = function () {};

/** Deprecated. Built-in Data Service is deprecated due to the lack of flexibility.
 * @public
 * @ignore
 * @return {Object} Always returns null
 */
DataCache.prototype.getSubscription = function () { return null; };

/** Deprecated. Built-in Data Service is deprecated due to the lack of flexibility.
 * @public
 * @ignore
 * @return {string} Always returns empty string
 */
DataCache.prototype.getPrimaryRic = function () { return ""; };

/** @public
 * @param {boolean=} opt_suppressEvent
 * @fires DataCache#dataChanged
 */
DataCache.prototype.clearAllData = function (opt_suppressEvent) {
	this._rows = {};

	if (!opt_suppressEvent) {
		this._dispatchDataChange({"globalChange": true});
	}
};

/**
 * @public
 * @param {Array.<string>|string} colId
 * @param {boolean=} opt_suppressEvent
 * @fires DataCache#dataChanged
 */
DataCache.prototype.clearColumnData = function (colId, opt_suppressEvent) {
	let cids = (typeof colId === "string") ? [/** @type {string} */(colId)] : /** @type {Array.<string>} */(colId);

	for (let i = cids.length; --i >= 0;) {
		let cid = cids[i];

		if (cid) {
			for (let rid in this._rows) {
				delete this._rows[rid][cid];
			}
		}
	}

	if (!opt_suppressEvent) {
		this._dispatchDataChange({"globalChange": true});
	}
};

/**
 * @public
 * @param {string} rid
 * @param {string} cid
 * @return {*}
 */
DataCache.prototype.getData = function (rid, cid) {
	let row = this.getRowData(rid);

	return (row) ? row[cid] : null;
};

/** Deprecated. Built-in Data Service is deprecated due to the lack of flexibility.
 * @public
 * @ignore
 * @return {Object.<string, Array.<string>>} Always returns null
 */
DataCache.prototype.getAllRics = function () { return null; };

/**
 * @public
 * @return {Array.<string>}
 */
DataCache.prototype.getAllRowIds = function () {
	let rids = [];

	for (let rid in this._rows) {
		rids.push(rid);
	}

	return rids;
};

/**
 * @public
 * @param {string} rid ID of a row to be checked for its presence.
 * @return {boolean} `true` if row was found, `false` if row was not found.
 */
DataCache.prototype.hasRowId = function (rid) {
	return !!this._rows[rid];
};

/**
 * @public
 * @param {string} cid
 * @return {!Array.<*>}
 */
DataCache.prototype.getColumnData = function (cid) {
	let fieldRows = [];

	for (let rid in this._rows) {
		fieldRows.push(this._rows[rid][cid]);
	}

	return fieldRows;
};

/**
 * @public
 * @param {string} rid
 * @return {Object.<string, *>}
 */
DataCache.prototype.getRowData = function (rid) {
	return this._rows[rid] || null;
};

/**
 * @public
 * @param {Array.<string>} rids
 * @param {number=} opt_from
 * @param {number=} opt_to EXCLUSIVE
 * @return {!Array.<Object>}
 */
DataCache.prototype.getMultipleRowData = function (rids, opt_from, opt_to) {
	let len = rids.length;

	if (opt_to == null) {
		opt_to = len;
	}

	if (opt_to > len) {
		opt_to = len;
	}

	if (!(opt_from > 0)) {
		opt_from = 0;
	}

	let out_ary = new Array(len);
	let rowMap = this._rows;

	for (let i = opt_from; i < opt_to; ++i) {
		out_ary[i] = rowMap[rids[i]] || null;
	}

	return out_ary;
};

/** Return a new copy of row data
 * @public
 * @return {!Array.<Object.<string, *>>}
 */
DataCache.prototype.getAllRowData = function () {
	let rows = [];
	let rowMap = this._rows;

	for (let rid in rowMap) {
		rows.push(rowMap[rid]);
	}

	return rows;
};

/**
 * @public
 * @param {string} rid Row Id
 * @param {string} cid Column Id
 * @param {*} value
 * @return {boolean} Return true if there is any change, and false otherwise
 * @fires DataCache#dataChanged
 */
DataCache.prototype.setData = function (rid, cid, value) { // Data changed event is always dispatched
	let values = {};

	values[cid] = value;

	return this.setRowData(rid, values);
};

/**
 * @public
 * @ignore
 * @param {string} rid
 */
DataCache.prototype.hasDataCloudData = function (rid) {};

/**
 * @public
 * @param {string} rid Row Id
 * @param {Object.<string, *>=} values Object where the key is column Id and value is the data to be set
 * @param {Object=} opt_eventArg Additional arguments to be attached to the event fired
 * @return {boolean} Return true if there is any change, and false otherwise
 * @fires DataCache#dataChanged
 */
DataCache.prototype.setRowData = function (rid, values, opt_eventArg) { // Data changed event is always dispatched
	let row = this.getRowData(rid);
	let dirty = false;
	let changes = null;

	if (values) {
		if (!row) {
			row = values; // WARNING: Dangerous use of external reference. User may modify data without using DataCache's API

			this._rows[rid] = row;
			dirty = true;

		} else {
			for (let cid in values) {
				row[cid] = values[cid];
				dirty = true;
			}

			changes = values;
		}
	} else {
		if (row) {
			delete this._rows[rid];
			dirty = true;
		}
	}

	if (dirty && !this._composing) {
		let e = opt_eventArg || {};

		e["rid"] = rid;

		if (changes) {
			e["changes"] = changes;
		}

		e["rowData"] = row;

		this._dispatchDataChange(e);
	}

	return dirty;
};

/**
 * @public
 * @param {string} rid
 * @return {Object}
 */
DataCache.prototype.cloneRowData = function (rid) {
	let fields = this._rows[rid];

	if (!fields) {
		return null;
	}

	let values = {};

	for (let key in fields) {
		if (key.indexOf("FREE_TEXT") === 0) { // TODO: This is not generic and will cause unintentional error. Also, it posts some performance penalty
			continue; // Skip free text column
		}

		values[key] = fields[key];
	}

	return values;
};

/**
 * @protected
 * @ignore
 * @param {string} fromRid
 * @param {string} toRid
 * @return {boolean}
 */
DataCache.prototype._replaceRowId = function(fromRid, toRid) {
	let rows = this._rows;
	if(rows[fromRid]) {
		rows[toRid] = rows[fromRid];
		delete rows[fromRid];
		return true;
	}
	return false;
};
/**
 * @public
 * @ignore
 * @param {Object} ridPair
 */
DataCache.prototype.replaceRowIds = function(ridPair) {
	if(typeof ridPair === "object") {
		for(let oldRid in ridPair) {
			this._replaceRowId(oldRid, ridPair[oldRid]);
		}
	}
};

/** Deprecated. Built-in Data Service is deprecated due to the lack of flexibility.
 * @public
 * @ignore
 */
DataCache.prototype.addStaticFields = function () {};

/** Deprecated. Built-in Data Service is deprecated due to the lack of flexibility.
 * @public
 * @ignore
 */
DataCache.prototype.removeStaticFields = function () {};

/** Deprecated. Built-in Data Service is deprecated due to the lack of flexibility.
 * @public
 * @ignore
 */
DataCache.prototype.resetStaticFields = function () {};

/** Deprecated. Built-in Data Service is deprecated due to the lack of flexibility.
 * @public
 * @ignore
 */
DataCache.prototype.getStaticFields = function () {};

/**
 * Construct a JSON object that is compatible with console.table()
 * @public
 * @param {Object=} options
 * @return {!Array.<Object>} Return a JSON object that is compatible with console.table()
 * @example
 * let options = {
 *   "colNames": Array.<string>,
 *   "rowCount": number,
 *   "startIndex": number,
 *   "getter": function(Object, string):*,
 *   "withRowIds": boolean
 * };
 */
DataCache.prototype.dump = function (options) {
	return DataCache.constructTable(this.getAllRowData(), options, Object.keys(this._rows));
};

/**
 * Print current data and structure to console panel
 * @public
 * @param {Object=} opt_options
 * @example
 * let options = {
 *   "colNames": Array.<string>,
 *   "rowCount": number,
 *   "startIndex": number,
 *   "getter": function(Object, string):*,
 *   "withRowIds": boolean
 * };
 */
DataCache.prototype.log = function (opt_options) {
	console.table(this.dump(opt_options));
};

/** Deprecated. Built-in Data Service is deprecated due to the lack of flexibility.
 * @public
 * @ignore
 */
DataCache.prototype.setDataCloudSettings = function () {};

/** Deprecated. Built-in Data Service is deprecated due to the lack of flexibility.
 * @public
 * @ignore
 */
DataCache.prototype.getDataCloudFields = function () {};

/** Deprecated. Built-in Data Service is deprecated due to the lack of flexibility.
 * @public
 * @ignore
 */
DataCache.prototype.addDataCloudFields = function () {};

/** Deprecated. Built-in Data Service is deprecated due to the lack of flexibility.
 * @public
 * @ignore
 */
DataCache.prototype.removeDataCloudField = function () {};
//#endregion Public

/**
 * @protected
 * @ignore
 * @param {!Object} e
 * @fires DataTable#dataComposed
 * @fires DataTable#dataChanged
 */
DataCache.prototype._dispatchDataChange = function (e) {
	if (this._composing) {
		return;
	}

	this._composing = true;
	this._dispatch("dataComposed", e);
	this._composing = false;
	this._dispatch("dataChanged", e);
};

/**
 * @private
 * @param {string} propertyName
 * @param {Object} rowData
 * @param {string} colName
 * @return {*}
 */
DataCache._defaultPropertyGetter = function (propertyName, rowData, colName) {
	let d = rowData[colName];

	return d ? d[propertyName] : null;
};

/**
 * @private
 * @param {Object} rowData
 * @param {string} colName
 * @return {*}
 */
DataCache._defaultGetter = function (rowData, colName) {
	return rowData[colName];
};


/** Construct a JSON object that is compatible with console.table()
 * @public
 * @ignore
 * @param {!Array.<Object>} dataset
 * @param {Object=} opt_options { "rowCount": number,  "colNames": Array.<string>,  "getter": function(Object, string):*, startIndex: number, "withRowIds": boolean }
 * @param {Array.<string>=} opt_rowIds Optional row ids corresponding to the given dataset
 * @return {!Array.<Object>} Return a JSON object that is compatible with console.table()
 */
DataCache.constructTable = function (dataset, opt_options, opt_rowIds) {
	let startIndex = 0;
	let rLen = dataset.length;
	let rowLim = rLen;
	let colMap = null;
	let rids = opt_rowIds || null;
	let getter = DataCache._defaultGetter;
	let transformation = false;

	if (opt_options) {
		if (opt_options["rowCount"]) {
			rowLim = opt_options["rowCount"];
			transformation = true;
		}

		if (opt_options["colNames"]) {
			colMap = {};

			let len = opt_options["colNames"].length;

			for (let i = 0; i < len; ++i) {
				colMap[opt_options["colNames"][i]] = true;
			}

			transformation = true;
		}

		if (opt_options["getter"]) {
			if (typeof opt_options["getter"] == "string") {
				getter = DataCache._defaultPropertyGetter.bind(null, opt_options["getter"]);
			} else { // Function
				getter = opt_options["getter"];
			}

			transformation = true;
		}

		if (opt_options["startIndex"]) {
			startIndex = /** @type{number} */(opt_options["startIndex"]);
			transformation = true;
		}

		if (opt_options["withRowIds"] === false) {
			rids = null;
		}
	}

	if (!transformation && !rids) {
		return dataset; // The dataset is already compatible with console.table
	}

	let rows = [];

	for (let r = startIndex; r < rLen; ++r) {
		if (rows.length >= rowLim) {
			break;
		}

		let rowData = dataset[r];
		let columns = (rids) ? {"rid": rids[r]} : {};

		if (rowData) { // Header and footer don't have any row data
			let colName;

			if (colMap) {
				for (colName in colMap) {
					if (rowData[colName] != null) {
						columns[colName] = getter(rowData, colName);
					}
				}
			} else {
				for (colName in rowData) {
					columns[colName] = getter(rowData, colName);
				}
			}
		}

		rows.push(columns);
	}

	return rows;
};


/* harmony default export */ const data_DataCache = (DataCache);


;// ./node_modules/@grid/core/es6/data/Segment.js



/** @constructor
* @extends {EventDispatcher}
* @param {string} rid
* @param {!Object} sharedObj
*/
let Segment = function(rid, sharedObj) {
	this._rid = rid;
	this._childIds = [];
	this._shared = sharedObj;
	if(sharedObj.defaultCollapsing) {
		this._collapsed = true;
	}
};
es6_Ext.inherits(Segment, es6_EventDispatcher);

/** @public
* @type {string}
* @constant
*/
const UNCATEGORY = "Uncategorized";

/** @private
* @function
* @param {string} a
* @param {string} b
* @return {number}
*/
Segment._subSegSortLogic = function(a, b) {
	if(a === UNCATEGORY) {
		return 1;
	}
	if(b === UNCATEGORY) {
		return -1;
	}

	if(a < b) {
		return -1;
	}
	if(b < a) {
		return 1;
	}

	return 0;
};

/** @type {Object}
* @private
*/
Segment.prototype._shared = null;

/** @type {string}
* @private
*/
Segment.prototype._rid;
/** @type {!Array.<string>}
* @private
*/
Segment.prototype._childIds;
/** @type {Object}
* @private
*/
Segment.prototype._childDataIds = null;
/** @type {boolean}
* @private
*/
Segment.prototype._collapsed = false;
/** @type {number}
* @private
*/
Segment.prototype._order = 0;
/** @type {number}
* @private
*/
Segment.prototype._offsetOrder = 0;
/** @type {number}
* @private
*/
Segment.prototype._depth = 0;
/** @type {boolean}
* @private
*/
Segment.prototype._disposed = false;

/** @type {Object}
* @private
*/
Segment.prototype._subSegDef = null;
/** @type {boolean}
* @private
*/
Segment.prototype._subSegment = false; // This indicates that the segment is autogenerated (subsegment)
/** @type {Object.<string, Segment>}
* @private
*/
Segment.prototype._subSegMap = null; // For immediate sub-segment children
/** @type {Array.<string>}
* @private
*/
Segment.prototype._subSegNames = null; // For immediate sub-segment child names/ids
/** @type {string}
* @private
*/
Segment.prototype._subSegName = "";
/** @type {*}
* @private
*/
Segment.prototype._subSegVal;


/** @public
*/
Segment.prototype.dispose = function() {
	if(this._disposed) {
		return;
	}
	this._disposed = true;

	this.removeAllEventListeners();
	let segmentNames = this._subSegNames;
	if(segmentNames) {
		let segmentCount = segmentNames.length;
		let segmentMap = this._subSegMap;
		for(let i = 0; i < segmentCount; ++i) {
			segmentMap[segmentNames[i]].dispose();
		}
		this._subSegMap = this._subSegNames = null;
	}
	if(this._collapsed) {
		this.markCollapsingStateDirty();
	}

	this._childDataIds = null;
	this._childIds.length = 0;
	this._shared = null;
	this._subSegDef = this._subSegVal = null;
};
/** @public
* @return {string}
*/
Segment.prototype.getId = function() {
	return this._rid;
};
/** @public
* @return {string}
*/
Segment.prototype.getParentId = function() {
	return this._shared.childToSegment[this._rid] || "";
};
/** @public
* @return {Segment}
*/
Segment.prototype.getParent = function() {
	let parentId = this.getParentId();
	if(parentId) {
		return this._shared.segments[parentId] || null;
	}
	return null;
};
/** @public
* @param {Array.<string>=} out_ary
* @return {Array.<string>}
*/
Segment.prototype.getSubSegmentIds = function(out_ary) {
	let segmentNames = this._subSegNames;
	if(segmentNames) {
		if(!out_ary) {
			out_ary = [];
		}
		let segmentCount = segmentNames.length;
		let segmentMap = this._subSegMap;
		for(let i = 0; i < segmentCount; ++i) {
			let segmentName = segmentNames[i];
			let segment = segmentMap[segmentName];
			out_ary.push(segment.getId());
			segment.getSubSegmentIds(out_ary);
		}

		return out_ary;
	}
	return null;
};
/** @public
* @param {string} rid
* @param {string=} dataId Row id for retrieving data
* @return {boolean} Returns true only a new child is added
*/
Segment.prototype.addChild = function(rid, dataId) {
	if(!rid) {
		return false;
	}
	let prevSegmentId = this._shared.childToSegment[rid];
	if(prevSegmentId) {
		if(prevSegmentId !== this._rid) {
			let prevSegment = this._shared.segments[prevSegmentId];
			if(prevSegment && !prevSegment.hasSubSegments()) { // Children of a classified segment always stick the root segment
				prevSegment.removeChild(rid);
			}
			this._shared.childToSegment[rid] = this._rid;
		}
	} else {
		this._shared.childToSegment[rid] = this._rid;
	}

	if(dataId != null) { // Update data id
		if(dataId && dataId !== rid) {
			if(!this._childDataIds) {
				this._childDataIds = {};
			}
			this._childDataIds[rid] = dataId;
		} else if(this._childDataIds) {
			delete this._childDataIds[rid];
		}
	}

	let at = this._childIds.indexOf(rid);
	if(at < 0) {
		if(this._collapsed) {
			this._shared.dirtyCollapsingState = true; // TODO: Check if we need to update this only when new child is added
		}
		this._childIds.push(rid);
		return true;
	}
	return false;
};
/** @public
* @param {Array.<string>} rids
* @param {Array.<string>=} dataIds Row ids for retrieving data
* @return {boolean}
*/
Segment.prototype.addChildren = function(rids, dataIds) {
	if(!rids) {
		return false;
	}
	let rowIds = Array.isArray(rids) ? rids : [rids];
	let rowCount = rowIds.length;
	if(!rowCount) {
		return false;
	}

	let dirty = 0;
	let i;
	if(dataIds != null) {
		dataIds = Array.isArray(dataIds) ? dataIds : [dataIds];
		for(i = 0; i < rowCount; ++i) {
			dirty |= this.addChild(rowIds[i], dataIds[i]);
		}
	} else {
		for(i = 0; i < rowCount; ++i) {
			dirty |= this.addChild(rowIds[i]);
		}
	}
	return dirty ? true : false;
};
/** @public
* @param {string} rid
* @return {boolean}
*/
Segment.prototype.containsChild = function(rid) {
	return this._childIds.indexOf(rid) >= 0;
};
/** @public
* @param {string} rid
* @return {number}
*/
Segment.prototype.getChildIndex = function(rid) {
	return this._childIds.indexOf(rid);
};
/** @public
* @param {string} rid
* @return {boolean}
*/
Segment.prototype.removeChild = function(rid) {
	if(this._subSegment) {
		return false; // Sub segments are not allowed to remove its children
	}
	if(!this._childIds.length) {
		return false;
	}
	let at = this._childIds.indexOf(rid);
	if(at < 0) {
		return false; // The specified rid is not a child of this segment
	}

	if(this._shared.childToSegment[rid] === this._rid) {
		delete this._shared.childToSegment[rid];
	}
	this._childIds.splice(at, 1); // Slow

	if(this._collapsed) {
		this._shared.dirtyCollapsingState = true;
	}
	return true;
};
/** @public
* @param {Array.<string>} rids
* @return {boolean}
*/
Segment.prototype.removeChildren = function(rids) {
	if(this._subSegment) {
		return false; // Sub segments are not allowed to remove its children
	}
	if(!this._childIds.length) {
		return false;
	}
	let rowIds = Array.isArray(rids) ? rids : [rids];
	let rowCount = rowIds.length;
	let dirty = 0;
	for(let i = 0; i < rowCount; ++i) {
		dirty |= this.removeChild(rowIds[i]);
	}
	return dirty ? true : false;
};
/** @public
* @return {boolean}
*/
Segment.prototype.removeAllChildren = function() {
	if(this._subSegment) {
		return false; // Sub segments are not allowed to remove its children
	}
	let childCount = this._childIds.length;
	if(!this._childIds.length) {
		return false;
	}
	let segmentId = this._rid;
	let objMap = this._shared.childToSegment;
	let chdr = this._childIds;
	for(let i = 0; i < childCount; ++i) {
		let rid = chdr[i];
		if(objMap[rid] === segmentId) {
			delete objMap[rid];
		}
	}
	this._childIds.length = 0;
	this._childDataIds = null;

	if(this._collapsed) {
		this._shared.dirtyCollapsingState = true;
	}
	return true;
};
/** @public
* @return {!Array.<string>}
*/
Segment.prototype.getChildIds = function() {
	return this._childIds; // WARNING: Returns private members
};
/** @public
* @return {!Object}
*/
Segment.prototype.getChildren = function() {
	let obj = {};
	let chdr = this._childIds;
	let childCount = chdr.length;
	let dataIds = this._childDataIds || {};
	for(let i = 0; i < childCount; ++i) {
		let rid = chdr[i];
		obj[rid] = dataIds[rid] || rid;
	}
	return obj;
};
/** @public
* @return {number}
*/
Segment.prototype.getChildCount = function() {
	return this._childIds.length;
};
/** When a segment is not empty, the visibility of its content need to be updated.
* @public
*/
Segment.prototype.markCollapsingStateDirty = function() {
	// A segment can have a child and/or autogenerated segment (subsegment) to not be considered as empty.
	if(this._childIds.length || this._subSegDef) {
		this._shared.dirtyCollapsingState = true;
	}
};


/** @public
* @return {Array.<string>}
*/
Segment.prototype.getClassification = function() {
	if(this._subSegDef) {
		return this._subSegDef.classifiers || null;
	}
	return null;
};
/** @public
* @param {string|Array.<string>} fields
* @return {boolean}
*/
Segment.prototype.setClassification = function(fields) {
	if(this._subSegment) {
		return false; // subsegment cannot be classified
	}
	let classifiers = null;
	if(this._subSegDef) {
		classifiers = this._subSegDef.classifiers;
	}

	let newClassifiers = null;
	if(fields) {
		if(typeof fields === "string") {
			newClassifiers = [fields];
		} else if(Array.isArray(fields)) {
			newClassifiers = fields;
		}
	}
	let i;
	let fieldCount = newClassifiers ? newClassifiers.length : 0;

	if(fieldCount) {
		let curCount = classifiers ? classifiers.length : 0;
		if(curCount === fieldCount) { // Check duplication
			for(i = 0; i < fieldCount; ++i) {
				if(newClassifiers[i] !== classifiers[i]) {
					break;
				}
			}
			if(i >= fieldCount) {
				return false; // nothing has change
			}
		}

		if(!this._subSegDef) {
			this._subSegDef = {
				root: this,
				subSegments: null
			};
		}

		this._subSegDef.classifiers = newClassifiers;
		// this._subSegDef.classifierChanged = true;
		return true;
	} else if(classifiers) { // Remove existing ones
		this._subSegDef.classifiers = null;
		// this._subSegDef.classifierChanged = true;
		this._subSegDef = null; // WARNING: All subsegments remain existing
		return true;
	}
	return false;
};
/** @public
* @param {Object.<string, Object>=} rows Object maps between row id and its record
* @return {boolean}
*/
Segment.prototype.classify = function(rows) {
	let classifiers = this._subSegDef ? this._subSegDef.classifiers : null;
	let classifierCount = classifiers ? classifiers.length : 0;

	let segmentNames = this._subSegNames;
	let segmentCount = segmentNames ? segmentNames.length : 0;

	if(!segmentCount) {
		if(this._depth >= classifierCount) {
			return false; // Current segment level is beyond existing classification level and this segment should already be removed
		}
	}

	let sharedObj = this._shared;
	if(this._collapsed) {
		sharedObj.dirtyCollapsingState = true;
	}

	// Prepare existing subsegments for checking change in its members
	let i;
	let segmentName = "";
	let nonExistenceGroups = {};
	let removalCount = 0;
	let segmentMap = this._subSegMap;
	let segment = null;
	if(segmentCount) {
		removalCount = segmentCount;
		for(i = 0; i < removalCount; ++i) {
			segmentName = segmentNames[i];
			nonExistenceGroups[segmentName] = 1;

			segment = segmentMap[segmentName];
			if(segment._childIds.length) { // Quick cleaning up
				segment._childIds.length = 0;
			}
			if(segment._collapsed) {
				sharedObj.dirtyCollapsingState = true;
			}
		}
	}

	// Loop through row children and assign them to their corresponding subsegment
	let rootSegment = !this._subSegment;
	let dataIds = this._childDataIds || {};
	let rid;
	let chdr = this._childIds;
	let childCount = chdr.length;
	if(this._depth < classifierCount && rows) {
		if(!segmentMap) {
			segmentMap = this._subSegMap = {};
			segmentNames = this._subSegNames = [];
		}

		let classifier = classifiers[this._depth];

		for(i = 0; i < childCount; ++i) {
			rid = chdr[i];
			let dataId = dataIds[rid] || rid;
			let record = rows[dataId];
			let val = record ? record[classifier] : null; // WARNING: row could already be removed

			sharedObj.childToSegment[rid] = this._rid; // Relocate child in case of it has been moved to a non existence group

			segmentName = UNCATEGORY;
			if(val || val === 0 || val === false) { // Check for null, undefined, "", and NaN value
				segmentName = val + "";
			}
			if(nonExistenceGroups[segmentName]) {
				nonExistenceGroups[segmentName] = 0;
				--removalCount;
			}

			segment = segmentMap[segmentName];
			if(!segment) { // New group is detected
				let subSegId = this._rid + "/" + segmentName;
				segment = new Segment(subSegId, sharedObj);
				segment._subSegDef = this._subSegDef;
				segment._subSegment = true; // this indicates that the segment is autogenerated (subsegment)
				segment._depth = this._depth + 1;
				segment._subSegName = segmentName;
				segment._subSegVal = val;
				sharedObj.childToSegment[subSegId] = this._rid; // WARNING: this will mix autogenerated rows with actual rows

				segmentMap[segmentName] = segment;
				segmentNames.push(segmentName);

				this._dispatch("subSegmentAdded", {
					"rid": subSegId,
					"segment": segment
				});
			}

			segment.addChild(rid, dataId);
		}
	} else if(rootSegment) { // In case of no classification
		for(i = 0; i < childCount; ++i) {
			rid = chdr[i];
			sharedObj.childToSegment[rid] = this._rid; // Relocate child in case of it has been moved to a non existence group
		}
	}

	// Remove all subsegments with no members
	if(removalCount > 0) {
		if(removalCount >= segmentNames.length) {
			segmentNames.length = 0;
		}
		for(segmentName in nonExistenceGroups) {
			if(nonExistenceGroups[segmentName]) {
				segment = segmentMap[segmentName];
				delete segmentMap[segmentName];
				// TODO: Slow
				let at = segmentNames.indexOf(segmentName);
				if(at >= 0) {
					segmentNames.splice(at, 1);
				}

				this._dispatch("subSegmentRemoved", {
					"rid": segment.getId(),
					"segment": segment
				});

				// segment.dispose(); Already done by segment collection
			}
		}
		if(!segmentNames.length) {
			segmentNames = this._subSegMap = this._subSegNames = null;
		}
	}

	// Sort and classify existing subsegments
	segmentCount = segmentNames ? segmentNames.length : 0;
	if(segmentCount) {
		segmentNames.sort(Segment._subSegSortLogic);
		for(i = 0; i < segmentCount; ++i) {
			segment = segmentMap[segmentNames[i]];
			segment.classify(rows);
		}
	}

	return true;
};
/** SubSegment implies being classified
* @public
* @return {boolean}
*/
Segment.prototype.hasSubSegments = function() {
	if(this._subSegNames) {
		return this._subSegNames.length ? true : false;
	}
	return false;
};
/** SubSegment implies autogenerated segment
* @public
* @return {boolean}
*/
Segment.prototype.isSubSegment = function() {
	return this._subSegment;
};
/** @public
* @return {boolean}
*/
Segment.prototype.isRootSegment = function() {
	return this._shared.childToSegment[this._rid] ? false : true;
};
/** @public
* @return {Segment}
*/
Segment.prototype.getFirstAncestor = function() {
	let ancestor = null;
	if(this._subSegment && this._subSegDef) { // Quick way to get root
		ancestor = this._subSegDef.root;
	} else { // Slow
		ancestor = this.getParent();
		if(ancestor) {
			let parentSeg = ancestor.getParent();
			while(parentSeg) { // This could cause infinite loop
				ancestor = parentSeg;
				parentSeg = ancestor.getParent();
			}
		}
	}
	return ancestor || null;
};
/** @public
* @param {Array.<Segment>=} out_ary
* @return {Array.<Segment>}
*/
Segment.prototype.getAllSubSegments = function(out_ary) {
	let segmentNames = this._subSegNames;
	if(segmentNames) {
		if(!out_ary) {
			out_ary = [];
		}
		let segmentMap = this._subSegMap;
		let segmentCount = segmentNames.length;
		for(let i = 0; i < segmentCount; ++i) {
			let segment = segmentMap[segmentNames[i]];
			out_ary.push(segment);
			segment.getAllSubSegments(out_ary);
		}
	}
	return out_ary || null;
};
/** This method sets order, last order, and depth to entire tree structure in the segment, including the segment itself
* @public
* @param {number} counter
* @param {number} rootOrder
* @param {Object} ridMap Row id map that stores row order of the entire tree
* @return {number} Last order of the segment
*/
Segment.prototype.updateTreeStructure = function(counter, rootOrder, ridMap) {
	if(!counter) {
		counter = 0;
		if(!this._subSegment) { // Subsegment's depth cannot be reset back to 0
			this._depth = 0; // WARNING: this assumes counter at 0 is the root segment
		}
	}
	if(!rootOrder) {
		rootOrder = 0;
	}
	if(!ridMap) {
		ridMap = {};
	}

	let i;
	let segmentNames = this._subSegNames;
	let subSegmentCount = segmentNames ? segmentNames.length : 0;
	if(subSegmentCount) { // Classified segment
		let segmentMap = this._subSegMap;
		for(i = 0; i < subSegmentCount; ++i) {
			let subSegment = segmentMap[segmentNames[i]];
			++counter;
			ridMap[subSegment.getId()] = rootOrder + counter;
			subSegment.setOrder(counter);
			counter = subSegment.updateTreeStructure(counter, rootOrder, ridMap);
		}
	} else {
		let childIds = this._childIds;
		let childCount = childIds.length;
		let segmentSeparators = this._shared.segments;
		let prevSeg = "";
		for(i = 0; i < childCount; ++i) {
			let childId = childIds[i];
			let childSegment = segmentSeparators[childId];
			let segmentId = (childSegment) ? childSegment.getId() : UNCATEGORY;
			if(prevSeg !== segmentId) {
				++counter;
				prevSeg = segmentId;
			}
			ridMap[childId] = rootOrder + counter;
			if(childSegment) {
				childSegment._depth = this._depth + 1;
				childSegment.setOrder(counter);
				counter = childSegment.updateTreeStructure(counter, rootOrder, ridMap);
			}
		}
	}

	return this.setLastOrder(counter);
};
/** Deprecated.
* @ignore
* @public
* @function
* @param {number} counter
* @return {number}
*/
Segment.prototype.calcSubSegmentOrder = Segment.prototype.updateTreeStructure;
/** @public
* @return {number}
*/
Segment.prototype.getSegmentLevel = function() {
	return this._depth;
};
/** This method will be called on subsegments only
* @public
* @param {Object=} rows
* @param {Object=} clsSource
*/
Segment.prototype.setRowData = function(rows, clsSource) {
	if(!rows) {
		return;
	}
	let row = rows[this._rid];
	if(!row) {
		row = rows[this._rid] = {};
	}

	if(!clsSource) {
		clsSource = rows;
	}
	row = clsSource[this._rid];
	if(!row) {
		row = clsSource[this._rid] = {};
	}

	let segment = this;
	while(segment && segment.isSubSegment()) {
		segment.getSubSegmentName(row);
		segment = segment.getParent();
	}
};
/** @public
* @param {Object=} row
* @return {string}
*/
Segment.prototype.getSubSegmentName = function(row) {
	if(row && this._subSegment) {
		let classifiers = this.getClassification();
		let field = classifiers[this._depth - 1];
		if(field) {
			row[field] = this._subSegName;
		}
	}
	return this._subSegName;
};
/**
* @public
* @param {!Function} comparer
* @returns {boolean} Returns true if there is any change
*/
Segment.prototype.sortSegments = function(comparer) {
	if(!comparer || this.hasSubSegments()) {
		return false;
	}

	let childIds = this._childIds;
	let childCount = childIds.length;
	let segments = this._shared.segments;
	for(let i = 0; i < childCount; ++i) {
		if(segments[childIds[i]]) {
			this._childIds.sort(comparer); // Perform segment sorting only if children have at least one segment
			return true;
		}
	}

	return false;
};
/** Get segment id and ids of children, including nested segments, but excluding sub segments
* @public
* @param {Array.<string>} ary
* @return {!Array.<string>}
*/
Segment.prototype.getFlattenTreeIds = function(ary) {
	if(!ary) {
		ary = [];
	}

	if(!this._subSegment) { // Sub segments are excluded from tree
		ary.push(this._rid);
	}
	let i;
	let segmentNames = this._subSegNames;
	if(segmentNames) { // Classified segment
		let segmentCount = segmentNames.length;
		let segmentMap = this._subSegMap;
		for(i = 0; i < segmentCount; ++i) {
			segmentMap[segmentNames[i]].getFlattenTreeIds(ary);
		}
	} else {
		let childIds = this._childIds;
		let childCount = childIds.length;
		let segments = this._shared.segments;
		for(i = 0; i < childCount; ++i) {
			let childId = childIds[i];
			let childSegment = segments[childId];
			if(childSegment) {
				childSegment.getFlattenTreeIds(ary);
			} else {
				ary.push(childId);
			}
		}
	}

	return ary;
};

/** @public
* @param {boolean=} bool
* @return {boolean}
*/
Segment.prototype.collapse = function(bool) {
	bool = (bool !== false);
	if(this._collapsed !== bool) {
		this._collapsed = bool;
		this.markCollapsingStateDirty();
		return true;
	}
	return false;
};
/** @public
* @param {boolean=} bool
* @return {boolean}
*/
Segment.prototype.expand = function(bool) {
	return this.collapse(bool === false);
};
/** @public
* @return {boolean}
*/
Segment.prototype.isCollapsed = function() {
	return this._collapsed;
};
/** Get all collapsing state from all children (subsegments and child segments), excluding the segment itself
* @public
* @param {Object=} objMap
* @param {boolean=} parentState=false Collapsing state from parent segment
* @return {boolean}
*/
Segment.prototype.getCollapsingStates = function(objMap, parentState) {
	let childList = this._subSegNames;
	let segmentMap = this._subSegMap;
	let subSegment = true; // Normal segments can have subsegments without being subsegment themselve
	if(!childList && this._shared) { // Ensure that segment has not been disposed
		childList = this._childIds;
		segmentMap = this._shared.segments;
		subSegment = false;
	}
	let childCount = childList ? childList.length : 0;
	if(!childCount) {
		return false;
	}

	let dirty = false;
	let childCollapsed = (parentState || this._collapsed) ? true : false;

	if(!objMap) {
		objMap = {};
	}
	for(let i = 0; i < childCount; ++i) {
		let rid = "";
		let segment = null;
		if(subSegment) {
			segment = segmentMap[childList[i]]; // Use segment name to retrieve subsegment
			rid = segment.getId();
		} else {
			rid = childList[i];
			segment = segmentMap[rid]; // Use row id to retrieve segment from shared map
		}
		if(childCollapsed) {
			objMap[rid] = childCollapsed; // Collapsing states for all children is registered here
			dirty = true;
		}
		if(segment && segment.getCollapsingStates(objMap, childCollapsed)) {
			dirty = true;
		}
	}
	return dirty;
};

/** @public
* @return {number}
*/
Segment.prototype.getOrder = function() {
	let ancestor = this.getFirstAncestor();
	if(ancestor) {
		return ancestor._order + this._order;
	}
	return this._order; // This is root order
};
/** Get the last (highest) order from the entire tree regardless of the current position segment in the hierachy
* @public
* @return {number}
*/
Segment.prototype.getLastOrder = function() {
	let ancestor = this.getFirstAncestor();
	if(ancestor) {
		return ancestor.getLastOrder();
	}
	return this._order + this._offsetOrder; // This is root last order
};
/** @public
* @param {number} val
*/
Segment.prototype.setOrder = function(val) {
	this._order = val;
};
/** WARNING: This actually sets offset from the starting order
* @public
* @param {number} val
* @returns {number} Returns the number set
*/
Segment.prototype.setLastOrder = function(val) {
	return (this._offsetOrder = val); // WARNING: this._offsetOrder cannot be greater than 9999
};

/** @private
* @type {Array.<string>}
*/
Segment._tabs = null;
/** @public
* @param {Array.<string>=} lines
* @param {number=} tabLevel
* @return {!Array.<string>} lines
*/
Segment.prototype.log = function(lines, tabLevel) {
	if(!lines) {
		lines = [];
	}

	let i;
	let tabs = Segment._tabs;
	if(!tabs) {
		tabs = Segment._tabs = [];
		let tabCh = "";
		for(i = 0; i < 11; ++i) {
			tabs[i] = tabCh;
			tabCh += "  ";
		}
	}
	if(!tabLevel) {
		tabLevel = 0;
	}

	let collapsedCh = this._collapsed ? "+ " : "- ";
	lines.push(tabs[tabLevel] + collapsedCh + this._rid);

	let childLevel = tabLevel + 1;
	let childIndent = tabs[childLevel];

	let childList = this._subSegNames;
	let segmentMap = this._subSegMap;
	let subSegment = true; // Normal segments can have subsegments without being subsegment themselve
	if(!childList && this._shared) { // Ensure that segment has not been disposed
		childList = this._childIds;
		segmentMap = this._shared.segments;
		subSegment = false;
	}
	let childCount = childList ? childList.length : 0;

	for(i = 0; i < childCount; ++i) {
		let rid = "";
		let segment = null;
		if(subSegment) {
			segment = segmentMap[childList[i]]; // Use segment name to retrieve subsegment
			rid = segment.getId();
		} else {
			rid = childList[i];
			segment = segmentMap[rid]; // Use row id to retrieve segment from shared map
		}

		if(segment) {
			segment.log(lines, childLevel);
		} else {
			lines.push(childIndent + "- " + rid);
		}
	}
	return lines;
};


/* harmony default export */ const data_Segment = ((/* unused pure expression or super */ null && (Segment)));


;// ./node_modules/@grid/core/es6/data/SegmentCollection.js




/** @constructor
* @extends {EventDispatcher}
*/
let SegmentCollection = function() {
	this._onSubSegmentAdded = this._onSubSegmentAdded.bind(this);
	this._onSubSegmentRemoved = this._onSubSegmentRemoved.bind(this);

	this._segments = {};
	this._insertionList = [];
	this._removalList = [];

	this._shared = {
		segments: this._segments,
		childToSegment: {}, // child Id to segment Id
		dirtyCollapsingState: false,
		defaultCollapsing: false
	};
};
es6_Ext.inherits(SegmentCollection, es6_EventDispatcher);


/** @type {!Object.<string, Segment>}
* @private
*/
SegmentCollection.prototype._segments; // Contains both segment and their sub segments
/** @type {number}
* @private
*/
SegmentCollection.prototype._segmentCount = 0;
/** @type {Object}
* @private
*/
SegmentCollection.prototype._collapsedRids = null;
/** @type {!Object}
* @private
*/
SegmentCollection.prototype._shared;
/** @type {Array.<Segment>}
* @private
*/
SegmentCollection.prototype._segmentList = null; // Array of main/root segments
/** @type {Array.<Segment>}
* @private
*/
SegmentCollection.prototype._insertionList = null; // Array of sub segments
/** @type {Array.<string>}
* @private
*/
SegmentCollection.prototype._removalList = null; // Array of sub segment ids
/** @type {boolean}
* @private
*/
SegmentCollection.prototype._classification = false;
/** @type {boolean}
* @private
*/
SegmentCollection.prototype._classifierChanged = false;
/** @type {Object.<string, number>}
* @private
*/
SegmentCollection.prototype._rowOrders = null;

/** @public
*/
SegmentCollection.prototype.dispose = function() {
	this.removeAllSegments();
	this.removeAllEventListeners();
	this._collapsedRids = null;
	this._segmentList = this._rowOrders = this._insertionList = this._removalList = null;
};
/** @public
* @param {string} rid
* @param {Array.<string>=} childRids Children of the newly added segment. This is a shorthand to addSegmentChildren method
* @return {boolean} Returns true if there is any change. Otherwise, returns false
*/
SegmentCollection.prototype.addSegment = function(rid, childRids) {
	if(rid && !this._segments[rid]) {
		let segment = this._segments[rid] = new Segment(rid, this._shared);
		segment.addEventListener("subSegmentAdded", this._onSubSegmentAdded);
		segment.addEventListener("subSegmentRemoved", this._onSubSegmentRemoved);
		++this._segmentCount;
		this.invalidateSegmentOrder(); // order could be changed

		if(childRids && childRids.length) {
			segment.addChildren(childRids);
		}
		return true;
	}
	return false;
};
/** @public
* @param {Array.<string>} rids
* @return {boolean} Returns true if there is any change. Otherwise, returns false
*/
SegmentCollection.prototype.addSegments = function(rids) {
	let rowIds = Array.isArray(rids) ? rids : [rids];
	let rowCount = rowIds.length;
	let dirty = 0;
	for(let i = 0; i < rowCount; ++i) {
		dirty |= this.addSegment(rowIds[i]);
	}
	return dirty ? true : false;
};
/** @public
* @param {string} rid
* @return {boolean}
*/
SegmentCollection.prototype.containsSegment = function(rid) {
	return this._segments[rid] ? true : false;
};
/** @public
* @param {string} rid
* @return {string} parent row id of this segmentation. If the parent row id for this segmentation cannot be found, return ""
*/
SegmentCollection.prototype.getParentRowId = function(rid) {
	return this._shared.childToSegment[rid] || "";
};
/** @public
* @param {string} rid
* @return {boolean} Returns true if there is any change. Otherwise, returns false
*/
SegmentCollection.prototype.removeSegment = function(rid) {
	let segment = this._segments[rid];
	if(!segment) {
		return false;
	}

	if(this._segmentCount <= 1) {
		return this.removeAllSegments();
	}
	let subSegment = segment.isSubSegment();
	if(subSegment) {
		this._removalList.push(segment.getId());
	}
	let subSegIds = segment.getSubSegmentIds();
	if(subSegIds) {
		let len = subSegIds.length;
		for(let i = 0; i < len; ++i) {
			let subSegId = subSegIds[i];
			if(this._segments[subSegId]) {
				this._removalList.push(subSegId);
				delete this._segments[subSegId]; // Slow
				--this._segmentCount;
			}
		}
	}
	if(!subSegment) {
		let parentSeg = segment.getParent();
		if(parentSeg) { // Move existing children to its parent
			parentSeg.addChildren(segment.getChildIds()); // WARNING: passing private member
		}
	}
	segment.removeAllChildren(); // This is important for updating childToSegment
	segment.dispose();

	delete this._segments[rid]; // Slow
	--this._segmentCount;
	this.invalidateSegmentOrder();
	return true;
};
/** @public
* @return {boolean} Returns true if there is any change. Otherwise, returns false
*/
SegmentCollection.prototype.removeAllSegments = function() {
	if(this._segmentCount) {
		for(let key in this._segments) {
			this._segments[key].dispose();
		}
		this._segments = {};
		this._segmentCount = 0;
		this._segmentList = this._rowOrders = null;
		this._shared.segments = this._segments;
		this._shared.childToSegment = {};

		this._classification = this._classifierChanged = false;
		return true;
	}
	return false;
};

/** @public
* @return {number}
*/
SegmentCollection.prototype.getSegmentCount = function() {
	return this._segmentCount;
};
/** @public
* @param {string} rid
* @return {Segment}
*/
SegmentCollection.prototype.getSegment = function(rid) {
	return this._segments[rid] || null;
};
/** @public
* @param {string} rid
* @return {Segment}
*/
SegmentCollection.prototype.getSegmentParent = function(rid) {
	let parentRowId = this.getParentRowId(rid);
	if(parentRowId) {
		return this.getSegment(parentRowId);
	}
	return null;
};
/** @public
* @return {Object.<string, Segment>}
*/
SegmentCollection.prototype.getSegments = function() {
	return this._segmentCount ? this._segments : null; // WARNING: Return private member
};
/** @public
* @return {!Array.<string>}
*/
SegmentCollection.prototype.getSegmentIds = function() {
	return Object.keys(this._segments);
};
/** @public
* @param {!Function} comparer
* @return {!Array.<Segment>}
*/
SegmentCollection.prototype.sortSegments = function(comparer) {
	let rootSegments = [];
	let segmentSeparators = this._segments;
	for(let rid in segmentSeparators) {
		let segment = segmentSeparators[rid];
		segment.sortSegments(comparer);
		if(segment.isRootSegment()) {
			rootSegments.push(segment);
		}
	}
	rootSegments.sort(comparer);
	this.invalidateSegmentOrder();
	return rootSegments;
};


/** @public
* @param {Array.<string>} segmentIds
* @param {boolean=} bool
* @return {boolean} Returns true if there is any change. Otherwise, returns false
*/
SegmentCollection.prototype.collapseSegments = function(segmentIds, bool) {
	let dirty = 0;
	if(this._segmentCount) {
		let len = segmentIds.length;
		for (let i = 0; i < len; i++) {
			let segment = this._segments[segmentIds[i]];
			if(segment) {
				dirty |= segment.collapse(bool);
			}
		}
	}
	return dirty ? true : false;
};

/** @public
* @param {string} segmentId
* @param {boolean=} bool
* @return {boolean} Returns true if there is any change. Otherwise, returns false
*/
SegmentCollection.prototype.collapseSegment = function(segmentId, bool) {
	let segment = this._segments[segmentId];
	if(segment) {
		return segment.collapse(bool);
	}
	return false;
};
/** @public
* @param {string} segmentId
* @param {boolean=} bool
* @return {boolean}
*/
SegmentCollection.prototype.expandSegment = function(segmentId, bool) {
	return this.collapseSegment(segmentId, bool === false);
};
/** @public
* @return {boolean} Returns true if there is any change. Otherwise, returns false
*/
SegmentCollection.prototype.expandAllSegments = function() {
	if(this._segmentCount) {
		let segmentSeparators = this._segments;
		let dirty = 0;
		for(let rid in segmentSeparators) {
			dirty |= segmentSeparators[rid].expand();
		}
		if(dirty) {
			return true;
		}
	}
	return false;
};
/** @public
* @param {string} segmentId
* @return {boolean} Returns true if there is any change. Otherwise, returns false
*/
SegmentCollection.prototype.isCollapsedSegment = function(segmentId) {
	let segment = this._segments[segmentId];
	if(segment) {
		return segment.isCollapsed();
	}
	return false;
};
/** @public
* @return {Object}
*/
SegmentCollection.prototype.getCollapsedRows = function() {
	if(this._shared.dirtyCollapsingState) {
		this._shared.dirtyCollapsingState = false;
		let collapsedRids = null;
		let count = 0;
		if(this._segmentCount) {
			let segmentSeparators = this._segments;
			collapsedRids = {};
			for(let rid in segmentSeparators) {
				let segment = segmentSeparators[rid];
				if(segment.isRootSegment()) {
					if(segment.getCollapsingStates(collapsedRids)) {
						++count;
					}
				}
			}
		}
		this._collapsedRids = count ? collapsedRids : null;
	}
	return this._collapsedRids;
};

/** Invalidate segment order cache
* @public
* @returns {boolean} Returns true if there is any change
*/
SegmentCollection.prototype.invalidateSegmentOrder = function() {
	if(this._segmentList) {
		this._segmentList = this._rowOrders = null;
		return true;
	}
	return false;
};
/** @public
* @param {string} segmentId
* @param {string} rid
* @param {string=} dataId Row id for retrieving data
* @return {boolean} Returns true if there is any change. Otherwise, returns false
*/
SegmentCollection.prototype.addSegmentChild = function(segmentId, rid, dataId) {
	let segment = this._segments[segmentId];
	if(segment && !segment.isSubSegment()) {
		// If a segment becomes a child of other segment, then the segment order needs to be recalculated
		this.invalidateSegmentOrder();
		return segment.addChild(rid, dataId);
	}
	return false;
};
/** @public
* @param {string} segmentId
* @param {Array.<string>} rids
* @param {Array.<string>=} dataIds Row ids for retrieving data
* @return {boolean} Returns true if there is any change. Otherwise, returns false
*/
SegmentCollection.prototype.addSegmentChildren = function(segmentId, rids, dataIds) {
	let segment = this._segments[segmentId];
	if(segment && !segment.isSubSegment()) {
		// If a segment becomes a child of other segment, then the segment order needs to be recalculated
		this.invalidateSegmentOrder();
		return segment.addChildren(rids, dataIds);
	}
	return false;
};
/** This only works for immediate children of the specified segment
* @public
* @param {string} segmentId
* @param {string} rid
* @return {boolean} Returns true if there is any change. Otherwise, returns false
*/
SegmentCollection.prototype.containsSegmentChild = function(segmentId, rid) {
	let segment = this._segments[segmentId];
	if(segment) {
		return segment.containsChild(rid);
	}
	return false;
};
/** @public
* @param {string} segmentId
* @param {string} rid
* @return {boolean} Returns true if there is any change. Otherwise, returns false
*/
SegmentCollection.prototype.removeSegmentChild = function(segmentId, rid) {
	let segment = this._segments[segmentId];
	if(segment) {
		this.invalidateSegmentOrder();
		return segment.removeChild(rid);
	}
	return false;
};
/** @public
* @param {string} segmentId
* @param {Array.<string>} rids
* @return {boolean} Returns true if there is any change. Otherwise, returns false
*/
SegmentCollection.prototype.removeSegmentChildren = function(segmentId, rids) {
	let segment = this._segments[segmentId];
	if(segment) {
		this.invalidateSegmentOrder();
		return segment.removeChildren(rids);
	}
	return false;
};
/** @public
* @return {boolean} Returns true if there is any change. Otherwise, returns false
*/
SegmentCollection.prototype.removeAllSegmentChildren = function() {
	this._shared.childToSegment = {};
	let segmentSeparators = this._segments;
	let dirty = false;
	for(let rid in segmentSeparators) {
		let segment = segmentSeparators[rid];
		if(segment.removeAllChildren()) {
			dirty = true;
		}
	}

	if(dirty) {
		this.invalidateSegmentOrder();
		this.classify(null);
	}

	return dirty;
};

/** @public
* @param {string} segmentId
* @param {Array.<string>} rids
*/
SegmentCollection.prototype.fillSegment = function(segmentId, rids) {
	let rowCount = Array.isArray(rids) ? rids.length : 0;
	if(!rowCount) {
		return;
	}
	let segmentSeparators = this._segments;
	let curSegment = segmentSeparators[segmentId];
	if(curSegment && !curSegment.isSubSegment()) {
		let segmentAt = rids.indexOf(segmentId);
		if(segmentAt >= 0) {
			curSegment.removeAllChildren();
			for(let r = segmentAt + 1; r < rowCount; ++r) {
				let rid = rids[r];
				let segmentSeparator = segmentSeparators[rid];
				if(segmentSeparator) {
					break;
				}
				curSegment.addChild(rid);
			}
			this.invalidateSegmentOrder();
		}
	}
};
/** @public
* @param {Array.<string>} rids
* @return {boolean} Return true if the fill segments have changed, otherwise return false
*/
SegmentCollection.prototype.fillSegments = function(rids) {
	this._shared.childToSegment = {};

	let rowCount = Array.isArray(rids) ? rids.length : 0;
	let segmentSeparators = this._segments;
	let curSegment = null;
	let change = false;
	for(let r = 0; r < rowCount; ++r) {
		let rid = rids[r];
		let segmentSeparator = segmentSeparators[rid];
		if(segmentSeparator && !segmentSeparator.isSubSegment()) {
			curSegment = segmentSeparator;
			curSegment.removeAllChildren();
		} else if(curSegment && !curSegment.isSubSegment()) {
			curSegment.addChild(rid);
			change = true;
		}
	}
	if(change) {
		this.invalidateSegmentOrder();
	}
	return change;
};

/** The number represent maximum nested segments that can be created in a single segment
* @private
* @type {number}
* @constant
*/
const SEGMENT_STEP = 10000;

/** @public
* @param {Array.<string>} rids
* @param {boolean=} useCache=false If this is true, skip the calculation when there is already a cache for segment order
*/
SegmentCollection.prototype.calcSegmentOrder = function(rids, useCache) {
	let segmentList = this._segmentList;
	if(segmentList) {
		if(useCache) {
			return; // Use previous cache for segment order
		}
		segmentList.length = 0;
	} else {
		segmentList = this._segmentList = [];
	}

	this._rowOrders = {};
	let rowOrders = this._rowOrders;
	let segmentSeparators = this._segments;
	let childToSegmentId = this._shared.childToSegment;
	let prevSeg = UNCATEGORY;
	let ridCount = rids ? rids.length : 0;
	let order = 0;
	for(let i = 0; i < ridCount; ++i) {
		let rid = rids[i];
		if(!childToSegmentId[rid]) {
			let segment = segmentSeparators[rid];
			let segmentId = (segment) ? segment.getId() : UNCATEGORY;
			if(prevSeg !== segmentId) {
				++order; // WARNING: Segments and sub segments start with 1
				prevSeg = segmentId;
			}

			let rootOrder = order * SEGMENT_STEP;
			rowOrders[rid] = rootOrder;
			if(segment) {
				this._segmentList.push(segment);
				segment.setOrder(rootOrder);
				segment.updateTreeStructure(0, rootOrder, rowOrders);
			}
		}
	}
};
/** @public
* @param {!Array.<string>} rids Array of row ids
* @param {boolean=} partial Indicating that the given ids are not the whole list
* @return {Array.<number>} Returns Array of segment values, if there are at least one segment, otherwise returns null
*/
SegmentCollection.prototype.getSegmentValues = function(rids, partial) {
	let rowCount = rids ? rids.length : 0;
	if(!rowCount) {
		return null;
	}
	let rowOrders = this._rowOrders;
	if(!rowOrders) { // calcSegmentOrder has not been called (no cache detected)
		return null;
	}

	let segmentValues = new Array(rowCount);
	let highestVal = 0;
	for(let r = 0; r < rowCount; ++r) {
		let rid = rids[r];
		let segmentVal = rowOrders[rid];
		if(segmentVal != null) {
			if(highestVal < segmentVal) {
				highestVal = segmentVal;
			}
		} else {
			segmentVal = highestVal ? highestVal + SEGMENT_STEP : 0;
		}
		segmentValues[r] = segmentVal;
	}

	return highestVal ? segmentValues : null;
};
/** @public
* @return {string}
*/
SegmentCollection.prototype.logStructure = function() {
	let segmentList = this._segmentList;
	if(!segmentList) {
		return "";
	}
	let segmentCount = segmentList.length;
	let lines = [];
	for(let i = 0; i < segmentCount; ++i) {
		let segment = segmentList[i];
		if(segment.isRootSegment()) {
			segment.log(lines);
		}
	}

	return lines.join("\n");
};
/** @public
* @return {string}
*/
SegmentCollection.prototype.logRowIdMap = function() {
	let lines = [];
	let segmentSeparators = this._segments;
	let childToSegmentId = this._shared.childToSegment;
	for(let rid in childToSegmentId) {
		let segmentId = childToSegmentId[rid];
		let segment = segmentSeparators[segmentId];
		if(!segment || !segment.isSubSegment()) {
			lines.push(rid + " > " + segmentId);
		}
	}

	return lines.join("\n");
};

/** @public
* @param {string} segmentId
* @param {string|Array.<string>} fields
* @return {boolean}
*/
SegmentCollection.prototype.setSegmentClassification = function(segmentId, fields) {
	let segment = this._segments[segmentId];
	if(segment) {
		if(segment.setClassification(fields)) {
			if(segment.getClassification()) {
				this._classification = true;
			}
			this._classifierChanged = true;

			return true;
		}
	}
	return false;
};
/** @public
* @return {boolean}
*/
SegmentCollection.prototype.hasClassification = function() {
	// WARNING: This include the time when classification is removed
	return this._classification || this._classifierChanged;
};
/** @public
* @param {Object.<string, Object>} rows Object maps between row id and its record
* @return {boolean}
*/
SegmentCollection.prototype.classify = function(rows) {
	if(!this._segmentCount) {
		return false;
	}
	if(!this.hasClassification()) {
		return false;
	}
	this._classification = this._classifierChanged = false;

	let segmentSeparators = this._segments;
	for(let rid in segmentSeparators) {
		let segment = this._segments[rid];
		if(!segment.isSubSegment()) {
			if(segment.getClassification()) {
				this._classification = true;
			}
			segment.classify(rows);
		}
	}
	if(this._insertionList.length || this._removalList.length) {
		this.invalidateSegmentOrder();
		this._dispatch("subSegmentChanged", {
			"insertionList": this._insertionList,
			"removalList": this._removalList
		});
		this._insertionList.length = 0;
		this._removalList.length = 0;

		this._dispatch("classified", {});
		return true;
	}

	this._dispatch("classified", {});
	return false;
};

/** @private
* @param {!Object} e
*/
SegmentCollection.prototype._onSubSegmentAdded = function(e) {
	let rid = e["rid"];
	if(!this._segments[rid]) {
		let segment = /** @type{Segment} */(e["segment"]);
		this._insertionList.push(segment);
		this._segments[rid] = segment;
		this._segmentCount++;

		segment.addEventListener("subSegmentAdded", this._onSubSegmentAdded);
		segment.addEventListener("subSegmentRemoved", this._onSubSegmentRemoved);
	} else {
		console.log("Incorrect logic detected.");
	}
};
/** @private
* @param {!Object} e
*/
SegmentCollection.prototype._onSubSegmentRemoved = function(e) {
	let rid = e["rid"];
	this.removeSegment(rid);
};

/** @public
* @param {boolean} bool
* @return {boolean}
*/
SegmentCollection.prototype.setDefaultCollapsing = function(bool) {
	let newVal = bool ? true : false;
	if(this._shared.defaultCollapsing !== newVal) {
		this._shared.defaultCollapsing = newVal;
		return true;
	}
	return false;
};


/* harmony default export */ const data_SegmentCollection = (SegmentCollection);


;// ./node_modules/@grid/core/es6/data/DataTable.js



 // eslint-disable-line


/** Trigger when data within the table has been changed
* @event DataTable#dataChanged
* @property {boolean} globalChange Indicates a big change. User should expect all data has been change
* @property {string} type Type of changes. Possible values are "inserted", "removed", "updated", and undefined
* @property {string} rid Row Id
* @property {Object.<string, *>} rowData Column values of the changed row in JSON object format
*/

/** Trigger before DataTable#dataChanged. Perform any data update during the event will NOT cause more dataChanged events
* @event DataTable#dataComposed
*/

/** @constructor
* @extends {DataCache}
*/
let DataTable = function() {
	DataTable.base(this, 'constructor');

	this._removeRows = this._removeRows.bind(this);
	this._bySegmentSeparator = this._bySegmentSeparator.bind(this);
	this._onClassifyingTimer = this._onClassifyingTimer.bind(this);
	this._onSubSegmentChanged = this._onSubSegmentChanged.bind(this);

	this._prevData = {};
	this._rids = [];

	this._compMap = {};

	this._addEvent("subSegmentChanged");
};
es6_Ext.inherits(DataTable, data_DataCache);

/** @private
* @type {Object.<string, Object.<string, *>>}
*/
DataTable.prototype._prevData = null; // Mainly used for blinking
/** @private
* @type {!Array.<string>}
*/
DataTable.prototype._rids;
/** @private
* @type {boolean}
*/
DataTable.prototype._frozen = false;
/** @private
* @type {number}
*/
DataTable.prototype._runningId = 0;

/** Comparer map that will be shared across all child DataViews
* @private
* @type {Object.<string, DataTable.SortLogic>}
*/
DataTable.prototype._compMap = null;

/** @private
* @type {SegmentCollection}
*/
DataTable.prototype._segments = null;
/** @private
* @type {boolean}
*/
DataTable.prototype._autoSegmentFilling = false;
/** @private
* @type {Object}
*/
DataTable.prototype._removedRows = null;
/** @private
* @type {Function}
*/
DataTable.prototype._userSegmentComparer = null;
/** @private
* @type {number}
*/
DataTable.prototype._segmentSortOrder = 0;
/** @private
* @type {*}
*/
DataTable.prototype._segmentSortContext = null;
/** @private
* @type {Object.<string, Object>}
*/
DataTable.prototype._clsSource = null;
/** @private
* @type {number}
*/
DataTable.prototype._classifyingTimer = 0;
/** @private
* @type {boolean}
*/
DataTable.prototype._segmentDefaultCollapsing = false;

/** @typedef {Function} DataTable~SortLogic
* @description SortLogic function is used for comparison between two values during sorting. <br>
* The function should return:<br>
* negative value to put val1 before val2, <br>
* positive value to put val2 before val1. <br>
* zero or falsy value to keep val1 and val2 unchanged.
* @property {*} val1 Value from a row
* @property {*} val2 Value from another row
* @property {number=} order 1 indicates ascending order, and -1 indicates descending order
* @property {*=} contextObject Any given user context
* @example
* let customComparer = function(a, b, order) {
*   if(a + 10 < b) { return -order; }
*   if(b + 10 < a) { return order; }
*   return 0; // a and b are less than 10 difference apart
* };
*/
DataTable.SortLogic;


/** @override
*/
DataTable.prototype.dispose = function() {
	this.unlistenAll();
	this.clearAllData(true);

	if(this._classifyingTimer) {
		clearTimeout(this._classifyingTimer);
		this._classifyingTimer = 0;
	}

	this._compMap = null; // Release user function that may be bound
	this._clsSource = null;
	DataTable._removeIndexArray();
	if(this._segments) {
		this._segments.dispose();
		this._segments = null;
	}
};

/** {@link DataCache#getColumnData}
* @override
*/
DataTable.prototype.getColumnData = function(cid) {
	let rows = new Array(this._rids.length);
	for(let i = this._rids.length; --i >= 0;) {
		rows[i] = this._rows[this._rids[i]][cid];
	}
	return rows;
};
/** @public
* @param {number} rowIndex
* @return {string}
*/
DataTable.prototype.getRowId = function(rowIndex) {
	return this._rids[rowIndex] || "";
};
/** {@link DataCache#getAllRowIds}
* @override
* @param {boolean=} opt_shallowCopy Default value is false. Set to true for speed
*/
DataTable.prototype.getAllRowIds = function(opt_shallowCopy) {
	if(opt_shallowCopy) {
		return this._rids;
	}
	return this._rids.slice(0); // Fastest way to clone an array
};
/** @public
* @param {string} rid
* @return {number}
*/
DataTable.prototype.getRowIndex = function(rid) {
	return this._rids.indexOf(rid);
};
/** @public
* @return {number}
*/
DataTable.prototype.getRowCount = function() {
	return this._rids.length;
};
/** @public
* @param {number} rowIndex
* @param {string} cid
* @return {Object.<string, *>}
*/
DataTable.prototype.getDataAt = function(rowIndex, cid) {
	return this.getData(this._rids[rowIndex], cid);
};
/** @public
* @param {number} rowIndex
* @return {Object.<string, *>}
*/
DataTable.prototype.getRowDataAt = function(rowIndex) {
	return this.getRowData(this._rids[rowIndex]);
};
/** Get previously set data. This is good for blinking changed data
* @public
* @param {string} rid
* @param {string} cid
* @return {Object.<string, *>}
*/
DataTable.prototype.getPreviousData = function(rid, cid) {
	let row = this._prevData[rid] || null;
	if(row) {
		return row[cid];
	}
	return null;
};


/** Set data to individual cell
* @override
* @param {string} rid Row Id
* @param {string} cid Column Id
* @param {*} value
* @return {boolean} Return true if there is any change, and false otherwise
* @fires DataTable#dataChanged
*/
DataTable.prototype.setData = function(rid, cid, value) { // Data changed event may be dispatched
	if(!this._needFiring()) {
		return this._setData(rid, cid, value);
	}
	let values = {};
	values[cid] = value;
	return this.setRowData(rid, values);
};
/** @private
* @param {string} rid Row Id
* @returns {boolean}
*/
DataTable.prototype._removeRidFromSegments = function(rid) {
	if(this._segments) {
		let segment = this._segments.getSegment(rid);
		if(segment) {
			if(this._segments.removeSegment(rid)) {
				// TODO: Handle sub segment removal
				if(!this._segments.getSegmentCount()) {
					this._segments.dispose();
					this._segments = null;
				}
				return true;
			}
		} else {
			let segmentParentId = this._segments.getParentRowId(rid);
			if(segmentParentId) {
				this._segments.removeSegmentChild(segmentParentId, rid);
			}
		}
	}
	return false;
};
/** Set data in row manner
* @override
* @public
* @param {string} rid Row Id
* @param {Object.<string, *>=} values Object where the key is column Id and value is the data to be set
* @param {Object=} eventArg Additional arguments to be attached to the event fired
* @return {boolean} Return true if there is any change, and false otherwise
* @fires DataTable#dataChanged
* @example
* let dt = new DataTable();
* dt.setRowData("row 1", {"column 1": "a", "column 2": 1, "column 3": null});
* dt.log(); // See console for the results
*/
DataTable.prototype.setRowData = function(rid, values, eventArg) { // Data changed event may be dispatched
	if(!rid) {
		return false;
	}

	let row = this.getRowData(rid);
	let dirty = false;
	let type = "updated";
	let segmentChanged = false;

	if(values) {
		if(!row) {
			type = "inserted";
			row = values;
			this._rows[rid] = row; // WARNING: Dangerous use of external reference. User may modify data without using DataTable's API
			this._prevData[rid] = {};
			if(eventArg) {
				let nextRid = /** @type{string} */(eventArg["nextRid"]); // Used for insertion only
				let rowIndex = (nextRid) ? this.getRowIndex(nextRid) : -1;
				if(rowIndex < 0) {
					rowIndex = /** @type{number} */(eventArg["fallback"]);
				}
				if(rowIndex == null || rowIndex < 0 || rowIndex >= this._rids.length) {
					this._rids.push(rid);
				} else {
					this._rids.splice(rowIndex, 0, rid);
					if(this._segments) { // for no sorting case
						this._segments.invalidateSegmentOrder();
					}
					segmentChanged = !this._isLastSegment(rowIndex); // This is slow. Check if this is required
				}
			} else {
				this._rids.push(rid);
			}
		} else {
			let prevData = this._prevData[rid];
			for (let cid in values) {
				prevData[cid] = row[cid];
				row[cid] = values[cid];
			}
		}
		dirty = true;
	} else {
		if(row) {
			type = "removed";
			// WARNING: Even if row is removed we still keep previous data
			// delete this._prevData[rid];
			this._prevData[rid] = this._rows[rid]; // Move reference from the removed row to history
			delete this._rows[rid];
			segmentChanged = this._removeRidFromSegments(rid);
			DataTable._removeArrayItem(this._rids, rid);
			dirty = true;
		}
	}

	if(dirty && this._needFiring()) {
		if(this._autoFillSegments()) {
			this.dispatchGlobalChange();
		} else {
			let e = eventArg || {};
			e["rid"] = rid;
			e["changes"] = values; // WARNING: External user data is passed along
			e["rowData"] = row;
			e["type"] = type;
			if(segmentChanged) {
				e["segmentChanged"] = true;
			}
			this._dispatchDataChange(e);
		}
		this.requestClassifying(); // TODO: Check if needFiring method should have an effect on this request
	}
	return dirty;
};
/** Set data in a column manner. This is faster than repeatedly calling {@link DataTable#setData}
* @public
* @param {string} cid
* @param {Array.<*>} valueList
* @param {Array.<string>=} opt_ridList Specify row id to be set corresponding to the data item
* @example
* let dt = new DataTable();
* dt.setColumnData("column 1", ["1st row data", "2nd row data", "3rd row data"]);
* dt.log(); // See console for the results
*/
DataTable.prototype.setColumnData = function(cid, valueList, opt_ridList) { // Data changed event may be dispatched
	let rLen = (valueList) ? valueList.length : 0;
	if(rLen < 0) { return; }

	let prevFrozen = this._frozen;
	this.freeze();
	let rids = this._generateRids(rLen, opt_ridList);

	for(let r = 0; r < rLen; ++r) {
		this._setData(rids[r], cid, valueList[r]);
	}

	this.freeze(prevFrozen);
};
/** Add, update, and remove data in batch by using JSON object. This is faster than repeatedly calling {@link DataTable#setRowData}. <br>
Any existing row, that is not matched by the same row id provided in the argument, remains the same. <br>
Any existing row with the same row id from the provided argument will be updated. <br>
Row id with null value will cause a removal of the row with the same id.
* @public
* @param {Object.<string, Object.<string, *>>} obj Object is 2 level nested object where 1st level key is row Id and 2nd level key is column Id
* @see {@link DataTable#addRows}
* @see {@link DataTable#setDataFrom2DArray}
* @example
* let data = {
*   "rowId 1": {"column 1": "a", "column 2": 1, "column 3": 2},
*   "rowId 2": {"column 1": "b", "column 2": 3, "column 3": 4},
*   "rowId 4": {"column 1": "d", "group_criteria": 1} // Rows don't need to be in uniform structure
* };
* let dt = new DataTable();
* dt.setDataFromObjectMap(data);
* dt.log(); // See console for the results
* dt.setDataFromObjectMap({"rowId 1": null}); // Remove the row with "rowId 1"
* dt.setDataFromObjectMap({"rowId 2": {"column 1": "newValue"} }); // Update existing row
*/
DataTable.prototype.setDataFromObjectMap = function(obj) {
	if(!obj) {
		return;
	}
	if(Array.isArray(obj)) {
		this.addRows(/** @type{Array.<Object>} */(obj));
		return;
	}

	let prevFrozen = this._frozen;
	this.freeze();

	for(let rid in obj) {
		this.setRowData(rid, obj[rid]);
	}

	this.freeze(prevFrozen);
};
/** Set data in batch from 2 dimensional array. This is faster than repeatedly calling {@link DataTable#setRowData}
* @public
* @param {Array.<Array.<*>>} ary Row based array
* @param {Array.<string>=} opt_cidList
* @param {Array.<string>=} opt_ridList
* @see {@link DataTable#addRows}
* @see {@link DataTable#setDataFromObjectMap}
* @example
* let data = [
*   ["a", 1, 2],
*   ["b", 3, 4],
*   ["c", 5, 6]
* ];
* let dt = new DataTable();
* dt.setDataFrom2DArray(data, ["column 1", "column 2", "column 3"]);
* dt.log(); // See console for the results
*/
DataTable.prototype.setDataFrom2DArray = function(ary, opt_cidList, opt_ridList) {
	let rLen = (ary) ? ary.length : 0;
	if(rLen < 0) { return; }

	let prevFrozen = this._frozen;
	this.freeze();

	let c = 0;
	let rids = this._generateRids(rLen, opt_ridList);

	for(let r = 0; r < rLen; ++r) {
		let row = ary[r];
		let rid = rids[r];
		if(opt_cidList) {
			for(c = row.length; --c >= 0;) {
				this._setData(rid, opt_cidList[c], row[c]);
			}
		} else {
			for(c = row.length; --c >= 0;) {
				this._setData(rid, "" + c, row[c]); // Auto-generate column id
			}
		}
	}

	this.freeze(prevFrozen);
};

/** Set data to individual cell by specified row index instead of row Id
* @public
* @param {number} rowIndex Invalid row index will not produce any effect
* @param {string} cid
* @param {*} value
* @return {boolean} Return true if there is any change, and false otherwise
* @fires DataTable#dataChanged
*/
DataTable.prototype.setDataAt = function(rowIndex, cid, value) { // Data changed event may be dispatched
	let rid = this.getRowId(rowIndex);
	return (rid) ? this.setData(rid, cid, value) : false;
};
/** @public
* @param {number} rowIndex Invalid row index will not produce any effect
* @param {Object.<string, *>=} values Object where the key is column Id and value is the data to be set
* @param {Object=} opt_eventArg
* @return {boolean} Return true if there is any change, and false otherwise
* @fires DataTable#dataChanged
*/
DataTable.prototype.setRowDataAt = function(rowIndex, values, opt_eventArg) { // Data changed event may be dispatched
	let rid = this.getRowId(rowIndex);
	return (rid) ? this.setRowData(rid, values, opt_eventArg) : false;
};
/** Add new row of data to the end of the Data Table
* @public
* @param {Object.<string, *>=} values field-value pair object
* @param {string|null=} rid If the given row Id already exists, data is updated and no new row is added. Leave blank or null to let DataTable generate unique row Id
* @return {string} Return Row id that has been inserted or changed
* @fires DataTable#dataChanged
* @example
* let dt = new DataTable();
* dt.addRow({"column 1": "a", "column 2": 1, "column 3": null});
* dt.addRow({"column 1": "b", "column 2": 5, "column 3": {"someKey": 10}});
* dt.log(); // See console for the results
*/
DataTable.prototype.addRow = function(values, rid) {
	if(!values) { values = {}; }
	rid = this._generateUniqueRid(values, rid);

	this.setRowData(rid, values);
	return rid;
};
/** Add multiple rows all at once. This is used to improve the performance. dataChagned with globalChange flag will be fired, if multiple rows are added.
* @public
* @param {Array.<Object>} rows Array of field-value pair object.
* @param {Array.<string>=} rids If the given row Id already exists, data is updated and no new row is added. Leave blank or null to let DataTable generate unique row Id
* @fires DataTable#dataChanged
* @see {@link DataTable#setDataFrom2DArray}
* @see {@link DataTable#setDataFromObjectMap}
* @example
* let dt = new DataTable();
* dt.addRows([
*   {"field1": "a", "field2": 1, "field3": null},
*   {"field1": "b", "field2": 5, "field3": {"someKey": 10}},
*   {"field1": "c", "field2": 7, "field3": [1, 2, 3]},
*   {"field1": "c", "field4": "some value"} // Rows don't need to be in uniform structure
* ]); // 4 rows are added
* dt.log(); // See console for the results
*/
DataTable.prototype.addRows = function(rows, rids) {
	if(!Array.isArray(rows)) {
		return;
	}
	let len = rows.length;
	if(!len) {
		return;
	} else if(len === 1) {
		this.addRow(rows[0], rids ? rids[0] : null);
		return;
	}
	if(!rids) {
		rids = [];
	}
	let prevFrozen = this._frozen;
	this.freeze();
	for(let i = 0; i < len; ++i) {
		this.addRow(rows[i], rids[i]);
	}
	this.freeze(prevFrozen);
};

/** Insert a new row as the first row
* @public
* @param {Object.<string, *>=} opt_values
* @param {string|null=} opt_rid If the given row Id already exists, data is updated and no new row is added. Leave blank or null to let DataTable generate unique row Id
* @return {string} Return Row id that has been inserted or changed
* @fires DataTable#dataChanged
* @see {@link DataTable#insertRow}
*/
DataTable.prototype.unshiftRow = function(opt_values, opt_rid) {
	return this.insertRow(0, opt_values, opt_rid);
};
/** @public
* @param {(string|number|null)=} rowRef
* @param {Object.<string, *>=} values Key/value pair object map
* @param {string|null=} rid If the given row Id already exists, data is updated and no new row is added. Leave blank or null to let DataTable generate unique row Id
* @return {string} Return Row id that has been inserted or changed
* @fires DataTable#dataChanged
* @see {@link DataTable#unshiftRow}
* @see {@link DataTable#removeRow}
*/
DataTable.prototype.insertRow = function(rowRef, values, rid) {
	let arg = null;
	if(typeof rowRef === "number") {
		arg = {};
		arg["nextRid"] = this._rids[rowRef];
		arg["fallback"] = rowRef; // Fallback index in case of no nextRid found
	} else if(rowRef) {
		arg = {};
		arg["nextRid"] = /** @type{string} */(rowRef);
	}

	if(!values) { values = {}; }
	rid = this._generateUniqueRid(values, rid);

	this.setRowData(rid, values, arg);
	return rid;
};
/** @public
 * @param {string|number} rowRef
 * @param {Object=} opt_values
 * @param {string=} opt_rid
 * @return {string}
 * @fires DataTable#dataChanged
 */
DataTable.prototype.insertAfter = function(rowRef, opt_values, opt_rid) {
	let rowIndex = (typeof rowRef === "string") ? this.getRowIndex(rowRef) : /** @type{number}*/ (rowRef);

	if(!(rowIndex >= 0) || rowIndex >= this._rids.length - 1) {
		return this.addRow(opt_values, opt_rid);
	}
	return this.insertRow(rowIndex + 1, opt_values, opt_rid);
};

/** @public
* @param {number|string} rowRef Reference to the row to be removed
* @see {@link DataTable#removeRows}
*/
DataTable.prototype.removeRow = function(rowRef) { // Data changed event may be dispatched
	if(typeof rowRef === "number") {
		this.setRowData(this.getRowId(rowRef), null);
	} else {
		this.setRowData(/** @type{string} */(rowRef), null);
	}
};
/** @public
* @param {Array.<number|string>} refs Reference to the rows to be removed
* @see {@link DataTable#removeAllRows}
*/
DataTable.prototype.removeRows = function(refs) {
	let rid;
	if(refs.length === 1) {
		rid = refs[0];
		if (typeof rid === "number") {
			this.removeRow(rid);
		} else {
			this.setRowData(rid, null);
		}
		return;
	}

	let i;
	let dirty = false;
	this._removedRows = {};
	for (i = refs.length; --i >= 0;) {
		rid = refs[i];
		if (typeof rid === "number") {
			rid = this.getRowId(rid);
		}

		let row = this.getRowData(rid);
		if(row) {
			this._prevData[rid] = this._rows[rid];
			this._removedRows[rid] = 1;
			delete this._rows[rid];
			this._removeRidFromSegments(rid);
			dirty = true;
		}
	}
	if(dirty) {
		this._rids = this._rids.filter(this._removeRows);
		this._removedRows = null;
		this.dispatchGlobalChange();
	} else {
		this._removedRows = null;
	}
};
/** @private
* @param {string} rid
* @return {boolean}
*/
DataTable.prototype._removeRows = function(rid) {
	return !this._removedRows[rid];
};

/** @private
* @param {Array} ary
* @param {*} item
* @return {boolean}
*/
DataTable._removeArrayItem = function(ary, item) {
	let at = ary.indexOf(item);
	if(at >= 0) {
		if(at >= ary.length) {
			ary.pop(); // Array.pop is 100 times faster than Array.splice
		} else {
			ary.splice(at, 1);
		}
		return true;
	}
	return false;
};
/** Remove all existing rows
* @override
*/
DataTable.prototype.clearAllData = function(suppressEvent) {
	DataTable.base(this, "clearAllData", true);
	this._prevData = {};

	if(this._segments) {
		this._segments.dispose();
		this._segments = null;
	}

	if(this._rids.length) {
		this._rids.length = 0;

		this.dispatchGlobalChange(suppressEvent);
	}
};
/** Alias to {@link DataTable#clearAllData}
* @public
* @function
* @param {boolean=} suppressEvent
* @fires DataTable#dataChanged
*/
DataTable.prototype.removeAllRows = DataTable.prototype.clearAllData;
/** @private
* @function
* @param {number} a
* @param {number} b
* @return {number}
*/
DataTable._ascendingOrder = function(a, b) { return a - b; };
/** @public
* @param {Array.<number>|number} fromIndex
* @param {number} toIndex
* @param {boolean=} suppressEvent
* @return {Array.<string>|string} Return row id(s). Return null if nothing has changed
*/
DataTable.prototype.moveRow = function(fromIndex, toIndex, suppressEvent) {
	let output = null;
	let rid = "";
	if(!(toIndex >= 0 && toIndex < this._rids.length)) {
		toIndex = this._rids.length;
	}

	if (Array.isArray(fromIndex)) {
		let fromIndices = fromIndex;
		fromIndices.sort(DataTable._ascendingOrder);
		let movedRids = [];
		for (let i = fromIndices.length; --i >= 0;) {
			let idx = fromIndices[i];
			rid = this._rids[idx];
			if(rid) {
				movedRids.unshift(rid);
				this._rids.splice(idx, 1); // WARNING: splice method is slow
				if (idx < toIndex) {
					--toIndex;
				}
			}
		}
		if(movedRids.length) {
			this._rids.splice.apply(this._rids, [toIndex, 0].concat(movedRids));
			output = movedRids;
		}
	} else {
		if (fromIndex < 0) {
			fromIndex = 0;
		}

		if(fromIndex !== toIndex && (fromIndex + 1) !== toIndex) {
			rid = this._rids.splice(fromIndex, 1)[0];
			if (rid) {
				if (fromIndex < toIndex) {
					--toIndex;
				}
				this._rids.splice(toIndex, 0, rid);
				output = rid;
			}
		}
	}

	if(output) {
		if(this._segments) {
			this._segments.invalidateSegmentOrder();
			this._sort(null); // Reorder segment members
		}
		this._dispatchPositionChange(suppressEvent);
	}
	return output;
};
/** @public
* @ignore
* @param {Array.<string>} fromRids
* @param {string} toRid If rid is empty string, insert the rid to the last index
* @param {boolean=} suppressEvent
* @return {Array.<string>|string} rid
*/
DataTable.prototype._moveRow = function(fromRids, toRid, suppressEvent) {
	let len = fromRids.length;
	if(!len) {
		return null;
	}

	let fromIndices = new Array(len);
	for(let i = 0; i < len; ++i) {
		fromIndices[i] = this.getRowIndex(fromRids[i]);
	}

	let toIndex = (toRid) ? this.getRowIndex(toRid) : this._rids.length;
	if(len > 1) {
		return this.moveRow(fromIndices, toIndex, suppressEvent);
	}
	return this.moveRow(fromIndices[0], toIndex, suppressEvent);
};

/** Move rows in chunk by specifying chunk size in the third parameter.
* @public
* @param {number|string} startIndex
* @param {number|string} toIndex
* @param {number=} opt_count
* @param {boolean=} suppressEvent
* @return {Array.<string>} rids Return null if nothing has changed
* @see {@link DataTable#moveRow}
*/
DataTable.prototype.moveRows = function(startIndex, toIndex, opt_count, suppressEvent) {
	if(typeof startIndex === "string") {
		startIndex = this.getRowIndex(startIndex);
	}
	if(typeof toIndex === "string") {
		toIndex = this.getRowIndex(toIndex);
	}
	if(startIndex < 0) { startIndex = 0; }
	if(toIndex < 0) { toIndex = this._rids.length; }
	if(!opt_count) { opt_count = 1; }

	let endIndex = startIndex + opt_count - 1;
	let targetIndex = toIndex;
	if (toIndex >= startIndex) {
		if (toIndex <= endIndex) {
			targetIndex = startIndex;
		} else {
			targetIndex = toIndex - opt_count;
		}
	}

	if ((opt_count > 0) && (endIndex <= this._rids.length)) {
		if (targetIndex > this._rids.length) {
			targetIndex = this._rids.length;
		}

		let rows = this._rids.splice(startIndex, opt_count);
		let args = [targetIndex, 0].concat(rows);
		Array.prototype.splice.apply(this._rids, args);

		if(this._segments) {
			this._sort(null); // Reorder segment members
		}
		this._dispatchPositionChange(suppressEvent);
		return rows;
	}

	return null;
};

/** This method is deprecated in favor of more sophisticate {@link DataTable#moveRow}
* @public
* @param {number} fromIndex
* @param {number} toIndex
*/
DataTable.prototype.swapRow = function(fromIndex, toIndex) { // No event is fired. No effect on other views
	let len = this._rids.length;
	if(fromIndex < 0 || fromIndex >= len) { return; }
	if(toIndex < 0 || toIndex >= len) { return; }

	let rid = this._rids[fromIndex];
	this._rids[fromIndex] = this._rids[toIndex];
	this._rids[toIndex] = rid;
};

/**
* @protected
* @override
* @ignore
* @param {string} fromRid
* @param {string} toRid
* @return {boolean}
*/
DataTable.prototype._replaceRowId = function(fromRid, toRid) {
	let rids = this._rids;
	let idx = rids.indexOf(fromRid);
	if(idx >= 0) {
		rids[idx] = toRid;
		this._rows[toRid] = this._rows[fromRid];
		delete this._rows[fromRid];
		this._prevData[toRid] = this._prevData[fromRid];
		delete this._prevData[fromRid];
		return true;
	}
	return false;
};
/**
* @public
* @override
* @ignore
* @param {Object} ridPair
*/
DataTable.prototype.replaceRowIds = function(ridPair) {
	if(typeof ridPair === "object") {
		let dirty = false;
		for(let oldRid in ridPair) {
			dirty |= this._replaceRowId(oldRid, ridPair[oldRid]);
		}
		if(dirty) {
			this.dispatchGlobalChange();
		}
	}
};

/** @public
* @function
* @param {string} rid
*/
DataTable.prototype.clearRowData = DataTable.prototype.removeRow;
/** {@link DataCache#clearColumnData}
* @override
*/
DataTable.prototype.clearColumnData = function(colId, suppressEvent) {
	let cids = (typeof colId === "string") ? [/** @type {string} */(colId)] : /** @type {Array.<string>} */(colId);
	for(let i = cids.length; --i >= 0;) {
		let cid = cids[i];
		if(cid) {
			for(let rid in this._rows) {
				delete this._rows[rid][cid];
				delete this._prevData[rid][cid];
			}
		}
	}

	this.dispatchGlobalChange(suppressEvent);
};

/** Get parent data source. Note that DataTable cannot be child of other data source. So the returned value will always be null.
* @public
* @return {null}
*/
DataTable.prototype.getDataSource = function() {
	return null; // No other source
};
/** @public
* @param {*} source
*/
DataTable.prototype.setDataSource = function(source) {};

/**
* @public
* @ignore
* @return {Object.<string, DataTable.SortLogic>}
*/
DataTable.prototype.getSortingLogics = function() {
	return this._compMap;
};
/** @public
* @param {string|Array.<string>} cid Column id
* @param {(string|number|Array.<string|number>)=} sortOrders "a"|"d"|"n"
* @param {DataTable.SortLogic=} customComparer
* @param {*=} contextObj Context object that will be provided as the forth parameter of the given comparer method
* @return {boolean} Return true if there is any change, otherwise false
* @fires DataTable#dataChanged
*/
DataTable.prototype.sortOnce = function(cid, sortOrders, customComparer, contextObj) {
	let sortingDefs = DataTable._buildSortContext(
		[],
		cid,
		sortOrders,
		customComparer || this._compMap,
		contextObj // User context
	);

	if(this._sort(sortingDefs)) {
		this._dispatchPositionChange();
		return true;
	}
	return false;
};
/** @public
* @param {DataTable.SortLogic} func
* @example
* customLogic = function(a, b, sortOrder) {
*   return (a - b) * sortOrder; // for numeric comparison
* };
* let dt = new DataTable();
* dt.setSortingLogic(customLogic);
* dt.sortOnce("Column id", "ascending");
*/
DataTable.prototype.setSortingLogic = function(func) {
	this._compMap["_default"] = func;
};
/** Specified sorting logic for the column with the given column id. This will override logic set by {@link DataTable#setSortingLogic}, when the column is sorted
* @public
* @param {string} cid
* @param {DataTable.SortLogic} func Use null to remove current sorting logic
*/
DataTable.prototype.setColumnSortingLogic = function(cid, func) {
	if(cid) {
		this._compMap[cid] = func;
	}
};
/** Get sorting logic for the specified field. Default logic is returned, if no logic is specified for the column
* @public
* @param {string=} cid
* @return {DataTable.SortLogic}
*/
DataTable.prototype.getColumnSortingLogic = function(cid) {
	if(cid) {
		let logic = this._compMap[cid];
		if(logic) {
			return logic;
		}
	}

	return this._compMap["_default"] || null;
};

/** Freeze data table so that no event is fired for data processing until executing {@link DataTable#unfreeze} method
* @public
* @param {boolean=} bool
* @return {boolean} Previous freeze state before the execution
* @fires DataTable#dataChanged
* @example
* let prevState = dt.freeze();
* for(let i = 0; i < 100; ++i) {
*   dt.insertRow(); // no event is fired
* }
* dt.freeze(prevState); // Restore previous freeze states
*/
DataTable.prototype.freeze = function(bool) {
	bool = bool !== false;
	let prevState = this._frozen;
	if(prevState !== bool) {
		this._frozen = bool;
		if(!bool && this._segments) { // for no sorting case
			this._segments.invalidateSegmentOrder();
		}
		this._autoFillSegments();
		this.dispatchGlobalChange();
	}
	return prevState;
};
/** @public
* @param {boolean=} bool
* @return {boolean} Previous freeze state before the execution
*/
DataTable.prototype.unfreeze = function(bool) {
	return this.freeze(bool === false);
};
/** @public
* @return {boolean} Current freeze state
*/
DataTable.prototype.isFrozen = function() {
	return this._frozen;
};

/** @private
*/
DataTable.prototype._initSegmentCollection = function() {
	if (!this._segments) {
		this._segments = new data_SegmentCollection();
		this._segments.setDefaultCollapsing(this._segmentDefaultCollapsing);
		this._segments.addEventListener("subSegmentChanged", this._onSubSegmentChanged);
	}
};
/** Add or remove all segments from the given row ids
* @public
* @param {Array.<string>} rids
* @param {boolean=} enabled
* @return {boolean} Return true if there is any change
*/
DataTable.prototype.setSegmentSeparators = function(rids, enabled) {
	if (!rids) {
		return false;
	}
	let adding = (enabled !== false);
	let segmentAdded = 0;
	let segmentRemoved = 0;

	let len = rids.length;
	for (let i = 0; i < len; i++) {
		let rid = rids[i];
		if(adding) {
			this._initSegmentCollection();
			if(this._autoSegmentFilling) {
				let parentId = this._segments.getParentRowId(rid);
				if(parentId) {
					this._segments.removeSegmentChild(parentId, rid);
				}
			}
			segmentAdded |= this._segments.addSegment(rid);
		} else if (this._segments) { // Removing
			let segment = this._segments.getSegment(rid);
			if(segment) {
				if(this._segments.removeSegment(rid)) {
					segmentRemoved = 1;
					if(!this._segments.getSegmentCount()) {
						this._segments = null;
					}
				}
			}
		}
	}
	let changed = segmentAdded || segmentRemoved;
	if(changed && this._needFiring()) {
		this.dispatchGlobalChange();
	}
	return changed ? true : false;
};

/**
* @public
* @param {string} rid
* @param {*=} options=null Segment options. If the value is false, segment separator will be stripped off from the given rid
* @return {boolean} Return true if there is any change
*/
DataTable.prototype.setSegmentSeparator = function(rid, options) {
	let change = false;
	let memberCount = 0;
	if(rid && typeof rid === "string") {
		if(options !== false) {
			this._initSegmentCollection();
			if(this._autoSegmentFilling) {
				let parentId = this._segments.getParentRowId(rid);
				if(parentId) {
					this._segments.removeSegmentChild(parentId, rid);
				}
			}
			let children = (options && options["children"]) ? options["children"] : null;
			if(this._segments.addSegment(rid, children)) {
				change = true;
			}
		} else if(this._segments) { // Remove the separator
			let segment = this._segments.getSegment(rid);
			if(segment) {
				memberCount = segment.getChildCount();
				if(this._segments.removeSegment(rid)) {
					change = true;
					if(!this._segments.getSegmentCount()) {
						this._segments = null;
					}
				}
			}
		}
	}
	if(change && this._needFiring()) {
		if(this._autoFillSegments() || memberCount) {
			this.dispatchGlobalChange();
		} else {
			let rowData = this._rows[rid] || null;
			if(rowData) {
				this._dispatchDataChange({
					"rid": rid,
					"changes": {},
					"rowData": rowData,
					"type": "updated",
					"segmentChanged": true
				});
			}
		}
	}

	return change;
};
/** @public
* @return {boolean} Return true if there is any change
*/
DataTable.prototype.unsetAllSegmentSeparators = function() {
	if(this._segments) {
		this._segments.dispose();
		this._segments = null;
		this.dispatchGlobalChange();
		return true;
	}
	return false;
};
/**
* @public
* @return {boolean}
*/
DataTable.prototype.hasSegmentation = function() {
	return this._segments ? true : false;
};
/**
* @public
* @return {boolean}
*/
DataTable.prototype.hasSegmentClassification = function() {
	return this._segments ? this._segments.hasClassification() : false;
};
/**
* @public
* @param {string} rid
* @return {boolean}
*/
DataTable.prototype.isSegmentSeparator = function(rid) {
	if(this._segments) {
		return this._segments.containsSegment(rid);
	}
	return false;
};
/** Get Segment object
* @public
* @param {string} rid Row id
* @return {Segment} Returns null if a segment cannot be found
*/
DataTable.prototype.getSegment = function(rid) {
	if(this._segments) {
		return this._segments.getSegment(rid);
	}
	return null;
};
/**
* @public
* @param {string} rid Row id of a segment child
* @return {Segment} Returns null if a segment cannot be found
*/
DataTable.prototype.getSegmentParent = function(rid) {
	if(this._segments) {
		return this._segments.getSegmentParent(rid);
	}
	return null;
};
/** Segment level starts from 1
* @public
* @param {string} rid
* @return {number} Return 0 if the given rid is not a segment
*/
DataTable.prototype.getSegmentLevel = function(rid) {
	let segment = this.getSegment(rid);
	if(segment) {
		return segment.getSegmentLevel() + 1;
	}
	return 0;
};
/**
* @public
* @param {string} rid
* @return {string} parent row id of this segmentation. If the parent row id for this segmentation cannot be found, return ""
*/
DataTable.prototype.getSegmentParentRowId = function(rid) {
	if(this._segments) {
		return this._segments.getParentRowId(rid);
	}
	return "";
};
/**
* @public
* @param {Array.<string>=} rids If no row id is given, row ids of this data table is used instead
* @param {boolean=} partial Indicating that the given ids are not the whole list
* @return {Array.<number>} Return null if there is no segmentation
*/
DataTable.prototype.getSegmentValues = function(rids, partial) {
	if(this._segments) {
		this._segments.calcSegmentOrder(this._rids, true); // use cache
		return this._segments.getSegmentValues(rids || this._rids, partial);
	}
	return null;
};
/**
* @private
* @param {number} at Row index
* @return {boolean} Return true if the given row is in the last segment
*/
DataTable.prototype._isLastSegment = function(at) {
	let segmentSeparators = this._segments;
	if(segmentSeparators) {
		let rids = this._rids;
		let rowCount = rids.length;
		for(let r = at + 1; r < rowCount; ++r) {
			if(segmentSeparators.containsSegment(rids[r])) {
				return false;
			}
		}
	}
	return true;
};
/**
* @public
* @ignore
* @return {SegmentCollection} Return null if there is no segmentation
*/
DataTable.prototype._getSegmentSeparators = function() {
	return this._segments;
};
/** @public
* @param {boolean=} enabled
* @ignore
*/
DataTable.prototype.enableAutoSegmentFilling = function(enabled) {
	this._autoSegmentFilling = enabled != false;
	this._autoFillSegments();
};
/** @private
* @return {boolean} Return true if the fill segments have changed, otherwise return false
*/
DataTable.prototype._autoFillSegments = function() {
	if(this._autoSegmentFilling) {
		return this.fillSegments();
	}
	return false;
};
/** Remove existing segment children and fill the segments with all content rows before the next segment separator
* @public
* @param {string} segmentId Row id
*/
DataTable.prototype.fillSegment = function(segmentId) {
	if(this._segments) {
		this._segments.fillSegment(segmentId, this._rids);
		this.dispatchGlobalChange();
		this.requestClassifying();
	}
};
/** Remove all existing segment children in each segment and fill the segments with all contnet rows before the next segment separator
* @public
* @return {boolean} Return true if the fill segments have changed, otherwise return false
*/
DataTable.prototype.fillSegments = function() {
	if(this._segments) {
		let dirty = this._segments.fillSegments(this._rids);
		if(dirty) {
			this.dispatchGlobalChange();
			this.requestClassifying();
		}
		return dirty;
	}
	return false;
};
/** @private
* @param {boolean=} adding This indicates that segment is changed by adding a new child
*/
DataTable.prototype._onSegmentChildChanged = function(adding) {
	if(adding !== false) {
		this._sort(null);
	}
	this._dispatchPositionChange(); // Force rerendering, even if there is no position change

	this.requestClassifying();
};
/** @public
* @param {string} segmentId Row id
* @param {string} rid Row id
* @param {string=} dataId Row id for retrieving data
* @return {boolean} Return true if there is any change
*/
DataTable.prototype.addSegmentChild = function(segmentId, rid, dataId) {
	if(this._segments) {
		let dirty = this._segments.addSegmentChild(segmentId, rid, dataId);
		if(dirty) {
			this._onSegmentChildChanged();
			return true;
		}
	}
	return false;
};
/** @public
* @param {string} segmentId Row id
* @param {Array.<string>} rids Row id
* @param {Array.<string>=} dataIds Row ids for retrieving data
* @return {boolean} Return true if there is any change
*/
DataTable.prototype.addSegmentChildren = function(segmentId, rids, dataIds) {
	if(this._segments) {
		let dirty = this._segments.addSegmentChildren(segmentId, rids, dataIds);
		if(dirty) {
			this._onSegmentChildChanged();
			return true;
		}
	}
	return false;
};

/** Deprecated. Remove all existing children and add new children according to the give array
* @public
* @param {Array.<Object>} segmentArr Segment array that contain "segmentId", "rowIds" to set segment children
* @return {boolean} Return true if there is any change
*/
DataTable.prototype.setSegmentChildren = function(segmentArr) {
	if(!this._segments) {
		return false;
	}
	let dirty = this._segments.removeAllSegmentChildren();
	let len = segmentArr.length;
	for (let i = 0; i < len; i++) {
		let obj = segmentArr[i];
		if(this._segments.addSegmentChildren(obj["segmentId"], obj["rowIds"])) {
			dirty = true;
		}
	}
	if(dirty) {
		this._onSegmentChildChanged();
		return true;
	}

	return false;
};
/** @public
* @param {string} segmentId Row id
* @param {string} rid Row id
* @return {boolean} Return true if there is any change
*/
DataTable.prototype.removeSegmentChild = function(segmentId, rid) {
	if(this._segments) {
		let dirty = this._segments.removeSegmentChild(segmentId, rid);
		if(dirty) {
			this._onSegmentChildChanged(false);
		}
		return dirty;
	}
	return false;
};
/** @public
* @param {string} segmentId Row id
* @param {Array.<string>} rids Row id
* @return {boolean} Return true if there is any change
*/
DataTable.prototype.removeSegmentChildren = function(segmentId, rids) {
	if(this._segments) {
		let dirty = this._segments.removeSegmentChildren(segmentId, rids);
		if(dirty) {
			this._onSegmentChildChanged(false);
		}
		return dirty;
	}
	return false;
};
/** @public
* @return {boolean} Return true if there is any change
*/
DataTable.prototype.removeAllSegmentChildren = function() {
	if(this._segments) {
		let dirty = this._segments.removeAllSegmentChildren(); // This immediately remove all sub segments
		if (dirty) {
			this._onSegmentChildChanged(false);
		}
		return dirty;
	}
	return false;
};
/** Get row ids of all segment separator
* @public
* @return {Array.<string>} Return null if there is no segment separator
*/
DataTable.prototype.getSegmentIds = function() {
	if(this._segments) {
		return this._segments.getSegmentIds();
	}
	return null;
};
/** Get row ids of all children from the specified segment
* @public
* @param {string} segmentId
* @return {Array.<string>} Returns array of row ids. Returns null if there is no child or no segment separator found
*/
DataTable.prototype.getSegmentChildIds = function(segmentId) {
	if(this._segments) {
		let segment = this._segments.getSegment(segmentId);
		if(segment) {
			let chdr = segment.getChildIds();
			return chdr.length ? chdr : null;
		}
	}
	return null;
};
/**
* @private
* @param {(Array|number)} sortOrder
* @param {*} context
*/
DataTable.prototype._setSegmentSortContext = function (sortOrder, context) {
	this._segmentSortOrder = sortOrder;
	this._segmentSortContext = context;
};
/** Sort all of existing segments by multiple sort logics
* @public
* @param {Function|Array.<Function>|Object} sortLogics
* @param {Array.<number|string>=} sortOrders
* @param {Array.<string>=} cids
* @return {boolean}
*/
DataTable.prototype.sortSeparators = function (sortLogics, sortOrders, cids) {
	if(!this._segments){
		return false;
	}
	if(typeof sortLogics === "function"){
		this._setSegmentSortContext(DataTable._getSortOrder(sortOrders), cids);
		return this.sortSegments(sortLogics);
	}
	if(!cids) {
		return false;
	}
	let sortingDefs = DataTable._buildSortContext(
		[],
		cids,
		sortOrders,
		sortLogics || this._compMap
	);
	let defCount = sortingDefs.length;
	let sortLogic = null;
	if(defCount > 1) {
		sortLogic = DataTable._multiColumnSeparatorCompareLogic;
		this._setSegmentSortContext(1, sortingDefs);
	} else if(defCount === 1) { // Single level sorting
		sortLogic = DataTable._singleColumnSeparatorCompareLogic;
		this._setSegmentSortContext(sortingDefs[0][3], sortingDefs[0]); // sortOrder and context
	} else {
		return false;
	}
	return this.sortSegments(sortLogic);
};
/** Sort all of existing segments by given compare function
* @public
* @param {Function} compare
* @return {boolean}
*/
DataTable.prototype.sortSegments = function (compare) {
	if(!this._segments) {
		return false;
	}
	if(typeof compare !== "function") {
		return false;
	}

	this._userSegmentComparer = compare;

	let segments = this._segments;
	let rootSegments = segments.sortSegments(this._bySegmentSeparator);

	this._userSegmentComparer = null;
	this._setSegmentSortContext(0, null);

	let rids = this._rids;
	let origRids = rids.slice();
	let rowCount = rids.length;
	rids.length = 0;

	let segmentCount = 0;
	let dirty = false;
	for(let i = 0; i < rowCount; ++i) {
		let rid = origRids[i];
		let segment = segments.getSegment(rid);
		if(segment) {
			if(segment.isRootSegment()) {
				rootSegments[segmentCount++].getFlattenTreeIds(rids);
			}
		} else if(!segments.getParentRowId(rid)) {
			rids.push(rid);
		}
		if(rid !== rids[i]) {
			dirty = true;
		}
	}

	if(dirty) {
		this._sort(null);
		this._dispatchPositionChange();
	}
	return dirty;
};
/** Sort all of existing segments by given compare function
* @private
* @param {*} rowA
* @param {*} rowB
* @return {number}
*/
DataTable.prototype._bySegmentSeparator = function (rowA, rowB) {
	return /** @type{number} */(this._userSegmentComparer(
		this._getRowDataFromSegment(rowA),
		this._getRowDataFromSegment(rowB),
		this._segmentSortOrder,
		this._segmentSortContext
	));
};
/** Get row data from either Segment or row id
* @private
* @param {*} obj
* @return {Object}
*/
DataTable.prototype._getRowDataFromSegment = function (obj) {
	var rowData = null;
	if(obj) {
		if(typeof obj == "string") {
			rowData = this.getRowData(obj);
		} else if(obj.getId) {
			rowData = this.getRowData(obj.getId());
		}
	}
	return rowData;
};

/** A data source to be used with segment classification
* @public
* @param {DataCache} dc
*/
DataTable.prototype.setClassificationSource = function(dc) {
	this._clsSource = null;

	if(dc && dc._rows) {
		this._clsSource = dc._rows;
	}
};
/**
* @public
* @param {string} segmentId
* @param {string|Array.<string>} fields
* @return {boolean}
*/
DataTable.prototype.setSegmentClassification = function(segmentId, fields) {
	if(this._segments) {
		let dirty = this._segments.setSegmentClassification(segmentId, fields);
		if(dirty) {
			return this.classifySegments();
		}
	}
	return false;
};
/** @public
* @return {boolean}
*/
DataTable.prototype.classifySegments = function() {
	if(this._segments) {
		if(this._rids.length) {
			return this._segments.classify(this._clsSource || this._rows);
		}
	}
	return false;
};

/** @public
*/
DataTable.prototype.requestClassifying = function() {
	if(this._segments && !this._classifyingTimer) {
		if(this._segments.hasClassification()) {
			this._classifyingTimer = setTimeout(this._onClassifyingTimer, 10);
		}
	}
};
/** @private
*/
DataTable.prototype._onClassifyingTimer = function() {
	this._classifyingTimer = 0;
	this.classifySegments();
};
/** @private
* @param {Object} e
*/
DataTable.prototype._onSubSegmentChanged = function(e) {
	let insertionList = /** @type{Array.<Segment>} */(e["insertionList"]);
	let removalList = /** @type{Array.<string>} */(e["removalList"]);

	let dirty = false;
	let rows = this._rows;
	let clsSource = this._clsSource || rows;
	let rids = this._rids;
	let prevData = this._prevData;
	let i;

	let removalCount = removalList.length;
	let removedRows = {};
	for(i = 0; i < removalCount; i++) {
		let rid = removalList[i];
		if(rows[rid]) {
			removedRows[rid] = prevData[rid] = rows[rid];
			delete rows[rid];
			DataTable._removeArrayItem(rids, rid);
			dirty = true;
			// TODO: Remove classification source
		}
	}

	let insertionCount = insertionList.length;
	for(i = 0; i < insertionCount; i++) {
		let segment = insertionList[i];
		let parentId = segment.getParentId();
		let segmentId = segment.getId();

		if(!rows[segmentId]) {
			prevData[segmentId] = {};
			segment.setRowData(rows, clsSource);

			let parentAt = rids.indexOf(parentId);
			if(parentAt < 0 || parentAt + 1 >= rids.length) {
				rids.push(segmentId);
			} else {
				rids.splice(parentAt + 1, 0, segmentId);
			}
			dirty = true;
		}
	}
	if(dirty) {
		e["removedRows"] = removedRows;
		this._dispatch("subSegmentChanged", e);
		this.dispatchGlobalChange(); // Rerender everything
	}
};

/**
* @public
* @param {boolean} bool
* @return {boolean}
*/
DataTable.prototype.setSegmentDefaultCollapsing = function(bool) {
	if(this._segmentDefaultCollapsing !== bool) {
		this._segmentDefaultCollapsing = bool;
		if(this._segments) {
			this._segments.setDefaultCollapsing(bool);
		}
		return true;
	}
	return false;
};


/**
* @public
* @ignore
* @param {Array.<Array>} sortingDefs
* @return {boolean} Return true if there is any change, otherwise false
*/
DataTable.prototype._sort = function(sortingDefs) {
	let rids = this._rids;
	let dataRows = this.getMultipleRowData(rids);
	let segmentValues = this.getSegmentValues(rids);
	let change = DataTable._performMultiLevelSorting(dataRows, sortingDefs, segmentValues);

	if(change) {
		this._rids = DataTable._reorderItems(rids, change);
		return true;
	}
	return false;
};

/** Simulate data changed event by firing global change event argument to all listeners<br>
* Normally this will force data view to do sorting, filtering, and grouping again
* @public
* @param {boolean=} suppressEvent
* @fires DataTable#dataComposed
* @fires DataTable#dataChanged
*/
DataTable.prototype.dispatchGlobalChange = function(suppressEvent) {
	if(!suppressEvent && this._needFiring()) {
		this._dispatchDataChange(DataTable._globalChangeArg);
	}
};
/** Position change will not cause dataComposed to be fired
* @private
* @param {boolean=} suppressEvent
* @fires DataTable#dataChanged
*/
DataTable.prototype._dispatchPositionChange = function(suppressEvent) {
	if(!suppressEvent && this._needFiring()) {
		this._dispatch("dataChanged", DataTable._positionChangeArg);
	}
};

/** {@link DataCache#dump} Construct a JSON object that is compatible with console.table()
* @override
* @param {Object=} opt_options {@link DataCache#dump}
* @return {!Array.<Object>} Return a JSON object that is compatible with console.table()
* @see {@link DataCache#dump}
*/
DataTable.prototype.dump = function(opt_options) {
	return data_DataCache.constructTable(this.getMultipleRowData(this._rids), opt_options, this._rids);
};


/** @private
* @param {string} rid
* @param {string} cid
* @param {*} value
* @return {boolean} Return true if there is any change, and false otherwise
*/
DataTable.prototype._setData = function(rid, cid, value) {
	let row = this.getRowData(rid);

	if(!row) {
		row = {};
		this._rows[rid] = row;
		this._prevData[rid] = {};
		this._rids.push(rid);
	}
	this._prevData[rid][cid] = row[cid];
	row[cid] = value;
	return true;
};
/** @private
* @param {number} len
* @param {Array.<string>=} opt_ridList
* @returns {Array.<string>}
*/
DataTable.prototype._generateRids = function(len, opt_ridList) {
	let r = 0, rids;
	if(opt_ridList) {
		rids = opt_ridList;
	} else {
		rids = [];
		let rowCount = this._rids.length;
		while(r < rowCount && r < len) {
			rids.push(this._rids[r++]);
		}
	}
	r = rids.length;
	while(r < len) {
		rids.push("" + r++); // Auto-generate row id
	}
	return rids;
};
/** @private
* @param {Object} values
* @param {string|null=} rid
* @return {string}
*/
DataTable.prototype._generateUniqueRid = function(values, rid) {
	if(!rid) {
		if(values["ROW_ID"]) {
			return values["ROW_ID"];
		}
		rid = "_" + this._runningId++;
		while(this._rows[rid]) {
			rid = "_" + this._runningId++;
		}
	}
	return rid;
};

/** @private
* @return {boolean}
*/
DataTable.prototype._needFiring = function() {
	return !this._frozen && !this._composing;
};

/** Deprecated
* @deprecated
* @public
* @function
*/
DataTable.getSortingDefinitions = util._deprecatedFunction;
/** @public
* @const
* @ignore
* @type {!Object}
*/
DataTable._globalChangeArg = {"globalChange": true};
/** @public
* @const
* @ignore
* @type {!Object}
*/
DataTable._positionChangeArg = {"globalChange": true, "positionChangeOnly": true};


/**
* @function
* @public
* @ignore
* @param {!Array.<Array>} out_defs Output object. Array is used to optimize property accessing time.
* @param {string|Array.<string>} cids
* @param {string|number|Array.<string|number>=} sortOrders
* @param {Function|Array.<Function>|Object.<string, DataTable.SortLogic>=} logics
* @param {*=} contextObj Context object that will be provided as the forth parameter of the given comparer method
* @return {!Array.<Array>}
*/
DataTable._buildSortContext = function(out_defs, cids, sortOrders, logics, contextObj) {
	let aryColumnIds, aryOrders;
	if(Array.isArray(cids)) {
		aryColumnIds = /** @type{Array.<string>} */(cids);
		aryOrders = /** @type{Array.<string|number>} */(sortOrders);
	} else {
		aryColumnIds = [/** @type{string} */(cids)];
		aryOrders = (cids) ? [/** @type{string|number} */(sortOrders)] : [0];
	}

	let logic = null;
	let logicMap = null;
	if(typeof logics == "function") {
		logic = logics;
	} else {
		logicMap = /** @type{Array.<Function>|Object.<string, DataTable.SortLogic>} */(logics);
	}
	let len = aryColumnIds.length;
	out_defs.length = len;
	for(let c = len; --c >= 0;) {
		if (Array.isArray(logicMap)) {
			logic = logicMap[c];
		}
		let sortingDef = out_defs[c];
		if(!sortingDef) {
			sortingDef = out_defs[c] = new Array(5);
		}
		let columnId = aryColumnIds[c];
		sortingDef[0] = columnId; // text field
		sortingDef[1] = null; // Reserved for row data
		sortingDef[2] = logic || logicMap[columnId] || logicMap["_default"]; // sort logic
		if(!sortingDef[2]) {
			sortingDef[2] = DataTable._defaultComparer;
		}
		sortingDef[3] = DataTable._getSortOrder(aryOrders[c]); // sort order
		sortingDef[4] = contextObj;
	}
	return out_defs;
};
/** Performance is extremely vital in this method
* @public
* @ignore
* @function
* @param {number} a
* @param {number} b
* @param {number} order Not used by in this method
* @param {Array.<Array>} sortingDefs
* @return {number}
*/
DataTable._multiColumnCompareLogic = function(a, b, order, sortingDefs) {
	let count = sortingDefs.length;
	let result = 0;
	for(let c = 0; c < count; ++c) {
		let sortingDef = sortingDefs[c];
		result = DataTable._singleColumnCompareLogic(a, b, sortingDef[3], sortingDef);
		if(result) {
			return result;
		}
	}
	return result;
};
/** Performance is extremely vital in this method
* @public
* @ignore
* @function
* @param {number} a
* @param {number} b
* @param {number} order
* @param {Array} sortingDef
* @return {number}
*/
DataTable._singleColumnCompareLogic = function(a, b, order, sortingDef) {
	let values = /** @type{Array} */(sortingDef[1]);
	return /** @type{number} */(sortingDef[2](
		values[a], // Value1
		values[b], // Value2
		order, // Sort order (3)
		sortingDef[4] // Context object
	));
};
/** Performance is extremely vital in this method
* @public
* @ignore
* @function
* @param {*} a
* @param {*} b
* @param {number} order Not used by in this method
* @param {Array.<Array>} sortingDefs
* @return {number}
*/
DataTable._multiColumnSeparatorCompareLogic = function(a, b, order, sortingDefs) {
	let count = sortingDefs.length;
	let result = 0;
	for(let c = 0; c < count; ++c) {
		let sortingDef = sortingDefs[c];
		result = DataTable._singleColumnSeparatorCompareLogic(a, b, sortingDef[3], sortingDef);
		if(result) {
			return result;
		}
	}
	return result;
};
/** Performance is extremely vital in this method
* @public
* @ignore
* @function
* @param {*} a
* @param {*} b
* @param {number} order
* @param {Array} sortingDef
* @return {number}
*/
DataTable._singleColumnSeparatorCompareLogic = function(a, b, order, sortingDef) {
	let key = /** @type{string} */(sortingDef[0]);
	return /** @type{number} */(sortingDef[2](
		a[key], // Value1
		b[key], // Value2
		order, // Sort order (3)
		sortingDef[4] // Context object
	));
};
/** @public
* @function
* @ignore
* @param {!Array.<Object>} dataRows
* @param {Array.<Array>} sortingDefs
* @param {Array.<number>=} segmentValues
* @return {Array.<number>} Returns new item positions if there is any change, otherwise returns null
*/
DataTable._performMultiLevelSorting = function(dataRows, sortingDefs, segmentValues) {
	let rowCount = dataRows ? dataRows.length : 0;
	let defCount = sortingDefs ? sortingDefs.length : 0;
	if(!rowCount) {
		return null;
	}

	let sortLogic;
	// Populate data for each definition
	for(let c = 0; c < defCount; ++c) {
		let sortingDef = sortingDefs[c];
		let rows = sortingDef[1];
		sortLogic = sortingDef[2];
		let rowSorting = sortLogic["rowSorting"];
		let r;
		if(rowSorting) {
			if(rows) {
				rows.length = rowCount;
				for(r = 0; r < rowCount; ++r) {
					rows[r] = dataRows[r];
				}
			} else {
				rows = sortingDef[1] = dataRows;
			}
		}	else {
			if(rows) {
				rows.length = rowCount;
			} else {
				rows = sortingDef[1] = new Array(rowCount);
			}
			let columnId = sortingDef[0];
			for(r = 0; r < rowCount; ++r) {
				rows[r] = dataRows[r][columnId];
			}
		}
	}

	if(segmentValues) {
		let segmentDef = [ // TODO: Make this static variable
			"ROW_SEGMENT", // Dummy column id
			segmentValues, // Segment values
			DataTable._segmentComparer,
			1, // Ascending order
			null // Context object
		];
		if(sortingDefs) {
			sortingDefs = sortingDefs.slice();
			sortingDefs.unshift(segmentDef);
		} else {
			sortingDefs = [segmentDef];
		}
		++defCount;
	}

	if(!defCount) {
		return null;
	}

	let sortOrder = 0;
	let sortContext;
	if(defCount > 1) {
		sortLogic = DataTable._multiColumnCompareLogic;
		sortContext = sortingDefs;
		sortOrder = 1; // sortOrder is not used by _multiColumnCompareLogic
	} else { // Single level sorting
		sortLogic = DataTable._singleColumnCompareLogic;
		sortContext = sortingDefs[0];
		sortOrder = /** @type{number} */(sortContext[3]);
	}

	return DataTable._mergeSort(rowCount, sortOrder, sortLogic, sortContext);
};
/**
* @function
* @public
* @ignore
* @param {number} len Length of array item
* @param {number} order
* @param {DataTable.SortLogic} comparer
* @param {*=} contextObj Context object that will be provided as the forth parameter of the given comparer method
* @return {Array.<number>} Returns new item positions if there is any change, otherwise returns null
*/
DataTable._mergeSort = function(len, order, comparer, contextObj) {
	let change = false;

	let grpSize = 1;
	let iter, lhsFrom, rhsFrom, lhsTo, rhsTo;
	let subj = DataTable._createIndexArray(len);
	let obj = subj.slice();

	while(grpSize <= len) {
		iter = rhsTo = 0; // start from 0
		while(iter < len) {
			lhsFrom = rhsTo;
			lhsTo = lhsFrom + grpSize;
			rhsFrom = lhsTo;
			rhsTo = rhsFrom + grpSize;
			if(rhsTo > len) rhsTo = len;

			if(rhsFrom < rhsTo) {
				while(true) { // Merge two groups together
					if(comparer(subj[lhsFrom], subj[rhsFrom], order, contextObj) <= 0) {
						obj[iter++] = subj[lhsFrom];
						if(++lhsFrom >= lhsTo) {
							while(rhsFrom < rhsTo) {
								obj[iter++] = subj[rhsFrom];
								++rhsFrom;
							}
							break;
						}
					} else {
						change = true;
						obj[iter++] = subj[rhsFrom];
						if(++rhsFrom >= rhsTo) {
							while(lhsFrom < lhsTo) {
								obj[iter++] = subj[lhsFrom];
								++lhsFrom;
							}
							break;
						}
					}
				}
			} else {
				while(iter < len) {
					obj[iter] = subj[iter];
					++iter;
				}
			}
		}

		let tmp = subj;
		subj = obj;
		obj = tmp;
		grpSize <<= 1;
	}
	return change ? subj : null;
};
/** reorderItems(["a", "b", "c", "d"], [1, 3, 0, 2]) === ["b", "d", "a", "c"]; (Move the first item to where its corresponding index is) <br>
* Not in-place reordering. Orders array will become the results
* @function
* @public
* @ignore
* @param {!Array} ary
* @param {Array} orders
* @return {!Array}
*/
DataTable._reorderItems = function (ary, orders) {
	if(orders) {
		for(let i = ary.length; --i >= 0;) {
			orders[i] = ary[orders[i]];
		}
		return orders;
	}
	return ary;
};
/** Create array of incremental series starting from 0 (e.g. [0, 1, 2, 3, ...])
* @private
* @function
* @ignore
* @param {number} len Length of the generated array
* @return {!Array.<number>}
*/
DataTable._createIndexArray = function(len) {
	let ary = DataTable._idxAryMap[len];
	if(!ary) {
		if(DataTable._idxAryCount > 50) { // Clear cache to avoid caching huge memory
			DataTable._idxAryMap = {};
			DataTable._idxAryCount = 0;
		}
		ary = DataTable._idxAryMap[len] = new Array(len);
		DataTable._idxAryCount++;

		for(let i = 0; i < len; ++i) {
			ary[i] = i;
		}
	}
	return ary.slice(); // Fastest way to clone an array is to perform slice() method
};
/** Remove _idxAryMap and _idAryCount
* @private
*/
DataTable._removeIndexArray = function() {
	DataTable._idxAryMap = {};
	DataTable._idxAryCount = 0;
};
/** @private
* @type {Object.<number, Array.<number>>}
*/
DataTable._idxAryMap = {};
/** @private
* @type {number}
*/
DataTable._idxAryCount = 0;

/** @private
* @function
* @param {number} a
* @param {number} b
* @return {number}
*/
DataTable._segmentComparer = function(a, b) {
	if(a === b) {
		return 0;
	}
	return (a < b) ? -1 : 1;
};
/** @private
* @function
* @param {*} a
* @param {*} b
* @param {number} order
* @return {number}
*/
DataTable._defaultComparer = function(a, b, order) { // User data may contain invalid value such as undefined or null
	if(a == null || a !== a) {
		if(b == null || b !== b) {
			return 0;
		}
		return 1;
	}
	if(b == null || b !== b) {
		return -1;
	}

	if(a < b) {
		return -order;
	}
	if(b < a) {
		return order;
	}
	return 0;
};
/** @private
* @function
* @param {string|number=} sortOrder
* @returns {number}
*/
DataTable._getSortOrder = function(sortOrder) {
	if(typeof sortOrder === "number") {
		return sortOrder;
	} else if(sortOrder && typeof sortOrder === "string") {
		sortOrder = sortOrder.charAt(0).toLowerCase();
		if(sortOrder === "a") {
			return 1;
		} else if(sortOrder === "d") {
			return -1;
		}
	}
	return 0;
};

DataTable._proto = DataTable.prototype;

/* harmony default export */ const data_DataTable = (DataTable);


;// ./src/js/RowDefinition.js

/* eslint-disable */


/* eslint-enable */

/** @typedef {Object} RowDefinition~Options
* @description RowDefinition options
* @property {string=} ric="" RIC for real-time fields. Prefixing 0# to the ric is equivalent to specifying asChain property.
* @property {string=} permId="" Organization PermId to be used instead of ric for private company.
* @property {(Array|Object)=} values=null Values for static data
* @property {Array.<string>=} fields=null Field that corresponds to the given static values
* @property {boolean=} asChain=false The given ric will be treated as a chain
* @property {string=} chainRic="" RIC to be used for chain request (overiding ric property)
* @property {boolean=} collapsed Chain is collapsed by default. Segment is expanded by default.
* @property {(string|null)=} label=null
* @property {boolean=} hidden=true When this row is hidden
* @property {boolean=} realTime=true Realtime row, able to request for JET/RTK
* @property {Object=} info=null For storing any additional information to the row
* @property {string=} rowId Row identifier used for referencing the row. The value cannot be in "_x_" format.
*/

/** @typedef {Object} RowDefinition~RowTypes
* @property {string} CONTENT="CONTENT" Regular row
* @property {string} CHAIN="CHAIN" Chain RIC row
* @property {string} CONSTITUENT="CONSTITUENT" Constituent row
* @property {string} GROUP_HEADER="GROUP_HEADER" Row of segment separator at the very top
* @property {string} SUBGROUP_HEADER="SUBGROUP_HEADER" Row of nested segment separator
* @property {string} GROUP_MEMBER="GROUP_MEMBER" Child row displayed under segment separator
*/

/** @type {string}
* @public
* @const
*/
const ROW_DEF = "ROW_DEF";

/** @type {!RowDefinition~RowTypes}
* @public
* @const
*/
const ROW_TYPES = {
	CONTENT: "CONTENT",
	CHAIN: "CHAIN",
	CONSTITUENT: "CONSTITUENT",
	GROUP_HEADER: "GROUP_HEADER",
	SUBGROUP_HEADER: "SUBGROUP_HEADER",
	GROUP_MEMBER: "GROUP_MEMBER"
};

/** @private
* @function
* @param {Object} obj
* @returns {!Object} Always returns a new object
*/
let _cloneObject = function(obj) {
	let newObj = {};
	for(let key in obj) {
		newObj[key] = obj[key];
	}
	return newObj;
};

/** @constructor
* @param {RowDefinition~Options=} rowOptions
*/
let RowDefinition = function(rowOptions) {
	this._changes = {};
	if(rowOptions && rowOptions["segmentId"]) { // This row will be classified header row (subsegment header row)
		this._rowId = rowOptions["segmentId"];
		this._autoGenerated = true;
		this._subSegment = true;
		return;
	}

	this._rowId = "_" + RowDefinition._runningId++ + "_";

	if(rowOptions) {
		if(rowOptions["asConstituent"]) { // Create row definition as a constituent of a chain
			this._initializeAsConstituent(rowOptions); // WARNING: This does not follow normal initialization process
		} else {
			this.initialize(rowOptions);
		}
	}
};

/** @type {number}
* @private
*/
RowDefinition._runningId = 0;
/** @type {string}
* @private
*/
RowDefinition._childDataField = "CHILD_VALUES";
/** @type {RegExp}
* @private
* @const
*/
RowDefinition.ROW_ID_PATTERN = /^_[^_]+_$/;
//#region Private Members
/** @type {string}
* @private
*/
RowDefinition.prototype._userInput = "";
/** @type {string}
* @private
*/
RowDefinition.prototype._ric = ""; // Contains no 0#
/** @type {string}
* @private
*/
RowDefinition.prototype._permId = "";
/** @type {string|null}
* @private
*/
RowDefinition.prototype._label = null; // Label overrides _ric and _permId
/** @type {boolean|null}
* @private
*/
RowDefinition.prototype._isChain = null;
/** @type {boolean}
* @private
*/
RowDefinition.prototype._asSegment = false;
/** @type {boolean}
* @private
*/
RowDefinition.prototype._realTime = true;

/** @type {string}
* @private
*/
RowDefinition.prototype._rowId = ""; // Row Id must be unique
/** @type {boolean}
* @private
*/
RowDefinition.prototype._userId = false;
/** @type {DataCache}
* @private
*/
RowDefinition.prototype._dc = null;
/** Quotes2 subscription object
* @type {Object}
* @private
*/
RowDefinition.prototype._subs = null;
/** @type {DataView}
* @private
*/
RowDefinition.prototype._view = null;

/** @type {string}
* @private
*/
RowDefinition.prototype._subId = "";

/** @type {boolean|null}
* @private
*/
RowDefinition.prototype._collapsed = null;
/** @type {boolean}
* @private
*/
RowDefinition.prototype._autoGenerated = false;
/** @type {boolean}
* @private
*/
RowDefinition.prototype._subSegment = false;
/** @type {Object.<string, *>}
* @private
*/
RowDefinition.prototype._staticValues = null;
/** @type {!Object.<string, number>}
* @private
*/
RowDefinition.prototype._changes;
/** @type {number}
* @private
*/
RowDefinition.prototype._updateCount = 0;

/** @type {Array.<RowDefinition>}
* @private
*/
RowDefinition.prototype._children = null;
/** @type {RowDefinition}
* @private
*/
RowDefinition.prototype._parent = null;
/** @type {number}
* @private
*/
RowDefinition.prototype._groupOrder = 0; // For group sorting
/** @type {number}
* @private
*/
RowDefinition.prototype._depthLevel = 0;

/** @type {Object}
* @private
*/
RowDefinition.prototype._userModel = null;
/** @type {Object}
* @private
*/
RowDefinition.prototype._info = null;

//#endregion Private Members

/** @public
*/
RowDefinition.prototype.dispose = function() {
	if(this.isDisposed()) {
		return;
	}

	if(this._children) {
		this._children.forEach(RowDefinition.dispose);
		this._children = null;
	}

	if(this._dc) {
		this._dc.setRowData(this._rowId, null); // Clear all data held by this row def
		this._dc = null;
	}
	this._staticValues = null; // Static values contain user data
	this._parent = null; // WARNING: This does not impact children count of the parent. A child must be destroyed by its parent only
	this._depthLevel = 0;
	this.unsubscribeForUpdates();
	this._subs = null;
	this._view = null; // WARNING: This does not remove this row from the UI view

	this._subId = this._rowId = "";
	this._userId = false;
	this._autoGenerated = false;
	this._userModel = null;
};
/** @public
* @return {boolean}
*/
RowDefinition.prototype.isDisposed = function() {
	return !this._rowId;
};
/** @public
* @param {RowDefinition~Options=} rowOptions
*/
RowDefinition.prototype.initialize = function(rowOptions) {
	if(!rowOptions) {
		return;
	}

	if(!this._autoGenerated) {
		let userRowId = rowOptions["rowId"];
		if(userRowId && typeof userRowId === "string") {
			if(userRowId.match(RowDefinition.ROW_ID_PATTERN)) {
				console.warn("Please change the rowId format to avoid duplicated rows' id causing unexpected behavior.");
			} else {
				this._rowId = userRowId;
				this._userId = true;
			}
		}
	}

	let extractedOptions = RowDefinition.extractRowOptions(rowOptions);

	let val = rowOptions["permId"];
	if(val != null) {
		this._permId = val;
	}
	val = rowOptions["ric"];
	if(val != null) {
		this._ric = val;
	}
	val = rowOptions["label"];
	// eslint-disable-next-line no-undefined
	if(val !== undefined) { // Empty string and null are allowed
		this._label = val;
	}
	val = rowOptions["chainRic"];
	if(val != null) {
		this._chainRic = val;
	}

	val = extractedOptions["asChain"];
	if(val != null) {
		this._isChain = val ? true : false;
	}

	val = rowOptions["realTime"];
	if(val != null) {
		this._realTime = val;
	}

	val = rowOptions["asSegment"];
	if(val != null) {
		this._asSegment = val ? true : false;
	}

	if(this._isChain || this._asSegment) {
		this._collapsed = extractedOptions["collapsed"]; // Temporary state
	}

	val = rowOptions["keepModel"];
	if(val) {
		this._userModel = rowOptions;
	}
	val = rowOptions["info"];
	if(val) {
		this._info = val;
	}

	let symbol = this._ric || this._chainRic;
	if(symbol || this._permId){
		this.setContent(symbol, extractedOptions);
	}

	val = rowOptions["values"];
	// eslint-disable-next-line no-undefined
	if(val !== undefined) {
		this.setStaticRowData(val, rowOptions["fields"]);
	}
};
/** @description Constituents will have subscription id, but have no subscription. It's row id will be different from subscription id. It will have a reference to its parent. It cannot be generated by themselve -- always be autogenerated.
* @private
* @param {!Object} rowOptions
*/
RowDefinition.prototype._initializeAsConstituent = function(rowOptions) {
	this._autoGenerated = true;
	let parentDef = /** @type{RowDefinition} */(rowOptions["parent"]);
	if(this.setParent(parentDef)) {
		this._ric = /** @type{string} */(rowOptions["ric"]);
		this._dc = parentDef._dc; // Parent chain must have data cache
		this._subId = parentDef._subId; // Child constituent uses the same subscription as its parent
	}
	this._staticValues = rowOptions["values"] || null;
	this.resetRowData();
};
/** @private
* @param {DataView} view
* @param {string} rowId
* @returns {string}
*/
let _getEndOfSegmentRowId = function(view, rowId) {
	let rowIndex = view.getRowIndex(rowId);
	do {
		rowId = view.getRowId(++rowIndex);
		if(rowId && !view.getSegmentParentRowId(rowId)) {
			break;
		}
	} while(rowId);
	return rowId;
};

/** @private
* @param {DataView} view
* @param {boolean} newState
* @param {boolean} prevState
* @returns {boolean} Current state
*/
let _stallSorting = function(view, newState, prevState) {
	if(view && view.isSorting()) {
		newState = newState ? true : false;
		if(newState !== prevState) {
			if(newState) {
				view.synchronizeRowOrder();
			}
			view.stallSorting(newState);
		}
		return newState;
	}
	return false;
};
/** @public
* @ignore
* @param {string} userInput RIC
* @param {Object} extractedOptions
* @return {boolean} True if there is any change otherwise false
*/
RowDefinition.prototype.setContent = function(userInput, extractedOptions) {
	if(this._autoGenerated) { // AutoGenerated RowDefinition cannot be changed by user input
		return false;
	}

	if(!userInput) { // 0 is not a valid input
		userInput = "";
	} else if(typeof userInput !== "string") {
		userInput += "";
	}

	let dirty = (this._userInput !== userInput);
	let permId = extractedOptions["permId"] || "";
	if(this._permId !== permId){
		this._permId = permId;
		dirty = true;
	}
	if(!dirty) {
		return false;
	}

	let asChain = extractedOptions["asChain"];
	let realtimeRow = true;
	if(userInput.charAt(0) === "'") { // Single quote is used as a marker for non realtime row
		realtimeRow = false;
		this._ric = this._chainRic = ""; // No ric for realtime request
	}

	let dv = this._view;
	let stalledSorting = _stallSorting(dv, realtimeRow && asChain, false); // To preserve current position of the segment/chain

	this.unsubscribeForUpdates(); // Static data remains intact

	this._userInput = userInput;
	let collapsed = extractedOptions["collapsed"];
	if(realtimeRow) {
		let chainRic = extractedOptions["chainRic"];
		if(asChain === true){
			this._ric = collapsed === true ? userInput : userInput.replace("0#", "");
		} else {
			this._ric = userInput;
		}
		this._isChain = asChain != null ? asChain : null; // this could be null or undefined
		this._chainRic = chainRic || "";
	}

	if(dv) {
		let segmentId = dv.getSegmentParentRowId(this._rowId);
		if(segmentId) {
			if(this._isChain){ // If the row was a normal row and has been changed to a chain, remove it from existing segment
				let targetRowId = _getEndOfSegmentRowId(dv, this._rowId);
				dv.removeSegmentChild(segmentId, this._rowId);
				dv.moveRow(this._rowId, targetRowId);

				segmentId = "";
			}
		} else if(dv.getSegment(this._rowId)) {
			dv.setSegmentSeparator(this._rowId, false); // Remove existing segment
		}
		if(this._isChain) {
			dv.setSegmentSeparator(this._rowId, true);
		}

		if(collapsed !== this.isChainCollapsed()) { // WARNING: This is not optimizsed
			dv.collapseSegment(this._rowId, collapsed);
		}
		this._collapsed = null;

		_stallSorting(dv, false, stalledSorting);
		if(segmentId) { // If data id is changed and the row is a child of a segment, then segment child data id must be updated
			dv.addSegmentChild(segmentId, this._rowId);
		}
	}

	this.subscribeForUpdates();
	return true;
};
/** @public
* @return {string} Row Id will never be empty string unless the row has been removed
*/
RowDefinition.prototype.getRowId = function() {
	return this._rowId;
};
/** @public
* @param {RowDefinition~Options=} rowOptions
* @return {RowDefinition~Options}
*/
RowDefinition.prototype.getConfigObject = function(rowOptions) {
	let obj = rowOptions || {};

	if(this._userId) {
		obj["rowId"] = this._rowId;
	}

	let val = this._ric;
	if(val) {
		obj["ric"] = val;
	}

	// The user passed a single quote (') case
	if(!this._ric && this._userInput.charAt(0) === "'") {
		obj["ric"] = this._userInput;
	}

	val = this._realTime;
	if(!val) {
		obj["realTime"] = false;
	}

	val = this._permId;
	if(val) {
		obj["permId"] = val;
	}

	val = this._chainRic;
	if(val) {
		obj["chainRic"] = val;
	}

	val = this._label;
	if(val) {
		obj["label"] = val;
	}

	val = this._userModel;
	if(val) {
		obj["keepModel"] = obj; // This is bad
	}

	val = this._isChain;
	if(val != null) {
		obj["asChain"] = val;
	}

	val = this._collapsed;
	if(val != null) {
		obj["collapsed"] = val;
	}

	val = this._view ? this._view.isHiddenRow(this._rowId) : false;
	if(val) {
		obj["hidden"] = val;
	}

	val = this._info;
	if(val) {
		obj["info"] = this._info;
	}

	val = this._cloneStaticRowData();
	if(val) {
		obj["values"] = val;
	}

	// obtain the static values of constituent rows
	if(this.isChain()) {
		let children = this.getChildren();
		if(children) {
			let childValues = val ? val[RowDefinition._childDataField] : {};
			if(!childValues) {
				childValues = {};
			}
			let dirty = false;
			let len = children.length;
			let i, rowDef, staticValues;
			for(i = 0; i < len; i++) {
				rowDef = children[i];
				staticValues = rowDef._cloneStaticRowData();
				if(staticValues) {
					dirty = true;
					childValues[rowDef.getRic()] = staticValues;
				}
			}

			if(dirty) {
				if(!obj["values"]) {
					obj["values"] = {};
				}
				obj["values"][RowDefinition._childDataField] = childValues;
			}
		}
	}

	return obj;
};
/** Data id is deprecated
* @public
* @return {string} Always return empty string
*/
RowDefinition.prototype.getDataId = function() {
	return "";
};
/** @public
* @return {string}
*/
RowDefinition.prototype.getType = function() {
	if(this._isChain) {
		return ROW_TYPES.CHAIN;
	} else if(this._parent) {
		return ROW_TYPES.CONSTITUENT;
	} else {
		let dv = this._view;
		if(dv) {
			let rid = this.getRowId();
			let separator = dv.isSegmentSeparator(rid);
			let level = dv.getSegmentLevel(rid);
			if(separator) {
				if(level === 1) {
					return ROW_TYPES.GROUP_HEADER;
				} else {
					return ROW_TYPES.SUBGROUP_HEADER;
				}
			} else {
				if(dv.getSegmentParentRowId(rid)) {
					return ROW_TYPES.GROUP_MEMBER;
				}
			}
		} else {
			return "";
		}
	}
	return ROW_TYPES.CONTENT;
};
/** This method should always be called right after the initialization
* @public
* @param {DataCache} dataSource
* @param {Object=} subs Quotes2 subscription object
* @return {boolean} Returns true if there is an data update
*/
RowDefinition.prototype.setDataSource = function(dataSource, subs) {
	this._dc = dataSource || null;
	let dataUpdated = false;
	if(this._dc) {
		if(this._staticValues) {
			let rowData = _cloneObject(this._staticValues);
			dataUpdated = this.addUpdate(rowData);
			this._dc.setRowData(this._rowId, rowData, {
				"rowDef": this, // The _onDataChanged and _onDataComposed handlers cannot get rowDef from DT because row does not register to view yet.
				"initialChanges": this.getUpdates() // Used by _onDataComposed to keep backward compatibility.
			}); // Trigger dataComposed and add updates
		} else {
			this._dc.setRowData(this._rowId, {});
		}
	}

	this._subs = subs || null;
	// This will work for runtime row insertion, but not for first initilization.
	this.subscribeForUpdates();
	return dataUpdated;
};
/** @public
* @return {DataCache} dataSource
*/
RowDefinition.prototype.getDataSource = function() {
	return this._dc;
};

/** @public
* @return {Object} rowData
*/
RowDefinition.prototype.getRowData = function() {
	return this._dc ? this._dc.getRowData(this._rowId) : {};
};
/** @public
* @param {string} field
* @return {*} data
*/
RowDefinition.prototype.getData = function(field) {
	return this._dc ? this._dc.getData(this._rowId, field) : null;
};

/** This method replaces all existing static data by the given data -- all existing static data are removed.
* @public
* @param {Object.<string, *>|Array} data
* @param {Array.<string>=} indexToFieldMap In case of the given data is an array, this param will be used for mapping index to field
*/
RowDefinition.prototype.setStaticRowData = function(data, indexToFieldMap) {
	this._clearStaticData();

	if(Array.isArray(data)) { // Convert array to object
		data = arrayToObject(data, indexToFieldMap);
	}

	this._staticValues = (typeof data === "object") ? data : null;

	if(this._staticValues) {
		this.setRowData(data);
	}
};
/** @private
* @return {Object.<string, *>}
*/
RowDefinition.prototype._cloneStaticRowData = function() {
	return this._staticValues ? _cloneObject(this._staticValues) : null;
};
/** @public
* @param {string} field
* @param {*} value
*/
RowDefinition.prototype.setStaticData = function(field, value) {
	if(!this._staticValues) {
		this._staticValues = {};
	}
	this._staticValues[field] = value;

	this.setData(field, value);
};
/** @public
* @param {Object.<string, *>|Array} data
* @param {Array.<string>=} indexToFieldMap In case of the given data is an array, this param will be used for mapping index to field
*/
RowDefinition.prototype.updateRowData = function(data, indexToFieldMap) {
	if(Array.isArray(data)) { // Convert array to object
		data = arrayToObject(data, indexToFieldMap);
	}

	if(data) {
		this.setRowData(data);
	}
};
/** @public
* @param {RowDefinition} srcRowDef Original source of row data to be copied from
*/
RowDefinition.prototype.copyRowData = function(srcRowDef) {
	if(srcRowDef && srcRowDef !== this) {
		let rowData = this.getRowData();
		srcRowDef.cloneRowData(rowData, this._staticValues);
	}
};
/** @public
 * @param {Object=} obj
 * @param {Object=} exceptionObj
 * @return {!Object}
*/
RowDefinition.prototype.cloneRowData = function(obj, exceptionObj) {
	let rowObj = obj || {};
	let field;
	let uncopiable_fields = {
		"ROW_DEF": 1,
		"SUB_ID": 1
	};
	let staticValues = this._staticValues;
	for(field in staticValues) {
		uncopiable_fields[field] = 1;
	}

	for(field in exceptionObj) {
		uncopiable_fields[field] = 1;
	}

	let rowData = this.getRowData();
	for(field in rowData) {
		if(!uncopiable_fields[field]) {
			rowObj[field] = rowData[field];
		}
	}
	return rowObj;
};
/** @public
* @param {Object.<string, *>} data
*/
RowDefinition.prototype.setRowData = function(data) {
	if(this._dc) {
		this._dc.setRowData(this._rowId, data);
	}
};
/** @public
*/
RowDefinition.prototype.resetRowData = function() {
	let dc = this._dc;
	if(dc) {
		let rowId = this._rowId;
		dc.setRowData(rowId, null);
		dc.setRowData(rowId, {}); // Row data is guaranteed to be existed
		if(this._staticValues) {
			dc.setRowData(rowId, this._staticValues);  // Trigger dataComposed and add updates
		}
	}
};
/** @public
* @param {string} field
* @param {*} value
*/
RowDefinition.prototype.setData = function(field, value) {
	if(this._dc) {
		this._dc.setData(this._rowId, field, value);
	}
};
/** @private
*/
RowDefinition.prototype._clearStaticData = function() {
	let staticValues = this._staticValues;
	if(staticValues) {
		if(this._dc) {
			let obj = {};
			for(let key in staticValues) {
				obj[key] = null;
			}
			this._dc.setRowData(this._rowId, obj);
		}
		this._staticValues = null;
	}
};

/** @public
* @return {string}
*/
RowDefinition.prototype.getUserInput = function() {
	return this._userInput;
};
/** @public
* @return {string}
*/
RowDefinition.prototype.getRic = function() {
	return this._ric;
};
/** @public
* @return {string}
*/
RowDefinition.prototype.getPermId = function() {
	return this._permId;
};
/** @public
* @return {string}
*/
RowDefinition.prototype.getSymbol = function() {
	return this._ric || this._permId;
};
/** @public
* @return {string}
*/
RowDefinition.prototype.getDisplayText = function() {
	if(this._label != null) { // Empty string is allowed
		return this._label;
	}

	if(this._ric) { // Use chainRic instead if this is a chain
		return this._ric;
	}

	if(this._permId) {
		return this._permId;
	}

	return this._userInput;
};
/** @public
* @return {string|null}
*/
RowDefinition.prototype.getLabel = function() {
	return this._label;
};

/** @public
* @return {boolean}
*/
RowDefinition.prototype.isRowHeader = function() {
	return (this._userInput.charAt(0) === "'");
};
/** @public
* @return {boolean}
*/
RowDefinition.prototype.isChain = function() {
	return this._isChain;
};
/** @public
* @return {boolean}
*/
RowDefinition.prototype.isConstituent = function() {
	return (this._autoGenerated && this._subId && this._rowId !== this._subId) ? true : false;
};
/** @public
* @function
* @param {RowDefinition} rowDef
* @return {boolean}
*/
RowDefinition.hasChain = function(rowDef) {
	return rowDef.isChain();
};
/** @public
* @return {boolean}
*/
RowDefinition.prototype.isChainCollapsed = function() {
	if(this._view) {
		return this._view.isSegmentCollapsed(this._rowId);
	}
	return false;
};
/** @public
* @return {boolean}
*/
RowDefinition.prototype.isChainExpanded = function() {
	return !this.isChainCollapsed();
};
/** @public
* @ignore
* @return {boolean}
*/
RowDefinition.prototype.isFixedPosition = function() {
	return this._isChain && this.isChainExpanded();
};
/** @public
* @return {boolean}
*/
RowDefinition.prototype.isAutoGenerated = function() {
	return this._autoGenerated;
};
/** @public
* @return {boolean}
*/
RowDefinition.prototype.isRealTimeRow = function() {
	if(!this.getRic()) { // Empty row
		return false;
	}
	if(!this._realTime) {
		return false;
	}

	if(this._autoGenerated) { // Constituents in chain are not real-time row
		return false;
	}

	if(this.isRowHeader()) {
		return false;
	}

	return true;
};

/** @public
* @param {Object=} subs
* @return {boolean} If a subscription is made, return true.
*/
RowDefinition.prototype.subscribeForUpdates = function(subs) {
	if(!(subs  || this._subs)) {
		return false;
	}
	if(!this.isRealTimeRow() && !this.getPermId()) {
		return false;
	}
	// TODO: Check if the same subscription is being made.
	this.unsubscribeForUpdates();
	if(subs) {
		this._subs = subs;
	} else {
		subs = this._subs;
	}
	if(this.isChain()) {
		let symbol = this._chainRic;
		if(!symbol){
			symbol = this._ric;
			if(symbol.indexOf("0#") < 0){
				let count = (symbol.match(/\./g) || []).length;
				if(count < 2){
					symbol = "0#" + symbol;
				}
			}
		}
		this._subId = subs["addChain"](symbol, this._rowId); // Some chain require 0# symbol to populate its constituents
	} else if(this._ric) {
		this._subId = subs["addRic"](this._ric, this._rowId);
	}

	let xRicName = this.getDisplayText(); // WARNING: This does not cover real-time X_RIC_NAME updates
	this.setRowData({"X_RIC_NAME": xRicName}); // Trigger data update immediately

	return true;
};
/** Unsubscribe existing real-time data service. Static data is maintained
* @public
* @param {boolean=} keepData
* @returns {null} Always return null
*/
RowDefinition.prototype.unsubscribeForUpdates = function(keepData) {
	if(this.isSubscribing()) { // Only normal real-time rows and chains have both subId and subscription object
		this._subs["removeSubscription"](this._subId);
		this._subId = "";
		if(!keepData) {
			this.resetUpdates();
			this.resetRowData(); // Real-time data is removed while static data is maintained
		}
	}
	return null;
};
/** @public
* @return {boolean}
*/
RowDefinition.prototype.isSubscribing = function() {
	return (this._subId && this._subs) ? true : false;
};
/** @public
* @return {string}
*/
RowDefinition.prototype.getSubId = function() {
	return this._subId;
};

/** Take event argument from streaming update with "changes" array property
* @public
* @param {Object} changes
* @return {boolean} Returns true if there is any change
*/
RowDefinition.prototype.addUpdate = function(changes) {
	if(changes) {
		let dirty = 0;
		for(let field in changes) {
			this._changes[field] = dirty = 1;
		}
		if(dirty) {
			++this._updateCount;
			return true;
		}
	}
	return false;
};
/** If there is no update, this method returns null
* @public
* @return {Object.<string, number>}
*/
RowDefinition.prototype.getUpdates = function() {
	return this._updateCount ? this._changes : null;
};
/** @public
*/
RowDefinition.prototype.resetUpdates = function() {
	if(this._updateCount) {
		this._changes = {};
		this._updateCount = 0;
	}
};

/** @public
* @param {DataView} view
* @param {string=} destRowId Destination position where the row will be placed BEFORE the specified position
* @returns {boolean} Returns true for successful registration
*/
RowDefinition.prototype.registerToView = function(view, destRowId) {
	if(!view || this._view === view) {
		return false; // Already in the view
	}

	this._view = view;
	let rowId = this._rowId;
	let rowData = view.getRowData(rowId);
	if(rowData && rowData[ROW_DEF]) {
		console.warn("Duplicate row id detected");
		return false;
	}

	if(this._subSegment) {
		if(rowData) { // Row data for sub segment has been created by data view
			rowData[ROW_DEF] = this; // It only needs to register row def to the row data
		}
		return true;
	}

	rowData = {};
	rowData[ROW_DEF] = this;

	let parentRowId = "";
	let isSegment = this._isChain || this._asSegment;
	if(destRowId) {
		parentRowId = view.getSegmentParentRowId(destRowId);
		if(parentRowId) {
			if(isSegment) { // A chain or a segment cannot be put inside another segment
				destRowId = _getEndOfSegmentRowId(view, destRowId);
			} else if(!this.isConstituent()) { // Normal row cannot be put inside another chain
				let parentRowDef = view.getData(parentRowId, ROW_DEF);
				if(parentRowDef.isChain()) {
					parentRowId = null;
					destRowId = _getEndOfSegmentRowId(view, destRowId);
				} // else { // Normal row is inserted into a segment
			} // Constituent can be inserted inside a chain
		}
	}

	let stalledSorting = _stallSorting(view, isSegment, false);
	view.insertRow(destRowId, rowData, rowId);

	if(isSegment) {
		view.setSegmentSeparator(rowId);
		_stallSorting(view, false, stalledSorting);
		if(this._collapsed != null) {
			view.collapseSegment(rowId, this._collapsed);
			this._collapsed = null;
		}
	} else if(!this._parent && parentRowId) { // Constituent cannot be added to another segment
		view.addSegmentChild(parentRowId, rowId);
	}

	return true;
};
/** @private
* @param {Array.<string>=} rowIds
* @return {string}
*/
RowDefinition.prototype._deregisterFromView = function(rowIds) {
	if(this._view) {
		if(rowIds) {
			rowIds.push(this._rowId);
		}

		this._view = null;
		return this._rowId;
	}
	return "";
};
/** Deprecated.
* @public
* @ignore
* @function
* @param {Array.<string>} rowIds
* @param {RowDefinition} rowDef
* @return {Array.<string>}
* @example
* let rowIds = descendants.reduce(RowDefinition.deregisterFromView, []);
*/
RowDefinition.deregisterFromView = function(rowIds, rowDef) {
	rowDef._deregisterFromView(rowIds);
	return rowIds;
};

/** @public
* @ignore
* @param {string} ric
* @return {RowDefinition}
*/
RowDefinition.prototype.getConstituent = function(ric) {
	if(this._children && ric) {
		for(let i = this._children.length; --i >= 0;) {
			if(this._children[i]._ric === ric) {
				return this._children[i];
			}
		}
	}
	return null;
};

/** Check whether the given ric can be a constituent of this row definition. WARNING: permId is not handled by this method
* @public
* @ignore
* @param {string} ric
* @return {boolean} Returns true if the given ric can be its constituent
*/
RowDefinition.prototype.verifyConstituent = function(ric) {
	if(this._isChain && ric) { // A constituent must have a valid ric
		if(this._ric !== ric) { // A constituent cannot have the same ric as its parent
			if(this._ric !== ("0#" + ric)) { // Real-time service may return ric without 0#
				return this._ric !== ("/" + ric); // Real-time service may return ric with slash (delayed ric)
			}
		}
	}
	return false;
};
/** verifyConstituent should be called before calling this method
* @public
* @ignore
* @param {string} ric
* @return {RowDefinition} Return newly created child definition, if success
*/
RowDefinition.prototype.addConstituent = function(ric) {
	if(!(this._rowId && this._subId && this._isChain)) { // Chain must be alive (has not been disposed). Chain must be subscribed as chain
		return null;
	}

	if(!this._children) {
		this._children = [];
	}

	let rowOptions = {
		"asConstituent": true,
		"ric": ric,
		"parent": this
	};

	// Get child static values stored in this row
	if(this._staticValues) {
		let children = this._staticValues[RowDefinition._childDataField];
		if(children) {
			let childValues = children[ric];
			if(childValues) {
				rowOptions["values"] = childValues;
				children[ric] = null;
			}
		}
	}

	return new RowDefinition(rowOptions); // childDef is added to this._children in the constructor
};

/** Used to convert autogenerated row to regular real-time row
* @public
* @ignore
* @param {string=} userRowId
*/
RowDefinition.prototype.toRealTimeRow = function(userRowId) {
	if(!this.isConstituent()) {
		return; // Only a constituent can be converted to a real-time row
	}

	let subs = this._parent._subs;
	this._realTime = true;
	this._autoGenerated = false;
	this._parent = null;
	this._depthLevel = 0;

	if(userRowId) {
		this._rowId = userRowId;
		this._userId = true;
	}

	this.subscribeForUpdates(subs); // Static data remains intact
};

/** @public
*/
RowDefinition.prototype.unlinkChain = function() {
	if(!this._isChain) {
		return;
	}

	this.unsubscribeForUpdates(true); // Static data remains intact

	let view = this._view;
	if(view) {
		let rid = this.getRowId();
		let segment = view.getSegment(rid);
		if(segment) {
			segment.setClassification(null);
		}
		view.setSegmentSeparator(rid, false);
	}

	this._isChain = false;
	this._chainRic = "";
	this._userInput = this._ric;
	this._children = null;

	this.subscribeForUpdates();
};

/** @public
* @return {boolean} Returns true if there is a change in view
*/
RowDefinition.prototype.collapseChain = function() {
	if(this._isChain && this._view) {
		return this._view.collapseSegment(this._rowId, true);
	}
	return false;
	// if(this._children) {
	// this._children.forEach(RowDefinition.dispose); // All descendants will be removed too
	// this._children = null;
	// }
	// unsub chain
	// resub index
};
/** @public
* @return {boolean} Returns true if there is a change in view
*/
RowDefinition.prototype.expandChain = function() {
	if(this._isChain && this._view) {
		return this._view.collapseSegment(this._rowId, false);
	}
	return false;
	// unsub index
	// resub chain
};

/** Toggle between chain expanding and collapsing
* @public
* @return {boolean} Returns true if there is a change in view
*/
RowDefinition.prototype.toggleChain = function() {
	if(this.isChainExpanded()) {
		return this.collapseChain();
	} else {
		return this.expandChain();
	}
};

/** @public
* @ignore
* @param {RowDefinition} childDef
* @return {boolean} Returns true if success
*/
RowDefinition.prototype.addChild = function(childDef) {
	if(!childDef) {
		return false;
	}
	return childDef.setParent(this);
};
/** @public
* @return {Array.<RowDefinition>}
*/
RowDefinition.prototype.getChildren = function() {
	return this._children;
};
/** Get all descendants (i.e. children and nested children).
* @public
* @param {Array.<RowDefinition>=} ary
* @return {!Array.<RowDefinition>} Flat list of row definitions. The order is corresponded to the order of the rows in the view
*/
RowDefinition.prototype.getDescendants = function(ary) {
	if(!ary) {
		ary = [];
	}
	if(this._children) {
		let chdr = this._children;
		let len = chdr.length;
		for(let i = 0; i < len; ++i) {
			let ch = chdr[i];
			ary.push(ch);
			ch.getDescendants(ary);
		}
	}
	return ary;
};
/** Get rowIds of all descendants (i.e. children and nested children).
* @public
* @return {Array.<string>} Null if there is no row Id
*/
RowDefinition.prototype.getAllDescendantIds = function() {
	let descendants = this.getDescendants();
	if(descendants.length) {
		return descendants.map(RowDefinition.toRowId);
	} else {
		return null;
	}
};
/** @public
* @return {number}
*/
RowDefinition.prototype.getChildCount = function() {
	return (this._children) ? this._children.length : 0;
};
/** @public
* @return {number}
*/
RowDefinition.prototype.countChildInView = function() {
	if(this._children) {
		let viewCount = 0;
		let childCount = this._children.length;
		for(let i = 0; i < childCount; ++i) {
			if(this._children[i]._view) {
				++viewCount;
			}
		}
		return viewCount;
	}
	return 0;
};
/** @public
* @return {RowDefinition}
*/
RowDefinition.prototype.getParent = function() {
	return this._parent;
};
/** @public
* @ignore
* @param {RowDefinition} parentDef
* @return {boolean} Returns true if success
*/
RowDefinition.prototype.setParent = function(parentDef) {
	if(this._parent === parentDef || parentDef === this) { // The same parent is given
		return false;
	}
	if(this._parent) { // Release from previous parent
		let at = this._parent._children.lastIndexOf(this);
		if(at >= 0) {
			this._parent._children.splice(at, 1);
		}
		this._parent = null;
		this._depthLevel = 0; // TODO: Update depth of children
	}

	if(parentDef) {
		if(!parentDef._children) {
			parentDef._children = [];
		}
		// WARNING: No duplication checking
		parentDef._children.push(this);
		this._parent = parentDef;
		this._depthLevel = parentDef._depthLevel + 1; // TODO: Update depth of children
		return true;
	}
	return false;
};
/** @public
* @return {number}
*/
RowDefinition.prototype.getDepthLevel = function() {
	return this._depthLevel;
};
/** @public
* @ignore
* @return {number}
*/
RowDefinition.prototype.getGroupOrder = function() {
	if(this._parent) {
		return this._parent.getGroupOrder() + 0.1; // Plus level. TODO: Support nested level
	}
	return this._groupOrder;
};
/** @public
* @ignore
* @param {number} num
* @return {boolean} Return true if there is any change
*/
RowDefinition.prototype.setGroupOrder = function(num) {
	if(!this._parent) { // Only root node can have group order
		if(this._groupOrder !== num) {
			this._groupOrder = num;
			return true;
		}
	}
	return false;
};

/** @public
* @ignore
* @return {Object}
*/
RowDefinition.prototype.getUserModel = function() {
	return this._userModel;
};

/** @public
* @param {Object} obj
*/
RowDefinition.prototype.setRowInfo = function(obj) {
	this._info = obj;
};
/** @public
* @return {Object}
*/
RowDefinition.prototype.getRowInfo = function() {
	return this._info;
};

/** For mapping row Id by Array.map()
* @public
* @function
* @param {RowDefinition} rowDef
* @return {string} Row Id will never be empty string unless the row has been removed
*/
RowDefinition.toRowId = function(rowDef) {
	return rowDef.getRowId();
};
/** For transforming row options into standard format
* @public
* @ignore
* @function
* @param {RowDefinition~Options} rowOptions
* @return {RowDefinition~Options}
*/
RowDefinition.extractRowOptions = function(rowOptions) {
	let ric = rowOptions["ric"];
	let permId = rowOptions["permId"];
	let chainRic = rowOptions["chainRic"];
	let collapsed = rowOptions["collapsed"];
	let asSegment = rowOptions["asSegment"];
	let asChain = rowOptions["asChain"];
	if(asChain == null && chainRic){
		asChain = true;
	}
	let expanded = null;
	if(ric && ric.indexOf("0#") >= 0){
		if(asChain == null){
			asChain = true;
		}
		expanded = true;
	} else if(asChain) {
		expanded = false;
	} else if(asSegment) {
		expanded = true;
	}

	let extractedOptions = {};
	if(collapsed == null) {
		extractedOptions["collapsed"] = !expanded;
	} else {
		extractedOptions["collapsed"] = collapsed;
	}
	extractedOptions["ric"] = ric;
	extractedOptions["permId"] = permId || "";
	extractedOptions["chainRic"] = chainRic;
	extractedOptions["asChain"] = asChain;
	return extractedOptions;
};
/** For mapping row Id by Array.map()
* @public
* @function
* @param {RowDefinition} rowDef
*/
RowDefinition.dispose = function(rowDef) {
	rowDef.dispose();
};


/* harmony default export */ const js_RowDefinition = ((/* unused pure expression or super */ null && (RowDefinition)));

;// ./src/js/FieldDefinition.js
// TODO: Merge this with Fields in mobile-mon repository
/* eslint-disable */


/* eslint-enable */


/** @type {string}
* @private
* @constant
*/
const SYNAPSE_URL =
	"/synapse/service/suggestions/suggest/?"
	+ "hits=1" // search only 1 result
	+ "&profile=" + encodeURIComponent("Field Selector");

/** @type {!RegExp}
* @private
* @constant
* @description
* ^TR.						=> start with TR.
* \w+						=> any field with string and value
* [\(]						=> open bucket (
* [\w\-=,]*				=> any property name and follow by = EX. SDATE=2011-11-11, PRIOD=123123
* (?:EDATE|SDATE)	=> non-capturing group match EDATE or SDATE
* [\w\-=,]+				=> another propertie param
* [\)]$						=> end with only )
* i								=> for match both upper and lower cases
*/
const TimeSeriesRegEx = /^TR\.\w+\([\w\-=,]*(?:EDATE|SDATE)=+[\w\-=,]*[ \w]*\)$/i;

/* @namespace */
let FieldDefinition = {};

/** @type {Object.<string, Object>}
* @private
*/
FieldDefinition._defs = {};
/** @type {Object.<string, Object>}
* @private
*/
FieldDefinition._defaults = {
	"CF_NAME": {
		name: "Name",
		rank: 2800003,
		fieldDataType: "ALPHANUMERIC",
		id: "6307",
		IsMonitorOnlyField: false,
		IsRealtimePortfolioField: false,
		IsRealtimeField: true,
		source: "IDN",
		sourceRank: 100003,
		description: "The name of the instrument"
	},
	"CF_LAST": {
		name: "Last",
		rank: 2800024,
		fieldDataType: "PRICE",
		id: "5364",
		IsMonitorOnlyField: false,
		IsRealtimePortfolioField: false,
		IsRealtimeField: true,
		source: "IDN",
		sourceRank: 100024,
		description: "The latest trade price or value."
	},
	"PCTCHNG": {
		name: "Pct. Chng",
		rank: 2709440,
		fieldDataType: "PRICE",
		id: "56",
		IsMonitorOnlyField: false,
		IsRealtimePortfolioField: false,
		IsRealtimeField: true,
		source: "IDN",
		sourceRank: 9440,
		description: "Percentage change in the latest trade price or value from the historic close."
	},
	"CF_NETCHNG": {
		name: "Net. Chng",
		rank: 2800027,
		fieldDataType: "PRICE",
		id: "5367",
		IsMonitorOnlyField: false,
		IsRealtimePortfolioField: false,
		IsRealtimeField: true,
		source: "IDN",
		sourceRank: 100027,
		description: "Net Change is the difference between latest trade price or value and the historic closing value or settlement price."
	},
	"CF_CLOSE": {
		name: "Close",
		rank: 2800023,
		fieldDataType: "PRICE",
		id: "5360",
		IsMonitorOnlyField: false,
		IsRealtimePortfolioField: false,
		IsRealtimeField: true,
		source: "IDN",
		sourceRank: 100023,
		description: "The last trade price, settlement value or closing value."
	},
	"CF_CURR": {
		name: "Currency",
		rank: 2800021,
		fieldDataType: "ENUMERATED",
		id: "6452",
		IsMonitorOnlyField: false,
		IsRealtimePortfolioField: false,
		IsRealtimeField: true,
		source: "IDN",
		sourceRank: 100021,
		description: "The currency that the instrument is quoted in"
	},
	"CF_BID": {
		name: "Bid",
		rank: 2800025,
		fieldDataType: "PRICE",
		id: "5359",
		IsMonitorOnlyField: false,
		IsRealtimePortfolioField: false,
		IsRealtimeField: true,
		source: "IDN",
		sourceRank: 100025,
		description: "The latest Bid price."
	},
	"CF_ASK": {
		name: "Ask",
		rank: 2800026,
		fieldDataType: "PRICE",
		id: "5358",
		IsMonitorOnlyField: false,
		IsRealtimePortfolioField: false,
		IsRealtimeField: true,
		source: "IDN",
		sourceRank: 100026,
		description: "The latest Ask price."
	}
};

/** @type {Object.<string, Object>}
* @private
*/
FieldDefinition._timeSeriesChildren = {};

/** api-key to call synapse service
* @type {string}
* @private
* @const
*/
FieldDefinition._synapse = "";
/**
* @type {string}
* @private
* @const
*/
FieldDefinition._lang = "en";
/**
* @type {boolean}
* @private
*/
FieldDefinition._timeSeriesExpansion = true;

/**
 * @type {Object.<string, boolean>}
 * @private
 */
FieldDefinition._loadingField = {};

/** @public
* @function
* @param {string} field
* @param {Object} def
*/
FieldDefinition.set = function(field, def) {
	if (!FieldDefinition._defs[field]) {
		FieldDefinition._defs[field] = def;
	} else {
		for (let key in def) {
			FieldDefinition._defs[field][key] = def[key];
		}
	}
};
/** Get field definition object
* @public
* @function
*/
FieldDefinition.resetAll = function() {
	let defs = FieldDefinition._defs = {};
	let defaultFields = FieldDefinition._defaults;
	for(let key in defaultFields) {
		defs[key] = defaultFields[key];
	}
};
/** Get field definition object
* @public
* @function
* @param {string} field
* @return {Object}
*/
FieldDefinition.get = function(field) {
	return FieldDefinition._defs[field] || null;
};
/** Get field definition object
* @public
* @function
* @param {string} field
* @return {Object}
*/
FieldDefinition.getFieldInfo = FieldDefinition.get;

/** @public
* @function
* @param {string} field
* @return {boolean}
*/
FieldDefinition.hasFieldInfo = function(field) {
	let val = FieldDefinition.get(field);
	return (val && val.field) ? true : false; // Already preventing an error caused by accessing a property on a null value
};

/** @public
* @function
* @param {string} field
* @return {Array}
*/
FieldDefinition.getTimeSeriesChildren = function(field) {
	return FieldDefinition._timeSeriesChildren[field] || null;
};
/** @public
* @function
* @param {string} tsDef
* @param {Object} childDef
*/
FieldDefinition.addTimeSeriesChild = function(tsDef, childDef) {
	if(!FieldDefinition._timeSeriesChildren[tsDef]) {
		FieldDefinition._timeSeriesChildren[tsDef] = [];
	}
	FieldDefinition._timeSeriesChildren[tsDef].push(childDef);
};
/** @public
* @function
* @param {string} field
*/
FieldDefinition.remove = function(field) {
	delete FieldDefinition._defs[field];
};

/** @public
* @function
* @param {Grid~SynapseConfig} config
*/
FieldDefinition.setSynapseConfig = function (config) {
	FieldDefinition._synapse = config;
};

/** @public
* @function
* @param {boolean} disabled=true if disable it, time series will not be expand
*/
FieldDefinition.disableTimeSeriesExpansion = function(disabled) {
	FieldDefinition._timeSeriesExpansion = disabled === false;
};

/** @public
* @param {string} field
* @return {boolean}
*/
FieldDefinition.isFormula = function(field) {
	return field.charAt(0) === "=";
};
/** @public
* @param {string} field
* @return {boolean}
*/
FieldDefinition.isAdc = function(field) {
	return field.indexOf("TR.") === 0;
};

/** @public
* @param {string} field
* @return {boolean}
*/
FieldDefinition.isRealTimeField = function(field) {
	if (!field) {
		return false;
	}

	if(FieldDefinition.isAdc(field)) { // Note that all TimeSeries fields are ADC field
		return false;
	}

	if(FieldDefinition.isFormula(field)) {
		return false;
	}

	if(FieldDefinition.isTimeSeriesChild(field)) {
		return false;
	}

	return FieldDefinition.getFieldProperty(field, "IsRealtimeField") !== false;
};

/** @public
* @param {string} field
* @return {boolean}
*/
FieldDefinition.isTimeSeriesChild = function(field) {
	return FieldDefinition.getFieldProperty(field, "timeSeriesParent") ? true : false;
};

/** @public
* @param {string} field
* @return {boolean}=true if field is time series field
*/
FieldDefinition.isTimeSeries = function (field) {
	if (!field) {
		return false;
	}

	// If we disable time series expansion, the grid won't have a time series field.
	if(!FieldDefinition._timeSeriesExpansion) {
		return false;
	}

	// We can check time series using a cache to avoid duplicating checks in regular expressions.
	let timeSeriesField = FieldDefinition.getFieldProperty(field, "timeSeries");
	if (timeSeriesField != null) {
		return timeSeriesField ? true : false;
	}

	return TimeSeriesRegEx.test(field);
};

/**
 * Set property value into field definition
 * @private
 * @param {string} field field definition
 * @param {string} propertyName
 * @param {*} value
 */
FieldDefinition.setFieldProperty = function(field, propertyName, value) {
	if(!FieldDefinition._defs[field]) {
		FieldDefinition._defs[field] = {};
	}
	FieldDefinition._defs[field][propertyName] = value;
};

/**
 * Set property value into field definition
 * @private
 * @param {string} field field definition
 * @param {string} propertyName
 * @return {*}
 */
FieldDefinition.getFieldProperty = function(field, propertyName) {
	let fieldDef = FieldDefinition._defs[field];
	if(fieldDef) {
		return fieldDef[propertyName];
	}
	return null;
};

/** To get more info about field via synapse service
* @private
* @param {string} field
* @returns {Promise}
*/
FieldDefinition.loadFieldInfo = function (field) {
	if (FieldDefinition._loadingField[field]) {
		return FieldDefinition._loadingField[field].promise;
	}

	let defer = new Deferred();
	let synapse = FieldDefinition._synapse;
	// cannot call request without synapse config
	if (!synapse) {
		defer.resolve(null);
	}
	// already have field definition then return
	else if (FieldDefinition.hasFieldInfo(field)) {
		defer.resolve(FieldDefinition.get(field));
	}
	// in debug using mock data instead
	else if (synapse.debug) {
		let mockData = this._mockOnLoadEndData(field, defer);
		setTimeout(onLoadEnd.bind(null, mockData), 1000);
	}
	// cannot call request without apiKey and  contextApp
	else if (!synapse.apiKey || !synapse.contextApp) {
		defer.resolve(null); // cannot request without apiKey and contextApp
	}
	// everything fine, call synapse service
	else {
		let queryUrl = SYNAPSE_URL
			+ "&api-key=" + encodeURIComponent(synapse.apiKey)
			+ "&contextApp=" + encodeURIComponent(synapse.contextApp)
			+ "&language=" + encodeURIComponent(FieldDefinition._lang)
			+ "&query=" + encodeURIComponent(field);
		if (synapse.auth) {
			queryUrl += "&auth=" + encodeURIComponent(synapse.auth);
		}
		// TODO: handle client timeout
		let xmr = new XMLHttpRequest();
		xmr._defer = defer;
		xmr._field = field;
		FieldDefinition._loadingField[field] = defer;

		// for now we care only successful case
		xmr.addEventListener("loadend", onLoadEnd);
		xmr.addEventListener("timeout", onError);
		xmr.addEventListener("error", onError);
		xmr.open("GET", queryUrl, true); // true for asynchronous
		xmr.send();
	}
	return defer.promise;
};

/**
* @private
* @param {Event} e
*/
function onError(e) {
	// provide information for timeout and error cases and will be used in onLoadEnd handler
	e.currentTarget._error = {
		type: e.type
	};
}

/** @private
* @function
* @param {string} field
* @param {Object} defer
* @returns {Object}
*/
FieldDefinition._mockOnLoadEndData = function (field, defer) {
	let fieldUpperCase = field.toUpperCase();
	let fieldLowerCase = fieldUpperCase.toLowerCase();
	let label;
	if (fieldLowerCase.indexOf("_") === 2) { // transform XX_ABCD -> Abcd
		label = fieldUpperCase[3] + fieldLowerCase.substring(4);
	} else {
		label = fieldUpperCase[0] + fieldLowerCase.substring(1);
	}
	let dataType = (fieldUpperCase.indexOf("DATE") >= 0) ? "DATE" : "PRICE";
	let item = {
		cmd: label,
		explanation: null,
		fr: false,
		id: "1635614732",
		navigation: null,
		p: {
			Rank: 2800024,
			fdt: dataType,
			fid: "5364",
			fimo: false,
			firp: false,
			firt: true,
			fl: label,
			fn: fieldUpperCase,
			fsrc: "IDN",
			fsrnk: 100024
		},
		position: 0,
		preview: false,
		relations: {},
		score: 2840024,
		source: "87",
		subtitle: "",
		symbol: label,
		title: "title mockup of " + fieldUpperCase,
		vc: "FLD"
	};

	let response = {
		action: '\"' + fieldUpperCase + '\" SRCH',
		assetClassifierExplanation: "AssetClassifier is disabled.",
		assetClassifierStatus: "NOT_INVOKED",
		// header: { request: { … }, response: { … } }, // whatever
		result: [{ name: "Fields", default: false, size: 1, hits: [item], hasMore: false }],
		search: "cpurl://apps.cp./apps/SearchAll?Search.Value=" + fieldLowerCase + "*"
	};
	response = JSON.stringify(response);
	return {
		currentTarget: {
			_defer: defer,
			_field: field,
			status: 200,
			response: response,
			responseText: response
		}
	};
};

/**
* @private
* @param {Event} e
*/
function onLoadEnd(e) {
	let xmr = e.currentTarget;
	let defer = xmr._defer;
	let field = xmr._field;

	delete FieldDefinition._loadingField[field];

	if (xmr.status === 200) { // case success
		let res = JSON.parse(xmr.responseText);
		let fieldDef = null;

		let result = res.result && res.result[0];
		if (result) {
			let item = result.hits && result.hits[0];
			if (item && item.p) {
				let profile = item.p;
				if (profile.fn.toUpperCase() === field.toUpperCase()) {
					fieldDef = FieldDefinition.get(field) || {};

					fieldDef.field = profile.fn; // name
					fieldDef.name = profile.fl; // label
					fieldDef.rank = profile.Rank;
					fieldDef.fieldDataType = profile.fdt; // data type
					fieldDef.id = profile.fid;
					fieldDef.IsMonitorOnlyField = profile.fimo;
					fieldDef.IsRealtimePortfolioField = profile.firp;
					fieldDef.IsRealtimeField = profile.firt;
					fieldDef.source = profile.fsrc;
					fieldDef.sourceRank = profile.fsrnk;
					fieldDef.description = item.title;

					FieldDefinition.set(field, fieldDef);
				}
			}
		}
		if(fieldDef) {
			defer.resolve(fieldDef);
		} else {
			defer.reject("No definition for " + field);
		}
	} else if (xmr._error) { // case error && time out
		xmr._error.field = field;
		defer.reject(xmr._error);
	} else { // case status not 200
		defer.reject({
			field: field,
			status: xmr.status,
			statusText: xmr.statusText,
			request: xmr
		});
	}
}

(function() { // Add "field" property to all default field definitions. This reduces file size and redundancy
	for(let key in FieldDefinition._defaults) {
		FieldDefinition._defaults[key]["field"] = key;
		FieldDefinition._defs[key] = FieldDefinition._defaults[key];
	}
})();


/* harmony default export */ const js_FieldDefinition = (FieldDefinition);

;// ./src/js/PredefinedFormula.js

/* @constructor */
let PredefinedFormula = {};
/** @private
* @type {Object<string, string>}
*/
PredefinedFormula._formulas = {};
/** @public
* @function
* @param {string} field
* @param {string} formula
*/
PredefinedFormula.set = function(field, formula) {
	PredefinedFormula._formulas[field] = formula;
};
/** @public
* @function
* @param {string} field
* @return {string}
*/
PredefinedFormula.get = function(field) {
	return PredefinedFormula._formulas[field] || "";
};
/** @public
* @function
* @param {string} field
*/
PredefinedFormula.remove = function(field) {
	delete PredefinedFormula._formulas[field];
};


/* harmony default export */ const js_PredefinedFormula = (PredefinedFormula);

;// ./src/js/ColumnDefinition.js
/* eslint-disable */




/* eslint-enable */

/** @callback ColumnDefinition~SortLogic
* @param {*} valA Data or RowDefinition object, depending on rowSorting flag, used for comparison
* @param {*} valB Data or RowDefinition object, depending on rowSorting flag, used for comparison
* @param {number} order Order code. 1 means ascending order and -1 means descending order
* @param {Object} ctx Context Object of the current sorting operation
*/

/** @typedef {Object} ColumnDefinition~Options
* @description ColumnDefinition options
* @property {string=} field="" Field for real-time, ADC, formula sources
* @property {string=} dataType="" data type eg. string, datetime, number
* @property {string=} formulaReference=""
* @property {Array.<string>=} require=null Extra fields required by this column will be requested for real-time data.
* @property {string=} name="" Label of the column
* @property {boolean=} notRealTimeField=false If enabled, the `field` will not be requested for real-time data. By enabling this, `rowSorting` will also be disabled, if not specified.
* @property {(boolean|string|null)=} tooltip=null Show tooltip in header section
* @property {Function=} headerBinding=null Method for custom header rendering.
* @property {Function=} headerRenderer=null Alias to `headerBinding`
* @property {Function=} binding=null Method for custom rendering. It will be executed on each data update and scrolling
* @property {Function=} renderer=null Alias to `binding`
* @property {ColumnDefinition~SortLogic=} sortLogic=null
* @property {ColumnDefinition~SortLogic=} sortingLogic=null Alias to `sortLogic`
* @property {ColumnDefinition~SortLogic=} sorter=null Alias to `sortLogic`
* @property {boolean=} rowSorting=null If set to true, RowDefinition object will be used for sorting comparison. If set to false, data from the given `field` will be used for sorting comparison.
* @property {boolean=} sortable=true If disabled, the column cannot be sorted by user click
* @property {string=} sort="" Deprecated Alias to `sortOrder`
* @property {string=} sortOrder="" The column can be sorted in ascending and descending order by specifying the order "a", "d". This option will sort the column when it is initialized or inserted at runtime.
* @property {Object=} styles=null Key/value pair Object specifying CSS styles
* @property {number=} width=NaN If specified, column will have a fixed width in pixel
* @property {number=} minWidth=NaN If specified, column will have a minimum width in pixel
* @property {boolean=} scalable=true Column scalability forces column to be scaled and filled the grid. `width` property will be used as a weight/ratio
* @property {string=} className=""
* @property {string=} class="" Alias to `className`
* @property {string=} textAlign="" Text alignment for content and column header. Available values are: left|center|right
* @property {string=} alignment="" Alias to `textAlign`
* @property {string=} headerAlignment="" Text alignment for column header. This will override `textAlign` option for column header.
* @property {string=} titleAlignment="" Alias to `headerAlignment`
* @property {boolean=} hidden=false
* @property {string=} id A unique identifier for the column
* @property {boolean=} textSelect=false If enabled, user can select text in this column
* @property {boolean=} keepModel If enabled, initial column's options will be kept
* @property {boolean=} stationary=false If enabled, the column order cannot be changed (i.e., this column and any column to its left cannot be moved)
* @property {boolean=} leftPinned=false If enabled, the column will not be part of the scrollable area and is pinned to the left side
* @property {boolean=} rightPinned=false If enabled, the column will not be part of the scrollable area and is pinned to the right side
* @property {Object=} info=null For storing any additional information to the column
* @property {boolean=} focusable=false If enabled, the column will be used to find focusable element when pressing tab key
* @property {string=} backgroundColor CSS color value
*/

/** mapping of field type to javascript type
* @type {Object.<string, string>}
* @private
*/
const TYPE_MAPPING = {
	STRING: "string",
	BOOLEAN: "boolean",
	NUMBER: "number",

	DATE: "datetime",
	DATETIME: "datetime",
	TIME: "datetime",

	ENUMERATED: "string",
	ALPHANUMERIC: "string",
	PERCENT: "string",

	PRICE: "number",
	DOUBLE: "number",
	MONEY: "number",

	INTEGER: "number",
	FLOAT: "number",
	SCORE: "number"
};

/** @type {string}
* @public
* @const
*/
const COL_DEF = "COL_DEF";

/** @private
* @param {string} str
* @return {string}
*/
let _toAlignment = function(str) {
	if(str) {
		let ch = str.charAt(0).toLowerCase();
		if(ch === "l") {
			return "left";
		}
		if(ch === "r") {
			return "right";
		}
		if(ch === "c") {
			return "center";
		}
	}
	return "";
};

/** @constructor
* @param {ColumnDefinition~Options=} columnOption
* @param {*=} hostGrid
*/
let ColumnDefinition = function(columnOption, hostGrid) {
	this._defaultRenderer = this._defaultRenderer.bind(this);
	this._customRenderer = this._customRenderer.bind(this);

	this._internalRenderer = this._defaultRenderer;
	this._internalSorter = this._defaultSorter;

	this._eventArg = {"colDef": this}; // WARNING: Circular reference
	if(hostGrid) {
		this._updateContext("grid", hostGrid);
		this._updateContext("core", hostGrid["getCoreGrid"]()); // TODO: Handle multiple core grids
	}

	this._classes = [];
	this._requiredFields = [];
	this._children = [];

	if(columnOption) {
		if(columnOption["parent"]) { // WARNING: This assume time series child, when have a parent
			this._initializeTimeSeriesChild(columnOption);
		} else {
			this.initialize(columnOption);
		}
	}

};
//#region Private Members
/** This is used as a cache for better performance. The actual field is stored in the core
* @type {string}
* @private
*/
ColumnDefinition.prototype._field = "";
/** @type {string}
* @private
*/
ColumnDefinition.prototype._dataType = "";
/** @type {string}
* @private
*/
ColumnDefinition.prototype._name = "";
/** @type {boolean}
* @private
*/
ColumnDefinition.prototype._defaultName = true;
/** @type {boolean}
* @private
*/
ColumnDefinition.prototype._defaultField = true;
/** @type {string|null}
* @private
*/
ColumnDefinition.prototype._tooltip = null;
/** @type {!Array.<string>}
* @private
*/
ColumnDefinition.prototype._requiredFields;
/** @type {Function}
* @private
*/
ColumnDefinition.prototype._headerRenderer = null;
/** @type {string}
* @private
*/
ColumnDefinition.prototype._headerAlignment = "";
/** @type {!Function}
* @private
*/
ColumnDefinition.prototype._internalRenderer;
/** @type {Array.<Function>}
* @private
*/
ColumnDefinition.prototype._userRenderers = null;
/** @type {Function}
* @private
*/
ColumnDefinition.prototype._activatedRenderer = null;
/** @type {Object.<string, Function>}
* @private
*/
ColumnDefinition.prototype._rendererMap = null;
/** @type {Function}
* @private
*/
ColumnDefinition.prototype._userSorter = null;
/** @type {boolean}
* @private
*/
ColumnDefinition.prototype._sortable = true;

/** @type {!Object}
* @private
*/
ColumnDefinition.prototype._eventArg;
/** @type {boolean}
* @private
*/
ColumnDefinition.prototype._autoGenerated = false;

/** @type {!Array.<string>}
* @private
*/
ColumnDefinition.prototype._classes;

/** @type {Engine}
* @private
*/
ColumnDefinition.prototype._fnEngine = null;
/** @type {Formula}
* @private
*/
ColumnDefinition.prototype._formula = null;
/** @type {string}
* @private
*/
ColumnDefinition.prototype._formulaRef = "";
/** @type {string}
 * @deprecated
* @private
*/
ColumnDefinition.prototype._initialOrder = "";
/** @type {boolean|null}
* @private
*/
ColumnDefinition.prototype._rowSorting = null;
/** @type {boolean}
* @private
*/
ColumnDefinition.prototype._textSelect = false;

/** @type {Object}
* @private
*/
ColumnDefinition.prototype._userModel = null;

/** @type {Grid~ColumnDefinition}
* @private
*/
ColumnDefinition.prototype._parent = null;
/** @type {Array<ColumnDefinition>}
* @private
*/
ColumnDefinition.prototype._children = null;
/** @type {Object}
* @private
*/
ColumnDefinition.prototype._info = null;
/** @type {Object}
* @private
*/
ColumnDefinition.prototype._coreColDef = null;
/** @type {boolean}
* @private
*/
ColumnDefinition.prototype._focusable = false;
//#endregion Private Members


/** @public
*/
ColumnDefinition.prototype.dispose = function() {
	this._eventArg = {}; // Clear all references
	this._coreColDef = null;
	this._activatedRenderer = this._rendererMap = null;
	this.setRenderer(null); // this._userRenderers are removed
	this.setSorter(null);
	this._userModel = null;
	this._parent = null;
	this._children = null;
	// TODO: Remove any related reference from this._fnEngine
};

/** @private
* @param {ColumnDefinition~Options|string=} columnOption
*/
ColumnDefinition.prototype._initializeTimeSeriesChild = function(columnOption) {
	this._autoGenerated = true;
	let parentDef = /** @type{ColumnDefinition} */(columnOption["parent"]);
	this._setParent(parentDef);
	this.initialize(columnOption);
};
/** @public
* @param {ColumnDefinition~Options=} columnOption
*/
ColumnDefinition.prototype.initialize = function(columnOption) {
	if(!columnOption) {
		return;
	}

	let i, len, key, val;
	//#region Apply pre-defined option
	let field = columnOption["field"]; // Field could be null or undefined here
	let defaultOption = js_FieldDefinition.get(field);
	if(defaultOption && defaultOption !== columnOption) { // The column match the default field list
		// WARNING: This changes reference of the user object. Any new property added won't affect anything outside of this method
		let userOption = columnOption;
		columnOption = {}; // Create a new object for cloning process

		for(key in defaultOption) {
			columnOption[key] = defaultOption[key];
		}

		for(key in userOption) {
			columnOption[key] = userOption[key]; // User option will override default option with the same key
		}
	}
	//#endregion Apply pre-defined option

	val = columnOption["formulaEngine"];
	if(val && val["addFormula"]) {
		this._fnEngine = /** @type{Engine} */(val);
	}
	val = columnOption["formulaReference"];
	if(val) {
		this._formulaRef = val + "";
	}

	val = columnOption["require"] || columnOption["requiredFields"];
	if(val != null) { // Empty string is allowed
		if(typeof val === "string") {
			this._requiredFields = val.split(" ");
		} else if(Array.isArray(val)) {
			this._requiredFields = val.slice();
		}
	}

	this._setField(field, columnOption); // Perform some field manipulation

	val = columnOption["name"];
	if(val == null) {
		val = columnOption["title"]; // For backward compatability
	}
	if(val != null) { // Name can be empty string
		this._name = (typeof val === "string") ? val : (val + "");
		this._defaultName = false;
	}

	val = columnOption["dataType"];
	if(val != null) {
		this._dataType = val;
	}

	val = columnOption["notRealTimeField"];
	if(val != null) {
		js_FieldDefinition.setFieldProperty(field, "IsRealtimeField", !val);
	}

	val = columnOption["tooltip"];
	if(val != null) { // Tooltip can be empty string
		this._tooltip = val;
	}

	val = columnOption["headerAlignment"] || columnOption["titleAlignment"];
	if(val != null) {
		this._headerAlignment = _toAlignment(val);
	}

	val = columnOption["headerBinding"] || columnOption["headerRenderer"];
	// eslint-disable-next-line no-undefined
	if(val !== undefined) { // null is allowed for headerBinding
		this._headerRenderer = val;
	}

	val = columnOption["binding"] || columnOption["renderer"];
	if(!val) {
		val = columnOption["formatter"]; // Migrate from Composite Grid
	}
	// eslint-disable-next-line no-undefined
	if(val !== undefined) { // null is allowed for renderer
		if(val instanceof Array) {
			len = val.length;
			for(i = 0; i < len; ++i) {
				this.addRenderer(/** @type{Function} */(val[i]));
			}
		} else if(val) {
			if(typeof val === "function") {
				this.setRenderer(val);
			} else if(val["binding"]) {
				this.setRenderer(val["binding"]);
			}
		} else {
			this.setRenderer(null);
		}
	}

	val = columnOption["rowSorting"];
	if(val != null) {
		this._rowSorting = val ? true : false;
	}

	val = columnOption["sortLogic"] || columnOption["sortingLogic"] || columnOption["sorter"];
	// eslint-disable-next-line no-undefined
	if(val !== undefined) { // null is allowed for sorter
		this.setSorter(/** @type{Function} */(val));
	}

	// Deprecate
	val = columnOption["sortable"];
	if(val != null) {
		this._sortable = val ? true : false;
	}

	val = columnOption["focusable"];
	if(val != null) {
		this._focusable = val ? true : false;
	}

	val = columnOption["className"] || columnOption["class"];
	if(typeof val === "string") {
		this._classes = val.split(" ");
	}

	// Deprecate
	val = columnOption["sortOrder"] || columnOption["sort"];
	if(val) {
		this._initialOrder = val;
	}

	val = columnOption["textSelect"];
	if(val) {
		this._textSelect = val;
	}

	val = columnOption["info"];
	if(val) {
		this._info = val;
	}

	this._userModel = columnOption; // WARNING: This may not actually the user object
};

/** @public
* @return {string}
*/
ColumnDefinition.prototype.getId = function() {
	if(this._coreColDef) {
		return this._coreColDef["id"] || "";
	}
	return "";
};
/** @public
* @return {!Array.<string>}
*/
ColumnDefinition.prototype.getRequireFields = function() {
	// TODO: Validate the fields
	return this._requiredFields;
};
/** @public
* @function
* @return {!Array.<string>}
*/
ColumnDefinition.prototype.getRequiredFields = ColumnDefinition.prototype.getRequireFields;
/** @public
* @return {!Function}
*/
ColumnDefinition.prototype.getRenderer = function() {
	return this._internalRenderer;
};
/** @public
* @return {Function}
*/
ColumnDefinition.prototype.getHeaderRenderer = function() {
	return this._headerRenderer;
};

/** @public
* @return {string}
*/
ColumnDefinition.prototype.getName = function() {
	if(this._defaultName) {
		if(this._defaultField) {
			return "Column " + this.getId();
		}
		return this.getField();
	}
	return this._name;
};
/** @public
* @return {boolean}
*/
ColumnDefinition.prototype.isDefaultName = function() {
	return this._defaultName;
};
/** @public
* @return {string}
*/
ColumnDefinition.prototype.getFieldDefinition = function () {
	return js_FieldDefinition.get(this.getField());
};
/** get realtime field data type
* @public
* @return {string}
*/
ColumnDefinition.prototype.getFieldDataType = function () {
	let fieldDef = this.getFieldDefinition();
	if(fieldDef && fieldDef.fieldDataType) {
		return fieldDef.fieldDataType;
	}
	return "";
};
/** get data type
* @public
* @return {string}
*/
ColumnDefinition.prototype.getDataType = function () {
	if(this._dataType) return this._dataType;

	let fieldDataType = this.getFieldDataType();
	if(fieldDataType) {
		return TYPE_MAPPING[fieldDataType.toUpperCase()];
	}
	return "";
};
/** set data type
* @public
* @param {string} dataType
*/
ColumnDefinition.prototype.setDataType = function (dataType) {
	let val = TYPE_MAPPING[dataType.toUpperCase()];
	if(val) {
		dataType = val;
	}
	this._dataType = dataType;
};
/** get data type
* @public
* @param {string} field
* @return {string}
*/
ColumnDefinition.getDataType = function(field) {
	let fieldDef = js_FieldDefinition.get(field);
	if(fieldDef) {
		let fieldDataType = fieldDef.fieldDataType ? fieldDef.fieldDataType.toUpperCase() : null;
		return TYPE_MAPPING[fieldDataType] ? TYPE_MAPPING[fieldDataType] : "";
	}
	return "";
};
/** @public
* @ignore
* @return {boolean|string|null}
*/
ColumnDefinition.prototype.getTooltipValue = function() {
	return this._tooltip;
};
/** @public
* @return {string}
*/
ColumnDefinition.prototype.getTooltip = function() {
	if(this._tooltip != null) {
		if(typeof this._tooltip === "string") {
			return this._tooltip; // There is a custom tooltip
		} else if(!this._tooltip) {
			return "";
		}
	}

	let name = this.getName();
	if(name) {
		return name;
	}

	let field = this.getField();
	if(field) {
		return field;
	}

	return "";
};
/** @public
* @return {string}
*/
ColumnDefinition.prototype.getField = function() {
	if(this._coreColDef) {
		return this._coreColDef["field"] || "";
	}
	return "";
};
/** @public
* @return {!Array.<string>}
*/
ColumnDefinition.prototype.getAllFields = function() {
	let field = this.getField();
	if(field) {
		return this._requiredFields.concat(field);
	}
	return this._requiredFields.slice();
};
/** @public
* @return {boolean}
*/
ColumnDefinition.prototype.isRealTimeField = function() {
	return js_FieldDefinition.isRealTimeField(this.getField() || this._field);
};
/** @public
* @return {boolean}
*/
ColumnDefinition.prototype.isTimeSeries = function() {
	return js_FieldDefinition.isTimeSeries(this.getField());
};
/** @public
* @return {boolean}
*/
ColumnDefinition.prototype.isTimeSeriesChild = function() {
	return js_FieldDefinition.isTimeSeriesChild(this.getField());
};
/** @public
* @return {boolean}
*/
ColumnDefinition.prototype.isFormulaField = function() {
	return this._formula ? true : false;
};
/** @public
* @param {string} field
* @return {boolean}
*/
ColumnDefinition.isFormulaField = function(field) {
	return js_FieldDefinition.isFormula(field);
};
/** @public
* @param {string} field
* @return {boolean}
*/
ColumnDefinition.isAdcField = function(field) {
	return js_FieldDefinition.isAdc(field);
};
/** @public
* @param {string} field
* @return {boolean}
*/
ColumnDefinition.isRealTimeField = function(field) {
	return js_FieldDefinition.isRealTimeField(field);
};
/** @public
* @function
* @param {Array.<string>} fields To be used by Array.reduce method
* @param {ColumnDefinition} colDef
* @return {!Array.<string>}
*/
ColumnDefinition.getRealTimeFields = function(fields, colDef) {
	if(colDef.isRealTimeField()) {
		fields.push(colDef.getField());
	}

	// TODO: Exclude any formula references from requesting real-time field
	let reqFields = colDef.getRequiredFields();
	let len = reqFields.length;
	for(let i = 0; i < len; ++i) {
		let field = reqFields[i];
		if(ColumnDefinition.isRealTimeField(field)) {
			fields.push(field);
		}
	}
	return fields;
};
/** @public
 * @deprecated
* @return {boolean}
*/
ColumnDefinition.prototype.isSortable = function() {
	return this._sortable;
};
/** @public
* @return {boolean}
*/
ColumnDefinition.prototype.isScalable = function() {
	let core = this._eventArg["core"];
	let grid = this._eventArg["grid"];
	let colIndex = grid.getColumnIndex(this);
	return core.getColumnScalability(colIndex);
};
/** Deprecated, this function will be return initial sort order of this column, The current sort state may be different from initial sort state.
* @public
* @deprecated
* @return {string}
*/
ColumnDefinition.prototype.getInitialSortOrder = function() {
	// WARNING: This state is different from `sortOrder`, it is the state when grid is initialized.
	return this._initialOrder;
};
/** @public
* @return {number}
*/
ColumnDefinition.prototype.getWidth = function() {
	let core = this._eventArg["core"];
	let grid = this._eventArg["grid"];
	let colIndex = grid.getColumnIndex(this);
	return core.getColumnWidth(colIndex);
};
/** @public
* @return {number}
*/
ColumnDefinition.prototype.getMinWidth = function() {
	let core = this._eventArg["core"];
	let grid = this._eventArg["grid"];
	let colIndex = grid.getColumnIndex(this);
	return core.getMinimumColumnWidth(colIndex);
};
/** @public
* @return {string}
*/
ColumnDefinition.prototype.getTextAlign = function() {
	let core = this._eventArg["core"];
	let grid = this._eventArg["grid"];
	let colIndex = grid.getColumnIndex(this);
	return core.getColumnAlignment(colIndex);
};
/** @public
* @return {boolean}
*/
ColumnDefinition.prototype.isFieldEmpty = function() {
	return !this.getField();
};

/** @public
* @return {string}
*/
ColumnDefinition.prototype.getHeaderAlignment = function() {
	return this._headerAlignment || "";
};
/** @public
* @return {!Array.<string>}
*/
ColumnDefinition.prototype.getClasses = function() {
	return this._classes;
};
/** @public
* @param {Object=} colOptions
* @return {!Object}
*/
ColumnDefinition.prototype.getConfigObject = function(colOptions) {
	let obj = colOptions || {};

	if(this._info){
		obj["info"] = this._info;
	}

	if(obj["field"] == null) {
		if(!this._defaultField) {
			let field = this.getField();
			if(field) {
				obj["field"] = field;
			}
		}
	}

	let value = this.getDataType();
	if(value) {
		obj["dataType"] = value;
	}

	if(this._formulaRef) {
		obj["formulaReference"] = this._formulaRef;
	}

	if(this._requiredFields && this._requiredFields.length) {
		obj["require"] = this._requiredFields;
	}

	if(!this._defaultName) {
		obj["name"] = this.getName();
	}

	// The 'IsRealtimeField' property will only be set if the user sets 'notRealTimeField' in the column options. It will be returned if the user has this option enabled, otherwise it will not be returned
	value = js_FieldDefinition.getFieldProperty(this.getField(), "IsRealtimeField") === false;
	if(value) {
		obj["notRealTimeField"] = value;
	}

	if(this._tooltip != null) {
		obj["tooltip"] = this._tooltip;
	}

	if(this._rowSorting != null) {
		obj["rowSorting"] = this._rowSorting;
	}

	if(!this._sortable) {
		obj["sortable"] = false;
	}

	let classes = this._classes;
	if(classes && classes.length > 0) {
		obj["className"] = classes.join(" ");
	}

	if(this._textSelect) {
		obj["textSelect"] = this._textSelect;
	}

	if(this._headerAlignment) {
		obj["headerAlignment"] = this._headerAlignment;
	}

	if(this._autoGenerated) {
		obj["autoGenerated"] = this._autoGenerated;
	}

	if(this._focusable) {
		obj["focusable"] = true;
	}

	let core = this._eventArg["core"];
	let grid = this._eventArg["grid"];
	let colIndex = grid.getColumnIndex(this);

	// The scalable, width and minWidth cannot be shifted into core as getConfigObject is accessible from colDef.
	let scalable = core.getColumnScalability(colIndex);
	let width = core.getColumnCustomLaneSize(colIndex);

	if (scalable !== true || width !== 1) {
		obj["scalable"] = scalable;
		obj["width"] = width;
	}

	value = core.getMinimumColumnWidth(colIndex);
	if(value > 0) {
		obj["minWidth"] = value;
	}

	// If "hidden" property already available from core/extensions, don't override this property
	if(obj["hidden"] == null) {
		if(!core.getColumnVisibility(colIndex, 0)) {
			obj["hidden"] = true;
		}
	}

	value = core.getColumnAlignment(colIndex);
	if(value !== "default") {
		obj["textAlign"] = value;
	}

	return obj;
};
/** @public
* @return {boolean}
*/
ColumnDefinition.prototype.isHidden = function() {
	let core = this._eventArg["core"];
	let grid = this._eventArg["grid"];
	let colIndex = grid.getColumnIndex(this);
	return !core.isColumnVisible(colIndex);
};
/** @public
* @return {boolean}
*/
ColumnDefinition.prototype.isTextSelect = function () {
	return this._textSelect;
};
/** @public
* @param {boolean=} bool
*/
ColumnDefinition.prototype.hide = function(bool) {
	let core = this._eventArg["core"];
	let grid = this._eventArg["grid"];
	let colIndex = grid.getColumnIndex(this);
	core.hideColumn(colIndex, bool !== false);
};
/** This will clear all previously stored renderers and set a new one.
* @public
* @param {Function} func
*/
ColumnDefinition.prototype.setHeaderRenderer = function(func) {
	this._headerRenderer = func;
};
/** This will clear all previously stored renderers and set a new one.
* @public
* @param {Function} func
*/
ColumnDefinition.prototype.setRenderer = function(func) {
	if(typeof func === "function") {
		this._internalRenderer = this._customRenderer;
		if(this._activatedRenderer !== func) {
			if(this._userRenderers) {
				this._userRenderers.length = 0;
			} else {
				this._userRenderers = [];
			}
			this._userRenderers.push(func);
		}
	} else {
		this._internalRenderer = this._defaultRenderer;
		this._userRenderers = null;
	}
};


/** @public
* @return {ColumnDefinition}
*/
ColumnDefinition.prototype.getParent = function() {
	return this._parent;
};
/**
* @private
* @param {ColumnDefinition} parentDef
* @return {boolean}=true if have any change
*/
ColumnDefinition.prototype._setParent = function(parentDef) {
	if(this._parent === parentDef || parentDef === this) { // The same parent is given
		return false;
	}

	if(parentDef) {
		this._parent = parentDef;
		this._parent._addChild(this); // sync child and parent
	}
	return false;
};
/**
* @public
* @return {Array.<ColumnDefinition>}
*/
ColumnDefinition.prototype.getChildren = function() {
	return this._children;
};
/**
* @private
* @param {ColumnDefinition} colDef Child column definition
*/
ColumnDefinition.prototype._addChild = function(colDef) {
	if(this === colDef || this._children.indexOf(colDef) > -1) {
		return;
	}
	this._children.push(colDef);
};

/** Add more renderer to the existing list
* @public
* @param {Function} func
*/
ColumnDefinition.prototype.addRenderer = function(func) {
	if(func) {
		if(this._userRenderers) {
			this._userRenderers.push(func);
		} else {
			this.setRenderer(func);
		}
	}
};
/** Make the given renderer the first priority, overriding any existing renderer. The method still maintain existing renderers for restoring back to original state.
* @public
* @param {string=} id ID for renderer mapping. If not specified the default renderer will be picked up
* @param {Function=} func If not specified the renderer which corresponded id from mapping will be picked up. If false is given, the corresponding id will be cleared.
* @return {boolean} Current activation state after the call
*/
ColumnDefinition.prototype.activateRenderer = function(id, func) {
	let renderer = null;
	if (id != null) {
		let rendererMap = this._rendererMap;
		if (!rendererMap) {
			rendererMap = this._rendererMap = {};
		}

		if (typeof func === "function") {
			renderer = rendererMap[id] = func;
		} else if(func == null) { // Use cached method if null or undefined value is given
			renderer = rendererMap[id];
		} else if(!func) { // Empty string, zero, or false value
			renderer = rendererMap[id] = null;
		}
	}

	this._activatedRenderer = renderer || null;
	if(!renderer) { // Restore back existing user renderers if there is no activated renderer
		renderer = this._userRenderers ? this._userRenderers[0] : null;
	}
	this.setRenderer(renderer);
	return this._activatedRenderer ? true : false;
};

/** @public
* @return {Function}
*/
ColumnDefinition.prototype.getSorter = function() {
	return this._userSorter;
};
/** @public
* @param {ColumnDefinition~SortLogic=} func
*/
ColumnDefinition.prototype.setSorter = function(func) {
	this._userSorter = (typeof func === "function") ? func : null;
};
/** @public
* @return {boolean}
*/
ColumnDefinition.prototype.isRowSorting = function() {
	if(this._rowSorting == null) {
		return !js_FieldDefinition.getFieldProperty(this.getField(), "IsRealtimeField");
	}
	return this._rowSorting ? true : false;
};

/** @public
* @return {boolean}
*/
ColumnDefinition.prototype.isAutoGenerated = function() {
	return this._autoGenerated;
};

/** To allow user change column name in run-time (language change or localization)
* @public
* @param {string} str
*/
ColumnDefinition.prototype.setName = function(str) {
	this._name = str;
	this._defaultName = false;
};

/** @private
* @param {string|null=} field
* @param {ColumnDefinition~Options=} columnOption
*/
ColumnDefinition.prototype._setField = function(field, columnOption) {
	this._defaultField = (field == null); // undefined or null
	if(!field) {
		field = "";
	}
	field = field.trim();

	let formulaStr = columnOption ? columnOption["formula"] : "";
	if(this._fnEngine) {
		let uppercasedF = field.toUpperCase(); // For comparison only
		let predefinedF = formulaStr || js_PredefinedFormula.get(uppercasedF);
		if(predefinedF || ColumnDefinition.isFormulaField(field)) {
			field = formula_Formula.toUpperCase(field);
			this._formula = this._fnEngine.addFormula(predefinedF || field); // Always a success
			if(predefinedF) {
				this._formula.setAlias(field); // This field is used for retrieving data for dependent fields
				this._fnEngine.addReference(this._formula, field); // Add reference name for dependencies calculation
			}

			this._retrieveNestedFields(this._formula.getFieldNames());
		}
		if(this._formulaRef) { // Allow user input to be used as a shorthand of the field
			// WARNING: Formula Reference must be unique, or else it will be changed
			this._formulaRef = this._fnEngine.addReference(this._formula || field, this._formulaRef);
		}
	}

	this._field = field; // WARNING: This could be out of sync with core. It is used as a cache for better performance

	// We need to cache time series in field definition for improve performance of checking methond
	js_FieldDefinition.setFieldProperty(field, "timeSeries", js_FieldDefinition.isTimeSeries(field) ? true : false);

	let parentColDef = columnOption["parent"];
	if(columnOption && parentColDef) {
		js_FieldDefinition.setFieldProperty(field, "timeSeriesParent", parentColDef.getField());
	}

	this._updateContext("field", field);
	if(this.isRealTimeField()) { // Only realtime field will have a formatted field
		this._updateContext("formattedField", field + "_FORMATTED");
	}
};
/** @private
* @param {Array.<string>} fields
*/
ColumnDefinition.prototype._retrieveNestedFields = function(fields) {
	if(!fields) {
		return;
	}
	for(let i = fields.length; --i >= 0;) {
		let field = fields[i];
		let uppercasedF = field.toUpperCase();
		let predefinedF = js_PredefinedFormula.get(uppercasedF);
		if(predefinedF) { // Predefined formula
			let f = this._fnEngine.getFormula(field); // Get formula by reference name
			if(f) {
				f.addFormulaCount(); // Add reference count
			} else {
				f = this._fnEngine.addFormula(predefinedF);
				f.setAlias(field); // This field is used for retrieving data for dependent fields
				this._fnEngine.addReference(f, field); // Add reference name for dependencies calculation
			}
			this._retrieveNestedFields(f.getFieldNames()); // Recursion
		} else { // Any field that is not a predefined formula will be sent for requesting real-time data
			this._requiredFields.push(field); // TODO: Exclude column reference
		}
	}
};
/** @private
* @param {Object} e
*/
ColumnDefinition.prototype._defaultRenderer = function(e) {
	let rowDef = /** @type{RowDefinition} */(e["dataValue"]);
	e["cell"].setContent(rowDef ? rowDef.getData(this._field) : null);
};
/** @private
* @param {Object} e
*/
ColumnDefinition.prototype._customRenderer = function(e) {
	let rowDef = /** @type{RowDefinition} */(e["dataValue"]);

	let arg = this._eventArg; // Reuse the same object
	if(rowDef) {
		let rowData = rowDef.getRowData();
		arg["rowDef"] = rowDef;
		arg["rowData"] = rowData;
		arg["rowIndex"] = e["rowIndex"];
		if(rowData) {
			arg["data"] = rowData[this._field];
		} else {
			arg["data"] = null;
		}
	} else {
		arg["data"] = null;
	}
	arg["colIndex"] = e["colIndex"];
	arg["cell"] = e["cell"];
	arg["section"] = e["section"];

	if(this._activatedRenderer) {
		this._activatedRenderer(arg);
	} else {
		let urs = this._userRenderers;
		let len = urs.length;
		for(let i = 0; i < len; ++i) {
			urs[i](arg);
		}
	}
};

/** @private
* @param {string} propName
* @param {*} val
*/
ColumnDefinition.prototype._updateContext = function(propName, val) {
	this._eventArg[propName] = val;
};

/** @public
* @return {Object}
*/
ColumnDefinition.prototype.getUserModel = function() {
	return this._userModel;
};

/** @public
*/
ColumnDefinition.prototype.clearUserModel = function() {
	this._userModel = null;
};

/** @public
* @param {Object} obj
*/
ColumnDefinition.prototype.setColumnInfo = function(obj) {
	this._info = obj;
};

/** @public
* @return {Object}
*/
ColumnDefinition.prototype.getColumnInfo = function() {
	return this._info;
};

/** @public
* @ignore
* @param {Object} obj
*/
ColumnDefinition.prototype._setCoreColumnDef = function(obj) {
	this._coreColDef = obj || null;
};

/** @public
* @return {boolean}
*/
ColumnDefinition.prototype.isFocusable = function() {
	return this._focusable;
};
/** @public
* @return {string}
*/
ColumnDefinition.prototype.getBackgroundColor = function() {
	let core = this._eventArg["core"];
	let grid = this._eventArg["grid"];
	let colIndex = grid.getColumnIndex(this);
	return core.getColumnBackgroundColor(colIndex);
};
/** @public
* @param {string} color
*/
ColumnDefinition.prototype.setBackgroundColor = function(color) {
	let core = this._eventArg["core"];
	let grid = this._eventArg["grid"];
	let colIndex = grid.getColumnIndex(this);
	core.setColumnBackgroundColor(colIndex, color);
};


/* harmony default export */ const js_ColumnDefinition = (ColumnDefinition);

;// ./src/js/SnapshotFiller.js
/* eslint-disable */






/** This class make requesting ADC field easier by encapsulating JET.Data APIs and mapping data to the row
* @constructor
* @extends {EventDispatcher}
*/
let SnapshotFiller = function () {
	this._onRequest = this._onRequest.bind(this);

	this._rics = {};
	this._fields = {};
	this._adcOptions = { // TODO: support requesting level parameter
		productId: "001"
	};
};
Ext.inherits(SnapshotFiller, EventDispatcher);

/** @private
* @type {Object}
*/
SnapshotFiller._mockAdc = null;

/** @type {number}
* @private
*/
SnapshotFiller.prototype._timerId = 0;
/** @type {!Object.<string, number>}
* @private
*/
SnapshotFiller.prototype._rics;
/** @type {!Object.<string, number>}
* @private
*/
SnapshotFiller.prototype._fields;
/** @type {!*}
* @private
*/
SnapshotFiller.prototype._rtk;
/** @type {!Grid~ADCOptions}
* @private
*/
SnapshotFiller.prototype._adcOptions = null;


/** @public
* @param {*} rtk RTK instance
*/
SnapshotFiller.prototype.dispose = function () {
	this.removeAllEventListeners();
	if(this._timerId) {
		clearTimeout(this._timerId);
		this._timerId = 0;
	}
	this._rics = this._fields = this._rtk = this._adcOptions = null;
};

/** @public
* @param {*} rtk RTK instance
*/
SnapshotFiller.prototype.setRTK = function (rtk) {
	this._rtk = rtk;
};
/** @public
* @param {string} str 
* @param {*} value 
*/
SnapshotFiller.setMockAdc = function(str, value) {
	if(!SnapshotFiller._mockAdc) {
		SnapshotFiller._mockAdc  = {};
	}
	SnapshotFiller._mockAdc[str] = value;
};

/** @public
* @param {string} str 
* @return {*} 
*/
SnapshotFiller.getMockAdc = function(str) {
	if(!SnapshotFiller._mockAdc) {
		return null;
	}
	return SnapshotFiller._mockAdc[str];
};

/** @public
*/
SnapshotFiller.clearMockAdc = function() {
	SnapshotFiller._mockAdc = null;
}

/** @public
* @param {Grid~ADCOptions} adcOptions ADC requesting level parameter options
*/
SnapshotFiller.prototype.setADCOptions = function (adcOptions) {

	let val = adcOptions["productId"];
	if(val) {
		this._adcOptions.productId = val;
	}
	// TODO: support requesting level parameter https://confluence.refinitiv.com/display/ADC/Request+level+parameters
	// let val = adcOptions["lang"];
	// if(val) {
	// 	this._adcOptions.lang = val;
	// }
	// let val = adcOptions["cache"];
	// if(val) {
	// 	this._adcOptions.cache = val;
	// }
};

/** @public
* @param {string} ric
*/
SnapshotFiller.prototype.addRic = function (ric) {
	// TODO: since we use ric instead of rowDef
	// We need to find a way to check isSubscribing and isAutoGenerated before add ric
	// if (!rowDef.isSubscribing()) { // // Only real-time row (non-empty non-header) can be registered for snapshot.
	// 	if (!rowDef.isAutoGenerated()) {  // However, chain constituents (auto generated) are still eligible.
	// 		return;
	// 	}
	// }

	if (ric && !this._rics[ric]) {
		this._rics[ric] = 1;
		this._request();
	}
};
/** @public
* @param {Array.<string>} rics
* @return {boolean}
*/
SnapshotFiller.prototype.addRics = function (rics) {
	let len = rics ? rics.length : 0;
	let dirty = false;
	for(let i = 0; i < len; ++i) {
		let ric = rics[i];
		if (ric && !this._rics[ric]) {
			this._rics[ric] = 1;
			dirty = true;
		}
	}
	if(dirty) {
		this._request();
		return true;
	}
	return false;
};
/** @public
* @param {string} str
*/
SnapshotFiller.prototype.addField = function (str) {
	if(this._addField(str)) {
		this._request();
	}
};
/** @public
* @param {Array.<string>} fields
* @return {boolean}
*/
SnapshotFiller.prototype.addFields = function (fields) {
	let len = fields.length;
	let dirty = 0;
	for(let i = 0; i < len; ++i) {
		dirty |= this._addField(fields[i]);
	}
	if(dirty) {
		this._request();
		return true;
	}
	return false;
};

/** @private
* @param {string} str
* @return {number}
*/
SnapshotFiller.prototype._addField = function (str) {
	if(SnapshotFiller.isValidField(str)) {
		if(!this._fields[str]) {
			this._fields[str] = 1;
			return 1;
		}
	}
	return 0;
};
/** @public
* @function
* @param {string} str
* @returns {boolean}
*/
SnapshotFiller.isValidField = function(str) {
	// Only field with TR. is considered as ADC field
	return (str && str.indexOf("TR.") === 0) ? true : false;
};

/** @private
*/
SnapshotFiller.prototype._request = function () {
	if(!this._timerId) {
		this._timerId = setTimeout(this._onRequest, 0);
	}
};
/** @private
*/
SnapshotFiller.prototype._onRequest = function () {
	this._timerId = 0;

	let jet = window["JET"]; // Assume that JET is already exist.
	if (this._rtk) {
		if (!this._rtk.Data) {
			return; // Require RTK.Data plugin to get snapshot data
		}
	} else if (!jet || !jet["ContainerDescription"] || (typeof jet["Data"] !== "function")) {
		return; // Require JET to get snapshot data or JET is not ready to use
	}

	// Retrieve instruments and ADC fields
	let instruments = Object.keys(this._rics);

	let i;
	let fields = [];
	let timeSeriesFields = [];
	for (let field in this._fields) {
		if(!FieldDefinition.isTimeSeries(field)) {
			fields.push(field);
		} else {
			timeSeriesFields.push(field);
		}
	}
	let fieldLen = fields.length;
	let timeSeriesFieldLen = timeSeriesFields.length;

	if ( (!fieldLen && !timeSeriesFieldLen ) || !instruments.length) { // No ADC field or real-time RIC
		return;
	}

	// Clean up members, preparing for the next request
	this._rics = {};
	this._fields = {};

	let onSuccess, payload;
	if (this._rtk) {
		let strFields;
		// Request time serie fields
		if(timeSeriesFields.length > 0) {
			// request time series field
			let strtTimeSeriesFields = timeSeriesFields.join(',');
			payload = {
				"method": "select",
				"formula": strtTimeSeriesFields,
				"identifiers": instruments,
				"productId": this._adcOptions.productId,
				"output": "Col,date|,Row,In|,va,T,NoEmptyTickers" // For customize output server, for more information please visit "https://confluence.refinitiv.com/display/ADC/Data+Cloud+Output+Format"
			};
			onSuccess = this._onRTKTimeSeriesSuccess.bind(this, timeSeriesFields);
			let responseMockAdc = SnapshotFiller.getMockAdc(payload.formula);
			if(responseMockAdc) {
				setTimeout(function() {
					onSuccess(responseMockAdc);
				}, 10); // Mock Async operation
			} else {
				this._rtk.Data.Adc
				.request(payload)
				.then(onSuccess)
				.catch(function (err) {
					console.log(err);
				});
			}
		}

		// Request normal adc field
		if(fields.length > 0) {
			strFields = fields.join(',');
			payload = {
				"method": "select",
				"formula": strFields,
				"identifiers": instruments,
				"productId": this._adcOptions.productId,
				"output": "Col,In,va,T,NoEmptyTickers" // For customize output server, for more information please visit "https://confluence.refinitiv.com/display/ADC/Data+Cloud+Output+Format"
			};
			onSuccess = this._onRTKSuccess.bind(this, fields);
			this._rtk.Data.Adc
				.request(payload)
				.then(onSuccess)
				.catch(function (err) {
					console.log(err);
				});
		}


	} else {
		let reqFields = [];
		for(i = 0; i < fieldLen; i++) {
			reqFields.push({ "name": fields[i] });
		}
		payload = {
			"instruments": instruments,
			"fields": reqFields
		};
		onSuccess = this._onJETSuccess.bind(this, fields);
		jet["Data"]("datagrid")
			.then(function (service) { return service["request"](payload); })
			.then(JSON.parse)
			.then(onSuccess)
			.catch(function (err) {
				console.log(err);
			});
	}
};

/** @private
* @function
* @param {Array.<string>} fields
* @param {string} serverResult
*/
SnapshotFiller.prototype._onJETSuccess = function (fields, serverResult) {
	this._dispatch("adcDataReceived", serverResult);
	let data2D = serverResult["data"];
	let svHeaders = serverResult["headers"] && serverResult["headers"][0];

	if (!Array.isArray(data2D) || !Array.isArray(svHeaders)) {
		return; //TODO: Return Promise.reject(errMsg);
	}


	// Server will return field only in UPPERCASE
	// ex. requestField = TR.Volume ===> serverField = TR.VOLUME
	// so we need convert UPPERCASE to be original
	let i, field, ric;
	let j = 1; //  to skip instrument index, use j = 1
	let fLength = fields.length;
	let hLength = svHeaders.length;
	let headers = new Array(hLength);
	for (i = 0; i < fLength && j < hLength; i++) {
		field = fields[i];
		if (svHeaders[j].field === field.toUpperCase()) {
			headers[j] = field;
			j++;
		}
	}

	let len = data2D.length;
	let fieldLen = headers.length;
	let ricMap = {};

	// TODO: Freeze the data view before setting multiple data
	for (i = 0; i < len; i++) {
		let dataRow = data2D[i];
		ric = dataRow[0];

		if (ricMap[ric]) {
			continue; // we interested only first row of ric
		}

		let snapData = ricMap[ric] = {};

		// loop for create rowData for update
		for (j = 1; j < fieldLen; j++) { //  to skip instrument index, use j = 1
			let value = dataRow[j];
			if (value != null && value !== "") {
				field = headers[j];
				snapData[field] = value;
			}
		}
	}

	// return result only ric that has update data
	let updatedData = {};
	for (ric in ricMap) {
		let obj = ricMap[ric];
		if (!isEmptyObject(obj)) {
			updatedData[ric] = obj;
		}
	}

	this._dispatch("dataChanged", {
		data: updatedData
	});
};

/** @private
* @function
* @param {Array.<string>} fields
* @param {string} serverResult
*/
SnapshotFiller.prototype._onRTKSuccess = function (fields, serverResult) {
	this._dispatch("adcDataReceived", serverResult);
	let data2D = serverResult["rows"];
	let svHeaders = serverResult["rows"] && serverResult["rows"][0];
	if (!Array.isArray(data2D) || !Array.isArray(svHeaders)) {
		return; //TODO: Return Promise.reject(errMsg);
	}

	// Server will return field only in UPPERCASE
	// ex. requestField = TR.Volume ===> serverField = TR.VOLUME
	// so we need convert UPPERCASE to be original
	let i, field, ric;
	let j = 1; //  to skip instrument index, use j = 1
	let fLength = fields.length;
	let hLength = svHeaders.length;
	let headers = new Array(hLength);
	for (i = 0; i < fLength && j < hLength; i++) {
		field = fields[i];
		if (svHeaders[j].r.toUpperCase() === field.toUpperCase()) {
			headers[j] = field;
			j++;
		}
	}

	let len = data2D.length;
	let fieldLen = headers.length;
	let ricMap = {};

	// TODO: Freeze the data view before setting multiple data
	for (i = 1; i < len; i++) { // to skip column header index, use i = 1
		let dataRow = data2D[i];
		ric = dataRow[0];

		if (ricMap[ric]) {
			continue; // we interested only first row of ric
		}
		let snapData = ricMap[ric] = {};

		// loop for create rowData for update
		for (j = 1; j < fieldLen; j++) { // to skip instrument index, use j = 1
			let value = dataRow[j];
			if (value != null && value !== "") {
				if(typeof value !== 'object') {
					field = headers[j];
					snapData[field] = value;
				}
				// TODO : handled when a cell has a mistake and the value appears as {f: "1"} ( description error in fault attribute at index 1 ),
				// Therefore, we need to store information to the error field for this cell.
				// else {}

			}
		}
	}

	// return result only ric that has update data
	let updatedData = {};
	for (ric in ricMap) {
		let obj = ricMap[ric];
		if (!isEmptyObject(obj)) {
			updatedData[ric] = obj;
		}
	}

	this._dispatch("dataChanged", {
		data: updatedData
	});
};


/** @private
* @function
* @param {Array.<string>} fields
* @param {string} serverResult
*/
SnapshotFiller.prototype._onRTKTimeSeriesSuccess = function (fields, serverResult) {

	// TODO: noti the user for change structure
	this._dispatch("adcDataReceived", serverResult);
	let data2D = serverResult["rows"];
	let svHeaders = serverResult["rows"] && serverResult["rows"][0];
	if (!Array.isArray(data2D) || !Array.isArray(svHeaders)) {
		return; // TODO: Return Promise.reject(errMsg);
	}
	// Extract header dates
	let headerDates = svHeaders.slice(2).map(header => header.v);
	// Create necessary variables
	let fileNameToTSField = {};
	let count = 0;
	// Convert map to fieldMap
	let data2DLength = data2D.length;
	for (let i = 1; i < data2DLength; i++) {
		let dataRow = data2D[i];
		let fieldName = dataRow[1];
		if (!fileNameToTSField[fieldName]) {
			fileNameToTSField[fieldName] = fields[count++];
		}
	}
	// Process data and populate ricMap and childrenFieldToParent
	let ricMap = {};
	let childrenFieldToParent = {};
	for (let i = 1; i < data2DLength; i++) {
		let dataRow = data2D[i];
		let ric = dataRow[0];
		let fieldName = dataRow[1];

		if (!ricMap[ric]) {
			ricMap[ric] = {};
		}
		let snapData = ricMap[ric];
		if (headerDates.length === 1) {
			// Time series with only 1 column, it will be a normal adc field
			snapData[fileNameToTSField[fieldName]] = dataRow[2];
		} else {
			// Multiple columns, it will be a time series field
			for (let k = 0; k < headerDates.length; k++) {
				let field = fileNameToTSField[fieldName].replace("TR.", "") + "_" + headerDates[k];
				let celLData =  dataRow[2 + k];
				if(celLData !== null) {
					childrenFieldToParent[field] = fileNameToTSField[fieldName];
					snapData[field] = celLData;
				}
			}
		}
	}

	// Return result only for ric that has updated data
	let dataMapping = {};
	for (let ric in ricMap) {
		let obj = ricMap[ric];
		if (!isEmptyObject(obj)) {
			dataMapping[ric] = obj;
		}
	}

	// Dispatch events based on the header length
	let eventData = {
		data: dataMapping,
	};
	if (headerDates.length > 1) {
		eventData.timeSeries = true;
		eventData.childrenFieldToParent = childrenFieldToParent;
	}
	this._dispatch("dataChanged", eventData);
};


/* harmony default export */ const js_SnapshotFiller = (SnapshotFiller);

;// ./node_modules/tr-grid-util/es6/GroupDefinitions.js


/** @constructor
*/
let GroupDefinitions = function () {
	this._groupMap = {};
	this._childToParent = {};
};

/** @type {!Object.<string, Object>}
* @description A map of group id to group defintion
* @private
*/
GroupDefinitions.prototype._groupMap;
/** @type {!Object.<string, string>}
* @description A map of child id to parent id
* @private
*/
GroupDefinitions.prototype._childToParent;


/** @public
* @function
* @param {Object} groupDef
* @return {string}
*/
GroupDefinitions.getGroupId = function(groupDef) {
	if(groupDef) {
		return groupDef.id || "";
	}
	return "";
};
/** @public
* @param {Object.<string, Object>} groupMap
* @param {Object} groupDef
* @return {number} Return total number of parents. Return 0 if there is no parent.
*/
GroupDefinitions.calcTreeDepth = function (groupMap, groupDef) {
	let curDepth = -1;
	let curNode = groupDef;
	while(curNode) { // WARNING: infinite loop could occured, if parentId is cycle back to one of the child group
		if(++curDepth > 15) {
			console.log("WARNING: Infinite loop detected during column group creation");
			break;
		}
		let parentId = curNode.parentId;
		curNode = groupMap[parentId];
	}
	return curDepth;
};
/** @public
* @param {Object.<string, Object>} groupMap
* @param {string} groupId
* @return {Array.<string>}
*/
GroupDefinitions.getLeafDescendants = function (groupMap, groupId) {
	let groupDef = groupMap[groupId];
	if(!groupDef) {
		return null;
	}

	let leaves = [];
	let unvisitedGroups = [groupDef];
	let visitedCount = 0;
	let visitedMap = {};
	while(visitedCount < unvisitedGroups.length) {
		groupDef = unvisitedGroups[visitedCount++];
		visitedMap[groupDef.id] = true;
		let chdr = groupDef.children;
		let len = chdr ? chdr.length : 0;
		for(let i = 0; i < len; ++i) {
			let childId = chdr[i];
			groupDef = groupMap[childId];
			if(groupDef) {
				if(!visitedMap[groupDef.id]) { // Prevent infinite loop
					unvisitedGroups.push(groupDef);
				}
			} else {
				leaves.push(childId);
			}
		}
	}
	return leaves;
};


/** @private
* @function
* @param {Object} obj
* @return {Object} Return a new object with guaranteed children property
*/
GroupDefinitions._cloneObject = function(obj) {
	let newObj = cloneObject(obj);
	if(Array.isArray(newObj.children)) { // This is to prevent modification from the outside source
		newObj.children = newObj.children.slice();
	} else {
		newObj.children = [];
	}
	return newObj;
};

/** @private
* @function
* @param {Array.<string>|Object} obj
* @param {string=} groupId
* @return {Object} Return a new object with guaranteed children property
*/
GroupDefinitions._toGroupDefinition = function(obj, groupId) {
	let groupDef = null;
	if(obj) {
		if(Array.isArray(obj)) {
			groupDef = {
				children: obj.slice()
			};
		} else {
			groupDef = GroupDefinitions._cloneObject(obj);
		}
		if(groupId) {
			if(!groupDef.name) {
				groupDef.name = groupId;
			}
			groupDef.id = groupId;
		}
	}
	return groupDef;
};

/** @public
* @return {string}
*/
GroupDefinitions.prototype.toString = function() {
	let groupMap = this._groupMap;
	let lines = [];
	lines.push("=== groupDefs ===");
	for(let key in groupMap) {
		let group = groupMap[key];
		lines.push(key + ": " + JSON.stringify(group, ["id", "parentId", "children"]));
	}

	lines.push("=== childToParent ===");
	lines.push(JSON.stringify(this._childToParent, null, 2));
	lines.push("");

	return lines.join("\n");
};

/** Get group definition object. This method can be used to check whether the given id is a group id or not (i.e., isGroupId).
* @public
* @param {string} groupId
* @return {Object}
*/
GroupDefinitions.prototype.getGroup = function (groupId) {
	if(groupId) {
		return this._groupMap[groupId] || null;
	}
	return null;
};
/** @public
* @function
* @param {string} groupId
* @return {Object}
*/
GroupDefinitions.prototype.getDefinition = GroupDefinitions.prototype.getGroup;
/** Get array of all existing group definitions
* @public
* @return {!Array.<Object>}
*/
GroupDefinitions.prototype.getGroups = function () {
	let groupDefs = [];
	let groupMap = this._groupMap;
	for(let key in groupMap) {
		groupDefs.push(groupMap[key]);
	}
	return groupDefs;
};
/** Get array of all existing group ids
* @public
* @return {!Array.<string>}
*/
GroupDefinitions.prototype.getGroupIds = function () {
	return Object.keys(this._groupMap);
};
/** @public
* @return {!Object.<string, Object>}
*/
GroupDefinitions.prototype.getGroupMap = function () {
	return this._groupMap; // WARNING: Allow access to private member
};
/** @public
* @return {!Object.<string, Object>}
*/
GroupDefinitions.prototype.cloneGroupMap = function () {
	let groupMap = this._groupMap;
	let outMap = {};
	for(let groupId in groupMap) {
		let groupDef = groupMap[groupId];
		let obj = GroupDefinitions._cloneObject(groupDef);
		outMap[groupId] = obj;
	}
	return outMap;
};
/** In case of a children array has been modified outside of GroupDefinitions, this method is needed to re-create internal maps to reflect existing structure
* @public
*/
GroupDefinitions.prototype.rebuildMap = function () {
	let groupMap = this._groupMap;
	let childToParent = this._childToParent = {};

	// Create child to parent map
	let groupIds = Object.keys(groupMap);
	let grpCount = groupIds.length;
	let i, groupId;
	for(i = 0; i < grpCount; ++i) {
		groupId = groupIds[i];
		let chdr = groupMap[groupId].children;
		let childCount = chdr ? chdr.length : 0;
		for (let j = 0; j < childCount; j++) {
			childToParent[chdr[j]] = groupId;
		}
	}

	// Apply a parent id to group definition to make it easier to find depth
	for(i = 0; i < grpCount; ++i) {
		groupId = groupIds[i];
		let parentId = childToParent[groupId];
		if(parentId) {
			groupMap[groupId].parentId = parentId;
		}
	}
};

/** Get immediate child ids of the specified group. The returned array can contain both child ids and group ids
* @public
* @param {string} groupId
* @return {Array.<string>}
*/
GroupDefinitions.prototype.getGroupChildren = function (groupId) {
	let groupDef = this._groupMap[groupId];
	return groupDef ? groupDef.children : null;
};
/** Get all non-group descendants of the given group id.
* @public
* @param {string} groupId
* @return {Array.<string>}
*/
GroupDefinitions.prototype.getLeafDescendants = function (groupId) {
	return GroupDefinitions.getLeafDescendants(this._groupMap, groupId);
};
/** @public
* @param {string} groupId
* @return {number} Return -1 if the given groupDef is null. Return 0 if there is no parent. Return 1 or more if it has a parent
*/
GroupDefinitions.prototype.getGroupLevel = function (groupId) {
	return GroupDefinitions.calcTreeDepth(this._groupMap, this._groupMap[groupId]);
};
/** @public
* @param {string} groupId
* @return {Object}
*/
GroupDefinitions.prototype.getRootGroup = function (groupId) {
	if (!groupId) {
		return null;
	}
	let groupMap = this._groupMap;
	let groupDef = groupMap[groupId] || null;
	// TODO: Support column id
	if(groupDef) {
		while (groupDef.parentId) {
			let parentDef = groupMap[groupDef.parentId];
			if(parentDef) {
				groupDef = parentDef;
			} else {
				break;
			}
		}
	}
	return groupDef;
};
/** @public
* @param {string} childId
* @return {Object}
*/
GroupDefinitions.prototype.getParentGroup = function (childId) {
	return this.getGroup(this._childToParent[childId]);
};
/** @public
* @param {string} childId
* @return {Array.<string>}
*/
GroupDefinitions.prototype.getParentIds = function(childId) {
	if (childId && typeof childId === "string") {
		let groupId = this._childToParent[childId];
		if (groupId) {
			let groupIds = [groupId];
			let group = this._groupMap[groupId];
			while (group && group.parentId) {
				group = this._groupMap[group.parentId];
				if (group) {
					groupIds.push(group.id);
				}
			}
			return groupIds;
		}
	}
	return null;
};
/** @public
* @param {string} childId
* @param {number=} groupLevel Default is to retrieve immediate parent id. Use 0 to get root (topmost) group id
* @return {string}
*/
GroupDefinitions.prototype.getParentId = function (childId, groupLevel) {
	let parentId = this._childToParent[childId];
	if(groupLevel != null) {
		let currentLevel = this.getGroupLevel(parentId);
		while(currentLevel > groupLevel && parentId){
			parentId = this._childToParent[parentId];
			currentLevel--;
		}
	}

	return parentId || "";
};

/** Remove all existing group definitions
* @public
* @return {boolean}
*/
GroupDefinitions.prototype.removeAllGroups = function () {
	for(let groupId in this._groupMap) { // eslint-disable-line
		this._groupMap = {};
		this._childToParent = {};
		return true;
	}
	return false;
};
/** Remove all existing group definitions and replace them with the given definitions. Note that this method does not check for circular referencing nor duplication of ids
* @public
* @param {Array.<Object>=} groupDefs Use null or empty array to remove all existing groups
*/
GroupDefinitions.prototype.setGroups = function (groupDefs) {
	// Clear existing group structure
	let groupMap = this._groupMap = {};

	// Create group map
	let grpCount = groupDefs ? groupDefs.length : 0;
	for (let i = 0; i < grpCount; i++) {
		let groupDef = groupDefs[i];
		let groupId = groupDef.id;
		if(groupId) {
			groupMap[groupId] = GroupDefinitions._cloneObject(groupDef);
		}
	}

	this.rebuildMap();
};
/** Add new group definition to existing group structure. Existing group with the same id will be replaced.
* @public
* @param {Object} groupDef Group definition object
* @return {string} Return group ID
*/
GroupDefinitions.prototype.addGroup = function (groupDef) {
	let groupId = GroupDefinitions.getGroupId(groupDef);
	if(groupId) {
		return this.setGroup(groupId, groupDef);
	}
	return groupId;
};
/** @public
* @param {string} groupId
* @return {boolean}
*/
GroupDefinitions.prototype.removeGroup = function (groupId) {
	let curDef = this._groupMap[groupId];
	if(curDef) {
		this.removeAllChildren(groupId);
		this.unsetParent(groupId);
		delete this._groupMap[groupId];

		return true;
	}
	return false;
};
/** Replace and update existing group definition. A new group will be added, only if the given id has no match. Existing group will be removed of no new definition is given.
* @public
* @param {string} groupId
* @param {(Array.<string>|Object)=} groupDef
* @return {string} Return group Id. Return empty string, if nothing has been changed
*/
GroupDefinitions.prototype.setGroup = function (groupId, groupDef) {
	if(!groupId) {
		return "";
	}

	if(groupDef) {
		let newDef = GroupDefinitions._toGroupDefinition(groupDef, groupId);
		this._ungroupChildren(newDef.children);

		let curDef = this._groupMap[groupId];
		if(curDef) { // Replace
			this.removeAllChildren(groupId);
		}
		let parentId = this._childToParent[groupId];
		if(parentId) {
			newDef.parentId = parentId;
		}
		this._groupMap[groupId] = newDef;

		let chdr = newDef.children; // newDef is guaranteed to have children property
		let len = chdr.length;
		for(let i = 0; i < len; ++i) {
			let childId = chdr[i];
			this._childToParent[childId] = groupId;
			let childDef = this._groupMap[childId];
			if(childDef) {
				childDef.parentId = groupId;
			}
		}
	} else { // Remove
		if(!this.removeGroup(groupId)) {
			return "";
		}
	}

	return groupId;
};

/** Remove each child from any group
 * @private
 * @param {Array.<string>} children
 */
GroupDefinitions.prototype._ungroupChildren = function(children) {
	if (Array.isArray(children)) {
		let len = children.length;
		for(let i = 0; i < len; ++i) {
			this.unsetParent(children[i]);
		}
	}
};

/** Check if the given child id is an immediate child of the given group
* @public
* @param {string} parentId Group id
* @param {string} childId
* @return {boolean}
*/
GroupDefinitions.prototype.hasGroupChild = function (parentId, childId) {
	let groupDef = this._groupMap[parentId];
	if(childId && groupDef) {
		let chdr = groupDef.children;
		if(chdr) {
			return chdr.indexOf(childId) >= 0; // TODO: Use childToParent map to improve performance
		}
	}
	return false;
};
/** Check if the specified id is contained within the given group regardless of any group level that the id is in
* @public
* @param {string} groupId
* @param {string} childId
* @return {boolean}
*/
GroupDefinitions.prototype.contains = function (groupId, childId) {
	if(groupId && childId) {
		if(groupId === childId) {
			return true;
		}
		let levelLimit = 20;
		let parentId = this._childToParent[childId];
		while(parentId && levelLimit) { // WARNING: Circular dependency could happen
			if(groupId === parentId) {
				return true;
			}
			--levelLimit;
			parentId = this._childToParent[parentId];
		}
	}
	return false;
};
/** @public
* @param {string} parentId Group id
* @param {string} childId
* @param {number=} position
* @return {boolean}
*/
GroupDefinitions.prototype.addGroupChild = function (parentId, childId, position) {
	let groupDef = this._groupMap[parentId];

	if(childId && groupDef) {
		let chdr = groupDef.children;
		if(chdr && chdr.indexOf(childId) < 0) {
			this.unsetParent(childId); // Remove previous parent
			// Add new child to group structures
			this._childToParent[childId] = parentId;
			let childDef = this._groupMap[childId];
			if(childDef) {
				childDef.parentId = parentId;
			}
			if(position != null && position >= 0) {
				chdr.splice(position, 0, childId);
			} else {
				chdr.push(childId);
			}
			return true;
		}
	}
	return false;
};
/** Remove the given child from the specified parent. If childId is not given, unsetParent will be used on parentId instead of childId.
* @public
* @param {string} parentId Group id
* @param {string=} childId
* @return {boolean}
*/
GroupDefinitions.prototype.removeGroupChild = function (parentId, childId) {
	if(childId == null) {
		return this.unsetParent(parentId);
	}
	if(this.hasGroupChild(parentId, childId)) {
		return this.unsetParent(childId);
	}
	return false;
};
/** Remove the given child from its own parent (i.e., unset Parent of the given child).
* @public
* @param {string} childId
* @return {boolean}
*/
GroupDefinitions.prototype.unsetParent = function (childId) {
	let parentId = this._childToParent[childId];
	if(!parentId) {
		return false;
	}
	this._childToParent[childId] = "";
	let childDef = this._groupMap[childId];
	if(childDef) {
		childDef.parentId = "";
	}
	let parentDef = this._groupMap[parentId];
	if(parentDef) {
		let chdr = parentDef.children;
		if(chdr && chdr.length) {
			let at = chdr.indexOf(childId);
			if (at >= 0) {
				chdr.splice(at, 1); // splice is slow
			}
		}
	}
	return true;
};
/** Remove all children from the specified group
* @public
* @param {string} groupId
* @return {boolean}
*/
GroupDefinitions.prototype.removeAllChildren = function(groupId) {
	let grpDef = this._groupMap[groupId];
	if(grpDef) {
		let chdr = grpDef.children;
		let len = chdr ? chdr.length : 0;
		if(len) {
			grpDef.children = [];
			for(let i = 0; i < len; ++i) {
				let childId = chdr[i];
				if(this._childToParent[childId]) {
					this._childToParent[childId] = "";
				}
				let childDef = this._groupMap[childId];
				if(childDef) {
					childDef.parentId = "";
				}
			}
			return true;
		}
	}
	return false;
};
/** Replace and update existing group definition.
* @public
* @param {string} groupId
* @param {Array.<string>} newChildList If null is given, all existing children in the group will be removed
* @return {boolean}
*/
GroupDefinitions.prototype.setGroupChildren = function (groupId, newChildList) {
	let groupDef = this._groupMap[groupId];
	if(groupDef) {
		if(Array.isArray(newChildList)) {
			let chdr = newChildList.slice();
			this._ungroupChildren(chdr);
			this.removeAllChildren(groupId);
			groupDef.children = chdr;

			let parentId = groupDef.id;
			let len = chdr.length;
			for(let i = 0; i < len; ++i) {
				let childId = chdr[i];
				this._childToParent[childId] = parentId;
				let childDef = this._groupMap[childId];
				if(childDef) {
					childDef.parentId = parentId;
				}
			}

			return true;
		} else if(!newChildList && groupDef.children.length) {
			this.removeAllChildren(groupId);
			return true;
		}
	}
	return false;
};

/** @public
* @param {string} groupId
* @return {string}
*/
GroupDefinitions.prototype.getGroupName = function (groupId) {
	let groupDef = this._groupMap[groupId];
	if(groupDef) {
		return groupDef.name || "";
	}
	return "";
};
/** @public
* @param {string} groupId
* @param {string} groupName
* @return {boolean}
*/
GroupDefinitions.prototype.setGroupName = function (groupId, groupName) {
	let groupDef = this._groupMap[groupId];
	if(groupDef) {
		if(groupDef.name !== groupName) {
			groupDef.name = groupName;
			return true;
		}
	}

	return false;
};

/* harmony default export */ const es6_GroupDefinitions = ((/* unused pure expression or super */ null && (GroupDefinitions)));


;// ./node_modules/@grid/core/es6/grid/components/ElementWrapper.js






/** A helper class that add access to native HTML element APIs through simple JavaScript
 * @constructor
 * @extends {EventDispatcher}
 * @param {Element=} opt_element If not specified "div" element is instantly created.
 */
let ElementWrapper = function (opt_element) {
	this._element = opt_element ? opt_element : document.createElement("div");
};
es6_Ext.inherits(ElementWrapper, event_EventDispatcher);

/** @protected
* @ignore
* @type{Element}
*/
ElementWrapper.prototype._element = null;
/** @protected
* @ignore
* @type{Element|NodeList}
*/
ElementWrapper.prototype._content = null;
/** @protected
* @ignore
* @type{Element}
*/
ElementWrapper.prototype._textSpan = null;

/** @private
* @type{Object.<string, !EventListeners>}
*/
ElementWrapper.prototype._nativeEvents = null;
/** @private
* @type {Object}
*/
ElementWrapper._passiveEvents = null;


//#region Content Manipulation
/**
* @public
* @param {boolean=} opt_allContent If true all children will be returned
* @return {Element|Node|NodeList} Returns the last child, if there are multiple children. Returns null if there is no child
*/
ElementWrapper.prototype.getContent = function (opt_allContent) {
	if(opt_allContent) {
		return this._element.children; // WARNING: Returns private member
	}
	return /** @type{Element|Node} */(this._element.lastChild);
};
/** Replaces all existing child elements with the specified content. This method automatically converts all non-element to string wrapped with <span> for proper display.
* @public
* @param {string|Element|ElementWrapper|*} content Use null to clear content. If string is given, <span> tag is automatically added to host the string.
* @param {string=} opt_tooltip If undefined, content is added without modification. If string is given, it is added to title attribute of the content
* @return {Element}
*/
ElementWrapper.prototype.setContent = function (content, opt_tooltip) {
	if (content == null || content !== content) {
		content = null;
	} else if (content.nodeType !== 1 && content.nodeType !== 11) { // Element type checking (1 = DOM, 11 = DocumentFragment)
		if (content["getElement"] == null) {
			if (!util.isString(content)) { content = content + ""; }

			if (this._textSpan == null) {
				this._textSpan = document.createElement("span"); // For fastest speed
			}
			this._textSpan.textContent = content;

			content = this._textSpan;
		} else {
			content = content["getElement"]();
		}
	}

	if (content !== this._content) {
		this._content = /** @type{Element} */(content);
		while (this._element.firstChild !== null) {
			this._element.removeChild(this._element.firstChild);
		}
		if (this._content !== null) {
			this._element.appendChild(this._content);
		}
	}

	if (opt_tooltip && content) {
		content.setAttribute("title", opt_tooltip);
	}
	return /** @type{Element} */(this._content); // Can only be an element at this point
};
/** Append the specified content to the existing children list. Note that string is not acceptable
* Use setParent() if you just want to put an element into the document. This method is specifically designed for user content, but not for element reordering
* @public
* @param {Element|Node|ElementWrapper} content will be appended as a child of this element
* @return {Element}
*/
ElementWrapper.prototype.addContent = function (content) {
	content = util.getElement(content);
	if (content == null) { return null; }

	this._content = this._element.children;
	this._element.appendChild(content);
	return content;
};
/** Removes the specified content from the existing children list
* @public
* @param {Element|Node|ElementWrapper} content If null is given, nothing is done
* @return {boolean} true if the content is removed
*/
ElementWrapper.prototype.removeContent = function (content) {
	content = util.getElement(content);
	if (content == null) { return false; }

	this._content = this._element.children;
	this._element.removeChild(content);
	return true;
};
//#endregion Content Manipulation

//#region Event Related Methods
/** Add a event listener to either native or custom event provided from the instance.
* @public
* @param {string} type All standard HTML element event types are allowed
* @param {Function} handler Callback method
* @param {number=} opt_priority The higher the priority, the sooner the listener get executed. Undefined priority listener will be executed last
*
* @example
* let grid = new Core();
* grid.listen("click", function(e) { console.log("Grid is clicked"); });
*/
ElementWrapper.prototype.listen = function (type, handler, opt_priority) {
	if(this._customEvents && this._customEvents[type]) {
		this._customEvents[type].listen(handler, opt_priority);
		return;
	}

	if (!this._nativeEvents) {
		this._nativeEvents = {};
	}
	let e = this._nativeEvents[type];
	if (!e) {
		e = new event_EventListeners();
		this._nativeEvents[type] = e;

		e.dispatch = e.dispatch.bind(e);

		if (!ElementWrapper._passiveEvents) {
			// https://github.com/WICG/EventListenerOptions/blob/gh-pages/explainer.md#feature-detection
			let supportsPassive = false;
			try {
				let opts = /** @type{boolean} */(Object.defineProperty({}, 'passive', {
					/**
					 * @return {boolean}
					 */
					"get": function() {
						supportsPassive = true;
						return false;
					}
				}));
				window.addEventListener("testPassive", null, opts);
				window.removeEventListener("testPassive", null, opts);
			} catch (err) {}

			if(supportsPassive) {
				let passiveOption = { "passive": true };

				// "wheel" and "mousewheel" have default behavior that needs to be prevented. We have to exclude them from the list
				ElementWrapper._passiveEvents = {
					// "wheel": passiveOption,
					// "mousewheel": passiveOption,
					"touchstart": passiveOption,
					"touchmove": passiveOption
				};
			} else {
				ElementWrapper._passiveEvents = {};
			}
		}

		this._element.addEventListener(type, e.dispatch, ElementWrapper._passiveEvents[type] || false);
	}

	e.listen(handler, opt_priority);
};
/** Removes an event listener which was added with listen(). The same function handler must be sent in, or else it will not be removed
* @public
* @param {string} type The name of the event without the 'on' prefix.
* @param {Function} handler The listener function to remove.
*/
ElementWrapper.prototype.unlisten = function (type, handler) {
	if(this._customEvents && this._customEvents[type]) {
		this._customEvents[type].unlisten(handler);
		return;
	}

	if (this._nativeEvents) {
		let e = this._nativeEvents[type];
		if (e) {
			e.unlisten(handler);
		}
	}
};
/** Removes all listeners from an object. If no type is specified, it will remove all listeners that have been registered.
* You can also optionally remove listeners of a particular type.
* @public
* @param {string=} opt_type Type of event to, default is all types.
*/
ElementWrapper.prototype.unlistenAll = function (opt_type) {
	if(this._customEvents) {
		if(opt_type) {
			if(this._customEvents[opt_type]) {
				this._customEvents[opt_type].unlistenAll();
			}
		} else {
			for (let customEvt in this._customEvents) {
				this._customEvents[customEvt].unlistenAll();
			}
		}
	}

	if (this._nativeEvents) {
		if (opt_type) {
			if (this._nativeEvents[opt_type]) {
				this._unlistenAll(opt_type);
				delete this._nativeEvents[opt_type];
			}
		} else {
			for (let key in this._nativeEvents) {
				this._unlistenAll(key);
			}
			this._nativeEvents = null;
		}
	}
};
/** @protected
* @ignore
* @param {string} type
*/
ElementWrapper.prototype._unlistenAll = function (type) {
	let e = this._nativeEvents[type];
	this._element.removeEventListener(type, e.dispatch, false);
	e.unlistenAll();
};
/** Calls this method, if it is NO LONGER needed and we want to free up memory.
* Any usage after disposition will cause exception error
* @public
*/
ElementWrapper.prototype.dispose = function () {
	this._dispose();
};
/** @protected
* @ignore
*/
ElementWrapper.prototype._dispose = function () {
	this.setParent(null);
	this.unlistenAll();

	this._element = this._content = this._textSpan = null;
	this._customEvents = this._nativeEvents = null;
};
/** Set Extender method that will be executed before dispatcing an event to provide additional event argument. This only works for custom event
* @ignore
* @public
* @param {string} type
* @param {Function} func
*/
ElementWrapper.prototype.setEventArgExtender = function (type, func) {
	if(this._customEvents) {
		let evt = this._customEvents[type];
		if(evt) {
			evt.setEventArgExtender(func);
		}
	}
};
//#endregion Event Related Methods

//#region DOM Manipulation
/** Gets top element of this control
* @public
* @return {Element}
*/
ElementWrapper.prototype.getElement = function () {
	return this._element;
};
/**
* @public
* @return {Element|Node}
*/
ElementWrapper.prototype.getNextSibling = function () {
	return this._element.nextSibling;
};
/**
* @public
* @return {Element|Node}
*/
ElementWrapper.prototype.getParent = function () {
	return this._element.parentNode;
};
/**
* @public
* @param {ElementWrapper|Element|Node} parent Uses null to clear parent
* @param {boolean=} opt_prepend If false (default), this element is appended to child list of the parent
*/
ElementWrapper.prototype.setParent = function (parent, opt_prepend) {
	this._setParent(parent, opt_prepend);
};
/** @protected
* @ignore
* @param {Element} parent
* @param {boolean=} opt_prepend
*/
ElementWrapper.prototype._setParent = function (parent, opt_prepend) {
	parent = util.getElement(parent);
	if (parent !== null) {
		if (opt_prepend) {
			parent.insertBefore(this._element, parent.firstChild);
		} else {
			parent.appendChild(this._element);
		}
	} else if (this._element.parentNode !== null) {
		this._element.parentNode.removeChild(this._element);
	}
};
/** Inserts this element before the specified sibling. Parent of this element will be changed to sibling's parent
* If sibling parent does not exist, nothing is performed.
* If nextSibling is null, this element will place itself at the end of its parent child list
* @public
* @param {ElementWrapper|Element|Node} nextSibling
*/
ElementWrapper.prototype.insertBefore = function (nextSibling) {
	this._insertBefore(nextSibling);
};
/** @protected
* @ignore
* @param {Element} nextSibling
*/
ElementWrapper.prototype._insertBefore = function (nextSibling) {
	nextSibling = util.getElement(nextSibling);
	if (nextSibling == null) {
		this.setParent(this.getParent());
		return;
	}

	let parent = nextSibling.parentNode;
	if (parent == null) { return; }

	parent.insertBefore(this._element, nextSibling);
};
/** Inserts this element after the specified sibling. Parent of this element will be changed to sibling's parent
* If sibling parent does not exist, nothing is performed.
* If previousSibling is null, this element will place itself at the end of its parent child list
* @public
* @param {ElementWrapper|Element|Node} previousSibling
*/
ElementWrapper.prototype.insertAfter = function (previousSibling) {
	previousSibling = util.getElement(previousSibling);
	if(previousSibling) {
		if(previousSibling.nextSibling) {
			this.insertBefore(previousSibling.nextSibling);
		} else {
			this.setParent(previousSibling.parentNode);
		}
	} else {
		this.setParent(this.getParent(), true);
	}
};
//#endregion DOM Manipulation

//#region HTML Styling
/** Checks if the given CSS class is in className of the element
* @public
* @param {string} str
* @return {boolean}
*/
ElementWrapper.prototype.hasClass = function (str) {
	return (this._element) ? this._element.classList.contains(str) : false;
};
/** If the given CSS class is already in className, the class is not added twice
* @public
* @param {string} str
*/
ElementWrapper.prototype.addClass = function (str) {
	if (this._element) {
		this._element.classList.add(str);
	}
};
/**
* @public
* @param {string} str
*/
ElementWrapper.prototype.removeClass = function (str) {
	if (this._element) {
		this._element.classList.remove(str);
	}
};
/** Toggles the specified class
* @public
* @param {string} cls
* @param {(boolean|null)=} forcedVal If specified, force CSS class state as the specified value
*/
ElementWrapper.prototype.toggleClass = function (cls, forcedVal) {
	let elem = this._element;
	if(elem) {
		if(forcedVal == null) {
			forcedVal = !elem.classList.contains(cls);
		}
		if(forcedVal) {
			elem.classList.add(cls);
		} else {
			elem.classList.remove(cls);
		}
	}
};
/** Adds or removes CSS class by specifying "enabled" parameter
* @public
* @param {string} str
* @param {boolean=} opt_enabled Default is true
*/
ElementWrapper.prototype.enableClass = function (str, opt_enabled) {
	if (opt_enabled !== false) {
		this.addClass(str);
	} else {
		this.removeClass(str);
	}
};
/**
* @public
* @return {string}
*/
ElementWrapper.prototype.getClass = function () { return this._element.className; };
/**
* @public
* @param {string} str All existing classes will be placed by this value
*/
ElementWrapper.prototype.setClass = function (str) { this._element.className = str; };

/** @public
* @param {string} str
* @return {string}
*/
ElementWrapper.prototype.getAttribute = function (str) { return this._element.getAttribute(str); };
/** @public
* @param {string} str
* @return {boolean}
*/
ElementWrapper.prototype.hasAttribute = function (str) { return this._element.hasAttribute(str); };
/** Sets attribute to DOM element
* @public
* @param {string} str
* @param {string} val Uses empty string ("") to remove the attribute value
*/
ElementWrapper.prototype.setAttribute = function (str, val) { this._element.setAttribute(str, val); };
/** Sets multiple attributes all at once
* @public
* @param {Object} json Key is the attribute names and value is string value (i.e. "readonly", "50", "checkbox")
*/
ElementWrapper.prototype.setAttributes = function (json) {
	for (let key in json) {
		this._element.setAttribute(key, json[key]);
	}
};
/** Completely removes attribute from DOM element for both key and values
 * @public
 * @param {string} str
 */
ElementWrapper.prototype.removeAttribute = function (str) { this._element.removeAttribute(str); };

/**
* @public
* @param {string} str
* @return {string}
*/
ElementWrapper.prototype.getStyle = function (str) { return this._element.style[str]; };
/**
* @public
* @param {string} str
* @param {string} val
*/
ElementWrapper.prototype.setStyle = function (str, val) { this._element.style[str] = val; };
/** Sets multiple css properties all at once
* @public
* @param {Object} json Key is the property names and value is style value (i.e. "center", "#FF00FF", "10px")
*/
ElementWrapper.prototype.setStyles = function (json) {
	let styles = this._element.style;
	for (let key in json) {
		styles[key] = json[key];
	}
};
//#endregion HTML Styling

//#region Common HTML Helpers
/**
* @public
* @return {string}
*/
ElementWrapper.prototype.getId = function () {
	return this._element.id;
};
/**
* @public
* @param {string} val
*/
ElementWrapper.prototype.setId = function (val) {
	this._element.id = val;
};
/** Return a string that represents the object mainly for debugging
* @public
* @return {string}
*/
ElementWrapper.prototype.toString = function () {
	let outer = this._element.outerHTML;
	outer = outer.replace(this._element.innerHTML, "");
	return outer;
};

/** Gets textContent (IE9+) value from this element
* @public
* @return {string}
*/
ElementWrapper.prototype.getTextContent = function () {
	return this._element.textContent;
};
/** Sets textContent (IE9+) value this will replace all childNodes with a single TextNode
* @public
* @param {string} str
*/
ElementWrapper.prototype.setTextContent = function (str) {
	this._element.textContent = str;
};

/** Gets the left position that is relative to its parent's client area
* @public
* @return {number}
*/
ElementWrapper.prototype.getLeft = function () { return this._element.offsetLeft; };
/**
* @public
* @param {number} val in pixel
* @return {boolean} true if there is any change, otherwise false
*/
ElementWrapper.prototype.setLeft = function (val) {
	this._element.style.left = val + "px";
	return true;
};

/** Gets the top position that is relative to its parent's client area
* @public
* @return {number}
*/
ElementWrapper.prototype.getTop = function () { return this._element.offsetTop; };
/**
* @public
* @param {number} val in pixel
* @return {boolean} ture if there is any change, otherwise false
*/
ElementWrapper.prototype.setTop = function (val) {
	this._element.style.top = val + "px";
	return true;
};

/** Gets offsetWidth
* @public
* @return {number}
*/
ElementWrapper.prototype.getWidth = function () { return this._element.offsetWidth; };
/** For complex sizing, use setStyle("width", "-webkit-calc(100% - 10px)") instead.
* @public
* @param {number} val in pixel
* @return {boolean} true if there is any change, otherwise false
*/
ElementWrapper.prototype.setWidth = function (val) {
	this._element.style.width = val + "px";
	return true;
};

/** Gets offsetHeight
* @public
* @return {number} in pixel
*/
ElementWrapper.prototype.getHeight = function () { return this._element.offsetHeight; };
/** For complex sizing, use setStyle("height", "-webkit-calc(100% - 10px)") instead.
* @public
* @param {number} val in pixel
* @return {boolean} true if there is any change, otherwise false
*/
ElementWrapper.prototype.setHeight = function (val) {
	if (val < 0) { val = 0; }
	this._element.style.height = val + "px";
	return true;
};

/** The returned value is a TextRectangle object, which contains read-only left,
 * top, right and bottom properties describing the border-box, in pixels, with the top-left
 * relative to the top-left of the viewport.
 * @public
 * @return {Object}
 */
ElementWrapper.prototype.getBoundingClientRect = function () {
	return this._element.getBoundingClientRect();
};

/** Sets width in percentage corresponding to the parent
* @public
* @param {number=} opt_percent If not defined, 100% width is used
* @return {boolean} true if there is any change, otherwise false
*/
ElementWrapper.prototype.fillParentWidth = function (opt_percent) {
	if (opt_percent == null) { opt_percent = 100; }

	this._element.style.width = opt_percent + "%";
	return true;
};
/** Sets height in percentage corresponding to the parent
* @public
* @param {number=} opt_percent If not defined, 100% height is used
* @return {boolean} true if there is any change, otherwise false
*/
ElementWrapper.prototype.fillParentHeight = function (opt_percent) {
	if (opt_percent == null) { opt_percent = 100; }

	this._element.style.height = opt_percent + "%";
	return true;
};

/** Gets tooltip text from "title" attribute
* @public
* @return {string}
*/
ElementWrapper.prototype.getTooltip = function () { return this.getAttribute("title"); };
/** Sets tooltip text to "title" attribute
* @public
* @param {string} str
*/
ElementWrapper.prototype.setTooltip = function(str) {
	if(str) {
		this.setAttribute("title", str);
	} else {
		this.removeAttribute("title");
	}
};

/** Sets display style as inline style to the element with value of empty string
* @public
* @param {boolean=} opt_shown If false the element will be hidden
*/
ElementWrapper.prototype.show = function (opt_shown) {
	this._element.style.display = (opt_shown !== false) ? "" : "none";
};
/** Sets display style as inline style to the element with value of "none"
* @public
* @param {boolean=} opt_hidden If false the element will be shown
*/
ElementWrapper.prototype.hide = function (opt_hidden) {
	this.show(opt_hidden === false);
};
/** Check to see if display has "none" as a inline style. Note that any change in visibility and class name may not be reflected by the returned value.
* @public
* @return {boolean}
*/
ElementWrapper.prototype.isVisible = function () { return this._element.style.display !== "none"; };

/** Enable control element
* @public
* @param {boolean=} opt_enabled If false the element will be hidden
*/
ElementWrapper.prototype.enable = function (opt_enabled) {
	opt_enabled = opt_enabled !== false;
	if (opt_enabled) {
		this.removeAttribute("disabled");
	} else {
		this.setAttribute("disabled", "disabled");
		this._element.disabled = !opt_enabled;
	}
};
/** Disable control element
* @public
* @param {boolean=} opt_disabled If false the element will be shown
*/
ElementWrapper.prototype.disable = function (opt_disabled) {
	this.enable(opt_disabled === false);
};
/** Check if control element is enabled
* @public
* @return {boolean} true if element is enabled
*/
ElementWrapper.prototype.isEnabled = function () {
	return !this._element.hasAttribute("disabled");
};

/** Note that if element display style is set to "none" or not in the document, scroll value will automatically reset to zero and will not be restored when it is back to visible state
* @public
* @return {number}
*/
ElementWrapper.prototype.getScrollLeft = function () { return this._element.scrollLeft; };
/**
* @public
* @return {number}
*/
ElementWrapper.prototype.getScrollTop = function () { return this._element.scrollTop; };
/**
* @public
* @return {boolean}
*/
ElementWrapper.prototype.isEndOfHorizontalScroll = function () { return this._element.scrollLeft >= this._element.scrollWidth - this.getWidth(); };
/**
* @public
* @return {boolean}
*/
ElementWrapper.prototype.isEndOfVerticalScroll = function () { return this._element.scrollTop >= this._element.scrollHeight - this.getHeight(); };
//#endregion Common HTML Helpers

ElementWrapper._proto = ElementWrapper.prototype;

/* harmony default export */ const components_ElementWrapper = (ElementWrapper);


;// ./node_modules/@grid/core/es6/grid/components/CellFloatingPanel.js
/* eslint-disable */


/* eslint-enable */

/** @constructor
* @extends {ElementWrapper}
*/
let CellFloatingPanel = function () {
	this._items = [];
	this._element = null;
};
es6_Ext.inherits(CellFloatingPanel, components_ElementWrapper);

/** @public
* @type {Array.<Element>}
*/
CellFloatingPanel.prototype._items = [];

/** @public
 * @returns {boolean}
*/
CellFloatingPanel.prototype.hasItem = function () {
	return !!this._items.length;
};
/** @public
 * @param {Element} icon Element of item
 * @returns {boolean}
*/
CellFloatingPanel.prototype.containItem = function (icon) {
	return this._items.indexOf(icon) > -1;
};
/** @public
* @param {Element} item
* @param {number=} opt_order The greater the order, the farther the position to the right
*/
CellFloatingPanel.prototype.insertItem = function (item, opt_order) {
	let items = this._items;
	if(items.indexOf(item) > -1) {
		return; // There is no need to insert the same icon
	}
	items.push(item);

	// Ordering icons
	if(opt_order) {
		item._order = opt_order;
	}
	if (items.length) {
		items.sort(CellFloatingPanel._iconComparer);
	}
	this._updatePanelItems();
};
/** @public
* @param {(Element|string)=} elemRef This can be the element or className (string) of the icon. If not specified, the last item will be remove.
* @return {Element|null} removed element
*/
CellFloatingPanel.prototype.removeItem = function (elemRef) {
	let items = this._items;
	let targetIndex = -1;
	if (!elemRef) {
		targetIndex = items.length - 1;
	} else {
		for(let i = items.length; --i >= 0;) {
			let ch = items[i];
			if (typeof elemRef === "string") {
				if(ch.classList.contains(elemRef)) {
					targetIndex = i;
					break;
				}
			} else {
				if (ch === elemRef) {
					targetIndex = i;
					break;
				}
			}
		}
	}
	let removedItem = null;
	if (targetIndex > -1) {
		removedItem = items.splice(targetIndex, 1)[0];
	}
	this._updatePanelItems();
	return removedItem;
};
/** @public
*/
CellFloatingPanel.prototype.clearItems = function () {
	let items = this._items;
	if (!items.length) {
		return;
	}
	items.length = 0;
	this._updatePanelItems();
};

/** @private
* @param {Element} elemA
* @param {Element} elemB
* @return {number}
*/
CellFloatingPanel._iconComparer = function(elemA, elemB) {
	if(!elemA._order) {
		return (!elemB._order) ? 0 : -1;
	} else if(!elemB._order) {
		return 1;
	}
	return (elemA._order - elemB._order);
};

/** @private
*/
CellFloatingPanel.prototype._updatePanelItems = function() {
	let items = this._items;
	let elem = this._element;
	if (!items.length) {
		return;
	}
	if (!this._element) {
		elem = this._element = document.createElement("div");
	}
	while (elem.lastChild) {
		elem.removeChild(elem.lastChild);
	}
	for (let i = 0; i < items.length; i++) {
		elem.appendChild(items[i]);
	}
};

/* harmony default export */ const components_CellFloatingPanel = (CellFloatingPanel);


;// ./node_modules/@grid/core/es6/grid/components/Cell.js
/* eslint-disable */





/* eslint-enable */

/** Cell container. Please see {@link ElementWrapper} for full APIs and parameter lists
* @constructor
* @extends {ElementWrapper}
* @param {Element=} opt_element
* @param {ILayoutGrid=} opt_section Parent section
*/
let Cell = function (opt_element, opt_section) {
	this._id = Cell._runningId++;
	this._element = document.createElement("div");
	this._element.className = "cell";
	if (opt_section) {
		this._section = opt_section;
	}
};
es6_Ext.inherits(Cell, components_ElementWrapper);

/** Cell's aligner
* @private
* @type {Element}
*/
Cell.prototype._aligner = null;
/** @type {number}
* @private
*/
Cell._runningId = 0;

/** @type {number}
* @private
*/
Cell.prototype._id = 0;
/** @type {number}
* @private
*/
Cell.prototype._width = -1;  // Negative value to allow using actual element size
/** @type {number}
* @private
*/
Cell.prototype._height = -1;  // Negative value to allow using actual element size
/** @type {boolean}
* @private
*/
Cell.prototype._visible = true;

/**
* @private
* @type {*}
*/
Cell.prototype._rawContent;

/** Parent section of the cell
* @private
* @type {ILayoutGrid}
*/
Cell.prototype._section = null;
/** @private
* @type {boolean}
*/
Cell.prototype._contentLocked = false;
/** @private
* @type {number}
*/
Cell.prototype._collapsedState = 0;
/** @private
* @type {CellFloatingPanel}
*/
Cell.prototype._floatingPanel = null;
/** @private
* @type {CellFloatingPanel}
*/
Cell.prototype._frontIcon = null;
/** @type {Object}
* @private
*/
Cell.prototype._tooltipInfo = null;

//#region ElementWrapper
/** {@link ElementWrapper#getContent}
* @override
*/
Cell.prototype.getContent = function (opt_allContent) {
	if(opt_allContent) {
		if(this._element) {
			return this._element.children;
		}
	}
	return /** @type{Node} */(this._content) || null;
};
/** {@link ElementWrapper#setContent}
* @override
*/
Cell.prototype.setContent = function (content, opt_tooltip) {
	if (this._contentLocked) { return null; }

	if (content == null || content !== content) { // null or NaN
		content = null;
	} else if (content.nodeType !== 1 && content.nodeType !== 11) { // Element type checking (1 = DOM, 11 = DocumentFragment)
		if (content["getElement"] == null) {
			if (!(typeof content === "string")) { content = content + ""; }
		} else {
			content = /** @type{Element} */(content["getElement"]());
		}
	}

	if(this._aligner) {// Hack: Someone has changed the content without notice
		let fc = this._aligner.firstChild;
		if(fc !== this._content) {
			this._rawContent = fc;
			this._content = /** @type{Element} */(fc);
		}
	}

	let ret = this._setRawContent(content);

	if (opt_tooltip != null && this._content) { // Apply tooltip after the real content has been populated
		this._content.setAttribute("title", opt_tooltip);
	}
	return ret;
};
/** Fastest way to set simple text to the Cell
* @override
* @public
* @param {string} str
*/
Cell.prototype.setTextContent = function (str) {
	if (str == null) { str = ""; }
	if(this._rawContent === str) { return; }

	if(typeof this._rawContent !== "string") {
		if(this._rawContent == null && !this._textSpan) { // First time initialization
			this.initialize();
			this._rawContent = str;
			this._textSpan = document.createElement("span");
			this._textSpan.className = "text";
			this._textSpan.textContent = str;
			this._content = this._textSpan;
			this._aligner.className = "valigner";
			this._aligner.appendChild(this._textSpan);
		} else {
			this.setContent(str);
		}
		return;
	}
	this._rawContent = str;
	this._textSpan.textContent = str;
};


/** {@link ElementWrapper#dispose}
* @override
*/
Cell.prototype.dispose = function () {
	this.removeParent();
	this.unlistenAll();

	this._element = this._section = this._aligner = null;
	this._textSpan = this._rawContent = this._content = null;

	this._frontIcon = this._floatingPanel = null;
};

/** {@link ElementWrapper#getNextSibling}
* @override
*/
Cell.prototype.getNextSibling = function () {
	return (this._element !== null) ? this._element.nextSibling : null;
};
/** {@link ElementWrapper#getParent}
* @override
*/
Cell.prototype.getParent = function () {
	return (this._element !== null) ? this._element.parentNode : null;
};
/** {@link ElementWrapper#setParent}
* @override
*/
Cell.prototype.setParent = function (obj, opt_prepend) {
	let parent = util.getElement(obj);
	if (parent) {
		this.initialize();
		if(this._collapsedState === 1) {
			return; // Cell is being collapsed and not in the document
		}

		let sibling;
		if (opt_prepend === true) {
			sibling = parent.firstChild;
		}
		if (sibling) {
			parent.insertBefore(this._element, sibling);
		} else {
			parent.appendChild(this._element);
		}
	} else {
		this.removeParent();
	}
};
/** @public
* @ignore
*/
Cell.prototype.removeParent = function () {
	let el = this._element;
	if (el && el.parentNode) {
		el.parentNode.removeChild(el);
	}
};
/** {@link ElementWrapper#insertBefore}
* @override
*/
Cell.prototype.insertBefore = function (nextSibling) {
	let nextNode = util.getElement(nextSibling);
	if (nextNode == null) {
		this.setParent(this.getParent());
		return;
	}

	let parent = nextNode.parentNode;
	if (parent == null) { return; }

	this.initialize();
	parent.insertBefore(this._element, nextNode);
};


/** {@link ElementWrapper#getTextContent}
* @override
*/
Cell.prototype.getTextContent = function () {
	if(this._rawContent) {
		return (typeof this._rawContent == "string") ? /** @type{string} */(this._rawContent) : this._rawContent.textContent;
	}
	return "";
};
/** @public
* @return {Element} return the element of the text content
*/
Cell.prototype.getTextElement = function () {
	return this._textSpan;
};

/** {@link ElementWrapper#getWidth}
* @override
*/
Cell.prototype.getWidth = function () {
	if(this._width >= 0) {
		return this._width;
	}

	return this._element.offsetWidth;
};
/** {@link ElementWrapper#setWidth}
* @override
*/
Cell.prototype.setWidth = function (px) {
	if(px < 0) { px = -1; }
	if(this._width !== px) {
		this._width = px;
		this._element.style.width = (px >= 0) ? px + "px" : "";
		this._updateDisplay();
		return true;
	}
	return false;
};

/** {@link ElementWrapper#getHeight}
* @description WARNING: Even if this element is hidden, its height is still returned as positive value
* @override
*/
Cell.prototype.getHeight = function () {
	if(this._height >= 0) {
		return this._height;
	}

	return this._element.offsetHeight;
};
/** {@link ElementWrapper#setHeight}
* @override
*/
Cell.prototype.setHeight = function (px) {
	if(px < 0) { px = -1; }
	if(this._height !== px) {
		this._height = px;
		this._element.style.height = (px >= 0) ? px + "px" : "";
		this._updateDisplay();
		return true;
	}
	return false;
};

/** Cell cannot be resized by this way {@link ElementWrapper#fillParentWidth}
* @override
*/
Cell.prototype.fillParentWidth = function (percent) { return false; };
/** Cell cannot be resized by this way {@link ElementWrapper#fillParentHeight}
* @override
*/
Cell.prototype.fillParentHeight = function (percent) { return false; };
/** {@link ElementWrapper#show}
* @override
*/
Cell.prototype.show = function (opt_shown) {
	let shown = opt_shown !== false;
	if(shown !== this._visible) {
		this._visible = shown;
		this._updateDisplay();
	}
};
/** {@link ElementWrapper#isVisible}
* @override
*/
Cell.prototype.isVisible = function () {
	return this._visible;
};

/** @deprecated
 * @returns {number}
*/
Cell.prototype.getScrollLeft = function() { return 0; };
/** @deprecated
 * @returns {number}
 */
Cell.prototype.getScrollTop = function() { return 0; };
/** @deprecated
 * @returns {boolean}
 */
Cell.prototype.isEndOfHorizontalScroll = function() { return true; };
/** @deprecated
 * @returns {boolean}
 */
Cell.prototype.isEndOfVerticalScroll = function() { return true; };
//#endregion ElementWrapper

//#region Cell Specific Methods
/** @public
* @return {boolean}
*/
Cell.prototype.hasElement = function () {
	return this._element !== null;
};
/** @public
* @return {ILayoutGrid}
*/
Cell.prototype.getSection = function () {
	return this._section;
};
/** Use for lazy initialization only. Force an element to be created, if it does not exist.
* @public
* @return {!Element}
*/
Cell.prototype.initialize = function () {
	if(this._aligner == null) {
		if (util.isIE) {
			this._aligner = document.createElement("div");
		} else {
			this._aligner = document.createElement("button");
		}
		this._aligner.className = "valigner";
		this._aligner.role = "gridcell";
		if(this._frontIcon) {
			this._element.appendChild(this._frontIcon.getElement());
		}
		this._element.appendChild(this._aligner);
		if(this._floatingPanel) {
			this._element.appendChild(this._floatingPanel.getElement());
		}
	}
	return /** @type{!Element} */(this._element);
};

/** @public
* @param {Element} icon
*/
Cell.prototype.updateIcon = function(icon) {
	if (!icon) {
		return;
	}

	let fi = this._frontIcon;
	if (fi) {
		if(fi.containItem(icon)) { // Check same icon which duplicate with insertItem in _frontIcon
			return;
		}
		fi.clearItems();
	}
	this.insertFrontIcon(icon, 0);
};
/** @public
*/
Cell.prototype.removeIcon = function() {
	let fi = this._frontIcon;
	if (fi) {
		fi.clearItems();
		this._element.removeChild(fi.getElement());
		fi = this._frontIcon = null;
	}
};

/** @public
* @deprecated
*/
Cell.prototype.isFrozen = util._deprecatedFunction;
/** @public
* @deprecated
* @param {boolean=} opt_bool
*/
Cell.prototype.freezeLayout = util._deprecatedFunction;

/** @public
* @deprecated
* @param {boolean=} opt_bool
*/
Cell.prototype.setAutoContentClipping = util._deprecatedFunction;

/** @public
* @param {string} str
* @return {boolean}
*/
Cell.prototype.setContentTooltip = function (str) {
	if (str != null && this._content) {
		this._content.setAttribute("title", str);
		return true;
	}
	return false;
};

/** @public
* @param {boolean} opt_locked
*/
Cell.prototype.lockContent = function (opt_locked) {
	this._contentLocked = (opt_locked !== false);
};

/** Reset all styles, classes, and content in this cell and restore its default classes
* @public
*/
Cell.prototype.reset = function() {
	if(!this._element) { return; }

	let width = this._element.style.width;
	let height = this._element.style.height;
	this.removeAttribute("style");
	if(width) {
		this._element.style.width = width;
	}
	if(height) {
		this._element.style.height = height;
	}
	this.resetClass();

	// HACK!!
	let blinkState = this["blinkState"];
	if(blinkState) {
		if (blinkState.timerId >= 0) {
			clearInterval(blinkState.timerId);
		}
		delete this["blinkState"];
	}
	delete this["trendState"];
	let scope = this["blinking"];
	if(scope) {
		if(scope["blinking"] >= 0) {
			clearTimeout(scope["blinking"]);
		}
		scope["cell"] = null;
		scope["styles"] = null;
		delete this["blinking"];
	}

	this.removeIcon();

	this.setContent(null);
	if(this._textSpan) {
		this._textSpan.className = "text";
		this._textSpan.textContent = "";
	}
};

/** Reset all classes in this cell and restore its default classes
 * @public
 */
Cell.prototype.resetClass = function() {
	let className = "cell";
	if(this.hasClass("spanned")) {
		className += " spanned";
	}
	if(this.hasClass("selected-row")) {
		className += " selected-row";
	}
	if(this.hasClass("highlighted-row")) {
		className += " highlighted-row";
	}
	this.setClass(className);
};
/** Reset everything to initial state
 * @public
 */
Cell.prototype.resetToInitialState = function() {
	if(!this._element) { return; }

	this.setParent(null);
	this.unlistenAll();

	this._element.className = "cell";
	this.removeIcon();
	this.setContent(null);
	this._contentLocked = false;
	this._collapsedState = 0;
};

/** @public
* @param {Element} elem
* @param {number=} opt_order The greater the order, the farther the position to the right
*/
Cell.prototype.insertFloatingIcon = function (elem, opt_order) {
	if(!elem) {
		return;
	}

	let fp = this._floatingPanel;
	if(!fp) {
		fp = this._floatingPanel = new components_CellFloatingPanel();
	}
	fp.insertItem(elem, opt_order);
	fp.addClass("floating-panel");

	let element = this._element;
	if (!element.contains(fp.getElement())) {
		element.appendChild(fp.getElement());
	}
};
/** @public
* @param {Element|string} elemRef This can be the element or className (string) of the icon
* @return {Element|null} removed element
*/
Cell.prototype.removeFloatingIcon = function (elemRef) {
	let fp = this._floatingPanel;
	if (!fp) {
		return null;
	}

	let removeElem = fp.removeItem(elemRef);
	if (!fp.hasItem()) {
		this._element.removeChild(fp.getElement());
		this._floatingPanel = null;
	}
	return removeElem;
};

/** @public
* @param {Element} elem
* @param {number=} opt_order The greater the order, the farther the position to the right
*/
Cell.prototype.insertFrontIcon = function (elem, opt_order) {
	if(!elem) {
		return;
	}

	let fi = this._frontIcon;
	if(!fi) {
		fi = this._frontIcon = new components_CellFloatingPanel();
	}
	fi.insertItem(elem, opt_order);
	fi.addClass("front-panel");

	let element = this._element;
	if (!element.contains(fi.getElement())) {
		element.insertBefore(fi.getElement(), element.firstChild);
	}
};
/** @public
* @param {Element|string} elemRef This can be the element or className (string) of the icon
* @return {Element|null} removed element
*/
Cell.prototype.removeFrontIcon = function (elemRef) {
	let fi = this._frontIcon;
	if (!fi) {
		return null;
	}

	let removeElem = fi.removeItem(elemRef);
	if (!fi.hasItem()) {
		removeElem = fi.getElement();
		this._element.removeChild(removeElem);
		this._frontIcon = null;
	}
	return removeElem;
};
//#endregion Cell Specific Methods

//#region Internal Public Methods
/** @public
* @ignore
* @return {number}
*/
Cell.prototype.getCellId = function () {
	return this._id;
};
/** Force updating cell padding due to the floating icons in the cell
* @public
* @deprecated
*/
Cell.prototype.updatePadding = function() {
	// Do nothing
};

/** @public
* @ignore
* @param {boolean=} opt_collapsed
* @param {boolean=} opt_remainedInDoc
* @return {boolean}
*/
Cell.prototype.collapse = function (opt_collapsed, opt_remainedInDoc) {
	let collapsed = opt_collapsed !== false;
	(opt_remainedInDoc ? 2 : 1);
	let collapsedState;
	if (collapsed) {
		collapsedState = (opt_remainedInDoc ? 2 : 1);
	} else {
		collapsedState = 0;
	}
	if(this._collapsedState === collapsedState) {
		return false;
	}

	if(collapsed) {
		if(opt_remainedInDoc) {
			this.addClass("spanned");
		} else {
			this.setParent(null);
		}
	} else {
		if(this._collapsedState === 2) {
			this.removeClass("spanned");
		}
	}
	this._collapsedState = collapsedState;
	return true;
};
/** @public
* @ignore
* @return {boolean}
*/
Cell.prototype.isCollapsed = function () {
	return this._collapsedState !== 0;
};
/** @public
* @ignore
* @return {boolean}
*/
Cell.prototype.isKeptInDoc = function () {
	return this._collapsedState !== 1;
};

/** @public
* @ignore
* @return {number}
*/
Cell.prototype.getClientWidth = function () {
	return (this._element) ? this._element.clientWidth : this.getWidth();
};
/** @public
* @ignore
* @return {number}
*/
Cell.prototype.getClientHeight = function () {
	return (this._element) ? this._element.clientHeight : this.getHeight();
};

/** To remove the tooltip, pass str as an empty string. To allow cell calculation for the correct tooltip, set str to null.
* @public
* @param {string|null} str
*/
Cell.prototype.setTooltip = function(str) {
	this.setTooltipInfo("userTooltip", str);
	this.updateTooltip();
};
/** @public
* @ignore
* @param {string} type
* @param {*=} tooltip
*/
Cell.prototype.setTooltipInfo = function(type, tooltip) {
	let tooltipInfo = this._tooltipInfo;
	if(!tooltipInfo) {
		tooltipInfo = this._tooltipInfo = {};
	}

	tooltipInfo[type] = tooltip;
};
/** @public
* @ignore
*/
Cell.prototype.updateTooltip = function() {
	let tooltipInfo = this._tooltipInfo;
	if(!tooltipInfo) {
		return;
	}

	let defaultTooltip = "";
	let customizedTooltip = null;

	// Clipped text tooltip takes precedence over default group header and column tooltip
	if(tooltipInfo["clippedText"]) { // boolean
		defaultTooltip = customizedTooltip = tooltipInfo["clippedTextTooltip"];
	}

	if(tooltipInfo["groupHeaderTooltip"] == null) {
		if(tooltipInfo["columnDefault"] !== false) {
			if(tooltipInfo["columnDefault"] != null) { // true, "", string
				customizedTooltip = tooltipInfo["columnTooltip"];
			} else { // null
				defaultTooltip = tooltipInfo["columnTooltip"];
			}
		}
	} else { // Group header tooltip takes precedence over column tooltip
		if(tooltipInfo["groupHeaderDefault"] !== false) {
			if(tooltipInfo["groupHeaderDefault"] != null) {
				customizedTooltip = tooltipInfo["groupHeaderTooltip"];
			} else {
				defaultTooltip = tooltipInfo["groupHeaderTooltip"];
			}
		}
	}

	// User tooltip take the highest precedence
	if(tooltipInfo["userTooltip"] != null) { // "", string
		customizedTooltip = tooltipInfo["userTooltip"];
	}

	if(customizedTooltip == null) {
		customizedTooltip = defaultTooltip;
	}

	if(customizedTooltip) {
		let curTooltip = this.hasAttribute("tooltip") ? this.getAttribute("tooltip") : this.getAttribute("title");
		if(curTooltip !== customizedTooltip) {
			this.setAttribute("title", customizedTooltip);
		}
	} else {
		if(this.hasAttribute("title")) {
			this.removeAttribute("title");
		}
		if(this.hasAttribute("tooltip")) {
			this.removeAttribute("tooltip");
		}
	}
};
/** @public
* @ignore
* @param {string} type
* @return {*}
*/
Cell.prototype.getTooltipInfo = function(type) {
	if(this._tooltipInfo) {
		return this._tooltipInfo[type];
	}
	return null;
};
//#region Internal Public Methods

//#region Private Methods
/** @private
* @param {*} value
* @return {Element}
*/
Cell.prototype._setRawContent = function (value) {
	if (this._rawContent !== value) {
		this._rawContent = value;

		let actualContent = /** @type{Element|NodeList} */(this._rawContent);
		if (typeof this._rawContent == "string") {
			if (!this._textSpan) {
				this._textSpan = document.createElement("span");
				this._textSpan.className = "text";
			}
			this._textSpan.textContent = this._rawContent;
			actualContent = this._textSpan;
		}

		if (this._content !== actualContent) { // Compare with previous element
			this._content = actualContent;

			this.initialize();

			let aligner = this._aligner;
			while (aligner.lastChild ) {
				aligner.removeChild(aligner.lastChild);
			}
			if (this._content) {
				aligner.appendChild(/** @type{Element} */(this._content));
			}
		}

	}
	return /** @type{Element} */(this._content);
};

/** @private
*/
Cell.prototype._updateDisplay = function () {
	let visibility = false;
	if(this.isVisible()) {
		if(this._width && this._height) {
			visibility = true;
		}
	}
	if (visibility) {
		this._element.style.display = "";
	} else {
		this._element.style.display = "none";
	}
};
//#endregion Private Methods


Cell._proto = Cell.prototype;

/* harmony default export */ const components_Cell = (Cell);


;// ./node_modules/@grid/core/es6/grid/util/TrackLayout.js


/** TrackLayout is like a running track in a football field which contains multiple lanes.
* TrackLayout defines position and size for all its lanes, as well as support binarySearch for hit testing
* @constructor
* @ignore
*/
let TrackLayout = function () {
	this._ends = [];
	this._cols = [];
};

/** @type {number}
* @private
*/
TrackLayout.prototype._laneCount = 0;
/** @type {number}
* @private
*/
TrackLayout.prototype._defaultSize = 10;
/** @type {boolean}
* @private
*/
TrackLayout.prototype._defaultScalability = true;
/** @type {boolean}
* @private
*/
TrackLayout.prototype._homogeneous = true;
/** @type {boolean}
* @private
*/
TrackLayout.prototype._visibility = true;
/** Cumulative end position of each lane
* @type {!Array.<number>}
* @private
*/
TrackLayout.prototype._ends;
/** @type {!Array.<Object>}
* @private
*/
TrackLayout.prototype._cols;

/** @type {!Array.<number>}
* @private
*/
TrackLayout.prototype._bits = [1, 2, 4, 8]; // Static variable



/** If track size is 50 and start from 0, [0, 49] counts as a hit, while 50 does not
* @public
* @param {number} val
* @return {number} Positive index if hit, otherwise negative
*/
TrackLayout.prototype.hitTest = function (val) {
	if (val < 0) { return -1; }

	// getTrackSize has updated the cache for usage. Do not call this._updateEndPoints()
	let totalSize = this.getTrackSize();
	if (val >= totalSize) { return ~this._laneCount; } // totalSize does not count as a hit

	let hitIndex = -1;
	let validRange = 0;
	if (this._homogeneous) { // Each row has the same size
		let laneSize = this._defaultSize;
		hitIndex = Math.floor(val / laneSize);
		validRange = ((hitIndex + 1) * laneSize);
	} else {
		hitIndex = util.upperBound(this._ends, val);
		validRange = this._ends[hitIndex];
	}

	if (hitIndex >= this._laneCount || val >= validRange) {
		hitIndex = ~hitIndex; // invert to negative value
	}
	return hitIndex;
};

/**
* @public
*/
TrackLayout.prototype.dispose = function () {
	// TODO: Column dragging extension refers to this variable, so the reference instance should be deleted instead of set a new pointer
	this._cols = this._ends = null;
};

/** If optional index is not given, the entire track is taken into account
* @public
* @param {number=} opt_index
* @return {boolean}
*/
TrackLayout.prototype.isAtDefaultSize = function (opt_index) {
	if(this._homogeneous) {
		return true; // All lanes are at the default size
	}
	if (opt_index != null) {
		if(!this._isAtDefaultSize(opt_index)) {
			return false;
		}
	} else {
		for (let i = this._cols.length; --i >= 0; ) {
			if (!this._isAtDefaultSize(i)) {
				return false;
			}
		}
		this._homogeneous = true; // All lane are at the default size
	}
	return true;
};
/** @private
* @param {number} index
* @return {boolean}
*/
TrackLayout.prototype._isAtDefaultSize = function (index) {
	let col = this._cols[index];
	if (col) {
		if (col.actualSize !== this._defaultSize) {
			return false;
		}
		if (col.customSize != null) {
			return false;
		}
		if(col.invisibility) {
			return false;
		}
	}
	return true;
};
/** @public
* @return {boolean}
*/
TrackLayout.prototype.isHomogeneous = function () {
	return this._homogeneous;
};

/** If optional index is not given, the entire track is taken into account
* @public
* @param {number=} opt_index
* @return {boolean}
*/
TrackLayout.prototype.isAtMinimumSize = function (opt_index) {
	let minSize = this._getMinimumScalingSize(opt_index);
	let trackSize = (opt_index != null) ? this.getLaneSize(opt_index) : this.getTrackSize();
	return trackSize <= minSize;
};

/** Track is fixed size if all lane are not scalable
* @public
* @return {boolean}
*/
TrackLayout.prototype.isFixedSize = function () {
	for (let i = 0; i < this._laneCount; ++i) {
		if (this.getLaneScalability(i)) {
			return false;
		}
	}
	return true;
};

/** Clear all customizations of all lanes including visibility, custom size, minimum size, and scalability
* @public
*/
TrackLayout.prototype.clearLaneCustomizations = function () {
	this._ends.length = 0;
	this._cols.length = 0;
	this._homogeneous = true;
};

/** @public
* @return {number}
*/
TrackLayout.prototype.getLaneCount = function () {
	return this._laneCount;
};
/** @public
* @param {number} val
*/
TrackLayout.prototype.setLaneCount = function (val) {
	if (val < 0) { val = 0; }
	if (val === this._laneCount) { return; }

	this._markDirtyIndex(val);
	this._cols.length = val;
	this._laneCount = val;
	if (val === 0) { this._homogeneous = true; }
};
/**
* @public
* @return {number}
*/
TrackLayout.prototype.getDefaultLaneSize = function () {
	return this._defaultSize;
};
/** @public
* @param {number} val
* @return {boolean}
*/
TrackLayout.prototype.setDefaultLaneSize = function (val) {
	if (val < 0) { val = 0; }
	if (this._defaultSize !== val) {
		this._defaultSize = val;
		this._markDirtyIndex(0);
		return true;
	}
	return false;
};
/** @public
* @param {number} index
* @return {number}
*/
TrackLayout.prototype.getLaneDefaultSize = function (index) {
	if (index < 0 || index >= this._laneCount) { return -1; }
	let col = this._newColumn(index);
	if(col.defaultSize) {
		return col.defaultSize;
	} else {
		return -1;
	}
};
/** @public
* @param {number} index
* @param {number} val
*/
TrackLayout.prototype.setLaneDefaultSize = function (index, val) {
	if (index < 0 || index >= this._laneCount) { return; }
	if (val >= 0) {
		let col = this._newColumn(index);
		col.defaultSize = val;
	}
};
/** @public
* @return {boolean}
*/
TrackLayout.prototype.resetLaneToDefaultSize = function () {
	let dirty = false;
	for (let i = 0; i < this._laneCount; ++i) {
		let col = this._cols[i];
		if (col) {
			if (!col.invisibility) {
				if (col.defaultSize != null) {
					dirty |= this.setLaneScalability(i, false);
					dirty |= this.setLaneSize(i, col.defaultSize);
				} else {
					dirty |= this.setLaneScalability(i, true);
					dirty |= this.setLaneSize(i, 1);
				}
			}
		}
	}
	return dirty ? true : false;
};
/**
* @public
* @return {boolean}
*/
TrackLayout.prototype.getDefaultScalability = function () {
	return this._defaultScalability;
};
/** @public
* @param {boolean} val
* @return {boolean}
*/
TrackLayout.prototype.setDefaultScalability = function (val) {
	if (this._defaultScalability !== val) {
		this._defaultScalability = val;
		// this._markDirtyIndex(0); WARNING: Scalability does not have any impact on lane size
		return true;
	}
	return false;
};


/** @public
* @return {number}
*/
TrackLayout.prototype.getTrackSize = function () {
	if (!this._visibility || this._laneCount <= 0) { return 0; }

	if (this._homogeneous) {
		return this._laneCount * this._defaultSize;
	}

	this._updateEndPoints();
	return this._ends[this._laneCount - 1]; // Return the end of the last item
};
/** Scale all lanes based on the given available space. Only the specified flexible lanes are adjusted.
* If all lanes are eqaul in size (even zero size), they all will be weighted equally
* @public
* @param {number} targetSize
* @return {Array.<number>|null} Array of size adjustment. Return null if there is no change
*/
TrackLayout.prototype.setTrackSize = function (targetSize) {
	let i = 0, fixedSize = 0;
	let indices = [];
	// Collecting current data for calculation
	for (i = 0; i < this._laneCount; ++i) {
		let col = this._cols[i];
		if (col) {
			if (!col.invisibility) { // If column is visible
				if (this._getLaneScalability(col) !== false) {
					indices.push(i);
				} else {
					fixedSize += this._getFixedSize(col);
				}
			}
		} else {
			indices.push(i);
		}
	}
	if (indices.length <= 0) { return null; }

	let distributions = this._distributeLaneSize(targetSize - fixedSize, indices);

	// Find any change to flexible lane size
	let dirty = false;
	let changes = [];
	for (i = distributions.length; --i >= 0; ) {
		changes[i] = 0;
		let val = distributions[i];
		if (val != null) {
			let cal = this.getLaneSize(i);
			this._newColumn(i);
			if (this._setCalculatedLaneSize(i, val)) {
				changes[i] = val - cal;
				dirty = true;
			}
		}
	}
	return (dirty) ? changes : null;
};

/** This method is forgiving and often used for size predictions. Thus, out of bounds index is accepted
* Negative index will be treated as 0. Any exceeding lane will be treated to have a default size
* @public
* @param {number} index
* @return {number}
*/
TrackLayout.prototype.getLaneStart = function (index) {
	if (index <= 0) return 0;

	if (this._homogeneous) {
		return index * this._defaultSize;
	}

	this._updateEndPoints();
	let exceedings = index - this._laneCount;
	let exceededPortion = 0;
	if (exceedings > 0) {
		index -= exceedings;
		exceededPortion = exceedings * this._defaultSize;
	}
	// Return the end point of the previous item
	return this._ends[index - 1] + exceededPortion;
};
/** Any invalid index will be automatically adjusted to be in a valid range.
* @public
* @param {number} index
* @return {number} if there is no lane, NaN value is returned
*/
TrackLayout.prototype.getLaneEnd = function (index) {
	if (this._laneCount <= 0) { return NaN; }
	if (index < 0) { index = 0; }
	else if (index >= this._laneCount) { index = this._laneCount - 1; }

	if (this._homogeneous) {
		return (index + 1) * this._defaultSize;
	}

	this._updateEndPoints();

	return this._ends[index]; // Return the end point of the previous item
};

/** @public
* @param {number} index
* @return {number}
*/
TrackLayout.prototype.getLaneSize = function (index) {
	if (this._visibility) {
		let col = this._cols[index];
		return (col) ? col.actualSize : this._defaultSize;
	}
	return 0;
};
/** @public
* @param {number} index
* @return {number}
*/
TrackLayout.prototype.getCustomLaneSize = function (index) {
	let col = this._cols[index];
	return (col && col.customSize != null) ? col.customSize : this._defaultSize;
};
/** @public
* @param {number} index
* @param {number} val
* @return {boolean}
*/
TrackLayout.prototype.setLaneSize = function (index, val) {
	if (index < 0 || index >= this._laneCount) { return false; }
	if (val < 0 || val !== val) { return false; }

	let col = this._cols[index];
	if (col == null || col.customSize !== val) {
		col = this._newColumn(index);
		col.customSize = val;
		return this._setCalculatedLaneSize(index, val);
	}

	return false;
};

/** @public
* @param {number} index
* @param {string} propName
* @param {*} val
* @return {boolean}
*/
TrackLayout.prototype.setLaneProperty = function (index, propName, val) {
	if (index < 0 || index >= this._laneCount) { return false; }

	let col = this._cols[index];
	if (col == null || col[propName] !== val) {
		col = this._newColumn(index);
		col[propName] = val;
		return true;
	}

	return false;
};
/** @public
* @param {number} index
* @param {string} propName
* @return {*}
*/
TrackLayout.prototype.getLaneProperty = function (index, propName) {
	let col = this._cols[index];
	return (col && col[propName] != null) ? col[propName] : null;
};
/** @public
* @param {number} index
* @return {number}
*/
TrackLayout.prototype.getMinimumLaneSize = function (index) {
	let col = this._cols[index];
	return (col && col.minSize != null) ? col.minSize : 0;
};
/** Minimum lane size governs custom lane size in calculation
* @public
* @param {number} index
* @param {number} val
* @param {boolean=} proportion This will make the lane expand in proportion of its min width when there is enough space.
* @return {boolean} Return true if there is a change in the custom lane size, false otherwise
*/
TrackLayout.prototype.setMinimumLaneSize = function (index, val, proportion) {
	if (index < 0 || index >= this._laneCount) { return false; }
	if (val < 0) { val = 0; }
	if (this.getMinimumLaneSize(index) !== val) {
		let col = this._newColumn(index);
		col.minSize = val;

		if(proportion) {
			if (this.getLaneScalability(index)) {
				col.customSize = val;
			}
		}
		return this._setCalculatedLaneSize(index);
	}
	return false;
};
/** @public
* @param {number} index
* @return {boolean}
*/
TrackLayout.prototype.isLaneVisible = function (index) {
	let col = this._cols[index];
	return col ? !col.invisibility : true;
};
/** @public
* @param {number} index
* @param {boolean=} opt_val
* @return {boolean}
*/
TrackLayout.prototype.showLane = function (index, opt_val) {
	return this.hideLane(index, opt_val === false);
};
/** @public
* @param {number} index
* @param {boolean=} hidden
* @param {number=} bitIndex
* @return {boolean}
*/
TrackLayout.prototype.hideLane = function (index, hidden, bitIndex) {
	if (index < 0 || index >= this._laneCount) { return false; }

	let col = this._newColumn(index);
	let prevVis = !col.invisibility;

	let bit = (bitIndex != null) ? this._bits[bitIndex] : 1;
	if(hidden !== false) {
		col.invisibility |= bit; // Add invisibility bit
	} else {
		col.invisibility &= ~bit; // Remove invisibility bit
	}

	if (prevVis !== (!col.invisibility)) {
		return this._setCalculatedLaneSize(index);
	}
	return false;
};
/** Return true if the lane does not exist nor is not hidden. Return false only if the lane is hidden by the specified bit
* @public
* @param {number} index
* @param {number=} bitIndex
* @return {boolean}
*/
TrackLayout.prototype.getLaneVisibilityBit = function (index, bitIndex) {
	if (index >= 0 && index < this._laneCount) {
		let col = this._cols[index];
		if(col) {
			let bit = (bitIndex != null) ? this._bits[bitIndex] : 1;
			if(col.invisibility & bit) {
				return false;
			}
		}
	}
	return true;
};
/** @public
* @param {boolean=} opt_shown
*/
TrackLayout.prototype.show = function (opt_shown) {
	opt_shown = opt_shown !== false;
	if (this._visibility !== opt_shown) {
		this._visibility = opt_shown;
		if (opt_shown) {
			for (let i = 0; i < this._laneCount; ++i) {
				this._newColumn(i);
				this._setCalculatedLaneSize(i);
			}
		} else {
			this._markDirtyIndex(0);
		}
	}
};

/** @public
* @param {number} index
* @return {boolean}
*/
TrackLayout.prototype.getLaneScalability = function (index) {
	let col = this._cols[index];
	return (col && col.scalability != null) ? col.scalability : this._defaultScalability;
};
/** @public
* @param {number} index
* @param {boolean} val
* @return {boolean}
*/
TrackLayout.prototype.setLaneScalability = function (index, val) {
	if (index < 0 || index >= this._laneCount) { return false; }

	let col = this._cols[index];
	if (!col || col.scalability !== val) {
		col = this._newColumn(index);
		col.scalability = val;
		return true;
	}
	return false;
};

/** @public
* @param {number} index
* @param {number=} opt_count Default is 1
* @return {Object|null}
*/
TrackLayout.prototype.removeLaneAt = function (index, opt_count) {
	if(opt_count == null) { opt_count = 1; }
	if(index < 0) { return null; }
	if(index + opt_count > this._laneCount) {
		opt_count = this._laneCount - index;
	}
	if(opt_count < 1) { // Nothing to be removed
		return null;
	}

	this._markDirtyIndex(index);
	let userSettings = this.serializeLane(index); // TODO: This is unnecessary
	this._cols.splice(index, opt_count);

	this._laneCount -= opt_count;
	return userSettings;
};
/** @public
* @param {number} index
* @param {number|Object=} opt_json Numeric value means number of lane to be inserted, while object value means the settings of the lane
*/
TrackLayout.prototype.insertLane = function (index, opt_json) {
	if (index < 0) { index = 0; }
	let chunkSize = 1;
	if (util.isNumber(opt_json)) {
		if (/** @type{number} */(opt_json) <= 0) { return; }
		chunkSize = /** @type{number} */(opt_json);
		opt_json = null;
	}

	if (index >= this._laneCount) {
		index = this._laneCount;
		if (!opt_json) { // No special setting
			this.setLaneCount(this._laneCount + chunkSize);
			return;
		}
	}

	this._laneCount += chunkSize;
	this._markDirtyIndex(index); // Only mark at the lowest index

	util.insertEmptyItems(this._cols, index, chunkSize);
	if (opt_json) {
		let toIndex = index + chunkSize;
		for (let i = index; i < toIndex; ++i) {
			this.deserializeLane(i, /** @type{Object} */(opt_json));
		}
	}
};
/** @public
* @param {number} from
* @param {number} to
*/
TrackLayout.prototype.moveLane = function (from, to) {
	if (from < 0) { from = 0; }
	else if (from > this._laneCount) { from = this._laneCount; }
	if (to < 0) { to = 0; }
	else if (to > this._laneCount) { to = this._laneCount; }
	if (from === to) { return; }

	let minIndex = (from < to) ? from : to;
	this._markDirtyIndex(minIndex);

	util.moveArrayItem(this._cols, from, to);
};

/** @public
* @param {number} index
* @return {Object|null}
*/
TrackLayout.prototype.serializeLane = function (index) {
	if (index < 0 || index >= this._laneCount) { return null; }

	let userSettings = {};
	userSettings["index"] = index;

	let col = this._cols[index];
	if (!col) { return userSettings; }

	userSettings["size"] = col.customSize;
	userSettings["minSize"] = col.minSize;
	userSettings["visible"] = !col.invisibility;
	userSettings["scalable"] = col.scalability;

	return userSettings;
};
/** @public
* @param {number} index
* @param {Object=} opt_json
*/
TrackLayout.prototype.deserializeLane = function (index, opt_json) {
	if (!opt_json) { return; }
	if (index < 0 || index >= this._laneCount) { return; }

	let val = opt_json["size"];
	if (val != null) {
		this.setLaneSize(index, /** @type{number} */(val));
	}
	val = opt_json["minSize"];
	if (val != null) {
		this.setMinimumLaneSize(index, /** @type{number} */(val));
	}
	val = opt_json["visible"];
	if (val != null) {
		this.showLane(index, /** @type{boolean} */(val));
	}
	val = opt_json["scalable"];
	if (val != null) {
		this.setLaneScalability(index, /** @type{boolean} */(val));
	}
};

/** @private
*/
TrackLayout.prototype._updateEndPoints = function () {
	let dirtyIndex = this._ends.length;
	if (dirtyIndex >= this._laneCount) return;

	this._ends.length = this._laneCount;

	let i = 0;
	let sum = (dirtyIndex > 0) ? this._ends[dirtyIndex - 1] : 0;

	for (i = dirtyIndex; i < this._laneCount; ++i) {
		sum += this.getLaneSize(i);
		this._ends[i] = sum;
	}
};

/** Calculated size is the result from processing custom size, min size, and visibility
* @private
* @param {number} index
* @param {number=} opt_val
* @return {boolean}
*/
TrackLayout.prototype._setCalculatedLaneSize = function (index, opt_val) {
	let col = this._cols[index];
	if (this._visibility && !col.invisibility) { // If column is visible
		if (opt_val == null) {
			opt_val = (col.customSize != null) ? col.customSize : this._defaultSize;
		}
		if(!col.scalability) { // minSize doesn't applied when lane scalability
			let minSize = (col.minSize != null) ? col.minSize : 0;
			if (opt_val < minSize) {
				opt_val = minSize;
			}
		}
	} else {
		opt_val = 0;
	}

	if (col.actualSize !== opt_val) { // actualSize can be undefined value
		col.actualSize = opt_val;
		this._homogeneous = false;
		this._markDirtyIndex(index);
		return true;
	}

	return false;
};

/** @private
* @param {number=} opt_index
* @return {number}
*/
TrackLayout.prototype._getMinimumScalingSize = function (opt_index) {
	let sum = 0;
	if (opt_index != null) {
		let col = this._cols[opt_index];
		if (!col.invisibility) {
			if (col.minSize != null) {
				if (this._getLaneScalability(col) === false && col.customSize != null) {
					if (col.customSize > col.minSize) {
						return col.customSize;
					}
				}
				return col.minSize;
			}
		}
	} else {
		for (let i = this._laneCount; --i >= 0; ) {
			sum += this._getMinimumScalingSize(i);
		}
	}
	return sum;
};
/** @private
* @param {Object} col
* @return {number}
*/
TrackLayout.prototype._getFixedSize = function (col) {
	if(col.customSize != null) {
		if(col.minSize != null) {
			return (col.customSize > col.minSize) ? col.customSize : col.minSize;
		}
		return col.customSize;
	} else if(col.minSize != null) {
		return col.minSize;
	}
	return this._defaultSize;
};
/** @private
* @param {Object} col
* @return {boolean}
*/
TrackLayout.prototype._getLaneScalability = function (col) {
	if (col.scalability == null) { return this._defaultScalability; }
	return /** @type{boolean} */(col.scalability);
};

/** Distribute available space to every lane if have enough space. If not enough, priority goes to lane with minimum size.
* @private
* @param {number} availSpace
* @param {Array.<number>} indices
* @return {!Array.<number>} distributions
*/
TrackLayout.prototype._distributeLaneSize = function (availSpace, indices) {
	if (indices.length <= 0) { return []; }
	if (availSpace < 0) { availSpace = 0; }

	let i = 0, totalWeight = 0, index = 0, expectedSize = 0;
	let minSizes = [], laneSizes = [], distributions = [];

	for (i = indices.length; --i >= 0; ) {
		index = indices[i];
		minSizes[index] =  this.getMinimumLaneSize(index); // Cache for multiple usage
		laneSizes[index] = this.getCustomLaneSize(index); // Cache for multiple usage
		totalWeight += laneSizes[index];
	}

	let undistributable = true;
	while(undistributable) { // All lane sizes must at least equal to its minimum width
		undistributable = false;
		for (i = indices.length; --i >= 0; ) {
			index = indices[i];
			let minSize = minSizes[index];
			let laneSize = laneSizes[index];
			expectedSize = availSpace * laneSize / totalWeight;
			if (expectedSize <= minSize) {
				undistributable = true;
				distributions[index] = minSize;
				availSpace -= minSize;
				totalWeight -= laneSize;
				indices.splice(i, 1);
			}
		}
	}

	let lastIndex = indices.length - 1;
	let occupiedSpace = 0;
	let fraction = 0;
	for (i = lastIndex; --i >= 0; ) {
		index = indices[i];
		expectedSize = availSpace * laneSizes[index] / totalWeight;
		let integerSize = Math.floor(expectedSize);
		fraction += (expectedSize - integerSize);
		if(fraction >= 1) {
			++integerSize;
			--fraction;
		}
		distributions[index] = integerSize;
		occupiedSpace += integerSize;
	}
	distributions[indices[lastIndex]] = Math.floor(availSpace - occupiedSpace);
	return distributions;
};

/** @private
* @param {number} index
*/
TrackLayout.prototype._markDirtyIndex = function (index) {
	if (index < this._ends.length) {
		this._ends.length = index;
	}
};
/** @private
* @param {number} index
* @return {!Object}
*/
TrackLayout.prototype._newColumn = function (index) {
	let col = this._cols[index];
	if (!col) {
		this._cols[index] = col = {};
		// Each bit represents visibility state as the followings:
		// Bit 0 is used for normal show/hide method
		// Bit 1 is used for expanded/collapsed state of columm grouping
		// Bit 2 is used for stacking state of column stacking
		col.invisibility = 0; // Column is visible by default
	}
	return col;
};

/** @public
* @param {TrackLayout} src
* @param {number=} offset
*/
TrackLayout.prototype.copyFrom = function (src, offset) {
	if(this._homogeneous && src._homogeneous) {
		if(this._defaultSize === src._defaultSize) {
			return;
		}
	}
	if(!offset) { offset = 0; }

	let srcCount = src.getLaneCount();
	let curCount = this.getLaneCount();
	let limit = offset + srcCount;
	if(limit > curCount) {
		limit = curCount;
	}
	let srcIndex = 0;
	for(let i = offset; i < limit; ++i) {
		let val = src.getLaneSize(srcIndex++);
		let col = this._newColumn(i);
		if (col.customSize !== val) {
			col.customSize = val;
			if(col.actualSize !== val) { // WARNING: This does not obey visibility and minSize
				col.actualSize = val;
				this._homogeneous = false; // TODO: Improve this flag
				this._markDirtyIndex(i);
			}
		}
	}
};

/* harmony default export */ const util_TrackLayout = (TrackLayout);


;// ./node_modules/@grid/core/es6/grid/ILayoutGrid.js
/* eslint-disable */




/**
* @interface
* @extends {ElementWrapper}
*/
let ILayoutGrid = function () {};

/** @public
* @param {number} x
* @param {number} y
* @param {boolean=} opt_omitSpan if true cell spans will be omitted from calculation
* @return {Object|null} Return object with "colIndex" and "rowIndex" property
*/
ILayoutGrid.prototype.hitTest = function (x, y, opt_omitSpan) { };
/** @public
* @param {number} x
* @return {number} Positive index if hit, otherwise negative
*/
ILayoutGrid.prototype.hitTestX = function (x) { };
/** @public
* @param {number} y
* @return {number} Positive index if hit, otherwise negative
*/
ILayoutGrid.prototype.hitTestY = function (y) { };

/** @public
* @return {number}
*/
ILayoutGrid.prototype.getDefaultColumnWidth = function () { };
/** @public
* @param {number} val
*/
ILayoutGrid.prototype.setDefaultColumnWidth = function (val) { };
/** @public
* @return {number}
*/
ILayoutGrid.prototype.getDefaultRowHeight = function () { };
/** @public
* @param {number} val
*/
ILayoutGrid.prototype.setDefaultRowHeight = function (val) { };
/** @public
* @param {number} colIndex
* @return {number}
*/
ILayoutGrid.prototype.getColumnWidth = function (colIndex) { };
/** @public
* @param {number} colIndex
* @param {number} val
* @return {boolean}
*/
ILayoutGrid.prototype.setColumnWidth = function (colIndex, val) { };
/** @public
* @param {number} rowIndex
* @return {number}
*/
ILayoutGrid.prototype.getRowHeight = function (rowIndex) { };
/** @public
* @param {number} rowIndex
* @param {number} val
* @return {boolean}
*/
ILayoutGrid.prototype.setRowHeight = function (rowIndex, val) { };
/** @public
* @param {number} colIndex
* @return {boolean}
*/
ILayoutGrid.prototype.isColumnVisible = function (colIndex) { };
/** @public
* @param {number} colIndex
* @param {boolean=} val
* @return {boolean}
*/
ILayoutGrid.prototype.hideColumn = function (colIndex, val) { };
/** @public
* @param {number} rowIndex
* @return {boolean}
*/
ILayoutGrid.prototype.isRowVisible = function (rowIndex) { };
/** @public
* @param {number} rowIndex
* @param {boolean=} val
* @return {boolean}
*/
ILayoutGrid.prototype.hideRow = function (rowIndex, val) { };
/** @public
* @param {number} colIndex
* @return {number}
*/
ILayoutGrid.prototype.getMinimumColumnWidth = function (colIndex) { };
/** @public
* @param {number} colIndex
* @param {number} val
* @return {boolean}
*/
ILayoutGrid.prototype.setMinimumColumnWidth = function (colIndex, val) { };
/** @public
* @param {number} rowIndex
* @return {number}
*/
ILayoutGrid.prototype.getMinimumRowHeight = function (rowIndex) { };
/** @public
* @param {number} rowIndex
* @param {number} val
* @return {boolean}
*/
ILayoutGrid.prototype.setMinimumRowHeight = function (rowIndex, val) { };

/** @public
* @param {number} colIndex
* @return {number}
*/
ILayoutGrid.prototype.getCellLeft = function (colIndex) { };
/** @public
* @param {number} rowIndex
* @return {number}
*/
ILayoutGrid.prototype.getCellTop = function (rowIndex) { };
/** @public
* @param {number} colIndex
* @param {number} rowIndex
* @return {number}
*/
ILayoutGrid.prototype.getCellWidth = function (colIndex, rowIndex) { };
/** @public
* @param {number} colIndex
* @param {number} rowIndex
* @return {number}
*/
ILayoutGrid.prototype.getCellHeight = function (colIndex, rowIndex) { };

/** @public
* @return {boolean}
*/
ILayoutGrid.prototype.hasCellSpan = function () { };
/** @description Returns zero or negative value, if the cell is spanned over (i.e. occupied by the spanning cell). <br>
* Negative value is distance from the spanning cell (i.e. -1 means one cell from the spanning cell and -2 means two cells from the spanning cell) <br>
* Return one, if the cell has no span (normal cell). <br>
* Return value greater than one, if the cell is spanning (getting expanded)
* @public
* @param {number} colIndex
* @param {number} rowIndex
* @return {number}
*/
ILayoutGrid.prototype.getCellRowSpan = function (colIndex, rowIndex) { };
/** @public
* @param {number} colIndex
* @param {number} rowIndex
* @param {number} rowSpan
* @return {boolean}
*/
ILayoutGrid.prototype.setCellRowSpan = function (colIndex, rowIndex, rowSpan) { };
/** @description Returns zero or negative value, if the cell is spanned over (i.e. occupied by the spanning cell). <br>
* Negative value is distance from the spanning cell (i.e. -1 means one cell from the spanning cell and -2 means two cells from the spanning cell) <br>
* Return one, if the cell has no span (normal cell). <br>
* Return value greater than one, if the cell is spanning (getting expanded)
* @public
* @param {number} colIndex
* @param {number} rowIndex
* @return {number}
*/
ILayoutGrid.prototype.getCellColSpan = function (colIndex, rowIndex) { };
/** @public
* @param {number} colIndex
* @param {number} rowIndex
* @param {number} colSpan
* @return {boolean}
*/
ILayoutGrid.prototype.setCellColSpan = function (colIndex, rowIndex, colSpan) { };
/** @public
* @return {boolean}
*/
ILayoutGrid.prototype.clearCellSpans = function () { };
/** @public
* @param {number} colIndex
* @return {boolean}
*/
ILayoutGrid.prototype.clearColumnSpans = function (colIndex) { };

/** @public
*/
ILayoutGrid.prototype.clearColumnCustomizations = function () { };
/** @public
*/
ILayoutGrid.prototype.clearRowCustomizations = function () { };

/** @public
* @param {number} colIndex
* @return {boolean}
*/
ILayoutGrid.prototype.getColumnScalability = function (colIndex) { };
/** @public
* @param {number} colIndex
* @param {boolean} val
* @return {boolean}
*/
ILayoutGrid.prototype.setColumnScalability = function (colIndex, val) { };
/** @public
* @param {number} rowIndex
* @return {boolean}
*/
ILayoutGrid.prototype.getRowScalability = function (rowIndex) { };
/** @public
* @param {number} rowIndex
* @param {boolean} val
* @return {boolean}
*/
ILayoutGrid.prototype.setRowScalability = function (rowIndex, val) { };

/** WARNING: Any modification to individual cell outside this class will not reflect the change as a whole. Please use with great care.
* @public
* @param {number} colIndex
* @param {number} rowIndex
* @param {boolean=} opt_includeSpan Default is false (cells are retrieved by their index)
* @return {ElementWrapper}
*/
ILayoutGrid.prototype.getCell = function (colIndex, rowIndex, opt_includeSpan) { };
/** @public
* @param {number} colIndex
* @param {number} rowIndex
* @return {Element|Node|NodeList}
*/
ILayoutGrid.prototype.getCellContent = function (colIndex, rowIndex) { };
/** Set data to the specified cell
* @public
* @param {number} colIndex
* @param {number} rowIndex
* @param {string|Element|ElementWrapper|*} data this can be HTML element, string, number, date, image
* @return {Element}
*/
ILayoutGrid.prototype.setCellContent = function (colIndex, rowIndex, data) { };

/** @public
* @param {number} index
* @return {ElementWrapper}
*/
ILayoutGrid.prototype.getColumn = function (index) { };
/** @public
* @return {number}
*/
ILayoutGrid.prototype.getColumnCount = function () { };
/** @public
* @param {number} val
*/
ILayoutGrid.prototype.setColumnCount = function (val) { };
/** @public
* @return {number}
*/
ILayoutGrid.prototype.getRowCount = function () { };
/** @public
* @param {number} val
* @param {boolean=} noBinding
*/
ILayoutGrid.prototype.setRowCount = function (val, noBinding) { };
/** @public
* @return {number}
*/
ILayoutGrid.prototype.getVisibleRowCount = function () { };
/** @public
* @param {number} val Number of row to be added from the last row
*/
ILayoutGrid.prototype.addRow = function (val) { };
/** @public
* @param {number} val Number of row to be removed from the last row
*/
ILayoutGrid.prototype.removeRow = function (val) { };

/** Deprecated. Shorthand method to set border visibility.
* @public
* @param {boolean} lft
* @param {boolean} top
* @param {boolean} rgt
* @param {boolean} btm
*/
ILayoutGrid.prototype.setBorders = function (lft, top, rgt, btm) { };

/** @public
* @param {number} colIndex
* @param {string} styleName
* @return {string|null} value
*/
ILayoutGrid.prototype.getColumnStyle = function (colIndex, styleName) { };

/** @public
* @param {number} colIndex
* @param {string} styleName
* @param {string} value
*/
ILayoutGrid.prototype.setColumnStyle = function (colIndex, styleName, value) { };

/** @public
* @param {number} colIndex
* @param {string} className
* @param {boolean=} enabled
*/
ILayoutGrid.prototype.enableColumnClass = function (colIndex, className, enabled) { };
/** @public
 * @return {boolean}
 */
ILayoutGrid.prototype.hasColumnClass = function (colIndex, className) { };
/** Apply CSS class to each cell in the row
* @public
* @param {number} rowIndex
* @param {string} className
* @param {boolean} enabled
* @see {@link ILayoutGrid.#setRowStyle}
*/
ILayoutGrid.prototype.enableRowClass = function (rowIndex, className, enabled) { };
/** Set CSS style to each cell in the row
* @public
* @param {number} rowIndex
* @param {string} styleName
* @param {string|null=} value
* @param {boolean=} enabled If set to false, the specified style will be removed instead
* @see {@link ILayoutGrid.#enableRowClass}
*/
ILayoutGrid.prototype.setRowStyle = function (rowIndex, styleName, value, enabled) { };

/** @public
* @param {!ElementWrapper|Element|Event|MouseEvent} obj ElementWrapper, element or mouse event whose position we're calculating.
* @return {!Object}
*/
ILayoutGrid.prototype.getRelativePosition = function (obj) { };

/** @public
* @return {number}
*/
ILayoutGrid.prototype.getContentWidth = function () { };
/** @public
* @return {number}
*/
ILayoutGrid.prototype.getContentHeight = function () { };
/** @public
* @param {number} rowIndex
* @return {Array.<ElementWrapper>} Return null for non existing row
* @see {@link ILayoutGrid.#setRowStyle}
* @see {@link ILayoutGrid.#enableRowClass}
*/
ILayoutGrid.prototype.getCellsByRowIndex = function(rowIndex) { };

/** @public
* @param {boolean=} opt_bool
*/
ILayoutGrid.prototype.disableHorizontalGridLine = function (opt_bool) { };
/** @public
* @param {boolean=} opt_bool
*/
ILayoutGrid.prototype.disableVerticalGridLine = function (opt_bool) { };

/** @public
* @param {boolean} enabled
*/
ILayoutGrid.prototype.enableShading = function (enabled) { };
/** @public
* @param {boolean} enabled
*/
ILayoutGrid.prototype.setTransposedShading = function (enabled) { };

/** When mouse hovering over a row, it will be highlighted
* @public
* @param {boolean=} opt_bool
*/
ILayoutGrid.prototype.enableRowHighlighting = function (opt_bool) { };
/** @public
* @return {number} rowIndex Negative number means no row is highlighted
*/
ILayoutGrid.prototype.getRowHighlight = function () { };
/** @public
* @param {number} rowIndex Negative number will cause deselection
*/
ILayoutGrid.prototype.setRowHighlight = function (rowIndex) { };
/** @public
* @return {Array.<number>}
*/
ILayoutGrid.prototype.getSelectedRows = function () { };
/** @public
* @return {number}
*/
ILayoutGrid.prototype.getSelectedRowCount = function () { };
/** @public
 * @param {number} at
 * @return {boolean}
 */
ILayoutGrid.prototype.isSelectedRow = function (at) { };
/** @public
 * @return {number}
 */
ILayoutGrid.prototype.getRowAnchor = function () { };
/** @public
 * @param {number} at
 */
ILayoutGrid.prototype.setRowAnchor = function (at) { };
/** @public
* @param {number} rowIndex
* @param {boolean=} opt_selected
*/
ILayoutGrid.prototype.setSelectedRow = function (rowIndex, opt_selected) { };

/** @public
 * @param {number} rowIndex
 */
ILayoutGrid.prototype.selectSingleRow = function (rowIndex) { };
/** @public
 * @param {number} rowIndex
 * * @param {number} length can be negative to select upward
 */
ILayoutGrid.prototype.selectRowRange = function (rowIndex, length) { };
/** @public
* @return {number} Number of selection removed
 */
ILayoutGrid.prototype.clearSelectedRows = function () { };
/** @public
 * @return {number}
 */
ILayoutGrid.prototype.getFirstSelectedRow = function () { };
/** @public
 * @return {number}
 */
ILayoutGrid.prototype.getLastSelectedRow = function () { };

/** @public
* @ignore
* @param {number} colIndex
* @param {boolean=} opt_bool
*/
ILayoutGrid.prototype.activateColumn = function (colIndex, opt_bool) { };
/** @public
* @param {number=} from INCLUSIVE
* @param {number=} to EXCLUSIVE
* @param {Array.<number>=} opt_updateRange
*/
ILayoutGrid.prototype.activateRows = function (from, to, opt_updateRange) { };
/** @public
* @ignore
* @param {number} numRows
*/
ILayoutGrid.prototype.moveRowsToTop = function (numRows) {};

/** @public
 * @return {number}
 */
ILayoutGrid.prototype.getFirstIndexInView = function () { };

/** @public
 * @return {number}
 */
ILayoutGrid.prototype.getLastIndexInView = function () { };

/** @public
 * @param {number} index
 * @return {boolean}
 */
ILayoutGrid.prototype.isInCurrentView = function (index) { };

/** @public
* @ignore
* @return {boolean}
*/
ILayoutGrid.prototype.getFrozenLayout = function () { };
/** @public
* @ignore
* @param {boolean} bool
*/
ILayoutGrid.prototype.setFrozenLayout = function (bool) { };

/** @public
* @ignore
* @param {number} index
* @param {Object|null=} opt_json
* @return {!ElementWrapper}
*/
ILayoutGrid.prototype.insertColumn = function (index, opt_json) { };

/** @public
* @ignore
* @param {number} index
* @return {Object|null}
*/
ILayoutGrid.prototype.removeColumnAt = function (index) { };

/** @public
* @ignore
* @param {number} from INCLUSIVE
* @param {number} to INCLUSIVE
*/
ILayoutGrid.prototype.moveColumn = function (from, to) { };

/** @public
* @ignore
* @param {boolean} bool
*/
ILayoutGrid.prototype.enableRowVirtualization = function (bool) { };
/** @public
* @ignore
* @param {boolean} bool
*/
ILayoutGrid.prototype.enableColumnVirtualization = function (bool) { };

/** @public
* @param {Element|Node} colElem
* @return {number}
*/
ILayoutGrid.prototype.getColumnIndex = function (colElem) { };
/** @public
* @param {number} colIndex
* @param {Element} cellElem
* @return {number}
*/
ILayoutGrid.prototype.getCellIndex = function (colIndex, cellElem) { };

/** @public
* @return {number}
*/
ILayoutGrid.prototype.getIndex = function () { };
/** @public
* @ignore
* @param {number} index
*/
ILayoutGrid.prototype.setIndex = function (index) { };
/** @public
* @return {number}
*/
ILayoutGrid.prototype.getRowOffset = function () { };
/** @public
* @ignore
* @param {number} index
*/
ILayoutGrid.prototype.setRowOffset = function (index) { };

/** @public
* @ignore
*/
ILayoutGrid.prototype.updateLayout = function () { };

/** This will make the specified cell horizontally span to fit entire row regardless of position of the cell -- cell will shift to the left most of the section.<br>
* If onlyToTheRight is true, the cell will not extend to the left most position<br>
* This is different from cell colSpan, which the cell remain in the same position. tr-stretched class will also be added to the cell
* @public
* @param {number|Cell} cellRef Either column index or cell reference is acceptable
* @param {number=} rowIndex If the cell instance is given as the first parameter, this parameter will be ignored.
* @param {boolean=} opt_stretching Default is true
* @param {boolean=} onlyToTheRight=false By default, cell will be stretched to both left and right directions, covering entire row. Set to true to expand only to the right
* @return {Cell} The cell being stretched
* @example
* grid.stretchCell(1, 0); // The second cell on the first row will be stretched covering the entire row
* grid.stretchCell(cellInstance, 0, true); // The given cell will be stretched covering the entire row
*/
ILayoutGrid.prototype.stretchCell = function (cellRef, rowIndex, opt_stretching, onlyToTheRight) {};
/**
* @public
* @param {number|Cell} cellRef
* @param {number=} rowIndex
* @return {Cell} The cell being un-stretched
* @see {@link ILayoutGrid#stretchCell}
*/
ILayoutGrid.prototype.unstretchCell = function (cellRef, rowIndex) {};
/**
* @public
*/
ILayoutGrid.prototype.unstretchAllCells = function () {};

/**
* @public
* @param {number|Cell} cellRef
* @param {number=} rowIndex
* @return {Cell}
*/
ILayoutGrid.prototype.getStretchedCell = function (cellRef, rowIndex) {};
/**
* @public
* @return {!Array.<Cell>} The cells that are stretched
* @see {@link ILayoutGrid#stretchCell}
*/
ILayoutGrid.prototype.getStretchedCells = function () {};

/**
* @public
* @see {@link ILayoutGrid#stretchCell}
* @return {boolean} Return true if there is any change
*/
ILayoutGrid.prototype.updateStretchedCells = function () {};

/** Get immediate parent of all child columns (i.e. base grid element)
* @public
* @return {Element}
*/
ILayoutGrid.prototype.getColumnHost = function () {};

/**
* @public
* @ignore
* @param {string} method
*/
ILayoutGrid.prototype.setHidingMethod = function (method) {};
/**
* @public
* @ignore
* @return {string}
*/
ILayoutGrid.prototype.getHidingMethod = function () {};
/**
* @public
* @ignore
* @param {number} colIndex
* @param {boolean} bool
* @param {number} flag Default is 0
* @return {boolean}
*/
ILayoutGrid.prototype.setColumnVisibility = function (colIndex, bool, flag) {};
/**
* @public
* @ignore
* @return {TrackLayout}
*/
ILayoutGrid.prototype.getRowLayout = function () {};

/** @public
* @param {number} size
* @return {Element}
*/
ILayoutGrid.prototype.reserveRightSpace = function (size) {};
/** @public
* @return {Element}
*/
ILayoutGrid.prototype.getReservedSpace = function () {};

/** @public
* @return {CSSStyleDeclaration} CSS live style object
* @see {@link https://developer.mozilla.org/en-US/docs/Web/API/Window/getComputedStyle}
*/
ILayoutGrid.prototype.getComputedStyle = function () {};

/** Column can be virtualized. Column that is in view will be active, while the one that is not in view will be inactive
* @public
* @param {number} colIndex
* @return {boolean}
*/
ILayoutGrid.prototype.isColumnActive = function (colIndex) { };

/**
* @public
* @param {number} rowIndex
* @param {function(number):Array.<{attribute:string, value:string, remove:boolean}>} func
*/
ILayoutGrid.prototype.setRowAttributes = function (rowIndex, func) {};

/** @public
* @param {number} rowIndex
* @return {Object}
*/
ILayoutGrid.prototype.getContextRow = function (rowIndex) {};

/** @public
* @param {number} colIndex
* @param {boolean=} selected
*/
ILayoutGrid.prototype.selectColumn = function (colIndex, selected) {};
/** @public
* @param {number} colIndex
* @return {boolean}
*/
ILayoutGrid.prototype.isSelectedColumn = function (colIndex) {};
/** @public
* @param {number} colIndex
* @param {number} rowIndex
* @param {boolean=} selected
*/
ILayoutGrid.prototype.selectCell = function (colIndex, rowIndex, selected) {};
/** @public
* @param {number} colIndex
* @param {number} rowIndex
* @param {number} width
* @param {number} height
*/
ILayoutGrid.prototype.setCellBounds = function (colIndex, rowIndex, width, height) {};
/** @public
* @ignore
* @return {!TrackLayout}
*/
ILayoutGrid.prototype.getHorizontalLayout = function () {};
/** @public
* @ignore
* @param {number} lftIdx
* @param {number} rgtIdx
* @param {!Array.<number>} outPositions
* @param {!Array.<boolean>} outNoBorders
*/
ILayoutGrid.prototype.calculateColumnBounds = function (lftIdx, rgtIdx, outPositions, outNoBorders) {};
/** @public
* @ignore
* @param {!Array.<Array>} posAry Left and right bound positions in pixel
* @param {!Array.<Array>} noBorderAry Boolean values indicating existence of left and right CSS borders
* @param {number=} topPx Top position of bound
*/
ILayoutGrid.prototype.updateColumnBounds = function (posAry, noBorderAry, topPx) {};
/** @public
* @ignore
*/
ILayoutGrid.prototype.updateColumnSeparators = function () {};

/* harmony default export */ const grid_ILayoutGrid = ((/* unused pure expression or super */ null && (ILayoutGrid)));


;// ./node_modules/@grid/core/es6/grid/components/Column.js
/* eslint-disable */





/* eslint-enable */

/** @constructor
* @ignore
* @extends {ElementWrapper}
* @param {ILayoutGrid} section Parent section
*/
let Column = function (section) {
	let elem = this._element = document.createElement("div");
	elem.className = "column";
	this._cells = [];
	this._section = section;
};
es6_Ext.inherits(Column, components_ElementWrapper);

/** @type {Column}
* @private
*/
Column.prototype._nextSibling = null;

/** @type {Node}
* @private
*/
Column.prototype._parent = null;
/** @type {ILayoutGrid}
* @private
*/
Column.prototype._section = null;
/** @type {Array.<!Cell>}
* @private
*/
Column.prototype._cells = null;
/** @type {number}
* @private
*/
Column.prototype._cellStart = 0;
/** @type {number}
* @private
*/
Column.prototype._cellEnd = 0;
/** @type {boolean}
* @private
*/
Column.prototype._autoStyleUpdating = false;
/** @type {boolean}
* @description In some use case column cannot be deactivated, e.g. column is stretched or spanned
* @private
*/
Column.prototype._deactivation = false;
/** @type {Array}
* @private
*/
Column.prototype._sizeClasses = null;
/** @type {number}
* @private
*/
Column.prototype._colLeft = 0;
/** @type {boolean}
* @private
*/
Column.prototype._absolute = false;
/** @type {Node}
* @description Scroll pane, only the left pinned column will have this property,
*              used to keep order of inline-block element.
* @private
*/
Column.prototype._scrollFrame = null;

/** @type {number}
* @description Each bit represents visibility state as the followings:
* Bit 0 is used for normal show/hide method
* Bit 1 is used for zero width
* Bit 2 is used for zero height
* Bit 3 is used for active/inactive state of virtualization
* @private
*/
Column.prototype._invisibility = 0;
/**
* @type {!Array.<number>}
* @private
*/
Column.prototype._bits = [1, 2, 4, 8]; // Static variable
/** @type {boolean}
* @private
*/
Column.prototype._pinned = true;

/** @public
*/
Column.prototype.clearContent = function() {
	for(let r = this._cells.length; --r >= 0;) {
		this._cells[r].setContent(null);
	}
};

/** {@link ElementWrapper#setLeft}
* @override
*/
Column.prototype.setLeft = function (val) {
	this._colLeft = val;
	if(this._absolute) {
		this._element.style.left = val + "px";
	}
	return true;
};
/** @public
* @ignore
* @param {boolean=} bool
*/
Column.prototype.enableAbsolutePosition = function (bool) {
	bool = bool !== false;
	if(bool !== this._absolute) {
		this._absolute = bool;
		if(bool) {
			this._element.style.left = this._colLeft + "px";
		} else {
			this._element.style.left = "";
		}
	}
};
/** {@link ElementWrapper#dispose}
* @override
*/
Column.prototype.dispose = function () {
	this._dispose();
	for(let r = this._cells.length; --r >= 0;) {
		this._cells[r].dispose();
	}
	this._scrollFrame = this._nextSibling = this._parent = this._cells = this._section = null;
};
/** {@link ElementWrapper#setStyle}
* @override
*/
Column.prototype.setStyle = function (str, val) {
	let elem = this._element;
	elem.style[str] = val;

	let bgc = elem.style.backgroundColor;
	if(bgc) {
		if(!elem.classList.contains("no-shading")) { elem.classList.add("no-shading"); }
	} else {
		if(elem.classList.contains("no-shading")) { elem.classList.remove("no-shading"); }
	}
};
/** {@link ElementWrapper#setStyles}
* @override
*/
Column.prototype.setStyles = function (json) {
	let elem = this._element;
	let styles = elem.style;
	for (let key in json) {
		styles[key] = json[key];
	}

	let bgc = elem.style.backgroundColor;
	if(bgc) {
		if(!elem.classList.contains("no-shading")) { elem.classList.add("no-shading"); }
	} else {
		if(elem.classList.contains("no-shading")) { elem.classList.remove("no-shading"); }
	}
};

/** {@link ElementWrapper#show}
* @override
*/
Column.prototype.show = function (opt_shown) {
	this.setVisibility(0, opt_shown !== false);
};

/** {@link ElementWrapper#setWidth}
* @override
*/
Column.prototype.setWidth = function (val) {
	this._element.style.width = val + "px";
	let magnitude = 0;
	if(val > 0 && val <= 24){
		magnitude = 3;
	}
	else if(val > 24 && val <= 40){
		magnitude = 2;
	}
	else if(val > 40 && val < 60){
		magnitude = 1;
	}

	let i;
	let sc = this._sizeClasses;
	if(sc) {
		if(sc.length > magnitude) {
			for(i = sc.length; --i >= magnitude;) {
				this.removeClass(sc[i]);
			}
			sc.length = magnitude;
		}
		if(!magnitude) {
			this._sizeClasses = null;
		}
	}

	if(magnitude) {
		if(!sc) {
			sc = [];
			this._sizeClasses = sc;
		}
		for(i = sc.length; i < magnitude; ++i) {
			sc[i] = Column._sizeClasses[i];
			this.addClass(sc[i]);
		}
	}

	this.setVisibility(1, val > 0);
	return true;
};
/** @type {Array.<string>}
* @private
*/
Column._sizeClasses = ["small-col", "very-small-col", "tiny-col"];

/** {@link ElementWrapper#setHeight}
* @override
*/
Column.prototype.setHeight = function (val) {
	this._element.style.height = val + "px";
	this.setVisibility(2, val > 0);
	return true;
};
/** {@link ElementWrapper#getParent}
* @override
*/
Column.prototype.getParent = function () {
	return this._parent;
};
/** {@link ElementWrapper#setParent}
* @override
*/
Column.prototype.setParent = function (parent, opt_prepend) {
	let el = this._element;
	if(!el) { return; }

	this._parent = util.getElement(parent);
	let hidingMethod = this._section ? this._section.getHidingMethod() : "";

	// hidden column with hidingMethod should appear in tree
	// to prevent from unability of showing
	if(!this._invisibility || hidingMethod) {
		this._setParent(this._parent, opt_prepend);
	}
};
/** {@link ElementWrapper#insertBefore}
* @override
*/
Column.prototype.insertBefore = function (nextSibling) {
	if(this._parent) {
		if(this._scrollFrame) { // Only left pinned columns have this property
			// This column must be moved within the same parent and before its scroll frame sibling.
			if(nextSibling && this._parent === nextSibling.getParent()) {
				this._insertBefore(nextSibling);
			} else {
				this._insertBefore(this._scrollFrame);
			}
		} else if(nextSibling) {
			if(this._parent === nextSibling.getParent()) { // Move column, only if their parents are the same
				this._insertBefore(nextSibling);
			}
			// else {
			// cannot move this column to differrent parent
			// }
		} else { // Place the column at the end when there is no next sibling
			this._setParent(this._parent);
		}
	} else {
		this._insertBefore(nextSibling); // TODO: Perform: column virtualization
		this._parent = nextSibling.getParent(); // TODO: Check if this is necessary
	}
};
//#endregion Public Methods

//#region Internal Public Methods
/** @public
* @ignore
* @param {boolean=} enabled
*/
Column.prototype.enableAutoStyleUpdating = function (enabled) {
	enabled = enabled !== false;

	if(this._autoStyleUpdating !== enabled) {
		this._autoStyleUpdating = enabled;
		// if(this._autoStyleUpdating) {
		// this.updateStyle();
		// }
	}
};
/** @public
* @param {Column} sibling
*/
Column.prototype.setNextSibling = function (sibling) {
	this._nextSibling = sibling;
};
/** @public
* @return {boolean}
*/
Column.prototype.isActive = function () {
	return !(this._invisibility & this._bits[3]); // Bit 3 has value, then column is inactive
};
/** @public
* @ignore
* @param {boolean=} opt_disabled
*/
Column.prototype.disableDeactivation = function (opt_disabled) {
	this._deactivation = opt_disabled !== false;
	if(this._deactivation) {
		if(!this.isActive()) {
			this.activate();
		}
	}
};
/** @public
* @ignore
* @param {boolean=} opt_enabled
* @return {boolean} Return true if there is any change
*/
Column.prototype.activate = function (opt_enabled) {
	opt_enabled = this._pinned || this._deactivation || (opt_enabled !== false);
	if (this.isActive() !== opt_enabled) {
		this.setVisibility(3, opt_enabled);
		return true;
	}
	return false;
};
/** @public
* @ignore
* @param {boolean=} opt_disabled
*/
Column.prototype.deactivate = function (opt_disabled) {
	this.activate(opt_disabled === false);
};
/** @public
* @ignore
* @param {number} count
*/
Column.prototype.setCellCount = function (count) {
	let r;
	for(r = this._cells.length; r < count; ++r) {
		this._cells.push(new components_Cell(null, this._section));
	}
	for(r = this._cells.length; --r >= count;) {
		this._cells[r].dispose();
	}
	this._cells.length = count;
	if(this._cellStart > count) {
		this._cellStart = this._cellEnd = count;
	} else if(this._cellEnd > count) {
		this._cellEnd = count;
	}
};
/** @public
* @ignore
* @param {number} rowIndex
* @return {Cell}
*/
Column.prototype.getCell = function (rowIndex) {
	return this._cells[rowIndex] || null;
};
/** @public
* @ignore
* @param {number} from
* @param {number} length
* @param {boolean} collapsed
* @param {boolean} excludeFirstCell
* @param {boolean} keptInDoc
*/
Column.prototype.collapseCells = function (from, length, collapsed, excludeFirstCell, keptInDoc) {
	if(length <= 0) { return; }

	let dirty = false;
	let cell = this._cells[from];
	if(cell && !excludeFirstCell) { // If we need to process the first cell
		dirty |= cell.collapse(collapsed, keptInDoc); // If we have multiple rows kept the cell in the doc for positioning
	}
	for(let i = 1; i < length; ++i) {
		cell = this._cells[from + i];
		if(cell) {
			dirty |= cell.collapse(collapsed, keptInDoc);
		} else {
			break;
		}
	}
	if(dirty) {
		if(collapsed) {
			if(length > 1) {
				this._hasCollapsedCells = true;
			}
		} else {
			this.updateCellOrders(); // WARNING: Re-inserting all cells is slow
		}
	}
};
/** @public
* @deprecated
* @ignore
*/
Column.prototype.uncollapseAllCells = util._deprecatedFunction;

/** @public
* @ignore
* @param {number} from
* @param {number} to
*/
Column.prototype.activateCells = function (from, to) {
	let r;
	let parent = this._element;
	if(this._hasCollapsedCells) { // There is a vertical span and no row virtualization. We only need to appendChild incrementally
		from = this._cellEnd;
		if(to > this._cellEnd) {
			this._cellEnd = to;
		}
	} else {
		if(this._cellStart === from) {
			let diff = this._cellEnd - to;
			if(diff > 0) {
				for(r = this._cellEnd; --r >= to;) {
					this._cells[r].removeParent();
				}
				this._cellEnd = to;
				return;
			} else if(diff < 0) {
				from = this._cellEnd; // Append new cells without removing any child
				this._cellEnd = to;
			} else {
				return; // Activate the same range
			}
		} else {
			for(r = this._cellEnd; --r >= this._cellStart;) {
				this._cells[r].removeParent();
			}
			this._cellStart = from;
			this._cellEnd = to;
		}
	}

	for(r = from; r < to; ++r) {
		let cell = this._cells[r];
		if(cell.isKeptInDoc()) {
			cell.initialize();
			parent.appendChild(cell.getElement());
		}
	}
};
/** @public
* @ignore
* @param {number} numCells Index of the first cell to be moved to the top
*/
Column.prototype.moveCellsToTop = function (numCells) {
	if(!this.isActive()) {
		return; // There is no point in cycling cells when the column is inactive
	}

	let parent = this._element;
	for(let i = 0; i < numCells; ++i) {
		let cell = this._cells[i];
		if(cell.isKeptInDoc()) {
			parent.appendChild(cell.getElement());
		}
	}
	this._cells = util.cycleArray(this._cells, numCells);
};
/** @public
* @ignore
*/
Column.prototype.updateCellOrders = function () {
	this._hasCollapsedCells = false;
	let parent = this._element;
	for(let i = this._cellStart; i < this._cellEnd; ++i) {
		let cell = this._cells[i];
		if(cell.isKeptInDoc()) {
			parent.appendChild(cell.getElement());
		} else {
			this._hasCollapsedCells = true;
		}
	}
};
/** @public
* @ignore
* @return {boolean}
*/
Column.prototype.hasCollapsedCells = function () {
	return this._hasCollapsedCells;
};
/** @public
* @ignore
* @param {Element} cellElem
* @returns {number}
*/
Column.prototype.getCellIndex = function (cellElem) {
	if(!cellElem) { return -1; }
	for(let i = this._cellStart; i < this._cellEnd; ++i) {
		let cell = this._cells[i];
		if(cell.getElement() === cellElem) {
			return i;
		}
	}
	return -1;
};
/**
* Get section instance (ILayoutGrid) which is the parent of this column
* @public
* @return {ILayoutGrid}
*/
Column.prototype.getSection = function () {
	return this._section;
};

/**
* @public
* @ignore
* @return {Array.<string>}
*/
Column.prototype.log = function () {
	let ary = [];
	let len = this._cells.length;
	for(let i = 0; i < len; ++i) {
		ary.push(this._cells[i].getTextContent());
	}
	return ary;
};

/** @public
* @ignore
* @return {boolean}
*/
Column.prototype.getVisibility = function () {
	return !this._invisibility;
};
/** @public
* @ignore
* @param {number} bitIndex
* @param {*} shown
*/
Column.prototype.setVisibility = function (bitIndex, shown) {
	let prevVis = !this._invisibility;

	let bit = this._bits[bitIndex] || 1;
	if(shown) {
		this._invisibility &= ~bit; // Remove invisibility bit
	} else {
		this._invisibility |= bit; // Add invisibility bit
	}

	let newVis = !this._invisibility;
	if (prevVis === newVis) {
		return; // Nothing has changed
	}

	let hidingMethod = this._section ? this._section.getHidingMethod() : "";
	if(hidingMethod) {
		this.enableClass("hidden", !newVis); // Custom class method
		// this._element.style.display = (newVis !== false) ? "" : "none";
	} else {
		if(newVis) { // Find a proper place to insert (show)
			// this._element.style.display = "";
			let curParent = this._parent;
			if(!curParent) {
				return; // WARNING: It could possibly be a bug, if there is no parent at this moment
			}
			let nextSibling = this._nextSibling; // Next sibling can be an empty object
			while(nextSibling) {
				if (nextSibling._parent !== curParent) {
					// This could mean either this is pinned left column or column within the scroll frame (i.e. any column that is not pinned to the right)
					nextSibling = null;
					break;
				}
				if(!nextSibling._invisibility) {
					break;
				}
				nextSibling = nextSibling._nextSibling;
			}
			this.insertBefore(nextSibling);
		} else { // Hide by removing from the DOM
			this._setParent(null);
		}
	}
};
/** @public
* @ignore
*/
Column.prototype.updateLayout = function () {};

/** @public
* @ignore
* @param {Node} pane Scroll pane is required by pinned column on the left side to maintain the order
* @param {Node} slider
* @param {boolean} absolutePos
*/
Column.prototype.setScrollState = function (pane, slider, absolutePos) {
	this._scrollFrame = pane;

	this.enableAbsolutePosition(absolutePos);

	this.setParent(slider || this._section.getColumnHost());

	if(!slider) {
		// Re-activate any column that is not inside the scroll slider (pinned columns on either left or right side)
		this.activate(); // This may change invisibility state
	}

	this._pinned = !slider;
};

Column._proto = Column.prototype;

/* harmony default export */ const components_Column = (Column);


;// ./node_modules/@grid/core/es6/grid/components/VirtualItems.js


/** @constructor
* @ignore
*/
let VirtualItems = function () {
	this._items = [];
};
/** @type {Array.<!Object>}
* @private
*/
VirtualItems.prototype._items = null;

/** @public
*/
VirtualItems.prototype.dispose = function () {
	this._items = null;
};

/** @public
* @ignore
* @param {number} count
*/
VirtualItems.prototype.setItemCount = function (count) {
	let ary = this._items;
	let len = ary.length;
	if(len !== count) {
		ary.length = count;
		while(len < count) {
			ary[len++] = {};
		}
	}
};
/** @public
* @ignore
* @param {number} idx
* @return {Object}
*/
VirtualItems.prototype.getItem = function (idx) {
	return this._items[idx] || null;
};
/** @public
* @ignore
* @param {number} numItems Index of the first item to be moved to the top
*/
VirtualItems.prototype.moveItemsToTop = function (numItems) {
	this._items = util.cycleArray(this._items, numItems);
};

/* harmony default export */ const components_VirtualItems = (VirtualItems);


;// ./node_modules/@grid/core/es6/grid/components/StretchedCells.js

 // eslint-disable-line
 // eslint-disable-line



/** Update cell's top and height
* @private
* @param {Cell} cell
* @param {TrackLayout} trackY
* @param {number} rowIndex
*/
let _updateYAxis = function (cell, trackY, rowIndex) {
	cell.setTop(trackY.getLaneStart(rowIndex));
	cell.setHeight(trackY.getLaneSize(rowIndex));
};
/** Update cell's left and width
* @private
* @param {Cell} cell
* @param {TrackLayout} trackX
* @param {number} colIndex
* @param {number} width
*/
let _updateXAxis = function (cell, trackX, colIndex, width) {
	if(colIndex) {
		let leftStart = trackX.getLaneStart(colIndex);
		cell.setLeft(leftStart);
		cell.setWidth(width - leftStart);
	} else {
		cell.setStyle("left", "");
		cell.setWidth(width); // Cell width is always modified (colSpan styling will be reset)
	}
};


/** @constructor
* @ignore
* @param {!TrackLayout} trackX
* @param {!TrackLayout} trackY
* @param {!ILayoutGrid} section
*/
let StretchedCells = function (trackX, trackY, section) {
	this._trackX = trackX;
	this._trackY = trackY;
	this._section = section;
	this._cells = [];
};
/** @type {!TrackLayout}
* @private
*/
StretchedCells.prototype._trackX;
/** @type {!TrackLayout}
* @private
*/
StretchedCells.prototype._trackY;
/** @type {!ILayoutGrid}
* @private
*/
StretchedCells.prototype._section;
/** @type {!Array.<Cell>}
* @private
*/
StretchedCells.prototype._cells;
/** @type {number}
* @private
*/
StretchedCells.prototype._activeCount = 0;
/** @type {number}
* @private
*/
StretchedCells.prototype._stretchSize = 0;

/** @public
*/
StretchedCells.prototype.dispose = function () {
	let cells = this._cells;
	if(this._activeCount) {
		this._activeCount = 0;
		for(let i = cells.length; --i >= 0;) {
			let cell = cells[i];
			if(cell) {
				cell.dispose();
			}
		}
	}
	cells.length = 0;
};

/** @public
* @ignore
* @param {number} num
*/
StretchedCells.prototype.setStretchSize = function(num) {
	this._stretchSize = num;
};
/** @public
* @ignore
* @return {number}
*/
StretchedCells.prototype.getStretchSize = function() {
	return this._stretchSize;
};

/** @public
* @ignore
* @param {number} count
*/
StretchedCells.prototype.setCellCount = function (count) {
	let cells = this._cells;
	let len = cells.length;
	if(len < count) {
		cells.length = count;
		while(len < count) {
			cells[len++] = null;
		}
	} else if(len > count) {
		while(--len >= count) {
			let cell = cells[len];
			if(cell) {
				cell.dispose();
			}
		}
		cells.length = count;
	}
};
/** @public
* @ignore
* @param {number} rowIndex
* @return {Cell}
*/
StretchedCells.prototype.getCell = function (rowIndex) {
	let cell = this._cells[rowIndex];
	if(cell && cell["getParent"]()) {
		return cell;
	}
	return null;
};
/** @public
* @ignore
* @return {Array.<Cell>}
*/
StretchedCells.prototype.getCells = function () {
	let outAry = [];
	if(this._activeCount) {
		let cells = this._cells;
		let rowCount = cells.length;
		for(let r = 0; r < rowCount; ++r) {
			let cell = cells[r];
			if(cell && cell.getParent()) {
				outAry.push(cell);
			}
		}
		this._activeCount = outAry.length;
	}
	return outAry;
};
/** @public
* @ignore
* @param {Cell|Element} cellRef
* @return {number}
*/
StretchedCells.prototype.getColumnIndex = function (cellRef) {
	if(cellRef) {
		if(cellRef["getElement"]) {
			let cellElement = cellRef["getElement"]();
			if(cellElement) {
				return cellElement._colIndex != null ? cellElement._colIndex : -1;
			}
		} else {
			return cellRef._colIndex != null ? cellRef._colIndex : -1;
		}
	}
	return -1;
};
/** @public
* @ignore
* @param {Cell|Element} cellRef
* @return {number}
*/
StretchedCells.prototype.getRowIndex = function (cellRef) {
	if(cellRef) {
		if(cellRef["getElement"]) {
			return cellRef["getParent"]() ? this._cells.indexOf(cellRef) : -1;
		} else {
			let cells = this._cells;
			let rowCount = cells.length;
			for(let r = 0; r < rowCount; ++r) {
				let cell = cells[r];
				if(cell && cell["getParent"]() && cell["getElement"]() === cellRef) {
					return r;
				}
			}
		}
	}
	return -1;
};

/** @public
* @ignore
* @param {number} rowIndex
* @param {number=} colIndex Column index indicates the position of the cell. If not defined, the cell will always position at the left most
* @return {Cell}
*/
StretchedCells.prototype.stretchCell = function (rowIndex, colIndex) {
	// Increase the active count regardless of redundancy. This acts as a dirty flag. It does not need to be accurate
	++this._activeCount;
	if(!colIndex || colIndex < 0) {
		colIndex = 0;
	}

	let cell = this._cells[rowIndex] || null;
	if(!cell) {
		cell = new components_Cell(null, this._section);
		cell.addClass("tr-stretched");
		cell["stretched"] = true;
		this._cells[rowIndex] = cell;
	}
	let cellElem = cell["getElement"]();
	cellElem["stretched"] = true;
	if(cellElem._colIndex !== colIndex) {
		cellElem._colIndex = colIndex;
		if(colIndex > 0) {
			cellElem.classList.add("right-stretching");
		} else {
			cellElem.classList.remove("right-stretching");
		}
	}
	if(cellElem._rowIndex !== rowIndex) {
		cellElem._rowIndex = rowIndex;
	}

	let width = this._stretchSize || this._trackX.getTrackSize();
	_updateXAxis(cell, this._trackX, colIndex, width);
	_updateYAxis(cell, this._trackY, rowIndex);

	return cell;
};
/** @public
* @ignore
* @param {number} rowIndex
* @return {Cell}
*/
StretchedCells.prototype.unstretchCell = function(rowIndex) {
	let cell = this._cells[rowIndex] || null;
	if(cell) {
		cell.setParent(null);
	}
	return cell;
};
/** @public
*/
StretchedCells.prototype.unstretchAllCells = function () {
	if(this._activeCount) {
		this._activeCount = 0;
		let cells = this._cells;
		for(let i = cells.length; --i >= 0;) {
			let cell = cells[i];
			if(cell) {
				cell.setParent(null);
			}
		}
	}
};

/** Update cell's left, top, width and height for all active cells
* @public
* @ignore
*/
StretchedCells.prototype.updateCells = function() {
	if(this._activeCount) {
		let activeCount = 0;
		let cells = this._cells;
		let rowCount = cells.length;
		let trackX = this._trackX;
		let trackY = this._trackY;
		let width = this._stretchSize || trackX.getTrackSize();
		for(let r = 0; r < rowCount; ++r) {
			let cell = cells[r];
			if(cell && cell.getParent()) {
				++activeCount;
				_updateXAxis(cell, trackX, cell.getElement()._colIndex, width);
				_updateYAxis(cell, trackY, r);
			}
		}
		this._activeCount = activeCount;
	}
};
/** Update cell's left, top, width and height for all active cells
* @public
* @ignore
*/
StretchedCells.prototype.updateXAxis = function() {
	if(this._activeCount) {
		let activeCount = 0;
		let cells = this._cells;
		let rowCount = cells.length;
		let trackX = this._trackX;
		let width = this._stretchSize || trackX.getTrackSize();
		for(let r = 0; r < rowCount; ++r) {
			let cell = cells[r];
			if(cell && cell.getParent()) {
				++activeCount;
				_updateXAxis(cell, trackX, cell.getElement()._colIndex, width);
			}
		}
		this._activeCount = activeCount;
	}
};
/** Update cell's top and height for all active cells
* @public
* @ignore
*/
StretchedCells.prototype.updateYAxis = function () {
	if(this._activeCount) {
		let activeCount = 0;
		let cells = this._cells;
		let rowCount = cells.length;
		let trackY = this._trackY;
		for(let r = 0; r < rowCount; ++r) {
			let cell = cells[r];
			if(cell && cell.getParent()) {
				++activeCount;
				_updateYAxis(cell, trackY, r);
			}
		}
		this._activeCount = activeCount;
	}
};

/** @public
* @ignore
* @param {number} numCells Index of the first item to be moved to the top
*/
StretchedCells.prototype.moveCellsToTop = function (numCells) {
	if(this._activeCount) {
		this._cells = util.cycleArray(this._cells, numCells);
		this.updateYAxis(); // Reposition cells due to moving row index
	}
};

/* harmony default export */ const components_StretchedCells = (StretchedCells);


;// ./node_modules/@grid/core/es6/grid/components/CellSpan.js

/** @constructor
* @param {number} x
* @param {number} y
*/
let CellSpan = function (x, y) {
	this.indexX = x;
	this.indexY = y;
};

/** @public
 * @param {number} x
 * @param {number} y
* @return {boolean}
*/
CellSpan.prototype.isSpanIndices = function (x, y) {
	return x === this.indexX && y === this.indexY;
};
/** @public
 * @param {number} x
 * @param {number} y
* @return {boolean}
*/
CellSpan.prototype.isOccupiedIndices = function (x, y) {
	return x !== this.indexX || y !== this.indexY;
};

/** @public
* @param {!Array.<CellSpan>} allSpans
* @param {!Object.<string, CellSpan>} occupiedMap
*/
CellSpan.prototype.register = function (allSpans, occupiedMap) {
	this._register(allSpans, occupiedMap, true);
};
/** @public
* @param {!Array.<CellSpan>} allSpans
* @param {!Object.<string, CellSpan>} occupiedMap
*/
CellSpan.prototype.unregister = function (allSpans, occupiedMap) {
	this._register(allSpans, occupiedMap, false);
};
/** @private
* @param {!Array.<CellSpan>} allSpans
* @param {!Object.<string, CellSpan>} occupiedMap
* @param {boolean} bool
*/
CellSpan.prototype._register = function (allSpans, occupiedMap, bool) {
	let at = allSpans.indexOf(this);
	if(bool) {
		if(at < 0) {
			allSpans.push(this);
		}
	} else {
		if(at >= 0) {
			allSpans.splice(at, 1);
		}
	}

	let obj = bool ? this : null;
	let endCol = this.indexX + this.colSpan;
	let endRow = this.indexY + this.rowSpan;
	for (let c = this.indexX; c < endCol; ++c) {
		for (let r = this.indexY; r < endRow; ++r) {
			occupiedMap[c + "," + r] = obj;
		}
	}
};

/** @public
* @type {number}
*/
CellSpan.prototype.indexX;
/** @public
* @type {number}
*/
CellSpan.prototype.indexY;
/** @public
* @type {number}
*/
CellSpan.prototype.colSpan = 1;
/** @public
* @type {number}
*/
CellSpan.prototype.rowSpan = 1;
/** @public
* @type {number}
*/
CellSpan.prototype.prevColSpan = 1;
/** @public
* @type {number}
*/
CellSpan.prototype.prevRowSpan = 1;
/** @public
* @type {number}
*/
CellSpan.prototype.retVal = 0;

/* harmony default export */ const components_CellSpan = (CellSpan);


;// ./node_modules/@grid/core/es6/grid/components/CellSpans.js


/** @constructor */
let CellSpans = function () {
	this._spans = [];
	this._occupiedMap = {};
};

/** @public
* @param {number} at
* @return {boolean} Return true if there is any change
*/
CellSpans.prototype.removeColumn = function (at) { // Use when a column is removed
	let dirty = false;
	if(this.removeSpanByColIndex(at)) {
		dirty = true;
	}
	if(this.shiftColumn(at, -1)) {
		dirty = true;
	}
	return dirty;
};
/** @public
* @param {boolean=} bool
*/
CellSpans.prototype.freezeMapping = function (bool) {
	this._frozenMapping = bool !== false;
	if(!this._frozenMapping) {
		this._mapCellSpans();
	}
};


/** @private
*/
CellSpans.prototype._mapCellSpans = function () {
	let cellSpans = this._spans;
	this._spans = [];
	this._occupiedMap = {};
	let i, cellSpan;
	for(i = cellSpans.length; --i >= 0;) {
		cellSpan = cellSpans[i];
		if(cellSpan.indexX >= 0) {
			cellSpan.register(this._spans, this._occupiedMap);
		}
	}
};


/** @public
* @param {number} from
* @param {number} amount This can be negative number
* @return {boolean} Return true if there is any change
*/
CellSpans.prototype.shiftColumn = function (from, amount) { // Use when a column is inserted or removed
	if(this._spans.length <= 0 || amount === 0) { return false; }

	let dirty = false;
	for (let i = this._spans.length; --i >= 0;) {
		let cellSpan = this._spans[i];
		if(cellSpan.indexX >= from) {
			dirty = true;
			cellSpan.indexX += amount;
		}
	}
	if(!dirty || this._frozenMapping) { return false; }

	// Re-registers all cellspans. Some of the cellspans may be lost due to shifting
	this._mapCellSpans();
	return true;
};
/** @public
 * @param {number} indexX
 * @param {number} indexY
* @return {CellSpan}
*/
CellSpans.prototype.removeSpan = function (indexX, indexY) {
	if(this._spans.length <= 0) { return null; }

	let key = indexX + "," + indexY;
	let cellSpan = this._occupiedMap[key];
	if(!cellSpan || cellSpan.isOccupiedIndices(indexX, indexY)) { return null; }

	cellSpan.unregister(this._spans, this._occupiedMap);
	return cellSpan;
};
/** @public
* @param {number} at
* @param {Array.<CellSpan>=} opt_ary Array that contains removed CellSpan.
* @return {Array.<CellSpan>} Return null if no CellSpan is removed
*/
CellSpans.prototype.removeSpanByColIndex = function (at, opt_ary) {
	let len = this._spans.length;
	if(len <= 0) { return null; }

	let cellSpan;
	let removedCellSpans = opt_ary || [];
	let count = 0;
	let i;
	for (i = 0; i < len; ++i) {
		cellSpan = this._spans[i];
		if(cellSpan && cellSpan.indexX === at) {
			removedCellSpans.push(cellSpan);
			count++;
		}
	}
	if(count <= 0) { return null; }

	len = removedCellSpans.length;
	for(i = 0; i < count; ++i) {
		removedCellSpans[len - i - 1].unregister(this._spans, this._occupiedMap);
	}
	return removedCellSpans;
};
/** @public
* @param {number} at
* @param {Array.<CellSpan>=} opt_ary Array that contains removed CellSpan.
* @return {Array.<CellSpan>} Return null if no CellSpan is removed
*/
CellSpans.prototype.removeSpanStartedFromRowIndex = function (at, opt_ary) {
	let len = this._spans.length;
	if(len <= 0) { return null; }

	let cellSpan;
	let removedCellSpans = opt_ary || [];
	let count = 0;
	let i;
	for (i = 0; i < len; ++i) {
		cellSpan = this._spans[i];
		if(cellSpan && cellSpan.indexY >= at) {
			removedCellSpans.push(cellSpan);
			count++;
		}
	}
	if(count <= 0) { return null; }

	len = removedCellSpans.length;
	for(i = 0; i < count; ++i) {
		removedCellSpans[len - i - 1].unregister(this._spans, this._occupiedMap);
	}
	return removedCellSpans;
};

/** @public
* @return {Array.<CellSpan>}
*/
CellSpans.prototype.clearAllSpans = function () {
	if(this._spans.length <= 0) { return null; }

	let allCellSpans = this._spans;
	this._spans = [];
	this._occupiedMap = {};
	return allCellSpans;
};
/** @public
* @return {boolean}
*/
CellSpans.prototype.hasSpan = function () {
	return (this._spans.length  > 0);
};

/** @public
* @param {number} indexX
* @param {number} indexY
* @return {CellSpan|null}
*/
CellSpans.prototype.getSpan = function (indexX, indexY) {
	let cellSpan = this.getOccupyingSpan(indexX, indexY);
	if(cellSpan && cellSpan.isSpanIndices(indexX, indexY)) {
		return cellSpan;
	}
	return null;
};
/** @public
* @param {number} indexX
* @param {number} indexY
* @return {CellSpan|null}
*/
CellSpans.prototype.getOccupyingSpan = function (indexX, indexY) {
	if(this._spans.length <= 0) { return null; }
	return this._occupiedMap[indexX + "," + indexY] || null;
};
/** Shallow copy is used for better performance. The returned cellspans are still referenced.
* @public
* @return {!Array.<CellSpan>}
*/
CellSpans.prototype.getAllSpans = function () {
	return this._spans;
};
/** @description Returns zero or negative value, if the cell is spanned over (i.e. occupied by the spanning cell). <br>
* Negative value is distance from the spanning cell (i.e. -1 means one cell from the spanning cell and -2 means two cells from the spanning cell) <br>
* Return one, if the cell has no span (normal cell). <br>
* Return value greater than one, if the cell is spanning (getting expanded)
* @public
* @param {number} indexX
* @param {number} indexY
* @return {number}
*/
CellSpans.prototype.getColSpan = function (indexX, indexY) {
	let cellSpan = this._occupiedMap[indexX + "," + indexY];
	if(cellSpan == null) {
		return 1;
	}
	if (cellSpan.isOccupiedIndices(indexX, indexY)) {
		return cellSpan.indexX - indexX;
	}
	return cellSpan.colSpan;
};
/** @description Returns zero or negative value, if the cell is spanned over (i.e. occupied by the spanning cell). <br>
* Negative value is distance from the spanning cell (i.e. -1 means one cell from the spanning cell and -2 means two cells from the spanning cell) <br>
* Return one, if the cell has no span (normal cell). <br>
* Return value greater than one, if the cell is spanning (getting expanded)
* @public
* @param {number} indexX
* @param {number} indexY
* @return {number}
*/
CellSpans.prototype.getRowSpan = function (indexX, indexY) {
	let cellSpan = this._occupiedMap[indexX + "," + indexY];
	if(cellSpan == null) {
		return 1;
	}
	if (cellSpan.isOccupiedIndices(indexX, indexY)) {
		return cellSpan.indexY - indexY;
	}
	return cellSpan.rowSpan;
};

/** @public
* @param {number} indexX
* @param {number} indexY
* @param {number} spanVal
* @return {CellSpan} Return effected CellSpan definition
*/
CellSpans.prototype.setColSpan = function (indexX, indexY, spanVal) {
	return this._updateSpan(indexX, indexY, true, spanVal);
};
/** @public
* @param {number} indexX
* @param {number} indexY
* @param {number} spanVal
* @return {CellSpan} Return effected CellSpan definition
*/
CellSpans.prototype.setRowSpan = function (indexX, indexY, spanVal) {
	return this._updateSpan(indexX, indexY, false, spanVal);
};

/** @private
* @param {number} c0
* @param {number} r0
* @param {boolean} isColumn
* @param {number} spanVal
* @return {CellSpan} Return effected CellSpan definition
*/
CellSpans.prototype._updateSpan = function (c0, r0, isColumn, spanVal) {
	if (spanVal < 1) { spanVal = 1; }
	let cellSpan = this.getSpan(c0, r0);
	if(cellSpan) { cellSpan.retVal = 0; }

	let c1 = c0;
	let r1 = r0;
	let cs = (cellSpan) ? cellSpan.colSpan : 1;
	let rs = (cellSpan) ? cellSpan.rowSpan : 1;
	let c2, r2, target, key;
	let isExpanding = true;
	if (isColumn) {
		if (cs === spanVal) { return cellSpan; }
		if (cs > spanVal) {
			c1 = c0 + spanVal;
			c2 = c0 + cs;
			isExpanding = false;
		} else {
			target = this._occupiedMap[c0 + "," + r0];
			if (target && target !== cellSpan) {
				return cellSpan;
			}
			c1 = c0 + cs;
			c2 = c0 + spanVal;
		}
		r2 = r0 + rs;
	} else {
		if (rs === spanVal) { return cellSpan; }
		if (rs > spanVal) {
			r1 = r0 + spanVal;
			r2 = r0 + rs;
			isExpanding = false;
		} else {
			target = this._occupiedMap[c0 + "," + r0];
			if (target && target !== cellSpan) {
				return cellSpan;
			}
			r1 = r0 + rs;
			r2 = r0 + spanVal;
		}
		c2 = c0 + cs;
	}

	// Collecting affected cells
	let c, r;
	if (isExpanding) { // Check all affected cells for occupying
		let keys = [];
		for (c = c1; c < c2; ++c) {
			for (r = r1; r < r2; ++r) {
				key = c + "," + r;
				if (this._occupiedMap[key]) { //Invalid input is given
					return cellSpan;
				}
				keys.push(key);
			}
		}

		if (cellSpan == null) {
			cellSpan = new components_CellSpan(c0, r0);
			key = c0 + "," + r0;
			this._spans.push(cellSpan);
			this._occupiedMap[key] = cellSpan;
		}

		for (c = keys.length; --c >= 0; ) { // Update occupied cell map
			this._occupiedMap[keys[c]] = cellSpan;
		}
	} else {
		let noLongerSpan = (isColumn) ? (spanVal + rs <= 2) : (spanVal + cs <= 2);
		if (noLongerSpan) {
			key = c0 + "," + r0;
			let at = this._spans.indexOf(cellSpan);
			this._spans.splice(at, 1);
			this._occupiedMap[key] = null;
		}

		for (c = c1; c < c2; ++c) { // Update occupied cell map
			for (r = r1; r < r2; ++r) {
				this._occupiedMap[c + "," + r] = null;
			}
		}
	}

	if (isColumn) {
		cellSpan.prevColSpan = cellSpan.colSpan;
		cellSpan.colSpan = spanVal;
	} else {
		cellSpan.prevRowSpan = cellSpan.rowSpan;
		cellSpan.rowSpan = spanVal;
	}
	cellSpan.retVal = isExpanding ? 1 : -1;
	return cellSpan;
};

/** A list contains all CellSpan definitions
* @private
* @type {!Array.<CellSpan>}
*/
CellSpans.prototype._spans;
/** Map of occupied cells using string as key : _occupiedMap[colIndex + "," +rowIndex]
* @private
* @type {!Object.<string, CellSpan>}
*/
CellSpans.prototype._occupiedMap;

/**
* @private
* @type {boolean}
*/
CellSpans.prototype._frozenMapping = false;

/* harmony default export */ const components_CellSpans = (CellSpans);



;// ./node_modules/@grid/core/es6/grid/util/SelectionList.js

/** @constructor
* @ignore
*/
let SelectionList = function () {
	this._selections = [];
};
/** Clone all members
* @public
* @return {!SelectionList}
*/
SelectionList.prototype.clone = function () {
	// IMPORTANT NOTE: If you add any field or member to this class, clone it over here too.
	let newObj = new SelectionList();
	newObj._selections = this._selections.concat();
	newObj._count = this._count;
	newObj._anchor = this._anchor;
	newObj._firstIndex = this._firstIndex;
	newObj._lastIndex = this._lastIndex;
	return newObj;
};

//#region ======  Public Methods ======//
/** @public
* @return {string}
*/
SelectionList.prototype.toString = function() {
	return this._selections.join(", ");
};
/** If selection count reach its limit, new selection will not be set
* @public
* @param {number} at Negative index is not allowed
* @return {boolean} True new selection is added. False otherwise (i.e. selecting the selected index)
*/
SelectionList.prototype.select = function(at) {
	if(at < 0) {
		return false;
	}

	if(this._select(at)) {
		this._anchor = at;
		if(this._firstIndex < 0 || at < this._firstIndex) {
			this._firstIndex = at;
		}
		if(this._lastIndex < 0 || at > this._lastIndex) {
			this._lastIndex = at;
		}
		return true;
	}
	return false;
};
/** WARNING: Prefer clearAllSelections() to this method
* @public
* @param {number} at
* @return {boolean}
*/
SelectionList.prototype.deselect = function (at) {
	if (this._selections[at]) {
		this._selections[at] = false;
		--this._count;
		if (this._anchor === at) {
			this._anchor = -1; // No anchor
		}

		if (this._firstIndex === at) {
			this._firstIndex = this._findNextSelection(at);
		}
		if (this._lastIndex === at) {
			this._lastIndex = this._findPrevSelection(at);
		}
		return true;
	}
	return false;
};
/** Deselect all selections starting from the specified index
* @public
* @param {number} at
* @return {boolean}
*/
SelectionList.prototype.deselectFrom = function (at) {
	if(this._lastIndex < at) {
		return false;
	}
	if(this._firstIndex >= at) {
		this.clearAllSelections();
		return true;
	}

	let lastIndex = this._lastIndex;
	let sels = this._selections;
	for(let i = at; i <= lastIndex; ++i) {
		if (sels[i]) {
			sels[at] = false;
			--this._count;
		}
	}
	if (this._anchor >= at) {
		this._anchor = -1; // No anchor
	}
	this._lastIndex = this._findPrevSelection(at);
	return true;
};
/** @public
* @param {number} at
*/
SelectionList.prototype.toggleSelection = function(at) {
	if(this._selections[at]) {
		this.deselect(at);
	} else {
		this.select(at);
	}
};
/** Force selection to be left only the selected one, in case of containing multiple selections
* Additionally, move the previous selection to the new one, if one selection has been selected
* Anchor is also moved.
* @public
* @param {number} at Negative index will clear all selections
* @return {boolean} True new selection is added. False otherwise (i.e. selecting the selected index)
*/
SelectionList.prototype.singularlySelect = function (at) {
	if (this._count !== 1 || at < 0) {
		this.clearAllSelections();
		this.select(at);
		return true;
	}
	if (this._firstIndex === at) {
		return false;
	}
	// Move the old selection to the new place
	this._selections[this._anchor] = false;
	this._selections[at] = true;
	this._anchor = at;
	this._firstIndex = at;
	this._lastIndex = at;
	return true;
};

/** Superior performance than calling select multiple times
* Use this method to immitate selectAll() behavior
* Negative length is allowed. It will be inverse selectRange
* @public
* @param {number} anchor Negative value is allowed and will be adjusted to zero
* @param {number} length This will be adjusted according to max selection index
*/
SelectionList.prototype.selectRange = function (anchor, length) {
	if(length === 0) { return; }
	if (anchor < 0) { anchor = 0; }
	if (length === 1 || length === -1) {
		this.select(anchor);
		return;
	}

	let at = anchor;
	let endIndex = at + length;
	if (length < 0) {
		endIndex = at + 1;
		at = endIndex + length;
	}

	if (at >= endIndex) {
		return;
	}
	this._anchor = anchor;

	if (this._firstIndex < 0 || at < this._firstIndex) {
		this._firstIndex = at;
	}
	if (this._lastIndex < 0 || endIndex >= this._lastIndex) {
		this._lastIndex = endIndex - 1;
	}

	for (let i = at; i < endIndex; ++i) {
		this._select(i);
	}
};

/** Superior performance than calling deselect multiple times
* Negative length is allowed. It will be inverse selectRange
* @public
* @param {number} at Negative value is allowed and will be adjusted to zero
* @param {number} length This will be adjusted according to max selection index
*/
SelectionList.prototype.deselectRange = function (at, length) {
	if (this._count <= 0) { return; }
	if (at < 0) { at = 0; }
	if (length === 1 || length === -1) {
		this.deselect(at);
		return;
	}

	let endIndex = at + length;
	if (length < 0) {
		endIndex = at + 1;
		at = endIndex + length;
	}

	if (this._firstIndex >= at && this._lastIndex < endIndex) {
		this.clearAllSelections();
		return;
	}

	if (this._anchor >= at && this._anchor < endIndex) {
		this._anchor = -1;
	}
	if (this._firstIndex >= at && this._firstIndex < endIndex) {
		this._firstIndex = this._findNextSelection(endIndex - 1);
	}
	if (this._lastIndex >= at && this._lastIndex < endIndex) {
		this._lastIndex = this._findPrevSelection(at);
	}

	let i = 0;
	for (i = at; i < endIndex; ++i) {
		if (this._selections[i]) {
			this._selections[i] = false;
			--this._count;
		}
	}
};
/** @public
* @param {number} at
* @param {number} length
* @return {boolean}
*/
SelectionList.prototype.hasSelection = function(at, length) {
	if(this._count <= 0) {
		return false;
	}
	let endIndex = at + length;
	if (length < 0) {
		endIndex = at + 1;
		at = endIndex + length;
	}
	if(endIndex < this._firstIndex || at > this._lastIndex) {
		return false;
	}

	for(let i = at; i < endIndex; ++i) {
		if(this._selections[i]) {
			return true;
		}
	}
	return false;
};

/** @public
 * @param {number} at
* @return {boolean} True, if the specified index is the selected one, otherwise false.
*/
SelectionList.prototype.getSelection = function(at) {
	return (this._selections[at] === true);
};
/** @public
* @param {number} at
* @param {boolean=} opt_selected Select if this is true or undefined
* @return {boolean} True, if there is any change
*/
SelectionList.prototype.setSelection = function(at, opt_selected) {
	return (opt_selected !== false) ? this.select(at) : this.deselect(at);
};
/** Selection anchor is set when a selection is set to true only by select() and toggleSelection()
* @public
* @return {number} Negative index is returned if there is no anchor
*/
SelectionList.prototype.getSelectionAnchor = function() {
	return this._anchor;
};
/** Sometimes anchor doesn't have to be where selection happens, exp shift + select
 * @public
 * @param {number} at
 */
SelectionList.prototype.setSelectionAnchor = function(at) {
	this._anchor = at;
};
/** @public
* @return {number}
*/
SelectionList.prototype.getSelectionCount = function() {
	return this._count;
};

/** This is for optimization purpose, in case of one has to loop through selected index
* @public
* @return {number} Negative index is returned, if there is no selection
*/
SelectionList.prototype.getFirstSelectedIndex = function() {
	return this._firstIndex;
};
/** This is for optimization purpose, in case of one has to loop through selected index
* @public
* @return {number} Negative index is returned, if there is no selection
*/
SelectionList.prototype.getLastSelectedIndex = function() {
	return this._lastIndex;
};

/** WARNING: It will creates a new(EXPENSIVE) defragmented array of selected Index. The selected indices will always be sorted in ascending order
* @public
* @return {!Array.<number>}
*/
SelectionList.prototype.getAllSelections = function() {
	if(this._count > 0) {
		let ary = new Array(this._count); // Fastest way to create an array
		let count = 0;
		let sels = this._selections;
		let lastIdx = this._lastIndex;
		for(let i = this._firstIndex; i <= lastIdx; ++i) {
			if(sels[i]) {
				ary[count++] = i;
			}
		}
		return ary;
	}
	return [];
};
/** Get array of connected selection ranges. For intances, if indices 1, 2, 5, and 5 are selected, array of [1, 2] and [5, 5] are returned.
* @public
* @param {number=} from
* @param {number=} to EXCLUSIVE
* @return {!Array.<Array.<number>>}
*/
SelectionList.prototype.getConnectedRanges = function(from, to) {
	if(this._count > 0) {
		let ary = [];
		if(from == null || from < this._firstIndex) {
			from = this._firstIndex;
		}
		if(to == null || to > this._lastIndex) {
			to = this._lastIndex + 1;
		}

		let pair = null;
		for(let i = from; i < to; ++i) {
			if(this._selections[i]) {
				if(!pair) {
					pair = [i, -1];
				}
			} else if(pair) {
				pair[1] = i - 1;
				ary.push(pair);
				pair = null;
			}
		}

		if(pair) {
			pair[1] = this._lastIndex;
			ary.push(pair);
			pair = null;
		}
		return ary;
	}
	return [];
};

/**
* @public
* @return {Array.<boolean>}
*/
SelectionList.prototype.getSelectionMap = function () {
	return this._selections;
};
/** @public
* @return {number}
*/
SelectionList.prototype.clearAllSelections = function() {
	if(this._count > 0) {
		let count = 0;
		this._selections.length = 0;
		this._count = 0;
		this._anchor = -1;
		this._firstIndex = -1;
		this._lastIndex = -1;
		return count;
	}
	return 0;
};

/** Copy selections from the source selection list for the specified length at the specified index. Note that selection anchor won't be changed
* @public
* @param {SelectionList} srcSelections
* @param {number} fromSrcIndex
* @param {number} offset Offset from the source index to map to destination index of this SelectionList. Use 0 if there is no shifting.
* @param {number} forLength Positive value only. negative valie is not allowed
*/
SelectionList.prototype.copyFrom = function (srcSelections, fromSrcIndex, offset, forLength) {
	if (forLength <= 0) { return; }

	let toThisIndex = fromSrcIndex + offset;
	if (srcSelections == null) {
		this.deselectRange(toThisIndex, forLength);
		return;
	}
	if (srcSelections.hasSelection(fromSrcIndex, forLength) === false) {
		this.deselectRange(toThisIndex, forLength);
		return;
	}


	for (let i = 0; i < forLength; ++i) {
		if (srcSelections._selections[fromSrcIndex + i]) {
			this.select(toThisIndex + i);
		} else {
			this.deselect(toThisIndex + i);
		}
	}
	this._anchor = -1;
	if (srcSelections._anchor >= 0) {
		if (srcSelections._anchor >= fromSrcIndex && srcSelections._anchor < (fromSrcIndex + forLength)) {
			this._anchor = srcSelections._anchor + offset;
		}
	}
};
//#endregion

//#region ====== Private Methods ======//
/** @private
* @param {number} at
* @returns {boolean}
*/
SelectionList.prototype._select = function (at) {
	if(this._selections[at]) { return false; }

	this._selections[at] = true;
	++this._count;

	return true;
};
/** WARNING: Perform LINEAR SEARCH
* @private
* @param {number} exclusiveStart
* @return {number} Negative if not found
*/
SelectionList.prototype._findNextSelection = function(exclusiveStart) {
	if(this._lastIndex >= 0) {
		while(++exclusiveStart <= this._lastIndex) {
			if(this._selections[exclusiveStart]) {
				return exclusiveStart;
			}
		}
	}
	return -1;
};
/** WARNING: Perform LINEAR SEARCH
* @private
* @param {number} exclusiveStart
* @return {number} Negative if not found
*/
SelectionList.prototype._findPrevSelection = function(exclusiveStart) {
	if(this._firstIndex >= 0) {
		while(--exclusiveStart >= this._firstIndex) {
			if(this._selections[exclusiveStart]) {
				return exclusiveStart;
			}
		}
	}
	return -1;
};
//#endregion

//#region ====== Private Fields ======//
/** Non-nullable array of selected index
* @private
* @type {Array.<boolean>}
*/
SelectionList.prototype._selections = null;

/**
* @private
* @type {number}
*/
SelectionList.prototype._count = 0;
/** Anchor is set (>= 0) only when select is successfully processed.
* Anchor is cleared (-1) when the selection at the anchor index is set to false
* @private
* @type {number}
*/
SelectionList.prototype._anchor = -1; // No anchor
/**
* @private
* @type {number}
*/
SelectionList.prototype._firstIndex = -1;
/** Inclusive last selected index
* @private
* @type {number}
*/
SelectionList.prototype._lastIndex = -1;
//#endregion

/* harmony default export */ const util_SelectionList = (SelectionList);


;// ./node_modules/@grid/core/es6/grid/util/Reverter.js

/** @description Reverter is used executed the given method after the specified delay. If Reverter is activated the second time within the delay, the delay time will be extended.
* @constructor
* @param {Function|number=} opt_func
* @param {number|Function=} opt_ms The delay time in millisecond before executing the function
*/
let Reverter = function (opt_func, opt_ms) {
	this._onInterval = this._onInterval.bind(this);

	if(typeof opt_func == "function") {
		this._func = opt_func;
	} else if (typeof opt_ms == "function") { // Parameter can be swapped places
		this._func = opt_ms;
	}

	if(typeof opt_ms == "number") {
		this._delay = opt_ms;
	} else if (typeof opt_func == "number") { // Parameter can be swapped places
		this._delay = opt_func;
	}
};

/** @type {Function}
* @private
*/
Reverter.prototype._func = null;
/** @type {number}
* @private
*/
Reverter.prototype._delay = 0;
/** @type {number}
* @private
*/
Reverter.prototype._timerId = 0;
/** @type {boolean}
* @private
*/
Reverter.prototype._pending = false;


/** @public
*/
Reverter.prototype.start = function () {
	if(this._func) {
		if(!this._timerId) {
			this._timerId = setInterval(this._onInterval, this._delay);
		} else {
			this._pending = true;
		}
	}
};

/** @public
*/
Reverter.prototype.dispose = function () {
	if(this._timerId) {
		clearInterval(this._timerId);
		this._timerId = 0;
	}
	this._func = null; // Release user bound function
};


/** @private
*/
Reverter.prototype._onInterval = function () {
	if(this._pending) {
		this._pending = false;
	} else {
		this._func();
		clearInterval(this._timerId);
		this._timerId = 0;
	}
};

Reverter._proto = Reverter.prototype;

/* harmony default export */ const util_Reverter = (Reverter);


;// ./node_modules/@grid/core/es6/grid/components/Scrollbar.js
/* eslint-disable */




/* eslint-enable */

/** @constructor
* @extends {ElementWrapper}
*/
let Scrollbar = function () {
	let t = this; // Create a shorthand to shorthen file size
	t._addEvents(
		"scroll",
		"activated",
		"deactivated",
		"scrollStart",
		"scrollEnd",
		"thicknessChanged",
		"layoutChanged",
		"enabled",
		"disabled"
	);

	t._onTrackScroll = t._onTrackScroll.bind(t);
	t._onKeyDown = t._onKeyDown.bind(t);
	t._onStartFading = t._onStartFading.bind(t);
	t._onMouseWheel = t._onMouseWheel.bind(t);
	t._flash = t._flash.bind(t);

	t._onTouchStart = t._onTouchStart.bind(t);
	t._onTouchMove = t._onTouchMove.bind(t);
	t._onTouchEnd = t._onTouchEnd.bind(t);
	t._smoothScroll = t._smoothScroll.bind(t);
	t._updateEffectiveArea = t._updateEffectiveArea.bind(t);
	t._onRestingPosition = t._onRestingPosition.bind(t);

	t._element = document.createElement("div");
	t._element.className = "grid-scrollbar";

	t._trackContent = document.createElement("div");

	t._panes = [];
	t._paneSliders = [];
	t._startPos = {};
	t._endPos = {};
	t._reverter = new util_Reverter(t._onRestingPosition, 300); // WORKAROUND: GRID-402

	t._element.addEventListener("scroll", t._onTrackScroll);
	t._element.addEventListener("mouseover", t._flash); // Allow interaction if scrollbar is fading
	t._element.appendChild(t._trackContent); // WARNING: trackContent is not registered as ElementWrapper's content

	t.disableKeyboardInput(false); // Enable keyboard input by default
	t._onThicknessChanged = t._dispatch.bind(t, "thicknessChanged");
	Scrollbar._queryNativeTrackThickness(t._onThicknessChanged);

	if(!t._updateEffectiveArea()) {
		t.listen("thicknessChanged", t._updateEffectiveArea);
	}
};
es6_Ext.inherits(Scrollbar, components_ElementWrapper);

/** @type {boolean}
* @protected
* @ignore
*/
Scrollbar.prototype._vertical = true;
/** @type {number}
* @protected
* @ignore
*/
Scrollbar.prototype._step = 0;
/** @type {Element}
* @protected
* @ignore
*/
Scrollbar.prototype._parentNode = null;
/** @type {string}
* @protected
* @ignore
*/
Scrollbar.prototype._hostClassName = "";

/** @type {Element}
* @protected
* @ignore
*/
Scrollbar.prototype._trackContent = null;
/** @type {Array.<ElementWrapper>}
* @protected
* @ignore
*/
Scrollbar.prototype._panes = null;
/** @type {Array.<ElementWrapper>}
* @protected
* @ignore
*/
Scrollbar.prototype._paneSliders = null;

/** @type {boolean}
* @private
*/
Scrollbar.prototype._enabled = true;
/** @type {boolean}
* @private
*/
Scrollbar.prototype._active = false;
/** if freeze, scroll can't be moved. Event will stopPropagation and preventDefault
* @type {boolean}
* @private
*/
Scrollbar.prototype._isFrozen = false;
/** @type {Scrollbar}
* @protected
* @ignore
*/
Scrollbar.prototype._otherScrollbar = null;
/** @type {boolean}
* @private
*/
Scrollbar.prototype._autoHideScrollbar = true;

/** @type {number}
* @protected
* @ignore
*/
Scrollbar.prototype._trackSize = 120;
/** @type {number}
* @private
*/
Scrollbar.prototype._tContentSize = 0;
/** @type {number}
* @private
*/
Scrollbar.prototype._tScrollVal = 0; // Track and pane may have different size and content
/** @type {number}
* @private
*/
Scrollbar.prototype._tPercentVal = 0; // Percentage [0, 1] of track scroll value

/** @type {number}
* @private
*/
Scrollbar.prototype._paneSize = 0;
/** @type {number}
* @private
*/
Scrollbar.prototype._pContentSize = 0;
/** @type {number}
* @private
*/
Scrollbar.prototype._pScrollVal = 0; // Track and pane may have different size and content

/** @type {number}
* @protected
* @ignore
*/
Scrollbar.prototype._footerCount = 0;

/** @type {number}
* @private
*/
Scrollbar.prototype._fadingId = 0;
/** @type {boolean}
* @private
*/
Scrollbar.prototype._postponeFading = false;

/** @type {boolean}
* @private
*/
Scrollbar.prototype._layoutChanged = false;
/** @type {boolean}
* @private
*/
Scrollbar.prototype._contentDirty = true;
/** @type {boolean}
* @private
*/
Scrollbar.prototype._layoutDirty = true;

/** @type {boolean}
* @private
*/
Scrollbar.prototype._touchInput = true;
/** @type {boolean}
* @private
*/
Scrollbar.prototype._swiping = false;
/** @type {boolean}
* @private
*/
Scrollbar.prototype._touchDrag = false;
/** @type {!Object}
* @private
*/
Scrollbar.prototype._startPos;
/** @type {!Object}
* @private
*/
Scrollbar.prototype._endPos;

/** @type {boolean}
* @private
*/
Scrollbar.prototype._smoothing = true;
/** @type {number}
* @private
*/
Scrollbar.prototype._smoothingId = 0;
/** @type {number}
* @private
*/
Scrollbar.prototype._smoothingSpeed = 0;

/** @type {string}
* @private
*/
Scrollbar.prototype._wheelScrolling = "";
/** @type {Function}
* @private
* @ignore
*/
Scrollbar.prototype._mouseWheelLogic = null;
/** @type {Function}
* @private
* @ignore
*/
Scrollbar.prototype._onThicknessChanged = null;


/** @type {number}
* @private
*/
Scrollbar._trackThickness = 0;
/** @type {number}
* @private
*/
Scrollbar._queryTimeout = 0;
/** @type {boolean}
* @private
*/
Scrollbar._queryStarted = false;
/** @type {Element}
* @private
*/
Scrollbar._outerDummy = null;
/** @type {Element}
* @private
*/
Scrollbar._innerDummy = null;
/** @type {Array.<Function>}
* @private
*/
Scrollbar._listeners = null;



/** @private
* @return {boolean}
*/
Scrollbar.prototype._updateEffectiveArea = function() {
	let thickness = this.getTrackThickness();
	if(thickness) {
		let prop = (this._vertical) ? "width" : "height";
		this._element.style[prop] = (thickness + 1) + "px";
	}
	return thickness ? true : false;
};
/** @private
* @function
* @param {Function=} listener
*/
Scrollbar._queryNativeTrackThickness = function (listener) {
	if(!Scrollbar._queryStarted) { // We only need one query for all scrollbars
		Scrollbar._repeatQuery();
	}

	if(listener && Scrollbar._trackThickness <= 0) {
		let listeners = Scrollbar._listeners;
		if(!listeners) {
			listeners = Scrollbar._listeners = [];
		}
		listeners.push(listener);
	}
};
/** @private
* @function
*/
Scrollbar._repeatQuery = function () {
	Scrollbar._queryStarted = true;
	if(Scrollbar._retrieveNativeTrackThinkness()) {
		return;
	}
	//Re-query if the thickness is not yet available. Add some delay for each query to increase chance of getting the value.

	// TODO: Use window.addEventListener("load", ); in conjunction with this to speed things up
	Scrollbar._queryTimeout += 100;
	if(Scrollbar._queryTimeout < 3000) { // Prevent infinite loop
		setTimeout(Scrollbar._repeatQuery, Scrollbar._queryTimeout);
	} else {
		Scrollbar._trackThickness = 13; // Set to 13px as a last ditch effort
		Scrollbar._clearStaticResources();
	}
};
/** @private
* @function
* @return {boolean} Returns true if valid track thickness
*/
Scrollbar._retrieveNativeTrackThinkness = function () {
	if(Scrollbar._trackThickness > 0) {
		return true;
	}

	let inner = Scrollbar._innerDummy;
	let outer = Scrollbar._outerDummy;
	if(!inner) {
		inner = Scrollbar._innerDummy = document.createElement('p');
		inner.style.width = "100%";
		inner.style.height = "200px";

		outer = Scrollbar._outerDummy = document.createElement('div');
		outer.style.position = "absolute";
		outer.style.top = "0px";
		outer.style.left = "0px";
		outer.style.visibility = "hidden";
		outer.style.width = "200px";
		outer.style.height = "150px";
		outer.appendChild(inner);
	}

	outer.style.overflow = "hidden";
	document.body.appendChild(outer);
	let w1 = inner.offsetWidth;
	outer.style.overflow = 'scroll';
	let w2 = inner.offsetWidth;

	// Sometimes, w1 may be equal to w2 on certain browsers or devices, such as a Macbook when opened on the built-in screen. In such cases, the outer.style.overflow scroll may not change the offsetWidth, and the outer.clientWidth will be the same as the inner offsetWidth. As a result, the native track thickness may not be found
	if(w1 == w2) {
		w2 = outer.clientWidth;
	}
	document.body.removeChild(outer);
	let scrollbarWidth = Scrollbar._trackThickness = (w1 - w2);
	if(scrollbarWidth > 0) {
		Scrollbar._clearStaticResources();
		return true;
	}
	return false;
};
/** @private
* @function
*/
Scrollbar._clearStaticResources = function () {
	Scrollbar._innerDummy = Scrollbar._outerDummy = null;
	let listeners = Scrollbar._listeners;
	if(listeners) {
		Scrollbar._listeners = null;

		let arg = {};
		for(let i = listeners.length; --i >= 0;) {
			listeners[i](arg);
		}
	}
};
/** @public
* @function
*/
Scrollbar.updateTrackThickness = function () {
	Scrollbar._trackThickness = 0;
	Scrollbar._queryTimeout = 0;
	Scrollbar._queryStarted = false;
	Scrollbar._queryNativeTrackThickness();
};


/** @override */
Scrollbar.prototype.dispose = function () {
	let sbListeners = Scrollbar._listeners;
	if(sbListeners) {
		let idx = sbListeners.indexOf(this._onThicknessChanged);
		if(idx >= 0 ) {
			sbListeners.splice(idx, 1);
			this._onThicknessChanged = null;
		}
	}

	this.unlistenAll();
	if(this._smoothingId) {
		clearInterval(this._smoothingId);
		this._smoothingId = 0;
	}

	this._active = this._layoutChanged = false;
	this._dispose();
	for(let i = this._panes.length; --i >= 0;) {
		this._panes[i].dispose();
		this._paneSliders[i].dispose();
	}
	this._reverter.dispose();
};
/** @override */
Scrollbar.prototype.getContent = function (opt_allContent) {
	return this._paneSliders[0].getContent(opt_allContent);
};
/** @override */
Scrollbar.prototype.getParent = function () {
	return this._parentNode;
};
/** @override */
Scrollbar.prototype.setParent = function (p, opt_prepend) {
	let elem = util.getElement(p);

	let isDifferentParent = this._parentNode !== elem;
	if(isDifferentParent && this._parentNode) {
		this.removeWheelListener(this._parentNode);
		this._parentNode.classList.remove(this._hostClassName);
	}

	if(this.isActive()) {
		this._setParent(elem, opt_prepend); // This also reorder scrollbar to the top or bottom (z-order) of the view
		this._parentNode = /** @type{Element} */(this._element.parentNode); // TODO:Change to parentElement
		if(this._parentNode) {
			this._parentNode.classList.add(this._hostClassName);
		}
	} else {
		this._parentNode = elem;
	}
	if(isDifferentParent) {
		this.addWheelListener(this._parentNode);
	}
};
/** @public
* @param {Element|Node} host
*/
Scrollbar.prototype.attachToExternalElement = function (host) {
	this.setParent(host);
	let scrollElem = this.getElement();
	scrollElem.style.right = "0";
	if(this._vertical) {
		scrollElem.style.overflowX = "hidden";
	} else {
		scrollElem.style.overflowY = "hidden";
	}
	scrollElem.style.boxSizing = "border-box";
	scrollElem.style.position = "absolute";
	scrollElem.style.outline = "none";
};

/** @private
*/
Scrollbar.prototype._onActiveStateChange = function () {
	let t = this; // Minimize file size
	t._scrollTo(0, true); // WARNING: Reset scroll position
	let i;
	if(t._active) {
		// t._element.style.display = "";
		if(t._parentNode) {
			t._layoutDirty = true;
			t._setParent(t._parentNode);
			t._parentNode.classList.add(t._hostClassName);
		}
		t._flash(); // WARNING: Auto flashing

		// Activate scrollbar on all panes
		for (i = t._panes.length; --i >= 0; ) {
			t._panes[i].removeClass("scroll-disabled");
		}
		t._dispatch("activated", {});
	} else {
		// t._element.style.display = "none";
		t._setParent(null);
		if(t._parentNode) {
			t._parentNode.classList.remove(t._hostClassName);
		}

		// Remove scrollbar on all panes
		let prop = (t._vertical) ? "height" : "width";
		for (i = t._panes.length; --i >= 0; ) {
			t._panes[i].addClass("scroll-disabled");
			t._panes[i].setStyle(prop, "");
		}
		t._dispatch("deactivated", {});
	}
	if(t._vertical) {
		if(t._otherScrollbar && t._otherScrollbar.isActive()) {
			t._otherScrollbar.invalidate(); // Vertical scrollbar can have an impact on horizontal track size
		}
	}
};
/** @private
*/
Scrollbar.prototype._onEnablingStateChange = function () {
	let t = this; // Minimize file size
	if(t._enabled) {
		t._dispatch("enabled", {});
	} else {
		t._dispatch("disabled", {});
	}
};

/** @public
* @return {boolean}
*/
Scrollbar.prototype.isActive = function () {
	return this._active;
};
/** @public
* @return {boolean}
*/
Scrollbar.prototype.isEnabled = function () {
	return this._enabled;
};
/** @public
* @param {boolean=} opt_enabled
*/
Scrollbar.prototype.enable = function (opt_enabled) {
	opt_enabled = opt_enabled !== false;
	if(this._enabled != opt_enabled) {
		this._enabled = opt_enabled;
		this._updateLayout();
		this._onEnablingStateChange();
	}
};
/** @public
* @param {boolean=} opt_disabled
*/
Scrollbar.prototype.disable = function (opt_disabled) {
	this.enable(opt_disabled === false);
};

/** @public
* @return {number}
*/
Scrollbar.prototype.getTrackSize = function () { return this._trackSize; };
/** This is equivalent to clientHeight or clientWidth (i.e. view port).
* @public
* @ignore
* @return {number} Pane size without zoom factor
*/
Scrollbar.prototype.getPaneSize = function () { return this._paneSize; };
/** This is equivalent to scrollHeight or scrollWidth
* @public
* @return {number} Content size without zoom factor
*/
Scrollbar.prototype.getContentSize = function () { return this._pContentSize; };
/** @public
 * @returns {number}
*/
Scrollbar.prototype.getTrackThickness = function () { return Scrollbar._trackThickness; };

/** Get available space for scrolling which is equivalent to content size subtracted by view size
* @public
* @return {number}
* @see {@link https://developer.mozilla.org/en-US/docs/Web/API/Element/scrollHeight|scrollHeight}
* @see {@link https://developer.mozilla.org/en-US/docs/Web/API/Element/scrollWidth|scrollWidth}
*/
Scrollbar.prototype.getScrollGap = function () {
	return this._pContentSize - this.getPaneSize();
};

/** @public
* @param {number} size
* @param {number} paneSize
*/
Scrollbar.prototype.setTrackSize = function (size, paneSize) {
	this._trackSize = size > 0 ? size : 0;
	this._paneSize = paneSize > 0 ? paneSize : 0;

	// Actually it should invoke _updateLayout() immediately for active-state updating here.
	// But grid always invokes _updateLayout() implicitly after invokes setTrackSize(),
	// so _updateLayout() invoking is unneccessory.
};
/** Must be implemented by the inheritance
* @protected
* @return {number}
*/
Scrollbar.prototype.calculateContentSize = function () { return 0; };

/** @public
* @return {boolean} Return true if there is any change, otherwise false
*/
Scrollbar.prototype.update = function () {
	if(this._contentDirty) { // WARNING: Causes page recalculation
		this._contentDirty = false;
		let size = this.calculateContentSize();
		if(!size || size < 0) {
			size = 0;
		}
		if(this._pContentSize !== size) {
			this._pContentSize = size;
			this._layoutDirty = true;
		}
	}
	if(this._layoutDirty) {
		this._layoutDirty = false;
		this._updateLayout();
		return true;
	}
	return false;
};
/** @public
* @param {boolean=} forceUpdate
* @param {number=} scrollVal
* @ignore
*/
Scrollbar.prototype.invalidate = function (forceUpdate, scrollVal) {
	this._contentDirty = true;
	this._layoutDirty = true;
	if(scrollVal != null) {
		this._pScrollVal = scrollVal;
	}
	if(forceUpdate) {
		this.update();
	}
};
/** @public
* @ignore
*/
Scrollbar.prototype.invalidateContent = function () {
	this._contentDirty = true;
};
/** Validates scrollbar-position against real content-size and pane-size
* @public
* @ignore
* @param {number} contentSize
* @return {boolean}
*/
Scrollbar.prototype.isScrollPosValid = function(contentSize) {
	let scrollGap = contentSize - this.getPaneSize();
	let pScrollVal = this._pScrollVal;
	return (pScrollVal === 0) || ((pScrollVal > 0) &&
		(scrollGap > 0) && (pScrollVal < scrollGap));
};
/** @public
*/
Scrollbar.prototype.flash = function () {
	this.update();
	this._flash();
};

/** @protected
* @ignore
* @param {number} pScrollVal
*/
Scrollbar.prototype._scroll = function (pScrollVal) {
	this._scrollTo(this._pScrollVal + pScrollVal);
};
/** @protected
* @ignore
* @param {number} pScrollVal Scroll value of the pane (not the track)
* @param {boolean=} layoutChanged Force triggering scroll event
*/
Scrollbar.prototype._scrollTo = function (pScrollVal, layoutChanged) {
	if(!this._active && !layoutChanged) {
		return;
	}
	pScrollVal = this._calcProperScrollValue(pScrollVal);

	let dirty = false;
	if(this._pScrollVal !== pScrollVal) {
		this._pScrollVal = pScrollVal; // Fix asynchronous of tracking issue
		dirty = true;
	}

	this._setTrackScrollVal(pScrollVal);

	if(dirty) {
		if(layoutChanged) {
			this._layoutChanged = true;
		}
		this._dispatchScrollEvent(null); // Event is fire synchronously
	} else if(layoutChanged) {
		this._dispatch("layoutChanged", {});
	}
};
/** @private
* @param {number} scrollVal
* @return {number}
*/
Scrollbar.prototype._calcProperScrollValue = function (scrollVal) {
	if(!scrollVal || scrollVal - 0.5 < 0) {
		return 0;
	}
	if(this._tPercentVal >= 1 && Math.abs(this._pScrollVal - scrollVal) <= 2) {
		return this._pScrollVal; // Zooming could cause recalculation errors, so we try to snap the value to the end of the scrollbar
	}

	let scrollGap = this.getScrollGap();
	if(scrollGap <= 0) { // Scrollbar is disabled or inactive
		return 0;
	} else if(scrollVal + 0.5 >= scrollGap) {
		return scrollGap;
	}

	let step = this._step;
	if(step) { // Step scrolling
		scrollVal = Math.round(scrollVal / step) * step;
		if(scrollVal - step / 2 <= 0) {
			return 0;
		} else if(scrollVal + step / 2 >= scrollGap) {
			return scrollGap;
		}
	}
	return Math.round(scrollVal); // Scroll value can only be integers
};
/** @private
* @param {number} tScrollVal Track scroll value
* @return {number}
*/
Scrollbar.prototype._convertTrackToPane = function (tScrollVal) {
	tScrollVal = Math.round(tScrollVal); // tScrollVal may not be an integer due to zooming

	// Track scroll could be changed due to browser's zoom. We need to avoid unnecessary changes
	let trackGap = this._getTrackGap();
	if(this._tPercentVal >= 1) {
		if(Math.abs(this._tScrollVal - tScrollVal) <= 5) {
			tScrollVal = trackGap;
		}
	}

	this._tScrollVal = tScrollVal;
	this._tPercentVal = trackGap ? tScrollVal / trackGap : 0;

	// Convert track value to pane value
	let paneGap = this.getScrollGap();
	let pScrollVal = (trackGap === paneGap) ? tScrollVal : this._tPercentVal * paneGap;
	return pScrollVal;
};
/** @private
* @param {number} pScrollVal Pane scroll value
* @return {boolean} Return true, if there is any change to track value
*/
Scrollbar.prototype._setTrackScrollVal = function(pScrollVal) {
	let trackGap = this._getTrackGap();
	let tScrollVal = 0;
	if(trackGap > 0) {
		if(pScrollVal > 0) {
			let paneGap = this.getScrollGap();
			tScrollVal = (trackGap === paneGap) ? pScrollVal : pScrollVal / paneGap * trackGap;
			tScrollVal = Math.round(tScrollVal); // Avoid calculation error due to fractions from conversion
		}
		if(tScrollVal >= trackGap) {
			tScrollVal = trackGap;
		}
	}

	if(tScrollVal !== this._tScrollVal) {
		this._tScrollVal = tScrollVal;
		this._tPercentVal = trackGap ? tScrollVal / trackGap : 0;

		// Scroll event will be fired asynchronously, if there is any change
		let track = this._element;
		if(this._vertical) {
			track.scrollTop = tScrollVal;
		} else {
			track.scrollLeft = tScrollVal;
		}
		return true;
	}
	return false;
};
/** Get gap between thumb and its track
* @private
* @return {number}
*/
Scrollbar.prototype._getTrackGap = function () {
	let trackGap = this._tContentSize - this.getTrackSize();
	return trackGap > 0 ? trackGap : 0;
};

/** @protected
* @ignore
* @return {number}
*/
Scrollbar.prototype._getScrollValue = function () {
	return this._pScrollVal;
};
/** @protected
* @ignore
* @return {boolean}
*/
Scrollbar.prototype._isEndOfScroll = function () {
	if(!this._active) {
		return true; // TODO: Check if inactive state means end of scrolling
	}

	return this._pScrollVal >= this.getScrollGap();
};
/** @protected
* @ignore
* @return {boolean}
*/
Scrollbar.prototype._isStartOfScroll = function() {
	return !this._pScrollVal;
};
/** @protected
* @ignore
* @param {number} diff
* @return {boolean}
*/
Scrollbar.prototype._isOnEdge = function (diff) {
	return (diff < 0 && this._isStartOfScroll()) || (diff > 0 && this._isEndOfScroll());
};
/** @protected
* @ignore
*/
Scrollbar.prototype._flash = function () {
	if(!this._active || !this._autoHideScrollbar) { return; }

	if(this._fadingId) {
		this._postponeFading = true;
	} else {
		if(this._element._fading) { // This can improve performance a lot
			this._element.classList.remove("grid-scroll-fadeout");
			this._element._fading = false;
		}
		this._fadingId = setTimeout(this._onStartFading, 1000);
	}
};

/** @protected
* @ignore
*/
Scrollbar.prototype._clearAllPanes = function() {
	for (let i = this._panes.length; --i >= 0; ) {
		this._panes[i].setParent(null);
		let paneSlider = this._paneSliders[i];
		paneSlider._content = paneSlider._columns = null;
		let elem = paneSlider.getElement();
		while (elem.firstChild !== null) {
			elem.removeChild(elem.firstChild);
		}
	}
};

/** @public
* @param {boolean=} opt_disabled
*/
Scrollbar.prototype.disableKeyboardInput = function (opt_disabled) {
	if(opt_disabled === false) {
		this._element.setAttribute("tabindex", "-1"); // tabindex makes the element focusable. The negative value exclude it from tab key navigation
		this._element.addEventListener("keydown", this._onKeyDown, false);
	} else {
		this._element.removeAttribute("tabindex");
		this._element.removeEventListener("keydown", this._onKeyDown, false);
	}
};
/** @public
* @param {boolean=} opt_disabled
*/
Scrollbar.prototype.disableTouchInput = function (opt_disabled) {
	this._touchInput = (opt_disabled === false);
};

/** Set scrollbar autohide status If set to false, scrollbar will always be shown and no fading
* @public
* @param {boolean} hide true if autohide, false if not
*/
Scrollbar.prototype.setAutoHide = function (hide) {
	this._autoHideScrollbar = hide;
	if(!hide) { // Always show scrollbar
		if(this._element._fading) {
			this._element.classList.remove("grid-scroll-fadeout");
			this._element._fading = false;
		}
		if(this._fadingId) {
			clearTimeout(this._fadingId);
			this._fadingId = 0;
		}
	}
};

/** Get scrollbar autohide status
* @public
* @return {boolean} true if autohide, false if not
*/
Scrollbar.prototype.getAutoHide = function () {
	return this._autoHideScrollbar;
};

/** Set reference to other scrollbar. For internal use only
* @public
* @ignore
* @param {Scrollbar} scrollbar Other scrollbar object
*/
Scrollbar.prototype.setOtherScrollbar = function(scrollbar){
	this._otherScrollbar = scrollbar;
};

/** Set number of expected frozen footer count. For internal use only.
* @public
* @ignore
* @param {number} num
*/
Scrollbar.prototype.setFooterCount = function(num){
	this._footerCount = num;
};

/** @private
* @return {boolean}
*/
Scrollbar.prototype._calcActiveState = function () {
	if(this._enabled) {
		if(this.getTrackSize() > 20) {
			let paneSize = this.getPaneSize();
			if(paneSize > 0) {
				return (this._pContentSize - paneSize > 0);
			}
		}
	}
	return false;
};
/** @private
*/
Scrollbar.prototype._updateLayout = function () {
	let active = this._calcActiveState();
	if(active !== this._active) {
		this._active = active;
		this._onActiveStateChange();
	}
	if(!this._active) {
		return;
	}

	let trackSize = this.getTrackSize();
	let paneSize = this.getPaneSize();
	this._tContentSize = this._pContentSize / paneSize * trackSize;
	if (this._vertical) {
		this._trackContent.style.height = this._tContentSize + "px";

		this._element.style.height = trackSize + "px";
		this._panes[0].setHeight(paneSize);
	} else {
		this._trackContent.style.width = this._tContentSize + "px";

		this._element.style.width = trackSize + "px";
		for (let i = this._panes.length; --i >= 0;) {
			this._panes[i].setWidth(paneSize);
		}
	}

	// Since track could be smaller, scrollVal needs to be updated. Also UI needs to be always updated too
	this._scrollTo(this._pScrollVal, true); // Scroll event may be fired
};

/** @public
* @ignore
*/
Scrollbar.prototype.resetPaneSize = function () {
	let prop = (this._vertical) ? "height" : "width";
	for (let i = this._panes.length; --i >= 0;) {
		this._panes[i].setStyle(prop, "");
	}
};

/** @private
* @param {MouseEvent} e
*/
Scrollbar._preventVScrolling = function(e) {
	e.currentTarget.scrollTop = 0;
};
/** @private
* @param {MouseEvent} e
*/
Scrollbar._preventHScrolling = function(e) {
	e.currentTarget.scrollLeft = 0;
};

/** @protected
* @ignore
* @param {number=} opt_at
* @returns {ElementWrapper}
*/
Scrollbar.prototype._addPaneAt = function (opt_at) {
	if(opt_at == null) {
		opt_at = this._panes.length;
	}

	let pane = this._panes[opt_at];
	if(!pane) {
		pane = new components_ElementWrapper(document.createElement("div"));
		pane.setClass("grid-pane");
		if(this._vertical) {
			pane.addClass("sections"); // The pane will host multiple sections
			pane.listen("scroll", Scrollbar._preventVScrolling);
		} else {
			pane.addClass("columns"); // The pane will host multiple columns
			pane.listen("scroll", Scrollbar._preventHScrolling);
		}
		if(!this._active) {
			pane.addClass("scroll-disabled");
		}
		if(this._paneSize && this._active) {
			let paneSize = this.getPaneSize();
			if(this._vertical) {
				pane.setHeight(paneSize);
			} else {
				pane.setWidth(paneSize);
			}
		}
		pane.listen("touchstart", this._onTouchStart);
		pane.listen("touchmove", this._onTouchMove);
		pane.listen("touchend", this._onTouchEnd);
		pane.listen("touchcancel", this._onTouchEnd);

		let paneSlider = new components_ElementWrapper(document.createElement("div"));
		paneSlider.setClass("grid-pane-slider");
		paneSlider.setParent(pane);

		this._panes[opt_at] = pane;
		this._paneSliders[opt_at] = paneSlider;
	}
	return pane;
};
/** @private
* @param {!Event} e
*/
Scrollbar.prototype._onStartFading = function (e) {
	this._fadingId = 0;
	if(!this._active) { return; }
	if(this._postponeFading) {
		this._postponeFading = false;
		this._flash();
	} else {
		if(!this._element._fading) {
			this._element.classList.add("grid-scroll-fadeout");
			this._element._fading = true;
		}
	}
};
/** Scrolling from the UI
* @private
* @param {Object} e
*/
Scrollbar.prototype._onTrackScroll = function (e) {
	if(!this._element) {
		return; // Element has been disposed
	}

	let tScrollVal = (this._vertical) ? e.target["scrollTop"] : e.target["scrollLeft"];
	let pScrollVal = this._convertTrackToPane(tScrollVal);

	pScrollVal = this._calcProperScrollValue(pScrollVal);

	if(pScrollVal !== this._pScrollVal) { // Prevent duplicate event firing
		this._pScrollVal = pScrollVal;
		this._dispatchScrollEvent(e);
	}
};

/** @private
*/
Scrollbar.prototype._onRestingPosition = function () {
	this._setSliderPosition(-this._pScrollVal, true);
};
/** @private
* @param {number} scrollVal
* @param {boolean=} resting
*/
Scrollbar.prototype._setSliderPosition = function (scrollVal, resting) {
	let translateStr = "";
	let relativeStr = "";
	let direction = this._vertical ? "top" : "left";
	if(scrollVal) {
		if(resting) {
			relativeStr = scrollVal + "px";
		} else {
			translateStr = this._vertical ? "translateY" : "translateX";
			translateStr += "(" + scrollVal + "px)";
			this._reverter.start();
		}
	}
	for (let i = this._paneSliders.length; --i >= 0; ) {
		let paneSlider = this._paneSliders[i];
		let paneSliderStyle = paneSlider.getElement().style;
		if(paneSlider._titleSection) {
			paneSliderStyle[direction] = scrollVal + "px";
		} else {
			paneSliderStyle.transform = translateStr;
			paneSliderStyle[direction] = relativeStr; // WORKAROUND: GRID-402
		}
	}
	if(resting) {
		this._dispatch("scrollEnd", {});
	} else {
		this._dispatch("scrollStart", {});
	}
};
/** @private
* @param {Object=} e
*/
Scrollbar.prototype._dispatchScrollEvent = function (e) {
	this._setSliderPosition(-this._pScrollVal, false);

	this._flash(); // Slider has been updated. Flash the track's UI

	// Fire scroll event synchronously
	if(!this._isEventDispatching("scroll")) { // VScrollbar may change scroll value
		if(!e) {
			let target = this._element;
			e = {
				"currentTarget": target,
				"srcElement": target,
				"target": target
			};
		}
		e["layoutChanged"] = this._layoutChanged;	// Do not close the editor - the scroll is due to a layout change - this a not an actual scroll - see QuoteList.js (monitor-grid)
		if(this._vertical) {
			e["scrollTop"] = this._pScrollVal; // For backward compatibility
		} else {
			e["scrollLeft"] = this._pScrollVal;
		}

		this._dispatch("scroll", e);
	}
};
/** @private
* @param {!Event} e
*/
Scrollbar.prototype._onKeyDown = function (e) {
	if (this._isFrozen) {
		util._preventDefault(e);
		return;
	}
	if (!this._active) { return; }

	let isArrow = false;
	if(this._vertical) {
		isArrow = (e.keyCode === 38 || e.keyCode === 40);
	} else {
		isArrow = (e.keyCode === 37 || e.keyCode === 39);
	}

	if(isArrow) {
		let scrollGap = this.getScrollGap();
		let scrollVal = Math.round(scrollGap * 0.07);
		if(scrollVal < 10) {
			scrollVal = 10;
		} else if(scrollVal > 100) {
			scrollVal = 100;
		}
		if(e.keyCode === 37 || e.keyCode === 38) { // Left or Up
			scrollVal *= -1;
		}
		this._scroll(scrollVal);
		util._preventDefault(e);
	}
};

/** @private
* @param {Event} e
*/
Scrollbar.prototype._onTouchStart = function (e) {
	if (this._isFrozen) {
		util._preventDefault(e);
		return;
	}
	if(this._swiping || !this._touchInput) {
		return;
	}
	if(this._vertical) { // Vertical scrollbar is optimized, so that it will not be activated on row change event
		this.update();
	}
	this._flash();

	if(this._smoothingId) {
		clearInterval(this._smoothingId);
		this._smoothingId = 0;
	}

	this._swiping = true;
	this._getTouchInfo(e, this._startPos);
};
/** @private
* @param {Event} e
*/
Scrollbar.prototype._onTouchMove = function (e) {
	let t = this;

	if (t._isFrozen) {
		util._preventDefault(e);
		return;
	}
	// Stop scrolling when scrollbar becomes inactive
	if(!t._swiping || !t._active || !t._touchInput) {
		return;
	}

	if(e.touches && e.touches.length > 1) {
		return;
	}

	let sp = t._startPos;
	let ep = t._endPos;
	t._getTouchInfo(e, ep);
	let diff = (this._vertical) ? sp["y"] - ep["y"] : sp["x"] - ep["x"];

	if(diff > -5 && diff < 5) {
		return;
	} else if(t._isOnEdge(diff)) {
		return;
	}

	if(t._touchDrag) {
		t._scrollTo(sp["scrollValue"] + diff);
	} else {
		let duration = ep["timeStamp"] - sp["timeStamp"];
		if(duration > 30) {
			t._touchDrag = true;
		}
	}

	if(e.cancelable) {
		e.preventDefault();
	}
};
/** @private
* @param {Event} e
*/
Scrollbar.prototype._onTouchEnd = function (e) {
	if (this._isFrozen) {
		util._preventDefault(e);
		return;
	}
	this._swiping = false;
	if(this._touchDrag) {
		this._touchDrag = false;

		if(e.cancelable) {
			e.preventDefault();
		}

		if(this._smoothing && !this._smoothingId) {
			let sp = this._startPos;
			let ep = this._getTouchInfo(e, this._endPos);
			let duration = ep["timeStamp"] - sp["timeStamp"];
			let diff = (this._vertical) ? (sp["y"] - ep["y"]) : (sp["x"] - ep["x"]);
			this._smoothingSpeed = diff * 30 / duration; // pixel per milliseconds
			if(this._smoothingSpeed > 10 || this._smoothingSpeed < -10) {
				this._smoothingId = setInterval(this._smoothScroll, 25);
			}
		}
	}
};
/** @private
* @param {Event} e
* @param {Object=} obj
* @return {!Object}
*/
Scrollbar.prototype._getTouchInfo = function(e, obj) {
	let touch = e.changedTouches[0];
	if(!obj) {
		obj = {};
	}
	obj["x"] = touch.pageX;
	obj["y"] = touch.pageY;
	obj["timeStamp"] = e.timeStamp;
	obj["scrollValue"] = this._getScrollValue();
	return obj;
};
/** @private
*/
Scrollbar.prototype._smoothScroll = function() {
	let ss = this._smoothingSpeed;
	ss = (ss * 0.9) | 0; // OR operator is not equivalent to Math.floor()
	this._smoothingSpeed = ss;
	if(ss > 1 || ss < -1) {
		this._scroll(ss);
	} else {
		clearInterval(this._smoothingId);
		this._smoothingId = 0;
	}
};

/** @public
* @param {boolean=} opt_disabled
*/
Scrollbar.prototype.disableMouseWheel = function (opt_disabled) {
	if(opt_disabled === false) {
		this.setMouseWheelLogic(this._mouseWheelLogic || this._onMouseWheel);
	} else {
		this.setMouseWheelLogic(null);
	}
};
/** @public
* @ignore
* @param {Function} handler
*/
Scrollbar.prototype.setMouseWheelLogic = function (handler) {
	this.removeWheelListener(this._parentNode);

	this._mouseWheelLogic = handler;

	if(this._mouseWheelLogic) {
		// Wheel listener should be added to parentNode (utilizing event bubbling)
		this.addWheelListener(this._parentNode);
	}
};
/** @public
* @param {Element} elem Element that will trigger scrolling by mouse wheel
*/
Scrollbar.prototype.addWheelListener = function (elem) {
	if(elem && this._mouseWheelLogic) {
		// Note that this wheel event can not use passive options ({passive: false}) because of the need to prevent default behavior
		elem.addEventListener("wheel", this._mouseWheelLogic, false);
	}
};
/** @public
* @param {Element} elem Element that will trigger scrolling by mouse wheel
*/
Scrollbar.prototype.removeWheelListener = function (elem) {
	if(elem && this._mouseWheelLogic) {
		elem.removeEventListener("wheel", this._mouseWheelLogic, false);
	}
};
/** @public
* @param {string} val
*/
Scrollbar.prototype.setMouseWheelSpeed = function (val) {
	this._wheelScrolling = typeof val == "string" ? val : "";
};
/** @public
* @ignore
* @return {string}
*/
Scrollbar.prototype.getMouseWheelSpeed = function () {
	return this._wheelScrolling;
};
/** @public
* @return {function(Event)} Mouse wheel logic that is attached to this instance
*/
Scrollbar.prototype.getDefaultMouseWheelLogic = function () {
	return this._onMouseWheel;
};
/** TODO: Support mouse wheel (currently works only touchpad wheel)
* @private
* @param {Event} e
*/
Scrollbar.prototype._onMouseWheel = function (e) {

	// Blacklist for prevent triggering the scrollbar when the dialog is opened.
	if(e.target.opened) { // get attribute method doesn't work in elf element
		return;
	}

	if (this._isFrozen) {
		util._preventDefault(e);
		return;
	}
	if(e.ctrlKey || e.altKey ) { return; }
	if (!this.isActive()) { return; }

	let delta = (this._vertical) ? e["deltaY"] : e["deltaX"];

	if(!delta) { return; }

	if(this._isOnEdge(delta)) { return; }

	// We must prevent on a passive event such as "wheel" to prevent default scrolling of the window
	util._preventDefault(e);

	// Only Firefox implements deltaMode. All other browsers use DOM_DELTA_PIXEL in most of the cases
	// See https://github.com/w3c/uievents/issues/181 and https://github.com/svgdotjs/svg.panzoom.js/issues/67
	// Hence, we need to normalize the delta value, so every browser have similar mousewheel behaviors.
	if(e.deltaMode) { // https://developer.mozilla.org/en-US/docs/Web/API/WheelEvent/deltaMode
		delta = (delta > 0) ? 100 : -100; // WARNING: We use estimated value, since there is no simple way to calculate actual pixel value
	}

	if(this._wheelScrolling != "linear") {
		let scrollSize = this.getContentSize();
		if(scrollSize >= 4000) { // Scaling for large content
			delta *= 1 + scrollSize / 4000 | 0; // If scrollSize == 4000, delta *= 2. If scrollSize == 8000, delta *= 3
		}
	}

	this.update();
	this._scroll(delta);
};
/** @public
* @param {number} px
*/
Scrollbar.prototype.setScrollingStep = function (px) {
	this._step = (px > 0) ? px : 0;
};

/** prevent all scrollbar action using stopPropagation and preventDefault
* @public
* @param {boolean=} frozen=true
* @return {boolean} return previous frozen status
*/
Scrollbar.prototype.freezeScrolling = function (frozen) {
	let prev = this._isFrozen;
	this._isFrozen = frozen !== false;
	return prev;
};
/** @public
*/
Scrollbar.prototype.restoreTrackPosition = function () {
	// Scroll event will be fired asynchronously, if there is any change
	let track = this._element;
	if(this._vertical) {
		track.scrollTop = this._tScrollVal;
		if(track.scrollTop){
			this._tScrollVal = track.scrollTop; //Check to ensure that cache equal to element's scroll
		}
	}
	//TODO: handle restore scrollLeft for hscroll
};

Scrollbar._proto = Scrollbar.prototype;

/* harmony default export */ const components_Scrollbar = (Scrollbar);


;// ./node_modules/@grid/core/es6/grid/components/HScrollbar.js
/* eslint-disable */



/* eslint-enable */

/** @constructor
* @extends {Scrollbar}
*/
let HScrollbar = function () {
	let t = this; // Create a shorthand to shorthen file size
	t._vertical = false;
	t._onDeactivated = t._onDeactivated.bind(t);
	t._onActivated = t._onActivated.bind(t);
	t._hostClassName = "hscrollbar-host";
	HScrollbar.base(this, 'constructor'); // Call base constructor

	// Styling
	t._element.classList.add("grid-hscroll");
	t._trackContent.style.height = "1px";

	t.disableMouseWheel(false); // Mousewheel for horizontal scrollbar is enabled by default

	t.listen("activated", t._onActivated);
	t.listen("deactivated", t._onDeactivated);
};
es6_Ext.inherits(HScrollbar, components_Scrollbar);

/** @type {number}
* @private
*/
HScrollbar.prototype._pinnedLeftColumnCount = 0;
/** @type {number}
* @private
*/
HScrollbar.prototype._pinnedRightColumnCount = 0;

/** For internal use only
* @public
* @ignore
* @param {ElementWrapper} grid Grid object is not used in this case
* @param {!Array.<ElementWrapper>} sections
* @param {number} startColIndex
* @param {number=} pinnedRightColumn Number of pinned columns on the right side
*/
HScrollbar.prototype.setScrollContent = function (grid, sections, startColIndex, pinnedRightColumn) {
	this._clearAllPanes();

	let sectionCount = sections.length;
	if(sectionCount <= 0) { return; }

	this._pinnedLeftColumnCount = startColIndex >= 0 ? startColIndex : 0;
	this._pinnedRightColumnCount = pinnedRightColumn > 0 ? pinnedRightColumn : 0;
	let section = sections[0];
	let colCount = section.getColumnCount();
	let endColIndex = colCount - pinnedRightColumn; // WARNING: If number of pinned column is greater than the scrollable range, the pinning will not be applied
	if(!endColIndex || endColIndex < 0) {
		endColIndex = colCount;
	}

	// Insert panes in the correct slot
	for (let s = 0; s < sectionCount; ++s) {
		section = sections[s];

		let pane = this._addPaneAt(s);
		let paneSlider = this._paneSliders[s];
		let paneElem = pane.getElement();
		let sliderElem = paneSlider.getElement();

		let c;
		let column;
		for (c = 0; c < startColIndex; ++c) {
			column = /** @type{Column} */(section.getColumn(c));
			column.setScrollState(paneElem, null, false);
		}
		let scrollHost = section.getColumnHost();
		pane.setParent(scrollHost);
		let columns = [];
		let content = [];
		for (c = startColIndex; c < endColIndex; ++c) {
			column = /** @type{Column} */(section.getColumn(c));
			column.setScrollState(null, sliderElem, this.isActive());

			columns.push(column);
			content.push(column.getElement());
		}
		for (c = endColIndex; c < colCount; ++c) {
			column = /** @type{Column} */(section.getColumn(c));
			column.setScrollState(null, null, false);
		}

		let rs = section.getReservedSpace();
		if(rs) {
			if(this.isActive() && !this._pinnedRightColumnCount) {
				paneSlider.addContent(rs);
				content.push(rs);
			} else {
				scrollHost.appendChild(rs);
			}
			section._updateRightSpaceStyle();
		}
		section._updateLayers(); // Order of the layers may be changed due to change in rightspace and/or column pinning
		paneSlider._titleSection = section.hasClass("title"); // To cache virtualized title section
		paneSlider._columns = columns; // To cache virtualized columns
		paneSlider._content = content; // To cache virtualized columns
	}
	this.invalidateContent(); // Delay content update
};

/** @override
*/
HScrollbar.prototype.setWidth = function (val) {
	this.setTrackSize(val, val);
	return true;
};
/** @override
*/
HScrollbar.prototype.getWidth = function () {
	return this.getPaneSize(); // Track size may not represent what the client want
};

/** @override */
HScrollbar.prototype.getScrollLeft = function () {
	return this._getScrollValue();
};
/** @public
* @param {number} px
*/
HScrollbar.prototype.setScrollLeft = function (px) {
	this.update();
	this._scrollTo(px); // "scroll" event may be fired
};

/** @public
* @return {number}
*/
HScrollbar.prototype.getPinnedLeftColumnCount = function () {
	return this.isEnabled() ? this._pinnedLeftColumnCount : 0;
};
/** @public
* @return {number}
*/
HScrollbar.prototype.getPinnedRightColumnCount = function () {
	return this.isEnabled() ? this._pinnedRightColumnCount : 0;
};

/** @public
* @ignore
* @param {number} val
*/
HScrollbar.prototype.setPaneLeft = function (val) {
	let px = val && this.isActive() ? (-val + "px") : "";
	for (let i = this._panes.length; --i >= 0; ) {
		this._paneSliders[i].setStyle("marginLeft", px);
	}
};

/** @public
* @override
* @return {boolean}
*/
HScrollbar.prototype.isEndOfHorizontalScroll = function () {
	return this._isEndOfScroll();
};
/** @public
* @return {number}
*/
HScrollbar.prototype.getContentWidth = function () {
	let paneSlider = this._paneSliders[0]; // Get content width from the first pane slider
	let sum = 0;
	if (paneSlider) {
		let colList = /** @type{Array} */(paneSlider._content);
		for (let i = colList.length; --i >= 0; ) {
			let elem = colList[i];
			let width = elem ? elem.style.width : 0; // HACK
			if(width) {
				sum += parseFloat(width); // TODO: parseFloat is not fast
			}
		}
	}
	return sum;
};
/** @override
* @protected
* @return {number}
*/
HScrollbar.prototype.calculateContentSize = function () {
	return this.getContentWidth();
};

/** Horizontal scrollbar will be shrunk if vertical scrollbar exists at the same time
* @override
* @public
* @return {number}
*/
HScrollbar.prototype.getTrackSize = function () {
	// Horizontal scrollbar may be overlap with vertical scrollbar
	if(this._footerCount <= 0) { // Horizontal scrollbar is NOT on top of a footer section and may be overlap
		if(this._otherScrollbar) { // The other scrollbar exists
			if(this._otherScrollbar.isActive() && !this._pinnedRightColumnCount) { // The other scrollbar is active and hence it is overlap with this scrollbar
				return this._trackSize - this.getTrackThickness();
			}
		}
	}
	return this._trackSize;
};

/** @private
* @param {Object} e
*/
HScrollbar.prototype._onActivated = function (e) {
	this._updateActivation(true);
};
/** @private
* @param {Object} e
*/
HScrollbar.prototype._onDeactivated = function (e) {
	this._updateActivation(false);
};
/** @private
* @param {boolean} bool
*/
HScrollbar.prototype._updateActivation = function (bool) {
	let sectCount = this._paneSliders.length;
	for(let s = 0; s < sectCount; ++s) {
		let paneSlider = this._paneSliders[s];
		let columns = paneSlider._columns;
		let colCount = columns ? columns.length : 0;
		for(let c = 0; c < colCount; ++c) {
			let column = columns[c];
			if(column.getElement()) { // Column may have been disposed
				column.enableAbsolutePosition(bool);
			}
		}
	}
};

HScrollbar._proto = HScrollbar.prototype;

/* harmony default export */ const components_HScrollbar = (HScrollbar);


;// ./node_modules/@grid/core/es6/grid/util/CellBoundPainter.js
/** @private
* @function
* @param {number} idx
* @param {number} limit
* @return {number}
*/
let _validateIndex = function(idx, limit) {
	if(idx > limit) {
		return limit;
	} else if(idx < 0) {
		return 0;
	}
	return idx;
};

/** @constructor
* @param {Object} ctx
*/
let CellBoundPainter = function (ctx) {
	this._boundLayer = ctx.boundLayer;
	this._layoutX = ctx.layoutX;
	this._layoutY = ctx.layoutY;
	this._hscrollbar = ctx.hscrollbar;
	this._calculateColumnBounds = ctx.calculateColumnBounds;
};

/** @type {Element}
* @private
*/
CellBoundPainter.prototype._boundLayer = null;
/** @type {!TrackLayout}
* @private
*/
CellBoundPainter.prototype._layoutX;
/** @type {!TrackLayout}
* @private
*/
CellBoundPainter.prototype._layoutY;
/** @type {HScrollbar}
* @private
*/
CellBoundPainter.prototype._hscrollbar;
/** @type {Function}
* @private
*/
CellBoundPainter.prototype._calculateColumnBounds;
/** @type {Element}
* @private
*/
CellBoundPainter.prototype._cellBound = null;
/** @type {number}
* @private
*/
CellBoundPainter.prototype._cbLftIdx = 0;
/** @type {number}
* @private
*/
CellBoundPainter.prototype._cbRgtIdx = 0;
/** @type {number}
* @private
*/
CellBoundPainter.prototype._cbTopIdx = 0;
/** @type {number}
* @private
*/
CellBoundPainter.prototype._cbBtmIdx = 0;

/** @public
*/
CellBoundPainter.prototype.dispose = function () {
	let cellBound = this._cellBound;
	if(cellBound) {
		let pn = cellBound.parentNode;
		if(pn) {
			pn.removeChild(cellBound);
		}
		this._cellBound = null;
	}
	this._boundLayer = null;
	this._layoutX = null;
	this._layoutY = null;
	this._hscrollbar = null;
	this._calculateColumnBounds = null;
};

/** @public
* @param {number} colIndex
* @param {number} rowIndex
* @param {number} width
* @param {number} height
* @param {number} colCount
*/
CellBoundPainter.prototype.setCellBounds = function (colIndex, rowIndex, width, height, colCount) {
	let cellBound = this._cellBound;
	if(!cellBound) {
		cellBound = this._cellBound = document.createElement("div");
		cellBound.className = "selection-bound";
	}

	// Validate inputs
	let rgtIndex = colIndex + width; // Exclusive
	let btmIndex = rowIndex + height; // Exclusive

	let rowCount = this._layoutY.getLaneCount();

	this._cbLftIdx = _validateIndex(colIndex, colCount);
	this._cbRgtIdx = _validateIndex(rgtIndex, colCount);
	this._cbTopIdx = _validateIndex(rowIndex, rowCount);
	this._cbBtmIdx = _validateIndex(btmIndex, rowCount);

	this.updateCellBounds();
};

/** @public
*/
CellBoundPainter.prototype.updateCellBounds = function() {
	let cellBound = this._cellBound;
	if(!cellBound) {
		return;
	}

	let layoutX = this._layoutX;
	let layoutY = this._layoutY;
	let lftIdx = this._cbLftIdx;
	let rgtIdx = this._cbRgtIdx; // Exclusive
	let topIdx = this._cbTopIdx;
	let btmIdx = this._cbBtmIdx; // Exclusive
	let lftPx, rgtPx, topPx, btmPx;
	lftPx = rgtPx = topPx = btmPx = 0;
	if(lftIdx < rgtIdx && topIdx < btmIdx) {
		lftPx = layoutX.getLaneStart(lftIdx);
		rgtPx = layoutX.getLaneEnd(rgtIdx - 1);
		topPx = layoutY.getLaneStart(topIdx);
		btmPx = layoutY.getLaneEnd(btmIdx - 1);
	}

	let width = rgtPx - lftPx;
	let height = btmPx - topPx;
	let noBorders = [false, false];
	if(width > 0 && height > 0 && this._hscrollbar) {
		let positions = [0, 0];
		this._calculateColumnBounds(lftIdx, rgtIdx - 1, positions, noBorders);
		lftPx = positions[0];
		rgtPx = positions[1];
		width = rgtPx - lftPx;
	}

	if(width > 0) {
		cellBound.style.left = lftPx + "px";
		cellBound.style.top = topPx + "px";
		cellBound.style.width = width + "px";
		cellBound.style.height = height + "px";

		cellBound.classList.toggle("no-left-bound", noBorders[0]);
		cellBound.classList.toggle("no-right-bound", noBorders[1]);

		if(this._boundLayer) {
			this._boundLayer.appendChild(cellBound);
		}
	} else {
		let pn = cellBound.parentNode;
		if(pn) {
			pn.removeChild(cellBound);
		}
	}
};

CellBoundPainter._proto = CellBoundPainter.prototype;

/* harmony default export */ const util_CellBoundPainter = (CellBoundPainter);


;// ./node_modules/@grid/core/es6/grid/LayoutGrid.js
/* eslint-disable */














/* eslint-enable */

//#region Events
/**
 * @event LayoutGrid#rowHighlighted
 */
/**
 * @event LayoutGrid#rowCountChanged
 */
/**
 * @event LayoutGrid#rowHeightChanged
 */
/**
 * @event LayoutGrid#rowAvailable
 * @ignore
 */
//#endregion Events

/**
 * Please see {@link ILayoutGrid} for full APIs and parameter lists.
 * @constructor
 * @param {Object=} options
 * @extends {ElementWrapper}
 */
let LayoutGrid = function (options) {
	let _t = this;

	_t._onMouseMove = _t._onMouseMove.bind(_t);
	_t._onMouseOut = _t._onMouseOut.bind(_t);
	_t._onResetTransformIE = _t._onResetTransformIE.bind(_t);

	_t._trackX = new util_TrackLayout();
	_t._trackY = new util_TrackLayout();
	_t._trackX.setDefaultLaneSize(100);
	_t._trackY.setDefaultLaneSize(32);

	_t._cellSpans = new components_CellSpans();

	_t._element = document.createElement("div");
	_t._element.className = "tr-lg tr-even-shading";

	_t._ctxRows = new components_VirtualItems(); // Context rows act as a virtual column storing UI context or metadata for rows
	_t._columns = [];
	_t._cells = [];
	_t._highlightedCells = [];
	_t._stretchedCells = new components_StretchedCells(_t._trackX, _t._trackY, _t);
	_t._selectionList = new util_SelectionList();

	if (options) {
		_t._virOwner = options["owner"] || null;
		if(options["horizontalScrollbar"]) {
			_t._hscrollbar = options["horizontalScrollbar"];
		}
	}
	if(!_t._virOwner) {
		_t._element.classList.add("section");
	}

	_t._addEvent("rowHighlighted");
	_t._addEvent("rowCountChanged");
	_t._addEvent("rowHeightChanged");
	_t._addEvent("rowAvailable");
};

es6_Ext.inherits(LayoutGrid, components_ElementWrapper);

/**
 * @type {number}
 * @private
 */
LayoutGrid.prototype._index = -1;

/**
 * For virtualization in Y-Axis
 * @type {number}
 * @private
 */
LayoutGrid.prototype._availableRowCount = 0;

/**
 * For virtualization in Y-Axis
 * @type {number}
 * @private
 */
LayoutGrid.prototype._activeRow = 0;

/**
 * For virtualization in Y-Axis
 * @type {number}
 * @private
 */
LayoutGrid.prototype._activeRowEnd = 0;

/**
 * Cache of column count.
 * @type {number}
 * @private
 */
LayoutGrid.prototype._colCount = 0;

/** Cache of row count
 * @type {number}
 * @private
 */
LayoutGrid.prototype._rowCount = 0;

/**
 * Cache of hidden row count
 * @type {number}
 * @private
 */
LayoutGrid.prototype._hiddenRowCount = 0;

/**
 * @type {!TrackLayout}
 * @private
 */
LayoutGrid.prototype._trackX;

/**
 * @type {!TrackLayout}
 * @private
 */
LayoutGrid.prototype._trackY;

/**
 * @type {!CellSpans}
 * @private
 */
LayoutGrid.prototype._cellSpans;

/**
 * Div elements that govern column styles
 * @type {!Array.<Column>}
 * @private
 */
LayoutGrid.prototype._columns;

/**
 * @type {Column}
 * @private
 */
LayoutGrid.prototype._firstColumn;

/**
 * @type {Column}
 * @private
 */
LayoutGrid.prototype._lastColumn;

/** @type {Element}
* @private
*/
LayoutGrid.prototype._rightSpace;
/** @type {number}
* @private
*/
LayoutGrid.prototype._rightSpaceSize = 0;

/**
 * @type {boolean}
 * @private
 */
LayoutGrid.prototype._frozenLayout = false;

/**
 * @type {boolean}
 * @private
 */
LayoutGrid.prototype._rowHighlightEnabled = false;

/**
 * Negative value means no row is highlight
 * @type {number}
 * @private
 */
LayoutGrid.prototype._rowHighlightIndex = -1;

/**
 * @type {!Array.<Cell>}
 * @private
 */
LayoutGrid.prototype._highlightedCells;

/**
 * @type {Element}
 * @private
 */
LayoutGrid.prototype._coverLayer = null;
/**
 * @type {StretchedCells}
 * @private
 */
LayoutGrid.prototype._stretchedCells = null;
/**
 * @type {*}
 * @private
 */
LayoutGrid.prototype._ctx = null;

/**
 * @type {!SelectionList}
 * @private
 */
LayoutGrid.prototype._selectionList;

/** @type {boolean}
 * @private
 */
LayoutGrid.prototype._colVir = false;

/**
 * @type {boolean}
 * @private
 */
LayoutGrid.prototype._initialized = false;

/**
 * @type {string}
 * @private
 */
LayoutGrid.prototype._hidingMethod = "";

/**
 * @type {ILayoutGrid}
 * @private
 */
LayoutGrid.prototype._virOwner = null;
/** @type {number}
* @private
*/
LayoutGrid.prototype._transformIETimer = 0;
/** @type {!VirtualItems}
* @description Context rows act as a virtual column storing UI context or metadata for rows
* @private
*/
LayoutGrid.prototype._ctxRows;
/** @type {Element}
* @private
*/
LayoutGrid.prototype._boundLayer = null;
/** @type {Array.<Element>}
* @private
*/
LayoutGrid.prototype._colBounds = null;
/** @type {Array.<Element>}
* @private
*/
LayoutGrid.prototype._colBoundCache = null;
/** @type {HScrollbar}
* @private
*/
LayoutGrid.prototype._hscrollbar = null;
/** @type {Element}
* @private
*/
LayoutGrid.prototype._leftColumnSeparator = null;
/** @type {Element}
* @private
*/
LayoutGrid.prototype._rightColumnSeparator = null;
/** @type {Element}
* @private
*/
LayoutGrid.prototype._cellBoundPainter = null;
/**
 * {@link ElementWrapper#dispose}
 * @override
 */
LayoutGrid.prototype.dispose = function () {
	this.setRowCount(0); // Fire rowCountChanged

	this.unlistenAll();
	//this.clearSelectedRows();

	this._ctxRows.dispose();
	this._stretchedCells.dispose();
	for (let i = this._columns.length; --i >= 0; ) {
		this._columns[i].dispose();
	}

	this._colCount = this._rowCount = this._activeRowEnd = this._availableRowCount = 0;
	this._colBounds = this._colBoundCache = null;

	this._highlightedCells.length = 0;
	this._ctx = null;
	this._hscrollbar = null;

	if(this._cellBoundPainter) {
		this._cellBoundPainter.dispose();
	}

	this._dispose();
};
/**
 * {@link ElementWrapper#getWidth}
 * @override
 */
LayoutGrid.prototype.getWidth = function () {
	return this.isVisible() ? this._getWidth() : 0;
};
/** @private
* @return {number}
 */
LayoutGrid.prototype._getWidth = function () {
	return this.getContentWidth() + this._rightSpaceSize;
};

/**
 * {@link ElementWrapper#setWidth}
 * @override
 */
LayoutGrid.prototype.setWidth = function (px) {
	if (px < 0) {
		return false;
	}

	let changes = this._trackX.setTrackSize(px - this._rightSpaceSize); // WARNING: Perform scalling instead of setting width directly

	if (changes == null) {
		return false;
	}

	// The size will be applied, only if minimum specification is satisfied.
	this._syncLayoutToColumns(0);

	return true;
};

/**
 * {@link ElementWrapper#getHeight}
 * @override
 */
LayoutGrid.prototype.getHeight = function () {
	if (!this.isVisible()) {
		return 0;
	}

	return this.getContentHeight();
};
/**
 * {@link ElementWrapper#setHeight}
 * @override
 */
LayoutGrid.prototype.setHeight = function (px) {
	if (px < 0) {
		return false;
	}

	let changes = this._trackY.setTrackSize(px); // WARNING: Perform scalling instead of setting height directly

	if (changes == null) {
		return true;
	}

	// The size will be applied, only if minimum specification is satisfied.
	this._syncLayoutToRows(0, this._rowCount);

	return true;
};
/**
 * {@link ElementWrapper#show}
 * @override
 */
LayoutGrid.prototype.show = function (opt_shown) {
	// warning: this method affect to row only
	this._element.style.display = opt_shown !== false ? "" : "none";
	this._trackY.show(opt_shown);
	this.setFrozenLayout(opt_shown === false);
	this._dispatchRowHeightChangedEvent(0, this._rowCount); // TODO: This is redundant with setFrozenLayout
};

/**
 * {@link ILayoutGrid#hitTest}
 * @inheritDoc
 */
LayoutGrid.prototype.hitTest = function (x, y, opt_omitSpan) {
	if (x < 0) {
		return null;
	}

	if (y < 0) {
		return null;
	}

	if (x >= this.getContentWidth()) {
		return null;
	}

	if (y >= this.getHeight()) {
		return null;
	}

	let hitX = this._trackX.hitTest(x);

	if (hitX < 0) {
		return null;
	}

	let hitY = this._trackY.hitTest(y);

	if (hitY < 0) {
		return null;
	}

	let result = {};
	result["colIndex"] = hitX;
	result["rowIndex"] = hitY;

	if (opt_omitSpan !== true) {
		let cellSpan = this._cellSpans.getOccupyingSpan(hitX, hitY);

		if (cellSpan) {
			result["colIndex"] = cellSpan.indexX;
			result["rowIndex"] = cellSpan.indexY;
		}
	}

	return result;
};

/**
 * {@link ILayoutGrid#hitTestX}
 * @inheritDoc
 */
LayoutGrid.prototype.hitTestX = function (x) {
	if (x < 0) {
		return -1;
	}

	if (x >= this.getContentWidth()) {
		return ~this._colCount;
	}

	return this._trackX.hitTest(x);
};

/**
 * {@link ILayoutGrid#hitTestY}
 * @inheritDoc
 */
LayoutGrid.prototype.hitTestY = function (y) {
	if (y < 0) {
		return -1;
	}

	if (y >= this.getHeight()) {
		return ~this._rowCount;
	}

	return this._trackY.hitTest(y);
};

/**
 * {@link ILayoutGrid#getDefaultColumnWidth}
 * @inheritDoc
 */
LayoutGrid.prototype.getDefaultColumnWidth = function () {
	return this._trackX.getDefaultLaneSize();
};

/**
 * {@link ILayoutGrid#setDefaultColumnWidth}
 * @inheritDoc
 */
LayoutGrid.prototype.setDefaultColumnWidth = function (val) {
	if (this._trackX.getDefaultLaneSize() !== val) {
		this._trackX.setDefaultLaneSize(val);
		this._syncLayoutToColumns(0);
	}
};

/**
 * {@link ILayoutGrid#getDefaultRowHeight}
 * @inheritDoc
 */
LayoutGrid.prototype.getDefaultRowHeight = function () {
	return this._trackY.getDefaultLaneSize();
};

/**
 * {@link ILayoutGrid#setDefaultRowHeight}
 * @inheritDoc
 */
LayoutGrid.prototype.setDefaultRowHeight = function (val) {
	let defaultH = this._trackY.getDefaultLaneSize();

	if (defaultH !== val) {
		this._trackY.setDefaultLaneSize(val);

		for (let i = 0; i < this._colCount; ++i) {
			this._columns[i].setHeight(val);
		}

		this._syncLayoutToRows(0, this._rowCount);
		this._dispatchRowHeightChangedEvent(0, this._rowCount);
	}
};

/**
 * {@link ILayoutGrid#getColumnWidth}
 * @inheritDoc
 */
LayoutGrid.prototype.getColumnWidth = function (indexX) {
	return this._trackX.getLaneSize(indexX);
};

/**
 * {@link ILayoutGrid#setColumnWidth}
 * @inheritDoc
 */
LayoutGrid.prototype.setColumnWidth = function (indexX, val) {
	if (this._setColumnWidth(indexX, val)) {
		this._syncLayoutToColumns(indexX);

		return true;
	}

	return false;
};
/** @public
 * @ignore
 * @param {number} indexX
 * @param {number} val
 * @return {boolean}
 */
LayoutGrid.prototype._setColumnWidth = function (indexX, val) {
	return this._trackX.setLaneSize(indexX, val);
};
/** @public
 * @ignore
 */
LayoutGrid.prototype._updateColumnLayout = function () {
	this._syncLayoutToColumns(0);
};
/** @public
 * @ignore
 * @param {boolean} enabled
 */
LayoutGrid.prototype._startBindingSession = function (enabled) {
	// Prevent error when calling _startBindingSession in LayoutGrid
};
/** @public
 * @ignore
 * @return {Array<boolean>}
 */
LayoutGrid.prototype._getTempRowHeights = function () {
	return null;
};

/**
 * {@link ILayoutGrid#getRowHeight}
 * @inheritDoc
 */
LayoutGrid.prototype.getRowHeight = function (indexY) {
	return this._trackY.getLaneSize(indexY);
};

/**
 * {@link ILayoutGrid#setRowHeight}
 * @inheritDoc
 */
LayoutGrid.prototype.setRowHeight = function (rowIndex, val) {
	if (rowIndex < 0 || rowIndex >= this._rowCount) {
		return false;
	}

	let dirty = this._trackY.setLaneSize(rowIndex, val);

	this._syncLayoutToRows(rowIndex, rowIndex + 1); // Update UI. WARNING: Very slow

	if (dirty) {
		this._dispatchRowHeightChangedEvent(rowIndex, rowIndex + 1);
	}

	this._resetTransformIETimer();

	return dirty;
};

/**
 * @description Use internally to quickly update all row heights
 * @public
 * @ignore
 * @param {TrackLayout} track
 * @param {number} start
 * @returns {boolean}
 */
LayoutGrid.prototype._updateRowHeights = function (track, start) {
	let dirty = 0;

	for (let i = 0; i < this._rowCount; ++i) {
		let val = track.getLaneSize(start);

		dirty |= this._trackY.setLaneSize(i, val);
		++start;
	}

	this._syncLayoutToRows(0, this._rowCount);
	// if (dirty) { // TODO: Check if need to dispatch the event
	// this._dispatchRowHeightChangedEvent(0, this._rowCount);
	// }
	return dirty;
};

/**
 * {@link ILayoutGrid#isColumnVisible}
 * @inheritDoc
 */
LayoutGrid.prototype.isColumnVisible = function (indexX) {
	return this._trackX.isLaneVisible(indexX);
};

/**
 * {@link ILayoutGrid#hideColumn}
 * @inheritDoc
 */
LayoutGrid.prototype.hideColumn = function (indexX, val) {
	return this.setColumnVisibility(indexX, (val === false), 0);
};

/**
 * {@link ILayoutGrid#isRowVisible}
 * @inheritDoc
 */
LayoutGrid.prototype.isRowVisible = function (indexY) {
	return this._trackY.isLaneVisible(indexY);
};

/**
 * {@link ILayoutGrid#hideRow}
 * @inheritDoc
 */
LayoutGrid.prototype.hideRow = function (indexY, val) {
	if (this._trackY.hideLane(indexY, val)) {
		if ((val == null || val) && !this.isRowVisible(indexY)) {
			this._hiddenRowCount++;
		} else if (this.isRowVisible(indexY)){
			this._hiddenRowCount--;
		}

		this._syncLayoutToRows(indexY, this._rowCount);
		this._dispatchRowHeightChangedEvent(indexY, indexY + 1);

		return true;
	}

	return false;
};

/**
 * {@link ILayoutGrid#getMinimumColumnWidth}
 * @inheritDoc
 */
LayoutGrid.prototype.getMinimumColumnWidth = function (indexX) {
	return this._trackX.getMinimumLaneSize(indexX);
};

/**
 * {@link ILayoutGrid#setMinimumColumnWidth}
 * @inheritDoc
 */
LayoutGrid.prototype.setMinimumColumnWidth = function (indexX, val) {
	if (this._trackX.setMinimumLaneSize(indexX, val)) {
		this._syncLayoutToColumns(indexX);

		return true;
	}

	return false;
};

/**
 * {@link ILayoutGrid#getMinimumRowHeight}
 * @inheritDoc
 */
LayoutGrid.prototype.getMinimumRowHeight = function (indexY) {
	return this._trackY.getMinimumLaneSize(indexY);
};

/**
 * {@link ILayoutGrid#setMinimumRowHeight}
 * @inheritDoc
 */
LayoutGrid.prototype.setMinimumRowHeight = function (indexY, val) {
	if (this._trackY.setMinimumLaneSize(indexY, val)) {
		this._syncLayoutToRows(indexY, this._rowCount);
		this._dispatchRowHeightChangedEvent(indexY, indexY + 1);

		return true;
	}

	return false;
};

/**
 * {@link ILayoutGrid#getCellLeft}
 * @inheritDoc
 */
LayoutGrid.prototype.getCellLeft = function (indexX) {
	return this._trackX.getLaneStart(indexX);
};

/**
 * {@link ILayoutGrid#getCellTop}
 * @inheritDoc
 */
LayoutGrid.prototype.getCellTop = function (indexY) {
	return this._trackY.getLaneStart(indexY);
};

/**
 * {@link ILayoutGrid#getCellWidth}
 * @inheritDoc
 */
LayoutGrid.prototype.getCellWidth = function (indexX, indexY) {
	let cellSpan = this._cellSpans.getOccupyingSpan(indexX, indexY);

	if (cellSpan) {
		let spanStart = this._trackX.getLaneStart(cellSpan.indexX);
		let spanEnd = this._trackX.getLaneEnd(cellSpan.indexX + cellSpan.colSpan - 1);

		return spanEnd - spanStart;
	}

	return this.getColumnWidth(indexX);
};

/**
 * {@link ILayoutGrid#getCellHeight}
 * @inheritDoc
 */
LayoutGrid.prototype.getCellHeight = function (indexX, indexY) {
	let cellSpan = this._cellSpans.getOccupyingSpan(indexX, indexY);

	if (cellSpan) {
		let spanStart = this._trackY.getLaneStart(cellSpan.indexY);
		let spanEnd = this._trackY.getLaneEnd(cellSpan.indexY + cellSpan.rowSpan - 1);

		return spanEnd - spanStart;
	}

	return this.getRowHeight(indexY);
};

/**
 * {@link ILayoutGrid#hasCellSpan}
 * @inheritDoc
 */
LayoutGrid.prototype.hasCellSpan = function () {
	return this._cellSpans.hasSpan();
};

/**
 * {@link ILayoutGrid#getCellRowSpan}
 * @inheritDoc
 */
LayoutGrid.prototype.getCellRowSpan = function (indexX, indexY) {
	return this._cellSpans.getRowSpan(indexX, indexY);
};

/**
 * {@link ILayoutGrid#setCellRowSpan}
 * @inheritDoc
 */
LayoutGrid.prototype.setCellRowSpan = function (indexX, indexY, spanVal) {
	let cellSpan = this._cellSpans.setRowSpan(indexX, indexY, spanVal);

	this._updateCellSpanSize(indexX, indexY, cellSpan);

	if (cellSpan) {
		if (cellSpan.retVal >= 0) { // Expansion
			this._updateCellSpanClass(indexX, indexY, cellSpan.colSpan, cellSpan.rowSpan, true);
		} else { // Shrinking
			if (cellSpan.colSpan > 1) { // Spanning still exists
				this._updateCellSpanClass(indexX, indexY, cellSpan.colSpan, cellSpan.prevRowSpan, false);
				this._updateCellSpanClass(indexX, indexY, cellSpan.colSpan, cellSpan.rowSpan, true);
			} else { // Removing span definition
				this._updateCellSpanClass(indexX, indexY, cellSpan.colSpan, cellSpan.prevRowSpan, false);
			}
		}

		return cellSpan.retVal !== 0;
	}

	let cell = this._getCell(indexX, indexY);

	if (cell && cell._rowSpan > 1) {
		this._updateCellSpanClass(indexX, indexY, cell._colSpan, cell._rowSpan, false);
	}

	return false;
};

/**
 * {@link ILayoutGrid#getCellColSpan}
 * @inheritDoc
 */
LayoutGrid.prototype.getCellColSpan = function (indexX, indexY) {
	return this._cellSpans.getColSpan(indexX, indexY);
};

/**
 * {@link ILayoutGrid#setCellColSpan}
 * @inheritDoc
 */
LayoutGrid.prototype.setCellColSpan = function (indexX, indexY, spanVal) {
	let cellSpan = this._cellSpans.setColSpan(indexX, indexY, spanVal);

	this._updateCellSpanSize(indexX, indexY, cellSpan);

	if (cellSpan) {
		if (cellSpan.retVal >= 0) { // Expansion
			this._updateCellSpanClass(indexX, indexY, cellSpan.colSpan, cellSpan.rowSpan, true);
		} else { // Shrinking
			if (cellSpan.colSpan > 1) { // Spanning still exists
				this._updateCellSpanClass(indexX, indexY, cellSpan.prevColSpan, cellSpan.rowSpan, false);
				this._updateCellSpanClass(indexX, indexY, cellSpan.colSpan, cellSpan.rowSpan, true);
			} else { // Removing span definition
				this._updateCellSpanClass(indexX, indexY, cellSpan.prevColSpan, cellSpan.rowSpan, false);
			}
		}

		return cellSpan.retVal !== 0;
	}

	let cell = this._getCell(indexX, indexY);

	if (cell && cell._colSpan > 1) {
		this._updateCellSpanClass(indexX, indexY, cell._colSpan, cell._rowSpan, false);
	}

	return false;
};

/**
 * {@link ILayoutGrid#clearCellSpans}
 * @inheritDoc
 */
LayoutGrid.prototype.clearCellSpans = function () {
	let cellSpans = this._cellSpans.clearAllSpans();

	if (cellSpans == null) {
		return false;
	}

	for (let i = cellSpans.length; --i >= 0; ) {
		let cellSpan = cellSpans[i];

		this._updateCellSpan(cellSpan.indexX, cellSpan.indexY, cellSpan, false);
	}

	return true;
};

/**
 * {@link ILayoutGrid#clearCellSpans}
 * @inheritDoc
 */
LayoutGrid.prototype.clearColumnSpans = function (indexX) {
	let cellSpans = this._cellSpans.removeSpanByColIndex(indexX);

	if (cellSpans == null) {
		return false;
	}

	for (let i = cellSpans.length; --i >= 0; ) {
		let cellSpan = cellSpans[i];

		this._updateCellSpan(cellSpan.indexX, cellSpan.indexY, cellSpan, false);
	}

	return true;
};

/**
 * {@link ILayoutGrid#clearColumnCustomizations}
 * @inheritDoc
 */
LayoutGrid.prototype.clearColumnCustomizations = function () {
	this._trackX.clearLaneCustomizations();
	this._syncLayoutToColumns(0);
};

/**
 * {@link ILayoutGrid#clearRowCustomizations}
 * @inheritDoc
 */
LayoutGrid.prototype.clearRowCustomizations = function () {
	this._trackY.clearLaneCustomizations();
	this._syncLayoutToRows(0, this._rowCount);
};

/**
 * {@link ILayoutGrid#getColumnScalability}
 * @inheritDoc
 */
LayoutGrid.prototype.getColumnScalability = function (indexX) {
	return this._trackX.getLaneScalability(indexX);
};

/**
 * {@link ILayoutGrid#setColumnScalability}
 * @inheritDoc
 */
LayoutGrid.prototype.setColumnScalability = function (indexX, val) {
	return this._trackX.setLaneScalability(indexX, val);
};

/**
 * {@link ILayoutGrid#getRowScalability}
 * @inheritDoc
 */
LayoutGrid.prototype.getRowScalability = function (indexY) {
	return this._trackY.getLaneScalability(indexY);
};

/**
 * {@link ILayoutGrid#setRowScalability}
 * @inheritDoc
 */
LayoutGrid.prototype.setRowScalability = function (indexY, val) {
	return this._trackY.setLaneScalability(indexY, val);
};

/**
 * {@link ILayoutGrid#getCell}
 * @inheritDoc
 */
LayoutGrid.prototype.getCell = function (indexX, indexY, opt_includeSpan) {
	if (opt_includeSpan && this._cellSpans.hasSpan()) {
		let colSpan = this._cellSpans.getColSpan(indexX, indexY);
		let rowSpan = this._cellSpans.getRowSpan(indexX, indexY);

		if (colSpan < 0) {
			indexX += colSpan;
		}

		if (rowSpan < 0) {
			indexY += rowSpan;
		}
	}

	return this._getCell(indexX, indexY);
};

/**
 * {@link ILayoutGrid#getCellContent}
 * @inheritDoc
 */
LayoutGrid.prototype.getCellContent = function (indexX, indexY) {
	let cell = this.getCell(indexX, indexY);

	if (cell !== null) {
		return cell.getContent();
	}

	return null;
};

/**
 * {@link ILayoutGrid#setCellContent}
 * @inheritDoc
 */
LayoutGrid.prototype.setCellContent = function (indexX, indexY, data) {
	let cell = this.getCell(indexX, indexY);

	if (cell == null) {
		window.console.log("Error: Accessing nonexistent cell in Grid's Table");

		return null;
	}

	return cell.setContent(data);
};

/**
 * {@link ILayoutGrid#getColumnCount}
 * @inheritDoc
 */
LayoutGrid.prototype.getColumnCount = function () {
	return this._colCount;
};

/**
 * {@link ILayoutGrid#setColumnCount}
 * @inheritDoc
 */
LayoutGrid.prototype.setColumnCount = function (val) {
	if (val < 0) {
		val = 0;
	}

	if (this._colCount === val) {
		return;
	}

	let oldColCount = this._colCount;

	this._trackX.setLaneCount(val);  // Need to be set before syncing layout
	this._colCount = this._trackX.getLaneCount();

	let i = 0;
	let column = null;

	if (oldColCount > val) {
		for (i = oldColCount; --i >= val; ) {
			column = this._columns.pop();
			column.dispose();

			if (column === this._firstColumn) {
				this._firstColumn = null;
			}

			if (column === this._lastColumn) {
				this._lastColumn = null;
			}
		}

		// this._updateCssWidth(); // Core grid will guarantee updating width to each section
	} else {
		for (i = oldColCount; i < val; ++i) {
			column = this._newColumn(i);
		}

		// this._syncLayoutToColumns(oldColCount); // Core grid will guarantee updating width to each section
	}

	this._updateColumnIndices();

	for (i = oldColCount; i < val; ++i) {
		this._columns[i].activate(!this._colVir);
	}

	if (val <= 0) { // If there is no column at all, clear all selections
		this._selectionList.clearAllSelections(); // No UI change
	}
};

/**
 * {@link ILayoutGrid#getRowCount}
 * @inheritDoc
 */
LayoutGrid.prototype.getRowCount = function () {
	return this._rowCount;
};

/**
 * {@link ILayoutGrid#setRowCount}
 * @inheritDoc
 */
LayoutGrid.prototype.setRowCount = function (val, noBinding) {
	if (val < 0) {
		val = 0;
	}

	if (this._rowCount === val) {
		return;
	}

	let c;
	let oldRowCount = this._rowCount;

	this._trackY.setLaneCount(val); // Need to be set before syncing layout
	this._rowCount = this._trackY.getLaneCount();

	this._ctxRows.setItemCount(val);
	this._stretchedCells.setCellCount(val);
	for (c = 0; c < this._colCount; ++c) {
		this._columns[c].setCellCount(val);
	}

	if (oldRowCount > val) {
		if (val < this._rowHighlightIndex) {
			this.setRowHighlight(-1);
		}

		if (val < this._activeRowEnd) {
			this._activeRowEnd = val;
		}

		if (val < this._availableRowCount) {
			this._availableRowCount = val;
		}

		this._cellSpans.removeSpanStartedFromRowIndex(val);
		this._updateViewHeight();

		// Clear selection of the removed rows
		if (val > 0) {
			for (let i = val; i < oldRowCount; ++i) {
				this._selectionList.deselect(i); // No UI change
			}
		} else {
			this._selectionList.clearAllSelections(); // No UI change
		}
	} else {
		this._syncLayoutToRows(oldRowCount, val);
	}

	let e = {};

	e["prevCount"] = oldRowCount;
	e["rowCount"] = this._rowCount;
	this._dispatch("rowCountChanged", e); // This will trigger virtualizer

	if (oldRowCount < val) { // In case of no event is sent back
		this._insertCells(0, this._rowCount); // Dispatch row available
	}
};

/**
 * {@link ILayoutGrid#getVisibleRowCount}
 * @inheritDoc
 */
LayoutGrid.prototype.getVisibleRowCount = function () {
	return (this._rowCount - this._hiddenRowCount);
};

/**
 * {@link ILayoutGrid#addRow}
 * @inheritDoc
 */
LayoutGrid.prototype.addRow = function (val) {
	if (val <= 0) {
		return;
	}

	if (this._colCount <= 0) {
		window.console.log("WARNING: Row cannot be added, unless at least one column is added");

		return;
	}

	this.setRowCount(this._rowCount + val);
};

/**
 * {@link ILayoutGrid#removeRow}
 * @inheritDoc
 */
LayoutGrid.prototype.removeRow = function (val) {
	if (val <= 0) {
		return;
	}

	if (val > this._rowCount) {
		val = this._rowCount;
	}

	this.setRowCount(this._rowCount - val);
};

/**
 * {@link ILayoutGrid#setBorders}
 * @inheritDoc
 */
LayoutGrid.prototype.setBorders = function (lft, top, rgt, btm) {

};

/**
 * {@link ILayoutGrid#getColumnStyle}
 * @inheritDoc
 */
LayoutGrid.prototype.getColumnStyle = function (colIndex, styleName) {
	let column = this._columns[colIndex];

	if (column) {
		return column.getStyle(styleName);
	}

	return null;
};

/**
 * {@link ILayoutGrid#setColumnStyle}
 * @inheritDoc
 */
LayoutGrid.prototype.setColumnStyle = function (colIndex, styleName, value) {
	let column = this._columns[colIndex];
	if (column) {
		column.setStyle(styleName, value);
	}
};

/**
 * {@link ILayoutGrid#enableColumnClass}
 * @inheritDoc
 */
LayoutGrid.prototype.enableColumnClass = function (colIndex, className, enabled) {
	let column = this._columns[colIndex];

	if (column) {
		column.enableClass(className, enabled);
	}
};

/**
 * {@link ILayoutGrid#enableColumnClass}
 * @inheritDoc
 */
LayoutGrid.prototype.hasColumnClass = function (colIndex, className) {
	let column = this._columns[colIndex];

	return (column) ? column.hasClass(className) : false;
};

/**
 * {@link ILayoutGrid#enableRowClass}
 * @inheritDoc
 */
LayoutGrid.prototype.enableRowClass = function (rowIndex, className, enabled) {
	if (rowIndex >= 0 && rowIndex < this._rowCount) {
		for (let c = 0; c < this._colCount; ++c) {
			this._columns[c].getCell(rowIndex).enableClass(className, enabled);
		}
	}
};

/**
 * {@link ILayoutGrid#setRowStyle}
 * @inheritDoc
 */
LayoutGrid.prototype.setRowStyle = function (rowIndex, styleName, value, enabled) {
	if (rowIndex >= 0 && rowIndex < this._rowCount) {
		if (!value || (enabled != null && !enabled)) {
			value = "";
		}

		for (let c = 0; c < this._colCount; ++c) {
			this._columns[c].getCell(rowIndex).setStyle(styleName, value);
		}
	}
};

/**
 * {@link ILayoutGrid#getRelativePosition}
 * @inheritDoc
 */
LayoutGrid.prototype.getRelativePosition = function (obj) {
	return util.getRelativePosition(/** @type{Element|MouseEvent|Event} */(obj), this._element);
};

/**
 * {@link ILayoutGrid#getContentWidth}
 * @inheritDoc
 */
LayoutGrid.prototype.getContentWidth = function () {
	return this._trackX.getTrackSize();
};

/**
 * {@link ILayoutGrid#getContentHeight}
 * @inheritDoc
 */
LayoutGrid.prototype.getContentHeight = function () {
	let layoutSize = this._trackY.getTrackSize();

	if (layoutSize <= 0) { // All of the cell has default size of zero
		if (this._colCount * this.getVisibleRowCount() > 0) {
			let column = this._columns[0];
			let bottomCell = column.getCell(this._rowCount - 1);
			let bottom = bottomCell.getTop() + bottomCell.getHeight();

			return bottom - column.getCell(0).getTop();
		}
	}

	return layoutSize;
};

/**
 * {@link ILayoutGrid#getCellsByRowIndex}
 * @inheritDoc
 */
LayoutGrid.prototype.getCellsByRowIndex = function (rowIndex) {
	if (rowIndex < 0 || rowIndex >= this._rowCount) {
		return null;
	}

	let ary = new Array(this._colCount);

	for (let c = 0; c < this._colCount; ++c) {
		ary[c] = this._columns[c].getCell(rowIndex);
	}

	return ary;
};

/**
 * {@link ILayoutGrid#disableHorizontalGridLine}
 * @inheritDoc
 */
LayoutGrid.prototype.disableHorizontalGridLine = function (opt_bool) {
	this.enableClass("no-horizontal-grid-line", opt_bool !== false);
};

/**
 * {@link ILayoutGrid#disableVerticalGridLine}
 * @inheritDoc
 */
LayoutGrid.prototype.disableVerticalGridLine = function (opt_bool) {
	this.enableClass("no-vertical-grid-line", opt_bool !== false);
};

/**
 * {@link ILayoutGrid#enableShading}
 * @inheritDoc
 */
LayoutGrid.prototype.enableShading = function (enabled) {
	if (enabled !== false) {
		this.addClass("tr-even-shading");
	} else {
		this.removeClass("tr-even-shading");
		this.removeClass("tr-odd-shading");
	}
};

/**
 * {@link ILayoutGrid#setTransposedShading}
 * @inheritDoc
 */
LayoutGrid.prototype.setTransposedShading = function (enabled) {
	this.enableClass("transposed-shading", enabled !== false);
};

/**
 * {@link ILayoutGrid#enableRowHighlighting}
 * @inheritDoc
 */
LayoutGrid.prototype.enableRowHighlighting = function (opt_bool) {
	if (opt_bool == null) {
		opt_bool = true;
	}

	if (this._rowHighlightEnabled === opt_bool) {
		return;
	}

	this._rowHighlightEnabled = opt_bool;

	if (this._rowHighlightEnabled) {
		this.listen("mousemove", this._onMouseMove);
		this.listen("mouseout", this._onMouseOut);
	} else {
		this.setRowHighlight(-1);
		this.unlisten("mousemove", this._onMouseMove);
		this.unlisten("mouseout", this._onMouseOut);
	}
};

/**
 * {@link ILayoutGrid#getRowHighlight}
 * @inheritDoc
 */
LayoutGrid.prototype.getRowHighlight = function () {
	return this._rowHighlightIndex;
};

/**
 * {@link ILayoutGrid#setRowHighlight}
 * @inheritDoc
 */
LayoutGrid.prototype.setRowHighlight = function (rowIndex) {
	if (!this._rowHighlightEnabled || rowIndex < -1 || rowIndex >= this._rowCount) {
		rowIndex = -1;
	}

	if (this._rowHighlightIndex === rowIndex) {
		return;
	}

	let prevIndex = this._rowHighlightIndex;

	this._rowHighlightIndex = rowIndex;

	for (let i = this._highlightedCells.length; --i >= 0;) {
		this._highlightedCells[i].removeClass("highlighted-row"); // This will guarantee that there is no left over highlighted cell
	}

	if (rowIndex >= 0 && rowIndex < this._rowCount) {
		this._highlightedCells.length = this._colCount;

		for (let c = this._colCount; --c >= 0;) {
			let cell = this._columns[c].getCell(rowIndex);

			this._highlightedCells[c] = cell;
			cell.addClass("highlighted-row");
		}
		let stretchEl = this.getStretchedCell(rowIndex); // check stretched cell
		if(stretchEl) {
			this._highlightedCells.push(stretchEl);
			stretchEl.addClass("highlighted-row");
		}
	} else {
		this._highlightedCells.length = 0;
	}

	if (this._hasListener("rowHighlighted")) {
		let e = {};

		e["prevRowIndex"] = prevIndex;
		e["rowIndex"] = rowIndex;
		this._dispatch("rowHighlighted", e);
	}
};

/**
 * {@link ILayoutGrid#getSelectedRows}
 * @inheritDoc
 */
LayoutGrid.prototype.getSelectedRows = function () {
	if (this.getSelectedRowCount() > 0) {
		return this._selectionList.getAllSelections();
	}

	return null;
};

/**
 * {@link ILayoutGrid#getSelectedRowCount}
 * @inheritDoc
 */
LayoutGrid.prototype.getSelectedRowCount = function () {
	return this._selectionList.getSelectionCount();
};

/**
 * {@link ILayoutGrid#isSelectedRow}
 * @inheritDoc
 */
LayoutGrid.prototype.isSelectedRow = function (at) {
	return this._selectionList.getSelection(at);
};

/**
 * {@link ILayoutGrid#setSelectedRow}
 * @inheritDoc
 */
LayoutGrid.prototype.setSelectedRow = function (rowIndex, opt_selected) {
	if (rowIndex < 0 || rowIndex >= this._rowCount) {
		return;
	}

	this._selectionList.setSelection(rowIndex, opt_selected);
	this._updateSelectionUI(rowIndex);
};

/**
 * {@link ILayoutGrid#selectSingleRow}
 * @inheritDoc
 */
LayoutGrid.prototype.selectSingleRow = function (rowIndex) {
	if (rowIndex < 0 || rowIndex >= this._rowCount) {
		return;
	}

	this.clearSelectedRows();
	this._selectionList.select(rowIndex);
	this._addSelectionUI(rowIndex);
};

/**
 * {@link ILayoutGrid#selectRowRange}
 * @inheritDoc
 */
LayoutGrid.prototype.selectRowRange = function (rowIndex, len) {
	if (rowIndex < 0 || rowIndex >= this._rowCount) {
		return;
	}

	if (rowIndex + len > this._rowCount) {
		len = this._rowCount - rowIndex;
	}

	this._selectionList.selectRange(rowIndex, len);

	for (let r = 0; r < len; ++r) {
		this._addSelectionUI(r + rowIndex);
	}
};

/**
 * {@link ILayoutGrid#clearSelectedRows}
 * @inheritDoc
 */
LayoutGrid.prototype.clearSelectedRows = function () {
	let count = this._selectionList.getSelectionCount();

	if (count) {
		let sels = this._selectionList.getAllSelections(); // Slow

		this._selectionList.clearAllSelections();

		for (let i = 0; i < count; ++i) {
			this._removeSelectionUI(sels[i]);
		}

		return count;
	}

	return 0;
};

/**
 * {@link ILayoutGrid#getFirstSelectedRow}
 * @inheritDoc
 */
LayoutGrid.prototype.getFirstSelectedRow = function () {
	return this._selectionList.getFirstSelectedIndex();
};

/**
 * {@link ILayoutGrid#getLastSelectedRow}
 * @inheritDoc
 */
LayoutGrid.prototype.getLastSelectedRow = function () {
	return this._selectionList.getLastSelectedIndex();
};

/**
 * {@link ILayoutGrid#getRowAnchor}
 * @inheritDoc
 */
LayoutGrid.prototype.getRowAnchor = function () {
	return this._selectionList.getSelectionAnchor();
};

/**
 * {@link ILayoutGrid#setRowAnchor}
 * @inheritDoc
 */
LayoutGrid.prototype.setRowAnchor = function (at) {
	this._selectionList.setSelectionAnchor(at);
};

/**
 * @inheritDoc
 */
LayoutGrid.prototype.activateColumn = function (colIndex, opt_bool) {
	let column = this._columns[colIndex];

	if (column) {
		let dirty = column.activate(opt_bool);
		if (opt_bool && dirty && this._rightSpace) {
			let pn = this._rightSpace.parentNode;
			if (pn) {
				pn.appendChild(this._rightSpace);
			}
		}
	}
};

/**
 * {@link ILayoutGrid#activateRows}
 * @inheritDoc
 */
LayoutGrid.prototype.activateRows = function () {
	this._onEnterDocument();
};

/**
 * @inheritDoc
 */
LayoutGrid.prototype.moveRowsToTop = function (numRows) {
	this._ctxRows.moveItemsToTop(numRows);
	this._stretchedCells.moveCellsToTop(numRows);
	for (let c = 0; c < this._colCount; ++c) {
		this._columns[c].moveCellsToTop(numRows);
	}
};

/**
 * {@link ILayoutGrid#getFirstIndexInView}
 * @inheritDoc
 */
LayoutGrid.prototype.getFirstIndexInView = function () {
	return 0;
};

/**
 * {@link ILayoutGrid#getLastIndexInView}
 * @inheritDoc
 */
LayoutGrid.prototype.getLastIndexInView = function () {
	return this._availableRowCount - 1;
};

/**
 * {@link ILayoutGrid#isInCurrentView}
 * @inheritDoc
 */
LayoutGrid.prototype.isInCurrentView = function (index) {
	return this._rowCount === 0 ? false : (index >= 0 && index < this._rowCount);
};

/**
 * {@link ILayoutGrid#getFrozenLayout}
 * @ignore
 * @inheritDoc
 */
LayoutGrid.prototype.getFrozenLayout = function () {
	return this._frozenLayout;
};

/**
 * {@link ILayoutGrid#setFrozenLayout}
 * @ignore
 * @inheritDoc
 */
LayoutGrid.prototype.setFrozenLayout = function (bool) {
	if (!this.isVisible()) {
		bool = true;
	}

	if (this._frozenLayout !== bool) {
		this._frozenLayout = bool;

		this._cellSpans.freezeMapping(this._frozenLayout);
		if (!this._frozenLayout) {
			this._syncLayoutToColumns(0);
			this._syncLayoutToRows(0, this._rowCount);
			this._dispatchRowHeightChangedEvent(0, this._rowCount);
		}
	}
};

/**
 * {@link ILayoutGrid#getColumn}
 * @inheritDoc
 */
LayoutGrid.prototype.getColumn = function (index) {
	return this._columns[index] || null;
};

/**
 * {@link ILayoutGrid#insertColumn}
 * @ignore
 * @inheritDoc
 */
LayoutGrid.prototype.insertColumn = function (index, opt_json) {
	if (index < 0) {
		index = 0;
	}	else if (index > this._colCount) {
		index = this._colCount;
	}

	let atTheMiddle = index < this._colCount;
	let cellSpans = this._cellSpans.getAllSpans();

	this._updateCellSpans(cellSpans, false); // Clear all styling
	this._trackX.insertLane(index, opt_json);
	this._colCount = this._trackX.getLaneCount();

	let column = this._newColumn(index);

	this._updateColumnIndices();

	if (this._cellSpans.shiftColumn(index, 1)) {
		cellSpans = this._cellSpans.getAllSpans();
	}

	this._updateCellSpans(cellSpans, true); // Restore all styling
	this._calculateViewSize(true);
	this._stretchedCells.updateXAxis();
	// this._syncLayoutToColumns(index); // Core grid will guarantee updating width to each section

	// Select the newly added cell, if there is a row selection
	let count = this._selectionList.getSelectionCount();

	if (count) {
		let sels = this._selectionList.getAllSelections(); // Slow

		for (let i = sels.length; --i >= 0;) {
			let cell = column.getCell(sels[i]);

			if (cell) {
				cell.addClass("selected-row");
			}
		}
		// TODO: Check if it can insert column with stretch cell
	}

	column.activate(atTheMiddle || !this._colVir);

	return column;
};

/**
 * {@link ILayoutGrid#removeColumnAt}
 * @ignore
 * @inheritDoc
 */
LayoutGrid.prototype.removeColumnAt = function (index) {
	let definition = this._trackX.removeLaneAt(index);

	if (definition == null) {
		return null;
	}

	let cellSpans = this._cellSpans.getAllSpans();

	this._updateCellSpans(cellSpans, false); // Clear all styling
	this._colCount = this._trackX.getLaneCount();

	let column = this._columns.splice(index, 1)[0];

	column.dispose();
	definition["control"] = column;
	this._updateColumnIndices();

	if (this._cellSpans.removeColumn(index)) {
		cellSpans = this._cellSpans.getAllSpans();
	}

	this._updateCellSpans(cellSpans, true); // Restore all styling
	this._syncLayoutToColumns(index);

	if (index >= this._colCount) {
		this._updateCssWidth();
	}

	return definition;
};

/**
 * {@link ILayoutGrid#moveColumn}
 * @ignore
 * @inheritDoc
 */
LayoutGrid.prototype.moveColumn = function (from, to) {
	if (from < 0) {
		from = 0;
	} else if (from >= this._colCount) {
		return;
	}

	if (to < 0) {
		to = 0;
	}	else if (to >= this._colCount) {
		to = this._colCount - 1;
	}

	if (from === to) {
		return;
	}

	let cellSpans = this._cellSpans.getAllSpans();

	this._updateCellSpans(cellSpans, false); // Clear all styling
	this._trackX.moveLane(from, to);

	let movedColumn = this._columns[from];

	util.moveArrayItem(this._columns, from, to);

	if(movedColumn.getElement().offsetParent) {
		let siblingIndex = to + 1;
		let nextSibling = this._columns[siblingIndex] || null;
		while (nextSibling && !nextSibling.getElement().offsetParent) {
			nextSibling = this._columns[++siblingIndex];
		}
		movedColumn.insertBefore(nextSibling);
	}

	this._updateColumnIndices();
	this._updateCellSpans(cellSpans, true); // Restore all styling

	let minIndex = (from < to) ? from : to;

	this._syncLayoutToColumns(minIndex);
};

/**
 * {@link ILayoutGrid#enableRowVirtualization}
 * @ignore
 * @inheritDoc
 */
LayoutGrid.prototype.enableRowVirtualization = function () {
	console.log("WARNING: This function is deprecated");
};

/**
 * {@link ILayoutGrid#enableRowVirtualization}
 * @ignore
 * @inheritDoc
 */
LayoutGrid.prototype.enableColumnVirtualization = function (bool) {
	this._colVir = bool !== false;
};

/**
 * {@link ILayoutGrid#getColumnIndex}
 * @ignore
 * @inheritDoc
 */
LayoutGrid.prototype.getColumnIndex = function (colElem) {
	if (colElem) {
		if(colElem["stretched"]) {
			return this._stretchedCells.getColumnIndex(colElem);
		}
		let len = this._columns.length;

		for (let i = 0; i < len; ++i) {
			if (this._columns[i].getElement() === colElem) {
				return i;
			}
		}
	}

	return -1;
};

/**
 * {@link ILayoutGrid#getCellIndex}
 * @ignore
 * @inheritDoc
 */
LayoutGrid.prototype.getCellIndex = function (colIndex, cellElem) {
	if (cellElem) {
		if(cellElem["stretched"]) {
			return this._stretchedCells.getRowIndex(cellElem);
		}

		let column = this._columns[colIndex];
		if(column) {
			return column.getCellIndex(cellElem);
		}
	}
	return -1;
};

/**
 * {@link ILayoutGrid#getIndex}
 * @inheritDoc
 */
LayoutGrid.prototype.getIndex = function () {
	return this._index;
};

/**
 * {@link ILayoutGrid#setIndex}
 * @ignore
 * @inheritDoc
 */
LayoutGrid.prototype.setIndex = function (index) {
	this._index = index;
};

/**
 * {@link ILayoutGrid#getRowOffset}
 * @ignore
 * @inheritDoc
 */
LayoutGrid.prototype.getRowOffset = function () {
	return this._rowOffset;
};

/**
 * {@link ILayoutGrid#setRowOffset}
 * @ignore
 * @inheritDoc
 */
LayoutGrid.prototype.setRowOffset = function (index) {
	this._rowOffset = index;
};

/** @public
* @ignore
*/
LayoutGrid.prototype.updateLayout = function () {
	this._calculateViewSize(true); // Column bounds will be updated by trigger from Core

	this._stretchedCells.updateCells();
	this._updateRightSpaceStyle();
};

/**
 * {@link ILayoutGrid#stretchCell}
 * @inheritDoc
 */
LayoutGrid.prototype.stretchCell = function (cellRef, rowIndex, opt_stretching, onlyToTheRight) {
	let colIndex = -1;
	if (typeof cellRef === "number") {
		colIndex = cellRef;
		if(rowIndex == null) {
			rowIndex = 0;
		}
	} else if (cellRef) {
		if(cellRef["stretched"]) {
			colIndex = this._stretchedCells.getColumnIndex(cellRef);
			if(rowIndex == null) {
				rowIndex = this._stretchedCells.getRowIndex(cellRef);
			}
		} else if(cellRef["getParent"]) {
			colIndex = this.getColumnIndex(cellRef["getParent"]());
		}
	}
	if(colIndex < 0 || rowIndex < 0 || rowIndex == null) {
		return null;
	}

	let stretching = (opt_stretching !== false);
	let cell = null;

	if (stretching) {
		let coverLayer = this._coverLayer;
		if(!coverLayer) {
			coverLayer = this._coverLayer = document.createElement("div");
			coverLayer.className = "cover-layer";
			this._updateLayers();
		}

		this._calculateViewSize();
		cell = this._stretchedCells.stretchCell(rowIndex, onlyToTheRight ? colIndex : 0);
		cell.setParent(coverLayer);
	} else {
		cell = this._stretchedCells.unstretchCell(rowIndex);
	}
	if(cell) {
		let selected = this._selectionList.getSelection(rowIndex);
		cell.enableClass("selected-row", selected); // It's can enable class without get stretch cell again.
	}
	return cell;
};

/**
 * {@link ILayoutGrid#unstretchCell}
 * @inheritDoc
 */
LayoutGrid.prototype.unstretchCell = function (cellRef, rowIndex) {
	return this.stretchCell(cellRef, rowIndex, false);
};
/**
 * {@link ILayoutGrid#unstretchAllCells}
 * @inheritDoc
 */
LayoutGrid.prototype.unstretchAllCells = function () {
	this._stretchedCells.unstretchAllCells();
};

/**
 * {@link ILayoutGrid#getStretchedCell}
 * @inheritDoc
 */
LayoutGrid.prototype.getStretchedCell = function (cellRef, rowIndex) {
	if(rowIndex == null) {
		if(typeof cellRef === "number") {
			rowIndex = cellRef;
		} else {
			rowIndex = this._stretchedCells.getRowIndex(cellRef);
		}
	}

	return (typeof rowIndex === "number") ? this._stretchedCells.getCell(rowIndex) : null;
};
/**
 * {@link ILayoutGrid#getStretchedCells}
 * @inheritDoc
 */
LayoutGrid.prototype.getStretchedCells = function () { // TODO: Deprecate this method
	return this._stretchedCells.getCells();
};

/** {@link ILayoutGrid#updateStretchedCells}
* @inheritDoc */
LayoutGrid.prototype.updateStretchedCells = function () {
	this._updateLayers();
	let childCount = this._coverLayer ? this._coverLayer.children.length : 0; // TODO: Currently assume only stretched cells exists
	if (childCount <= 0) {
		return false;
	}
	this._calculateViewSize(true);

	this._stretchedCells.updateCells();
	return childCount ? true : false;
};

/** @private
* @param {boolean=} forceRecal
* @return {number}
*/
LayoutGrid.prototype._calculateViewSize = function (forceRecal) {
	if(this._frozenLayout) {
		return this._stretchedCells.getStretchSize();
	}

	let ctx = this._ctx;
	let stretchSize = forceRecal ? 0 : this._stretchedCells.getStretchSize();
	if(!stretchSize && ctx) {
		if(ctx["getWidth"]) {
			let contentWidth = this.getContentWidth();
			stretchSize = ctx["getWidth"]() - this._rightSpaceSize;
			if(stretchSize) { // Grid may not be in the document
				if(contentWidth < stretchSize) { // If view width is larger than actual content
					stretchSize = contentWidth;
				}
				this._stretchedCells.setStretchSize(stretchSize);
				return stretchSize;
			} else {
				return contentWidth;
			}
		}
	}
	return stretchSize;
};

/**
* @private
* @param {number} rowIndex
* @param {string} className
* @param {boolean} enabled
*/
LayoutGrid.prototype._enableStretchCellClass = function (rowIndex, className, enabled ) {
	let stretchEl = this.getStretchedCell(rowIndex); // check stretched cell
	if(stretchEl) {
		stretchEl.enableClass(className, enabled);
	}
};
/** View size is width of container (Grid's pane) or content (LayoutGrid's columns), whichever is smaller.
* @public
* @ignore
* @return {number}
*/
LayoutGrid.prototype._getViewSize = function () {
	return this._calculateViewSize();
};

/**
 * {@link ILayoutGrid#getColumnHost}
 * @inheritDoc
 */
LayoutGrid.prototype.getColumnHost = function () {
	return this._element;
};

/**
 * {@link ILayoutGrid#setHidingMethod}
 * @inheritDoc
 */
LayoutGrid.prototype.setHidingMethod = function (method) {
	this._hidingMethod = method;
};

/**
 * {@link ILayoutGrid#getHidingMethod}
 * @inheritDoc
 */
LayoutGrid.prototype.getHidingMethod = function () {
	return this._hidingMethod;
};

/**
 * {@link ILayoutGrid#setColumnVisibility}
 * @inheritDoc
 */
LayoutGrid.prototype.setColumnVisibility = function (colIndex, bool, flag) {
	if (this._trackX.hideLane(colIndex, !bool, flag)) {
		this._syncLayoutToColumns(colIndex);

		return true;
	}

	return false;
};

/**
 * {@link ILayoutGrid#setColumnVisibility}
 * @inheritDoc
 */
LayoutGrid.prototype.getRowLayout = function () {
	return this._trackY;
};

/**
 * {@link ILayoutGrid#reserveRightSpace}
 * @inheritDoc
 */
LayoutGrid.prototype.reserveRightSpace = function (size) {
	if (size && size > 0) {
		if (!this._rightSpace) {
			let rs = this._rightSpace = document.createElement("div");
			rs.className = "tr-reserved-space";
			rs.style.height = this.getContentHeight() + "px";

			// warning: The element's style is not completed by this function
			// so this method should not be called directly in the application level.
			this._updateLayers();
		}

		this._rightSpace.style.width = size + "px";
		this._rightSpaceSize = size;

		this._updateCssWidth();
	} else if (this._rightSpace) {
		let pn = this._rightSpace.parentNode;

		if (pn) {
			pn.removeChild(this._rightSpace);
		}

		this._rightSpace = null;
		this._rightSpaceSize = 0;

		this._updateCssWidth();
	}

	return this._rightSpace;
};

/** @public
* @ignore
*/
LayoutGrid.prototype._updateRightSpaceStyle = function() {
	let rs = this._rightSpace;
	if(!rs) {
		return;
	}
	let isScrollbarActive = false;
	let pinnedRgtCount = 0;
	if(this._hscrollbar) {
		isScrollbarActive = this._hscrollbar.isActive();
		pinnedRgtCount = this._hscrollbar.getPinnedRightColumnCount();
	}
	let absolutePos = isScrollbarActive && !pinnedRgtCount; // if true, the right space element will be positioned absolutely

	if (absolutePos) {
		rs.style.top = this._trackY.getLaneStart(this._activeRow) + "px";
		rs.style.left = this._trackX.getLaneStart(this._colCount) + "px";
		rs.style.position = "";
		rs.style.display = "";
	} else {
		rs.style.position = "static";
		rs.style.display = "inline-block";
	}
};

/** @public
* @ignore
*/
LayoutGrid.prototype._updateLayers = function() {
	if(this._rightSpace) {
		let pn = this._rightSpace.parentNode; // rightSpace can be located in two places: layout grid and pane slider
		if(pn) {
			pn.appendChild(this._rightSpace);
		} else {
			this._element.appendChild(this._rightSpace);
		}
	}
	if(this._boundLayer) {
		this._element.appendChild(this._boundLayer);
	}
	if(this._coverLayer) {
		this._element.appendChild(this._coverLayer); // The layer should always on top
	}
};

/**
 * {@link ILayoutGrid#reserveRightSpace}
 * @inheritDoc
 */
LayoutGrid.prototype.getReservedSpace = function () {
	return this._rightSpace;
};

/**
 * {@link ILayoutGrid#getComputedStyle}
 * @inheritDoc
 */
LayoutGrid.prototype.getComputedStyle = function () {
	let elem = this._element;

	return elem.offsetParent ? getComputedStyle(elem) : null;
};

/**
 * {@link ILayoutGrid#isColumnActive}
 * @inheritDoc
 */
LayoutGrid.prototype.isColumnActive = function (colIndex) {
	let column = this._columns[colIndex];

	return column ? column.isActive() : false;
};

/**
 * {@link ILayoutGrid#setRowAttributes}
 * @inheritDoc
 */
LayoutGrid.prototype.setRowAttributes = function (rowIndex, func) {
	if (rowIndex < 0 || (rowIndex >= this._rowCount)) {
		return;
	}

	for (let c = 0; c < this._colCount; ++c) {
		let result = func(c);
		for (let i = result.length; --i >= 0;) {
			let entry = result[i];
			let cell = this._columns[c].getCell(rowIndex);

			if (!cell) {
				continue;
			}

			if (entry["remove"]) {
				cell.removeAttribute(entry["attribute"]);
			} else {
				cell.setAttribute(entry["attribute"], entry["value"]);
			}
		}
	}
};
/** @public
* @param {number} rowIndex
* @return {Object}
*/
LayoutGrid.prototype.getContextRow = function (rowIndex) {
	return this._ctxRows.getItem(rowIndex);
};

/** @public
* @param {number} colIndex
* @param {boolean=} selected
*/
LayoutGrid.prototype.selectColumn = function (colIndex, selected) {
	this.enableColumnClass(colIndex, "selected-column", selected);

	if(selected) {
		let boundLayer = this._boundLayer;
		if(!boundLayer) {
			this._initBoundLayer();
			this._updateLayers();
		}
	}
};
/** @public
* @param {number} colIndex
* @return {boolean}
*/
LayoutGrid.prototype.isSelectedColumn = function (colIndex) {
	return this.hasColumnClass(colIndex, "selected-column");
};
/** @public
* @param {number} colIndex
* @param {number} rowIndex
* @param {boolean=} selected
*/
LayoutGrid.prototype.selectCell = function (colIndex, rowIndex, selected) {
	let cell = this._getCell(colIndex, rowIndex);
	if(cell) {
		cell.enableClass("selection", selected);
	}
};
/** @public
* @param {number} colIndex
* @param {number} rowIndex
* @param {number} width
* @param {number} height
*/
LayoutGrid.prototype.setCellBounds = function (colIndex, rowIndex, width, height) {
	let boundLayer = this._initBoundLayer();
	let cellBoundPainter = this._cellBoundPainter;
	if(!cellBoundPainter) {
		cellBoundPainter = this._cellBoundPainter = new util_CellBoundPainter({
			boundLayer: boundLayer,
			layoutX: this._trackX,
			layoutY: this._trackY,
			hscrollbar: this._hscrollbar,
			calculateColumnBounds: this.calculateColumnBounds.bind(this)
		});
	}

	let colCount = this.getColumnCount();
	cellBoundPainter.setCellBounds(colIndex, rowIndex, width, height, colCount);
};
/** @public
* @ignore
* @return {!TrackLayout}
*/
LayoutGrid.prototype.getHorizontalLayout = function () {
	return this._trackX;
};
/** @public
* @ignore
* @param {number} lftIdx
* @param {number} rgtIdx Inclusive index
* @param {!Array.<number>} outPositions
* @param {!Array.<boolean>} outNoBorders
*/
LayoutGrid.prototype.calculateColumnBounds = function (lftIdx, rgtIdx, outPositions, outNoBorders) {
	let lftPx = 0;
	let rgtPx = 0;
	let noLftBorder = false;
	let noRgtBorder = false;
	let pinnedRgtIdx = this._trackX.getLaneCount();
	if(lftIdx >= 0) {
		lftPx = this._trackX.getLaneStart(lftIdx);
		rgtPx = this._trackX.getLaneEnd(rgtIdx);
	}
	if(lftPx < rgtPx) {
		let lftView = 0;
		let rgtView, rgtLimit;
		let scrollLeft = 0;
		let pinnedLftCount = 0;
		let pinnedRgtCount = 0;
		if(this._hscrollbar) {
			scrollLeft = this._hscrollbar.getScrollLeft();
			pinnedLftCount = this._hscrollbar.getPinnedLeftColumnCount();
			pinnedRgtCount = this._hscrollbar.getPinnedRightColumnCount();
		}
		rgtView = rgtLimit = this._trackX.getTrackSize();
		if(pinnedLftCount) {
			lftView = this._trackX.getLaneStart(pinnedLftCount);
		}
		if(pinnedRgtCount) {
			pinnedRgtIdx -= pinnedRgtCount;
			rgtView = this._getViewSize() - rgtLimit + this._trackX.getLaneStart(pinnedRgtIdx);
		}
		if(lftIdx >= pinnedLftCount && lftIdx < pinnedRgtIdx) {
			lftPx -= scrollLeft;
			if(lftPx < lftView) {
				lftPx = lftView;
				noLftBorder = true;
			} else if(lftPx > rgtView) {
				lftPx = rgtView;
				noLftBorder = true;
			}
		} else if(lftIdx >= pinnedRgtIdx) {
			lftPx = this._getViewSize() - rgtLimit + this._trackX.getLaneStart(lftIdx);
		}
		if(rgtIdx >= pinnedLftCount && rgtIdx < pinnedRgtIdx) {
			rgtPx -= scrollLeft;
			if(rgtPx < lftView) {
				rgtPx = lftView;
				noRgtBorder = true;
			} else if(rgtPx > rgtView) {
				rgtPx = rgtView;
				noRgtBorder = true;
			}
		} else if(rgtIdx >= pinnedRgtIdx) {
			rgtPx = this._getViewSize() - rgtLimit + this._trackX.getLaneEnd(rgtIdx);
		}
	}
	outPositions[0] = lftPx;
	outPositions[1] = rgtPx;
	outNoBorders[0] = noLftBorder;
	outNoBorders[1] = noRgtBorder;
};
/** @public
* @ignore
* @param {!Array.<Array>} posAry Left and right bound positions in pixel
* @param {!Array.<Array>} noBorderAry Boolean values indicating existence of left and right CSS borders
* @param {number=} topPx Top position of bound
*/
LayoutGrid.prototype.updateColumnBounds = function (posAry, noBorderAry, topPx) {
	let cbs = this._colBounds;
	let cbc = this._colBoundCache;
	if(!cbs) {
		cbs = this._colBounds = [];
	}
	if(!cbc) {
		cbc = this._colBoundCache = [];
	}

	let rangeCount = posAry.length;
	let i;
	let pn = null; // parentNode
	let columnBound = null;

	// Remove unused bounds from document
	let activeCount = cbs.length;
	for(i = rangeCount; i < activeCount; ++i) {
		columnBound = cbs[i];
		pn = columnBound.parentNode;
		if(pn) {
			pn.removeChild(columnBound);
		}
	}
	cbs.length = activeCount = rangeCount;

	if(!rangeCount) {
		return;
	}

	for(i = 0; i < rangeCount; ++i) {
		let positions = posAry[i];
		let noBorders = noBorderAry[i];
		let lftPx = /** @type{number} */(positions[0]);
		let rgtPx = /** @type{number} */(positions[1]);

		columnBound = cbc[i];
		if(!columnBound) {
			columnBound = cbc[i] = document.createElement("div");
			columnBound.className = "selection-bound column-bound";
		}
		columnBound.style.left = lftPx + "px";
		columnBound.style.width = (rgtPx - lftPx) + "px";

		topPx = topPx || 0;
		columnBound.style.top = topPx + "px";
		columnBound.style.height = (this._trackY.getTrackSize() - topPx) + "px";
		columnBound.classList.toggle("no-left-bound", noBorders[0]);
		columnBound.classList.toggle("no-right-bound", noBorders[1]);
		if(this._boundLayer) {
			if(!cbs[i]) {
				cbs[i] = columnBound;
				this._boundLayer.appendChild(columnBound);
			}
		}
	}
};

/** @public
*/
LayoutGrid.prototype.updateColumnSeparators = function () {
	let pinnedLeftCount = this._hscrollbar.getPinnedLeftColumnCount();
	let pinnedRightCount = this._hscrollbar.getPinnedRightColumnCount();
	if ((pinnedLeftCount || pinnedRightCount) && !this._boundLayer) {
		this._initBoundLayer();
	}

	let isScrollbarActive = false;
	if(this._hscrollbar) {
		isScrollbarActive = this._hscrollbar.isActive();
	}

	let boundLayer = this._boundLayer;

	let colSeparator = this._leftColumnSeparator;
	if (isScrollbarActive && pinnedLeftCount) {
		if (!colSeparator) {
			colSeparator = this._leftColumnSeparator = this._createColumnSeparator();
		}
		if (!colSeparator.parentNode) {
			if (boundLayer.children.length) {
				boundLayer.insertBefore(colSeparator, boundLayer.children[0]);
			} else {
				boundLayer.appendChild(colSeparator);
			}
		}

		let rightPos = this._trackX.getLaneStart(pinnedLeftCount);
		colSeparator.style.left = (rightPos - 1) + "px";
		colSeparator.style.height = this._trackY.getTrackSize() + "px";
	} else {
		if (colSeparator && colSeparator.parentNode) {
			this._boundLayer.removeChild(colSeparator);
		}
	}

	colSeparator = this._rightColumnSeparator;
	if (isScrollbarActive && pinnedRightCount) {
		if (!colSeparator) {
			colSeparator = this._rightColumnSeparator = this._createColumnSeparator();
		}

		if (!colSeparator.parentNode) {
			if (boundLayer.children.length) {
				boundLayer.insertBefore(colSeparator, boundLayer.children[0]);
			} else {
				boundLayer.appendChild(colSeparator);
			}
		}
		let colCount = this.getColumnCount();
		let colWidth = this._trackX.getLaneEnd(colCount - 1) - this._trackX.getLaneStart(colCount - pinnedRightCount);
		let viewSize = this._getViewSize();

		colSeparator.style.left = (viewSize - colWidth) + "px";
		colSeparator.style.height = this._trackY.getTrackSize() + "px";
	} else {
		if (colSeparator && colSeparator.parentNode) {
			this._boundLayer.removeChild(colSeparator);
		}
	}
};
/** @private
* @return {Element}
*/
LayoutGrid.prototype._createColumnSeparator = function() {
	let colSeparator = document.createElement("div");
	colSeparator.classList.add("column-separator");
	return colSeparator;
};

/** @private
* @return {Element}
*/
LayoutGrid.prototype._initBoundLayer = function () {
	let boundLayer = this._boundLayer;
	if(!boundLayer) {
		boundLayer = this._boundLayer = document.createElement("div");
		boundLayer.className = "cover-layer";
		this._element.appendChild(boundLayer);
	}
	return boundLayer;
};

/**
 * @private
 * @param {number} indexX
 * @param {number} indexY
 * @return {Cell}
 */
LayoutGrid.prototype._getCell = function (indexX, indexY) {
	let column = this._columns[indexX];

	return (column) ? column.getCell(indexY) : null;
};

/**
 * @private
 * @param {number} from
 * @param {number} to
 */
LayoutGrid.prototype._insertCells = function (from, to) {
	this._activeRow = from;
	this._activeRowEnd = to;

	let top = this._trackY.getLaneStart(from);

	for (let c = 0; c < this._colCount; ++c) {
		let column = this._columns[c];

		if (!column.hasCollapsedCells()) {
			column.setTop(top);
		}

		column.activateCells(from, to);
	}

	if (this._rightSpace) {
		this._rightSpace.style.top = top + "px";
	}

	if (to > this._availableRowCount) {
		let e = {};

		e["prevRowIndex"] = this._availableRowCount;
		e["rowIndex"] = to;

		this._availableRowCount = to;
		this._dispatch("rowAvailable", e);
	}
};

/**
 * @private
 * @param {number} indexX
 * @return {!ElementWrapper}
 */
LayoutGrid.prototype._newColumn = function (indexX) {
	let column = new components_Column(this._virOwner || this);

	column.setHeight(this._trackY.getDefaultLaneSize());
	column.setCellCount(this._rowCount);
	column.setTop(this._trackY.getLaneStart(this._activeRow));
	column.activateCells(this._activeRow, this._activeRowEnd);

	for (let r = 0; r < this._rowCount; ++r) {
		if (!this._trackY.isAtDefaultSize(r)) {
			column.getCell(r).setHeight(this._trackY.getLaneSize(r));
		}
	}

	if (indexX >= this._columns.length) {
		this._columns.push(column);

		column.setParent(this._element);
	} else {
		this._columns.splice(indexX, 0, column);

		let nextColumn = this._columns[indexX + 1];

		column.insertBefore(nextColumn);
	}

	return column;
};

/** Update LayoutGrid CSS width. Grid width is calculated from summation of each column width plus reserved right space
 * @private
 */
LayoutGrid.prototype._updateCssWidth = function () { // This method is also called by _syncLayoutToColumns()
	if (!this._frozenLayout) {
		this._calculateViewSize(true);
		this._stretchedCells.updateXAxis();

		this._element.style.width = this._getWidth() + "px"; // View is kept in sync with content size
	}
};

/**
 * @private
 */
LayoutGrid.prototype._updateViewHeight = function () {
	if (this._frozenLayout) {
		return;
	}

	let contentHeight = this.getContentHeight();

	this._element.style.height = contentHeight + "px"; // View is kept in sync with content size

	if (this._rightSpace) {
		this._rightSpace.style.height = contentHeight + "px";
	}
};

/** @private
*/
LayoutGrid.prototype._updateColumnIndices = function () {
	let columnCount = this._columns.length;
	let first = this._columns[0];

	if (first !== this._firstColumn) {
		if (this._firstColumn) {
			this._firstColumn.removeClass("first");
		}

		this._firstColumn = first;

		if (this._firstColumn) {
			this._firstColumn.addClass("first");
		}
	}

	let last = this._columns[columnCount - 1];

	if (last !== this._lastColumn) {
		if (this._lastColumn) {
			this._lastColumn.removeClass("last");
		}

		this._lastColumn = last;

		if (this._lastColumn) { this._lastColumn.addClass("last"); }
	}

	let sibling = null;

	for (let i = this._columns.length; --i >= 0;) {
		let column = this._columns[i];

		column.setNextSibling(sibling);
		sibling = column;
	}
};

/**
 * @private
 * @param {number} from
 * @returns {boolean}
 */
LayoutGrid.prototype._syncLayoutToColumns = function (from) {
	if (this._frozenLayout) {
		return false;
	}

	if (this._rightSpace) { // Reserved right space must be updated regardless of the given index
		this._rightSpace.style.left = this._trackX.getLaneStart(this._colCount) + "px";
	}

	if (from >= this._colCount) {
		return false;
	}

	for (let c = from; c < this._colCount; ++c) {
		let column = this._columns[c];

		column.setLeft(this._trackX.getLaneStart(c));
		column.setWidth(this._trackX.getLaneSize(c));
	}

	this._updateCellSpanLayout(this._cellSpans.getAllSpans());
	this._updateCssWidth();
	return true;
};

/**
 * @private
 * @param {number} indexY
 * @param {number} toY Exclusive index of end cell in Y-Axis
 */
LayoutGrid.prototype._syncLayoutToRows = function (indexY, toY) {
	if (this._frozenLayout) {
		return;
	}

	for (let r = indexY; r < toY; ++r) {
		let height = (this._trackY.isAtDefaultSize(r)) ? -1 : this._trackY.getLaneSize(r);

		for (let c = 0; c < this._colCount; ++c) {
			this._columns[c].getCell(r).setHeight(height);
		}
	}

	this._stretchedCells.updateYAxis();
	if(indexY) {
		this._updateCellSpanLayout(this._cellSpans.getAllSpans());
	} else {
		this._updateCellSpans(this._cellSpans.getAllSpans(), true); // Re-collapsing cells when spans are set before setting row count
	}
	this._updateViewHeight();
};

/**
 * @private
 * @param {number} from
 * @param {number} to
 */
LayoutGrid.prototype._dispatchRowHeightChangedEvent = function (from, to) {
	if (!this._hasListener("rowHeightChanged")) {
		return;
	}

	if (from >= to) {
		return;
	}

	let e = {};

	e["rowIndex"] = from;
	e["changeLength"] = to - from;
	this._dispatch("rowHeightChanged", e);
};

/**
 * @private
 * @param {number} indexX
 * @param {number} indexY
 * @param {CellSpan} cellSpan
 */
LayoutGrid.prototype._updateCellSpanSize = function (indexX, indexY, cellSpan) {
	let cell = this._getCell(indexX, indexY);

	if (cell == null) {
		return;
	}

	if (cellSpan && cellSpan.colSpan > 1) {
		cell.setWidth(this.getCellWidth(indexX, indexY));
		this._columns[indexX].disableDeactivation(); // HACK: If a cell has column span, the column cannot be virtualized
	} else {
		cell.setWidth(-1);
	}

	if (cellSpan && cellSpan.rowSpan > 1) {
		cell.setHeight(this.getCellHeight(indexX, indexY)); /////
	} else {
		let height = this._trackY.isAtDefaultSize(indexY) ? -1 : this._trackY.getLaneSize(indexY);

		cell.setHeight(height);
	}
};

/**
 * @private
 * @param {Array.<CellSpan>} cellSpans
 */
LayoutGrid.prototype._updateCellSpanLayout = function (cellSpans) {
	if (cellSpans == null) { return; }

	for (let i = cellSpans.length; --i >= 0; ) {
		let cellSpan = cellSpans[i];

		this._updateCellSpanSize(cellSpan.indexX, cellSpan.indexY, cellSpan);
	}
};

/**
 * @private
 * @param {number} indexX
 * @param {number} indexY
 * @param {!CellSpan} cellSpan
 * @param {boolean} adding
 */
LayoutGrid.prototype._updateCellSpan = function (indexX, indexY, cellSpan, adding) {
	this._updateCellSpanSize(indexX, indexY, adding ? cellSpan : null);
	this._updateCellSpanClass(indexX, indexY, cellSpan.colSpan, cellSpan.rowSpan, adding);
};

/**
 * @private
 * @param {number} indexX
 * @param {number} indexY
 * @param {number} colSpan
 * @param {number} rowSpan
 * @param {boolean} adding
 */
LayoutGrid.prototype._updateCellSpanClass = function (indexX, indexY, colSpan, rowSpan, adding) {
	let column;

	for (let c = 0; c < colSpan; ++c) {
		column = this._columns[indexX + c];

		if (!column) {
			break;
		}

		column.collapseCells(indexY, rowSpan, adding, (c === 0), (c > 0));
	}

	let cell = this._getCell(indexX, indexY);

	if (cell) {
		cell._colSpan = (adding) ? colSpan : 1;
		cell._rowSpan = (adding) ? rowSpan : 1; // For quick rendering
	}
};

/**
 * @private
 * @param {Array.<CellSpan>} cellSpans
 * @param {boolean} adding
 */
LayoutGrid.prototype._updateCellSpans = function (cellSpans, adding) {
	if (cellSpans == null) { return; }

	for (let i = cellSpans.length; --i >= 0; ) {
		let cellSpan = cellSpans[i];

		this._updateCellSpan(cellSpan.indexX, cellSpan.indexY, cellSpan, adding);
	}
};

/**
 * @private
 * @param {Object} e
 */
LayoutGrid.prototype._onMouseMove = function (e) {
	let target = e["target"];

	let cellElement = util.closestElement(target, "cell");
	let colIndex = this._stretchedCells.getColumnIndex(cellElement);
	if(colIndex < 0) { // Not found colIndex in stretching cell, then get from normal row
		let colElement = util.closestElement(target, "column");
		colIndex =	this.getColumnIndex(colElement);
	}
	let rowIndex = this.getCellIndex(colIndex, cellElement);

	this.setRowHighlight(rowIndex);
};

/**
 * @private
 * @param {Object} e
 */
LayoutGrid.prototype._onMouseOut = function (e) {
	let rel = e.relatedTarget;

	if (rel) {
		let thisElem = this._element;

		while (rel.parentNode !== null) {
			rel = rel.parentNode;

			if (rel === thisElem) {
				return;
			}
		}
	}

	this.setRowHighlight(-1);
};


/** @public
* @ignore
* @return {!Object}
*/
LayoutGrid.prototype._getEventHandlers = function () {
	return {
		"mouseout": this._onMouseOut,
		"mousemove": this._onMouseMove
	};
};



/**
 * @private
 * @param {number} rowIndex
 */
LayoutGrid.prototype._updateSelectionUI = function (rowIndex) { // Update UI of the specified row index
	let selected = this._selectionList.getSelection(rowIndex);
	this._enableStretchCellClass(rowIndex, "selected-row", selected);
	this.enableRowClass(rowIndex, "selected-row", selected);
};

/** @private
* @param {number} rowIndex
*/
LayoutGrid.prototype._addSelectionUI = function (rowIndex) {
	this._enableStretchCellClass(rowIndex, "selected-row", true);
	this.enableRowClass(rowIndex, "selected-row", true);
};

/**
 * @private
 * @param {number} rowIndex
 */
LayoutGrid.prototype._removeSelectionUI = function (rowIndex) {
	this._enableStretchCellClass(rowIndex, "selected-row", false);
	this.enableRowClass(rowIndex, "selected-row", false);
};

/**
 * @public
 * @ignore
 */
LayoutGrid.prototype._onEnterDocument = function () {
	if (!this._initialized && this.getParent()) {
		for (let c = 0; c < this._colCount; ++c) {
			this._columns[c].enableAutoStyleUpdating();
		}

		this._syncLayoutToColumns(0);
		this._initialized = true;
	}
};

/** @public
* @ignore
* @param {*} ctx
*/
LayoutGrid.prototype._setContext = function(ctx) {
	this._ctx = ctx;
};

/** @public
* @ignore
* @return {*} ctx
*/
LayoutGrid.prototype._getContext = function () {
	return this._ctx;
};

/** This is a WORKAROUND to fix Internet Explorer couldn't render top 50% with translateX -50% correctly
 * @private
*/
LayoutGrid.prototype._resetTransformIETimer = function () {
	if (util.isIE) {
		if(!this._transformIETimer) {
			this.enableClass("reset-transform");
			this._transformIETimer = setTimeout(this._onResetTransformIE, 20);
		}
	}
};
/** @private
*/
LayoutGrid.prototype._onResetTransformIE = function () {
	this._transformIETimer = 0;
	this.enableClass("reset-transform", false);
};

LayoutGrid._proto = LayoutGrid.prototype;

/* harmony default export */ const grid_LayoutGrid = (LayoutGrid);


;// ./node_modules/@grid/core/es6/data/ColumnStats.js

/** @constructor
* @param {*=} source
* @param {string=} field
*/
let ColumnStats = function(source, field) {
	this._onDataChanged = this._onDataChanged.bind(this);

	this._refs = [];
	if(source) {
		this._dt = source;
		this._dt["listen"]("dataChanged", this._onDataChanged, 0); // Setting priority to be prioritized first
	}
	if(field) {
		this._field = field;
	}
};

//#region Public
/** @public
*/
ColumnStats.prototype.dispose = function() {
	if(this._dt) {
		this._dt["unlisten"]("dataChanged", this._onDataChanged);
		this._dt = null;
	}
};
/** @public
* @param {*} source
*/
ColumnStats.prototype.setDataSource = function(source) {
	if(this._dt === source) { return; }

	if(this._dt) {
		this._dt["unlisten"]("dataChanged", this._onDataChanged);
	}

	this._dt = source;

	if(this._dt) {
		this._dt["listen"]("dataChanged", this._onDataChanged, 0); // Setting priority to be prioritized first
	}

	this._requestRecalculation();
};
/** @public
* @param {string} field
*/
ColumnStats.prototype.setField = function(field) {
	if(this._field !== field) {
		this._field = field;

		this._requestRecalculation();
	}
};
/** @public
* @param {boolean=} opt_disabled
*/
ColumnStats.prototype.disableOnDemandMode = function(opt_disabled) {
	let enabled = opt_disabled === false;
	if(this._onDemandMode !== enabled) {
		this._onDemandMode = enabled;
		this._requestRecalculation();
	}
};
/** @public
* @param {Function} func
*/
ColumnStats.prototype.setColumnDataGetter = function(func) {
	if(this._colDataGetter !== func) {
		this._colDataGetter = func;
		this._requestRecalculation();
	}
};
/** @public
* @return {Function}
*/
ColumnStats.prototype.getColumnDataGetter = function() {
	return this._colDataGetter;
};
/** @public
* @return {number} Summation of all values
*/
ColumnStats.prototype.getSum = function() {
	this._updateStats();

	return this._sum;
};
/** @public
* @return {number} The count of valid data, excluding all invalid data such as undefined, null, string, and NaN
*/
ColumnStats.prototype.getDataCount = function() {
	this._updateStats();

	return this._count;
};

/** @public
* @return {number} Minimum value
*/
ColumnStats.prototype.getMin = function() {
	this._updateStats();

	return this._min;
};
/** @public
* @return {number} Maximum value
*/
ColumnStats.prototype.getMax = function() {
	this._updateStats();

	return this._max;
};
/** @public
* @return {number} Maximum absolute value
*/
ColumnStats.prototype.getAbsoluteMax = function() {
	this._updateStats();

	let absMax = (this._max >= 0) ? this._max : -this._max;
	let absMin = (this._min >= 0) ? this._min : -this._min;

	return (absMax >= absMin) ? absMax : absMin;
};

/** @public
* @return {number} Average. NaN is returned if data count is zero.
*/
ColumnStats.prototype.getAverage = function() {
	this._updateStats();

	return this._sum / this._count;
};
/** @function
* @public
* @return {number} Mean. NaN is returned if data count is zero.
*/
ColumnStats.prototype.getMean = ColumnStats.prototype.getAverage;
/** @public
* @return {number} Mid-extreme. The arithmetic mean of the maximum and minimum values in a data set. NaN is returned if there is no valid data.
*/
ColumnStats.prototype.getMidRange = function() {
	this._updateStats();

	return (this._max + this._min) * 0.5;
};
/** @public
* @return {Array.<number>}
*/
ColumnStats.prototype.getCachedColumnData = function() {
	if(this._dirty) {
		this._recalculate();
	} else if(!this._cacheData) {
		this._cacheData = this._getColData();
	}

	return this._cacheData;
};

/** Mark this ColumnStats object as dirty to force re-calculation next time the data is retrieved
* @public
* @return {boolean}
*/
ColumnStats.prototype.markDirty = function() {
	if(this._onDemandMode) {
		this._dirty = true;
		return this._dirty;
	}
	return false;
};

//#endregion Public

//#region Private
/** @private
* @return {Array.<number>}
*/
ColumnStats.prototype._getColData = function() {
	if(this._colDataGetter) {
		return /** @type{Array.<number>} */(this._colDataGetter(this._field, this._dt));
	}

	return /** @type{Array.<number>} */(this._dt["getColumnData"](this._field));
};
/** @private
*/
ColumnStats.prototype._requestRecalculation = function() {
	if(!this.markDirty()) {
		this._recalculate();
	}
};
/** @private
*/
ColumnStats.prototype._updateStats = function() {
	if(this._dirty) {
		this._recalculate();
	}
};
/** @private
*/
ColumnStats.prototype._recalculate = function() {
	this._dirty = false;
	this._cacheData = null;
	this._sum = 0;
	this._min = NaN;
	this._max = NaN;
	this._count = 0;

	if (!this._dt || !this._field) {
		return;
	}

	let cacheData = this._cacheData = this._getColData();
	let len = cacheData.length;

	if (!len) {
		return;
	}

	let isDataNumeric = ColumnStats._isDataNumeric;

	let i = 0;
	let data;

	while (i < len) { // Find first valid index for max and min value
		data = +cacheData[i++];

		if (isDataNumeric(data)) {
			this._sum = this._min = this._max = data;
			++this._count;

			break;
		}
	}

	while (i < len) {
		data = +cacheData[i];

		if (isDataNumeric(data)) {
			this._sum += data;

			if (data < this._min) {
				this._min = data;
			} else if (data > this._max) {
				this._max = data;
			}

			++this._count;
		}

		++i;
	}
};

/** @private
* @param {Object} e
*/
ColumnStats.prototype._onDataChanged = function(e) {
	if(e["rid"]) {
		if(!this._colDataGetter && e["changes"]) {
			if(e["changes"][this._field] == null) {
				return; // No change in the interested field
			}
		}

		if(this.markDirty()) {
			return;
		}

		this._updateRowData(e["rid"]);
	} else if(e["rids"]) {
		if(this.markDirty()) {
			return;
		}

		let rids = e["rids"];
		if(rids.length === 1) {
			this._updateRowData(rids[0]);
		} else {
			this._recalculate();
		}
	} else if(e["globalChange"]) {
		if(e["positionChangeOnly"]) {
			this._cacheData = null;
			return; // No data change in this update
		}

		this._requestRecalculation();
	}
};
/** @private
 * @param {string} rid
 * @returns {void}
*/
ColumnStats.prototype._updateRowData = function(rid) {
	let prevData = this._dt["getPreviousData"](rid, this._field);
	let newData = this._dt["getData"](rid, this._field);
	if(prevData === newData) {
		return;
	}

	let newValid = ColumnStats._isDataNumeric(newData);

	if(ColumnStats._isDataNumeric(prevData)) {
		if(this._min === prevData) {
			if(!newValid || newData > this._min) { // New value is actually greater than previous data
				this._recalculate();
				return;
			}
		}
		if(this._max === prevData) {
			if(!newValid || newData < this._max) { // New value is actually lesser than previous data
				this._recalculate();
				return;
			}
		}

		this._sum -= prevData;
		--this._count;
	}

	if(newValid) {
		this._sum += newData;
		if(!(newData >= this._min)) { // this._min can be NaN
			this._min = newData;
		}
		if(!(newData <= this._max)) { // this._max can be NaN
			this._max = newData;
		}
		++this._count;
	}
};
/**
* @private
* @function
* @param {*} data
* @return {boolean}
*/
ColumnStats._isDataNumeric = function(data) {
	return (typeof data === "number") && (data === data);
};

/** @public
* @ignore
* @param {string=} ref
* @return {number}
*/
ColumnStats.prototype.addRef = function(ref) {
	if(ref) {
		if(this._refs.indexOf(ref) < 0) {
			this._refs.push(ref);
		}
	}
	return this._refs.length;
};
/** @public
* @ignore
* @param {string=} ref
* @return {number}
*/
ColumnStats.prototype.deRef = function(ref) {
	if(ref) {
		let at = this._refs.indexOf(ref);
		if(at >= 0) {
			this._refs.splice(at, 1);
		}
	}
	return this._refs.length;
};
/** @public
* @ignore
* @return {number}
*/
ColumnStats.prototype.getRefCount = function() {
	return this._refs.length;
};


/** @private
* @type {*}
*/
ColumnStats.prototype._dt = null;
/** @private
* @type {string}
*/
ColumnStats.prototype._field = "";
/** @private
* @type {boolean}
*/
ColumnStats.prototype._onDemandMode = true;
/** @private
* @type {boolean}
*/
ColumnStats.prototype._dirty = true;

/** @private
* @type {Array.<number>}
*/
ColumnStats.prototype._cacheData = null;

/** @private
* @type {number}
*/
ColumnStats.prototype._sum = 0;
/** @private
* @type {number}
*/
ColumnStats.prototype._min = NaN;
/** @private
* @type {number}
*/
ColumnStats.prototype._max = NaN;
/** @private
* @type {number}
*/
ColumnStats.prototype._count = 0;
/** @private
* @type {Function}
*/
ColumnStats.prototype._colDataGetter = null;
/** @private
* @type {!Array.<string>}
*/
ColumnStats.prototype._refs;
//#endregion Private

/* harmony default export */ const data_ColumnStats = (ColumnStats);


;// ./node_modules/@grid/core/es6/data/WrappedView.js


 // eslint-disable-line
 // eslint-disable-line
 // eslint-disable-line
 // eslint-disable-line

/** @event WrappedView#dataChanged
* @description Trigger when data within the data view has been changed. Not only actual change in data, but also position change will trigger this event.
* @property {boolean} globalChange Indicates a big change. User should expect all data has been change. With active sorting or filtering, this flag will always be true due to multiple changes in row position.
* @property {string} type Type of changes. Possible values are "inserted", "removed", "updated", and undefined
* @property {string} rid Row Id
* @property {Object.<string, *>} rowData Column values of the changed row in JSON object format
*/

/** WrappedView doesn't hold the actual data. It is just a transform view (by applying wrap operation) of the data from the {@link DataView}
* @constructor
* @extends {EventDispatcher}
* @param {WrappedView~Options} options
*/
let WrappedView = function(options) {
	this._onDataChanged = this._onDataChanged.bind(this);
	this._onWrapSizeChanged = this._onWrapSizeChanged.bind(this);

	this._dv = options["dataView"];
	this._dt = this._dv.getDataTable();

	this._dv.listen("dataChanged", this._onDataChanged);
	this._dv.listen("wrapSizeChanged", this._onWrapSizeChanged);

	this._wrapIndex = options["wrapIndex"];

	this._refreshRids(options["rids"]);

	this._addEvents("dataChanged");
};
es6_Ext.inherits(WrappedView, event_EventDispatcher);

/** The configuration object for WrappedView
* @typedef {Object} WrappedView~Options
* @property {DataView} dataView
* @property {number} wrapIndex
* @property {Array.<string> rids
*/
WrappedView.Options;

/** @private
* @type {DataView}
*/
WrappedView.prototype._dv = null;
/** @private
* @type {DataTable}
*/
WrappedView.prototype._dt = null;
/** @private
* @type {number}
*/
WrappedView.prototype._wrapIndex = 0;
/** List of row Ids
* @private
* @type {!Array.<string>}
*/
WrappedView.prototype._rids;
/** @private
* @type {Conflator}
*/
WrappedView.prototype._dataConflator = null;


/** @private
* @param {Object} e
*/
WrappedView.prototype._onDataChanged = function(e) {
	if(this._dv) {
		let rids = this._dv.getVisibleRowIds(true);
		if(this._refreshRids(rids)) {
			if(!e["globalChange"]) {
				e["globalChange"] = true; // TODO: Optimize how we to dispatch globalChange
			}
			this._dispatch("dataChanged", e);
		}
	}
};
/** @private
* @param {Object} e
*/
WrappedView.prototype._onWrapSizeChanged = function(e) {
	if(this._refreshRids(e["rids"])) {
		this._dispatch("dataChanged", data_DataTable._globalChangeArg);
	}
};

/** @private
* @param {Array.<string>} rids
* @return {boolean} Returns true if there is any change
*/
WrappedView.prototype._refreshRids = function(rids) {
	let dirty = false;
	if(this._dv && Array.isArray(rids)) {
		let wrapSize = this.getWrapSize();
		if(wrapSize > 0) {
			let startIndex = wrapSize * this._wrapIndex;

			if(startIndex < rids.length) {
				dirty = true;
				this._rids = rids.slice(startIndex, startIndex + wrapSize);
			} else if(this._rids.length) {
				dirty = true;
				this._rids.length = 0;
			}
		}
	}
	return dirty;
};
/** @public
*/
WrappedView.prototype.dispose = function() {
	if(this._dv) {
		this.unlistenAll();
		this._dv.unlisten("dataChanged", this._onDataChanged);
		this._dv.unlisten("wrapSizeChanged", this._onWrapSizeChanged);
		this._dt = this._dv = null;
	}
};

/** Get data by row id and column id (field)
* @public
* @param {string} rid
* @param {string} cid
* @returns {Object.<string, *>}
*/
WrappedView.prototype.getData = function(rid, cid) {
	return this._dt.getData(rid, cid);
};
/** Get entire row data by row id
* @public
* @param {string} rid
* @return {Object.<string, *>}
*/
WrappedView.prototype.getRowData = function(rid) {
	return this._dt.getRowData(rid);
};
/** @public
* @param {string} cid
* @param {boolean=} inclNonDataRow=false If enabled, group header rows are included
* @return {!Array}
*/
WrappedView.prototype.getColumnData = function(cid, inclNonDataRow) {
	let rids = this.getAllRowIds(true, inclNonDataRow);

	let rows = [];
	let len = rids.length;
	for(let r = 0; r < len; ++r) {
		let rowData = this._dt.getRowData(rids[r]);
		if(rowData) {
			rows.push(rowData[cid]);
		} else if(inclNonDataRow) {
			rows.push(null);
		}
	}
	return rows;
};
/** @public
* @param {number} rowIndex
* @return {string}
* @see {@link WrappedView#getRowIndex}
*/
WrappedView.prototype.getRowId = function(rowIndex) {
	return this._rids[rowIndex] || "";
};
/** @private
* @param {number|string} rowRef
* @return {string}
*/
WrappedView.prototype._toRowId = function(rowRef) {
	if(typeof rowRef === "number") {
		return this._rids[rowRef] || "";
	}
	return /** @type{string} */(rowRef || "");
};
/** @private
* @param {Array.<number|string>} rowRefs
* @return {!Array.<string>}
*/
WrappedView.prototype._toRowIds = function(rowRefs) {
	let rowCount = Array.isArray(rowRefs) ? rowRefs.length : 0;
	let ary = new Array(rowCount);
	let rids = this._rids;
	for(let r = 0; r < rowCount; ++r) {
		let rowRef = rowRefs[r];
		if(typeof rowRef === "number") {
			ary[r] = rids[rowRef] || "";
		} else {
			ary[r] = rowRef || "";
		}
	}
	return ary;
};
/** Return all row ids with row data, excluding the group header rows (auto generated rows).
* @public
* @param {boolean=} shallowCopy=false If enabled, no new copy of Array is created (better for performance).
* @param {boolean=} inclNonDataRow=false If enabled, group header rows are included. In pagination mode, this flag will always be true.
* @return {!Array.<string>}
* @see {@link WrappedView#getAllRowData}
* @see {@link WrappedView#getMultipleRowData}
* @see {@link WrappedView#getSortedRowIds}
* @see {@link WrappedView#getVisibleRowIds}
*/
WrappedView.prototype.getAllRowIds = function(shallowCopy, inclNonDataRow) {
	return (shallowCopy) ? this._rids : this._rids.slice(0); // Fastest way to clone an array is Array.slice()
};
/** This method returns current visible rows (i.e. Rows in other pages or in collapsed groups are excluded). Group header rows may be included in the results.
* @public
* @param {boolean=} shallowCopy=false If enabled, no new copy of Array is created (better for performance).
* @return {!Array.<string>}
* @see {@link WrappedView#getAllRowData}
*/
WrappedView.prototype.getVisibleRowIds = function(shallowCopy) {
	return (shallowCopy) ? this._rids : this._rids.slice(0); // Fastest way to clone an array is Array.slice()
};
/** Return all data rows that is filtered and sorted, excluding header rows or paging. This method ignores pagination.
* @public
* @param {boolean=} shallowCopy=false If enabled, no new copy of Array is created (better for performance).
* @return {!Array.<string>}
* @see {@link WrappedView#getAllRowData}
*/
WrappedView.prototype.getSortedRowIds = function(shallowCopy) {
	return (shallowCopy) ? this._rids : this._rids.slice(0); // Fastest way to clone an array is Array.slice()
};
/** This method is quite slow
* @public
* @param {string} rid
* @return {number}
* @see {@link WrappedView#getRowId}
*/
WrappedView.prototype.getRowIndex = function(rid) {
	return this._rids.indexOf(rid);
};
/** @public
* @return {number}
*/
WrappedView.prototype.getRowCount = function() {
	return this._rids.length;
};
/** @public
* @param {number} rowIndex
* @param {string} cid
* @returns {Object.<string, *>}
*/
WrappedView.prototype.getDataAt = function(rowIndex, cid) {
	return this._dt.getData(this.getRowId(rowIndex), cid);
};
/** This method is slow. When working with huge amount of rows, use getRowData in combination with getAllRowIds(true) instead of calling getRowDataAt multiple times.
* @public
* @param {number} rowIndex
* @return {Object.<string, *>}
*/
WrappedView.prototype.getRowDataAt = function(rowIndex) {
	return this._dt.getRowData(this.getRowId(rowIndex));
};
/** @public
* @param {string} rid
* @param {string} cid
* @returns {Object.<string, *>}
*/
WrappedView.prototype.getPreviousData = function(rid, cid) {
	return this._dt.getPreviousData(rid, cid);
};
/** @public
* @param {Array.<string>} rids
* @param {number=} opt_from
* @param {number=} opt_to EXCLUSIVE
* @return {!Array.<Object>}
*/
WrappedView.prototype.getMultipleRowData = function(rids, opt_from, opt_to) {
	return this._dt.getMultipleRowData(rids, opt_from, opt_to);
};
/** Return all rows with row data, excluding the group header rows (auto generated rows)
* @public
* @return {!Array.<Object>}
*/
WrappedView.prototype.getAllRowData = function() {
	return this.getMultipleRowData(this.getAllRowIds(true));
};

/** @public
* @param {string} rid
* @param {string} cid
* @param {*} value
* @return {boolean} Return true if there is any change, and false otherwise
*/
WrappedView.prototype.setData = function(rid, cid, value) {
	if(this._dt) {
		return this._dt.setData(rid, cid, value);
	}
	return false;
};
/** @public
* @param {string} rid
* @param {Object.<string, *>=} values
* @param {Object=} opt_eventArg
* @return {boolean} Return true if there is any change, and false otherwise
*/
WrappedView.prototype.setRowData = function(rid, values, opt_eventArg) {
	return this._dv.setRowData(rid, values, opt_eventArg);
};
/** Set data in a column manner. This is faster than repeatedly calling {@link DataTable#setData}
* @public
* @param {string} cid
* @param {Array} valueList
* @param {Array.<string>=} opt_ridList Specify row id to be set corresponding to the data item
*/
WrappedView.prototype.setColumnData = function(cid, valueList, opt_ridList) {
	//  Note: This function should filter out row expansion before calling dt.setColumnData
	if(this._dt) {
		this._dt.setColumnData(cid, valueList, opt_ridList);
	}
};
/** This method is slow. When working with huge amount of rows, use setData in combination with getAllRowIds(true) instead of calling setDataAt multiple times.
* @public
* @param {number} rowIndex
* @param {string} cid
* @param {*} value
* @return {boolean} Return true if there is any change, and false otherwise
*/
WrappedView.prototype.setDataAt = function(rowIndex, cid, value) {
	let rid = this.getRowId(rowIndex);
	if(rid && this._dt) {
		return this._dt.setData(rid, cid, value);
	}
	return false;
};
/** This method is slow. When working with huge amount of rows, use setRowData in combination with getAllRowIds(true) instead of calling setRowDataAt multiple times.
* @public
* @param {number} rowIndex
* @param {Object.<string, *>=} values
* @param {Object=} opt_eventArg
* @return {boolean} Return true if there is any change, and false otherwise
*/
WrappedView.prototype.setRowDataAt = function(rowIndex, values, opt_eventArg) {
	let rid = this.getRowId(rowIndex);
	return this.setRowData(rid, values, opt_eventArg);
};

/** @public
* @param {string|number} rowRef The position of the new row
* @param {Object.<string, *>=} opt_values
* @param {string|null=} opt_rid If the given row Id already exists, data is updated and no new row is added. Leave blank or null to let DataTable generate unique row Id
* @return {string} Return Row id that has been inserted or changed
*/
WrappedView.prototype.insertRow = function(rowRef, opt_values, opt_rid) { // Data changed event may be dispatched
	let rowId = this._toRowId(rowRef);
	return this._dv.insertRow(rowId, opt_values, opt_rid);
};

/** Permanently remove the specified row by row id or WrappedView's row index. If you want to temporarily remove the row, use {@link WrappedView#hideRows} instead
* @public
* @param {number|string} rowRef
*/
WrappedView.prototype.removeRow = function(rowRef) {
	if(this._dt) {
		this._dt.removeRow(this._toRowId(rowRef)); // Trigger dataChanged
	}
};
/** Permanently remove the specified rows by row id or WrappedView's row index. If you want to temporarily remove the rows, use {@link WrappedView#hideRows} instead
* @public
* @param {Array.<number|string>} rowRefs
*/
WrappedView.prototype.removeRows = function(rowRefs) {
	if(this._dt) {
		this._dt.removeRows(this._toRowIds(rowRefs)); // Trigger dataChanged
	}
};
/** Move a single row or multiple rows to the position BEFORE the specified target row.
* @public
* @param {Array.<number|string>|number|string} fromRef The value could be row index, row id, or array of row indices or row ids.
* @param {number|string} toRef Destination target could be row index or row id
* @return {Array.<string>|string} rid rowId or array of the rowId of the moved row(s)
* @example
* ary = [0, 1, 2];
* move(ary, 0, 1); // [0, 1, 2] Move index 0 to the position before index 1 (nothing is changed)
* move(ary, 1, 0); // [1, 0, 2] Move index 1 to the position before index 0
*/
WrappedView.prototype.moveRow = function(fromRef, toRef) {
	let toRid = this._toRowId(toRef);
	let fromIndices = /** @type{!Array.<number>} */(Array.isArray(fromRef) ? fromRef : [fromRef]);
	let fromRids = this._toRowIds(fromIndices);

	// if(this._dt.hasSegmentation()) {
	// this.synchronizeRowOrder();
	// }

	return this._dv.moveRow(fromRids, toRid);
};

/** Returns the root data source (DataTable) of all data views
* @public
* @return {DataTable}
* @see {@link WrappedView#getDataSource}
*/
WrappedView.prototype.getDataTable = function() {
	return this._dt;
};
/** Return source for this data view which could be another data view or a data table.
* @public
* @return {DataView}
* @see {@link WrappedView#getDataTable}
*/
WrappedView.prototype.getDataSource = function() {
	return this._dv;
};
/** @public
* @param {DataView=} source
*/
WrappedView.prototype.setDataSource = function(source) {

};

/** @public
* @param {number} milliSecond
*/
WrappedView.prototype.setConflationRate = function (milliSecond) {
	this._dv.setConflationRate(milliSecond);
};

/** @public
* @return {number}
*/
WrappedView.prototype.getConflationRate = function () {
	return this._dv.getConflationRate();
};

/**
* @public
* @ignore
* @return {Object.<string, DataTable.SortLogic>}
*/
WrappedView.prototype.getSortingLogics = function() {
	return this._dt ? this._dt.getSortingLogics() : null;
};
/** @public
* @param {string|Array.<string>} cid
* @param {string|number|Array.<string|number>=} opt_sortOrder
* @param {Function|Array.<Function>=} sortLogics
* @param {boolean=} noOptimization=false Force to sort without optimization to refresh all rowIds again.
*/
WrappedView.prototype.sort = function (cid, opt_sortOrder, sortLogics, noOptimization) {
	this._dv.sort(cid, opt_sortOrder, sortLogics, noOptimization);
};
/** @public
* @param {string|Array.<string>} cid
* @param {string|number|Array.<string|number>=} opt_sortOrder
*/
WrappedView.prototype.sortOnce = function(cid, opt_sortOrder) {
	this._dv.sortOnce(cid, opt_sortOrder);
};
/** @public
* @param {DataTable.SortLogic} func
* @example
* customLogic = function(a, b, sortOrder) {
*   return (a - b) * sortOrder; // for numeric comparison
* };
* let dt = new DataTable();
* let dv = new DataView(dt);
* dv.setSortingLogic(customLogic);
* dv.sort("Column id", "ascending");
*/
WrappedView.prototype.setSortingLogic = function(func) {
	if(this._dt) {
		this._dt.setSortingLogic(func);
	}
};
/** Specified sorting logic for the column with the given column id. This will override logic set by {@link WrappedView#setSortingLogic}, when the column is sorted
* @public
* @param {string} cid
* @param {DataTable.SortLogic} func Use null to remove current sorting logic
*/
WrappedView.prototype.setColumnSortingLogic = function(cid, func) {
	if(this._dt) {
		this._dt.setColumnSortingLogic(cid, func);
	}
};
/** Check if this view is in sorting mode
* @public
* @return {boolean}
*/
WrappedView.prototype.isSorting = function() {
	return this._dv.isSorting();
};

/** Hide rows in the data view
 * @param {string|number} rId Row id to hide. If the rId is a number, it will be treated as a row index
 * @param {boolean=} opt_show show if false
 * @public
 */
WrappedView.prototype.hideRow = function(rId, opt_show) {
	this.showRows([rId], opt_show === false);
};
/** Hide rows in the data view
 * @param {Array.<string|number>} rowRefs Array of row ids or indices to hide
 * @param {boolean=} opt_show show if false
 * @public
 */
WrappedView.prototype.hideRows = function(rowRefs, opt_show) {
	this.showRows(rowRefs, opt_show === false);
};
/**
 * Show/hide rows in the data view
 * @param {Array.<string|number>} rowRefs Array of row ids or indices to hide
 * @param {boolean=} opt_show hide instead show if false
 * @public
 */
WrappedView.prototype.showRows = function(rowRefs, opt_show) {
	let rids = this._toRowIds(rowRefs);
	this._dv.showRows(rids, opt_show);
};
/** Unhide all rows
* @public
*/
WrappedView.prototype.unhideAllRows = function(){
	this._dv.unhideAllRows();
};
/** Check if there is any hidden row
* @public
* @return {boolean}
*/
WrappedView.prototype.hasHiddenRow = function(){
	return this._dv.hasHiddenRow();
};

/** @public
* @param {string|function(string, Object.<string, *>):boolean} cid Use string to identify column name. Use function for custom filter logic. Function will take rowId and column values as parameters, and return boolean value
* @param {*=} value
* @param {boolean=} opt_filteringOut
* @return {boolean} Return true if there is any change. Otherwise, return false.
* @example
* dv.filterIn("FIELD", "Some value");
* dv.filterIn(function(rowId, rowData)) { // This is equivalent with the above statement
*     return rowData["FIELD"] === "Some value";
* });
*/
WrappedView.prototype.filterIn = function(cid, value, opt_filteringOut) {
	return this._dv.filterIn(cid, value, opt_filteringOut);
};
/** @public
* @param {string|function(string, Object.<string, *>):boolean} cid Use string to identify column name. Use function for custom filter logic. Function will take rowId and column values as parameters, and return boolean value
* @param {*=} value
* @return {boolean} Return true if there is any change. Otherwise, return false.
* @example
* dv.filterIn("FIELD", "Some value");
* dv.filterOut(function(rowId, rowData)) { // This is equivalent with the above statement
*     return rowData["FIELD"] !== "Some value";
* });
*/
WrappedView.prototype.filterOut = function(cid, value) {
	return this.filterIn(cid, value, true);
};
/** @public
* @param {string|Function} cid Use string to identify column name. Use function for custom filter logic. Function will take rowId and column values as parameters, and return boolean value
* @param {*=} value
* @param {boolean=} opt_filteringOut
*/
WrappedView.prototype.filterInOnce = function(cid, value, opt_filteringOut) {
	this._dv.filterInOnce(cid, value, opt_filteringOut);
};
/** @public
* @suppress {checkTypes}
* @param {string|function(string, Object.<string, *>):boolean} cid Use string to identify column name. Use function for custom filter logic. Function will take rowId and column values as parameters, and return boolean value
* @param {*=} value
*/
WrappedView.prototype.filterOutOnce = function(cid, value) {
	this.filterInOnce(cid, value, true);
};
/** @public
*/
WrappedView.prototype.clearFilter = function() {
	this._dv.clearFilter();
};

/** Disable the page index correction
* @public
* @param {boolean=} opt_disable
*/
WrappedView.prototype.disablePageCorrection = function(opt_disable) {
};
/** Goes to the specified page
* @public
* @param {number} pageIndex
* @return {boolean} True if there is any change, otherwise false
*/
WrappedView.prototype.setPage = function (pageIndex) { return false; };
/** {@link WrappedView#setPage}
* @public
* @function
*/
WrappedView.prototype.gotoPage = WrappedView.prototype.setPage;
/** Goes to the next page. If current page is already the last page, nothing is performed
* @public
* @return {boolean} True if there is any change, otherwise false
*/
WrappedView.prototype.nextPage = function () {
	return false;
};
/** Goes back to the previous page. If current page is already the first page, nothing is performed
* @public
* @return {boolean} True if there is any change, otherwise false
*/
WrappedView.prototype.previousPage = function () {
	return false;
};
/** {@link WrappedView#previousPage}
* @public
* @function
*/
WrappedView.prototype.prevPage = WrappedView.prototype.previousPage;
/** Goes to the last page
* @public
* @return {boolean} True if there is any change, otherwise false
*/
WrappedView.prototype.lastPage = function () {
	return false;
};
/** Goes to the first page
* @public
* @return {boolean} True if there is any change, otherwise false
*/
WrappedView.prototype.firstPage = function () {
	return false;
};

/**
* @public
* @return {number}
*/
WrappedView.prototype.getCurrentPage = function () {
	return 1;
};
/** {@link WrappedView#getCurrentPage}
* @public
* @function
*/
WrappedView.prototype.getPageIndex = WrappedView.prototype.getCurrentPage;
/** Verifies that current page is the last page
* @public
* @return {boolean}
*/
WrappedView.prototype.isOnLastPage = function () {
	return true;
};
/** Verifies that current page is the first page
* @public
* @return {boolean}
*/
WrappedView.prototype.isOnFirstPage = function () {
	return true;
};

/** Get total number of rows in a single page
* @public
* @return {number}
*/
WrappedView.prototype.getPageSize = function () {
	return 0;
};
/** Get total number of pages
* @public
* @return {number}
*/
WrappedView.prototype.getPageCount = function () {
	return 1;
};
/** {@link WrappedView#getTotalPages}
* @public
* @function
*/
WrappedView.prototype.getTotalPages = WrappedView.prototype.getPageCount;
/** Shrink current view to the specified size. Set it 0 to reset to original size
* @public
* @param {number} pageSize
*/
WrappedView.prototype.setPageSize = function (pageSize) {};

/** Group all rows by the specified criteria. The criteria can be field name (field value will be used as a group id) or a function that takes row data and returns group id.
* @public
* @param {string|Function|Array.<string|Function>} cid Column id or grouper method that take one parameter for row data in JSON format, and returns groupId (string)
* @param {Array=} opt_predefinedGroups
*/
WrappedView.prototype.groupBy = function (cid, opt_predefinedGroups) {
	this._dv.groupBy(cid, opt_predefinedGroups);
};
/** addGroup is used to add a single predefined group or multiple predefined groups.
* @public
* @param {*|Array} value
*/
WrappedView.prototype.addGroup = function (value) {
	this._dv.addGroup(value);
};
/** Sort all of its descendants based on given compare function
* @public
* @param {Function=} compare Default is sorting by text. If null or false value is given, no sorting is performed
*/
WrappedView.prototype.sortGroups = function (compare) {
	this._dv.sortGroups(compare);
};
/** Move the specified group within its group
* @public
* @param {number|string} groupRef
* @param {number} destIndex
*/
WrappedView.prototype.moveGroup = function (groupRef, destIndex) {
	this._dv.moveGroup(groupRef, destIndex);
};
/** WARNING: This could be very slow to perform
* @public
* @param {Array} predefinedGroups Use two dimensional array to define groups for multi-level grouping. The first item is for the top level groups. The second item is for the second level groups, and so on. If one dimentional array is given, it is treated as the first item in the two-dimentionsal array.
* @example
* dv.setPredefinedGroups(["a", "b", "c"]); // Create empty three empty groups
* dv.setPredefinedGroups([ // Create two empty groups, and each group has 3 sub-groups
*   ["Main Group 1", "Main 2"],
*   ["Sub 1", "Sub 2", "Sub 3"]
* ]);
*/
WrappedView.prototype.setPredefinedGroups = function (predefinedGroups) {
	this._dv.setPredefinedGroups(predefinedGroups);
};

/** Add a footer to current group
* @public
* @param {number=} opt_num Number of footer to be added. If not specified, the number is one.
*/
WrappedView.prototype.addFooter = function (opt_num) {
	this._dv.addFooter(opt_num);
};
/** Remove all footers in the gurrent view
* @public
*/
WrappedView.prototype.removeFooter = function () {
	this._dv.removeFooter();
};

/** Enable/Disable multi-group row mode
* @public
* @ignore
* @param {boolean=} enabled
*/
WrappedView.prototype.enableMultiGroup = function(enabled) {
	this._dv.enableMultiGroup(enabled);
};

/** Get a group DataView by group row id. If content row id is given, null is returned (unlike getGroup method).<br>
* Nested groups and other groups outside of this group branch will be searched.
* @public
* @param {string} rowId Group Id, row Id or group index (for immediate child groups)
* @return {DataView}
* @see {@link WrappedView#getGroup}
*/
WrappedView.prototype.getGroupByRowId = function (rowId) {
	return this._dv.getGroupByRowId(rowId);
};
/** Get a corresponding DataView by the given group Id, row Id or group index. If row Id of content row is given, a parent group will be returned. <br>
* Nested groups and other groups outside of this group branch will be searched.
* @public
* @param {number|string} groupRef Group Id, row Id or group index (for immediate child groups)
* @return {DataView}
* @see {@link WrappedView#getGroupByRowId}
*/
WrappedView.prototype.getGroup = function (groupRef) {
	return this._dv.getGroup(groupRef);
};
/** Get all group descendants
* @public
* @param {Array=} out_ary
* @return {!Array.<DataView>}
*/
WrappedView.prototype.getAllGroups = function (out_ary) {
	return this._dv.getAllGroups(out_ary);
};
/** Get immediate group members row ids in array list
* @public
* @return {Array.<string>}
*/
WrappedView.prototype.getGroupList = function () {
	return this._dv.getGroupList();
};
/** Get group id (string) of this data view. Only exists if this data view is a child data view
* @public
* @return {string}
*/
WrappedView.prototype.getGroupId = function () {
	return this._dv.getGroupId();
};
/** Get group value (any data type) of this data view. Only exists if this data view is a child data view
* @public
* @return {*}
*/
WrappedView.prototype.getGroupValue = function () {
	return this._dv.getGroupValue();
};
/** Row id used by parent data view as a header row for referencing
* @public
* @return {string}
*/
WrappedView.prototype.getGroupRowId = function () {
	return this._dv.getGroupRowId();
};
/**
* @public
* @ignore
* @param {number} rid
* @return {number}
*/
WrappedView.prototype.getFooterRowIndex = function (rid) {
	return this._dv.getFooterRowIndex(rid);
};
/** A List of the footer row ids. Footer row is placed at the bottom of all other rows in this view
* @public
* @ignore
* @return {Array.<string>} Null if there is no footer row
*/
WrappedView.prototype.getFooterRowIds = function () {
	return this._dv.getFooterRowIds();
};
/**
* @public
* @return {boolean}
*/
WrappedView.prototype.hasGroup = function () {
	return this._dv.hasGroup();
};
/** Deprecated in favor of getGroupByRowId
* @public
* @return {null}
* @see {@link WrappedView#getGroupByRowId}
*/
WrappedView.prototype.getGroupRows = function () {
	return null;
};
/** Get group level of the corresponding row. If the given index is not specified, header row (the view itself) is used instead. Group level 0 means it is the root data view.
* @public
* @param {(number|string)=} rowRef
* @return {number}
*/
WrappedView.prototype.getGroupLevel = function (rowRef) {
	let rowId = this._toRowId(rowRef);
	return this._dv.getGroupLevel(rowId);
};

/** Statistics (e.g. Sum, Avg, Max, and Min) of the specified column will be collected on every update. WARNING: CPU usage is intensive. Do disable, if the column is no longer used.
* @public
* @param {string} cid
* @param {boolean=} opt_enabled
* @param {string=} opt_refId
* @return {ColumnStats} ColumnStats Object
*/
WrappedView.prototype.enableColumnStats = function(cid, opt_enabled, opt_refId) {
	return this._dv.enableColumnStats(cid, opt_enabled, opt_refId);
};
/** @public
 * @param {string} cid
* @return {ColumnStats} ColumnStats. Return null if the column has not been enabled
*/
WrappedView.prototype.getColumnStats = function(cid) {
	return this._dv.getColumnStats(cid);
};

/** This method temporarily remove all rows from the view, which can improve performance as further updates and operations will be ignored
* @public
* @param {boolean=} opt_hidden
* @returns {boolean}
*/
WrappedView.prototype.hide = function(opt_hidden) {
	return this._dv.hide(opt_hidden);
};
/** @public
* @return {boolean}
*/
WrappedView.prototype.isHidden = function() {
	return this._dv.isHidden();
};
/** @public
* @param {string} rowId
* @return {boolean} Returns true when this row is hidden
*/
WrappedView.prototype.isHiddenRow = function(rowId) {
	return this._dv.isHiddenRow(rowId);
};

/** Collapse current view. This works only if the view is generated as a child view
* @public
* @param {boolean=} opt_collapsed
*/
WrappedView.prototype.collapse = function(opt_collapsed) {
	this._dv.collapse(opt_collapsed);
};
/** Collapse all children and descendants, excluding itself and its parent
* @public
* @param {boolean=} opt_collapsed
*/
WrappedView.prototype.collapseAllGroups = function(opt_collapsed) {
	this._dv.collapseAllGroups(opt_collapsed);
};
/** @public
*/
WrappedView.prototype.toggleCollapsedState = function() {
	this.collapse(!this.isCollapsed());
};
/** @public
* @return {boolean}
*/
WrappedView.prototype.isCollapsed = function() {
	return this._dv.isCollapsed();
};
/** Content as header mode indicates that the group headers cannot be collapsed and the first row in the group will always be shown.
* @public
* @param {boolean=} bool
*/
WrappedView.prototype.enableContentAsHeader = function(bool) {
	this._dv.enableContentAsHeader(bool);
};

/** This will not include collapsed group and rows
* @public
* @return {number}
*/
WrappedView.prototype.getVisibleRowCount = function() {
	return this._rids.length;
};

/** @public
* @description check weather row object is in group
* @param {Object} rowData
* @param {string=} groupID Default group id is the group id of the DataView that is being called.
* @return {boolean}
*/
WrappedView.prototype.isRowDataInGroup = function (rowData, groupID) {
	return this._dv.isRowDataInGroup(rowData, groupID);
};

/** @public
* @description check weather row ID is in group
* @param {string} rowId
* @param {string=} groupID Default group id is the group id of the DataView that is being called.
* @return {boolean}
*/
WrappedView.prototype.isRowIdInGroup = function (rowId, groupID) {
	return this._dv.isRowIdInGroup(rowId, groupID);
};

/** The row expansion will be directly below the specified row, even if this data view is sorted.
* @public
* @param {string} rid Row Id
* @param {number=} count=1
*/
WrappedView.prototype.addRowExpansion = function(rid, count) {
	this._dv.addRowExpansion(rid, count);
};
/**
* @public
* @param {string} rid Row Id
*/
WrappedView.prototype.removeRowExpansion = function(rid) {
	this._dv.removeRowExpansion(rid);
};
/**
* @public
* @param {string|number} rowRef Row Id or index
* @param {boolean=} force
* @param {number=} count=1
*/
WrappedView.prototype.toggleRowExpansion = function(rowRef, force, count) {
	let rid = this._toRowId(rowRef);
	this._dv.toggleRowExpansion(rid, force, count);
};
/** @public
*/
WrappedView.prototype.removeAllRowExpansions = function() {
	this._dv.removeAllRowExpansions();
};
/** Deprecated. Use {@link WrappedView#getExpansionParents} instead. Get all rows with expansion (not row expansion itself) will contains row id, otherwise undefined.
* @public
* @ignore
* @param {number=} from
* @param {number=} to EXCLUSIVE
* @return {!Array.<string|undefined>}
*/
WrappedView.prototype.getRowsWithExpansion = function(from, to) {
	return this._dv.getRowsWithExpansion(from, to); // TODO: Convert row index to row id
};
/** Get row expansion states from the specified range. The returned value will be row id for row expansion, otherwise empty string for other types of row (e.g. normal row, or group header row)
* @public
* @param {number=} from
* @param {number=} to EXCLUSIVE
* @return {Array.<string>} object mapping of rowIndex and parentRowId
*/
WrappedView.prototype.getExpansionParents = function (from, to) {
	return this._dv.getExpansionParents(from, to); // TODO: Convert row index to row id
};

/** Check if the specified row is itself a row expansion.
* @public
* @param {string} rid rowId
* @return {boolean}
*/
WrappedView.prototype.isRowExpansion = function(rid) {
	return this._dv.isRowExpansion(rid);
};
/** Check if the specified row has row expansion (not row expansion itself).
* @public
* @param {string} rid rowId
* @return {boolean}
*/
WrappedView.prototype.hasRowExpansion = function(rid) {
	return this._dv.hasRowExpansion(rid);
};
/** Get row Id of the row expansion by providing the row Id of the row with row expansion (not row expansion itself). This is NOT recommended for normal usage, since the row expansion contains no data in the DataView
* @public
* @param {string} rid rowId
* @param {number=} idx=0
* @return {string} Empty string is returned if the provided row does not contains a row expansion.
*/
WrappedView.prototype.getRowExpansionId = function(rid, idx) {
	return this._dv.getRowExpansionId(rid, idx);
};

/** Get index from the row expansion id. Any row other than row expansion will yield negative value (-1)
* @public
* @param {string} rid Row expansion id
* @return {number}
*/
WrappedView.prototype.getRowExpansionIndex = function(rid) {
	return this._dv.getRowExpansionIndex(rid);
};
/**
* @public
* @param {function(Object):boolean} searchLogic Function that takes row data object and return boolean value
* @param {Array.<string>=} opt_out Array for containing row ids
* @return {!Array.<string>} List of row ids
*/
WrappedView.prototype.searchAll = function(searchLogic, opt_out) {
	return this._dv.searchAll(searchLogic, opt_out);
};
/**
* @public
* @param {number|string} rowRef
* @param {function(Object):boolean} searchLogic Function that takes row data object and return boolean value
* @return {number} Row index
*/
WrappedView.prototype.searchPrev = function(rowRef, searchLogic) {
	let rid = this._toRowId(rowRef);
	return this._dv.searchPrev(rid, searchLogic);
};
/**
* @public
* @param {number|string} rowRef
* @param {function(Object):boolean} searchLogic Function that takes row data object and return boolean value
* @return {number} Row index
*/
WrappedView.prototype.searchNext = function(rowRef, searchLogic) {
	let rid = this._toRowId(rowRef);
	return this._dv.searchNext(rid, searchLogic);
};

/** Blocks all row insertion and removal, but keep updating the data
* @public
* @param {boolean=} opt_bool
* @return {boolean} True if there is any change
*/
WrappedView.prototype.stall = function(opt_bool) {
	return this._dv.stall(opt_bool);
};
/** Automatically and asyncronuosly remove group that has no member or no content. Predefined groups will not be removed in this way.
* @public
* @param {boolean=} opt_bool
* @return {boolean} True if there is any change
*/
WrappedView.prototype.enableAutoGroupRemoval = function(opt_bool) {
	return this._dv.enableAutoGroupRemoval(opt_bool);
};
/** Automatically hide null or undefined group header, while keeping the group's items intact.
* @public
* @param {boolean=} opt_bool
* @return {boolean} True if there is any change
*/
WrappedView.prototype.enableAutoGroupHiding = function(opt_bool) {
	return this._dv.enableAutoGroupHiding(opt_bool);
};

/** Copy row order in data view to its data table
* @public
*/
WrappedView.prototype.synchronizeRowOrder = function() {
	this._dv.synchronizeRowOrder();
};
/** Set visible row as segment separator (hidden or filtered rows cannot be a segment separator)
* @public
* @param {string|number} rowRef Row id or row index
* @param {boolean=} enabled
* @return {boolean} Return true if there is any change
*/
WrappedView.prototype.setSegmentSeparator = function(rowRef, enabled) {
	let rowId = this._toRowId(rowRef);
	return this._dv.setSegmentSeparator(rowId, enabled);
};
/**
* @public
* @return {boolean} Return true if there is any change
*/
WrappedView.prototype.unsetAllSegmentSeparators = function() {
	return this._dt.unsetAllSegmentSeparators();
};
/**
* @public
* @return {boolean}
*/
WrappedView.prototype.hasSegmentation = function() {
	return this._dt.hasSegmentation();
};
/**
* @public
* @param {string} rid
* @return {boolean}
*/
WrappedView.prototype.isSegmentSeparator = function(rid) {
	return this._dt.isSegmentSeparator(rid);
};

/**
* @public
* @param {string} rid
* @return {string} parent row id of this segmentation. If the parent row id for this segmentation cannot be found, return ""
*/
WrappedView.prototype.getSegmentParentRowId = function(rid) {
	return this._dt.getSegmentParentRowId(rid);
};
/** @public
* @param {string|number} rowRef Row id or row index
* @param {boolean=} collapsed
* @return {boolean} Return true if there is any change
*/
WrappedView.prototype.collapseSegment = function(rowRef, collapsed) {
	let rid = this._toRowId(rowRef);
	return this._dv.collapseSegment(rid, collapsed);
};
/** @public
* @param {string|number} rowRef Row id or row index
* @param {boolean=} expanded
* @return {boolean} Return true if there is any change
*/
WrappedView.prototype.expandSegment = function(rowRef, expanded) {
	return this.collapseSegment(rowRef, expanded === false);
};
/** @public
* @return {boolean} Return true if there is any change
*/
WrappedView.prototype.expandAllSegments = function() {
	return this._dv.expandAllSegments();
};
/** Deprecated.
* @public
* @ignore
*/
WrappedView.prototype.setSegmentCollapsingLogic = function() {};
/** @public
* @param {string|number} rowRef Row id or row index
* @return {boolean} Return true if only the given row is a collapsed segment separator
*/
WrappedView.prototype.isSegmentCollapsed = function(rowRef) {
	let rid = this._toRowId(rowRef);
	return this._dv.isSegmentCollapsed(rid);
};
/** Remove existing segment children and fill the segments with all contnet rows before the next segment separator
* @public
* @param {string|number} segmentRef Row id or row index
*/
WrappedView.prototype.fillSegment = function(segmentRef) {
	let rid = this._toRowId(segmentRef);
	this._dv.fillSegment(rid);
};
/** Remove all existing segment children in each segment and fill the segments with all contnet rows before the next segment separator
* @public
*/
WrappedView.prototype.fillSegments = function() {
	this._dv.fillSegments();
};
/** @public
* @param {string|number} segmentRef Row id or row index
* @param {string|number} rowRef Row id, row index
* @return {boolean} Return true if there is any change
*/
WrappedView.prototype.addSegmentChild = function(segmentRef, rowRef) {
	let segmentId = this._toRowId(segmentRef);
	let rowId = this._toRowId(rowRef);
	return this._dv.addSegmentChild(segmentId, rowId);
};
/** @public
* @param {string|number} segmentRef Row id or row index
* @param {Array.<string|number>} rowRefs Array of row ids or row indices
* @return {boolean} Return true if there is any change
*/
WrappedView.prototype.addSegmentChildren = function(segmentRef, rowRefs) {
	let segmentId = this._toRowId(segmentRef);
	let rowIds = this._toRowIds(rowRefs);
	return this._dv.addSegmentChildren(segmentId, rowIds);
};
/** @public
* @param {string|number} segmentRef Row id or row index
* @param {string|number} rowRef Row id, row index
* @return {boolean} Return true if there is any change
*/
WrappedView.prototype.removeSegmentChild = function(segmentRef, rowRef) {
	let segmentId = this._toRowId(segmentRef);
	let rowId = this._toRowId(rowRef);
	return this._dv.removeSegmentChild(segmentId, rowId);
};
/** @public
* @param {string|number} segmentRef Row id or row index
* @param {Array.<string|number>} rowRefs Array of row ids or row indices. If null is given, no child will be removed.
* @return {boolean} Return true if there is any change
*/
WrappedView.prototype.removeSegmentChildren = function(segmentRef, rowRefs) {
	let segmentId = this._toRowId(segmentRef);
	let rowIds = this._toRowIds(rowRefs);
	return this._dv.removeSegmentChildren(segmentId, rowIds);
};
/** @public
* @return {boolean} Return true if there is any change
*/
WrappedView.prototype.removeAllSegmentChildren = function() {
	return this._dt.removeAllSegmentChildren();
};
/** Get row ids of all segment separator
* @public
* @return {Array.<string>} Return null if there is no segment separator
*/
WrappedView.prototype.getSegmentIds = function() {
	return this._dt.getSegmentIds();
};
/** Get row ids of all children from the specified segment
* @public
* @param {string|number} segmentRef Row id or row index
* @return {Array.<string>} Returns array of row ids. Returns null if there is no child or no segment separator found
*/
WrappedView.prototype.getSegmentChildIds = function(segmentRef) {
	return this._dt.getSegmentChildIds(this._toRowId(segmentRef));
};
/** Sort all of existing segments by given compare function
* @public
* @param {Function} compare
*/
WrappedView.prototype.sortSegments = function (compare) {
	this._dt.sortSegments(compare);
};

/** Return 0, if there is no wrapping
* @public
* @return {number}
*/
WrappedView.prototype.getWrapSize = function () {
	return this._dv.getWrapSize();
};
/** Return null, if there is no wrapping
* @public
* @return {Array.<WrappedView>}
*/
WrappedView.prototype.getWrappedViews = function () {
	return this._dv.getWrappedViews();
};
/**
* @public
* @param {number=} wrapSize
* @fires DataView#wrapCountChanged
*/
WrappedView.prototype.wrapView = function (wrapSize) {
	this._dv.wrapView(wrapSize);
};

/** Construct a JSON object that is compatible with console.table()
* @public
* @param {Object=} opt_options {@link DataCache#dump}
* @return {!Array.<Object>} Return a JSON object that is compatible with console.table()
* @see {@link DataCache#dump}
*/
WrappedView.prototype.dump = function(opt_options) {
	return data_DataCache.constructTable(this.getMultipleRowData(this._rids), opt_options, this._rids);
};
/** Print current data and structure to console panel
* @public
* @param {Object=} opt_options
* @example
// Options have the following format:
// {
//  "colNames": Array.<string>,
//  "rowCount": number,
//  "startIndex": number,
//  "getter": function(Object, string):*,
//  "withRowIds": boolean
//}
dv.log({"rowCount": 10}); // Showing 10 records
dv.log({"getter": function(rowData, colName){
	let complexData = rowData[colName];
	return complextData.someProperty;
}});
*/
WrappedView.prototype.log = function(opt_options) {
	console.table(this.dump(opt_options));
};


/** Check if the row should be filtered out
* @public
* @param {string} rid
* @param {Object} rowData
* @return {boolean}
*/
WrappedView.prototype.isRowFiltered = function(rid, rowData) {
	return this._dv.isRowFiltered(rid, rowData);
};

/* harmony default export */ const data_WrappedView = (WrappedView);


;// ./node_modules/@grid/core/es6/data/DataView.js



 // eslint-disable-line

 // eslint-disable-line




/** @event DataView#dataChanged
* @description Trigger when data within the data view has been changed. Not only actual change in data, but also position change will trigger this event.
* @property {boolean} globalChange Indicates a big change. User should expect all data has been change. With active sorting or filtering, this flag will always be true due to multiple changes in row position.
* @property {string} type Type of changes. Possible values are "inserted", "removed", "updated", and undefined
* @property {string} rid Row Id
* @property {Object.<string, *>} rowData Column values of the changed row in JSON object format
*/
/** @event DataView#pageIndexChanged
* @property {number} pageIndex
*/
/** @event DataView#pageCountChanged
* @property {number} pageCount
*/
/** @event DataView#preDisposed
*/
/** @event DataView#groupAdded
* @property {DataView} dataView Parent data view that create the new group
* @property {DataView} newGroup Child data view that is newly created
*/
/** @event DataView#beforeGroupAdded
* @description Fired each time before group added. Set property `cancel` to true to cancel adding of the new group
* @type {Object}
* @property {DataView} dataView Parent data view that create the new group
* @property {string} newGroupId
* @property {*} newGroupValue
* @property {string} newGroupRowId Unique row Id given for the group
*/
/** @event DataView#groupCriteriaChanged
* @type {Object}
*/
/** @event DataView#beforeFiltering
* @description Fired once before actual row filtering (before multiple execution of filter logic). This allow optimization by preparing data needed for the filtering
* @type {Object}
*/
/** @event DataView#wrapCountChanged
* @property {number} prevCount
* @property {number} newCount
*/
/** @event DataView#wrapSizeChanged
* @property {number} wrapSize
* @property {Array.<string>} rids
*/

/** DataView doesn't hold the actual data. It is just a transform view (by applying filtering or sorting) of the data from the {@link DataTable}
* @constructor
* @extends {EventDispatcher}
* @param {DataTable|DataView=} source
* @see {@link DataTable}
*/
let DataView = function(source) {
	let t = this; // For file minification
	t._groupFilterLogic = t._groupFilterLogic.bind(t);
	t._internalGroupSortingLogic = t._internalGroupSortingLogic.bind(t);

	t._onDataChanged = t._onDataChanged.bind(t);
	t._onGroupAdded = t._onGroupAdded.bind(t);
	t._onBeforeGroupAdded = t._onBeforeGroupAdded.bind(t);
	t._onRefreshTimeout = t._onRefreshTimeout.bind(t);
	t._updateWrapCount = t._updateWrapCount.bind(t);

	t._byRemovalMap = t._byRemovalMap.bind(t);

	t._rids = [];
	t._sortingDefs = [];
	t._columnStats = {};
	t._shared = {};

	t._conflator = new Conflator(this._onDataChanged, 0);
	t._conflator.disable();

	t._addEvents("dataChanged",
		"preDisposed",
		"pageIndexChanged",
		"pageCountChanged",
		"groupAdded",
		"beforeGroupAdded",
		"groupCriteriaChanged",
		"beforeFiltering",
		"wrapCountChanged",
		"wrapSizeChanged"
	);

	t.setDataSource(source);
};
es6_Ext.inherits(DataView, event_EventDispatcher);

/** List of row Ids with filtering and sorting transformation
* @private
* @type {!Array.<string>}
*/
DataView.prototype._rids; // Contains only data rows that have been filtered
/** List of row Ids with grouping transformation
* @private
* @type {Array.<string>}
*/
DataView.prototype._groupView = null; // Contains non-data rows (e.g. group headers, group footers) and exclude collapsed group data rows
/** List of row Ids with paging transformation
* @private
* @type {Array.<string>}
*/
DataView.prototype._pageView = null; // Cut-off version of _rids or _groupView
/** List of row Ids with expanding transformation
* @private
* @type {Array.<string>}
*/
DataView.prototype._expansionView = null; // The rids with row expansion

/** @private
* @type {Object.<string, boolean>}
*/
DataView.prototype._hiddenRids = null;
/** @private
* @type {Object.<string, boolean>}
*/
DataView.prototype._collapsedRids = null; // for segmentation
/** @private
* @type {Object.<string, number>}
*/
DataView.prototype._excludedRids = null;
/** @private
* @type {boolean}
*/
DataView.prototype._emptySegmentFiltering = false;
/** @private
* @type {boolean}
*/
DataView.prototype._separatorFiltering = false;

/** @private
* @type {Object.<string, number>}
*/
DataView.prototype._expansionMap = null;
/** @private
* @type {Object.<string, string>}
*/
DataView.prototype._expansionChildren = null;
/** @private
* @type {number}
*/
DataView.prototype._expansionCount = 0;

/** @private
* @type {DataTable}
*/
DataView.prototype._dt = null;
/** @private
* @type {DataTable|DataView}
*/
DataView.prototype._parent = null;
/** @private
* @type {Object}
*/
DataView.prototype._shared = null; // Shared options between data view in the same hierarchy

/** @private
* @type {Array.<Array>}
*/
DataView.prototype._sortingDefs = null;

/** @private
* @type {function(string, Object.<string, *>)|null}
*/
DataView.prototype._userFilter = null;
/** @private
* @type {boolean}
*/
DataView.prototype._filteringOut = false;

/** @private
* @type {number}
*/
DataView.prototype._refreshTimerId = 0;

/** @private
* @type {Function}
*/
DataView.prototype._groupSortingLogic = null;

/** @private
* @type {boolean}
*/
DataView.prototype._pageCorrection = true;
/** @private
* @type {number}
*/
DataView.prototype._pageSize = 0;
/** @private
* @type {number}
*/
DataView.prototype._pageIndex = 1;
/** @private
* @type {number}
*/
DataView.prototype._lastPageIndex = 0;

/** @private
* @type {Array.<Function>}
*/
DataView.prototype._groupCriteria = null; // Shared between groups
/** @private
* @type {Array.<Array>}
*/
DataView.prototype._preDefGroups = null;
/** @private
* @type {boolean}
*/
DataView.prototype._preDefGroupsDirty = false;
/** @private
* @type {Object.<string, DataView>}
*/
DataView.prototype._groupMembers = null; // Map of immediate child data views (key is id)
/** @private
* @type {Array.<string>}
*/
DataView.prototype._groupList = null; // List of immediate child's ids
/** A map object that contains all groups with key is group row id and value is the group DataView
* @private
* @type {Object.<string, DataView>}
*/
DataView.prototype._groupRows = null; // Shared between groups
/** @private
* @type {string}
*/
DataView.prototype._groupId = "";
/** @private
* @type {string}
*/
DataView.prototype._groupColor = "";
/** @private
* @type {*}
*/
DataView.prototype._groupValue;
/** @private
* @type {string}
*/
DataView.prototype._groupRowId = "";
/** @private
* @type {Array.<string>}
*/
DataView.prototype._footerRowIds = null;
/** @private
* @type {number}
*/
DataView.prototype._groupLevel = 0;
/** @private
* @type {boolean}
*/
DataView.prototype._groupViewDirty = false;

/** @private
* @type {Object.<string, ColumnStats>}
*/
DataView.prototype._columnStats;

/** @private
* @type {boolean}
*/
DataView.prototype._hidden = false;
/** @private
* @type {boolean}
*/
DataView.prototype._hiddenHeaderRow = false;
/** @private
* @type {boolean}
*/
DataView.prototype._collapsed = false;
/** @private
* @type {boolean}
*/
DataView.prototype._collapsible = true;
/** @private
* @type {boolean}
*/
DataView.prototype._showOnlyFirstMember = false;
/** @private
* @type {boolean}
*/
DataView.prototype._beingAdded = false;
/** @private
* @type {Conflator}
*/
DataView.prototype._conflator = null;
/** @private
* @type {Array.<WrappedView>}
*/
DataView.prototype._wrappedViews = null;
/** @private
* @type {number}
*/
DataView.prototype._wrapSize = 0;
/** @private
* @type {number}
*/
DataView.prototype._wrapTimerId = 0;
/** @private
* @type {boolean}
*/
DataView.prototype._stalledSorting = false;

/** @public
* @fires DataView#preDisposed
*/
DataView.prototype.dispose = function() {
	this._dispatch("preDisposed", {});

	this.unlistenAll();
	this.wrapView(0);

	this.setDataSource(null); // Remove this._dt and this._parent. WARNING: an event is fired
	this.groupBy(null); // Remove this._groupCriteria, this._groupRows and all of its children. WARNING: an event is fired

	this._conflator.dispose();
	this._sortingDefs = null;
	this._shared = null; // Remove shared options

	this._userFilter = this._groupSortingLogic = null;

	if(this._wrapTimerId) {
		clearTimeout(this._wrapTimerId);
		this._wrapTimerId = 0;
	}
	if(this._refreshTimerId) {
		clearTimeout(this._refreshTimerId);
		this._refreshTimerId = 0;
	}

	if(this._columnStats) {
		for(let field in this._columnStats) {
			this._columnStats[field].dispose();
		}
		this._columnStats = null;
	}
};

/** Get data by row id and column id (field)
* @public
* @param {string} rid
* @param {string} cid
* @returns {Object.<string, *>}
*/
DataView.prototype.getData = function(rid, cid) {
	return this._dt.getData(rid, cid);
};
/** Get entire row data by row id
* @public
* @param {string} rid
* @return {Object.<string, *>}
*/
DataView.prototype.getRowData = function(rid) {
	return this._dt.getRowData(rid);
};
/** @public
* @param {string} cid
* @param {boolean=} inclNonDataRow=false If enabled, group header rows are included
* @return {!Array}
*/
DataView.prototype.getColumnData = function(cid, inclNonDataRow) {
	let rids = this.getAllRowIds(true, inclNonDataRow);

	let rows = [];
	let len = rids.length;
	for(let r = 0; r < len; ++r) {
		let rowData = this._dt.getRowData(rids[r]);
		if(rowData) {
			rows.push(rowData[cid]);
		} else if(inclNonDataRow) {
			rows.push(null);
		}
	}
	return rows;
};
/** @public
* @param {number} rowIndex
* @return {string}
* @see {@link DataView#getRowIndex}
*/
DataView.prototype.getRowId = function(rowIndex) {
	this._rebuildRowIdsFromChildren();

	return this._getRids()[rowIndex] || "";
};
/** @private
* @param {number|string} rowRef
* @return {string}
*/
DataView.prototype._toRowId = function(rowRef) {
	if(typeof rowRef === "number") {
		this._rebuildRowIdsFromChildren();
		return this._getRids()[rowRef] || "";
	}
	return /** @type{string} */(rowRef || "");
};
/** @private
* @param {Array.<number|string>} rowRefs
* @return {!Array.<string>}
*/
DataView.prototype._toRowIds = function(rowRefs) {
	let rowCount = Array.isArray(rowRefs) ? rowRefs.length : 0;
	let ary = new Array(rowCount);
	let rids = this._getRids();
	for(let r = 0; r < rowCount; ++r) {
		let rowRef = rowRefs[r];
		if(typeof rowRef === "number") {
			this._rebuildRowIdsFromChildren();
			ary[r] = rids[rowRef] || "";
		} else {
			ary[r] = rowRef || "";
		}
	}
	return ary;
};
/** Return all row ids with row data, excluding the group header rows (auto generated rows).
* @public
* @param {boolean=} shallowCopy=false If enabled, no new copy of Array is created (better for performance).
* @param {boolean=} inclNonDataRow=false If enabled, group header rows are included. In pagination mode, this flag will always be true.
* @return {!Array.<string>}
* @see {@link DataView#getAllRowData}
* @see {@link DataView#getMultipleRowData}
* @see {@link DataView#getSortedRowIds}
* @see {@link DataView#getVisibleRowIds}
*/
DataView.prototype.getAllRowIds = function(shallowCopy, inclNonDataRow) {
	this._rebuildRowIdsFromChildren();

	// WARNING: this._pageView can contain non-data rows (group header rows), even if inclNonDataRow flag is false
	let rids = (inclNonDataRow) ? this._getRids() : (this._pageView || this._rids);

	return (shallowCopy) ? rids : rids.slice(0); // Fastest way to clone an array is Array.slice()
};
/** This method returns current visible rows (i.e. Rows in other pages or in collapsed groups are excluded). Group header rows may be included in the results.
* @public
* @param {boolean=} shallowCopy=false If enabled, no new copy of Array is created (better for performance).
* @return {!Array.<string>}
* @see {@link DataView#getAllRowData}
*/
DataView.prototype.getVisibleRowIds = function(shallowCopy) {
	this._rebuildRowIdsFromChildren();
	let rids = this._getRids();
	return shallowCopy ? rids : rids.slice(0); // Fastest way to clone an array
};
/** Return all data rows that is filtered and sorted, excluding header rows or paging. This method ignores pagination.
* @public
* @param {boolean=} shallowCopy=false If enabled, no new copy of Array is created (better for performance).
* @return {!Array.<string>}
* @see {@link DataView#getAllRowData}
*/
DataView.prototype.getSortedRowIds = function(shallowCopy) {
	this._rebuildRowIdsFromChildren();
	return shallowCopy ? this._rids : this._rids.slice(0); // Fastest way to clone an array
};
/** This method is quite slow
* @public
* @param {string} rid
* @return {number}
* @see {@link DataView#getRowId}
*/
DataView.prototype.getRowIndex = function(rid) {
	this._rebuildRowIdsFromChildren();

	return this._getRids().indexOf(rid);
};
/** @public
* @return {number}
*/
DataView.prototype.getRowCount = function() {
	return this._getRids().length;
};
/** @public
* @param {number} rowIndex
* @param {string} cid
* @returns {Object.<string, *>}
*/
DataView.prototype.getDataAt = function(rowIndex, cid) {
	return this._dt.getData(this.getRowId(rowIndex), cid);
};
/** This method is slow. When working with huge amount of rows, use getRowData in combination with getAllRowIds(true) instead of calling getRowDataAt multiple times.
* @public
* @param {number} rowIndex
* @return {Object.<string, *>}
*/
DataView.prototype.getRowDataAt = function(rowIndex) {
	return this._dt.getRowData(this.getRowId(rowIndex));
};
/** @public
* @param {string} rid
* @param {string} cid
* @returns {Object.<string, *>}
*/
DataView.prototype.getPreviousData = function(rid, cid) {
	return this._dt.getPreviousData(rid, cid);
};
/** @public
* @param {Array.<string>} rids
* @param {number=} opt_from
* @param {number=} opt_to EXCLUSIVE
* @return {!Array.<Object>}
*/
DataView.prototype.getMultipleRowData = function(rids, opt_from, opt_to) {
	return this._dt.getMultipleRowData(rids, opt_from, opt_to);
};
/** Return all rows with row data, excluding the group header rows (auto generated rows)
* @public
* @return {!Array.<Object>}
*/
DataView.prototype.getAllRowData = function() {
	return this.getMultipleRowData(this.getAllRowIds(true));
};


/** @public
* @param {string} rid
* @param {string} cid
* @param {*} value
* @return {boolean} Return true if there is any change, and false otherwise
* @fires DataView#dataChanged
*/
DataView.prototype.setData = function(rid, cid, value) {
	if(this._dt) {
		return this._dt.setData(rid, cid, value);
	}
	return false;
};
/** @public
* @param {string} rid
* @param {Object.<string, *>=} values
* @param {Object=} opt_eventArg
* @return {boolean} Return true if there is any change, and false otherwise
* @fires DataView#dataChanged
*/
DataView.prototype.setRowData = function(rid, values, opt_eventArg) {
	if(rid && this._dt) {
		if (!this.isRowExpansion(rid)) {
			return this._dt.setRowData(rid, values, opt_eventArg);
		}
	}
	return false;
};
/** Set data in a column manner. This is faster than repeatedly calling {@link DataTable#setData}
* @public
* @param {string} cid
* @param {Array} valueList
* @param {Array.<string>=} opt_ridList Specify row id to be set corresponding to the data item
*/
DataView.prototype.setColumnData = function(cid, valueList, opt_ridList) {
	//  Note: This function should filter out row expansion before calling dt.setColumnData
	if(this._dt) {
		this._dt.setColumnData(cid, valueList, opt_ridList);
	}
};
/** This method is slow. When working with huge amount of rows, use setData in combination with getAllRowIds(true) instead of calling setDataAt multiple times.
* @public
* @param {number} rowIndex
* @param {string} cid
* @param {*} value
* @return {boolean} Return true if there is any change, and false otherwise
* @fires DataView#dataChanged
*/
DataView.prototype.setDataAt = function(rowIndex, cid, value) {
	let rid = this.getRowId(rowIndex);
	if(rid && this._dt) {
		return this._dt.setData(rid, cid, value);
	}
	return false;
};
/** This method is slow. When working with huge amount of rows, use setRowData in combination with getAllRowIds(true) instead of calling setRowDataAt multiple times.
* @public
* @param {number} rowIndex
* @param {Object.<string, *>=} values
* @param {Object=} opt_eventArg
* @return {boolean} Return true if there is any change, and false otherwise
* @fires DataView#dataChanged
*/
DataView.prototype.setRowDataAt = function(rowIndex, values, opt_eventArg) {
	let rid = this.getRowId(rowIndex);
	return this.setRowData(rid, values, opt_eventArg);
};

/** @public
* @param {string|number} rowRef The position of the new row
* @param {Object.<string, *>=} opt_values
* @param {string|null=} opt_rid If the given row Id already exists, data is updated and no new row is added. Leave blank or null to let DataTable generate unique row Id
* @return {string} Return Row id that has been inserted or changed
*/
DataView.prototype.insertRow = function(rowRef, opt_values, opt_rid) { // Data changed event may be dispatched
	this._rebuildRowIdsFromChildren(); // TODO: Check if we need to rebuild row ids
	let nextRid = this._toRowId(rowRef);

	if(this._dt) {
		return this._dt.insertRow(nextRid, opt_values, opt_rid);
	}
	return "";
};

/** Permanently remove the specified row by row id or DataView's row index. If you want to temporarily remove the row, use {@link DataView#hideRows} instead
* @public
* @param {number|string} rowRef
* @fires DataView#dataChanged
*/
DataView.prototype.removeRow = function(rowRef) {
	if(this._dt) {
		this._dt.removeRow(this._toRowId(rowRef)); // Trigger dataChanged
	}
};
/** Permanently remove the specified rows by row id or DataView's row index. If you want to temporarily remove the rows, use {@link DataView#hideRows} instead
* @public
* @param {Array.<number|string>} rowRefs
*/
DataView.prototype.removeRows = function(rowRefs) {
	if(this._dt) {
		this._dt.removeRows(this._toRowIds(rowRefs)); // Trigger dataChanged
	}
};
/** Move a single row or multiple rows to the position BEFORE the specified target row.
* @public
* @param {Array.<number|string>|number|string} fromRef The value could be row index, row id, or array of row indices or row ids.
* @param {number|string} toRef Destination target could be row index or row id
* @return {Array.<string>|string} rid rowId or array of the rowId of the moved row(s)
* @example
* ary = [0, 1, 2];
* move(ary, 0, 1); // [0, 1, 2] Move index 0 to the position before index 1 (nothing is changed)
* move(ary, 1, 0); // [1, 0, 2] Move index 1 to the position before index 0
*/
DataView.prototype.moveRow = function(fromRef, toRef) {
	let toRid = this._toRowId(toRef);

	let fromIndices = /** @type{!Array.<number|string>} */(Array.isArray(fromRef) ? fromRef : [fromRef]);
	let fromRids = this._toRowIds(fromIndices);

	if(this._dt.hasSegmentation()) {
		this.synchronizeRowOrder();
	}

	return this._dt._moveRow(fromRids, toRid);
};

/** Deprecated
* @public
* @function
* @param {number} fromIndex
* @param {number} toIndex INCLUSIVE
*/
DataView.prototype.swapRow = util._deprecatedFunction;

/** Returns the root data source (DataTable) of all data views
* @public
* @return {DataTable}
* @see {@link DataView#getDataSource}
*/
DataView.prototype.getDataTable = function() {
	return this._dt;
};
/** Return source for this data view which could be another data view or a data table.
* @public
* @return {DataTable|DataView}
* @see {@link DataView#getDataTable}
*/
DataView.prototype.getDataSource = function() {
	return this._parent;
};
/** @public
* @param {(DataTable|DataView)=} source
* @fires DataView#dataChanged
*/
DataView.prototype.setDataSource = function(source) {
	if(source) {
		if(this._groupLevel > 0) {
			return; // Sub group cannot be changed data source
		}
		this._setParent(source);

		this._onDataChanged(data_DataTable._globalChangeArg); // Update current view
		this._parent.listen("dataChanged", this._onDataChanged);
	} else {
		if(this._parent) {
			this._parent.unlisten("dataChanged", this._onDataChanged);
			this._resetAllViews();
			this._groupViewDirty = false;
		}
		this._parent = this._dt = null;
		this._shared = {}; // The options are no longer shared
	}
};

/** @public
* @param {number} milliSecond
*/
DataView.prototype.setConflationRate = function (milliSecond) {
	this._conflator.disable(!milliSecond);
	this._conflator.setConflatingTime(milliSecond);
};

/** @public
* @return {number}
*/
DataView.prototype.getConflationRate = function () {
	return this._conflator.getConflatingTime();
};

/** @private
*/
DataView.prototype._resetAllViews = function() {
	this._rids.length = 0;
	if(this._groupView && this._groupView.length) { this._groupView.length = 0; }
	if(this._pageView && this._pageView.length) { this._pageView.length = 0; }
	if(this._expansionView && this._expansionView.length) { this._expansionView.length = 0; }
};

/**
* @public
* @ignore
* @return {Object.<string, DataTable.SortLogic>}
*/
DataView.prototype.getSortingLogics = function() {
	return this._dt ? this._dt.getSortingLogics() : null;
};
/** @public
* @param {string|Array.<string>} cid
* @param {string|number|Array.<string|number>=} opt_sortOrder
* @param {Function|Array.<Function>=} sortLogics
* @param {boolean=} noOptimization=false Force to sort without optimization to refresh all rowIds again.
* @fires DataView#dataChanged
*/
DataView.prototype.sort = function (cid, opt_sortOrder, sortLogics, noOptimization) {
	if(this._groupLevel > 0) {
		window.console.log("ERROR: Sub-group cannot be sorted individually");
		return;
	}

	let compMap = this.getSortingLogics();
	let sortingDefs = this._sortingDefs;
	let curSortLen = sortingDefs.length;

	if (Array.isArray(sortLogics)) {
		for(let c = sortLogics.length; --c >= 0;) {
			if (typeof sortLogics[c] !== "function") {
				let columnId = Array.isArray(cid) ? cid[c] : cid;
				sortLogics[c] = compMap[columnId] || compMap["_default"];
			}
		}
	}
	data_DataTable._buildSortContext(
		sortingDefs,
		cid,
		opt_sortOrder,
		sortLogics || compMap
	);
	if(sortingDefs.length === 1 && sortingDefs[0][3] === 0) { // Sort order is zero which is resetting to original order
		sortingDefs.length = 0;
	}
	if(curSortLen <= 0 && sortingDefs.length <= 0) {
		return; // Nothing is changed
	}
	if(this._shared.stalledRids) { // The stalled view cannot be changed
		return;
	}

	let change = false;

	if (this._groupMembers || sortingDefs.length <= 0 || noOptimization) {
		change = true;
		this._updateRowIds(); // WARNING: Slow!
	} else { // filtering should already be done beforehand
		change = this._sort();
	}
	if (change) {
		this._dispatchDataChange(data_DataTable._positionChangeArg);
	}
};
/** @public
* @param {string|Array.<string>} cid
* @param {string|number|Array.<string|number>=} opt_sortOrder
* @fires DataView#dataChanged
*/
DataView.prototype.sortOnce = function(cid, opt_sortOrder) {
	this.sort(cid, opt_sortOrder); // Global change event is dispatched
	this._sortingDefs.length = 0;
};
/** @public
* @param {DataTable.SortLogic} func
* @example
* customLogic = function(a, b, sortOrder) {
*   return (a - b) * sortOrder; // for numeric comparison
* };
* let dt = new DataTable();
* let dv = new DataView(dt);
* dv.setSortingLogic(customLogic);
* dv.sort("Column id", "ascending");
*/
DataView.prototype.setSortingLogic = function(func) {
	if(this._dt) {
		this._dt.setSortingLogic(func);
	}
};
/** Specified sorting logic for the column with the given column id. This will override logic set by {@link DataView#setSortingLogic}, when the column is sorted
* @public
* @param {string} cid
* @param {DataTable.SortLogic} func Use null to remove current sorting logic
*/
DataView.prototype.setColumnSortingLogic = function(cid, func) {
	if(this._dt) {
		this._dt.setColumnSortingLogic(cid, func);
	}
};
/** Get sorting logic for the specified field. Default logic is returned, if no logic is specified for the column
* @public
* @param {string=} cid
* @return {DataTable.SortLogic}
*/
DataView.prototype.getColumnSortingLogic = function(cid) {
	if(this._dt) {
		return this._dt.getColumnSortingLogic(cid);
	}
	return null;
};
/** Check if this view is in sorting mode
* @public
* @return {boolean}
*/
DataView.prototype.isSorting = function() {
	return this._sortingDefs.length > 0;
};

/** Hide rows in the data view
 * @param {string|number} rId Row id to hide. If the rId is a number, it will be treated as a row index
 * @param {boolean=} hidden=true If the value is false, the specified row will be shown.
 * @public
 */
DataView.prototype.hideRow = function(rId, hidden) {
	this.hideRows([rId], hidden);
};
/** Hide rows in the data view
 * @param {Array.<string|number>} rowRefs Array of row ids or indices to hide
 * @param {boolean=} hidden=true If the value is false, the specified rows will be shown.
 * @public
 */
DataView.prototype.hideRows = function(rowRefs, hidden) {
	hidden = hidden !== false;
	let hiddenRids = this._hiddenRids;

	if(hidden){
		if(!hiddenRids) {
			hiddenRids = this._hiddenRids = {};
		}
	} else if(!hiddenRids) {
		return; // All rows are visible
	}

	let rids = this._toRowIds(rowRefs);
	let dirty = false;

	for(let i = rids.length; --i >= 0;) {
		let rid = rids[i];
		if(rid) { // undefined, null, and an empty string value are not a valid row id
			if(hidden) {
				if(!hiddenRids[rid]) {
					hiddenRids[rid] = true;
					dirty = true;
				}
			} else if(hiddenRids[rid]) {
				delete hiddenRids[rid];
				dirty = true;
			}
		}
	}

	if(dirty) {
		if(!hidden) {
			let hasHiddenRow = false;
			for(let key in hiddenRids) { // eslint-disable-line
				hasHiddenRow = true;
				break;
			}
			if(!hasHiddenRow) {
				hiddenRids = this._hiddenRids = null;
			}
		}
		this._refreshAndNotify(); // Very slow
	}

};
/**
 * Show/hide rows in the data view
 * @param {Array.<string|number>} rowRefs Array of row ids or indices to hide
 * @param {boolean=} shown=true If the value is false, the specified rows will be hidden.
 * @public
 */
DataView.prototype.showRows = function(rowRefs, shown) {
	this.hideRows(rowRefs, shown === false);
};
/** Unhide all rows
* @public
*/
DataView.prototype.unhideAllRows = function(){
	if(this._hiddenRids) {
		this._hiddenRids = null;
		this._refreshAndNotify();
	}
};
/** Check if there is any hidden row
* @public
* @return {boolean}
*/
DataView.prototype.hasHiddenRow = function(){
	return this._hiddenRids ? true : false;
};

/** @public
* @param {string|function(string, Object.<string, *>):boolean} cid Use string to identify column name. Use function for custom filter logic. Function will take rowId and column values as parameters, and return boolean value
* @param {*=} value
* @param {boolean=} opt_filteringOut
* @return {boolean} Return true if there is any change. Otherwise, return false.
* @fires DataView#pageCountChanged
* @fires DataView#pageIndexChanged
* @fires DataView#dataChanged
* @example
* dv.filterIn("FIELD", "Some value");
* dv.filterIn(function(rowId, rowData)) { // This is equivalent with the above statement
*     return rowData["FIELD"] === "Some value";
* });
*/
DataView.prototype.filterIn = function(cid, value, opt_filteringOut) {
	let newLogic = this._getFilterLogic(cid, value);
	let filteringOut = (opt_filteringOut === true);
	if(this._userFilter === newLogic && typeof cid !== "function") {
		if(!newLogic || filteringOut === this._filteringOut) {
			return false; // Nothing to update
		}
	}
	this._shared.filterLogic = newLogic;
	this._userFilter = newLogic;
	this._filteringOut = filteringOut;
	this._refreshAndNotify();
	return true;
};
/** @public
* @param {string|function(string, Object.<string, *>):boolean} cid Use string to identify column name. Use function for custom filter logic. Function will take rowId and column values as parameters, and return boolean value
* @param {*=} value
* @return {boolean} Return true if there is any change. Otherwise, return false.
* @fires DataView#pageCountChanged
* @fires DataView#pageIndexChanged
* @fires DataView#dataChanged
* @example
* dv.filterIn("FIELD", "Some value");
* dv.filterOut(function(rowId, rowData)) { // This is equivalent with the above statement
*     return rowData["FIELD"] !== "Some value";
* });
*/
DataView.prototype.filterOut = function(cid, value) {
	return this.filterIn(cid, value, true);
};
/** @public
* @param {string|Function} cid Use string to identify column name. Use function for custom filter logic. Function will take rowId and column values as parameters, and return boolean value
* @param {*=} value
* @param {boolean=} opt_filteringOut
* @fires DataView#pageCountChanged
* @fires DataView#pageIndexChanged
* @fires DataView#dataChanged
*/
DataView.prototype.filterInOnce = function(cid, value, opt_filteringOut) {
	let checker = this._getFilterLogic(cid, value);
	let removalMap = {};
	if(!this._getRemovalMap(
		removalMap,
		checker,
		(opt_filteringOut === true)
	)) {
		return;
	}

	let firstChange = this._removeRowIds(removalMap);

	let pageIndexChanged = this._updatePageCount();
	if(pageIndexChanged || this._transformView(firstChange)) {
		this._dispatchGlobalChange();
	} else {
		this._dispatchMultiRowRemoved(Object.keys(removalMap));
	}
};
/** @public
* @suppress {checkTypes}
* @param {string|function(string, Object.<string, *>):boolean} cid Use string to identify column name. Use function for custom filter logic. Function will take rowId and column values as parameters, and return boolean value
* @param {*=} value
* @fires DataView#pageCountChanged
* @fires DataView#pageIndexChanged
* @fires DataView#dataChanged
*/
DataView.prototype.filterOutOnce = function(cid, value) {
	this.filterInOnce(cid, value, true);
};
/** @private
* @param {!Object} removalMap
* @return {number}
*/
DataView.prototype._removeRowIds = function(removalMap) {
	let firstChange = DataView._removeArrayItems(this._rids, removalMap);
	if(this._groupView) {
		firstChange = DataView._removeArrayItems(this._groupView, removalMap);
	}

	if(this._groupMembers) {
		let len = this._groupList.length;
		for(let i = 0; i < len; ++i) {
			let groupId = this._groupList[i];
			this._groupMembers[groupId]._removeRowIds(removalMap);
		}
	}
	return firstChange;

};
/** @public
* @fires DataView#pageCountChanged
* @fires DataView#pageIndexChanged
* @fires DataView#dataChanged
*/
DataView.prototype.clearFilter = function() {
	if(this._userFilter) {
		this._userFilter = null;
		this._shared.filterLogic = null;
		this._refreshAndNotify();
	}
};

/** Disable the page index correction
* @public
* @param {boolean=} opt_disable
*/
DataView.prototype.disablePageCorrection = function(opt_disable) {
	this._pageCorrection = (opt_disable === false);
};
/** Goes to the specified page
* @public
* @param {number} pageIndex
* @return {boolean} True if there is any change, otherwise false
* @fires DataView#pageIndexChanged
* @fires DataView#dataChanged
*/
DataView.prototype.setPage = function (pageIndex) {
	let result = this._setPage(pageIndex);
	if(result) {
		this._dispatchGlobalChange();
	}
	return result;
};
/** {@link DataView#setPage}
* @public
* @function
* @fires DataView#pageIndexChanged
* @fires DataView#dataChanged
*/
DataView.prototype.gotoPage = DataView.prototype.setPage;
/** Goes to the next page. If current page is already the last page, nothing is performed
* @public
* @return {boolean} True if there is any change, otherwise false
* @fires DataView#pageIndexChanged
* @fires DataView#dataChanged
*/
DataView.prototype.nextPage = function () {
	return this.gotoPage(this._pageIndex + 1);
};
/** Goes back to the previous page. If current page is already the first page, nothing is performed
* @public
* @return {boolean} True if there is any change, otherwise false
* @fires DataView#pageIndexChanged
* @fires DataView#dataChanged
*/
DataView.prototype.previousPage = function () {
	return this.gotoPage(this._pageIndex - 1);
};
/** {@link DataView#previousPage}
* @public
* @function
* @fires DataView#pageIndexChanged
* @fires DataView#dataChanged
*/
DataView.prototype.prevPage = DataView.prototype.previousPage;
/** Goes to the last page
* @public
* @return {boolean} True if there is any change, otherwise false
* @fires DataView#pageIndexChanged
* @fires DataView#dataChanged
*/
DataView.prototype.lastPage = function () {
	return this.gotoPage(this._lastPageIndex);
};
/** Goes to the first page
* @public
* @return {boolean} True if there is any change, otherwise false
* @fires DataView#pageIndexChanged
*/
DataView.prototype.firstPage = function () {
	return this.gotoPage(0);
};

/**
* @public
* @return {number}
*/
DataView.prototype.getCurrentPage = function () {
	return this._pageIndex;
};
/** {@link DataView#getCurrentPage}
* @public
* @function
*/
DataView.prototype.getPageIndex = DataView.prototype.getCurrentPage;
/** Verifies that current page is the last page
* @public
* @return {boolean}
*/
DataView.prototype.isOnLastPage = function () {
	return (this._pageIndex === this._lastPageIndex);
};
/** Verifies that current page is the first page
* @public
* @return {boolean}
*/
DataView.prototype.isOnFirstPage = function () {
	return (this._pageIndex === 1);
};

/** Get total number of rows in a single page
* @public
* @return {number}
*/
DataView.prototype.getPageSize = function () {
	return this._pageSize;
};
/** Get total number of pages
* @public
* @return {number}
*/
DataView.prototype.getPageCount = function () {
	return this._lastPageIndex;
};
/** {@link DataView#getTotalPages}
* @public
* @function
*/
DataView.prototype.getTotalPages = DataView.prototype.getPageCount;
/** Shrink current view to the specified size. Set it 0 to reset to original size
* @public
* @param {number} pageSize
* @fires DataView#pageCountChanged
* @fires DataView#pageIndexChanged
* @fires DataView#dataChanged
*/
DataView.prototype.setPageSize = function (pageSize) {
	if(!(pageSize >= 0)) { pageSize = 0; }
	if(this._pageSize === pageSize) { return; }

	this._pageSize = pageSize;
	this._updatePageCount();

	if(pageSize > 0) {
		if(!this._pageView) {
			this._pageView = [];
		}
	} else {
		if(this._pageView) {
			this._pageView = null;
		}
	}

	if(!this._pageView || pageSize === 0) {
		this._updateRowIds();
	} else {
		// filtering and sorting should already be done beforehand
		this._transformView();
	}
	this._dispatchGlobalChange();
};

/** Group all rows by the specified criteria. The criteria can be field name (field value will be used as a group id) or a function that takes row data and returns group id.
* @public
* @param {string|Function|Array.<string|Function>} cid Column id or grouper method that take one parameter for row data in JSON format, and returns groupId (string)
* @param {Array=} opt_predefinedGroups
* @fires DataView#groupAdded
* @fires DataView#groupCriteriaChanged
* @fires DataView#pageCountChanged
* @fires DataView#pageIndexChanged
*/
DataView.prototype.groupBy = function (cid, opt_predefinedGroups) {
	// always deactivate multi-group row mode, it will automatically activate when populating group
	this._shared.multiGroupRow = false;

	if(this._groupCriteria) {
		if(this._groupList && this._groupList.length > 0) { // Dispose any existing member
			for(let key in this._groupMembers) {
				this._groupMembers[key].dispose(); // Clear everything in the data view
			}
			this._groupMembers = {}; // Clear all members
			this._groupList.length = 0;
		}
		this._groupCriteria = null;
	}
	if(opt_predefinedGroups != null) { // null is allowed
		this._setPredefinedGroups(opt_predefinedGroups);
	}

	let i, len;
	if(cid && this._groupLevel === 0) { // Sub group cannot be re-group without its parent noticing it
		this._groupCriteria = [];
		let ary;
		if(!Array.isArray(cid)) {
			ary = [cid];
		} else {
			ary = /** @type{Array.<string|Function>} */(cid);
		}
		len = ary.length;
		for(i = 0; i < len; ++i) {
			let criteria = ary[i];
			if(typeof criteria === "string") {
				this._groupCriteria.push(this._defaultGroupCriteria.bind(this, criteria));
			} else if(criteria) {
				this._groupCriteria.push(/** @type{Function} */(criteria));
			}
		}
	}

	if(this._groupCriteria) { // Only root view can have group criteria at this point
		if(!this._groupMembers) {
			this._groupMembers = {};
			this._groupList = [];
			this._groupRows = {};
		}
		if(!this._groupView) {
			this._groupView = [];
		}
	} else {
		this._groupMembers = null;
		this._groupList = null;
		this._groupView = null;
		if(this._groupRows) { // this._groupRows is a shared member
			if(this._groupLevel <= 0) {
				this._footerRowIds = null;
			} else {
				if(this._groupRowId) {
					delete this._groupRows[this._groupRowId];
				}
				if(this._footerRowIds) {
					len = this._footerRowIds.length;
					for(i = 0; i < len; ++i) {
						delete this._groupRows[this._footerRowIds[i]];
					}
					this._footerRowIds = null;
				}
			}
			this._groupRows = null;
		}
		this._groupValue = null;
	}
	this._groupLevel = 0; // View that is set criteria will always be in the first group level
	this._updateRowIds();

	this._dispatch("groupCriteriaChanged", {});
	this._dispatchGlobalChange();
};
/** addGroup is used to add a single predefined group or multiple predefined groups.
* @public
* @param {*|Array} value
* @fires DataView#groupAdded
* @fires DataView#pageCountChanged
* @fires DataView#pageIndexChanged
*/
DataView.prototype.addGroup = function (value) {
	if(!this._groupMembers) {
		// window.console.log("Group criteria must be specified first before adding any group");
		return;
	}

	if(!isArray(value)) {
		this._addGroup(value);
	} else {
		let ary = /** @type{Array} */(value);
		let len = ary.length;
		for(let i = 0; i < len; ++i) {
			this._addGroup(ary[i]);
		}
	}
};
/** Sort all of its descendants based on given compare function
* @public
* @param {Function=} compare Default is sorting by text. If null or false value is given, no sorting is performed
*/
DataView.prototype.sortGroups = function (compare) {
	if(!this._groupMembers) { return; }

	if(compare == null) {
		this._groupSortingLogic = DataView._defaultGroupSortingLogic; // Temporary set the variable
	} else {
		this._groupSortingLogic = compare || null;
	}
	if(!this._groupSortingLogic) {
		return; // No logic is defined
	}

	this._groupList.sort(this._internalGroupSortingLogic);
	for(let i = this._groupList.length; --i >= 0;){
		let groupId = this._groupList[i];
		this._groupMembers[groupId].sortGroups(this._groupSortingLogic);
	}
	this._groupViewDirty = true;
	if(this._groupLevel <= 0) {
		this._dispatchGlobalChange();
	}
};
/** Move the specified group within its group
* @public
* @param {number|string} groupRef
* @param {number} destIndex
*/
DataView.prototype.moveGroup = function (groupRef, destIndex) {
	let movedGroup = this.getGroup(groupRef);
	if(!movedGroup) {
		return; // Cannot find the given group
	}

	let parentGroup = movedGroup.getDataSource();
	let groupId = movedGroup.getGroupId();
	let curGroupIndex = parentGroup._groupList.indexOf(groupId);
	if(curGroupIndex === destIndex || curGroupIndex + 1 === destIndex) {
		return; // Nothing is needed to be moved
	}

	parentGroup._groupList.splice(curGroupIndex, 1);
	if(curGroupIndex < destIndex) {
		--destIndex;
	}
	parentGroup._groupList.splice(destIndex, 0, groupId);

	this._requestRebuildingRowIds(false);
};
/** WARNING: This could be very slow to perform
* @public
* @param {Array} predefinedGroups Use two dimensional array to define groups for multi-level grouping. The first item is for the top level groups. The second item is for the second level groups, and so on. If one dimentional array is given, it is treated as the first item in the two-dimentionsal array.
* @example
* dv.setPredefinedGroups(["a", "b", "c"]); // Create empty three empty groups
* dv.setPredefinedGroups([ // Create two empty groups, and each group has 3 sub-groups
*   ["Main Group 1", "Main 2"],
*   ["Sub 1", "Sub 2", "Sub 3"]
* ]);
*/
DataView.prototype.setPredefinedGroups = function (predefinedGroups) {
	this._setPredefinedGroups(predefinedGroups);
	if(this._groupMembers) {
		this._refreshAndNotify();
	}
};
/** @private
* @param {Array} predefinedGroups
*/
DataView.prototype._setPredefinedGroups = function (predefinedGroups) {
	if(this._preDefGroups === predefinedGroups) {
		return;
	}

	if(this._preDefGroups) {
		this._preDefGroups.length = 0; // Remove all predefined groups in all members (this let is shared)
	}
	let is2D = false;
	if(predefinedGroups) {
		for(let i = predefinedGroups.length; --i >= 0;) {
			if(Array.isArray(predefinedGroups[i])) {
				is2D = true;
				break;
			}
		}
	}

	if(is2D) {
		this._preDefGroups = predefinedGroups; // WARNING: Referencing user's object
	} else if(Array.isArray(predefinedGroups)) {
		this._preDefGroups = [predefinedGroups]; // Convert to 2D Array
	} else {
		this._preDefGroups = null;
	}
	this._preDefGroupsDirty = true;
};

/** Add a footer to current group
* @public
* @param {number=} opt_num Number of footer to be added. If not specified, the number is one.
*/
DataView.prototype.addFooter = function (opt_num) {
	if(this._groupLevel <= 0) {
		console.log("WARNING: Only group can have a footer");
		return;
	}
	if(opt_num == null) {
		opt_num = 1;
	}
	if(opt_num <= 0) { // 0 and negative number is not acceptable
		return;
	}

	if(!this._footerRowIds) {
		this._footerRowIds = [];
	}

	for(let i = 0; i < opt_num; ++i) {
		let footerRowId = this._groupRowId + "_footer" + (this._footerRowIds.length || "");
		this._footerRowIds.push(footerRowId);
		this._groupRows[footerRowId] = this;
	}

	if(!this._beingAdded) {
		this._requestRebuildingRowIds(this._collapsed); // Slow
	}
};
/** Remove all footers in the gurrent view
* @public
*/
DataView.prototype.removeFooter = function () {
	if(this._footerRowIds) {
		let len = this._footerRowIds.length;
		for(let i = 0; i < len; ++i) {
			delete this._groupRows[this._footerRowIds[i]];
		}
		this._footerRowIds = null;

		if(!this._beingAdded) {
			this._requestRebuildingRowIds(this._collapsed); // Slow
		}
	}
};

/** Enable/Disable multi-group row mode
* @public
* @ignore
* @param {boolean=} enabled
*/
DataView.prototype.enableMultiGroup = function(enabled) {
	this._shared.multiGroupRow = enabled !== false;
};

/** Get a group DataView by group row id. If content row id is given, null is returned (unlike getGroup method).<br>
* Nested groups and other groups outside of this group branch will be searched.
* @public
* @param {string} rowId Group Id, row Id or group index (for immediate child groups)
* @return {DataView}
* @see {@link DataView#getGroup}
*/
DataView.prototype.getGroupByRowId = function (rowId) {
	if(this._groupRows) {
		return this._groupRows[rowId] || null;
	}
	return null;
};
/** Get a corresponding DataView by the given group Id, row Id or group index. If row Id of content row is given, a parent group will be returned. <br>
* Nested groups and other groups outside of this group branch will be searched.
* @public
* @param {number|string} groupRef Group Id, row Id or group index (for immediate child groups)
* @return {DataView}
* @see {@link DataView#getGroupByRowId}
*/
DataView.prototype.getGroup = function (groupRef) {
	if(!this._groupMembers) {
		return null;
	}
	let groupId;
	if(typeof groupRef === "number") {
		groupId = this._groupList[groupRef];
		if(!groupId) {
			return null;
		}
	} else {
		groupId = groupRef + ""; // Convert to string
	}

	// Search immediate child groups by group Id
	let dv = this._groupMembers[groupId];
	if(dv) {
		return dv;
	}

	// A group cannot be found by group Id. It is now assumed to be row id
	let rowId = groupId;
	dv = this._groupRows[rowId]; // Groups of any level will be searched
	if(dv) {
		return dv;
	}

	// The given row id is not a group row. Check if it is actually content row of this group
	return this._getGroupByContentRowId(rowId); // Slow recursive function
};
/** Get group of this data view by content row id. Group outside of this group branch is NOT searched
* @private
* @param {string} rowId
* @return {DataView} Group of rowId that given
*/
DataView.prototype._getGroupByContentRowId = function (rowId) {
	if(this._groupMembers) {
		for (let i = 0; i < this._groupList.length; i++) {
			let groupId = this._groupList[i];
			let grp = this._groupMembers[groupId];
			let childGroup = grp._getGroupByContentRowId(rowId);
			if(childGroup) {
				return childGroup;
			}
		}
	} else { // Last childGroup
		if(this._rids.indexOf(rowId) !== -1) {
			return this; // return group if match with rowId
		}
	}
	return null;
};
/** Get all group descendants
* @public
* @param {Array=} out_ary
* @return {!Array.<DataView>}
*/
DataView.prototype.getAllGroups = function (out_ary) {
	if(!out_ary) { out_ary = []; }
	if(this._groupMembers) {
		let len = this._groupList.length;
		for(let i = 0; i < len; ++i) {
			let groupId = this._groupList[i];
			let grp = this._groupMembers[groupId];
			out_ary.push(grp);
			grp.getAllGroups(out_ary);
		}
	}
	return out_ary;
};
/** Get immediate group members row ids in array list
* @public
* @return {Array.<string>}
*/
DataView.prototype.getGroupList = function () {
	return this._groupList;
};
/** Get group id (string) of this data view. Only exists if this data view is a child data view
* @public
* @return {string}
*/
DataView.prototype.getGroupId = function () {
	return this._groupId;
};
/** Get group value (any data type) of this data view. Only exists if this data view is a child data view
* @public
* @return {*}
*/
DataView.prototype.getGroupValue = function () {
	return this._groupValue;
};
/** Row id used by parent data view as a header row for referencing
* @public
* @return {string}
*/
DataView.prototype.getGroupRowId = function () {
	return this._groupRowId;
};
/**
* @public
* @ignore
* @param {string} rid
* @return {number}
*/
DataView.prototype.getFooterRowIndex = function (rid) {
	if(this._footerRowIds) {
		for(let i = this._footerRowIds.length; --i >= 0;) {
			if(this._footerRowIds[i] === rid) {
				return i;
			}
		}
	}
	return -1;
};
/** A List of the footer row ids. Footer row is placed at the bottom of all other rows in this view
* @public
* @ignore
* @return {Array.<string>} Null if there is no footer row
*/
DataView.prototype.getFooterRowIds = function () {
	return this._footerRowIds;
};
/**
* @public
* @return {boolean}
*/
DataView.prototype.hasGroup = function () {
	return (this._groupMembers) ? true : false;
};
/** Deprecated in favor of getGroupByRowId
* @public
* @return {null}
* @see {@link DataView#getGroupByRowId}
*/
DataView.prototype.getGroupRows = function () {
	return null;
};
/** Get group level of the corresponding row. If the given index is not specified, header row (the view itself) is used instead. Group level 0 means it is the root data view.
* @public
* @param {(number|string)=} rowRef
* @return {number}
*/
DataView.prototype.getGroupLevel = function (rowRef) {
	if(rowRef != null && this._groupRows) {
		if(typeof rowRef === 'number'){ // TODO: Change logic to acquire row id from row index instead
			this._rebuildRowIdsFromChildren();
			let rids = this._getRids(); // TODO : Should include hidden rows
			let rid = rids[rowRef];
			while(rid) {
				let dv = this._groupRows[rid];
				if(dv) {
					return dv._groupLevel;
				}
				rid = rids[--rowRef];
			}
		} else {
			let currentGroup = this.getGroup(rowRef); // Slow becuase recursive
			if(currentGroup) {
				return currentGroup._groupLevel;
			}
			return -1; // group not found
		}
	}
	return this._groupLevel;
};

/** Statistics (e.g. Sum, Avg, Max, and Min) of the specified column will be collected on every update. WARNING: CPU usage is intensive. Do disable, if the column is no longer used.
* @public
* @param {string} cid
* @param {boolean=} opt_enabled
* @param {string=} opt_refId
* @return {ColumnStats} ColumnStats Object
*/
DataView.prototype.enableColumnStats = function(cid, opt_enabled, opt_refId) {
	if (!cid) { return null; }
	let enabled = opt_enabled !== false;
	let cs = this._columnStats[cid];
	if(enabled) {
		if(!cs) {
			cs = new data_ColumnStats(this, cid);
			this._columnStats[cid] = cs;
		}
		cs.addRef(opt_refId);
	} else {
		if(cs) {
			if(!cs.deRef(opt_refId)) {
				cs.dispose();
				delete this._columnStats[cid];
			}
		}
	}
	if(this._groupMembers) {
		for(let i = this._groupList.length; --i >= 0;) {
			let groupId = this._groupList[i];
			this._groupMembers[groupId].enableColumnStats(cid, opt_enabled, opt_refId);
		}
	}
	return cs || null;
};
/** @public
 * @param {string} cid
* @return {ColumnStats} ColumnStats. Return null if the column has not been enabled
*/
DataView.prototype.getColumnStats = function(cid) {
	return this._columnStats[cid] || null;
};

/** This method temporarily remove all rows from the view, which can improve performance as further updates and operations will be ignored
* @public
* @param {boolean=} opt_hidden
* @returns {boolean}
*/
DataView.prototype.hide = function(opt_hidden) {
	let val = opt_hidden !== false;
	if(this._hidden === val) {
		return false;
	}

	this._hidden = val;
	this._refreshAndNotify();
	return true;
};
/** @public
* @return {boolean}
*/
DataView.prototype.isHidden = function() {
	return this._hidden;
};
/** @public
* @param {string} rowId
* @return {boolean} Returns true when this row is hidden
*/
DataView.prototype.isHiddenRow = function(rowId) {
	if(this._hiddenRids) {
		return this._hiddenRids[rowId] === true;
	}
	return false;
};

/** @private
* @return {boolean}
*/
DataView.prototype._isHeaderRowShown = function() {
	if (this._hiddenHeaderRow) {
		return false;
	}
	if (this._shared.filterLogic && this._rids.length === 0) {
		// When filter is active, group without its member will be hidden
		return false;
	}

	// Group member with option autoGroupHiding enabled should always set `_collapsed` to false. Otherwise group will be collapsed and cannot be expanded by UI.
	if (this._shared.autoGroupHiding && this._groupValue == null) {
		// In auto grouping mode, if group value is either undifined or null the header row will be hidden
		return false;
	}
	return true;
};
/** @private
* @param {boolean} collapsed
* @return {boolean} Return true if there is any change
*/
DataView.prototype._setCollapse = function(collapsed) {
	if (this._shared.contentAsHeader) {
		// In this mode, only groups with hidden group header can be expanded/collapsed.
		// Ignore any group with visible group header
		if (this._hiddenHeaderRow) {
			if (this._showOnlyFirstMember !== collapsed) {
				this._showOnlyFirstMember = collapsed;
				return true;
			}
		}
	} else if (this._isHeaderRowShown()) { // Only groups with visible group header can be expanded/collapsed
		if (this._collapsible && this._collapsed !== collapsed) {
			this._collapsed = collapsed;
			return true;
		}
	}
	return false;
};
/** Collapse current view. This works only if the view is generated as a child view
* @public
* @param {boolean=} opt_collapsed
*/
DataView.prototype.collapse = function(opt_collapsed) {
	let dirty = this._setCollapse(opt_collapsed !== false);

	if(dirty) {
		if (this._shared.contentAsHeader) {
			if (this._hiddenHeaderRow) {
				this._requestRebuildingRowIds(this._showOnlyFirstMember); // Slow
			}
		} else {
			if(this._groupLevel > 0) {
				this._requestRebuildingRowIds(this._collapsed);
			}
		}
	}
};
/** Collapse all children and descendants, excluding itself and its parent
* @public
* @param {boolean=} opt_collapsed
*/
DataView.prototype.collapseAllGroups = function(opt_collapsed) {
	let grpList = this.getAllGroups();
	let len = grpList.length;
	if(len) {
		let collapsed = (opt_collapsed !== false);
		for(let i = 0; i < len; ++i) {
			let grpItem = grpList[i];
			grpItem._setCollapse(collapsed);
		}
		this._requestRebuildingRowIds(false);
	}
};
/** @public
*/
DataView.prototype.toggleCollapsedState = function() {
	this.collapse(!this.isCollapsed());
};
/** @public
* @return {boolean}
*/
DataView.prototype.isCollapsed = function() {
	return this._collapsed || this._showOnlyFirstMember;
};
/** Content as header mode indicates that the group headers cannot be collapsed and the first row in the group will always be shown.
* @public
* @param {boolean=} bool
*/
DataView.prototype.enableContentAsHeader = function(bool) {
	this._shared.contentAsHeader = bool !== false;
};
/** set default collapse when new groups added
* @public
* @param {boolean=} collapse=false, if enable it, it will be set collapsing by default
*/
DataView.prototype.setDefaultCollapse = function(collapse) {
	this._shared.defaultCollapse = !!collapse;
};
/** get default collapse when new groups added
* @public
* @return {boolean}
*/
DataView.prototype.getDefaultCollapse = function() {
	return !!this._shared.defaultCollapse; // it can be null, convert to boolean
};
/**
* @public
* @return {boolean}
*/
DataView.prototype.isCollapsible = function() {
	return this._collapsible;
};
/**
* @public
* @param {boolean=} bool=true, if set it to false, it will be enable collapsing
*/
DataView.prototype.disableCollapsing = function(bool) {
	this._collapsible = !(bool !== false);
};
/**
* @public
* @param {string} groupColor
*/
DataView.prototype.setGroupColor = function(groupColor) {
	if(this._groupLevel === 1) {
		this._groupColor = groupColor;
	}
};
/**
* @public
* @return {string}
*/
DataView.prototype.getGroupColor = function() {
	if(this._groupLevel === 0) {
		return "";
	} else if(this._groupLevel === 1) {
		return this._groupColor;
	}
	return this._parent.getGroupColor();
};
/** @private
* @return {?Array.<string>}
*/
DataView.prototype._getFirstHiddenRids = function() {
	if(this._shared.contentAsHeader) {
		if(this._showOnlyFirstMember) {
			let rids = this._rids;
			if (Array.isArray(rids) && rids.length) {
				return [rids[0]]; // Get only the first member of rids
			}
		}
	}
	return null;
};

/** This will not include collapsed group and rows
* @public
* @return {number}
*/
DataView.prototype.getVisibleRowCount = function() {
	if(this._collapsed) {
		return 0;
	}
	this._rebuildRowIdsFromChildren();
	return this._getRids().length;
};

/** @public
* @description check weather row object is in group
* @param {Object} rowData
* @param {string=} groupID Default group id is the group id of the DataView that is being called.
* @return {boolean}
*/
DataView.prototype.isRowDataInGroup = function (rowData, groupID) {
	let criteria = this._groupCriteria;
	let level = this._groupLevel;

	// Because self is a leaf group, only check against self.
	if (!groupID || groupID === this.getGroupId()) {
		return this._isAllowedInGroup(null, rowData);
	}

	// Check against self and all sub groups within self
	for (let i = level; i < criteria.length; i++) {
		if (criteria[i](rowData).indexOf(groupID) >= 0) {
			return true;
		}
	}
	return false;
};

/** @public
* @description check weather row ID is in group
* @param {string} rowId
* @param {string=} groupID Default group id is the group id of the DataView that is being called.
* @return {boolean}
*/
DataView.prototype.isRowIdInGroup = function (rowId, groupID) {
	let rowData = this.getRowData(rowId);
	if(rowData) {
		return this.isRowDataInGroup(rowData, groupID);
	}
	return false;
};

/** @private
* @param {string} rid Row Id
* @param {number} _index child index
* @return {string}
*/
DataView._getDataViewExpansionId = function (rid, _index) {
	return rid + "_ex" + _index;
};

/** The row expansion will be directly below the specified row, even if this data view is sorted.
* @public
* @param {string} rid Row Id
* @param {number=} count=1
*/
DataView.prototype.addRowExpansion = function(rid, count) {
	if(this._groupLevel > 0) {
		return; // Sub-group is not allowed to add a row expansion
	}

	if(!count || count < 0) {
		if(count == null) {
			count = 1;
		} else {
			return;
		}
	}

	let rowIndex = this._rids.indexOf(rid);
	if(rowIndex < 0) {
		// The given row id must be normal row. header and expansion row are not in the this._rids list
		return;
	}

	let expCount = 0;
	if(this._expansionMap) {
		expCount = this._expansionMap[rid] || 0;
	} else {
		this._expansionMap = {};
		this._expansionChildren = {};
	}
	let totalChild = expCount + count;
	this._expansionMap[rid] = totalChild;

	// create _expansionChildren mapping

	for (let i = expCount; i < totalChild; i++) {
		this._expansionChildren[DataView._getDataViewExpansionId(rid, i)] = rid;
	}

	if(!expCount) {
		// There is no row expansion for the row before, add number of entries to the map
		this._expansionCount++;
	}

	if(this._groupViewDirty) { // If group view is already dirty, we don't need to update row expansion right away
		return;
	}

	if(this._updateExpansionView()) {
		let e = {}; // Slow
		e["rid"] = rid;
		e["type"] = "inserted";
		e["nextRid"] = this._rids[rowIndex + 1];
		e["fallback"] = rowIndex; // Fallback index in case of no nextRid is found in the receiver
		this._dispatchDataChange(e);
	}
};
/**
* @public
* @param {string} rid Row Id
*/
DataView.prototype.removeRowExpansion = function(rid) {
	if(!this._expansionMap) {
		return;
	}
	let expCount = this._expansionMap[rid];
	if(!expCount) {
		return;
	}

	let rids = new Array(expCount);
	for (let i = 0; i < expCount; ++i) {
		rids[i] = DataView._getDataViewExpansionId(rid, i);
		delete this._expansionChildren[rids[i]];
	}

	delete this._expansionMap[rid];
	this._expansionCount--;

	if(!this._groupViewDirty) { // If group view is already dirty, we don't need to update row expansion right away
		if(this._updateExpansionView()) {
			if(expCount === 1) {
				this._dispatchRowRemoved(rids[0]); // TODO: Expansion rid may not need to be sent
			} else {
				this._dispatchMultiRowRemoved(rids);
			}
		}
	}
};
/**
* @public
* @param {string|number} rowRef Row Id or index
* @param {boolean=} force
* @param {number=} count=1
*/
DataView.prototype.toggleRowExpansion = function(rowRef, force, count) {
	let rid = this._toRowId(rowRef);

	if(force != null) {
		force = force === true;
	} else {
		force = !this.hasRowExpansion(rid);
	}

	if (force) {
		this.addRowExpansion(rid, count);
	} else {
		this.removeRowExpansion(rid);
	}
};
/** @public
*/
DataView.prototype.removeAllRowExpansions = function() {
	if(this._expansionMap) {
		this._expansionMap = null;
		this._expansionCount = 0;
		this._expansionView = null;
		this._expansionChildren = null;
		this._refreshAndNotify();
	}
};
/** Deprecated. Use {@link DataView#getExpansionParents} instead. Get all rows with expansion (not row expansion itself) will contains row id, otherwise undefined.
* @public
* @ignore
* @param {number=} from
* @param {number=} to EXCLUSIVE
* @return {!Array.<string|undefined>}
*/
DataView.prototype.getRowsWithExpansion = function(from, to) {
	if(!this._expansionMap) {
		return [];
	}
	let rids = this._expansionView;
	let len = rids.length;
	if(to == null) { to = len; }
	else if(to > len) { to = len; }

	if(!(from > 0)) { from = 0; } // Handle NaN, undefined, null, and other invalid value

	let entryCount = this._expansionCount;
	let out_ary = new Array(len);
	for(let i = from; i < to; ++i) {
		let rid = rids[i];
		if(this._expansionMap[rid]) {
			out_ary[i] = rid;
			if(--entryCount <= 0) {
				break;
			}
		}
	}
	return out_ary;
};
/** Get row expansion states from the specified range. The returned value will be row id for row expansion, otherwise empty string for other types of row (e.g. normal row, or group header row)
* @public
* @param {number=} from
* @param {number=} to EXCLUSIVE
* @return {Array.<string>} object mapping of rowIndex and parentRowId
*/
DataView.prototype.getExpansionParents = function (from, to) {
	let parentMapping = [];
	if (this._expansionMap) {
		let rids = this._expansionView;
		let len = rids.length;

		// Handle NaN, undefined, null, and other invalid values
		if (to == null || to > len) {
			to = len;
		}
		if (!from || from < 0) {
			from = 0;
		}

		for (let i = from; i < to; ++i) {
			parentMapping[i] = this._expansionChildren[rids[i]] || "";
		}
	}
	return parentMapping;
};

/**
* @public
* @param {string} rid
* @return {string} parent row id
*/
DataView.prototype.getExpansionParentByRowId = function(rid) {
	if(this._expansionChildren) {
		return this._expansionChildren[rid] || "";
	}
	return "";
};
/** Check if the specified row is itself a row expansion.
* @public
* @param {string} rid rowId
* @return {boolean}
*/
DataView.prototype.isRowExpansion = function(rid) {
	if(this._expansionChildren) {
		return this._expansionChildren[rid] ? true : false;
	}
	return false;
};
/** Check if the specified row has row expansion (not row expansion itself).
* @public
* @param {string} rid rowId
* @return {boolean}
*/
DataView.prototype.hasRowExpansion = function(rid) {
	if(this._expansionMap) {
		return this._expansionMap[rid] ? true : false;
	}
	return false;
};
/**
* @public
* @param {string} rowId
* @return {number}
*/
DataView.prototype.getRowExpansionCount = function(rowId) {
	if(this._expansionMap) {
		return this._expansionMap[rowId] || 0;
	}
	return 0;
};

/** Get row Id of the row expansion by providing the row Id of the row with row expansion (not row expansion itself). This is NOT recommended for normal usage, since the row expansion contains no data in the DataView
* @public
* @param {string} rid rowId
* @param {number=} idx=0
* @return {string} Empty string is returned if the provided row does not contains a row expansion.
*/
DataView.prototype.getRowExpansionId = function(rid, idx) {
	if(this._expansionMap) {
		let expCount = this._expansionMap[rid] || 0;
		if(!idx) {
			idx = 0;
		}
		if (idx < expCount) {
			return DataView._getDataViewExpansionId(rid, idx); // WARNING: Internal logic is exposed
		}
	}
	return "";
};

/** Get index from the row expansion id. Any row other than row expansion will yield negative value (-1)
* @public
* @param {string} rid Row expansion id
* @return {number}
*/
DataView.prototype.getRowExpansionIndex = function(rid) {
	if(rid) {
		let matches = rid.match(/_ex(\d+)$/);
		if(matches) {
			return +matches[1]; // TODO: Check for NaN value
		}
	}
	return -1;
};
/** @private
* @return {boolean}
*/
DataView.prototype._updateExpansionView = function() {
	let expMap = this._expansionMap;
	if(expMap) {
		if(!this._expansionCount || this._expansionCount < 0) {
			this._expansionCount = 0;
			this._expansionMap = null;
			this._expansionView = null;
			this._expansionChildren = null;
			return true;
		}

		let rids = this._pageView || this._groupView || this._rids;
		let rowLen = rids.length;
		let origLen = this._expansionView ? this._expansionView.length : 0;
		if(!this._expansionView) {
			this._expansionView = new Array(rowLen);
		} else {
			this._expansionView.length = rowLen;
		}
		let entryCount = this._expansionCount;
		for(let i = 0, j = 0; i < rowLen; ++i) {
			let rid = rids[i];
			this._expansionView[j++] = rid;
			if(entryCount) {
				let expCount = expMap[rid] || 0;
				if(expCount) {
					for(let k = 0; k < expCount; ++k) {
						this._expansionView[j++] = DataView._getDataViewExpansionId(rid, k);
					}
					--entryCount;
				}
			}
		}
		return origLen !== this._expansionView.length; // TODO: This does not completely check the change of all cases
	} else {
		return false;
	}
};
/** Guarantee that expansion is added after the pagination view
* @private
* @param {number=} firstChange
* @return {boolean}
*/
DataView.prototype._transformView = function(firstChange) {
	let dirty = this._updatePaginationView(firstChange);
	dirty |= this._updateExpansionView();
	return dirty ? true : false;
};

/**
* @public
* @param {function(Object):boolean} searchLogic Function that takes row data object and return boolean value
* @param {Array.<string>=} opt_out Array for containing row ids
* @return {!Array.<string>} List of row ids
*/
DataView.prototype.searchAll = function(searchLogic, opt_out) {
	if(!opt_out) { opt_out = []; }

	this._rebuildRowIdsFromChildren();
	let rids = this._getRids();
	let len = rids.length;

	for(let i = 0; i < len; ++i) {
		let rid = rids[i];
		let row = this._dt.getRowData(rid);
		if(row) {
			if(searchLogic(row)) {
				opt_out.push(rid);
			}
		}
	}
	return opt_out;
};
/**
* @public
* @param {number|string} rowRef
* @param {function(Object):boolean} searchLogic Function that takes row data object and return boolean value
* @return {number} Row index
*/
DataView.prototype.searchPrev = function(rowRef, searchLogic) {
	this._rebuildRowIdsFromChildren();
	let rids = this._getRids();
	let rowIndex = -1;
	if(typeof rowRef === "number") {
		if(this._dt.getRowData(rids[rowRef])) {
			rowIndex = rowRef;
		}
	} else {
		rowIndex = rids.indexOf(rowRef);
	}

	let foundAt = -1;
	let len = rids.length;
	if(rowIndex > 0) {
		foundAt = this._searchRow(rids, searchLogic, rowIndex - 1, -1, -1);
		if(foundAt >= 0) {
			return foundAt;
		}
	}
	if(rowIndex >= 0) { // loop the search by starting from the end
		foundAt = this._searchRow(rids, searchLogic, len - 1, rowIndex, -1);
		if(foundAt >= 0) {
			return foundAt;
		}
	}
	return -1;
};
/**
* @public
* @param {number|string} rowRef
* @param {function(Object):boolean} searchLogic Function that takes row data object and return boolean value
* @return {number} Row index
*/
DataView.prototype.searchNext = function(rowRef, searchLogic) {
	this._rebuildRowIdsFromChildren();
	let rids = this._getRids();
	let rowIndex = -1;
	if(typeof rowRef === "number") {
		if(this._dt.getRowData(rids[rowRef])) {
			rowIndex = rowRef;
		}
	} else {
		rowIndex = rids.indexOf(rowRef);
	}
	if(rowIndex < 0) { // Cannot find the specified row
		return -1;
	}

	let foundAt = -1;
	let len = rids.length;
	if(rowIndex < len - 1) {
		foundAt = this._searchRow(rids, searchLogic, rowIndex + 1, len, 1);
		if(foundAt >= 0) {
			return foundAt;
		}
	}
	if(rowIndex > 0) { // loop the search by starting from the start
		foundAt = this._searchRow(rids, searchLogic, 0, rowIndex, 1);
		if(foundAt >= 0) {
			return foundAt;
		}
	}
	return -1;
};
/**
* @private
* @param {!Array.<string>} rids
* @param {function(Object):boolean} searchLogic
* @param {number} start
* @param {number} end
* @param {number} inc
* @return {number}
*/
DataView.prototype._searchRow = function(rids, searchLogic, start, end, inc) {
	while(start !== end) {
		let rid = rids[start];
		let row = this._dt.getRowData(rid);
		if(row) {
			if(searchLogic(row)) {
				return start;
			}
		}
		start += inc;
	}
	return -1;
};

/** Blocks all row insertion and removal, but keep updating the data
* @public
* @param {boolean=} bool
* @return {boolean} Returns true, if there is any change
*/
DataView.prototype.stall = function(bool) {
	let newBool = bool !== false;
	let curBool = this._shared.stalledRids ? true : false;
	if(curBool === newBool) { return false; }
	if(newBool) {
		let srs = {};
		this._shared.stalledRids = srs;
		let rids = this._getRids();

		for(let i = rids.length; --i >= 0;) {
			srs[rids[i]] = 1;
		}
	} else {
		this._shared.stalledRids = null;
		this._refreshAndNotify();
	}
	return true;
};
/** Blocks existing sorting from performing on every data change
* @public
* @param {boolean=} bool
* @return {boolean} Returns true, if there is any change
*/
DataView.prototype.stallSorting = function(bool) {
	let newBool = bool !== false;
	if(this._stalledSorting !== newBool) {
		this._stalledSorting = newBool;

		if(!newBool && this.isSorting()) {
			if(this._sort()) {
				this._dispatchDataChange(data_DataTable._positionChangeArg);
			}
		}
		return true;
	}
	return false;
};

/** Automatically and asyncronuosly remove group that has no member or no content. Predefined groups will not be removed in this way.
* @public
* @param {boolean=} opt_bool
* @return {boolean} True if there is any change
*/
DataView.prototype.enableAutoGroupRemoval = function(opt_bool) {
	opt_bool = opt_bool !== false;
	let autoGroupRemoval = this._shared.autoGroupRemoval ? true : false;
	if(autoGroupRemoval !== opt_bool) {
		this._shared.autoGroupRemoval = opt_bool;
		this._triggerGroupRefreshing();
		return true;
	}
	return false;
};
/** Automatically hide null or undefined group header, while keeping the group's items intact.
* @public
* @param {boolean=} opt_bool
* @return {boolean} True if there is any change
*/
DataView.prototype.enableAutoGroupHiding = function(opt_bool) {
	opt_bool = opt_bool !== false;
	let autoGroupHiding = this._shared.autoGroupHiding ? true : false;
	if(autoGroupHiding !== opt_bool) {
		this._shared.autoGroupHiding = opt_bool;
		this._triggerGroupRefreshing();
		return true;
	}
	return false;
};

/** Copy row order in data view to its data table
* @public
*/
DataView.prototype.synchronizeRowOrder = function() {
	if(!this._stalledSorting && this.isSorting()) {
		this._dt._sort(this._sortingDefs);
	}
};
/**
* @public
* @param {Array<string>} rowIds
* @param {boolean=} enabled
* @return {boolean} Return true if there is any change
*/
DataView.prototype.setSegmentSeparators = function(rowIds, enabled) {
	if(rowIds) {
		enabled = enabled !== false;
		if(enabled) {
			this.synchronizeRowOrder();
		}
		// TODO: Force expanding of segment before unsetting segment separator
		return this._dt.setSegmentSeparators(rowIds, enabled);
	}
	return false;
};
/** Set visible row as a segment separator (hidden or filtered rows cannot be a segment separator)
* @public
* @param {string|number} rowRef Row id or row index
* @param {*=} options=null Segment options. If the value is false, segment separator will be stripped off from the given rid
* @return {boolean} Return true if there is any change
*/
DataView.prototype.setSegmentSeparator = function(rowRef, options) {
	let rowId = "";
	if(typeof rowRef === "number") {
		rowId = this.getRowId(rowRef);
	} else if(typeof rowRef === "string") {
		rowId = rowRef; // WARNING: Invisible row id can still be set as segment separator as long as it is in the data table
	}

	if(rowId) {
		if(options !== false) { // undefined, null, object, or true value
			this.synchronizeRowOrder();
		}
		// TODO: Force expanding of segment before unsetting segment separator
		return this._dt.setSegmentSeparator(rowId, options);
	}
	return false;
};
/**
* @public
* @return {boolean} Return true if there is any change
*/
DataView.prototype.unsetAllSegmentSeparators = function() {
	return this._dt.unsetAllSegmentSeparators();
};
/**
* @public
* @return {boolean}
*/
DataView.prototype.hasSegmentation = function() {
	return this._dt.hasSegmentation();
};
/**
* @public
* @param {string} rid
* @return {boolean}
*/
DataView.prototype.isSegmentSeparator = function(rid) {
	return this._dt.isSegmentSeparator(rid);
};

/** Get Segment object
* @public
* @param {string} rid Row id of a segment
* @return {Segment} Returns null if a segment cannot be found
*/
DataView.prototype.getSegment = function(rid) {
	return this._dt.getSegment(rid);
};
/**
* @public
* @param {string} rid Row id of a segment child
* @return {Segment} Returns null if a segment cannot be found
*/
DataView.prototype.getSegmentParent = function(rid) {
	return this._dt.getSegmentParent(rid);
};
/** Segment level starts from 1
* @public
* @param {string} rid Row id
* @return {number} Return 0 if the given rid is not a segment
*/
DataView.prototype.getSegmentLevel = function(rid) {
	return this._dt.getSegmentLevel(rid);
};
/**
* @public
* @param {string} rid
* @return {string} parent row id of this segmentation. If the parent row id for this segmentation cannot be found, return ""
*/
DataView.prototype.getSegmentParentRowId = function(rid) {
	return this._dt.getSegmentParentRowId(rid);
};
/** @public
* @param {string|number} rowRef Row id or row index
* @param {boolean=} collapsed
* @return {boolean} Return true if there is any change
*/
DataView.prototype.collapseSegment = function(rowRef, collapsed) {
	collapsed = collapsed !== false;
	let segments = this._dt._getSegmentSeparators();
	if(segments) {
		let segmentRid = this._toRowId(rowRef);

		if(segments.collapseSegment(segmentRid, collapsed)) {
			this._refreshAndNotify(); // dispatch global change event

			return true;
		}
	}

	return false;
};
/** @public
* @param {Array<string|number>} rowIds
* @param {boolean=} collapsed
* @return {boolean} Return true if there is any change
*/
DataView.prototype.collapseSegments = function(rowIds, collapsed) {
	collapsed = collapsed !== false;
	let segments = this._dt._getSegmentSeparators();
	if(segments) {
		if(segments.collapseSegments(rowIds, collapsed)) {
			this._refreshAndNotify(); // dispatch global change event
			return true;
		}
	}
	return false;
};
/** @public
* @param {string|number} rowRef Row id or row index
* @param {boolean=} expanded
* @return {boolean} Return true if there is any change
*/
DataView.prototype.expandSegment = function(rowRef, expanded) {
	return this.collapseSegment(rowRef, expanded === false);
};
/** @public
* @return {boolean} Return true if there is any change
*/
DataView.prototype.expandAllSegments = function() {
	let segments = this._dt._getSegmentSeparators();
	if(segments) {
		if(segments.expandAllSegments()) {
			this._refreshAndNotify();
			return true;
		}
	}
	return false;
};
/** Deprecated.
* @public
* @ignore
*/
DataView.prototype.setSegmentCollapsingLogic = function() {};
/** @public
* @param {string|number} rowRef Row id or row index
* @return {boolean} Return true if only the given row is a collapsed segment separator
*/
DataView.prototype.isSegmentCollapsed = function(rowRef) {
	let segments = this._dt._getSegmentSeparators();
	if(segments) {
		return segments.isCollapsedSegment(this._toRowId(rowRef));
	}
	return false;
};
/** Remove existing segment children and fill the segments with all contnet rows before the next segment separator
* @public
* @param {string|number} segmentRef Row id or row index
*/
DataView.prototype.fillSegment = function(segmentRef) {
	this._dt.fillSegment(this._toRowId(segmentRef));
};
/** Remove all existing segment children in each segment and fill the segments with all contnet rows before the next segment separator
* @public
*/
DataView.prototype.fillSegments = function() {
	this._dt.fillSegments();
};
/** @public
* @param {string|number} segmentRef Row id or row index
* @param {string|number} rowRef Row id, row index
* @param {string=} dataId Row id for retrieving data
* @return {boolean} Return true if there is any change
*/
DataView.prototype.addSegmentChild = function(segmentRef, rowRef, dataId) {
	if(this._dt._getSegmentSeparators()) {
		let segmentId = this._toRowId(segmentRef);
		let rowId = this._toRowId(rowRef);
		return this._dt.addSegmentChild(segmentId, rowId, dataId);
	}
	return false;
};
/** @public
* @param {string|number} segmentRef Row id or row index
* @param {Array.<string|number>} rowRefs Array of row ids or row indices
* @param {Array.<string>=} dataIds Row ids for retrieving data
* @return {boolean} Return true if there is any change
*/
DataView.prototype.addSegmentChildren = function(segmentRef, rowRefs, dataIds) {
	if(this._dt._getSegmentSeparators()) {
		let segmentId = this._toRowId(segmentRef);
		let rowIds = this._toRowIds(rowRefs);
		return this._dt.addSegmentChildren(segmentId, rowIds, dataIds);
	}
	return false;
};
/** @public
* @param {Array<Object>} segmentArr Segment array that contain "segmentId", "rowIds" to set segment children
* @return {boolean} Return true if there is any change
*/
DataView.prototype.setSegmentChildren = function(segmentArr) {
	if(this._dt._getSegmentSeparators()) {
		return this._dt.setSegmentChildren(segmentArr);
	}
	return false;
};
/** @public
* @param {string|number} segmentRef Row id or row index
* @param {string|number} rowRef Row id, row index
* @return {boolean} Return true if there is any change
*/
DataView.prototype.removeSegmentChild = function(segmentRef, rowRef) {
	if(this._dt._getSegmentSeparators()) {
		let segmentId = this._toRowId(segmentRef);
		let rowId = this._toRowId(rowRef);
		return this._dt.removeSegmentChild(segmentId, rowId); // TODO: Handle the case where a child is remove from its collapsed parent. Child should be made visible
	}
	return false;
};
/** @public
* @param {string|number} segmentRef Row id or row index
* @param {Array.<string|number>} rowRefs Array of row ids or row indices. If null is given, no child will be removed.
* @return {boolean} Return true if there is any change
*/
DataView.prototype.removeSegmentChildren = function(segmentRef, rowRefs) {
	if(this._dt._getSegmentSeparators()) {
		let segmentId = this._toRowId(segmentRef);
		let rowIds = this._toRowIds(rowRefs);
		return this._dt.removeSegmentChildren(segmentId, rowIds); // TODO: Handle the case where a child is remove from its collapsed parent. Child should be made visible
	}
	return false;
};
/** @public
* @return {boolean} Return true if there is any change
*/
DataView.prototype.removeAllSegmentChildren = function() {
	return this._dt.removeAllSegmentChildren();
};
/** Get row ids of all segment separator
* @public
* @return {Array.<string>} Return null if there is no segment separator
*/
DataView.prototype.getSegmentIds = function() {
	return this._dt.getSegmentIds();
};
/** Get row ids of all children from the specified segment
* @public
* @param {string|number} segmentRef Row id or row index
* @return {Array.<string>} Returns array of row ids. Returns null if there is no child or no segment separator found
*/
DataView.prototype.getSegmentChildIds = function(segmentRef) {
	return this._dt.getSegmentChildIds(this._toRowId(segmentRef));
};
/** Sort all of existing segments by multiple sort logics
* @public
* @param {Function|Array.<Function>|Object} sortLogics
* @param {(number|Array.<number>)=} sortOrders
* @param {(string|Array.<string>)=} cids
*/
DataView.prototype.sortSeparators = function (sortLogics, sortOrders, cids) {
	let sortingDefs = this._sortingDefs;
	if(sortingDefs.length) {
		if(!cids) {
			cids = sortingDefs[0][0]; // field
		}
		if(!sortOrders) {
			sortOrders = sortingDefs[0][3];
		}
	} else if(!sortLogics) {
		return;
	}

	this._dt.sortSeparators(sortLogics, sortOrders, cids);
};
/** Sort all of existing segments by given compare function
* @public
* @param {Function} compare
*/
DataView.prototype.sortSegments = function (compare) {
	this._dt.sortSegments(compare);
};
/** Automatically hide empty segment when all of its member are filtered out. An empty segment will NOT be hidden, if there is no active filter. Collapsed segment does not count as filtering. A segment with no child is treated the same way as an empty segment.
* @public
* @param {boolean=} enabled
*/
DataView.prototype.enableEmptySegmentFiltering = function (enabled) {
	enabled = enabled !== false;
	if(this._emptySegmentFiltering !== enabled) {
		this._emptySegmentFiltering = enabled;
		if(this._userFilter) {
			this._refreshAndNotify();
		}
	}
};
/** Allow filtering of segment separators as if they were normal rows. Note that even if a separator row is filtered out, its child row may remain in the view and not be filtered out.
* @public
* @param {boolean=} enabled
*/
DataView.prototype.enableSeparatorFiltering = function (enabled) {
	enabled = enabled !== false;
	if(this._separatorFiltering !== enabled) {
		this._separatorFiltering = enabled;
		if(this._userFilter) {
			this._refreshAndNotify();
		}
	}
};
/**
* @public
* @param {string|number} segmentRef Row id or row index
* @param {string|Array.<string>} fields
* @return {boolean}
*/
DataView.prototype.setSegmentClassification = function(segmentRef, fields) {
	return this._dt.setSegmentClassification(this._toRowId(segmentRef), fields);
};
/**
* @public
* @param {boolean} bool
* @return {boolean}
*/
DataView.prototype.setSegmentDefaultCollapsing = function(bool) {
	return this._dt.setSegmentDefaultCollapsing(bool);
};

/** @public
* @ignore
* @return {Object}
*/
DataView.prototype._getSharedObject = function () {
	return this._shared;
};
/** Return 0, if there is no wrapping
* @public
* @return {number}
*/
DataView.prototype.getWrapSize = function () {
	return this._wrapSize;
};
/** Return null, if there is no wrapping
* @public
* @return {Array.<WrappedView>}
*/
DataView.prototype.getWrappedViews = function () {
	return this._wrappedViews;
};
/**
* @public
* @param {number=} wrapSize
* @fires DataView#wrapCountChanged
*/
DataView.prototype.wrapView = function (wrapSize) {
	if(wrapSize != null) {
		if(!wrapSize || wrapSize < 0) {
			wrapSize = 0;
		}
		if(this._wrapSize !== wrapSize) {
			this._wrapSize = wrapSize;
			this._dispatch("wrapSizeChanged", {
				"wrapSize": wrapSize,
				"rids": this.getVisibleRowIds(true)
			});
		}
	}
	this._updateWrapCount();
};

/** @private
* @fires DataView#wrapCountChanged
*/
DataView.prototype._updateWrapCount = function () {
	this._wrapTimerId = 0;
	let wrapSize = this._wrapSize;

	let i;
	let views = this._wrappedViews;
	let viewCount = views ? views.length : 0;
	let targetCount = 0;
	if(wrapSize) {
		let rids = this.getVisibleRowIds(true);
		let rowCount = rids.length;
		targetCount = Math.ceil(rowCount / wrapSize);
		if(viewCount < targetCount) {
			if(!views) {
				views = this._wrappedViews = [];
			}
			for(i = viewCount; i < targetCount; ++i) {
				this._wrappedViews[i] = new data_WrappedView({
					"dataView": this,
					"wrapIndex": i,
					"rids": rids
				});
			}
		}
	}

	if(targetCount < viewCount) {
		for(i = viewCount; --i >= targetCount;) {
			this._wrappedViews[i].dispose();
		}
		this._wrappedViews.length = targetCount;
	}
	if(!wrapSize) {
		this._wrappedViews = null;
	}

	if(viewCount !== targetCount) {
		this._dispatch("wrapCountChanged", {
			"prevCount": viewCount,
			"newCount": targetCount
		});
	}
};

/** Construct a JSON object that is compatible with console.table()
* @public
* @param {Object=} opt_options {@link DataCache#dump}
* @return {!Array.<Object>} Return a JSON object that is compatible with console.table()
* @see {@link DataCache#dump}
*/
DataView.prototype.dump = function(opt_options) {
	this._rebuildRowIdsFromChildren();
	let rids = this._getRids();

	return data_DataCache.constructTable(this.getMultipleRowData(rids), opt_options, rids);
};
/** Print current data and structure to console panel
* @public
* @param {Object=} opt_options
* @example
// Options have the following format:
// {
//  "colNames": Array.<string>,
//  "rowCount": number,
//  "startIndex": number,
//  "getter": function(Object, string):*,
//  "withRowIds": boolean
//}
dv.log({"rowCount": 10}); // Showing 10 records
dv.log({"getter": function(rowData, colName){
	let complexData = rowData[colName];
	return complextData.someProperty;
}});
*/
DataView.prototype.log = function(opt_options) {
	console.table(this.dump(opt_options));
};


/** @private
* @return {!Array.<string>}
*/
DataView.prototype._getRids = function() {
	return this._expansionView || this._pageView || this._groupView || this._rids;
};

/** @private
* @param {Array.<string>=} opt_rowIds Provide this parameter to avoid unnecessary group filtering
* @fires DataView#pageCountChanged
* @fires DataView#pageIndexChanged
*/
DataView.prototype._updateRowIds = function(opt_rowIds) {
	this._groupViewDirty = false;
	if(!this._parent || this._hidden) {
		this._resetAllViews();
		return;
	}
	if(this._shared.stalledRids) { // The stalled view cannot be changed
		return;
	}

	// Perform the following sequences: parent view cloning >> row hiding >> row filtering >> row grouping >> sorting >> paging
	this._rids = opt_rowIds || this._parent.getAllRowIds(); // Get all data ids

	this._dispatch("beforeFiltering", {});

	this._excludedRids = {};
	let exclusionCount = 0;
	exclusionCount += DataView._copyValidObjectKeys(this._excludedRids, this._hiddenRids);

	// Segment separators should not be filtered out (hidden)
	let segments = this._dt._getSegmentSeparators();
	let segmentIds = segments ? segments.getSegments() : null;
	let userRemoval = this._getRemovalMap(
		this._excludedRids,
		this._userFilter,
		this._filteringOut,
		this._separatorFiltering ? null : segmentIds
	);
	exclusionCount += userRemoval;

	this._collapsedRids = null;
	if(segments) {
		if(userRemoval && this._emptySegmentFiltering) {
			exclusionCount += this._getEmptySegments(this._excludedRids, segmentIds);
		}
		this._collapsedRids = segments.getCollapsedRows();
		 // Children of collapsed segments must be filtered out (hidden)
		exclusionCount += DataView._copyValidObjectKeys(this._excludedRids, this._collapsedRids);
	}

	if(this._groupLevel > 0 && !opt_rowIds) { // WARNING: The line below is quite slow
		exclusionCount += this._getRemovalMap(this._excludedRids, this._groupFilterLogic, false); // Filter In
	}
	if(exclusionCount) {
		this._rids = this._rids.filter(this._byRemovalMap);
	}
	this._excludedRids = null;

	if(this._groupMembers) { // Has grouping
		this._populateGroups(); // View will be properly re-populate inside _populateGroups()
	} else { // No grouping
		let pageIndexChanged = this._updatePageCount();
		let sorted = this._sort();
		if(!sorted && !pageIndexChanged) {
			this._transformView();
		}
	}
};
/** @private
* @param {Object} e
*/
DataView.prototype._onDataChanged = function (e) {
	if (this._conflator.conflate(e)) return;

	let events = this._conflator.popAllData() || [e];

	// process all events
	let eventsCount = events.length;
	if (eventsCount === 1) {
		let event = events[0];
		if (event["globalChange"]) {
			this._updateRowIds(); // Reset all row ids and perform sorting
			this._dispatchDataChange(event);
			return;
		}

		let type = event["type"];
		if (type === "inserted") {
			this._onRowInserted(event);
		} else if (type === "removed") {
			this._onRowRemoved(event);
		} else {
			this._onRowUpdated(event);
		}
	} else {
		// multi event should occur globalchange
		this._updateRowIds();  // Reset all row ids and perform sorting
		let positionChangeOnly = true;
		for (let i = 0; i < eventsCount; i++) {
			if (!events[i].positionChangeOnly) {
				positionChangeOnly = false;
				break;
			}
		}
		this._dispatchDataChange({
			"globalChange": true,
			"positionChangeOnly": positionChangeOnly,
			"events": events
		});
	}
};
/** @private
* @param {!Object} e
* @return {number} 1 if there is any change, zero otherwise.
*/
DataView.prototype._onRowInserted = function(e) { // onInsert
	if(this._shared.stalledRids) {
		return 0;
	}

	if(this.isRowFiltered(e["rid"], e["rowData"])) {
		return 0;
	}

	let ret = 0;
	if(this._groupMembers) {
		let gids = this._resolveGroupId(e["rowData"]);
		let groupLen = gids.length;
		if(groupLen > 1) {
			// activate multi-group row mode
			this._shared.multiGroupRow = true;
		}
		let gid;
		for(let i = 0; i < groupLen; i++) {
			gid = gids[i];
			if(this._groupMembers[gid]) { // Insert to existing member
				ret = this._groupMembers[gid]._onRowInserted(e);
			} else {
				let member = this._addGroup(gid);
				if(member) {
					e["globalChange"] = true;
					member._onRowInserted(e);
					ret = 1;
				}
			}
		}
		if(e["globalChange"]) {
			this._groupViewDirty = true;
		}
	}

	if(this._hidden) {
		return ret; // the view is hidden and there is no need to be updated
	}

	let rid = e["rid"];
	let rowData = e["rowData"];

	if(!this._isAllowedInGroup(rid, rowData)) {
		return ret;
	}

	let nextRid = /** @type{string} */(e["nextRid"]);
	let rowIndex;
	if(this.isSorting() && !e["segmentChanged"]) {
		rowIndex = this._addDataRow(rid);
	} else {
		rowIndex = this._addDataRow(rid, nextRid, e["fallback"]);
	}

	let pageIndexChanged = this._updatePageCount();
	let sorted = this._sort();
	if(sorted) {
		e["globalChange"] = true; // Notify parent
	}
	if(e["globalChange"]) {
		if(!sorted && !pageIndexChanged) {
			this._transformView();
		}
	} else {
		e = { // Slow
			"rid": rid,
			"rowData": rowData,
			"type": "inserted"
		};
		if(nextRid != null) {
			e["nextRid"] = nextRid;
			e["fallback"] = rowIndex;
		}
		if(this._transformView(rowIndex)) {
			e["globalChange"] = true;
		}
	}
	this._dispatchDataChange(e);

	return 1;
};
/** @private
* @param {!Object} e
* @return {number}
*/
DataView.prototype._onRowRemoved = function(e) { // onRemove
	if(this._shared.stalledRids) {
		return 0;
	}

	if(e["segmentChanged"] && this.isSorting()) {
		this._refreshAndNotify();
		return 1; // may not be a correct number
	}

	let removeCount = 0;
	let rids = e["rids"];
	let i;
	if(this._groupMembers) {
		let rowData = e["rowData"];
		let groupIds, groupId;
		if(!rids && rowData) {
			groupIds = this._resolveGroupId(rowData);
			for(i = 0; i < groupIds.length; i++) {
				groupId = groupIds[i];
				if(this._groupMembers[groupId]) { // remove existing member
					removeCount += this._groupMembers[groupId]._onRowRemoved(e);
				}
			}

			// should allow rebuilding because rows which the same rid were removed from many groups
			// sectionSetting will not know the proper changed row index
			if(removeCount > 1) {
				this._groupViewDirty = true;
			}
		} else {
			let len = this._groupList.length;
			for(i = 0; i < len; ++i) {
				groupId = this._groupList[i];
				removeCount += this._groupMembers[groupId]._onRowRemoved(e);
			}
		}
	}

	let firstModifiedIndex = NaN;
	let removedRids = null;
	let rid, at;

	// TODO: Change removal mechanism to object map based logic to speed up the process
	removeCount = 0;
	if(rids) {
		removedRids = [];
		for(i = rids.length; --i >= 0;) {
			rid = rids[i];
			at = this._removeDataRow(rid);
			if(at >= 0) {
				removedRids.push(rid);
				if(!(at >= firstModifiedIndex)) {
					firstModifiedIndex = at;
				}
			}
		}
		removeCount = removedRids.length;
	} else {
		rid = e["rid"];
		at = this._removeDataRow(rid);
		if(at >= 0) {
			removedRids = [rid];
			firstModifiedIndex = at;
			removeCount = 1;
		}
	}
	if(removeCount > 0) {
		// No sorting is required, since no order should be changed during the removal
		let pageIndexChanged = this._updatePageCount();
		if(pageIndexChanged || this._transformView(firstModifiedIndex)) {
			this._dispatchGlobalChange();
		} else {
			this._dispatchMultiRowRemoved(removedRids);
		}
		if(this._shared.autoGroupRemoval &&
			this._groupRows &&
			this._rids.length <= 0) {
			this._triggerGroupRefreshing(); // Set a timer to refresh group view
		}
		if (this._hiddenHeaderRow) { // If the first row is removed while th group is collapsing, the next row should take place.
			this._requestRebuildingRowIds(true); // TODO: Optimize
		}
	}
	return removeCount;
};
/** @private
* @param {!Object} e
* @return {number}
*/
DataView.prototype._onRowUpdated = function(e) { // onUpdate
	let rid = e["rid"];
	let rowData = e["rowData"];
	let i, gid;
	if(this.isRowFiltered(rid, rowData)) { // The row is qualified but it is filtered out
		if(this._onRowRemoved(e)) {
			return 1;
		}
		return 0; // The row does not belong to this view
	}

	if(this._groupMembers) {
		let flags = [];
		let len = this._groupList.length;
		for(i = 0; i < len; ++i) {
			gid = this._groupList[i];
			flags[this._groupMembers[gid]._onRowUpdated(e)] = true;
		}

		if(e["globalChange"]) { // If current dataView is sorting, globalChange will always be true
			this._groupViewDirty = true;
		}

		// conclude flag
		let processingFlag = 0;
		if(flags[2]) {
			processingFlag = 2;
		} else if(flags[1]) {
			if(!this._isAllowedInGroup(rid, rowData)) {
				processingFlag = 1;
			}
		} else if(flags[4]) {
			processingFlag = 4;
		}

		if(this._groupLevel > 0) {
			if(processingFlag === 1) { // The row is moved to the other group
				if(DataView._removeArrayItem(this._rids, rid) >= 0) {
					if(this._groupView) {
						DataView._removeArrayItem(this._groupView, rid);
					}
				}
				if(this._shared.multiGroupRow) {
					this._groupViewDirty = true;
				}
			} else if(processingFlag === 2) { // The row is added to its sub-group
				if(!this._hidden) {
					if(this._rids.indexOf(rid) < 0) {
						this._rids.push(rid);
					}

					this._groupViewDirty = true; // Adding row to sub-group both single and multiple-group require parent group to rebuild rows

					let groupLen = this._resolveGroupId(rowData).length;
					if(groupLen > 1) {
						// activate multi-group row mode
						this._shared.multiGroupRow = true;
					}
				}
			}
		} else { // This is the root node
			let noNewGroup = (flags[1] && flags[2]) || flags[4];
			if(this._shared.multiGroupRow || !noNewGroup) {
				// 0 - updated data from non-existing group to new group
				// 1 - updated data from existing group to new group
				// 2 - updated data from a non-existing group to existing group
				// 3 - updated data from a existing group to another existing group
				// 4 - updated data with in a single group and no group member changed
				let gids = this._resolveGroupId(rowData);
				for(i = 0; i < gids.length; i++) {
					gid = gids[i];
					let member = this._addGroup(gid); // Add header row
					if(member) {
						member._onRowInserted(e);
						processingFlag = 1;
					}
				}
			}

			if(!noNewGroup && !this._hidden && this._rids.indexOf(rid) < 0) {
				this._addDataRow(rid);
			}

			if(processingFlag === 4) {
				if(this._shared.multiGroupRow) {
					this._dispatchGlobalChange();
				} else {
					this._dispatchDataChange(e);
				}
			} else {
				this._groupViewDirty = true;
				this._dispatchGlobalChange();
			}
		}
		return processingFlag;
	}

	// recursive task for leaf group
	if(this._isAllowedInGroup(rid, rowData)) {
		let newRowIndex = this._rids.indexOf(rid);
		if(newRowIndex < 0) { // The row is qualified to be in the group, but not in the current view, so insert the row
			this._onRowInserted(e);
			return 2;
		}

		// Perform normal data updating
		if (!this._shared.stalledRids) { // Stalled view cannot be sorted
			if(this._sort()) {
				e["globalChange"] = true;
			}
		}

		this._dispatchDataChange(e);
		return 4; // Update is within the group
	} else {
		if(this._onRowRemoved(e)) {
			return 1;
		}
		return 0; // The row does not belong to this view
	}
};
/** @private
* @param {!Object} e
*/
DataView.prototype._onGroupAdded = function(e) {
	if(!this._hidden) {
		let newGroup = /** @type{DataView} */(e["newGroup"]);
		let rid = newGroup.getGroupRowId();
		this._groupView.push(rid); // Add header row
		this._groupViewDirty = true;
		this._dispatch("groupAdded", e);

		util.pushArrayItems(this._groupView, newGroup.getFooterRowIds());
	}
};
/** @private
* @param {!Object} e
*/
DataView.prototype._onBeforeGroupAdded = function(e) {
	this._dispatch("beforeGroupAdded", e);
};
/** NOTE: This is not handle well in case of calling this function infinitely
 * @private
* @param {boolean} bottomUp
*/
DataView.prototype._requestRebuildingRowIds = function(bottomUp) {
	if(this._groupMembers) {
		this._groupViewDirty = true;
	}
	if(this._groupLevel > 0) {
		this._parent._requestRebuildingRowIds(bottomUp);
	} else {
		if(bottomUp) { // Regenerate row Ids from children. Use when something is changed and can be resolved from the requester.
			this._rebuildRowIdsFromChildren();
		} else { // Use when something is changed in the hierarchy tree but cannot be resolved by any of the children. Root view then repopulates the entire tree (slow).
			this._updateRowIds();
		}
		this._dispatchGlobalChange();
	}
};
/** There should be no other operation after calling this method
* @private
*/
DataView.prototype._refreshAndNotify = function() {
	this._updateRowIds();
	this._dispatchGlobalChange();
};
/** @private
*/
DataView.prototype._onRefreshTimeout = function() {
	this._refreshTimerId = 0;
	this._refreshAndNotify();
};

/** @private
* @param {string} rid
* @param {string=} opt_nextRid
* @param {number=} opt_fallback
* @return {number}
*/
DataView.prototype._addDataRow = function(rid, opt_nextRid, opt_fallback) {
	let rowIndex = this._getRowIndex(this._rids, opt_nextRid, opt_fallback);
	this._insertItems(this._rids, rowIndex, rid);
	if(this._groupView) {
		if(this._groupMembers) {
			this._groupViewDirty = true;
			return this._insertItems(this._groupView, -1, rid);
		} else if(rowIndex < 0) { // No nextRid, No fallback
			return this._insertItems(this._groupView, -1, rid);
		}

		rowIndex = this._getRowIndex(this._groupView, opt_nextRid, opt_fallback);
		return this._insertItems(this._groupView, rowIndex, rid);
	}

	return (rowIndex >= 0) ? rowIndex : this._rids.length - 1;
};
/** @private
* @param {Array.<string>} ary
* @param {number} idx
* @param {string} item1
* @return {number} Return the first position of the inserted item
*/
DataView.prototype._insertItems = function(ary, idx, item1) {
	if(idx < 0){
		idx = ary.length;
		ary.push(item1);
	} else {
		ary.splice(idx, 0, item1);
	}
	return idx;
};
/** @private
* @param {Array.<string>} rids
* @param {string=} opt_nextRid
* @param {number=} opt_fallback Fallback index in case of no nextRid found
* @return {number} Return negative number if not found
*/
DataView.prototype._getRowIndex = function(rids, opt_nextRid, opt_fallback) {
	if(opt_nextRid == null) {
		return -1;
	}

	let rowIndex = this._rids.indexOf(opt_nextRid);
	if(rowIndex >= 0) {
		return rowIndex;
	}

	if(opt_fallback == null || opt_fallback >= rids.length) {
		return -1;
	}
	return /** @type{number} */(opt_fallback);
};
/**
* @private
* @param {string} rid
* @returns {number}
*/
DataView.prototype._removeDataRow = function(rid) {
	let at = DataView._removeArrayItem(this._rids, rid);
	if(this._groupView && at >= 0) {
		at = DataView._removeArrayItem(this._groupView, rid);
	}
	return at;
};
/** @private
* @param {!Array} ary
* @param {*} item
* @return {number} Index of the removed item
*/
DataView._removeArrayItem = function(ary, item) {
	let at = ary.indexOf(item);
	if(at >= 0) {
		ary.splice(at, 1);
	}
	return at;
};
/** Remove multiple array items
* @private
* @param {Array.<string>} ary
* @param {Object} items
* @return {number} First item index that is being removed. NaN if no item is removed
*/
DataView._removeArrayItems = function(ary, items) {
	let f = NaN;
	let c = 0;
	for(let i = ary.length; --i >= 0;) {
		if (items[ary[i]]) {
			++c;
		} else if(c) {
			f = i + 1;
			ary.splice(f, c);
			c = 0;
		}
	}
	if(c) {
		ary.splice(0, c);
		f = 0;
	}
	return f;
};
/** @private
* @param {Object} baseObj
* @param {Object} masterObj
* @returns {number}
*/
DataView._copyValidObjectKeys = function(baseObj, masterObj) {
	if(masterObj) {
		let count = 0;

		for(let key in masterObj) {
			if(masterObj[key]) {
				baseObj[key] = 1;
				++count; // WARNING: duplicated key can be counted more than once
			}
		}
		return count;
	}
	return 0;
};

/** @private
* @param {string|null} rid
* @param {Object} rowData
* @return {boolean}
*/
DataView.prototype._isAllowedInGroup = function(rid, rowData) {
	// Check if it the row is belong to this group branch
	if(this._groupLevel > 0) {
		let dv = this;
		for(let i = this._groupLevel; --i >= 0;) {
			if(!dv._groupFilterLogic(rid, rowData)) {
				return false;
			}
			dv = dv.getDataSource();
		}
	}

	return true; // The row is in this group
};
/** Check if the row should be filtered out
* @public
* @param {string} rid
* @param {Object} rowData
* @return {boolean}
*/
DataView.prototype.isRowFiltered = function(rid, rowData) {
	if(this._shared.stalledRids) {
		return !this._shared.stalledRids[rid]; // Snapshot override everything else
	}
	if(this._hiddenRids) {
		if(this._hiddenRids[rid]) {
			return true;
		}
	}
	if(this._collapsedRids) {
		if(this._collapsedRids[rid]) {
			return true;
		}
	}

	if(this._userFilter) {
		if(this._userFilter(rid, rowData) === this._filteringOut) {
			return true;
		}
	}
	return false; // The row is not filtered out
};

/** Performance is extremely vital in this method.<br>
* _sort() will be executed by: _onRowUpdated(), _onRowInserted(), _updateRowIds(), and sort()
* @private
* @return {boolean} True if there is any change, otherwise false
*/
DataView.prototype._sort = function() {
	if(this._groupMembers) {
		return false; // DataView with children should not be sorted as sort operation will be done on each child
	}
	if(this._stalledSorting) {
		return false;
	}

	if(this._dt._getSegmentSeparators() || this.isSorting()) {
		let rids = this._rids;
		let dataRows = this._dt.getMultipleRowData(rids);
		let segmentValues = this._dt.getSegmentValues(rids, true); // WARNING: This does not work well for hidden/filtered rows
		let change = data_DataTable._performMultiLevelSorting(dataRows, this._sortingDefs, segmentValues);

		if(change) {
			this._rids = data_DataTable._reorderItems(rids, change); // Group view should be changed after this
			this._transformView();
			return true;
		}
	}

	return false;
};

/** @private
* @param {string} rid
* @returns {boolean}
*/
DataView.prototype._byRemovalMap = function(rid) {
	return !this._excludedRids[rid];
};
/** @private
* @param {Object} removalMap
* @param {Function} checker
* @param {boolean} filteringOut
* @param {Object=} exceptions
* @returns {number} Number of rids that would be filtered out
*/
DataView.prototype._getRemovalMap = function(removalMap, checker, filteringOut, exceptions) {
	if(!checker) {
		return 0;
	}

	let rids = this._rids; // Make local variable to speed up the process
	let dt = this._dt;
	let count = 0;
	let rid, i, values;
	let len = rids.length;

	if(exceptions) {
		for(i = len; --i >= 0;) {
			rid = rids[i];
			if(!exceptions[rid]) {
				values = dt.getRowData(rid);
				if (values) {
					if(checker(rid, values) === filteringOut) {
						removalMap[rid] = 1;
						++count;
					}
				} else {
					removalMap[rid] = 1;
					++count;
				}
			}
		}
	} else {
		for(i = len; --i >= 0;) {
			rid = rids[i];
			values = dt.getRowData(rid);
			if (values) {
				if(checker(rid, values) === filteringOut) {
					removalMap[rid] = 1;
					++count;
				}
			} else {
				removalMap[rid] = 1;
				++count;
			}
		}
	}
	return count;
};
/** @private
* @param {Object} removalMap
* @param {Object} segmentRids
* @returns {number} Number of rids that would be filtered out
*/
DataView.prototype._getEmptySegments = function(removalMap, segmentRids) {
	let segments = this._dt._getSegmentSeparators();
	let count = 0;
	for(let segmentId in segmentRids) {
		let segment = segments.getSegment(segmentId);
		if(segment) {
			let chdr = segment.getChildren();
			let emptySegment = true;
			for(let childId in chdr) {
				if(!removalMap[childId]) {
					emptySegment = false;
					break;
				}
			}
			if(emptySegment) {
				removalMap[segmentId] = 1;
				++count;
			}
		}
	}

	return count;
};

/** @private
* @param {string|Function|undefined} cid
* @param {*} value
* @return {Function} A new function object is created, if cid is a string
*/
DataView.prototype._getFilterLogic = function(cid, value) {
	if(cid != null) {
		if(typeof cid == "function") {
			return cid;
		} else if(typeof cid == "string") {
			return DataView._defaultFilterLogic.bind(null, cid, value);
		} else {
			return DataView._defaultFilterLogic.bind(null, cid + "", value);
		}
	}
	return null;
};

/**
* @private
* @param {number} pageIndex
* @return {boolean} True if there is any change, otherwise false
* @fires DataView#pageIndexChanged
*/
DataView.prototype._setPage = function (pageIndex) {
	if (pageIndex > this._lastPageIndex && this._pageCorrection) {
		pageIndex = this._lastPageIndex;
	}
	if (pageIndex < 1) {
		pageIndex = 1;
	}

	if (pageIndex == this._pageIndex) {
		return false;
	}

	this._pageIndex = pageIndex;
	this._transformView();
	this._dispatch("pageIndexChanged", {"pageIndex": this._pageIndex});
	return true;
};
/** @private
* @param {number=} firstChange Negative value is equivalent to 0
* @return {boolean} Return true if there is any change, otherwise false
*/
DataView.prototype._updatePaginationView = function(firstChange) {
	let dirty = false;
	if(this._pageView) {
		if(!firstChange) { firstChange = 0; }

		let pageStart = this._pageSize * (this._pageIndex - 1);

		let pageEnd = this._pageSize + pageStart;
		if(firstChange < pageEnd) { // The change happens before current page
			let rids = this._groupView || this._rids;
			if (pageEnd > rids.length) { // Shrink view if the last page is not a full page
				pageEnd = rids.length;
			}

			// Populate the page
			let ridsLen = pageEnd - pageStart;
			this._pageView.length = (ridsLen >= 0) ? ridsLen : 0;
			for(let i = pageStart; i < pageEnd; ++i) {
				this._pageView[i - pageStart] = rids[i];
			}
			dirty = true;
		}
	}
	return dirty;
};

/** WARNING: _transformView() may be called due to page index changed
* @private
* @return {boolean} Return true if page index is changed
* @fires DataView#pageCountChanged
* @fires DataView#pageIndexChanged
*/
DataView.prototype._updatePageCount = function () {
	let lastPageIndex = 1;
	if (this._pageSize > 0) {
		let rows = this._groupView || this._rids;
		let rowCount = (rows.length > 0) ? rows.length : 0;
		lastPageIndex = Math.ceil(rowCount / this._pageSize);
	}
	if(this._lastPageIndex !== lastPageIndex) {
		this._lastPageIndex = lastPageIndex;
		this._dispatch("pageCountChanged", {"pageCount": this._lastPageIndex});

		if(this._lastPageIndex < this._pageIndex && this._pageCorrection) {
			return this._setPage(this._lastPageIndex);
		}
	}
	return false;
};

/** @private
* @param {string} rid
*/
DataView.prototype._dispatchRowRemoved = function(rid) {
	if(rid) {
		this._dispatchDataChange({"type": "removed", "rid": rid});
	}
};
/** @private
* @param {Array.<string>} rids
*/
DataView.prototype._dispatchMultiRowRemoved = function(rids) {
	if(rids) {
		let count = rids.length;
		if(count === 1) {
			this._dispatchRowRemoved(rids[0]);
		} else if(count > 1) {
			this._dispatchDataChange({"type": "removed", "rids": rids});
		}
	}
};
/** @private
*/
DataView.prototype._dispatchGlobalChange = function() {
	this._dispatchDataChange(data_DataTable._globalChangeArg);
};
/** @private
* @param {!Object} evtArgument
*/
DataView.prototype._dispatchDataChange = function(evtArgument) {
	this._dispatch("dataChanged", evtArgument);
	if(this._wrapSize && !this._wrapTimerId) {
		this._wrapTimerId = setTimeout(this._updateWrapCount);
	}
};

/** @private
* @param {DataTable|DataView} source
*/
DataView.prototype._setParent = function (source) {
	this._parent = source;
	this._dt = /** @type{DataTable} */(source);
	do {
		source = source.getDataSource();
		if(source) {
			this._dt = /** @type{DataTable} */(source);
		}
	} while(source);

	if(this._parent && this._dt !== this._parent) { // Copy shared member from parent data view
		this._shared = this._parent._shared;

		let cids = this._parent._columnStats;
		if(cids) {
			for(let cid in cids) {
				let cs = this.enableColumnStats(cid);
				let pcs = this._parent.getColumnStats(cid);
				if(cs && pcs) {
					cs.setColumnDataGetter(pcs.getColumnDataGetter());
				}
			}
		}
	}
};

/** @private
*/
DataView.prototype._populateGroups = function () {
	let initialRids = this._rids;
	let i, len, member;
	this._rids = [];

	// Collect members of each group from the available data
	let groupId, groupIds, groupLen;
	let ridMap = {};
	let groupList = [];
	len = initialRids.length;
	for(i = 0; i < len; ++i) {
		let rid = initialRids[i];
		let rowData = this._dt.getRowData(rid);
		if(rowData) {
			groupIds = this._resolveGroupId(rowData);
			groupLen = groupIds.length;
			if(groupLen > 1) {
				// activate multi-group row mode
				this._shared.multiGroupRow = true;
			}
			for(let j = 0; j < groupLen; j++) {
				groupId = groupIds[j];
				let groupedRids = ridMap[groupId];
				if(groupedRids) {
					groupedRids.push(rid);
				} else {
					groupedRids = [rid];
					ridMap[groupId] = groupedRids;
					groupList.push(groupId);
				}
			}
		}
	}

	// Remove groups that have no member
	if(this._shared.autoGroupRemoval) {
		len = this._groupList.length;
		for(i = len; --i >= 0;) {
			groupId = this._groupList[i];
			if(!ridMap[groupId]) { // No member exists
				member = this._groupMembers[groupId];
				if(member && !member._predefinedGroup) { // Predefined group should not be removed automatically
					this._removeGroupMember(i, groupId);
				}
			}
		}
	}
	// Remove any non existing predefined groups
	let preDefGroups = null;
	if(this._preDefGroups && this._groupLevel < this._preDefGroups.length) {
		preDefGroups = this._preDefGroups[this._groupLevel];
	}

	len = this._groupList.length;
	if(this._preDefGroupsDirty) {
		this._preDefGroupsDirty = false;
		for(i = len; --i >= 0;) {
			groupId = this._groupList[i];
			member = this._groupMembers[groupId];
			if(member && member._predefinedGroup) {
				// TODO: Handle the case where predefined groups exist in normal groupList
				if(preDefGroups) {
					if(preDefGroups.indexOf(member._groupId) < 0) {
						this._removeGroupMember(i, groupId);
					}
				} else {
					this._removeGroupMember(i, groupId);
				}
			}
		}
	}


	// Add predefined groups first, allowing user to initially order the groups
	if(preDefGroups) {
		len = preDefGroups.length;
		for(i = 0; i < len; ++i) {
			let pdg = this._addGroup(preDefGroups[i]);
			if(pdg) {
				pdg._predefinedGroup = true;
			}
		}
	}
	// Add normal group
	len = groupList.length;
	for(i = 0; i < len; ++i) {
		this._addGroup(groupList[i]);
	}

	// Add row Id
	len = this._groupList.length;
	for(i = 0; i < len; ++i) { // Re-distribute this object content to all of its group members
		groupId = this._groupList[i];
		member = this._groupMembers[groupId];
		member._updateRowIds(ridMap[groupId]); // member will be sorted individually
		member._markColumnStatsDirty(); // Since there is no event fired for child members
	}

	this._groupViewDirty = true;
	this._rebuildRowIdsFromChildren();
};
/** @private
* @param {number} groupIndex
* @param {string} groupId
*/
DataView.prototype._removeGroupMember = function (groupIndex, groupId) {
	let member = this._groupMembers[groupId];
	if(member) {
		this._groupList.splice(groupIndex, 1);
		delete this._groupMembers[groupId];

		member.dispose(); // Clear everything. Note that this will call groupBy(null);
	}
};
/** @private
* @param {string} cid
* @param {Object} values
* @return {Array.<string>}
*/
DataView.prototype._defaultGroupCriteria = function(cid, values) {
	let val = values[cid];
	if(Array.isArray(val)) {
		return val.map(function(data) { // TODO: this is very slow
			return data + "";
		});
	} else {
		return [val + ""];
	}
};
/** @private
* @param {string|null} rid
* @param {Object} values
* @return {boolean}
*/
DataView.prototype._groupFilterLogic = function(rid, values) {
	let gids = this._groupCriteria[this._groupLevel - 1](values); // TODO: this is very slow
	return gids.indexOf(this._groupId) >= 0;
};

/** @private
* @param {string} strA
* @param {string} strB
* @return {number}
*/
DataView.prototype._internalGroupSortingLogic = function(strA, strB) {
	let dvA = this._groupMembers[strA];
	let dvB = this._groupMembers[strB];

	return this._groupSortingLogic(strA, strB, dvA, dvB);
};

/** @private
* @param {*} value
* @return {DataView} Newly added group member. If it already exists, this method returns null.
*/
DataView.prototype._addGroup = function (value) {
	let groupId = value + "";
	if(!groupId) {
		// window.console.log("WARNING: Group Id cannot be an empty string");
		return null;
	}
	let member = this._groupMembers[groupId];
	if(member) {
		return null;
	}
	let parentGrpRowId = this._groupRowId || "_grp";
	let newGrpRowId = parentGrpRowId + "_" + groupId;
	let arg;
	if(this._hasListener("beforeGroupAdded")) {
		arg = {
			"dataView": this, // parent
			"newGroupId": groupId,
			"newGroupValue": value,
			"newGroupRowId": newGrpRowId,
			"newGroupLevel": this._groupLevel + 1
		};
		this._dispatch("beforeGroupAdded", arg);
		if(arg["cancel"]) {
			return null;
		}
	}
	member = new DataView();
	let collapsed = this._shared.defaultCollapse;
	if(collapsed != null) {
		member._collapsed = collapsed;
	}
	member._groupId = groupId;
	if(typeof value == "string") { // TODO: Raw value should not always be a string
		if(value == "null") {
			member._groupValue = null;
		} else if(value != "undefined") {
			member._groupValue = value;
		} // else { value is undefined string
	} else {
		member._groupValue = value;
	}

	if (arg && arg["hideHeaderRow"] === true) {
		member._hiddenHeaderRow = true;
	}

	member._groupRowId = newGrpRowId;
	member._groupLevel = this._groupLevel + 1;
	member._groupCriteria = this._groupCriteria; // Shared Member
	member._preDefGroups = this._preDefGroups;
	if(member._groupLevel < member._groupCriteria.length) {
		member._groupMembers = {};
		member._groupList = [];
		member._groupView = [];
	}
	member._groupRows = this._groupRows; // Shared Member
	member._setParent(this);
	member._sortingDefs = this._sortingDefs; // Shared Member

	this._groupMembers[groupId] = member;
	this._groupList.push(groupId);
	this._groupRows[member._groupRowId] = member;

	if(!this._hidden) {
		if (member._isHeaderRowShown()) {
			this._groupView.push(member._groupRowId); // Add header row
		}

		this._groupViewDirty = true;
		member._beingAdded = true;
		this._dispatch("groupAdded", {
			"dataView": this,
			"newGroup": member
		});
		member._beingAdded = false;
	}

	member.listen("groupAdded", this._onGroupAdded);
	if(this._hasListener("beforeGroupAdded")) {
		member.listen("beforeGroupAdded", this._onBeforeGroupAdded);
	}
	return member;
};
/** @private
*/
DataView.prototype._triggerGroupRefreshing = function () {
	if(this._groupLevel > 0) {
		if(this._parent) {
			this._parent._triggerGroupRefreshing();
		}
	} else if(this._groupRows && !this._refreshTimerId) {
		this._refreshTimerId = setTimeout(this._onRefreshTimeout, 100);
	}
};
/** @private
* @fires DataView#pageCountChanged
* @fires DataView#pageIndexChanged
*/
DataView.prototype._rebuildRowIdsFromChildren = function () {
	if(!this._groupViewDirty) {
		return;
	}

	this._rids.length = 0;
	if(this._groupView) { this._groupView.length = 0; }
	if(!this._hidden) {
		let grpLen = this._groupList.length;
		for(let i = 0; i < grpLen; ++i) { // Re-distribute this object content to all of its group members
			let gid = this._groupList[i];
			let member = this._groupMembers[gid];
			member._rebuildRowIdsFromChildren();

			util.pushArrayItems(this._rids, member._rids);

			if(!this._collapsed && this._groupView) {
				if (member._isHeaderRowShown()) {
					this._groupView.push(member._groupRowId); // Add header row
				}
				if(!member._collapsed) { // WARNING: This gives different value from isCollapsed() method
					let rids = member._getFirstHiddenRids() || member._groupView || member._rids;
					util.pushArrayItems(this._groupView, rids);
					util.pushArrayItems(this._groupView, member._footerRowIds);
				}
			}
		}
	}
	this._groupViewDirty = false;

	// View has been re-created. So we need to add row expansion
	if(!this._updatePageCount()) {
		this._transformView();
	}
};
/** @private
* @param {Object} rowData
* @return {Array.<string>}
*/
DataView.prototype._resolveGroupId = function (rowData) {
	return /** @type{Array.<string>} */(this._groupCriteria[this._groupLevel](rowData));
};

/** @private
*/
DataView.prototype._markColumnStatsDirty = function() {
	for(let cid in this._columnStats) {
		this._columnStats[cid].markDirty();
	}
};

/** @private
* @function
* @param {string} cid
* @param {*} value
* @param {string} rid
* @param {Object.<string, *>} values
* @return {boolean}
*/
DataView._defaultFilterLogic = function(cid, value, rid, values) {
	return (values[cid] === value);
};
/** @private
* @function
* @param {string} strA
* @param {string} strB
* @return {number}
*/
DataView._defaultGroupSortingLogic = function(strA, strB) {
	if(strA == strB) {
		return 0;
	}
	return (strA < strB) ? -1 : 1;
};


/* harmony default export */ const data_DataView = (DataView);


;// ./node_modules/@grid/core/es6/grid/util/Conflator.js

/** @description Conflator is used to stop multiple executions of a function in the specified period of time by merging multiple calls into one single call.
* Conflator guarantees that only one single call will be executed.
* @constructor
* @param {Function|number=} opt_func Bound function that will use this conflator
* @param {number|Function=} opt_ms The delay time in millisecond before executing the function
* @param {*=} opt_thisObj "this" object to be bound with the given function. If the function is already bound, there is no need to provide thisObj parameter
* @example
* let c = new Conflator(function() { console.log("Executed"); }, 1000);
* for(let i = 10; --i >= 0;) {
*   c.conflate(i); // Only one "Executed" text will be logged to console after one second
* }
*/
let Conflator_Conflator = function (opt_func, opt_ms, opt_thisObj) {
	this._onConflated = this._onConflated.bind(this);

	this._data = [];

	if(typeof opt_func == "function") {
		this.setFunction(opt_func, opt_thisObj);
	} else if (typeof opt_ms == "function") { // Parameter can be swapped places
		this.setFunction(opt_ms, opt_thisObj);
	}

	if(typeof opt_ms == "number") {
		this.setConflatingTime(opt_ms);
	} else if (typeof opt_func == "number") { // Parameter can be swapped places
		this.setConflatingTime(opt_func);
	}
};

/** @type {!Array}
* @private
*/
Conflator_Conflator.prototype._data; // Array to store all conflated data
/** @type {Function}
* @private
*/
Conflator_Conflator.prototype._func = null;
/** @type {number}
* @private
*/
Conflator_Conflator.prototype._delay = 0;
/** @type {number}
* @private
*/
Conflator_Conflator.prototype._timerId = 0;
/** @type {boolean}
* @private
*/
Conflator_Conflator.prototype._conflated = false;
/** @type {boolean}
* @private
*/
Conflator_Conflator.prototype._disabled = false;


/** @description Any existing timer and stored data will be cleared.
* @public
* @return {Array} All stored data. Return null if there is no data
*/
Conflator_Conflator.prototype.reset = function () {
	if(this._timerId) {
		clearTimeout(this._timerId);
		this._timerId = 0;
	}

	return this.popAllData();
};
/** All stored data will be cleared
* @public
* @return {Array} All stored data. Return null if there is no data
*/
Conflator_Conflator.prototype.popAllData = function() {
	if(this._data.length) {
		let data = this._data;
		this._data = [];
		return data;
	}
	return null;
};
/** @public
*/
Conflator_Conflator.prototype.dispose = function () {
	if(this._timerId) {
		clearTimeout(this._timerId);
		this._timerId = 0;
	}
	if(this._data.length) {
		this._data.length = 0;
	}
	this._func = null; // Release user bound function
};

/** @public
* @param {*=} opt_data
* @return {boolean}
*/
Conflator_Conflator.prototype.conflate = function (opt_data) {
	if(this._disabled) {
		return false;
	}
	if(opt_data != null) {
		this._data.push(opt_data);
	}
	if(this._conflated) {
		return false; // Conflating operation is already completed
	}
	if(!this._timerId) {
		this._timerId = setTimeout(this._onConflated, this._delay);
	}
	return true;
};
/** @public
* @return {boolean}
*/
Conflator_Conflator.prototype.isConflating = function () {
	return (this._timerId > 0 && !this._conflated && !this._disabled);
};

/** @private
*/
Conflator_Conflator.prototype._onConflated = function () {
	this._conflated = true;

	try {
		this._func();
	} catch (err) {
		console.error(err.message || err);
	}

	this._timerId = 0;
	this._data.length = 0;
	this._conflated = false;
};

/** @public
* @return {!Array}
*/
Conflator_Conflator.prototype.getData = function () {
	return this._data;
};
/** @public
* @param {Function} func Bound function that will use this conflator
* @param {*=} opt_thisObj "this" object to be bound with the given function. If the function is already bound, there is no need to provide thisObj parameter
*/
Conflator_Conflator.prototype.setFunction = function (func, opt_thisObj) {
	this._func = func;
	if(func) {
		if(opt_thisObj) {
			this._func = func.bind(opt_thisObj);
		} else {
			this._func = func;
		}
	} else {
		this._func = null;
	}
};
/** @public
* @param {number} ms The delay time in millisecond before executing the function
*/
Conflator_Conflator.prototype.setConflatingTime = function (ms) {
	this._delay = ms || 0;
	if(this._delay < 0) {
		this._delay = 0;
	}
};

/** @public
* @param {boolean=} opt_disabled
*/
Conflator_Conflator.prototype.disable = function (opt_disabled) {
	this._disabled = opt_disabled !== false;
};
/** @public
* @return {boolean}
*/
Conflator_Conflator.prototype.isEnabled = function () {
	return !this._disabled;
};
/** @public
* @param {boolean=} opt_enabled
*/
Conflator_Conflator.prototype.enable = function (opt_enabled) {
	this._disabled = opt_enabled === false;
};

Conflator_Conflator._proto = Conflator_Conflator.prototype;

/* harmony default export */ const util_Conflator = (Conflator_Conflator);


;// ./node_modules/@grid/core/es6/grid/util/SectionSettings.js
/* eslint-disable */








/* eslint-enable */

/** @constructor
* @extends {EventDispatcher}
* @param {!ILayoutGrid} grid
*/
let SectionSettings = function (grid) {
	let _t = this;
	_t._onDataChanged = _t._onDataChanged.bind(_t);
	_t.extendEventArg = _t.extendEventArg.bind(_t);

	_t._dataChangedConflator = new util_Conflator(100, _t._onDataChanged);

	_t._grid = grid;
	_t._grid.setEventArgExtender("rowCountChanged", _t.extendEventArg);
	_t._grid.setEventArgExtender("rowHeightChanged", _t.extendEventArg);
	_t._grid.setEventArgExtender("rowAvailable", _t.extendEventArg);
	_t._grid.listen("rowAvailable", _t._dispatch.bind(this, "rowAvailable"));

	_t._addEvent("dataChanged");
	_t._addEvent("rowAvailable");
};
es6_Ext.inherits(SectionSettings, event_EventDispatcher); // Inherits() will replace prototype of the derived object

/** @type {number}
* @private
*/
SectionSettings._runningId = 0;
/** @type {string}
* @private
*/
SectionSettings.prototype._id = "";
/** @type {string}
* @private
*/
SectionSettings.prototype._type = "";
/** @type {ILayoutGrid}
* @private
*/
SectionSettings.prototype._grid = null;
/** @type {ILayoutGrid}
* @private
*/
SectionSettings.prototype._snapshot = null;

/** @type {DataView}
* @private
*/
SectionSettings.prototype._dataView = null;
/** @type {Object}
* @private
*/
SectionSettings.prototype._dataArg = null;

/** @type {boolean}
* @description When the parameter is not specifically set by user, The mode will be switched automatically based on SectionSettings type
* @private
*/
SectionSettings.prototype._autoSyncRowCount; // Intentionally left undefined
/** @type {boolean}
* @private
*/
SectionSettings.prototype._bindingEnabled = true;
/** @type {SectionSettings}
* @private
*/
SectionSettings.prototype._parent = null;
/** @type {Array.<SectionSettings>}
* @private
*/
SectionSettings.prototype._children = null;
/** @type {!Conflator}
* @private
*/
SectionSettings.prototype._dataChangedConflator;


/** @public
* @return {string}
*/
SectionSettings.prototype.getId = function () {
	return this._id;
};
/** @public
* @param {string=} str
* @return {string}
*/
SectionSettings.prototype.setId = function (str) {
	this._id = (str) ? str : "section" + SectionSettings._runningId++;

	this._grid.setAttribute("id", this._id);
	return this._id;
};
/** @public
* @return {ILayoutGrid}
*/
SectionSettings.prototype.getSection = function () {
	return this._snapshot || this._grid;
};
/** @public
* @return {string}
*/
SectionSettings.prototype.getSectionName = function () {
	return this._grid.getId();
};
/** @public
* @return {number}
*/
SectionSettings.prototype.getIndex = function () {
	return this._grid.getIndex();
};
/** @public
* @return {string}
*/
SectionSettings.prototype.getType = function () {
	return this._type;
};
/** @public
* @param {string} type
*/
SectionSettings.prototype.setType = function (type) {
	if(this._type !== type) {
		if(this._type) {
			this._grid.removeClass(this._type);
		}
		this._type = type || "";
		if(type) {
			this._grid.addClass(type);
		}
	}
};

/** @public
* @return {DataView}
*/
SectionSettings.prototype.getDataSource = function () {
	return this._dataView;
};
/** @public
* @param {DataView} dataView
*/
SectionSettings.prototype.setDataSource = function (dataView) {
	let t = this;
	if(!t.getAutoSyncRowCount() || t._dataView === dataView) {
		return;
	}

	if (t._dataView) {
		t._dataView.unlisten("dataChanged", this._onDataChanged);
	}

	t._dataChangedConflator.reset();
	t._dataArg = null;

	t._dataView = dataView;

	if (t._dataView !== null) {
		t._dataView.listen("dataChanged", t._onDataChanged);
	}
	t._dispatchDataChanged(0, NaN);
};
/** @public
* @return {DataTable|DataView}
*/
SectionSettings.prototype.getRootDataSource = function () {
	if (!this._dataView) { return null; }

	let dataTable = this._dataView.getDataSource();
	if (!dataTable) { return null; }

	while (dataTable.getDataSource()) {
		dataTable = dataTable.getDataSource();
	}
	return dataTable;
};
/** @public
* @param {string} colName
* @param {number|string} rowRef Number is treated as row index relative to this section data view. String is data view row id
* @return {*} Returns undefined if not success
*/
SectionSettings.prototype.getData = function (colName, rowRef) {
	if (!this._dataView) { return null; }

	rowRef = this._getRowId(rowRef);
	return this._dataView.getData(rowRef, colName);
};
/** @public
* @param {string} colName
* @param {number|string} rowRef Number is treated as row index relative to this section data view. String is data view row id
* @param {*} data
* @return {boolean} Returns true for successful data setting
*/
SectionSettings.prototype.setData = function (colName, rowRef, data) {
	if (!this._dataView) { return false; }

	rowRef = this._getRowId(rowRef);
	this._dataView.setData(rowRef, colName, data);
	return true;
};
/** @public
* @param {number|string} rowRef Row Index of the display section or row id of the data table
* @return {Object.<string, *>}
*/
SectionSettings.prototype.getRowData = function (rowRef) {
	if (!this._dataView) { return null; }

	rowRef = this._getRowId(rowRef);
	return this._dataView.getRowData(rowRef);
};
/** @public
* @param {number|string} rowRef Row Index of the display section or row id of the data table
* @return {boolean} Returns true for successful data removing
*/
SectionSettings.prototype.removeRowData = function (rowRef) {
	if (!this._dataView) { return false; }

	let dataTable = /** @type{DataTable} */(this.getRootDataSource());
	if (!dataTable) { return false; }

	rowRef = this._getRowId(rowRef);
	let rowIndex = dataTable.getRowIndex(/** @type{string} */(rowRef));
	if (rowIndex < 0) { return false; }

	dataTable.removeRow(rowIndex);
	return true;
};
/** @public
* @param {Array.<boolean|undefined>} activations
* @param {number} firstIndex
* @param {number} lastIndex
*/
SectionSettings.prototype.activateColumns = function (activations, firstIndex, lastIndex) {
	// Column virtual rendering should only work with content section only
	if (this._type === "content") {
		for(let c = lastIndex + 1; --c >= firstIndex;) {
			let activation = activations[c];
			if (activation != null) {
				this._grid.activateColumn(c, activation);
			}
		}
	}
};

/** Clone a copy of the list of SectionSettingss. If there is no tree relationship, return null
* @public
* @return {Array.<SectionSettings>}
*/
SectionSettings.prototype.getChildren = function () { // TODO: Deprecate this method
	return this._children;
};
/**
* @public
* @ignore
* @param {SectionSettings} parent
*/
SectionSettings.prototype.setParent = function (parent) { // TODO: Deprecate this method
	if(this._parent) {
		let at = this._parent._children.indexOf(this);
		if(at >= 0) {
			this._parent._children.splice(at, 1);
			if(this._parent._children.length <= 0) {
				this._parent._children = null;
			}
		}
		this._parent = null;
	}
	this._parent = parent;
	if(this._parent) {
		if(!this._parent._children) {
			this._parent._children = [];
		}
		this._parent._children.push(this);
	}
};
/**
* @public
* @return {SectionSettings}
*/
SectionSettings.prototype.getParent = function () { // TODO: Deprecate this method
	return this._parent;
};

/** @public
* @return {!Conflator}
*/
SectionSettings.prototype.getDataChangedConflator = function () {
	return this._dataChangedConflator;
};
/** @public
* @param {boolean} opt_disabled
*/
SectionSettings.prototype.disableConflation = function (opt_disabled) {
	this._dataChangedConflator.disable(opt_disabled);
};
/** @public
* @param {boolean=} opt_disabled
*/
SectionSettings.prototype.disableDataBinding = function (opt_disabled) {
	this._bindingEnabled = opt_disabled === false;
};
/** When the parameter is not specifically set by user, The mode will be switched automatically based on SectionSettings type
* @public
* @return {boolean}
*/
SectionSettings.prototype.getAutoSyncRowCount = function () {
	return (this._autoSyncRowCount != null) ? this._autoSyncRowCount : (this._type === "content");
};
/** @public
* @param {boolean} bool
*/
SectionSettings.prototype.setAutoSyncRowCount = function (bool) {
	this._autoSyncRowCount = bool;
	if (this.getAutoSyncRowCount()) {
		let newRowCount = (this._dataView) ? this._dataView.getVisibleRowCount() : 0;
		this._grid.setRowCount(newRowCount);
	}
};

/** @public
* @ignore
* @return {boolean}
*/
SectionSettings.prototype.isDataBindable = function () {
	return (this._bindingEnabled && this._dataView) ? true : false;
};


/** WARNING: Do not call dispose directly, unless you know what you are doing
* @public
* @ignore
*/
SectionSettings.prototype.dispose = function () {
	let _t = this;
	_t._dataChangedConflator.reset();
	_t._dataArg = null;
	if (_t._grid) {
		_t._grid.dispose(); // Release from DOM document
	}
	if (_t._dataView) {
		_t._dataView.unlisten("dataChanged", _t._onDataChanged);
	}
	_t.unlistenAll();

	_t.setParent(null);
	_t._grid = null; // It is important release grid object
	_t._dataView = null;
};

/** @public
* @param {Object=} e
* @return {!Object}
*/
SectionSettings.prototype.extendEventArg = function (e) {
	if(!e) { e = {}; }
	if(e["sectionSettings"] !== this) {
		e["sectionSettings"] = this; // Be careful of circular reference
		e["section"] = this._snapshot || this._grid;
		e["sectionName"] = this._grid.getId();
		e["sectionIndex"] = this._grid.getIndex();
		e["sectionType"] = this._type;

		e["dataSource"] = this._dataView;

		if(this._snapshot) {
			e["print"] = true; // Add print flag to tell renderers of the current state
		} else if(e["print"]) {
			e["print"] = false;
		}
	}
	return e;
};
/** @public
* @param {Object} e
* @param {number} fromR
* @param {number} toR
* @return {!Object}
*/
SectionSettings.prototype.extendRenderEventArg = function (e, fromR, toR) {
	e = this.extendEventArg(e);
	if(!(fromR >= 0)) {
		if(this._snapshot) {
			fromR = 0;
			toR = this._grid.getRowCount();
		} else {
			fromR = this._grid.getFirstIndexInView();
			toR = this._grid.getLastIndexInView() + 1;
		}
	}
	e["fromRowIndex"] = fromR;
	e["toRowIndex"] = toR;
	return e;
};

/** @public
* @param {Object=} e
* @param {number=} begin INCLUSIVE. The value can be undefined or NaN
* @param {number=} end INCLUSIVE. The value can be undefined or NaN
* @return {!Object}
*/
SectionSettings.prototype.extendDataEventArg = function (e, begin, end) {
	if (!e) { e = {}; }
	// WARNING: Dynamic property adding for internal use
	e.firstUpdate = begin;
	e.lastUpdate = end;

	let firstInView, endOfView;
	if(this._snapshot) {
		firstInView = 0;
		endOfView = this._grid.getRowCount();
	} else {
		firstInView = this._grid.getFirstIndexInView();
		endOfView = this._grid.getLastIndexInView() + 1;
	}

	e["fromRowIndex"] = (begin > firstInView) ? begin : firstInView; // Handles NaN value
	if (this._dataView) {
		let dataRowCount = this._dataView.getVisibleRowCount();
		e["dataRowCount"] = dataRowCount;
		if (endOfView > dataRowCount) {
			endOfView = dataRowCount;
		}
	}
	e["toRowIndex"] = ((end + 1) < endOfView) ? (end + 1) : endOfView; // Handles NaN value

	this.extendEventArg(e);
	return e;
};

/** @public
* @ignore
* @param {number} val
*/
SectionSettings.prototype.setIndex = function (val) {
	this._grid.setIndex(val);
};
/** @public
* @ignore
* @return {number}
*/
SectionSettings.prototype.getRowOffset = function () {
	return this._grid.getRowOffset();
};
/** @public
* @ignore
* @param {number} val
*/
SectionSettings.prototype.setRowOffset = function (val) {
	this._grid.setRowOffset(val);
};

/** @public
* @param {boolean=} opt_enabled
*/
SectionSettings.prototype.enableColumnVirtualization = function (opt_enabled) {
	opt_enabled = (opt_enabled !== false);
	if (this._type === "content") {
		this._grid.enableColumnVirtualization(opt_enabled);
	}
};
/** @public
* @return {boolean}
*/
SectionSettings.prototype.isBinding = function () {
	return this._isEventDispatching("dataChanged");
};


/** @private
* @param {Object} e
*/
SectionSettings.prototype._onDataChanged = function (e) {
	if(this.isBinding()) { // During data binding, changing data is not allowed
		return;
	}

	if(this._bindingEnabled) {
		if (e) {
			this._mergeEvent(e);
		} else if (!this._dataArg) { // e does not exist due to the conflator
			return; // Invalid logic detected!
		}
	} else if(!this.getAutoSyncRowCount()) {
		return;
	}

	if (this._dataChangedConflator.conflate()) {
		return;
	}
	this._calculateChangeIndices();
	if(this._dataArg) {
		this._dispatchDataChanged(this._dataArg.firstUpdate, this._dataArg.lastUpdate);
	} else {
		this._dispatchDataChanged(0, NaN);
	}
};
/** @private
* @param {number} firstUpdate
* @param {number} lastUpdate
 */
SectionSettings.prototype._dispatchDataChanged = function (firstUpdate, lastUpdate) {
	let isAutoSync = this.getAutoSyncRowCount();
	let newCount = (this._dataView) ? this._dataView.getVisibleRowCount() : 0;
	let prevCount = this._grid.getRowCount();
	if (isAutoSync) {
		if (newCount !== prevCount) {
			this._grid.setRowCount(newCount, true);
		}
	} else {
		if (newCount > prevCount) {
			lastUpdate = prevCount - 1;
		}
	}

	this.updateRowData(firstUpdate, lastUpdate, this._dataArg);
	this._dataArg = null; // Data argument must be cleared to prevent memory leak
};
/** Fires data binding event without actual change in the data source.
* This will force visual elements to be re-rendered with the latest data in the data source.
* @public
* @param {number=} fromRowIndex INCLUSIVE If the value is undefined, the first row index will be used
* @param {number=} lastRowIndex INCLUSIVE If the value is undefined, the last row index will be used
* @param {Object=} e Additional parameters to be fired with the event
*/
SectionSettings.prototype.updateRowData = function (fromRowIndex, lastRowIndex, e) {
	if(this.isDataBindable()) { // Prevent dispatching dataChanged event without the data view
		if(this._grid._getTempRowHeights()){
			fromRowIndex = this._grid.getFirstIndexInView();
			lastRowIndex = this._grid.getLastIndexInView() + 1;
		}

		e = this.extendDataEventArg(e, fromRowIndex, lastRowIndex);
		if(e["fromRowIndex"] < e["toRowIndex"]) {
			this._dispatch("dataChanged", e);
		}
	}
};
/** @description Rerender section content. If the section is bound with the data source, dataChanged event is fired. Otherwise, rowAvailable event is fired instead.
* @public
*/
SectionSettings.prototype.rerender = function () {
	if(this.isDataBindable()) {
		this.updateRowData();
	} else {
		this._dispatch("rowAvailable", this.extendEventArg());
	}
};
/** Regenerate the section (UI part) on a different object
* @public
* @param {ILayoutGrid} clone A LayoutGrid that will be rendered
*/
SectionSettings.prototype.snapshot = function (clone) {
	if(clone) {
		this._snapshot = clone;
		try {
			clone["setColumnCount"](this._grid.getColumnCount());
			clone["setRowCount"](this._grid.getRowCount());
			this._dispatch("rowAvailable", this.extendEventArg());
		} catch (err) {
			console.warn("Unable to take a snapshot: " + err.message);
		}
		this._snapshot = null;

		// Since states could be changed after taking the snapshot, force rerendering again
		this._dispatch("rowAvailable", this.extendEventArg());
	}
};

/** @private
* @param {string|number|null=} rowRef
* @return {string}
*/
SectionSettings.prototype._getRowId = function (rowRef) {
	if (util.isNumber(rowRef)) {
		return this._dataView.getRowId(/** @type{number} */(rowRef));
	}
	return /** @type{string} */(rowRef);
};

/** @private
* @param {!Object} e
*/
SectionSettings.prototype._mergeEvent = function (e) {
	let arg = this._dataArg;
	if(!arg) {
		arg = this._dataArg = {};
		arg["actualUpdate"] = true; // Indicate that this event is fired because of some changes from data view
		arg["positionChangeOnly"] = true; // The change only cause position changes
		arg["events"] = [];
		arg.firstUpdate = NaN;
		arg.lastUpdate = NaN;
		arg.updatedRids = {};
	}
	if(!e["positionChangeOnly"]) {
		arg["positionChangeOnly"] = false;
	}

	if (e.events) {
		arg["events"] = arg["events"].concat(e.events);
	} else {
		arg["events"].push(e);
	}

	if(arg["globalChange"]) {
		return;
	}
	if(e["globalChange"]) {
		arg["globalChange"] = true;
		arg.firstUpdate = 0;
		arg.lastUpdate = NaN;
		return;
	}

	let rids = e["rids"];
	if(rids) {
		for(let i = rids.length; --i >= 0;) {
			arg.updatedRids[rids[i]] = true;
		}
	} else if(e["rid"]) {
		arg.updatedRids[e["rid"]] = true;
	}

	if(e["type"] !== "updated") { // inserted or removed
		arg.indexShifted = true;
		if(e["type"] === "removed") {
			arg.dataRemoved = true;
		}
	}
};
/** @private
*/
SectionSettings.prototype._calculateChangeIndices = function () {
	if(!this.isDataBindable() || this._dataArg["globalChange"]) {
		return;
	}

	// Calculate the first and last update indices.
	let rid, rowIndex;
	if(this._dataArg.indexShifted) {
		if(this._dataArg.dataRemoved) {
			this._dataArg.firstUpdate = 0;
		} else {
			for(rid in this._dataArg.updatedRids) {
				rowIndex = this._dataView.getRowIndex(rid);
				if(rowIndex >= 0) {
					if (!(this._dataArg.firstUpdate <= rowIndex)) {
						this._dataArg.firstUpdate = rowIndex;
					}
				}
			}
		}
		this._dataArg.lastUpdate = this._dataView.getVisibleRowCount();
	} else {
		for(rid in this._dataArg.updatedRids) {
			rowIndex = this._dataView.getRowIndex(rid);
			if(rowIndex >= 0) {
				if (!(this._dataArg.firstUpdate <= rowIndex)) {
					this._dataArg.firstUpdate = rowIndex;
				}
				if (!(this._dataArg.lastUpdate >= rowIndex)) {
					this._dataArg.lastUpdate = rowIndex;
				}
			}
		}
	}
};

SectionSettings._proto = SectionSettings.prototype;

/* harmony default export */ const util_SectionSettings = (SectionSettings);


;// ./node_modules/@grid/core/es6/grid/util/Virtualizer.js


 // eslint-disable-line

/** @constructor
* @param {!TrackLayout} trackLayout
* @extends {EventDispatcher}
*/
let Virtualizer = function (trackLayout) {
	this._layout = trackLayout;
	this._arg = {};
	this._addEvent("indexChanged");
};
es6_Ext.inherits(Virtualizer, event_EventDispatcher); // Inherits() will replace prototype of the derived object

/** @private
* @type {!TrackLayout}
*/
Virtualizer.prototype._layout;
/** @private
* @type {number}
*/
Virtualizer.prototype._viewSize = 0;
/** @private
* @type {number}
*/
Virtualizer.prototype._viewOffset = 0;
/** @private
* @type {number}
*/
Virtualizer.prototype._startOffsetCount = 0;
/** @private
* @type {number}
*/
Virtualizer.prototype._endOffsetCount = 0;
/** @private
* @type {number}
*/
Virtualizer.prototype._bufferSize = 0;

/** @private
* @type {Object}
*/
Virtualizer.prototype._arg = null; // a cache to event argument

/** @private
* @type {boolean}
*/
Virtualizer.prototype._enabled = true;
/** @private
* @type {boolean}
*/
Virtualizer.prototype._active = true;
/** @private
* @type {boolean}
*/
Virtualizer.prototype._virtualizable = false;

/** @private
* @return {number}
*/
Virtualizer.prototype._getMinIndex = function () {
	return this._startOffsetCount;
};
/** @private
* @return {number}
*/
Virtualizer.prototype._getMaxIndex = function () {
	let index = this._layout.getLaneCount() - this._endOffsetCount;
	return index > this._startOffsetCount ? index : this._startOffsetCount;
};

/** @public
* @param {number} index
* @return {number}
*/
Virtualizer.prototype.getContentStart = function (index) {
	return this._layout.getLaneStart(index);
};
/** @public
* @return {number}
*/
Virtualizer.prototype.getContentInViewCount = function () {
	let first = this.getFirstIndexInView();
	let last = this.getLastIndexInView();
	if (first >= 0 && first < last) {
		return last - first + 1;
	}
	return 0;
};

/** @private
 * @returns {number}
*/
Virtualizer.prototype._getContentSizeInBounds = function () {
	let minIndex = this._getMinIndex();
	let maxIndex = this._getMaxIndex();
	return this._layout.getLaneStart(maxIndex) - this._layout.getLaneStart(minIndex);
};

/** @public
* @return {number}
*/
Virtualizer.prototype.getViewSize = function () {
	return this._viewSize;
};


/** @public
* @return {number}
*/
Virtualizer.prototype.getViewOffset = function () {
	let minIndex = this._getMinIndex();
	return this._viewOffset + this._layout.getLaneStart(minIndex);
};
/** @public
* @return {number}
*/
Virtualizer.prototype.getIndexOffset = function () {
	return this._startOffsetCount;
};

/** @public
* @return {boolean}
*/
Virtualizer.prototype.isVirtualizable = function () {
	return this._virtualizable;
};

/** @public
* @return {number}
*/
Virtualizer.prototype.getFirstIndexInView = function () {
	if (!this._virtualizable) {
		return 0;
	}
	let vo = this.getViewOffset();
	let trackSize = this._layout.getTrackSize();
	if(vo >= trackSize) {
		vo = trackSize - this._viewSize;  // WARNING: View offset already includes minIndex and this will override that
	}
	if(vo < 0) {
		return 0;
	}

	return this._layout.hitTest(vo);
};
/** @public
* @return {number} INCLUSIVE last index in view
*/
Virtualizer.prototype.getLastIndexInView = function () {
	if (!this._virtualizable) {
		return this._layout.getLaneCount() - 1;
	} else {
		let viewEnd = this.getViewOffset() + this._viewSize;

		// prevent from invalid-index because of verical buffering
		let maxIndex = this._getMaxIndex();
		let endBound = this._layout.getLaneStart(maxIndex);
		if (viewEnd >= endBound) {
			return maxIndex - 1; // maxIndex is exclusive
		}

		return this._layout.hitTest(viewEnd);
	}
};

/** @public
* @return {number}
*/
Virtualizer.prototype.getPrevFirstIndexInView = function () {
	return (this._arg["prevFirstIndex"] != null) ? this._arg["prevFirstIndex"] : -1;
};
/** @public
* @return {number}
*/
Virtualizer.prototype.getPrevLastIndexInView = function () {
	return (this._arg["prevLastIndex"] != null) ? this._arg["prevLastIndex"] : -1;
};

/** @public
* @param {boolean=} opt_force
* @return {boolean}
*/
Virtualizer.prototype.update = function (opt_force) {
	if (opt_force !== true) {
		if (!(this._enabled && this._active)) {
			return false;
		}
	}

	if (this._isEventDispatching("indexChanged")) { return false; }

	let fi = this.getFirstIndexInView();
	let li = this.getLastIndexInView(); // INCLUSIVE
	let changed = (this._arg["prevFirstIndex"] !== fi) || (this._arg["prevLastIndex"] !== li);

	if (opt_force || changed) {
		this._arg["firstIndex"] = fi;
		this._arg["lastIndex"] = li;

		this._dispatch("indexChanged", this._arg);

		this._arg["prevFirstIndex"] = fi;
		this._arg["prevLastIndex"] = li;
	}
	return changed;
};

/** @public
* @return {boolean}
*/
Virtualizer.prototype.isEnabled = function () {
	return this._enabled;
};
/** @public
* @param {boolean=} opt_enabled
*/
Virtualizer.prototype.enable = function (opt_enabled) {
	opt_enabled = (opt_enabled !== false);
	if (this._enabled !== opt_enabled) {
		this._enabled = opt_enabled;
		this.validateVirtualization();
		this.update(true);
	}
};
/** @public
* @param {boolean=} opt_disabled
*/
Virtualizer.prototype.disable = function (opt_disabled) {
	this.enable(opt_disabled === false);
};

/** @public
* @ignore
* @return {boolean}
*/
Virtualizer.prototype.isActive = function () {
	return this._active;
};
/**
* @public
* @ignore
* @param {boolean=} opt_enabled
*/
Virtualizer.prototype.activate = function (opt_enabled) {
	opt_enabled = (opt_enabled !== false);
	if (this._active !== opt_enabled) {
		this._active = opt_enabled;
		this.validateVirtualization();
		this.update(true);
	}
};
/**
* @public
* @ignore
* @param {boolean=} opt_disabled
*/
Virtualizer.prototype.deactivate = function (opt_disabled) {
	this.activate(opt_disabled === false);
};
/** @public
* @ignore
* @return {number}
*/
Virtualizer.prototype.getBufferSize = function () {
	return this._bufferSize;
};

/** @public
* @param {!TrackLayout} trackLayout
* @ignore
*/
Virtualizer.prototype.setContentLayout = function (trackLayout) {
	if (this._layout !== trackLayout) {
		this._layout = trackLayout;
		this.validateVirtualization();
	}
};
/** @public
* @ignore
* @param {number} px
*/
Virtualizer.prototype.setViewSize = function (px) {
	if (px < 0) { px = 0; }

	if (this._viewSize !== px) {
		this._viewSize = px;
		this.validateVirtualization();
	}
};
/** @public
* @ignore
* @param {number} px
*/
Virtualizer.prototype.setViewOffset = function (px) {
	this._viewOffset = px > 0 ? px : 0;
	this.update();
};
/** @public
* @ignore
* @param {number} startItemCount
* @param {number} endItemCount
*/
Virtualizer.prototype.setViewBounds = function (startItemCount, endItemCount) {
	this._startOffsetCount = startItemCount > 0 ? startItemCount : 0;
	this._endOffsetCount = endItemCount > 0 ? endItemCount : 0;
	this.validateVirtualization(); // Everytime row height is changed
};
/** @public
* @ignore
* @param {number} px
*/
Virtualizer.prototype.setBufferSize = function (px) {
	if (px < 0) { px = 0; }
	if (this._bufferSize !== px) {
		this._bufferSize = px;
		this.validateVirtualization();
	}
};

/** @public
* @ignore
*/
Virtualizer.prototype.validateVirtualization = function () {
	this._virtualizable = false;
	if (this._enabled && this._active) {
		// If either content or view have no size, or content is smaller than view, entire content is contained within view
		if (this._viewSize > 0) {
			if (this._getContentSizeInBounds() > this._viewSize) {
				this._virtualizable = true;
			}
		}
	}
};

Virtualizer._proto = Virtualizer.prototype;

/* harmony default export */ const util_Virtualizer = (Virtualizer);


;// ./node_modules/@grid/core/es6/grid/components/VScrollbar.js
/* eslint-disable */



/* eslint-enable */

/** @constructor
* @extends {Scrollbar}
*/
let VScrollbar = function () {
	let t = this; // Create a shorthand to shorthen file size
	t._vertical = true;
	t._hostClassName = "vscrollbar-host";
	VScrollbar.base(this, 'constructor'); // Call base constructor

	t._onScroll = t._onScroll.bind(t);

	t._addPaneAt(); // Vertical scroll bar will have only one pane for multiple sections

	// Styling
	t._element.classList.add("grid-vscroll");
	t._trackContent.style.width = "1px";

	t.disableMouseWheel(false); // Mousewheel for vertical scrollbar is enabled by default

	t.listen("scroll", t._onScroll); // For backward compatability
	t._addEvent("contentScrolled");
};
es6_Ext.inherits(VScrollbar, components_Scrollbar);


/** For internal usage only
* @public
* @ignore
* @param {ElementWrapper} grid
* @param {Array.<ElementWrapper>} sections
* @param {number} startingSection
*/
VScrollbar.prototype.setScrollContent = function (grid, sections, startingSection) {
	this._clearAllPanes();

	let sectionCount = sections.length;
	if(sectionCount <= 0) { return; }

	// Insert panes in the correct slot
	let section;
	let pane = this._panes[0]; // Vertical scrollbar has only one pane and one paneSlider
	let paneSlider = this._paneSliders[0];

	let s = 0;
	while(s < startingSection) {
		section = sections[s++];
		section.setParent(grid);
	}

	pane.setParent(grid);
	let dest = sectionCount - this._footerCount;

	while(s < dest) {
		section = sections[s++];
		section.setParent(paneSlider);
	}

	while(s < sectionCount) {
		section = sections[s++];
		section.setParent(grid);
	}
	this.invalidateContent(); // Delay content update
};

/** @override
*/
VScrollbar.prototype.setHeight = function (val) {
	this.setTrackSize(val, val);
	return true;
};
/** @override
*/
VScrollbar.prototype.getHeight = function () {
	return this.getPaneSize(); // Track size may not represent what the client want
};

/** @override */
VScrollbar.prototype.getScrollTop = function () {
	return this._getScrollValue();
};
/** @public
* @param {number} px
*/
VScrollbar.prototype.setScrollTop = function (px) {
	this.update();
	this._scrollTo(px); // "scroll" event may be fired
};

/** @public
* @override
* @return {boolean}
*/
VScrollbar.prototype.isEndOfVerticalScroll = function () {
	return this._isEndOfScroll();
};
/** @public
* @return {number}
*/
VScrollbar.prototype.getContentHeight = function () {
	let nodeList = this._paneSliders[0].getContent(true);
	let sum = 0;
	for (let i = nodeList.length; --i >= 0; ) {
		sum += nodeList[i].offsetHeight; // TODO: Slow
	}
	return sum;
};
/** @override
* @protected
* @return {number}
*/
VScrollbar.prototype.calculateContentSize = function () {
	return this.getContentHeight();
};

/** @public
* @ignore
* @return {ElementWrapper}
*/
VScrollbar.prototype.getPane = function () {
	return this._panes[0];
};

/** @private
* @param {!Object} e
*/
VScrollbar.prototype._onScroll = function (e) {
	this._dispatch("contentScrolled", e); // For backward compatibility
};

VScrollbar._proto = VScrollbar.prototype;

/* harmony default export */ const components_VScrollbar = (VScrollbar);


;// ./node_modules/@grid/core/es6/grid/VirtualizedLayoutGrid.js
/* eslint-disable */









/* eslint-enable */


/** @constructor
 * @ignore
 * @param {Object=} options
 * @extends {ElementWrapper}
*/
let VirtualizedLayoutGrid = function (options) {
	let _t = this;
	_t._onRestingPosition = _t._onRestingPosition.bind(_t);
	_t._updateRowBounds = _t._updateRowBounds.bind(_t);

	_t._element = document.createElement("div");
	_t._element.className = "tr-vlg section";

	if(!options) {
		options = {};
	}
	if(options["horizontalScrollbar"]) {
		_t._hscrollbar = options["horizontalScrollbar"];
	}
	options["owner"] = this;

	_t._grid = new grid_LayoutGrid(options);
	_t._grid.setParent(_t._element);

	_t._layoutY = new util_TrackLayout();
	_t._selectionList =  new util_SelectionList();
	_t._reverter = new util_Reverter(_t._onRestingPosition, 200); // WORKAROUND: GRID-402

	_t._addEvent("rowHighlighted");
	_t._addEvent("rowCountChanged");
	_t._addEvent("rowHeightChanged");
	_t._addEvent("rowAvailable");
};
es6_Ext.inherits(VirtualizedLayoutGrid, components_ElementWrapper);


/**
 * @private
 * @type {number}
 */
VirtualizedLayoutGrid.prototype._index = -1;

/** Number of rows offsetting from the first section
 * @private
 * @type {number}
 */
VirtualizedLayoutGrid.prototype._rowOffset = 0;
/** For virtualization in Y-Axis
 * @private
 * @type {number}
 */
VirtualizedLayoutGrid.prototype._firstIndex = 0;

/** @private
* @type {!LayoutGrid}
*/
VirtualizedLayoutGrid.prototype._grid;

/** @private
* @type {!TrackLayout}
*/
VirtualizedLayoutGrid.prototype._layoutY;

/** @private
 * @type {SelectionList}
 */
VirtualizedLayoutGrid.prototype._selectionList = null;

/** @type {Reverter}
* @private
*/
VirtualizedLayoutGrid.prototype._reverter = null;
/** @type {Array.<Element>}
* @private
*/
VirtualizedLayoutGrid.prototype._rowBounds = null;
/** @type {Array.<Element>}
* @private
*/
VirtualizedLayoutGrid.prototype._rowBoundCache = null;
/** @type {boolean}
* @private
*/
VirtualizedLayoutGrid.prototype._rowSelDirty = false;
/** @type {Object}
* @private
*/
VirtualizedLayoutGrid.prototype._cellBoundPainter = null;
/** @type {number}
* @private
*/
VirtualizedLayoutGrid.prototype._rowBoundTimer = 0;
/** @type {Element}
* @private
*/
VirtualizedLayoutGrid.prototype._boundLayer = null;
/** @type {HScrollbar}
* @private
*/
VirtualizedLayoutGrid.prototype._hscrollbar = null;
/** @type {string}
* @private
*/
VirtualizedLayoutGrid.prototype._session = "A";
/** @type {Array<boolean>}
* @private
*/
VirtualizedLayoutGrid.prototype._tempRowHeights = null;

//#region ====== Override ElementWrapper ======//
/** @override */
VirtualizedLayoutGrid.prototype.getWidth = function () {
	if(this.isVisible()) {
		return this._grid.getWidth();
	}
	return 0;
};
/** @override */
VirtualizedLayoutGrid.prototype.getHeight = function () {
	if (this.isVisible()) {
		return this._element.offsetHeight;
	}
	return 0;
};
/** @override */
VirtualizedLayoutGrid.prototype.dispose = function () {
	this.setRowCount(0); // Fire rowCountChanged
	this.unlistenAll();

	this._hscrollbar = null;
	this._grid.dispose();
	this._dispose();
	this._reverter.dispose();

	this._rowBounds = this._rowBoundCache = null;
	this._rowSelDirty = false;
	if(this._rowBoundTimer) {
		clearTimeout(this._rowBoundTimer);
		this._rowBoundTimer = 0;
	}
	if(this._cellBoundPainter) {
		this._cellBoundPainter.dispose();
	}
};
/** @override */
VirtualizedLayoutGrid.prototype.setWidth = function (px) {
	return this._grid.setWidth(px);
};
/** @override */
VirtualizedLayoutGrid.prototype.setHeight = function (px) {
	return false;
};
/** {@link ElementWrapper#show}
* @override */
VirtualizedLayoutGrid.prototype.show = function (opt_shown) {
	this._element.style.display = (opt_shown !== false) ? "" : "none";
	this._dispatchRowHeightChangedEvent(0, this._layoutY.getLaneCount());
};
//#endregion

//#region ====== Implementing ILayoutGrid ======//
/** @inheritDoc */
VirtualizedLayoutGrid.prototype.hitTest = function (x, y) {
	let e = [];
	e["colIndex"] = this._grid.hitTestX(x);
	e["rowIndex"] = this._layoutY.hitTest(y);
	return e;
};
/** @inheritDoc */
VirtualizedLayoutGrid.prototype.hitTestX = function (x) {
	return this._grid.hitTestX(x);
};
/** @inheritDoc */
VirtualizedLayoutGrid.prototype.hitTestY = function (y) {
	return this._layoutY.hitTest(y);
};

/** @inheritDoc */
VirtualizedLayoutGrid.prototype.getDefaultColumnWidth = function () {
	return this._grid.getDefaultColumnWidth();
};
/** @inheritDoc */
VirtualizedLayoutGrid.prototype.setDefaultColumnWidth = function (val) {
	this._grid.setDefaultColumnWidth(val);
};

/** @inheritDoc */
VirtualizedLayoutGrid.prototype.getDefaultRowHeight = function () {
	return this._layoutY.getDefaultLaneSize();
};
/** @inheritDoc */
VirtualizedLayoutGrid.prototype.setDefaultRowHeight = function (val) {
	let prevVal = this._layoutY.getDefaultLaneSize();
	if (prevVal !== val) {
		this._layoutY.setDefaultLaneSize(val);
		this._element.style.height = this._layoutY.getTrackSize() + "px";
		this._grid.setDefaultRowHeight(val);
		this._dispatchRowHeightChangedEvent(0, this._layoutY.getLaneCount());
	}
};

/** @inheritDoc */
VirtualizedLayoutGrid.prototype.getColumnWidth = function (index) {
	return this._grid.getColumnWidth(index);
};
/** @inheritDoc */
VirtualizedLayoutGrid.prototype.setColumnWidth = function (index, val) {
	return this._grid.setColumnWidth(index, val);
};
/** @public
 * @ignore
 * @param {number} indexX
 * @param {number} val
 * @return {boolean}
 */
VirtualizedLayoutGrid.prototype._setColumnWidth = function (indexX, val) {
	return this._grid._setColumnWidth(indexX, val);
};
/** @public
 * @ignore
 */
VirtualizedLayoutGrid.prototype._updateColumnLayout = function () {
	this._grid._updateColumnLayout();
};
/** @public
 * @ignore
 * @param {boolean} enabled
 */
VirtualizedLayoutGrid.prototype._startBindingSession = function (enabled) {
	this._isBinding = enabled;
	if(!enabled){
		// Clear old session
		if(this._tempRowHeights){
			let defaultRowHeight = this.getDefaultRowHeight();
			let hasSession = true;
			let removedIndices = [];

			for(let key in this._tempRowHeights){
				let index = +key;
				let rowSession = this._layoutY.getLaneProperty(index, "sizeSession");
				if(rowSession) {
					if(rowSession != this._session){
						this.setRowHeight(index, defaultRowHeight);
						this._layoutY.setLaneProperty(index, "sizeSession", null);
						removedIndices.push(index);
					}
				}
			}
			let removedCount = removedIndices.length;
			for(let i = 0; i < removedCount; i++){
				delete this._tempRowHeights[removedIndices[i]];
			}
			if(removedCount){
				for(let remainingKey in this._tempRowHeights){
					hasSession = remainingKey != null;
					break;
				}
			}
			if(!hasSession){
				this._tempRowHeights = null;
			}

		}
		this._session = this._session === "A" ? "B" : "A";
	}
};
/** @public
 * @ignore
 * @return {Array<boolean>}
 */
VirtualizedLayoutGrid.prototype._getTempRowHeights = function () {
	return this._tempRowHeights;
};
/** @inheritDoc */
VirtualizedLayoutGrid.prototype.getRowHeight = function (index) {
	return this._layoutY.getLaneSize(index);
};
/** @inheritDoc */
VirtualizedLayoutGrid.prototype.setRowHeight = function (index, val) {
	if(this._isBinding && val != this.getDefaultRowHeight()){
		this._layoutY.setLaneProperty(index, "sizeSession", this._session);
		if(!this._tempRowHeights){
			this._tempRowHeights = [];
		}
		this._tempRowHeights[index] = true; // Store row index that set temporary row height
	}
	if(this._layoutY.setLaneSize(index, val)) {
		this._grid.setRowHeight(index - this._firstIndex, val);
		this._element.style.height = this._layoutY.getTrackSize() + "px";
		this._requestUpdatingRowBounds();
		return this._dispatchRowHeightChangedEvent(index, index + 1);
	}
	return false;
};

/** @inheritDoc */
VirtualizedLayoutGrid.prototype.isColumnVisible = function (index) {
	return this._grid.isColumnVisible(index);
};
/** @inheritDoc */
VirtualizedLayoutGrid.prototype.hideColumn = function (index, val) {
	return this._grid.hideColumn(index, val);
};

/** @inheritDoc */
VirtualizedLayoutGrid.prototype.isRowVisible = function (index) {
	return this._grid.isRowVisible(index - this._firstIndex);
};
/** @inheritDoc */
VirtualizedLayoutGrid.prototype.hideRow = function (index, val) {
	return this._grid.hideRow(index - this._firstIndex, val);
};

/** @inheritDoc */
VirtualizedLayoutGrid.prototype.getMinimumColumnWidth = function (index) {
	return this._grid.getMinimumColumnWidth(index);
};
/** @inheritDoc */
VirtualizedLayoutGrid.prototype.setMinimumColumnWidth = function (index, val) {
	return this._grid.setMinimumColumnWidth(index, val);
};

/** @inheritDoc */
VirtualizedLayoutGrid.prototype.getMinimumRowHeight = function (index) {
	return this._layoutY.getMinimumLaneSize(index);
};
/** @inheritDoc */
VirtualizedLayoutGrid.prototype.setMinimumRowHeight = function (index, val) {
	return false;
};

/** @inheritDoc */
VirtualizedLayoutGrid.prototype.getCellLeft = function (indexX) {
	return this._grid.getCellLeft(indexX);
};
/** @inheritDoc */
VirtualizedLayoutGrid.prototype.getCellTop = function (indexY) {
	return this._layoutY.getLaneStart(indexY);
};
/** @inheritDoc */
VirtualizedLayoutGrid.prototype.getCellWidth = function (indexX, indexY) {
	return this._grid.getCellWidth(indexX, 0); //In virtualization, every cell in the same column has the same width
};
/** @inheritDoc */
VirtualizedLayoutGrid.prototype.getCellHeight = function (indexX, indexY) {
	return this._layoutY.getLaneSize(indexY); //In virtualization, every cell in the same row has the same height
};

/** @inheritDoc */
VirtualizedLayoutGrid.prototype.hasCellSpan = function () {
	return this._grid.hasCellSpan();
};
//Not support row spanning in virtualization
/** @inheritDoc */
VirtualizedLayoutGrid.prototype.getCellRowSpan = function (indexX, indexY) {
	return this._grid.getCellRowSpan(indexX, indexY - this._firstIndex);
};
/** @inheritDoc */
VirtualizedLayoutGrid.prototype.setCellRowSpan = function (indexX, indexY, rowSpan) {
	return this._grid.setCellRowSpan(indexX, indexY - this._firstIndex, rowSpan);
};
/** @inheritDoc */
VirtualizedLayoutGrid.prototype.getCellColSpan = function (indexX, indexY) {
	return this._grid.getCellColSpan(indexX, indexY - this._firstIndex);
};
/** @inheritDoc */
VirtualizedLayoutGrid.prototype.setCellColSpan = function (indexX, indexY, colSpan) {
	return this._grid.setCellColSpan(indexX, indexY - this._firstIndex, colSpan);
};
/** @inheritDoc */
VirtualizedLayoutGrid.prototype.clearCellSpans = function () {
	return this._grid.clearCellSpans();
};
/** @inheritDoc */
VirtualizedLayoutGrid.prototype.clearColumnSpans = function (indexX) {
	return this._grid.clearColumnSpans(indexX);
};

/** @inheritDoc */
VirtualizedLayoutGrid.prototype.clearColumnCustomizations = function () {
	this._grid.clearColumnCustomizations();
};
/** @inheritDoc */
VirtualizedLayoutGrid.prototype.clearRowCustomizations = function () {
	this._layoutY.clearLaneCustomizations();
	this._grid.clearRowCustomizations();
};

/** @inheritDoc */
VirtualizedLayoutGrid.prototype.getColumnScalability = function (indexX) {
	return this._grid.getColumnScalability(indexX);
};
/** @inheritDoc */
VirtualizedLayoutGrid.prototype.setColumnScalability = function (indexX, val) {
	return this._grid.setColumnScalability(indexX, val);
};

/** @inheritDoc */
VirtualizedLayoutGrid.prototype.getRowScalability = function (indexY) {
	return this._layoutY.getLaneScalability(indexY);
};
/** @inheritDoc */
VirtualizedLayoutGrid.prototype.setRowScalability = function (index, val) {
	return false;
};

/** @inheritDoc */
VirtualizedLayoutGrid.prototype.getCell = function (indexX, indexY, opt_includeSpan) {
	return this._grid.getCell(indexX, indexY - this._firstIndex, opt_includeSpan);
};
/** @inheritDoc */
VirtualizedLayoutGrid.prototype.getCellContent = function (indexX, indexY) {
	return this._grid.getCellContent(indexX, indexY - this._firstIndex);
};
/** @inheritDoc */
VirtualizedLayoutGrid.prototype.setCellContent = function (indexX, indexY, data) {
	return this._grid.setCellContent(indexX, indexY - this._firstIndex, data);
};

/** @inheritDoc */
VirtualizedLayoutGrid.prototype.getColumnCount = function () {
	return this._grid.getColumnCount();
};
/** @inheritDoc */
VirtualizedLayoutGrid.prototype.setColumnCount = function (val) {
	this._grid.setColumnCount(val);
};

/** @inheritDoc */
VirtualizedLayoutGrid.prototype.getRowCount = function () {
	return this._layoutY.getLaneCount();
};
/** @inheritDoc */
VirtualizedLayoutGrid.prototype.setRowCount = function (val, noBinding) {
	let prevCount = this._layoutY.getLaneCount();
	if(prevCount !== val){
		this._layoutY.setLaneCount(val);
		if(this._tempRowHeights){
			this._tempRowHeights.length = val;
		}
		this._element.style.height = this._layoutY.getTrackSize() + "px";
		this._requestUpdatingRowBounds();
		//After rowCountChanged fires, virtualizer will update virtual row count if
		//the range of virtual layout changed.
		this._dispatch("rowCountChanged", { "rowCount": val, "prevCount": prevCount, "noBinding": noBinding });
	}
};
/** @inheritDoc */
VirtualizedLayoutGrid.prototype.getVisibleRowCount = function () {
	return this.getRowCount();
};
/** @inheritDoc */
VirtualizedLayoutGrid.prototype.addRow = function (val) {
	if (val <= 0) { return; }
	if (this._grid.getColumnCount() <= 0) {
		console.log("WARNING: Row cannot be added, unless at least one column is added");
		return;
	}
	this.setRowCount(this._layoutY.getLaneCount() + val);
};
/** @inheritDoc */
VirtualizedLayoutGrid.prototype.removeRow = function (val) {
	if (val <= 0) { return; }
	let rowCount = this._layoutY.getLaneCount();
	if (val > rowCount) { val = rowCount; }
	this.setRowCount(rowCount - val);
};

/** @inheritDoc */
VirtualizedLayoutGrid.prototype.setBorders = function (lft, top, rgt, btm) {
	this._grid.setBorders(lft, top, rgt, btm);
};
/** @inheritDoc */
VirtualizedLayoutGrid.prototype.getColumnStyle = function (colIndex, styleName) {
	return this._grid.getColumnStyle(colIndex, styleName);
};
/** @inheritDoc */
VirtualizedLayoutGrid.prototype.setColumnStyle = function (colIndex, styleName, value) {
	this._grid.setColumnStyle(colIndex, styleName, value);
};

/** @inheritDoc */
VirtualizedLayoutGrid.prototype.enableColumnClass = function (colIndex, className, enabled) {
	this._grid.enableColumnClass(colIndex, className, enabled);
};
/** @inheritDoc */
VirtualizedLayoutGrid.prototype.hasColumnClass = function (colIndex, className) {
	return this._grid.hasColumnClass(colIndex, className);
};
/** @inheritDoc */
VirtualizedLayoutGrid.prototype.enableRowClass = function (rowIndex, className, enabled) {
	this._grid.enableRowClass(rowIndex - this._firstIndex, className, enabled);
};
/** @inheritDoc */
VirtualizedLayoutGrid.prototype.setRowStyle = function (rowIndex, styleName, value, enabled) {
	this._grid.setRowStyle(rowIndex - this._firstIndex, styleName, value, enabled);
};

/** @inheritDoc */
VirtualizedLayoutGrid.prototype.getContentWidth = function () {
	return this._grid.getContentWidth();
};
/** @inheritDoc */
VirtualizedLayoutGrid.prototype.getContentHeight = function () {
	return this._layoutY.getTrackSize();
};

/** @inheritDoc */
VirtualizedLayoutGrid.prototype.getCellsByRowIndex = function(rowIndex) {
	return this._grid.getCellsByRowIndex(rowIndex - this._firstIndex);
};

/** @inheritDoc */
VirtualizedLayoutGrid.prototype.disableHorizontalGridLine = function (opt_bool) {
	this._grid.disableHorizontalGridLine(opt_bool);
};
/** @inheritDoc */
VirtualizedLayoutGrid.prototype.disableVerticalGridLine = function (opt_bool) {
	this._grid.disableVerticalGridLine(opt_bool);
};

/** @inheritDoc */
VirtualizedLayoutGrid.prototype.enableShading = function (enabled) {
	this._grid.enableShading(enabled);
};

/** @inheritDoc */
VirtualizedLayoutGrid.prototype.setTransposedShading = function (enabled) {
	this._grid.setTransposedShading(enabled);
};

/** @inheritDoc */
VirtualizedLayoutGrid.prototype.enableRowHighlighting = function (opt_bool) {
	this._grid.enableRowHighlighting(opt_bool);
};

/** @inheritDoc */
VirtualizedLayoutGrid.prototype.getRowHighlight = function () {
	let index = this._grid.getRowHighlight();
	if(index >= 0){
		index += this._firstIndex;
	}
	return index;
};

/** @inheritDoc */
VirtualizedLayoutGrid.prototype.setRowHighlight = function (rowIndex) {
	let prevIndex = this._grid.getRowHighlight();
	this._grid.setRowHighlight(rowIndex - this._firstIndex);

	if (this._hasListener("rowHighlighted")) {
		rowIndex = this._grid.getRowHighlight();

		this._dispatch("rowHighlighted", {
			"prevRowIndex": (prevIndex >= 0) ? prevIndex + this._firstIndex : prevIndex,
			"rowIndex": (rowIndex >= 0) ? rowIndex + this._firstIndex : rowIndex
		});
	}
};

/** @inheritDoc */
VirtualizedLayoutGrid.prototype.getSelectedRows = function () {
	if(this.getSelectedRowCount() > 0){
		return this._selectionList.getAllSelections();
	}
	return null;
};
/** @inheritDoc */
VirtualizedLayoutGrid.prototype.getSelectedRowCount = function () {
	return this._selectionList.getSelectionCount();
};

/** @inheritDoc */
VirtualizedLayoutGrid.prototype.isSelectedRow = function (at) {
	return this._selectionList.getSelection(at);
};

/** @inheritDoc */
VirtualizedLayoutGrid.prototype.setSelectedRow = function (rowIndex, opt_selected) {
	this._selectionList.setSelection(rowIndex, opt_selected);

	let selected = this._selectionList.getSelection(rowIndex);
	this._grid.setSelectedRow(rowIndex - this._firstIndex, selected);

	if(selected) {
		this._rowSelDirty = true;
		this._initBoundLayer();
	}
	this._requestUpdatingRowBounds();
};

/** @inheritDoc */
VirtualizedLayoutGrid.prototype.selectSingleRow = function (rowIndex) {
	this._selectionList.clearAllSelections();
	this._grid.clearSelectedRows();
	this.setSelectedRow(rowIndex);
};
/** @inheritDoc */
VirtualizedLayoutGrid.prototype.selectRowRange = function (rowIndex, length) {
	this._selectionList.selectRange(rowIndex, length);
	this._updateRowSelection();
	this._rowSelDirty = true;
	this._initBoundLayer();
	this._requestUpdatingRowBounds();
};
/** @inheritDoc */
VirtualizedLayoutGrid.prototype.clearSelectedRows = function () {
	let count = this._selectionList.clearAllSelections();
	this._grid.clearSelectedRows();
	this._requestUpdatingRowBounds(); // WARNING: Row bounds are not removed from the document immediately
	return count;
};
/** @inheritDoc */
VirtualizedLayoutGrid.prototype.getFirstSelectedRow = function () {
	return this._selectionList.getFirstSelectedIndex();
};
/** @inheritDoc */
VirtualizedLayoutGrid.prototype.getLastSelectedRow = function () {
	return this._selectionList.getLastSelectedIndex();
};
/** @inheritDoc */
VirtualizedLayoutGrid.prototype.getRowAnchor = function () {
	return this._selectionList.getSelectionAnchor();
};
/** @inheritDoc */
VirtualizedLayoutGrid.prototype.setRowAnchor = function (at) {
	this._selectionList.setSelectionAnchor(at);
};

/** @inheritDoc */
VirtualizedLayoutGrid.prototype.activateColumn = function (colIndex, opt_bool) {
	this._grid.activateColumn(colIndex, opt_bool);
};
/** @inheritDoc */
VirtualizedLayoutGrid.prototype.activateRows = function (from, to, updateRange) {
	let maxLen = this._layoutY.getLaneCount();
	if(!from) { from = this._rowOffset; }
	if(!to && to !== 0) { to = from + maxLen; }

	from -= this._rowOffset; // Convert to relative index
	if(from < 0) { from = 0; }
	to -= this._rowOffset; // Convert to relative index
	if(to < 0) { to = 0; }

	let prevLen = this._getInnerRowCount();
	let pfi = this._firstIndex;
	let fi = 0;

	fi = from - (from & 1); // This is done to maintain zebra background color.
	if (fi < 0) { fi = 0; }
	else if (fi > maxLen) { fi = maxLen; }

	let newLen = (to + (to & 1)) - fi;
	if(newLen + 2 === prevLen) { // To remove unnecessary shrinking during scrolling
		newLen = prevLen;
	}

	if(newLen < 0) { newLen = 0; }
	else if(newLen + fi > maxLen) { newLen = maxLen - fi; }

	this._firstIndex = fi;
	let ei = fi + newLen;

	// Re-position inner grid
	let px = this._layoutY.getLaneStart(fi);
	let gridElem = this._grid.getElement(); // GRID-347 Chrome 86 does not recognize top property of grand children
	if(gridElem) {
		gridElem.style.transform = px ? "translateY(" + px + "px)" : ""; // Replacement for this._grid.setTop(px);
		gridElem.style.top = "";
		this._reverter.start();
	}
	this._grid._onEnterDocument();
	let force = false; // Forcing flag to update entire range

	if (newLen !== prevLen) {
		this._grid.setRowCount(newLen);
		if(newLen > prevLen) { // New row is available. Fire event to render the cells.
			let e = {};
			e["prevRowIndex"] = fi + prevLen;
			e["rowIndex"] = fi + newLen;
			if(updateRange) {
				e["noDataUpdate"] = true; // Data update will be triggered after this, so we notify grid not to update
			}
			this._dispatch("rowAvailable", e);
		}
		force = true; // Force update entire range
	}
	if(updateRange) { // Find actual update range
		if(!force) {
			// TODO: Check where to clearSelectedRows() more properly
			this._grid.clearSelectedRows(); // Because row will be moving up and down, we need to clear the selection first
			let pei = pfi + prevLen;
			if(fi < pfi && pfi < ei && pei >= ei) {
				this.moveRowsToTop(ei - pfi);
				updateRange[0] = fi;
				updateRange[1] = pfi;
			} else if(pfi < fi && fi < pei && ei >= pei) {
				this.moveRowsToTop(fi - pfi);
				updateRange[0] = pei;
				updateRange[1] = ei;
			} else if(fi === pfi && ei === pei) {
				updateRange[0] = 0;
				updateRange[1] = 0;
			} else {
				force = true;
			}
		}

		if(force) {
			updateRange[0] = fi;
			updateRange[1] = ei;
		}
	}

	if(!this._layoutY.isHomogeneous()) { // Row heights need to be updated on each scrolling
		this._grid._updateRowHeights(this._layoutY, fi); // TODO: Set height only for the actual update range
	}

	this._updateRowSelection();
};
/** @inheritDoc */
VirtualizedLayoutGrid.prototype.moveRowsToTop = function (numRows) {
	this._grid.moveRowsToTop(numRows);
};

/** @inheritDoc */
VirtualizedLayoutGrid.prototype.getFirstIndexInView = function () {
	return this._firstIndex;
};

/** @inheritDoc */
VirtualizedLayoutGrid.prototype.getLastIndexInView = function () {
	let li = this._firstIndex + this._getInnerRowCount() - 1;
	let core = this._grid._getContext();
	let dv = core.getDataSource();
	let actualLastIndex = dv ? dv.getRowCount() - 1 : -1;
	if (li > actualLastIndex) { //Avoid incorrect index because the delay of row count synchronization after filtering
		return actualLastIndex;
	}
	return li;
};

/** @inheritDoc */
VirtualizedLayoutGrid.prototype.isInCurrentView = function(index){
	return (this._firstIndex <= index) && (index < this._firstIndex + this._getInnerRowCount());
};

/** @inheritDoc */
VirtualizedLayoutGrid.prototype.getFrozenLayout = function () {
	return this._grid.getFrozenLayout();
};
/** @inheritDoc */
VirtualizedLayoutGrid.prototype.setFrozenLayout = function (bool) {
	this._grid.setFrozenLayout(bool);
};

/** @inheritDoc */
VirtualizedLayoutGrid.prototype.getColumn = function (index) {
	return this._grid.getColumn(index);
};

/** @inheritDoc */
VirtualizedLayoutGrid.prototype.insertColumn = function (index, opt_json) {
	return this._grid.insertColumn(index, opt_json);
};

/** @inheritDoc */
VirtualizedLayoutGrid.prototype.removeColumnAt = function (index) {
	return this._grid.removeColumnAt(index);
};

/** @inheritDoc */
VirtualizedLayoutGrid.prototype.moveColumn = function (from, to) {
	this._grid.moveColumn(from, to);
};

/** @inheritDoc */
VirtualizedLayoutGrid.prototype.enableRowVirtualization = function (bool) { };

/** @inheritDoc */
VirtualizedLayoutGrid.prototype.enableColumnVirtualization = function (bool) {
	this._grid.enableColumnVirtualization(bool);
};

/** @inheritDoc */
VirtualizedLayoutGrid.prototype.getColumnIndex = function (colElem) {
	return this._grid.getColumnIndex(colElem);
};
/** @inheritDoc */
VirtualizedLayoutGrid.prototype.getCellIndex = function (colIndex, cellElem) {
	let at = this._grid.getCellIndex(colIndex, cellElem);
	if(at < 0) {
		return -1;
	}
	return at + this._firstIndex;
};

/** @inheritDoc */
VirtualizedLayoutGrid.prototype.getIndex = function () {
	return this._index;
};
/** @inheritDoc */
VirtualizedLayoutGrid.prototype.setIndex = function (index) {
	this._index = index;
};

/** @inheritDoc */
VirtualizedLayoutGrid.prototype.setRowOffset = function(val){
	this._rowOffset = val;
};

/** @inheritDoc */
VirtualizedLayoutGrid.prototype.getRowOffset = function(){
	return this._rowOffset;
};

/** @public
* @ignore
*/
VirtualizedLayoutGrid.prototype.updateLayout = function () {
	this._grid.updateLayout();
};

/** {@link ILayoutGrid#stretchCell}
* @inheritDoc */
VirtualizedLayoutGrid.prototype.stretchCell = function (cellRef, rowIndex, opt_stretching, onlyToTheRight) {
	let stretchedCell = this._grid.stretchCell(
		cellRef,
		(rowIndex != null) ? rowIndex - this._firstIndex : null,
		opt_stretching,
		onlyToTheRight
	);

	if (stretchedCell){
		let cellElem = stretchedCell.getElement();
		if(rowIndex === 0) {
			cellElem.classList.add("top");
		} else {
			cellElem.classList.remove("top");
		}
	}
	return stretchedCell;
};

/** {@link ILayoutGrid#unstretchCell}
* @inheritDoc */
VirtualizedLayoutGrid.prototype.unstretchCell = function (cellRef, rowIndex) {
	return this._grid.unstretchCell(
		cellRef,
		(rowIndex != null) ? rowIndex - this._firstIndex : null
	);
};
/** {@link ILayoutGrid#unstretchAllCells}
* @inheritDoc */
VirtualizedLayoutGrid.prototype.unstretchAllCells = function () {
	this._grid.unstretchAllCells();
};

/** {@link ILayoutGrid#getStretchedCell}
* @inheritDoc */
VirtualizedLayoutGrid.prototype.getStretchedCell = function (cellRef, rowIndex) {
	return this._grid.getStretchedCell(
		cellRef,
		(rowIndex != null) ? rowIndex - this._firstIndex : null
	);
};
/** {@link ILayoutGrid#getStretchedCells}
* @inheritDoc */
VirtualizedLayoutGrid.prototype.getStretchedCells = function () {
	return this._grid.getStretchedCells();
};
/** {@link ILayoutGrid#updateStretchedCells}
* @inheritDoc */
VirtualizedLayoutGrid.prototype.updateStretchedCells = function () {
	return this._grid.updateStretchedCells();
};

/** {@link ILayoutGrid#getColumnHost}
* @inheritDoc */
VirtualizedLayoutGrid.prototype.getColumnHost = function () {
	return this._grid.getElement();
};
/** {@link ILayoutGrid#setHidingMethod}
* @inheritDoc */
VirtualizedLayoutGrid.prototype.setHidingMethod = function (method) {
	this._grid.setHidingMethod(method);
};
/** {@link ILayoutGrid#getHidingMethod}
* @inheritDoc */
VirtualizedLayoutGrid.prototype.getHidingMethod = function () {
	return this._grid.getHidingMethod();
};
/** {@link ILayoutGrid#setColumnVisibility}
* @inheritDoc */
VirtualizedLayoutGrid.prototype.setColumnVisibility = function (colIndex, bool, flag) {
	return this._grid.setColumnVisibility(colIndex, bool, flag);
};
/** {@link ILayoutGrid#setColumnVisibility}
* @inheritDoc */
VirtualizedLayoutGrid.prototype.getRowLayout = function () { return this._layoutY; };

/** {@link ILayoutGrid#reserveRightSpace}
* @inheritDoc */
VirtualizedLayoutGrid.prototype.reserveRightSpace = function (size) {
	return this._grid.reserveRightSpace(size);
};
/** {@link ILayoutGrid#reserveRightSpace}
* @inheritDoc */
VirtualizedLayoutGrid.prototype.getReservedSpace = function () {
	return this._grid.getReservedSpace();
};
/** {@link ILayoutGrid#getComputedStyle}
* @inheritDoc */
VirtualizedLayoutGrid.prototype.getComputedStyle = function () {
	return this._grid.getComputedStyle();
};
/** {@link ILayoutGrid#isColumnActive}
* @inheritDoc
*/
VirtualizedLayoutGrid.prototype.isColumnActive = function (colIndex) {
	return this._grid.isColumnActive(colIndex);
};
/** {@link ILayoutGrid#setRowAttributes}
* @inheritDoc
*/
VirtualizedLayoutGrid.prototype.setRowAttributes = function (rowIndex, func) {
	return this._grid.setRowAttributes(rowIndex, func);
};
/** @public
* @param {number} rowIndex
* @return {Object}
*/
VirtualizedLayoutGrid.prototype.getContextRow = function (rowIndex) {
	return this._grid.getContextRow(rowIndex - this._firstIndex);
};

/** @public
* @param {number} colIndex
* @param {boolean=} selected
*/
VirtualizedLayoutGrid.prototype.selectColumn = function (colIndex, selected) {
	this._grid.selectColumn(colIndex, selected);
};
/** @public
* @param {number} colIndex
* @return {boolean}
*/
VirtualizedLayoutGrid.prototype.isSelectedColumn = function (colIndex) {
	return this._grid.isSelectedColumn(colIndex);
};
/** @public
* @param {number} colIndex
* @param {number} rowIndex
* @param {boolean=} selected
*/
VirtualizedLayoutGrid.prototype.selectCell = function (colIndex, rowIndex, selected) {
	this._grid.selectCell(colIndex, rowIndex - this._firstIndex, selected);
};
/** @public
* @param {number} colIndex
* @param {number} rowIndex
* @param {number} width
* @param {number} height
*/
VirtualizedLayoutGrid.prototype.setCellBounds = function (colIndex, rowIndex, width, height) {
	let boundLayer = this._initBoundLayer();
	let cellBoundPainter = this._cellBoundPainter;
	if(!cellBoundPainter) {
		cellBoundPainter = this._cellBoundPainter = new util_CellBoundPainter({
			boundLayer: boundLayer,
			layoutX: this.getHorizontalLayout(),
			layoutY: this._layoutY,
			hscrollbar: this._hscrollbar,
			calculateColumnBounds: this.calculateColumnBounds.bind(this)
		});
	}

	let colCount = this.getColumnCount();
	cellBoundPainter.setCellBounds(colIndex, rowIndex, width, height, colCount);
};
/** @public
* @ignore
* @return {!TrackLayout}
*/
VirtualizedLayoutGrid.prototype.getHorizontalLayout = function () {
	return this._grid.getHorizontalLayout();
};
/** @public
* @ignore
* @param {number} lftIdx
* @param {number} rgtIdx Inclusive index
* @param {!Array.<number>} outPositions
* @param {!Array.<boolean>} outNoBorders
*/
VirtualizedLayoutGrid.prototype.calculateColumnBounds = function (lftIdx, rgtIdx, outPositions, outNoBorders) {
	this._grid.calculateColumnBounds(lftIdx, rgtIdx, outPositions, outNoBorders);
};
/** @public
* @ignore
* @param {!Array.<Array>} posAry Left and right bound positions in pixel
* @param {!Array.<Array>} noBorderAry Boolean values indicating existence of left and right CSS borders
* @param {number=} topPx Top position of bound
*/
VirtualizedLayoutGrid.prototype.updateColumnBounds = function (posAry, noBorderAry, topPx) {
	this._grid.updateColumnBounds(posAry, noBorderAry, topPx);
	this._updateRowBounds();
};
/** @public
* @ignore
*/
VirtualizedLayoutGrid.prototype.updateColumnSeparators = function () {
	this._grid.updateColumnSeparators();
};
/** @private
* @return {Element}
*/
VirtualizedLayoutGrid.prototype._initBoundLayer = function () {
	let boundLayer = this._boundLayer;
	if(!boundLayer) {
		boundLayer = this._boundLayer = document.createElement("div");
		boundLayer.className = "cover-layer";
		this._element.appendChild(boundLayer);
	}
	return boundLayer;
};
/** @private
*/
VirtualizedLayoutGrid.prototype._requestUpdatingRowBounds = function () {
	if(!this._rowBoundTimer) {
		this._rowBoundTimer = setTimeout(this._updateRowBounds, 10);
	}
};
/** @private
*/
VirtualizedLayoutGrid.prototype._updateRowBounds = function () {
	this._rowBoundTimer = 0;

	let cellBoundPainter = this._cellBoundPainter;
	if(cellBoundPainter) {
		cellBoundPainter.updateCellBounds();
	}

	if(!this._rowSelDirty) {
		return;
	}
	let rbs = this._rowBounds;
	let rbc = this._rowBoundCache;
	if(!rbs) {
		rbs = this._rowBounds = [];
	}
	if(!rbc) {
		rbc = this._rowBoundCache = [];
	}

	let selList = this._selectionList;
	let rowCount = this._layoutY.getLaneCount();
	selList.deselectFrom(rowCount); // TODO: move this to setRowCount

	let selRanges = selList.getConnectedRanges();
	let rangeCount = selRanges.length;
	let i;
	let pn = null; // parentNode
	let rowBound = null;

	// Remove unused bounds from document
	let activeCount = rbs.length;
	for(i = rangeCount; i < activeCount; ++i) {
		rowBound = rbs[i];
		pn = rowBound.parentNode;
		if(pn) {
			pn.removeChild(rowBound);
		}
	}
	rbs.length = activeCount = rangeCount;

	if(!rangeCount) {
		let selCount = selList.getSelectionCount();
		if(!selCount) {
			this._rowSelDirty = false;
		}
		return;
	}

	// Prepare shared parameters
	let scrollLeft = 0;
	let pinnedLftCount = 0;
	let pinnedRgtCount = 0;
	let endOfScroll = false;
	if(this._hscrollbar) {
		scrollLeft = this._hscrollbar.getScrollLeft();
		pinnedLftCount = this._hscrollbar.getPinnedLeftColumnCount();
		pinnedRgtCount = this._hscrollbar.getPinnedRightColumnCount();
		endOfScroll = this._hscrollbar.isEndOfHorizontalScroll();
	}
	let noLeftBound = !pinnedLftCount && scrollLeft > 0;
	let noRightBound = !pinnedRgtCount && !endOfScroll;
	let boundWidth = this._grid._getViewSize();

	// Create row bound elements based on number of selection ranges
	for(i = 0; i < rangeCount; ++i) {
		let pair = selRanges[i];
		let topIdx = pair[0];
		let btmIdx = pair[1]; // Inclusive
		let topPx = this._layoutY.getLaneStart(topIdx);
		let btmPx = this._layoutY.getLaneEnd(btmIdx);

		rowBound = rbc[i];
		if(!rowBound) {
			rowBound = rbc[i] = document.createElement("div");
			rowBound.className = "selection-bound";
		}

		rowBound.style.top = topPx + "px";
		rowBound.style.height = (btmPx - topPx) + "px";
		rowBound.style.width = boundWidth + "px";

		rowBound.classList.toggle("no-left-bound", noLeftBound);
		rowBound.classList.toggle("no-right-bound", noRightBound);

		if(this._boundLayer) {
			if(!rbs[i]) {
				rbs[i] = rowBound;
				this._boundLayer.appendChild(rowBound);
			}
		}
	}
};
//#endregion public methods

//#region ====== Private methods ======//
/** @private
* @return {number}
*/
VirtualizedLayoutGrid.prototype._getInnerRowCount = function () {
	return this._grid.getRowCount();
};

/** @private */
VirtualizedLayoutGrid.prototype._updateRowSelection = function () {
	this._grid.clearSelectedRows();
	let sl = this._selectionList;
	let count = sl.getSelectionCount();
	if(count <= 0) { return; }

	let r;
	let fi = this._firstIndex;
	let ei = fi + this._getInnerRowCount();
	if(count === 1) {
		r = sl.getFirstSelectedIndex();
		if (fi <= r && r < ei) {
			this._grid.setSelectedRow(r - fi, true);
		}
	} else {
		if(count >= ei - fi) {
			for(r = fi; r < ei; ++r) {
				if(sl.getSelection(r)) {
					this._grid.setSelectedRow(r - fi, true);
				}
			}
		} else {
			let selections = sl.getAllSelections();
			for(let i = 0; i < count; ++i){
				r = selections[i];
				if (fi <= r && r < ei) {
					this._grid.setSelectedRow(r - fi, true);
				}
			}
		}
	}
};

/** @private
 * @param {number} from
 * @param {number} to
 * @return {boolean}
 */
VirtualizedLayoutGrid.prototype._dispatchRowHeightChangedEvent = function (from, to) {
	if (from >= to) { return false; }

	let e = {};
	e["rowIndex"] = from;
	e["changeLength"] = to - from;
	this._dispatch("rowHeightChanged", e);
	return true;
};
/** @private
 */
VirtualizedLayoutGrid.prototype._onRestingPosition = function () {
	let gridElem = this._grid.getElement();
	if(gridElem) {
		let px = this._layoutY.getLaneStart(this._firstIndex);
		gridElem.style.transform = "";
		gridElem.style.top = px ? px + "px" : ""; // WORKAROUND: GRID-402
	}
};
/** @public
* @ignore
*/
VirtualizedLayoutGrid.prototype._updateRightSpaceStyle = function() {
	this._grid._updateRightSpaceStyle();
};
/** @public
* @ignore
*/
VirtualizedLayoutGrid.prototype._updateLayers = function() {
	this._grid._updateLayers();
};
/** @public
* @ignore
* @param {*} ctx
*/
VirtualizedLayoutGrid.prototype._setContext = function(ctx) {
	this._grid._setContext(ctx);
};

/** @public
* @ignore
* @return {*} ctx
*/
VirtualizedLayoutGrid.prototype._getContext = function () {
	return this._grid._getContext();
};
//#endregion

VirtualizedLayoutGrid._proto = VirtualizedLayoutGrid.prototype;

/* harmony default export */ const grid_VirtualizedLayoutGrid = (VirtualizedLayoutGrid);


;// ./node_modules/@grid/core/es6/grid/Core.js

 // eslint-disable-line


 // eslint-disable-line












//#region Events
/** @event Core#sectionAdded */
/** @event Core#columnAdded */
/** @event Core#columnRemoved */
/** @event Core#columnVisibilityChanged */
/** @event Core#columnAlignmentChanged */
/** @event Core#widthChanged */
/** @event Core#dataSourceChanged */
/** @event Core#rowExpansionBinding
* @description Trigger after column data binding. Fires for all rows within the view.
* @property {Object.<string, *>} originalRowData
* @property {string} originalRowId The row that rowExpansion belongs to
* @property {boolean} rowExpansion
* @property {string} rowId
* @property {number} rowIndex
*/
/** @event Core#beforeContentBinding */
/** @event Core#postSectionDataBinding */
/** @event Core#firstRendered */
/** @event Core#afterContentBinding */
/** @event Core#rowHighlighted */

/** @event Core#rowAdded
* @ignore
*/
/** @event Core#postDataSourceChanged
* @ignore
*/
/** @event Core#preSectionRender
* @ignore
*/
/** @event Core#preSectionDataBinding
* @ignore
*/
/** @event Core#preForcedUpdate
* @ignore
*/
/** @event Core#tabNavigation
* @ignore
*/
//#endregion Events

/** @private
* @param {number} a
* @param {number} b
* @return {number}
*/
let ascNumberSorter = function (a, b) {
	return a - b;
};
/** @private
* @return {!Element}
*/
let _createHiddenInput = function () {
	let hiddenInput = document.createElement("input");
	let styleObj = hiddenInput.style;
	styleObj.position = "absolute";
	styleObj.width = styleObj.height = styleObj.padding = styleObj.border = "0";
	hiddenInput.value = "0";
	hiddenInput.role = "grid";
	hiddenInput.readOnly = true;
	return hiddenInput;
};
/** @private
* @param {Object} e
* @return {boolean}
*/
let _isTabCommand = function (e) {
	if(e.keyCode === 9) {
		return !e.ctrlKey && !e.altKey && !e.metaKey;
	}
	return false;
};
/** @private
* @param {Element} elem
* @return {Element}
*/
let _getActiveElement = function (elem) {
	if(elem) {
		if(elem.getRootNode) {
			let rootNode = elem.getRootNode(); // Get uncomposed root node
			if(rootNode && rootNode !== elem) { // The root node could be the element itself, if it is not attached to the DOM tree
				return rootNode.activeElement || null;
			}
		} else { // Older browser does not support getRootNode
			return document.activeElement;
		}
	}
	return null;
};

/** @constructor
* @param {Element=} opt_initializer this can be either element id (string) or DOM element.
* @extends {ElementWrapper}
*/
let Core = function (opt_initializer) {
	if (!Core._sectionType) { // Initialize static variables
		Core._sectionType = {};
		Core._sectionType["title"] = true;
		Core._sectionType["header"] = true;
		Core._sectionType["content"] = true;
		Core._sectionType["footer"] = true;
	}

	let _t = this;

	// Initialize method binding
	_t.getColumnId = _t.getColumnId.bind(_t);
	_t.getColumnIndex = _t.getColumnIndex.bind(_t);

	_t._onMouseMove = _t._onMouseMove.bind(_t);
	_t._onRowHightlighted = _t._onRowHightlighted.bind(_t);
	_t._onGridClicked = _t._onGridClicked.bind(_t);
	_t._onKeyDown = _t._onKeyDown.bind(_t);

	_t._onWindowResize = _t._onWindowResize.bind(_t);
	_t._onSectionDataChanged = _t._onSectionDataChanged.bind(_t);
	_t._onRowCountChanged = _t._onRowCountChanged.bind(_t);
	_t._onRowHeightChanged = _t._onRowHeightChanged.bind(_t);
	_t._onRowAvailable = _t._onRowAvailable.bind(_t);

	_t._onVScroll = _t._onVScroll.bind(_t);
	_t._onHScroll = _t._onHScroll.bind(_t);
	_t._onSyncVScroll = _t._onSyncVScroll.bind(_t);
	_t._onSyncHScroll = _t._onSyncHScroll.bind(_t);
	_t.updateLayout = _t.updateLayout.bind(_t);
	_t._onRowRefresh = _t._onRowRefresh.bind(_t);
	_t._onVScrollEnabled = _t._onVScrollEnabled.bind(_t);
	_t._onVScrollDisabled = _t._onVScrollDisabled.bind(_t);

	_t._onRowInViewChanged =  _t._onRowInViewChanged.bind(_t);
	_t._onColInViewChanged =  _t._onColInViewChanged.bind(_t);

	_t._updateVScrollbar = _t._updateVScrollbar.bind(_t);
	_t.updateColumnBounds = _t.updateColumnBounds.bind(_t);
	_t._dispatchColumnPositionChanged = _t._dispatchColumnPositionChanged.bind(_t);
	_t._dispatchRowPositionChanged = _t._dispatchRowPositionChanged.bind(_t);
	_t._requestScrollbarUpdate = _t._requestScrollbarUpdate.bind(_t);

	// Text nodes are unintentionally getting in the tag.
	if(opt_initializer) { // Any node other than element node is not allowed within the tag.
		for(let i = opt_initializer.childNodes.length; --i >= 0;) {
			if(!util.isElement(opt_initializer.childNodes[i])) {
				opt_initializer.removeChild(opt_initializer.childNodes[i]);
			}
		}
	}

	// Initialize members
	_t._element = opt_initializer ? opt_initializer : document.createElement("div");
	_t._element.classList.add("tr-grid");

	_t._layoutX = new util_TrackLayout();
	_t._layoutY = new util_TrackLayout();
	_t._layoutX.setDefaultLaneSize(100); //Default column width
	_t._layoutY.setDefaultLaneSize(32); //Default row height

	_t._settings = [];
	_t._plugins = {};
	_t._columnDefs = [];
	_t._sectionStarts = [];
	_t._rowHeightException = "title";

	_t._rowVirtualizer = new util_Virtualizer(_t._layoutY);
	_t._rowVirtualizer.deactivate();
	_t._rowVirtualizer.setViewSize(360); // Minimum view size

	_t._colVirtualizer = new util_Virtualizer(_t._layoutX);
	_t._colVirtualizer.enable(false);
	_t._colVirtualizer.deactivate();
	_t._colVirtualizer.setViewSize(400); // Minimum view size

	// Initialize vertical scrollbar
	_t._vscrollbar = new components_VScrollbar();
	_t._vscrollbar.disable();
	_t._vscrollbar.setParent(_t.getParent() || _t.getElement());

	_t._vscrollbar.listen("scroll", _t._onVScroll);
	_t._vscrollbar.listen("layoutChanged", _t._onVScroll);
	_t._vscrollbar.listen("activated", _t.updateLayout); // TODO: remove column virtualization logic
	_t._vscrollbar.listen("deactivated", _t.updateLayout); // TODO: remove column virtualization logic
	_t._vscrollbar.listen("thicknessChanged", _t.updateLayout); // TODO: remove column virtualization logic
	_t._vscrollbar.listen("enabled", _t._onVScrollEnabled);
	_t._vscrollbar.listen("disabled", _t._onVScrollDisabled);

	// Initialize horizontal scrollbars
	_t._hscrollbar = new components_HScrollbar();
	_t._hscrollbar.disable();
	_t._hscrollbar.setParent(_t.getParent() || _t.getElement());

	_t._hscrollbar.listen("scroll", _t._onHScroll);
	_t._hscrollbar.listen("layoutChanged", _t._onHScroll);
	_t._hscrollbar.listen("activated", _t.updateLayout);
	_t._hscrollbar.listen("deactivated", _t.updateLayout);

	// cross-reference scrollbars
	_t._hscrollbar.setOtherScrollbar(_t._vscrollbar);
	_t._vscrollbar.setOtherScrollbar(_t._hscrollbar);


	_t._element.addEventListener("keydown", _t._onKeyDown);
	if (util.isMobile || util.isTouchDevice) {
		_t._element.addEventListener("touchmove", _t._onMouseMove, false);
	} else {
		_t._element.addEventListener("mousemove", _t._onMouseMove, false);
	}

	if(util.isSafari){
		_t._element.addEventListener("click", _t._onGridClicked);
	}

	window.addEventListener("resize", _t._onWindowResize, false); // Should be unlistened after destroyed
	_t._rowVirtualizer.listen("indexChanged", _t._onRowInViewChanged);
	_t._colVirtualizer.listen("indexChanged", _t._onColInViewChanged);
	_t._rowHeightConflator = new util_Conflator(_t._onRowHeightChanged, 50);
	_t._vScrollbarConflator = new util_Conflator(_t._updateVScrollbar, 200);
	_t._columnBoundConflator = new util_Conflator(_t.updateColumnBounds, 10);
	_t._columnPositionConflator = new util_Conflator(_t._dispatchColumnPositionChanged, 10);
	_t._rowPositionConflator = new util_Conflator(_t._dispatchRowPositionChanged, 10);

	_t._firstHiddenInput = _createHiddenInput();
	_t._firstHiddenInput.className = "first-input";
	_t._lastHiddenInput = _createHiddenInput();
	_t._lastHiddenInput.className = "last-input";
	_t._element.insertBefore(_t._firstHiddenInput, _t._element.firstChild);
	_t._element.appendChild(_t._lastHiddenInput);

	// Initialize events for external users
	_t._addEvents(
		"sectionAdded",
		"columnAdded",
		"columnRemoved",
		"columnMoved",
		"columnVisibilityChanged",
		"columnAlignmentChanged",
		"widthChanged",
		"dataSourceChanged",
		"postDataSourceChanged",
		"preSectionRender",
		"postSectionRender",
		"beforeContentBinding",
		"preSectionDataBinding",
		"postSectionDataBinding",
		"firstRendered",
		"afterContentBinding",
		"rowExpansionBinding",
		"rowHighlighted",
		"preForcedUpdate",
		"rowAdded",
		"rowRemoved",
		"columnPositionChanged",
		"rowPositionChanged",
		"beforeColumnBoundUpdate",
		"beforeBatchOperation",
		"afterBatchOperation",
		"pinningChanged",
		"tabNavigation"
	);

	// For debugging in advanced optimization mode
	let map = Core["map"];
	if(!map) {
		map = {};
		Core["map"] = map;
	}

	_t._element["_control"] = _t;
	let id = _t._element.id || _t._element.name;
	if(!id || map[id]) {
		id = "_grid" + Core._runningGridId;
	}
	_t["_id"] = id;
	map[id] = _t;
	Core._runningGridId++;

	// Ensure all affected plugins are loaded prior zoom plugin
	// use as entity to trigger updateLayout once zoom is changed
	Object.defineProperty(_t, "zoomFactor", {
		set: function(factor) {
			if(!factor || factor < 0) {
				return;  // Cannot have negative factor or factor of zero
			}
			if(this._zoomFactor !== factor) {
				this._zoomFactor = factor;
				// Panes and view port are affected and need to be updated
				_t._updateLayout(); // Super slow. Do not execute this so often
			}
		},
		get: function() {
			return this._zoomFactor;
		}
	});
};
es6_Ext.inherits(Core, components_ElementWrapper);

/** @typedef {number|string|ILayoutGrid|SectionSettings} Core~SectionReference
* @description A section in core grid can be refered by the following object <br>
* `{@link ILayoutGrid}` : A pointer to Section instance <br>
* `{@link SectionSettings}` : A pointer to SectionSettings instance <br>
* `number` : Section index <br>
* `string` : Section type (e.g. "content", "title", "header", "footer"), or Section id (element id)
*/
Core.SectionReference;

/** @typedef {Object} Core~MouseInfo
* @description Object returned from Core#getRelativePosition method
* @property {boolean} hit - Indicates whether the given position is on the grid
* @property {number} x - Horizontal coordinate that is relative to the top-left of the grid (Zero means left-most)
* @property {number} y - Vertical coordinate that is relative to the top-left of the grid (Zero means top-most)
* @property {number} colIndex
* @property {number} sectionIndex
* @property {number} rowIndex
* @property {string} sectionType
* @property {SectionSettings} sectionSettings
* @property {ILayoutGrid} section
* @property {Element} cellElement
* @property {DataView} dataSource
*/
Core.MouseInfo;

/** @typedef {Object} Core~BatchInfo
* @private
* @property {string=} reset //set columns
* @property {string=} insertion //add cols
* @property {string=} removal //remove cols
* @property {string=} moving //reorder
*/
Core.BatchInfo;

/** @typedef {Core.MouseInfo|ElementWrapper|Element} Core~CellReference
* @description A section in core grid can be refered by the following object <br>
* `{Core.MouseInfo}` : Object with valid x, y coordinates and section index <br>
* `{@link ElementWrapper}` : a pointer to ElementWrapper instance <br>
* `Element` : cell element
*/
Core.CellReference;

/** @typedef {Object} Core~ColumnOptions
* @description Column options that can be provided to {@link Core#insertColumn}
* @property {number=} width Column width
* @property {number=} minWidth Minimum column width
* @property {number=} minSize Alias to `minWidth`
* @property {number=} size Alias to `width`
* @property {boolean=} visible Column visibility
* @property {boolean=} scalable Column scalability
* @property {Object.<string, string>=} styles Key/value pair object specifying CSS styles
* @property {Object.<string, string>=} classes Key/value pair object specifying CSS classes
* @property {string=} dataColumnName Data column for sorting
* @property {Function=} renderingHandler - Deprecated
* @property {Function=} dataBindingHandler Column binding function
* @property {string=} columnData For attaching context (user data)
* @property {boolean=} stationary=false If enabled, the column order cannot be changed (i.e., this column and any column to its left cannot be moved)
* @property {boolean=} leftPinned=false If enabled, the column will not be part of the scrollable area and is pinned to the left side
* @property {boolean=} rightPinned=false If enabled, the column will not be part of the scrollable area and is pinned to the right side
* @property {TrackLayout=} layout - Deprecated
*/
Core.ColumnOptions;

/** For runtime debugging!
* @type {number}
* @private
*/
Core._runningGridId = 0; // Static variable
/** For identifying column
* @type {number}
* @private
*/
Core._runningColumnId = 0; // Static variable

/** @type {!Object.<string, boolean>}
* @private
* @const
*/
Core._sectionType; // Static variable

/** @type {!Object.<string, *>}
* @private
*/
Core.prototype._plugins;
/** @type {DataView}
* @private
*/
Core.prototype._dataSource = null;

/** @type {!TrackLayout}
* @private
*/
Core.prototype._layoutX;
/** @type {!TrackLayout}
* @private
*/
Core.prototype._layoutY;
/** @type {!VScrollbar}
* @private
*/
Core.prototype._vscrollbar; // vertical scrollbar is non-nullable
/** @type {!HScrollbar}
* @private
*/
Core.prototype._hscrollbar; // horizontal scrollbar is non-nullable


/** reference of top frozen section. It's a temporaly use between function _freezeTopSectionAt and _onSectionCountChanged
* @type {string}
* @private
*/
Core.prototype._tempFrozenSectionRef = "";
/** first index of section that vertical scroll placed, negative value mean no vertical scroll
* @type {number}
* @private
*/
Core.prototype._startVScrollbarIndex = -1;
/** @type {boolean}
* @private
*/
Core.prototype._hScrollbarEnabled = true;
/** @type {number}
* @private
*/
Core.prototype._pinnedLeftColumnCount = 0; // Cached value, that should always the same as the actual value in the UI
/** @type {number}
* @private
*/
Core.prototype._pinnedRightColumnCount = 0; // Cached value, that should always the same as the actual value in the UI
/** number of footer that being freeze right now
* @type {number}
* @private
*/
Core.prototype._frozenFooterCount = 0;
/** If true, the number of frozen top sections will not be changed automatically
* @type {boolean}
* @private
*/
Core.prototype._fixFrozenTopSections = false;
/** If true, the number of frozen bottom sections will not be changed automatically
* @type {boolean}
* @private
*/
Core.prototype._fixFrozenBottomSection = false;
/** cumulative row indices
* @type {Array.<number>}
* @private
*/
Core.prototype._sectionStarts;

/** @type {!Array.<SectionSettings>}
* @private
*/
Core.prototype._settings;
/** @type {SectionSettings}
* @private
*/
Core.prototype._firstSettings;
/** @type {SectionSettings}
* @private
*/
Core.prototype._lastSettings;

/** @type {boolean}
* @private
*/
Core.prototype._fitContentWidth = false;

/** @type {!Array.<Object.<string, *>>}
* @private
*/
Core.prototype._columnDefs;

/** @type {!Virtualizer}
* @private
*/
Core.prototype._rowVirtualizer;
/** @type {!Virtualizer}
* @private
*/
Core.prototype._colVirtualizer;

/** when set datasource without loading DataSourceTreePlugin, this datasource is used for every section.
* @type {boolean}
* @private
*/
Core.prototype._autoSetDataSource = true;

/** @type {Function}
* @private
*/
Core.prototype._renderingHandler = null;
/** @type {Function}
* @private
*/
Core.prototype._dataBindingHandler = null;

/** @type {boolean}
* @private
*/
Core.prototype._frozenLayout = false;
/** @type {boolean}
* @private
*/
Core.prototype._noBinding = false;
/** @type {boolean}
* @private
*/
Core.prototype._rowHeightSync = true;
/** @type {boolean}
* @private
*/
Core.prototype._rowHighlighting = false;
/** @type {boolean}
* @private
*/
Core.prototype._dispatchingDataChanged = false;

/** @type {string}
* @private
*/
Core.prototype._rowHeightException = "";
/** @type {number}
* @private
*/
Core.prototype._rowScrollingStep = 0;
/** @type {number}
* @private
*/
Core.prototype._zoomFactor = 1;
/** @type {number}
* @private
*/
Core.prototype._reservedSpace = 0;

/** @type {string}
* @private
*/
Core.prototype._hidingMethod = "";
/** @type {Conflator}
* @private
*/
Core.prototype._rowHeightConflator = null;
/** @type {Conflator}
* @private
*/
Core.prototype._vScrollbarConflator = null;
/** @type {Conflator}
* @private
*/
Core.prototype._columnBoundConflator = null;
/** @type {number}
* @private
*/
Core.prototype._rowRefreshTimer = 0;
/** @type {boolean}
* @private
*/
Core.prototype._layoutUpdating = false;
/** A hidden input that allows grid to receive keyboard input and focus
* @type {!Element}
* @private
*/
Core.prototype._firstHiddenInput;
/** A hidden input that allows grid to receive keyboard input and focus
* @type {!Element}
* @private
*/
Core.prototype._lastHiddenInput;
/** @type {number}
* @private
*/
Core.prototype._bottomPadding = 6;
/** @type {ILayoutGrid}
* @private
*/
Core.prototype._bottomPaddingSect = null;
/** @type {boolean}
* @private
*/
Core.prototype._preserveProportion = false;
/** @type {boolean}
* @private
*/
Core.prototype._preserveGridSize = false;
/** @type {number}
* @private
*/
Core.prototype._rowHeightTimerId = 0;
/** @type {GroupDefinitions}
* @private
*/
Core.prototype._groupDefs = null;
/** @type {BatchInfo}
* @private
*/
Core.prototype._batches = null;
/** @type {boolean}
* @private
*/
Core.prototype._firstRendered = false;
/** @type {boolean}
* @private
*/
Core.prototype._hasPendingRowChange = false;
//#region Public Methods

/**
* @public
* @return {string}
*/
Core.getVersion = function () {
	return "5.2.3";
};
/** {@link ElementWrapper#dispose}
* @override
*/
Core.prototype.dispose = function () {
	let internalId = this["_id"];
	let map = Core["map"];
	if (map[internalId]) {
		map[internalId] = null;
	} else {
		console.log("WARNING: Core grid is disposed twice");
		return;
	}
	this._disposed = true;
	if(this._rowRefreshTimer) {
		clearTimeout(this._rowRefreshTimer);
		this._rowRefreshTimer = 0;
	}

	// Clear all events
	this.unlistenAll();
	window.removeEventListener("resize", this._onWindowResize, false);

	for (let i = this._columnDefs.length; --i >= 0; ) {
		let colDef = this._getColumnDef(i);
		colDef["renderingHandler"] = colDef["dataBindingHandler"] = null;
	}
	for (let key in this._plugins) {
		let plugin = this._plugins[key];
		if (plugin["unload"]) {
			plugin["unload"](this);
		}
	}

	// Clear all Elements
	this.removeAllSections(); // Remove all Sections and LayoutGrids
	let mainScrolbar = this._vscrollbar._mainScrollbar;
	if(mainScrolbar) {
		mainScrolbar.unlisten("scroll", this._onSyncVScroll);
		this._vscrollbar._mainScrollbar = null;
	}
	mainScrolbar = this._hscrollbar._mainScrollbar;
	if(mainScrolbar) {
		mainScrolbar.unlisten("scroll", this._onSyncHScroll);
		this._hscrollbar._mainScrollbar = null;
	}

	this._vscrollbar.dispose();
	this._hscrollbar.dispose();
	this._rowHeightConflator.dispose();
	this._vScrollbarConflator.dispose();
	this._columnBoundConflator.dispose();
	this._columnPositionConflator.dispose();
	this._rowPositionConflator.dispose();
	this._layoutX.dispose();
	this._layoutY.dispose();


	// Clean Top node
	let elem = this._element;
	if (elem) {
		if (elem["_control"]) {
			delete elem["_control"];
		}
		elem.removeChild(this._firstHiddenInput);
		elem.removeChild(this._lastHiddenInput);
	}
	this._dispose();

	this._columnDefs.length = 0;
	delete this._plugins;
};
/** Indicates if the object has been disposed or not
* @public
* @return {boolean}
*/
Core.prototype.isDisposed = function () {
	return this._disposed;
};

/** {@link ElementWrapper#setParent}
* @override
*/
Core.prototype.setParent = function (obj, opt_prepend) {
	this._setParent(obj, opt_prepend);
	this._onParentChange();
};
/** {@link ElementWrapper#insertBefore}
* @override
*/
Core.prototype.insertBefore = function (obj) {
	this._insertBefore(obj);
	this._onParentChange();
};
/** @private */
Core.prototype._onParentChange = function () {
	let newParent = this.getParent() || this.getElement();
	if(!this._vscrollbar._mainScrollbar) { // HACK
		this._vscrollbar.setParent(newParent);
	}
	this._hscrollbar.setParent(newParent);

	this._updateLayout();
};

/** @public
* @param {Object=} gridOptions
* @return {!Object}
*/
Core.prototype.getConfigObject = function (gridOptions) {
	let obj = gridOptions || {};

	let columnCount = this.getColumnCount();
	if (!obj["columns"]) {
		obj["columns"] = [];
	}

	for (let colIndex = 0; colIndex < columnCount; colIndex++) {
		let column = obj["columns"][colIndex];
		if (!column) {
			column = obj["columns"][colIndex] = {};
		}

		let columnDef = this._getColumnDef(colIndex);
		if(columnDef["userId"]) {
			column["id"] = columnDef["userId"];
		}
		if(columnDef["field"]) {
			column["field"] = columnDef["field"];
		}
		if (columnDef["styles"]) {
			column["styles"] = columnDef["styles"];
		}
		if (columnDef["stationary"]) {
			column["stationary"] = columnDef["stationary"];
		}
		if (columnDef["leftPinned"]) {
			column["leftPinned"] = columnDef["leftPinned"];
		}
		if (columnDef["rightPinned"]) {
			column["rightPinned"] = columnDef["rightPinned"];
		}
		if (columnDef["backgroundColor"]) {
			column["backgroundColor"] = columnDef["backgroundColor"];
		}
	}

	// It will be overwrite in rt-grid or atlas-blotter
	obj["rowVirtualization"] = this.isRowVirtualizationEnabled();
	obj["columnVirtualization"] = this.isColumnVirtualizationEnabled();
	obj["rowHighlighting"] = this._rowHighlighting;

	if(this._fixFrozenTopSections) {
		obj["topFreezingCount"] = this._startVScrollbarIndex >= 0 ? (this._startVScrollbarIndex + 1) : false;
	}
	if(this._fixFrozenBottomSection) {
		obj["bottomFreezingCount"] = this._frozenFooterCount;
	}


	if(this._rowScrollingStep) {
		obj["stepScroll"] = (this._rowScrollingStep === 1) ? true : this._rowScrollingStep;
	}

	let val = this._vscrollbar.getAutoHide();
	if(val !== true) {
		// TODO: check this._hscrollbar and this._vscrollbar can be difference config
		obj["autoHideScrollbar"] = val; // this._hscrollbar has the same settings
	}

	let wheelSpeed = this._vscrollbar.getMouseWheelSpeed();
	if(wheelSpeed) {
		obj["linearWheelScrolling"] = wheelSpeed;
	}

	if(this.hasClass("no-borders")) {
		obj["borders"] = false;
	}
	if(this.hasClass("no-gridlines")) {
		obj["gridlines"] = false;
	}
	if(this.hasClass("no-vertical-grid-line")) {
		obj["verticalLines"] = false;
	}
	if(this.hasClass("no-horizontal-grid-line")) {
		obj["horizontalLines"] = false;
	}
	if(this.hasClass("no-content-vertical-line")) {
		obj["contentVerticalLines"] = false;
	}
	if(this._bottomPaddingSect) {
		obj["contentBottomPadding"] = this._bottomPadding;
	}
	if(this._reservedSpace > 0) {
		obj["contentRightPadding"] = this._reservedSpace;
	}

	let conflationRate = this.getDataConflationRate();
	if (conflationRate) {
		obj["dataConflationRate"] = conflationRate;
	}

	for(let key in this._plugins) {
		let pluginInstance = this._plugins[key];
		if(typeof pluginInstance["getConfigObject"] === "function") {
			pluginInstance["getConfigObject"](obj);
		}
	}

	return obj;
};

/** {@link ElementWrapper#fillParentWidth}
* @override
*/
Core.prototype.fillParentWidth = function(pct) {
	if(this._fitContentWidth) { return false; }
	if(this._layoutX.isFixedSize()) { return false; }

	// uses getBoundingClientRect() instead of clientWidth to prevents
	// fraction error occured on Chrome (Windows 10) which make clientWidth
	// exceeds expected value by 1px and will cause incorrect scrollbar active frag
	let el = this._element;
	let px = (el.getBoundingClientRect().width | 0) - el.clientLeft;
	let changes = this._layoutX.setTrackSize(px - this._reservedSpace);
	return changes !== null;
};
/** {@link ElementWrapper#show}
* @override
*/
Core.prototype.show = function (opt_shown) {
	this._element.style.display = (opt_shown !== false) ? "" : "none";
	if (opt_shown !== false) {
		this.updateLayout();
	}
};
/** @public
* @return {number}
*/
Core.prototype.getContentWidth = function() {
	return this._layoutX.getTrackSize();
};

/** @public
* @return {number}
*/
Core.prototype.getSectionCount = function() {
	return this._settings.length;
};

/** @private */
Core.prototype._updateFrozenTopSections = function () {
	if (!this._fixFrozenTopSections) {
		let topCount = this._getTopSectionCount();
		this._freezeTopSectionAt(topCount - 1);
	}
};

/** @private */
Core.prototype._updateFrozenBottomSections = function () {
	if (!this._fixFrozenBottomSection) {
		let bottomCount = this._getBottomSectionCount();
		this._setFrozenFooter(bottomCount);
	}
};

/** @private
* @return {number}
*/
Core.prototype._getTopSectionCount = function () {
	let sectionSettings = this._settings;
	let length = sectionSettings.length;
	let count = 0;
	for (let i = 0; i < length; i++) {
		let type = sectionSettings[i].getType();
		if (type === "header" || type === "title") {
			count++;
		} else {
			break;
		}
	}
	return count;
};

/** @private
* @return {number}
*/
Core.prototype._getBottomSectionCount = function () {
	let sectionSettings = this._settings;
	let length = sectionSettings.length;
	let count = 0;
	for (let i = length - 1; i >= 0; i--) {
		let type = sectionSettings[i].getType();
		if (type === "footer") {
			count++;
		} else {
			break;
		}
	}
	return count;
};

/** @public
* @param {string=} opt_type
* @param {string=} opt_sectionName
* @return {ILayoutGrid|null}
*/
Core.prototype.addSection = function (opt_type, opt_sectionName) {
	return this.addSectionAt(this.getSectionCount(), opt_type, opt_sectionName);
};
/** @public
* @param {number} at
* @param {string=} opt_type
* @param {string=} opt_sectionName
* @return {ILayoutGrid|null}
* @fires Core#sectionAdded
*/
Core.prototype.addSectionAt = function (at, opt_type, opt_sectionName) {
	let sectionCount = this._settings.length;
	if (at < 0) { at = 0; }
	if (at > sectionCount) { at = sectionCount; }

	let sectType = this._toSectionType(opt_type);
	let isBottomPadding = (opt_sectionName === "bottomPadding");
	let sectionSettings  = this._newSection(
		isBottomPadding ? "footer" : sectType, // Always use footer as a setup for bottom padding section
		opt_sectionName
	);
	if(!sectionSettings){ return null; }

	let section = sectionSettings.getSection();
	if(isBottomPadding) {
		sectionSettings.setAutoSyncRowCount(false); // Important
		this._bottomPaddingSect = section;
	}

	if (at >= sectionCount) {
		this._putToLast(section);
		this._settings.push(sectionSettings);
	} else {
		this._settings.splice(at, 0, sectionSettings);
		section.insertBefore(this._settings[at + 1].getSection());
	}

	this._updateSectionIndices(at);
	if (this._autoSetDataSource) {
		// This could trigger data binding and column virtualization
		sectionSettings.setDataSource(this._dataSource);
	}

	sectType = sectionSettings.getType();
	if (sectType === "footer") {
		this._updateFrozenBottomSections();
	} else if (sectType === "header" || sectType === "title") {
		this._updateFrozenTopSections();
	}

	this._onColumnCountChanged(); // When a new section is added, new columns are also added
	this._onSectionCountChanged(true);

	if(this._colVirtualizer.isEnabled()) {
		let activations = this._getColActivationList();
		let colCount = this.getColumnCount();
		sectionSettings.activateColumns(activations, 0, colCount - 1);
	}

	if (this._hasListener("sectionAdded")) {
		let e = sectionSettings.extendEventArg();
		this._dispatch("sectionAdded", e);
	}

	return section;
};
/** @public
* @param {Core.SectionReference} sectionRef
* @return {ILayoutGrid}
*/
Core.prototype.removeSection = function(sectionRef) {
	let section = this.getSection(sectionRef);
	if(section !== null) {
		return this.removeSectionAt(section.getIndex());
	}
	return null;
};
/** @public
* @param {number} at
* @return {ILayoutGrid}
*/
Core.prototype.removeSectionAt = function (at) {
	let sectionCount = this._settings.length;
	if (at >= 0 && at < sectionCount) {
		let sectionSettings = this._settings[at];
		let removedIndex = sectionSettings.getIndex();
		let section = sectionSettings.getSection();
		let sectionType = sectionSettings.getType();

		section.setRowCount(0); // Row count changed
		this._settings.splice(at, 1);

		this._updateSectionIndices(removedIndex);

		if (sectionType === "footer") {
			this._updateFrozenBottomSections();
			this._onColumnCountChanged(); // Update footer count in hScrollbar
		} else if (sectionType === "header" || sectionType === "title") {
			this._updateFrozenTopSections();
		}

		this._onSectionCountChanged();  // Update vScrollbar

		sectionSettings.dispose();
		return section;
	}
	return null;
};
/** @public
* @return {ILayoutGrid}
*/
Core.prototype.removeLastSection = function() {
	return this.removeSectionAt(this._settings.length - 1);
};
/** @public */
Core.prototype.removeAllSections = function () {
	let len = this._settings.length;
	for (let i = 0; i < len; ++i) {
		let sectionSettings = this._settings[i];
		sectionSettings.dispose();
	}
	this._bottomPaddingSect = null;
	this._firstSettings = this._lastSettings = null;
	this._settings.length = 0;
	this._updateSectionIndices(0);
	this._onSectionCountChanged();
};
/** @public
* @param {boolean|number} paddingSize=6 Add special section at the bottom. If falsy value is specified, the section is removed.
*/
Core.prototype.setBottomPaddingSection = function (paddingSize) {
	let sect = this._bottomPaddingSect;
	if(paddingSize) {
		if(typeof paddingSize === "number") {
			this._bottomPadding = paddingSize;
		}
		if(!sect) {
			sect = this.addSection("content", "bottomPadding");
		}
		if(sect) {
			sect.setDefaultRowHeight(this._bottomPadding);
			if(!sect.getRowCount()) {
				sect.addRow(1);
			}
		}
	} else if(sect) {
		this.removeSection(sect);
	}
};

/** @private
* @return {boolean} Returns true if there is any change
*/
Core.prototype._updatePaddingBottomSect = function () {
	let bps = this._bottomPaddingSect;
	if(bps) { // Always move bottom padding sect to the last
		let sectCount = this._settings.length;
		let last = sectCount - 1;
		if(last > 0) {
			let lastSettings = this._settings[last];
			if(lastSettings.getSection() !== bps) {
				let bpss = null;
				for(let i = last; --i >= 0;) {
					let settings = this._settings[i];
					if(settings.getSection() === bps) {
						bpss = settings;
						break;
					}
				}
				if(i >= 0) {
					this._putToLast(bps);
					this._settings.splice(i, 1);
					this._settings.push(bpss);
					// Blend styling with previous one
					bpss.setType(lastSettings.getType() === "footer" ? "footer" : "content");
					return true;
				} else {
					this._bottomPaddingSect = null; // The settings and section must have been removed
				}
			} else {
				let prevSettings = this._settings[last - 1];
				if(prevSettings) {
					// Blend styling with previous one
					lastSettings.setType(prevSettings.getType() === "footer" ? "footer" : "content");
				}
			}
		}
	}
	return false;
};
/** @public
* @ignore
* @param {number} from INCLUSIVE
* @param {number} dest INCLUSIVE
*/
Core.prototype.moveSection = function (from, dest) {
	let sectionCount = this.getSectionCount();
	if (from < 0) { from = 0; }
	else if (from >= sectionCount) { return; }
	if (dest < 0) { dest = 0; }
	else if (dest >= sectionCount) { dest = sectionCount - 1; }
	if (from === dest) { return; }

	let fromSettings = this._settings[from];
	let destSettings = this._settings[dest];
	util.moveArrayItem(this._settings, from, dest);

	if (destSettings) {
		fromSettings.getSection().insertBefore(destSettings.getSection());
	} else {
		this._putToLast(fromSettings.getSection());
	}

	this._updateSectionIndices((from <= dest) ? from : dest);
	this._syncRowHeights();
	this._onSectionCountChanged();
	if (!this._frozenLayout) {
		this._rowVirtualizer.update(true); // Force firing of row activation
	}
};

/** @public
* @param {Core.SectionReference} sectionRef
* @return {ILayoutGrid}
*/
Core.prototype.getSection = function (sectionRef) {
	let settings = this.getSectionSettings(sectionRef);
	return (settings !== null) ? settings.getSection() : null;

};
/** @public
* @param {Core.SectionReference} sectionRef
* @return {SectionSettings}
*/
Core.prototype.getSectionSettings = function (sectionRef) {
	if(util.isString(sectionRef)) {
		if(Core._sectionType[/** @type{string} */(sectionRef)]) {
			return this._getNextSection(1, null, /** @type{string} */(sectionRef));
		}
		for(let i = this._settings.length; --i >= 0;) {
			if(this._settings[i].getId() === sectionRef) {
				return this._settings[i];
			}
		}
	} else {
		if(sectionRef != null) {
			let num = -1;
			if(sectionRef.getIndex != null) {
				num = /** @type{number} */(sectionRef.getIndex());
			} else if(util.isNumber(sectionRef)) {
				num = /** @type{number} */(sectionRef);
			}
			if(num >= 0 && num < this._settings.length) {
				return this._settings[num];
			}
		}
	}
	return null;
};

/** @public
* @param {string=} opt_type
* @return {!Array.<ILayoutGrid>}
*/
Core.prototype.getAllSections = function (opt_type) {
	if(opt_type == null) { return this._getAllSections(); }

	let list = this.getAllSectionSettings(opt_type);
	let ary = /** @type{!Array.<ILayoutGrid>} */(list);
	for (let i = ary.length; --i >= 0; ) {
		ary[i] = list[i].getSection();
	}
	return ary;
};
/** @public
* @param {string=} opt_type
* @return {!Array.<SectionSettings>}
*/
Core.prototype.getAllSectionSettings = function (opt_type) {
	if(!opt_type) {
		return this._settings; // WARNING: Return internal private member
	}

	let len = this._settings.length;
	let ary = [];
	opt_type = this._toSectionType(opt_type);
	for (let i = 0; i < len; ++i) {
		let sectionSettings = this._settings[i];
		if(opt_type === sectionSettings.getType()) {
			ary.push(sectionSettings);
		}
	}
	return ary;
};

/** @public
* @param {(MouseEvent|Core.SectionReference)=} identifier This can be MouseEvent, index, or section id or pointer reference
* @return {number} Return negative value, if the section does not exist
*/
Core.prototype.getSectionIndex = function(identifier) {
	if(util.isNumber(identifier)) {
		return /** @type{number} */(identifier);
	}

	if(identifier instanceof MouseEvent) {
		let mPos = util.getRelativePosition(/** @type{MouseEvent} */(identifier), this._element);
		let obj = {};
		this._hitTestY(mPos["y"], obj); // TODO: This will not work with zoom feature
		return obj["sectionIndex"];
	}

	let section = this.getSection(/** @type{string} */(identifier));
	if(section == null) { return -1; }

	return section.getIndex();
};
/** @public
* @param {Core.SectionReference} sectionRef
* @return {number} Return zero, if identifier does not exist
*/
Core.prototype.getSectionTop = function (sectionRef) {
	let section = this.getSection(sectionRef);
	if (section == null) { return 0; }
	let top = this._layoutY.getLaneStart(section.getRowOffset());

	if (this._startVScrollbarIndex >= 0 && section.getIndex() >= this._startVScrollbarIndex) {
		top -= this._vscrollbar.getScrollTop();
	}
	return top;
};

/** @public
* @param {(MouseEvent|Core.SectionReference)=} identifier This can be MouseEvent, index, or section id or pointer reference <br>
* If not specified, the last item is returned
* @param {string=} opt_type The previous section with the specified type. Default is any type
* @return {ILayoutGrid}
*/
Core.prototype.getPreviousSection = function (identifier, opt_type) {
	if(identifier == null) { identifier = this._settings.length; }
	let s = this._getNextSection(-1, identifier, opt_type);
	return (s !== null) ? s.getSection() : null;
};
/** @public
* @param {(MouseEvent|Core.SectionReference)=} identifier This can be MouseEvent, index, or section id or pointer reference <br>
* If not specified, the first item is returned
* @param {string=} opt_type The next section with the specified type. Default is any type
* @return {ILayoutGrid}
*/
Core.prototype.getNextSection = function (identifier, opt_type) {
	let s = this._getNextSection(1, identifier, opt_type);
	return (s !== null) ? s.getSection() : null;
};
/** @public
* @return {ILayoutGrid}
*/
Core.prototype.getLastSection = function () {
	return this.getSection(this._settings.length - 1);
};

/** @public
* @param {Core.SectionReference} sectionRef
* @param {number} colIndex
* @param {number} rowIndex
* @param {boolean=} opt_includeSpan Default is true
* @return {ElementWrapper}
*/
Core.prototype.getCell = function (sectionRef, colIndex, rowIndex, opt_includeSpan) {
	let s = this.getSection(sectionRef);
	return (s) ? s.getCell(colIndex, rowIndex, opt_includeSpan) : null;
};
/** Get information about the cell, inclusing column index, row index, and section that the cell resides
* @public
* @param {Core.CellReference} cellRef Reference to cell element. This can be something inside the cell as well.
* @return {Core.MouseInfo} Information about the given cell
* @see {@link Core#getRelativePosition}
* @example
* core.getCellInfo({"colIndex": 0, "rowIndex": 0}); // Get cell information about the top left cell in the content section
* core.getCellInfo(document.createElement("some_element"); // Get cell information for cell that contains the element with id "some_element"
* core.getCellInfo(mouseEventArg); // Get cell information from the given MouseEvent argument (this is the same as getRelativePosition() method)
* let cellInfo = core.getCellInfo({"colIndex": 1, "rowIndex": 1});
* cellInfo.colIndex = cellInfo.colIndex + 1; // Get next cell to the right
* core.getCellInfo(cellInfo);
*/
Core.prototype.getCellInfo = function (cellRef) {
	if(!cellRef) { return null; }
	let colIndex = /** @type{number} */(cellRef["colIndex"]);
	let rowIndex = /** @type{number} */(cellRef["rowIndex"]);
	let sectionName = /** @type{string} */(cellRef["sectionName"]);
	let mouseInfo = null;
	if(colIndex == null || rowIndex == null) { // Core.MouseInfo Non Mouse Info
		mouseInfo = this.getRelativePosition(/** @type{ElementWrapper|Element} */(cellRef));
		if(!mouseInfo["hit"]) {
			mouseInfo = null;
		}
	} else {
		if(!sectionName) {
			sectionName = "content";
		}
		let ss = this.getSectionSettings(sectionName);
		if(ss) {
			let s = ss.getSection();
			// WARNING: getCellInfo does not required cell to exist in the DOM (i.e., virtualized or hidden cells are included)
			if(colIndex >= 0 && colIndex < s.getColumnCount() &&
				rowIndex >= 0 && rowIndex < s.getRowCount()) {
				mouseInfo = {
					"colIndex": colIndex,
					"rowIndex": rowIndex
				};
				ss.extendEventArg(mouseInfo);
			}
		}
	}
	return mouseInfo;
};
/** Get cell information of the next cell in the same row. If there is no next cell in the row, `cell` parameter is set to null
* @public
* @param {Core.CellReference} cellRef Reference to cell element. This can be something inside the cell as well.
* @param {number=} step=1
* @return {Core.MouseInfo} Information about the next cell in the row. Returns null if the given parameter is invalid
*/
Core.prototype.getNextCellInRow = function (cellRef, step) {
	return this._getNextCellInRow(cellRef, (step != null) ? step : 1);
};
/** Get cell information of the previous cell in the same row. If there is no previous cell in the row, `cell` parameter is set to null
* @public
* @param {Core.CellReference} cellRef Reference to cell element. This can be something inside the cell as well.
* @param {number=} step=1
* @return {Core.MouseInfo} Information about the previous cell in the row. Returns null if the given parameter is invalid
*/
Core.prototype.getPrevCellInRow = function (cellRef, step) {
	return this._getNextCellInRow(cellRef, (step != null) ? -step : -1);
};
/** @private
* @param {Core.CellReference} cellRef
* @param {number} step
* @return {Core.MouseInfo}
*/
Core.prototype._getNextCellInRow = function (cellRef, step) {
	let mouseInfo = this.getCellInfo(cellRef);
	if(mouseInfo) {
		let colIndex = /** @type{number} */(mouseInfo["colIndex"]) + step;
		let rowIndex = /** @type{number} */(mouseInfo["rowIndex"]);
		let section = /** @type{ILayoutGrid} */(mouseInfo["section"]);

		mouseInfo["colIndex"] = colIndex;
		mouseInfo["cell"] = section.getCell(colIndex, rowIndex);
	}
	return mouseInfo;
};
/** Get cell information of the next cell. If bottom right has been reached, `cell` parameter is set to null
* @public
* @param {Core.CellReference} cellRef Reference to cell element. This can be something inside the cell as well.
* @return {Core.MouseInfo} Information about the next cell from left to right and top to bottom. Returns null if the given parameter is invalid
*/
Core.prototype.getNextCell = function (cellRef) {
	let mouseInfo = this.getNextCellInRow(cellRef);
	if(mouseInfo) {
		let cell = /** @type{Core.Cell} */(mouseInfo["cell"]);
		if(!cell) { // End of the row
			let section = /** @type{ILayoutGrid} */(mouseInfo["section"]);
			let colIndex = mouseInfo["colIndex"] = 0;
			let rowIndex = /** @type{number} */(mouseInfo["rowIndex"]);

			mouseInfo["rowIndex"] = ++rowIndex;
			mouseInfo["cell"] = section.getCell(colIndex, rowIndex);
		}
	}
	return mouseInfo;
};
/** Get cell information of the previous cell. If top left cell has been reached, `cell` parameter is set to null
* @public
* @param {Core.CellReference} cellRef Reference to cell element. This can be something inside the cell as well.
* @return {Core.MouseInfo} Information about the previous cell in the row. Returns null if the given parameter is invalid
*/
Core.prototype.getPrevCell = function (cellRef) {
	let mouseInfo = this.getPrevCellInRow(cellRef);
	if(mouseInfo) {
		let cell = /** @type{Core.Cell} */(mouseInfo["cell"]);
		if(!cell) { // End of the row
			let section = /** @type{ILayoutGrid} */(mouseInfo["section"]);
			let colIndex = mouseInfo["colIndex"] = section.getColumnCount() - 1;
			let rowIndex = /** @type{number} */(mouseInfo["rowIndex"]);

			mouseInfo["rowIndex"] = --rowIndex;
			mouseInfo["cell"] = section.getCell(colIndex, rowIndex);
		}
	}
	return mouseInfo;
};

/** @public
* @return {number}
*/
Core.prototype.getColumnCount = function () {
	return this._layoutX.getLaneCount();
};
/** Column count can be independently set regardless of row count and section count
* @public
* @param {number} num
* @fires Core#columnAdded
* @fires Core#preSectionRender
* @fires Core#columnRender
* @fires Core#beforeContentBinding
* @fires Core#preSectionDataBinding
* @fires Core#columnDataBinding
* @fires Core#postSectionDataBinding
* @fires Core#afterContentBinding
*/
Core.prototype.setColumnCount = function(num) {
	let colCount = this._layoutX.getLaneCount();
	if(num >= colCount) {
		this._layoutX.setLaneCount(num);
		for (let i = this._settings.length; --i >= 0; ) {
			let section = this._settings[i].getSection();
			section.setColumnCount(num); // Column may not be added to the document due to virtualization
		}

		this._onColumnCountChanged();
		this._syncLayoutToColumns(colCount, num);
		this._dispatchColumnAddedEvent(colCount, num - colCount, false);
	} else {
		this._removeColumn(colCount - num);
	}
};

/** @public
* @param {number} index
* @param {Core.ColumnOptions=} jsonObj Column definition object
* @fires Core#columnAdded
* @fires Core#preSectionRender
* @fires Core#columnRender
* @fires Core#beforeContentBinding
* @fires Core#preSectionDataBinding
* @fires Core#columnDataBinding
* @fires Core#postSectionDataBinding
* @fires Core#afterContentBinding
*/
Core.prototype.insertColumn = function (index, jsonObj) {
	let prevCount = this.getColumnCount();
	if (!(index < prevCount)) { index = prevCount; }

	this._layoutX.insertLane(index);
	for (let i = this._settings.length; --i >= 0; ) {
		let section = this._settings[i].getSection();
		section.insertColumn(index);
	}

	this._columnDefs.splice(index, 0, null); // TODO: Always initialize column def
	this._deserializeColumn(index, jsonObj); // Set leftPinned and other properties

	this._onColumnCountChanged();
	this._syncLayoutToColumns(index, index + 1); // _updateScrollbarWidth should be called

	this._dispatchColumnAddedEvent(index, 1, (index < prevCount), jsonObj);
};

/** @public
* @param {number} at
* @param {(number|Core.ColumnOptions|Array.<Core.ColumnOptions>)=} cols Column definition object or column count
*/
Core.prototype.insertColumns = function (at, cols) {
	let prevCount = this.getColumnCount();
	if (!(at < prevCount)) { at = prevCount; }

	let count = 0;
	let colDefs;
	if (Array.isArray(cols)) {
		count = cols.length;
		if(count === 1) {
			this.insertColumn(at, cols[0]);
			return;
		}

		colDefs = cols;
	} else if (typeof cols === "number") {
		count = cols;
		if(count === 1) {
			this.insertColumn(at, null);
			return;
		}
	} else {
		this.insertColumn(at, cols);
		return;
	}
	if (count <= 0) { return; }

	let prevState = this.freezeLayout();
	let args = [at, 0].concat(new Array(count));
	Array.prototype.splice.apply(this._columnDefs, args);

	for (let i = 0; i < count; i++) {
		let index = at + i;
		this._layoutX.insertLane(index);
		for (let s = this._settings.length; --s >= 0; ) {
			let section = this._settings[s].getSection();
			section.insertColumn(index);
		}
		let colDef = colDefs && colDefs[i];
		this._deserializeColumn(index, colDef);
	}

	this._dispatchColumnAddedEvent(at, count, (at < prevCount), colDefs);

	this.freezeLayout(prevState); // call _onColumnCountChanged and _syncLayoutToColumns internally
};

/** @public
* @param {number} index
* @return {Object} Return column definition object
* @see {@link Core#removeAllColumns}
* @fires Core#widthChanged
* @fires Core#columnRemoved
*/
Core.prototype.removeColumnAt = function (index) {
	let colCount = this.getColumnCount();
	if(index < 0 || index >= colCount) { return null; }

	let vEnd = -1;
	if(this._colVirtualizer.isVirtualizable()) {
		vEnd = this._colVirtualizer.getLastIndexInView();
	}

	let layoutSettings = this._layoutX.removeLaneAt(index);
	let sectionCount = this._settings.length;
	for (let i = 0; i < sectionCount; ++i) {
		this._settings[i].getSection().removeColumnAt(index);
	}

	let leftPinnedCount = this._pinnedLeftColumnCount;
	let rightPinnedCount = this._pinnedRightColumnCount;

	let colDef = this._columnDefs.splice(index, 1)[0]; // width and styles

	this._onColumnCountChanged();
	 // TODO: Column count changed may cause scroll pane to changed, and thus _updateScrollbarWidth should be included in the event handler
	if (index < leftPinnedCount) {
		this._updateScrollbarWidth(true, true);
	} else if (index >= (colCount - rightPinnedCount)) {
		this._updateScrollbarWidth(true, true);
	}
	this._syncLayoutToColumns(index, colCount - 1, true); // WARNiNG: _updateScrollbarWidth may be called again here

	if(!colDef) {
		colDef = {};
	}
	colDef["layout"] = layoutSettings;

	if (this._hasListener("columnRemoved")) {
		let e = {};
		e["atTheMiddle"] = true;
		e["colIndex"] = index;
		e["columns"] = "deprecated";
		e["columnData"] = colDef["columnData"];
		e["colId"] = colDef["id"] || "";
		this._dispatchColumnEvent("columnRemoved", e);
	}

	// Last index in view here might be different from before moving column if columns have different width.
	if(index <= vEnd && vEnd === this._colVirtualizer.getLastIndexInView()) {
		this._colVirtualizer.update(); // Since columns are shifted to the left, we need to activate the columns to the right
	}

	return colDef;
};
/** @public
* @see {@link Core#removeColumnAt}
* @fires Core#widthChanged
* @fires Core#columnRemoved
*/
Core.prototype.removeAllColumns = function () {
	if (this.getColumnCount() > 0) {
		this._pinnedLeftColumnCount = 0;
		this._pinnedRightColumnCount = 0;
		this.setColumnCount(0);
	}
};

/** @public
* @return {number}
*/
Core.prototype.getStationaryColumnIndex = function () {
	let colCount = this.getColumnCount();
	for (let i = colCount - 1; i >= 0; i--) {
		if (this._getColumnDef(i)["stationary"]) {
			return i;
		}
	}
	return -1;
};

/** Moving columns from or to steady area marked by stationary flag is not allowed. This method is used for checking ability of move.
* @public
* @param {number|Array.<number>} fromColIndex index or array of indices of the column(s) to be moved.
* @param {number} toColIndex target column index
* @return {boolean} Return true if the moving is doable, and false otherwise
*/
Core.prototype.isColumnMoveAllowed = function (fromColIndex, toColIndex) {
	if (!Array.isArray(fromColIndex)) {
		fromColIndex = [fromColIndex];
	}

	if (fromColIndex.length === 0 || toColIndex == null) {
		return false;
	}

	let stationaryIndex = this.getStationaryColumnIndex();

	if (stationaryIndex > -1) {
		if (toColIndex <= stationaryIndex) {
			return false;
		}

		for (let i = 0; i < fromColIndex.length; i++) {
			if (fromColIndex[i] <= stationaryIndex) {
				return false;
			}
		}
	}

	return true;
};

/** This method is used for checking movability of column.
* @public
* @param {number} colIndex Index of the column.
* @return {boolean} Return true if the column moving is movable, and false otherwise
*/
Core.prototype.isColumnMovable = function (colIndex) {
	let stationaryIndex = this.getStationaryColumnIndex();

	if (stationaryIndex > -1) {
		if (colIndex <= stationaryIndex) {
			return false;
		}
	}

	return true;
};

/** @public
* @param {number|Array.<number>} fromCol INCLUSIVE index/indices of the column to be moved
* @param {number} destCol INCLUSIVE
* @return {boolean} Return true if there is any change, and false otherwise
* @fires Core#columnMoved
*/
Core.prototype.moveColumn = function (fromCol, destCol) {
	if(typeof fromCol === "number") {
		return this._moveColumn(fromCol, destCol);
	}
	if(!Array.isArray(fromCol)) {
		return false;
	}

	// Validate user inputs
	let i;
	let len = fromCol.length;
	let colIndices = [];
	let colIndex = 0;
	let colCount = this.getColumnCount();
	let dict = {};
	for(i = 0; i < len; ++i) {
		colIndex = fromCol[i];
		if(colIndex >= 0 && colIndex < colCount) { // Filter out invalid index
			if(dict[colIndex] == null) { // Remove duplication
				dict[colIndex] = 1;
				colIndices.push(colIndex);
			}
		}
	}
	if (destCol < 0) {
		destCol = 0;
	} else if (destCol >= colCount) {
		destCol = colCount - 1;
	}

	// Skip unneccessary cases
	len = colIndices.length;
	if(!len) {
		return false;
	}
	if(len === 1) {
		return this._moveColumn(colIndices[0], destCol);
	}
	if(!this.isColumnMoveAllowed(colIndices, destCol)) {
		return false;
	}

	colIndices.sort(ascNumberSorter);
	if(dict[destCol]) { // Destination falls in the same place as source columns
		// Check if source columns are in continuous sequence
		let continuousSequence = true;
		colIndex = colIndices[0];
		for(i = 1; i < len; ++i) {
			if(++colIndex !== colIndices[i]) {
				continuousSequence = false;
				break;
			}
		}
		if(continuousSequence) {
			return false; // No moving operation is required
		}
	}
	let prevState = this.freezeLayout(true);

	// Start moving multiple items
	let rhsCount = 0;
	let srcIndex, destIndex;
	// Perform move operation on the right hand side
	for(i = 0; i < len; ++i) {
		srcIndex = colIndices[i];
		if(srcIndex >= destCol) {
			destIndex = destCol + rhsCount;
			this._moveColumn(srcIndex, destIndex);
			++rhsCount;
		}
	}
	// Perform move operation on the left hand side
	let lhsCount = len - rhsCount;
	destIndex = rhsCount ? destCol - 1 : destCol;
	for(i = 0; i < lhsCount; ++i) {
		srcIndex = colIndices[i] - i;
		this._moveColumn(srcIndex, destIndex);
	}

	this.freezeLayout(prevState);
	return true;
};
/** @private
* @param {number} fromCol INCLUSIVE
* @param {number} destCol INCLUSIVE
* @return {boolean} Return true if there is any change, and false otherwise
* @fires Core#columnMoved
*/
Core.prototype._moveColumn = function (fromCol, destCol) {
	let colCount = this.getColumnCount();
	if (fromCol < 0) { fromCol = 0; }
	else if (fromCol >= colCount) { fromCol = colCount - 1; }
	if (destCol < 0) { destCol = 0; }
	else if (destCol >= colCount) { destCol = colCount - 1; }
	if (fromCol === destCol) { return false; }

	if (!this.isColumnMoveAllowed(fromCol, destCol)) {
		return false;
	}

	let vEnd = -1;
	if(this._colVirtualizer.isVirtualizable()) {
		vEnd = this._colVirtualizer.getLastIndexInView();
	}

	let colId = this.getColumnId(fromCol);
	this._layoutX.moveLane(fromCol, destCol);
	let sectionCount = this._settings.length;
	for (let i = 0; i < sectionCount; ++i) {
		let sectionSettings = this._settings[i];
		let section = sectionSettings.getSection();
		section.moveColumn(fromCol, destCol);
	}

	let colDef = this._columnDefs[fromCol];
	let leftPinnedCount = this._countPinnedLeftColumns();
	let rightPinnedCount = this._countPinnedRightColumns();
	// let minColumn = (fromCol < destCol) ? fromCol : destCol;
	util.moveArrayItem(this._columnDefs, fromCol, destCol);

	if(this._hScrollbarEnabled) {
		if(fromCol < leftPinnedCount) {
			// Left pinned column

			if(destCol >= (colCount - rightPinnedCount)) {
				// moved into right frozen zone
				this.freezeColumn(leftPinnedCount - 2, rightPinnedCount + 1);
			} else if(destCol >= leftPinnedCount) {
				// moved out
				this.freezeColumn(leftPinnedCount - 2, rightPinnedCount);
			}
		} else if(colDef["rightPinned"]) {
			// Right pinned column

			if(destCol < leftPinnedCount) {
				// moved into left frozen zone
				this.freezeColumn(leftPinnedCount, rightPinnedCount - 1);
			} else if(destCol < (colCount - rightPinnedCount)) {
				// moved out
				this.freezeColumn(leftPinnedCount - 1, rightPinnedCount - 1);
			}
		} else {
			// unpinned column

			if(destCol < leftPinnedCount) {
				// moved into left frozen zone
				this.freezeColumn(leftPinnedCount, rightPinnedCount);
			} else if(destCol >= (colCount - rightPinnedCount)) {
				// moved into right frozen zone
				this.freezeColumn(leftPinnedCount - 1, rightPinnedCount + 1);
			}
		}
	}

	// The deactivated column may be moved directly or indirectly to the current view. It's necessary to make all columns in view activated.
	if(!this._frozenLayout) {
		if(this._colVirtualizer.isEnabled()) {
			let vBegin = this._colVirtualizer.getFirstIndexInView();
			if(!((fromCol < vBegin && destCol < vBegin) || (fromCol > vEnd && destCol > vEnd))) { // Columns does not move between hidden columns
				this._activateColumns(vBegin, vEnd, vBegin, vEnd); // To confirm that all columns in view are activated
			}
		}
	}

	// no need to invoke because moving column does not change column-width
	// this._syncLayoutToColumns(minColumn, this.getColumnCount());

	if (this._hasListener("columnMoved")) {
		let e = {};
		e["fromColIndex"] = fromCol;
		e["toColIndex"] = destCol;
		e["colId"] = colId; // TODO: Id may not needed
		this._dispatchColumnEvent("columnMoved", e); // add remove move
	}

	// Last index in view here might be different from before moving column if columns have different width.
	if(vEnd === this._colVirtualizer.getLastIndexInView()) {
		if (fromCol <= vEnd && destCol > vEnd) { // Columns are shifted to the left, we need to activate the columns to the right
			this._colVirtualizer.update();
		} else if (destCol <= vEnd && fromCol > vEnd) { // Move virtualized column into view, need to rerender destination column
			this._colVirtualizer.update();
		}
	}
	this.updateColumnBounds();
	this._updateColumnSeparators();
	return true;
};

/** If source column is not found, no operation is performed. If destination column is not found, the source column will be moved to the last position.<br>
* Note: this method behaves slightly different from moveColumn method in that it always put source column at the position before the specified destination column, while moveColumn method will put column at exactly at the destination index.
* @public
* @param {number|string} srcCol Column Id or index
* @param {(number|string)=} destCol Column Id or index of the destination
* @return {boolean} Return true if there is any change, and false otherwise
* @see {@link Core#moveColumn}
* @example
* grid.moveColumnById(3, 1); // Move column 3 to position before column 1
* grid.moveColumnById(0, 2); // Move column 0 to position before column 2 (column index 1)
* grid.moveColumnById(0, 1); // Nothing is moved
* grid.moveColumnById("sourceColumnId", "anotherId");
* grid.moveColumnById("sourceColumnId", ""); // move to the last position
*/
Core.prototype.moveColumnById = function (srcCol, destCol) {
	let colCount = this.getColumnCount();
	let srcIndex = this.getColumnIndex(srcCol);
	if(srcIndex < 0 || srcIndex >= colCount) {
		return false;
	}
	let destIndex = destCol != null ? this.getColumnIndex(destCol) : -1;
	if(destIndex < 0) {
		destIndex = colCount;
	}
	return this._moveColumnByIndex(srcIndex, destIndex);
};
/** Move column without verification for better performance
* @private
* @param {number} srcIndex Column index
* @param {number} destIndex Column index of the destination
* @return {boolean} Return true if there is any change, and false otherwise
*/
Core.prototype._moveColumnByIndex = function (srcIndex, destIndex) {
	if(srcIndex < destIndex) { // Ensure that the source column is put in front of the destination index
		--destIndex;
	}
	if(srcIndex === destIndex) {
		return false;
	}
	return this.moveColumn(srcIndex, destIndex);
};
/** @public
* @param {number|string|Array.<number|string>} colRefs List of column index or column id to be moved
* @param {(number|string)=} destCol Destination position where the moved columns will be placed BEFORE the specified position. This can be column id or index
* @return {boolean} Return true if there is any change, and false otherwise
*/
Core.prototype.reorderColumns = function (colRefs, destCol) {
	let destId = "";
	if(typeof destCol === "number") {
		destId = this.getColumnId(destCol);
	} else if(destCol) {
		destId = destCol;
	}

	this.startBatch("move");

	let dirty = 0;
	if(Array.isArray(colRefs)) {
		let srcLen = colRefs.length;
		if(srcLen > 1) {
			let colIds = this.getColumnIds();
			let colCount = colIds.length;
			let srcIds = [];
			let invalidDest = false;
			let i, srcId, srcIdx;
			for(i = 0; i < srcLen; ++i) {
				let colRef = colRefs[i];
				if(typeof colRef === "number") {
					srcIdx = colRef;
					srcId = colIds[colRef] || "";
				} else {
					srcId = colRef;
					srcIdx = colIds.indexOf(srcId);
				}
				if(srcId && srcIdx >= 0) {
					srcIds.push(srcId);
					if(destId === srcId) {
						invalidDest = true; // Destination must not exist in source columns
					}
				}
			}

			let destIdx;
			srcLen = srcIds.length;
			if(invalidDest) { // Find the next valid destination where it is not contained in the source columns
				destIdx = this.getColumnIndex(destId);
				if(destIdx >= 0) {
					while(++destIdx < colCount) {
						destId = colIds[destIdx];
						if(srcIds.indexOf(destId) < 0) {
							break;
						}
					}
				}
				if(destIdx < 0 || destIdx >= colCount) {
					destId = "";
				}
			}

			for(i = srcLen; --i >= 0;) {
				srcId = srcIds[i]; // Only valid source columns are left at this point
				srcIdx = this.getColumnIndex(srcId);
				destIdx = this.getColumnIndex(destId);
				if(destIdx < 0) { // Insert to the back when id is not found
					destIdx = colCount;
				}
				dirty |= this._moveColumnByIndex(srcIdx, destIdx);
				destId = srcId;
			}
		} else {
			dirty = this.moveColumnById(colRefs[0], destId);
		}
	} else if(colRefs != null) {
		// colRefs will be a number or string
		dirty = this.moveColumnById(colRefs, destId);
	}
	this.stopBatch("move");
	return dirty ? true : false;
};

/** @public
* @ignore
* @return {!TrackLayout}
*/
Core.prototype.getColumnLayout = function () {
	return this._layoutX;
};

/** @public
* @ignore
* @fires Core#widthChanged
*/
Core.prototype.updateColumnLayout = function () {
	this._syncLayoutToColumns(0, this.getColumnCount());
};

/** @private
* @param {number} index
* @param {Object=} jsonObj
*/
Core.prototype._deserializeColumn = function (index, jsonObj) {
	if (!jsonObj) { return; }

	let layout = jsonObj["layout"] || jsonObj;
	let val = jsonObj["width"];
	if (val != null) { layout["size"] = val; }
	val = jsonObj["scalable"];
	if (val != null) { layout["scalable"] = val; }
	val = jsonObj["minWidth"];
	if (val != null) { layout["minSize"] = val; }
	if (layout["scalable"] == null) {
		layout["scalable"] = (layout["size"] == null);
	}

	this._layoutX.deserializeLane(index, layout);

	let name;
	let styles = jsonObj["styles"];
	for (name in styles) {
		this.setColumnStyle(index, name, styles[name]);
	}
	let classes = jsonObj["classes"];
	for (name in classes) {
		this.enableColumnClass(index, name, classes[name]);
	}
	// TODO: Load specific styles for each column type (e.g. "content", "title", "header")

	let colDef = this._getColumnDef(index);
	let colId = jsonObj["id"];
	if(colId && typeof colId === "string") {
		colDef["id"] = colId; // WARNING: We do not guarantee uniqueness of user id
		colDef["userId"] = colId;
	}
	let field = jsonObj["field"];
	if(field && typeof field === "string") {
		colDef["field"] = field;
	}

	let value = jsonObj["dataColumnName"]; // Deprecated
	if (value != null) {
		colDef["dataColumnName"] = value;
	}

	value = jsonObj["stationary"];
	if (value != null) {
		colDef["stationary"] = value ? true : false;
	}
	value = jsonObj["leftPinned"];
	if (value != null) {
		colDef["leftPinned"] = value ? true : false;
	}
	value = jsonObj["rightPinned"];
	if (value != null) {
		colDef["rightPinned"] = value ? true : false;
	}
	value = jsonObj["backgroundColor"];
	if (value != null) {
		this.setColumnBackgroundColor(index, value);
	}

	this.setColumnRenderingHandler(index, jsonObj["renderingHandler"]);
	this.setColumnDataBindingHandler(index, jsonObj["dataBindingHandler"]);

	value = jsonObj["columnData"];
	if(value) {
		this.setColumnData(index, value);
	}

	let formatter = jsonObj["binding"] || jsonObj["formatter"]; // support both composite and real-time grid, binding have piority first
	if(Array.isArray(formatter)) {
		formatter = formatter[0]; // Support only one formatter in the array
	}
	if(formatter && typeof formatter !== "function") {
		let formatterInitializer = formatter["init"];
		if(typeof formatterInitializer === "function") {
			formatterInitializer(jsonObj, this); // Pass on userJson object and core grid instance
		}
	}
};

/** Add specified number of rows to the last section. This method is intended for fast layout generation. <br>
* If column count is zero, one column will be added automatically. <br>
* If there is no section, a new section is created automatically. <br>
* No row is added to the data source.
* @public
* @param {number=} opt_num Default is one row
* @fires Core#preSectionRender
* @fires Core#columnRender
* @fires Core#beforeContentBinding
* @fires Core#preSectionDataBinding
* @fires Core#columnDataBinding
* @fires Core#postSectionDataBinding
* @fires Core#afterContentBinding
*/
Core.prototype.addRow = function (opt_num) {
	if (opt_num == null) { opt_num = 1; }
	else if (opt_num <= 0) { return; }

	if (this.getColumnCount() <= 0) {
		this.setColumnCount(1); // WARNING: Auto column adding
	}

	let section = this.getLastSection();
	if (section == null) {
		section = this.addSection(); // WARNING: Auto section adding
	}

	section.addRow(opt_num);
};
/** Remove specified number of rows from the last section. <br>
* Note that this will NOT remove row in the DataTable, as well as the section. <br>
* Use removeRowData to remove the supporting data row from the data source
* @public
* @param {number=} opt_num Default is one row
*/
Core.prototype.removeRow = function (opt_num) {
	let section = this.getLastSection();
	if (section == null) { return; }

	if (opt_num == null) { opt_num = 1; }
	section.removeRow(opt_num);
};
/** @public
* @return {number}
*/
Core.prototype.getRowCount = function () {
	return this._layoutY.getLaneCount();
};
/** The specified section and columns must exist prior to calling this method
* @public
* @param {Core.SectionReference} sectionRef
* @param {number} num
*/
Core.prototype.setSectionRowCount = function(sectionRef, num) {
	let section = this.getSection(sectionRef);
	if (section == null) { return; }
	section.setRowCount(num);
};
/** @public
* @param {number} sectionIndex
* @param {number} rowIndex relative to the given section
* @return {number} rowIndex relative to the first section (index as a whole)
*/
Core.prototype.getRowIndex = function(sectionIndex, rowIndex) {
	return this._sectionStarts[sectionIndex] + rowIndex;
};
/** @private
* @function
*/
Core.prototype.getRelativeIndex = util._deprecatedFunction;

/** WARNING: This only removes all rows in each section, but the sections and columns remain intact
* @public
*/
Core.prototype.removeAllRows = function() {
	let len = this._settings.length;
	for(let i = 0; i < len; ++i) {
		this._settings[i].getSection().setRowCount(0);
		this._sectionStarts[i] = 0;
	}
	this._sectionStarts[len] = 0;
};
/** @public
* @return {number}
*/
Core.prototype.getDefaultRowHeight = function () {
	return this._layoutY.getDefaultLaneSize();
};
/** This default row height only applies to header and content section. <br>
* Use the second flag to apply the change to title section too. <br>
* To set default row height for a specific section, please call setDefaultRowHeight directly from the section
* @public
* @param {number} val
* @param {boolean=} opt_includeTitle
*/
Core.prototype.setDefaultRowHeight = function (val, opt_includeTitle) {
	if (this._layoutY.setDefaultLaneSize(val)) {
		this._rowHeightSync = false;
		let defaultHeight = this._layoutY.getDefaultLaneSize();
		this._rowHeightException = (opt_includeTitle) ? "" : "title";
		let sectCount = this._settings.length;
		for (let i = 0; i < sectCount; ++i) {
			let sectionSettings = this._settings[i];
			if (
				sectionSettings.getType() !== this._rowHeightException &&
				sectionSettings.getId() !== "bottomPadding"
			) {
				sectionSettings.getSection().setDefaultRowHeight(defaultHeight); // An event will be fired for each row
			}
		}
		this._syncRowHeights();
		this._rowHeightSync = true;
		this.setRowScrollingStep(this._rowScrollingStep);
		this._updateColumnSeparators();
	}
};

/** @public
* @return {DataView} dataSource
*/
Core.prototype.getDataSource = function () {
	return this._dataSource;
};

/** set Data Conflation for better performance
* @public
* @param {number} milliSecond
*/
Core.prototype.setDataConflationRate = function (milliSecond) {
	this._dataSource.setConflationRate(milliSecond);
};

/**
* @public
* @return {number}
*/
Core.prototype.getDataConflationRate = function () {
	return this._dataSource.getConflationRate();
};

/** @public
* @param {DataTable|DataView} dataSource
* @fires Core#dataSourceChanged
* @fires Core#postDataSourceChanged
* @return {DataView} Returns the root data view
*/
Core.prototype.setDataSource = function (dataSource) {
	if (dataSource) {
		if (dataSource instanceof data_DataTable) { // Sometimes, user put in a wrong parameter
			dataSource = new data_DataView(dataSource);
		}
	}

	let e = {};
	e["prevDataSource"] = this._dataSource;
	this._dataSource = dataSource || null;
	e["dataSource"] = this._dataSource;

	this._dispatch("dataSourceChanged", e);

	if (this._autoSetDataSource) {
		let len = this._settings.length;
		for (let i = 0; i < len; ++i) {
			this._settings[i].setDataSource(dataSource);
		}
	}

	this._dispatch("postDataSourceChanged", e);

	return this._dataSource;
};

/** @private
* @function
*/
Core.prototype.setColumnTemplate = util._deprecatedFunction;
/** @private
* @function
*/
Core.prototype.getColumnTemplate = util._deprecatedFunction;

/** Deprecated. Use setColumnRenderingHandler instead
* @public
* @function
*/
Core.prototype.getColumnRenderEvent = util._deprecatedFunction;
/** Deprecated. Use setColumnDataBindingHandler instead
* @public
* @function
* @param {number} c
* @return {Object}
* @see {@link Core#setColumnDataBindingHandler}
*/
Core.prototype.getColumnDataBindingEvent = function(c) {
	console.log("getColumnDataBindingEvent() is deprecated. Use setColumnDataBindingHandler() instead");
	let t = this;
	return {
		"listen": function(f) {
			t.setColumnDataBindingHandler(c, f);
		},
		"unlisten": function() {
			t.setColumnDataBindingHandler(c, null);
		}
	};
};


/** @public
* @param {number} colIndex
* @param {string} style
* @param {string} value
* @param {string=} opt_type Apply the style only to the specified row type. Default is applying to all rows
*/
Core.prototype.setColumnStyle = function (colIndex, style, value, opt_type) {
	if (!style) { return; }
	if (!util.isNumber(colIndex)) {
		colIndex = 0;
		console.log("WARNING: the first parameter for setColumnStyle must be column index");
	}

	let styles;
	if (opt_type == null) {
		styles = this._getNestedColumnDef(colIndex, "styles");
	} else {
		opt_type = this._toSectionType(opt_type);
		styles = this._getNestedColumnDef(colIndex, opt_type, "styles");
	}
	styles[style] = value;

	for (let i = this._settings.length; --i >= 0; ) {
		let sectionSettings = this._settings[i];
		if (opt_type == null || opt_type === sectionSettings.getType()) {
			let section = sectionSettings.getSection();
			section.setColumnStyle(colIndex, style, value);
		}
	}
};

/** @public
* @param {number} colIndex
* @return {string}
*/
Core.prototype.getColumnBackgroundColor = function(colIndex) {
	let colDef = this._getColumnDef(colIndex);
	return colDef["backgroundColor"] || "";
};
/** @public
* @param {number} colIndex
* @param {string=} color
*/
Core.prototype.setColumnBackgroundColor = function(colIndex, color) {
	if(colIndex == null || typeof colIndex !== "number") {
		return;
	}

	let colDef = this._getColumnDef(colIndex);
	if(colDef["backgroundColor"] === color) {
		return;
	}

	colDef["backgroundColor"] = color;
	color = color != null ? color : "";

	let sectionCount = this._settings.length;
	for(let i = 0; i < sectionCount; ++i) {
		let settings = this._settings[i];
		let section = settings.getSection();
		if(!section) {
			continue;
		}

		if(settings.getType() !== "content") {
			this._applyCellsBackgroundColor(section, colIndex, color);
		} else {
			let column = section.getColumn(colIndex);
			column.setStyle("backgroundColor", color);
		}
	}
};
/** @private
* @param {ILayoutGrid} section
* @param {number} colIndex
* @param {string} color
*/
Core.prototype._applyCellsBackgroundColor = function(section, colIndex, color) {
	let rowCount = section.getRowCount();
	for(let r = 0; r < rowCount; r++) {
		let cellSpan = section.getCellColSpan(colIndex, r);
		if(cellSpan > 1) { continue; }
		let cell = section.getCell(colIndex, r);
		if(cell) {
			cell.setStyle("backgroundColor", color);
		}
	}
};
/** @public
* @ignore
* @param {ILayoutGrid} section
*/
Core.prototype.applyColumnsBackgroundColor = function(section) {
	if(!section) {
		return;
	}

	let colCount = section.getColumnCount();
	for(let c = 0; c < colCount; c++) {
		let colDef = this._getColumnDef(c);
		let color = colDef["backgroundColor"] || "";
		this._applyCellsBackgroundColor(section, c, color);
	}
};

/** @public
* @param {number} colIndex
* @param {string} clsName
* @param {boolean=} enabled
* @param {string=} opt_type Apply the class only to the specified row type. Default is applying to all rows
* @return {boolean} Return true, if there is any change
*/
Core.prototype.enableColumnClass = function (colIndex, clsName, enabled, opt_type) {
	if (typeof colIndex !== "number") {
		colIndex = 0;
	}

	let classes;
	if (opt_type == null) {
		classes = this._getNestedColumnDef(colIndex, "classes");
	} else {
		opt_type = this._toSectionType(opt_type);
		classes = this._getNestedColumnDef(colIndex, opt_type, "classes");
	}

	let dirty = false;
	enabled = (enabled !== false);
	if (enabled) {
		if(!classes[clsName]) {
			dirty = classes[clsName] = true;
		}
	} else if (classes[clsName]) {
		delete classes[clsName];
		dirty = true;
	}

	if(dirty) {
		for (let i = this._settings.length; --i >= 0; ) {
			let sectionSettings = this._settings[i];
			if (opt_type == null || opt_type === sectionSettings.getType()) {
				sectionSettings.getSection().enableColumnClass(colIndex, clsName, enabled);
			}
		}
	}
	return dirty;
};
/** @public
* @param {number} colIndex
* @param {string} clsName
* @param {string=} opt_type Row type
* @return {boolean}
*/
Core.prototype.hasColumnClass = function (colIndex, clsName, opt_type) {
	let classes;
	if (opt_type == null) {
		classes = this._getNestedColumnDef(colIndex, "classes");
	} else {
		opt_type = this._toSectionType(opt_type);
		classes = this._getNestedColumnDef(colIndex, opt_type, "classes");
	}
	return (classes && classes[clsName]) ? true : false;
};

/** @public
* @param {number} colIndex
* @param {string=} alignment Available options are "center"|"right"|"left"|"c"|"r"|"l". Use empty string (falsy value) to reset back to default aliganment.
*/
Core.prototype.setColumnAlignment = function(colIndex, alignment) {
	let ch = alignment ? alignment.charAt(0).toLowerCase() : "";

	let dirty = 0;
	dirty |= this.enableColumnClass(colIndex, "tr-align-left", ch === "l");
	dirty |= this.enableColumnClass(colIndex, "tr-align-right", ch === "r");
	dirty |= this.enableColumnClass(colIndex, "tr-align-center", ch === "c");

	if(dirty) {
		this._dispatch("columnAlignmentChanged", {
			"colIndex": colIndex,
			"alignment": ch
		});
	}
};
/** @public
* @param {number} colIndex
* @return {string} Possible returned values are "default"|"left"|"center"|"right"
*/
Core.prototype.getColumnAlignment = function(colIndex) {
	if(this.hasColumnClass(colIndex, "tr-align-right")) {
		return "right";
	} else if(this.hasColumnClass(colIndex, "tr-align-center")) {
		return "center";
	} else if(this.hasColumnClass(colIndex, "tr-align-left")) {
		return "left";
	}
	return "default";
};


/** @public
* @param {number} colIndex
* @return {number}
*/
Core.prototype.getColumnLeft = function (colIndex) {
	let colDef = this._columnDefs[colIndex];
	if(colDef && colDef["rightPinned"]) {
		let gridWidth = this.getElement().clientWidth;
		let contentWidth = this._layoutX.getTrackSize();
		let colLeft = contentWidth < gridWidth ? contentWidth : gridWidth;
		let lastIndex = this.getColumnCount() - 1;
		for(let i = lastIndex; i >= colIndex; --i) {
			colLeft -= this._layoutX.getLaneSize(i);
		}
		return colLeft;
	} else if (this._hScrollbarEnabled && colIndex >= this._pinnedLeftColumnCount) {
		return this._layoutX.getLaneStart(colIndex) - this._hscrollbar.getScrollLeft();
	} else {
		return this._layoutX.getLaneStart(colIndex);
	}
};
/** @public
* @param {number} colIndex
* @return {number}
*/
Core.prototype.getColumnRight = function (colIndex) {
	return this.getColumnLeft(colIndex) + this._layoutX.getLaneSize(colIndex);
};
/** @public
* @param {number} colIndex
* @return {number}
*/
Core.prototype.getColumnWidth = function (colIndex) {
	return this._layoutX.getLaneSize(colIndex);
};
/** @public
* @param {number} colIndex
* @return {number}
*/
Core.prototype.getColumnCustomLaneSize = function (colIndex) {
	return this._layoutX.getCustomLaneSize(colIndex);
};
/** @private
* @param {Array.<number>} indices
* @param {Array.<number>} widths
* @return {Array.<number>}
*/
Core.prototype._calculateColumnWidths = function (indices, widths) {
	if(!this._preserveGridSize) {
		return widths;
	}

	let i, newWidth = 0, settingCols = {};
	let len = indices.length;
	let calWidths = new Array(len);
	for (i = 0; i < len; i++) {
		settingCols[indices[i]] = true;
		calWidths[i] = widths[i];
		newWidth += widths[i];
	}
	let fixedSize = 0, scalableCount = 0;
	let colCount = this.getColumnCount();
	let layoutX = this._layoutX;

	for(i = 0; i < colCount; i++) {
		if (settingCols[i]) { continue; }
		if (layoutX.isLaneVisible(i)) { // If column is visible
			if (!layoutX.getLaneScalability(i)) {
				fixedSize += layoutX.getLaneSize(i);
			} else {
				fixedSize += layoutX.getMinimumLaneSize(i);
				scalableCount++;
			}
		}
	}
	let gridSize = this.getContentWidth();
	let excess = newWidth + fixedSize - gridSize;

	if (scalableCount < 1 || excess > 0) {
		layoutX.resetLaneToDefaultSize();
		layoutX.setTrackSize(gridSize - this._reservedSpace);
		fixedSize = 0;
		for(i = 0; i < colCount; i++) {
			if (settingCols[i]) { continue; }
			if (layoutX.isLaneVisible(i)) { // If column is visible
				if (!layoutX.getLaneScalability(i)) {
					fixedSize += layoutX.getLaneSize(i);
				} else {
					fixedSize += layoutX.getMinimumLaneSize(i);
				}
			}
		}
		excess = newWidth + fixedSize - gridSize;
		if (excess > 0) {
			excess = excess / len;
			for (i = 0; i < len; i++) {
				calWidths[i] -= excess;
			}
		}
	}
	return calWidths;
};
/** @public
* @param {number} colIndex
* @param {number} px
* @param {boolean=} opt_scalable If not defined false value is used instead
* @return {boolean} True if there is any changed
* @fires Core#widthChanged
* @example
* core.setColumnWidth(0, 100); // The first column will have a fixed width
* core.setColumnWidth(1, 100, true); // The second column will have a percentage width where 100 is the weight comparing with other percentage column
*/
Core.prototype.setColumnWidth = function (colIndex, px, opt_scalable) {
	let calWidths = this._calculateColumnWidths([colIndex], [px]);
	let dirty = this._setColumnWidth(colIndex, calWidths[0], opt_scalable);
	if (dirty) {
		this._syncLayoutToColumns(colIndex, colIndex + 1);
	}
	return dirty;
};
/** Specifies multiple column widths all at once.
* @public
* @param {Array.<number>} columnWidths in pixel. If the specified value is undefined or null, nothing is performed on that column.
* @param {(boolean|Array.<boolean>)=} opt_proportion This will make columns scale in proportion specified by the given width
* @return {boolean} True if there is any changed
* @fires Core#widthChanged
* @example
* core.setColumnWidths([1, 2, 1, 1], true); // The first four columns have percentage width with the second column is twice as big as the others
* core.setColumnWidths([1, 200, 1, 150], [true, false, true]); // The first column make column scale, seconds make column pixel, third make column scale and four make column pixel
*/
Core.prototype.setColumnWidths = function (columnWidths, opt_proportion) {
	let dirty = 0;
	let indices = [], widths = [], propotions = [], i;
	for(i = columnWidths.length; --i >= 0;) {
		let px = columnWidths[i];
		if(px != null) {
			indices.push(i);
			widths.push(px);
		}
		let proportionMode = Array.isArray(opt_proportion) ? opt_proportion[i] : opt_proportion;
		propotions.push(proportionMode);
	}
	let calWidths = this._calculateColumnWidths(indices, widths);
	for(i = 0; i < calWidths.length; i++) {
		dirty |= this._setColumnWidth(indices[i], calWidths[i], propotions[i] );
	}

	if (dirty) {
		this._syncLayoutToColumns(0, columnWidths.length);
	}
	return dirty ? true : false;
};
/** Use {@link Core#hideColumn} to hide the column
* @public
* @param {number} colIndex
* @return {boolean}
*/
Core.prototype.isColumnVisible = function (colIndex) {
	return this._layoutX.isLaneVisible(colIndex);
};
/** Note that the hidden columns continue to exist and occupy the index. So there is no index shifted after this function call.
* @public
* @param {number} colIndex
* @param {boolean=} hidden If the value is true or undefined, hide the specified column. Otherwise, show the column
* @fires Core#widthChanged
* @fires Core#columnVisibilityChanged
* @see {@link Core#isColumnVisible}
* @see {@link Core#hideColumns}
* @see {@link Core#showAllColumns}
*/
Core.prototype.hideColumn = function (colIndex, hidden) {
	let visibility = hidden === false;
	this.setColumnVisibility(colIndex, visibility, 0);
};
/** Hide multiple columns at once, improving the performance. Note that the hidden columns continue to exist and occupy the index. So there is no index shifted after this function call.
* @public
* @param {Array.<number>} colIndices
* @param {boolean=} hidden If the value is true or undefined, hide the specified columns. Otherwise, show the columns
* @fires Core#widthChanged
* @fires Core#columnVisibilityChanged
* @see {@link Core#hideColumn}
* @see {@link Core#showAllColumns}
*/
Core.prototype.hideColumns = function (colIndices, hidden) {
	let len = Array.isArray(colIndices) ? colIndices.length : 0;
	let changeIndices = [];
	let colCount = this.getColumnCount();
	hidden = hidden !== false;
	for (let i = 0; i < len; ++i) {
		let colIndex = colIndices[i];
		if (colIndex >= 0 && colIndex < colCount) {
			if (this._layoutX.hideLane(colIndex, hidden, 0)) {
				changeIndices.push(colIndex);
			}
		}
	}
	if (changeIndices.length) {
		this._syncLayoutToColumns();
		this.requestRowRefresh();

		if (this._hasListener("columnVisibilityChanged")) {
			let e = {};
			e["colIndices"] = changeIndices;
			e["hidden"] = hidden;
			this._dispatch("columnVisibilityChanged", e);
		}
	}
};
/** @public
* @fires Core#widthChanged
* @fires Core#columnVisibilityChanged
* @see {@link Core#hideColumn}
* @see {@link Core#hideColumns}
*/
Core.prototype.showAllColumns = function () {
	let changeIndices = [];
	let colCount = this.getColumnCount();
	for (let c = 0; c < colCount; ++c) {
		if (this._layoutX.hideLane(c, false, 0)) {
			changeIndices.push(c);
		}
	}
	if (changeIndices.length) {
		this._syncLayoutToColumns();
		this.requestRowRefresh();

		if (this._hasListener("columnVisibilityChanged")) {
			let e = {};
			e["colIndices"] = changeIndices;
			e["hidden"] = false;
			this._dispatch("columnVisibilityChanged", e);
		}
	}
};
/** @public
* @param {number} colIndex
* @return {number}
*/
Core.prototype.getMinimumColumnWidth = function (colIndex) {
	return this._layoutX.getMinimumLaneSize(colIndex);
};
/** @public
* @param {number} colIndex
* @param {number} width
* @fires Core#widthChanged
*/
Core.prototype.setMinimumColumnWidth = function (colIndex, width) {
	if (this._layoutX.setMinimumLaneSize(colIndex, width)) {
		this._syncLayoutToColumns(colIndex, colIndex + 1);
	}
};
/** @public
* @param {Array.<number|null>} minWidths
* @param {boolean=} proportion This will make the column expand in proportion of its minimum width when there is enough space.
* @fires Core#widthChanged
* @return {boolean} True if there is any change
*/
Core.prototype.setMinimumColumnWidths = function (minWidths, proportion) {
	let dirty = 0;
	let first = NaN;
	let last = NaN;
	for (let c = minWidths.length; --c >= 0; ) {
		let minWidth = minWidths[c];
		if (minWidth != null) {
			if(!last) { last = c; }
			first = c;
			dirty |= this._layoutX.setMinimumLaneSize(c, minWidth, proportion);
		}
	}
	if (dirty) {
		this._syncLayoutToColumns(first, last + 1);
	}
	return dirty ? true : false;
};
/** @public
* @param {number} colIndex
* @return {boolean}
*/
Core.prototype.getColumnScalability = function (colIndex) {
	return this._layoutX.getLaneScalability(colIndex);
};
/** @public
* @param {number|boolean} val column index or default column scalability
* @param {boolean=} opt_scalable
* @return {boolean}
* @fires Core#widthChanged
*/
Core.prototype.setColumnScalability = function (val, opt_scalable) {
	let dirty = false;
	if (util.isNumber(val)) {
		opt_scalable = (opt_scalable !== false);
		dirty = this._layoutX.setLaneScalability(/** @type{number} */(val), opt_scalable);
		if (dirty) {
			this._syncLayoutToColumns(/** @type{number} */(val), val + 1);
		}
	} else {
		this.setDefaultColumnScalable(/** @type{boolean} */(val));
	}
	return dirty;
};
/** @public
* @param {boolean} scalable
*/
Core.prototype.setDefaultColumnScalable = function (scalable) {
	this._layoutX.setDefaultScalability(!!scalable);
};
/** @public
* @return {boolean}
*/
Core.prototype.resetColumnWidths = function () {
	let dirty = this._layoutX.resetLaneToDefaultSize();
	if (dirty) {
  	this._syncLayoutToColumns(0, this.getColumnCount());
	}
	return dirty;
};
/** @public
* @param {number} colIndex
* @param {number} width
*/
Core.prototype.setColumnDefaultWidth = function (colIndex, width) {
	this._layoutX.setLaneDefaultSize(colIndex, width);
};
/** @public
* @param {boolean} preserve=false
*/
Core.prototype.preserveProportion = function (preserve) {
	this._preserveProportion = preserve === true;
};
/** @public
* @return {boolean} Proportion preserved
*/
Core.prototype.isProportionPreserved = function () {
	return this._preserveProportion;
};
/** @public
* @param {boolean} preserve=false
*/
Core.prototype.preserveGridSize = function (preserve) {
	this._preserveGridSize = preserve === true;
};
/** @public
* @param {number} colIndex
* @return {!Object} Column data object
*/
Core.prototype.getColumnData = function (colIndex) {
	return this._getColumnDef(colIndex)["columnData"];
};
/** @public
* @deprecated
* @ignore
* @param {number} colIndex
* @param {Object} userData
* @return {Object}
*/
Core.prototype.setColumnData = function (colIndex, userData) {
	this._getColumnDef(colIndex)["columnData"] = userData || {};
	return userData;
};
/** @public
* @function
* @deprecated
* @ignore
* @param {number} colIndex
* @return {!Object}
*/
Core.prototype.newColumnData = Core.prototype.getColumnData;
/** @public
* @param {number} colIndex
* @param {boolean=} alwaysRender
*/
Core.prototype.setAlwaysRenderColumn = function(colIndex, alwaysRender) {
	this.getColumnData(colIndex)["alwaysRender"] = (alwaysRender !== false);
};
/** @private
* @param {number} colIndex
* @return {!boolean}
*/
Core.prototype._isAlwaysRenderColumn = function (colIndex) {
	let colData = this.getColumnData(colIndex);
	return colData["alwaysRender"] === true;
};

/** @public
* @return {boolean}
*/
Core.prototype.getFitContentWidth = function () {
	return this._fitContentWidth;
};
/** @public
* @param {boolean} bool
*/
Core.prototype.setFitContentWidth = function (bool) {
	if (this._fitContentWidth !== bool) {
		this._fitContentWidth = bool;
		this.enableClass("fit-width", this._fitContentWidth);
	}
};

/** A plugin can be loaded only once
* @public
* @param {*} pluginRef Plugin/extension instance. Plugin or extension name (string) is deprecated in favor of html import syntax
* @param {Object=} opt_options Option for initialize the given plugin
* @return {*} Return the plugin that has been successfully loaded. Returns null if the plugin is not loaded
* @example
* core.loadPlugin(new InCellEditingExtension());
*/
Core.prototype.loadPlugin = function (pluginRef, opt_options) {
	let pluginInstance = null;
	if (typeof pluginRef === "string") {
		let trNamespace = window["tr"];
		let gridNamespace = trNamespace ? trNamespace["grid"] : null;
		if(!gridNamespace) {
			return null;
		}
		if (gridNamespace[pluginRef]) {
			pluginInstance = new gridNamespace[pluginRef]();
		} else if (gridNamespace[pluginRef + "Plugin"] ) {
			pluginInstance = new gridNamespace[pluginRef + "Plugin"]();
		} else {
			return null;
		}
	} else {
		pluginInstance = pluginRef;
	}

	if (!pluginInstance || !pluginInstance["getName"]) {
		return null;
	}

	let pluginName = pluginInstance["getName"]();
	if(!pluginName) {
		return null;
	}

	let prevInstance = this._plugins[pluginName];
	if (prevInstance) {
		if(prevInstance === pluginInstance) {
			return prevInstance; // The same instance is loaded twice
		}
		console.log("WARNING: Two of the same " + pluginName + "s cannot be loaded on a single grid. The existing plugin is replaced.");
		this.unloadPlugin(pluginName);
	}

	if (pluginInstance["initialize"]) {
		pluginInstance["initialize"](this, opt_options);
	}
	this._plugins[pluginName] = pluginInstance;

	return pluginInstance;
};
/** @public
* @param {*} pluginRef Plugin name (string) or instance
* @return {*|null}
*/
Core.prototype.unloadPlugin = function (pluginRef) {
	let pluginInstance;
	if (pluginRef) {
		let pluginName = "";
		if (typeof pluginRef === "string") {
			pluginName = pluginRef;
		} else if (pluginRef["getName"]) {
			pluginName = pluginRef["getName"]();
		}

		pluginInstance = this._plugins[pluginName];
		if(!pluginInstance) {
			pluginName += "Plugin";
			pluginInstance = this._plugins[pluginName];
		}

		if (pluginInstance) {
			if (pluginInstance["unload"]) {
				pluginInstance["unload"](this);
			}

			delete this._plugins[pluginName];
		}
	}
	return pluginInstance || null;
};
/** Return null if the specified plugin has not been loaded
* @public
* @param {string} pluginName
* @return {*|null} plugin Return null if the specified plugin has not been loaded
*/
Core.prototype.getPlugin = function (pluginName) {
	let p = this._plugins[pluginName];
	if (!p) {
		p = this._plugins[pluginName + "Plugin"];
	}
	return p || null;
};
/** Get a list of all plugin instances
* @public
* @return {!Array}
*/
Core.prototype.getPlugins = function () {
	let list = [];
	for (let key in this._plugins) {
		list.push(this._plugins[key]);
	}
	return list;
};
/** Get a list of existing plugin names
* @public
* @return {!Array.<string>}
*/
Core.prototype.getPluginList = function () {
	let list = [];
	for (let key in this._plugins) {
		list.push(key);
	}
	return list;
};

/** Vertical scrollbar will automatically appear for the rest of the sections, when container of Grid does NOT have enough space for Grid's content.
* @public
* @param {(string|number|null)=} opt_frozenSection the last section that will be fixed in place. <br>
* Negative value means applying the scrollbar to all sections. <br>
* null value means disabling the feature.
*/
Core.prototype.freezeSection = function (opt_frozenSection) {
	this._fixFrozenTopSections = true;
	this._freezeTopSectionAt(opt_frozenSection);
};
/** @public
*/
Core.prototype.resetSectionFreezing = function () {
	this._fixFrozenTopSections = false;
	this._updateFrozenTopSections();
};

/** Vertical scrollbar will automatically appear for the rest of the sections, when container of Grid does NOT have enough space for Grid's content.
* @private
* @param {(string|number|null)=} opt_frozenSection the last section that will be fixed in place. <br>
* Negative value means applying the scrollbar to all sections. <br>
* null value means disabling the feature.
*/
Core.prototype._freezeTopSectionAt = function (opt_frozenSection) {
	if (typeof opt_frozenSection == "number") {
		let newIndex = opt_frozenSection + 1;
		if (newIndex === this._startVScrollbarIndex) return; // same position then do nothing
		this._startVScrollbarIndex = newIndex;
	} else {
		this._startVScrollbarIndex = -1;
		if (typeof opt_frozenSection == "string") {
			this._tempFrozenSectionRef = opt_frozenSection;
		}
	}
	this._onSectionCountChanged();
};

/** @public
* @param {number} frozenCount the amount of sections that will be fixed in place (from bottom up).
*/
Core.prototype.freezeFooter = function (frozenCount) {
	this._fixFrozenBottomSection = true;
	if (this._frozenFooterCount === frozenCount) return;
	this._setFrozenFooter(frozenCount);
	this._onSectionCountChanged();
};
/** @private
* @param {number} frozenCount
*/
Core.prototype._setFrozenFooter = function (frozenCount) {
	if (this._frozenFooterCount === frozenCount) return;
	this._frozenFooterCount = frozenCount;
	this._vscrollbar.setFooterCount(frozenCount); // Vertical scrollbar needs to know number of body sections for framing (excluding header and footer sections)
	this._hscrollbar.setFooterCount(frozenCount); // Horizontal scrollbar needs this for determining the scrollbar track width
};

/** @public
* @return {number} Total footer count
*/
Core.prototype.getFooterCount = function () {
	return this._frozenFooterCount;
};

/** Left columns will be frozen and horizontal scrollbar will automatically appear for the rest columns. This will happens only when the container of grid does NOT have enough space for the grid's content.
* @public
* @param {(number|null)=} frozenColIndex Last column index to be fixed/pinned on the left side. <br>
* Use negative value (-1) make horizontal scrollbar bar cover the first column. <br>
* Use null value to disable the scrollbar
* @param {number=} numRightColumn Number of columns on the right side to be fixed/pinned on the right side
* @example
* core.freezeColumn(0); // The first column is pinned to the left side
* core.freezeColumn(1); // The first two columns are pinned to the left side
* core.freezeColumn(-1); // Grid's horizontal scrollbar will cover entire are of the grid.
* core.freezeColumn(null); // Turn off grid's horizontal scrollbar
* core.freezeColumn(-1, 1); // The last column is pinned to the right side
* core.freezeColumn(-1, 2); // The last two columns are pinned to the right side
* core.freezeColumn(0, 2); // Grid's horizontal scrollbar will affect only the middle part
*/
Core.prototype.freezeColumn = function (frozenColIndex, numRightColumn) {
	let i, colDef;
	let colCount = this.getColumnCount();
	let leftPinnedCount = 0;
	let rightPinnedCount = 0;
	let dirty = false;
	if (frozenColIndex || frozenColIndex === 0) {
		this._hScrollbarEnabled = true;
		leftPinnedCount = (frozenColIndex >= 0) ? frozenColIndex + 1 : 0;
		if (this._pinnedLeftColumnCount !== leftPinnedCount) {
			dirty = true;
			this._pinnedLeftColumnCount = leftPinnedCount; // This variable is used for caching
		}

		for (i = 0; i < colCount; ++i) {
			colDef = this._getColumnDef(i);
			colDef["leftPinned"] = (--leftPinnedCount >= 0);
		}
	} else {
		this._hScrollbarEnabled = false;
	}
	if(numRightColumn != null) {
		rightPinnedCount = numRightColumn > 0 ? numRightColumn : 0;
		if (this._pinnedRightColumnCount !== rightPinnedCount) {
			dirty = true;
			this._pinnedRightColumnCount = rightPinnedCount; // This variable is used for caching
		}

		for (i = colCount; --i >= 0;) {
			colDef = this._getColumnDef(i);
			colDef["rightPinned"] = (--rightPinnedCount >= 0);
		}
	}

	this._onColumnCountChanged(); // Activate horizontal scrollbar and column virtualization
	this._updateScrollbarWidth(true, true);
	this._updateColumnSeparators();

	if (dirty) {
		if (!this._isEventDispatching("pinningChanged")) {
			this._dispatch("pinningChanged", {});
		}
	}
};

/** @private
* @return {number} Array of number of left pinned column count and right pinned column count
*/
Core.prototype._countPinnedLeftColumns = function () {
	let leftPinnedCount = 0;
	if(this._hScrollbarEnabled) {
		for (let i = this._columnDefs.length; --i >= 0;) {
			let colDef = this._columnDefs[i];
			if(colDef) {
				if(colDef["leftPinned"]) {
					leftPinnedCount = i + 1; // Use the first right most pinned column
					break;
				}
			}
		}
	}
	this._pinnedLeftColumnCount = leftPinnedCount; // This variable is used as a cache
	return leftPinnedCount;
};
/** @private
* @return {number} numRightColumn Number of columns on the right side to be fixed/pinned on the right side
*/
Core.prototype._countPinnedRightColumns = function () {
	let rightPinnedCount = 0;
	if(this._hScrollbarEnabled) {
		let colCount = this._columnDefs.length;
		let i;
		for(i = 0; i < colCount; ++i) {
			let colDef = this._columnDefs[i];
			if(colDef && colDef["rightPinned"]) {
				break;
			}
		}
		rightPinnedCount = colCount - i;
	}
	this._pinnedRightColumnCount = rightPinnedCount;
	return rightPinnedCount;
};

/** @public
* @param {number} colIndex
* @return {boolean} return weather colIndex in pinned or not
*/
Core.prototype.isPinnedColumn = function (colIndex) {
	return colIndex < this.getFrozenColumnCount() || colIndex >= this.getFirstPinnedRightIndex();
};
/** Returns number of pinned left columns.
* @public
* @return {number}
*/
Core.prototype.getFrozenColumnCount = function() {
	return this._hScrollbarEnabled ? this._pinnedLeftColumnCount : 0;
};
/** Returns number of pinned left columns. This is an alias to getFrozenColumnCount
* @public
* @function
* @return {number}
*/
Core.prototype.getPinnedLeftColumnCount = Core.prototype.getFrozenColumnCount;
/** Returns number of pinned right columns.
* @public
* @return {number}
*/
Core.prototype.getPinnedRightColumnCount = function() {
	return this._pinnedRightColumnCount;
};
/** @public
* @return {number} If there is no pinned right column, column count is returned
*/
Core.prototype.getFirstPinnedRightIndex = function () {
	return this.getColumnCount() - this._pinnedRightColumnCount;
};

/** Sometimes there is a change in layout without Grid noticing it (e.g tab pane is switched, or parent is hidden from view). <br>
* Hence, the layout must be manually and explicitly update by calling this method.
* @public
* @fires Core#widthChanged
*/
Core.prototype.updateLayout = function () {
	if(this._disposed || this._layoutUpdating) { return; }
	this._layoutUpdating = true;

	this._dispatch("preForcedUpdate", {});
	this._updateLayout();

	if(this._colVirtualizer.isEnabled()) {
		this._activateColumns();
	}

	this._layoutUpdating = false;
};

/** @public
* @param {boolean} updateHeight
* @param {boolean} updateWidth
*/
Core.prototype.updateScrollbars = function(updateHeight, updateWidth) {
	if (updateWidth) {
		this._updateScrollbarWidth(true, false);
	}

	if (updateHeight) {
		this._updateScrollbarHeight(true, false);
	}
};

/** Stop grid from updating layout to improve performance during multiple layout changes. Unfreeze the layout once all changes have been made.
* @public
* @param {boolean=} opt_bool
* @return {boolean} Previous freezing state (Used for restoring original state)
* @fires Core#widthChanged
* @see {@link Core#isLayoutFrozen}
*/
Core.prototype.freezeLayout = function (opt_bool) {
	opt_bool = opt_bool !== false;
	let prevState = this._frozenLayout;
	if (prevState !== opt_bool) {
		this._frozenLayout = opt_bool;

		let stp = this.getPlugin("SortableTitlePlugin");
		if(this._frozenLayout) {
			if(stp) {
				stp.freezeIndicator(true);
			}
		}

		if (!this._frozenLayout) {
			// Width has not yet changed so we need to disable it first
			this._disableEvent("widthChanged");
			this._onSectionCountChanged();
			this._onColumnCountChanged(); // Update horizontal scrollbar
			if(this._colVirtualizer.isEnabled()) {
				this._activateColumns();
			}
		}

		this._rowHeightSync = false;
		let s, section;
		let sectionCount = this._settings.length;
		for (s = 0; s < sectionCount; s++) {
			section = this._settings[s].getSection();
			section.setFrozenLayout(this._frozenLayout);
		}

		if (!this._frozenLayout) {
			this._syncLayoutToColumns(0, this.getColumnCount()); // Column layout should be synchronized after sections are unfrozen
			// Layout has been changed dispatch it
			this._disableEvent("widthChanged", false);
			this._dispatch("widthChanged", {});

			let viewChanged = this._syncRowHeights();
			if(!viewChanged) { // Always update virtualizer
				this._rowVirtualizer.update(true); // Force section activation
			}
			if(stp) {
				stp.freezeIndicator(false);
			}
		}
		this._rowHeightSync = true;
	}
	return prevState;
};
/** Deprecated. Please use freezeLayout instead
* @public
* @param {boolean=} opt_bool
* @fires Core#widthChanged
* @see {@link Core#freezeLayout}
*/
Core.prototype.unfreezeLayout = function (opt_bool) {
	this.freezeLayout(opt_bool === false);
};
/** @public
* @return {boolean}
*/
Core.prototype.isLayoutFrozen = function () {
	return this._frozenLayout;
};

/** Deprecated
* @public
* @ignore
* @function
* @param {boolean=} opt_shown
* @see {@link Core#toggleBorders}
*/
Core.prototype.showBorders = util._deprecatedFunction;
/** Deprecated
* @public
* @ignore
* @function
* @param {boolean=} opt_hidden
* @see {@link Core#toggleBorders}
*/
Core.prototype.hideBorders = util._deprecatedFunction;
/** Toggle borders around core grid element. WARNING: You will need to manually call updateLayout() method, if you call this method at run-time
* @public
* @param {boolean=} forcedVal If value is true, show border lines, otherwise hide them
*/
Core.prototype.toggleBorders = function(forcedVal) {
	this.toggleClass("no-borders", forcedVal != null ? !forcedVal : null);
};
/** Toggle both horizontal and vertical gridlines. This applies to ONLY content sections.
* @public
* @param {boolean=} forcedVal If value is true, show gridlines, otherwise hide them
*/
Core.prototype.toggleGridlines = function(forcedVal) {
	this.toggleClass("no-gridlines", forcedVal != null ? !forcedVal : null);
};
/** Toggle vertical gridlines. This applies to all sections.
* @public
* @param {boolean=} forcedVal If value is true, show vertical lines, otherwise hide them
*/
Core.prototype.toggleVerticalLines = function(forcedVal) {
	this.toggleClass("no-vertical-grid-line", forcedVal != null ? !forcedVal : null);
};
/** Toggle horizontal gridlines. This applies to all sections.
* @public
* @param {boolean=} forcedVal If value is true, show horizontal lines, otherwise hide them
*/
Core.prototype.toggleHorizontalLines = function(forcedVal) {
	this.toggleClass("no-horizontal-grid-line", forcedVal != null ? !forcedVal : null);
};
/** Toggle vertical gridlines. This applies to ONLY content sections.
* @public
* @param {boolean=} forcedVal If value is true, show horizontal lines, otherwise hide them
*/
Core.prototype.toggleContentVerticalLines = function(forcedVal) {
	this.toggleClass("no-content-vertical-line", forcedVal != null ? !forcedVal : null);
};


/** @public
* @return {Array.<string>}
*/
Core.prototype.getDataColumnMap = function () {
	let dataMap = [];
	for (let i = this.getColumnCount(); --i >= 0; ) {
		dataMap[i] = this.getDataColumnName(i);
	}
	return dataMap;
};
/** For indirectional one to one data binding
* @public
* @param {Array.<string>=} opt_dataMap index in the array will be matched with the display column
*/
Core.prototype.setDataColumnMap = function (opt_dataMap) {
	let i, colDef;
	if (opt_dataMap) {
		for (i = opt_dataMap.length; --i >= 0; ) {
			colDef = this._getColumnDef(i);
			colDef["dataColumnName"] = opt_dataMap[i];
		}
	} else {
		for (i = this._columnDefs.length; --i >= 0; ) {
			colDef = this._columnDefs[i];
			if (colDef && colDef["dataColumnName"] != null) {
				colDef["dataColumnName"] = null;
			}
		}
	}
};
/** @public
* @param {number} colIndex
* @return {string}
*/
Core.prototype.getDataColumnName = function (colIndex) {
	let colDef = this._getColumnDef(colIndex);
	let dataColumnName = colDef["dataColumnName"];
	return dataColumnName || "";
};
/** @public
* @param {number} colIndex
* @param {(string|null)=} opt_dataColumnName Use string for normal mapping, null to disable, and undefined for default mapping
*/
Core.prototype.setDataColumnName = function (colIndex, opt_dataColumnName) {
	this._getColumnDef(colIndex)["dataColumnName"] = opt_dataColumnName;
};

/** If the given column has no rendering handler, this default handler will be executed instead
* @public
* @param {Function} handler that takes json object as a first parameter
* @see {Core#setColumnRenderingHandler}
*/
Core.prototype.setDefaultRenderingHandler = function (handler) {
	this._renderingHandler = (typeof handler === "function") ? handler : null;
};
/** If the given column has no data binding handler, this handler will be executed instead
* @public
* @param {Function} handler that takes json object as a first parameter
* @see {Core#setColumnDataBindingHandler}
*/
Core.prototype.setDefaultDataBindingHandler = function (handler) {
	this._dataBindingHandler = (typeof handler === "function") ? handler : null;
};
/** @public
* @param {number} c
* @param {*} func
* @see {Core#setDefaultRenderingHandler}
*/
Core.prototype.setColumnRenderingHandler = function(c, func) {
	this._getColumnDef(c)["renderingHandler"] = (typeof func === "function") ? func : null;
};
/** @public
* @param {number} c
* @param {*} func
* @see {Core#setDefaultDataBindingHandler}
*/
Core.prototype.setColumnDataBindingHandler = function(c, func) {
	this._getColumnDef(c)["dataBindingHandler"] = (typeof func === "function") ? func : null;
};
/** @public
* @param {number} c
* @return {Function} func
*/
Core.prototype.getColumnRenderingHandler = function(c) {
	return this._getColumnDef(c)["renderingHandler"] || null;
};
/** @public
* @param {number} c
* @return {Function} func
*/
Core.prototype.getColumnDataBindingHandler = function(c) {
	return this._getColumnDef(c)["dataBindingHandler"] || null;
};

/** Rows will be highlighted when mouse is hoved over rows. This will works only with content section
* @public
* @param {boolean=} opt_enabled
*/
Core.prototype.enableRowHighlighting = function (opt_enabled) {
	if (opt_enabled == null) { opt_enabled = true; }

	if (this._rowHighlighting !== opt_enabled) {
		this._rowHighlighting = opt_enabled;
		let contentSections = this.getAllSections("content");
		for (let i = contentSections.length; --i >= 0; ) {
			contentSections[i].enableRowHighlighting(this._rowHighlighting);
		}
	}
};

/** Get target from the given object for hit testing
* @private
* @param {!ElementWrapper|Element|Event|MouseEvent} obj Element, Browser's Event object, Browser's Mouse Event object are all valid
* @return {Element}
*/
Core.prototype._getTarget = function (obj) {
	let targetEl = /** @type{Element} */(obj["target"]);
	if(targetEl) { // The Given object is an Event object
		let path = null;
		if(!this._element.contains(targetEl)) {
			if(obj["composedPath"]) { // Handle the case where grid is used in the Shadow Dom
				path = /** @type{Array.<Element>} */(obj["composedPath"]());
			} else { // 'path' property is an alternative to composedPath, but it is a non-standard property
				path = /** @type{Array.<Element>} */(obj["path"]);
			}
		}
		if(path) {
			return path[0] || null;
		} else {
			return targetEl;
		}
	} else {
		if(obj["getElement"]) { // The Given object is implementing ElementWrapper
			return /** @type{Element} */(obj["getElement"]());
		} else if(util.isElement(obj)){ // The Given object is just an element
			return /** @type{Element} */(obj);
		}
	}
	return null;
};
/** Return all information, from Grid and its internal components, related to the given position.
* @public
* @param {!ElementWrapper|Element|Event|MouseEvent} obj Element, Browser's Event object, Browser's Mouse Event object are all valid
* @param {Object=} context An object that stores returned values
* @return {!Core.MouseInfo}
*/
Core.prototype.getRelativePosition = function (obj, context) {
	if(!obj) {
		return {};
	}

	let ret_obj = context || {};
	let targetEl = this._getTarget(obj);

	// Calculate object's x and y coordinates that are relative to the top left of this grid
	if(obj["target"]) { // The Given object is an Event object
		util.getRelativePosition(/** @type{Event} */(obj), this.getElement(), ret_obj);
	} else if(targetEl) {
		util.getRelativePosition(targetEl, this.getElement(), ret_obj);
	}

	// The coordinates must be scaled by the zoom factor
	if (this._zoomFactor !== 1) {
		ret_obj["x"] /= this._zoomFactor;
		ret_obj["y"] /= this._zoomFactor;
	}

	ret_obj["hit"] = false;
	ret_obj["grid"] = this;
	ret_obj["clientX"] = obj["clientX"];
	ret_obj["clientY"] = obj["clientY"];
	ret_obj["zoomFactor"] = this._zoomFactor;

	// Find cell element
	let cellElement = util.closestElement(targetEl, "cell");
	if(!cellElement) {
		return ret_obj;
	}
	ret_obj["cellElement"] = cellElement;

	// Find column element
	let colElement = util.closestElement(cellElement, "column");
	ret_obj["columnElement"] = colElement;

	// Find section element
	let sectionElement = null;
	if (colElement) {
		// Column element may not be in the document (the column could be hidden)
		sectionElement = util.closestElement(colElement, "tr-lg");
	} else {
		// Floating cell does have the column
		sectionElement = util.closestElement(cellElement, "tr-lg");
	}

	if(!sectionElement) {
		return ret_obj;
	}
	ret_obj["sectionName"] = sectionElement["id"] || util.closestElement(sectionElement, "tr-vlg")["id"];

	let sectionSettings = this.getSectionSettings(ret_obj["sectionName"]);
	if (!sectionSettings) {
		return ret_obj; // Cannot find a section from the given id
	}
	sectionSettings.extendEventArg(ret_obj);

	// Find column index
	let section = sectionSettings.getSection();
	let colIndex = section.getColumnIndex(colElement || cellElement);
	if (colIndex < 0) {
		return ret_obj; // The given columnElement does not resolve in any column index
	}
	ret_obj["colIndex"] = colIndex;

	// Find row index
	if (this._hitTestY(-1, ret_obj) === false) { // hitTestY requires only sectionSettings, colIndex, and cellElement
		return ret_obj;
	}

	if(cellElement["stretched"]) {
		ret_obj["cell"] = section.getStretchedCell(colIndex, ret_obj["rowIndex"]);
	} else {
		ret_obj["cell"] = section.getCell(colIndex, ret_obj["rowIndex"]);
	}
	ret_obj["hit"] = true;
	return ret_obj;
};

/** @public
* @return {!ElementWrapper}
*/
Core.prototype.getVScrollbar = function () {
	return this._vscrollbar;
};
/** @public
* @return {!ElementWrapper}
*/
Core.prototype.getHScrollbar = function () {
	return this._hscrollbar;
};
/** @public
* @param {boolean=} opt_hide True if autohide (default) or false to keep scrollbars permanently
*/
Core.prototype.autoHideScrollbars = function(opt_hide){
	opt_hide = opt_hide != null ? opt_hide : true;
	this._vscrollbar.setAutoHide(opt_hide);
	this._hscrollbar.setAutoHide(opt_hide);
};
/** @public
* @ignore
* @return {number}
*/
Core.prototype.getVScrollStartIndex = function () {
	return this._startVScrollbarIndex;
};
/** @public
* @ignore
* @return {number}
*/
Core.prototype.getHScrollStartIndex = function () {
	return this._hScrollbarEnabled ? this._pinnedLeftColumnCount : -1;
};
/** @public
* @param {number} colIndex
* @param {boolean=} leftOfView Default is false. If true, always make to specified column at the left most of the view.
* @return {boolean} Return true, if there is any change
*/
Core.prototype.scrollToColumn = function (colIndex, leftOfView) {
	if(!this._updateScrollbar(this._hscrollbar)) {
		return false;
	}

	let colCount = this._layoutX.getLaneCount();
	if (colIndex <= 0) {
		colIndex = 0;
	} else if (colIndex >= colCount) {
		colIndex = colCount - 1;
	}
	if(this.isPinnedColumn(colIndex)) {
		return false;
	}

	let widthOffset = this._hScrollbarEnabled ? this._layoutX.getLaneStart(this._pinnedLeftColumnCount) : 0;
	let scrollLeft = this._hscrollbar.getScrollLeft();
	let viewLeft = scrollLeft + widthOffset;
	let viewWidth = this._hscrollbar.getWidth();
	let viewTopIndex = this._layoutX.hitTest(viewLeft);
	let laneLeft = this._layoutX.getLaneStart(colIndex);
	let laneSize = this._layoutX.getLaneSize(colIndex);

	if (leftOfView || laneSize >= viewWidth || colIndex <= viewTopIndex) {
		let toLeftOfView = laneLeft - widthOffset;
		if(scrollLeft !== toLeftOfView) {
			this._hscrollbar.setScrollLeft(toLeftOfView);
			return true;
		}
	} else {
		let viewRight = viewLeft + viewWidth;
		let viewRightIndex = this._layoutX.hitTest(viewRight - 0.1);
		if (colIndex >= viewRightIndex) {
			let laneRight = laneLeft + laneSize;
			let toEndOfView = scrollLeft + laneRight - viewRight; // Fit end of column into the view

			this._hscrollbar.setScrollLeft(toEndOfView);
			return true;
		}
	}
	return false;
};
/** @public
* @override
* @return {number} pixels
*/
Core.prototype.getScrollLeft = function () {
	return this._updateScrollbar(this._hscrollbar) ? this._hscrollbar.getScrollLeft() : 0;
};
/** @public
* @param {number} pixels
*/
Core.prototype.setScrollLeft = function (pixels) {
	if(!this._updateScrollbar(this._hscrollbar)) {
		return;
	}

	this._hscrollbar.setScrollLeft(pixels);
};
/** @public
* @param {number} pixels
*/
Core.prototype.scrollRight = function (pixels) {
	this.setScrollLeft(this._hscrollbar.getScrollLeft() + pixels);
};
/** @public
 * @returns {number}
 */
Core.prototype.getScrollWidth = function () {
	if(!this._updateScrollbar(this._hscrollbar)) {
		return 0;
	}
	return this._hscrollbar.getContentWidth();
};
/** Return the scroll value (in pixels) to make specified row visible in the view
* @public
* @param {Core.SectionReference} sectionRef Specify section that the given rowIndex is related to. Use null for the first scrollable section (first non-frozen section)
* @param {number} rowIndex
* @param {boolean=} topOfView=false. If true, Make to specified row at the top of the view.
* @returns {number|null} The scroll value (in pixels)
*/
Core.prototype.getYScrollVal = function (sectionRef, rowIndex, topOfView) {
	if(!this._updateScrollbar(this._vscrollbar)) {
		return null;
	}
	let section = null;
	if(sectionRef) {
		section = this.getSection(sectionRef);
	}

	let rowIndexOffset = (section) ? section.getRowOffset() : this._sectionStarts[this._startVScrollbarIndex];
	let heightOffset = this._layoutY.getLaneStart(this._startVScrollbarIndex);

	let rowCount = this._layoutY.getLaneCount();
	if (rowIndex <= 0) { rowIndex = 0; }
	else if (rowIndex >= rowCount) { rowIndex = rowCount - 1; }

	if (topOfView) {
		return this._layoutY.getLaneStart(rowIndex + rowIndexOffset) - heightOffset;
	}

	let bufferSize = 2; // for more appealing space
	let viewInfo = this.getVerticalViewInfo();
	let firstFullRow = viewInfo.firstFullRow; // TODO: Make it work in zooming mode
	if(rowIndex < firstFullRow) { // Scroll up, as the target row is outside of view
		let targetIndex = rowIndex - bufferSize; // Have some spaces at the top for more appealing visual
		if(targetIndex < 0) {
			targetIndex = 0;
		}

		return this._layoutY.getLaneStart(targetIndex + rowIndexOffset) - heightOffset;
	}

	let lastFullRow = viewInfo.lastFullRow;
	if (rowIndex > lastFullRow) { // Scroll down, as the target row is outside of view
		let targetScroll = this._layoutY.getLaneStart(rowIndex + rowIndexOffset + bufferSize + 1);

		return targetScroll - viewInfo.viewHeight - heightOffset;
	}

	return null;
};
/** Scroll up or down to make specified row visible in the view
* @public
* @param {Core.SectionReference} sectionRef Specify section that the given rowIndex is related to. Use null for the first scrollable section (first non-frozen section)
* @param {number} rowIndex
* @param {boolean=} topOfView=false. If true, Make to specified row at the top of the view.
*/
Core.prototype.scrollToRow = function (sectionRef, rowIndex, topOfView) {
	let scrollVal = this.getYScrollVal(sectionRef, rowIndex, topOfView);
	if(scrollVal !== null) {
		this._vscrollbar.setScrollTop(scrollVal);
	}
};
/** @public
* @return {Object}
*/
Core.prototype.getVerticalViewInfo = function() {
	let rowIndexOffset = this._sectionStarts[this._startVScrollbarIndex];
	let heightOffset = this._layoutY.getLaneStart(rowIndexOffset);
	let viewHeight = this._vscrollbar.getHeight();
	let viewTop = this._vscrollbar.getScrollTop() + heightOffset;
	let viewBottom = viewTop + viewHeight;

	let topRowIndex = this._layoutY.hitTest(viewTop) - rowIndexOffset;
	let bottomRowIndex = this._layoutY.hitTest(viewBottom - 0.1);
	if(bottomRowIndex < 0) { // view is larger than existing row count
		bottomRowIndex = this._layoutY.getLaneCount() - 1;
	}
	bottomRowIndex -= rowIndexOffset;

	let laneTop = this._layoutY.getLaneStart(topRowIndex + rowIndexOffset);
	let laneBottom = this._layoutY.getLaneEnd(bottomRowIndex + rowIndexOffset);

	return {
		topRowIndex: topRowIndex,
		firstFullRow: (laneTop < viewTop) ? topRowIndex + 1 : topRowIndex,
		lastFullRow: (laneBottom > viewBottom) ? bottomRowIndex - 1 : bottomRowIndex,
		bottomRowIndex: bottomRowIndex,
		viewHeight: viewHeight,
		viewTop: viewTop,
		viewBottom: viewBottom
	};
};
/** @public
* @return {Object} Returns null if vscrollbar does not exists
*/
Core.prototype.getVScrollView = function () {
	if(!this._updateScrollbar(this._vscrollbar)) {
		return null;
	}

	let rowCount = this._layoutY.getLaneCount();
	if (rowCount === 0) { return null; }

	let rowIndexOffset = this._sectionStarts[this._startVScrollbarIndex];
	let heightOffset = this._layoutY.getLaneStart(rowIndexOffset);
	let viewTop = this._vscrollbar.getScrollTop() + heightOffset;
	let viewHeight = this._vscrollbar.getHeight();
	let viewBottom = viewTop + viewHeight;
	let topRowIndex = this._layoutY.hitTest(viewTop);
	let bottomRowIndex = this._layoutY.hitTest(viewBottom - 0.1);

	let laneTop = this._layoutY.getLaneStart(topRowIndex + rowIndexOffset);
	let laneBottom = this._layoutY.getLaneEnd(bottomRowIndex + rowIndexOffset);

	let prevAnchor = viewTop - viewHeight;
	let prevPage = (prevAnchor <= heightOffset) ? rowIndexOffset : this._layoutY.hitTest(prevAnchor) + 1;
	let nextAnchor = viewBottom + viewHeight;
	let trackSize = this._layoutY.getTrackSize();
	let nextPage = (nextAnchor >= trackSize) ? rowCount - 1 : this._layoutY.hitTest(nextAnchor - 0.1) - 1;

	topRowIndex -= rowIndexOffset;
	bottomRowIndex -= rowIndexOffset;
	let obj = {};
	obj["viewTop"] = viewTop - heightOffset;
	obj["viewBottom"] = viewBottom - heightOffset;
	obj["topRowIndex"] = topRowIndex;
	obj["bottomRowIndex"] = bottomRowIndex;
	obj["firstFullRow"] = (laneTop < viewTop) ? topRowIndex + 1 : topRowIndex;
	obj["lastFullRow"] = (laneBottom > viewBottom) ? bottomRowIndex - 1 : bottomRowIndex;
	obj["prevPageRowIndex"] = prevPage - rowIndexOffset;
	obj["nextPageRowIndex"] = nextPage - rowIndexOffset;
	return obj;
};
/** @public
* @override
* @return {number} pixels
*/
Core.prototype.getScrollTop = function () {
	return this._updateScrollbar(this._vscrollbar) ? this._vscrollbar.getScrollTop() : 0;
};
/** @public
* @param {number} pixels
*/
Core.prototype.setScrollTop = function (pixels) {
	if(!this._updateScrollbar(this._vscrollbar)) {
		return;
	}

	this._vscrollbar.setScrollTop(pixels);
};
/** @public
* @param {number} pixels
*/
Core.prototype.scrollDown = function (pixels) {
	this.setScrollTop(this._vscrollbar.getScrollTop() + pixels);
};
/** @public
 * @returns {number}
 */
Core.prototype.getScrollHeight = function () {
	if(!this._updateScrollbar(this._vscrollbar)) {
		return 0;
	}
	return this._vscrollbar.getContentHeight();
};
/** @public
*/
Core.prototype.restoreScrollbars = function () {
	this._vscrollbar.restoreTrackPosition();
};

/** @public
* @ignore
* @param {boolean} bool
*/
Core.prototype.enableAutoSetDataSource = function (bool) {
	this._autoSetDataSource = bool;
};

/** @public
* @return {!Virtualizer}
*/
Core.prototype.getRowVirtualizer = function () {
	return this._rowVirtualizer;
};


/** @public
* @return {!Virtualizer}
*/
Core.prototype.getColumnVirtualizer = function () {
	return this._colVirtualizer;
};

/** No runtime support
* @public
* @param {boolean} opt_enabled
*/
Core.prototype.enableRowVirtualization = function (opt_enabled) {
	this._rowVirtualizer.enable(opt_enabled);
	if (this._rowVirtualizer.isEnabled()) {
		if (this._vscrollbar.isEnabled()) {
			this._rowVirtualizer.activate();
		} else {
			this._rowVirtualizer.deactivate();
		}
	} else {
		this._rowVirtualizer.deactivate();
	}
};

/** @public
* @return {boolean}
*/
Core.prototype.isRowVirtualizationEnabled = function () {
	return this._rowVirtualizer.isEnabled();
};

/** No runtime support
* @public
* @param {boolean=} opt_enabled
*/
Core.prototype.enableColumnVirtualization = function (opt_enabled) {
	this._colVirtualizer.enable(opt_enabled);

	let colVir = this._colVirtualizer.isEnabled();
	for (let i = this._settings.length; --i >= 0; ) {
		let sectionSettings = this._settings[i];
		sectionSettings.enableColumnVirtualization(colVir);
	}
};

/** @public
* @return {boolean}
*/
Core.prototype.isColumnVirtualizationEnabled = function () {
	return this._colVirtualizer.isEnabled();
};

/** Scrolling step will be multiple of the default row height in pixels. For example, if default row height is 32 pixel and row scrolling step is 2, <br>
* then the actual scrolling step for vertical scrollbar will be (32 * 2) = 64 pixels
* @public
* @param {number} rowCount Number of rows for a single scrolling step. Use zero to disable this feature.
*/
Core.prototype.setRowScrollingStep = function (rowCount) {
	this._rowScrollingStep = (rowCount > 0) ? rowCount : 0;
	this._vscrollbar.setScrollingStep(this._layoutY.getDefaultLaneSize() * this._rowScrollingStep);
};

/** Allows other elements such as background to accept wheel event and scroll grid's vertical scrollbar
* @public
* @return {function(Event)}
*/
Core.prototype.getVMouseWheelHandler = function () {
	return this._vscrollbar.getDefaultMouseWheelLogic();
};
/** Synchronize two vertical scrollbars of two grid, by hiding its scrollbar and using the one from the given grid
* @public
* @param {Core} subGrid
*/
Core.prototype.synchronizeVScrollbar = function (subGrid) {
	subGrid.unlisten("mousemove", subGrid._onMouseMove);
	subGrid.listen("mousemove", this._onMouseMove);

	let vscrollbar = subGrid.getVScrollbar();
	vscrollbar._mainScrollbar = this._vscrollbar; // HACK

	vscrollbar.setStyle("visibility", "hidden");
	vscrollbar.setStyle("pointerEvents", "none");
	vscrollbar.disableMouseWheel(); // Disable sub-grid wheel behavior
	vscrollbar.attachToExternalElement(this._vscrollbar.getParent()); // MouseWheel event is still available on the main

	// TODO: Check if we need to re-append this._vscrollbar to move it to the front over other grid elements.
	this._vscrollbar.listen("scroll", subGrid._onSyncVScroll);
};

/** Synchronize two horizontal scrollbars of two grid, by hiding its scrollbar and using the one from the given grid
* @public
* @param {Core} subGrid
*/
Core.prototype.synchronizeHScrollbar = function (subGrid) {
	subGrid.unlisten("mousemove", subGrid._onMouseMove);
	subGrid.listen("mousemove", this._onMouseMove);

	let hscrollbar = subGrid.getHScrollbar();
	hscrollbar._mainScrollbar = this._hscrollbar; // HACK

	hscrollbar.setStyle("visibility", "hidden");
	hscrollbar.setStyle("pointerEvents", "none");
	hscrollbar.disableMouseWheel(); // Disable sub-grid wheel behavior
	hscrollbar.attachToExternalElement(this._hscrollbar.getParent()); // MouseWheel event is still available on the main

	// TODO: Check if we need to re-append this._hscrollbar to move it to the front over other grid elements.
	this._hscrollbar.listen("scroll", subGrid._onSyncHScroll);
};

/** Fires data binding event without actual change in the data source. <br>
* This will force visual elements to be re-rendered with the latest data in the data source.
* @public
* @param {Core.SectionReference=} sectionRef If not defined, "content" is used
* @param {number=} fromRowIndex INCLUSIVE If the value is undefined, the first row index will be used
* @param {number=} lastRowIndex INCLUSIVE If the value is undefined, the last row index will be used
* @param {Object=} userParam Addtional parameters to be fired with the event
* @fires Core#beforeContentBinding
* @fires Core#preSectionDataBinding
* @fires Core#columnDataBinding
* @fires Core#postSectionDataBinding
* @fires Core#afterContentBinding
*/
Core.prototype.updateRowData = function (sectionRef, fromRowIndex, lastRowIndex, userParam) {
	let ss = this.getSectionSettings(sectionRef || "content");
	if (ss) {
		ss.updateRowData(fromRowIndex, lastRowIndex, userParam);
	}
};
/** Re-render all sections (headers and contents) in grid. Layout will not be affected by this method
* @public
*/
Core.prototype.rerender = function () {
	let sectionCount = this._settings.length;
	for (let s = 0; s < sectionCount; s++) { // for each section
		this._settings[s].rerender();
	}
};
/** Set a timer to call updateRowData only once to avoid performance issue due to multiple call of updateRowData()
* @public
*/
Core.prototype.requestRowRefresh = function() {
	if(!this._rowRefreshTimer) {
		this._rowRefreshTimer = setTimeout(this._onRowRefresh, 100);
	}
};
/** Set a timer to call updateScrollbarHeight only once to avoid performance issue due to multiple call of _updateScrollbarHeight()
* @private
*/
Core.prototype._requestScrollbarUpdate = function() {
	this._updateScrollbarHeight(true, true);
	this._rowHeightTimerId = 0;
};

/** prevent bind data process
* @public
* @param {boolean=} disabled
*/
Core.prototype.deactivateRendering = function(disabled) {
	disabled = disabled !== false;

	// if option value not changed then do nothing
	if (disabled == this._noBinding) return;

	this._noBinding = disabled;
	let contentSection = this.getSectionSettings("content");
	if (!contentSection) return;

	contentSection.disableDataBinding(disabled);

	if (!disabled) {
		this.requestRowRefresh();
	}
};

/** @private */
Core.prototype._onRowRefresh = function() {
	this._rowRefreshTimer = 0;
	this.updateRowData();
};

/** @public
* @ignore
* @param {boolean=} opt_bool
*/
Core.prototype.disableLazyRendering = function(opt_bool) {
	console.log("WARNING: This function is deprecated.");
};

/** @param {Core.SectionReference} sectionRef
* @param {number|string} colRef Either column index or data column name
* @param {number|string} rowRef Row Index of the display section or row id of the data table
* @return {*} Returns undefined if not success
*/
Core.prototype.getData = function (sectionRef, colRef, rowRef) {
	let sectionSettings = this.getSectionSettings(sectionRef);
	if (!sectionSettings) { return null; }

	let colName = this._toDataColumnName(colRef);

	return sectionSettings.getData(colName, rowRef);
};
/** Do not used this method too often. This method is slow. It is always better to set data directly to the data table
* @param {Core.SectionReference} sectionRef
* @param {number|string} colRef Either column index or data column name
* @param {number|string} rowRef Row Index of the display section or row id of the data table
* @param {*} data
* @return {boolean} True for successful data setting
*/
Core.prototype.setData = function (sectionRef, colRef, rowRef, data) {
	let sectionSettings = this.getSectionSettings(sectionRef);
	if (sectionSettings && sectionSettings.isDataBindable()) {
		let colName = this._toDataColumnName(colRef);
		return sectionSettings.setData(colName, rowRef, data);
	}
	return false;
};

/** Do not used this method too often. This method is slow. It is always better to set data directly to the data table
* @param {Core.SectionReference} sectionRef
* @param {number|string} rowRef Row Index of the display section or row id of the data table
* @return {boolean} True for successful data setting
*/
Core.prototype.removeRowData = function (sectionRef, rowRef) {
	let sectionSettings = this.getSectionSettings(sectionRef);
	if (sectionSettings && sectionSettings.isDataBindable()) {
		return sectionSettings.removeRowData(rowRef);
	}
	return false;
};

/** @param {string} method */
Core.prototype.setHidingMethod = function (method) {
	if(!method) { method = ""; }
	if(this._hidingMethod !== method) {
		this._hidingMethod = method;
		let sections = this.getAllSections();
		for(let i = sections.length; --i >= 0;) {
			sections[i].setHidingMethod(method);
		}
	}
};

/** @public
* @ignore
* @return {!TrackLayout}
*/
Core.prototype.getHorizontalLayout = function () {
	return this._layoutX;
};
/** @public
* @ignore
* @param {number} colIndex
* @param {boolean} bool
* @param {number} flag
* @return {boolean}
*/
Core.prototype.setColumnVisibility = function (colIndex, bool, flag) {
	if (this._layoutX.hideLane(colIndex, !bool, flag)) {
		this._syncLayoutToColumns(colIndex, colIndex + 1);

		if (bool) { // If column turns back to visible, update ui. TODO: This will cause a rendering issue for column grouping
			this.requestRowRefresh();
		}
		if (this._hasListener("columnVisibilityChanged")) {
			let e = {};
			e["colIndex"] = colIndex;
			e["hidden"] = !bool;
			this._dispatch("columnVisibilityChanged", e);
		}
		return true;
	}
	return false;
};
/** Get visibility state for the specified flag. This is for internal use only
* @public
* @ignore
* @param {number} colIndex
* @param {number=} flag
* @return {boolean}
*/
Core.prototype.getColumnVisibility = function (colIndex, flag) {
	return this._layoutX.getLaneVisibilityBit(colIndex, flag);
};

/** @public
* @param {number} size
* @return {boolean}
*/
Core.prototype.reserveRightSpace = function (size) {
	if(this._reservedSpace !== size) {
		this._reservedSpace = size;
		for(let i = this._settings.length; --i >= 0;) {
			this._settings[i].getSection().reserveRightSpace(size);
		}
		this._onColumnCountChanged(); // Activate horizontal scrollbar and column virtualization
		this._updateScrollbarWidth(true, true);
		return true;
	}
	return false;
};

/** Get the hidden input. This input allows grid to receive keyboard input
* @public
* @ignore
* @param {boolean} firstInput
* @return {!Element}
*/
Core.prototype.getHiddenInput = function (firstInput) {
	return firstInput ? this._firstHiddenInput : this._lastHiddenInput;
};

/** Focus grid element without bringing grid into window's view. This is useful when grid is very wide or tall, since window can be scrolled to focused element by default in some browsers.
* @public
* @see {@link https://developer.mozilla.org/en/docs/Web/API/HTMLElement/focus}
* @see {@link http://help.dottoro.com/ljqmdirr.php}
*/
Core.prototype.focus = function () {
	let elem = this._lastHiddenInput;
	let activeElem = _getActiveElement(elem);
	if(elem && elem !== activeElem) {
		let x = window.pageXOffset;
		let y = window.pageYOffset;
		if(elem["setActive"]) { // Non-standard IE specific method
			try { // MON-909
				elem["setActive"]();
			} catch (error) {
				elem["focus"]({"preventScroll": true});
			}
		} else {
			elem["focus"]({"preventScroll": true});
		}
		if(y !== window.pageYOffset || x !== window.pageXOffset) {
			window.scrollTo(x, y); // Restore scrolling
		}
	}
};

/** @public
* @return {boolean}
*/
Core.prototype.isBinding = function() {
	return this._dispatchingDataChanged;
};

/**
 * Hide the specified rows by row id
 * @param {number | string} rowId rowId of the row to hide
 * @param {boolean=} hidden=true If the value is false, the specified rows will be shown.
 * @public
 */
Core.prototype.hideRow = function (rowId, hidden) {
	this._dataSource.hideRow(rowId, hidden);
};
/**
 * Hide the specified rows by rowIds
 * @param {number | string | Array.<number | string>} rowIds rowId(s) of the rows to hide
 * @param {boolean=} hidden=true If the value is false, the specified rows will be shown.
 * @public
 */
Core.prototype.hideRows = function (rowIds, hidden) {
	if (!Array.isArray(rowIds)) {
		this._dataSource.hideRow(/** @type{number | string} */(rowIds), hidden);
	} else {
		this._dataSource.hideRows(rowIds, hidden);
	}
};

/**
 * Show the specified rows by rowIds. To show all rows, use unhideAllRows method for better performance.
 * @param {number | string | Array.<number | string>} rowIds rowId(s) of the rows to show
 * @param {boolean=} shown=true If the value is false, the specified rows will be hidden.
 * @public
 */
Core.prototype.showRows = function (rowIds, shown) {
	if (!Array.isArray(rowIds)) {
		this._dataSource.showRows([rowIds], shown);
	} else {
		this._dataSource.showRows(rowIds, shown);
	}
};

/**
 * Unhide all hidden rows in the Grid
 * @public
 */
Core.prototype.unhideAllRows = function () {
	this._dataSource.unhideAllRows();
};

/**
 * @public
 * @return {boolean}
 */
Core.prototype.hasHiddenRow = function () {
	return this._dataSource.hasHiddenRow();
};

/** @public
 * @ignore
 * @param {Array=} columns
 */
Core.prototype.initSimpleTable = function (columns) {
	let colCount = 0;
	if(!columns && !this.getColumnCount()) {
		columns = ["Default"]; // Row cannot be added unless there is at least 1 column
	}

	if(columns) {
		colCount = columns.length;
		this.setColumnCount(colCount);
		this.setDataColumnMap(columns);
		for(let c = 0; c < colCount; ++c) {
			this.setColumnField(c, columns[c]);
		}
	}

	let titleSect = this.addSection("title");
	titleSect.addRow(1);

	if(!this._renderingHandler) { // Render title if there is no default rendering
		for(let c = 0; c < colCount; ++c) {
			titleSect.setCellContent(c, 0, columns[c]);
		}
	}

	this.addSection("content");
	let titleSectionSetting = this.getSectionSettings("title");
	titleSectionSetting.disableDataBinding();

	let contentSectionSetting = this.getSectionSettings("content");
	contentSectionSetting.setAutoSyncRowCount(true);
	contentSectionSetting.setParent(titleSectionSetting);

	if(!this._dataBindingHandler) {
		this.setDefaultDataBindingHandler(Core._defaultDataBinding);
	}
};
/** @private
 * @param {Object} e
 */
Core._defaultDataBinding = function(e) {
	e.cell.setContent(e.dataValue);
};

/** @public
* @param {number} colIndex
* @param {boolean=} selected
*/
Core.prototype.selectColumn = function (colIndex, selected) {
	for (let i = this._settings.length; --i >= 0; ) {
		this._settings[i].getSection().selectColumn(colIndex, selected);
	}
	this.updateColumnBounds();
};
/** @public
* @param {number} colIndex
* @return {boolean}
*/
Core.prototype.isSelectedColumn = function (colIndex) {
	let first = this._settings[0] || null;
	if(first) {
		return first.getSection().isSelectedColumn(colIndex);
	}
	return false;
};

/** @public
*/
Core.prototype.updateColumnBounds = function () {
	if(this._columnBoundConflator.conflate()) {
		return;
	}

	let sectCount = this._settings.length;
	if(!sectCount) {
		return;
	}

	// Collecting column selection and selection ranges
	let selRanges = [];
	let pair = null;
	let colCount = this.getColumnCount();
	let selIndices = [];
	let i;
	for(i = 0; i < colCount; i++) {
		if(this.isSelectedColumn(i)) {
			selIndices.push(i);
			if(!pair) {
				pair = [i, -1];
			}
		} else if(pair) {
			pair[1] = i - 1;
			selRanges.push(pair);
			pair = null;
		}
	}
	if(pair) {
		pair[1] = colCount - 1;
		selRanges.push(pair);
		pair = null;
	}

	let arg = {
		"selectedColumns": selIndices,
		"selectionRanges": selRanges
	};
	this._dispatch("beforeColumnBoundUpdate", arg);

	// Calculate position from ranges
	let rangeCount = selRanges.length;
	let posAry = [];
	let noBorderAry = [];
	let topSectionSettings = this._settings[0];
	let section = topSectionSettings.getSection();
	for(i = 0; i < rangeCount; ++i) {
		pair = selRanges[i];
		let positions = [0, 0];
		let noBorders = [false, false];
		section.calculateColumnBounds(pair[0], pair[1], positions, noBorders);
		if(positions[0] < positions[1]) {
			posAry.push(positions);
			noBorderAry.push(noBorders);
		}
	}

	// Render column bounds
	let topPx = 0;
	let topBoundIdx = -1;
	if(arg["topBoundRowIndex"] != null) {
		topBoundIdx = +arg["topBoundRowIndex"];
	}
	if(topBoundIdx >= 0 && topSectionSettings.getType() === "title") {
		topPx = this._layoutY.getLaneStart(topBoundIdx);
	}
	section.updateColumnBounds(posAry, noBorderAry, topPx);
	for(i = 1; i < sectCount; i++) {
		section = this._settings[i].getSection();
		section.updateColumnBounds(posAry, noBorderAry);
	}
};
/* @private
*/
Core.prototype._updateColumnSeparators = function() {
	let sectCount = this._settings.length;
	if(!sectCount) {
		return;
	}

	for(let i = 0; i < sectCount; i++) {
		let section = this._settings[i].getSection();
		if (section) {
			section.updateColumnSeparators();
		}
	}
};

/** @public
* @param {number} startColIndex INCLUSIVE
* @param {number} endColIndex INCLUSIVE
* @return {Object}
*/
Core.prototype.getColumnRect = function (startColIndex, endColIndex) {
	let gridWidth = this.getElement().clientWidth;
	let scrollViewSize = gridWidth;
	let hscrollbar = this._hscrollbar;
	if (hscrollbar.isActive()) {
		scrollViewSize = hscrollbar.getTrackSize();
	}

	let left = 0;
	let width = 0;
	if (this._settings.length) {
		let section = this._settings[0].getSection();
		let positions = [0, 0];
		section.calculateColumnBounds(startColIndex, endColIndex, positions, []);
		left = positions[0];
		width = positions[1] - left;
	}
	let rect = {
		top: 0, // TODO: Support column grouping
		left: left,
		width: width,
		height: this._layoutY.getTrackSize(),
		leftPinnedIndex: this._pinnedLeftColumnCount - 1,
		rightPinnedIndex: this.getFirstPinnedRightIndex(),
		scrollViewSize: scrollViewSize,
		gridWidth: gridWidth
	};
	return rect;
};

/** @public
* @param {number} startRowIndex INCLUSIVE
* @param {number} endRowIndex INCLUSIVE
* @return {Object}
*/
Core.prototype.getRowRect = function (startRowIndex, endRowIndex) {
	let contentWidth = this._layoutX.getTrackSize();
	let gridElem = this.getElement();
	let gridWidth = gridElem.clientWidth;
	contentWidth = contentWidth < gridWidth ? contentWidth : gridWidth;

	let section = this.getSection("content");
	let rowOffset = section ? section.getRowOffset() : 1;
	let layoutY = this._layoutY;
	let sectionOffsetPx = layoutY.getLaneStart(rowOffset);

	let scrollTop = 0;
	let scrollViewSize = section.getHeight();
	let vscrollbar = this._vscrollbar;
	if (vscrollbar.isActive()) {
		scrollTop = vscrollbar.getScrollTop();
		scrollViewSize = vscrollbar.getTrackSize();
	}
	let scrollViewBottom = scrollViewSize + sectionOffsetPx;

	let topPx = layoutY.getLaneStart(startRowIndex + rowOffset) - scrollTop;
	let bottomPx = layoutY.getLaneEnd(endRowIndex + rowOffset) - scrollTop;
	if (topPx < sectionOffsetPx) {
		topPx = sectionOffsetPx;
	} else if (topPx > scrollViewBottom) {
		topPx = scrollViewBottom;
	}
	if (bottomPx < sectionOffsetPx) {
		bottomPx = sectionOffsetPx;
	} else if (bottomPx > scrollViewBottom) {
		bottomPx = scrollViewBottom;
	}

	// Change origin point from grid to section
	topPx -= sectionOffsetPx;
	bottomPx -= sectionOffsetPx;

	let rect = {
		top: topPx,
		left: 0,
		width: contentWidth,
		height: bottomPx - topPx,
		scrollViewSize: scrollViewSize,
		scrollViewTop: sectionOffsetPx,
		scrollViewBottom: scrollViewBottom
	};
	return rect;
};
/** @public
* @param {Object} configObj
* @return {!Object}
*/
Core.prototype.normalizeConfig = function (configObj) {
	if(!configObj) {
		configObj = {};
	}
	let columns = configObj["columns"];
	let colCount = columns ? columns.length : 0;
	let i, column;

	let freezingIndex = configObj["columnFreezing"];
	if(freezingIndex == null) {
		freezingIndex = configObj["freezeColumn"];
	}
	let pinnedRightColumns = configObj["pinnedRightColumns"] || configObj["frozenRightColumns"];
	if (pinnedRightColumns) {
		delete configObj["pinnedRightColumns"];
		delete configObj["frozenRightColumns"];

		configObj["columnVirtualRendering"] = false;
		configObj["columnVirtualization"] = false;

		for(i = colCount; --i >= 0;) {
			if(--pinnedRightColumns >= 0) {
				column = columns[i];
				if(column) {
					column["rightPinned"] = true;
				}
			}
		}
	}
	if(freezingIndex != null) {
		delete configObj["columnFreezing"];
		delete configObj["freezeColumn"];

		if(freezingIndex >= 0) {
			if(columns) {
				column = columns[freezingIndex];
				if(column) {
					column["leftPinned"] = true;
				}
			}
		}
	}

	return configObj;
};
/** @public
* @param {GroupDefinitions} groupDefs
*/
Core.prototype.setColumnGrouping = function (groupDefs) {
	this._groupDefs = groupDefs || null;
};
/** @public
* @param {string|number} colRef
* @return {string} Return group id of immediate group parent. Return empty string, if no parent is found.
*/
Core.prototype.getColumnGroupParentId = function (colRef) {
	if(this._groupDefs) {
		let colId = (typeof colRef === "number") ? this.getColumnId(colRef) : colRef;
		return this._groupDefs.getParentId(colId);
	}
	return "";
};

/** @public
* @param {string} groupId
* @return {Array.<string>} Return array of column id. Return null if the specified group has no child
*/
Core.prototype.getColumnGroupChildIds = function (groupId) {
	if(this._groupDefs) {
		let ary = this._groupDefs.getLeafDescendants(groupId);
		if(ary && ary.length > 0) {
			return ary;
		}
	}
	return null;
};

/** @public
* @description Get a list of objects with column id and column index in sorted order
* @param {Array.<string>} colIds
* @param {Object=} columnMap
* @return {!Array.<string>} Return column array with corresponding order to UI
* @example
* core.getValidColumnList(["c1","c2","c5"]); // Get list of valid columns
* core.getValidColumnList(["c1","c2","c5"],{ "c2":true, "c5":true }); // Get list of valid columns from specific mapping
*/
Core.prototype.getValidColumnList = function (colIds, columnMap) {
	let colList = [];
	if(!colIds){
		return colList;
	}
	if(!columnMap){
		columnMap = this.createColumnMap(colIds);
	}
	let colCount = this.getColumnCount();
	for(let c = 0; c < colCount; ++c) {
		let id = this._getColumnDef(c)["id"];
		if(columnMap[id] != null){
			colList.push({"index": c, "id": id});
		}
	}
	return colList;
};

/** @public
* @description Create mapping object from an array of strings or column indices
* @param {Array.<string|number>=} colRefs
* @return {!Object} Column mapping object
*/
Core.prototype.createColumnMap = function (colRefs) {
	if(!colRefs){
		colRefs = this.getColumnIds();
	}
	let mappingObj = {};
	let count = colRefs.length;
	for(let i = 0; i < count; i++){
		let colId = colRefs[i];
		if(typeof colId ===  "number"){
			colId = this.getColumnId(colId);
		}
		if(colId){
			mappingObj[colId] = true;
		}
	}
	return mappingObj;
};

/** @public
* @param {string} batchType
* @return {boolean}
* @fires Core#beforeBatchOperation
*/
Core.prototype.startBatch = function (batchType) {
	if(!batchType){
		return false;
	}
	if(!this._batches){
		this._batches = {};
	}
	this._batches[batchType] = batchType;
	// The "columnVisibilityChanged" event is blocked while the "reset" batch operation is in progress, so this event will not be fired until the batch operation succeeds.
	if(batchType === "reset") {
		this._disableEvent("columnVisibilityChanged", true);
	}
	this._dispatch("beforeBatchOperation", { batches: this._batches, batchType: batchType });
	return true;
};
/** @public
* @param {string} batchType
* @return {boolean}
* @fires Core#afterBatchOperation
*/
Core.prototype.stopBatch = function (batchType) {
	if(!batchType){
		return false;
	}
	this._dispatch("afterBatchOperation", { batches: this._batches, batchType: batchType });
	if(batchType === "reset") {
		this._disableEvent("columnVisibilityChanged", false);
	}

	delete this._batches[batchType];
	if(isEmptyObject(this._batches)){
		this._batches = null;
	}
	return true;
};
//#endregion Public Methods

//#region Private Methods
/** @private
* @param {string} type
* @param {!Object} eventArg
*/
Core.prototype._dispatchColumnEvent = function (type, eventArg) {
	let batches = this._batches;
	if(batches){
		eventArg["batches"] = batches;
	}
	this._dispatch(type, eventArg);
};
/** @private
*/
Core.prototype._dispatchColumnPositionChanged = function () {
	if(this._columnPositionConflator.conflate()) {
		return;
	}

	this._dispatch("columnPositionChanged", { sender: this });
};

/** @private
*/
Core.prototype._dispatchRowPositionChanged = function () {
	if(this._rowPositionConflator.conflate()) {
		return;
	}

	this._dispatch("rowPositionChanged", { sender: this });
};
/** @private
* @return {!Array.<ILayoutGrid>}
*/
Core.prototype._getAllSections = function () {
	let sections = [];
	let sectionCount = this._settings.length;
	for (let i = 0; i < sectionCount; i++) {
		sections.push(this._settings[i].getSection());
	}
	return sections;
};
/** @private
* @param {number} increment
* @param {(MouseEvent|Core.SectionReference)=} identifier This can be MouseEvent, index, or element id or pointer reference
* @param {string=} opt_type The next section with the specified type. Default is any type
* @return {SectionSettings}
*/
Core.prototype._getNextSection = function (increment, identifier, opt_type) {
	let nextAt = this.getSectionIndex(identifier) + increment;
	if(opt_type == null) { return this.getSectionSettings(nextAt); }

	opt_type = this._toSectionType(opt_type);
	let count = this._settings.length;
	while (nextAt >= 0 && nextAt < count) {
		let sectionSettings = this._settings[nextAt];
		if (opt_type === sectionSettings.getType()) {
			return /**@type{SectionSettings}*/(sectionSettings);
		}
		nextAt += increment;
	}
	return null;
};

/** @private
* @param {*} val
* @return {string}
*/
Core.prototype._toSectionType = function (val) {
	if(Core._sectionType[/** @type{string} */(val)]) { return /** @type{string} */(val); }

	if (util.isString(val)) {
		let lowerCaseStr = val.charAt(0).toLowerCase();
		switch (lowerCaseStr) {
		case "t": return "title";
		case "h": return "header";
		case "s": return "header";
		case "f": return "footer";
		}
	}

	return "content";
};

/** @private
* @param {string=} opt_type Possible value are "content", "title", "header", and "footer"
* @param {string=} sectionName
* @return {SectionSettings|null}
*/
Core.prototype._newSection = function (opt_type, sectionName) {
	if(sectionName) {
		for(let i = this._settings.length; --i >= 0;) {
			if (this._settings[i].getId() === sectionName) { // check if section name isn't already used
				return null;
			}
		}
	}

	opt_type = this._toSectionType(opt_type);
	let sectionOpt = {
		"horizontalScrollbar": this._hscrollbar
	};
	let section = (opt_type === "content") ? new grid_VirtualizedLayoutGrid(sectionOpt) : new grid_LayoutGrid(sectionOpt);
	section._setContext(this);

	let sectionSettings = new util_SectionSettings(section);
	sectionSettings.setId(sectionName);
	sectionSettings.setType(opt_type);

	sectionSettings.listen("dataChanged", this._onSectionDataChanged);
	sectionSettings.listen("rowAvailable", this._onRowAvailable);

	let rowHighlighting = (opt_type === "content") ? this._rowHighlighting : false;
	section.enableRowHighlighting(rowHighlighting);
	section.enableShading(false); // No alternate shading
	section.setHidingMethod(this._hidingMethod);

	sectionSettings.enableColumnVirtualization(this._colVirtualizer.isEnabled());
	section.setColumnCount(this._layoutX.getLaneCount());
	section.reserveRightSpace(this._reservedSpace);
	// TODO: Add any column customization
	this._syncLayoutToSectionColumns(section);
	this._syncStyleToSectionColumns(section, opt_type);

	if (opt_type !== this._rowHeightException) { section.setDefaultRowHeight(this._layoutY.getDefaultLaneSize()); }
	section.listen("rowHighlighted", this._onRowHightlighted);
	section.listen("rowCountChanged", this._onRowCountChanged);
	section.listen("rowHeightChanged", this._onRowHeightChanged);

	return sectionSettings;
};

/** @private
* @param {ILayoutGrid} section
*/
Core.prototype._putToLast = function(section) {
	let sectionCount = this._settings.length;
	if (sectionCount === 0) {
		section.insertBefore(this._lastHiddenInput);
	} else {
		let lastGrid = this.getLastSection();
		let nextSibling = lastGrid.getElement().nextSibling;
		if (nextSibling !== null) {
			section.insertBefore(nextSibling);
		} else {
			section.insertBefore(this._lastHiddenInput);
		}
	}
};

/** @private
* @param {ILayoutGrid} section
* @param {string} sectionType
*/
Core.prototype._syncStyleToSectionColumns = function (section, sectionType) {
	let name;
	for (let i = this._columnDefs.length; --i >= 0; ) {
		let colDef = this._getColumnDef(i);
		let styles = colDef["styles"];
		for (name in styles) {
			section.setColumnStyle(i, name, styles[name]);
		}
		let classes = colDef["classes"];
		for (name in classes) {
			section.enableColumnClass(i, name, true);
		}

		let sectionDef = colDef[sectionType];
		if(sectionDef) {
			styles = sectionDef["styles"];
			for (name in styles) {
				section.setColumnStyle(i, name, styles[name]);
			}
			classes = sectionDef["classes"];
			for (name in classes) {
				section.enableColumnClass(i, name, true);
			}
		}
	}
};
/** @private
* @return {boolean}
*/
Core.prototype._syncRowHeights = function () {
	let sectionCount = this._settings.length;
	for (let s = 0; s < sectionCount; ++s) {
		this._syncSectionRowHeights(this._settings[s].getSection());
	}
	if (!this._frozenLayout) {
		return this._updateScrollbarHeight(true, true);
	}
	return false;
};
/** @private
* @param {ILayoutGrid} section
*/
Core.prototype._syncSectionRowHeights = function (section) {
	this._layoutY.copyFrom(
		section.getRowLayout(),
		this._sectionStarts[section.getIndex()]
	);
};

/** @private
* @param {number} colIndex
* @param {number} px If this is negative
* @param {boolean=} opt_scalable Default is false
* @return {boolean} True if there is any changed
*/
Core.prototype._setColumnWidth = function (colIndex, px, opt_scalable) {
	let dirty = false;
	let layoutX = this._layoutX;
	if(opt_scalable != null) {
		dirty = layoutX.setLaneScalability(colIndex, opt_scalable ? true : false);
	} else {
		let scalable = layoutX.getLaneScalability(colIndex);
		if(this._preserveProportion) {
			if(scalable) {
				//calculate the proportion
				let colCount = this.getColumnCount();
				let totalWeight = 0, totalWidth = 0;
				for (let j = 0; j < colCount; ++j) {
					if(!layoutX.getLaneScalability(j)) { continue; }
					let colWidth = layoutX.getLaneSize(j);
					let colWeight = layoutX.getCustomLaneSize(j);
					totalWeight += colWeight;
					totalWidth += colWidth;
				}
				let oldWeight = layoutX.getCustomLaneSize(colIndex);
				px = ((px * totalWeight) - (px * oldWeight)) / (totalWidth - px);
				if(!px || px < 0) {
					px = totalWeight; // In case of new width exceeds total available width, set new weight as total weight
				}
			}
		} else {
			dirty = layoutX.setLaneScalability(colIndex, false);
		}
	}
	dirty = layoutX.setLaneSize(colIndex, px) || dirty;
	return dirty;
};

/** @private
* @param {number} at
* @param {number} count
* @param {boolean=} atTheMiddle
* @param {*=} ctx
*/
Core.prototype._dispatchColumnAddedEvent = function (at, count, atTheMiddle, ctx) {
	let e = {};
	if (this._hasListener("columnAdded")) {
		e["atTheMiddle"] = atTheMiddle;
		if(count === 1) {
			e["colIndex"] = at;
			e["context"] = ctx;
			this._dispatchColumnEvent("columnAdded", e);
		} else {
			let ary = Array.isArray(ctx) ? ctx : [];
			for (let i = 0; i < count; ++i) {
				e["colIndex"] = at + i;
				e["context"] = ary[i];
				this._dispatchColumnEvent("columnAdded", e);
			}
		}
	}

	if(this._frozenLayout){
		e["noDataUpdate"] = true;
	}

	this._dispatchColumnRenderEvent(e,
		at, at + count,
		0, this._settings.length,
		NaN, NaN);
};

/** @private
* @param {!Object} e
* @param {number} fromC
* @param {number} toC EXCLUSIVE
* @param {number} fromS
* @param {number} toS EXCLUSIVE
* @param {number} fromR If NaN or negative value is given. The entire section will be updated
* @param {number} toR EXCLUSIVE
*/
Core.prototype._dispatchColumnRenderEvent = function (e, fromC, toC, fromS, toS, fromR, toR) {
	if (!(fromS < toS)) { // Column is added before section
		return;
	}

	e["sender"] = this; // Make sure that the sender is this grid
	e["fromColIndex"] = fromC;
	e["toColIndex"] = toC;

	let s;
	if (this._hasListener("preSectionRender")) {
		for (s = fromS; s < toS; ++s) {
			this._settings[s].extendRenderEventArg(e, fromR, toR);
			this._dispatch("preSectionRender", e);
		}
	}

	// Dispatch column render event
	for (let c = fromC; c < toC; ++c) { // For each column
		let handler = this.getColumnRenderingHandler(c) || this._renderingHandler;
		if (handler) {
			e["colIndex"] = c;
			e["dataColumnName"] = this.getDataColumnName(c);

			for (s = fromS; s < toS; ++s) { // For each section
				let sectionSettings = this._settings[s];
				sectionSettings.extendRenderEventArg(e, fromR, toR);

				let section = sectionSettings.getSection();
				let startR = e["fromRowIndex"];
				let destR = e["toRowIndex"];
				for (let r = startR; r < destR; ++r) { // For each row
					e["rowIndex"] = r;
					e["cell"] = section["getCell"](c, r, false); // Accessing cell by using bracket allows extenal object to mock Section
					handler(e);
				}
			}
		}
	}

	if (this._hasListener("postSectionRender")) {
		for (s = fromS; s < toS; ++s) {
			this._settings[s].extendRenderEventArg(e, fromR, toR);
			this._dispatch("postSectionRender", e);
		}
	}

	// Dispatch data binding event once after rendering for populating content of newly rendered cells
	if (!e["noDataUpdate"] && !this._noBinding) {
		for (s = fromS; s < toS; ++s) { // For each sectionSettings
			this.updateRowData(this._settings[s], fromR, toR - 1);
		}
	}
};
/** @private
* @param {Object} e
*/
Core.prototype._dispatchRowExpansionBinding = function (e) {
	if (!this._hasListener("rowExpansionBinding")) {
		return;
	}

	let dataView = /** @type{DataView} */(e["dataSource"]);
	if (!dataView || !dataView.getDataSource()) {
		return;
	}

	let fromR = /** @type{number} */(e["fromRowIndex"]);
	let toR = /** @type{number} */(e["toRowIndex"]);
	let rids = dataView.getVisibleRowIds(true);
	let section = /** @type{ILayoutGrid} */(e["section"]);

	e["rowData"] = null;
	e["cell"] = null;
	let lastRowIndexChanged = false;
	for (let r = toR - 1; r >= fromR; --r) {
		let rowId = dataView.getRowId(r);
		if(!lastRowIndexChanged) {
			let reCount = dataView.getRowExpansionCount(rowId);
			if(reCount > 0){
				r = r + reCount + 1; // include parent row
				lastRowIndexChanged = true;
				continue;
			}
		}

		let ctxRow = section.getContextRow(r);
		if(!ctxRow) {
			continue; // The row may have been pushed out of view due to row count or height changed during the binding
		}
		let parentId = dataView.getExpansionParentByRowId(rowId);
		if(parentId) { // dispatch to render row expansion
			e["originalRowData"] = dataView.getRowData(parentId);
			e["originalRowId"] = parentId;
			e["rowExpansion"] = ctxRow["rowExpansion"] = true;
		} else if(ctxRow["rowExpansion"] ){ // dispose cases
			e["originalRowData"] = null;
			e["originalRowId"] = null;
			e["rowExpansion"] = ctxRow["rowExpansion"] = false;
		} else { // normal row no need to dispatch
			continue;
		}
		e["rowId"] = rids[r];
		e["rowIndex"] = r;
		this._dispatch("rowExpansionBinding", e);
	}
	e["originalRowData"] = null;
	e["originalRowId"] = null;
	e["rowExpansion"] = false;
};

/** @private
* @param {number} num
*/
Core.prototype._removeColumn = function (num) {  // TODO: change the logic to use removeColumnAt
	let colCount = this._layoutX.getLaneCount();
	if (num > colCount) { num = colCount; }
	if (num <= 0) { return; }

	let newCount = colCount - num;
	this._layoutX.setLaneCount(newCount);

	for (let s = this._settings.length; --s >= 0; ) {
		let section = this._settings[s].getSection();
		section.setColumnCount(newCount);
	}

	let removedCols = this._columnDefs.slice(newCount); // Shallow cloning
	this._columnDefs.length = newCount;

	this._onColumnCountChanged();
	this._syncLayoutToColumns(newCount, 0, true); // Virtualizer's view bound will not be affected from column

	if (this._hasListener("columnRemoved")) {
		let e = {};
		for (let c = colCount; --c >= newCount; ) {
			let colDef = removedCols[c - newCount];
			e["colIndex"] = c;
			e["columnData"] = colDef ? colDef["columnData"] : null;
			this._dispatchColumnEvent("columnRemoved", e);
		}
	}
};

/** `section` variable in this scope can either be instance of LayoutGrid or SectionWriter (from tr-grid-printer).
 * Need to force calling public methods (section["isColumnActive"] and section["getCell"]) so it works while being minified.
 * @private
* @param {!Object} e
*/
Core.prototype._onSectionDataChanged = function (e) {
	if(this._dispatchingDataChanged) { return; } // Prevent infinite loop

	let colCount = this._layoutX.getLaneCount();
	this._dispatchingDataChanged = colCount ? true : false;
	if(!colCount) {
		return;
	}

	let fromR = /** @type{number} */(e["fromRowIndex"]);
	let toR = /** @type{number} */(e["toRowIndex"]);
	let section = /** @type{ILayoutGrid} */(e["section"]);
	let dataView = /** @type{DataView} */(e["dataSource"]);
	let hasDataView = (dataView && dataView.getDataSource()) ? 1 : 0;
	let rids, rowDataCollection;

	section._startBindingSession(true);

	if(hasDataView) {
		rids = dataView.getVisibleRowIds(true);
		rowDataCollection = dataView.getMultipleRowData(rids, fromR, toR);
		e["dataRows"] = rowDataCollection;
	}

	let isContentSection = e["sectionType"] === "content";
	if(isContentSection){
		this._dispatch("beforeContentBinding", e);
	}
	this._dispatch("preSectionDataBinding", e);

	let dataMap = this.getDataColumnMap();
	for (let c = 0; c < colCount; ++c) {
		if(section["isColumnActive"](c) && (this.isColumnVisible(c) || this._isAlwaysRenderColumn(c))) {
			let handler = this.getColumnDataBindingHandler(c) || this._dataBindingHandler;
			if (handler) {
				let cid = dataMap[c];
				e["dataColumnName"] = cid;
				e["colIndex"] = c;

				for (let r = fromR; r < toR; ++r) {
					let rowData = hasDataView ? rowDataCollection[r] : null;
					if(rowData) {
						e["rowData"] = rowData;
						e["rowId"] = rids[r];
						e["dataValue"] = rowData ? rowData[cid] : null;
						e["rowIndex"] = r;
						e["cell"] = section["getCell"](c, r, false); // Accessing cell by using bracket allows extenal object to mock Section

						handler(e);
					}
				}
			}
		}
	}

	this._dispatch("postSectionDataBinding", e);
	this._dispatchRowExpansionBinding(e);

	if(isContentSection){
		if(!this._firstRendered) {
			this._dispatch("firstRendered", e);
			this._firstRendered = true;
		}
		this._dispatch("afterContentBinding", e);
	}
	section._startBindingSession(false);
	this._dispatchingDataChanged = false;
};

/** @public
* @ignore
*/
Core.prototype.resetInternalState = function() {
	this._firstRendered = false;
};

/** @private
* @param {VScrollbar|HScrollbar} scrollbar
* @returns {boolean}
*/
Core.prototype._updateScrollbar = function (scrollbar) {
	if(scrollbar) {
		if(scrollbar.isEnabled()) { scrollbar.update(); }
		return scrollbar.isActive();
	}
	return false;
};
/** @private
* @param {Object} e
*/
Core.prototype._onVScroll = function (e) {
	this._rowVirtualizer.setViewOffset(this._vscrollbar.getScrollTop());  // Trigger virtualization event
	this._dispatchRowPositionChanged();
};
/** @private
* @param {Object} e
*/
Core.prototype._onHScroll = function (e) {
	let scrollVal = this._hscrollbar.getScrollLeft();
	this._colVirtualizer.setViewOffset(scrollVal); // Trigger virtualization event
	this.updateColumnBounds();
	this._dispatchColumnPositionChanged();
};
/** @private
*/
Core.prototype._onVScrollEnabled = function () {
	this._updateScrollbarHeight(true, false);
	if(this._rowVirtualizer.isEnabled()) {
		this._rowVirtualizer.activate();
	}
	if(!this._vscrollbar.getAutoHide()) {
		this._updateVScrollbar();
	}
};
/** @private
*/
Core.prototype._onVScrollDisabled = function () {
	if(this._rowVirtualizer.isEnabled()) {
		this._rowVirtualizer.deactivate();
	}
};
/** @private
* @param {Object} e
*/
Core.prototype._onRowInViewChanged = function (e) { // Triggered from virtualizer
	let fi = e["firstIndex"];
	let ei = e["lastIndex"] + 1; // End index
	let sectionCount = this._settings.length;
	let updateRange = [0, 0];

	for (let s = 0; s < sectionCount; ++s) { // For each section
		let sectionSettings = this._settings[s];
		let section = sectionSettings.getSection();
		if(section && !section.getFrozenLayout()) { // Section may have been disposed or frozen
			section.activateRows(fi, ei, updateRange);
			this.updateRowData(sectionSettings,
				updateRange[0],
				updateRange[1] - 1
			); // The updateRange are relative to the section
		}
	}
};
/** @private
* @param {Object} e
*/
Core.prototype._onColInViewChanged = function (e) {
	if(!this.getColumnCount()) {
		return; // No column for activation and no cell to be rendered
	}
	let pfi = e["prevFirstIndex"];
	let pli = e["prevLastIndex"]; // INCLUSIVE
	let fi = e["firstIndex"];
	let li = e["lastIndex"]; // INCLUSIVE

	let first = pfi < fi ? pfi : fi;
	let last = pli > li ? pli : li; // INCLUSIVE

	this._activateColumns(fi, li, first, last);
	let sectionCount = this.getSectionCount();
	for (let s = 0; s < sectionCount; ++s) { // For each section
		let sectionSettings = this._settings[s];
		this.updateRowData(sectionSettings); // It will be render all rows in view
	}
};
/** @private
* @param {number=} fi First index in view
* @param {number=} li Last index in view (inclusive)
* @param {number=} first Starting index of range
* @param {number=} last Ending index of range (inclusive)
*/
Core.prototype._activateColumns = function(fi, li, first, last) {
	if (first == null) {
		first = 0;
	}

	if (last == null) {
		last = this.getColumnCount() - 1;
	}

	let activations = this._getColActivationList(fi, li, first, last);
	let sectionCount = this.getSectionCount();
	for(let s = sectionCount; --s >= 0;) {
		let sectionSettings = this._settings[s];
		sectionSettings.activateColumns(activations, first, last);
	}
};
/** @private
* @param {number=} fi First index in view
* @param {number=} li Last index in view (inclusive)
* @param {number=} first Starting index of range
* @param {number=} last Ending index of range (inclusive)
* @return {Array.<boolean>}
*/
Core.prototype._getColActivationList = function (fi, li, first, last) {
	if (first == null) { first = 0; }

	let colCount = this.getColumnCount();
	if (last == null) { last = colCount - 1; }

	if (fi == null) {
		fi = this._colVirtualizer.getFirstIndexInView();
	}

	if (li == null) {
		li = this._colVirtualizer.getLastIndexInView();
	}

	let leftPinnedIndex = this.getHScrollStartIndex();
	let rightPinnedIndex = this.getFirstPinnedRightIndex();
	let activations = new Array(last + 1);
	for (let c = first; c <= last; ++c) {
		activations[c] = (c < leftPinnedIndex) ||
			(c >= fi && c <= li) ||
			(c >= rightPinnedIndex);
	}
	return activations;
};
/** @public
* @ignore
* @param {number} colIndex
* @return {!Object}
*/
Core.prototype._getColumnDef = function (colIndex) {
	if (colIndex >= 0) {
		let def = this._columnDefs[colIndex];
		if (!def) {
			def = this._columnDefs[colIndex] = {
				"id": "" + Core._runningColumnId++, // Unique identifier
				"columnData": {}
			};
		}
		return def;
	}
	return {
		"columnData": {}
	};
};
/** Retrieve 1 or 2 level of nested object definition
* @private
* @param {number} colIndex
* @param {string=} firstLvl
* @param {string=} secondLvl
* @return {!Object}
*/
Core.prototype._getNestedColumnDef = function (colIndex, firstLvl, secondLvl) {
	let def = this._getColumnDef(colIndex);
	if(firstLvl) {
		let parentDef = def;
		def = parentDef[firstLvl];
		if(!def) {
			def = parentDef[firstLvl] = {};
		}
		if(secondLvl) {
			parentDef = def;
			def = parentDef[secondLvl];
			if(!def) {
				def = parentDef[secondLvl] = {};
			}
		}
	}
	return def;
};

/** @public
* @param {number} colIndex
* @return {string} Return empty string if the specified column does not exist
*/
Core.prototype.getColumnId = function (colIndex) {
	if(colIndex >= 0 && colIndex < this.getColumnCount()) {
		return this._getColumnDef(colIndex)["id"] || "";
	}
	return "";
};
/** @deprecated
* @public
* @function
* @ignore
* @param {number} colIndex
* @return {string}
*/
Core.prototype._getColumnId = Core.prototype.getColumnId;
/** @public
* @return {!Array.<string>} Return all column ids from existing column
*/
Core.prototype.getColumnIds = function () {
	let colCount = this.getColumnCount();
	let ary = new Array(colCount);
	for(let c = 0; c < colCount; ++c) {
		ary[c] = this._getColumnDef(c)["id"] || "";
	}
	return ary;
};

/** @public
* @param {number} colIndex
* @param {string} field
*/
Core.prototype.setColumnField = function (colIndex, field) {
	this._getColumnDef(colIndex)["field"] = field || "";
};
/** @public
* @param {number} colIndex
* @return {string} Return empty string if the specified column does not exist
*/
Core.prototype.getColumnField = function (colIndex) {
	if(colIndex >= 0 && colIndex < this.getColumnCount()) {
		return this._getColumnDef(colIndex)["field"] || "";
	}
	return "";
};
/** @public
* @return {!Array.<string>} Return all column fields from existing columns
*/
Core.prototype.getColumnFields = function () {
	let colCount = this.getColumnCount();
	let ary = new Array(colCount);
	for(let c = 0; c < colCount; ++c) {
		ary[c] = this._getColumnDef(c)["field"] || "";
	}
	return ary;
};

/** Get column index by column id or column field
* @public
* @param {string|number} colRef Column id or data column name
* @return {number} Return negative value if there is no match
*/
Core.prototype.getColumnIndex = function (colRef) {
	if(typeof colRef === "number") {
		return colRef;
	} else if(colRef) {
		let str = colRef;
		let indexByField = -1;
		let colCount = this.getColumnCount();
		for(let c = 0; c < colCount; ++c) {
			let colDef = this._getColumnDef(c);
			if(str === colDef["id"]){
				return c;
			}
			if(str === colDef["field"]) { // In case colId and field are the same, use colId first and field as a fallback
				if(indexByField < 0) {
					indexByField = c;
				}
			}
		}
		return indexByField;
	}
	return -1;
};
/** Any invalid column reference will be excluded from the output array
* @public
* @param {Array.<number|string>} colRefs
* @return {!Array.<number>} Return negative value if there is no match
*/
Core.prototype.getColumnIndices = function (colRefs) {
	let ary = [];
	let colCount = this.getColumnCount();
	let inputAry = Array.isArray(colRefs) ? colRefs : [colRefs];
	let len = inputAry.length;
	// Verify user input
	for(let i = 0; i < len; ++i) {
		let colIndex = this.getColumnIndex(inputAry[i]);
		if(colIndex >= 0 && colIndex < colCount) {
			ary.push(colIndex); // WARNING: We have not check for duplication
		}
	}
	return ary;
};

/** Get a map from column id and field to column index. This should be useful when finding multiple column indices.
* @public
* @return {!Object.<string, number>}
*/
Core.prototype.getColumnIndexMap = function () {
	let colCount = this.getColumnCount();
	let obj = {};
	let str = "";
	for(let c = 0; c < colCount; ++c) {
		let colDef = this._getColumnDef(c);

		str = colDef["field"];
		if(str) {
			obj[str] = c;
		}

		str = colDef["id"];
		if(str) {
			obj[str] = c;
		}
	}
	return obj;
};


/** @private */
Core.prototype._onWindowResize = function() {
	this._updateLayout();
	this._rowVirtualizer.setViewOffset(this._vscrollbar.getScrollTop());
};
/** @private */
Core.prototype._onMouseMove = function () {
	this._vscrollbar.flash();
	this._hscrollbar.flash();
};
/** @private */
Core.prototype._onGridClicked = function () {
	let selection = window.getSelection();
	if(!selection.toString()){
		if(!this._element.contains(_getActiveElement(this._element))){
			this.focus();
		}
	}
};

/** @private
* @param {Object} e
*/
Core.prototype._onKeyDown = function (e) {
	if(!_isTabCommand(e)) {
		return;
	}
	let activeElement = _getActiveElement(this._element);
	let onTheEdge = false;
	if(this._firstHiddenInput === activeElement) {
		onTheEdge = -1;
	} else if(this._lastHiddenInput === activeElement) {
		onTheEdge = 1;
	}

	this._dispatch("tabNavigation", {
		"activeElement": activeElement,
		"firstHiddenInput": this._firstHiddenInput,
		"lastHiddenInput": this._lastHiddenInput,
		"onTheEdge": onTheEdge,
		"shiftKey": e.shiftKey,
		"event": e
	});

	if(onTheEdge && !e.defaultPrevented) {
		if(onTheEdge > 0 && e.shiftKey) {
			this._firstHiddenInput.focus(); // jump to the top and move out of grid
		} else if(onTheEdge < 0 && !e.shiftKey) {
			this._lastHiddenInput.focus(); // skip to the end and move out of grid
		}
	}
};

/** @private
* @param {Object} e
*/
Core.prototype._onRowHightlighted = function (e) {
	if (!this._hasListener("rowHighlighted")) { return; }

	let section = e["sender"];
	let settings = this.getSectionSettings(section.getId());
	if (settings == null) { return; }

	settings.extendEventArg(e);

	let dataSource = settings.getDataSource();
	if (dataSource !== null) {
		e["rowId"] = dataSource.getRowId(/** @type{number} */(e["rowIndex"]));
		e["dataRow"] = dataSource.getRowData(e["rowId"]);
	}
	this._dispatch("rowHighlighted", e);
};
/** @private
* @param {Object} e
*/
Core.prototype._onRowCountChanged = function (e) {
	if(this._disposed) {
		return; // There is no need to do anything after grid is disposed
	}
	let section = /** @type {ILayoutGrid} */(e["sender"]);
	let prevRowCount = e["prevCount"];
	let newRowCount = e["rowCount"];
	let sectionIndex = section.getIndex();
	let offset = this._updateSectionIndices(sectionIndex);
	let i = 0;

	if(prevRowCount > newRowCount) { // case row removed
		this._layoutY.removeLaneAt(offset, prevRowCount - newRowCount);
		if(!this._layoutY.isHomogeneous()) {
			this._syncSectionRowHeights(section);
		}
	} else { // case row added
		let defaultHeight = this._layoutY.getDefaultLaneSize();
		let customHeight = section.getDefaultRowHeight();

		this._layoutY.insertLane(offset + prevRowCount, newRowCount - prevRowCount);
		if (customHeight !== defaultHeight) {
			for (i = prevRowCount; i < newRowCount; ++i) {
				this._layoutY.setLaneSize(offset + i, customHeight);
			}
		}
	}

	let noBinding = e["noBinding"];
	let paneChanged = (sectionIndex <= this._startVScrollbarIndex) || (sectionIndex >= this._getFooterStartIndex());
	let forceUpdate = this._frozenFooterCount > 0 ? true : false; // Prevent from footer section flashing
	let viewChanged = this._updateScrollbarHeight(paneChanged, forceUpdate, noBinding); // Virtualization is triggered
	if(noBinding){
		this._hasPendingRowChange = true;
	}
	if(!this._frozenLayout && !viewChanged) { // Grid must activate newly created section if no event is dispatched from the row virtualizer
		if(this._rowVirtualizer.isVirtualizable()) {
			section.activateRows(this._rowVirtualizer.getFirstIndexInView(),
				this._rowVirtualizer.getLastIndexInView());
		} else {
			section.activateRows();
		}
	}
	this._hasPendingRowChange = false;

	if(!forceUpdate) {
		this._updateVScrollbar(); // Asynchronous
	}

	this._updateColumnSeparators();
	if(prevRowCount < newRowCount) {
		// Apply column background color
		let settings = this.getSectionSettings(section);
		if(settings.getType() !== "content") {
			this.applyColumnsBackgroundColor(section);
		}
		this._dispatch("rowAdded", e);
	} else if(prevRowCount > newRowCount) {
		this._dispatch("rowRemoved", e);
	}
};

/** @private
* @param {Object} e
*/
Core.prototype._onRowHeightChanged = function (e) {
	if (!this._rowHeightSync) { return; }
	if (this._disposed) { return; } // There is no need to sync anything after disposition
	if(this._rowHeightConflator.conflate(e)) {
		return;
	}

	let evts = this._rowHeightConflator.popAllData();

	let len = evts.length;
	let minSectionIndex = NaN;

	// TODO: Set row height one by one is not good
	for(let j = 0; j < len; ++j) {
		let evt = evts[j];
		let section = /** @type {ILayoutGrid} */(evt["section"]);
		if(!this._containsSection(section)) {
			continue; // The section is no longer contained in this grid
		}

		let sectIndex = section.getIndex();
		let relIndex = evt["rowIndex"];
		let rowIndex = this.getRowIndex(sectIndex, relIndex);

		let changeLength = evt["changeLength"];
		for (let i = 0; i < changeLength; ++i) {
			if(this._layoutY.setLaneSize(rowIndex + i, section.getRowHeight(relIndex + i))) {
				if(!(sectIndex >= minSectionIndex)) {
					minSectionIndex = sectIndex;
				}
			}
		}
	}

	if(this._rowHeightConflator._needScrollbarUpdate) {
		this._rowHeightConflator._needScrollbarUpdate = false;
		this._updateScrollbarHeight(true, true);
	} else if(minSectionIndex >= 0) {
		if(this._dispatchingDataChanged){
			if(!this._rowHeightTimerId){
				this._rowHeightTimerId = setTimeout(this._requestScrollbarUpdate, 0);
			}
		} else {
			this._updateScrollbarHeight(minSectionIndex < this._startVScrollbarIndex,
				minSectionIndex >= this._startVScrollbarIndex);
		}
	}
	this._updateColumnSeparators();
	this._dispatchRowPositionChanged();
};
/** @private
* @param {ILayoutGrid} s
* @returns {boolean}
*/
Core.prototype._containsSection = function (s) {
	for(let i = this._settings.length; --i >= 0;) {
		if(this._settings[i].getSection() === s) {
			return true;
		}
	}
	return false;
};

/** @private
* @param {!Object} e
*/
Core.prototype._onRowAvailable = function (e) {
	if(this._hasPendingRowChange){
		return;
	}
	let sectionIndex = /** @type{number} */(e["sectionIndex"]);
	this._dispatchColumnRenderEvent(e,
		0, this.getColumnCount(),
		sectionIndex, sectionIndex + 1,
		e["prevRowIndex"], e["rowIndex"]);
};

/** @private
* @param {boolean=} opt_suppressLayout
*/
Core.prototype._onSectionCountChanged = function (opt_suppressLayout) {
	if(this._frozenLayout) { return; }

	if (this._tempFrozenSectionRef) {
		let startVScrollbarSection = this.getNextSection(this._tempFrozenSectionRef);
		if (startVScrollbarSection !== null) {
			this._startVScrollbarIndex = startVScrollbarSection.getIndex();
			this._tempFrozenSectionRef = "";
		}
	}

	if (this._startVScrollbarIndex >= 0 && this._startVScrollbarIndex < this._getFooterStartIndex()) {
		// TODO: should enable scrollbar inside DataSourceTreePlugin instead
		this._vscrollbar.enable();

		// Reinsert sections
		this._vscrollbar.setScrollContent(this, this._getAllSections(), this._startVScrollbarIndex);
		this._element.appendChild(this._lastHiddenInput); // Ensure that the hidden input is always at the last position

		if(!opt_suppressLayout) {
			this._updateScrollbarHeight(true, true);
		}
		this._syncLayoutToColumns();
	} else {
		this._vscrollbar.disable();
	}
};
/** @private */
Core.prototype._onColumnCountChanged = function () {
	if(this._frozenLayout || this._disposed) { return; }

	// TODO: Optimize these methods during initialization
	let pinnedLeft = this._countPinnedLeftColumns();
	let pinnedRight = this._countPinnedRightColumns();

	this.updateColumnBounds();
	this._updateColumnSeparators();

	if (this._hScrollbarEnabled && pinnedLeft + pinnedRight < this.getColumnCount()) {
		this._hscrollbar.enable();
		this._colVirtualizer.activate();

		this._hscrollbar.setScrollContent(
			this,
			this._getAllSections(),
			pinnedLeft,
			pinnedRight
		);
	} else { // All columns are pinned and thus no column is scrollable
		this._hscrollbar.disable();
		this._colVirtualizer.deactivate();

		pinnedLeft = pinnedRight = 0;
	}

	this._element.classList.toggle("right-pinning", pinnedRight > 0 ? true : false);
	this._dispatchColumnPositionChanged();
};

/** @private
* @param {Object} e
*/
Core.prototype._onSyncVScroll = function (e) {
	this._vscrollbar.setScrollTop(e.scrollTop);
};
/** @private
* @param {Object} e
*/
Core.prototype._onSyncHScroll = function (e) {
	this._hscrollbar.setScrollLeft(e.scrollLeft);
};
/** @private
* @return {number} index of footer section
*/
Core.prototype._getFooterStartIndex = function () {
	return this.getSectionCount() - this._frozenFooterCount;
};
/** @private
* @return {number} pixel without zoom factor
*/
Core.prototype._getVScrollStart = function () {
	return this._getSectionSize(0, this._startVScrollbarIndex);
};
/** @private
* @return {number} pixel without zoom factor
*/
Core.prototype._getFooterSize = function () {
	return (this._frozenFooterCount > 0) ? this._getSectionSize(this._getFooterStartIndex()) : 0;
};
/** @private
* @param {number=} startIndex
* @param {number=} count
* @return {number} pixel without zoom factor
*/
Core.prototype._getSectionSize = function (startIndex, count) {
	if(!startIndex) {
		startIndex = 0;
	} else if(startIndex < 0) {
		return 0;
	}
	if(count == null) {
		count = this.getSectionCount() - startIndex;
	}
	let startPos = this._layoutY.getLaneStart(
		this._sectionStarts[startIndex]
	);

	let endIndex = startIndex + count;
	let endPos = count > 0 ? this._layoutY.getLaneStart(
		this._sectionStarts[endIndex]
	) : 0;
	return endPos - startPos;
};

/** @private
* @param {boolean} paneChanged
* @param {boolean} contentChanged
* @param {boolean=} noBinding
* @return {boolean}
*/
Core.prototype._updateScrollbarHeight = function (paneChanged, contentChanged, noBinding) {
	if(!this._vscrollbar.isEnabled()) {
		return false;
	}

	let forceUpdateLayout = contentChanged;
	let recalculation = paneChanged && this._startVScrollbarIndex >= 0;
	let hasRowHeightChanged = this._rowHeightConflator.isConflating();
	if(hasRowHeightChanged) {
		this._rowHeightConflator._needScrollbarUpdate = true;

		// Should not force layout-update while rowHeightConflator is executing
		// because states have not updated and will actives scrollbar unintentionally.
		forceUpdateLayout = false;
	}

	if (recalculation && !hasRowHeightChanged) {
		// Scroll Frame
		let gridElem = this.getElement();
		if(!gridElem) {
			return false;
		}

		let gridParent = this.getParent();
		let scrollFrame = this._vscrollbar.getParent();
		if(scrollFrame) {
			if(scrollFrame !== gridParent) { // If vscrollbar is attached to an external element
				// Assume that there is always a horizontal scrollbar host underneath
				if(scrollFrame.firstElementChild) {
					scrollFrame = scrollFrame.firstElementChild;
				}
			}
		} else {
			scrollFrame = gridElem;
			this._vscrollbar.setParent(scrollFrame);
		}

		let sectionStart = this._getVScrollStart();
		let footerSize = this._getFooterSize();

		if(scrollFrame !== gridElem) {
			sectionStart += gridElem.clientTop;
		}

		// Scroll Track and Pane
		this._vscrollbar.resetPaneSize(); // scrollFrame could have a dynamic height that rely on pane's content.
		let containerHeight = scrollFrame.clientHeight - (sectionStart + footerSize) * this._zoomFactor;

		// Pane size needs to be bigger when zooming out to keep the same view size
		this._vscrollbar.setTop(sectionStart * this._zoomFactor);
		this._vscrollbar.setTrackSize(containerHeight, containerHeight / this._zoomFactor);

		// This calculation is for finding a appropriate dynamic buffer size
		// because in the large content number, buffer need to be bigger.
		// TODO: move buffer calculation to virtualizer
		let defaultSize = this._layoutY.getDefaultLaneSize();
		let viewSize = defaultSize * 12; // Display at least 12 rows

		if (containerHeight > viewSize) {
			viewSize = (Math.ceil(containerHeight / defaultSize) + 4) * defaultSize;
		}

		// HACK: Due to fixed layout size we need to scale view size instead of content size, when zooming
		// TODO: Check if zoom factor is used for virtualization correctly
		this._rowVirtualizer.setViewSize(viewSize / this._zoomFactor);

		let offsetRowCount = this._sectionStarts[this._startVScrollbarIndex];
		let footerCount = this.getFooterCount();
		let footerRowCount = 0;
		if(footerCount) {
			let sectionCount = this.getSectionCount();
			footerRowCount = this._sectionStarts[sectionCount] - this._sectionStarts[sectionCount - footerCount];
		}
		this._rowVirtualizer.setViewBounds(offsetRowCount, footerRowCount);
	} else {
		this._rowVirtualizer.validateVirtualization(); // Content height may be changed
	}

	let viewChanged = false;
	if(!noBinding) {
		viewChanged = this._rowVirtualizer.update(); // May fire onRowInViewChanged
	}

	this._vscrollbar.invalidate(forceUpdateLayout); // Update visual for vertical scrollbar

	return viewChanged;
};
/** @private
* @param {boolean} paneChanged
* @param {boolean} contentChanged
*/
Core.prototype._updateScrollbarWidth = function (paneChanged, contentChanged) {
	if (this._frozenLayout) {
		return;
	}

	if (paneChanged && this._hScrollbarEnabled) {
		// Scroll Frame
		let gridElem = this.getElement();
		let scrollFrame = this._hscrollbar.getParent();
		if (!scrollFrame) {
			scrollFrame = this.getParent() || gridElem;
			this._hscrollbar.setParent(scrollFrame);
		}

		// Frozen Area
		let leftFrozenArea = this._layoutX.getLaneStart(this._pinnedLeftColumnCount);

		let colCount = this._layoutX.getLaneCount();
		let rightFrozenArea = 0;
		let rightPinnedCount = this._countPinnedRightColumns(); // TODO: Check if we need to do a recount here
		if (rightPinnedCount) {
			rightFrozenArea = this._layoutX.getLaneStart(colCount) -
				this._layoutX.getLaneStart(colCount - rightPinnedCount);
		}

		// Scroll Track and Pane
		let gridClientWidth = (scrollFrame !== gridElem) ? gridElem.clientLeft : 0;
		let containerWidth = scrollFrame.clientWidth -
				(leftFrozenArea + gridClientWidth + rightFrozenArea) * this._zoomFactor;

		if (rightPinnedCount > 0) {
			containerWidth = containerWidth - this._reservedSpace;
		}

		this._hscrollbar.setPaneLeft(leftFrozenArea);
		this._hscrollbar.setLeft((leftFrozenArea + gridClientWidth) * this._zoomFactor);
		this._hscrollbar.setTrackSize(containerWidth, containerWidth / this._zoomFactor);

		let defaultSize = this._layoutX.getDefaultLaneSize();
		let viewSize = defaultSize * 4; // Display at least 4 column
		if (containerWidth > viewSize) {
			viewSize = containerWidth;
		}
		this._colVirtualizer.setViewSize(viewSize / this._zoomFactor);
		this._colVirtualizer.setViewBounds(this._pinnedLeftColumnCount, rightPinnedCount);
	} else {
		this._colVirtualizer.validateVirtualization(); // Content width may be changed
	}

	this._colVirtualizer.update();

	this._hscrollbar.invalidate(contentChanged);  // Update content width
};

/** @private
* @ignore
* @param {number} relativeY y position that is relative to the top left this object.
* @param {!Object} returnedObj
* @return {boolean}
*/
Core.prototype._hitTestY = function (relativeY, returnedObj) {
	let sectionSettings;
	if (returnedObj["sectionSettings"]) {
		sectionSettings = returnedObj["sectionSettings"];
	} else {
		let rowIndex;
		if (returnedObj["rowIndex"] == 'number') {
			rowIndex = returnedObj['rowIndex'];
		} else {
			rowIndex = this._layoutY.hitTest(relativeY);
		}
		let validRowIndex = (rowIndex >= 0) ? rowIndex : ~rowIndex; // TODO: Check if this is necessary
		let sectionIndex = util.upperBound(this._sectionStarts, validRowIndex) - 1;
		returnedObj["sectionIndex"] = sectionIndex;
		sectionSettings = this._settings[sectionIndex];
		if(!sectionSettings) { // Cannot find SectionSettings
			return false;
		}
		sectionSettings.extendEventArg(returnedObj);
	}

	let section = sectionSettings.getSection();
	if(returnedObj["colIndex"] != null && returnedObj["cellElement"]) {
		returnedObj["rowIndex"] = section.getCellIndex(returnedObj["colIndex"], returnedObj["cellElement"]);
	} else {
		returnedObj["rowIndex"] = rowIndex + section.getRowOffset();
	}

	return true;
};

/** @private
* @param {number=} from
* @param {number=} to
* @param {boolean=} opt_forceDispatching
* @fires Core#widthChanged
*/
Core.prototype._syncLayoutToColumns = function (from, to, opt_forceDispatching) {
	if (this._frozenLayout) { return; }
	let colCount = this._layoutX.getLaneCount();
	let forceUpdate = this.fillParentWidth(100); // Resize all columns if there is a scaling column
	if (forceUpdate || (from == null) || (to == null)) {
		from = 0;
		to = colCount;
		forceUpdate = true;
	}

	let sectCount = this._settings.length;
	if (sectCount <= 0) { return; }

	let dirty = false;
	if(from < to) {
		for (let i = 0; i < sectCount; ++i) {
			dirty = this._syncLayoutToSectionColumns(this._settings[i].getSection(), from, to) || dirty;
		}
	}

	// TODO: Check if "to" should be greater than or equal to first pinnied right index
	let paneChanged = forceUpdate || (from < this.getHScrollStartIndex()) || (to > this.getFirstPinnedRightIndex());
	this._updateScrollbarWidth(paneChanged, true /* contentChanged */);
	this.updateColumnBounds();
	this._updateColumnSeparators();
	this._dispatchColumnPositionChanged();

	if (dirty || opt_forceDispatching) {
		this._dispatch("widthChanged", {
			"from": from,
			"to": to,
			"colChanged": from
		});
	}
};
/** @private
* @param {ILayoutGrid} section
* @param {number=} opt_from
* @param {number=} opt_to
* @return {boolean} True if there is any change
*/
Core.prototype._syncLayoutToSectionColumns = function (section, opt_from, opt_to) {
	if (opt_from == null) { opt_from = 0; }
	if (opt_to == null) { opt_to = this._layoutX.getLaneCount(); }

	let dirty = 0;
	for (let i = opt_from; i < opt_to; ++i) {
		dirty |= section._setColumnWidth(i, this._layoutX.getLaneSize(i));
	}
	if(dirty && !this._frozenLayout) {
		section._updateColumnLayout(); // TODO: This could be further improved by identifying first dirty index
		return true;
	}
	return false;
};

/** @private
* @param {number|string} colRef
* @return {string}
*/
Core.prototype._toDataColumnName = function (colRef) {
	if (util.isNumber(colRef)) {
		return this.getDataColumnName(/** @type{number} */(colRef));
	}
	return /** @type{string} */(colRef);
};

/** @private
* @param {number} from
* @return {number} offset value at the given index
*/
Core.prototype._updateSectionIndices = function (from) {
	if(this._updatePaddingBottomSect()) { // Bottom padding must always be at the last section
		--from; // The section has been moved, so we shift the position required for update
	}

	if (from < 0) { from = 0; }

	// Adjust Index offsets for hittest
	let sectionCount = this._settings.length;
	let first = this._settings[0] || null;
	if (first !== this._firstSettings) {
		if (this._firstSettings) { this._firstSettings.getSection().removeClass("first"); }
		this._firstSettings = first;
		if (this._firstSettings) { this._firstSettings.getSection().addClass("first"); }
	}
	let last = this._settings[sectionCount - 1] || null;
	if (last !== this._lastSettings) {
		if (this._lastSettings) { this._lastSettings.getSection().removeClass("last"); }
		this._lastSettings = last;
		if (this._lastSettings) { this._lastSettings.getSection().addClass("last"); }
	}

	this._sectionStarts.length = sectionCount + 1;
	this._sectionStarts[0] = 0;
	for (let i = from; i < sectionCount; ++i) {
		let section = this._settings[i].getSection();
		section.setIndex(i);
		section.setRowOffset(this._sectionStarts[i]);
		this._sectionStarts[i + 1] = this._sectionStarts[i] + section.getRowCount();
	}

	return this._sectionStarts[from];
};

/** @private */
Core.prototype._updateLayout = function () {
	let element =  this.getElement();
	if(this._disposed || !element.offsetParent) { return; }

	this._syncLayoutToColumns(); // Update only if need

	this._updateScrollbarHeight(true, true);
	this._updateScrollbarWidth(true, true); // WARNING this may be redundant with _syncLayoutToColumns

	let sectionCount = this.getSectionCount();
	for (let s = 0; s < sectionCount; ++s) {
		let section = this._settings[s].getSection();
		section.updateLayout(); // Notify section about forced recalculation of the layout
	}
	this._updateColumnSeparators();
};

/** @private */
Core.prototype._updateVScrollbar = function() {
	if (this._disposed || !this._vscrollbar.isEnabled()) { return; } // There is no need to sync anything after disposition
	if(this._vScrollbarConflator.conflate()) {
		return;
	}
	this._vscrollbar.update();
};

//#endregion Private Methods

Core._proto = Core.prototype;

/* harmony default export */ const grid_Core = ((/* unused pure expression or super */ null && (Core)));


;// ./src/js/StyleLoader.js
/* eslint-disable */


/* eslint-enable */

/**
* @constructor
*/
let StyleLoader = {};

/** @public
 * @type {string}
 */
StyleLoader.chainIcon = '';

/** @type {string}
* @public
*/
StyleLoader.styles = prettifyCss([
	".tr-grid .chain-icon", [
		"cursor: pointer;"
	],
	".tr-grid .chain-icon .fallback-arrow", [
		"vertical-align: middle;",
		"font-size: 7px;",
		"line-height: 1;",
		"display: inline-block;", // For transformation
		"transform: scaleX(1.5);",
		"font-family: Arial;"
	],
	".tr-grid .closed .chain-icon", [
		"transform: translate(0px, -1px) rotate(-90deg);"
	]
]);

/** @public
* @param {Core} grid
*/
StyleLoader.applyStyle = function (grid) {
	if (grid && !grid._rowGroupingStyles) { // TODO: Support run-time theme loading
		grid._rowGroupingStyles = true; // Prevent loading the same style twice
		injectCss(StyleLoader.styles, grid.getElement());
	}
};


/* harmony default export */ const js_StyleLoader = (StyleLoader);

;// ./src/js/ReferenceCounter.js
/** @constructor
*/
let ReferenceCounter = function() {
	this._counter = {};
	this._session = {};
	this._referers = {};
};

/** @type {Object.<string, number>}
* @private
**/
ReferenceCounter.prototype._counter = null;
/** @type {Object.<string, number>}
* @private
**/
ReferenceCounter.prototype._session = null;
/** @type {Object.<string, Object>}
* @private
**/
ReferenceCounter.prototype._referers = null;


/** @public
*/
ReferenceCounter.prototype.reset = function() {
	this._counter = {};
	this._referers = {};
	this.newSession();
};
/** @public
*/
ReferenceCounter.prototype.newSession = function() {
	this._session = {};
};

/** @public
* @return {!Array.<string>}
*/
ReferenceCounter.prototype.getAllReferences = function() {
	return Object.keys(this._counter);
};

/** @public
* @return {boolean}
*/
ReferenceCounter.prototype.hasReference = function() {
	for (const key in this._counter) {
		return true;
	}
	return false;
};

/** @public
* @return {!Array.<string>}
*/
ReferenceCounter.prototype.getAllReferers = function() {
	return Object.keys(this._referers);
};
/** Alias to ReferenceCounter.getAllReferers
* @public
* @function
* @return {!Array.<string>}
*/
ReferenceCounter.prototype.getAllReferrers = ReferenceCounter.prototype.getAllReferers;

/** @public
* @return {Object} Object with newEntries and removedEntries
*/
ReferenceCounter.prototype.getSession = function() {
	let newEntries = [];
	let removedEntries = [];
	for(let key in this._session) {
		let val = this._session[key];
		if(val > 0) {
			newEntries.push(key);
		} else if(val < 0) {
			removedEntries.push(key);
		}
		// else {} // when val 0 do nothing, doesn't change anything
	}
	return {
		newEntries: newEntries.filter(Boolean),
		removedEntries: removedEntries.filter(Boolean)
	};
};

/** @public
* @param {string} key
* @param {string=} referer Referer can only add specific key only once
* @return {boolean} True if new reference is added
*/
ReferenceCounter.prototype.addReference = function(key, referer) {
	if(key) {
		if(referer) {
			let counter = this._referers[referer];
			if(counter) {
				if(counter[key]) { // Single Referer can only add specific key only once
					return false;
				}
			} else {
				counter = this._referers[referer] = {};
			}
			counter[key] = 1;
		}

		if(this._counter[key]) {
			++this._counter[key];
			// The session will not change when a field already exists and a counter is attempted to be added
		} else {
			this._counter[key] = 1;
			if(this._session[key] === -1) {
				this._session[key] = 0;
			} else {
				this._session[key] = 1;
			}

			return true;
		}
	}
	return false;
};
/** @public
* @function
* @param {string} key
* @return {boolean} True if new reference is added
*/
ReferenceCounter.prototype.addRef = ReferenceCounter.prototype.addReference;

/** @public
* @param {string} key
* @param {string=} referer Referer can only add specific key only once
* @param {number=} count Use negative value to remove all counter from the given reference
* @return {boolean} True if existing reference is removed
*/
ReferenceCounter.prototype.removeReference = function(key, referer, count) {
	if(!key) {
		return false;
	}
	let val = this._counter[key];
	if(!val) {
		return false;
	}
	if(count == null) {
		count = 1;
	} else if(count < 0) {
		count = val;
	} else if(!count) {
		return false;
	}

	if(referer) {
		let counter = this._referers[referer];
		if(!counter) {
			return false;
		}
		if(!counter[key]) {
			return false;
		}

		delete counter[key];
	}

	val -= count;
	if(!val || val < 0) {
		delete this._counter[key];
		if(this._session[key] === 1) {
			this._session[key] = 0;
		} else {
			this._session[key] = -1;
		}
		return true;
	}

	this._counter[key] = val;
	return false;
};
/** @public
* @function
* @param {string} key
*/
ReferenceCounter.prototype.release = ReferenceCounter.prototype.removeReference;

/** @public
* @param {(string|Array.<string>|Object)} keys
* @param {string=} referer
* @return {number} Number of newly added entries
*/
ReferenceCounter.prototype.addReferences = function(keys, referer) {
	if(!keys) {
		return 0;
	}
	let count = 0;
	if(Array.isArray(keys)) {
		let len = keys.length;
		for(let i = 0; i < len; ++i) {
			if(this.addReference(keys[i], referer)) {
				++count;
			}
		}
	} else if (typeof keys == "string") {
		if(this.addReference(keys, referer)) {
			++count;
		}
	} else {
		for(let key in keys) {
			if(this.addReference(key, referer)) {
				++count;
			}
		}
	}
	return count;
};

/** Remove all references from specific referer
* @public
* @param {string=} referer
* @return {number} Number of removed entries
*/
ReferenceCounter.prototype.removeReferer = function(referer) {
	let count = 0;
	if(referer) {
		let counter = this._referers[referer];
		if(counter) {
			for(let key in counter) {
				if(this.removeReference(key, referer)) {
					++count;
				}
			}
			delete this._referers[referer];
		}
	}
	return count;
};
/** Alias to ReferenceCounter.removeReferer
* @public
* @function
* @param {string=} referer
* @return {number} Number of removed entries
*/
ReferenceCounter.prototype.removeReferrer = ReferenceCounter.prototype.removeReferer;

/** Log current reference count to dev console
* @public
*/
ReferenceCounter.prototype.log = function() {
	console.log(this._counter);
};



/* harmony default export */ const js_ReferenceCounter = (ReferenceCounter);

;// ./src/js/DataConnector.js





 // eslint-disable-line

/**
* @constructor
* @extends {EventDispatcher}
*/
let DataConnector = function () {
	let t = this;
	t._commitRicsChanges = t._commitRicsChanges.bind(t);
	t._commitFieldsChanges = t._commitFieldsChanges.bind(t);
	t._fieldChangedConflator = new Conflator(0, t._commitFieldsChanges);
	t._ricChangedConflator = new Conflator(0, t._commitRicsChanges);
	t._fields = new js_ReferenceCounter();
	t._rics = new js_ReferenceCounter();
	t._rowDefMap = {};
};
Ext.inherits(DataConnector, EventDispatcher);

/** @type {Conflator}
* @private
*/
DataConnector.prototype._fieldChangedConflator = null;
/** @type {Conflator}
* @private
*/
DataConnector.prototype._ricChangedConflator = null;
/** @type {Object.<string, Array.<RowDefinition>>}
* @private
*/
DataConnector.prototype._rowDefMap = null;
/** @type {ReferenceCounter}
* @private
*/
DataConnector.prototype._rics = null;
/** @type {ReferenceCounter}
* @private
*/
DataConnector.prototype._fields = null;

/** @public
*/
DataConnector.prototype.dispose = function () {
	this.removeAllEventListeners();

	this._rowDefMap = this._fields = this._rics = null;
	this._fieldChangedConflator.reset();
	this._ricChangedConflator.reset();
};

/** Get all ric with no duplication
 * @public
 * @returns {Array.<string>}
*/
DataConnector.prototype.getAllRics = function () {
	return this._rics.getAllReferences();
};

/**
 * @public
 * @returns {boolean}
*/
DataConnector.prototype.hasRic = function () {
	return this._rics.hasReference();
};

/** @public
 * @returns {Array.<RowDefinition>}
*/
DataConnector.prototype.getAllRowDefs = function () {
	let results = [];
	for (let i in this._rowDefMap) { // TODO: Check if push is better than concat
		results = results.concat(this._rowDefMap[i]);
	}
	return results;
};

/** Get all fields, including required fields and data fields, with no duplicate
* @public
* @return {!Array.<string>} New array is created
*/
DataConnector.prototype.getAllFields = function () {
	return this._fields.getAllReferences();
};

/** @public
 * @param {string} ric
 * @returns {Array.<RowDefinition>}
*/
DataConnector.prototype.getRowDefByRic = function (ric) {
	return this._rowDefMap[ric] || null;
};

/** @public
* @param {RowDefinition} rowDef
* @return {boolean} True if new reference is added.
*/
DataConnector.prototype.addRic = function (rowDef) {
	let ric = rowDef ? rowDef.getSymbol() : "";
	if(!ric) {
		return false;
	}

	let rowDefMap = this._rowDefMap;
	let rowDefs = rowDefMap[ric];
	if (!rowDefs) {
		rowDefs = rowDefMap[ric] = [rowDef];
	} else {
		if (rowDefs.indexOf(rowDef) < 0) {
			rowDefs.push(rowDef);
		}
	}

	// Referer could be added, even if no new reference is added.
	let newReference = this._rics.addReference(ric, rowDef.getRowId());

	this._commitRicsChanges();
	return newReference;
};

/** @public
* @param {RowDefinition} rowDef Row defintion object used as a referer
* @param {string=} ric RIC to be removed
*/
DataConnector.prototype.removeRic = function (rowDef, ric) {
	if(!ric) {
		// TODO: RIC or permId maybe change during run-time, need to check before remove RIC/permId
		ric = rowDef ? rowDef.getSymbol() : "";
	}
	if(!ric) {
		return;
	}

	let rowDefMap = this._rowDefMap;
	let rowDefs = rowDefMap[ric];
	if (rowDefs) {
		let index = rowDefs.indexOf(rowDef);
		if (index >= 0) {
			rowDefs.splice(index, 1);
		}
	}

	this._rics.removeReference(ric, rowDef.getRowId());
	this._commitRicsChanges();
};

/** @public
*/
DataConnector.prototype.removeAllRics = function () {
	let referers = this._rics.getAllReferers(); // TODO: Add method to removeAllReferers at once
	for (let i in referers) {
		this._rics.removeReferer(referers[i]);
	}
	this._rowDefMap = {};
	this._commitRicsChanges();
};

/** @private
*/
DataConnector.prototype._commitRicsChanges = function () {
	if (this._ricChangedConflator.conflate()) {
		return;
	}

	let rics = this._rics;
	let session = rics.getSession();
	let removedEntries = session.removedEntries;
	let newEntries = session.newEntries;

	if (removedEntries.length) {
		this._doDispatch({
			type: "ricRemoved",
			removedRics: removedEntries
		});
	}
	if (newEntries.length) {
		this._doDispatch({
			type: "ricAdded",
			addedRics: newEntries
		});
	}

	rics.newSession();
};

/** @public
* @param {string|Array.<string>|ColumnDefinition} fieldRef
* @param {string=} referer
*/
DataConnector.prototype.addFields = function (fieldRef, referer) {
	if (!fieldRef) { return; }

	let fields = this._fields;

	if (fieldRef instanceof js_ColumnDefinition) {
		fields.addReferences(fieldRef.getAllFields(), fieldRef.getId());
	} else {
		fields.addReferences(fieldRef, referer);
	}
	this._commitFieldsChanges();
};

/** @public
* @param {ColumnDefinition|string} fieldRef
* @param {string=} referer
*/
DataConnector.prototype.removeFields = function (fieldRef, referer) {
	if (!fieldRef) { return; }

	if (fieldRef instanceof js_ColumnDefinition) {
		this._fields.removeReferer(fieldRef.getId());
	} else {
		this._fields.removeReference(fieldRef, referer);
	}
	this._commitFieldsChanges();
};

/** @public
* @param {string} referrer
*/
DataConnector.prototype.removeFieldReferrer = function (referrer) {
	this._fields.removeReferer(referrer);
	this._commitFieldsChanges();
};

/** @public
*/
DataConnector.prototype.removeAllFields = function () {
	let referers = this._fields.getAllReferers();
	for (let i in referers) { // TODO: Add method to removeAllReferers at once
		this._fields.removeReferer(referers[i]);
	}
	this._commitFieldsChanges();
};

/** @private
*/
DataConnector.prototype._commitFieldsChanges = function () {
	if (this._fieldChangedConflator.conflate()) {
		return;
	}

	let fields = this._fields;
	let session = fields.getSession();
	let removedEntries = session.removedEntries;
	let newEntries = session.newEntries;

	if (removedEntries.length) {
		this._doDispatch({
			type: "fieldRemoved",
			removedFields: removedEntries
		});
	}
	if (newEntries.length) {
		this._doDispatch({
			type: "fieldAdded",
			addedFields: newEntries
		});
	}
	fields.newSession();
};

/** @private
* @param {Object} payload
*/
DataConnector.prototype._doDispatch = function (payload) {
	let t = this;
	payload.fields = t.getAllFields();
	payload.rics = t.getAllRics();
	payload.rowDefs = t.getAllRowDefs();
	t._dispatch(payload.type, payload);
};

/** @public
*/
DataConnector.prototype.reset = function () {
	let t = this;
	t._rowDefMap = {};
	t._fields.reset();
	t._rics.reset();
	t._fieldChangedConflator.reset();
	t._ricChangedConflator.reset();
};


/* harmony default export */ const js_DataConnector = (DataConnector);

;// ./src/js/RowDefSorter.js


/** @private
* @param {*} a
* @param {*} b
* @param {number} order
* @return {number}
*/
let _defaultCompare = function(a, b, order) {
	if(a == null || a !== a) {
		if(b == null || b !== b) {
			return 0;
		}
		return 1;
	}
	if(b == null || b !== b) {
		return -1;
	}

	if(a < b) {
		return -order;
	}
	if(b < a) {
		return order;
	}
	return 0;
};

/** @constructor
*/
let RowDefSorter = function() {
	this._defaultSorter = this._defaultSorter.bind(this);
	this._dataSorter = this._dataSorter.bind(this);
	this._rowDefSorter = this._rowDefSorter.bind(this);
	this._multiColumnsSorter = this._multiColumnsSorter.bind(this);

	this._globalContext = {};
	this._sortParams = [];
	this._ctxCaches = [];
};

/** @type {!Object}
* @private
*/
RowDefSorter.prototype._globalContext;
/** @type {!Array.<Array>}
* @private
*/
RowDefSorter.prototype._sortParams;
/** @type {!Array.<Object>}
* @private
*/
RowDefSorter.prototype._ctxCaches;
/** @type {Array}
* @private
*/
RowDefSorter.prototype._primaryParams;


/** @public
*/
RowDefSorter.prototype.dispose = function() {
	this._globalContext = {}; // Clear any existing reference
	this._sortParams.length = 0;
	this._ctxCaches.length = 0;
	this._primaryParams = null;
};

/** @public
* @return {Function}
*/
RowDefSorter.prototype.getSorter = function() {
	this._primaryParams = null;
	let sortCount = this._sortParams.length;
	if(sortCount === 1) {
		let params = this._primaryParams = this._sortParams[0];
		let sortLogic = params[1];
		if(sortLogic) {
			let rowSorting = params[3];
			return rowSorting ? this._rowDefSorter : this._dataSorter;
		} else {
			return this._defaultSorter;
		}
	} else if(sortCount > 1) {
		return this._multiColumnsSorter;
	}

	return RowDefSorter._noSorting;
};

/** @public
* @param {string} key
* @param {*} value
*/
RowDefSorter.prototype.reset = function() {
	if(this._sortParams.length) {
		this._sortParams.length = 0;
	}
};
/** @public
* @param {string} key
* @param {*} value
*/
RowDefSorter.prototype.setContext = function(key, value) {
	this._globalContext[key] = value;
};
/** @public
* @param {string} field
* @param {Function} logic
* @param {boolean} rowSorting
* @param {string} order
* @param {number} colIndex
* @param {*} colDef
*/
RowDefSorter.prototype.addColumnContext = function(field, logic, rowSorting, order, colIndex, colDef) {
	if(!field) {
		field = "";
	}
	let sortPriority = this._sortParams.length;
	let ctx = this._ctxCaches[sortPriority];
	if(!ctx) {
		ctx = this._ctxCaches[sortPriority] = cloneObject(this._globalContext);
	}
	let orderNum = 0;
	if(order === "a") {
		orderNum = 1;
	} else if(order === "d") {
		orderNum = -1;
	}

	let params = [
		field, // 0
		(typeof logic === "function") ? logic : null,  // 1
		ctx, // 2
		rowSorting, // 3
		orderNum  // 4
	];

	ctx["colIndex"] = colIndex;
	ctx["field"] = field;
	ctx["formattedField"] = field + "_FORMATTED";
	ctx["colDef"] = colDef;

	this._sortParams.push(params);
};

/** @private
* @param {RowDefinition} rowDefA
* @param {RowDefinition} rowDefB
* @param {number} order
* @return {number}
*/
RowDefSorter._noSorting = function(rowDefA, rowDefB, order) {
	return 0;
};
/** @private
* @param {RowDefinition} rowDefA
* @param {RowDefinition} rowDefB
* @param {number} order
* @return {number}
*/
RowDefSorter.prototype._defaultSorter = function(rowDefA, rowDefB, order) {
	let field = this._primaryParams[0];
	return _defaultCompare(
		rowDefA.getData(field),
		rowDefB.getData(field),
		order
	);
};
/** @private
* @param {RowDefinition} rowDefA
* @param {RowDefinition} rowDefB
* @param {number} order
* @return {number}
*/
RowDefSorter.prototype._dataSorter = function(rowDefA, rowDefB, order) {
	let params = this._primaryParams;
	let field = params[0];
	let sortLogic = params[1];
	return sortLogic(
		rowDefA.getData(field),
		rowDefB.getData(field),
		order,
		params[2]
	);
};
/** @private
* @param {RowDefinition} rowDefA
* @param {RowDefinition} rowDefB
* @param {number} order
* @return {number}
*/
RowDefSorter.prototype._rowDefSorter = function(rowDefA, rowDefB, order) {
	let params = this._primaryParams;
	let sortLogic = params[1];
	return sortLogic(rowDefA, rowDefB, order, params[2]);
};
/** @private
* @param {RowDefinition} rowDefA
* @param {RowDefinition} rowDefB
* @param {number} primaryOrder
* @return {number}
*/
RowDefSorter.prototype._multiColumnsSorter = function(rowDefA, rowDefB, primaryOrder) {
	let sortParams = this._sortParams;
	let sortCount = sortParams.length;
	for(let i = 0; i < sortCount; ++i) {
		let params = sortParams[i];
		let field = params[0];
		let sortLogic = params[1];
		let ctx = params[2];
		let rowSorting = params[3];
		let orderNum = params[4];
		let ret = 0;
		if(sortLogic) {
			if(rowSorting) {
				ret = sortLogic(rowDefA, rowDefB, orderNum, ctx);
			} else {
				ret = sortLogic(rowDefA.getData(field), rowDefB.getData(field), orderNum, ctx);
			}
		} else {
			ret = _defaultCompare(rowDefA.getData(field), rowDefB.getData(field), orderNum);
		}
		if(ret) {
			return ret;
		}
	}
	return 0;
};

/* harmony default export */ const js_RowDefSorter = (RowDefSorter);

;// ./node_modules/@grid/core/es6/grid/plugins/SortableTitlePlugin.js
/* eslint-disable */







/* eslint-enable */

/** Fired when user click and before the sorting operation.
* @event SortableTitlePlugin#preClicked
* @property {number} colIndex
* @property {Element} target Element being clicked
* @example
* function onPreClicked(e) {
*   e.cancel = true; // Cancel sorting operation
* }
*/

/** Fired when user click and after the sorting operation
* @event SortableTitlePlugin#clicked
* @property {number} colIndex
* @property {string} sortOrder "a" for "ascending, "d" for descending, and "n" for none
* @property {string} dataColumnName Field that is used for sorting
*/

/** Fired just before sorting data. This allows additional modification before the actual sorting
* @event SortableTitlePlugin#preDataSorting
* @property {boolean=} isUserAction This has true value, if event initiate from clicking title/header section
*/

/** Fired once column is sorted and before {@link SortableTitlePlugin#clicked}
* @event SortableTitlePlugin#columnSorted
* @property {number} colIndex
* @property {string} sortOrder "a" for "ascending, "d" for descending, and "n" for none
* @property {string} sortedField Field that is used for sorting
* @property {Array.<number>} colIndices An array of colIndex
* @property {Array.<string>} sortOrders An array of sortOrder
* @property {Array.<string>} sortedFields An array of sortedField
*/

/** @private
* @type {string}
*@constant
*/
const SortableTitlePlugin_ROW_DEF = "ROW_DEF";

/** @constructor
* @extends {EventDispatcher}
* @param {SortableTitlePlugin.Options=} options
*/
let SortableTitlePlugin = function (options) { // TODO: Extract SortableTitlePlugin to grid extension.
	let _t = this;
	_t._onSectionAdded = _t._onSectionAdded.bind(_t);
	_t._onMouseDown = _t._onMouseDown.bind(_t);

	_t._onUIUpdated = _t._onUIUpdated.bind(_t);

	_t._onColumnAdded = _t._onColumnAdded.bind(_t);
	_t._onColumnRemoved = _t._onColumnRemoved.bind(_t);
	_t._onItemSortingClicked = _t._onItemSortingClicked.bind(_t);
	_t.refresh = _t.refresh.bind(_t);
	_t._rowDefSorter = _t._rowDefSorter.bind(_t);

	_t._hosts = [];
	_t._sortLogic = {};

	_t._addEvents(
		"preClicked"
		, "clicked"
		, "preDataSorting"
		, "columnSorted"
	);

	_t._sortStates = [];
	_t._sortingSequence = ["a", "d"];

	if(options) {
		_t.config({ "sorting": options });
	}
};
es6_Ext.inherits(SortableTitlePlugin, event_EventDispatcher);

/**
 * @typedef {"a" | "ascending" | "d" | "descending" | "n" | null} SortableTitlePlugin~SortOrder
 */

/** An object that specifies how sorting should be performed
 * @typedef {Object} SortableTitlePlugin~InitialSort
 * @property {number=} colIndex Column index of a column to be sorted
 * @property {string=} colId Column id of a column to be sorted
 * @property {string=} field Field of a column to be sorted
 * @property {SortableTitlePlugin~SortOrder=} sortOrder=null Set to "d" for descending order and "a" for ascending order
 * @property {SortableTitlePlugin~SortOrder=} order Alias to sortOrder
 */
SortableTitlePlugin.InitialSort;

/** An object that specifies how sorting should be performed
 * @typedef {Object} SortableTitlePlugin~SortingDefinition
 * @property {number=} colIndex Column index of a column to be sorted
 * @property {string=} colId Column id of a column to be sorted
 * @property {string=} field Field of a column to be sorted
 * @property {SortableTitlePlugin~SortOrder=} sortOrder=null Set to "d" for descending order and "a" for ascending order
 * @property {SortableTitlePlugin~SortOrder=} order Alias to sortOrder
 */
SortableTitlePlugin.SortingDefinition;

/** The sorting properties for column option.
 * @typedef {Object} SortableTitlePlugin~ColumnOptions
 * @property {boolean=} sortable=false If enable, the column can be sorted by user click
 * @property {string=} sortBy Field to be used for sorting. If defined, user can click at the header section to sort the column
 * @property {string=} sort="" Deprecated Alias to `sortOrder`
 * @property {string=} sortOrder="" The column can be sorted in ascending and descending order by specifying the order "a", "d". This option will sort the column when it is initialized or inserted at runtime.
 * @property {DataTable.SortLogic=} sortLogic=null Custom compare function for sorting
 * @property {DataTable.SortLogic=} sortingLogic Alias to `sortLogic`
 * @property {DataTable.SortLogic=} sorter Alias to `sortLogic`
 * @property {Array.<SortableTitlePlugin~SortOrder>=} sortingSequence Collection of sorting direction for the particular field.
 */
SortableTitlePlugin.ColumnOptions;

/** The configuration object for sorting options. This can be defined by `sorting` property in CompositeGrid configuration object.
 * @typedef {Object} SortableTitlePlugin~Options
 * @property {(SortableTitlePlugin~InitialSort|Array.<SortableTitlePlugin.SortingDefinition>)=} initialSort=null Set this config to initiate sorting after grid is created.
 * @property {(boolean|number)=} multicolumn=false Allow multiple columns to be sorted at the same time. If number is defined, it is treated as maximum number of column to be sorted.
 * @property {(boolean|number)=} multiColumn=false Alias to `multicolumn`
 * @property {boolean=} threeStatesSorting=false If enabled, sorting will cycle through ascending, descending and original order states. This option is deprecated in favor of sortingSequence option
 * @property {Object.<string, DataTable.SortLogic>=} sortLogic=null Custom logic for sorting based on the specified field. The key is field name and the value pair is a function.
 * @property {boolean=} disableDoubleClickToSort=true If disabled, double click will trigger sorting operation
 * @property {boolean=} sortableColumns=false If enabled, all columns will be sortable by field automatically
 * @property {boolean=} sortableIndicator=false If enabled, sort indicator (diamond) icon will always show regardless of sorting states. In Halo theme, this option will be enabled by default.
 * @property {boolean=} indicatorOnly=false If enabled, data will not be sorted, but the UI is rendered normally. This is useful for server side sorting.
 * @property {string=} mode Set to "manual" to disable data sorting. Deprecated in favor of `indicatorOnly`
 * @property {boolean=} disabled=false If enabled, header section cannot be clicked and no sort UI is rendered. Sorting can still be triggered only through `sortColumn` method.
 * @property {Function=} columnSorted A shortcut to define the columnSorted event handler
 * @property {Function=} preClicked A shortcut to define the preClicked event handler
 * @property {Function=} clicked A shortcut to define the clicked event handler
 * @property {Function=} preDataSorting A shortcut to define the preDataSorting event handler
 * @property {Array.<SortableTitlePlugin~SortOrder>=} sortingSequence=["a", "d"] Sequence in which sort order will be changed. The default value is ["a", "d"]. This option overrides threeStatesSorting
 * @property {boolean=} rowSorting=false A flag indicating that the given sortLogic takes entire row data instead of a field value for comparison
 */
SortableTitlePlugin.Options;


/** @type {Array.<Core>}
* @private
*/
SortableTitlePlugin.prototype._hosts = null; // For a plug-in that support multi-table

/** Order of item indicates sort priorities used in DataView. <br>
* Each state contains column name{string}, sort order {string}, and rendering cell {ElementWrapper}
* @private
* @type {!Array.<Object>}
*/
SortableTitlePlugin.prototype._sortStates;
/** Maximum possible number of sorting columns at any time. how a column is sorted.
* @private
* @type {number}
*/
SortableTitlePlugin.prototype._maxCount = 1;

/** @private
* @type {boolean}
*/
SortableTitlePlugin.prototype._dataSorting = true;
/** @private
* @type {boolean}
*/
SortableTitlePlugin.prototype._dblClickAllowed = false;

/** @private
* @type {number}
*/
SortableTitlePlugin.prototype._clickTimer = 0;
/** @private
* @type {number}
*/
SortableTitlePlugin.prototype._delayInterval = 0;
/** @private
* @type {number}
*/
SortableTitlePlugin.prototype._lastMouseDown = 0;
/** @private
* @type {SortableTitlePlugin.SortingDefinition}
*/
SortableTitlePlugin.prototype._initialSort = null;
/** @private
* @type {string}
*/
SortableTitlePlugin.prototype._mode = "";
/** @private
* @type {boolean}
*/
SortableTitlePlugin.prototype._sortableColumns = true;
/** @private
* @type {boolean}
*/
SortableTitlePlugin.prototype._sortableIndicator = false;
/** @private
 * @type {boolean}
 */
SortableTitlePlugin.prototype._solarTheme = false;
/** Logic map based on field name
* @private
* @type {Object.<string, Function>}
*/
SortableTitlePlugin.prototype._sortLogic;
/** To allow users/clients to store and provide all sorting logics by themselve
* @private
* @type {boolean}
*/
SortableTitlePlugin.prototype._userManagedLogic = false;
/** @private
* @type {boolean}
*/
SortableTitlePlugin.prototype._disabled = false;
/** @private
* @type {boolean}
*/
SortableTitlePlugin.prototype._frozenIndicator = false;
/** Since DataView in real-time grid has only one ROW_DEF column, we need to re-map sorting field to sort correctly.
* @private
* @type {boolean}
*/
SortableTitlePlugin.prototype._rowDefMode = false;
/**
* @private
* @type {Function}
*/
SortableTitlePlugin.prototype._userComparer = null;

/** @private
* @type {!Object.<string, string>}
*/
SortableTitlePlugin._icons = {};

/** @private
* @type {!Array.<string>}
*/
SortableTitlePlugin.prototype._sortingSequence;
/** @private
* @type {Object.<string, Array>}
*/
SortableTitlePlugin.prototype._sortingSequenceMap = null;


/** @type {string}
* @private
* @const
*/
let CODE_UP = "\u25B2";
/** @type {string}
* @private
* @const
*/
let CODE_DOWN = "\u25BC";
/** @type {string}
* @private
* @const
*/
let CODE_DIAMOND = "\u25CA";

/** The name must be readonly and unique among all other plugins
* @public
* @return {string}
*/
SortableTitlePlugin.prototype.getName = function () {
	return "SortableTitlePlugin"; // Read Only
};

/** @public
* @param {!Core} host
* @param {Object=} options
* @description Suppress google compiler for argument options because Internal Plugin doesn't support.
* @suppress {checkTypes}
*/
SortableTitlePlugin.prototype.initialize = function (host, options) {
	if (this._hosts.indexOf(host) >= 0) { return; }
	this._hosts.push(host);

	if (this._hosts.length === 1) {
		host.listen("columnAdded", this._onColumnAdded);
		host.listen("columnRemoved", this._onColumnRemoved);
	}
	host.listen("sectionAdded", this._onSectionAdded);
	host.listen("postSectionRender", this._onUIUpdated);
	host.listen("dataSourceChanged", this.refresh);

	this.config(options);
};

/**For disposing any resource
* @public
* @param {!Core} host
*/
SortableTitlePlugin.prototype.unload = function (host) {
	let at = this._hosts.indexOf(host);
	if (at < 0) { return; }

	this._hosts.splice(at, 1);
	host.unlisten("sectionAdded", this._onSectionAdded);
	host.unlisten("postSectionRender", this._onUIUpdated);
	host.unlisten("dataSourceChanged", this.refresh);

	host.unlisten("columnAdded", this._onColumnAdded);
	host.unlisten("columnRemoved", this._onColumnRemoved);

	if (this._hosts.length <= 0) {
		if (this._clickTimer) {
			clearTimeout(this._clickTimer);
			this._clickTimer = 0;
		}
	}
};

/** @public
*/
SortableTitlePlugin.prototype.afterInit = function () {
	let t = this;

	// Set logic to the data source, since the data source not available during the initialization
	t.setSortLogics(t._sortLogic);

	let userInput = t._initialSort; // This will be passed from the user when they want to start with initialize sort
	if (userInput != null) {
		if(Array.isArray(userInput)) {
			if(t._maxCount === 1) { // TODO: max count could be part of sortColumns method
				t.sortColumns(userInput.slice(0, 1));
			} else {
				t.sortColumns(userInput);
			}
		} else {
			t.sortColumns([userInput]);
		}
	}
};

/** @public
* @param {Object=} options Core model object
*/
SortableTitlePlugin.prototype.config = function (options) {
	if (!options) { return; }

	let t = this;

	let icons = options["icons"];
	if (icons != null) {
		if (icons["sorting"]) {
			SortableTitlePlugin._icons = icons["sorting"];
		}
	}
	let themeName = options["themeName"];
	if (!themeName || themeName.indexOf("solar") >= 0) {
		t._solarTheme = true;
	} else {
		t._solarTheme = false;
	}
	t._initialSort = t._initialSortByColumnField(options);

	let extOptions = options["sorting"];
	if (!extOptions) {
		return;
	}

	if(extOptions["ROW_DEF"]) {
		this._rowDefMode = true;
	}
	if(extOptions["userManagedLogic"]) {
		t._userManagedLogic = true;
	}

	let val = extOptions["multiColumn"] || extOptions["multicolumn"];
	if (val != null) {
		if (val === true) val = -1;
		if (val === false) val = 1;
		t.setSortableColumnCount((typeof val == "number" && val > 0) ? val : -1);
	}

	val = extOptions["sortableIndicator"];
	if (val != null) {
		t._sortableIndicator = val; // This can override default halo theme
	}

	val = extOptions["disabled"];
	if (val != null) {
		t._disabled = val ? true : false;
	}

	val = extOptions["threeStatesSorting"];
	if (val != null) {
		t.disableTwoStateSorting(val);
	}

	val = extOptions["sortingSequence"];
	if (val != null) {
		if(Array.isArray(val)) {
			t._sortingSequence = val;
		}
	}

	val = extOptions["mode"];
	if (val != null) {
		t._mode = val;
		if (val === "manual") {
			t.disableDataSorting();
		}
	}

	val = extOptions["indicatorOnly"];
	if (val != null) {
		t.disableDataSorting(val);
	}

	val = extOptions["disableDoubleClickToSort"];
	if (val != null) {
		t.disableDoubleClickToSort(val);
	}

	val = extOptions["columnSorted"];
	if (typeof val === "function") {
		t.listen("columnSorted", val);
	}
	val = extOptions["preClicked"];
	if (typeof val === "function") {
		t.listen("preClicked", val);
	}
	val = extOptions["clicked"];
	if (typeof val === "function") {
		t.listen("clicked", val);
	}
	val = extOptions["preDataSorting"];
	if (typeof val === "function") {
		t.listen("preDataSorting", val);
	}

	val = extOptions["sortableColumns"];
	if (val != null) {
		t._sortableColumns = val;
	}

	// Override _initialSortByColumnField
	val = extOptions["initialSort"];
	if (typeof val === "object") {
		t._initialSort = val;
	}

	val = extOptions["sortLogic"];
	if (typeof val === "object") {
		for (let field in val) {
			if (typeof val[field] === "function") {
				t._sortLogic[field] = val[field];
			}
		}
	}
};

/** @public
* @param {Object=} gridOptions
* @return {!Object}
*/
SortableTitlePlugin.prototype.getConfigObject = function (gridOptions) {
	let obj = gridOptions || {};

	let columns = obj["columns"];
	if (!columns) {
		columns = obj["columns"] = [];
	}

	let host = this._hosts[0];

	if (!host) return obj;

	let sortedColumns = this.getSortedColumns();
	let col, i, len;
	// Multiple column sorting cannot retain the sort order in the column option.
	if(this._maxCount === 1 && sortedColumns != null) {
		// assign sorting state to each columns
		len = sortedColumns.length;
		for (i = 0; i < len; i++) {
			let sortedColumn = sortedColumns[i];
			let colIndex = sortedColumn["colIndex"];
			col = columns[colIndex];
			if (!col) {
				col = columns[colIndex] = {};
			}
			col["sortOrder"] = sortedColumn["sortOrder"];
		}
	}

	len = host.getColumnCount();
	let opt, field;
	for(i = 0; i < len; ++i) {
		col = columns[i];
		if (!col) {
			col = columns[i] = {};
		}

		// TODO: We should get the state from the core, instead of getting the sort options.
		opt = this._getSortOptions(i);

		if (!opt) { continue; }

		if(this._sortableColumns) {
			if (opt["disabled"]) {
				col["sortable"] = false;
			}
		} else if(!opt["disabled"]) {
			col["sortable"] = true;
		}
		field = opt["field"];
		if (field) {
			// WARNING: When Core's column has no field, sortBy property does not get returned
			if(col["field"] != null && col["field"] !== field) {
				col["sortBy"] = field;
			} // else The default value is an empty sortBy, which refers to the same column field by default.

			if (this._sortingSequenceMap && this._sortingSequenceMap[field]) {
				col["sortingSequence"] = this._sortingSequenceMap[field];
			}
		}

	}

	let extOptions = obj["sorting"];
	if(!extOptions) {
		extOptions = obj["sorting"] = {};
	}

	let val = sortedColumns;
	if(val != null) {
		extOptions["initialSort"] = val;
	}

	val = this._sortableIndicator;
	if(val !== false) {
		extOptions["sortableIndicator"] = val;
	}

	if(this._maxCount !== 1) {
		extOptions["multiColumn"] = this._maxCount === -1 ? true : this._maxCount;
	}
	if(this._dblClickAllowed) {
		extOptions["disableDoubleClickToSort"] = !this._dblClickAllowed;
	}

	val = this._sortableColumns;
	if(val !== true) {
		extOptions["sortableColumns"] = val;
	}

	val = this._dataSorting;
	if(val !== true) {
		extOptions["indicatorOnly"] = val;
	}
	// extOptions["mode"] = this._mode;
	if(this._disabled) {
		extOptions["disabled"] = true;
	}

	let sortingSeq = this._sortingSequence;
	let notDefaultSeq = true;
	if (sortingSeq.length === 2) {
		if (sortingSeq[0] === "a" && sortingSeq[1] === "d") { // Sorting Sequence is not the default one
			notDefaultSeq = false;
		}
	}
	if(notDefaultSeq) {
		extOptions["sortingSequence"] = sortingSeq.slice();
	}

	if(isEmptyObject(obj["sorting"])) {
		delete obj["sorting"];
	}

	return obj;
};

/** @public
* @description Function for ColumnMenuExtension to collect config of column menu
* @param {number} colIndex Column index
* @param {Object} config Column Menu configuration
* @return {!Object}
*/
SortableTitlePlugin.prototype.getColumnMenu = function (colIndex, config) {
	if (!config) {
		config = {};
	}

	let menus = config["menus"];
	if (!menus) {
		menus = config["menus"] = [];
	}

	let menu = menus[0];
	if (!menu) {
		menu = menus[0] = {
			"menuItems": []
		};
	}

	let order = this.getSortOrder(colIndex);

	// Menu items for this extension
	let extMenuItem = {
		"indexScore": 100,
		"label": "Sort Column",
		"icon": "sort-a-to-z",
		"group": "FREQUENTLY_USED",
		"subMenu": {
			"menuItems": [{
				"indexScore": 3,
				"label": "Ascending",
				"radioMode": true,
				"value": {
					"colIndex": colIndex,
					"order": "a"
				},
				"selected": order === "a",
				"click": this._onItemSortingClicked
			}, {
				"indexScore": 2,
				"label": "Descending",
				"radioMode": true,
				"value": {
					"colIndex": colIndex,
					"order": "d"
				},
				"selected": order === "d",
				"click": this._onItemSortingClicked
			}, {
				"indexScore": 1,
				"label": "Un-Sort",
				"radioMode": true,
				"value": {
					"colIndex": colIndex,
					"order": "n"
				},
				"selected": order === "n",
				"click": this._onItemSortingClicked
			}]
		}
	};

	// Push menu items to config
	menu["menuItems"].push(extMenuItem);

	return config;
};

/** @private
* @param {Object} e
*/
SortableTitlePlugin.prototype._onItemSortingClicked = function (e) {
	let host = this._hosts[0];
	let columnMenuExt = host["getPlugin"]("ColumnMenuPlugin");
	columnMenuExt["hide"]();

	let value = e["currentTarget"]["value"];
	let colIndex = value["colIndex"];
	let order = value["sortOrder"];
	this.sortColumn(colIndex, order);
};

/** @public
* @return {Object.<string, string>} "a" for ascending, "d" for descending, and "n" for no sorting
*/
SortableTitlePlugin.prototype.getIcons = function () {
	return SortableTitlePlugin._icons;
};

/** @public
* @param {Object.<string, Function>} sortLogic
*/
SortableTitlePlugin.prototype.setSortLogics = function (sortLogic) { // TODO: This should be deprecated
	if (sortLogic != null) {
		let host = this._hosts[0];
		if (host) {
			let ds = host.getDataSource();
			for (let field in sortLogic) {
				this._setSortLogic(ds, field, sortLogic[field]);
			}
		}
	}
};

/** @public
* @param {string} field
* @param {Function} func
*/
SortableTitlePlugin.prototype.setSortLogic = function (field, func) { // TODO: This should be deprecated
	if (typeof field === "string") {
		let host = this._hosts[0];
		if (host) {
			let ds = host.getDataSource();
			this._setSortLogic(ds, field, func);
		}
	}
};

/** @private
* @param {Object} ds
* @param {string} field
* @param {Function} func
*/
SortableTitlePlugin.prototype._setSortLogic = function (ds, field, func) { // TODO: This is deprecated
	if (ds && typeof field === "string") {
		if (field === "*") {
			ds.setSortingLogic(func);
		} else {
			ds.setColumnSortingLogic(field, func);
		}
	}
};

/** @public
* @param {number=} colIndex If colIndex is undefined or null the first sorted column is used instead
* @return {string} "a" for ascending, "d" for descending, and "n" for no sorting
*/
SortableTitlePlugin.prototype.getSortOrder = function (colIndex) {
	let priority = (colIndex != null) ? this.getSortPriority(colIndex) : 0;
	let state = this._sortStates[priority];
	let sortOrder = state ? state["sortOrder"] : "";
	return sortOrder || "n";
};
/** Return the first (highest priority) sorted column's index
* @public
* @param {number=} priority
* @return {number} Return negative value if there is no sorted column
*/
SortableTitlePlugin.prototype.getSortedColumnIndex = function (priority) {
	return this._getColumnIndex(this._sortStates[priority || 0]);
};
/** Start with 0 (the first column that is being sorted). The highest number will be sorted last. Negative value means the specified column is not sorted
* @public
* @param {number} colIndex
* @param {string=} colRef field or column id
* @return {number}
*/
SortableTitlePlugin.prototype.getSortPriority = function (colIndex, colRef) {
	let host = this._hosts[0];
	if(host) {
		let i;
		let len = this._sortStates.length;
		if(colIndex >= 0) {
			let colId = host.getColumnId(colIndex);
			if(colId) {
				for(i = 0; i < len; ++i) {
					if(this._sortStates[i]["colId"] === colId) {
						return i;
					}
				}
			}
		} else if(colRef) {
			for(i = 0; i < len; ++i) {
				let sortState = this._sortStates[i];
				if(sortState["colId"] === colRef) {
					return i;
				}
				if(sortState["field"] === colRef) {
					return i;
				}
			}
		}
	}
	return -1;
};
/** @public
* @param {number} colIndex
* @return {boolean}
*/
SortableTitlePlugin.prototype.isColumnSorted = function (colIndex) {
	return this.getSortPriority(colIndex) >= 0;
};
/** @public
* @return {Array.<Object>} Array of objects with colIndex, colId, field, and sortOrder properties
*/
SortableTitlePlugin.prototype.getSortedColumns = function () {
	let len = this._sortStates.length;
	if (len <= 0) { return null; }

	let arr = new Array(len);
	for (let i = 0; i < len; ++i) {
		let colIndex = this.getSortedColumnIndex(i);
		let state = this._sortStates[i];
		let retObj = {};
		let colId = "";
		if(colIndex >= 0) {
			retObj["colIndex"] = colIndex;
			colId = this._getColumnId(colIndex);
		}
		if(colId) {
			retObj["colId"] = colId;
		}
		let field = state["field"] || "";
		if(field) {
			retObj["field"] = field;
		}
		retObj["sortOrder"] = state["sortOrder"] || "n";
		arr[i] = retObj;
	}
	return arr;
};
/** @public
* @return {boolean}
*/
SortableTitlePlugin.prototype.isSorting = function () {
	return this._sortStates.length ? true : false;
};

/** @public
* @param {number|string} colRef Column index or field
* @param {string=} sortOrder "a" for ascending, "d" for descending, and "n" for no sorting
* @param {Object=} opt_arg Event argument to be sent with preDataSorting event
* @fires SortableTitlePlugin#preDataSorting
* @fires SortableTitlePlugin#columnSorted
*/
SortableTitlePlugin.prototype.sortColumn = function (colRef, sortOrder, opt_arg) {
	let sortOptions = this._prepareSorting(colRef, sortOrder);
	this._sortColumn(sortOptions, opt_arg);
	if (opt_arg && opt_arg["isUserAction"]) { // Currently, the 'isUserAction' flag is triggered by a user clicking on the title and clicking the filter dialog. TODO: preClicked should be firing too.
		let ce = {};
		ce["colIndex"] = sortOptions.colIndex;
		ce["sortOrder"] = this.getSortOrder(sortOptions.colIndex);
		ce["dataColumnName"] = this.getColumnSortingField(sortOptions.colIndex); // This should be deprecated
		this._dispatch("clicked", ce);
	}
};

/** Sort multiple columns at once
* @public
* @param {Array.<SortableTitlePlugin.SortingDefinition>} sortOptions Input an array to specify the multiple column sort order
* @param {Object=} opt_arg Event argument to be sent with preDataSorting event
* @fires SortableTitlePlugin#preDataSorting
* @fires SortableTitlePlugin#columnSorted
*/
SortableTitlePlugin.prototype.sortColumns = function (sortOptions, opt_arg) {
	if (Array.isArray(sortOptions)) {
		let len = sortOptions.length;
		let states = new Array(len);
		for (let i = 0; i < len; i++) {
			states[i] = this._prepareSorting(sortOptions[i]);
		}
		this._sortColumn(states, opt_arg);
	}
};
/** @description Clear all sorting states
* @public
* @param {Object=} opt_arg Event argument to be sent with preDataSorting event
*/
SortableTitlePlugin.prototype.clearSortState = function (opt_arg) {
	if(this._popSortState(0, opt_arg)) {
		this.updateSortSymbols();
	}
};
/** @description Perform sorting with the same parameter.
* @public
* @fires SortableTitlePlugin#preDataSorting
*/
SortableTitlePlugin.prototype.refresh = function () {
	if (this._sortStates.length) {
		this._sortDataView();
		this.updateSortSymbols();
	}
};

/** @description Perform sorting with the same parameter. Alias to {@link SortableTitlePlugin#refresh}
* @public
* @function
*/
SortableTitlePlugin.prototype.restoreUserActions = SortableTitlePlugin.prototype.refresh;

/** @public
* @param {number} count
*/
SortableTitlePlugin.prototype.setSortableColumnCount = function (count) {
	this._maxCount = count ? count : 1;
	this._popSortState(this._maxCount);
};

/** Deprecated. Use getSortedColumns instead of this
* @public
* @return {Array.<Object>} Array of sorted column states ordered by priority
* @see {@link SortableTitlePlugin#getSortedColumns}
*/
SortableTitlePlugin.prototype.getSortingStates = function () {
	return this._sortStates; // WARNING: Exposing internal variable
};

/** @public
*/
SortableTitlePlugin.prototype.updateSortSymbols = function () {
	if(!this._disabled) {
		for (let i = this._hosts.length; --i >= 0;) {
			this._updateSortableIndicator(i);
		}
	}
};

/** Get Array of fields used for sorting
* @public
* @return {!Array.<string>}
*/
SortableTitlePlugin.prototype.getDataColumnMap = function () {
	let host = this._hosts[0];
	let dataMap = [];
	if (host) {
		let colCount = host.getColumnCount();
		for (let i = 0; i < colCount; ++i) {
			dataMap[i] = this.getColumnSortingField(i);
		}
	}
	return dataMap;
};
SortableTitlePlugin.prototype.getDataColumnNames = SortableTitlePlugin.prototype.getDataColumnMap;

/** Set sorting field for multiple columns at once
* @public
* @param {Array.<string|null>} dataMap
*/
SortableTitlePlugin.prototype.setDataColumnMap = function (dataMap) { // TODO: This should be deprecated
	// This should work just like "sortBy"
	if (Array.isArray(dataMap)) {
		for (let i = dataMap.length; --i >= 0;) {
			this.setColumnSortingField(i, dataMap[i]);
		}
	}
};
SortableTitlePlugin.prototype.setDataColumnNames = SortableTitlePlugin.prototype.setDataColumnMap;

/** @public
* @param {number} colIndex
* @return {string} field A field used for sorting
*/
SortableTitlePlugin.prototype.getColumnSortingField = function (colIndex) {
	let sortOptions = this._getSortOptions(colIndex);
	// options["field"] is already consolidated from field, sortBy, and sortable properties.
	let field = (sortOptions) ? sortOptions["field"] : null;

	if (field == null && !this._rowDefMode) { // Old CompositeGrid may store sorting field in DataColumnName
		let host = this._hosts[0];
		if (host) {
			field = host.getColumnField(colIndex);
			if(!field) {
				field = host.getDataColumnName(colIndex);
			}
			if(field && sortOptions) {
				sortOptions["field"] = field; // Cache for later use
			}
		}
	}

	return field || "";
};
/** This is equivalent to setting sortBy in the configuration object.
* @public
* @param {number} colIndex
* @param {string|null} field A field used for sorting
* @param {Function=} sortLogic=null
* @param {boolean=} rowSorting=false A flag indicating that the given sortLogic takes entire row data instead of a field value for comparison
*/
SortableTitlePlugin.prototype.setColumnSortingField = function (colIndex, field, sortLogic, rowSorting) {
	if (typeof colIndex == "number" && field != null) {
		let options = this._newSortOptions(colIndex);
		if (!field) {
			field = "";
		}
		if (options["field"] !== field) {
			options["field"] = field;
		}
		if(sortLogic != null && !this._userManagedLogic) {
			if (options["sortLogic"] !== sortLogic) {
				options["sortLogic"] = sortLogic || null;
				if(rowSorting && sortLogic) {
					sortLogic["rowSorting"] = true; // Tell data view/table about the argument
				}
			}
		}
	}
};

/** @private
* @param {Object} sortState
* @returns {string}
*/
let _toField = function(sortState) {
	return sortState["field"] || "";
};
/** Return array of the fields from the currently sorted columns
* @public
* @return {!Array.<string>} fields of each sorting states
*/
SortableTitlePlugin.prototype.getColumnSortingFields = function () {
	return this._sortStates.map(_toField);
};

/** This is deprecated in favor of setSortingSequence method. Move existing sort order to the front of sorting sequence. New sort order will be added, if the specified sort order does not exist in current sorting sequence.
* @public
* @param {string|number|Array.<string|number>} colIdentifier string is data column name, while number is display column index. If null value is given, first sort order is set to grid level configuration
* @param {string=} sortOrder="a" "ascending", "descending", "none"
* @see {@link SortableTitlePlugin#setSortingSequence}
*/
SortableTitlePlugin.prototype.setFirstSortOrder = function (colIdentifier, sortOrder) {
	if (sortOrder) {
		sortOrder = SortableTitlePlugin._toSortOrder(sortOrder);
	} else {
		sortOrder = "a";
	}
	if (sortOrder !== "a" && sortOrder !== "d") {
		sortOrder = "a"; // There is no "none" as the first sorting order
	}

	let fields = null;
	if (Array.isArray(colIdentifier)) {
		fields = colIdentifier;
	} else {
		fields = [/** @type{string|number} */(colIdentifier)];
	}

	let len = fields.length;
	for (let i = 0; i < len; ++i) {
		let field = fields[i];
		let sortingSeq = this.getSortingSequence(field).slice();
		let index = sortingSeq.indexOf(sortOrder);
		if (index >= 0) {
			sortingSeq.splice(index, 1);
		}
		sortingSeq.unshift(sortOrder);
		this.setSortingSequence(sortingSeq, field);

	}
};

/** This is deprecated in favor of setSortingSequence method. If two-state sorting is disabled, "n" (original order) will be added to current sorting sequence. If two-state sorting is enabled, "n" will be removed from current sorting sequence.
* @public
* @param {boolean=} disabled
* @see {@link SortableTitlePlugin#setSortingSequence}
*/
SortableTitlePlugin.prototype.disableTwoStateSorting = function (disabled) {
	let at = this._sortingSequence.indexOf("n");
	if(disabled === false) { // two-state sorting
		if(at >= 0) {
			this._sortingSequence.splice(at, 1);
		}
	} else { // three-state sorting
		if(at < 0) {
			this._sortingSequence.push("n");
		}
	}

};
/** @public
* @param {boolean=} bool=true, if set to false it will be unfreeze indicator
*/
SortableTitlePlugin.prototype.freezeIndicator = function (bool) {
	let prevState = this._frozenIndicator;
	this._frozenIndicator = bool !== false;
	if(prevState && !this._frozenIndicator) { // from frozen to unfrozen, update the ui
		for (let i = this._hosts.length; --i >= 0;) {
			this._updateSortableIndicator(i);
		}
	}

};
/** @public
* @param {boolean=} disabled
*/
SortableTitlePlugin.prototype.disableSortSymbols = function (disabled) {
	// this._sortSymbols = disabled === false;
	// TODO: Clear existing UIs at run-time
};
/** @public
* @param {boolean=} disabled
*/
SortableTitlePlugin.prototype.disableDataSorting = function (disabled) {
	this._dataSorting = disabled === false;
};
/** @public
* @description To allow double click action to sort. Default value is false.
* @param {boolean=} disabled=false
*/
SortableTitlePlugin.prototype.disableDoubleClickToSort = function (disabled) {
	this._dblClickAllowed = !disabled;
};
/** This api is deprecated. Use {@link SortableTitlePlugin.prototype.disableColumnSorting} instead
 * @public
* @param {ElementWrapper} cell
* @param {boolean=} disabled
*/
SortableTitlePlugin.prototype.disableCellClick = function (cell, disabled) {
	if (cell) {
		cell.enableClass("no-sort", disabled !== false);
	}
};
/** @public
* @param {number|Array.<number>} columns
* @param {boolean=} disabled
*/
SortableTitlePlugin.prototype.disableColumnSorting = function (columns, disabled) {
	disabled = disabled !== false;
	if (typeof columns === "number") {
		this._disableColumnSorting(columns, disabled);
	} else if (Array.isArray(columns)) {
		let len = columns.length;
		for (let i = 0; i < len; ++i) {
			this._disableColumnSorting(columns[i] || 0, disabled);
		}
	}
};
/** @private
* @param {number} colIndex
* @param {boolean} disabled
*/
SortableTitlePlugin.prototype._disableColumnSorting = function (colIndex, disabled) {
	let sortOptions = this._newSortOptions(colIndex);
	if (sortOptions["disabled"] !== disabled) {
		sortOptions["disabled"] = disabled;
		// TODO: Remove existing sort
	}
};

/** @public
* @param {number=} opt_ms Delay before actual sorting in milliseconds. Setting undefined value will be 500ms. Setting 0 to instantly perform sorting
*/
SortableTitlePlugin.prototype.setClickDelayInterval = function (opt_ms) {
	this._delayInterval = (opt_ms != null) ? opt_ms : 500;
};

/** @private
* @description Get initial sort from column option `defaultSort` or `sort`
* @param {Object} options Core model object
* @returns {Object}
*/
SortableTitlePlugin.prototype._initialSortByColumnField = function (options) {
	let columns = options["columns"];
	let colCount = columns ?  columns.length : 0;
	for (let i = 0; i < colCount; i++) {
		let column = columns[i];
		let sortOrder = column["defaultSort"]; // composite and rt grid option for default sort order
		if (sortOrder) {
			// TODO: Support multi-column sorting
			return {
				"colIndex": i,
				"sortOrder": sortOrder
			};
		}
	}
	return null;
};

/** @private
* @param {Object} e
*/
SortableTitlePlugin.prototype._onSectionAdded = function (e) {
	if (e["sectionType"] !== "title") { return; }

	let section = /** @type{ILayoutGrid} */(e["section"]);
	if (section.getIndex() !== 0) return; // only topmost header can click
	section.listen("mousedown", this._onMouseDown);
	section.listen("click", this._onClickTitle.bind(this, e["sender"]), 0); // Sorting has higher priority
};

/** @private
* @param {Object} e
*/
SortableTitlePlugin.prototype._onUIUpdated = function (e) {
	if (e["sectionType"] === "title") {
		this.updateSortSymbols();
	}
};

/** @private
* @param {Object} hitObj
* @return {boolean}
* @fires SortableTitlePlugin#preClicked
*/
SortableTitlePlugin.prototype._canClickToSort = function (hitObj) {
	if(this._disabled) {
		return false;
	}

	let grid = hitObj["grid"];
	if (!grid || !grid.getElement()) { // The given grid has been disposed
		return false;
	}

	let cep = this._getPlugin("CellEditingPlugin");
	if (cep && cep["isEditing"]()) { // Current cell is being edited, so skip the sort
		return false;
	}

	this._dispatch("preClicked", hitObj);
	if (hitObj["cancel"]) { // User want to cancel the operation
		return false;
	}

	let colIndex = hitObj["colIndex"];
	let section = hitObj["section"];
	let rowIndex = hitObj["rowIndex"];

	if (!this._isCellSortable(section, colIndex, rowIndex)) {
		return false;
	}

	return true;
};

/** @private
* @param {Object} hitObj
* @fires SortableTitlePlugin#preDataSorting
* @fires SortableTitlePlugin#columnSorted
* @fires SortableTitlePlugin#clicked
*/
SortableTitlePlugin.prototype._proceedSorting = function (hitObj) {
	this._clickTimer = 0;

	if (this._canClickToSort(hitObj)) {
		let colIndex = hitObj["colIndex"];
		// _proceedSorting() method is always produced by user clicking at the header // Thus, this operation is guaranteed to be a user action
		this.sortColumn(colIndex, null, { "isUserAction": true });

		let grid = hitObj["grid"];
		if(grid && grid["focus"]) {
			grid["focus"]();
		}
	}
};

/** @private
* @param {MouseEvent} e
 */
SortableTitlePlugin.prototype._onMouseDown = function (e) {
	if (!e.button) { // left mouse only
		this._lastMouseDown = e.timeStamp;
	}
};
/** @private
* @param {string} pluginName
* @return {*|null}
 */
SortableTitlePlugin.prototype._getPlugin = function(pluginName) {
	let host = this._hosts[0];
	return (host) ? host.getPlugin(pluginName) : null;
};

/** @public
* @description Click title to sort with mouse event. This is for testing purpose.
* @ignore
* @param {Core} grid
* @param {Object=} mouseEvt
*/
SortableTitlePlugin.prototype.clickTitleByMouse = function (grid, mouseEvt) {
	this._onClickTitle(grid, mouseEvt);
};

/** @private
* @param {Core} grid
* @param {MouseEvent} e
 */
SortableTitlePlugin.prototype._onClickTitle = function (grid, e) {
	if(this._disabled) {
		return;
	}
	if (e.detail === 2 && !this._dblClickAllowed) { // dblclick
		if (this._clickTimer) {
			clearTimeout(this._clickTimer);
			this._clickTimer = 0;
		}
		return; // This is registered as double click in modern browser so we ignored the click
	} else if (e.shiftKey || e.ctrlKey || e.altKey || e.metaKey) { // If any modifier key is pressed, ignored the click
		return;
	}

	// Avoid click by dragging
	if (!(e.timeStamp - this._lastMouseDown < 500)) {
		return;
	}

	let hitObj = grid.getRelativePosition(e);
	if (!hitObj["hit"]) { // Click across multiple elements (dragging)
		return;
	}

	let csp = this._getPlugin("ColumnSelectionPlugin");
	if (csp && csp["isEnabled"]()) {
		let colIndex = hitObj["colIndex"];
		if (!csp["isSelectedColumn"](colIndex)) { // Sort only if the column is selected
			return;
		}
	}
	hitObj["target"] = e["target"];

	if (this._delayInterval > 0) {
		if (!this._clickTimer) {
			this._clickTimer = setTimeout(
				this._proceedSorting.bind(this, hitObj),
				this._delayInterval
			);
		}
	} else {
		this._proceedSorting(hitObj);
	}
};

/** @private
* @param {number} toIndex INCLUSIVE
* @param {Object=} opt_arg Event argument to be sent with preDataSorting event
* @return {boolean}
* @fires SortableTitlePlugin#preDataSorting
*/
SortableTitlePlugin.prototype._popSortState = function (toIndex, opt_arg) {
	if (toIndex < 0 || this._sortStates.length <= toIndex) { return false; }

	let dirty = false;
	for (let i = this._sortStates.length; --i >= toIndex;) {
		this._clearSortSymbols(this._sortStates[i]);
		dirty = true;
	}
	this._sortStates.length = toIndex;
	this._sortDataView(opt_arg);
	return dirty;
};

/** @private
* @param {string=} order "ascending", "descending", "none"
* @return {string}
*/
SortableTitlePlugin._toSortOrder = function(order) {
	if (order && typeof order === "string") {
		order = order.charAt(0).toLowerCase();
		if (order === "d" || order === "a") {
			return order;
		}
	}
	return "n";
};

/** Returns current sorting sequence for either a grid (all columns), or a single column.
* @public
* @param {(number|string)=} colRef Column index or column field. If no value is given or no field is found, grid level sorting sequence is returned
* @return {!Array.<string>}
*/
SortableTitlePlugin.prototype.getSortingSequence = function (colRef) {
	if (this._sortingSequenceMap && (colRef || colRef === 0)) {
		if(typeof colRef !== "string") {
			colRef = this.getColumnSortingField(/** @type{number} */(colRef));
		}
		if(this._sortingSequenceMap[colRef]) {
			return /** @type{!Array.<string>} */(this._sortingSequenceMap[colRef]);
		}
	}
	return this._sortingSequence;
};
/** Specify sorting sequence for either a grid (all columns), or a single column.
* @public
* @param {Array.<string>} sequence Collection of sorting direction. If null value or empty Array is given, sorting sequence will be reverted to default.
* @param {(number|string)=} colRef Column index or column field. If no value is given, sorting sequence will be set to grid level configuration.
* @example
* stPlugin.setSortingSequence(["d", "a", "n"]); // Set three-state sorting and start with "d"
* stPlugin.setSortingSequence(["a"]); // Force to sort with ascending order only and cannot change to other order
* stPlugin.setSortingSequence(["d", "a"], "field"); // The specified column will have different sorting sequence
*/
SortableTitlePlugin.prototype.setSortingSequence = function (sequence, colRef) {
	if (Array.isArray(sequence)) {
		if(!sequence.length) {
			sequence = null; // Empty array is treated as null value
		}
	} else if(sequence) { // true, 1, "string", and other datatypes are not allowed
		return;
	}

	let field;
	if (colRef != null) {
		if (typeof colRef === "string") {
			field = colRef;
		} else if (typeof colRef === "number") {
			field = this.getColumnSortingField(colRef);
		}
	}

	if (sequence) {
		// Ensure that user sequence has only valid sort order
		let sortingSequence = sequence.map(SortableTitlePlugin._toSortOrder);
		if (field) {
			if (!this._sortingSequenceMap) {
				this._sortingSequenceMap = {};
			}

			this._sortingSequenceMap[field] = sortingSequence;
		} else {
			this._sortingSequence = sortingSequence;
		}
	} else { // No sequence is specified, revert existing sequence to default
		if(field) {
			if (this._sortingSequenceMap) {
				delete this._sortingSequenceMap[field];
			}
		} else { // Reset current sorting sequence to default
			this._sortingSequence = this._sortingSequence.length > 2 ? ["a", "d", "n"] : ["a", "d"];
		}
	}
};
/** @public
*/
SortableTitlePlugin.prototype.clearAllColumnSortingSequences = function () {
	this._sortingSequenceMap = null;
};
/** @private
* @param {Object} rowA
* @param {Object} rowB
* @param {number} sortOrder
* @param {*} context
* @returns {number}
*/
SortableTitlePlugin.prototype._rowDefSorter = function (rowA, rowB, sortOrder, context) {
	return this._userComparer(
		rowA[SortableTitlePlugin_ROW_DEF].getRowData(),
		rowB[SortableTitlePlugin_ROW_DEF].getRowData(),
		sortOrder,
		context
	);
};
/** @public
* @description Sorting all separators according to compare function. If compare function is not specified, separators will will be sort by current sorting states.
* @param {Function} comparer Compare function
*/
SortableTitlePlugin.prototype.sortSeparators = function (comparer) {
	let host = this._hosts[0];
	let dv = host.getDataSource();
	if(comparer){
		if(this._rowDefMode) {
			this._userComparer = comparer;
			dv.sortSeparators(this._rowDefSorter);
			this._userComparer = null;
		} else {
			dv.sortSeparators(comparer);
		}
	} else {
		let sortLogics = dv.getSortingLogics();
		let sortOrders = [];
		let sortFields = [];
		let sortStateCount = this._sortStates.length;
		for(let i = 0; i < sortStateCount; i++){
			let sortState = this._sortStates[i];
			let field = this._rowDefMode ? SortableTitlePlugin_ROW_DEF : sortState["field"];
			sortOrders.push(sortState["sortOrder"]);
			sortFields.push(field);
		}
		dv.sortSeparators(sortLogics, sortOrders, sortFields);
	}
};


/** @private
* @param {(number|string|SortableTitlePlugin.SortingDefinition)} sortingDef This could be column index, field, or SortingDefinition
* @param {string=} sortOrder "a" for ascending, "d" for descending, and "n" for no sorting
* @returns {Object} state Returns null, if nothing is changed
*/
SortableTitlePlugin.prototype._prepareSorting = function (sortingDef, sortOrder) {
	if (!this._maxCount) {
		return null;
	}
	let host = this._hosts[0];
	if(!host) {
		return null;
	}

	let colIndex = -1;
	let colRef = "";
	if(typeof sortingDef === "number") {
		colIndex = sortingDef;
	} else if(typeof sortingDef === "string") { // This would be either column id or field
		colRef = sortingDef;
		colIndex = this._getColumnIndex(colRef);
	} else if(sortingDef && typeof sortingDef === "object") {
		colRef = sortingDef["colId"] || sortingDef["field"];
		if(colRef) {
			colIndex = this._getColumnIndex(colRef);
		} else if(typeof sortingDef["colIndex"] === "number") {
			colIndex = sortingDef["colIndex"];
		}
		if(!sortOrder) {
			sortOrder = sortingDef["sortOrder"] || sortingDef["order"] || null;
		}
	}

	let field = "";
	let sortLogic = null;
	if(colIndex >= 0) {
		field = this.getColumnSortingField(colIndex);
		let sortOptions = this._getSortOptions(colIndex);
		if(sortOptions) {
			sortLogic = sortOptions["sortLogic"];
		}
	} else if(colRef) {
		field = colRef; // Assume that colRef is a field and not a column id
	}

	let priority = this.getSortPriority(colIndex, field);
	let curState = (priority >= 0) ? this._sortStates[priority] : null;
	let curOrder = curState ? curState["sortOrder"] : "n";

	// Getting next sort order
	let nextOrder = "n";
	if (sortOrder == null) { // Auto order switch
		let sortingSequence = this.getSortingSequence(field);

		let sequenceIndex = sortingSequence.indexOf(curOrder);
		if (sequenceIndex < 0) {
			sequenceIndex = 0;
		} else {
			sequenceIndex = (sequenceIndex + 1) % sortingSequence.length;
		}
		nextOrder = sortingSequence[sequenceIndex];
	} else {
		nextOrder = SortableTitlePlugin._toSortOrder(sortOrder);
	}

	if (curState) { // Remove existing state to rearrange priority later
		if (curOrder === nextOrder) {
			this.updateSortSymbols(); // Update Display
			return null; // Nothing has changed
		}

		this._sortStates.splice(priority, 1); // Remove sort states
		this._clearSortSymbols(curState); // Clear any previous symbols
	}

	if (nextOrder === "a" || nextOrder === "d") { // Reorder sort states
		if(colIndex < 0 && !field) {
			return null;
		}
		if (!curState) {
			curState = {};
			let colId = host.getColumnId(colIndex);
			if(colId) {
				curState["colId"] = colId;
			}
		}
		curState["field"] = field;
		curState["sortLogic"] = sortLogic || null;

		this._sortStates.push(curState); // Latest sorting have lowest priority (multiColumn sorting)
	}

	if (curState) { // Update current state
		if(curOrder !== "n") {
			curState["prevOrder"] = curOrder; // For backward compatibility
		} else {
			delete curState["prevOrder"];
		}
		curState["order"] = nextOrder; // For backward compatibility
		curState["sortOrder"] = nextOrder;
	}

	let clearedSortColIndexes = [];
	if (this._maxCount > 0) {
		while (this._sortStates.length > this._maxCount) { // Shift excess
			let index = this._clearSortSymbols(this._sortStates.shift());
			if (index !== null) {
				clearedSortColIndexes.push(index);
			}
		}
	}

	return { // TODO: Refactor this code as it is only used by "columnSorted" event (unnecessarily complex).
		"colIndex": colIndex,
		"sortedField": field,
		"sortOrder": nextOrder || "n",
		"prevOrder": curOrder,
		"clearedSortColIndexes": clearedSortColIndexes
	};
};

/** @private
* @param {Object|Array.<Object>} states States from _prepareSorting for dispatching columnSorted event.
* @param {Object=} opt_arg Event argument to be sent with preDataSorting event
* @fires SortableTitlePlugin#preDataSorting
* @fires SortableTitlePlugin#columnSorted
*/
SortableTitlePlugin.prototype._sortColumn = function (states, opt_arg) {
	if(!states) {
		return;
	}

	this._sortDataView(opt_arg); // Update Data
	this.updateSortSymbols(); // Update Display

	let evtArg = null;
	if(Array.isArray(states)) {
		let len = states.length;
		for(let i = 0; i < len; ++i) {
			let state = states[i];
			if(state) {
				if(evtArg) {
					if(!evtArg["colIndices"]) {
						evtArg["colIndices"] = [];
						evtArg["sortedFields"] = [];
						evtArg["sortOrders"] = [];
						evtArg["prevOrders"] = [];
					}

					evtArg["colIndices"].push(state["colIndex"]);
					evtArg["sortedFields"].push(state["sortedField"]);
					evtArg["sortOrders"].push(state["sortOrder"]);
					evtArg["prevOrders"].push(state["prevOrder"]);
				} else {
					evtArg = state;
				}
			}
		}
		evtArg = states[0];
	} else {
		evtArg = states;
	}
	this._dispatch("columnSorted", evtArg);
};

/** @private
* @param {Object} e
*/
SortableTitlePlugin.prototype._onColumnAdded = function (e) {
	let t = this;
	let colIndex = e["colIndex"];
	let column = e["context"] || {};

	let sortable = !!t._sortableColumns;
	let field = column["field"];
	let sortBy = column["sortBy"];

	if (sortBy != null) { // composite grid option
		sortable = !!sortBy;
		field = sortBy;
	}
	if (column["sortable"] != null) { // rt grid option
		sortable = !!column["sortable"];
	}

	let sortLogic = column["sortLogic"] || column["sortingLogic"] || column["sorter"]; // Originally, rt grid support option for sortLogic by column
	if(!sortLogic) {
		sortLogic = t._sortLogic[field];
	}
	if(typeof sortLogic !== "function") {
		sortLogic = null;
	}

	let sortingSequence = column["sortingSequence"];
	if (sortingSequence != null && field) { // Make sure that column config does not affect grid level config
		t.setSortingSequence(sortingSequence, field);
	}

	if (sortable) { // TODO: Check if empty field should be sortable
		t.setColumnSortingField(colIndex, field, sortLogic, column["rowSorting"]);
	} else {
		t.disableColumnSorting(colIndex, true);
	}

	let sortOrder = column["sortOrder"] || column["sort"];
	if(sortOrder != null) {
		sortOrder = SortableTitlePlugin._toSortOrder(sortOrder); // Invalid input will return "n"
		if(sortOrder !== "n") {
			t.clearSortState(); // clear previous sorting state
			t.sortColumn(colIndex, sortOrder);
		}
	}
};

/** @private
* @param {Object} e
* @fires SortableTitlePlugin#preDataSorting
* @fires SortableTitlePlugin#columnSorted
*/
SortableTitlePlugin.prototype._onColumnRemoved = function (e) {
	let sortCount = this._sortStates.length;
	if (!sortCount) {
		return;
	}
	let colId = e["colId"];
	if (!colId) {
		return;
	}
	// WARNING: columnRemoved event could be fired repeatedly. Sorting could be triggered multiple times
	for(let i = 0; i < sortCount; ++i) {
		let sortState = this._sortStates[i];
		if (sortState["colId"] === colId) {
			this._sortStates.splice(i, 1);
			this._clearSortSymbols(sortState);
			this._sortDataView();
			break;
		}
	}
};

/** @private
* @return {string}
*/
SortableTitlePlugin._toRowDefField = function() {
	return SortableTitlePlugin_ROW_DEF;
};
/** @private
* @param {Object=} opt_action Action that raised the sort. This is optional
* @fires SortableTitlePlugin#preDataSorting
*/
SortableTitlePlugin.prototype._sortDataView = function (opt_action) {
	opt_action = opt_action || {};
	this._dispatch("preDataSorting", opt_action);

	if (!this._dataSorting) { return; }

	let sortCount = this._sortStates.length;
	if(this._userManagedLogic && sortCount > 1) { // The logic is managed by the user. There is no point in using multi-column sorting
		sortCount = 1;
	}
	let orders = null;
	let sortLogics = null;
	let c_ref = null;
	if (sortCount) {
		orders = new Array(sortCount);
		sortLogics = new Array(sortCount);
		for (let i = 0; i < sortCount; ++i) {
			orders[i] = this._sortStates[i]["sortOrder"];
			sortLogics[i] = this._sortStates[i]["sortLogic"]; // sortLogic can be null
		}
		if(this._rowDefMode) { // DataView in real-time grid has only ROW_DEF column
			c_ref = this._sortStates.map(SortableTitlePlugin._toRowDefField);
		} else {
			c_ref = this.getColumnSortingFields();
		}
		if(this._userManagedLogic && c_ref.length > 1) {
			c_ref = c_ref.slice(0, 1);
		}
	}

	// Perform sorting even if there is no sort state
	for (let j = this._hosts.length; --j >= 0;) {
		let host = this._hosts[j];
		// TODO: In wrap mode all hosts does not need to be sorted
		let dataSource = host.getDataSource();
		if (dataSource) {
			let noOptimization = opt_action["isUserAction"] == true;
			dataSource.sort(c_ref, orders, sortLogics, noOptimization); // If length === 0 data is not sorted
		}
	}
};
/**`section` and `cell` variable in this scope can either be instance of LayoutGrid or SectionWriter (from tr-grid-printer).
 * Need to force calling public methods (section["isColumnActive"] and section["getCell"]) so it works while being minified.
 * @private
* @param {Object} state
* @return {number|null} The column index of cleared sort
*/
SortableTitlePlugin.prototype._clearSortSymbols = function (state) {
	let cells = state["targetCells"];
	if (!cells) {
		return null;
	}
	let colIndex = null;
	for (let j = cells.length; --j >= 0;) {	// One single cell per host
		let cell = cells[j];
		cell["removeClass"]("sorting");
		cell["removeClass"]("sortable");
		cell["removeFloatingIcon"]("sort-symbol");
		cell["removeFloatingIcon"]("priority-symbol");
		let section;
		if(colIndex == null && (section = cell["getSection"]())) {
			colIndex = section["getColumnIndex"](cell["getParent"]());
		}
	}
	cells.length = 0;
	return colIndex;
};
/**`section` and `cell` variable in this scope can either be instance of LayoutGrid or SectionWriter (from tr-grid-printer).
 * Need to force calling public methods (section["getColumnCount"] and section["getCell"]) so it works while being minified.
 * @private
* @param {number} hostIndex
*/
SortableTitlePlugin.prototype._updateSortableIndicator = function (hostIndex) {
	let t = this;
	let host = t._hosts[hostIndex];
	if (!host || this._frozenIndicator) { return; }

	let section = host.getSection("title");
	if (section == null) { return; }
	let colCount = section["getColumnCount"]();
	for (let col = colCount; --col >= 0;) {

		let rowCount = section["getRowCount"]();
		for (let r = 0; r < rowCount; r++) {

			let cell = section["getCell"](col, r, true);
			if (!cell) { continue; }

			// Clear all sorting icons
			cell["removeClass"]("sorting");
			cell["removeClass"]("sortable");
			cell["removeClass"]("asc");
			cell["removeClass"]("desc");
			cell["removeClass"]("edge-indicator");
			cell["removeFloatingIcon"]("sort-symbol");
			cell["removeFloatingIcon"]("priority-symbol");
			cell["removeFloatingIcon"]("sortable-indicator");

			if (r !== rowCount - 1) { continue; } // Sort icon should only appear in last row of section otherwise skip
			if (!t._isCellSortable(section, col, r)) { continue; }

			cell["addClass"]("sortable");

			let priority = t.getSortPriority(col);
			let state = t._sortStates[priority];

			let isSorting = state && state["sortOrder"] !== "n";
			let symbol, icon;
			if (isSorting) {
				if (!state["targetCells"]) {
					state["targetCells"] = [];
				}

				state["targetCells"][hostIndex] = cell; // NOTE: This can be improved.

				if (state["sortOrder"] === "a") {
					cell["addClass"]("asc");
				} else {
					cell["addClass"]("desc");
				}
				cell["addClass"]("sorting");

				if(t._solarTheme) { // Insert icon when use solar theme
					let textContent;

					if (state["sortOrder"] === "a") {
						textContent = CODE_UP;
						icon = SortableTitlePlugin._icons["ascending"];
					} else {
						textContent = CODE_DOWN;
						icon = SortableTitlePlugin._icons["descending"];
					}

					symbol = t._createIconElem(icon, textContent);
					symbol.className = "sort-symbol";
					cell["insertFloatingIcon"](symbol, 0);
				} else {
					cell["addClass"]("edge-indicator");
				}

				if (t._sortStates.length > 1) {
					symbol = document.createElement("span");
					symbol.className = "priority-symbol";
					symbol.textContent = priority + 1;
					cell["insertFloatingIcon"](symbol, 1);
				}
			} else if (t._sortableIndicator) { // Can sort but currently not sorting
				icon = SortableTitlePlugin._icons["sortable"];
				symbol = t._createIconElem(icon, CODE_DIAMOND);
				symbol.className = "sortable-indicator";
				cell["insertFloatingIcon"](symbol, 0);
			}
		}
	}
};

/** @private
* @param {string} icon - valid elf icon name
* @param {string} fallback - a fallback in case icon in not defined
* @return {Element}
*/
SortableTitlePlugin.prototype._createIconElem = function (icon, fallback) {
	let elem;
	if (!icon || typeof icon !== "string") {
		elem = document.createElement("span");
		elem.textContent = fallback;
	} else {
		elem = document.createElement("ef-icon");
		elem.setAttribute("icon", icon);
	}
	return elem;
};

/** @private
* @param {string|number|Object} colRef This can be sortState, column id, column field, or column index
* @return {number}
*/
SortableTitlePlugin.prototype._getColumnIndex = function (colRef) {
	if(colRef != null) {
		let host = this._hosts[0];
		if(host) {
			if(typeof colRef === "number") {
				return colRef;
			} else if(typeof colRef === "string") {
				return host.getColumnIndex(colRef);
			} else {
				return host.getColumnIndex(colRef["colId"]);
			}
		}
	}
	return -1;
};

/** @private
* @param {string|number} colRef
* @return {string} column id or empty string for not found
*/
SortableTitlePlugin.prototype._getColumnId = function (colRef) {
	let host = this._hosts[0];
	if(host && colRef != null) {
		return host.getColumnId(colRef);
	}
	return "";
};

/** @private
* @param {number} colIndex
* @return {Object}
*/
SortableTitlePlugin.prototype._getSortOptions = function (colIndex) {
	let host = this._hosts[0];
	if (host) {
		return host.getColumnData(colIndex)["sortableTitle"] || null;
	}
	return null;
};
/** @private
* @param {number} colIndex
* @return {!Object}
*/
SortableTitlePlugin.prototype._newSortOptions = function (colIndex) {
	let host = this._hosts[0];
	let sortOptions = null;
	if (host && colIndex >= 0) {
		let colData = host.getColumnData(colIndex); // colData is guaranteed to exist
		sortOptions = colData["sortableTitle"];
		if (!sortOptions) {
			sortOptions = colData["sortableTitle"] = {};
		}
	}
	return sortOptions || {};
};
/** @private
* @param {ILayoutGrid} section
* @param {number} colIndex
* @param {number} rowIndex
* @return {boolean}
*/
SortableTitlePlugin.prototype._isCellSortable = function (section, colIndex, rowIndex) {
	let options = this._getSortOptions(colIndex);
	if (options) {
		if (options["disabled"]) {
			return false;
		}

		let field = options["field"];
		if (field != null) {
			if (!field) {
				return false;
			}
		}
	}

	let targetCell = section["getCell"](colIndex, rowIndex, true);
	if (targetCell.hasClass("no-sort")) {
		return false; // The cell is not allowed for sorting
	}
	let bottomCell = section["getCell"](colIndex, section["getRowCount"]() - 1, true);
	if (targetCell !== bottomCell) {
		return false;
	}
	let parentCell = targetCell.getParent();
	let classList = parentCell["classList"];
	// "collapsed" class in column element only enable by column stack in spreading mode
	if (classList.contains("collapsed")) { // HACK: unsortable for collapsed stack in spreading mode
		return false;
	}

	return true; // By default all cells are sortable
};

SortableTitlePlugin._proto = SortableTitlePlugin.prototype;

/* harmony default export */ const plugins_SortableTitlePlugin = ((/* unused pure expression or super */ null && (SortableTitlePlugin)));


;// ./src/js/Grid.js
/* eslint-disable */






















/* eslint-enable */

/** @typedef {Object} Grid~SynapseConfig
* @description Configuration object that can be provided directly at the initialization phase
* @property {string} apiKey Synapse api key
* @property {string} contextApp Synapse context app name
* @property {string=} auth Synapse authen token
* @property {boolean=} debug=false If true, Synapse response will be mock
*/

/** @typedef {Object} Grid~ADCOptions
* @description ADC requesting level config from adc team
* @property {string=} productId=001 required parameter, it specifies the product for which you request data. Contact the adc staff to create one.
*/

/** @typedef {Object} Grid~GridOptions
* @description Configuration object that can be provided directly at the initialization phase
* @property {Array.<ColumnDefinition~Options|string>=} columns Collection of the column definitions
* @property {ColumnDefinition~Options=} defaultColumnOptions All Columns will inherit from this default object
* @property {Array.<RowDefinition~Options>=} rows Collection of the row definitions
* @property {Array.<string>=} fields Shorthand for setting fields
* @property {Array.<string>=} rics Shorthand for creating row definition
* @property {Array=} plugins Additional plugins for core grid (from getBlotter()/getCoreGrid())
* @property {Array=} extensions Alias to `plugins`
* @property {boolean=} scrollbar=true Enable both vertical and horizontal scrollbar in grid
* @property {boolean=} linearWheelScrolling=false If enabled, wheel scrolling will move the content linearly regardless of how long the content is (i.e. no skipping).
* @property {boolean=} stepScroll=false If enabled, a full row will be scrolled at a time instead of actual scroll value.
* @property {boolean=} autoHideScrollbar=true Fading in/out Scrollbar on hover.
* @property {string=} pageSize=0 Make grid show specified number of row
* @property {number=} rowHeight=32 Set default row height for body (content) section
* @property {number=} headerRowHeight=32 Set default row height for header section
* @property {boolean=} noColumnDragging=false All columns can be moved by dragging. If this option is true, the feature is disabled
* @property {boolean=} columnReorder=true Alias with noColumnDragging but in opposite value. This property deprecated in favor of noColumnDragging.
* @property {boolean=} rowHighlighting=true Highlight a row when hovering mouse over grid
* @property {(boolean|number)=} autoLayoutUpdate=false Grid will check for layout change periodically.
* @property {number=} columnFreezing Deprecated in favor of column's leftPinned property. Enable horizontal scrollbar after `columnFreezing` index
* @property {number=} freezeColumn Deprecated in favor of column's leftPinned property. An alias to columnFreezing
* @property {number=} pinnedRightColumns=0 Deprecated in favor of column's rightPinned property. Number of columns to be frozen/pinned on the right side.
* @property {Array=} staticDataRows Shorthand for calling {@link Grid#addStaticDataRows}
* @property {Array.<string>=} dataFields Shorthand for calling {@link Grid#addDataFields}
* @property {boolean=} rowVirtualization=true If disabled, all data rows will be rendered. This will greatly impact grid's performance if the data set is huge.
* @property {boolean=} columnVirtualization=false If enabled, columns that are out of grid's view will NOT be rendered. This could greatly improve grid's performance when there are a lot of columns.
* @property {(number|null|boolean)=} topFreezingCount=null If number >= 0 will fix number of frozen title section, If false = disabled scrollbar, if null then title section will freeze auto when new section added, this option will not work with scrollbar option.
* @property {(number|null)=} bottomFreezingCount=null If number >= 0 will fix nuber of frozen footer section, if null then footer section will freeze auto when new section added, this option will not work with scrollbar option.
* @property {boolean=} borders=false Lines around grid element
* @property {boolean=} gridlines Horizontal and Vertical lines for ONLY content sections
* @property {boolean=} verticalLines=false Vertical lines for all sections
* @property {boolean=} contentVerticalLines=false Vertical lines for all content section
* @property {boolean=} horizontalLines=true Horizontal lines for all sections
* @property {*=} RTK=null rtk toolkit instance
* @property {Grid~ADCOptions=} ADC=null ADC requesting level config object from adc team
* @property {Grid~SynapseConfig=} synapse=null synapse config object
* @property {number=} contentRightPadding=0 Padding that is added next to the right most column. The padding is still a part of scrollable content.
* @property {number=} contentBottomPadding=0 Padding that is added below the last section. The padding is still a part of scrollable content.
* @property {number=} dataConflationRate=0 set delay for data to avoid to much sorting operation when data changed
* @property {boolean=} autoDateConversion=false If enabled, date-time field with numeric value will be automatically converted to native date object.
* @property {boolean=} textSelect=false If enabled, user can select content text by using mouse drag.
* @property {string=} lang laguage for config localization date time, if null the default language is "en"
* @property {Element=} scrollbarParent=null Element to be attached by grid's scrollbars, allowing them to move outside and avoid overlapping with the content.
* @property {boolean=} formulaEngine=false If enabled, field with leading equal sign will be treated as a formula and rows will be filled with the calculated values.
* @property {number=} adcPollingInterval=0 Length of polling interval for refreshing ADC data in milliseconds. The default value (0) means no polling.
* @property {boolean=} fieldCaching=false If enabled, field definition will be caching internal mechanism
* @property {boolean=} timeSeriesExpansion=true If disabled, the time series field will be treated as a normal field and not expanded.
* @property {string=} childDataField=CHILD_VALUES The given field will be used to store children's static data, such as row color assignment.
* @property {boolean=} topSection=true If disabled, title section will not be rendered
* @property {SortableTitlePlugin~Options=} sorting Options for sorting
* @property {Function=} dataComposed Handler for dataComposed event
* @property {Function=} beforeContentBinding Handler for beforeContentBinding event
* @property {Function=} firstRendered Handler for firstRendered event
* @property {Function=} afterContentBinding Handler for afterContentBinding event
*/

/** @typedef {number|string|RowDefinition} Grid~RowReference
* @description A row in Grid can be referred by the following values: <br>
* <b>number</b> : Row index of a visible rows, excluding invisible (hidden) rows<br>
* <b>RowDefinition</b> : Row definition object<br>
* <b>string</b> : Unique row Id existing in current data table, including invisible rows<br>
*/

/** @typedef {number|string|ColumnDefinition} Grid~ColumnReference
* @description A column in Grid can be referred by the following values:<br>
* <b>number</b> : Column index including all invisible (hidden) columns<br>
* <b>string</b> : Field name or column id of the column<br>
* <b>ColumnDefinition</b> : Column definition object<br>
*/

/** @event Grid#fieldAdded
* @description Fired after the field has been added to the data table.
* @property {string} type "fieldAdded"
* @property {Array.<string>} addedFields An array of newly added fields
* @property {Array.<string>} fields All fields
* @property {Array.<string>} rics All rics
* @property {Array.<RowDefinition>} rowDefs All row definitions
*/
/** @event Grid#fieldRemoved
* @description Fired after the field has been removed from the data table.
* @property {string} type "fieldRemoved"
* @property {Array.<string>} removedFields An array of recently removed fields
* @property {Array.<string>} fields All fields
* @property {Array.<string>} rics All rics
* @property {Array.<RowDefinition>} rowDefs All row definitions
*/
/** @event Grid#ricAdded
* @description Fired after the ric has been added to the data table.
* @property {string} type "ricAdded"
* @property {Array.<string>} addedRics An array of newly added rics
* @property {Array.<string>} fields All fields
* @property {Array.<string>} rics All rics
* @property {Array.<RowDefinition>} rowDefs All row definitions
*/
/** @event Grid#ricRemoved
* @description Fired after the ric has been removed from the data table.
* @property {string} type "ricRemoved"
* @property {Array.<string>} removedRics An array of recently removed rics
* @property {Array.<string>} fields All fields
* @property {Array.<string>} rics All rics
* @property {Array.<RowDefinition>} rowDefs All row definitions
*/

/** @event Grid#adcDataReceived
* @description Fired after ADC data or response received from the server
* @property {Array.<Array>} data Data part of the server response
*/
/** @event Grid#pageIndexChanged
* @description Fired after page index in the data view is changed during the active pagination mode
*/
/** @event Grid#pageCountChanged
* @description Fired after page count in the data view is changed during the active pagination mode
*/
/** @event Grid#dataComposed
* @description Trigger before dataChanged. Perform any data update during the event will NOT cause more dataChanged events
* @property {Grid} grid
* @property {RowDefinition} rowDef
*/

/** @event Grid#beforeContentBinding
* @property {Object} e Event of beforeContentBinding, we can use e.actualUpdate to check the actual data update, otherwise It will be empty when it is rendered by row virtualization or only the UI changes.
* @description Trigger before content binding.
*/

/** @event Grid#firstRendered
* @property {Object} e Event of firstRendered
* @description This event is triggered when the content is rendered for the first time.
*/

/** @event Grid#afterContentBinding
* @property {Object} e Event of afterContentBinding
* @description Trigger after content binding.
*/

/** @event Grid#beforeRowRemoved
* @description Fired only when a row will be removed through Grid's API and before occurring of the actual removal
*/

/** @event Grid#beforeUnlinked
* @description Trigger before unlinking a chain row.
* @type {Object}
* @property {RowDefinition} chain An row definition object of a chain row.
* @property {boolean} collapsed Collapsing state of a chain row.
* @property {Object} ridMap A map of constituent rics and row ids used to customize row ids.
*/

/** @private
* @param {RowDefinition} rowDef
* @return {Object}
*/
let toRowData = function(rowDef) {
	return rowDef ? rowDef.getRowData() : null;
};

/** @private
* @param {RowDefinition} rowDef
* @return {boolean}
*/
let excludeAutoGenerated = function (rowDef) {
	return !rowDef.isAutoGenerated();
};

/** @private
* @param {Object} colConfig column config
* @return {boolean}
*/
let _byNonAutoGeneratedColumn = function (colConfig) {
	return !colConfig.autoGenerated;
};
/** @private
* @param {string} rowDefA
* @param {string} rowDefB
* @param {string} sortOrder
* @param {string} fieldName
* @return {number} The outcome of the value comparison
*/
let compareNumber = function(rowDefA, rowDefB, sortOrder, fieldName) { // edit name
	return (rowDefA.getData(fieldName) - rowDefB.getData(fieldName)) * sortOrder; // for numeric comparison
};

/**
 * @private
* @param {string} rowDefA
* @param {string} rowDefB
* @param {string} sortOrder
* @return {number} The outcome of the value comparison
*/
let _sortChildrenOfChain = function (rowDefA, rowDefB, sortOrder) {
	let parentA = rowDefA.getParent();

	if (!parentA) {
		return 0;
	}

	let parentB = rowDefB.getParent();

	if (!parentB) {
		return 0;
	}

	if (parentA !== parentB) {
		return 0;
	}

	let a = rowDefA.getData('CHILD_ORDER');

	if (a == null) {
		return 0;
	}

	let b = rowDefB.getData('CHILD_ORDER');

	if (b == null) {
		return 0;
	}

	return ( a - b ) * sortOrder;
};

/** @private
* @param {ColumnDefinition} colDef
* @return {string}
*/
let _getName = function(colDef) {
	if(colDef) {
		return colDef.getName();
	}
	return "";
};
/** @private
* @param {ColumnDefinition} colDef
* @param {string} str
* @return {boolean}
*/
let _hasFieldOrId = function(colDef, str) {
	return (colDef.getField() === str) || (colDef.getId() === str);
};

/** Compare the difference in the 'id' property.
* @private
* @param {Object} obj1
* @param {Object} obj2
* @returns {boolean} If the id property of two objects is equal, the return will be true, otherwise it will be false.
*/
let _hasMatchingId = function(obj1, obj2) {
	if(!obj1 || !obj2 || !obj1.id || !obj2.id) { // Handle nullable, if the object or id have null, it's means difference value
		return false;
	}
	return obj1.id === obj2.id;
};

/** @private
* @param {Object} e
*/
let _preventDefault = function(e) {
	if(e) {
		e.preventDefault();
	}
};
/** @private
* @param {number=} id
* @returns {number} Always return 0
*/
let _clearTimeout = function(id) {
	if(id) {
		clearTimeout(id);
	}
	return 0;
};

/** @constructor
* @extends {EventDispatcher}
* @param {(Element|null)=} placeholder
* @param {Grid~GridOptions=} config
*/
let Grid_Grid = function(placeholder, config) {
	let t = this; // This is to primarily reduce file size

	t._onDataChanged = t._onDataChanged.bind(t);
	t._onQuote2PostUpdate = t._onQuote2PostUpdate.bind(t);
	t._onQ2DataChanged = t._onQ2DataChanged.bind(t);
	t._onDataComposed = t._onDataComposed.bind(t);
	t._onSubSegmentChanged = t._onSubSegmentChanged.bind(t);
	t._recalculateFormulas = t._recalculateFormulas.bind(t);
	t.updateColumnTitle = t.updateColumnTitle.bind(t);
	t._populateTimeSeriesChildren = t._populateTimeSeriesChildren.bind(t);

	t._onPostSectionDataBinding = t._onPostSectionDataBinding.bind(t);
	t._asyncClearDataUpdates = t._asyncClearDataUpdates.bind(t);
	t._clearDataUpdates = t._clearDataUpdates.bind(t);
	t._onPreDataSorting = t._onPreDataSorting.bind(t);
	t._mainSorter = t._mainSorter.bind(t);
	t._updateRowData = t._updateRowData.bind(t);
	t._onFormulaDataChanged = t._onFormulaDataChanged.bind(t);
	t._onFormulaDataRequired = t._onFormulaDataRequired.bind(t);
	t._registerConstituents = t._registerConstituents.bind(t);
	t._onColumnAdded = t._onColumnAdded.bind(t);
	t._onRowExpansionBinding = t._onRowExpansionBinding.bind(t);
	t._onColumnHeaderBinding = t._onColumnHeaderBinding.bind(t);
	t._getRowId = t._getRowId.bind(t);
	t._onFieldLoadedError = t._onFieldLoadedError.bind(t);

	t._snapshotFillerDataChanged = t._snapshotFillerDataChanged.bind(t);
	t._onPollingInterval = t._onPollingInterval.bind(t);

	t._onTabNavigation = t._onTabNavigation.bind(t);
	t._requestScroll = t._requestScroll.bind(t);
	t._onVScroll = t._onVScroll.bind(t);
	t._selfScrollToRow = t._selfScrollToRow.bind(t);

	t._dcConflator = new Conflator(50, t._onDataChanged);
	t._formulaConflator = new Conflator(300, t._onFormulaDataChanged);
	t._chainConflator = new Conflator(100, t._registerConstituents);
	t._columnTitleConflator = new Conflator(0, t.updateColumnTitle);
	t._timeSeriesChildConflator = new Conflator(0, t._populateTimeSeriesChildren);


	t._defaultColumnOptions = {};

	t._topNode = placeholder || Dom.div();
	t._topNode.classList.add("rt-grid");
	t._grid = new Core();
	t._grid._wrapper = this; // For back referencing

	t._sorter = new js_RowDefSorter();
	t._sorter.setContext("grid", this);
	t._sorter.setContext("core", t._grid); // TODO: Handle multiple core grids

	t._dateTimeUtil = DateTime;

	t._grid.listen("columnAdded", t._onColumnAdded); // Columns could be added by some plugins

	if(config) {
		if(config["dataView"]) {
			t._sharedDataSource = true;
			t._dv = config["dataView"];
			t._dt = t._dv.getDataTable();
			t._mainGrid = config["grid"];

			let core = t._mainGrid.getCoreGrid();
			t._stp = core.getPlugin("SortableTitlePlugin");
		}
		if(config["SortableTitle"]) { // Exception for built-in plugin
			t._stp = config["SortableTitle"];
		}
		if(config["topSection"] === false) {
			t._topSection = false;
		}
	}
	if(!t._sharedDataSource) {
		t._dc = new DataCache();
		t._dc.listen("dataChanged", t._onDataChanged);
		t._dc.listen("dataComposed", t._onDataComposed);

		t._dt = new DataTable();
		t._dt.setSortingLogic(/** @type{Function} */(t._mainSorter));
		t._dt.setClassificationSource(t._dc);
		t._dt.listen("subSegmentChanged", t._onSubSegmentChanged);
		t._dv = new DataView(t._dt);
		t._dv.listen("pageIndexChanged", t._dispatch.bind(t, "pageIndexChanged"));
		t._dv.listen("pageCountChanged", t._dispatch.bind(t, "pageCountChanged")); // TODO: When implementing filtered row, it may need to implement the conflator
	}
	t._grid.setDataSource(t._dv); // Avoid triggering sorting

	if(t._stp) {
		t._sharedSorter = true;
		t._stp.listen("preDataSorting", t._onPreDataSorting);
		t._grid.loadPlugin(t._stp); // The plugin instance is shared and don't need to be reconfigured.
	} else { // Create built-in plugin
		t._stp = new SortableTitlePlugin({
			"userManagedLogic": true,
			"sortableColumns": true,
			"disableDoubleClickToSort": true,
			"ROW_DEF": true // Enable Row Definition Mode
		});
		t._stp.listen("preDataSorting", t._onPreDataSorting);

		// TODO: Built-in plugins don't have normal workflow unlike other extensions. Make them have the same workflow like the others
		// defaultColumnOptions property does not work with them.
		// beforeInit is not called.
		t._grid.loadPlugin(t._stp, config);
	}

	t._grid.listen("beforeContentBinding", t._dispatch.bind(t, "beforeContentBinding"));
	t._grid.listen("preSectionRender", t._onColumnHeaderBinding);
	t._grid.listen("postSectionDataBinding", t._onPostSectionDataBinding);
	t._grid.listen("firstRendered", t._dispatch.bind(t, "firstRendered"));
	t._grid.listen("afterContentBinding", t._dispatch.bind(t, "afterContentBinding"));
	t._grid.listen("tabNavigation", t._onTabNavigation);

	t._grid.getVScrollbar().listen("scroll", t._onVScroll);

	t._grid.enableRowHighlighting(true);

	t._addGridSections();
	t._grid.setParent(t._topNode);

	t._snapshot = new js_SnapshotFiller();
	t._snapshot.addEventListener("adcDataReceived", t._dispatch.bind(t, "adcDataReceived"));
	t._snapshot.addEventListener("dataChanged", this._snapshotFillerDataChanged);

	t._connector = new js_DataConnector();
	t._connector.addEventListener("ricAdded", t._onRicAdded.bind(t));
	t._connector.addEventListener("ricRemoved", t._onRicRemoved.bind(t));
	t._connector.addEventListener("fieldAdded", t._onFieldAdded.bind(t));
	t._connector.addEventListener("fieldRemoved", t._onFieldRemoved.bind(t));

	t.initialize(config);

	js_StyleLoader.applyStyle(t._grid);
};
Ext.inherits(Grid_Grid, EventDispatcher);

/** @private
 * @type {Element}
 */
Grid_Grid.prototype._topNode = null;
/** @private
 * @type {Grid}
 */
Grid_Grid.prototype._mainGrid = null;
/** @private
 * @type {Core}
 */
Grid_Grid.prototype._grid = null;
/** Store all actual data including streaming data, ADC data, snapshot data, user input data.
 * @type {DataCache}
 * @private
 */
Grid_Grid.prototype._dc = null;
/** Used for hiding and filtering out rows. Always have greater or equal number of rows than that in grid
 * @type {DataTable}
 * @private
 */
Grid_Grid.prototype._dt = null;
/** Used for sorting. Number of rows is always the same as that in grid
 * @type {DataView}
 * @private
 */
Grid_Grid.prototype._dv = null;
/** @type {boolean}
 * @private
 */
Grid_Grid.prototype._sharedDataSource = false;
/** @type {boolean}
 * @private
 */
Grid_Grid.prototype._sharedSorter = false;

/** JET/RTK's Quotes2 subscription object
 * @type {Object}
 * @private
 */
Grid_Grid.prototype._subs = null;

/** @type {SortableTitlePlugin}
 * @private
 */
Grid_Grid.prototype._stp = null;
/** @type {Function}
 * @private
 */
Grid_Grid.prototype._columnSorter = null;

/** @private
* @type {Conflator}
*/
Grid_Grid.prototype._dcConflator = null;
/** @private
* @type {SnapshotFiller}
*/
Grid_Grid.prototype._snapshot = null;
/** @private
* @type {DataConnector}
*/
Grid_Grid.prototype._connector = null;
/** @private
* @type {RowDefSorter}
*/
Grid_Grid.prototype._sorter = null;

/** @private
* @type {Engine}
*/
Grid_Grid.prototype._fnEngine = null;
/** @private
* @type {Conflator}
*/
Grid_Grid.prototype._formulaConflator = null;

/** @private
* @type {Conflator}
*/
Grid_Grid.prototype._chainConflator = null;
/** @private
* @type {Object}
*/
Grid_Grid.prototype._constituentMap = null;
/** @private
* @type {number}
*/
Grid_Grid.prototype._clientWidth = NaN;
/** @private
* @type {number}
*/
Grid_Grid.prototype._autoLayoutTimer = 0;
/** @private
 * @type {ColumnDefinition~Options}
 */
Grid_Grid.prototype._defaultColumnOptions = null;
/** @private
* @type {*}
*/
Grid_Grid.prototype._RTK = null;
/** @private
* @type {Grid~ADCOptions}
*/
Grid_Grid.prototype._ADCOptions = null;
/** use for synapse service
* @private
* @type {string}
*/
Grid_Grid.prototype._synapse = null;
/** @private
* @type {boolean}
*/
Grid_Grid.prototype._autoDateConversion = false;
/** @private
* @type {boolean}
*/
Grid_Grid.prototype._textSelect = false;
/** @type {string}
* @private
*/
Grid_Grid.prototype._lang = null;
/** @type {Object}
* @private
*/
Grid_Grid.prototype._dateTimeUtil = null;
/** @type {boolean}
* @private
*/
Grid_Grid.prototype._initializing = false;
/** @type {number}
* @private
*/
Grid_Grid.prototype._pollingInterval = 0;
/** @type {number}
* @private
*/
Grid_Grid.prototype._pollingTimerId = 0;
/** @type {number}
* @private
*/
Grid_Grid.prototype._lastPollingRequest = 0;
/** @type {boolean}
* @private
*/
Grid_Grid.prototype._pollingEnabled = true;
/** @type {boolean}
* @private
*/
Grid_Grid.prototype._fieldCaching = false;
/** @type {boolean}
* @private
*/
Grid_Grid.prototype._timeSeriesExpansion = true;

/** @type {string}
* @private
*/
Grid_Grid.prototype._childDataField = "";
/** @type {boolean}
* @private
*/
Grid_Grid.prototype._topSection = true;
/** @type {Object}
* @private
*/
Grid_Grid.prototype._focusingArgs = null;
/** @type {boolean}
* @private
*/
Grid_Grid.prototype._hasNewUpdates = false;

/** @public
*/
Grid_Grid.prototype.dispose = function() {
	this.removeAllEventListeners();
	if(this._autoLayoutTimer) {
		clearInterval(this._autoLayoutTimer);
		this._autoLayoutTimer = 0;
	}
	this._pollingTimerId = _clearTimeout(this._pollingTimerId);

	this.removeAllColumns(); // Some conflators are reset
	this.removeAllRows(); // Some conflators are reset
	this._sorter.dispose();
	this._grid.dispose();
	this._snapshot.dispose();
	this._connector.dispose();
	this._connector = null;

	if(!this._sharedDataSource) { // Make sure that this is the final grid, and its data is disposed
		if(this._subs) {
			this._subs.removeEventListener("postUpdate", this._onQuote2PostUpdate);
			this._subs.removeEventListener("dataChanged", this._onQ2DataChanged);
			this._subs["dispose"]();
		}
		if(this._dc) {
			this._dc.dispose();
		}
		if(this._dt) {
			this._dt.dispose();
		}
		if(this._dv) {
			this._dv.dispose();
		}
	}
	this._mainGrid = this._dc = this._dt = this._dv = this._subs = null;

	if(this._focusingArgs) {
		_clearTimeout(this._focusingArgs.id);
		_clearTimeout(this._focusingArgs.timeoutId);
		this._focusingArgs = null;
	}
};
/** @public
* @return {Element}
*/
Grid_Grid.prototype.getElement = function() {
	return this._topNode;
};
/** Alias to {@link EventDispatcher#removeAllEventListeners}
* @public
* @function
*/
Grid_Grid.prototype.removeEventListeners = Grid_Grid.prototype.removeAllEventListeners;
/** Alias to {@link EventDispatcher#addEventListener}
* @public
* @function
* @param {string} type Event name
* @param {Function} handler Event handler
*/
Grid_Grid.prototype.listen = Grid_Grid.prototype.addEventListener;
/** @public
*/
Grid_Grid.prototype.initSubscription = function() {
	if(this._subs) { // Subscription is already initialized
		return;
	}
	if(this._sharedDataSource) {
		this._subs = this._mainGrid._subs;
		return;
	}

	let jet, q, s;
	jet = window["JET"]; // Assume that JET is already exist.
	if (this._RTK) {
		q = this._RTK.Quotes;
	} else if(jet) {
		q = jet["Quotes2"];
	}
	if(q) {
		s = q["create"]();
		if(this._RTK && this._RTK.prefetchWorkaround) { // The workaround for RTK Quotes has a bug concerning the removal of the last row; it isn't removed.
			s.addRic("DUMMY.WORKAROUND", "DUMMY.WORKAROUND");
		}
	}
	if(!s) {
		return;
	}
	if(s["filter"]) {
		s["filter"](-2, -1); // Enable skip header for chains
	}

	this._subs = s;
	this._subs["start"]();
	this._subs.addEventListener("postUpdate", this._onQuote2PostUpdate);
	this._subs.addEventListener("dataChanged", this._onQ2DataChanged);

	// TODO: Subscriptions should be registered per row.
	// However, chain subscription cannot be integrated with DataConnector in this current implementation.
	let rowDefs = this._getAllRowDefinitions();
	let len = rowDefs.length;
	for(let i = 0; i < len; ++i) {
		let rowDef = rowDefs[i];
		if(rowDef) {
			rowDef.subscribeForUpdates(s);
		}
	}
};

/** @public
*/
Grid_Grid.prototype.updateLayout = function() {
	this._grid.updateLayout();
	this._stp.updateSortSymbols(); // HACK: There is a chance that grid may not yet be in the document
};
/** Force rerender of grid rows. TODO: Check this could be deprecated in favor of Core's requestRowRefresh method
* @private
* @param {number=} rowIndex
*/
Grid_Grid.prototype._updateRowData = function(rowIndex) {
	if(rowIndex >= 0) {
		let rowId = this._dv.getRowId(rowIndex);
		if(rowId) {
			this._dt.setRowData(rowId, {});
		}
	} else {
		this._dt.dispatchGlobalChange(); // Trigger sorting too
	}
};
/** @public
* @function
* @param {number=} opt_rowIndex
*/
Grid_Grid.prototype.updateRowData = Grid_Grid.prototype._updateRowData;

/** @private
 */
Grid_Grid.prototype._addGridSections = function () {
	let title = this._grid.addSection("title");
	if(this._topSection == false){
		title.setRowCount(0);
	} else {
		title.setRowCount(1);
	}
	this._grid.addSection("content");

	let titleSettings = this._grid.getSectionSettings("title");
	titleSettings.disableDataBinding();

	let contentSettings = this._grid.getSectionSettings("content");
	contentSettings.setAutoSyncRowCount(true);
	contentSettings.setParent(titleSettings);
};
/** @public
* @param {string=} sectionName
* @return {*} Return Section instance
 */
Grid_Grid.prototype.addHeaderSection = function (sectionName) {
	let titleSects = this._grid.getAllSections("title");

	let headerSect = this._grid.addSectionAt(titleSects.length, "title", sectionName);
	if(headerSect) {
		headerSect.setRowCount(1);

		let headerSettings = this._grid.getSectionSettings(headerSect);
		headerSettings.disableDataBinding();
	}

	return headerSect;
};
/** @public
* @param {string=} sectionName
* @return {*} Return Section instance
*/
Grid_Grid.prototype.addFooterSection = function (sectionName) {
	let footerSect = this._grid.addSection("footer", sectionName);
	if(footerSect) {
		footerSect.setRowCount(1);

		let footerSettings = this._grid.getSectionSettings(footerSect);
		footerSettings.disableDataBinding();
	}

	return footerSect;
};

/** @public
* @param {Element} elem Parent element
*/
Grid_Grid.prototype.setParent = function (elem) {
	Dom.appendChild(elem, this._topNode);
	this.updateLayout();
};
/** @public
* @param {Element} elem Sibling element
*/
Grid_Grid.prototype.insertBefore = function (elem) {
	if(elem) {
		let pn = elem.parentNode;
		if(pn) {
			pn.insertBefore(this._topNode, elem);
			this.updateLayout();
		}
	}
};
/** The returned object contains <br>
* `boolean` hit : Indicates whether the given position is on the grid <br>
* `number` x : Horizontal coordinate that is relative to the top-left of the grid (Zero means left-most) <br>
* `number` y : Vertical coordinate that is relative to the top-left of the grid (Zero means top-most) <br>
* `number` colIndex <br>
* `number` sectionIndex <br>
* `number` rowIndex <br>
* and other related property to the given position.
* @public
* @param {!ElementWrapper|Element|Event|MouseEvent} obj Element, Browser's Event object, Browser's Mouse Event object are all valid
* @return {!Object}
*/
Grid_Grid.prototype.getRelativePosition = function (obj) {
	return this._grid.getRelativePosition(obj);
};
/** The returned core grid
* @public
* @return {Core}
*/
Grid_Grid.prototype.getCoreGrid = function () {
	return this._grid;
};

/** @public
* @param {Grid~GridOptions=} gridOption
*/
Grid_Grid.prototype.initialize = function(gridOption) {
	if (!gridOption) {
		this.initSubscription();
		return;
	}
	// TODO: clear all data before re-initialization
	let t = this; // For minimizing file size
	t._initializing = true;
	let grid = t._grid; // core grid
	grid.resetInternalState();
	gridOption = grid.normalizeConfig(gridOption);

	let exts = gridOption["plugins"] || gridOption["extensions"];

	let cols = /** @type{Array} */(gridOption["fields"] || gridOption["columns"]);
	if (gridOption["defaultColumnOptions"]) {
		t._defaultColumnOptions = gridOption["defaultColumnOptions"];
		t._defaultColumnSetup(t._defaultColumnOptions, cols);
	}

	// Tell all plugins that the initialization phase has started
	t._processExtension(exts, "beforeInit", [grid.getElement(), gridOption]);

	// option flag text-select
	// allow user to select text for copy or so something
	if (gridOption['textSelect']) {
		t._textSelect = true;
		grid.addClass('tr-text-select');
	}

	if (gridOption["RTK"]) {
		t._RTK = gridOption["RTK"];
		t._snapshot.setRTK(t._RTK);
	}

	if (gridOption["ADC"]) {
		t._ADCOptions = gridOption["ADC"];
		t._snapshot.setADCOptions(t._ADCOptions);
	}

	if (gridOption["synapse"]) {
		t._synapse = gridOption["synapse"];
		js_FieldDefinition.setSynapseConfig(t._synapse);
	}

	if (gridOption["fieldCaching"]) {
		t._fieldCaching = gridOption["fieldCaching"];
	}

	if(gridOption["timeSeriesExpansion"] != null) {
		t._timeSeriesExpansion = gridOption["timeSeriesExpansion"];
		js_FieldDefinition.disableTimeSeriesExpansion(!t._timeSeriesExpansion);
	}

	if (gridOption["icons"] && gridOption["icons"]["rowGrouping"] && gridOption["icons"]["rowGrouping"]["expander"]) {
		js_StyleLoader.chainIcon = gridOption["icons"]["rowGrouping"]["expander"];
	}

	if(gridOption["formulaEngine"]) {
		t._fnEngine = new Engine();
		t._fnEngine.addEventListener("dataChanged", t._onFormulaDataChanged);
		t._fnEngine.addEventListener("dataRequired", t._onFormulaDataRequired);
	}

	let borders = gridOption["borders"];
	if (borders != null) {
		grid.toggleBorders(borders);
	}
	let gridlines = gridOption["gridlines"];
	if (gridlines != null) {
		grid.toggleGridlines(gridlines);
	}
	let vLines = gridOption["verticalLines"];
	if (vLines != null) {
		grid.toggleVerticalLines(vLines);
	}
	let cvLines = gridOption["contentVerticalLines"];
	if (cvLines != null) {
		grid.toggleContentVerticalLines(cvLines);
	}
	let hLines = gridOption["horizontalLines"];
	if (hLines != null) {
		grid.toggleHorizontalLines(hLines);
	}

	if(gridOption["pageSize"]) {
		t._dv.setPageSize(+gridOption["pageSize"]);
	}

	let rowHeight = +gridOption["rowHeight"];
	if(rowHeight) { // Non empty or zero
		grid.setDefaultRowHeight(rowHeight);
	}
	let hRowHeight = +gridOption["headerRowHeight"];
	if(hRowHeight) { // Non empty or zero
		grid.getSection("title").setDefaultRowHeight(hRowHeight);
	}

	let val = gridOption["autoLayoutUpdate"];
	if(!t._autoLayoutTimer && val) {
		if(val !== "0" && val !== "false") { // HACK: Temporarily support incorrect data type
			t._autoLayoutTimer = setInterval(t._onAutoLayoutUpdate.bind(t), 2000);
		}
	}

	let bool = gridOption["rowHighlighting"];
	if(bool != null) {
		grid.enableRowHighlighting(!!bool);
	}

	let rowExpansionBinding = gridOption["rowExpansionBinding"];
	if(typeof rowExpansionBinding === "function") {
		grid.listen("rowExpansionBinding", t._onRowExpansionBinding); // TODO: Support multi-table feature
		t.listen("rowExpansionBinding", rowExpansionBinding);
	}

	this.addListener(gridOption, "dataComposed");
	this.addListener(gridOption, "beforeContentBinding");
	this.addListener(gridOption, "firstRendered");
	this.addListener(gridOption, "afterContentBinding");
	this.addListener(gridOption, "tabNavigation");

	if(gridOption["autoDateConversion"]) {
		t._autoDateConversion = true;
	}

	if(gridOption["stepScroll"]) {
		grid.setRowScrollingStep(1);
	}
	if(gridOption["linearWheelScrolling"]) {
		grid.getVScrollbar().setMouseWheelSpeed("linear");
	}
	if(gridOption["autoHideScrollbar"] != null) {
		grid.autoHideScrollbars(gridOption["autoHideScrollbar"] ? true : false);
	}
	if(gridOption["scrollbarParent"] != null) {
		t._setScrollbarParent(gridOption["scrollbarParent"]);
	}
	let pollingInterval = gridOption["adcPollingInterval"];
	if(pollingInterval != null) {
		t._pollingInterval = pollingInterval ? +pollingInterval : 0;
	}

	// Column operations
	t.setColumns(cols);

	let rowVir = true;
	if (gridOption["rowVirtualRendering"] != null) {
		rowVir = gridOption["rowVirtualRendering"];
	}
	if (gridOption["rowVirtualization"] != null) {
		rowVir = gridOption["rowVirtualization"];
	}
	grid.enableRowVirtualization(!!rowVir);

	let colVir = false;
	if (gridOption["columnVirtualRendering"] != null) {
		colVir = gridOption["columnVirtualRendering"];
	}
	if (gridOption["columnVirtualization"] != null) {
		colVir = gridOption["columnVirtualization"];
	}
	grid.enableColumnVirtualization(!!colVir);

	let scrollbar = gridOption["scrollbar"] != null ? gridOption["scrollbar"] : true;
	if (scrollbar) {
		t._topNode.style.overflow = "hidden"; // TODO: Move this to CSS Class
	} else {
		grid.freezeSection(null); // Disable vertical scrollbar
		t._topNode.style.overflow = "";
	}

	let topFreezingCount = gridOption["topFreezingCount"];
	if (typeof topFreezingCount === "number") {
		topFreezingCount = (topFreezingCount > 0) ? topFreezingCount : 0;
		let sectionIndex = topFreezingCount - 1;
		grid.freezeSection(sectionIndex);
	} else if (topFreezingCount === false) {
		grid.freezeSection(null);
	}

	let bottomFreezingCount = gridOption["bottomFreezingCount"];
	if (typeof bottomFreezingCount === "number") {
		bottomFreezingCount = (bottomFreezingCount > 0) ? bottomFreezingCount : 0;
		grid.freezeFooter(bottomFreezingCount);
	}

	t.addDataFields(gridOption["dataFields"]);

	// Plugins
	gridOption[ROW_DEF] = true; // Enable ROW_DEF mode
	if (Array.isArray(exts)) {
		for (let i = 0; i < exts.length; ++i) {
			let ext = exts[i];
			let extInstance = null;
			if (ext) {
				let extName = ext["name"];
				if(extName) {
					extInstance = grid.loadPlugin(extName, ext); // pass plugin object as an argument for initialization
				} else {
					extInstance = grid.loadPlugin(ext, gridOption);
				}
			}
			if (extInstance && extInstance["setGridWrapper"]) {
				let hasApi = (typeof extInstance["getGridApi"] === "function") ? extInstance["getGridApi"]() : false;
				if(!hasApi) {
					extInstance["setGridWrapper"]("realTimeGrid", t);
				}
			}
		}
		t._initializing = false;
	}

	let lang = gridOption["lang"];
	if(lang != null) {
		t._lang = lang;
		t._dateTimeUtil.setLocale(lang);
	}

	// Row operations
	if(gridOption["childDataField"] != null) {
		this._childDataField = RowDefinition._childDataField = gridOption["childDataField"];
	}
	let rows = gridOption["rows"];
	if(!rows) {
		rows = gridOption["rics"] || null; // Make "rics" an alias to "rows"
	}
	let bodies = /** @type{Array} */(gridOption["bodies"]);
	if(!bodies) {
		bodies = [];
		if(gridOption["body"]) {
			bodies.push(gridOption["body"]);
		}
		if(rows) {
			for(let m = 0; m < rows.length; m++) {
				if(typeof rows[m] === "string") {
					rows[m] = { "ric": rows[m] };
				}
			}
			bodies.push({ "rows": rows });
		}
	}

	// Merge rows with static data rows
	let staticRows = gridOption["staticDataRows"] || gridOption["dataModel"];
	if(rows && staticRows) {
		let ary = staticRows["data"] ? staticRows["data"] : staticRows;
		let fields = staticRows["fields"] || t.getColumnFields();
		let values, data;
		let len = rows.length;
		for(let n = 0; n < len; n++) {
			if(rows[n]["values"]) { // merge row's values, values can be object or array
				values = arrayToObject(rows[n]["values"], fields);
				data = arrayToObject(ary[n], fields);
				rows[n]["values"] = extendObject(values, data);
			} else {
				rows[n]["values"] = ary[n];
			}
		}
		ary = ary.slice(len, ary.length);
		if(staticRows["data"]) {
			staticRows["data"] = ary;
		} else {
			staticRows = ary;
		}
	}

	for(let j = 0; j < bodies.length; ++j) {
		t.insertRows(bodies[j]["rows"]);
		// break; // TODO: The first version supports only 1 grid
	}

	t.addStaticDataRows(staticRows);

	if(gridOption.contentBottomPadding) {
		grid.setBottomPaddingSection(gridOption.contentBottomPadding);
	}

	let contentRightPadding = gridOption.contentRightPadding;
	if(contentRightPadding) {
		if (typeof contentRightPadding !== "number") {
			contentRightPadding = 6;
		}

		grid.reserveRightSpace(contentRightPadding);
	}

	// Let everything init first
	// then set conflationRate to grid for better perfomance on rapidly updating
	let dataConflationRate = gridOption['dataConflationRate'];
	if (dataConflationRate != null) {
		grid.setDataConflationRate(dataConflationRate);
	}

	// Tell all plugins that the initialization phase has ended
	let extObjects = grid.getPluginList().map(grid.getPlugin.bind(grid));
	t._processExtension(extObjects, "afterInit");
	t.initSubscription();
};

/** @public
* @param {Object=} gridOptions
* @return {!Object}
*/
Grid_Grid.prototype.getConfigObject = function (gridOptions) {
	let obj = gridOptions || {};

	let columns = obj.columns;
	if(!columns) {
		columns = obj.columns = [];
	}

	let grid = this._grid;
	if(grid.getConfigObject) {
		grid.getConfigObject(obj);
	}

	// Difference from composite grid in default config
	if(obj["columnVirtualization"] === false) {
		delete obj["columnVirtualization"];
	}

	if(obj["rowVirtualization"] === true) {
		delete obj["rowVirtualization"];
	}

	if(obj["rowHighlighting"] === true) {
		delete obj["rowHighlighting"];
	}

	if(obj["contentVerticalLines"] === false) { // the default value change in wrapper (atlas-blotter)
		delete obj["contentVerticalLines"];
	}

	if(obj["borders"] === false) { // the default value change in wrapper (atlas-blotter)
		delete obj["borders"];
	}


	let i, len, prevTimeSeriesField;
	len = this.getColumnCount();
	for (i = 0; i < len; ++i) {
		let column = columns[i];
		if(!column) {
			column = columns[i] = {};
		}
		let colDef = this.getColumnDefinition(i);
		// TODO: Update width and scalability changed by the extensions
		colDef.getConfigObject(column);
		let parentField = js_FieldDefinition.getFieldProperty(colDef.getField(), "timeSeriesParent");
		if(parentField && !prevTimeSeriesField) { // Assume first column of time series child to convert to time series parent
			prevTimeSeriesField = parentField;
			column.field = parentField;
			delete column.name; // WARNING: Name of child should be retained
			delete column.autoGenerated; // Convert to parent time series field
		}

		if(!parentField && prevTimeSeriesField) { // Handled duplicate parent time series field in difference position of column
			prevTimeSeriesField = null;
		}
	}
	obj.columns = columns = columns.filter(_byNonAutoGeneratedColumn); // Time series child will be filtered out

	if(this._topNode.style.overflow === "") {
		obj["scrollbar"] = false;
	}

	let val = this._dv.getPageSize();
	if(val > 0) {
		obj["pageSize"] = val;
	}

	val = grid.getDefaultRowHeight();
	if(val !== 28) { // the default value change in wrapper (atlas-blotter)
		obj["rowHeight"] = val;
	}

	val = grid.getSection("title").getDefaultRowHeight();
	if(val !== 28) { // the default value change in wrapper (atlas-blotter)
		obj["headerRowHeight"] = val;
	}

	if(this._autoLayoutTimer) {
		obj["autoLayoutUpdate"] = true;
	}

	val = isEmptyObject(this._defaultColumnOptions);
	if (!val) { // Currently, this variable is the object, we need to check empty array for default value
		obj["defaultColumnOptions"] = this._defaultColumnOptions;
	}

	if(this._autoDateConversion) {
		obj["autoDateConversion"] = true;
	}

	if(this._textSelect) {
		obj["textSelect"] = true;
	}

	if(this._lang) {
		obj["lang"] = this._lang;
	}
	if(this._fnEngine) {
		obj["formulaEngine"] = true;
	}
	if(this._pollingInterval) {
		obj["adcPollingInterval"] = this._pollingInterval;
	}
	if(this._fieldCaching) {
		obj["fieldCaching"] = this._fieldCaching;
	}
	if(this._timeSeriesExpansion != true) {
		obj["timeSeriesExpansion"] = this._timeSeriesExpansion;
	}

	if(this._childDataField) {
		obj["childDataField"] = this._childDataField;
	}

	// get all rows config
	let rowDefs = this.getAllRowDefinitions();
	let rows = obj["rows"] = [];
	let rowDef, rowConfig;
	len = rowDefs.length;
	for (i = 0; i < len; i++) {
		rowDef = rowDefs[i];
		rowConfig = rowDef.getConfigObject();
		rows.push(rowConfig);
	}

	// get row config from extensions
	let extensions = grid.getPlugins();
	let pluginInstance, row;
	for(let key in extensions) {
		pluginInstance = extensions[key];
		if(typeof pluginInstance["getRowConfigObject"] === "function") {
			len = rows.length;
			for (i = 0; i < len; i++) {
				rowDef = rowDefs[i];
				row = rows[i];
				pluginInstance["getRowConfigObject"](row, rowDef.getRowId());
			}
		}
	}

	// TODO: The following states need to be retrieved
	// topFreezingCount, bottomFreezingCount
	// scrollbarParent
	// linearWheelScrolling
	// stepScroll
	// noColumnDragging
	// columnReorder
	// topSection

	// NOTE: no need to export synapseApiKey, ADC and RTK

	return obj;
};

/** @public
* @ignore
* @param {Object} dateTimeUtil
*/
Grid_Grid.prototype.setDateTimeUtil = function(dateTimeUtil) {
	if(this._dateTimeUtil !== dateTimeUtil) {
		this._dateTimeUtil = dateTimeUtil;
		this._dateTimeUtil.setLocale(this._lang);
	}
};
/** @public
* @param {string} lang
*/
Grid_Grid.prototype.setLocale = function(lang) {
	if(lang && this._lang !== lang) {
		this._lang = lang;
		this._dateTimeUtil.setLocale(lang);
		this._grid.requestRowRefresh();
	}
};

/** @private
* @param {Object} defaultCol
* @param {Array.<Object>} userColumns
*/
Grid_Grid.prototype._defaultColumnSetup = function (defaultCol, userColumns) {
	let c = userColumns.length;
	for (let i = 0; i < c; i++) {
		let column = userColumns[i];
		if (typeof column === 'object') {
			for (let key in defaultCol) {
				if (column[key] == null) {
					column[key] = defaultCol[key];
				}
			}
		}
		// TODO: Handle columnOption type string
		// else if (typeof column === 'string') { }
	}
};

/** @private
* @param {Array.<Object>} exts List of plugin instance
* @param {string} field A function name of plugin instance
* @param {Array.<*>} params Inputs that will pass to a function call
*/
Grid_Grid.prototype._processExtension = function (exts, field, params) {
	if(Array.isArray(exts)) {
		for (let i = 0; i < exts.length; i++) {
			if (exts[i][field]) {
				exts[i][field].apply(exts[i], params);
			}
		}
	}
};

/** @private
* @param {Object} e
*/
Grid_Grid.prototype._onRicAdded = function(e) {
	let addedRics = e.addedRics;
	this._snapshot.addFields(e.fields);
	let len = addedRics.length;
	for (let i = 0; i < len; ++i) {
		let ric = addedRics[i];
		let rowDefs = this._connector.getRowDefByRic(ric);
		let jLen = rowDefs ? rowDefs.length : 0;
		for (let j = 0; j < jLen; ++j) {
			let rowDef = rowDefs[j];
			if (rowDef.getSymbol() === ric) {
				this._snapshot.addRic(ric); // ADC
			}
		}
	}

	this._dispatch(e.type, e);
};

/** @private
* @param {Object} e
*/
Grid_Grid.prototype._onRicRemoved = function(e) {
	this._dispatch(e.type, e);
};

/** @private
* @param {Object} e
*/
Grid_Grid.prototype._onFieldAdded = function(e) {
	let rowDefs = e.rowDefs;
	let addedFields = e.addedFields;

	// ADC
	for (let i in rowDefs) {
		this._snapshot.addRic(rowDefs[i].getSymbol());
	}
	this._snapshot.addFields(addedFields);

	// JET
	if (this._subs) {
		let realtimeFields = addedFields.filter(js_FieldDefinition.isRealTimeField);
		if(realtimeFields.length > 0) {
			this._subs["addFields"](realtimeFields);
		}
	}

	this._dispatch(e.type, e);
};
/** @private
* @param {Object} e
*/
Grid_Grid.prototype._onFieldRemoved = function(e) {
	let removedFields = e.removedFields;

	// TODO: ADC fields have an interval load. Currently, we only keep the field but do not delete it.
	// JET
	if(this._subs) {
		this._subs["removeFields"](removedFields);
	}

	this._dispatch(e.type, e);
};

/** @public
* @param {ColumnDefinition~Options|string} columnOption String will be treated as field, while object is treated as the column options
* @param {number=} idx
*/
Grid_Grid.prototype.insertColumn = function (columnOption, idx) {
	let colCount = this.getColumnCount();
	if(idx == null || idx > colCount) {
		idx = colCount;
	}

	// no need to merge defaultColumn during initialize process
	// because columnOption already mutate by initialize function
	let configObj = (this._initializing) ? {} : cloneObject(this._defaultColumnOptions);

	if (typeof columnOption === "string") {
		configObj["field"] = columnOption;
	} else { // columnOption is assumed to be an object
		for (let key in columnOption) {
			configObj[key] = columnOption[key];
		}
	}

	if(this._fnEngine) {
		configObj["formulaEngine"] = this._fnEngine; // Adding extra option
	}

	let colDef = new ColumnDefinition(configObj, this);
	configObj[COL_DEF] = colDef;

	// WARNING: width is processed twice by tr-grid and rt-grid
	// Inserting column also initializes prefined formatter
	this._grid.insertColumn(idx, configObj); // columnAdded is fired
};

/**
* @private
* @param {Object} e snapshort change event object
*/
Grid_Grid.prototype._updateTimeSeriesFields = function (e) {
	let childField;
	let snapShotData = e.data;
	let childFields = {};
	for (let ric in snapShotData) {
		// childFields = snapShotData[ric];
		for (childField in snapShotData[ric]) {
			childFields[childField] = true; // Merge child fields to parent (Don't truth each ric it will be have some empty data)
		}
		this.setRicData(ric, snapShotData[ric]); // update cell data if exist
	}
	let parentField, field, colIndex, parentColDef, childColIndex;
	let parentMap = {}; // map parent time series field and their first child
	for (field in childFields) { // Warning: data change can be differ time series child field
		parentField = e.childrenFieldToParent[field];
		if(!parentMap[parentField]) {
			parentMap[parentField] = [field];
		} else {
			parentMap[parentField].push(field);
		}
	}

	for (parentField in parentMap) { // WARNING: one dataChanged event can be multiple fields
		let childrenField = parentMap[parentField];
		colIndex = this.getColumnIndex(parentField);
		parentColDef = this._getColumnDefinition(colIndex);
		let i, len;
		len = childrenField.length;
		for (i = 0; i < len; i++) {
			childField = childrenField[i];
			childColIndex = this.getColumnIndex(childField);
			if(childColIndex < 0) { // not found column index in view tried to clone from parent
				colIndex = this.getColumnIndex(parentField);
				parentColDef = this._getColumnDefinition(colIndex);
				this._cloneTimeSeriesColumn(parentColDef, childField, colIndex);
				js_FieldDefinition.addTimeSeriesChild(parentField, childField); // TODO: time series child can be difference of response even if the input is the same, It should be dynamic add/remove fields
			}
		}
		this.removeColumn(parentField);
	}

};


/** @private
* @param {ColumnDefinition} colDef
*/
Grid_Grid.prototype._populateTimeSeriesChildren = function (colDef) {
	if(this._timeSeriesChildConflator.conflate(colDef) ) {
		return;
	}

	let colDefs = this._timeSeriesChildConflator.popAllData();
	let i, j, len, childField, idx;

	for (i = 0; i < colDefs.length; i++) {
		colDef = colDefs[i];
		idx = this.getColumnIndex(colDef);

		if(!colDef) {
			continue;
		}
		let parentField = colDef.getField();
		let childFields = js_FieldDefinition.getTimeSeriesChildren(parentField);
		if(!childFields) { // not found time series field
			return;
		}

		len = childFields.length;
		if(len < 1) {
			return; // normal field
		}

		for (j = 0; j < len; j++) {
			idx = this.getColumnIndex(colDef);
			childField = childFields[j];
			this._cloneTimeSeriesColumn(colDef, childField, idx);
		}
	}

};

/** @private
* @param {ColumnDefinition} parentColDef Parent definition
* @param {string} childField field
* @param {number} idx index of insertion column
*/
Grid_Grid.prototype._cloneTimeSeriesColumn = function (parentColDef, childField, idx) {
	let parentConfig, columnOption;
	parentConfig = parentColDef.getConfigObject();
	columnOption = cloneObject(parentConfig);
	columnOption["field"] = childField.replace("TR.", ""); // We need to remove the 'TR' prefix from the field to avoid confusion with time series fields.
	columnOption["name"] = childField.split("_")[1].split("T")[0]; // Currently, response server format utc date ex "2022-11-23T00:00:00"
	columnOption["parent"] = parentColDef;
	columnOption["timeSeriesName"] = parentColDef.getName();
	columnOption["timeSeriesId"] = parentColDef.getId();
	this.insertColumn(columnOption, idx++);

};

/** @public
* @param {ColumnDefinition~Options|string} columnOption String will be treated as field, while object is treated as the column options
* @param {Grid~ColumnReference} colRef
*/
Grid_Grid.prototype.replaceColumn = function (columnOption, colRef) {
	let colIndex = this.getColumnIndex(colRef);
	if(colIndex < 0) {
		return;
	}
	let colConfig = {};
	let core = this._grid;
	let columnDef = core._getColumnDef(colIndex);

	let value = core.getColumnScalability(colIndex);
	colConfig["scalable"] = value;

	value = core.getColumnCustomLaneSize(colIndex);
	colConfig["width"] = value;

	value = core.getMinimumColumnWidth(colIndex);
	if(value > 0) {
		colConfig["minWidth"] = value;
	}

	value = core.getColumnVisibility(colIndex, 0); // flag 0 means get only core api hidden state
	if(!value) {
		colConfig["hidden"] = true;
	}

	value = columnDef["stationary"];
	if (value) {
		colConfig["stationary"] = value;
	}

	value = columnDef["leftPinned"];
	if (value) {
		colConfig["leftPinned"] = value;
	}

	value = columnDef["rightPinned"];
	if (value) {
		colConfig["rightPinned"] = value;
	}

	if(typeof columnOption === "string") {
		colConfig["field"] = columnOption;
	} else { // type object from user
		for (let key in columnOption) {
			colConfig[key] = columnOption[key];
		}
	}

	if(columnOption["width"] && !columnOption["scalable"]) {
		colConfig["scalable"] = false;
	}

	if(columnOption["scalable"] && !columnOption["width"]) {
		colConfig["width"] = 1;
	}

	let colDef = this.getColumnDefinition(colIndex);
	if(colDef && colConfig.id == null) {
		colConfig.id = colDef.getId(); // retain ID
	}
	this._grid.startBatch("reset");
	this.removeColumn(colIndex);
	this.insertColumn(colConfig, colIndex);
	this._grid.stopBatch("reset");
};

/** to update column name when field info is loaded
 * @private
 * @param {string} field
 * @param {ColumnDefinition} colDef
 * @param {Object} response
 */
Grid_Grid.prototype._onFieldLoadedSuccess = function (field, colDef, response) {
	if(this._connector && response && response.id) {
		let fieldDef = response;
		if (colDef && colDef.getField() === field) {
			if (colDef.isDefaultName() && fieldDef.name) {
				colDef.setName(fieldDef.name);
				this.updateColumnTitle(); // has conflator -> async operation
			}
		}
	}
};

/**
 * @private
 * @param {Object} err
 */
Grid_Grid.prototype._onFieldLoadedError = function (err) {
	// TODO: May implement retry mechanism here, if need
};

/**
 * @private
 * @param {string} field
 * @param {string} referrer
 */
Grid_Grid.prototype._onFieldLoaded = function (field, referrer) {
	if(this._connector) {
		// For time series, we need to wait until the field is loadedm, then we can insert a child from the field data.
		if(js_FieldDefinition.isTimeSeries(field)) {
			let colDef = this.getColumnDefinitionById(referrer); // The 'referrer' is a column ID that was just added
			this._populateTimeSeriesChildren(colDef);
		}
		this._connector.addFields(field, referrer);
	}
};

/**
 * @private
 * @param {Element} host
 */
Grid_Grid.prototype._setScrollbarParent = function (host) {
	host.style.boxSizing = "border-box";
	host.style.overflow = "hidden";
	host.style.position = "relative";

	this._topNode.style.height = "100%";

	this._grid.autoHideScrollbars(false);
	this._grid.getVScrollbar().attachToExternalElement(host);
	this._grid.getHScrollbar().attachToExternalElement(host);
};

/** Get stored field information. If field information has not been requested or no data has been received yet, null value is returned.
* @public
* @function
* @param {string} field
* @return {Object}
*/
Grid_Grid.prototype.getFieldInfo = function(field) {
	return js_FieldDefinition.getFieldInfo(field);
};
/** Request field information from Synapse service. If field information already exists, a resolved promise is returned. Synapse config must be supplied before the request can be made.
* @public
* @function
* @param {string} field
* @return {Promise}
* @example
* let gridConfig = {
*   synapse: { // define synapse configuration
*     apiKey: "xxx",
*     contextApp: "xxx",
*     auth: "xxx" (optional)
*   }
* };
* let promise = grid.loadFieldInfo("CF_LAST");
*/
Grid_Grid.prototype.loadFieldInfo = function(field) {
	return js_FieldDefinition.loadFieldInfo(field);
};

/**
* @private
* @param {string} field
* @returns {boolean}
*/
Grid_Grid.prototype._shouldLoadFieldInfo = function (field) {

	let val = this._RTK || window["JET"]; // Fastest checking can be performed by checking the first condition.
	if(!val) {
		return false;
	}

	// WARNING: If field caching is disabled, it shouldn't load field info
	if(!this._fieldCaching) {
		return false;
	}

	val = js_FieldDefinition.hasFieldInfo(field);
	if(val) {
		return false;
	}

	val = js_FieldDefinition.isAdc(field) || js_FieldDefinition.isRealTimeField(field);
	if(!val) {
		return false;
	}

	return true;
};
/** Remove all existing columns and add new columns based on the given objects
* @public
* @param {Array.<Object>} columns Array of column options
*/
Grid_Grid.prototype.setColumns = function(columns) {
	let grid = this._grid;
	let colCount = (columns) ? columns.length : 0;

	grid.startBatch("reset");
	this.removeAllColumns();
	if(colCount > 0) {
		let prevState = false;
		if(colCount > 1) {
			prevState = grid.freezeLayout(true); // Insert multiple columns can be a huge time consuming
		}
		for(let i = 0; i < colCount; ++i) {
			this.insertColumn(columns[i], i);
		}
		if(colCount > 1) {
			grid.freezeLayout(prevState);
		}
	}
	grid.stopBatch("reset");
};


/** Remove, add and keep column based on the given column data
* @public
* @param {Array.<Object>} columns Array of column options
* @param {boolean=} byId=false, if enable it, this method will only check for differences in the 'id' property
*/
Grid_Grid.prototype.restoreColumns = function(columns, byId) {
	let grid = this._grid;
	grid.startBatch("reset");
	let configObj = this.getConfigObject();
	let previousColumns = configObj.columns;

	let preColLen = previousColumns.length;
	let newColLen = columns.length;

	let compareLogic = byId ? _hasMatchingId : deepEqual;
	let removingFields = [];
	let keepingColumns = [];
	let columnOrdering = [];

	let i, j, found;
	// Slow, time complexity BigO(n ^ 2)
	for (i = 0; i < preColLen; i++) {
		found = false;
		for (j = 0; j < newColLen; j++) {
			if(compareLogic(previousColumns[i], columns[j])) {
				keepingColumns.push(previousColumns[i]);
				found = true;
				break;
			}
		}
		if (!found) {
			removingFields.push(i);
		}
	}

	this.removeColumns(removingFields);

	let keepingLen = keepingColumns.length;
	let prevState = false;
	if(newColLen > 1) {
		prevState = this._grid.freezeLayout(true); // Insert multiple columns can be a huge time consuming
	}
	for (i = 0; i < newColLen; i++) {
		found = false;
		for (j = 0; j < keepingLen; j++) { // loop only keeping column
			if(compareLogic(columns[i], keepingColumns[j])) {
				found = true;
				let colIndex = this.getColumnIndex(columns[i].id || columns[i].field); // We cannot use 'i' (colIndex) in this case, as it will sort the columns. Instead, we need to obtain a new column index from the field.
				columnOrdering.push(this.getColumnId(colIndex));
				break;
			}
		}
		if (!found) {
			this.insertColumn(columns[i], i);
			columnOrdering.push(this.getColumnId(i)); // Use new insertion column index
		}
	}

	if(newColLen > 1) {
		this._grid.freezeLayout(prevState);
	}

	grid.reorderColumns(columnOrdering);
	grid.stopBatch("reset");
};

/** Remove all existing columns and add new columns based on the given texts/fields
* @public
* @function
* @param {Array.<string>} ary Array of fields (string)
* @see {@link Grid#setColumns}
*/
Grid_Grid.prototype.setFields = Grid_Grid.prototype.setColumns;

/** @private
* @param {Object} e
*/
Grid_Grid.prototype._onColumnAdded = function(e) {
	let colDef = /** @type{ColumnDefinition} */(e.context[COL_DEF]);
	delete e.context[COL_DEF];
	let idx = e.colIndex;

	let coreColDef = this._grid._getColumnDef(idx);
	colDef._setCoreColumnDef(coreColDef); // For column id and field

	let colData = this._grid.getColumnData(idx);
	if(!colData) { // Save column inside grid
		colData = this._grid.setColumnData(idx, {});
	}
	colData[COL_DEF] = colDef;
	let userModel = colDef.getUserModel();
	// binding
	this._grid.setColumnDataBindingHandler(idx, colDef.getRenderer());
	// width and scalable
	let value = userModel["width"];
	// eslint-disable-next-line no-undefined
	if(value !== undefined) {
		value = +value;
		let scalable;
		if(userModel["scalable"] != null) {
			scalable = userModel["scalable"] ? true : false;
		} else {
			scalable = value !== value; // NaN means scalable
		}
		this._grid.setColumnWidth(idx, value, scalable);
	} else {
		this._grid.setColumnWidth(idx, 1, true);
	}
	// minWidth
	value = userModel["minWidth"];
	if(value >= 0) {
		this._grid.setMinimumColumnWidth(idx, value);
	}
	// hidden
	value = userModel["hidden"];
	if(value) {
		this._grid.hideColumn(idx, true);
	}
	// textAlign
	value = userModel["textAlign"] || userModel["alignment"];
	if(typeof value === "string") {
		this._grid.setColumnAlignment(idx, value);
	}
	if(!userModel["keepModel"]) {
		colDef.clearUserModel();
	}

	// text select flag per column
	// with this flag enabled user can select text in cell of this column
	if (colDef.isTextSelect()) {
		this._grid.enableColumnClass(idx, 'tr-text-select', true, "content");
	}

	// className
	let i, classes = colDef.getClasses();
	for(i = 0; i < classes.length; i++) {
		this._grid.enableColumnClass(idx, classes[i]);
	}
	let colField = colDef.getField();
	this._grid.setDataColumnName(idx, ROW_DEF); // This make ColumnDefinition renderer work
	let fields = colDef.getAllFields();
	let referrer = colDef.getId();
	let len = fields.length;
	let field, dataType, prom, onLoaded;
	for(i = 0; i < len; i++) {
		field = fields[i];
		if(this._shouldLoadFieldInfo(field)) {
			if(field === colField) {
				dataType = colDef.getDataType(); // Data-type from user's column options
			} else { // Other required fields
				dataType = ColumnDefinition.getDataType(field); // Data-type evaluated from FieldDefinition
			}
			prom = js_FieldDefinition.loadFieldInfo(field)
				.then(this._onFieldLoadedSuccess.bind(this, field, colDef))
				.catch(this._onFieldLoadedError);

			if(!dataType) {
				// wait field-info to be loaded before adding field
				onLoaded = this._onFieldLoaded.bind(this, field, referrer);
				prom = prom.then(onLoaded).catch(onLoaded);
			} else {
				if(colDef.isTimeSeries()) {
					this._populateTimeSeriesChildren(colDef);
				}
				this._connector.addFields(field, referrer);
			}
		} else {
			if(colDef.isTimeSeries()) {
				this._populateTimeSeriesChildren(colDef);
			}
			this._connector.addFields(field, referrer);
		}
	}
};

/** @public
* @param {Grid~ColumnReference} colRef
* @see {@link Grid#removeColumns}
* @see {@link Grid#removeAllColumns}
*/
Grid_Grid.prototype.removeColumn = function(colRef) {
	let colIndex = this.getColumnIndex(colRef);
	if(colIndex < 0) {
		return;
	}

	let colDef = this.getColumnDefinition(colIndex);

	this._grid.removeColumnAt(colIndex);
	this._connector.removeFields(colDef);

	colDef.dispose(); // Dispose at the last line to avoid making properties inaccessible
	// TODO: Remove fields that are related to the column (e.g. fields for coloring)
};
/** Remove multiple columns through array. To remove all columns, use removeAllColumns() instead of this method
* @public
* @param {Array.<Grid~ColumnReference>} colRefs
* @see {@link Grid#removeColumn}
* @see {@link Grid#removeAllColumns}
*/
Grid_Grid.prototype.removeColumns = function(colRefs) {
	let indices = this.getColumnIndices(colRefs);
	let len = indices.length;
	if(len <= 1) {
		if(len === 1) {
			this.removeColumn(indices[0]);
		}
		return;
	}
	indices.sort(Grid_Grid._descendingOrder); // Removal must be done from the back

	let prevState = this._grid.freezeLayout(); // To prevent multiple UI updates

	for(let i = 0; i < len; ++i) {
		let colIndex = indices[i];
		let colDef = this.getColumnDefinition(colIndex);
		colDef.dispose();
		this._grid.removeColumnAt(colIndex);
		this._connector.removeFields(colDef);
	}

	this._grid.freezeLayout(prevState);
};
/** Sorting is also cleared.
* @public
* @see {@link Grid#removeColumn}
* @see {@link Grid#removeColumns}
*/
Grid_Grid.prototype.removeAllColumns = function() {
	let colCount = this.getColumnCount();
	if(colCount <= 0) {
		return;
	}

	if(!this._sharedSorter) {
		this.clearSort();
	}
	for(let i = colCount; --i >= 0;) {
		// WARNING: This may affect plugins event listeners
		this._grid.setColumnDataBindingHandler(i, null);
		let colDef = this.getColumnDefinition(i);
		colDef.dispose();
	}
	// TODO: Remove fields that are related to the column (e.g. fields for coloring)

	this._columnTitleConflator.reset();
	this._timeSeriesChildConflator.reset();

	this._connector.removeAllFields();
	this._grid.setColumnCount(0);
};

/** Move the column at the specified index to the target index. When moving multiple columns, the original order of the source columns will be sorted by their index and the position of the columns will not be guaranteed to be exactly at the destination index.
* @public
* @param {number|Array.<number>} fromColIndex index or array of indices of the column(s) to be moved.
* @param {number} toColIndex target column index
* @return {boolean} Return true if there is any change, and false otherwise
* @see {@link Grid#moveColumnById}
*/
Grid_Grid.prototype.moveColumn = function (fromColIndex, toColIndex) {
	return this._grid.moveColumn(fromColIndex, toColIndex);
};
/** If source column is not found, no operation is performed. If destination column is not found, the source column will be moved to the last position.<br>
* Note: this method behaves slightly different from moveColumn method in that it always put source column at the position before the specified destination column, while moveColumn method will put column at exactly at the destination index.
* @public
* @param {number|string} srcCol Column Id or index
* @param {(number|string)=} destCol Column Id or index of the destination
* @return {boolean} Return true if there is any change, and false otherwise
* @see {@link Grid#moveColumn}
* @example
* grid.moveColumnById(3, 1); // Move column 3 to position before column 1
* grid.moveColumnById(0, 2); // Move column 0 to position before column 2 (column index 1)
* grid.moveColumnById(0, 1); // Nothing is moved
* grid.moveColumnById("sourceColumnId", "anotherId");
* grid.moveColumnById("sourceColumnId", ""); // move to the last position
*/
Grid_Grid.prototype.moveColumnById = function (srcCol, destCol) {
	return this._grid.moveColumnById(srcCol, destCol);
};

/** @public
* @param {number|string|Array.<number|string>} colRefs List of column index or column id to be moved
* @param {(number|string)=} destCol Destination position where the moved columns will be placed BEFORE the specified position. This can be column id or index
* @return {boolean} Return true if there is any change, and false otherwise
*/
Grid_Grid.prototype.reorderColumns = function (colRefs, destCol) {
	return this._grid.reorderColumns(colRefs, destCol);
};

/** The hidden column still occupies the same index.
* @public
* @param {Grid~ColumnReference} colRef
* @param {boolean=} hidden
* @see {@link Grid#hideColumns}
* @see {@link Grid#showAllColumns}
*/
Grid_Grid.prototype.hideColumn = function(colRef, hidden) {
	let colIndex = this.getColumnIndex(colRef);
	if(colIndex < 0) { // not found
		return;
	}
	this._grid.hideColumn(colIndex, hidden);

};
/** Hide multiple columns at once. The hidden columns still occupy the same index.
* @public
* @param {Array.<Grid~ColumnReference>} colRefs
* @param {boolean=} hidden
* @see {@link Grid#hideColumn}
* @see {@link Grid#showAllColumns}
*/
Grid_Grid.prototype.hideColumns = function(colRefs, hidden) {
	this._grid.hideColumns(this.getColumnIndices(colRefs), hidden);
};
/** @public
* @see {@link Grid#hideColumn}
* @see {@link Grid#hideColumns}
*/
Grid_Grid.prototype.showAllColumns = function() {
	this._grid.showAllColumns();
};
/** @private
* @function
* @param {number} a
* @param {number} b
* @return {number}
*/
Grid_Grid._descendingOrder = function(a, b) {
	return b - a;
};

/** Add fields for real-time data without adding column nor UIs. No duplication is allowed
* @public
* @param {string|Array.<string>} fieldRef
* @param {string=} referrer
* @return {boolean}
*/
Grid_Grid.prototype.addDataFields = function(fieldRef, referrer) {
	if(!fieldRef) {
		return false;
	}

	let fields = Array.isArray(fieldRef) ? fieldRef : [fieldRef];
	let len = fields.length;
	let i, field, dataType, prom, onLoaded;
	for(i = 0; i < len; i++) {
		field = fields[i];
		if(this._shouldLoadFieldInfo(field)) {
			dataType = ColumnDefinition.getDataType(field);
			prom = js_FieldDefinition.loadFieldInfo(field)
				.catch(this._onFieldLoadedError);

			if(!dataType) {
				// wait field-info to be loaded before adding field
				onLoaded = this._onFieldLoaded.bind(this, field, referrer);
				prom = prom.then(onLoaded).catch(onLoaded);
			} else {
				this._connector.addFields(field, referrer || "grid");
			}
		} else {
			this._connector.addFields(field, referrer || "grid");
		}
	}

	return true;
};
/** Add fields for real-time data without adding column nor UIs. No duplication is allowed
* @public
* @param {string|Array.<string>} fieldRef
* @param {string=} referrer
* @return {boolean}
*/
Grid_Grid.prototype.removeDataFields = function(fieldRef, referrer) {
	if(!fieldRef) {
		return false;
	}

	this._connector.removeFields(fieldRef, referrer || "grid");
	return true;
};
/** @public
* @param {string} referrer
*/
Grid_Grid.prototype.removeFieldReferrer = function(referrer) {
	if(!referrer) {
		return;
	}

	this._connector.removeFieldReferrer(referrer);
};

/** @public
* @param {Grid~ColumnReference} colRef
* @param {string=} opt_order This can be "ascending", "descending", "none", "noOrder", or "originalOrder"
*/
Grid_Grid.prototype.sortColumn = function(colRef, opt_order) {
	if(typeof colRef === "string") {
		this._stp.sortColumn(colRef, opt_order); // Allow sorting by field
		return;
	}

	let colIndex = this.getColumnIndex(colRef);
	if(colIndex < 0) {
		this._stp.sortColumn(this.getSortedColumnIndex(), "n"); // reset current sorting
	} else {
		this._stp.sortColumn(colIndex, opt_order);
	}
};
/** @public
* @param {number} colIndex
* @param {string} str
*/
Grid_Grid.prototype.setColumnName = function(colIndex, str) {
	let colDef = this.getColumnDefinition(colIndex);
	if(colDef) {
		colDef.setName(str);
		this.updateColumnTitle();
	}
};
/** @public
* @param {Grid~ColumnReference} colRef
* @return {string}
*/
Grid_Grid.prototype.getColumnBackgroundColor = function(colRef) {
	let colDef = this.getColumnDefinition(colRef);
	if(colDef) {
		return colDef.getBackgroundColor();
	}
	return "";
};
/** @public
* @param {Grid~ColumnReference} colRef
* @param {string} color
*/
Grid_Grid.prototype.setColumnBackgroundColor = function(colRef, color) {
	let colDef = this.getColumnDefinition(colRef);
	if(colDef) {
		colDef.setBackgroundColor(color);
	}
};
/** @public
* @param {Grid~ColumnReference} colRef
* @param {Function=} func
*/
Grid_Grid.prototype.setColumnRenderer = function(colRef, func) {
	let colIndex = this.getColumnIndex(colRef);
	this.setColumnFormatter(colIndex, func);
};

/** @public
* @param {Grid~ColumnReference} colRef Column index or id
* @param {string=} id ID for renderer mapping. If not specified the default renderer will be picked up
* @param {Function=} func If not specified the renderer which corresponded id from mapping will be picked up
*/
Grid_Grid.prototype.activateColumnRenderer = function(colRef, id, func) {
	let colIndex = this.getColumnIndex(colRef);
	if (colIndex == null) { return; }

	let colDef = this.getColumnDefinition(colIndex);
	if(!colDef) { return; }

	colDef.activateRenderer(id, func);

	this._grid.setColumnDataBindingHandler(colIndex, colDef.getRenderer());
	this._grid.requestRowRefresh();
};

/**
 * @public
 * @param {number} colIndex Column index of the position of the column to be updated
 * @param {Object|Function=} formatter Formatter object or bind function
 */
Grid_Grid.prototype.setColumnFormatter = function (colIndex, formatter) {
	if (colIndex == null) { return; }

	let colDef = this.getColumnDefinition(colIndex);
	if(!colDef) {
		return;
	}

	let func = null;
	if (formatter) {
		if(typeof formatter === "object") {
			if(typeof formatter.init === "function") { // Initialize predefined formatter
				// WARNING: Column definition has not been updated
				formatter.init(colDef.getConfigObject(), this._grid);
			}
			func = formatter.binding;
		} else {
			func = formatter;
		}
		if(typeof func !== "function") {
			func = null;
		}
	}

	if(func !== colDef.getRenderer()) {
		let section = this._grid.getSection("content");
		let col = /** @type {Column} */section.getColumn(colIndex);
		if (col) {
			col.clearContent();
		}

		this._grid.setColumnDataBindingHandler(colIndex, null);

		colDef.setRenderer(func);

		this._grid.setColumnDataBindingHandler(colIndex, colDef.getRenderer());
		this._grid.requestRowRefresh();
	}
};

/** @public
* @param {Grid~ColumnReference} colRef
* @param {ColumnDefinition~SortLogic=} func
*/
Grid_Grid.prototype.setColumnSorter = function(colRef, func) {
	let colIndex = this.getColumnIndex(colRef);
	let colDef = this.getColumnDefinition(colIndex);
	if(colDef) {
		colDef.setSorter(func);

		let sortedColumn = this.getSortedColumnIndex(); // TODO: Support multi-column sorting
		if(sortedColumn === colIndex) {
			this._stp.refresh();
		}
	}
};


/** Initialize data for the given rowDef from another rowDef with the same RIC
* @private
* @param {!RowDefinition} rowDef
*/
Grid_Grid.prototype._initDuplicateRicData = function(rowDef) {
	if(!rowDef) {
		return;
	}
	let rowDefs = this._connector.getRowDefByRic(rowDef.getSymbol());
	if(rowDefs && rowDefs.length > 0) { // Found at least 1 Duplicate chain/ric data
		let firstRowDef = rowDefs[0];
		rowDef.copyRowData(firstRowDef);
		if(rowDef.isChain()) {
			let children = firstRowDef.getChildren();
			if(children && children.length > 0) {
				setTimeout(this._cloneChain.bind(this, rowDef), 0); // Need to delay to wait row inserted
			}
		}
	}
};

/** @private
* @param {Object} newRowDef
*/
Grid_Grid.prototype._cloneChain = function(newRowDef) {
	let rowDefs = this._connector.getRowDefByRic(newRowDef.getSymbol());
	let firstRowDef = rowDefs ? rowDefs[0] : null;
	let constituents = firstRowDef ? firstRowDef.getChildren() : null;
	let count = constituents ? constituents.length : 0;
	if(count < 0) {
		return;
	}

	let subId = newRowDef.getSubId();
	if(!subId) {
		return; // A chain without subscription cannot have constituents
	}
	let evtArg = {
		"subId": subId
	};
	for (let i = 0; i < count; i++) {
		let childRowDef = constituents[i];
		evtArg["ric"] = childRowDef.getRic();
		evtArg["values"] = childRowDef.cloneRowData();
		evtArg["values"]["SUB_ID"] = subId; // Imitate real-time service responses

		this._onQ2DataChanged(evtArg); // WARNING: evtArg is shared across multiple calls
	}
};
/** @public
* @param {(RowDefinition~Options|string)=} rowOption
* @param {Grid~RowReference=} rowRef Reference (i.e. row index, row id, or row definition) of the insert position
* @returns {RowDefinition}
* @example
* let grid = new rt.Grid(grid_div, options);
* grid.insertRow({"ric": "RIC"}, 0); // A new row is added at the top
*/
Grid_Grid.prototype.insertRow = function(rowOption, rowRef) {
	if(this._mainGrid) {
		return this._mainGrid.insertRow(rowOption, this._getRowId(rowRef));
	}

	if(rowOption) {
		if(typeof rowOption === "string") {
			rowOption = { "ric": rowOption };
		} else if(Array.isArray(rowOption["values"])) {
			if(!rowOption["fields"]) { // Static values are defined but no fields
				rowOption["fields"] = this.getColumnFields(); // For initializing static value
			}
		}
	}
	let rowDef = new RowDefinition(rowOption);

	if(rowDef.setDataSource(this._dc, this._subs)) { // This could also subscribe chain index/ric to JET/RTK
		this._hasNewUpdates = true; // Mark data table for cleaning it up later
	}
	rowDef.registerToView(this._dv, this._getRowId(rowRef));

	this._initDuplicateRicData(rowDef);

	if(rowOption && rowOption["hidden"]) {
		this._dv.hideRow(rowDef.getRowId()); // Try to obtain rowId in rowDef since rowId is not assigned when new rows are created.
	}
	this._connector.addRic(rowDef);
	return rowDef;
};
/** Insert a row as a segment separator
* @public
* @param {RowDefinition~Options=} rowOption
* @param {Grid~RowReference=} rowRef Reference (i.e. row index, row id, or row definition) of the insert position
* @returns {RowDefinition}
*/
Grid_Grid.prototype.insertSegmentSeparator = function(rowOption, rowRef) {
	if(!rowOption) {
		rowOption = {};
	}
	if(typeof rowOption === "object") {
		rowOption.asSegment = true;
		return this.insertRow(rowOption, rowRef);
	}
	return null;
};
/** @public
* @param {Array.<RowDefinition~Options|string>} rowOptions Array of row option object
* @param {Grid~RowReference=} rowRef Reference (i.e. row index, row id, or row definition) of the insert position
* @param {Array.<string>=} opt_fields
* @example
* let grid = new rt.Grid(grid_div, options);
* grid.insertRows([
*  {ric: "RIC"},
*  {values: {"field1": 0, "field2": 1}},
*  {ric: "RIC2", values: [1, 2, 3, 4]},
*  null
* ]); // 4 rows are appended
*/
Grid_Grid.prototype.insertRows = function(rowOptions, rowRef, opt_fields) {
	if(this._mainGrid) {
		this._mainGrid.insertRows(rowOptions, this._getRowId(rowRef));
		return;
	}
	if(!Array.isArray(rowOptions)) {
		return;
	}

	let prevState = this._dt.freeze(); // Avoid sorting for each inserted row
	let fields = opt_fields || this.getColumnFields();
	let rowId = this._getRowId(rowRef);
	let len = rowOptions.length;
	for(let r = 0; r < len; ++r) {
		let rowOption = rowOptions[r];
		if(rowOption) {
			if(typeof rowOption === "string") {
				rowOption = { "ric": rowOption };
			}
			rowOption["fields"] = fields; // For initializing static value
		}
		this.insertRow(rowOption, rowId); // Note that data view is being frozen, so only using row id will work properly
	}
	this._dt.freeze(prevState);
};
/** Add multiple rows without `ric` property in {@link RowDefinition}
* @public
* @param {Array} dataRows 2D Array of data or Array of Field/Value pair object
* @param {Array.<string>=} fields If no field is given, field defined in column definition will be used instead
* @see {@link Grid#logDV}
* @see {@link Grid#logDT}
* @example
* let grid = new rt.Grid(grid_div, options);
* grid.addStaticDataRows([
*  [10, "a", true],
*  [20, "b", false],
*  [30, "c", true]
* ], ["field1", "field2", "field3"]); // 3 rows are appended
*
* grid.addStaticDataRows([ // Object format can also be used
*  {"field1": 40, "field2": "a", "field3": true},
*  {"field1": 50, "field2": "b"}, // each row doesn't have to be the same
*  {"field1": 60, "field4": "some thing"}
* ]); // 3 more rows are appended
* grid.logDV({"colNames": ["field1", "field2", "field3", "field4"]}); // See results in console
*/
Grid_Grid.prototype.addStaticDataRows = function(dataRows, fields) {
	if(this._mainGrid) {
		this._mainGrid.addStaticDataRows(dataRows, fields);
		return;
	}

	if(!Array.isArray(dataRows)) {
		let userObj = dataRows;
		dataRows = null;
		if(userObj) {
			if(userObj.fields && !fields) {
				fields = userObj.fields;
			}
			if(Array.isArray(userObj.data)) { // dataRows must has an array type
				dataRows = userObj.data;
			}
		}
	}
	if(dataRows) {
		this.insertRows(dataRows.map(Grid_Grid._dataToRowOptions), null, fields);
	}
};
/** Insert, update, remove and reorder data based on the given array of records
* @public
* @param {Array.<Object>} records Array of records for insert, remove, update, reorder in one operation
* @param {string=} rowIdentifier Check difference data with the row identifier map property for operate record.
* @example
* // Grid data set will be updated to 3 records. All rows in the grid will be cleared and new rows will be inserted
* grid.updateDataSet([
* 	{ id: "A", field2: 1, field3: 2, field4: "Something A" },
*	{ id: "B", field2: 1, field3: 2, field4: "Something B" },
* 	{ id: "C", field2: 1, field3: 2, field4: "Something C" },
* ], "id");
* // Update record id "B" in "field2", "field3" value 100, 200
* grid.updateDataSet([
* 	{ id: "A", field2: 1, field3: 2, field4: "Something A" },
* 	{ id: "B", field2: 100, field3: 200, field4: "Something B" },
* 	{ id: "C", field2: 1, field3: 2, field4: "Something C" },
* ], "id");
* // Delete record id "B"
* grid.updateDataSet([
* 	{ id: "A", field2: 1, field3: 2, field4: "Something A" },
* 	{ id: "C", field2: 1, field3: 2, field4: "Something C" },
* ], "id");
* // Insert record id "D"
* grid.updateDataSet([
* 	{ id: "A", field2: 1, field3: 2, field4: "Something A" },
* 	{ id: "C", field2: 1, field3: 2, field4: "Something C" },
* 	{ id: "D", field2: 1, field3: 2, field4: "Something D" },
* ], "id");
* // Reorder to "D", "C", "A"
* grid.updateDataSet([
* 	{ id: "D", field2: 1, field3: 2, field4: "Something D" },
* 	{ id: "C", field2: 1, field3: 2, field4: "Something C" },
* 	{ id: "A", field2: 1, field3: 2, field4: "Something A" },
* ], "id");
* // Insert, Update, Delete And Reorder
* grid.updateDataSet([
* 	{ id: "F", field2: 1, field3: 2, field4: "Something F" },
* 	{ id: "D", field2: 1, field3: 2, field4: "Something D" },
* 	{ id: "C", field2: 1, field3: 2, field4: "Something C" },
* 	{ id: "E", field2: 1, field3: 2, field4: "Something E" },
* ], "id");
*/
Grid_Grid.prototype.updateDataSet = function(records, rowIdentifier) {
	if(!Array.isArray(records) || records.length === 0 ) {
		this.removeAllRows();
		return;
	}

	// Map new data index
	let recordCount = records.length;
	let fieldSorting = "ROW_ORDER"; // TODO: Should be config by options
	let oldDataMap = {};
	let rowDef, id, record, i;
	let rowDefs = this.getAllRowDefinitions(); // Include the filter/hidden rows
	let rowDefCount = rowDefs.length;
	let prevState = false;
	if(rowIdentifier) {
		prevState = this._dt.freeze();
		for (i = 0; i < rowDefCount; i++) {
			rowDef = rowDefs[i];
			if(rowDef) {
				id = rowDef.getRowData()[rowIdentifier];
				if(id || id === 0) {
					oldDataMap[id] = rowDef;
				} else { // Any existing rowDef without row identifier should be removed
					this.removeRow(rowDef); // Slow
				}
			}
		}

		let idMap = {};
		let newDataMap = {};
		for (i = recordCount - 1; i >= 0; i--) {
			record = records[i];
			id = record[rowIdentifier];
			rowDef = oldDataMap[id];
			newDataMap[id] = record; // Assign a new data map to compare to the previous data
			record[fieldSorting] = i;
			if(idMap[id]) { // Prevent assign data in duplicate row
				continue;
			}
			idMap[id] = true;
			if (!rowDef) {
				this.insertRow({ values: record}); // Insert last position
			} else {
				rowDef.setRowData(record);
			}
		}
		// Check Remove previous data
		for (let rowIdentifierName in oldDataMap) {
			if (oldDataMap[rowIdentifierName] && !newDataMap[rowIdentifierName]) {
				this.removeRow(oldDataMap[rowIdentifierName]); // Slow
			}
		}
		this._dt.sortOnce(ROW_DEF, "a", compareNumber, fieldSorting);
		this._dt.freeze(prevState);
	} else {
		if(recordCount - rowDefCount > 1 || rowDefCount - recordCount > 1) {
			prevState = this._dt.freeze();
		}
		for (i = 0; i < rowDefCount; i++) {
			record = records[i];
			if(record) {
				oldDataMap[i] = rowDefs[i]; // Use index instead of rowIdentifier
			} else {
				this.removeRow(rowDefs[i]); // Need to use rowRef, can't use index in view
			}
		}

		for (i = 0; i < recordCount; i++) { // It will be sort by index when insert / update
			record = records[i];
			rowDef = oldDataMap[i];
			if (!rowDef) {
				this.insertRow({ values: record}); // Insert last position
			} else {
				rowDef.setRowData(record);
			}
		}
		if(recordCount - rowDefCount > 1 || rowDefCount - recordCount > 1) {
			this._dt.freeze(prevState);
		}
	}
};
/** @private
* @param {Array|Object} item
* @return {!Object}
*/
Grid_Grid._dataToRowOptions = function(item) {
	return {values: item};
};
/** @public
* @param {Grid~RowReference} rowRef
* @return {RowDefinition}
* @see {@link Grid#removeRows}
* @see {@link Grid#removeAllRows}
*/
Grid_Grid.prototype.removeRow = function(rowRef) {
	if(this._mainGrid) {
		return this._mainGrid.removeRow(this._getRowId(rowRef));
	}
	let rowDef = this._getRowDefinitionByRef(rowRef);
	if(rowDef) {
		if(!rowDef.isAutoGenerated()) { // Users cannot remove auto-generated row by themselves
			this._removeRow(rowDef);
		}
	}
	return rowDef;
};
/** Remove multiple rows at once. This prevents duplicate operation and improve performance. To remove all rows, use removeAllRows() method instead of this method.
* @public
* @param {Array.<Grid~RowReference>} rowRefs
* @return {Array.<RowDefinition>} Array of removed row definitions
* @see {@link Grid#removeRow}
* @see {@link Grid#removeAllRows}
*/
Grid_Grid.prototype.removeRows = function(rowRefs) {
	let rowDef;
	if(!Array.isArray(rowRefs)) {
		rowDef = this.removeRow(rowRefs);
		return rowDef ? [rowDef] : [];
	}
	if(this._mainGrid) {
		return this._mainGrid.removeRows(rowRefs.map(this._getRowId));
	}

	let i;
	let rowDefs = [];
	let len = rowRefs.length;

	// Verify user input
	for(i = 0; i < len; ++i) {
		rowDef = this._getRowDefinitionByRef(rowRefs[i]);
		if(rowDef) {
			if(!rowDef.isAutoGenerated()) {
				rowDefs.push(rowDef);
			}
		}
	}
	len = rowDefs.length;

	let prevState = false;
	if(len > 1) {
		prevState = this._dt.freeze(); // Prevent multiple changes that trigger sorting and filtering
	}

	for(i = 0; i < len; ++i) {
		this._removeRow(rowDefs[i]);
	}

	if(len > 1) {
		this._dt.freeze(prevState);
	}
	return rowDefs;
};
/** @public
* @see {@link Grid#removeRow}
* @see {@link Grid#removeRows}
*/
Grid_Grid.prototype.removeAllRows = function() {
	if(this._sharedDataSource) {
		return;
	}
	if(!this._dt.getRowCount()) {
		return;
	}
	// Data source is not shared at this point
	let rowDefs = this._getAllRowDefinitions();

	this._dc.clearAllData();
	this._dt.clearAllData();
	this._dt.setClassificationSource(this._dc);
	this._clearDataUpdates();

	rowDefs.forEach(RowDefinition.dispose); // Each individual subscription is unsubscribed along with disposed rowDef

	this._dcConflator.reset();
	this._formulaConflator.reset();
	this._chainConflator.reset();
	this._constituentMap = null;
	this._connector.removeAllRics();

	// TODO: This logic should also be in the core grid
	this._grid.getVScrollbar().setScrollTop(0);
};
/** WARNING: This does not remove data stored in the data cache
* @private
* @param {!RowDefinition} rowDef
*/
Grid_Grid.prototype._removeRow = function(rowDef) {
	if(this._sharedDataSource) {
		return;
	}
	this._dispatch("beforeRowRemoved", {});

	let childRowDefs = rowDef.getDescendants(); // TODO: Support nested child
	if(childRowDefs) {
		this._removeConstituentRows(childRowDefs);
	}
	this._connector.removeRic(rowDef);
	this._dt.removeRow(rowDef.getRowId()); // TODO: Merge this with the above removeRows() method
	rowDef.dispose(); // WARNING: This does not remove child reference from its parent
};

/** @private
* @param {Array.<RowDefinition>} rowDefs
*/
Grid_Grid.prototype._removeConstituentRows = function(rowDefs) {
	let connector = this._connector;
	let rowIds = [];
	for(let i = 0; i < rowDefs.length; i++) {
		let childRowDef = rowDefs[i];
		rowIds.push(childRowDef.getRowId());
		connector.removeRic(childRowDef);
		childRowDef.dispose();
	}

	this._dt.removeRows(rowIds);
};

/** @public
* @param {Grid~RowReference} rowRef
* @param {boolean=} hidden if false, show instead of hide
*/
Grid_Grid.prototype.hideRow = function(rowRef, hidden) {
	let rid = this._getRowId(rowRef);
	let grid = this._mainGrid || this._grid;
	grid.hideRow(rid, hidden);
};

/** Hide row(s) in the Grid by specifying the rIds
* @param {Grid~RowReference | Array.<Grid~RowReference>} rowRefs
* @param {boolean=} hidden if false, show instead of hide
* @public
*/
Grid_Grid.prototype.hideRows = function (rowRefs, hidden) {
	if (!Array.isArray(rowRefs)) {
		rowRefs = [rowRefs];
	}
	let rids = rowRefs.map(this._getRowId);
	let grid = this._mainGrid || this._grid;
	grid.hideRows(rids, hidden);
};
/**
* Show row(s) in the Grid by specifying the rIds
* @param {Grid~RowReference | Array.<Grid~RowReference>} rowRefs
* @param {boolean=} shown if false, hide instead of show
* @public
*/
Grid_Grid.prototype.showRows = function (rowRefs, shown) {
	this.hideRows(rowRefs, shown === false);
};
/**
 * Unhide all hidden rows in the Grid
 * @public
 */
Grid_Grid.prototype.unhideAllRows = function () {
	this._grid.unhideAllRows();
};
/**
* @public
* @returns {boolean}
*/
Grid_Grid.prototype.hasHiddenRow = function(){
	return this._grid.hasHiddenRow();
};

/** Set RIC to the specified row
* @public
* @param {Grid~RowReference} rowRef
* @param {string} str RIC to be set to a row
* @param {RowDefinition~Options=} options
* @returns {RowDefinition} Returns modified Row Definition instance
*/
Grid_Grid.prototype.setRic = function(rowRef, str, options) {
	if(this._mainGrid) {
		return this._mainGrid.setRic(this._toRowId(rowRef), str, options);
	}
	let rowDef = this._getRowDefinitionByRef(rowRef);
	if(rowDef) {
		let tempOpt = options || {};
		if(typeof str === "string") {
			tempOpt["ric"] = str;
		} else {
			if(str && !options) {
				tempOpt = str;
			}
			str = tempOpt["ric"] ? tempOpt["ric"] : "";
		}
		let extractedOptions = RowDefinition.extractRowOptions(tempOpt);
		let oldRic = rowDef.getSymbol();
		if(rowDef.setContent(str, extractedOptions)) { // The given string may not be a RIC
			this._connector.removeRic(rowDef, oldRic);
			this._initDuplicateRicData(rowDef);
			this._connector.addRic(rowDef);
		}

		let values = tempOpt["values"];
		if(values != null) {
			rowDef.setStaticRowData(values, tempOpt["fields"]);
		}
	}
	return rowDef;
};
/** Unlink the chain and its constituents. When the chain is expanded,
 * the chain row and its members are converted from autogenerated to
 * regular real-time rows. Only the chain row will be transformed to
 * a conventional real-time row if the chain is collapsed.
 * All converted rows will continue to have their data updated.
 *
 * @public
 * @param {Grid~RowReference} rowRef
 */
Grid_Grid.prototype.unlinkChain = function(rowRef) {
	let rowDef = this._getRowDefinitionByRef(rowRef);
	if(!rowDef) {
		return;
	}

	if(!rowDef.isChain()) {
		return;
	}

	let childRowDefs = rowDef.getDescendants(); // TODO: Support nested child
	if(childRowDefs) {
		let len = childRowDefs.length;
		let ridMap = {};
		for(let i = 0; i < len; i++) {
			ridMap[childRowDefs[i].getRic()] = "";
		}
		let collapsed = rowDef.isChainCollapsed();
		let args = {
			chain: rowDef,
			collapsed: collapsed,
			ridMap: ridMap
		};
		this._dispatch("beforeUnlinked", args);

		if(!collapsed) {
			let rowIdChanged = false;
			let ridPair = {};
			for(let i = 0; i < len; i++) {
				let childRowDef = childRowDefs[i];
				let mappedRowId = ridMap[childRowDef.getRic()];
				if(mappedRowId) {
					if(mappedRowId.match(RowDefinition.ROW_ID_PATTERN)) {
						console.warn("Please change the rowId format to avoid duplicated rows' id causing unexpected behavior.");
						mappedRowId = "";
					} else {
						rowIdChanged = true;
						ridPair[childRowDef.getRowId()] = mappedRowId;
					}
				}
				childRowDef.toRealTimeRow(mappedRowId);
			}
			if(rowIdChanged) {
				this._dc.replaceRowIds(ridPair);
				this._dt.replaceRowIds(ridPair);
			}
		} else {
			this._removeConstituentRows(childRowDefs);
		}
	}

	rowDef.unlinkChain();
};

/** Alias to setRic
* @public
* @function
* @param {Grid~RowReference} rowRef
* @param {string} str
* @see {@link Grid.setRic}
*/
Grid_Grid.prototype.setRowContent = Grid_Grid.prototype.setRic;

/**
* @public
* @param {string|number} rid Row Id or index
* @param {boolean=} force
*/
Grid_Grid.prototype.toggleRowExpansion = function(rid, force) {
	this._dv.toggleRowExpansion(rid, force);
};
/**
* @private
* @param {object} e
*/
Grid_Grid.prototype._onRowExpansionBinding = function(e) {
	if(!this.hasListener("rowExpansionBinding")) {
		return;
	}

	if(e["originalRowId"]) {
		let rowIndex = this._dv.getRowIndex(e["originalRowId"]);
		e["originalRowDef"] = this._getRowDefinitionByIndex(rowIndex);
		e["originalRowData"] = e["originalRowDef"].getRowData();
	} else {
		e["rowDef"] = this._getRowDefinitionByIndex(e["rowIndex"]);
		e["rowData"] = e["rowDef"].getRowData();
	}

	this._dispatch("rowExpansionBinding", e);
};

/** Reset existing rows. It is a combination of insertRows and removeAllRows.
* @public
* @param {Array.<string>} ary
*/
Grid_Grid.prototype.setRics = function(ary) {
	this.removeAllRows();
	if(ary && ary.length) {
		this.insertRows(ary);
	}
};

/** WARNING: This represents number of visible rows in a single grid
* @public
* @return {number}
*/
Grid_Grid.prototype.getRowCount = function() {
	return this._dv.getRowCount();
};
/** Get all rows without paging
* @public
* @return {number}
*/
Grid_Grid.prototype.getPageItemCount = function() {
	return this._dv.getRowCount(false, true);
};
/** WARNING: This represents number of rows in the DataTable which includes all invisible rows and rows in multiple wrapped grids
* @public
* @return {number}
*/
Grid_Grid.prototype.getDataRowCount = function() {
	return this._dt.getRowCount();
};
/** @public
* @return {number}
*/
Grid_Grid.prototype.getColumnCount = function() {
	return this._grid.getColumnCount();
};
/** @public
* @param {number} colIndex
* @return {string}
*/
Grid_Grid.prototype.getColumnDataType = function(colIndex) {
	let colDef = this.getColumnDefinition(colIndex);
	if (colDef) {
		return colDef.getDataType();
	}
	return '';
};
/** @public
* @param {number} colIndex
* @return {ColumnDefinition}
*/
Grid_Grid.prototype.getColumnDefinition = function(colIndex) {
	let colData = this._grid.getColumnData(colIndex);
	if(colData) {
		return /** @type{ColumnDefinition} */(colData[COL_DEF]) || null;
	}
	return null;
};
/** @public
* @return {!Array.<ColumnDefinition>}
*/
Grid_Grid.prototype.getColumnDefinitions = function() {
	let colCount = this.getColumnCount();
	let colDefs = new Array(colCount);
	for(let i = 0; i < colCount; i++) {
		colDefs[i] = this.getColumnDefinition(i);
	}
	return colDefs;
};

/** @public
* @param {string} colId
* @return {ColumnDefinition}
*/
Grid_Grid.prototype.getColumnDefinitionById = function(colId) {
	if(typeof colId === "string") {
		return this.getColumnDefinition(this.getColumnIndex(colId));
	}
	return null;
};

/** @public
* @param {Array.<string>} colIds
* @return {ColumnDefinition}
*/
Grid_Grid.prototype.getColumnDefinitionsById = function(colIds) {
	let colCount = Array.isArray(colIds) ? colIds.length : 0;
	if(colCount > 0) {
		let i, colId;
		let colDefs = new Array(colCount);
		for(i = 0; i < colCount; i++) {
			colId = colIds[i];
			colDefs[i] = this.getColumnDefinitionById(colId);
		}
		return colDefs;
	}
	return null;

};
/** @private
* @param {Grid~ColumnReference} colRef
* @return {ColumnDefinition}
*/
Grid_Grid.prototype._getColumnDefinition = function(colRef) {
	if(colRef instanceof ColumnDefinition) {
		return /** @type{!ColumnDefinition} */(colRef);
	} else if(typeof colRef === "number") {
		return this.getColumnDefinition(colRef);
	} else if(typeof colRef === "string") {
		let colCount = this.getColumnCount();
		for(let i = 0; i < colCount; ++i) {
			let colDef = this.getColumnDefinition(i);
			if(_hasFieldOrId(colDef, colRef)) {
				return colDef; // Return the first found column
			}
		}
	}
	return null;
};
/** @public
* @param {number|string} rowRef Row index as shown in the view or row id (string)
* @return {string}
*/
Grid_Grid.prototype.getRowType = function(rowRef) {
	let rowDef = this.getRowDefinition(rowRef);
	return rowDef ? rowDef.getType() : "";
};
/** Get RowDefinition object by either number or row id
* @public
* @param {number|string} rowRef Row index as shown in the view or row id (string)
* @return {RowDefinition}
*/
Grid_Grid.prototype.getRowDefinition = function(rowRef) {
	if(typeof rowRef === "number") {
		return this._getRowDefinitionByIndex(rowRef);
	} else if(typeof rowRef === "string") {
		return this._getRowDefinitionById(rowRef);
	}
	return null;
};
/** @private
* @param {number} rowIndex
* @return {RowDefinition}
*/
Grid_Grid.prototype._getRowDefinitionByIndex = function(rowIndex) {
	return this._dv.getDataAt(rowIndex, ROW_DEF) || null;
};
/** @private
* @param {string} rowId
* @return {RowDefinition}
*/
Grid_Grid.prototype._getRowDefinitionById = function(rowId) {
	return this._dt.getData(rowId, ROW_DEF) || null;
};
/** @private
* @param {Grid~RowReference} rowRef
* @return {RowDefinition}
*/
Grid_Grid.prototype._getRowDefinitionByRef = function(rowRef) {
	if(rowRef instanceof RowDefinition) {
		if(rowRef.getRowId()) { // The row may have been removed from the grid
			return /** @type{!RowDefinition} */(rowRef);
		}
	}
	return this.getRowDefinition(rowRef);
};
/** Get all row definitions shown in the view, excluding rows that are hidden by filtering or pagination. <br>
* A new array object is created every time. Do not call this method repeatedly.<br>
* Group header row will return null for its rowDefinition since the row is auto generated and has no data nor definition.
* @public
* @return {!Array.<RowDefinition>}
*/
Grid_Grid.prototype.getRowDefinitions = function() {
	return this._dv.getColumnData(ROW_DEF, true); // Include group header rows
};
/** Get all row definitions, including filtered and hidden rows, but without group header rows.
* @private
* @return {!Array.<RowDefinition>}
*/
Grid_Grid.prototype._getAllRowDefinitions = function() {
	return this._dt.getColumnData(ROW_DEF); // no group header rows
};

/** Get all row definitions, including filtered and hidden rows, but without auto generated rows (i.e., group header and chain constituent).
* @public
* @param {boolean=} inclAutoGenerated=false Set to true to include auto generated rows. Default value is false (exclude auto generated rows)
* @return {!Array.<RowDefinition>}
*/
Grid_Grid.prototype.getAllRowDefinitions = function(inclAutoGenerated) {
	if(inclAutoGenerated) {
		return this._getAllRowDefinitions();
	}
	return this._dt.getColumnData(ROW_DEF).filter(excludeAutoGenerated);
};
/** A shorthand to retrieve row data based on index of the specified row.
* @public
* @param {number} rowIndex Row index as shown in the view
* @return {Object} "FIELD1": value1, "FIELD2": value2, ...
*/
Grid_Grid.prototype.getRowData = function(rowIndex) {
	return toRowData(this._getRowDefinitionByIndex(rowIndex));
};
/** Get data from multiple rows. If null or undefined value is given, data from all rows are returned.
* @public
* @param {*=} obj The value can be rowIndex, array of row indices, array of row definitions, DataView instance, null or undefined
* @return {!Array.<Object>}
* @example
* grid.getMultipleRowData(); // All row data are returned
* grid.getMultipleRowData([0, 2, 4]); // Three row data are returned
* grid.getMultipleRowData(1); // One row data from the specified row index is returned
* grid.getMultipleRowData(DataView); // All row data from the specified DataView instance are returned
*/
Grid_Grid.prototype.getMultipleRowData = function(obj) {
	if(typeof obj === "number") {
		return [this.getRowData(obj)];
	}
	let rowDefs = null;
	if(Array.isArray(obj)) {
		let rowRefs = obj;
		let rowCount = rowRefs.length;
		let rows = new Array(rowCount);
		rowDefs = this.getRowDefinitions();

		for(let r = 0; r < rowCount; ++r) {
			let rowRef = rowRefs[r];
			if(typeof rowRef === "number") {
				rows[r] = toRowData(rowDefs[rowRef]);
			} else if(rowRef && rowRef.getRowData) {
				rows[r] = toRowData(rowRef);
			}
		}
		return rows;
	}

	if(obj && obj.getColumnData) { // Check if the given reference is DataView or DataTable
		rowDefs = obj.getColumnData(ROW_DEF, false); // Get all row data WITHOUT group header rows
	} else {
		rowDefs = this.getRowDefinitions(); // Get all row definition from current view INCLUDING group header rows
	}
	return rowDefs.map(toRowData);
};
/** Set row data base on RIC.
* @public
* @param {string} ric
* @param {Object} values {"FIELD1": value1, "FIELD2": value2, ...} Use null to remove current row data (not row in the view).
*/
Grid_Grid.prototype.setRicData = function(ric, values) {
	let rowDefs = this._connector.getRowDefByRic(ric);
	let rowCount = rowDefs ? rowDefs.length : 0;
	for (let i = 0; i < rowCount; ++i) {
		let rowDef = rowDefs[i];
		if (rowDef && rowDef.getSymbol() === ric) {
			rowDef.setRowData(values); // TODO: Need conflator to improve performance
		}
	}
};
/** Retrieve all RIC subscriptions. This method is not dependent on the number of rows, the RIC can vary from the current row in the grid.
* @public
* @return {Array.<string>}
*/
Grid_Grid.prototype.getAllRics = function() {
	return this._connector.getAllRics();
};

/** Returns true if there is at least 1 RIC in the grid. This method includes RICs not in the row.
* @public
* @return {boolean}
*/
Grid_Grid.prototype.hasRic = function() {
	return this._connector.hasRic();
};
/** Returns RIC of given row reference.
* @public
* @param {(string|number)=} rowRef
* @return {string}
*/
Grid_Grid.prototype.getRic = function(rowRef) {
	let rowDef = this.getRowDefinition(rowRef);
	return rowDef.getRic();
};
/** A shorthand to set row data based on index of the specified row. It is better to keep rowDefinition object for updating data directly as row index can be changed by sorting and filtering.
* @public
* @param {Grid~RowReference} rowRef
* @param {Object} values {"FIELD1": value1, "FIELD2": value2, ...} Use null to remove current row data (not row in the view).
*/
Grid_Grid.prototype.setRowData = function(rowRef, values) {
	let rowDef = this._getRowDefinitionByRef(rowRef);
	if(rowDef) {
		rowDef.setRowData(values);
	}
};

/**
* @public
* @param {Grid~RowReference} rowRef
* @param {Object} values
*/
Grid_Grid.prototype.setStaticRowData = function(rowRef, values) {
	let rowDef = this._getRowDefinitionByRef(rowRef);
	if(rowDef) {
		rowDef.setStaticRowData(values);
	}
};

/**
* @public
* @param {Grid~RowReference} rowRef
* @param {string} field
* @param {*} value
*/
Grid_Grid.prototype.setStaticData = function(rowRef, field, value) {
	let rowDef = this._getRowDefinitionByRef(rowRef);
	if(rowDef) {
		rowDef.setStaticData(field, value);
	}
};

/** @private
* @param {Grid~RowReference=} rowRef
* @return {string}
*/
Grid_Grid.prototype._getRowId = function(rowRef) {
	if(typeof rowRef === "string") {
		return rowRef;
	} else if(typeof rowRef === "number") { // WARNING: This is view index, excluding hidden row
		return this._dv.getRowId(rowRef);
	} else if(rowRef instanceof RowDefinition) {
		return rowRef.getRowId();
	}
	return "";
};

/** Clear all existing data from the specified column(s).
* @public
* @param {Grid~ColumnReference|Array.<Grid~ColumnReference>} colRefs
*/
Grid_Grid.prototype.clearColumnData = function(colRefs) {
	let colIndices = this.getColumnIndices(colRefs);
	let colCount = colIndices.length;
	if(!colCount) {
		return;
	}
	let emptyData = {};
	for(let c = 0; c < colCount; ++c) {
		let colIndex = colIndices[c];
		let field = this.getColumnField(colIndex);
		if(field) {
			emptyData[field] = null;
		}
	}

	let rowDefs = this.getAllRowDefinitions();
	let rowCount = rowDefs.length;
	for(let r = 0; r < rowCount; ++r) {
		rowDefs[r].setRowData(emptyData); // There is a conflator caching the data change in DataCache
	}
};

/** @public
* @param {Grid~ColumnReference} colRef
* @return {number}
*/
Grid_Grid.prototype.getColumnIndex = function(colRef) {
	if(colRef instanceof ColumnDefinition) {
		let colCount = this.getColumnCount();
		for(let i = 0; i < colCount; ++i) {
			let colDef = this.getColumnDefinition(i);
			if(colDef === colRef) {
				return i;
			}
		}
		return -1;
	}
	return this._grid.getColumnIndex(colRef);
};
/** Any invalid column reference will be excluded from the output array
* @public
* @param {Array.<Grid~ColumnReference>} colRefs
* @return {!Array.<number>}
*/
Grid_Grid.prototype.getColumnIndices = function(colRefs) {
	let ary = [];
	if(colRefs || colRefs === 0) {
		let colCount = this.getColumnCount();
		let inputAry = Array.isArray(colRefs) ? colRefs : [colRefs];
		let len = inputAry.length;
		// Verify user input
		for(let i = 0; i < len; ++i) {
			let colIndex = this.getColumnIndex(inputAry[i]);
			if(colIndex >= 0 && colIndex < colCount) {
				ary.push(colIndex); // WARNING: We have not checked for duplication
			}
		}
	}
	return ary;
};
/** Return column id that is set through configuration object
* @public
* @param {number} colIndex
* @return {string} Return empty string if no existing id or column
* @see {@link Grid#getColumnDefinition}
*/
Grid_Grid.prototype.getColumnId = function(colIndex) {
	return this._grid.getColumnId(colIndex);
};
/** Get ids from each column definition.
* @public
* @return {!Array.<string>} New array is created
*/
Grid_Grid.prototype.getColumnIds = function() {
	return this._grid.getColumnIds();
};
/** Return field defined in the column definition
* @public
* @param {number} colIndex
* @return {string}
* @see {@link Grid#getColumnDefinition}
*/
Grid_Grid.prototype.getColumnField = function(colIndex) {
	return this._grid.getColumnField(colIndex);
};
/** Get fields from each column definition. Note that this does not include any required field or data fields. Duplicates may exist.
* @public
* @return {!Array.<string>} New array is created
*/
Grid_Grid.prototype.getColumnFields = function() {
	return this._grid.getColumnFields();
};
/** Get column name from each column definition. Note that this does not include any required field or data fields. Duplicates may exist.
* @public
* @return {!Array.<string>} New array is created
*/
Grid_Grid.prototype.getColumnNames = function() {
	return this.getColumnDefinitions().map(_getName);
};
/** Get all fields, including required fields and data fields, with no duplicate
* @public
* @return {!Array.<string>} New array is created
*/
Grid_Grid.prototype.getAllFields = function() {
	return this._connector.getAllFields();
};
/** Freeze the column at the left side of the table starting from index 0 to the specified colIndex
* If no index is specified (null or undefined index), unfreeze all columns.
* @public
* @param {number=} colIndex Negative index is equivalent to null value
* @param {number=} pinnedRightColumns Number of columns to be pinned/snapped on the right side
*/
Grid_Grid.prototype.freezeColumn = function(colIndex, pinnedRightColumns) {
	if(colIndex == null) {
		colIndex = -1;
	}
	this._grid.freezeColumn(colIndex, pinnedRightColumns);
};

/** Pin column to the left side by moving the specified column to the rightmost of the frozen columns. <br>
* The method will do nothing if the specified column is already pinned to the left side
* @public
* @param {Grid~ColumnReference|Array.<Grid~ColumnReference>} colRef
* @param {string=} side Available values are: left|right. If no value is supplied, the column will be pinned to the left.
* @return {boolean}
*/
Grid_Grid.prototype.pinColumn = function(colRef, side) {
	if(Array.isArray(colRef)) {
		let ary = colRef;
		let len = ary.length;

		let dirty = 0;
		for(let i = 0; i < len; ++i) {
			dirty |= this._pinColumn(ary[i], side);
		}
		return dirty ? true : false;
	}
	return this._pinColumn(colRef, side);
};
/** @private
* @param {Grid~ColumnReference} colRef
* @param {string=} side Available values are: left|right. If no value is supplied, the column will be pinned to the left.
* @return {boolean}
*/
Grid_Grid.prototype._pinColumn = function(colRef, side) {
	let colIndex = this.getColumnIndex(colRef);
	let colCount = this.getColumnCount();
	if(colIndex < 0 || colIndex > colCount) {
		return false;
	}

	let leftPinnedCount = this._grid.getPinnedLeftColumnCount();
	let rightPinnedCount = this._grid.getPinnedRightColumnCount();
	let stationaryIdx = this._grid.getStationaryColumnIndex();

	if(side && side.toLowerCase() === "right") {
		let rightPinnedIndex = this._grid.getFirstPinnedRightIndex();
		if(colIndex >= rightPinnedIndex) {
			return false; // The column is already pinned area
		}

		if(stationaryIdx >= 0 && colIndex <= stationaryIdx) {
			return false;
		}

		this.moveColumnById(colIndex, rightPinnedIndex);
		rightPinnedCount += 1;
		leftPinnedCount -= 1;
	} else {
		if(colIndex < leftPinnedCount) {
			return false; // The column is already pinned area
		}
		if(!leftPinnedCount) {
			if(stationaryIdx >= 0) {
				leftPinnedCount = stationaryIdx;
				if(colIndex > stationaryIdx) {
					leftPinnedCount++;
				}
			}
		}
		this.moveColumnById(colIndex, leftPinnedCount);
	}

	this._grid.freezeColumn(leftPinnedCount, rightPinnedCount);
	return true;
};
/** Unpin column from the left side by moving the specified column to the end of the frozen columns. <br>
* The method will do nothing if the specified column is not pinned on the left side.
* @public
* @param {Grid~ColumnReference|Array.<Grid~ColumnReference>} colRef
* @param {Grid~ColumnReference=} dest The unpinned column will be placed before the destination position after the operation
* @return {boolean}
*/
Grid_Grid.prototype.unpinColumn = function(colRef, dest) {
	if(Array.isArray(colRef)) {
		let ary = colRef;
		let len = ary.length;

		let dirty = 0;
		for(let i = len; --i >= 0;) { // WARNING: unpinning is done in reversed order
			dirty |= this._unpinColumn(ary[i], dest);
		}
		return dirty ? true : false;
	}
	return this._unpinColumn(colRef, dest);
};
/** @private
* @param {Grid~ColumnReference} colRef
* @param {Grid~ColumnReference=} dest The unpinned column will be placed before the destination position after the operation
* @return {boolean}
*/
Grid_Grid.prototype._unpinColumn = function(colRef, dest) {
	let colIndex = this.getColumnIndex(colRef);
	if(colIndex < 0) {
		return false;
	}

	let leftPinnedCount = this._grid.getPinnedLeftColumnCount();
	let rightPinnedCount = this._grid.getPinnedRightColumnCount();
	let colCount = this.getColumnCount();
	let firstRightPinnedIndex = colCount - rightPinnedCount;

	if(colIndex >= leftPinnedCount && colIndex < firstRightPinnedIndex) {
		return false;
	}

	let srcId = null;
	let destId = null;
	if(dest != null) {
		let destIdx = this.getColumnIndex(dest);
		destId = this.getColumnId(destIdx);
		srcId = this.getColumnId(colIndex);
	}

	let stationaryIdx = this._grid.getStationaryColumnIndex();

	if(colIndex < leftPinnedCount) {
		if(colIndex > stationaryIdx) {
			this.moveColumnById(colIndex, leftPinnedCount);
		}

		this._grid.freezeColumn(leftPinnedCount - 2, rightPinnedCount); // Column index is used for freezing
	} else if(colIndex >= firstRightPinnedIndex) {
		if(colIndex > stationaryIdx) {
			this.moveColumnById(colIndex, firstRightPinnedIndex);
		}

		this._grid.freezeColumn(leftPinnedCount - 1, rightPinnedCount - 1); // Column index is used for freezing
	}

	if(colIndex > stationaryIdx) {
		if(destId != null) {
			this.moveColumnById(srcId, destId);
		}
	}

	return true;
};
/** A shorthand to unpin all columns from the left hand side
* @public
* @return {boolean}
*/
Grid_Grid.prototype.unpinAllColumns = function() {
	let leftPinnedCount = this._grid.getPinnedLeftColumnCount();
	let rightPinnedCount = this._grid.getPinnedRightColumnCount();
	if(!leftPinnedCount && !rightPinnedCount) {
		return false;
	}
	this._grid.freezeColumn(-1, 0); // Column index is used for left freezing and column count is used for right freezing
	return true;
};

/** @private
* @param {Object} e
*/
Grid_Grid.prototype._onColumnHeaderBinding = function(e) {
	if(e["sectionType"] !== "title" || e["sectionIndex"] > 0) {
		return; // WARNING: Assume that only the first title section is owned by the grid. Anything else is assumed to be generated by plugins
	}

	let c = e["fromColIndex"];
	let toC = e["toColIndex"];
	while(c < toC) {
		this._renderColumnHeader(c++, e);
	}
};
/** @private
* @param {number=} colIndex
* @param {Object} arg
*/
Grid_Grid.prototype._renderColumnHeader = function(colIndex, arg) {
	let colDef = this.getColumnDefinition(colIndex);
	if(!colDef) {
		return;
	}
	let customRenderer = colDef.getHeaderRenderer();
	let tSection = arg.section;
	let rowCount = tSection.getRowCount();
	arg["colIndex"] = colIndex;
	arg["columnDef"] = colDef;

	let colName = colDef.getName();
	let colTooltip = colDef.getTooltip();
	let headerAlignment = colDef.getHeaderAlignment();
	let tooltipValue = colDef.getTooltipValue();

	for(let r = 0; r < rowCount; ++r) {
		let tCell = tSection.getCell(colIndex, r, false);
		// Default behaviors
		tCell.setContent(colName);
		tCell.setTooltipInfo("columnDefault", tooltipValue);
		tCell.setTooltipInfo("columnTooltip", colTooltip);
		tCell.updateTooltip();
		tCell.setStyle("textAlign", headerAlignment);

		if(customRenderer) {
			arg["rowIndex"] = r;
			arg["cell"] = tCell;
			customRenderer(arg);
		}
	}
};
/** @public
*/
Grid_Grid.prototype.updateColumnTitle = function() {
	if(this._columnTitleConflator.conflate()) {
		return;
	}
	let tss = this._grid.getSectionSettings("title"); // WARNING: Assume that only the first title section is owned by the grid
	if(tss) {
		tss.rerender();
	}
};
/** @public
* @return {boolean}
*/
Grid_Grid.prototype.isSorting = function() {
	return this._stp.isSorting();
};
/** @public
* @return {number}
*/
Grid_Grid.prototype.getSortedColumnIndex = function() {
	return this._stp.getSortedColumnIndex();
};
/** @public
* @return {string} "a" for "ascending, "d" for descending, and "n" for none/noOrder
*/
Grid_Grid.prototype.getSortOrder = function() {
	return this._stp.getSortOrder();
};
/** @public
*/
Grid_Grid.prototype.clearSort = function() {
	this._stp.clearSortState(); // WARNING: No event is dispatched
};
/** Get sorting states from sorting columns
* @public
* @return {!Array.<Object>} Array of sorting states ordered by priority. If there is no sorting column, an empty array is returned
*/
Grid_Grid.prototype.getSortingStates = function () { // This method is mainly for backward compatability
	let ary = [];
	let states = this._stp.getSortingStates();
	let stateCount = states.length;

	for (let i = 0; i < stateCount; i++) {
		let state = states[i];
		let colIndex = this._stp.getSortedColumnIndex(i);

		ary.push({
			"colId": this.getColumnId(colIndex),
			"colIndex": colIndex,
			"order": state["sortOrder"]
		});
	}

	return ary;
};

/**
* @private
* @param {Object} e Event object from quote2
*/
Grid_Grid.prototype._onQuote2PostUpdate = function (e) {
	if(e.childOrderChange) { // For dynamic chain when CHILD_ORDER from the server change, it will be call sort children
		if(!this.isSorting()) {
			// Use rowDef for get CHILD_ORDER to sort
			this._dt.sortOnce("ROW_DEF", "a", _sortChildrenOfChain);
		}
	}
};

/**
* @private
* @param {Object} e
 */
Grid_Grid.prototype._onQ2DataChanged = function (e) {
	let rowDef = this._getRowDefinitionById(e["subId"]);
	if(!rowDef) {
		return; // WARNING: This should not be happened because row has been removed but the data is still received
	}

	let ric = e["ric"];
	let values = e["values"];
	if (values) {
		let duplicateRics = null;
		if(rowDef.verifyConstituent(ric)) {
			let parentDef = rowDef;
			let childDef = parentDef.getConstituent(ric);
			if(childDef) { // The constituent will share the same sub id as its parent
				rowDef = childDef;
			} else {
				rowDef = childDef = parentDef.addConstituent(ric);
				if(!childDef) {
					return; // Parent chain is not alive
				}
				duplicateRics = this._connector.getRowDefByRic(ric);
				this._connector.addRic(childDef); // TODO: JET/RTK should not re-subscribe this
				if(!this._constituentMap) {
					this._constituentMap = {};
				}
				this._constituentMap[childDef.getRowId()] = childDef;
				this._registerConstituents(childDef);
			}
		}

		if(duplicateRics && duplicateRics.length) {
			let duplicateRic = duplicateRics[0];
			duplicateRic.cloneRowData(values, values);
		}

		rowDef.setRowData(values); // Trigger data changes
	} else { // The constituent is requested to be removed by the real-time data provider
		let childDef = rowDef.getConstituent(ric); // WARNING: normal ric and its delayed version must match with the one first given
		if(childDef) {
			childDef.setParent(null); // Manually remove child reference from its parent
			this._removeRow(childDef);
		}
	}
};

/** @private
* @param {RowDefinition} rowDef
*/
Grid_Grid.prototype._registerConstituents = function(rowDef) {
	if(this._chainConflator.conflate(rowDef)) {
		return;
	}

	this._constituentMap = null;

	let view = this._dv;
	let dt = view ? view.getDataSource() : null;
	if(!dt) {
		return;
	}

	let childDefs = this._chainConflator.popAllData(); // This must have no duplication
	let childCount = childDefs ? childDefs.length : 0;

	// Validate row definition and collect its info
	let i;
	let validDefs = [];
	let maxCountMap = {};
	for(i = 0; i < childCount; ++i) {
		let childDef = childDefs[i];
		let parentDef = childDef.getParent();
		if(!parentDef) {
			break; // A constituent without parent cannot be added to the view
		}
		let parentRowId = parentDef.getRowId();
		let maxCount = maxCountMap[parentRowId];
		if(maxCount == null) {
			maxCount = maxCountMap[parentRowId] = parentDef.countChildInView();
		}
		validDefs.push(childDef);
	}

	childCount = validDefs.length;
	if(!childCount) {
		return;
	}

	let prevState = false;
	if(childCount > 1) {
		prevState = this._dt.freeze(); // Avoid sorting for each insertion
	}

	for(i = 0; i < childCount; ++i) {
		let childDef = validDefs[i];
		let parentDef = childDef.getParent();
		let parentRowId = parentDef.getRowId();
		let maxCount = maxCountMap[parentRowId];

		// TODO: Handle nested children
		view.addSegmentChild(parentRowId, childDef.getRowId());

		// Added order and child order can be different
		// CHILD_ORDER value from real-time service indicates the order of constituents.
		let destIndex = dt.getRowIndex(parentRowId) + 1; // WARNING: Not checking for negative index
		let childOrder = childDef.getData("CHILD_ORDER"); // CHILD_ORDER starts from 0
		if(childOrder != null && childOrder < maxCount) {
			destIndex += childOrder;
		} else {
			destIndex += maxCount;
		}
		let destRowId = dt.getRowId(destIndex);
		childDef.registerToView(view, destRowId);
		maxCountMap[parentRowId] = maxCount + 1; // Since new child has been added to the view, maxCount has to be updated
	}
	if(childCount > 1) {
		this._dt.freeze(prevState);
	}
};

/** @private
* @param {Object} e
*/
Grid_Grid.prototype._onDataChanged = function(e) {
	if(!this._dt) {
		return;
	}
	if(e) {
		if(e["initialChanges"] && !this._dt.hasSegmentClassification()) {
			return;
		}

		// TODO: Insertion should also be addressed here.
		if(e["rid"] && e["changes"]) {
			let rowDef = this._getRowDefinitionById(e["rid"]);
			if(rowDef) {
				if(rowDef.addUpdate(e["changes"])) { // This is the only place that update array can grow. It is used for blinking data.
					this._hasNewUpdates = true; // Mark data table for cleaning it up later
				}
			}
		}
	}

	if(this._dcConflator.conflate()) {
		return;
	}

	this._dt.classifySegments();
	this._dt.dispatchGlobalChange();
};

/** @private
* @param {Object} e
*/
Grid_Grid.prototype._onPreDataSorting = function (e) {
	let objs = this._stp.getSortedColumns();

	this._sorter.reset();
	if(Array.isArray(objs)) {
		let sortCount = objs.length;
		for(let i = 0; i < sortCount; ++i) {
			let obj = objs[i];
			let field = obj["field"] || ""; // This supports "sortBy" option (sort by another column)
			let colIndex = obj["colIndex"];
			let colDef = (colIndex >= 0) ? this.getColumnDefinition(colIndex) : null;

			let rowSorting = false;
			let sortLogic = null;
			if(colDef) {
				if(!field) {
					field = colDef.getField(); // WARNING: Field and logic could be out of sync
				}
				rowSorting = colDef.isRowSorting();
				sortLogic = colDef.getSorter();
			}
			// TODO: get sortLogic from DataView
			// if(!sortLogic && field) {
			// sortLogic = state["sortLogic"];
			// }

			this._sorter.addColumnContext(field, sortLogic, rowSorting, obj["sortOrder"], colIndex, colDef);
		}
	}

	this._columnSorter = this._sorter.getSorter();
};
/** @private
* @param {RowDefinition} rowDefA
* @param {RowDefinition} rowDefB
* @param {number} order
* @return {number}
*/
Grid_Grid.prototype._mainSorter = function (rowDefA, rowDefB, order) {
	return this._columnSorter(rowDefA, rowDefB, order);
};

/** @private
* @param {Object} e
*/
Grid_Grid.prototype._onPostSectionDataBinding = function(e) {
	if(e["actualUpdate"]) {
		this._asyncClearDataUpdates();
	}
};
/** @private
*/
Grid_Grid.prototype._asyncClearDataUpdates = function() {
	setTimeout(this._clearDataUpdates, 0);
};
/** @private
*/
Grid_Grid.prototype._clearDataUpdates = function() {
	if(!this._dt || !this._hasNewUpdates) {
		return;
	}
	this._hasNewUpdates = false;
	let rowIds = this._dt.getAllRowIds(true); // Use shallow copy for speed
	for(let i = rowIds.length; --i >= 0;) {
		let rowData = this._dt.getRowData(rowIds[i]);
		let rowDef = rowData[ROW_DEF];
		rowDef.resetUpdates();
	}
};

/** @public
* @return {DataView}
*/
Grid_Grid.prototype.getDataView = function() {
	return this._dv;
};
/** @public
* @param {number} pageIndex
* @return {boolean}
*/
Grid_Grid.prototype.setPage = function(pageIndex) {
	return this._dv.setPage(pageIndex);
};
/** @public
* @param {number} size
*/
Grid_Grid.prototype.setPageSize = function(size) {
	this._dv.setPageSize(size);
};
/** @public
* @return {number}
*/
Grid_Grid.prototype.getPageIndex = function() {
	return this._dv.getPageIndex();
};
/** @public
* @return {number}
*/
Grid_Grid.prototype.getPageSize = function() {
	return this._dv.getPageSize();
};
/** @public
* @return {number}
*/
Grid_Grid.prototype.getPageCount = function() {
	return this._dv.getPageCount();
};

/** @description Toggle between expanded and collapsed chain index
 * @public
 * @param {Grid~RowReference} rowRef
 */
Grid_Grid.prototype.toggleChain = function(rowRef) {
	let rowDef = this._getRowDefinitionByRef(rowRef);
	if(rowDef) {
		rowDef.toggleChain();
	}
};
/**
 * @public
 * @param {Grid~RowReference} rowRef
 * @param {Array.<string>} fields
 * @return {boolean}
 */
Grid_Grid.prototype.setClassification = function(rowRef, fields) {
	let rowDef = this._getRowDefinitionByRef(rowRef);
	if(rowDef) {
		return this._dt.setSegmentClassification(rowDef.getRowId(), fields);
	}
	return false;
};

/** Check element in the grid element
 * @public
 * @param {Element} elem
 * @return {boolean}=true if an element from the parameter is inside a grid element
 */
Grid_Grid.prototype.contains = function(elem) {
	if(elem) {
		// This will impact the contents within the rt-grid element, but not those outside of it, such as the wrapper elements atlas-blotter and ef-grid.
		return this._topNode.contains(elem);
	}
	return false;
};

/** Check grid element is focused
 * @public
 * @return {boolean}
 */
Grid_Grid.prototype.isFocused = function() {
	let activeElement = document.activeElement;
	if(!activeElement || !activeElement.shadowRoot) {
		// active element is not in the shadow DOM. try using contains method to check
		return this.contains(activeElement);
	}
 	// For wrapper scenarios, such as atlas-blotter or efx-grid
	return activeElement.shadowRoot === this._topNode.parentNode;
};
/** @description Focus grid element without moving window scrollbar
 * @public
 */
Grid_Grid.prototype.focus = function() {
	this._grid.focus();
};
/** Request re-rendering of content row by triggering data binding event without any change in data model
 * @public
 */
Grid_Grid.prototype.requestRowRefresh = function() {
	this._grid.requestRowRefresh();
};
/** Force recalculation of grid's layout
 * @public
 */
Grid_Grid.prototype.updateLayout = function() {
	this._grid.updateLayout();
};
/** Get extension instance
* @public
* @param {string} str Extension name
* @return {*} Returns null, if the specified extension does not exist
*/
Grid_Grid.prototype.getExtension = function(str) {
	if(str) {
		return this._grid.getPlugin(str.replace(/Extension/ig, ""));
	}
	return null;
};

/** @private
* @param {Object} e
*/
Grid_Grid.prototype._onFormulaDataChanged = function(e) {
	if (this._formulaConflator.conflate()) {
		return;
	}

	// TODO: Check if we need to recompose the data in the data composed method
	this._recalculateFormulas(); // Force recalculation of user formula
	this._updateRowData(); // Re-render the data
};
/** @private
* @param {Object} e
*/
Grid_Grid.prototype._onFormulaDataRequired = function(e) {
	// if(obj["field"]) {
	// this._registerFieldToDataProvider(obj); TODO
	// if(this._dataRequiredTimerId) {
	// window.clearTimeout(this._dataRequiredTimerId);
	// }
	// this._dataRequiredTimerId = window.setTimeout( this._recalculateFormulas, 0); // Force recalculation of all rows
	// }
};

/** @private
* @param {Object} e
*/
Grid_Grid.prototype._onDataComposed = function(e) {
	let changes = e["changes"];
	if(!changes) {
		if(e["initialChanges"]) {
			changes = e["changes"] = e["initialChanges"]; // Need to keep e["changes"] for backward compatibility.
		} else {
			return; // Cannot do data composition if there is no change in data
		}
	}

	let rowData = e["rowData"];
	if(!rowData) {
		return; // Row could already be removed or global change event is sent
	}

	let rowId = e["rid"];
	let rowDef = e["rowDef"] || this._getRowDefinitionById(rowId);
	if(!rowDef) {
		rowDef = this._constituentMap ? this._constituentMap[rowId] : null; // Row def could be in pending for adding to view
		if(!rowDef) {
			return; // Somehow the given row id is invalid
		}
		if(rowDef.isDisposed()) {
			this._constituentMap[rowId] = null;
			return;
		}
	}

	if(this._autoDateConversion) { // auto data conversion
		let dataType;
		for(let field in changes) {
			let colDef = this._getColumnDefinition(field);
			if(colDef) {
				dataType = colDef.getDataType();
			} else {
				dataType = ColumnDefinition.getDataType(field);
			}

			if(dataType === "datetime") {
				let value = rowData[field];
				rowDef.setData(field + "_RAW", value);
				value = DateTime.toDateObject(value);
				rowDef.setData(field, value); // TODO: Check if this trigger data change multiple times
			}
		}
	}

	if(this.hasListener("dataComposed")) {
		e["grid"] = this;
		e["rowDef"] = rowDef;
		this._dispatch("dataComposed", e);
	}

	this._recalculateFormulas(e);
};
/** @private
* @param {Object} e
*/
Grid_Grid.prototype._onSubSegmentChanged = function(e) {
	let insertionList = /** @type{Array.<Segment>} */(e["insertionList"]);
	let removalList = /** @type{Array.<string>} */(e["removalList"]);
	let removedRows = /** @type{Object} */(e["removedRows"]);

	let i;
	let removalCount = removalList.length;
	let rowDef = null;
	for(i = 0; i < removalCount; i++) {
		let rid = removalList[i];
		let removedRow = removedRows[rid];
		if(removedRow) {
			rowDef = removedRow[ROW_DEF];
			if(rowDef) {
				rowDef.dispose();
				removedRow[ROW_DEF] = null;
			}
		}
	}

	let insertionCount = insertionList.length;
	for(i = 0; i < insertionCount; i++) {
		let segment = insertionList[i];
		// let parentId = segment.getParentId();
		let segmentId = segment.getId();
		rowDef = new RowDefinition({
			"segmentId": segmentId // WARNING: This could cause row id duplication
		});
		if(rowDef.setDataSource(this._dc)) { // auto generated row does not require a subscription
			this._hasNewUpdates = true; // Mark data table for cleaning it up later
		}
		rowDef.registerToView(this._dv);
	}
};

/** @private
* @param {Object=} e
*/
Grid_Grid.prototype._recalculateFormulas = function(e) {
	if(!this._fnEngine || this._fnEngine.getFormulaCount() <= 0) {
		return;
	}

	if(e && e["rid"]) {
		this._fnEngine.calculate(/** @type{Object.<string, *>} */ (e["rowData"]), /** @type{Object.<string, *>} */ (e["changes"]));
	} else {
		let rows = this._dc ? this._dc.getAllRowData() : null; // Slow

		let rowCount = rows ? rows.length : 0;
		for(let r = 0; r < rowCount; ++r) { // WARNING: This is very slow
			this._fnEngine.calculate(rows[r]);
		}
	}
};

/** @private
* @param {Object} e The updated data from snapshot filler when it request data from JET/RTK
*/
Grid_Grid.prototype._snapshotFillerDataChanged = function (e) {
	if (!this._dt) return;

	let data = e.data;
	if(e.timeSeries) {
		this._updateTimeSeriesFields(e); // Increase or decrease time series field
	} else {
		for (let ric in data) {
			this.setRicData(ric, data[ric]);
		}
	}

	if(!this._lastPollingRequest) { // This is the first time we receive successful ADC response
		this._lastPollingRequest = 1; // Allow polling to be started
	}
	this._startPolling();
};
/** @private
*/
Grid_Grid.prototype._startPolling = function () {
	if(this._pollingEnabled && this._pollingInterval) {
		if(this._lastPollingRequest && !this._pollingTimerId) {
			let now = new Date();
			this._lastPollingRequest = now.getTime();
			this._pollingTimerId = setTimeout(this._onPollingInterval, this._pollingInterval);
		}
	}
};
/** @private
*/
Grid_Grid.prototype._onPollingInterval = function () {
	this._pollingTimerId = 0;
	if(this._pollingEnabled && this._pollingInterval) {
		this.refreshADCData();
		this._startPolling();
	}
};
/** @public
* @param {boolean=} activated=true
*/
Grid_Grid.prototype.activateADCPolling = function (activated) {
	if(activated !== false) {
		this._pollingEnabled = true;
		// Make immediate request after re-activation if it has been a long time
		let now = new Date();
		if(this._lastPollingRequest) {
			let diff = now.getTime() - this._lastPollingRequest;
			if(diff > this._pollingInterval) {
				this.refreshADCData();
			}
		}
		this._startPolling();
	} else {
		this._pollingEnabled = false;
	}
};
/** Force re-requesting ADC data to immediately update the data
* @public
* @return {boolean} Return true, if a request has been made
*/
Grid_Grid.prototype.refreshADCData = function () {
	let dirty = this._snapshot.addFields(this._connector.getAllFields());
	if(dirty) {
		dirty = this._snapshot.addRics(this._connector.getAllRics());
		if(dirty) {
			return true;
		}
	}
	this._lastPollingRequest = 0; // No ADC field or RIC detected. Prevent polling from continuing
	return false;
};

/** @private
*/
Grid_Grid.prototype._onAutoLayoutUpdate = function() {
	let elem = this._topNode;

	let cw = elem.clientWidth;
	if(cw !== this._clientWidth) {
		this._clientWidth = cw;
		this.updateLayout();
	}
};

/** @description Log DataTable to console. By default, this will log only fields specified in the column definition. To see other column data use "colNames" options
 * @public
 * @param {Object=} opt_options { "colNames": Array.<string>, "rowCount": number, "startIndex": number, "getter": function(Object, string):* }
 */
Grid_Grid.prototype.logDT = function(opt_options) {
	let rowDefs = this._getAllRowDefinitions();
	this._logData(rowDefs, opt_options);
};
/** @description Log DataView to console. By default, this will log only fields specified in the column definition. To see other column data use "colNames" options
 * @public
 * @param {Object=} opt_options { "colNames": Array.<string>, "rowCount": number, "startIndex": number, "getter": function(Object, string):* }
 * @example
// Options have the following format:
// {
//    "colNames": Array.<string>,
//    "rowCount": number,
//    "startIndex": number,
//    "getter": function(rowData:Object, colName:string):*,
//    "withRowIds": boolean
// }
grid.logDV({"rowCount": 10}); // Showing 10 records
 */
Grid_Grid.prototype.logDV = function(opt_options) {
	let rowDefs = this.getRowDefinitions();
	this._logData(rowDefs, opt_options);
};
/** Log the collection of data to console
 * @private
 * @param {!Array.<RowDefinition>} rowDefs
 * @param {Object=} options
 */
Grid_Grid.prototype._logData = function(rowDefs, options) {
	if(!options) {
		options = {};
	}
	if(!options["colNames"]) {
		// TODO: Get all required fields and eliminate duplicate fields
		options["colNames"] = this.getColumnFields();
	}
	let tbl = DataCache.constructTable(rowDefs.map(toRowData), options);

	console.table(tbl); // eslint-disable-line
};

/** @public
* @description Replace existing row by a new row. Row Id is always changed, after the row is replaced.
* If the rowId of the new row is identical to that of the replacing row. Grid will do nothing because
* similar rowIds indicate that they are the same row.
* @param {Grid~RowReference} rowRef Reference (i.e. row index, row id, or row definition) of the insert position
* @param {(RowDefinition~Options|string)=} rowOption
* @returns {RowDefinition} Returns null, if the row is not replaced. Otherwise, a newly created row is returned
*/
Grid_Grid.prototype.replaceRow = function(rowRef, rowOption) {
	let insertPos = this._getRowId(rowRef);
	let rowDef = this._getRowDefinitionById(insertPos);

	if(rowDef) {
		if(rowDef.isAutoGenerated()) {
			return null; // Users cannot replace an auto-generated row
		}

		let configObj = rowDef.getConfigObject();
		if(rowDef.getRowId() === rowOption["rowId"]) {
			return null; // Cannot replace a row with the same row id
		}

		if(deepEqual(configObj, rowOption)){
			return null;
		}
	}

	let newRow = this.insertRow(rowOption, insertPos);
	if(rowDef) {
		this.removeRow(insertPos);
	}
	return newRow;
};

/** Scroll the view to the specified column. If the column is already in the view, nothing happens. If the column is outside of the view, the view will be moved to the column with some additional offsets
* @public
* @param {number} colIndex
* @param {boolean=} leftOfView Default is false. If true, the specified column will be put at the leftmost of the view (no offset)
* @returns {boolean} Return true, if there is any change
*/
Grid_Grid.prototype.scrollToColumn = function (colIndex, leftOfView) {
	return this._grid.scrollToColumn(colIndex, leftOfView);
};
/** Scroll the view to the specified row. If the row is already in the view, nothing happens. If the row is outside of the view, the view will be moved to the row with some additional offsets
* @public
* @param {number} rowIndex
* @param {boolean=} topOfView=false If true, the specified row will be put at the top of the view (no offset)
*/
Grid_Grid.prototype.scrollToRow = function (rowIndex, topOfView) {
	this._grid.scrollToRow("content", rowIndex, topOfView);
};

/** Get scroll value in pixel from the horizontal scrollbar
* @public
* @returns {number} pixels
*/
Grid_Grid.prototype.getScrollLeft = function () {
	return this._grid.getScrollLeft();
};
/** Get scroll value in pixel from the vertical scrollbar
* @public
* @returns {number} pixels
*/
Grid_Grid.prototype.getScrollTop = function () {
	return this._grid.getScrollTop();
};

/** Set scroll value to the horizontal scrollbar. This will move the scrollbar to specific position
* @public
* @param {number} pixels
* @see {@link Grid.scrollRight}
*/
Grid_Grid.prototype.setScrollLeft = function (pixels) {
	this._grid.setScrollLeft(pixels);
};
/** Set scroll value to the vertical scrollbar. This will move the scrollbar to specific position
* @public
* @param {number} pixels
* @see {@link Grid.scrollDown}
*/
Grid_Grid.prototype.setScrollTop = function (pixels) {
	this._grid.setScrollTop(pixels);
};

/** Scroll the view to the right by the specified value. Use negative value to scroll the view to the left
* @public
* @param {number} pixels
* @see {@link Grid.setScrollLeft}
*/
Grid_Grid.prototype.scrollRight = function (pixels) {
	this._grid.scrollRight(pixels);
};
/** Scroll the view down by the specified value. Use negative value to scroll the view to the top
* @public
* @param {number} pixels
* @see {@link Grid.setScrollTop}
*/
Grid_Grid.prototype.scrollDown = function (pixels) {
	this._grid.scrollDown(pixels);
};

/** Get width of scrollable area from the horizontal scrollbar. This is useful for determining the end of scrollbar
* @public
* @returns {number}
*/
Grid_Grid.prototype.getScrollWidth = function () {
	return this._grid.getScrollWidth();
};
/** Get height of scrollable area from the vertical scrollbar. This is useful for determining the end of scrollbar
* @public
* @returns {number}
*/
Grid_Grid.prototype.getScrollHeight = function () {
	return this._grid.getScrollHeight();
};

/** Get information about current grid's view and the vertical scrollbar
* @public
* @returns {Object} Returns null, if vertical scrollbar does not exists
*/
Grid_Grid.prototype.getVScrollView = function () {
	return this._grid.getVScrollView();
};

/** @private
* @param {Object} cell
* @param {Object} args
* @return {boolean}
*/
Grid_Grid.prototype._focusCell = function(cell, args) {
	if(cell) {
		let cellContent = cell.getContent();
		if(cellContent) {
			let nfe = null;
			if(this.hasListener("tabNavigation")) {
				let tabNavArg = {
					"shiftKey": args.shiftKey,
					"activeElement": args.activeElement,
					"cellContent": cellContent,
					"cell": cell,
					"colIndex": args.colIndex,
					"rowIndex": args.rowIndex,
					"field": args.fields ? args.fields[args.colIndex] : ""
				};
				this._dispatch("tabNavigation", tabNavArg);
				nfe = tabNavArg.nextFocusableElement;
			} else if(cellContent.tagName !== "SPAN") {
				nfe = cellContent;
			}

			if(nfe && nfe !== args.activeElement && !nfe.disabled) {
				nfe.focus();
				return true;
			}
		}
	}
	return false;
};
/** @private
*/
Grid_Grid.prototype._onVScroll = function() {
	let args = this._focusingArgs;
	if(args) {
		args.timeoutId = _clearTimeout(args.timeoutId);
		this._focusingArgs = null;
		let cell = this._grid.getCell("content", args.colIndex, args.rowIndex);
		if(!this._focusCell(cell, args)) {
			if(args.shiftKey) {
				this._focusPrevCellContent(args);
			} else {
				this._focusNextCellContent(args);
			}
		}
	}
};
/** @private
*/
Grid_Grid.prototype._onScrollTimeout = function() {
	this._focusingArgs = null;
};
/** @private
*/
Grid_Grid.prototype._selfScrollToRow = function() {
	let args = this._focusingArgs;
	if(args) {
		args.id = 0;
		this.scrollToRow(args.rowIndex);
	}
};
/** @private
* @param {Object} args
*/
Grid_Grid.prototype._requestScroll = function(args) {
	if(!this._focusingArgs) {
		this._focusingArgs = args;
		args.event = null; // The event is invalid after the scroll
		args.id = setTimeout(this._selfScrollToRow, 0); // Avoid event loop protection
		args.timeoutId = setTimeout(this._onScrollTimeout, 100); // To avoid a fail case where scroll cannot be performed
	}
};
/** @private
* @param {Object} args
*/
Grid_Grid.prototype._focusNextCellContent = function(args) {
	let colIndex = args.colIndex;
	let rowIndex = args.rowIndex;
	if(rowIndex < 0 || rowIndex == null) {
		return;
	}

	let focusableColIndices = args.focusableColIndices;
	let len = focusableColIndices.length;
	let startIdx = 0;
	let i;
	if(colIndex >= 0) {
		for(i = 1; i < len; i++) {
			if(colIndex < focusableColIndices[i]) {
				break;
			}
			startIdx = i;
		}
	}

	let grid = this._grid;
	let section = grid.getSection("content");
	let viewInfo = grid.getVerticalViewInfo();
	let lastFullRow = viewInfo.lastFullRow;
	let rowCount = this.getRowCount();

	args.fields = grid.getColumnFields();
	for(let r = rowIndex; r < rowCount; r++) {
		args.rowIndex = r;
		for(i = startIdx; i < len; i++) {
			let c = focusableColIndices[i];
			args.colIndex = c;
			if(r > lastFullRow) {
				_preventDefault(args.event);
				this._requestScroll(args);
				return;
			} else {
				let cell = section.getCell(c, r);
				if(this._focusCell(cell, args)) {
					_preventDefault(args.event);
					return;
				}
			}
		}
		startIdx = 0;
	}

	// The current focus on the last focusable content
	this._grid.getHiddenInput().focus();
};
/** @private
* @param {Object} args
*/
Grid_Grid.prototype._focusPrevCellContent = function(args) {
	let colIndex = args.colIndex;
	let rowIndex = args.rowIndex;
	if(rowIndex < 0 || rowIndex == null) {
		return;
	}

	let focusableColIndices = args.focusableColIndices;
	let len = focusableColIndices.length;
	let startIdx = len - 1;
	let i;
	if(colIndex >= 0) {
		for(i = len - 1; --i >= 0;) {
			if(colIndex > focusableColIndices[i]) {
				break;
			}
			startIdx = i;
		}
	}

	let grid = this._grid;
	let section = grid.getSection("content");
	let viewInfo = grid.getVerticalViewInfo();
	let firstFullRow = viewInfo.firstFullRow;

	args.fields = this.getColumnFields();
	for(let r = rowIndex; r >= 0; r--) {
		args.rowIndex = r;
		for(i = startIdx; i >= 0; i--) {
			let c = focusableColIndices[i];
			args.colIndex = c;
			if(r < firstFullRow) {
				_preventDefault(args.event);
				this._requestScroll(args);
				return;
			} else {
				let cell = section.getCell(c, r);
				if(this._focusCell(cell, args)) {
					_preventDefault(args.event);
					return;
				}
			}
		}
		startIdx = len - 1;
	}

	// The current focus on the last focusable content
	this._grid.getHiddenInput(true).focus();
};

/** @private
* @param {Object} e
*/
Grid_Grid.prototype._onTabNavigation = function(e) {
	if(this._focusingArgs) {
		return; // Cannot do another tab navigation while waiting for scrolling
	}

	let colDefs = this.getColumnDefinitions();
	let colCount = colDefs.length;

	let focusableColIndices = [];
	for(let c = 0; c < colCount; c++) {
		if(colDefs[c].isFocusable()) {
			focusableColIndices.push(c);
		}
	}

	if(!focusableColIndices.length) {
		return;
	}

	let keyEvt = e.event;
	let pos = this.getRelativePosition(keyEvt);
	let startingRowIndex = pos["rowIndex"];
	if(e.onTheEdge) {
		let viewInfo = this._grid.getVerticalViewInfo();
		startingRowIndex = keyEvt.shiftKey ? viewInfo.lastFullRow : viewInfo.firstFullRow;
	}
	let args = {
		event: keyEvt,
		shiftKey: keyEvt.shiftKey,
		colIndex: pos["colIndex"],
		rowIndex: startingRowIndex,
		focusableColIndices: focusableColIndices,
		activeElement: e.activeElement
	};

	if(keyEvt.shiftKey) {
		this._focusPrevCellContent(args);
	} else {
		this._focusNextCellContent(args);
	}
};

/** @public
* @ignore
* @return {!Object}
*/
Grid_Grid.prototype._getEventHandlers = function() {
	return {
		"tabNavigation": this._onTabNavigation,
		"q2DataChanged": this._onQ2DataChanged
	};
};


/* harmony default export */ const js_Grid = ((/* unused pure expression or super */ null && (Grid_Grid)));

;// ./node_modules/tr-grid-util/es6/jet/DataSet.js
/**
* @namespace
*/
let DataSet = {};

DataSet.texts = [
	"If a dog and cat had a baby together that grew up and worked a desk job he'd be a Cog in the machine",
	"I'm still upset that Tie Domi didn't name his child Tyson",
	"A tagline for a car company that prides itself on its morals and ethics: Take the High Road",
	"Logan Broger is amazing and a wonderful boy according to Logan's mom",
	"You know the Grammys are a joke when Future doesn't win Best Everything",
	"If you were a member of the Bloods and became paralyzed do you then become a member of the Crips?",
	"Curling is the best sport named after something you do to your hair",
	"You say potatoe, I say starchy carbs",
	"If you work for an ad agency and getting paid for it aren't you the one who is being influenced by advertising?",
	"Are there Out of Stock photos? Gafuffle",
	"I think of a lot of good ideas when going to the bathroom I guess I have a real stream of consciousness",
	"Most streets are two way streets",
	"Why does that make love so special?",
	"A tagline for an airline: Take the High Road",
	"Rumour has it targeted online advertising was developed because the internet was upset that you could read it but it couldn't read you",
	"This is a true fact: I never had a fear of heights until I fell off a roof",
	"A tagline for a special highway that is easy to navigate while under the influence of drugs: Take the High Road",
	"I'm the only person in the world with my name",
	"We need more werkin and less twerkin if you ask me",
	"If Fantasy Hockey actually lived up to its name, every team would have Henrik Lundqvist and Joffrey Lupul on it",
	"I'm in a band that does Metallica covers with our private parts it's called Myphallica",
	"Smiling could easily be misinterpreted for showing your teeth to someone because they said something that made you happy",
	"We say we are walking the dog, but the dog always leads",
	"Tim Horton was a hockey player but is the name of a coffee chain, which means my dream of a goat sanctuary being my legacy is not unrealistic",
	"I started a sensory deprivation chamber business it involves really dark curtains, ear plugs, and a sleeping mask",
	"Do we make money or does money make us? Chezwich",
	"Pantone is a colour but also the singular version of pants",
	"Logan Ipsum will loop at some point",
	"Thank you for using my words in your work",
	"You should listen to my mixtape (check out the rest of my portfolio)",
	"I have never known a Jack that was in good enough shape to name bodybuilding after him",
	"Twitter is the rice of social media",
	"Injuries always keep you OUT of things",
	"For the name of an act as serious as killing someone, assassination literally translates to buttbuttination",
	"Why don't we call glasses duocles",
	"If you wake up with a giant zit, you are really facing your fears when you look in the mirror",
	"Cemeteries are just garbage dumps filled with humans",
	"I have a moral code, but I haven't figured out how to read it yet",
	"I bet most serial killers play the drums",
	"To Catch A Predator would have been a great name for a Steve Irwin show",
	"If the word kerning is kerned poorly, it kind of looks like learning which is appropriate because both are important",
	"North America should be called Russia since people are always moving so fast",
	"I don't need a big house, just a two floor condo you could say I have lofty expectations",
	"Even faithful and simple Sancho is forced to deceive him at certain points",
	"The novel is considered a satire of orthodoxy, veracity and even nationalism",
	"In exploring the individualism of his characters, Cervantes helped move beyond the narrow literary conventions of the chivalric romance literature that he spoofed",
	"which consists of straightforward retelling of a series of acts that redound to the knightly virtues of the hero",
	"The character of Don Quixote became so well known in its time that the word quixotic was quickly adopted by many languages",
	"Characters such as Sancho Panza and Don Quixote's steed, Rocinante, are emblems of Western literary culture",
	"It stands in a unique position between medieval chivalric romance and the modern novel",
	"The former consist of disconnected stories featuring the same characters and settings with little exploration of the inner life of even the main character"
];

DataSet.words = DataSet.texts.join(" ").split(" ").filter(function(w) { return w.length > 1; });

DataSet.companyName = [
	"Yost-Prohaska",
	"Wehner, Howe and Brown",
	"Gusikowski-Legros",
	"Wolff-Windler",
	"McGlynn, Douglas and Kilback",
	"MacGyver Group",
	"Spinka, Stracke and Huel",
	"Howell, Wolff and Emmerich",
	"Blick-O'Reilly",
	"Dibbert, Buckridge and Leffler",
	"Skiles, Kuhlman and Johns",
	"Daugherty Group",
	"Reilly, Toy and Bayer",
	"Bradtke, Mohr and Weber",
	"Thompson, Bruen and Friesen",
	"Rempel-Bednar",
	"Kunze LLC",
	"Pfannerstill Group",
	"Moen-Larkin",
	"Larson LLC",
	"Haag-Feest",
	"Hauck-Bartell",
	"Rohan-Kohler",
	"Waters, Homenick and Pagac",
	"Ziemann Group",
	"Reynolds, Turcotte and Bosco",
	"Graham, Morar and Waters",
	"Cassin Inc",
	"Bosco-Terry",
	"Stanton-Blick",
	"Prohaska and Sons",
	"Pouros, Daugherty and Pfannerstill",
	"Bogan-Cormier",
	"O'Kon-Mills",
	"Mueller-Schmeler",
	"Runte-Trantow",
	"Muller Inc",
	"Shanahan, Ryan and Orn",
	"Huel LLC",
	"Shanahan-Zulauf",
	"Emard-Schuster",
	"Muller Inc",
	"Collier, Muller and Yundt",
	"Wehner, Prohaska and Harber",
	"Ondricka-Stehr",
	"Towne LLC",
	"Feil Inc",
	"Braun, Kihn and Treutel",
	"Ebert-MacGyver",
	"Harris LLC"
];

DataSet.industry = [
	"Specialty Chemicals",
	"Office Equipment/Supplies/Services",
	"Electric Utilities: Central",
	"Trucking Freight/Courier Services",
	"Auto Parts:O.E.M.",
	"Oil Refining/Marketing",
	"Biotechnology: Biological Products (No Diagnostic Substances)",
	"Computer Software: Programming, Data Processing",
	"Farming/Seeds/Milling",
	"Biotechnology: Laboratory Analytical Instruments",
	"Finance: Consumer Services",
	"Major Banks",
	"Investment Managers",
	"Oilfield Services/Equipment",
	"Paper",
	"Transportation Services"
];

DataSet.market = [
	"NYSE",
	"NASDAQ",
	"SET",
	"DJI",
	"HSCEI",
	"HKEX",
	"NIKKEI"
];

DataSet.country = [
	{"name": "Afghanistan", "code": "AF"},
	{"name": "land Islands", "code": "AX"},
	{"name": "Albania", "code": "AL"},
	{"name": "Algeria", "code": "DZ"},
	{"name": "American Samoa", "code": "AS"},
	{"name": "AndorrA", "code": "AD"},
	{"name": "Angola", "code": "AO"},
	{"name": "Anguilla", "code": "AI"},
	{"name": "Antarctica", "code": "AQ"},
	{"name": "Antigua and Barbuda", "code": "AG"},
	{"name": "Argentina", "code": "AR"},
	{"name": "Armenia", "code": "AM"},
	{"name": "Aruba", "code": "AW"},
	{"name": "Australia", "code": "AU"},
	{"name": "Austria", "code": "AT"},
	{"name": "Azerbaijan", "code": "AZ"},
	{"name": "Bahamas", "code": "BS"},
	{"name": "Bahrain", "code": "BH"},
	{"name": "Bangladesh", "code": "BD"},
	{"name": "Barbados", "code": "BB"},
	{"name": "Belarus", "code": "BY"},
	{"name": "Belgium", "code": "BE"},
	{"name": "Belize", "code": "BZ"},
	{"name": "Benin", "code": "BJ"},
	{"name": "Bermuda", "code": "BM"},
	{"name": "Bhutan", "code": "BT"},
	{"name": "Bolivia", "code": "BO"},
	{"name": "Bosnia and Herzegovina", "code": "BA"},
	{"name": "Botswana", "code": "BW"},
	{"name": "Bouvet Island", "code": "BV"},
	{"name": "Brazil", "code": "BR"},
	{"name": "British Indian Ocean Territory", "code": "IO"},
	{"name": "Brunei Darussalam", "code": "BN"},
	{"name": "Bulgaria", "code": "BG"},
	{"name": "Burkina Faso", "code": "BF"},
	{"name": "Burundi", "code": "BI"},
	{"name": "Cambodia", "code": "KH"},
	{"name": "Cameroon", "code": "CM"},
	{"name": "Canada", "code": "CA"},
	{"name": "Cape Verde", "code": "CV"},
	{"name": "Cayman Islands", "code": "KY"},
	{"name": "Central African Republic", "code": "CF"},
	{"name": "Chad", "code": "TD"},
	{"name": "Chile", "code": "CL"},
	{"name": "China", "code": "CN"},
	{"name": "Christmas Island", "code": "CX"},
	{"name": "Cocos (Keeling) Islands", "code": "CC"},
	{"name": "Colombia", "code": "CO"},
	{"name": "Comoros", "code": "KM"},
	{"name": "Congo", "code": "CG"},
	{"name": "Congo, The Democratic Republic of the", "code": "CD"},
	{"name": "Cook Islands", "code": "CK"},
	{"name": "Costa Rica", "code": "CR"},
	{"name": "Cote D'Ivoire", "code": "CI"},
	{"name": "Croatia", "code": "HR"},
	{"name": "Cuba", "code": "CU"},
	{"name": "Cyprus", "code": "CY"},
	{"name": "Czech Republic", "code": "CZ"},
	{"name": "Denmark", "code": "DK"},
	{"name": "Djibouti", "code": "DJ"},
	{"name": "Dominica", "code": "DM"},
	{"name": "Dominican Republic", "code": "DO"},
	{"name": "Ecuador", "code": "EC"},
	{"name": "Egypt", "code": "EG"},
	{"name": "El Salvador", "code": "SV"},
	{"name": "Equatorial Guinea", "code": "GQ"},
	{"name": "Eritrea", "code": "ER"},
	{"name": "Estonia", "code": "EE"},
	{"name": "Ethiopia", "code": "ET"},
	{"name": "Falkland Islands (Malvinas)", "code": "FK"},
	{"name": "Faroe Islands", "code": "FO"},
	{"name": "Fiji", "code": "FJ"},
	{"name": "Finland", "code": "FI"},
	{"name": "France", "code": "FR"},
	{"name": "French Guiana", "code": "GF"},
	{"name": "French Polynesia", "code": "PF"},
	{"name": "French Southern Territories", "code": "TF"},
	{"name": "Gabon", "code": "GA"},
	{"name": "Gambia", "code": "GM"},
	{"name": "Georgia", "code": "GE"},
	{"name": "Germany", "code": "DE"},
	{"name": "Ghana", "code": "GH"},
	{"name": "Gibraltar", "code": "GI"},
	{"name": "Greece", "code": "GR"},
	{"name": "Greenland", "code": "GL"},
	{"name": "Grenada", "code": "GD"},
	{"name": "Guadeloupe", "code": "GP"},
	{"name": "Guam", "code": "GU"},
	{"name": "Guatemala", "code": "GT"},
	{"name": "Guernsey", "code": "GG"},
	{"name": "Guinea", "code": "GN"},
	{"name": "Guinea-Bissau", "code": "GW"},
	{"name": "Guyana", "code": "GY"},
	{"name": "Haiti", "code": "HT"},
	{"name": "Heard Island and Mcdonald Islands", "code": "HM"},
	{"name": "Holy See (Vatican City State)", "code": "VA"},
	{"name": "Honduras", "code": "HN"},
	{"name": "Hong Kong", "code": "HK"},
	{"name": "Hungary", "code": "HU"},
	{"name": "Iceland", "code": "IS"},
	{"name": "India", "code": "IN"},
	{"name": "Indonesia", "code": "ID"},
	{"name": "Iran, Islamic Republic Of", "code": "IR"},
	{"name": "Iraq", "code": "IQ"},
	{"name": "Ireland", "code": "IE"},
	{"name": "Isle of Man", "code": "IM"},
	{"name": "Israel", "code": "IL"},
	{"name": "Italy", "code": "IT"},
	{"name": "Jamaica", "code": "JM"},
	{"name": "Japan", "code": "JP"},
	{"name": "Jersey", "code": "JE"},
	{"name": "Jordan", "code": "JO"},
	{"name": "Kazakhstan", "code": "KZ"},
	{"name": "Kenya", "code": "KE"},
	{"name": "Kiribati", "code": "KI"},
	{"name": "Korea, Democratic People'S Republic of", "code": "KP"},
	{"name": "Korea, Republic of", "code": "KR"},
	{"name": "Kuwait", "code": "KW"},
	{"name": "Kyrgyzstan", "code": "KG"},
	{"name": "Lao People'S Democratic Republic", "code": "LA"},
	{"name": "Latvia", "code": "LV"},
	{"name": "Lebanon", "code": "LB"},
	{"name": "Lesotho", "code": "LS"},
	{"name": "Liberia", "code": "LR"},
	{"name": "Libyan Arab Jamahiriya", "code": "LY"},
	{"name": "Liechtenstein", "code": "LI"},
	{"name": "Lithuania", "code": "LT"},
	{"name": "Luxembourg", "code": "LU"},
	{"name": "Macao", "code": "MO"},
	{"name": "Macedonia, The Former Yugoslav Republic of", "code": "MK"},
	{"name": "Madagascar", "code": "MG"},
	{"name": "Malawi", "code": "MW"},
	{"name": "Malaysia", "code": "MY"},
	{"name": "Maldives", "code": "MV"},
	{"name": "Mali", "code": "ML"},
	{"name": "Malta", "code": "MT"},
	{"name": "Marshall Islands", "code": "MH"},
	{"name": "Martinique", "code": "MQ"},
	{"name": "Mauritania", "code": "MR"},
	{"name": "Mauritius", "code": "MU"},
	{"name": "Mayotte", "code": "YT"},
	{"name": "Mexico", "code": "MX"},
	{"name": "Micronesia, Federated States of", "code": "FM"},
	{"name": "Moldova, Republic of", "code": "MD"},
	{"name": "Monaco", "code": "MC"},
	{"name": "Mongolia", "code": "MN"},
	{"name": "Montenegro", "code": "ME"},
	{"name": "Montserrat", "code": "MS"},
	{"name": "Morocco", "code": "MA"},
	{"name": "Mozambique", "code": "MZ"},
	{"name": "Myanmar", "code": "MM"},
	{"name": "Namibia", "code": "NA"},
	{"name": "Nauru", "code": "NR"},
	{"name": "Nepal", "code": "NP"},
	{"name": "Netherlands", "code": "NL"},
	{"name": "Netherlands Antilles", "code": "AN"},
	{"name": "New Caledonia", "code": "NC"},
	{"name": "New Zealand", "code": "NZ"},
	{"name": "Nicaragua", "code": "NI"},
	{"name": "Niger", "code": "NE"},
	{"name": "Nigeria", "code": "NG"},
	{"name": "Niue", "code": "NU"},
	{"name": "Norfolk Island", "code": "NF"},
	{"name": "Northern Mariana Islands", "code": "MP"},
	{"name": "Norway", "code": "NO"},
	{"name": "Oman", "code": "OM"},
	{"name": "Pakistan", "code": "PK"},
	{"name": "Palau", "code": "PW"},
	{"name": "Palestinian Territory, Occupied", "code": "PS"},
	{"name": "Panama", "code": "PA"},
	{"name": "Papua New Guinea", "code": "PG"},
	{"name": "Paraguay", "code": "PY"},
	{"name": "Peru", "code": "PE"},
	{"name": "Philippines", "code": "PH"},
	{"name": "Pitcairn", "code": "PN"},
	{"name": "Poland", "code": "PL"},
	{"name": "Portugal", "code": "PT"},
	{"name": "Puerto Rico", "code": "PR"},
	{"name": "Qatar", "code": "QA"},
	{"name": "Reunion", "code": "RE"},
	{"name": "Romania", "code": "RO"},
	{"name": "Russian Federation", "code": "RU"},
	{"name": "RWANDA", "code": "RW"},
	{"name": "Saint Helena", "code": "SH"},
	{"name": "Saint Kitts and Nevis", "code": "KN"},
	{"name": "Saint Lucia", "code": "LC"},
	{"name": "Saint Pierre and Miquelon", "code": "PM"},
	{"name": "Saint Vincent and the Grenadines", "code": "VC"},
	{"name": "Samoa", "code": "WS"},
	{"name": "San Marino", "code": "SM"},
	{"name": "Sao Tome and Principe", "code": "ST"},
	{"name": "Saudi Arabia", "code": "SA"},
	{"name": "Senegal", "code": "SN"},
	{"name": "Serbia", "code": "RS"},
	{"name": "Seychelles", "code": "SC"},
	{"name": "Sierra Leone", "code": "SL"},
	{"name": "Singapore", "code": "SG"},
	{"name": "Slovakia", "code": "SK"},
	{"name": "Slovenia", "code": "SI"},
	{"name": "Solomon Islands", "code": "SB"},
	{"name": "Somalia", "code": "SO"},
	{"name": "South Africa", "code": "ZA"},
	{"name": "South Georgia and the South Sandwich Islands", "code": "GS"},
	{"name": "Spain", "code": "ES"},
	{"name": "Sri Lanka", "code": "LK"},
	{"name": "Sudan", "code": "SD"},
	{"name": "Suriname", "code": "SR"},
	{"name": "Svalbard and Jan Mayen", "code": "SJ"},
	{"name": "Swaziland", "code": "SZ"},
	{"name": "Sweden", "code": "SE"},
	{"name": "Switzerland", "code": "CH"},
	{"name": "Syrian Arab Republic", "code": "SY"},
	{"name": "Taiwan, Province of China", "code": "TW"},
	{"name": "Tajikistan", "code": "TJ"},
	{"name": "Tanzania, United Republic of", "code": "TZ"},
	{"name": "Thailand", "code": "TH"},
	{"name": "Timor-Leste", "code": "TL"},
	{"name": "Togo", "code": "TG"},
	{"name": "Tokelau", "code": "TK"},
	{"name": "Tonga", "code": "TO"},
	{"name": "Trinidad and Tobago", "code": "TT"},
	{"name": "Tunisia", "code": "TN"},
	{"name": "Turkey", "code": "TR"},
	{"name": "Turkmenistan", "code": "TM"},
	{"name": "Turks and Caicos Islands", "code": "TC"},
	{"name": "Tuvalu", "code": "TV"},
	{"name": "Uganda", "code": "UG"},
	{"name": "Ukraine", "code": "UA"},
	{"name": "United Arab Emirates", "code": "AE"},
	{"name": "United Kingdom", "code": "GB"},
	{"name": "United States", "code": "US"},
	{"name": "United States Minor Outlying Islands", "code": "UM"},
	{"name": "Uruguay", "code": "UY"},
	{"name": "Uzbekistan", "code": "UZ"},
	{"name": "Vanuatu", "code": "VU"},
	{"name": "Venezuela", "code": "VE"},
	{"name": "Viet Nam", "code": "VN"},
	{"name": "Virgin Islands, British", "code": "VG"},
	{"name": "Virgin Islands, U.S.", "code": "VI"},
	{"name": "Wallis and Futuna", "code": "WF"},
	{"name": "Western Sahara", "code": "EH"},
	{"name": "Yemen", "code": "YE"},
	{"name": "Zambia", "code": "ZM"},
	{"name": "Zimbabwe", "code": "ZW"}
];

/* harmony default export */ const jet_DataSet = (DataSet);


;// ./node_modules/tr-grid-util/es6/jet/DataGenerator.js


/** @typedef {Object} DataGenerator~DataOptions
* @description Options for generating data
* @property {number=} seed Seed for randomization. If seed is not specified (default), new set of data will be generated every time. Otherwise, the same set of data will be generated.
* @property {number=} numRows Number of rows to be generated
* @property {number=} rowCount Alias to numRows
*/

/** @typedef {Object} DataGenerator~FieldInformation
* @description Information object for defining how data are generated for a specific field
* @property {string=} type Available types are number, float, boolean, set, function, and isoDate
* @property {string=} prefix Add a text prefix to the data
* @property {string=} suffix Add a text suffix to the data
* @property {number=} min Minimum value of the numeric data
* @property {number=} max Maximum value of the numeric data. This is exclusive (not included in the result).
* @property {string=} prec Precision (place number after the decimal point) of the numeric data.
* @property {string=} fixedValue Single/static/constant value
* @property {Array=} members List of possible data when the type is "set"
* @property {Function=} generate Function for generating data when the type is "function"
*/

/** @private
* @param {Object} fInfo
* @param {number=} seed
* @return {number}
*/
let _generateId = function(fInfo, seed) {
	if(seed == null) {
		seed = fInfo._id != null ? fInfo._id : -1;
		fInfo._id = ++seed;
	}
	return seed || 0;
};
/** @private
* @param {Object} fInfo
* @param {number=} seed
* @return {Date}
*/
let _generateDate1 = function(fInfo, seed) {
	if(seed == null) {
		return randDate(6e11, 16e11, seed);
	}
	return new Date(1e12 + 1e9 * seed);
};

let POW10 = [1, 1e1, 1e2, 1e3, 1e4, 1e5, 1e6, 1e7, 1e8, 1e9, 1e10];

/** @type {Object.<string, DataGenerator~DataOptions>}
* @private
*/
let _fieldInfo = {
	"X_RIC_NAME": {type: "string"},
	"CF_NAME": {type: "string", prefix: true, suffix: " Name"},
	"CF_LAST": {type: "number", min: 0.01, max: 1000, prec: 2},
	"PCTCHNG": {type: "number", min: -1, max: 1, prec: 2, suffix: "%"},
	"PCTCHNG2": {type: "float", min: -20, max: 20, prec: 2, suffix: "%"},
	"CF_CURR": {type: "set", members: ["USD", "GBP", "EUR", "JPY", "THB"]},
	"INDICATOR": {type: "number", min: 0, max: 5},
	"CF_TICK": {type: "number", min: 1, max: 3, changeOnly: true},
	"PRCTCK_1": {type: "set", members: ["\u21e7", "B\u21e7", "\u2191", "B\u2191", "26", "1", "\"1\"", "-1", "\u21e9", "B\u21e9", "\u2193", "B\u2193", "27", "2", "\"2\""] },
	"CF_NETCHNG": {type: "number", min: -100, max: 100},
	"CF_VOLUME": {type: "number", min: 0, max: 1e7},
	"CHILD_ORDER": {type: "set", ignore: true, members: [undefined]}, // eslint-disable-line
	"RI_FLAG": {type: "set", members: ["R", "Q", "I", "_", ""]},
	"RI_CODE": {type: "number", min: 0, max: 2},
	"STATUS": {type: "number", min: 0, max: 6},
	"YRLOW": {type: "number", min: 0, max: 20},
	"YRHIGH": {type: "number", min: 200, max: 2000},
	"CFLOW": {type: "number", min: 0, max: 20},
	"CFHIGH": {type: "number", min: 200, max: 2000},
	"LOW_YLD": {type: "set", members: [undefined, 0, 1, 2, 3, 4, 5, 10, 11, 15, 20]}, // eslint-disable-line
	"HIGH_YLD": {type: "set", members: [undefined, 200, 300, 400, 500, 1000, 1500, 2000]}, // eslint-disable-line
	"CF_YIELD": {type: "set", members: [undefined, 0, 10, 20, 30, 40, 50, 100, 200, 300, 400, 500]}, // eslint-disable-line
	"TR.IPODate": {type: "isoDate", min: Date.now() - 63072000000 /* 2 years back from now */, max: Date.now()},
	"TR.Volume": {type: "number", min: 2000, max: 1e7},
	"TR.Price52WeekHigh": {type: "number", min: 100, max: 300, prec: 2},
	"TR.Price52WeekLow": {type: "number", min: 1, max: 200, prec: 2 },
	"TR.InvalidField": {type: "set", members: [undefined, null, NaN]}, // eslint-disable-line
	"falsyValues": {type: "set", members: [undefined, null, NaN, 0, false, ""]}, // eslint-disable-line
	"percent": {type: "number", min: 0, max: 101 },
	"number_1": {type: "number", min: 1, max: 100 },
	"number_2": {type: "number", min: -100, max: 100 },
	"index4": {type: "number", min: 0, max: 4 },
	"index5": {type: "number", min: 0, max: 5 },
	"index10": {type: "number", min: 0, max: 10 },
	"index100": {type: "number", min: 0, max: 100 },
	"group3": {type: "set", members: ["Group_A", "Group_B", "Group_C"]},
	"group4": {type: "set", members: ["Group_1", "Group_2", "Group_3", "Group_4"]},
	"float_1": {type: "float", min: 0, max: 1, prec: 2 },
	"float_2": {type: "float", min: 0, max: 100, prec: 2 },
	"float_3": {type: "float", min: -1, max: 1, prec: 2 },
	"precision4": {type: "float", min: -10, max: 10, prec: 4 },
	"precision8": {type: "float", min: -100, max: 100, prec: 8 },
	"boolean": {type: "boolean"},
	"date": {type: "date", min: 15e11, max: 16e11 },
	"date1": {type: "function", generate: _generateDate1 },
	"ISODate": {type: "isoDateObject", min: Date.now() - 63072000000 /* 2 years back from now */, max: Date.now()},
	"word": {type: "string", min: 1, max: 1},
	"phrase": {type: "string", min: 2, max: 5},
	"words": {type: "string", min: 1, max: 20},
	"words1": {type: "string", min: 2, max: 8},
	"words2": {type: "string", min: 3, max: 10},
	"words3": {type: "string", min: 5, max: 15},
	"sentence": {type: "string", min: 8, max: 20},
	"id": {type: "function", hash: 0, generate: _generateId},
	"companyName": {type: "set", members: jet_DataSet.companyName },
	"industry": {type: "set", members: jet_DataSet.industry },
	"market": {type: "set", members: jet_DataSet.market },
	"country": {type: "set", members: jet_DataSet.country }
};

/** @public
* @param {string} field
* @return {!Object}
*/
let getFieldInfo = function(field) {
	return _fieldInfo[field] || {};
};

/** @public
* @param {string} field
* @param {DataGenerator~FieldInformation|Function} options
*/
let addFieldInfo = function(field, options) {
	if(field) {
		let opt = options;
		if(typeof options === "function") {
			opt = {
				type: "function",
				generate: options
			};
		}
		_fieldInfo[field] = opt; // WARNING: This could replace existing info
	}
};

/** Return pseudo random number in the range of 0 to 1 (exclusive of 1)
* @public
* @param {number} seed
* @return {number}
*/
let pseudoRandNumber = function(seed) {
	let rand = Math.sin(seed * 113) * 10000;
	rand *= rand;
	return rand - (rand | 0);
};

/** @private
* @param {*=} seed
* @return {number}
*/
let _getRandomNumber = function(seed) {
	return (typeof seed === "number") ? pseudoRandNumber(seed) : Math.random();
};

/** Return a floating-point random number in the range min - max (inclusive of min, but not max) with prec digits of precision.
* @public
* @param {number=} min=0
* @param {number=} max=100 Exclusive value
* @param {(number|null)=} prec
* @param {number=} seed
* @return {number}
*/
let randNumber = function(min, max, prec, seed) {
	if(!min) { min = 0; }
	if(max == null) { max = 100; }
	let v = (max - min) * _getRandomNumber(seed) + min;

	if(prec > 0) {
		let pow = POW10[prec];
		return Math.floor(v * pow) / pow;
	} else if(prec != null) {
		return Math.floor(v);
	} else {
		return v;
	}
};

/** Return an interger random number in the range min - max (inclusive of min and max).
* @public
* @param {number} min
* @param {number} max Inclusive value
* @param {number=} seed
* @return {number}
*/
let randInt = function(min, max, seed) {
	return randNumber(min, max + 1, 0, seed);
};

/** Return a integer random number in the range min - max (inclusive of min, but not max).
* @public
* @param {number=} min
* @param {number=} max Exclusive value
* @param {number=} seed
* @return {number}
*/
let randBetween = function(min, max, seed) {
	return randNumber(min, max, 0, seed);
};

/** Return a integer random number in the range 0 - max (exclusive of max). It is usefull in some cases for example, <br>
* if you want to get a random member of an array, <br>
* just get a random index via randIndex(ary.length).
* @public
* @param {number} max Exclusive value
* @param {number=} seed
* @return {number}
*/
let randIndex = function(max, seed) {
	return randBetween(0, max, seed);
};

/** @public
* @param {number=} seed
* @return {boolean}
*/
let randBoolean = function(seed) {
	return randInt(0, 1, seed) === 0;
};

/** Return a random member of set or single character in the range of A to Z
* @public
* @param {Array} set
* @param {number=} seed
* @return {*}
*/
let randMember = function(set, seed) {
	var len = set ? (set.length || 0) : 0;
	if(len) {
		if(len === 1) {
			return set[0];
		}
		let index = randIndex(len, seed);
		return set[index];
	}
	return String.fromCharCode(randInt(65, 90, seed));
};

/** @public
* @param {(number|Object)=} min minimum timestamp or date object
* @param {(number|Object)=} max minimum timestamp or date object
* @param {number=} seed
* @return {Date}
*/
let randDate = function(min, max, seed) {
	min = min || (Date.now() - 63072000000); //2 years back from now
	if(min instanceof Date) {
		min = min.getTime();
	}

	max = max || Date.now();
	if(max instanceof Date) {
		max = max.getTime();
	}

	let ts = randInt(0, (max - min) / 1000, seed);
	return new Date(min + (ts * 1000));
};

/** @public
* @param {number=} min
* @param {number=} max Exclusive value
* @param {number=} seed
* @return {string}
*/
let randString = function(min, max, seed) {
	let wordCount = (min < max) ? randBetween(min, max, seed) : min;

	let ary = new Array(wordCount);
	for(let i = 0; i < wordCount; ++i) {
		if(seed != null) {
			seed *= 1.17 * (i + 1);
		}
		let idx = randIndex(jet_DataSet.words.length, seed);
		ary[i] = jet_DataSet.words[idx];
	}
	return ary.join(" ");
};


/** Convert 2D Array to Array of records
* @public
* @param {Array.<Array>} data2D Array of values
* @param {Array.<string|number>=} fields Keys to be mapped on the output records.
* @return {Array.<Object>} records
*/
let toRecords = function(data2D, fields) {
	if(!Array.isArray(data2D)) {
		return null;
	}
	if(!Array.isArray(fields)) {
		if(fields) {
			fields = [fields + ""];
		} else {
			fields = [];
		}
	}
	let len = data2D.length;
	let records = new Array(len);
	for(let i = 0; i < len; ++i) {
		let record = records[i] = {};
		let columns = data2D[i];
		let jLen = columns ? (columns.length || 0) : 0;
		for(let j = 0; j < jLen; ++j) {
			let value = columns[j];
			if(value !== undefined) { // eslint-disable-line
				let key = fields[j];
				if(!key && key !== 0) {
					key = j + "";
				}
				record[key] = value;
			}
		}
	}
	return records;
};

/** @public
* @return {number}
*/
let getSeed = function() {
	return DataGenerator.seed;
};

/** Generate 2 dimensional array of data from the specified field and options. Note that 2D array data structure is not recommended for usage due to the position of fields/columns can be changed at runtime.
* @public
* @param {string|Array.<string>} fields
* @param {DataGenerator~DataOptions=} options
* @return {!Array.<Array>} 2D Array of data
*/
let generate = function(fields, options) {
	let result = _generate2DArray(fields, options, DataGenerator.seed);
	DataGenerator.seed = result.seed;

	return result.data;
};

/** WARNING: This method does not modify global seed
* @public
* @param {string|Array.<string>} fields
* @param {(number|DataGenerator~DataOptions)=} options Configuration object or seed number
* @param {number=} seed
* @return {!Object} Object with the given fields as its keys
*/
let generateRecord = function(fields, options, seed) {
	if(!Array.isArray(fields)) {
		fields = [fields + ""];
	}
	let config = {};
	if(options != null) {
		if(typeof options === "number") {
			seed = options;
		} else if(options) {
			config = options;
		}
	}

	let fieldCount = fields.length;
	let record = {};
	for(let i = 0; i < fieldCount; ++i) {
		let field = fields[i];
		record[field] = _generateFieldData(field, config, seed).value;
	}
	return record;
};

/** @public
* @param {string|Array.<string>} fields
* @param {(number|DataGenerator~DataOptions)=} options Configuration object or number of rows
* @return {!Array.<Object>} Array of object with the given fields as its keys
*/
let generateRecords = function(fields, options) {
	let result = _generate2DArray(fields, options, DataGenerator.seed);
	DataGenerator.seed = result.seed;
	fields = result.fields;
	let fieldCount = fields.length;
	let records = result.data;
	for(let i = records.length; --i >= 0;) {
		let ary = records[i];
		let record = {};
		for(let j = 0; j < fieldCount; ++j) {
			let field = fields[j];
			record[field] = ary[j];
		}
		records[i] = record;
	}
	return records;
};

/** WARNING: This method does not modify global seed
* @public
* @param {string} field
* @param {DataGenerator~DataOptions=} options
* @param {number=} seed
* @return {!Object} Object with value, formattedValue and other properties
*/
let generateQuoteData = function(field, options, seed) {
	let fInfo = _generateFieldData(field, options, seed);

	//formatting
	let fmtValue = fInfo.value + "";
	if(fInfo.prefix){
		if(typeof fInfo.prefix == "string") {
			fmtValue = fInfo.prefix + fmtValue;
		} else if(options.prefix) {
			fmtValue = options.prefix + " " + fmtValue;
		}
	}
	if(fInfo.suffix){
		fmtValue = fmtValue + fInfo.suffix;
	}

	if(fInfo.type === "string") {
		fInfo.value = fmtValue; // Make sure that original value has correct data type
	}
	fInfo.formattedValue = fmtValue;
	return fInfo;
};

/** @private
* @param {string|Array.<string>} fields
* @param {(number|Object)=} options Configuration object or number of rows
* @param {number=} seed Default seed for randomization
* @return {!Object}
*/
let _generate2DArray = function(fields, options, seed) {
	let config = {};
	let numRows = 0;
	if(options) {
		if(typeof options === "number") {
			numRows = config.numRows = options;
		} else {
			config = options;
			numRows = options.numRows || options.numRow || options.rowCount;
			if(options.seed != null) {
				seed = options.seed; // Seed from option object overwrite the default seed
				if(seed === -1) {
					seed = null; // Negative one is a special value for reseting seed
				}
			}
		}
	} // else case is "", false, 0, null, undefined
	if(!numRows) {
		numRows = 1;
	}

	if(!fields) {
		fields = [""]; // Default field
	} else if(!Array.isArray(fields)) {
		fields = [fields + ""];
	}
	let fieldCount = fields.length;

	let dataRows = [];
	for(let i = 0; i < numRows; ++i) {
		let row = [];
		// Same field in a single record must be produced to the same value
		for(let j = 0; j < fieldCount; ++j) {
			let field = fields[j];
			row.push(_generateFieldData(field, options, seed).value);
		}
		if(seed != null) {
			++seed;
		}
		dataRows.push(row);
	}

	return {
		data: dataRows, // Array.<Array>
		seed: seed, // nullable
		fields: fields
	};
};
/** Simple hash
* @private
* @param {string} str
* @return {number}
*/
let _hash = function(str) {
	let sum = 0;
	let i = str ? str.length : 0;
	while(--i >= 0) {
		sum += str.charCodeAt(i) * (i + 0.9879);
	}
	return sum;
};
/** @private
* @param {string} field
* @param {DataGenerator~DataOptions=} options
* @param {number=} seed
* @return {!Object} Object with value and other properties
*/
let _generateFieldData = function(field, options, seed) {
	let value;
	let fInfo = getFieldInfo(field);
	if(!fInfo.type) {
		fInfo.type = "number";
		addFieldInfo(field, fInfo);
	}
	if(fInfo.fixedValue !== undefined){ // eslint-disable-line
		fInfo.value = fInfo.fixedValue;
		return fInfo;
	}
	if(seed != null) {
		if(fInfo.hash == null) {
			fInfo.hash = _hash(field);
		}
		seed += fInfo.hash; // Make each field unique for the same seed
	}

	let min = fInfo.min != null ? fInfo.min : 100; // WARNING: Default values are non-standard values
	let max = fInfo.max != null ? fInfo.max : 10000;

	if(fInfo.type === "string") {
		if(fInfo.min != null || fInfo.max != null) {
			value = randString(min, max, seed);
		} else {
			if(options) {
				value = options.text || "";
			} else {
				value = "";
			}
		}
	} else if(fInfo.type === "set") {
		value = randMember(fInfo.members, seed);
	} else if(fInfo.type === "date") {
		value = randDate(min, max, seed);
	} else if(fInfo.type === "isoDate") {
		let date = randDate(min, max, seed);
		value = date.toISOString().slice(0, 10);
	} else if(fInfo.type === "isoDateObject") {
		value = randDate(min, max, seed);
		value = new Date(value.toISOString().slice(0, 10));
	} else if(fInfo.type === "boolean") {
		value = randBoolean(seed);
	} else if(fInfo.type === "function") {
		fInfo.field = field;
		value = fInfo.generate(fInfo, seed);
	} else { // Default is number for all unknown type
		let prec = fInfo.prec != null ? fInfo.prec : 0;
		value = randNumber(min, max, prec, seed);
	}
	fInfo.value = value;
	return fInfo;
};


/** @constructor
* @param {number=} seed
*/
let DataGenerator = function(seed) {
	this._seed = seed != null ? seed : null;
};

/** @type {number=}
* @private
*/
DataGenerator.prototype._seed = null;

/** Generate 2 dimensional array of data from the specified field and options. Note that 2D array data structure is not recommended for usage due to the position of fields/columns can be changed at runtime.
* @public
* @param {string|Array.<string>} fields
* @param {(number|DataGenerator~DataOptions)=} options
* @return {!Array.<Array>} 2D Array of data
*/
DataGenerator.prototype.generate = function(fields, options) {
	let result = _generate2DArray(fields, options, this._seed);
	this._seed = result.seed;

	return result.data;
};

/** @public
* @param {string|Array.<string>} fields
* @param {(number|Object)=} options
* @return {!Array.<Object>} Array of data object
*/
DataGenerator.prototype.generateRecords = function(fields, options) {
	let result = _generate2DArray(fields, options, this._seed);
	this._seed = result.seed;
	fields = result.fields;
	let fieldCount = fields.length;
	let records = result.data;
	for(let i = records.length; --i >= 0;) {
		let ary = records[i];
		let record = {};
		for(let j = 0; j < fieldCount; ++j) {
			let field = fields[j];
			record[field] = ary[j];
		}
		records[i] = record;
	}
	return records;
};

/** @public
* @param {string} field
* @param {DataGenerator~DataOptions=} options
* @return {!Object} Object with value, formattedValue and other properties
*/
DataGenerator.prototype.generateQuoteData = function(field, options) {
	let fInfo = generateQuoteData(field, options, this._seed);
	if(this._seed != null) {
		++this._seed;
	}

	return fInfo;
};
/** @public
* @return {?number}
*/
DataGenerator.prototype.getSeed = function() {
	return this._seed;
};

/** @public
* @param {number} seed
*/
DataGenerator.prototype.setSeed = function(seed) {
	if(typeof seed === "number") {
		this._seed = seed;
	}
};

/** @public
* @param {number=} min
* @param {number=} max Exclusive value
* @return {number}
*/
DataGenerator.prototype.randBetween = function(min, max) {
	let result = randBetween(min, max, this._seed);
	if(this._seed != null) {
		++this._seed;
	}
	return result;
};

/** @public
* @param {number} min
* @param {number} max Inclusive value
* @return {number}
*/
DataGenerator.prototype.randInt = function(min, max) {
	let result = randInt(min, max, this._seed);
	if(this._seed != null) {
		++this._seed;
	}
	return result;
};

/** @public
* @param {number} max Exclusive value
* @return {number}
*/
DataGenerator.prototype.randIndex = function(max) {
	let result = randIndex(max, this._seed);
	if(this._seed != null) {
		++this._seed;
	}
	return result;
};

/** @public
* @param {number=} min=0
* @param {number=} max=100 Exclusive value
* @param {(number|null)=} prec
* @return {number}
*/
DataGenerator.prototype.randNumber = function(min, max, prec) {
	let result = randNumber(min, max, prec, this._seed);
	if(this._seed != null) {
		++this._seed;
	}
	return result;
};

/** @public
* @return {boolean}
*/
DataGenerator.prototype.randBoolean = function() {
	let result = randBoolean(this._seed);
	if(this._seed != null) {
		++this._seed;
	}
	return result;
};

/** @public
* @param {Array} set
* @return {*}
*/
DataGenerator.prototype.randMember = function(set) {
	let result = randMember(set, this._seed);
	if(this._seed != null) {
		++this._seed;
	}
	return result;
};

/** @public
* @param {(number|Object)=} min minimum timestamp or date object
* @param {(number|Object)=} max minimum timestamp or date object
* @return {Date}
*/
DataGenerator.prototype.randDate = function(min, max) {
	let result = randDate(min, max, this._seed);
	if(this._seed != null) {
		++this._seed;
	}
	return result;
};

/** @public
* @param {number=} min
* @param {number=} max Exclusive value
* @return {string}
*/
DataGenerator.prototype.randString = function(min, max) {
	let result = randString(min, max, this._seed);
	if(this._seed != null) {
		++this._seed;
	}
	return result;
};

/** @type {?number}
* @public
* @ignore
*/
DataGenerator.seed = null;
/** Generate 2 dimensional array of data from the specified field and options. Note that 2D array data structure is not recommended for usage due to the position of fields/columns can be changed at runtime.
* @public
* @function
* @param {string|Array.<string>} fields
* @param {DataGenerator~DataOptions=} options
* @return {!Array.<Array>} 2D Array of data
*/
DataGenerator.generate = generate;
/** @public
* @function
* @param {string|Array.<string>} fields
* @param {DataGenerator~DataOptions=} options
* @return {Object} Object with the given fields as its keys
*/
DataGenerator.generateRecord = generateRecord;
/** @public
* @function
* @param {string|Array.<string>} fields
* @param {(number|DataGenerator~DataOptions)=} options Configuration object or number of rows
* @return {!Array.<Object>} Array of object with the given fields as its keys
*/
DataGenerator.generateRecords = generateRecords;
/** @public
* @function
* @param {string} field
* @param {DataGenerator~FieldInformation|Function} options
*/
DataGenerator.addFieldInfo = addFieldInfo;
/** Return a floating-point random number in the range min - max (inclusive of min, but not max) with prec digits of precision.
* @public
* @function
* @param {number=} min=0
* @param {number=} max=100 Exclusive value
* @param {(number|null)=} prec
* @param {number=} seed
* @return {number}
*/
DataGenerator.randNumber = randNumber;
/** Return an interger random number in the range min - max (inclusive of min and max).
* @public
* @function
* @param {number} min
* @param {number} max Inclusive value
* @param {number=} seed
* @return {number}
*/
DataGenerator.randInt = randInt;
/** Return a integer random number in the range min - max (inclusive of min, but not max).
* @public
* @function
* @param {number=} min
* @param {number=} max Exclusive value
* @param {number=} seed
* @return {number}
*/
DataGenerator.randBetween = randBetween;
/** Return a integer random number in the range 0 - max <br>
* (exclusive of max). It is usefull in some cases for example, <br>
* if you want to get a random member of an array, <br>
* just get a random index via randIndex(ary.length).
* @public
* @function
* @param {number} max Exclusive value
* @param {number=} seed
* @return {number}
*/
DataGenerator.randIndex = randIndex;
/** @public
* @function
* @param {number=} seed
* @return {boolean}
*/
DataGenerator.randBoolean = randBoolean;
/** Return a random member of set or single character in the range of A to Z
* @public
* @function
* @param {Array} set
* @param {number=} seed
* @return {*}
*/
DataGenerator.randMember = randMember;
/** @public
* @function
* @param {(number|Object)=} min minimum timestamp or date object
* @param {(number|Object)=} max minimum timestamp or date object
* @param {number=} seed
* @return {Date}
*/
DataGenerator.randDate = randDate;
/** @public
* @function
* @param {number=} min
* @param {number=} max
* @param {string=} subType "w"/"W" = word, "s"/"S" = sentence
* @param {number=} seed
* @return {string}
*/
DataGenerator.randString = randString;

/** Convert 2D Array to Array of records
* @public
* @function
* @param {Array.<Array>} data2D Array of values
* @param {Array.<string>=} fields Keys to be mapped on the output records.
* @return {Array.<Object>} records
*/
DataGenerator.toRecords = toRecords;


/* harmony default export */ const jet_DataGenerator = ((/* unused pure expression or super */ null && (DataGenerator)));


;// ./node_modules/tr-grid-util/es6/jet/MockQuotes.js


/** @constructor */
let MockQuotes = function() {
};

/** @public
* @param {string=} opt_id
* @return {MockSubscription}
*/
MockQuotes.prototype.create = function(opt_id) {
	return new MockSubscription(opt_id);
};


/** @constructor
* @param {string=} opt_id
*/
let MockSubscription = function(opt_id) {
	this._onRowUpdated = this._onRowUpdated.bind(this);

	this._fieldMap = {};
	this._prevData = {};
	if(opt_id != null) {
		this["id"] = opt_id + "";
	} else {
		this["id"] = "_sub" + MockSubscription._runningId++;
	}
};

/** @type {Array.<string>}
* @private
*/
MockSubscription.prototype._rics = null;

/** @type {boolean}
* @private
*/
MockSubscription.prototype._isChain = false;

/** @type {Object}
* @private
*/
MockSubscription.prototype._fieldMap = null;

/** @type {Object}
* @private
*/
MockSubscription.prototype._prevData = null;

/** @type {Function}
* @private
*/
MockSubscription.prototype._updateHandler = null;

/** @type {number}
* @private
*/
MockSubscription.prototype._updateTimerId = 0;

/** @type {number}
* @private
*/
MockSubscription.prototype._updateInterval = 1000;

/** @type {number}
* @private
*/
MockSubscription._runningId = 0;

/** @public
* @param {Array.<string>} fields
* @return {MockSubscription}
*/
MockSubscription.prototype.formattedFields = function(fields) {
	for(let i = fields.length; --i >= 0;){
		this._fieldMap[fields[i]] = true;
	}
	return this;
};
/** @public
* @param {Array.<string>} fields
* @return {MockSubscription}
*/
MockSubscription.prototype.rawFields = function(fields) {
	for(let i = fields.length; --i >= 0;){
		this._fieldMap[fields[i]] = true;
	}
	return this;
};

/** @public
* @param {string} ric
* @return {MockSubscription}
*/
MockSubscription.prototype.chain = function(ric) {
	this._rics = Array.isArray(ric) ? ric[0] : [ric];
	this._isChain = true;
	return this;
};

/** @public
* @param {string|Array.<string>} ric
* @return {MockSubscription}
*/
MockSubscription.prototype.rics = function(ric) {
	this._rics = Array.isArray(ric) ? ric : [ric];
	return this;
};

/** @public
* @return {MockSubscription}
*/
MockSubscription.prototype.start = function() {
	this._onRowUpdated();
	return this;
};
/** @public
* @return {MockSubscription}
*/
MockSubscription.prototype.stop = function() {
	if(this._updateTimerId){
		clearInterval(this._updateTimerId);
		this._updateTimerId = 0;
	}
	return this;
};

/** @public
* @param {Function} handler
* @return {MockSubscription}
*/
MockSubscription.prototype.onNewRow = function(handler) {
	return this;
};
/** @public
* @param {Function} handler
* @return {MockSubscription}
*/
MockSubscription.prototype.onUpdate = function(handler) {
	this._updateHandler = handler;
	return this;
};
/** @public
* @param {Function} handler
* @return {MockSubscription}
*/
MockSubscription.prototype.onRemoveRow = function(handler) {
	return this;
};

/** @private */
MockSubscription.prototype._onRowUpdated = function() {
	if(!this._updateHandler){
		return;
	}
	//TODO: random which fields to be updated
	let len = this._rics ? this._rics.length : 0;
	let ric;
	if(len === 1) {
		ric = this._rics[0];
		this._updateHandler(this, ric, this._getUpdateData(ric));
	} else {
		for(let i = 0; i < len; ++i) {
			if(!randIndex(3)) {
				ric = this._rics[i];
				this._updateHandler(this, ric, this._getUpdateData(ric));
			}
		}
	}
	if(!this._updateTimerId) {
		this._updateTimerId = setInterval(this._onRowUpdated, this._updateInterval);
	}
};

/** @private
* @param {string} ric
* @return {Object}
*/
MockSubscription.prototype._getUpdateData = function(ric) {
	let data = {"id": this["id"]};
	let options = {"text": ric};
	for(let field in this._fieldMap) {
		let fInfo = generateQuoteData(field, options);
		let v = fInfo.value;
		let fv = fInfo.formattedValue;
		if(fInfo.changeOnly) {
			if(this._prevData[field] === v) {
				continue;
			}
		}
		this._prevData[field] = v;
		data[field] = {"raw": v, "formatted": fv || (v + "")};
	}
	return data;
};




;// ./node_modules/tr-grid-util/es6/jet/CollectionDict.js
/** @description CollectionDict stores a collection (Array) of any value using a text (string) as a key for accessing the collection.
* @constructor
*/
let CollectionDict = function() {
	this._dict = {};
};
/** @type {!Object.<string, Array>}
* @private
*/
CollectionDict.prototype._dict;
/** @type {number}
* @private
*/
CollectionDict.prototype._count = 0;

/** @public
* @param {string} key
* @param {*} item Null or undefined item is not allowed
* @return {boolean} Return true if the item has been added
*/
CollectionDict.prototype.addItem = function(key, item) {
	if(key && item != null) {
		let ary = this._dict[key];
		if(ary) {
			ary.push(item);
		} else {
			this._dict[key] = [item];
		}

		++this._count;
		return true;
	}
	return false;
};
/** @public
* @param {string} key
* @param {Array} items The given item list should not contain null or undefined
* @return {boolean} Return true if an item has been added
*/
CollectionDict.prototype.addItems = function(key, items) {
	if(key) {
		if(!Array.isArray(items)) {
			return this.addItem(key, items);
		}
		let len = items ? items.length : 0;
		if(len) {
			let ary = this._dict[key];
			if(!ary) {
				ary = this._dict[key] = [];
			}

			// Fastest way to add multiple items at once to an array. WARNING: There is a 10,000 item limit.
			Array.prototype.push.apply(ary, items);
			this._count += len;
			return true;
		}
	}
	return false;
};

/** @public
* @param {string} key
* @param {*} item
* @return {boolean} Return true if the item has been removed
*/
CollectionDict.prototype.removeItem = function(key, item) {
	let ary = this._dict[key];
	if(ary) {
		let at = ary.indexOf(item);
		if(at >= 0) {
			if(ary.length > 1) {
				ary.splice(at, 1);
			} else {
				delete this._dict[key];
			}
			--this._count;
			return true;
		}
	}
	return false;
};
/** @public
* @param {string} key
* @return {boolean} Return true if any item has been removed
*/
CollectionDict.prototype.removeItemsByKey = function(key) {
	let ary = this._dict[key];
	if(ary) {
		delete this._dict[key];
		this._count -= ary.length;
		return true;
	}
	return false;
};
/** @public
* @return {boolean} Return true if there is any change
*/
CollectionDict.prototype.removeAllItems = function() {
	if(this._count) {
		this._dict = {};
		this._count = 0;
		return true;
	}
	return false;
};

/** @public
* @param {string} key
* @return {Array} Return null if no item exist
*/
CollectionDict.prototype.getItems = function(key) {
	return this._dict[key] || null;
};
/** @public
* @return {Array} Return null if no item exist
*/
CollectionDict.prototype.getAllItems = function() {
	if(this._count > 0) {
		let ary = [];
		let dict = this._dict;
		for(let key in dict) {
			// Fastest way to add multiple items at once to an array. WARNING: There is a 10,000 item limit.
			Array.prototype.push.apply(ary, dict[key]);
		}
		return ary;
	}
	return null;
};
/** @public
* @param {string=} key
* @return {number}
*/
CollectionDict.prototype.getItemCount = function(key) {
	if(key) {
		let items = this._dict[key] || null;
		if(items) {
			return items.length;
		}
		return 0;
	}
	return this._count;
};
/** @public
* @return {!Array.<string>}
*/
CollectionDict.prototype.getAllKeys = function() {
	return Object.keys(this._dict);
};

/* harmony default export */ const jet_CollectionDict = ((/* unused pure expression or super */ null && (CollectionDict)));


;// ./node_modules/tr-grid-util/es6/jet/MockQuotes2.js



/** @private
* @param {Object} subA
* @param {Object} subB
* @return {string}
*/
let _joinSubKeys = function(subA, subB) {
	return subA.ric + "_" + subB.ric;
};

/** @private
* @param {string} ric
* @return {boolean}
*/
let _isDynamicChain = function(ric) {
	// Dynamic chain will be 2 . (dot) for example .PG.PA
	let matching = ric.match(/\./g); // Can be null when doesn't found dot(.)
	if(matching) {
		return matching.length > 1;
	}
	return false;
};

/** @private
* @param {Array<Object>} arr sub children
* @return {Array<Object>}
*/
let _shuffleArray = function(arr) {
	let arrayShuffled = arr; // Modify original array
	let i, randNumber, tmp;
	// Shuffle the array randomly using the Fisher-Yates algorithm
	for (i = arrayShuffled.length - 1; i > 0; i--) {
		randNumber = Math.floor(Math.random() * (i + 1)); // TODO: it can be use this._dataGen.randInt(min, max) range
		tmp = arrayShuffled[i];
		arrayShuffled[i] = arrayShuffled[randNumber];
		arrayShuffled[randNumber] = tmp;
	}

	// TODO: merge into 1 loop
	// Update the CHILD_ORDER property based on the new index
	arrayShuffled.forEach(_assignChildOrder);
	arrayShuffled = arrayShuffled.sort(_childOrderSort);

	return arrayShuffled;
};

let _assignChildOrder = function(obj, index) {
	obj["CHILD_ORDER"] = index;
};

let _childOrderSort = function(a, b) {
	return a["CHILD_ORDER"] - b["CHILD_ORDER"];
};

/** @constructor */
let MockQuotes2 = function() {

};
/** @private
* @type {*}
*/
MockQuotes2.prototype._lastSub = null;
/** @private
* @type {Object}
*/
MockQuotes2.prototype._configObj = null;

/** @public
* @param {Object=} options
*/
MockQuotes2.prototype.config = function(options) {
	if(options) {
		this._configObj = options;
	}
	if(this._lastSub && this._configObj) {
		this._lastSub.config(this._configObj);
	}
};
/** @public
* @param {Object=} options
* @return {MockSubscriptions}
*/
MockQuotes2.prototype.create = function(options) {
	this._lastSub = new MockSubscriptions(options || this._configObj);
	return this._lastSub;
};

/** @public
* @return {boolean}
*/
MockQuotes2.prototype.isSkipHeaderSupported = function() {
	return true;
};

/** @public
* @param {number=} min
* @param {number=} max
*/
MockQuotes2.prototype.setUpdateInterval = function(min, max) {
	if(this._lastSub) {
		this._lastSub.setUpdateInterval(min, max);
	} else {
		if(!this._configObj) {
			this._configObj = {};
		}
		this._configObj.minInterval = min;
		this._configObj.maxInterval = max;
	}
};
/** @public
* @param {number=} percent
*/
MockQuotes2.prototype.setPercentageDataUpdate = function(percent) {
	if(this._lastSub) {
		this._lastSub.setPercentageDataUpdate(percent);
	} else {
		if(!this._configObj) {
			this._configObj = {};
		}
		this._configObj.percentageDataUpdate = percent;
	}
};


/** @constructor
* @param {Object=} options
*/
let MockSubscriptions = function(options) {
	this._onSubscriptionResponse = this._onSubscriptionResponse.bind(this);

	this._subMap = {};
	this._dataMap = new CollectionDict();
	this._fields = {};
	this._events = {};
	this._dataGen = new DataGenerator();

	this.config(options);
};

/** @private
* @type {number}
*/
MockSubscriptions.prototype._minInterval = 750;
/** @private
* @type {number}
*/
MockSubscriptions.prototype._maxInterval = 850;
/** @private
* @type {number}
*/
MockSubscriptions.prototype._percentageDataUpdate = 0.1; // 10% by default
/** @private
* @type {number}
*/
MockSubscriptions.prototype._percentageOrderChange = 1; // 100%
/** @private
* @type {boolean}
*/
MockSubscriptions.prototype._constituentCache = false;

/** @public
* @param {Object=} options
*/
MockSubscriptions.prototype.config = function(options) {
	if(!options) {
		return;
	}
	let num = options.minInterval;
	if(typeof num === "number") {
		this._minInterval = num;
		if(this._minInterval > this._maxInterval) {
			this._maxInterval = this._minInterval;
		}
	}

	num = options.maxInterval;
	if(typeof num === "number") {
		this._maxInterval = num;
		if(this._minInterval > this._maxInterval) {
			this._minInterval = this._maxInterval;
		}
	}

	num = options.percentageDataUpdate;
	if(typeof num === "number") {
		this._percentageDataUpdate = num / 100;
	}

	num = options.percentageOrderChange;
	if(typeof num === "number") {
		this._percentageOrderChange = num / 100;
	}

	let value = options.constituentCache;
	if(value != null) {
		this._constituentCache = value;
	}

};
/** @public
* @param {number=} min
* @param {number=} max
*/
MockSubscriptions.prototype.setUpdateInterval = function(min, max) {
	if(typeof min === "number") {
		this._minInterval = min;
	}

	if(typeof max === "number") {
		this._maxInterval = max;
	}
};
/** @public
* @param {number=} percent
*/
MockSubscriptions.prototype.setPercentageDataUpdate = function(percent) {
	if(typeof percent === "number") {
		this._percentageDataUpdate = Math.round(percent / 100);
	}
};

/** @public
* @param {number=} percent
*/
MockSubscriptions.prototype.setPercentageOrderChanged = function(percent) {
	if(typeof percent === "number") {
		this._percentageOrderChange = Math.round(percent / 100);
	}
};

/** @public
* @param {number} seed
*/
MockSubscriptions.prototype.setSeed = function(seed) {
	this._dataGen.setSeed(seed);
};


/** @public
* @param {string} ric
* @param {string=} subId
* @return {string|null} subscription Id
*/
MockSubscriptions.prototype.addRic = function(ric, subId) {
	return this._addSymbol(ric, false, subId);
};
/** @public
* @param {string} ric
* @param {string=} subId
* @return {string|null} subscription Id
*/
MockSubscriptions.prototype.addChain = function(ric, subId) {
	return this._addSymbol(ric, true, subId);
};
/** @public
* @param {string|Array.<string>} fields
*/
MockSubscriptions.prototype.addFields = function(fields) {
	if(!fields) { return; }

	if(typeof fields === "string") {
		fields = [fields];
	}
	let len = fields.length;
	for(let i = 0; i < len; ++i) {
		this._fields[fields[i]] = true;
	}
};

/** @public
* @param {string} subId
*/
MockSubscriptions.prototype.removeSubscription = function(subId) {
	let sub = this._subMap[subId];
	if(!sub) {
		return;
	}
	this._dispatch("subscriptionRemoved", {"subs": [sub]});
	delete this._subMap[subId];
	this._dataMap.removeItem(sub["ric"], sub);
	if(sub["chain"]) {
		let children = sub["children"];
		let len = children.length;
		for(let i = 0; i < len; ++i) {
			let child = children[i];
			this._dataMap.removeItem(_joinSubKeys(sub, child), child);
			child["parent"] = null;
		}
		sub["children"] = null;
		sub["ricList"] = null;
	}
};
/** @public
* @function
*/
MockSubscriptions.prototype.removeRic = MockSubscriptions.prototype.removeSubscription;
/** @public
* @function
*/
MockSubscriptions.prototype.removeChain = MockSubscriptions.prototype.removeSubscription;
/** @public */
MockSubscriptions.prototype.removeAllSubscriptions = function() { // All channels will also be closed
	let working = this._working;
	this.stop();

	if(this._hasListener("subscriptionRemoved")) {
		let subs = [];
		for(let subId in this._subMap) {
			subs.push(this._subMap[subId]);
		}
		if(subs.length > 0) {
			this._dispatch("subscriptionRemoved", {"subs": subs});
		}
	}

	this._subMap = {};
	this._dataMap.removeAllItems();
	if(working) {
		this.start();
	}
};

/** @public
* @param {string|Array.<string>} fields
*/
MockSubscriptions.prototype.removeFields = function(fields) {
	if(typeof fields === "string") {
		fields = [fields];
	}
	let len = fields.length;
	for(let i = 0; i < len; ++i) {
		let field = fields[i];
		if(this._fields[field]) {
			delete this._fields[field];
		}
	}
};
/** @public */
MockSubscriptions.prototype.removeAllFields = function() {
	let dirty = false;
	for(let field in this._fields) { // eslint-disable-line no-unused-vars
		dirty = true;
		break;
	}
	if(!dirty) { return; }
	this._fields = {};
};

/** @public */
MockSubscriptions.prototype.start = function() {
	this._working = true;
	this._connect();
};
/** @public */
MockSubscriptions.prototype.stop = function() {
	this._working = false;
	if(this._timerId) {
		window.clearTimeout(this._timerId);
		this._timerId = 0;
	}
};

/** @public */
MockSubscriptions.prototype.dispose = function() {
	this.stop();

	this._subMap = null;
	if(this._dataMap) {
		this._dataMap.removeAllItems();
		this._dataMap = null;
	}

	this._fields = null;
	this._events = null;
};

/** @public
* @param {string} type
* @param {Function} handler
*/
MockSubscriptions.prototype.addEventListener = function(type, handler) {
	if(!handler) { return; }

	let listeners = this._events[type];
	if(listeners) {
		if(listeners.indexOf(handler) < 0) {
			listeners.push(handler);
		}
	} else {
		this._events[type] = [handler];
	}
};
/** @public
* @param {string} type
* @param {Function} handler
*/
MockSubscriptions.prototype.removeEventListener = function(type, handler) {
	let listeners = (this._events) ? this._events[type] : null;
	if(!listeners) { return; }

	let at = listeners.indexOf(handler);
	if(at >= 0) {
		listeners.splice(at, 1);
	}
};
/** @public */
MockSubscriptions.prototype.removeAllEventListeners = function() {
	this._events = {};
};

/** @public
* @param {string} subId
* @return {Object} subscription object
*/
MockSubscriptions.prototype.getSubscription = function(subId) {
	return this._subMap[subId] || null;
};
/** @public
* @param {string} subId
* @return {string} string
*/
MockSubscriptions.prototype.getRic = function(subId) {
	let sub = this._subMap[subId];
	if(sub) {
		return sub["ric"];
	}
	return "";
};
/** @public
* @return {Object.<string, boolean>} field map
*/
MockSubscriptions.prototype.getAllFields = function() {
	return this._fields;
};
/** @public
* @param {string} field
* @return {boolean}
*/
MockSubscriptions.prototype.hasField = function(field) {
	return this._fields[field] || false;
};
/** @public
* @param {number} start
* @param {number} count
* @return {MockSubscriptions}
*/
MockSubscriptions.prototype.filter = function (start, count) {
	return this;
};

/** Object.assign is not available in IE. This method is equivalent to Object.assign(toObject, fromObject);
* @private
* @function
* @param {Object} fromObj
* @param {!Object} toObj
* @return {!Object} Return toObj
*/
let copyValues = function(fromObj, toObj) {
	for(let key in fromObj) {
		toObj[key] = fromObj[key];
	}
	return toObj;
};
/** @private
* @param {string} ric
* @param {boolean} asChain
* @param {(string|number)=} subId
* @return {string|null} subscription id
*/
MockSubscriptions.prototype._addSymbol = function(ric, asChain, subId) {
	if(!ric) { return null; }

	if(!subId && subId !== 0) {
		do {
			subId = "_" + (this._runningSubId++) + "_";
		} while(this._subMap[subId]);
	} else {
		subId += ""; // Convert to string
	}
	if(this._subMap[subId]) {
		window.console.log("WARNING: Cannot add ric with duplicate id");
		return null;
	}

	if(asChain) {
		ric = ric.replace("0#", ""); // The receiver is not expecting 0# symbol
	}
	let sub = {};
	sub["ric"] = ric;
	sub["chain"] = asChain;
	sub["id"] = subId;
	sub["dataId"] = subId + ric;
	this._subMap[subId] = sub; // Collect all user subscriptions
	this._dataMap.addItem(ric, sub);
	let subs = this._dataMap.getItems(ric);
	let subCount = subs.length;

	this._dispatch("subscriptionAdded", {"subs": [sub]});

	let childSub = null;
	let childCount = 0;
	let i;
	if(asChain) {
		// Note that constituents should have no subscription object. They should share the same subscription as their parent. Hence we does not register it to the _subMap
		if(subCount === 1) { // The first chain detected
			sub["children"] = [];
			childCount = MockSubscriptions.simpleDigest(ric) % 10 + 4;
			for(i = 0; i < childCount; ++i) {
				childSub = {};
				childSub["ric"] = "Child_" + String.fromCharCode(65 + i);
				childSub["id"] = subId; // Child shares the same sub id as its chain parent
				childSub["dataId"] = subId + childSub["ric"];
				childSub["parent"] = sub; // This does not exist in real subscription
				sub["children"].push(childSub);
				childSub["CHILD_ORDER"] = i;
				this._dataMap.addItem(_joinSubKeys(sub, childSub), childSub);
			}
		} else { // Duplicate chain
			let firstSub = subs[0];
			let constituents = firstSub["children"];
			childCount = constituents.length;
			sub["children"] = new Array(childCount);

			for(i = 0; i < childCount; ++i) {
				childSub = {};
				childSub["ric"] = constituents[i]["ric"];
				childSub["id"] = subId; // Child shares the same sub id as its chain parent
				childSub["dataId"] = subId + childSub["ric"];
				childSub["parent"] = sub; // This does not exist in real subscription
				sub["children"][i] = childSub;

				this._dataMap.addItem(_joinSubKeys(sub, childSub), childSub);
			}
		}
	}
	if(subCount > 1) { // Duplicate RIC/CHAIN detected
		if(this._working) {
			setTimeout(this._updateDuplicateSymbol.bind(this, ric), 10);
		}
	}

	this._connect();
	return subId;
};
/** @public
* @param {string} str
* @return {number}
*/
MockSubscriptions.simpleDigest = function(str) {
	let len = str.length;
	let sum = 0;
	for(let i = 0; i < len; ++i) {
		sum += str.charCodeAt(i);
	}
	return sum;
};

/** @private */
MockSubscriptions.prototype._connect = function() {
	if(this._working && !this._timerId) {
		let delay = this._dataGen.randInt(this._minInterval, this._maxInterval);
		this._timerId = window.setTimeout(this._onSubscriptionResponse, delay); // This will be async for fire event to user
	}
};

MockSubscriptions.prototype._onSubscriptionResponse = function() {
	this._timerId = 0;

	let keys = this._dataMap.getAllKeys(); // list of all rics
	let len = keys ? keys.length : 0; // len include all row index (Constituent and normal ric)
	if(!len) { // No symbol has been added
		this._connect();
		return;
	}

	let minRow = (this._percentageDataUpdate - 0.02) * len;
	minRow = minRow > 0 ? minRow : 1; // update at least 1 row
	let maxRow = (this._percentageDataUpdate + 0.02) * len;
	maxRow = maxRow < len ? maxRow : len; // not more than all rows
	let numRows = this._dataGen.randInt(minRow, maxRow);
	let fields = this._fields;

	let updateCount = 0;
	let childOrderChange;
	for(let i = 0; i < numRows; i++) {
		// find key to random subs (key is subId)
		let key = keys[this._dataGen.randIndex(len)]; // WARNING: Same sub could be picked more than once
		let subs = this._dataMap.getItems(key); // Get all subs with the same RIC
		let subCount = subs.length;

		let j;
		let firstSub = subs[0]; // Only the first sub is need to generate data
		let subParent = firstSub.parent;
		let updatePosition = this._dataGen.randInt(0, 100) < this._percentageOrderChange * 100 ? true : false; // Random chance to change dynamic chain position (2% to change)

		let values = this._generateQuoteData(firstSub, fields);
		if(_isDynamicChain(key) && subParent && updatePosition) { // subParent in header of dynamic chain behaves like a normal ric
			let children = subParent.children;
			let childCount = children.length;

			children = _shuffleArray(children); // Random swap children in array
			let newOrder = children.indexOf(firstSub);
			firstSub["CHILD_ORDER"] = newOrder; // This is bad
			values["CHILD_ORDER"] = newOrder;

			childOrderChange = true;
			for(j = 0; j < subCount; ++j) { // It could be same ric and it need to dispatch with same ric number
				this._dispatchDataChanged(subs[j], values);
				// Update child order for all children with the same parent
				for (let k = 0; k < childCount; k++) { // This is not correct, if there is a duplicate dynamic chain
					if(k !== newOrder) {
						this._dispatchDataChanged(children[k], {
							X_RIC_NAME: children[k].ric,
							CHILD_ORDER: k
						});
					}
				}
			}
		} else {
			if(subCount > 1) { // Duplicate CHILD_ORDER from the first sub for other sub with the same RIC (not entirely correct)
				let childOrder = firstSub["CHILD_ORDER"];
				if(childOrder != null) { // Children of chain will have a CHILD_ORDER
					values["CHILD_ORDER"] = childOrder;
				}
			}
			for(j = 0; j < subCount; ++j) { // It could be same ric and it need to dispatch with same ric number
				this._dispatchDataChanged(subs[j], values);
				updateCount++;
			}
		}
	}

	let postObj = {};
	if(childOrderChange != null) {
		postObj["childOrderChange"] = childOrderChange;
	}
	if(updateCount != null) {
		postObj["updateCount"] = updateCount;
	}

	this._dispatchPostUpdate(postObj);

	this._connect();
};

/** @private
* @param {Object} obj
*/
MockSubscriptions.prototype._dispatchPostUpdate = function (obj) {
	// TODO: handlded another property of "postUpdate" event
	this._dispatch("postUpdate", obj);
};

/** @private
* @param {Object} sub
* @param {Object} fields
* @return {!Object}
*/
MockSubscriptions.prototype._generateQuoteData = function(sub, fields) {
	let ric = sub.ric;
	let prevData = sub.prevData;

	let values = {};
	let options = {
		text: ric,
		prefix: sub["parent"] ? sub["parent"]["ric"] : "" // prefix for constituents
	};
	for(let field in fields){
		let fInfo = this._dataGen.generateQuoteData(field, options);
		if(fInfo.ignore) {
			continue;
		}

		let formattedField = field + "_FORMATTED";
		if(prevData) {
			if(fInfo.changeOnly) {
				if(prevData[field] === fInfo.value) {
					continue;
				}
			}
			prevData[field] = fInfo.value;
			prevData[formattedField] = fInfo.formattedValue;
		}

		values[field] = fInfo.value;
		values[formattedField] = fInfo.formattedValue;
	}

	// The delay symbol for X_RIC_NAME will depend on the INDICATOR field.
	if(values["X_RIC_NAME"] != null && values["INDICATOR"] != null) {
		if(values["INDICATOR"] > 0) {
			values["X_RIC_NAME"] = "/" + values["X_RIC_NAME"];
		}
	}
	return values;
};


/** @private
* @param {Object} parentSub
* @param {string} ric
* @return {Object}
*/
MockSubscriptions.prototype._getChildSubByRic = function(parentSub, ric) {
	let children = parentSub["children"];
	if(children) {
		let childCount = children.length;
		for(let i = 0; i < childCount; ++i) {
			let child = children[i];
			if(child["ric"] === ric) {
				return child;
			}
		}
	}
	return null;
};
/** @private
* @param {string} ric
*/
MockSubscriptions.prototype._updateDuplicateSymbol = function(ric) {
	let subs = this._dataMap.getItems(ric);
	if(!subs) {
		return;
	}
	let subCount = subs.length;
	if(subCount < 2) {
		return;
	}
	let firstSub = subs[0];
	let prevData = firstSub["prevData"];
	let isChain = firstSub["chain"] ? true : false;
	let i;
	if(prevData) {
		for(i = 1; i < subCount; ++i) {
			let sub = subs[i];
			if(!sub["prevData"]) {
				// TODO: check in duplicate dynamic chain
				this._dispatchDataChanged(sub, prevData);
			}
		}
	}


	if(!isChain) {
		return;
	}
	if(!this._constituentCache) {
		return;
	}
	let ricList = firstSub["ricList"];
	if(!ricList) {
		return;
	}
	let childCount = ricList.length;

	for(i = 1; i < subCount; ++i) {
		let sub2 = subs[i];
		let ricList2 = sub2["ricList"];
		let childCount2 = ricList2 ? ricList2.length : 0;
		for(let j = childCount2; j < childCount; ++j) {
			let childRic = ricList[j];
			let childSub = this._getChildSubByRic(firstSub, childRic);
			let childSub2 = this._getChildSubByRic(sub2, childRic);
			if(childSub && childSub2) {
				if(childSub["prevData"]) {
					// TODO: check in duplicate dynamic chain
					this._dispatchDataChanged(childSub2, childSub["prevData"]);
				}
			}
		}
	}
};
/** @private
* @param {Object} sub
* @param {Object} dataUpdates
*/
MockSubscriptions.prototype._dispatchDataChanged = function(sub, dataUpdates) {
	let values = copyValues(dataUpdates, {});
	let evtArg = {};
	evtArg["ric"] = sub["ric"];
	evtArg["values"] = values;
	evtArg["subId"] = sub["id"];

	let prevData = sub.prevData; // For data generation on the next update
	if(!prevData) {
		prevData = {};
		sub.prevData = prevData;
		evtArg["type"] = "statusUpdated";

		values["STATUS"] =  1;
		values["STATUS_FORMATTED"] = this._statusMap["1"];
		values["SUB_ID"] = sub["id"];
		copyValues(values, prevData);

		let parentSub = sub.parent;
		if(parentSub) { // This is the first time constituent have the data
			let ricList = parentSub["ricList"];
			if(!ricList) {
				ricList = parentSub["ricList"] = [];
			}
			parentSub["ricList"].push(sub["ric"]);
		}
	}
	this._dispatch("dataChanged", evtArg);
};

/** @public
* @param {Array.<string>} feedNames
* @param {Function} handler
* @return {MockSubscriptions}
*/
MockSubscriptions.prototype.onFeedStatusChanged = function(feedNames, handler) {
	window.setTimeout(function() {
		handler([{"key": "Q", "name": "IDN_RDF", "state": "up"}]);
	}, 1000);
	return this;
};

/** @private
* @param {string} type
* @return {boolean}
*/
MockSubscriptions.prototype._hasListener = function(type) {
	let listeners = this._events[type];
	return listeners ? (listeners.length > 0) : false;
};
/** @private
* @param {string} type
* @param {Object} eventArg
*/
MockSubscriptions.prototype._dispatch = function(type, eventArg) {
	let listeners = this._events[type];
	if(!listeners) { return; }
	let len = listeners.length;
	for(let i = 0; i < len; ++i) {
		listeners[i](eventArg);
	}
};

/** @type {Object.<string, Object>}
* @private
*/
MockSubscriptions.prototype._subMap = null;
/** @type {Object.<string, boolean>}
* @private
*/
MockSubscriptions.prototype._fields = null;
/** @type {Object.<string, Array.<Function>>}
* @private
*/
MockSubscriptions.prototype._events = null;

/** @type {boolean}
* @private
*/
MockSubscriptions.prototype._working = false;
/** @type {number}
* @private
*/
MockSubscriptions.prototype._timerId = 0;

/** @type {number}
* @private
*/
MockSubscriptions.prototype._runningSubId = 0;

/** @type {Object.<string, string>}
* @private
*/
MockSubscriptions.prototype._statusMap = {
	"0": "PENDING",
	"1": "OK",
	"2": "STALE",
	"3": "INFO",
	"4": "CLOSED",
	"5": "DELAYED",
	"6": "NOPERM",
	"7": "REACHLIMIT",
	"8": "C_COMPLETE",
	"9": "C_CLOSE",
	"10": "C_EMPTY",
	"11": "C_NOTCHAIN",
	"12": "C_ERR",
	"13": "C_NOBEGIN",
	"14": "C_NOEND"
};



;// ./node_modules/tr-grid-util/es6/jet/MockArchive.js
/** @constructor */
let MockArchive = function() {
	this._data = {};
};
/** @type {!Object}
*/
MockArchive.prototype._data = null;
/** @public */
MockArchive.prototype.clear = function() {
	this._data = {};
};
/** @public
* @param {string} key
* @param {*} val
*/
MockArchive.prototype.put = function(key, val) {
	if(key) {
		this._data[key] = val;
	}
};
/** @public
* @param {string} key
* @return {*}
*/
MockArchive.prototype.get = function(key) {
	let val = this._data[key];
	return (val != null) ? val : "";
};

/** @public
* @return {Array.<string>} key
*/
MockArchive.prototype.getAllKeys = function() {
	return Object.keys(this._data);
};
/** @public */
MockArchive.prototype.save = function() {
};
/** @public
* @param {Function} func Function that take an item and return boolean
* @return {!Array}
*/
MockArchive.prototype.filter = function(func) {
	let out_ary = [];
	if(func) {
		for(let key in this._data) {
			let val = this._data[key];
			if(func(val)) {
				out_ary.push(val);
			}
		}
	}
	return out_ary;
};



;// ./node_modules/tr-grid-util/es6/jet/MockUtil.js
/**
* @public
* @type {Object}
*/
let invalidFieldDict = {
	'TR.NonExistField': true,
	'TR.NotExistField': true,
	'CF_IGNORE_FIELD': true
};

/** to mock invalid fields.
* @public
* @param {Array<string>|string} fields ex. TR.NonExistField, CF_IGNORE_FIELD
*/
function setInvalidFields(fields) {
	if (fields && typeof fields === 'string') {
		invalidFieldDict[fields] = true;
	} else if (Array.isArray(fields)) {
		for (let i = 0; i < fields.length; i++) {
			invalidFieldDict[fields[i]] = true;
		}
	}
}



;// ./node_modules/tr-grid-util/es6/jet/Adc.js




let dataGen = new DataGenerator();

/** @private
* @namespace
*/
let Adc = {};

/** @private
* @function
* @param {Object} payload
* @param {Object=} mockResponse
* @return {Promise}
*/
Adc.request = function (payload, mockResponse) {
	if (mockResponse) {
		return Promise.resolve(JSON.stringify(mockResponse));
	}

	// build row header
	let rows = [];
	let i, f, len, row, j;
	let identifiers, formula, fields, invalidDict;
	if(payload.output === "Col,date|,Row,In|,va,T,NoEmptyTickers") {

		identifiers = payload.identifiers;
		formula =  payload.formula.trim().split(/(?=,TR.)/);// TODO: check each field with another way, this doesn't work when user use some comma (,) formula
		fields = [];

		// invalidFieldDict is a dictionary of non exist field
		// so we must remove invalid field to make "mocking api" return result more like a "real api".
		invalidDict = invalidFieldDict;
		for (i = 0; i < formula.length; i++) {
			f = formula[i];
			if (!invalidDict[f]) {
				fields.push(f);
			}
		}

		let fieldCount = fields.length;
		// formula = payload.formula.trim().split(",TR"); // TODO: split wit

		rows.push([
			{
				"h": true,
				"i": "instrument",
				"v": "Instrument"
			},
			{
				"h": true,
				"i": "date",
				"v": "Date"
			}
		]);

		for (let index = 0; index < fieldCount; index++) {
			let timeSeriesField = fields[index];

			let regex = /TR.(.*)(?=\()/;
			let result = timeSeriesField.match(regex);
			let field = result[0].replace("TR.", "");
			let phrase = timeSeriesField.toLowerCase();
			let startDateRegex = /sdate=(.*)[,]/;
			let endDateRegex = /edate=(.*)[)]/;
			let startDateMatch = phrase.match(startDateRegex);
			let endDateMatch = phrase.match(endDateRegex);

			let startDate = startDateMatch ? startDateMatch[1] : DateTime.format(new Date(), "YYYY-MM-DD");
			let endDate = endDateMatch[1];

			let swapDateTmp;
			if(startDate > endDate) {
				swapDateTmp = startDate;
				startDate = endDate;
				endDate = swapDateTmp;
			}

			let msBetweenDate = (+new Date(endDate)) - (+new Date(startDate));

			let msInDay = 1000 * 60 * 60 * 24;
			let betweenDay = msBetweenDate / msInDay;
			// let betweenDay = Math.floor(Math.random() * 10); // For random length


			for (i = 0; i <= betweenDay; i++) {
				let date = new Date(new Date(startDate).setDate(new Date(startDate).getDate() + i));
				let fieldValue = {
					"h": true,
					"v": DateTime.format(date, "YYYY-MM-DDT00:00:00")
				};
				if(index === 0) { // add header 1 time
					rows[0].push(fieldValue);
				}
			}

			let rics = identifiers;
			for (i = 0; i < rics.length; i++) {

				let ric = rics[i];
				let rowValue = [
					ric,
					field
				];
				for (j = 1; j <= betweenDay + 1; j++) {
					let generatedValue = DataGenerator.generateRecord(field);
					rowValue.push(generatedValue[field]);

				}
				rows.push(rowValue);
			}
			// The example rows should be look like
			/*
			[
				[
				{
					"h": true,
					"i": "instrument",
					"v": "Instrument"
				},
				{
					"h": true,
					"i": "date",
					"v": "Date"
				},
				{
					"h": true,
					"v": "2011-08-11T00:00:00"
				},
				{
					"h": true,
					"v": "2011-08-12T00:00:00"
				},
				{
					"h": true,
					"v": "2011-08-15T00:00:00"
				}
				],
				[
				"IBM",
				"Price Close", // NOTE: this cannot be detech we join it with space Ex. PriceClose
				159.25449372,
				160.6585848,
				165.23382036
				],
				[
				"GOOGL.O",
				"Price Close",
				14.066881653,
				14.107921423,
				13.944262828
				]
			]
			*/
		}

	} else {

		identifiers = payload.identifiers;
		formula = Adc.splitFields(payload.formula);
		fields = [];

		// invalidFieldDict is a dictionary of non exist field
		// so we must remove invalid field to make "mocking api" return result more like a "real api".
		invalidDict = invalidFieldDict;
		for (i = 0; i < formula.length; i++) {
			f = formula[i];
			if (!invalidDict[f]) {
				fields.push(f);
			}
		}

		rows[0] = [{
			"h": true,
			"i": "instrument",
			"v": "Instrument"
		}];
		for (i = 0; i < fields.length; i++) {
			f = fields[i];
			rows[0].push({
				"h": true,
				"r": f.toUpperCase().trim()
				// "v": "Price Close", // Doesn't use
				// "p": "I_TRP_PH_PriceClose" // Doesn't use this property
			});
		}

		// build data
		let rowMap = {};

		len = identifiers.length;
		let rowData = dataGen.generate(fields, len);
		for (i = 0; i < len; ++i) {
			let inst = identifiers[i];
			row = rowMap[inst];
			if (!row) {
				row = rowMap[inst] = rowData[i];
				row.unshift(inst); // prepend instrument on each row
			}
			rows.push(row);
		}

		// There is a chance that rtk will return multiple row data per instrument
		// so we must create mock up for this case
		if (rows.length > 0) {
			let chance = dataGen.randInt(1, 10);
			if (chance <= 3) { // chance 30%
				let pos = dataGen.randInt(0, rows.length - 1); // random row pos
				row = rows[pos];
				len = row.length;
				let mockupRow = new Array(len);
				mockupRow[0] = row[0]; // 1st index is for instrument
				for (i = 1; i < len; i++) {
					mockupRow[i] = ''; // real case will return null or empty string
				}
				rows.splice(pos + 1, 0, mockupRow);
			}
		}
	}


	/*
	response type is :
	{
		cols: number,
		faults: Array<obj>,
		rows: Array<obj>,
		status: string
	}
	Currently, we use only the rows property for cache ADC data
	*/
	return Promise.resolve(JSON.stringify({
		rows: rows
		// TODO : supports faults property
	}));
};

/**
 * Splits a string of comma-separated fields into an array of individual field names with optional parentheses and contents.
 *
 * @param {string} strFields - The string of comma-separated fields to split.
 * @returns {Array<string>} - An array of individual field names with optional parentheses and contents.
 */
Adc.splitFields = function(strFields) {
	 if(!strFields) {
		return [];
	 }

	// Split the input string using the regular expression
	// regex is match commas that are not inside parentheses
	/*
		,          		- matches a comma
		(?![^()]*\) 	- negative lookahead assertion that matches if the comma is not followed by:
			[^()]*   	- any characters that are not opening or closing parentheses
			\)			- a closing parenthesis
		)          		- ending delimiter of the regular expression
	*/
	let fields = strFields.split(/,(?![^()]*\))/);
	fields = fields.map(function(field) {
		return field.trim();
	});

	return fields;
};



;// ./node_modules/tr-grid-util/es6/jet/mockDataAPI.js




let mockDataAPI_dataGen = new DataGenerator();

/** @private
* @namespace
*/
let DataGrid = {};

/** @private
* @function
* @param {Object} payload
* @param {Object=} mockResponse
* @return {Promise}
*/
DataGrid.request = function (payload, mockResponse) {
	if (mockResponse) {
		return Promise.resolve(JSON.stringify(mockResponse));
	}

	let i, f, len, row;
	let instruments = payload.instruments;
	let fields = payload.fields;

	// _invalidFieldDict is a dictionary of non exist field
	// so we must remove invalid field to make "mocking api" return result more like a "real api".
	let invalidDict = invalidFieldDict;
	fields = [];
	for (i = 0; i < payload.fields.length; i++) {
		f = payload.fields[i];
		if (!invalidDict[f.name]) {
			fields.push(f);
		}
	}

	// for filter duplicate instrument
	let rowMap = {};

	// build data
	len = instruments.length;
	let data2D = [];
	let rowData = mockDataAPI_dataGen.generate(fields, len);
	for (i = 0; i < len; ++i) {
		let inst = instruments[i];
		row = rowMap[inst];
		if (!row) {
			row = rowMap[inst] = rowData[i];
			row.unshift(inst); // prepend instrument on each row
		}
		data2D.push(row);
	}

	// There is a chance that rtk will return multiple row data per instrument
	// so we must create mock up for this case
	if (data2D.length > 0) {
		let chance = mockDataAPI_dataGen.randInt(1, 10);
		if (chance <= 3) { // chance 30%
			let pos = mockDataAPI_dataGen.randInt(0, data2D.length - 1); // random row pos
			row = data2D[pos];
			len = row.length;
			let mockupRow = new Array(len);
			mockupRow[0] = row[0]; // 1st index is for instrument
			for (i = 1; i < len; i++) {
				mockupRow[i] = ''; // real case will return null or empty string
			}
			data2D.splice(pos + 1, 0, mockupRow);
		}
	}

	// build header
	let headers = [{
		"displayName": "Instrument"
	}];
	for (i = 0; i < fields.length; i++) {
		f = fields[i];
		headers.push({
			"displayName": f.name,
			"field": f.name.toUpperCase() // server only return in uppercase
		});
	}

	return Promise.resolve(JSON.stringify({
		data: data2D,
		headers: [headers]
	}));
};

/** @public
* @function
* @param {string} dataType
* @return {Promise}
*/
let mockDataAPI = function (dataType) {
	if (dataType === "datagrid") {
		return Promise.resolve(DataGrid);
	}  else if(dataType === "adc") {
		return Promise.resolve(Adc);
	} else {
		return Promise.reject(dataType + " is not supported.");
	}
};

/** to mock invalid fields.
* @public
* @param {Array<string>} fields ex. TR.NonExistField, CF_IGNORE_FIELD
*/
mockDataAPI.setInvalidFields = setInvalidFields;



;// ./node_modules/tr-grid-util/es6/jet/MockJET.js





/** @constructor
* @param {Object=} options
*/
let MockJET = function(options) {
	this["Quotes"] = new MockQuotes();
	this["Quotes2"] = new MockQuotes2();
	this["Archive"] = new MockArchive();
	this["ContainerDescription"] = true;
	this["Data"] = mockDataAPI;
	if(options) {
		this["Quotes2"].config(options);
	}
};

/** @public
* @param {Object=} options
* @return {MockJET}
*/
MockJET.prototype.init = function(options) {
	if(options) {
		this["Quotes2"].config(options);
	}
	return this;
};

/** @public
* @param {Function} handler
* @return {MockJET}
*/
MockJET.prototype.onLoad = function(handler) {
	setTimeout(handler, 2000);
	return this;
};

/** @public
* @param {Object} obj
* @return {MockJET}
*/
MockJET.prototype.navigate = function(obj) {
	return this;
};
/** @public
* @param {string} name
* @param {string} type
* @param {string} property
*/
MockJET.prototype.appHit = function(name, type, property) {
};

/** @public
* @return {Promise}
*/
MockJET.prototype.getUserInfo = function() {
	return Promise.resolve({
		UUID: 'PAXTRA'
	});
};

/** @public
* @param {Array} contextAry
*/
MockJET.prototype.contextChange = function(contextAry) {
};

/* harmony default export */ const jet_MockJET = ((/* unused pure expression or super */ null && (MockJET)));


;// ./node_modules/tr-grid-util/es6/TickCodes.js
/** @type {Object.<string, number>}
* @public
*/
let TickCodes = {
	"\u21e7": 1,
	"B\u21e7": 1,
	"\u2191": 1,
	"B\u2191": 1,
	"26": 1,
	"1": 1,
	"\"1\"": 1,
	"-1": -1,
	"\u21e9": -1,
	"B\u21e9": -1,
	"\u2193": -1,
	"B\u2193": -1,
	"27": -1,
	"2": -1,
	"\"2\"": -1
};

/** @type {Object.<string, number>}
* @public
*/
let TickFields = {
	"CF_TICK": 1,
	"PRCTCK_1": 1,
	"BID_TICK": 1,
	"ASK_TICK": 1
};

/* harmony default export */ const es6_TickCodes = ((/* unused pure expression or super */ null && (TickCodes)));


;// ./node_modules/tr-grid-util/es6/ExpressionParser.js
/** @type {Object.<string, number>}
* @private
* @const
*/
let OperatorPrecedences = {
	"||": 1,
	"&&": 2,
	"<": 3,
	">": 3,
	"<=": 3,
	">=": 3,
	"!=": 3,
	"=": 3,
	"==": 3,
	"===": 3,
	"+": 4,
	"-": 4,
	"*": 5,
	"%": 6,
	"/": 6,
	"!": 7,
	"!!": 7
};
/** @type {Object.<string, Function>}
* @private
* @const
*/
let OperatorFunctions = {
	"+": function(lhs, rhs){ return lhs + rhs; },
	"-": function(lhs, rhs){ return lhs - rhs; },
	"*": function(lhs, rhs){ return lhs * rhs; },
	"/": function(lhs, rhs){ return lhs / rhs; },
	"%": function(lhs, rhs){ return lhs % rhs; },
	"<": function(lhs, rhs){ return lhs < rhs; },
	">": function(lhs, rhs){ return lhs > rhs; },
	"<=": function(lhs, rhs){ return lhs <= rhs; },
	">=": function(lhs, rhs){ return lhs >= rhs; },
	"==": function(lhs, rhs){ return lhs == rhs; },
	"!=": function(lhs, rhs){ return lhs != rhs; },
	"&&": function(lhs, rhs){ return lhs && rhs; },
	"||": function(lhs, rhs){ return lhs || rhs; },
	"!": function(lhs, rhs){ return !lhs; },
	"!!": function(lhs, rhs){ return !!lhs; }
};
/** @type {Object.<string, number>}
* @private
* @const
*/
let OperandRequirements = {
	"+": 2,
	"-": 2,
	"*": 2,
	"/": 2,
	"%": 2,
	"<": 2,
	">": 2,
	"<=": 2,
	">=": 2,
	"==": 2,
	"!=": 2,
	"&&": 2,
	"||": 2,
	"!": 1,
	"!!": 1
};
/** @private
* @function
* @param {Array.<Object>} ary
* @param {string|number} value
* @param {string} type
* @param {string=} dataType
* @return {number}
*/
let _addToken = function(ary, value, type, dataType) {
	let tok = {
		value: value,
		type: type
	};
	if(dataType) {
		tok.dataType = dataType;
	}
	if(type === "operator") {
		tok.precedence = OperatorPrecedences[value];
	} else if(type === "parenthesis") {
		tok.precedence = 0;
	}
	ary.push(tok);
	return ary.length - 1;
};
/** @private
* @function
* @param {Object} obj
* @returns {*}
*/
let _toValue = function(obj) { // eslint-disable-line
	return obj.value;
};

/** @public
* @namespace
*/
let ExpressionParser = {};
/** @type {Array.<Object>}
* @private
* @const
*/
ExpressionParser._tokens = [];


/** @private
* @function
* @param {string} str
* @return {string}
*/
ExpressionParser._withString = function(str) {
	let tokId = _addToken(ExpressionParser._tokens,
		str.substring(1, str.length - 1),
		"literal",
		"string"
	);
	return " #" + tokId + "#";
};
/** @private
* @function
* @param {string} field
* @return {string}
*/
ExpressionParser._withField = function(field) {
	let tokId = _addToken(ExpressionParser._tokens,
		field,
		"literal",
		"variable"
	);
	return " #" + tokId + "#";
};
/** @private
* @function
* @param {string} field
* @return {string}
*/
ExpressionParser._withFieldInBrackets = function(field) {
	return ExpressionParser._withField(field.substring(1, field.length - 1));
};
/** @private
* @function
* @param {string} numStr
* @return {string}
*/
ExpressionParser._withNegativeNumber = function(numStr) {
	let num = +numStr;
	if(num === num) {
		let tokId = _addToken(ExpressionParser._tokens,
			num,
			"literal",
			"number"
		);
		return " #" + tokId + "#";
	}
	return "";
};
/** @private
* @function
* @param {string} numStr
* @return {string}
*/
ExpressionParser._withPositiveNumber = function(numStr) {
	if(numStr.charCodeAt(0) === 35) {
		return numStr; // skip token (string with leading #)
	}
	return ExpressionParser._withNegativeNumber(numStr);
};
/** @private
* @function
* @param {string} opStr
* @return {string}
*/
ExpressionParser._withOperator = function(opStr) {
	if(OperatorPrecedences[opStr]) {
		if(opStr == "=" || opStr == "===") {
			opStr = "==";
		}
		let tokId = _addToken(ExpressionParser._tokens, opStr, "operator");
		return "#" + tokId + "#";
	}
	return "";
};
/** @private
* @function
* @param {string} paStr
* @return {string}
*/
ExpressionParser._withParenthesis = function(paStr) {
	let tokId = _addToken(ExpressionParser._tokens, paStr, "parenthesis");
	return "#" + tokId + "#";
};


/** @private
* @function
* @param {Object} ctx
* @return {boolean} Returns true if there is any error, otherwise returns false (everything is OK)
*/
ExpressionParser._tester = function(ctx) {
	let rpn = ctx._rpn;
	let inputCount = rpn.length;
	let operandCount = 0;
	let maxStack = 0;
	for(let i = 0; i < inputCount; ++i) {
		let tok = rpn[i];
		if(tok.type !== "literal") {
			let requiredCount = OperandRequirements[tok.value];
			if(operandCount >= requiredCount) {
				operandCount -= requiredCount;
			} else {
				console.warn("Cannot parse an expression with insufficient number of operands");
				return true;
			}
		}
		operandCount++;
		if(operandCount > maxStack) {
			maxStack = operandCount;
		}
	}

	if(operandCount !== 1) {
		return true;
	}

	ctx._stackSize = maxStack;
	return false;
};
/** @private
* @function
* @param {Object} ctx
* @param {Object} rowData
* @return {boolean}
*/
ExpressionParser._filter = function(ctx, rowData) {
	let rpn = ctx._rpn;
	let results = new Array(ctx._stackSize);
	let inputCount = rpn.length;
	let operandCount = 0;
	let curRes;
	for(let i = 0; i < inputCount; ++i) {
		let tok = rpn[i];
		let tokValue = tok.value;
		if(tok.type === "literal") {
			if(tok.dataType !== "variable") {
				curRes = tokValue;
			}	else {
				curRes = rowData ? rowData[tokValue] : null;
			}
		} else { // operator
			if(OperandRequirements[tokValue] === 1) {
				curRes = OperatorFunctions[tokValue](
					results[operandCount - 1]
				);
				operandCount -= 1;
			} else { // 2
				curRes = OperatorFunctions[tokValue](
					results[operandCount - 2],
					results[operandCount - 1]
				);
				operandCount -= 2;
			}
		}
		results[operandCount++] = curRes;
	}
	// return results[0];
	return results[0] ? true : false;
};

/** Parse an expression into a filter function (returning boolean value)
* @public
* @param {string|Function} expression
* @return {Function}
* @example
* func = ExpressionParser.parse("field1 >= 10");
* func = ExpressionParser.parse("[field with space] != 'text'");
* func = ExpressionParser.parse("(field1 + 1) / 2 < 10 && field2 - 10 > 3");
*/
ExpressionParser.parse = function(expression) {
	if(typeof expression === "function") {
		return  expression;
	}
	if(!expression || typeof expression !== "string") {
		return null;
	}

	let tokExp = expression.trim();
	if(!tokExp) {
		return null;
	}
	ExpressionParser._tokens.length = 0;

	tokExp = tokExp.replace(/".+?"/g, ExpressionParser._withString); // double quote string
	tokExp = tokExp.replace(/'.+?'/g, ExpressionParser._withString); // single quote string
	tokExp = tokExp.replace(/\[.+?\]/g, ExpressionParser._withFieldInBrackets); // field in square brackets
	tokExp = tokExp.replace(/-[0-9.]+/g, ExpressionParser._withNegativeNumber);
	tokExp = tokExp.replace(/[0-9.#]+/g, ExpressionParser._withPositiveNumber);
	tokExp = tokExp.replace(/[a-zA-Z_]\w*/g, ExpressionParser._withField);
	tokExp = tokExp.replace(/[\-\/+*%!=<>&|]+/g, ExpressionParser._withOperator);
	tokExp = tokExp.replace(/[()]/g, ExpressionParser._withParenthesis); // round brackets

	let tokens = ExpressionParser._tokens;
	let tokCount = tokens.length;
	if(!tokCount) {
		return null; // There is no recognizable character
	}

	// Generate infix token list
	let infixTokens = [];
	let matches = tokExp.match(/#[0-9]+#/g);
	let i, tok, tokType;
	for(i = 0; i < tokCount; ++i) {
		let tokId = matches[i];
		let tokIdx = +(tokId.substring(1, tokId.length - 1));
		tok = tokens[tokIdx];
		tokType = tok.type;
		if(tokType === "literal") {
			let prevTok = infixTokens[infixTokens.length - 1];
			if(prevTok && prevTok.type === "literal") { // Incorrect infix notation detected
				if(tok.value < 0 && tok.dataType === "number") {
					_addToken(infixTokens, "-", "operator");
					tok.value *= -1;
				} else {
					console.warn("Cannot parse an expression with insufficient number of operators");
					return null;
				}
			}
		}
		infixTokens.push(tok);
	}
	ExpressionParser._tokens.length = 0;

	// Convert Infix notation to Reverse Polish Notation (Postfix)
	// See https://en.wikipedia.org/wiki/Shunting_yard_algorithm#The_algorithm_in_detail
	tokCount = infixTokens.length;
	let rpn = [];
	let operators = [];
	let lastOp = null;
	let foundLeftPa = false;
	for(i = 0; i < tokCount; ++i) {
		tok = infixTokens[i];
		tokType = tok.type;

		if(tokType === "literal") {
			rpn.push(tok);
		} else if(tokType === "parenthesis") {
			if(tok.value === "(") {
				operators.push(tok);
			} else { // When finding right parenthesis, pop all operators until left parenthesis
				foundLeftPa = false;
				while(operators.length) {
					lastOp = operators.pop();
					if(lastOp.value === "(" && lastOp.type === "parenthesis") {
						foundLeftPa = true;
						break;
					} else {
						rpn.push(lastOp);
					}
				}
				if(!foundLeftPa) {
					console.warn("No left parenthesis paired with the right one");
				}
			}
		} else { // operator
			while(operators.length) {
				if(tok.precedence > operators[operators.length - 1].precedence) {
					break;
				}
				rpn.push(operators.pop());
			}
			operators.push(tok);
		}
	}

	foundLeftPa = false;
	while(operators.length) {
		lastOp = operators.pop();
		if(lastOp.value === "(" && lastOp.type === "parenthesis") {
			foundLeftPa = true;
		} else {
			rpn.push(lastOp);
		}
	}
	if(foundLeftPa) {
		console.warn("No right parenthesis paired with the left one");
	}

	let inputCount = rpn.length;
	if(!inputCount) {
		return null; // The expression contains nothing but parentheses
	}

	let ctx = {
		_rpn: rpn
	};

	if(ExpressionParser._tester(ctx)) {
		return null; // Insufficient number of operands
	}

	return ExpressionParser._filter.bind(null, ctx);
};

/* harmony default export */ const es6_ExpressionParser = ((/* unused pure expression or super */ null && (ExpressionParser)));


;// ./node_modules/tr-grid-util/es6/CellPainter.js





/** @typedef {Object} CellPainter~Expression
* @ignore
* @description This form of expression is deprecated
* @property {string=} name
* @property {string} text Text describes condition e.g. [FIELD] > 10
* @property {Array=} values
*/

/** @typedef {Object} CellPainter~Condition
* @property {string|Function} expression A function, text, or an object with text property that describes the condition
* @property {string=} field Field to be used in the condition
* @property {string=} backgroundColor CSS color string
* @property {string=} color CSS color string
* @property {string=} fontSize
* @property {string=} fontWeight
* @property {string=} fontStyle
* @property {string=} textAlign
* @property {string=} textDecoration
* @property {string=} cssClass Predefined color class name
*/

/** @typedef {Object.<string, string>} CellPainter~ThemeColors
* @description Theme colors in hex format. All of the values are optional
* @property {string=} baseGrid Background color of the grid element
* @property {string=} baseText Text color of the grid element
* @property {string=} up Color for positive value
* @property {string=} down Color for negative value
* @property {string=} level Color for neutral/unchanged value
* @property {string=} tickUp Color for positive tick icon
* @property {string=} tickDown Color for negative tick icon
*/

/** @constructor
*/
let CellPainter = function() {
	this._conditions = [];
	this._scopes = [];

	CellPainter._painters.push(this); // For later referencing
};
/** @private
* @function
* @param {Element} elem
* @param {string=} targetClass
*/
let _removeCssClass = function(elem, targetClass) {
	if (elem._coloringCssClass != null && elem._coloringCssClass !== targetClass) {
		elem.classList.remove(elem._coloringCssClass);
		elem._coloringCssClass = null;
	}
};

/** Enum for coloring types.
* @enum {number}
*/
CellPainter.ColoringTypes = {
	TEXT: 1,
	HEATMAP: 2,
	CONDITIONAL: 3
};

/** @type {CellPainter.ColoringTypes}
* @private
*/
CellPainter.prototype._coloringType = 0;
/** @type {Array.<Object>}
* @private
*/
CellPainter.prototype._conditions = null;
/** @type {Object}
* @private
*/
CellPainter.prototype._effectiveStyles = null;
/** @type {Object}
* @private
*/
CellPainter.prototype._blinkCondition = null;
/** @type {!Array}
* @private
*/
CellPainter.prototype._scopes;
/** @type {Object}
* @private
*/
CellPainter.prototype._columnStats = null;
/** @type {number}
* @private
*/
CellPainter.prototype._blinkingDuration = 250;

/** @type {number}
* @private
*/
CellPainter._runningBlinkId = 0;

/** @type {Array.<CellPainter>}
* @private
* @const
*/
CellPainter._painters = [];

/** @type {!Array}
* @private
* @const
*/
CellPainter._emptyAry = [];
/** @type {!Object}
* @private
* @const
*/
CellPainter._emptyObj = {};
/** @type {!Object}
* @private
* @const
*/
CellPainter._bgObj = {"backgroundColor": "", "color": ""};
/** @type {!Object}
* @private
* @const
*/
CellPainter._colorObj = {"color": ""};
/** @type {!Array.<string>}
* @private
* @const
*/
CellPainter.bgStyles = ["backgroundColor", "color"];
/** @type {!Array.<string>}
* @private
* @const
*/
CellPainter.nonBgStyles = ["fontSize", "fontWeight", "fontStyle", "textAlign", "textDecoration"];
/** @type {!Array.<string>}
* @private
* @const
*/
CellPainter.supportedStyles = CellPainter.bgStyles.concat(CellPainter.nonBgStyles);
/** @type {Promise<Object>}
* @public
*/
CellPainter.themeReady = null;
/** @type {Object}
* @private
*/
CellPainter._contrastColors = {};

/** Deprecated in favor of ExpressionParser
* @public
* @function
* @param {string|Function} expression
* @return {Function}
* @example
* let fn = CellPainter.parse("[CF_BID] >= 10 && [CF_BID] <= 100");
* window.console.log(fn(25));
*/
CellPainter.parse = ExpressionParser.parse;


/** @public */
CellPainter.prototype.dispose = function() {
	let at = CellPainter._painters.indexOf(this);
	if(at >= 0) {
		CellPainter._painters.splice(at, 1);
	}
	this.reset(); // Restore all styles and clear all blink timers

	this._conditions = null;
	this._scopes.length = 0;

	this._blinkCondition = null;
};
/** @public */
CellPainter.prototype.reset = function() {
	this._setColoringType(0);
	this.clearBlinking();
};

/** @public */
CellPainter.prototype.resetColoring = function() {
	// reset coloring type to allow other extensions to be able to set their new mode
	this._setColoringType(0);
	this._effectiveStyles = null;
	this._conditions.length = 0;
};

/** @public */
CellPainter.prototype.clearBlinking = function() {
	this._blinkCondition = null;
	let len = this._scopes.length;
	if(len) {
		for(let i = 0; i < len; ++i) {
			let scope = this._scopes[i];
			if(CellPainter._clearBlinkTimer(scope)) {
				scope._restorer();
			}
			let cell = scope["cell"];
			if(cell) { // cell may be removed before
				delete cell["blinking"]; // Delete cell blinking scope
			}

			scope["cell"] = null;
		}
		this._scopes.length = 0;
	}
};
/** @public
* @ignore
* @function
*/
CellPainter.prototype.resetBlinking = CellPainter.prototype.clearBlinking;

/** Return true if no one is using this painter
 * @public
 * @return {boolean}
*/
CellPainter.prototype.clearHeatMap = function() {
	if(this._blinkCondition) {
		this._setColoringType(0);
		this.resetHeatMap();
		return false;
	} else {
		this.dispose();
		return true;
	}
};
/** @public
* @ignore
* @function
*/
CellPainter.prototype.resetHeatMap = function() {
	this._conditions.length = 0;
	this._columnStats = null;
};

/** @public */
CellPainter.prototype.applyThemeColor = function() {
	let colors = ElfUtil.getColors();
	if(this._coloringType === CellPainter.ColoringTypes.HEATMAP) { // Heatmap uses grid color to blend the result color
		let cond = this._conditions[0]; // Heatmap, color text condition must be the first
		if(cond) {
			if(cond["textMode"]) {
				cond["baseColor"] = colors["baseText"];
			} else {
				cond["baseColor"] = colors["baseGrid"];
			}
			if(cond["useThemeColor"]) {
				let up = colors["up"];
				let down = colors["down"];
				let level = (colors["level"] === colors["baseText"]) ? "" : colors["level"];
				CellPainter._setUpDownColors(cond, up, down, level);
			}
		}
	}

	if(this._blinkCondition && this._blinkCondition["useThemeColor"]) {
		CellPainter._setUpDownColors(this._blinkCondition, colors["up"], colors["down"], colors["level"], true);
	}
};
/** Used by monitor saving
* @public
* @return {Array.<Object>}
*/
CellPainter.prototype.getConditions = function() {
	return this._conditions;
};
/** Used by monitor saving <br>
* Returns the coloring type ("text", "heatmap", "conditional")
* @public
* @return {CellPainter.ColoringTypes}
*/
CellPainter.prototype.getColoringType = function() {
	return this._coloringType;
};
/** @private
* @param {number} enumType Available values are TEXT, HEATMAP, CONDITIONAL
*/
CellPainter.prototype._setColoringType = function(enumType) {
	if(this._coloringType !== enumType) { // We can only have one type of coloring
		this._coloringType = enumType;
	}
};
/** @public
* @param {Object} mapping Effective styles mapping
*/
CellPainter.prototype.setEffectiveStyles = function(mapping) {
	this._effectiveStyles = mapping;
};
/** @public
* @return {Object}
*/
CellPainter.prototype.getEffectiveStyles = function() {
	return this._effectiveStyles;
};
/** @public
* @param {Array.<CellPainter~Condition>} conditions
*/
CellPainter.prototype.setConditions = function(conditions) {
	this._conditions.length = 0; // TODO: Clear existing cell styles first
	this._setColoringType(CellPainter.ColoringTypes.CONDITIONAL);

	let len = conditions ? conditions.length : 0;
	for (let i = 0; i < len; i++) {
		this._addCondition(conditions[i]);
	}
};
/** @public
* @param {Object} columnStats
*/
CellPainter.prototype.setColumnStats = function(columnStats) {
	this._columnStats = (columnStats && columnStats.getMin) ? columnStats : null;
};
/** @public
* @return {Object}
*/
CellPainter.prototype.getColumnStats = function() {
	return this._columnStats;
};
/** @private
* @param {CellPainter~Condition} condition
*/
CellPainter.prototype._addCondition = function(condition) {
	let exp = condition["expression"];
	if(exp) {
		condition._fn = ExpressionParser.parse(exp["text"] || exp);
		this._conditions.push(condition);
	}
};
/** @public
* @param {string} field
* @param {number} midPoint
* @param {string} upColor
* @param {string} downColor
* @param {string} baseColor
* @param {(string|boolean)=} opt_textMode
* @return {!Object}
*/
CellPainter.prototype.addHeatmap = function(field, midPoint, upColor, downColor, baseColor, opt_textMode) {
	this._setColoringType(CellPainter.ColoringTypes.HEATMAP);

	let condition = {};
	condition["heatmap"] = true;
	condition["field"] = field;
	condition["expression"] = CellPainter._heatMapCondition.bind(condition);
	condition["midPoint"] = midPoint;
	CellPainter._setUpDownColors(condition, upColor, downColor);

	condition["baseColor"] = baseColor;
	if(opt_textMode === true || opt_textMode === "text") {
		condition["textMode"] = true;
	}

	this._addCondition(condition);
	return condition;
};
/** @public
* @param {string} field
* @param {number} midPoint
* @param {(string|boolean)=} opt_textMode
* @return {!Object}
*/
CellPainter.prototype.addHeatmapWithTheme = function(field, midPoint, opt_textMode) {
	let colors = ElfUtil.themeColors;
	let condition = this.addHeatmap(
		field,
		midPoint,
		colors["up"], colors["down"], colors["baseGrid"],
		opt_textMode
	);

	if(opt_textMode === true || opt_textMode === "text") {
		condition["baseColor"] = colors["baseText"];
	}
	condition["useThemeColor"] = true;
	return condition;
};
/** @private
* @param {Function} expression
* @param {string} field
* @param {string=} upClass CSS class name
* @param {string=} downClass CSS class name
* @param {string=} levelClass CSS class name
* @return {!Object}
*/
CellPainter.prototype._addColorText = function(expression, field, upClass, downClass, levelClass) {
	this._setColoringType(CellPainter.ColoringTypes.TEXT);

	let condition = {};
	condition["field"] = field;
	condition["expression"] = expression.bind(null, field);
	condition["upClass"] = upClass;
	condition["downClass"] = downClass;
	condition["levelClass"] = levelClass;
	 // TODO: Make this consistent with other type (e.g., heat map and blinking)

	this._addCondition(condition);
	return condition;
};
/** @public
* @param {string} field
* @param {string=} upClass CSS class name
* @param {string=} downClass CSS class name
* @param {string=} levelClass CSS class name
* @return {!Object}
*/
CellPainter.prototype.addColorText = function(field, upClass, downClass, levelClass) {
	return this._addColorText(
		TickFields[field] ? CellPainter._tickColorTextCondition : CellPainter._colorTextCondition,
		field,
		upClass, downClass, levelClass
	);
};
/** @public
* @param {string} field
* @return {!Object}
*/
CellPainter.prototype.addColorTextWithTheme = function(field) {
	let condition = this.addColorText(field, "positive", "negative", "neutral");
	return condition;
};
/** @public
* @param {string} upColor
* @param {string} downColor
* @param {string} levelColor
* @return {!Object}
*/
CellPainter.prototype.addTickColorText = function(upColor, downColor, levelColor) {
	return this.addColorText("CF_TICK", upColor, downColor, levelColor);
};
/** @public
* @return {!Object}
*/
CellPainter.prototype.addTickColorTextWithTheme = function() {
	let colors = ElfUtil.themeColors;
	let condition = this.addTickColorText(colors["tickUp"], colors["tickDown"], colors["level"]);
	condition["useThemeColor"] = true;
	return condition;
};
/** @public
* @param {string} upColor
* @param {string} downColor
* @param {string} levelColor
* @param {(string|boolean)=} opt_border
* @return {!Object}
*/
CellPainter.prototype.addTickBlink = function(upColor, downColor, levelColor, opt_border) {
	return this.addBlink("CF_TICK", upColor, downColor, levelColor, opt_border);
};
/** @public
* @param {(string|boolean)=} opt_border
* @return {!Object}
*/
CellPainter.prototype.addTickBlinkWithTheme = function(opt_border) {
	let colors = ElfUtil.themeColors;
	let condition = this.addTickBlink(colors["tickUp"], colors["tickDown"], colors["level"], opt_border);
	condition["useThemeColor"] = true;
	return condition;
};
/** @public
* @param {string} field
* @param {string} upColor
* @param {string} downColor
* @param {string} levelColor
* @param {(string|boolean)=} opt_border
* @return {!Object}
*/
CellPainter.prototype.addBlink = function(field, upColor, downColor, levelColor, opt_border) {
	return this._addBlink(
		TickFields[field] ? CellPainter._tickBlinkCondition : CellPainter._fieldBlinkCondition,
		field,
		upColor, downColor, levelColor,
		opt_border
	);
};
/** @public
* @param {string} field
* @param {(string|boolean)=} opt_border
* @return {!Object}
*/
CellPainter.prototype.addBlinkWithTheme = function(field, opt_border) {
	let colors = ElfUtil.themeColors;
	let condition = this.addBlink(field, colors["up"], colors["down"], colors["level"], opt_border);
	condition["useThemeColor"] = true;
	return condition;
};
/** @private
* @param {Function} expression
* @param {string} field
* @param {string} upColor
* @param {string} downColor
* @param {string} levelColor
* @param {(string|boolean)=} blinkType
* @return {!Object}
*/
CellPainter.prototype._addBlink = function(expression, field, upColor, downColor, levelColor, blinkType) {
	let condition = this._blinkCondition = {};
	condition["blink"] = true;
	condition["blinkId"] = CellPainter._runningBlinkId++;
	condition["field"] = field;
	condition._fn = condition["expression"] = expression;

	if(blinkType === true || blinkType === "border") {
		condition["border"] = true;
	}
	CellPainter._setUpDownColors(condition, upColor, downColor, levelColor, true);
	return condition;
};

/** @public
* @param {Element} cell
* @param {Object} rowData e.g. { PCTCHNG: 0.53, CF_NETCHNG: 0.75 }
* @param {number} min
* @param {number} max
*/
CellPainter.prototype.renderForPrinting = function(cell, rowData, min, max) {
	if(!cell) {
		return;
	}
	let styles = this._getStyles(rowData, min, max);
	let cssClass = styles["cssClass"]; // Can be an empty string

	_removeCssClass(cell, cssClass);
	if (cssClass != null) { // Predefined colors mode
		if (cssClass) {
			cell.classList.add(cssClass);
			cell._coloringCssClass = cssClass;
		}
	} else {
		cell.style.backgroundColor = styles["backgroundColor"] || "";
		cell.style.color = styles["color"] || "";
	}
};


/** @private
* @param {string} field
* @param {Object} rowData
* @return {string|number}
*/
CellPainter._colorTextCondition = function(field, rowData) {
	let value = rowData[field];
	if(value || value === 0) {
		if(value > 0) {
			return 1;
		} else if(value < 0) {
			return -1;
		}
	}
	return "u"; // unchanged
};
/** @private
* @param {string} field
* @param {Object} rowData
* @return {undefined|number}
*/
CellPainter._tickColorTextCondition = function(field, rowData) {
	return TickCodes[rowData[field]];
};
/** @private
* @this {Object}
* @param {Object} rowData
* @param {number} min
* @param {number} max
* @return {number}
*/
CellPainter._heatMapCondition = function(rowData, min, max) {
	let value = rowData[this["field"]];
	let diff = (value || value === 0) ? value - this["midPoint"] : NaN;
	if (diff === 0 || diff !== diff) {
		return 0;
	}

	let ratio;
	let range = 1;
	if(diff > 0) {
		range = max - this["midPoint"];
		if(range < 0) {
			range = min - this["midPoint"];
		}
	} else {
		range = min - this["midPoint"];
		if(range > 0) {
			range = max - this["midPoint"];
		}
	}
	if(range < 0) {
		range = -range;
	}
	ratio = diff / range;
	return ratio;
};
/** @private
* @function
* @param {string} newValue
* @return {undefined|number}
*/
CellPainter._tickBlinkCondition = function(newValue) {
	return TickCodes[newValue];
};
/** @private
* @function
* @param {number} newValue
* @param {number} oldValue
* @return {number}
*/
CellPainter._fieldBlinkCondition = function(newValue, oldValue) {
	if(newValue > oldValue) {
		return 1; // eslint-disable-line
	} else if(newValue < oldValue) {
		return -1; // eslint-disable-line
	} else { // AlphaNumeric, unchanged newValue
		return 0; // eslint-disable-line
	}
};
/** @private
* @param {Object} scope
*/
CellPainter._borderRestorer = function(scope) {
	scope["blinkTimer"] = 0;
	let cell = scope["cell"];
	let elem = cell.getElement();

	if(elem) {
		elem.style.border = "";
	}
};
/** @private
* @param {Object} scope
*/
CellPainter._cellRestorer = function(scope) {
	scope["blinkTimer"] = 0;
	let cell = scope["cell"];
	let elem = cell.getElement();

	if(elem) {
		let rowData = scope["rowData"];
		let min = NaN;
		let max = NaN;
		if(this._coloringType === CellPainter.ColoringTypes.HEATMAP) {
			let columnStats = this._columnStats;
			if(columnStats) {
				min = columnStats.getMin();
				max = columnStats.getMax();
			}
		}

		let styles = this._getStyles(rowData, min, max);
		let cssClass = styles["cssClass"]; // Can be an empty string
		_removeCssClass(elem, cssClass);
		if (cssClass != null) { // Predefined colors mode
			if (cssClass) {
				elem.classList.add(cssClass);
				elem._coloringCssClass = cssClass;
			}
			// Remove blinking color
			elem.style.backgroundColor = "";
			elem.style.color = "";
		} else {
			elem.style.backgroundColor = styles["backgroundColor"] || "";
			elem.style.color = styles["color"] || "";
		}
	}
};

/** WARNING: This method is called very often and the performance is very critical here
* @private
* @param {Object} rowData
* @param {number=} min
* @param {number=} max
* @return {!Object}
*/
CellPainter.prototype._getStyles = function(rowData, min, max) {
	if (!rowData) {
		return CellPainter._emptyObj;
	}

	let len = this._conditions.length;
	for(let i = 0; i < len; ++i) {
		let curCond = this._conditions[i];
		let fn = curCond._fn;
		let ret;
		try { // WARNING: Try/Catch is very slow. TODO: The function should be checked for validity beforehand
			ret = fn && fn(rowData, min, max);
		} catch (err){
			// console.error(err.message); // This will produce tons of errors in the console
		}
		if (ret) { // Match condition with results 1, -1, true, or string
			if(this._coloringType === CellPainter.ColoringTypes.HEATMAP) {
				let blendedColor;
				if(ret > 0) {
					blendedColor = blendColor(curCond["baseColor"], curCond["upColor"], ret);
				} else {
					blendedColor = blendColor(curCond["baseColor"], curCond["downColor"], -ret);
				}

				if(curCond["textMode"]) {
					CellPainter._colorObj["color"] = num2Hex(blendedColor);
					return CellPainter._colorObj;
				} else {
					CellPainter._bgObj["backgroundColor"] = num2Hex(blendedColor);
					CellPainter._bgObj["color"] = getContrastColor(blendedColor);

					return CellPainter._bgObj;
				}
			} else if(this._coloringType === CellPainter.ColoringTypes.TEXT) {
				if(ret > 0) {
					curCond["cssClass"] = curCond["upClass"];
				} else if(ret < 0) {
					curCond["cssClass"] = curCond["downClass"];
				} else if(ret === 0) {
					curCond["cssClass"] = curCond["levelClass"];
				} // else case is no change in cssClass
				if(!curCond["cssClass"]) {
					curCond["cssClass"] = ""; // If no cssClass, remove existing one
				}
				return curCond;
			}

			// Expected return value is {
			//   "backgroundColor": bgc,
			//   "color": color,
			//   "fontSize": fSize,
			//   "fontWeight": fWeight,
			//   "fontStyle": fStyle,
			//   "textAlign": tAlign,
			//   "textDecoration": tDecor
			// };
			return curCond; // Conditional Coloring
		}
	}
	return CellPainter._emptyObj;
};

/** @public
* @function
* @param {string} baseColor
* @param {string} maxColor
* @param {number} ratio [0, 1]
* @return {Array.<number>} resultColor
*/
CellPainter.blendColor = blendColor; // For backward compatability
/** @private
* @function
* @param {Array.<number>} triplet
* @return {string} resultColor
*/
CellPainter.blackAndWhite = getContrastColor; // For backward compatability
/** @public
* @function
* @param {string} rgbCode
* @return {string} resultColor E.g. "#10FF0D"
*/
CellPainter.rgb2Hex = rgb2Hex; // For backward compatability
/** @public
* @function
* @param {Array.<number>} triplet
* @return {string} resultColor
*/
CellPainter.num2Hex = num2Hex; // For backward compatability
/** Note that Chrome, IE, and Firefox store color in rgb representation.
* @public
* @function
* @param {Array.<number>} triplet
* @return {string} Color string in RGB represetation (e.g. rgb(100, 44, 1))
*/
CellPainter.num2Rgb = num2Rgb; // For backward compatability
/** @public
* @function
* @param {string} hex
* @return {Array.<number>} array of size 3 which contains [red, green, blue]
*/
CellPainter.hex2Num = hex2Num; // For backward compatability
/** @public
* @function
* @param {string} hex Color string with leading # character (e.g. #FFAA00)
* @return {string} Color string in RGB represetation (e.g. rgb(100, 44, 1))
*/
CellPainter.hex2Rgb = hex2Rgb; // For backward compatability

/** Deprecated. Colors should be changed according to user's settings by CellPainter#loadThemeColors
* @private
* @function
*/
CellPainter.themeChanged = function() {};
/** Deprecated. Colors should be changed according to user's settings by CellPainter#loadThemeColors
* @public
* @function
*/
CellPainter.setThemeColors = function() {};
/** @private
* @function
*/
CellPainter._onThemeChanged = function() {
	// Apply newly changed theme to all the painters
	let painters = CellPainter._painters;
	for(let i = painters.length; --i >= 0;) {
		painters[i].applyThemeColor(); // Re-apply theme color
	}
};
/** Load theme colors and change colors for all instances of CellPainter
* @public
* @function
* @return {Promise<Object>}
*/
CellPainter.loadThemeColors = function() {
	if(!CellPainter.themeReady) {
		CellPainter.themeReady = ElfUtil.getThemeColors(CellPainter._onThemeChanged)
			.then(CellPainter._onThemeChanged)
			.catch(CellPainter._onThemeChanged);
	}
	return CellPainter.themeReady;
};

/** Reload theme colors and change colors for all instances of CellPainter
* @public
* @function
* @return {Promise<Object>}
*/
CellPainter.reloadThemeColors = function () {
	CellPainter.themeReady = ElfUtil.reloadThemeColors()
		.then(CellPainter._onThemeChanged)
		.catch(CellPainter._onThemeChanged);
	return CellPainter.themeReady;
};

/** @public
* @function
* @ignore
* @param {tr.grid.Cell} cell
* @param {Object=} styles
*/
CellPainter.clearCellStyle = function(cell, styles) {
	let elem = cell.getElement();
	if(!elem) {
		return; // Cell has been disposed
	}

	let scope = cell["blinking"];
	if(scope) { // Clear blinking scope
		CellPainter._clearBlinkTimer(scope);

		delete cell["blinking"]; // Delete cell blinking scope

		scope["cell"] = null;
		// WARNING: Scope is not removed from the this._scopes collection to speed things up
	}

	_removeCssClass(elem);

	if(!styles){
		styles = CellPainter.supportedStyles;
	}
	if(Array.isArray(styles)){
		for(let i = styles.length; --i >= 0;) {
			elem.style[styles[i]] = "";  // WARNING: Very slow
		}
	} else {
		for(let key in styles) {
			elem.style[key] = "";  // WARNING: Very slow
		}
	}
};
/** @private
* @param {!Object} scp Scope Object
* @param {string|null} upColor Color in hex representation
* @param {string|null} downColor
* @param {(string|null)=} levelColor
* @param {boolean=} withContrast
*/
CellPainter._setUpDownColors = function(scp, upColor, downColor, levelColor, withContrast) {
	scp["upColor"] = upColor;
	if(upColor != null) {
		if(withContrast) {
			scp["contrastUpColor"] = CellPainter.getOppositeColor(upColor);
		}
	}

	scp["downColor"] = downColor;
	if(downColor != null) {
		if(withContrast) {
			scp["contrastDownColor"] = CellPainter.getOppositeColor(downColor);
		}
	}

	scp["levelColor"] = levelColor;
	if(levelColor != null) {
		if(withContrast) {
			scp["contrastLevelColor"] = CellPainter.getOppositeColor(levelColor);
		}
	}
};
/** @private
* @param {!Object} scp Scope Object
* @param {boolean=} opt_restoreColor
* @return {boolean}
*/
CellPainter._clearBlinkTimer = function(scp, opt_restoreColor) {
	if(scp["blinkTimer"]) {
		clearTimeout(scp["blinkTimer"]);
		scp["blinkTimer"] = 0;
		return true;
	}
	return false;
};
/** @public
* @param {string|Array.<number>} hexCode
* @return {string}
*/
CellPainter.getOppositeColor = function (hexCode) {
	if(typeof hexCode === "string") {
		if(hexCode.charAt(0) === "#") {
			let contrastColor = CellPainter._contrastColors[hexCode];
			if (!contrastColor) {
				let triplet = hex2Num(hexCode);
				contrastColor = CellPainter._contrastColors[hexCode] = getContrastColor(triplet);
			}
			return contrastColor;
		}
	} else if(Array.isArray(hexCode)) {
		return getContrastColor(hexCode);
	}
	return "";
};

/**
* @public
* @param {tr.grid.Cell} cell
* @param {Object} rowData e.g. { PCTCHNG: 0.53, CF_NETCHNG: 0.75 }
* @param {number} min
* @param {number} max
*/
CellPainter.prototype.render = function (cell, rowData, min, max) {
	if (min != null && min === min) { // Render heatmap
		this._paintCell(cell, rowData, min, max);
	} else if (this._conditions.length && this._coloringType === CellPainter.ColoringTypes.CONDITIONAL) { // Render conditional-coloring
		this._paintCell(cell, rowData);
	}
};

/**
* @public
* @param {tr.grid.Cell} cell
* @param {Object} rowData e.g. { PCTCHNG: 0.53, CF_NETCHNG: 0.75 }
* @param {number} min
* @param {number} max
*/
CellPainter.prototype.renderHeatMap = function (cell, rowData, min, max) {
	this._paintCell(cell, rowData, min, max);
};

/**
* @public
* @param {tr.grid.Cell} cell
* @param {Object} rowData e.g. { PCTCHNG: 0.53, CF_NETCHNG: 0.75 }
*/
CellPainter.prototype.renderColoring = function (cell, rowData) {
	this._paintCell(cell, rowData);
};

/**
* @private
* @param {tr.grid.Cell} cell
* @param {Object} rowData e.g. { PCTCHNG: 0.53, CF_NETCHNG: 0.75 }
* @param {number=} min
* @param {number=} max
*/
CellPainter.prototype._paintCell = function(cell, rowData, min, max) {
	if (!cell) return;
	let  elem = cell.getElement();
	if (!elem) return; // Cell has been disposed

	let bc = this._blinkCondition;
	let scope;
	if (bc) {
		scope = cell["blinking"];
		if (scope && scope["blinkTimer"] && !bc["border"]) { // The cell is blinking
			return; // Let blinker calculate the style when restoring
		}
	}

	let styles = this._getStyles(rowData, min, max);

	let elStyle = elem.style;

	let cssClass = styles["cssClass"]; // Can be an empty string
	_removeCssClass(elem, cssClass);
	if (cssClass != null) { // Predefined colors mode
		if (cssClass) {
			elem.classList.add(cssClass);
			elem._coloringCssClass = cssClass;
		}
	} else {
		if(styles["backgroundColor"]){
			elStyle.backgroundColor = styles["backgroundColor"];
		} else if(!this._effectiveStyles || this._effectiveStyles["backgroundColor"]){
			elStyle.backgroundColor = "";
		}
		if(styles["color"]){
			elStyle.color = styles["color"];
		} else if(!this._effectiveStyles || this._effectiveStyles["color"]){
			elStyle.color = "";
		}
	}
};

/**
* @private
* @param {tr.grid.Cell} cell
* @return {Object}
*/
CellPainter.prototype._prepareScope = function(cell) {
	let scope = cell["blinking"];
	if (!scope) {
		scope = {};
		scope["cell"] = cell;
		cell["blinking"] = scope;

		this._scopes.push(scope);
	}

	let bc = this._blinkCondition;
	if (scope["blinkId"] !== bc["blinkId"]) {
		scope["blinkId"] = bc["blinkId"];
		scope["field"] = bc["field"];
		let restorer = bc["border"] ? CellPainter._borderRestorer : CellPainter._cellRestorer;
		scope._restorer = restorer.bind(this, scope);
	}

	return scope;
};

/**
* @public
* @param {tr.grid.Cell} cell
* @param {number} blinkSignal
* @param {Object} rowData to calculate original style e.g. { PCTCHNG: 0.53, CF_NETCHNG: 0.75 }
* @return {boolean}
*/
CellPainter.prototype.blink = function (cell, blinkSignal, rowData) {
	if (!cell) return false;

	let elem = cell.getElement();
	if (!elem) return false; // Cell has been disposed

	let bc = this._blinkCondition;
	if (!bc) return false;

	let scope = this._prepareScope(cell);

	scope["rowData"] = rowData;

	let blinkColor, contrastColor;
	if (blinkSignal > 0) {
		blinkColor = bc["upColor"];
		contrastColor = bc["contrastUpColor"];
	} else if (blinkSignal < 0) {
		blinkColor = bc["downColor"];
		contrastColor = bc["contrastDownColor"];
	} else {
		blinkColor = bc["levelColor"];
		contrastColor = bc["contrastLevelColor"];
	}

	this._blink(scope, elem, blinkColor, contrastColor);
	return true;
};

/** Blink cell with specific color or theme's neutral movement color.
* @public
* @param {tr.grid.Cell} cell
* @param {string|null} blinkColor
* @param {Object} rowData to calculate original style e.g. { PCTCHNG: 0.53, CF_NETCHNG: 0.75 }
* @return {boolean}
*/
CellPainter.prototype.flash = function(cell, blinkColor, rowData) {
	if (!cell) return false;

	let elem = cell.getElement();
	if (!elem) return false; // Cell has been disposed

	if (!this._blinkCondition) return false;

	let scope = this._prepareScope(cell);

	scope["rowData"] = rowData;

	if (typeof blinkColor !== "string") {
		blinkColor = ElfUtil.themeColors["level"];
	}

	this._blink(scope, elem, blinkColor, CellPainter.getOppositeColor(blinkColor));
	return true;
};

/** @private
* @param {Object} scope
* @param {Element} elem
* @param {string} blinkColor
* @param {string} contrastColor
* @return {boolean} Returns true if it is a background blinking, otherwise returns false
*/
CellPainter.prototype._blink = function (scope, elem, blinkColor, contrastColor) {
	CellPainter._clearBlinkTimer(scope);
	scope["blinkTimer"] = setTimeout(scope._restorer, this._blinkingDuration);

	if (this._blinkCondition["border"]) {
		elem.style.border = "1px solid " + blinkColor;
		return false;
	} else {
		elem.style.backgroundColor = blinkColor;
		elem.style.color = contrastColor;
	}

	return true;
};

/**
* @public
* @param {tr.grid.Cell} cell
* @param {number} newValue
* @param {number} oldValue
* @param {Object} rowData e.g. { PCTCHNG: 0.53, CF_NETCHNG: 0.75 }
* @return {boolean}
*/
CellPainter.prototype.blinkCell = function(cell, newValue, oldValue, rowData) {
	let bc = this._blinkCondition;
	if (!bc) return false;

	let blinkSignal = this._blinkCondition._fn(newValue, oldValue);
	return this.blink(cell, blinkSignal, rowData);
};

/**
* @public
* @ignore
* @param {number} newValue
* @param {number} oldValue
* @return {number}
*/
CellPainter.prototype.calcBlinkSignal = function(newValue, oldValue) {
	if (this._blinkCondition) {
		return this._blinkCondition._fn(newValue, oldValue);
	}
	return 0;
};

/**
* @public
* @param {tr.grid.Cell} cell
* @param {Object} rowData
*/
CellPainter.prototype.verifyBlinking = function(cell, rowData) {
	let scope = cell["blinking"];
	if (scope && scope["blinkTimer"]) {
		if (scope["rowData"] !== rowData) {
			scope["rowData"] = rowData; // Needed for style calculation by restorer
			if (CellPainter._clearBlinkTimer(scope)) {
				scope._restorer();
			}
		}
	}
};
/** Deprecated, the state has been moved out to support prioritization.
* @public
* @deprecated
* @param {boolean=} disabled
*/
CellPainter.prototype.disableLevelColor = function(disabled) {};
/**
* @public
* @param {number} duration
*/
CellPainter.prototype.setBlinkingDuration = function(duration) {
	if(typeof duration === "number"){
		this._blinkingDuration = duration;
	}
};

/* harmony default export */ const es6_CellPainter = ((/* unused pure expression or super */ null && (CellPainter)));


;// ./src/index.js
/* eslint-disable */

/* eslint-enable */

// tsd-disable




if (window) {
	var trNamespace = window['tr'];
	if(!trNamespace) {
		trNamespace = window['tr'] = {};
	}
	var rtNamespace = trNamespace['rt'];
	if(!rtNamespace) {
		rtNamespace = trNamespace['rt'] = {};
	}
	var gridNamespace = trNamespace['grid'];
	if(!gridNamespace) {
		gridNamespace = trNamespace['grid'] = {};
	}

	if(!trNamespace['MockJET']) {
		trNamespace.MockJET = MockJET;
	}
	if(!trNamespace['closestElement']) {
		trNamespace.closestElement = Dom.closestElement;
	}
	if(!trNamespace['CellPainter']) {
		trNamespace.CellPainter = CellPainter;
	}
	if(!rtNamespace['Grid']) {
		rtNamespace.Grid = Grid_Grid;
	}
	if(!gridNamespace['CellPainter']) {
		gridNamespace.CellPainter = CellPainter;
	}
}
// tsd-enable


/* harmony default export */ const src = ((/* unused pure expression or super */ null && (Grid)));

/******/ })()
;